<?php declare(strict_types=1);

namespace GraphQL\Language\AST;

/**
 * export type TypeSystemExtensionNode = SchemaExtensionNode | TypeExtensionNode;.
 */
interface TypeSystemExtensionNode extends DefinitionNode {}
