/*!
 * ---------------------------------------------------------------------
 *
 * GLPI - Gestionnaire Libre de Parc Informatique
 *
 * http://glpi-project.org
 *
 * @copyright 2015-2026 Teclib' and contributors.
 * @copyright 2003-2014 by the INDEPNET Development Team.
 * @licence   https://www.gnu.org/licenses/gpl-3.0.html
 *
 * ---------------------------------------------------------------------
 *
 * LICENSE
 *
 * This file is part of GLPI.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------
 */

.tab-content {
    border-top-left-radius: 0;
}

.card-tabs {
    #tabspanel {
        border-right: 1px solid var(--glpi-tabs-border-color);
    }

    #tabspanel.nav-tabs {
        .nav-link {
            background: var(--glpi-tabs-bg);
            color: var(--glpi-tabs-fg);
            border-color: var(--glpi-tabs-border-color);
            border-right-width: 0;

            &.active {
                border-right: 1px solid transparent !important;
                margin-right : -1px; // Hack for border overlap
                background: var(--glpi-tabs-active-bg);
                color: var(--glpi-tabs-active-fg);
                border-color: var(--glpi-tabs-active-border-color);
                font-weight: bold;

                .badge {
                    font-weight: bold;
                }
            }

            .badge {
                margin-left: 5px;
            }
        }
    }

    &.vertical {
        .nav:not(.nav-underline)
            .nav-link {
                border-bottom: 1px solid var(--glpi-tabs-border-color);
                border-top-right-radius: 0;

                &.active {
                    border-left-width: 5px;
                    border-right-color: transparent !important;
                }
            }
    }

    &.horizontal {
        #tabspanel.nav-tabs {
            .nav-link {
                border-bottom-width: 1px;
                border-bottom-style: solid;

                &:last-of-type {
                    border-right-width: 1px;

                    &.active {
                        border-right-color: var(--glpi-tabs-border-color) !important;
                    }
                }

                &.active {
                    border-bottom-color: transparent !important;
                }
            }
        }
    }
}
