<?php

declare(strict_types=1);

namespace Laminas\ServiceManager\Exception;

use Psr\Container\ContainerExceptionInterface;

/**
 * Base exception for all Laminas\ServiceManager exceptions.
 */
interface ExceptionInterface extends ContainerExceptionInterface
{
}
