{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

<div class="container">
   <div class="row justify-content-evenly">
      <div class="col-12 col-xxl-4">
         <div class="card">
            <div class="card-header">
               <h3>{{ __('External authentications') }}</h3>
            </div>
            <div class="list-group list-group-flush">
               {% if has_profile_right('config', constant('UPDATE')) %}
                  <a class="list-group-item list-group-item-action" href="{{ path('front/auth.settings.php') }}">
                     <i class="ti ti-adjustments me-1"></i>
                     <span>{{ __('Setup') }}</span>
                  </a>
               {% endif %}

               {% if can_use_ldap %}
                  <a class="list-group-item list-group-item-action" href="{{ path('front/authldap.php') }}">
                     <i class="{{ call('AuthLDAP::getIcon') }} me-1"></i>
                     <span>{{ 'AuthLDAP'|itemtype_name }}</span>
                  </a>
               {% else %}
                  <div class="alert alert-important alert-warning m-3">
                    <i class="ti ti-alert-triangle me-2"></i>
                    {{ __('The LDAP extension of your PHP parser isn’t installed') }}
                  </div>
               {% endif %}

               <a class="list-group-item list-group-item-action" href="{{ path('front/authmail.php') }}">
                  <i class="{{ call('AuthMail::getIcon') }} me-1"></i>
                  <span>{{ 'AuthMail'|itemtype_name }}</span>
               </a>

               <a class="list-group-item list-group-item-action" href="{{ path('front/auth.others.php') }}">
                  <i class="ti ti-login me-1"></i>
                  <span>{{ __('Others authentication methods') }}</span>
               </a>
            </div>
         </div>
      </div>
   </div>
</div>
