{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% if show_warning %}
    <span id="is_default_warning" class="text-warning d-none">
        {{ tooltip|raw }}
    </span>
{% endif %}

<script>
    $('select[name="is_default"]').on("change", function () {
        if ($(this).val() == 1) {
            $("#is_default_warning").removeClass("d-none");
            $('select[name="is_pending_per_default"]').prop("disabled", false);
        } else {
            $("#is_default_warning").addClass("d-none");
            $('select[name="is_pending_per_default"]').prop("disabled", true);
            $('select[name="is_pending_per_default"]').trigger('setValue', 0);
        }
    });
</script>
