{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{# @var can_update bool #}
{# @var form Glpi\Form\Form #}

<section
    role="option"
    aria-label="{{ __('Form horizontal block placeholder') }}"
    data-glpi-form-editor-horizontal-block-placeholder
    data-glpi-form-editor-on-click="set-active"
>
    {{ include('pages/admin/form/form_toolbar.html.twig', {
        'can_update'                               : can_update,
        'form'                                     : form,
        'ghost_button'                             : true,
        'hide_section_button'                      : true,
        'hide_new_horizontal_layout_button'        : true,
        'show_remove_horizontal_placeholder_button': true,
    }, with_context = false) }}
</section>


