/*!
 * ---------------------------------------------------------------------
 *
 * GLPI - Gestionnaire Libre de Parc Informatique
 *
 * http://glpi-project.org
 *
 * @copyright 2015-2026 Teclib' and contributors.
 * @copyright 2003-2014 by the INDEPNET Development Team.
 * @licence   https://www.gnu.org/licenses/gpl-3.0.html
 *
 * ---------------------------------------------------------------------
 *
 * LICENSE
 *
 * This file is part of GLPI.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------
 */

// The styles for the debug toolbar itself are in the Vue SFCs

body.debug-active:not(.debug-folded) {
    .search_page {
        .search-container {
            @include media-breakpoint-up(lg) {
                height: calc(100vh - var(--glpi-contextbar-height) - var(--glpi-content-margin) - 50px);
            }
        }
    }

    .tab-pane:not([id^="tab-Ticket_main"]) {
        margin-bottom: 50px;
    }

    .itil-object {
        @include media-breakpoint-up(lg) {
            height: calc(100vh - 187px - 46px);
        }
    }

    form.new-itil-object .itil-object {
        @include media-breakpoint-up(lg) {
            height: calc(100vh - 187px - 5px);
        }
    }


    &.horizontal-layout {
        .itil-object {
            @include media-breakpoint-up(lg) {
                height: calc(100vh - 257px - 46px);
            }
        }
    }

    .search-footer,
    .itil-footer {
        bottom: 46px;
    }
}
