<?php

namespace PhpOffice\PhpSpreadsheet\Style\ConditionalFormatting;

enum IconSetValues: string
{
    case ThreeArrows = '3Arrows';
    case ThreeArrowsGray = '3ArrowsGray';
    case ThreeFlags = '3Flags';
    case ThreeTrafficLights1 = '3TrafficLights1';
    case ThreeTrafficLights2 = '3TrafficLights2';
    case ThreeSigns = '3Signs';
    case ThreeSymbols = '3Symbols';
    case ThreeSymbols2 = '3Symbols2';
    case FourArrows = '4Arrows';
    case FourArrowsGray = '4ArrowsGray';
    case FourRedToBlack = '4RedToBlack';
    case FourRating = '4Rating';
    case FourTrafficLights = '4TrafficLights';
    case FiveArrows = '5Arrows';
    case FiveArrowsGray = '5ArrowsGray';
    case FiveRating = '5Rating';
    case FiveQuarters = '5Quarters';
    // The following icon sets are not implemented yet:
    // case ThreeStars = "3Stars";
    // case ThreeTriangles = "3Triangles";
    // case FiveBoxes = "5Boxes";
}
