/*!
 * ---------------------------------------------------------------------
 *
 * GLPI - Gestionnaire Libre de Parc Informatique
 *
 * http://glpi-project.org
 *
 * @copyright 2015-2026 Teclib' and contributors.
 * @licence   https://www.gnu.org/licenses/gpl-3.0.html
 *
 * ---------------------------------------------------------------------
 *
 * LICENSE
 *
 * This file is part of GLPI.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------
 */

$enable-negative-margins: true;
$link-hover-decoration: none;

@import "@tabler/core/scss/core";
@import "@fontsource/inter/scss/mixins";

$fontDir: "@fontsource/inter/files";

@include fontFace($weight: 100);
@include fontFace($weight: 200);
@include fontFace($weight: 300);
@include fontFace($weight: 400);
@include fontFace($weight: 500);
@include fontFace($weight: 600);
@include fontFace($weight: 700);
@include fontFace($weight: 800);
@include fontFace($weight: 900);

:root {
    --tblr-font-sans-serif: inter, -apple-system, blinkmacsystemfont, san francisco, segoe ui, roboto, helvetica neue, sans-serif;
}

$ti-font-path: "@tabler/icons-webfont/dist/fonts";

@import "@tabler/icons-webfont/dist/tabler-icons";
