{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

<div class="d-flex align-items-center">
    <button
        type="submit"
        form="ch_ent_o_{{ rand }}"
        name="id"
        value="{{ id }}"
        class="btn btn-link p-0 bg-transparent {{ is_recursive ? 'fw-bold' : '' }}">
        {{ name }}
    </button>

    {% if is_recursive %}
        <button
            type="submit"
            form="ch_ent_r_{{ rand }}"
            name="id"
            value="{{ id }}"
            class="btn btn-outline-secondary p-0 ms-1"
            aria-label="{{ __('Select %1s entity with all its sub entities')|format(name) }}"
        >
            <i
                class="ti ti-chevrons-down"
                data-bs-toggle="tooltip"
                data-bs-placement="right"
                title="{{ __('Select %1s entity with all its sub entities')|format(name) }}"
            ></i>
        </button>
    {% endif %}
</div>
