{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% import 'components/form/fields_macros.html.twig' as fields %}
{% import 'components/form/basic_inputs_macros.html.twig' as inputs %}

{% set rand = random() %}

<form method="post" action="{{ 'User'|itemtype_form_path }}">
   {{ inputs.hidden('_glpi_csrf_token', csrf_token()) }}
   {{ inputs.hidden('id', user.getID) }}

   {% if user.fields['authtype'] in [constant('Auth::LDAP')] %}
      {{ inputs.submit('force_ldap_resynch', __('Force synchronization'), 1, {
         icon: 'ti ti-refresh'
      }) }}
      {{ inputs.submit('clean_ldap_fields', __('Clean LDAP fields and force synchronisation'), 1, {
         icon: 'ti ti-recycle'
      }) }}
   {% endif %}
   {{ fields.htmlField('', call('Auth::dropdown', [{
      'name': 'authtype',
      'rand': rand,
      'display': false
   }]), __('Change of the authentication method'), {
      is_horizontal: false
   }) }}
   {% do call('Ajax::updateItemOnSelectEvent', ['dropdown_authtype' ~ rand, 'show_massiveaction_field', path('/ajax/dropdownMassiveActionAuthMethods.php'), {
      authtype: '__VALUE__',
      name: 'change_auth_method'
   }]) %}
   <span id="show_massiveaction_field"></span>
</form>
