/*!
 * ---------------------------------------------------------------------
 *
 * GLPI - Gestionnaire Libre de Parc Informatique
 *
 * http://glpi-project.org
 *
 * @copyright 2015-2026 Teclib' and contributors.
 * @copyright 2003-2014 by the INDEPNET Development Team.
 * @licence   https://www.gnu.org/licenses/gpl-3.0.html
 *
 * ---------------------------------------------------------------------
 *
 * LICENSE
 *
 * This file is part of GLPI.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------
 */

:root[data-glpi-theme="automn"] {
    --tblr-primary-rgb: 242, 178, 101;
    --tblr-primary-fg: #8b4d34;
    --tblr-secondary: #757575;
    --tblr-secondary-fg: #f4f6fa;
    --tblr-link-color-rgb: 192, 111, 61;
    --glpi-mainmenu-bg: #8b4d34; // hsv(17°, 46%, 37%)
    --glpi-helpdesk-header: hsl(18deg, 41%, 85%);
    --glpi-mainmenu-fg: #f4f6fa;
    --glpi-palette-color-1: #8b4d34;
    --glpi-palette-color-2: #58372e;
    --glpi-palette-color-3: #f2b265;
    --glpi-palette-color-4: #c06f3d;

    // Helpdesk header
    --glpi-illustrations-gradient-1: hsl(17deg, 46%, 92%);
    --glpi-illustrations-gradient-2: hsl(17deg, 46%, 72%);
    --glpi-illustrations-gradient-3: hsl(17deg, 46%, 45%);
}
