tinymce.addI18n("sq", {
    "Redo": "Rib\xebje",
    "Undo": "Zhb\xebje",
    "Cut": "Prije",
    "Copy": "Kopjoje",
    "Paste": "Ngjite",
    "Select all": "P\xebrzgjidhe krejt",
    "New document": "Dokument i ri",
    "Ok": "",
    "Cancel": "Anuloje",
    "Visual aids": "Ndihm\xebs pamor\xeb",
    "Bold": "T\xeb trasha",
    "Italic": "T\xeb pjerrta",
    "Underline": "N\xebnvij\xeb",
    "Strikethrough": "Hequrvije",
    "Superscript": "Sip\xebrshkrim",
    "Subscript": "Posht\xebshkrim",
    "Clear formatting": "Spastroju formatimin",
    "Remove": "",
    "Align left": "V\xebre majtas",
    "Align center": "V\xebre n\xeb qend\xebr",
    "Align right": "V\xebre djathtas",
    "No alignment": "",
    "Justify": "P\xebrligje",
    "Bullet list": "List\xeb me toptha",
    "Numbered list": "List\xeb e num\xebrtuara",
    "Decrease indent": "Zvog\xebloji shmangien e kryeradh\xebs",
    "Increase indent": "Rritja shmangien e kryeradh\xebs",
    "Close": "Mbylle",
    "Formats": "Formate",
    "Your browser doesn't support direct access to the clipboard. Please use the Ctrl+X/C/V keyboard shortcuts instead.": "Shfletuesi juaj s\u2019mbulon hyrje t\xeb drejtp\xebrdrejt\xeb n\xeb t\xeb papast\xebr. Ju lutemi, n\xeb vend t\xeb k\xebsaj, p\xebrdorni shkurtore tastiere Ctrl+X/C/V.",
    "Headings": "Krye",
    "Heading 1": "Krye 1",
    "Heading 2": "Krye 2",
    "Heading 3": "Krye 3",
    "Heading 4": "Krye 4",
    "Heading 5": "Krye 5",
    "Heading 6": "Krye 6",
    "Preformatted": "E paraformatuar",
    "Div": "",
    "Pre": "",
    "Code": "Kod",
    "Paragraph": "Paragraf",
    "Blockquote": "Citat",
    "Inline": "Brendazi",
    "Blocks": "Blloqe",
    "Paste is now in plain text mode. Contents will now be pasted as plain text until you toggle this option off.": "Ngjitjet tani b\xebhen n\xebn m\xebnyr\xebn \u201cthjesht tekst\u201d. L\xebnda tani do t\xeb ngjitet si tekst i thjesht\xeb, derisa ta \xe7aktivizoni k\xebt\xeb mund\xebsi.",
    "Fonts": "Shkronja",
    "Font sizes": "",
    "Class": "Klas\xeb",
    "Browse for an image": "Shfletoni p\xebr nj\xeb figur\xeb",
    "OR": "OSE",
    "Drop an image here": "Lini k\xebtu nj\xeb figur\xeb",
    "Upload": "Ngarkoje",
    "Uploading image": "",
    "Block": "Bllokoje",
    "Align": "Vendose",
    "Default": "Parazgjedhje",
    "Circle": "Rreth",
    "Disc": "Disk",
    "Square": "Katror",
    "Lower Alpha": "Alfa t\xeb Vogla",
    "Lower Greek": "Greke t\xeb Vogla",
    "Lower Roman": "Romake t\xeb Vogla",
    "Upper Alpha": "Alfa t\xeb M\xebdha",
    "Upper Roman": "Romake t\xeb\xa0M\xebdha",
    "Anchor...": "Spiranc\xeb\u2026",
    "Anchor": "",
    "Name": "Em\xebr",
    "ID": "",
    "ID should start with a letter, followed only by letters, numbers, dashes, dots, colons or underscores.": "",
    "You have unsaved changes are you sure you want to navigate away?": "Keni ndryshime t\xeb paruajtura, jeni i sigurt se doni t\xeb ikni prej k\xebtej?",
    "Restore last draft": "Rikthe skic\xebn e fundit",
    "Special character...": "Shenj\xeb speciale\u2026",
    "Special Character": "",
    "Source code": "Kodi burim",
    "Insert/Edit code sample": "Futni/P\xebrpunoni shembull kodi",
    "Language": "Gjuh\xeb",
    "Code sample...": "Shembull kodi\u2026",
    "Left to right": "Nga e majta n\xeb t\xeb djatht\xeb",
    "Right to left": "Nga e djathta n\xeb t\xeb majt\xeb",
    "Title": "Titull",
    "Fullscreen": "Sa Krejt Ekrani",
    "Action": "Veprim",
    "Shortcut": "Shkurtore",
    "Help": "Ndihm\xeb",
    "Address": "Adres\xeb",
    "Focus to menubar": "Fokusi te shtyll\xeb menuje",
    "Focus to toolbar": "Fokusi te panel",
    "Focus to element path": "Fokusi te shteg elementi",
    "Focus to contextual toolbar": "Fokusi te panel kontekstual",
    "Insert link (if link plugin activated)": "Futni lidhje (n\xebse \xebsht\xeb aktivizuar shtojca e lidhjeve)",
    "Save (if save plugin activated)": "Ruaje (n\xebse \xebsht\xeb aktivizuar shtojca e ruajtjeve)",
    "Find (if searchreplace plugin activated)": "Gjeni (n\xebse \xebsht\xeb aktivizuar shtojca p\xebr k\xebrkim dhe z\xebvend\xebsim)",
    "Plugins installed ({0}):": "Shtojca t\xeb instaluara ({0}):",
    "Premium plugins:": "Shtojca me pages\xeb:",
    "Learn more...": "M\xebsoni m\xeb tep\xebr\u2026",
    "You are using {0}": "Po p\xebrdorni {0}",
    "Plugins": "Shtojca",
    "Handy Shortcuts": "Shkurtore t\xeb Volitshme",
    "Horizontal line": "Vij\xeb horizontale",
    "Insert/edit image": "Futni/p\xebrpunoni figur\xeb",
    "Alternative description": "P\xebrshkrim alternativ",
    "Accessibility": "P\xebrdorim nga persona me aft\xebsi t\xeb kufizuara",
    "Image is decorative": "Figura \xebsht\xeb p\xebr zbukurim",
    "Source": "Burim",
    "Dimensions": "P\xebrmasa",
    "Constrain proportions": "Mbaji detyrimisht p\xebrpjestimet",
    "General": "T\xeb p\xebrgjithshme",
    "Advanced": "T\xeb m\xebtejshme",
    "Style": "Stil",
    "Vertical space": "Hapsir\xeb vertikale",
    "Horizontal space": "Hapsir\xeb horizontale",
    "Border": "An\xeb",
    "Insert image": "Futni figur\xeb",
    "Image...": "Figur\xeb\u2026",
    "Image list": "List\xeb figurash",
    "Resize": "Rip\xebrmasoje",
    "Insert date/time": "Futni dat\xeb/koh\xeb",
    "Date/time": "Dat\xeb/koh\xeb",
    "Insert/edit link": "Futni/p\xebrpunoni lidhje",
    "Text to display": "Tekst p\xebr t\u2019u shfaqur",
    "Url": "",
    "Open link in...": "Hape lidhje n\xeb:\u2026",
    "Current window": "Dritaren e tanishme",
    "None": "Asnj\xeb",
    "New window": "Dritare e re",
    "Open link": "Hape lidhjen",
    "Remove link": "Hiqe lidhjen",
    "Anchors": "Spiranca",
    "Link...": "Lidhni\u2026",
    "Paste or type a link": "Hidhni ose shtypni nj\xeb lidhje",
    "The URL you entered seems to be an email address. Do you want to add the required mailto: prefix?": "URL-ja q\xeb dhat\xeb duket se \xebsht\xeb adres\xeb email. Doni t\xeb shtohet parashtesa e domosdoshme mailto: ?",
    "The URL you entered seems to be an external link. Do you want to add the required http:// prefix?": "URL-ja q\xeb dhat\xeb duket se \xebsht\xeb lidhje e jashtme. Doni t\xeb shtohet parashtesa e domosdoshme https:// ?",
    "The URL you entered seems to be an external link. Do you want to add the required https:// prefix?": "URL-ja q\xeb dhat\xeb duket t\xeb jet\xeb nj\xeb lidhje e jashtme. Doni t\u2019i shtohet parashtesa e domosdoshme https:// ?",
    "Link list": "List\xeb lidhjesh",
    "Insert video": "Futni video",
    "Insert/edit video": "Futni/p\xebrpunoni video",
    "Insert/edit media": "Futni/p\xebrpunoni media",
    "Alternative source": "Burim alternativ",
    "Alternative source URL": "URL burimi alternativ",
    "Media poster (Image URL)": "Postues mediash (URL Figure)",
    "Paste your embed code below:": "Ngjiteni kodin tuaj t\xeb trup\xebzimit m\xeb posht\xeb:",
    "Embed": "Trup\xebzojeni",
    "Media...": "Media\u2026",
    "Nonbreaking space": "Hap\xebsir\xeb e pandashme",
    "Page break": "Nd\xebrprerje faqeje",
    "Paste as text": "Ngjite si tekst",
    "Preview": "Paraparje",
    "Print": "",
    "Print...": "Shtypeni\u2026",
    "Save": "Ruaje",
    "Find": "Gjej",
    "Replace with": "Z\xebvend\xebsoje me",
    "Replace": "Z\xebvend\xebsoje",
    "Replace all": "Z\xebvend\xebsoji krejt",
    "Previous": "I m\xebparsh\xebmi",
    "Next": "Pasuesi",
    "Find and Replace": "Gjej dhe Z\xebvend\xebso",
    "Find and replace...": "Gjeni dhe z\xebvend\xebsoni\u2026",
    "Could not find the specified string.": "S\u2019u gjet dot vargu i dh\xebn\xeb.",
    "Match case": "Si\xe7 \xebsht\xeb shkruar",
    "Find whole words only": "Gjej vet\xebm fjal\xeb t\xeb plota",
    "Find in selection": "Gjeni n\xeb p\xebrzgjedhje",
    "Insert table": "Futni tabel\xeb",
    "Table properties": "Veti tabele",
    "Delete table": "Fshi tabel\xebn",
    "Cell": "Kutiz\xeb",
    "Row": "Rresht",
    "Column": "Shtyll\xeb",
    "Cell properties": "Veti kutize",
    "Merge cells": "P\xebrzieji kutizat",
    "Split cell": "Ndaje kutiz\xebn",
    "Insert row before": "Futni rresht para",
    "Insert row after": "Futni rresht pas",
    "Delete row": "Fshije rreshtin",
    "Row properties": "Veti rreshti",
    "Cut row": "Prije rreshtin",
    "Cut column": "",
    "Copy row": "Kopjoje rreshtin",
    "Copy column": "",
    "Paste row before": "Ngjite rreshtin p\xebrpara",
    "Paste column before": "",
    "Paste row after": "Ngjite rreshtin pas",
    "Paste column after": "",
    "Insert column before": "Futni shtyll\xeb para",
    "Insert column after": "Futni shtyll\xeb pas",
    "Delete column": "Fshi shtyll\xeb",
    "Cols": "Shtylla",
    "Rows": "Rreshta",
    "Width": "Gjer\xebsi",
    "Height": "Lart\xebsi",
    "Cell spacing": "Hap\xebsir\xeb kutize",
    "Cell padding": "Mbushje kutize",
    "Row clipboard actions": "",
    "Column clipboard actions": "",
    "Table styles": "",
    "Cell styles": "",
    "Column header": "",
    "Row header": "",
    "Table caption": "",
    "Caption": "Legjend\xeb",
    "Show caption": "Shfaq titull",
    "Left": "Majtas",
    "Center": "N\xeb qend\xebr",
    "Right": "Djathtas",
    "Cell type": "Lloj kutize",
    "Scope": "Fokus",
    "Alignment": "Drejtim",
    "Horizontal align": "",
    "Vertical align": "",
    "Top": "N\xeb krye",
    "Middle": "N\xeb mes",
    "Bottom": "N\xeb fund",
    "Header cell": "Kutiz\xeb kryeje",
    "Row group": "Grup rreshtash",
    "Column group": "Grup shtyllash",
    "Row type": "Lloj rreshti",
    "Header": "Kryefaqe",
    "Body": "L\xebnd\xeb",
    "Footer": "Fundfaqe",
    "Border color": "Ngjyr\xeb ane",
    "Solid": "",
    "Dotted": "",
    "Dashed": "",
    "Double": "",
    "Groove": "",
    "Ridge": "",
    "Inset": "",
    "Outset": "",
    "Hidden": "",
    "Insert template...": "Futni gjedhe\u2026",
    "Templates": "Gjedhe",
    "Template": "Gjedhe",
    "Insert Template": "",
    "Text color": "Ngjyr\xeb teksti",
    "Background color": "Ngjyr\xeb sfondi",
    "Custom...": "Vetjake\u2026",
    "Custom color": "Ngjyr\xeb vetjake",
    "No color": "Pa ngjyr\xeb",
    "Remove color": "Hiqe ngjyr\xebn",
    "Show blocks": "Shfaqi blloqet",
    "Show invisible characters": "Shfaqi shenjat e padukshme",
    "Word count": "Num\xebr fjal\xebsh",
    "Count": "Num\xebr",
    "Document": "Dokument",
    "Selection": "P\xebrzgjedhje",
    "Words": "Fjal\xeb",
    "Words: {0}": "Fjal\xeb: {0}",
    "{0} words": "{0} fjal\xeb",
    "File": "Kartel\xeb",
    "Edit": "P\xebrpunoni",
    "Insert": "Futni",
    "View": "Shihni",
    "Format": "",
    "Table": "Tabel\xeb",
    "Tools": "Mjete",
    "Powered by {0}": "Bazuar n\xeb {0}",
    "Rich Text Area. Press ALT-F9 for menu. Press ALT-F10 for toolbar. Press ALT-0 for help": "Fush\xeb Teksti t\xeb Pasur. p\xebr menu, shtypni tastet ALT-F9. P\xebr panel, shtypnis tastet ALT-F10r. P\xebr ndihm\xeb, shtypni ALT-0",
    "Image title": "Titull figure",
    "Border width": "Gjer\xebsi an\xebsh",
    "Border style": "Stil an\xebsh",
    "Error": "Gabim",
    "Warn": "Sinjalizoje",
    "Valid": "E vlefshme",
    "To open the popup, press Shift+Enter": "Q\xeb t\xeb hapet flluska, shtypni tastet Shift+Enter",
    "Rich Text Area": "",
    "Rich Text Area. Press ALT-0 for help.": "Fush\xeb Teksti t\xeb Pasur. P\xebr ndihm\xeb, shtypni ALT-0.",
    "System Font": "Shkronja Sistemi",
    "Failed to upload image: {0}": "S\u2019u arrit t\xeb ngarkohej figur\xeb: {0}",
    "Failed to load plugin: {0} from url {1}": "S\u2019u arrit t\xeb ngarkohej shtojca: {0} prej url-je {1}",
    "Failed to load plugin url: {0}": "S\u2019u arrit t\xeb ngarkohej URL shtojce: {0}",
    "Failed to initialize plugin: {0}": "S\u2019u arrit t\xeb gatitej shtojca: {0}",
    "example": "shembull",
    "Search": "K\xebrko",
    "All": "Krejt",
    "Currency": "Monedh\xeb",
    "Text": "Tekst",
    "Quotations": "Citime",
    "Mathematical": "Matematikore",
    "Extended Latin": "Latinishte e Zgjeruar",
    "Symbols": "Simbole",
    "Arrows": "Shigjeta",
    "User Defined": "P\xebrcaktuar Nga P\xebrdoruesi",
    "dollar sign": "simbol dollari",
    "currency sign": "simbol monedhe",
    "euro-currency sign": "simbol euroje",
    "colon sign": "shenj\xeb dy pika",
    "cruzeiro sign": "shenj\xeb kruzeiroje",
    "french franc sign": "shenj\xeb frangu fr\xebng",
    "lira sign": "shenj\xeb sterline",
    "mill sign": "shenj\xeb mili",
    "naira sign": "shenj\xeb nairaje",
    "peseta sign": "shenj\xeb pesete",
    "rupee sign": "shenj\xeb rupie",
    "won sign": "shenj\xeb uoni",
    "new sheqel sign": "shenj\xeb shekeli t\xeb ri",
    "dong sign": "shenj\xeb dongu",
    "kip sign": "shenj\xeb kipi",
    "tugrik sign": "shenj\xeb tugriku",
    "drachma sign": "shenj\xeb drahme",
    "german penny symbol": "shenj\xeb peni gjerman",
    "peso sign": "shenj\xeb peso",
    "guarani sign": "shenj\xeb guaranie",
    "austral sign": "shenj\xeb australi",
    "hryvnia sign": "shenj\xeb hrivniaje",
    "cedi sign": "shenj\xeb cedi",
    "livre tournois sign": "shenj\xeb lire Turi",
    "spesmilo sign": "shenj\xeb spesmiloje",
    "tenge sign": "shenj\xeb tenge",
    "indian rupee sign": "shenj\xeb rupie indiane",
    "turkish lira sign": "shenj\xeb lire turke",
    "nordic mark sign": "shenj\xeb marke nordike",
    "manat sign": "shenj\xeb manati",
    "ruble sign": "shenj\xeb ruble",
    "yen character": "shenj\xeb jeni",
    "yuan character": "shenj\xeb juani",
    "yuan character, in hong kong and taiwan": "shenja e juanit, n\xeb Hong Kong dhe Tajvan",
    "yen/yuan character variant one": "shenja jen/juan n\xeb variantin nj\xeb",
    "Emojis": "",
    "Emojis...": "",
    "Loading emojis...": "",
    "Could not load emojis": "",
    "People": "Persona",
    "Animals and Nature": "Kafsh\xeb & Natyr\xeb",
    "Food and Drink": "Ushqim dhe Pije",
    "Activity": "Veprimtari",
    "Travel and Places": "Udh\xebtim dhe Vende",
    "Objects": "Objekte",
    "Flags": "Flamuj",
    "Characters": "Shenja",
    "Characters (no spaces)": "Shenja (pa hap\xebsira)",
    "{0} characters": "{0} shenja",
    "Error: Form submit field collision.": "Gabim: P\xebrplasje fushash parashtrimi formulari.",
    "Error: No form element found.": "Gabim: S\u2019u gjet element forme.",
    "Color swatch": "Palet\xeb ngjyrash",
    "Color Picker": "Zgjedh\xebs Ngjyrash",
    "Invalid hex color code: {0}": "",
    "Invalid input": "",
    "R": "",
    "Red component": "",
    "G": "",
    "Green component": "",
    "B": "",
    "Blue component": "",
    "#": "",
    "Hex color code": "",
    "Range 0 to 255": "",
    "Turquoise": "E bruzt\xeb",
    "Green": "E gjelb\xebr",
    "Blue": "E kalt\xebr",
    "Purple": "E purpurt",
    "Navy Blue": "Blu e err\xebt",
    "Dark Turquoise": "E bruzt\xeb e Err\xebt",
    "Dark Green": "E gjelb\xebr e Err\xebt",
    "Medium Blue": "Blu e Mesme",
    "Medium Purple": "E purpurt e Mesme",
    "Midnight Blue": "Blu Mesnate",
    "Yellow": "E verdh\xeb",
    "Orange": "Portokalli",
    "Red": "E kuqe",
    "Light Gray": "Gri e \xc7el\xebt",
    "Gray": "Gri",
    "Dark Yellow": "E verdh\xeb e Err\xebt",
    "Dark Orange": "Portokalli e Err\xebt",
    "Dark Red": "E kuqe e Err\xebt",
    "Medium Gray": "Gri e Mesme",
    "Dark Gray": "Gri e Err\xebt",
    "Light Green": "E gjelb\xebr e \xc7el\xebt",
    "Light Yellow": "E verdh\xeb e \xc7el\xebt",
    "Light Red": "E kuqe e \xc7el\xebt",
    "Light Purple": "E purpur e \xc7el\xebt",
    "Light Blue": "Blu e \xc7el\xebt",
    "Dark Purple": "E purpurt e Err\xebt",
    "Dark Blue": "Blu e Err\xebt",
    "Black": "E zez\xeb",
    "White": "E bardh\xeb",
    "Switch to or from fullscreen mode": "Kaloni n\xeb ose dilni nga m\xebnyra \u201cSa t\xebr\xeb ekrani\u201c",
    "Open help dialog": "Hapni dialogun e ndihm\xebs",
    "history": "historik",
    "styles": "stile",
    "formatting": "formatim",
    "alignment": "drejtim",
    "indentation": "shmangie kryeradhe",
    "Font": "Shkronja",
    "Size": "Madh\xebsi",
    "More...": "M\xeb tep\xebr\u2026",
    "Select...": "P\xebrzgjidhni\u2026",
    "Preferences": "Parap\xeblqime",
    "Yes": "Po",
    "No": "Jo",
    "Keyboard Navigation": "L\xebvizje me Tastier\xeb",
    "Version": "",
    "Code view": "Parje kodi",
    "Open popup menu for split buttons": "Hapni menu fllusk\xeb p\xebr butona ndarjeje",
    "List Properties": "Veti Liste",
    "List properties...": "Veti liste\u2026",
    "Start list at number": "Fillo list\xebn me numrin",
    "Line height": "Lart\xebsi rreshti",
    "Dropped file type is not supported": "",
    "Loading...": "",
    "ImageProxy HTTP error: Rejected request": "",
    "ImageProxy HTTP error: Could not find Image Proxy": "",
    "ImageProxy HTTP error: Incorrect Image Proxy URL": "",
    "ImageProxy HTTP error: Unknown ImageProxy error": ""
});