{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% extends 'layout/page_card_notlogged.html.twig' %}

{% block content_block %}
<div class="alert alert-warning">
    <div class="d-flex align-items-center">
        <div class="me-4">
            <i class="ti ti-alert-triangle fa-2x"></i>
        </div>
        <div>
            <h4 class="alert-title">
                {{ _n('Error', 'Errors', 1) }}
            </h4>
            <div>
                {% for error in errors %}
                    {{ error }}<br />
                {% endfor %}
            </div>

            <a href="{{ login_url }}" class="btn btn-primary mt-3">
                <i class="ti ti-login"></i>
                <span>{{ __('Log in again') }}</span>
            </a>
        </div>
    </div>
</div>
{% endblock %}
