tinymce.addI18n("nb_NO", {
    "Redo": "Gj\xf8r om",
    "Undo": "Angre",
    "Cut": "Klipp ut",
    "Copy": "Kopier",
    "Paste": "Lim inn",
    "Select all": "Marker alt",
    "New document": "Nytt dokument",
    "Ok": "Ok",
    "Cancel": "Avbryt",
    "Visual aids": "Visuelle hjelpemidler",
    "Bold": "Fet",
    "Italic": "Kursiv",
    "Underline": "Understreking",
    "Strikethrough": "Gjennomstreking",
    "Superscript": "Hevet skrift",
    "Subscript": "Senket skrift",
    "Clear formatting": "Fjern formateringer",
    "Remove": "Fjern",
    "Align left": "Venstrejuster",
    "Align center": "Midtstill",
    "Align right": "H\xf8yrejuster",
    "No alignment": "Ingen justering",
    "Justify": "Blokkjuster",
    "Bullet list": "Punktliste",
    "Numbered list": "Nummerliste",
    "Decrease indent": "Reduser innrykk",
    "Increase indent": "\xd8k innrykk",
    "Close": "Lukk",
    "Formats": "Stiler",
    "Your browser doesn't support direct access to the clipboard. Please use the Ctrl+X/C/V keyboard shortcuts instead.": "Nettleseren din st\xf8tter ikke direkte tilgang til utklippsboken. Bruk istedet tastatursnarveiene Ctrl+X/C/V.",
    "Headings": "Overskrifter",
    "Heading 1": "Overskrift 1",
    "Heading 2": "Overskrift 2",
    "Heading 3": "Overskrift 3",
    "Heading 4": "Overskrift 4",
    "Heading 5": "Overskrift 5",
    "Heading 6": "Overskrift 6",
    "Preformatted": "Forh\xe5ndsformatert",
    "Div": "Div",
    "Pre": "Pre",
    "Code": "Kode",
    "Paragraph": "Avsnitt",
    "Blockquote": "Blockquote",
    "Inline": "Innkapslet",
    "Blocks": "Blokker",
    "Paste is now in plain text mode. Contents will now be pasted as plain text until you toggle this option off.": "Lim inn er n\xe5 i ren tekst-modus. Kopiert innhold vil bli limt inn som ren tekst inntil du sl\xe5r av dette valget.",
    "Fonts": "Fonter",
    "Font sizes": "Fontst\xf8rrelser",
    "Class": "Klasse",
    "Browse for an image": "S\xf8k etter bilde",
    "OR": "QR",
    "Drop an image here": "Slipp et bilde her",
    "Upload": "Last opp",
    "Uploading image": "Laster opp bilde",
    "Block": "Blokk",
    "Align": "Juster",
    "Default": "Standard",
    "Circle": "Sirkel",
    "Disc": "Disk",
    "Square": "Firkant",
    "Lower Alpha": "Sm\xe5 bokstaver",
    "Lower Greek": "Greske minuskler",
    "Lower Roman": "Sm\xe5 romertall",
    "Upper Alpha": "Store bokstaver",
    "Upper Roman": "Store romertall",
    "Anchor...": "Lenke",
    "Anchor": "Anker",
    "Name": "Navn",
    "ID": "Id",
    "ID should start with a letter, followed only by letters, numbers, dashes, dots, colons or underscores.": "ID skal starte med en bokstav, kun etterfulgt av bokstaver, tall, bindestreker, prikker, kolon eller understreker.",
    "You have unsaved changes are you sure you want to navigate away?": "Du har ikke arkivert endringene. Vil du fortsette uten \xe5 arkivere?",
    "Restore last draft": "Gjenopprett siste utkast",
    "Special character...": "Spesialtegn...",
    "Special Character": "Spesialtegn",
    "Source code": "Kildekode",
    "Insert/Edit code sample": "Sett inn / endre kodeeksempel",
    "Language": "Spr\xe5k",
    "Code sample...": "Kodeeksempel",
    "Left to right": "Venstre til h\xf8yre",
    "Right to left": "H\xf8yre til venstre",
    "Title": "Tittel",
    "Fullscreen": "Fullskjerm",
    "Action": "Handling",
    "Shortcut": "Snarvei",
    "Help": "Hjelp",
    "Address": "Adresse",
    "Focus to menubar": "Fokus p\xe5 menylinje",
    "Focus to toolbar": "Fokus p\xe5 verkt\xf8ylinje",
    "Focus to element path": "Fokus p\xe5 elementsti",
    "Focus to contextual toolbar": "Fokus p\xe5 kontekstuell verkt\xf8ylinje",
    "Insert link (if link plugin activated)": "Sett inn lenke (dersom lenketillegg er aktivert)",
    "Save (if save plugin activated)": "Lagre (dersom lagretillegg er aktivert)",
    "Find (if searchreplace plugin activated)": "Finn (dersom tillegg for s\xf8k og erstatt er aktivert)",
    "Plugins installed ({0}):": "Installerte tillegg ({0}):",
    "Premium plugins:": "Premiumtillegg:",
    "Learn more...": "Les mer ...",
    "You are using {0}": "Du bruker {0}",
    "Plugins": "Programtillegg",
    "Handy Shortcuts": "Nyttige snarveier",
    "Horizontal line": "Horisontal linje",
    "Insert/edit image": "Sett inn / rediger bilde",
    "Alternative description": "Alternativ beskrivelse",
    "Accessibility": "Tilgjengelighet",
    "Image is decorative": "Bilde er dekorasjon",
    "Source": "Kilde",
    "Dimensions": "St\xf8rrelser",
    "Constrain proportions": "Begrens proporsjoner",
    "General": "Generelt",
    "Advanced": "Avansert",
    "Style": "Stil",
    "Vertical space": "Vertikal avstand",
    "Horizontal space": "Horisontal avstand",
    "Border": "Ramme",
    "Insert image": "Sett inn bilde",
    "Image...": "Bilde...",
    "Image list": "Bildeliste",
    "Resize": "Skaler",
    "Insert date/time": "Sett inn dato/tid",
    "Date/time": "Dato/tid",
    "Insert/edit link": "Sett inn / rediger lenke",
    "Text to display": "Tekst som skal vises",
    "Url": "Url",
    "Open link in...": "\xc5pne lenke i..",
    "Current window": "N\xe5v\xe6rende vindu",
    "None": "Ingen",
    "New window": "Nytt vindu",
    "Open link": "\xc5pne lenke",
    "Remove link": "Fjern lenke",
    "Anchors": "Forankringspunkter",
    "Link...": "Lenke...",
    "Paste or type a link": "Lim inn eller skriv en lenke",
    "The URL you entered seems to be an email address. Do you want to add the required mailto: prefix?": "Oppgitt URL ser ut til \xe5 v\xe6re en e-postadresse. \xd8nsker du \xe5 sette inn p\xe5krevet mailto: prefiks foran e-postadressen?",
    "The URL you entered seems to be an external link. Do you want to add the required http:// prefix?": "URL du skrev inn ser ut som en ekstern adresse. Vil du legge til det obligatoriske prefikset http://?",
    "The URL you entered seems to be an external link. Do you want to add the required https:// prefix?": "Nettadressen du fylte inn ser ut til \xe5 v\xe6re en ekstern. \xd8nsker du \xe5 legge til p\xe5krevd 'https://'-prefiks?",
    "Link list": "Liste over lenker",
    "Insert video": "Sett inn video",
    "Insert/edit video": "Sett inn / rediger video",
    "Insert/edit media": "Sett inn / endre media",
    "Alternative source": "Alternativ kilde",
    "Alternative source URL": "Alternativ kilde URL",
    "Media poster (Image URL)": "Mediaposter (bilde-URL)",
    "Paste your embed code below:": "Lim inn inkluderingskoden nedenfor:",
    "Embed": "Inkluder",
    "Media...": "Media..",
    "Nonbreaking space": "Hardt mellomrom",
    "Page break": "Sideskifte",
    "Paste as text": "Lim inn som tekst",
    "Preview": "Forh\xe5ndsvis",
    "Print": "Utskrift",
    "Print...": "Skriv ut...",
    "Save": "Lagre",
    "Find": "S\xf8k etter",
    "Replace with": "Erstatt med",
    "Replace": "Erstatt",
    "Replace all": "Erstatt alle",
    "Previous": "Forrige",
    "Next": "Neste",
    "Find and Replace": "Finn og erstatt",
    "Find and replace...": "Finn og erstatt...",
    "Could not find the specified string.": "Kunne ikke finne den spesifiserte teksten",
    "Match case": "Skill mellom store / sm\xe5 bokstaver",
    "Find whole words only": "Finn kun hele ord",
    "Find in selection": "Finn i utvalg",
    "Insert table": "Sett inn tabell",
    "Table properties": "Tabellegenskaper",
    "Delete table": "Slett tabell",
    "Cell": "Celle",
    "Row": "Rad",
    "Column": "Kolonne",
    "Cell properties": "Celleegenskaper",
    "Merge cells": "Sl\xe5 sammen celler",
    "Split cell": "Splitt celle",
    "Insert row before": "Sett inn rad f\xf8r",
    "Insert row after": "Sett inn rad etter",
    "Delete row": "Slett rad",
    "Row properties": "Radegenskaper",
    "Cut row": "Klipp ut rad",
    "Cut column": "Kutt kolonne",
    "Copy row": "Kopier rad",
    "Copy column": "Kopier kolonne",
    "Paste row before": "Lim inn rad f\xf8r",
    "Paste column before": "Lim inn kolonne f\xf8r",
    "Paste row after": "Lim inn rad etter",
    "Paste column after": "Lim inn kolonne etter",
    "Insert column before": "Sett inn kolonne f\xf8r",
    "Insert column after": "Sett inn kolonne etter",
    "Delete column": "Slett kolonne",
    "Cols": "Kolonner",
    "Rows": "Rader",
    "Width": "Bredde",
    "Height": "H\xf8yde",
    "Cell spacing": "Celleavstand",
    "Cell padding": "Cellemarg",
    "Row clipboard actions": "Rad-utklippstavlehandlinger",
    "Column clipboard actions": "Kolonne-utklippstavlehandlinger",
    "Table styles": "Tabellstiler",
    "Cell styles": "Cellestiler",
    "Column header": "Kolonneoverskrift",
    "Row header": "Radoverskrift",
    "Table caption": "Tabelloverskrift",
    "Caption": "Bildetekst",
    "Show caption": "Vis bildetekst",
    "Left": "Venstre",
    "Center": "Senter",
    "Right": "H\xf8yre",
    "Cell type": "Celletype",
    "Scope": "Omfang",
    "Alignment": "Justering",
    "Horizontal align": "Horisontal justering",
    "Vertical align": "Vertikal justering",
    "Top": "Topp",
    "Middle": "Sentrert",
    "Bottom": "Bunn",
    "Header cell": "Overskriftscelle",
    "Row group": "Radgruppe",
    "Column group": "Kolonnegruppe",
    "Row type": "Radtype",
    "Header": "Overskrift",
    "Body": "Br\xf8dtekst",
    "Footer": "Bunntekst",
    "Border color": "Rammefarge",
    "Solid": "Solid",
    "Dotted": "Stiplet",
    "Dashed": "Stiplet",
    "Double": "Dobbel",
    "Groove": "Rillekant",
    "Ridge": "Ridge",
    "Inset": "Innfelt",
    "Outset": "Utfelt",
    "Hidden": "Skjult",
    "Insert template...": "Sett inn mal..",
    "Templates": "Maler",
    "Template": "Mal",
    "Insert Template": "Sett inn mal",
    "Text color": "Tekstfarge",
    "Background color": "Bakgrunnsfarge",
    "Custom...": "Tilpasset...",
    "Custom color": "Tilpasset farge",
    "No color": "Ingen farge",
    "Remove color": "Fjern farge",
    "Show blocks": "Vis blokker",
    "Show invisible characters": "Vis skjulte tegn",
    "Word count": "Ordtelling",
    "Count": "Opptelling",
    "Document": "Dokument",
    "Selection": "Utvalg",
    "Words": "Ord",
    "Words: {0}": "Ord: {0}",
    "{0} words": "{0} ord",
    "File": "Fil",
    "Edit": "Rediger",
    "Insert": "Sett inn",
    "View": "Vis",
    "Format": "Format",
    "Table": "Tabell",
    "Tools": "Verkt\xf8y",
    "Powered by {0}": "Drevet av {0}",
    "Rich Text Area. Press ALT-F9 for menu. Press ALT-F10 for toolbar. Press ALT-0 for help": "Tekstredigering. Tast ALT-F9 for meny. Tast ALT-F10 for verkt\xf8ylinje. Tast ALT-0 for hjelp.",
    "Image title": "Bildetittel",
    "Border width": "Bordbredde",
    "Border style": "Bordstil",
    "Error": "Feil",
    "Warn": "Advarsel",
    "Valid": "Gyldig",
    "To open the popup, press Shift+Enter": "For \xe5 \xe5pne popup, trykk Shift+Enter",
    "Rich Text Area": "Rik tekst-omr\xe5de",
    "Rich Text Area. Press ALT-0 for help.": "Rik-tekstomr\xe5de. Trykk ALT-0 for hjelp.",
    "System Font": "Systemfont",
    "Failed to upload image: {0}": "Opplasting av bilde feilet: {0}",
    "Failed to load plugin: {0} from url {1}": "Kunne ikke laste tillegg: {0} from url {1}",
    "Failed to load plugin url: {0}": "Kunne ikke laste tillegg url: {0}",
    "Failed to initialize plugin: {0}": "Kunne ikke initialisere tillegg: {0}",
    "example": "eksempel",
    "Search": "S\xf8k",
    "All": "Alle",
    "Currency": "Valuta",
    "Text": "Tekst",
    "Quotations": "Sitater",
    "Mathematical": "Matematisk",
    "Extended Latin": "Utvidet latin",
    "Symbols": "Symboler",
    "Arrows": "Piler",
    "User Defined": "Brukerdefinert",
    "dollar sign": "dollartegn",
    "currency sign": "valutasymbol",
    "euro-currency sign": "Euro-valutasymbol",
    "colon sign": "kolon-symbol",
    "cruzeiro sign": "cruzeiro-symbol",
    "french franc sign": "franske franc-symbol",
    "lira sign": "lire-symbol",
    "mill sign": "mill-symbol",
    "naira sign": "naira-symbol",
    "peseta sign": "peseta-symbol",
    "rupee sign": "rupee-symbol",
    "won sign": "won-symbol",
    "new sheqel sign": "Ny sheqel-symbol",
    "dong sign": "dong-symbol",
    "kip sign": "kip-symbol",
    "tugrik sign": "tugrik-symbol",
    "drachma sign": "drachma-symbol",
    "german penny symbol": "tysk penny-symbol",
    "peso sign": "peso-symbol",
    "guarani sign": "quarani-symbol",
    "austral sign": "austral-symbol",
    "hryvnia sign": "hryvina-symbol",
    "cedi sign": "credi-symbol",
    "livre tournois sign": "livre tournois-symbol",
    "spesmilo sign": "spesmilo-symbol",
    "tenge sign": "tenge-symbol",
    "indian rupee sign": "indisk rupee-symbol",
    "turkish lira sign": "tyrkisk lire-symbol",
    "nordic mark sign": "nordisk mark-symbol",
    "manat sign": "manat-symbol",
    "ruble sign": "ruble-symbol",
    "yen character": "yen-symbol",
    "yuan character": "yuan-symbol",
    "yuan character, in hong kong and taiwan": "yuan-symbol, i Hongkong og Taiwan",
    "yen/yuan character variant one": "yen/yuan-symbol variant en",
    "Emojis": "Emojier",
    "Emojis...": "Emojier...",
    "Loading emojis...": "Laster emojier...",
    "Could not load emojis": "Kunne ikke laste inn emojier",
    "People": "Mennesker",
    "Animals and Nature": "Dyr og natur",
    "Food and Drink": "Mat og drikke",
    "Activity": "Aktivitet",
    "Travel and Places": "Reise og steder",
    "Objects": "Objekter",
    "Flags": "Flagg",
    "Characters": "Tegn",
    "Characters (no spaces)": "Tegn (uten mellomrom)",
    "{0} characters": "{0} tegn",
    "Error: Form submit field collision.": "Feil: Skjemafelt innsendingskollisjon.",
    "Error: No form element found.": "Feil: Intet skjemafelt funnet.",
    "Color swatch": "Fargepalett",
    "Color Picker": "Fargevelger",
    "Invalid hex color code: {0}": "Ugyldig heksadesimal fargekode: {0}",
    "Invalid input": "Ugyldig inndata",
    "R": "R",
    "Red component": "R\xf8d komponent",
    "G": "G",
    "Green component": "Gr\xf8nn komponent",
    "B": "B",
    "Blue component": "Bl\xe5 komponent",
    "#": "#",
    "Hex color code": "Heksadesimal fargekode",
    "Range 0 to 255": "Omr\xe5de 0 til 255",
    "Turquoise": "Turkis",
    "Green": "Gr\xf8nn",
    "Blue": "Bl\xe5",
    "Purple": "Lilla",
    "Navy Blue": "Marinebl\xe5",
    "Dark Turquoise": "M\xf8rketurkis",
    "Dark Green": "M\xf8rkegr\xf8nn",
    "Medium Blue": "Mellombl\xe5",
    "Medium Purple": "Mellomlilla",
    "Midnight Blue": "Midnattbl\xe5",
    "Yellow": "Gul",
    "Orange": "Oransje",
    "Red": "R\xf8d",
    "Light Gray": "Lysegr\xe5",
    "Gray": "Gr\xe5",
    "Dark Yellow": "M\xf8rkegul",
    "Dark Orange": "M\xf8rkeoransje",
    "Dark Red": "M\xf8rker\xf8d",
    "Medium Gray": "Mellomgr\xe5",
    "Dark Gray": "M\xf8rkegr\xe5",
    "Light Green": "Lysegr\xf8nn",
    "Light Yellow": "Lysegul",
    "Light Red": "Lyser\xf8d",
    "Light Purple": "Lyselilla",
    "Light Blue": "Lysebl\xe5",
    "Dark Purple": "M\xf8rkelilla",
    "Dark Blue": "M\xf8rkebl\xe5",
    "Black": "Svart",
    "White": "Hvit",
    "Switch to or from fullscreen mode": "Bytt til eller fra fullskjermmodus",
    "Open help dialog": "\xc5pne hjelp-dialog",
    "history": "historikk",
    "styles": "stiler",
    "formatting": "formatering",
    "alignment": "justering",
    "indentation": "innrykk",
    "Font": "Skrift",
    "Size": "St\xf8rrelse",
    "More...": "Mer...",
    "Select...": "Velg...",
    "Preferences": "Innstillinger",
    "Yes": "Ja",
    "No": "Nei",
    "Keyboard Navigation": "Navigering med tastaturet",
    "Version": "Versjon",
    "Code view": "Kodevisning",
    "Open popup menu for split buttons": "\xc5pne sprettoppmeny for splitt-knapper",
    "List Properties": "Listeegenskaper",
    "List properties...": "Listeegenskaper ...",
    "Start list at number": "Start liste p\xe5 nummer",
    "Line height": "Linjeh\xf8yde",
    "Dropped file type is not supported": "Filtype st\xf8ttes ikke",
    "Loading...": "Laster...",
    "ImageProxy HTTP error: Rejected request": "ImageProxy HTTP-feil: Avvist foresp\xf8rsel",
    "ImageProxy HTTP error: Could not find Image Proxy": "ImageProxy HTTP-feil: Fant ikke Image Proxy",
    "ImageProxy HTTP error: Incorrect Image Proxy URL": "ImageProxy HTTP-feil: Feil Image Proxy URL",
    "ImageProxy HTTP error: Unknown ImageProxy error": "ImageProxy HTTP-feil: Ukjent ImageProxy-feil"
});