{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% import 'components/form/fields_macros.html.twig' as fields %}

<input
    type="hidden"
    name="{{ input_name }}[itemtype]"
    value="{{ itemtype }}"
    data-glpi-conditions-editor-value
>

{% if input_value is iterable and input_value.itemtype is not same as(itemtype) %}
    {% set input_value = {
        'itemtype': itemtype,
        'items_id': 0,
    } %}
{% endif %}

{{ fields.dropdownField(
    itemtype,
    input_name ~ '[items_id]',
    input_value.items_id|default(0),
    '',
    {
        'no_label'     : true,
        'wrapper_class': '',
        'mb'           : '',
        'aria_label'   : input_label,
        'addicon'      : false,
        'comments'     : '',
        'specific_tags': {
            'data-glpi-conditions-editor-value': '',
        },
        'field_class'  : [
            'me-2 value-selector flex-grow-1',
        ]|join(' '),
    }
) }}
