{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{# The following template doesn't use generic_show_form because of issues with the CommonDBRelation create permission check (canEdit) #}
{% import 'components/form/fields_macros.html.twig' as fields %}
{% import 'components/form/basic_inputs_macros.html.twig' as inputs %}

<div class="asset mb-3">
    <form name="asset_form" method="post" action="{{ 'Calendar_Holiday'|itemtype_form_path }}" enctype="multipart/form-data" data-submit-once>
        <div class="d-flex flex-wrap">
            <div class="col-12 col-xxl-12 flex-column">
                <div class="row flex-row">
                    {{ inputs.hidden('_glpi_csrf_token', csrf_token()) }}
                    {{ inputs.hidden('calendars_id', calendar.getID()) }}
                    {{ fields.dropdownField('Holiday', 'holidays_id', 0, __('Add a close time'), {
                        used: used,
                        display_emptychoice: false,
                        entity: calendar.getEntityID(),
                    }) }}
                </div>
                <div class="d-flex flex-row-reverse">
                    {{ inputs.submit('add', _x('button', 'Add')) }}
                </div>
            </div>
        </div>
    </form>
</div>
