tinymce.addI18n("nl", {
    "Redo": "Opnieuw uitvoeren",
    "Undo": "Ongedaan maken",
    "Cut": "Knippen",
    "Copy": "Kopi\xebren",
    "Paste": "Plakken",
    "Select all": "Alles selecteren",
    "New document": "Nieuw document",
    "Ok": "OK",
    "Cancel": "Annuleren",
    "Visual aids": " Visuele hulpmiddelen",
    "Bold": "Vet",
    "Italic": "Cursief",
    "Underline": "Onderstrepen",
    "Strikethrough": "Doorhalen",
    "Superscript": "Superscript",
    "Subscript": "Subscript",
    "Clear formatting": "Opmaak wissen",
    "Remove": "Verwijderen",
    "Align left": "Links uitlijnen",
    "Align center": "Centreren",
    "Align right": "Rechts uitlijnen",
    "No alignment": "Geen uitlijning",
    "Justify": "Uitvullen",
    "Bullet list": "Lijst met opsommingstekens",
    "Numbered list": "Genummerde lijst",
    "Decrease indent": "Inspringing verkleinen",
    "Increase indent": "Inspringing vergroten",
    "Close": "Sluiten",
    "Formats": "Opmaken",
    "Your browser doesn't support direct access to the clipboard. Please use the Ctrl+X/C/V keyboard shortcuts instead.": "Jouw browser ondersteunt geen rechtstreekse toegang tot het klembord. Gebruik in plaats daarvan de sneltoetsen Ctrl+X/C/V.",
    "Headings": "Koppen",
    "Heading 1": "Kop 1",
    "Heading 2": "Kop 2",
    "Heading 3": "Kop 3",
    "Heading 4": "Kop 4",
    "Heading 5": "Kop 5",
    "Heading 6": "Kop 6",
    "Preformatted": "Vooraf opgemaakt",
    "Div": "Div",
    "Pre": "Pre",
    "Code": "Broncode",
    "Paragraph": "Alinea",
    "Blockquote": "Quote",
    "Inline": "Inline",
    "Blocks": "Blokken",
    "Paste is now in plain text mode. Contents will now be pasted as plain text until you toggle this option off.": "Plakken gebeurt nu zonder opmaak. De tekst zal dus zonder opmaak worden geplakt tenzij je deze optie uit zet.",
    "Fonts": "Lettertypes",
    "Font sizes": "Tekengroottes",
    "Class": "Klasse",
    "Browse for an image": "Afbeelding zoeken",
    "OR": "OF",
    "Drop an image here": "Hier een afbeelding neerzetten",
    "Upload": "Uploaden",
    "Uploading image": "Afbeelding uploaden",
    "Block": "Blok",
    "Align": "Uitlijnen",
    "Default": "Standaard",
    "Circle": "Cirkel",
    "Disc": "Schijf",
    "Square": "Vierkant",
    "Lower Alpha": "Kleine letters",
    "Lower Greek": "Kleine Griekse letters",
    "Lower Roman": "Kleine Romeinse cijfers",
    "Upper Alpha": "Hoofdletters",
    "Upper Roman": "Grote Romeinse cijfers",
    "Anchor...": "Anker...",
    "Anchor": "Anker",
    "Name": "Naam",
    "ID": "ID",
    "ID should start with a letter, followed only by letters, numbers, dashes, dots, colons or underscores.": "ID moet beginnen met een letter en daarna enkel gevolgd door letters, getallen, streepjes, punten, komma's of underscores.",
    "You have unsaved changes are you sure you want to navigate away?": "Je hebt niet alles opgeslagen, weet je zeker dat je de pagina wilt verlaten?",
    "Restore last draft": "Laatste concept herstellen",
    "Special character...": "Speciaal teken...",
    "Special Character": "Speciale tekens",
    "Source code": "Broncode",
    "Insert/Edit code sample": "Codevoorbeeld invoegen/bewerken",
    "Language": "Taal",
    "Code sample...": "Codevoorbeeld...",
    "Left to right": "Van links naar rechts",
    "Right to left": "Van rechts naar links",
    "Title": "Titel",
    "Fullscreen": "Volledig scherm",
    "Action": "Actie",
    "Shortcut": "Snelkoppeling",
    "Help": "Help",
    "Address": "Adres",
    "Focus to menubar": "Focus op menubalk instellen ",
    "Focus to toolbar": "Focus op werkbalk instellen ",
    "Focus to element path": "Focus op elementenpad instellen ",
    "Focus to contextual toolbar": "Focus op contextuele werkbalk instellen ",
    "Insert link (if link plugin activated)": "Link invoegen (als plug-in voor link geactiveerd is)",
    "Save (if save plugin activated)": "Opslaan (als plug-in voor opslaan geactiveerd is)",
    "Find (if searchreplace plugin activated)": "Zoeken (als plug-in voor zoeken/vervangen geactiveerd is)",
    "Plugins installed ({0}):": "Plug-ins ge\xefnstalleerd ({0}):",
    "Premium plugins:": "Premium plug-ins:",
    "Learn more...": "Leer meer...",
    "You are using {0}": "Je gebruikt {0}",
    "Plugins": "Plug-ins",
    "Handy Shortcuts": "Handige snelkoppelingen",
    "Horizontal line": "Horizontale lijn",
    "Insert/edit image": "Afbeelding invoegen/bewerken",
    "Alternative description": "Alternatieve beschrijving",
    "Accessibility": "Toegankelijkheid",
    "Image is decorative": "Afbeelding is decoratief",
    "Source": "Bron",
    "Dimensions": "Afmetingen",
    "Constrain proportions": "Verhoudingen behouden",
    "General": "Algemeen",
    "Advanced": "Geavanceerd",
    "Style": "Stijl",
    "Vertical space": "Verticale ruimte",
    "Horizontal space": "Horizontale ruimte",
    "Border": "Rand",
    "Insert image": "Afbeelding invoegen",
    "Image...": "Afbeelding...",
    "Image list": "Afbeeldingslijst",
    "Resize": "Formaat wijzigen",
    "Insert date/time": "Datum/tijd invoegen",
    "Date/time": "Datum/tijd",
    "Insert/edit link": "Link invoegen/bewerken",
    "Text to display": "Weer te geven tekst",
    "Url": "Url",
    "Open link in...": "Link openen in...",
    "Current window": "Huidige venster",
    "None": "Geen",
    "New window": "Nieuw venster",
    "Open link": "Open koppeling",
    "Remove link": "Link verwijderen",
    "Anchors": "Ankers",
    "Link...": "Link...",
    "Paste or type a link": "Een link plakken of typen",
    "The URL you entered seems to be an email address. Do you want to add the required mailto: prefix?": "De ingevoerde URL lijkt op een e-mailadres. Wil je er het vereiste voorvoegsel mailto: aan toevoegen?",
    "The URL you entered seems to be an external link. Do you want to add the required http:// prefix?": "De ingevoerde URL verwijst naar een extern adres. Wil je er het vereiste voorvoegsel http:// aan toevoegen?",
    "The URL you entered seems to be an external link. Do you want to add the required https:// prefix?": "De ingevoerde URL verwijst naar een extern adres. Wilt u er het vereiste voorvoegsel https:// aan toevoegen?",
    "Link list": "Lijst met links",
    "Insert video": "Video invoegen",
    "Insert/edit video": "Video invoegen/bewerken",
    "Insert/edit media": "Media invoegen/bewerken",
    "Alternative source": "Alternatieve bron",
    "Alternative source URL": "Alternatief bron-URL",
    "Media poster (Image URL)": "Mediaposter (afbeeldings-url)",
    "Paste your embed code below:": "Plak de invoegcode hieronder:",
    "Embed": "Insluiten",
    "Media...": "Media...",
    "Nonbreaking space": "Vaste spatie",
    "Page break": "Pagina-einde",
    "Paste as text": "Plakken als tekst",
    "Preview": "Voorbeeld",
    "Print": "Afdrukken",
    "Print...": "Afdrukken... ",
    "Save": "Opslaan",
    "Find": "Zoeken",
    "Replace with": "Vervangen door",
    "Replace": "Vervangen",
    "Replace all": "Alle vervangen",
    "Previous": "Vorige",
    "Next": "Volgende",
    "Find and Replace": "Zoek en vervang",
    "Find and replace...": "Zoeken en vervangen...",
    "Could not find the specified string.": "Kan opgegeven reeks niet vinden",
    "Match case": "Identieke hoofdletters/kleine letters",
    "Find whole words only": "Alleen hele woorden zoeken",
    "Find in selection": "Zoek in selectie",
    "Insert table": "Tabel invoegen",
    "Table properties": "Tabeleigenschappen",
    "Delete table": "Tabel verwijderen",
    "Cell": "Cel",
    "Row": "Rij",
    "Column": "Kolom",
    "Cell properties": "Celeigenschappen",
    "Merge cells": "Cellen samenvoegen",
    "Split cell": "Cel splitsen",
    "Insert row before": "Rij boven invoegen",
    "Insert row after": "Rij onder invoegen ",
    "Delete row": "Rij verwijderen ",
    "Row properties": "Rijeigenschappen",
    "Cut row": "Rij knippen",
    "Cut column": "Knip kolom",
    "Copy row": "Rij kopi\xebren",
    "Copy column": "Kopieer kolom",
    "Paste row before": "Rij plakken boven",
    "Paste column before": "Plak kolom voor",
    "Paste row after": "Rij plakken onder",
    "Paste column after": "Plak kolom na",
    "Insert column before": "Kolom invoegen voor ",
    "Insert column after": "Kolom invoegen na",
    "Delete column": "Kolom verwijderen",
    "Cols": "Kolommen",
    "Rows": "Rijen",
    "Width": "Breedte",
    "Height": "Hoogte",
    "Cell spacing": "Celafstand",
    "Cell padding": "Celopvulling",
    "Row clipboard actions": "Rij klembord acties",
    "Column clipboard actions": "Kolom klembord acties",
    "Table styles": "Tabel stijlen",
    "Cell styles": "Cel stijlen",
    "Column header": "Kolom kop",
    "Row header": "Rij kop",
    "Table caption": "Tabel bijschrift",
    "Caption": "Onderschrift",
    "Show caption": "Bijschrift weergeven",
    "Left": "Links",
    "Center": "Centreren",
    "Right": "Rechts",
    "Cell type": "Celtype",
    "Scope": "Bereik",
    "Alignment": "Uitlijning",
    "Horizontal align": "Horizontaal uitlijnen",
    "Vertical align": "Verticaal uitlijnen",
    "Top": "Boven",
    "Middle": "Centreren",
    "Bottom": "Onder",
    "Header cell": "Koptekstcel",
    "Row group": "Rijgroep",
    "Column group": "Kolomgroep",
    "Row type": "Rijtype",
    "Header": "Koptekst ",
    "Body": "Body",
    "Footer": "Voettekst",
    "Border color": "Randkleur",
    "Solid": "Massief",
    "Dotted": "Gestippeld",
    "Dashed": "Onderbroken",
    "Double": "Dubbel",
    "Groove": "Groef",
    "Ridge": "Ribbel",
    "Inset": "Inzet",
    "Outset": "Begin",
    "Hidden": "Verborgen",
    "Insert template...": "Sjabloon invoegen...",
    "Templates": "Sjablonen",
    "Template": "Sjabloon",
    "Insert Template": "Sjabloon invoegen",
    "Text color": "Tekstkleur",
    "Background color": "Achtergrondkleur",
    "Custom...": "Aangepast...",
    "Custom color": "Aangepaste kleur",
    "No color": "Geen kleur",
    "Remove color": "Kleur verwijderen",
    "Show blocks": "Blokken weergeven",
    "Show invisible characters": "Onzichtbare tekens weergeven",
    "Word count": "Aantal woorden",
    "Count": "Telling",
    "Document": "Dokument",
    "Selection": "Selectie",
    "Words": "Woorden",
    "Words: {0}": "Woorden: {0}",
    "{0} words": "{0} woorden",
    "File": "Bestand",
    "Edit": "Bewerken",
    "Insert": "Invoegen",
    "View": "Weergeven",
    "Format": "Opmaak",
    "Table": "Tabel",
    "Tools": "Extra",
    "Powered by {0}": "Mogelijk gemaakt met {0}",
    "Rich Text Area. Press ALT-F9 for menu. Press ALT-F10 for toolbar. Press ALT-0 for help": "Gebied met opgemaakte tekst. Druk op ALT-F9 voor menu. Druk op ALT-F10 voor werkbalk. Druk op ALT-0 voor help.",
    "Image title": "Afbeeldingstitel",
    "Border width": "Randbreedte",
    "Border style": "Randstijl",
    "Error": "Fout",
    "Warn": "Waarschuwen",
    "Valid": "Geldig",
    "To open the popup, press Shift+Enter": "Druk op Shift+Enter om de pop-up te openen",
    "Rich Text Area": "Rijk Tekst Gebied",
    "Rich Text Area. Press ALT-0 for help.": " Gebied met opgemaakte tekst. Druk op ALT-0 voor hulp.",
    "System Font": "Systeemlettertype",
    "Failed to upload image: {0}": "Niet gelukt om afbeelding te uploaden: {0}",
    "Failed to load plugin: {0} from url {1}": "Niet gelukt om plug-in te laden: {0} vanaf URL {1}",
    "Failed to load plugin url: {0}": "Niet gelukt om URL plug-in te laden: {0}",
    "Failed to initialize plugin: {0}": "Niet gelukt om plug-in te initialiseren: {0}",
    "example": "voorbeeld",
    "Search": "Zoeken",
    "All": "Alle",
    "Currency": "Valuta",
    "Text": "Tekst",
    "Quotations": "Citaten",
    "Mathematical": "Wiskundig",
    "Extended Latin": "Latijn uitgebreid ",
    "Symbols": "Symbolen",
    "Arrows": "Pijlen",
    "User Defined": "Door gebruiker gedefinieerd ",
    "dollar sign": "dollarteken",
    "currency sign": "valutateken",
    "euro-currency sign": "euroteken",
    "colon sign": "colon-teken",
    "cruzeiro sign": "cruzeiro-teken",
    "french franc sign": "franse franc-teken",
    "lira sign": "lire-teken",
    "mill sign": "mill-teken",
    "naira sign": "naira-teken",
    "peseta sign": "peseta-teken",
    "rupee sign": "roepie-teken",
    "won sign": "won-teken",
    "new sheqel sign": "nieuwe sheqel-teken",
    "dong sign": "dong-teken",
    "kip sign": "kip-teken",
    "tugrik sign": "tugrik-teken",
    "drachma sign": "drachme-teken",
    "german penny symbol": "duitse pfennig-teken",
    "peso sign": "peso-teken",
    "guarani sign": "guarani-teken",
    "austral sign": "austral-teken",
    "hryvnia sign": "hryvnia-teken",
    "cedi sign": "cedi-teken",
    "livre tournois sign": "livre tournois-teken",
    "spesmilo sign": "spesmilo-teken",
    "tenge sign": "tenge-teken",
    "indian rupee sign": "indiaase roepie-teken",
    "turkish lira sign": "turkse lire-teken",
    "nordic mark sign": "noorse mark-teken",
    "manat sign": "manat-teken",
    "ruble sign": "roebel-teken",
    "yen character": "yen-teken",
    "yuan character": "yuan-teken",
    "yuan character, in hong kong and taiwan": "yuan-teken (Hong Kong en Taiwan)",
    "yen/yuan character variant one": "yen/yuan variant 1-teken",
    "Emojis": "Emoji's",
    "Emojis...": "Emoji's...",
    "Loading emojis...": "Emoji's laden...",
    "Could not load emojis": "Kan de emoji's niet laden",
    "People": "Personen",
    "Animals and Nature": "Dieren en natuur",
    "Food and Drink": "Eten en drinken",
    "Activity": "Activiteit",
    "Travel and Places": "Reizen en plaatsen",
    "Objects": "Objecten",
    "Flags": "Vlaggen",
    "Characters": "Tekens",
    "Characters (no spaces)": "Tekens (geen spaties)",
    "{0} characters": "{0} karakters",
    "Error: Form submit field collision.": "Fout: Veldconflict bij versturen formulier.",
    "Error: No form element found.": "Fout: Geen formulierelement gevonden.",
    "Color swatch": "Kleurenwaaier",
    "Color Picker": "Kleurenkiezer",
    "Invalid hex color code: {0}": "Onjuiste hex kleurcode: {0}",
    "Invalid input": "Ongeldige invoer",
    "R": "R",
    "Red component": "Rood component",
    "G": "G",
    "Green component": "Groen component",
    "B": "B",
    "Blue component": "Blauw component",
    "#": "#",
    "Hex color code": "Hex kleurcode",
    "Range 0 to 255": "Bereik 0 tot 255",
    "Turquoise": "Turkoois",
    "Green": "Groen",
    "Blue": "Blauw",
    "Purple": "Paars",
    "Navy Blue": "Marineblauw",
    "Dark Turquoise": "Donkerturquoise",
    "Dark Green": "Donkergroen",
    "Medium Blue": "Middelblauw",
    "Medium Purple": "Middelpaars",
    "Midnight Blue": "Middernachtblauw",
    "Yellow": "Geel",
    "Orange": "Oranje",
    "Red": "Rood",
    "Light Gray": "Lichtgrijs",
    "Gray": "Grijs",
    "Dark Yellow": "Donkergeel",
    "Dark Orange": "Donkeroranje",
    "Dark Red": "Donkerrood",
    "Medium Gray": "Middelgrijs",
    "Dark Gray": "Donkergrijs",
    "Light Green": "Lichtgroen",
    "Light Yellow": "Lichtgeel",
    "Light Red": "Lichtrood",
    "Light Purple": "Lichtpaars",
    "Light Blue": "Lichtblauw",
    "Dark Purple": "Donkerpaars",
    "Dark Blue": "Donkerblauw",
    "Black": "Zwart",
    "White": "Wit",
    "Switch to or from fullscreen mode": "Overschakelen naar of vanuit de volledig scherm-modus",
    "Open help dialog": "Help-scherm openen",
    "history": "geschiedenis",
    "styles": "stijlen",
    "formatting": "opmaak",
    "alignment": "uitlijning",
    "indentation": "inspringing",
    "Font": "Lettertype",
    "Size": "Formaat",
    "More...": "Meer...",
    "Select...": "Selecteer...",
    "Preferences": "Voorkeuren",
    "Yes": "Ja",
    "No": "Nee",
    "Keyboard Navigation": "Toetsenbord navigatie",
    "Version": "Versie",
    "Code view": "Code bekijken",
    "Open popup menu for split buttons": "Open het pop-up menu voor gesplitste  knoppen",
    "List Properties": "Lijsteigenschappen",
    "List properties...": "Lijsteigenschappen...",
    "Start list at number": "Begin lijst met nummer",
    "Line height": "Lijnhoogte",
    "Dropped file type is not supported": "Gesleepte bestandstype wordt niet ondersteund",
    "Loading...": "Laden...",
    "ImageProxy HTTP error: Rejected request": "ImageProxy HTTP fout: Aanvraag afgewezen",
    "ImageProxy HTTP error: Could not find Image Proxy": "ImageProxy HTTP fout: Kan geen Image Proxy vinden",
    "ImageProxy HTTP error: Incorrect Image Proxy URL": "ImageProxy HTTP fout: Incorrecte Image proxy URL",
    "ImageProxy HTTP error: Unknown ImageProxy error": "ImageProxy HTTP fout: Onbekende ImageProxy fout"
});