<?php

/**
 * SCSSPHP
 *
 * @copyright 2012-2020 Leaf Corcoran
 *
 * @license http://opensource.org/licenses/MIT MIT
 *
 * @link http://scssphp.github.io/scssphp
 */

namespace ScssPhp\ScssPhp\Ast\Css;

/**
 * A plain CSS stylesheet.
 *
 * This is the root plain CSS node. It contains top-level statements.
 *
 * @internal
 */
interface CssStylesheet extends CssParentNode
{
}
