/*!
 * ---------------------------------------------------------------------
 *
 * GLPI - Gestionnaire Libre de Parc Informatique
 *
 * http://glpi-project.org
 *
 * @copyright 2015-2026 Teclib' and contributors.
 * @copyright 2003-2014 by the INDEPNET Development Team.
 * @licence   https://www.gnu.org/licenses/gpl-3.0.html
 *
 * ---------------------------------------------------------------------
 *
 * LICENSE
 *
 * This file is part of GLPI.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------
 */

.itil-footer {
    position: sticky;
    bottom: 0;

    @include media-breakpoint-down(sm) {
        position: fixed;
        left: 0;
        right: 0;
        bottom: 8px;
        z-index: var(--glpi-zindex-fixed);
    }

    .answer-action {
        &:hover {
            font-weight: bold;
        }
    }

    .action-task {
        background-color: var(--glpi-timeline-task-bg);
        color: var(--glpi-timeline-task-fg);
    }

    .action-solution {
        background-color: var(--glpi-timeline-solution-bg);
        color: var(--glpi-timeline-solution-fg);
    }

    .action-validation {
        background-color: var(--glpi-timeline-itil-content-bg);
        color: var(--glpi-timeline-itil-content-fg);
    }

    .action-document {
        background-color: var(--glpi-timeline-document-bg);
        color: var(--glpi-timeline-document-fg);
    }

    .buttons-bar {
        background-color: var(--glpi-itil-secondary-bg);

        .form-buttons {
            background-color: inherit;
            border-left: 1px solid var(--tblr-border-color);

            @include media-breakpoint-down(md) {
                border-left: 0 !important;
            }
        }
    }
}
