<?php declare(strict_types=1);

namespace GraphQL\Language\AST;

/**
 * export type ValueNode = VariableNode
 * | NullValueNode
 * | IntValueNode
 * | FloatValueNode
 * | StringValueNode
 * | BooleanValueNode
 * | EnumValueNode
 * | ListValueNode
 * | ObjectValueNode.
 */
interface ValueNode {}
