{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% set superheader_row %}
    <div class="relative">
        <span>{{ title|raw }}</span>
        <span class="float-end">
            {% if add_link is not empty %}
                <a href="{{ add_link }}" title="{{ __('Add') }}">
                    <i class="ti ti-plus"></i>
                </a>
            {% endif %}
        </span>
    </div>
{% endset %}

{% set superheader_colspan = rows is not empty ? rows[0]['values']|length : 1 %}

{% include('components/table.html.twig') with {
    class: 'table table-striped card-table table-hover',
    header_rows: [
        [{
            colspan: superheader_colspan,
            content: superheader_row,
        }]
    ]|merge(header_rows|default([])),
    rows: rows,
} only %}
