{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% import 'components/form/fields_macros.html.twig' as fields %}

<div class="row mx-n2">
   {{ fields.largeTitle(__('Support hours'), 'ti ti-calendar-clock') }}
   {{ fields.smallTitle(__('On week')) }}
   {{ fields.nullField({'field_class': 'col-12 col-sm-4'}) }}
   {{ fields.dropdownHoursField('week_begin_hour', item.fields['week_begin_hour'], __('Start'), {'field_class': 'col-12 col-sm-4'}) }}
   {{ fields.dropdownHoursField('week_end_hour', item.fields['week_end_hour'], __('End'), {'field_class': 'col-12 col-sm-4'}) }}

   {{ fields.smallTitle(__('On Saturday')) }}
   {{ fields.dropdownYesNo('use_saturday', item.fields['use_saturday'], __('Use Saturday'), {'field_class': 'col-12 col-sm-4'}) }}
   {{ fields.dropdownHoursField('saturday_begin_hour', item.fields['saturday_begin_hour'], __('Start'), {'field_class': 'col-12 col-sm-4'}) }}
   {{ fields.dropdownHoursField('saturday_end_hour', item.fields['saturday_end_hour'], __('End'), {'field_class': 'col-12 col-sm-4'}) }}

   {{ fields.smallTitle(__('Sundays and holidays')) }}
   {{ fields.dropdownYesNo('use_sunday', item.fields['use_sunday'], __('Use Sunday'), {'field_class': 'col-12 col-sm-4'}) }}
   {{ fields.dropdownHoursField('sunday_begin_hour', item.fields['sunday_begin_hour'], __('Start'), {'field_class': 'col-12 col-sm-4'}) }}
   {{ fields.dropdownHoursField('sunday_end_hour', item.fields['sunday_end_hour'], __('End'), {'field_class': 'col-12 col-sm-4'}) }}
</div>
