{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% if is_impersonate_active %}
<div class="banner-impersonate">
   <form name="form" method="post" action="{{ 'User'|itemtype_form_path }}" data-submit-once>
      {{ __('You are impersonating %s.')|format(get_item_name(user)) }}
      <input type="hidden" name="_glpi_csrf_token" value="{{ csrf_token() }}" />
      <button type="submit" name="impersonate" class="btn btn-outline-danger" value="0">
         {{ __('Stop impersonating') }}
      </button>
   </form>
</div>
{% endif %}
