{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% import 'components/form/fields_macros.html.twig' as fields %}

<div class="mb-3">
    <form method="post" action="{{ 'Contact_Supplier'|itemtype_form_path }}" data-submit-once>
        {{ fields.csrfField() }}
        <div class="d-flex">
            {% if get_class(peer) == 'Contact' %}
                <input type="hidden" name="contacts_id" value="{{ peer.getID() }}">
                {{ fields.dropdownField('Supplier', 'suppliers_id', '', __('Add a supplier'), {
                    used: used,
                    entity: peer.getEntityID(),
                    entity_sons: peer.isRecursive(),
                }) }}
            {% else %}
                <input type="hidden" name="suppliers_id" value="{{ peer.getID() }}">
                {{ fields.dropdownField('Contact', 'contacts_id', '', __('Add a contact'), {
                    used: used,
                    entity: peer.getEntityID(),
                    entity_sons: peer.isRecursive(),
                }) }}
            {% endif %}
            <div>
                <button type="submit" name="add" class="btn btn-primary ms-3">{{ _x('button', 'Add') }}</button>
            </div>
        </div>
    </form>
</div>
