{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% import 'components/form/fields_macros.html.twig' as fields %}

{% if not is_db_consistent %}
    <div class="alert alert-important alert-danger my-2 mx-4" role="alert">
        <i class="fas fa-2x fa-exclamation-triangle align-middle"></i>
        {{ __('The database schema is not consistent with the current GLPI version.') }}<br />
        {{ __('It is recommended to run the "%s" command to see the differences.')|format('php bin/console database:check_schema_integrity') }}
    </div>
{% endif %}

<h2>{{ __("One last thing before starting") }}</h2>

<p>
   {{ glpinetwork|raw }}
</p>

{% if not telemetry_enabled %}
   <hr>
   <p>
      {{ telemetry_info|raw }}
   </p>
{% endif %}

<hr>
<p>
   {{ reference_info|raw }}
</p>

<hr>
<form action="update.php" method="post" data-submit-once>
   <div class="text-center">
      <button type="submit" class="btn btn-primary">
         <i class="fas fa-thumbs-up me-1"></i>
         {{ __("Use GLPI") }}
      </button>
   </div>

   <input type="hidden" name="update_end" value="1">
   {{ fields.csrfField() }}
</form>
