{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% extends "generic_show_form.html.twig" %}
{% import 'components/form/fields_macros.html.twig' as fields %}

{% block form_fields %}
    {{ fields.textField('name', item.fields['name'], __('Name'), {
        full_width: true,
    }) }}
    {{ fields.dropdownItemTypes('itemtypes', null, _n('Type', 'Types', get_plural_number()), {
        types: config('link_types'),
        multiple: true,
        values: item.getLinkedItemtypes,
        full_width: true,
    }) }}
    {{ fields.codeField('link', item.fields['link'], __('Link or filename'), {
        single_line: true,
        language: 'twig',
        completions: tag_options|default([]),
        full_width: true,
    }) }}
    {{ fields.dropdownYesNo('open_window', item.fields['open_window'], __('Open in a new window'), {
        full_width: true,
    }) }}
    {{ fields.codeField('data', item.fields['data'], __('File content'), {
        language: 'twig',
        completions: tag_options|default([]),
        full_width: true,
        height: '300px'
    }) }}
{% endblock %}
