/*!
 * ---------------------------------------------------------------------
 *
 * GLPI - Gestionnaire Libre de Parc Informatique
 *
 * http://glpi-project.org
 *
 * @copyright 2015-2026 Teclib' and contributors.
 * @copyright 2003-2014 by the INDEPNET Development Team.
 * @licence   https://www.gnu.org/licenses/gpl-3.0.html
 *
 * ---------------------------------------------------------------------
 *
 * LICENSE
 *
 * This file is part of GLPI.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------
 */

#switchview {
    font-size: 2em;
    text-align: right;
    float: right;

    button {
        &.selected {
            color: green;
        }
    }
}

#viewlist {
    display: none;
}

#viewgraph {
    width: calc(100% - 80px);
    overflow-x: auto;

    table {
        width: 100%;
        table-layout: fixed;
        border-collapse: collapse;
    }

    .tipcontent {
        display: none;
    }

    th,
    td {
        height: 2em;
        text-align: center;
        padding: 0;
    }

    .mini_toggle {
        font-size: 11px;
        opacity: 0.5;
        cursor: pointer;
        position: relative;
        padding-left: 20px;
        margin: 15px 5px;
        float: left;

        &::before {
            content: "\f204";
            left: 0;
            top: 0;
            font: var(--fa-font-solid);
            font-size: 15px;
            position: absolute;
        }

        &.active::before {
            content: "\f205";
            font-weight: 900;
        }

        &:hover {
            opacity: 1;
        }
    }
}
