/*!
 * ---------------------------------------------------------------------
 *
 * GLPI - Gestionnaire Libre de Parc Informatique
 *
 * http://glpi-project.org
 *
 * @copyright 2015-2026 Teclib' and contributors.
 * @copyright 2003-2014 by the INDEPNET Development Team.
 * @licence   https://www.gnu.org/licenses/gpl-3.0.html
 *
 * ---------------------------------------------------------------------
 *
 * LICENSE
 *
 * This file is part of GLPI.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------
 */

.ct-series.ct-series-p {
    fill: #c2f040;
    stroke: #c2f040;
}

.ct-legend .ct-series-15::before {
    background-color: #c2f040;
    border-color: #c2f040;
}

.ct-series.ct-series-q {
    fill: #ca12d3;
    stroke: #ca12d3;
}

.ct-legend .ct-series-16::before {
    background-color: #ca12d3;
    border-color: #ca12d3;
}

.ct-series.ct-series-r {
    fill: #f75037;
    stroke: #f75037;
}

.ct-legend .ct-series-17::before {
    background-color: #f75037;
    border-color: #f75037;
}

.ct-series.ct-series-s {
    fill: #6224af;
    stroke: #6224af;
}

.ct-legend .ct-series-18::before {
    background-color: #6224af;
    border-color: #6224af;
}

.ct-series.ct-series-t {
    fill: #854f95;
    stroke: #854f95;
}

.ct-legend .ct-series-19::before {
    background-color: #854f95;
    border-color: #854f95;
}

.ct-series.ct-series-u {
    fill: #fcfefd;
    stroke: #fcfefd;
}

.ct-legend .ct-series-20::before {
    background-color: #fcfefd;
    border-color: #fcfefd;
}

.ct-series.ct-series-v {
    fill: #1739c0;
    stroke: #1739c0;
}

.ct-legend .ct-series-21::before {
    background-color: #1739c0;
    border-color: #1739c0;
}

.ct-series.ct-series-w {
    fill: #f83db1;
    stroke: #f83db1;
}

.ct-legend .ct-series-22::before {
    background-color: #f83db1;
    border-color: #f83db1;
}

.ct-series.ct-series-x {
    fill: #a52ab5;
    stroke: #a52ab5;
}

.ct-legend .ct-series-23::before {
    background-color: #a52ab5;
    border-color: #a52ab5;
}

.ct-series.ct-series-y {
    fill: #1f8870;
    stroke: #1f8870;
}

.ct-legend .ct-series-24::before {
    background-color: #1f8870;
    border-color: #1f8870;
}

.ct-series.ct-series-z {
    fill: #d7192e;
    stroke: #d7192e;
}

.ct-legend .ct-series-25::before {
    background-color: #d7192e;
    border-color: #d7192e;
}

/** Legends **/
.ct-legend {
    position: relative;
    z-index: 10;
    list-style: none;
    text-align: center;
}

.ct-legend li {
    position: relative;
    padding-left: 23px;
    margin-right: 10px;
    margin-bottom: 3px;
    cursor: pointer;
    display: inline-block;
}

.ct-legend li::before {
    width: 12px;
    height: 12px;
    position: absolute;
    left: 0;
    content: "";
    border: 3px solid transparent;
    border-radius: 2px;
}

.ct-legend li.inactive::before {
    background: transparent;
}

.ct-legend .ct-series-0::before {
    background-color: #d70206;
    border-color: #d70206;
}

.ct-legend .ct-series-1::before {
    background-color: #f05b4f;
    border-color: #f05b4f;
}

.ct-legend .ct-series-2::before {
    background-color: #f4c63d;
    border-color: #f4c63d;
}

.ct-legend .ct-series-3::before {
    background-color: #d17905;
    border-color: #d17905;
}

.ct-legend .ct-series-4::before {
    background-color: #453d3f;
    border-color: #453d3f;
}

.ct-legend .ct-series-5::before {
    background-color: #59922b;
    border-color: #59922b;
}

.ct-legend .ct-series-6::before {
    background-color: #0544d3;
    border-color: #0544d3;
}

.ct-legend .ct-series-7::before {
    background-color: #6b0392;
    border-color: #6b0392;
}

.ct-legend .ct-series-8::before {
    background-color: #f05b4f;
    border-color: #f05b4f;
}

.ct-legend .ct-series-9::before {
    background-color: #dda458;
    border-color: #dda458;
}

.ct-legend .ct-series-10::before {
    background-color: #eacf7d;
    border-color: #eacf7d;
}

.ct-legend .ct-series-11::before {
    background-color: #86797d;
    border-color: #86797d;
}

.ct-legend .ct-series-12::before {
    background-color: #b2c326;
    border-color: #b2c326;
}

.ct-legend .ct-series-13::before {
    background-color: #6188e2;
    border-color: #6188e2;
}

.ct-legend .ct-series-14::before {
    background-color: #a748ca;
    border-color: #a748ca;
}
