tinymce.addI18n("et", {
    "Redo": "Tee uuesti",
    "Undo": "V\xf5ta tagasi",
    "Cut": "L\xf5ika",
    "Copy": "Kopeeri",
    "Paste": "Kleebi",
    "Select all": "Vali k\xf5ik",
    "New document": "Uus dokument",
    "Ok": "",
    "Cancel": "Katkesta",
    "Visual aids": "N\xe4itevahendid",
    "Bold": "Rasvane",
    "Italic": "Kaldkiri",
    "Underline": "Allakriipsutatud",
    "Strikethrough": "L\xe4bikriipsutatud",
    "Superscript": "\xdclaindeks",
    "Subscript": "Alaindeks",
    "Clear formatting": "Puhasta vorming",
    "Remove": "",
    "Align left": "Joonda vasakule",
    "Align center": "Joonda keskele",
    "Align right": "Joonda paremale",
    "No alignment": "",
    "Justify": "Joonda r\xf6\xf6pselt",
    "Bullet list": "J\xe4rjestamata loend",
    "Numbered list": "J\xe4rjestatud loend",
    "Decrease indent": "V\xe4henda taanet",
    "Increase indent": "Suurenda taanet",
    "Close": "Sulge",
    "Formats": "Vormingud",
    "Your browser doesn't support direct access to the clipboard. Please use the Ctrl+X/C/V keyboard shortcuts instead.": "Sinu veebilehitseja ei toeta otsest ligip\xe4\xe4su l\xf5ikelauale. Palun kasuta selle asemel klaviatuuri kiirk\xe4sklusi Ctrl+X/C/V.",
    "Headings": "Pealkirjad",
    "Heading 1": "Pealkiri 1",
    "Heading 2": "Pealkiri 2",
    "Heading 3": "Pealkiri 3",
    "Heading 4": "Pealkiri 4",
    "Heading 5": "Pealkiri 5",
    "Heading 6": "Pealkiri 6",
    "Preformatted": "Eelvormindaud",
    "Div": "Sektsioon",
    "Pre": "Eelvormindatud",
    "Code": "Kood",
    "Paragraph": "L\xf5ik",
    "Blockquote": "Plokktsitaat",
    "Inline": "Reasisene",
    "Blocks": "Plokid",
    "Paste is now in plain text mode. Contents will now be pasted as plain text until you toggle this option off.": "Kleepimine on n\xfc\xfcd lihtteksti re\u017eiimis. Sisu kleebitakse lihttekstina seni, kuni Sa selle valiku v\xe4lja l\xfclitad.",
    "Fonts": "Fondid",
    "Font sizes": "Fondi suurused",
    "Class": "Klass",
    "Browse for an image": "Sirvi pilte",
    "OR": "V\xd5I",
    "Drop an image here": "Kukuta pilt siia",
    "Upload": "\xdcles laadimine",
    "Uploading image": "",
    "Block": "Plokk",
    "Align": "Joonda",
    "Default": "Vaikimisi",
    "Circle": "Ring",
    "Disc": "Plaat",
    "Square": "Ruut",
    "Lower Alpha": "Alam-Alfa",
    "Lower Greek": "Alamkreeka keel",
    "Lower Roman": "Alam-Rooma",
    "Upper Alpha": "\xdclem-Alfa",
    "Upper Roman": "\xdclemine Rooma keel",
    "Anchor...": "Ankur ...",
    "Anchor": "",
    "Name": "Nimi",
    "ID": "",
    "ID should start with a letter, followed only by letters, numbers, dashes, dots, colons or underscores.": "",
    "You have unsaved changes are you sure you want to navigate away?": "Teil on salvestamata muudatusi. Kas soovite kindlasti minema navigeerida?",
    "Restore last draft": "Taasta viimane mustand",
    "Special character...": "Eriline tegelane ...",
    "Special Character": "",
    "Source code": "L\xe4htekood",
    "Insert/Edit code sample": "Koodin\xe4idise sisestamine / muutmine",
    "Language": "Keel",
    "Code sample...": "Koodin\xe4ide ...",
    "Left to right": "Vasakult paremale",
    "Right to left": "Paremalt vasakule",
    "Title": "Pealkiri",
    "Fullscreen": "T\xe4isekraan",
    "Action": "Tegevus",
    "Shortcut": "Otsetee",
    "Help": "Abi",
    "Address": "Aadress",
    "Focus to menubar": "Keskenduge men\xfc\xfcribale",
    "Focus to toolbar": "Keskenduge t\xf6\xf6riistaribale",
    "Focus to element path": "Keskendumine elemendi rajale",
    "Focus to contextual toolbar": "Keskenduge kontekstip\xf5hisele t\xf6\xf6riistaribale",
    "Insert link (if link plugin activated)": "Sisesta link (kui lingi pistikprogramm on aktiveeritud)",
    "Save (if save plugin activated)": "Salvesta (kui pistikprogrammi salvestamine on aktiveeritud)",
    "Find (if searchreplace plugin activated)": "Leia (kui otsinguprogrammi plugin on aktiveeritud)",
    "Plugins installed ({0}):": "Installitud pistikprogrammid ({0}):",
    "Premium plugins:": "Premium pistikprogrammid:",
    "Learn more...": "Lisateave ...",
    "You are using {0}": "Kasutate seadet {0}",
    "Plugins": "Pistikprogrammid",
    "Handy Shortcuts": "K\xe4ep\xe4rased otseteed",
    "Horizontal line": "Horisontaalne joon",
    "Insert/edit image": "Sisesta / muuda pilti",
    "Alternative description": "",
    "Accessibility": "",
    "Image is decorative": "",
    "Source": "Allikas",
    "Dimensions": "M\xf5\xf5tmed",
    "Constrain proportions": "Piirake proportsioone",
    "General": "Kindral",
    "Advanced": "T\xe4psem",
    "Style": "Stiil",
    "Vertical space": "Vertikaalne ruum",
    "Horizontal space": "Horisontaalne ruum",
    "Border": "Piir",
    "Insert image": "Sisesta pilt",
    "Image...": "Pilt ...",
    "Image list": "Piltide loend",
    "Resize": "Suuruse muutmine",
    "Insert date/time": "Sisestage kuup\xe4ev / kellaaeg",
    "Date/time": "Kuup\xe4ev Kellaaeg",
    "Insert/edit link": "Sisesta / muuda link",
    "Text to display": "Kuvatav tekst",
    "Url": "URL",
    "Open link in...": "Ava link saidil ...",
    "Current window": "Praegune aken",
    "None": "Puudub",
    "New window": "Uus aken",
    "Open link": "",
    "Remove link": "Eemalda link",
    "Anchors": "Ankrud",
    "Link...": "",
    "Paste or type a link": "Kleepige v\xf5i tippige link",
    "The URL you entered seems to be an email address. Do you want to add the required mailto: prefix?": "Sisestatud URL n\xe4ib olevat e-posti aadress. Kas soovite lisadavajalik mailto: eesliide?",
    "The URL you entered seems to be an external link. Do you want to add the required http:// prefix?": "Teie sisestatud URL n\xe4ib olevat v\xe4line link. Kas soovite lisadavajalik http: // eesliide?",
    "The URL you entered seems to be an external link. Do you want to add the required https:// prefix?": "",
    "Link list": "Lingiloend",
    "Insert video": "Sisesta video",
    "Insert/edit video": "Video lisamine / muutmine",
    "Insert/edit media": "Sisestage / muutke meediumit",
    "Alternative source": "Alternatiivne allikas",
    "Alternative source URL": "Alternatiivne allika URL",
    "Media poster (Image URL)": "Meediaplakat (pildi URL)",
    "Paste your embed code below:": "Kleepige oma manustamiskood allpool:",
    "Embed": "",
    "Media...": "Meedia ...",
    "Nonbreaking space": "Murdmatu ruum",
    "Page break": "Lehek\xfcljevahe",
    "Paste as text": "Kleebi tekstina",
    "Preview": "Eelvaade",
    "Print": "",
    "Print...": "Prindi ...",
    "Save": "Salvesta",
    "Find": "Leidke",
    "Replace with": "Asenda s\xf5naga",
    "Replace": "Asenda",
    "Replace all": "Asenda k\xf5ik",
    "Previous": "Eelmine",
    "Next": "J\xe4rgmine",
    "Find and Replace": "",
    "Find and replace...": "Leidke ja asendage ...",
    "Could not find the specified string.": "M\xe4\xe4ratud stringi ei leitud.",
    "Match case": "Tikutoos",
    "Find whole words only": "Leidke ainult terved s\xf5nad",
    "Find in selection": "",
    "Insert table": "Sisesta tabel",
    "Table properties": "Tabeli omadused",
    "Delete table": "Kustuta tabel",
    "Cell": "Kamber",
    "Row": "Rida",
    "Column": "Veerg",
    "Cell properties": "Rakkude omadused",
    "Merge cells": "Lahtrite \xfchendamine",
    "Split cell": "Lahtrite jagamine",
    "Insert row before": "Sisestage rida enne",
    "Insert row after": "Lisage rida p\xe4rast",
    "Delete row": "Kustuta rida",
    "Row properties": "Rea omadused",
    "Cut row": "L\xf5ika rida",
    "Cut column": "",
    "Copy row": "Kopeeri rida",
    "Copy column": "",
    "Paste row before": "Kleepige rida enne",
    "Paste column before": "",
    "Paste row after": "Kleepige rida p\xe4rast",
    "Paste column after": "",
    "Insert column before": "Sisestage veerg enne",
    "Insert column after": "Sisestage veerg p\xe4rast",
    "Delete column": "Kustuta veerg",
    "Cols": "",
    "Rows": "Read",
    "Width": "Laius",
    "Height": "K\xf5rgus",
    "Cell spacing": "Lahtrite vahe",
    "Cell padding": "Lahtrite polsterdus",
    "Row clipboard actions": "",
    "Column clipboard actions": "",
    "Table styles": "",
    "Cell styles": "",
    "Column header": "",
    "Row header": "",
    "Table caption": "",
    "Caption": "Alapealkiri",
    "Show caption": "Kuva pealdis",
    "Left": "Vasakule",
    "Center": "Keskus",
    "Right": "\xd5ige",
    "Cell type": "Lahtri t\xfc\xfcp",
    "Scope": "Reguleerimisala",
    "Alignment": "Joondamine",
    "Horizontal align": "",
    "Vertical align": "",
    "Top": "\xdcles",
    "Middle": "Keskmine",
    "Bottom": "Alumine",
    "Header cell": "P\xe4ise lahter",
    "Row group": "R\xfchma r\xfchm",
    "Column group": "Veergude r\xfchm",
    "Row type": "Rea t\xfc\xfcp",
    "Header": "P\xe4is",
    "Body": "Keha",
    "Footer": "Jalus",
    "Border color": "\xc4\xe4rise v\xe4rv",
    "Solid": "",
    "Dotted": "",
    "Dashed": "",
    "Double": "",
    "Groove": "",
    "Ridge": "",
    "Inset": "",
    "Outset": "",
    "Hidden": "",
    "Insert template...": "Sisesta mall ...",
    "Templates": "Mallid",
    "Template": "Mall",
    "Insert Template": "",
    "Text color": "Teksti v\xe4rv",
    "Background color": "Taustav\xe4rv",
    "Custom...": "Kohandatud ...",
    "Custom color": "Kohandatud v\xe4rv",
    "No color": "Pole v\xe4rvi",
    "Remove color": "Eemaldage v\xe4rv",
    "Show blocks": "N\xe4ita plokke",
    "Show invisible characters": "Kuva n\xe4htamatud t\xe4hem\xe4rgid",
    "Word count": "S\xf5nade arv",
    "Count": "Krahv",
    "Document": "Dokument",
    "Selection": "Valik",
    "Words": "S\xf5nad",
    "Words: {0}": "S\xf5nad: {0}",
    "{0} words": "{0} s\xf5na",
    "File": "Fail",
    "Edit": "Muuda",
    "Insert": "Sisesta",
    "View": "Vaade",
    "Format": "Vormindus",
    "Table": "Tabel",
    "Tools": "T\xf6\xf6riistad",
    "Powered by {0}": "Toetaja: {0}",
    "Rich Text Area. Press ALT-F9 for menu. Press ALT-F10 for toolbar. Press ALT-0 for help": "Rikasteksti ala. Men\xfc\xfc avamiseks vajutage ALT-F9. T\xf6\xf6riistariba saamiseks vajutage ALT-F10. VajutageALT-0 abi saamiseks",
    "Image title": "Pildi pealkiri",
    "Border width": "\xc4\xe4rise laius",
    "Border style": "Piiri stiil",
    "Error": "Viga",
    "Warn": "Hoiatage",
    "Valid": "Kehtib",
    "To open the popup, press Shift+Enter": "H\xfcpikakna avamiseks vajutage t\xf5stuklahvi + sisestusklahvi",
    "Rich Text Area": "",
    "Rich Text Area. Press ALT-0 for help.": "Rikasteksti ala. Abi saamiseks vajutage ALT-0.",
    "System Font": "S\xfcsteemi font",
    "Failed to upload image: {0}": "Pildi \xfcleslaadimine eba\xf5nnestus: {0}",
    "Failed to load plugin: {0} from url {1}": "Pistikprogrammi laadimine nurjus: {0} URL-ist {1}",
    "Failed to load plugin url: {0}": "Pistikprogrammi URL-i laadimine nurjus: {0}",
    "Failed to initialize plugin: {0}": "Pistikprogrammi l\xe4htestamine eba\xf5nnestus: {0}",
    "example": "n\xe4ide",
    "Search": "Otsing",
    "All": "K\xf5ik",
    "Currency": "Valuuta",
    "Text": "Tekst",
    "Quotations": "Tsitaadid",
    "Mathematical": "Matemaatiline",
    "Extended Latin": "Laiendatud ladina keel",
    "Symbols": "S\xfcmbolid",
    "Arrows": "Nooled",
    "User Defined": "Kasutaja m\xe4\xe4ratud",
    "dollar sign": "dollari m\xe4rk",
    "currency sign": "v\xe4\xe4ringu m\xe4rk",
    "euro-currency sign": "euro v\xe4\xe4ringu m\xe4rk",
    "colon sign": "j\xe4mesoole m\xe4rk",
    "cruzeiro sign": "cruzeiro m\xe4rk",
    "french franc sign": "Prantsuse frangi m\xe4rk",
    "lira sign": "liiri m\xe4rk",
    "mill sign": "veskim\xe4rk",
    "naira sign": "naira m\xe4rk",
    "peseta sign": "peseta m\xe4rk",
    "rupee sign": "ruupia m\xe4rk",
    "won sign": "v\xf5itis m\xe4rgi",
    "new sheqel sign": "uus \u0161ekeli m\xe4rk",
    "dong sign": "dongi m\xe4rk",
    "kip sign": "kip m\xe4rk",
    "tugrik sign": "tugriku m\xe4rk",
    "drachma sign": "drahmam\xe4rk",
    "german penny symbol": "saksa senti s\xfcmbol",
    "peso sign": "peeso m\xe4rk",
    "guarani sign": "guarani m\xe4rk",
    "austral sign": "austraalne m\xe4rk",
    "hryvnia sign": "grivna m\xe4rk",
    "cedi sign": "cedi m\xe4rk",
    "livre tournois sign": "elav turniiri m\xe4rk",
    "spesmilo sign": "spesmilo m\xe4rk",
    "tenge sign": "tenge m\xe4rk",
    "indian rupee sign": "India ruupia m\xe4rk",
    "turkish lira sign": "t\xfcrgi liiri m\xe4rk",
    "nordic mark sign": "p\xf5hjam\xe4rgi m\xe4rk",
    "manat sign": "manati m\xe4rk",
    "ruble sign": "rubla m\xe4rk",
    "yen character": "jeeni tegelaskuju",
    "yuan character": "j\xfcaani tegelaskuju",
    "yuan character, in hong kong and taiwan": "j\xfcaani tegelane, Hongkongis ja Taiwanis",
    "yen/yuan character variant one": "jeeni / j\xfcaani t\xe4hem\xe4rgi variant \xfcks",
    "Emojis": "",
    "Emojis...": "",
    "Loading emojis...": "",
    "Could not load emojis": "",
    "People": "Inimesed",
    "Animals and Nature": "Loomad ja loodus",
    "Food and Drink": "Toit ja jook",
    "Activity": "Tegevus",
    "Travel and Places": "Reisimine ja kohad",
    "Objects": "Objektid",
    "Flags": "Lipud",
    "Characters": "Tegelased",
    "Characters (no spaces)": "M\xe4rgid (t\xfchikud puuduvad)",
    "{0} characters": "{0} t\xe4hem\xe4rki",
    "Error: Form submit field collision.": "Viga: vormi esitamise v\xe4lja kokkup\xf5rge.",
    "Error: No form element found.": "Viga: vormielementi ei leitud.",
    "Color swatch": "V\xe4rvivalik",
    "Color Picker": "V\xe4rvivalija",
    "Invalid hex color code: {0}": "",
    "Invalid input": "",
    "R": "",
    "Red component": "",
    "G": "",
    "Green component": "",
    "B": "",
    "Blue component": "",
    "#": "",
    "Hex color code": "",
    "Range 0 to 255": "",
    "Turquoise": "T\xfcrkiis",
    "Green": "Roheline",
    "Blue": "Sinine",
    "Purple": "Lilla",
    "Navy Blue": "Merev\xe4e sinine",
    "Dark Turquoise": "Tume t\xfcrkiissinine",
    "Dark Green": "Tumeroheline",
    "Medium Blue": "Keskmine sinine",
    "Medium Purple": "Keskmine lilla",
    "Midnight Blue": "Kesk\xf6ine sinine",
    "Yellow": "Kollane",
    "Orange": "Oran\u017e",
    "Red": "Punane",
    "Light Gray": "Helehall",
    "Gray": "Hall",
    "Dark Yellow": "Tumekollane",
    "Dark Orange": "Tumeoran\u017e",
    "Dark Red": "Tumepunane",
    "Medium Gray": "Keskmiselt hall",
    "Dark Gray": "Tumehall",
    "Light Green": "Heleroheline",
    "Light Yellow": "Helekollane",
    "Light Red": "Helepunane",
    "Light Purple": "Helelilla",
    "Light Blue": "Helesinine",
    "Dark Purple": "Tumelilla",
    "Dark Blue": "Tumesinine",
    "Black": "Must",
    "White": "Valge",
    "Switch to or from fullscreen mode": "L\xfclitumine t\xe4isekraanre\u017eiimile v\xf5i sellest v\xe4lja",
    "Open help dialog": "Ava dialoog",
    "history": "ajalugu",
    "styles": "stiilid",
    "formatting": "vormindamine",
    "alignment": "joondamine",
    "indentation": "taane",
    "Font": "",
    "Size": "Suurus",
    "More...": "Veel ...",
    "Select...": "Valige ...",
    "Preferences": "Eelistused",
    "Yes": "Jah",
    "No": "Ei",
    "Keyboard Navigation": "Klaviatuuril navigeerimine",
    "Version": "Versioon",
    "Code view": "",
    "Open popup menu for split buttons": "",
    "List Properties": "",
    "List properties...": "",
    "Start list at number": "",
    "Line height": "Reak\xf5rgus",
    "Dropped file type is not supported": "",
    "Loading...": "",
    "ImageProxy HTTP error: Rejected request": "",
    "ImageProxy HTTP error: Could not find Image Proxy": "",
    "ImageProxy HTTP error: Incorrect Image Proxy URL": "",
    "ImageProxy HTTP error: Unknown ImageProxy error": ""
});