{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% set unique_dom_id = 'service-catalog-tree-' ~ random() %}

<div class="col-12 col-sm-6 col-md-4 d-flex">
    <a
        class="card mx-2 my-2 flex-grow-1 border-0 rounded-4 hover-no-shadow hover-bg"
        {% if child is instanceof("Glpi\\Form\\ServiceCatalog\\ServiceCatalogLeafInterface") %}
            href="{{ child.getServiceCatalogLink() }}"
        {% elseif child is instanceof("Glpi\\Form\\ServiceCatalog\\ServiceCatalogCompositeInterface") %}
            href="?{{ child.getChildrenUrlParameters() }}"
            data-children-url-parameters="{{ child.getChildrenUrlParameters() }}"
            data-composite-item
        {% endif %}
    >
        <section
            class="card-body"
            aria-labelledby="{{ unique_dom_id }}"
        >
            <div class="d-flex">
                {{ render_illustration(
                    child.getServiceCatalogItemIllustration() ?? 'report-issue.svg',
                    100
                ) }}
                <div class="ms-4">
                    <h2
                        id="{{ unique_dom_id }}"
                        class="card-title mb-2 text-break"
                    >
                        {{ child.getServiceCatalogItemTitle() }}
                    </h2>
                    <div class="text-secondary remove-last-tinymce-margin">
                        {{ child.getServiceCatalogItemDescription()|safe_html }}
                    </div>
                </div>
            </div>
        </section>
    </a>
</div>
