{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% import 'components/form/fields_macros.html.twig' as fields %}

<div
    class="d-none"
    data-glpi-itildestination-field-config-display-condition="{{ CONFIG_SPECIFIC_VALUES }}"
>
    {{ fields.dropdownField(
        "ITILValidationTemplate",
        specific_value_extra_field.input_name,
        specific_value_extra_field.value,
        "",
        options|merge({
            field_class: '',
            mb: '',
            multiple: true,
            no_label: true,
            aria_label: specific_value_extra_field.aria_label,
        })
    ) }}
</div>

<div
    class="d-none"
    data-glpi-itildestination-field-config-display-condition="{{ CONFIG_SPECIFIC_ACTORS }}"
>
    {{ fields.dropdownField(
        'ValidationStep',
        specific_validation_step_extra_field.input_name,
        specific_validation_step_extra_field.value|default(0),
        '',
        options|merge({
            field_class: '',
            mb: '',
            no_label: true,
            emptylabel: specific_validation_step_extra_field.empty_label,
            aria_label: specific_validation_step_extra_field.aria_label,
            rand: random(),
        })
    ) }}

    {% set actors_dropdown = call('Glpi\\Form\\Dropdown\\FormActorsDropdown::show', [
        specific_values_extra_field.input_name,
        specific_values_extra_field.values,
        {
            multiple: true,
            allowed_types: specific_values_extra_field.allowed_types,
            aria_label: specific_values_extra_field.aria_label,
        }
    ]) %}

    {{ fields.htmlField(
        'default_value',
        actors_dropdown,
        '',
        {
            field_class: '',
            mb: '',
            no_label: true,
            wrapper_class: ''
        }
    ) }}
</div>

<div
    class="d-none"
    data-glpi-itildestination-field-config-display-condition="{{ CONFIG_SPECIFIC_ANSWERS }}"
>
    {{ fields.dropdownField(
        'ValidationStep',
        specific_validation_step_extra_field.input_name,
        specific_validation_step_extra_field.value|default(0),
        '',
        options|merge({
            field_class: '',
            mb: '',
            no_label: true,
            emptylabel: specific_validation_step_extra_field.empty_label,
            aria_label: specific_validation_step_extra_field.aria_label,
            rand: random(),
        })
    ) }}

    {{ fields.dropdownArrayField(
        specific_answers_extra_field.input_name,
        '',
        specific_answers_extra_field.possible_values,
        "",
        options|merge({
            field_class: '',
            mb: '',
            no_label: true,
            display_emptychoice: true,
            multiple: true,
            values: specific_answers_extra_field.values,
            aria_label: specific_answers_extra_field.aria_label,
        })
    ) }}
</div>
