{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% import 'components/form/fields_macros.html.twig' as fields %}

<div class="alert alert-warning">
   <strong>
      {% if is_dev %}
         {{ __('You are using a development version, be careful!') }}
      {% else %}
         {{ __('You are using a pre-release version, be careful!') }}
      {% endif %}
   </strong>
   <br>
   {{ fields.checkboxField('agree_unstable', 0, __('I know I am using a unstable version.'), {
      required: true,
      id: 'agree_unstable'
   }) }}
</div>
<script>
   $(function() {
       $('[name="continue"]').on('click', function(event){
           if (!$('#agree_unstable').is(':checked')) {
               event.preventDefault();
               alert('{{ __('Please check the unstable version checkbox.') }}');
           }
       });
   });
</script>
