{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% import 'components/form/fields_macros.html.twig' as fields %}

{{ fields.hiddenField(
    question.getEndUserInputName() ~ '[itemtype]',
    itemtype
) }}
{# Define empty choice manually to handle root entity #}
{% set dropdown_options = {
    'no_label'           : true,
    'right'              : 'all',
    'aria_label'         : aria_label,
    'mb'                 : '',
    'addicon'            : false,
    'comments'           : false,
    'condition'          : dropdown_restriction_params,
    'nochecklimit'       : true,
    'display_emptychoice': false,
    'toadd'              : {
        '-1': constant('Dropdown::EMPTY_VALUE'),
    },
} %}

{# Display additional fields in dropdown: IDs for ITIL objects, serial/user for assets #}
{% if displaywith is not empty %}
    {% set dropdown_options = dropdown_options|merge({'displaywith': displaywith}) %}
{% endif %}

{{ fields.dropdownField(
    itemtype,
    question.getEndUserInputName() ~ '[items_id]',
    default_items_id,
    '',
    dropdown_options
) }}
