<?php

namespace SourceSpan\Highlighter;

/**
 * @internal
 */
final class AsciiGlyph
{
    public const horizontalLine = '-';
    public const verticalLine = '|';
    public const topLeftCorner = ',';
    public const bottomLeftCorner = "'";
    public const cross = '+';
    public const upEnd = "'";
    public const downEnd = ',';
    public const horizontalLineBold = '=';
}
