/*!
 * ---------------------------------------------------------------------
 *
 * GLPI - Gestionnaire Libre de Parc Informatique
 *
 * http://glpi-project.org
 *
 * @copyright 2015-2026 Teclib' and contributors.
 * @copyright 2003-2014 by the INDEPNET Development Team.
 * @licence   https://www.gnu.org/licenses/gpl-3.0.html
 *
 * ---------------------------------------------------------------------
 *
 * LICENSE
 *
 * This file is part of GLPI.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------
 */

html:not([data-glpi-theme-dark="1"]) {
    @import "~tinymce/skins/ui/oxide/skin";
}

html[data-glpi-theme-dark="1"] {
    @import "~tinymce/skins/ui/oxide-dark/skin";
}

/* Tiny mce applied these styles to its toolbar but not to its content, we have to set them up manually */
.mce-content-readonly {
    opacity: 0.6;
    cursor: not-allowed;
}

/* Support placeholders */
.mce-content-body[data-mce-placeholder]:not(.mce-visualblocks)::before {
    color: var(--tblr-muted);
    content: attr(data-mce-placeholder);
    position: absolute;
}

/* Tinymce duplicate some tags on selection, unsure why but it does not fit our
expectations for forms tags and user mentions  */
.mce-offscreen-selection {
    display: none;
}
