/*!
 * ---------------------------------------------------------------------
 *
 * GLPI - Gestionnaire Libre de Parc Informatique
 *
 * http://glpi-project.org
 *
 * @copyright 2015-2026 Teclib' and contributors.
 * @copyright 2003-2014 by the INDEPNET Development Team.
 * @licence   https://www.gnu.org/licenses/gpl-3.0.html
 *
 * ---------------------------------------------------------------------
 *
 * LICENSE
 *
 * This file is part of GLPI.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------
 */

$is-dark: true;

:root[data-glpi-theme="auror_dark"] {
    --tblr-dark: #232e3c;
    --tblr-light: #9ca8c0;
    --tblr-body-bg: #1f2936; // hsl(213.9deg, 27.1%, 16.7%)
    --tblr-bg-surface: #232e3c;
    --tblr-body-color: #9ca8c0;
    --tblr-primary-rgb: 254, 201, 92;
    --tblr-primary-fg: #644e21;
    --tblr-secondary: #b0bbd4;
    --tblr-secondary-fg: #262d3d;
    --tblr-link-color-rgb: 182, 195, 224;
    --tblr-muted: #757d91;
    --tblr-badge-bg: var(--tblr-light);
    --tblr-badge-color: var(--tblr-dark);
    --glpi-form-header-fg: #6ba8f3;
    --glpi-form-header-border-color: transparent;
    --glpi-mainmenu-bg: #232e3c;
    --glpi-mainmenu-fg: #ffffffb3;
    --glpi-helpdesk-header: hsl(213.9deg, 27.1%, 30.7%);
    --glpi-helpdesk-tiles-section-bg: hsl(213.9deg, 27.1%, 25.7%);
    --glpi-helpdesk-tabs-section-bg: hsl(213.9deg, 27.1%, 20.7%);
    --glpi-tabs-bg: #1d2531;
    --glpi-tabs-fg: var(--tblr-light);
    --glpi-tabs-border-color: transparent;
    --glpi-tabs-active-bg: #2a3a4f;
    --glpi-tabs-active-fg: #d8dfe7;
    --glpi-tabs-active-border-color: transparent;
    --glpi-itil-secondary-bg: #1d2531;
    --glpi-timeline-fup-bg: #1c2430;
    --glpi-timeline-fup-fg: #8892a8;
    --glpi-timeline-fup-border: #2f3d50;
    --glpi-palette-color-1: #1b2f62;
    --glpi-palette-color-2: #131425;
    --glpi-palette-color-3: #3a5693;
    --glpi-palette-color-4: #fec95c;

    // Helpdesk header
    --glpi-illustrations-gradient-1: hsl(213.9deg, 27.1%, 70%);
    --glpi-illustrations-gradient-2: hsl(213.9deg, 27.1%, 40%);
    --glpi-illustrations-gradient-3: hsl(213.9deg, 27.1%, 20%);
}
