tinymce.addI18n("eu", {
    "Redo": "Berregin",
    "Undo": "Desegin",
    "Cut": "Ebaki",
    "Copy": "Kopiatu",
    "Paste": "Itsatsi",
    "Select all": "Aukeratu dena",
    "New document": "Dokumentu berria",
    "Ok": "Ados",
    "Cancel": "Utzi",
    "Visual aids": "Laguntza bisualak",
    "Bold": "Lodia",
    "Italic": "Etzana",
    "Underline": "Azpimarratua",
    "Strikethrough": "Marratua",
    "Superscript": "Goi-indizea",
    "Subscript": "Azpiindizea",
    "Clear formatting": "Garbitu formatua",
    "Remove": "Kendu",
    "Align left": "Lerrokatu ezkerrean",
    "Align center": "Lerrokatu erdian",
    "Align right": "Lerrokatu eskuinean",
    "No alignment": "Lerrokatzerik ez",
    "Justify": "Justifikatuta",
    "Bullet list": "Bulet zerrenda",
    "Numbered list": "Zenbaki-zerrenda",
    "Decrease indent": "Txikitu koska",
    "Increase indent": "Handitu koska",
    "Close": "Itxi",
    "Formats": "Formatuak",
    "Your browser doesn't support direct access to the clipboard. Please use the Ctrl+X/C/V keyboard shortcuts instead.": "Zure nabigatzaileak ez du arbela zuzenean erabiltzeko euskarririk. Mesedez erabili CTRL+X/C/V teklatuko lasterbideak.",
    "Headings": "Izenburuak",
    "Heading 1": "1. goiburua",
    "Heading 2": "2. goiburua",
    "Heading 3": "3. goiburua",
    "Heading 4": "4. goiburua",
    "Heading 5": "5. goiburua",
    "Heading 6": "6. goiburua",
    "Preformatted": "Aurrez formateatuta",
    "Div": "Div",
    "Pre": "Pre",
    "Code": "Kodea",
    "Paragraph": "Paragrafoa",
    "Blockquote": "Zita bat egiteko blokea (blockquote)",
    "Inline": "Txertatuta",
    "Blocks": "Blokeak",
    "Paste is now in plain text mode. Contents will now be pasted as plain text until you toggle this option off.": "Itsastea testu formatu arruntean egingo da orain. Edukiak testu arrunt moduan txertatuko dira aukera hau aldatu bitartean.",
    "Fonts": "Letra-motak",
    "Font sizes": "Letra tamainak",
    "Class": "Klasea",
    "Browse for an image": "Arakatu irudi baten bila",
    "OR": "EDO",
    "Drop an image here": "Ekarri irudia hona",
    "Upload": "Kargatu",
    "Uploading image": "Irudia kargatzen",
    "Block": "Blokea",
    "Align": "Lerrokatu",
    "Default": "Lehenetsita",
    "Circle": "Zirkulua",
    "Disc": "Diskoa",
    "Square": "Karratua",
    "Lower Alpha": "Behe alfa",
    "Lower Greek": "Behe grekoa",
    "Lower Roman": "Behe erromatarra",
    "Upper Alpha": "Goi alfa",
    "Upper Roman": "Goi erromatarra",
    "Anchor...": "Aingura...",
    "Anchor": "Aingura",
    "Name": "Izena",
    "ID": "IDa",
    "ID should start with a letter, followed only by letters, numbers, dashes, dots, colons or underscores.": "IDa hizki batekin hasi behar da, eta jarraian hizki gehiago, zenbakiak, marrak, puntuak komak edo azpiko marrak izan ditzake soilik.",
    "You have unsaved changes are you sure you want to navigate away?": "Gorde gabeko aldaketak dituzu, ziur zaude hemendik irten nahi duzula?",
    "Restore last draft": "Leheneratu azken zirriborroa",
    "Special character...": "Karaktere berezia...",
    "Special Character": "Karaktere Berezia",
    "Source code": "Iturburu-kodea",
    "Insert/Edit code sample": "Txertatu/Editatu kode adibidea",
    "Language": "Hizkuntza",
    "Code sample...": "Kode adibidea...",
    "Left to right": "Ezkerretik eskuinera",
    "Right to left": "Eskuinetik ezkerrera",
    "Title": "Izenburua",
    "Fullscreen": "Pantaila osoa",
    "Action": "Ekintza",
    "Shortcut": "Laster tekla",
    "Help": "Laguntza",
    "Address": "Helbidea",
    "Focus to menubar": "Fokua menu-barrara eraman",
    "Focus to toolbar": "Fokoa tresna-barrara eraman",
    "Focus to element path": "Fokua elementuaren bidera eraman",
    "Focus to contextual toolbar": "Fokua testuinguruko tresna-barrara eraman",
    "Insert link (if link plugin activated)": "Txertatu esteka (esteka plugina aktibatuta badago)",
    "Save (if save plugin activated)": "Gorde (gordetzeko plugina aktibatuta badago)",
    "Find (if searchreplace plugin activated)": "Bilatu (bilatu-ordezkatu plugina aktibatuta badago)",
    "Plugins installed ({0}):": "Instalatutako pluginak ({0}):",
    "Premium plugins:": "Premium pluginak:",
    "Learn more...": "Ikasi gehiago...",
    "You are using {0}": "{0} erabiltzen ari zara",
    "Plugins": "Pluginak",
    "Handy Shortcuts": "Laster-tekla erabilgarriak",
    "Horizontal line": "Marra horizontala",
    "Insert/edit image": "Txertatu/editatu irudia",
    "Alternative description": "Ordezko deskribapena",
    "Accessibility": "Irisgarritasuna",
    "Image is decorative": "Irudia apaingarria da",
    "Source": "Iturburua",
    "Dimensions": "Neurriak",
    "Constrain proportions": "Mugatu proportzioak",
    "General": "Orokorra",
    "Advanced": "Aurreratua",
    "Style": "Estiloa",
    "Vertical space": "Hutsune bertikala",
    "Horizontal space": "Hutsune horizontala",
    "Border": "Ertza",
    "Insert image": "Txertatu irudia",
    "Image...": "Irudia...",
    "Image list": "Irudi zerrenda",
    "Resize": "Aldatu tamaina",
    "Insert date/time": "Txertatu data/ordua",
    "Date/time": "Data/ordua",
    "Insert/edit link": "Txertatu/editatu esteka",
    "Text to display": "Bistaratzeko testua",
    "Url": "URLa",
    "Open link in...": "Ireki esteka hemen...",
    "Current window": "Leiho berean",
    "None": "Bat ere ez",
    "New window": "Leiho berria",
    "Open link": "Ireki esteka",
    "Remove link": "Kendu esteka",
    "Anchors": "Aingurak",
    "Link...": "Esteka...",
    "Paste or type a link": "Itsatsi edo idatzi esteka",
    "The URL you entered seems to be an email address. Do you want to add the required mailto: prefix?": "Sartu duzun URLa helbide elektroniko bat dela dirudi. Nahi duzu dagokion mailto: aurrizkia gehitzea?",
    "The URL you entered seems to be an external link. Do you want to add the required http:// prefix?": "Sartu duzun URLa kanpoko esteka bat dela dirudi. Nahi duzu dagokion http:// aurrizkia gehitzea?",
    "The URL you entered seems to be an external link. Do you want to add the required https:// prefix?": "Sartu duzun URLa kanpoko esteka bat dela dirudi. Nahi duzu dagokion https:// aurrizkia gehitzea?",
    "Link list": "Esteken zerrenda",
    "Insert video": "Txertatu bideoa",
    "Insert/edit video": "Txertatu/editatu bideoa",
    "Insert/edit media": "Txertatu/editatu multimedia",
    "Alternative source": "Ordezko jatorria",
    "Alternative source URL": "Ordezko jatorriaren URLa",
    "Media poster (Image URL)": "Multimedia posterra (irudiaren URLa)",
    "Paste your embed code below:": "Itsatsi azpian enbotatu beharreko zure kodea:",
    "Embed": "Enbotatzea",
    "Media...": "Multimedia...",
    "Nonbreaking space": "Zuriune zatiezina",
    "Page break": "Orrialde-jauzia",
    "Paste as text": "Itsatsi testu bezala",
    "Preview": "Aurrebista",
    "Print": "Inprimatu",
    "Print...": "Inprimatu...",
    "Save": "Gorde",
    "Find": "Bilatu",
    "Replace with": "Ordeztu honekin",
    "Replace": "Ordeztu",
    "Replace all": "Ordeztu guztiak",
    "Previous": "Aurrekoa",
    "Next": "Hurrengoa",
    "Find and Replace": "Bilatu eta ordezkatu",
    "Find and replace...": "Bilatu eta ordezkatu...",
    "Could not find the specified string.": "Ezin izan da zehaztutako katea aurkitu.",
    "Match case": "Maiuskula/minuskula",
    "Find whole words only": "Bilatu soilik hitz osoak",
    "Find in selection": "Bilatu hautapenean",
    "Insert table": "Txertatu taula",
    "Table properties": "Taularen propietateak",
    "Delete table": "Ezabatu taula",
    "Cell": "Gelaxka",
    "Row": "Errenkada",
    "Column": "Zutabea",
    "Cell properties": "Gelaxkaren propietateak",
    "Merge cells": "Batu gelaxkak",
    "Split cell": "Banatu gelaxka",
    "Insert row before": "Txertatu errenkada aurretik",
    "Insert row after": "Txertatu errenkada ostean",
    "Delete row": "Ezabatu errenkada",
    "Row properties": "Errenkadaren propietateak",
    "Cut row": "Ebaki errenkada",
    "Cut column": "Ebaki zutabea",
    "Copy row": "Kopiatu errenkada",
    "Copy column": "Kopiatu zutabea",
    "Paste row before": "Itsatsi errenkada aurretik",
    "Paste column before": "Itsatsi zutabea aurretik",
    "Paste row after": "Itsatsi errenkada ostean",
    "Paste column after": "Itsatsi zutabea ostean",
    "Insert column before": "Txertatu zutabea aurretik",
    "Insert column after": "Txertatu zutabea ostean",
    "Delete column": "Ezabatu zutabea",
    "Cols": "Zutabeak",
    "Rows": "Errenkadak",
    "Width": "Zabalera",
    "Height": "Altuera",
    "Cell spacing": "Gelaxka arteko tartea",
    "Cell padding": "Gelaxken betegarria",
    "Row clipboard actions": "Errenkadaren arbeleko ekintzak",
    "Column clipboard actions": "Zutabearen arbeleko ekintzak",
    "Table styles": "Taularen estiloak",
    "Cell styles": "Gelaxkaren estiloak",
    "Column header": "Zutabearen goiburua",
    "Row header": "Errenkadaren goiburua",
    "Table caption": "Taularen deskribapena",
    "Caption": "Irudi-oina",
    "Show caption": "Erakutsi irudi-oina",
    "Left": "Ezkerra",
    "Center": "Erdia",
    "Right": "Eskuina",
    "Cell type": "Gelaxka mota",
    "Scope": "Esparrua",
    "Alignment": "Lerrokatzea",
    "Horizontal align": "Lerrokatze horizontala",
    "Vertical align": "Lerrokatze bertikala",
    "Top": "Goian",
    "Middle": "Erdian",
    "Bottom": "Behean",
    "Header cell": "Goiburuko gelaxka",
    "Row group": "Errenkada taldea",
    "Column group": "Zutabe taldea",
    "Row type": "Errenkada mota",
    "Header": "Goiburua",
    "Body": "Gorputza",
    "Footer": "Oina",
    "Border color": "Ertzaren kolorea",
    "Solid": "Solidoa",
    "Dotted": "Puntuekin",
    "Dashed": "Marrekin",
    "Double": "Bikoitza",
    "Groove": "Ildaskatua",
    "Ridge": "Koska",
    "Inset": "Barruko marra",
    "Outset": "Kanpoko marra",
    "Hidden": "Ezkutua",
    "Insert template...": "Txertatu txantiloia...",
    "Templates": "Txantiloiak",
    "Template": "Txantiloia",
    "Insert Template": "Txertatu txantiloia",
    "Text color": "Testuaren kolorea",
    "Background color": "Atzeko planoaren kolorea",
    "Custom...": "Pertsonalizatu...",
    "Custom color": "Kolore pertsonalizatua",
    "No color": "Kolorerik ez",
    "Remove color": "Kendu kolorea",
    "Show blocks": "Erakutsi blokeak",
    "Show invisible characters": "Erakutsi karaktere ezkutuak",
    "Word count": "Hitz-kontagailua",
    "Count": "Zenbatu",
    "Document": "Dokumentua",
    "Selection": "Aukeraketa",
    "Words": "Hitzak",
    "Words: {0}": "Hitzak: {0}",
    "{0} words": "{0} hitz",
    "File": "Fitxategia",
    "Edit": "Editatu",
    "Insert": "Txertatu",
    "View": "Ikusi",
    "Format": "Formatua",
    "Table": "Taula",
    "Tools": "Tresnak",
    "Powered by {0}": "Garatzailea: {0}",
    "Rich Text Area. Press ALT-F9 for menu. Press ALT-F10 for toolbar. Press ALT-0 for help": "Testu Aberastun Eremua. Sakatu ALT-F9 menurako. Sakatu ALT-F10 tresna-barrarako. Sakatu ALT-0 laguntzarako",
    "Image title": "Irudiaren izenburua",
    "Border width": "Ertzaren zabalera",
    "Border style": "Ertzaren estiloa",
    "Error": "Errorea",
    "Warn": "Oharra",
    "Valid": "Zuzena",
    "To open the popup, press Shift+Enter": "Leiho berria irekitzeko, sakatu Shift+Enter",
    "Rich Text Area": "Testu Aberastun Eremua",
    "Rich Text Area. Press ALT-0 for help.": "Testu Aberastun Eremua. Sakatu ALT-0 laguntza lortzeko.",
    "System Font": "Sistemaren letra-mota",
    "Failed to upload image: {0}": "Errorea irudia igotzean: {0}",
    "Failed to load plugin: {0} from url {1}": "Errorea {0} plugina {1} URLtik kargatzean",
    "Failed to load plugin url: {0}": "Errorea gertatu da pluginaren URLa kargatzean: {0}",
    "Failed to initialize plugin: {0}": "Errorea plugina abiaraztean: {0}",
    "example": "adibidea",
    "Search": "Bilatu",
    "All": "Guztiak",
    "Currency": "Moneta",
    "Text": "Testua",
    "Quotations": "Aipuak",
    "Mathematical": "Matematika",
    "Extended Latin": "Latin zabaldua",
    "Symbols": "Ikurrak",
    "Arrows": "Geziak",
    "User Defined": "Erabiltzaileak definituta",
    "dollar sign": "dolarraren ikurra",
    "currency sign": "monetaren ikurra",
    "euro-currency sign": "euroaren ikurra",
    "colon sign": "bi puntuen ikurra",
    "cruzeiro sign": "cruzeiroaren ikurra",
    "french franc sign": "libera frantsesaren ikurra",
    "lira sign": "liraren ikurra",
    "mill sign": "millaren ikurra",
    "naira sign": "naira ikurra",
    "peseta sign": "pezetaren ikurra",
    "rupee sign": "rupiaren ikurra",
    "won sign": "wonaren ikurra",
    "new sheqel sign": "sheqel berriaren ikurra",
    "dong sign": "dongaren ikurra",
    "kip sign": "kiparen ikurra",
    "tugrik sign": "tugrikaren ikurra",
    "drachma sign": "drakmaren ikurra",
    "german penny symbol": "alemaniako peniaren ikurra",
    "peso sign": "pesoaren ikurra",
    "guarani sign": "guaraniaren ikurra",
    "austral sign": "australaren ikurra",
    "hryvnia sign": "hryvniaren ikurra",
    "cedi sign": "cediaren ikurra",
    "livre tournois sign": "libre tournoisaren ikurra",
    "spesmilo sign": "spesmiloaren ikurra",
    "tenge sign": "tengearen ikurra",
    "indian rupee sign": "indiako rupiaren ikurra",
    "turkish lira sign": "lira turkiarraren ikurra",
    "nordic mark sign": "iparraldeko markoaren ikurra",
    "manat sign": "manataren ikurra",
    "ruble sign": "rubloaren ikurra",
    "yen character": "yenaren karakterea",
    "yuan character": "yuanaren karakterea",
    "yuan character, in hong kong and taiwan": "yuanaren karakterea, hong kong eta taiwanen",
    "yen/yuan character variant one": "yen/yuan karakterearen 1go bariantea",
    "Emojis": "Emojiak",
    "Emojis...": "Emojiak...",
    "Loading emojis...": "Emojiak kargatzen...",
    "Could not load emojis": "Ezin izan dira emojiak kargatu",
    "People": "Jendea",
    "Animals and Nature": "Animaliak eta natura",
    "Food and Drink": "Janari eta edaria",
    "Activity": "Jarduera",
    "Travel and Places": "Bidaiak eta lekuak",
    "Objects": "Objektuak",
    "Flags": "Banderak",
    "Characters": "Karaktereak",
    "Characters (no spaces)": "Karaktereak (espaziorik gabe)",
    "{0} characters": "{0} karaktere",
    "Error: Form submit field collision.": "Errorea: formularioaren eremuetan talka gertatu da.",
    "Error: No form element found.": "Errorea: ez da formularioaren elementurik aurkitu.",
    "Color swatch": "Koloreak",
    "Color Picker": "Kolore-hautatzailea",
    "Invalid hex color code: {0}": "Kolore-kode hamaseitarra ez da zuzena: {0}",
    "Invalid input": "Sarrera ez da zuzena",
    "R": "R",
    "Red component": "Osagai gorria",
    "G": "G",
    "Green component": "Osagai berdea",
    "B": "B",
    "Blue component": "Osagai urdina",
    "#": "#",
    "Hex color code": "Kolore-kode hamaseitarra",
    "Range 0 to 255": "0-tik 255erako zenbakia",
    "Turquoise": "Turkesa",
    "Green": "Berdea",
    "Blue": "Urdina",
    "Purple": "Morea",
    "Navy Blue": "Itsas-urdina",
    "Dark Turquoise": "Turkesa iluna",
    "Dark Green": "Berde iluna",
    "Medium Blue": "Tarteko urdina",
    "Medium Purple": "Tarteko morea",
    "Midnight Blue": "Gauerdiko urdina",
    "Yellow": "Horia",
    "Orange": "Laranja",
    "Red": "Gorria",
    "Light Gray": "Gris argia",
    "Gray": "Grisa",
    "Dark Yellow": "Hori iluna",
    "Dark Orange": "Laranja iluna",
    "Dark Red": "Gorri iluna",
    "Medium Gray": "Tarteko grisa",
    "Dark Gray": "Gris iluna",
    "Light Green": "Berde argia",
    "Light Yellow": "Hori argia",
    "Light Red": "Gorri argia",
    "Light Purple": "More argia",
    "Light Blue": "Urdin argia",
    "Dark Purple": "More iluna",
    "Dark Blue": "Urdin iluna",
    "Black": "Beltza",
    "White": "Zuria",
    "Switch to or from fullscreen mode": "Aldatu pantaila osoko modura edo handik itzuli",
    "Open help dialog": "Ireki laguntza-mezua",
    "history": "historia",
    "styles": "estiloak",
    "formatting": "formatua ematen",
    "alignment": "lerrokatzea",
    "indentation": "koska",
    "Font": "Letra-mota",
    "Size": "Tamaina",
    "More...": "Gehiago...",
    "Select...": "Aukeratu...",
    "Preferences": "Hobespenak",
    "Yes": "Bai",
    "No": "Ez",
    "Keyboard Navigation": "Teklatuaren bidezko nabigazioa",
    "Version": "Bertsioa",
    "Code view": "Ikusi kodea",
    "Open popup menu for split buttons": "Ireki leiho-menua banatze-botoientzat",
    "List Properties": "Zerrendaren Ezaugarriak",
    "List properties...": "Zerrendaren ezaugarriak...",
    "Start list at number": "Hasi zerrenda zenbaki batean",
    "Line height": "Lerroaren altuera",
    "Dropped file type is not supported": "Jaregindako fitxategi mota ez da onartzen",
    "Loading...": "Kargatzen...",
    "ImageProxy HTTP error: Rejected request": "Irudi proxiaren HTTP errorea: eskaera baztertu egin da",
    "ImageProxy HTTP error: Could not find Image Proxy": "Irudi proxiaren HTTP errorea: ezin izan da irudi proxia aurkitu",
    "ImageProxy HTTP error: Incorrect Image Proxy URL": "Irudi proxiaren HTTP errorea: irudi proxiaren URLa ez da zuzena",
    "ImageProxy HTTP error: Unknown ImageProxy error": "Irudi proxiaren HTTP errorea: errore ezezaguna"
});