/*!
 * ---------------------------------------------------------------------
 *
 * GLPI - Gestionnaire Libre de Parc Informatique
 *
 * http://glpi-project.org
 *
 * @copyright 2015-2026 Teclib' and contributors.
 * @copyright 2003-2014 by the INDEPNET Development Team.
 * @licence   https://www.gnu.org/licenses/gpl-3.0.html
 *
 * ---------------------------------------------------------------------
 *
 * LICENSE
 *
 * This file is part of GLPI.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------
 */

:root[data-glpi-theme="hipster"] {
    --tblr-primary-rgb: 255, 210, 202;
    --tblr-primary-fg: #d6665a;
    --tblr-secondary: #d6665a;
    --tblr-secondary-fg: #fff;
    --tblr-link-color-rgb: 117, 164, 139;
    --glpi-mainmenu-bg: #d6665a; // hsl(6deg, 60%, 60%)
    --glpi-helpdesk-header: hsl(7deg, 54%, 85%);
    --glpi-mainmenu-fg: #ffd2ca;
    --glpi-palette-color-1: #dc9b74;
    --glpi-palette-color-2: #75a48b;
    --glpi-palette-color-3: #42282e;
    --glpi-palette-color-4: #d6665a;

    // Helpdesk header
    --glpi-illustrations-gradient-1: hsl(6deg, 60%, 92%);
    --glpi-illustrations-gradient-2: hsl(6deg, 60%, 65%);
    --glpi-illustrations-gradient-3: hsl(6deg, 60%, 40%);
}
