{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% import 'components/form/fields_macros.html.twig' as fields %}
{% import 'components/form/basic_inputs_macros.html.twig' as inputs %}

{% if can_edit %}
    <form method="post" action="{{ 'Item_Project'|itemtype_form_path }}">
    <div class="d-flex">
        <div class="col-auto">
        {{ inputs.hidden('_glpi_csrf_token', csrf_token()) }}
        {% if item is instanceof('Project') %}
            {{ inputs.hidden('projects_id', item.getID()) }}
            {{ fields.dropdownItemsFromItemtypes('items_id', '', {
                field_class: 'd-flex',
                width: 'auto',
                mb: '',
                no_label: true,
                itemtypes: config('project_asset_types'),
                used: used,
                entity: item.getEntityID(),
                entity_sons: item.isRecursive() ? 1 : 0
            }) }}
        {% else %}
            {{ inputs.hidden('items_id', item.getID()) }}
            {{ inputs.hidden('itemtype', get_class(item)) }}
            {{ 'Project'|itemtype_dropdown({
                'entity': item.getEntityID(),
                'used': used,
            }) }}
        {% endif %}
        </div>
        <div class="col-auto">
            {{ inputs.submit('add', _x('button', 'Add'), 1, {class: 'btn btn-primary ms-1', icon: 'ti ti-link'}) }}
        </div>
    </div>
    </form>
{% endif %}

{{ include('components/datatable.html.twig', datatable_params, with_context = false) }}
