<?php declare(strict_types=1);

namespace GraphQL\Type\Definition;

/*
export type GraphQLUnmodifiedType =
GraphQLScalarType |
GraphQLObjectType |
GraphQLInterfaceType |
GraphQLUnionType |
GraphQLEnumType |
GraphQLInputObjectType;
*/

interface UnmodifiedType {}
