/*!
 * ---------------------------------------------------------------------
 *
 * GLPI - Gestionnaire Libre de Parc Informatique
 *
 * http://glpi-project.org
 *
 * @copyright 2015-2026 Teclib' and contributors.
 * @copyright 2003-2014 by the INDEPNET Development Team.
 * @licence   https://www.gnu.org/licenses/gpl-3.0.html
 *
 * ---------------------------------------------------------------------
 *
 * LICENSE
 *
 * This file is part of GLPI.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------
 */

// reset change in tabler : https://github.com/tabler/tabler/commit/a9075f6717958300afad13fe51311f7e78bb7e91#diff-340d6988403d25d9ca8e073d798e79a801f5c4fcf9a67366dee207d2ed9fef54L2
@mixin tabler-badge-fix {
    display: inline-flex;
    font-size: 0.625rem;
    min-height: 1.25rem;
    height: auto;
    align-items: flex-start;
    vertical-align: middle;
    border: 1px solid transparent;

    .badge {
        min-height: 100%;
        margin: -0.25rem -0.25rem -0.25rem 0.5rem;

        a {
            color: inherit;
        }
    }
}
