{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% set display = false %}
{% if value_type == "specific" %}
   {% set out = call([item, "getValueToSelect"], [searchopt, inputname, value, options2]) %}
   {% if out is not empty %}
      {% set display = true %}
      {{ out|raw }}
   {% endif %}
   {% if not display and plug is not null %}
      {% set display = call_plugin_one_hook(plug['plugin'], 'searchOptionsValues', {
         name: inputname,
         searchtype: searchtype,
         searchoption: searchopt,
         value: value,
      }) %}
   {% endif %}
{% elseif value_type == "empty" %}
   <input type="hidden" name="{{ inputname }}" value="null">
{% endif %}

{% if not display %}
   <input type="text" class="form-control" size="13" aria-label="{{ __('Criteria value') }}" name="{{ inputname }}" value="{{ value }}">
{% endif %}
