{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

<div class="card">
   <div class="card-header">
      <h3 class="card-title">{{ __('Result details') }}</h3>
   </div>
   <div class="card-body">
      <table class="table">
         <tbody>
            {% for result in results %}
               <tr>
                  <td>{{ result['name'] }}</td>
                  <td>{{ result['result'] }}</td>
               </tr>
            {% endfor %}
         </tbody>
      </table>
      <table class="table mt-4">
         <thead>
            <tr>
               <th colspan="2" class="center">
                  <h4>
                     <span>{{ __('Rule results') }}</span>
                     <br>
                     <span class="text-{{ global_result_raw ? 'success' : 'danger' }}">
                        {{ _n('Validation', 'Validations', 1) }}: {{ global_result }}
                     </span>
                  </h4>
               </th>
            </tr>
         </thead>
         <tbody>
            {% for result_action in result_actions %}
               <tr>
                  <td>{{ result_action['name'] }}</td>
                  <td>{{ result_action['value'] }}</td>
               </tr>
            {% endfor %}
         </tbody>
      </table>
   </div>
</div>
