{#
 # ---------------------------------------------------------------------
 #
 # GLPI - Gestionnaire Libre de Parc Informatique
 #
 # http://glpi-project.org
 #
 # @copyright 2015-2026 Teclib' and contributors.
 # @licence   https://www.gnu.org/licenses/gpl-3.0.html
 #
 # ---------------------------------------------------------------------
 #
 # LICENSE
 #
 # This file is part of GLPI.
 #
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
 # (at your option) any later version.
 #
 # This program is distributed in the hope that it will be useful,
 # but WITHOUT ANY WARRANTY; without even the implied warranty of
 # MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 # GNU General Public License for more details.
 #
 # You should have received a copy of the GNU General Public License
 # along with this program.  If not, see <https://www.gnu.org/licenses/>.
 #
 # ---------------------------------------------------------------------
 #}

{% extends "generic_show_form.html.twig" %}
{% import 'components/form/fields_macros.html.twig' as fields %}
{% import 'components/form/basic_inputs_macros.html.twig' as inputs %}

{% block form_fields %}
    {{ inputs.hidden('users_id', session('glpiID')) }}
    {{ inputs.hidden('reminders_id', item.fields['reminders_id']) }}

    {% set field_options = {
        full_width: true
    } %}

    {% set lang_dropdown %}
        {% do call('Dropdown::showLanguages', ['language', {
            display_name: false,
            value: session('glpilanguage'),
            used: used_langs
        }]) %}
    {% endset %}
    {{ fields.htmlField('', lang_dropdown, __('Language'), field_options) }}

    {{ fields.textField('name', item.fields['name'], __('Name'), field_options) }}
    {{ fields.textareaField('text', item.fields['text']|safe_html, __('Description'), field_options|merge({
        enable_richtext: true,
        enable_fileupload: false
    })) }}
{% endblock %}
