<?php

// autoload_classmap.php @generated by Composer

$vendorDir = dirname(__DIR__);
$baseDir = dirname($vendorDir);

return array(
    'APIClient' => $baseDir . '/src/APIClient.php',
    'AbstractITILChildTemplate' => $baseDir . '/src/AbstractITILChildTemplate.php',
    'AbstractQuery' => $baseDir . '/src/AbstractQuery.php',
    'AbstractRightsDropdown' => $baseDir . '/src/AbstractRightsDropdown.php',
    'Agent' => $baseDir . '/src/Agent.php',
    'AgentType' => $baseDir . '/src/AgentType.php',
    'Ajax' => $baseDir . '/src/Ajax.php',
    'Alert' => $baseDir . '/src/Alert.php',
    'AllAssets' => $baseDir . '/src/AllAssets.php',
    'AltchaOrg\\Altcha\\Altcha' => $vendorDir . '/altcha-org/altcha/src/Altcha.php',
    'AltchaOrg\\Altcha\\BaseChallengeOptions' => $vendorDir . '/altcha-org/altcha/src/BaseChallengeOptions.php',
    'AltchaOrg\\Altcha\\Challenge' => $vendorDir . '/altcha-org/altcha/src/Challenge.php',
    'AltchaOrg\\Altcha\\ChallengeOptions' => $vendorDir . '/altcha-org/altcha/src/ChallengeOptions.php',
    'AltchaOrg\\Altcha\\CheckChallengeOptions' => $vendorDir . '/altcha-org/altcha/src/CheckChallengeOptions.php',
    'AltchaOrg\\Altcha\\Hasher\\Algorithm' => $vendorDir . '/altcha-org/altcha/src/Hasher/Algorithm.php',
    'AltchaOrg\\Altcha\\Hasher\\Hasher' => $vendorDir . '/altcha-org/altcha/src/Hasher/Hasher.php',
    'AltchaOrg\\Altcha\\Hasher\\HasherInterface' => $vendorDir . '/altcha-org/altcha/src/Hasher/HasherInterface.php',
    'AltchaOrg\\Altcha\\Obfuscator' => $vendorDir . '/altcha-org/altcha/src/Obfuscator.php',
    'AltchaOrg\\Altcha\\Payload' => $vendorDir . '/altcha-org/altcha/src/Payload.php',
    'AltchaOrg\\Altcha\\ServerSignaturePayload' => $vendorDir . '/altcha-org/altcha/src/ServerSignaturePayload.php',
    'AltchaOrg\\Altcha\\ServerSignatureVerification' => $vendorDir . '/altcha-org/altcha/src/ServerSignatureVerification.php',
    'AltchaOrg\\Altcha\\ServerSignatureVerificationData' => $vendorDir . '/altcha-org/altcha/src/ServerSignatureVerificationData.php',
    'AltchaOrg\\Altcha\\Solution' => $vendorDir . '/altcha-org/altcha/src/Solution.php',
    'Appliance' => $baseDir . '/src/Appliance.php',
    'ApplianceEnvironment' => $baseDir . '/src/ApplianceEnvironment.php',
    'ApplianceType' => $baseDir . '/src/ApplianceType.php',
    'Appliance_Item' => $baseDir . '/src/Appliance_Item.php',
    'Appliance_Item_Relation' => $baseDir . '/src/Appliance_Item_Relation.php',
    'Auth' => $baseDir . '/src/Auth.php',
    'AuthLDAP' => $baseDir . '/src/AuthLDAP.php',
    'AuthLdapReplicate' => $baseDir . '/src/AuthLdapReplicate.php',
    'AuthMail' => $baseDir . '/src/AuthMail.php',
    'AutoUpdateSystem' => $baseDir . '/src/AutoUpdateSystem.php',
    'BaconQrCode\\Common\\BitArray' => $vendorDir . '/bacon/bacon-qr-code/src/Common/BitArray.php',
    'BaconQrCode\\Common\\BitMatrix' => $vendorDir . '/bacon/bacon-qr-code/src/Common/BitMatrix.php',
    'BaconQrCode\\Common\\BitUtils' => $vendorDir . '/bacon/bacon-qr-code/src/Common/BitUtils.php',
    'BaconQrCode\\Common\\CharacterSetEci' => $vendorDir . '/bacon/bacon-qr-code/src/Common/CharacterSetEci.php',
    'BaconQrCode\\Common\\EcBlock' => $vendorDir . '/bacon/bacon-qr-code/src/Common/EcBlock.php',
    'BaconQrCode\\Common\\EcBlocks' => $vendorDir . '/bacon/bacon-qr-code/src/Common/EcBlocks.php',
    'BaconQrCode\\Common\\ErrorCorrectionLevel' => $vendorDir . '/bacon/bacon-qr-code/src/Common/ErrorCorrectionLevel.php',
    'BaconQrCode\\Common\\FormatInformation' => $vendorDir . '/bacon/bacon-qr-code/src/Common/FormatInformation.php',
    'BaconQrCode\\Common\\Mode' => $vendorDir . '/bacon/bacon-qr-code/src/Common/Mode.php',
    'BaconQrCode\\Common\\ReedSolomonCodec' => $vendorDir . '/bacon/bacon-qr-code/src/Common/ReedSolomonCodec.php',
    'BaconQrCode\\Common\\Version' => $vendorDir . '/bacon/bacon-qr-code/src/Common/Version.php',
    'BaconQrCode\\Encoder\\BlockPair' => $vendorDir . '/bacon/bacon-qr-code/src/Encoder/BlockPair.php',
    'BaconQrCode\\Encoder\\ByteMatrix' => $vendorDir . '/bacon/bacon-qr-code/src/Encoder/ByteMatrix.php',
    'BaconQrCode\\Encoder\\Encoder' => $vendorDir . '/bacon/bacon-qr-code/src/Encoder/Encoder.php',
    'BaconQrCode\\Encoder\\MaskUtil' => $vendorDir . '/bacon/bacon-qr-code/src/Encoder/MaskUtil.php',
    'BaconQrCode\\Encoder\\MatrixUtil' => $vendorDir . '/bacon/bacon-qr-code/src/Encoder/MatrixUtil.php',
    'BaconQrCode\\Encoder\\QrCode' => $vendorDir . '/bacon/bacon-qr-code/src/Encoder/QrCode.php',
    'BaconQrCode\\Exception\\ExceptionInterface' => $vendorDir . '/bacon/bacon-qr-code/src/Exception/ExceptionInterface.php',
    'BaconQrCode\\Exception\\InvalidArgumentException' => $vendorDir . '/bacon/bacon-qr-code/src/Exception/InvalidArgumentException.php',
    'BaconQrCode\\Exception\\OutOfBoundsException' => $vendorDir . '/bacon/bacon-qr-code/src/Exception/OutOfBoundsException.php',
    'BaconQrCode\\Exception\\RuntimeException' => $vendorDir . '/bacon/bacon-qr-code/src/Exception/RuntimeException.php',
    'BaconQrCode\\Exception\\UnexpectedValueException' => $vendorDir . '/bacon/bacon-qr-code/src/Exception/UnexpectedValueException.php',
    'BaconQrCode\\Exception\\WriterException' => $vendorDir . '/bacon/bacon-qr-code/src/Exception/WriterException.php',
    'BaconQrCode\\Renderer\\Color\\Alpha' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Color/Alpha.php',
    'BaconQrCode\\Renderer\\Color\\Cmyk' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Color/Cmyk.php',
    'BaconQrCode\\Renderer\\Color\\ColorInterface' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Color/ColorInterface.php',
    'BaconQrCode\\Renderer\\Color\\Gray' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Color/Gray.php',
    'BaconQrCode\\Renderer\\Color\\Rgb' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Color/Rgb.php',
    'BaconQrCode\\Renderer\\Eye\\CompositeEye' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Eye/CompositeEye.php',
    'BaconQrCode\\Renderer\\Eye\\EyeInterface' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Eye/EyeInterface.php',
    'BaconQrCode\\Renderer\\Eye\\ModuleEye' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Eye/ModuleEye.php',
    'BaconQrCode\\Renderer\\Eye\\PointyEye' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Eye/PointyEye.php',
    'BaconQrCode\\Renderer\\Eye\\SimpleCircleEye' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Eye/SimpleCircleEye.php',
    'BaconQrCode\\Renderer\\Eye\\SquareEye' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Eye/SquareEye.php',
    'BaconQrCode\\Renderer\\GDLibRenderer' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/GDLibRenderer.php',
    'BaconQrCode\\Renderer\\ImageRenderer' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/ImageRenderer.php',
    'BaconQrCode\\Renderer\\Image\\EpsImageBackEnd' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Image/EpsImageBackEnd.php',
    'BaconQrCode\\Renderer\\Image\\ImageBackEndInterface' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Image/ImageBackEndInterface.php',
    'BaconQrCode\\Renderer\\Image\\ImagickImageBackEnd' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Image/ImagickImageBackEnd.php',
    'BaconQrCode\\Renderer\\Image\\SvgImageBackEnd' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Image/SvgImageBackEnd.php',
    'BaconQrCode\\Renderer\\Image\\TransformationMatrix' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Image/TransformationMatrix.php',
    'BaconQrCode\\Renderer\\Module\\DotsModule' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Module/DotsModule.php',
    'BaconQrCode\\Renderer\\Module\\EdgeIterator\\Edge' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Module/EdgeIterator/Edge.php',
    'BaconQrCode\\Renderer\\Module\\EdgeIterator\\EdgeIterator' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Module/EdgeIterator/EdgeIterator.php',
    'BaconQrCode\\Renderer\\Module\\ModuleInterface' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Module/ModuleInterface.php',
    'BaconQrCode\\Renderer\\Module\\RoundnessModule' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Module/RoundnessModule.php',
    'BaconQrCode\\Renderer\\Module\\SquareModule' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Module/SquareModule.php',
    'BaconQrCode\\Renderer\\Path\\Close' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Path/Close.php',
    'BaconQrCode\\Renderer\\Path\\Curve' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Path/Curve.php',
    'BaconQrCode\\Renderer\\Path\\EllipticArc' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Path/EllipticArc.php',
    'BaconQrCode\\Renderer\\Path\\Line' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Path/Line.php',
    'BaconQrCode\\Renderer\\Path\\Move' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Path/Move.php',
    'BaconQrCode\\Renderer\\Path\\OperationInterface' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Path/OperationInterface.php',
    'BaconQrCode\\Renderer\\Path\\Path' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/Path/Path.php',
    'BaconQrCode\\Renderer\\PlainTextRenderer' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/PlainTextRenderer.php',
    'BaconQrCode\\Renderer\\RendererInterface' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/RendererInterface.php',
    'BaconQrCode\\Renderer\\RendererStyle\\EyeFill' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/RendererStyle/EyeFill.php',
    'BaconQrCode\\Renderer\\RendererStyle\\Fill' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/RendererStyle/Fill.php',
    'BaconQrCode\\Renderer\\RendererStyle\\Gradient' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/RendererStyle/Gradient.php',
    'BaconQrCode\\Renderer\\RendererStyle\\GradientType' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/RendererStyle/GradientType.php',
    'BaconQrCode\\Renderer\\RendererStyle\\RendererStyle' => $vendorDir . '/bacon/bacon-qr-code/src/Renderer/RendererStyle/RendererStyle.php',
    'BaconQrCode\\Writer' => $vendorDir . '/bacon/bacon-qr-code/src/Writer.php',
    'BarcodeManager' => $baseDir . '/src/BarcodeManager.php',
    'Blacklist' => $baseDir . '/src/Blacklist.php',
    'BlacklistedMailContent' => $baseDir . '/src/BlacklistedMailContent.php',
    'Brick\\Math\\BigDecimal' => $vendorDir . '/brick/math/src/BigDecimal.php',
    'Brick\\Math\\BigInteger' => $vendorDir . '/brick/math/src/BigInteger.php',
    'Brick\\Math\\BigNumber' => $vendorDir . '/brick/math/src/BigNumber.php',
    'Brick\\Math\\BigRational' => $vendorDir . '/brick/math/src/BigRational.php',
    'Brick\\Math\\Exception\\DivisionByZeroException' => $vendorDir . '/brick/math/src/Exception/DivisionByZeroException.php',
    'Brick\\Math\\Exception\\IntegerOverflowException' => $vendorDir . '/brick/math/src/Exception/IntegerOverflowException.php',
    'Brick\\Math\\Exception\\MathException' => $vendorDir . '/brick/math/src/Exception/MathException.php',
    'Brick\\Math\\Exception\\NegativeNumberException' => $vendorDir . '/brick/math/src/Exception/NegativeNumberException.php',
    'Brick\\Math\\Exception\\NumberFormatException' => $vendorDir . '/brick/math/src/Exception/NumberFormatException.php',
    'Brick\\Math\\Exception\\RoundingNecessaryException' => $vendorDir . '/brick/math/src/Exception/RoundingNecessaryException.php',
    'Brick\\Math\\Internal\\Calculator' => $vendorDir . '/brick/math/src/Internal/Calculator.php',
    'Brick\\Math\\Internal\\CalculatorRegistry' => $vendorDir . '/brick/math/src/Internal/CalculatorRegistry.php',
    'Brick\\Math\\Internal\\Calculator\\BcMathCalculator' => $vendorDir . '/brick/math/src/Internal/Calculator/BcMathCalculator.php',
    'Brick\\Math\\Internal\\Calculator\\GmpCalculator' => $vendorDir . '/brick/math/src/Internal/Calculator/GmpCalculator.php',
    'Brick\\Math\\Internal\\Calculator\\NativeCalculator' => $vendorDir . '/brick/math/src/Internal/Calculator/NativeCalculator.php',
    'Brick\\Math\\RoundingMode' => $vendorDir . '/brick/math/src/RoundingMode.php',
    'Budget' => $baseDir . '/src/Budget.php',
    'BudgetType' => $baseDir . '/src/BudgetType.php',
    'BusinessCriticity' => $baseDir . '/src/BusinessCriticity.php',
    'CAS_AuthenticationException' => $vendorDir . '/apereo/phpcas/source/CAS/AuthenticationException.php',
    'CAS_Client' => $vendorDir . '/apereo/phpcas/source/CAS/Client.php',
    'CAS_CookieJar' => $vendorDir . '/apereo/phpcas/source/CAS/CookieJar.php',
    'CAS_Exception' => $vendorDir . '/apereo/phpcas/source/CAS/Exception.php',
    'CAS_GracefullTerminationException' => $vendorDir . '/apereo/phpcas/source/CAS/GracefullTerminationException.php',
    'CAS_InvalidArgumentException' => $vendorDir . '/apereo/phpcas/source/CAS/InvalidArgumentException.php',
    'CAS_Languages_Catalan' => $vendorDir . '/apereo/phpcas/source/CAS/Languages/Catalan.php',
    'CAS_Languages_ChineseSimplified' => $vendorDir . '/apereo/phpcas/source/CAS/Languages/ChineseSimplified.php',
    'CAS_Languages_English' => $vendorDir . '/apereo/phpcas/source/CAS/Languages/English.php',
    'CAS_Languages_French' => $vendorDir . '/apereo/phpcas/source/CAS/Languages/French.php',
    'CAS_Languages_Galego' => $vendorDir . '/apereo/phpcas/source/CAS/Languages/Galego.php',
    'CAS_Languages_German' => $vendorDir . '/apereo/phpcas/source/CAS/Languages/German.php',
    'CAS_Languages_Greek' => $vendorDir . '/apereo/phpcas/source/CAS/Languages/Greek.php',
    'CAS_Languages_Japanese' => $vendorDir . '/apereo/phpcas/source/CAS/Languages/Japanese.php',
    'CAS_Languages_LanguageInterface' => $vendorDir . '/apereo/phpcas/source/CAS/Languages/LanguageInterface.php',
    'CAS_Languages_Portuguese' => $vendorDir . '/apereo/phpcas/source/CAS/Languages/Portuguese.php',
    'CAS_Languages_Spanish' => $vendorDir . '/apereo/phpcas/source/CAS/Languages/Spanish.php',
    'CAS_OutOfSequenceBeforeAuthenticationCallException' => $vendorDir . '/apereo/phpcas/source/CAS/OutOfSequenceBeforeAuthenticationCallException.php',
    'CAS_OutOfSequenceBeforeClientException' => $vendorDir . '/apereo/phpcas/source/CAS/OutOfSequenceBeforeClientException.php',
    'CAS_OutOfSequenceBeforeProxyException' => $vendorDir . '/apereo/phpcas/source/CAS/OutOfSequenceBeforeProxyException.php',
    'CAS_OutOfSequenceException' => $vendorDir . '/apereo/phpcas/source/CAS/OutOfSequenceException.php',
    'CAS_PGTStorage_AbstractStorage' => $vendorDir . '/apereo/phpcas/source/CAS/PGTStorage/AbstractStorage.php',
    'CAS_PGTStorage_Db' => $vendorDir . '/apereo/phpcas/source/CAS/PGTStorage/Db.php',
    'CAS_PGTStorage_File' => $vendorDir . '/apereo/phpcas/source/CAS/PGTStorage/File.php',
    'CAS_ProxiedService' => $vendorDir . '/apereo/phpcas/source/CAS/ProxiedService.php',
    'CAS_ProxiedService_Abstract' => $vendorDir . '/apereo/phpcas/source/CAS/ProxiedService/Abstract.php',
    'CAS_ProxiedService_Exception' => $vendorDir . '/apereo/phpcas/source/CAS/ProxiedService/Exception.php',
    'CAS_ProxiedService_Http' => $vendorDir . '/apereo/phpcas/source/CAS/ProxiedService/Http.php',
    'CAS_ProxiedService_Http_Abstract' => $vendorDir . '/apereo/phpcas/source/CAS/ProxiedService/Http/Abstract.php',
    'CAS_ProxiedService_Http_Get' => $vendorDir . '/apereo/phpcas/source/CAS/ProxiedService/Http/Get.php',
    'CAS_ProxiedService_Http_Post' => $vendorDir . '/apereo/phpcas/source/CAS/ProxiedService/Http/Post.php',
    'CAS_ProxiedService_Imap' => $vendorDir . '/apereo/phpcas/source/CAS/ProxiedService/Imap.php',
    'CAS_ProxiedService_Testable' => $vendorDir . '/apereo/phpcas/source/CAS/ProxiedService/Testable.php',
    'CAS_ProxyChain' => $vendorDir . '/apereo/phpcas/source/CAS/ProxyChain.php',
    'CAS_ProxyChain_AllowedList' => $vendorDir . '/apereo/phpcas/source/CAS/ProxyChain/AllowedList.php',
    'CAS_ProxyChain_Any' => $vendorDir . '/apereo/phpcas/source/CAS/ProxyChain/Any.php',
    'CAS_ProxyChain_Interface' => $vendorDir . '/apereo/phpcas/source/CAS/ProxyChain/Interface.php',
    'CAS_ProxyChain_Trusted' => $vendorDir . '/apereo/phpcas/source/CAS/ProxyChain/Trusted.php',
    'CAS_ProxyTicketException' => $vendorDir . '/apereo/phpcas/source/CAS/ProxyTicketException.php',
    'CAS_Request_AbstractRequest' => $vendorDir . '/apereo/phpcas/source/CAS/Request/AbstractRequest.php',
    'CAS_Request_CurlMultiRequest' => $vendorDir . '/apereo/phpcas/source/CAS/Request/CurlMultiRequest.php',
    'CAS_Request_CurlRequest' => $vendorDir . '/apereo/phpcas/source/CAS/Request/CurlRequest.php',
    'CAS_Request_Exception' => $vendorDir . '/apereo/phpcas/source/CAS/Request/Exception.php',
    'CAS_Request_MultiRequestInterface' => $vendorDir . '/apereo/phpcas/source/CAS/Request/MultiRequestInterface.php',
    'CAS_Request_RequestInterface' => $vendorDir . '/apereo/phpcas/source/CAS/Request/RequestInterface.php',
    'CAS_ServiceBaseUrl_AllowedListDiscovery' => $vendorDir . '/apereo/phpcas/source/CAS/ServiceBaseUrl/AllowedListDiscovery.php',
    'CAS_ServiceBaseUrl_Base' => $vendorDir . '/apereo/phpcas/source/CAS/ServiceBaseUrl/Base.php',
    'CAS_ServiceBaseUrl_Interface' => $vendorDir . '/apereo/phpcas/source/CAS/ServiceBaseUrl/Interface.php',
    'CAS_ServiceBaseUrl_Static' => $vendorDir . '/apereo/phpcas/source/CAS/ServiceBaseUrl/Static.php',
    'CAS_Session_PhpSession' => $vendorDir . '/apereo/phpcas/source/CAS/Session/PhpSession.php',
    'CAS_TypeMismatchException' => $vendorDir . '/apereo/phpcas/source/CAS/TypeMismatchException.php',
    'Cable' => $baseDir . '/src/Cable.php',
    'CableStrand' => $baseDir . '/src/CableStrand.php',
    'CableType' => $baseDir . '/src/CableType.php',
    'Calendar' => $baseDir . '/src/Calendar.php',
    'CalendarSegment' => $baseDir . '/src/CalendarSegment.php',
    'Calendar_Holiday' => $baseDir . '/src/Calendar_Holiday.php',
    'Cartridge' => $baseDir . '/src/Cartridge.php',
    'CartridgeItem' => $baseDir . '/src/CartridgeItem.php',
    'CartridgeItemType' => $baseDir . '/src/CartridgeItemType.php',
    'CartridgeItem_PrinterModel' => $baseDir . '/src/CartridgeItem_PrinterModel.php',
    'Central' => $baseDir . '/src/Central.php',
    'Certificate' => $baseDir . '/src/Certificate.php',
    'CertificateType' => $baseDir . '/src/CertificateType.php',
    'Certificate_Item' => $baseDir . '/src/Certificate_Item.php',
    'Change' => $baseDir . '/src/Change.php',
    'ChangeCost' => $baseDir . '/src/ChangeCost.php',
    'ChangeSatisfaction' => $baseDir . '/src/ChangeSatisfaction.php',
    'ChangeTask' => $baseDir . '/src/ChangeTask.php',
    'ChangeTemplate' => $baseDir . '/src/ChangeTemplate.php',
    'ChangeTemplateHiddenField' => $baseDir . '/src/ChangeTemplateHiddenField.php',
    'ChangeTemplateMandatoryField' => $baseDir . '/src/ChangeTemplateMandatoryField.php',
    'ChangeTemplatePredefinedField' => $baseDir . '/src/ChangeTemplatePredefinedField.php',
    'ChangeTemplateReadonlyField' => $baseDir . '/src/ChangeTemplateReadonlyField.php',
    'ChangeValidation' => $baseDir . '/src/ChangeValidation.php',
    'ChangeValidationStep' => $baseDir . '/src/ChangeValidationStep.php',
    'Change_Change' => $baseDir . '/src/Change_Change.php',
    'Change_Group' => $baseDir . '/src/Change_Group.php',
    'Change_Item' => $baseDir . '/src/Change_Item.php',
    'Change_Problem' => $baseDir . '/src/Change_Problem.php',
    'Change_Supplier' => $baseDir . '/src/Change_Supplier.php',
    'Change_Ticket' => $baseDir . '/src/Change_Ticket.php',
    'Change_User' => $baseDir . '/src/Change_User.php',
    'CleanSoftwareCron' => $baseDir . '/src/CleanSoftwareCron.php',
    'Cluster' => $baseDir . '/src/Cluster.php',
    'ClusterType' => $baseDir . '/src/ClusterType.php',
    'Com\\Tecnick\\Barcode\\Barcode' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Barcode.php',
    'Com\\Tecnick\\Barcode\\Exception' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Exception.php',
    'Com\\Tecnick\\Barcode\\Model' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Model.php',
    'Com\\Tecnick\\Barcode\\Type' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type.php',
    'Com\\Tecnick\\Barcode\\Type\\Convert' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Convert.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\Codabar' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/Codabar.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\CodeNineThree' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/CodeNineThree.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\CodeOneOne' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/CodeOneOne.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\CodeOneTwoEight' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/CodeOneTwoEight.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\CodeOneTwoEight\\CodeOneTwoEightA' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/CodeOneTwoEight/CodeOneTwoEightA.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\CodeOneTwoEight\\CodeOneTwoEightB' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/CodeOneTwoEight/CodeOneTwoEightB.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\CodeOneTwoEight\\CodeOneTwoEightC' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/CodeOneTwoEight/CodeOneTwoEightC.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\CodeOneTwoEight\\Process' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/CodeOneTwoEight/Process.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\CodeThreeNine' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/CodeThreeNine.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\CodeThreeNineCheck' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/CodeThreeNineCheck.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\CodeThreeNineExt' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/CodeThreeNineExt.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\CodeThreeNineExtCheck' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/CodeThreeNineExtCheck.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\EanEight' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/EanEight.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\EanFive' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/EanFive.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\EanOneThree' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/EanOneThree.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\EanTwo' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/EanTwo.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\Imb' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/Imb.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\ImbPre' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/ImbPre.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\InterleavedTwoOfFive' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/InterleavedTwoOfFive.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\InterleavedTwoOfFiveCheck' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/InterleavedTwoOfFiveCheck.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\KlantIndex' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/KlantIndex.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\Msi' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/Msi.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\MsiCheck' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/MsiCheck.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\Pharma' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/Pharma.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\PharmaTwoTracks' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/PharmaTwoTracks.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\Planet' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/Planet.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\Postnet' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/Postnet.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\Raw' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/Raw.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\RoyalMailFourCc' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/RoyalMailFourCc.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\StandardTwoOfFive' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/StandardTwoOfFive.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\StandardTwoOfFiveCheck' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/StandardTwoOfFiveCheck.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\UpcA' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/UpcA.php',
    'Com\\Tecnick\\Barcode\\Type\\Linear\\UpcE' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Linear/UpcE.php',
    'Com\\Tecnick\\Barcode\\Type\\Raw' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Raw.php',
    'Com\\Tecnick\\Barcode\\Type\\Square' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\Aztec' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/Aztec.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\Aztec\\Bitstream' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/Aztec/Bitstream.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\Aztec\\Codeword' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/Aztec/Codeword.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\Aztec\\Data' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/Aztec/Data.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\Aztec\\Encode' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/Aztec/Encode.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\Aztec\\ErrorCorrection' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/Aztec/ErrorCorrection.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\Aztec\\Layers' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/Aztec/Layers.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\Datamatrix' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/Datamatrix.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\Datamatrix\\Data' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/Datamatrix/Data.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\Datamatrix\\Encode' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/Datamatrix/Encode.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\Datamatrix\\EncodeTxt' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/Datamatrix/EncodeTxt.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\Datamatrix\\ErrorCorrection' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/Datamatrix/ErrorCorrection.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\Datamatrix\\Modes' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/Datamatrix/Modes.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\Datamatrix\\Placement' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/Datamatrix/Placement.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\Datamatrix\\Steps' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/Datamatrix/Steps.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\PdfFourOneSeven' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/PdfFourOneSeven.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\PdfFourOneSeven\\Compaction' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/PdfFourOneSeven/Compaction.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\PdfFourOneSeven\\Data' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/PdfFourOneSeven/Data.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\PdfFourOneSeven\\Sequence' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/PdfFourOneSeven/Sequence.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\QrCode' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/QrCode.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\QrCode\\ByteStream' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/QrCode/ByteStream.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\QrCode\\Data' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/QrCode/Data.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\QrCode\\Encode' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/QrCode/Encode.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\QrCode\\Encoder' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/QrCode/Encoder.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\QrCode\\EncodingMode' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/QrCode/EncodingMode.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\QrCode\\Estimate' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/QrCode/Estimate.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\QrCode\\Init' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/QrCode/Init.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\QrCode\\InputItem' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/QrCode/InputItem.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\QrCode\\Mask' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/QrCode/Mask.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\QrCode\\MaskNum' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/QrCode/MaskNum.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\QrCode\\Spec' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/QrCode/Spec.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\QrCode\\SpecRs' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/QrCode/SpecRs.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\QrCode\\Split' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/QrCode/Split.php',
    'Com\\Tecnick\\Barcode\\Type\\Square\\Raw' => $vendorDir . '/tecnickcom/tc-lib-barcode/src/Type/Square/Raw.php',
    'Com\\Tecnick\\Color\\Css' => $vendorDir . '/tecnickcom/tc-lib-color/src/Css.php',
    'Com\\Tecnick\\Color\\Exception' => $vendorDir . '/tecnickcom/tc-lib-color/src/Exception.php',
    'Com\\Tecnick\\Color\\Model' => $vendorDir . '/tecnickcom/tc-lib-color/src/Model.php',
    'Com\\Tecnick\\Color\\Model\\Cmyk' => $vendorDir . '/tecnickcom/tc-lib-color/src/Model/Cmyk.php',
    'Com\\Tecnick\\Color\\Model\\Gray' => $vendorDir . '/tecnickcom/tc-lib-color/src/Model/Gray.php',
    'Com\\Tecnick\\Color\\Model\\Hsl' => $vendorDir . '/tecnickcom/tc-lib-color/src/Model/Hsl.php',
    'Com\\Tecnick\\Color\\Model\\Rgb' => $vendorDir . '/tecnickcom/tc-lib-color/src/Model/Rgb.php',
    'Com\\Tecnick\\Color\\Model\\Template' => $vendorDir . '/tecnickcom/tc-lib-color/src/Model/Template.php',
    'Com\\Tecnick\\Color\\Pdf' => $vendorDir . '/tecnickcom/tc-lib-color/src/Pdf.php',
    'Com\\Tecnick\\Color\\Spot' => $vendorDir . '/tecnickcom/tc-lib-color/src/Spot.php',
    'Com\\Tecnick\\Color\\Web' => $vendorDir . '/tecnickcom/tc-lib-color/src/Web.php',
    'CommonDBChild' => $baseDir . '/src/CommonDBChild.php',
    'CommonDBConnexity' => $baseDir . '/src/CommonDBConnexity.php',
    'CommonDBConnexityItemNotFound' => $baseDir . '/src/CommonDBConnexityItemNotFound.php',
    'CommonDBRelation' => $baseDir . '/src/CommonDBRelation.php',
    'CommonDBTM' => $baseDir . '/src/CommonDBTM.php',
    'CommonDBVisible' => $baseDir . '/src/CommonDBVisible.php',
    'CommonDCModelDropdown' => $baseDir . '/src/CommonDCModelDropdown.php',
    'CommonDevice' => $baseDir . '/src/CommonDevice.php',
    'CommonDeviceModel' => $baseDir . '/src/CommonDeviceModel.php',
    'CommonDeviceType' => $baseDir . '/src/CommonDeviceType.php',
    'CommonDropdown' => $baseDir . '/src/CommonDropdown.php',
    'CommonGLPI' => $baseDir . '/src/CommonGLPI.php',
    'CommonGLPIInterface' => $baseDir . '/src/CommonGLPIInterface.php',
    'CommonITILActor' => $baseDir . '/src/CommonITILActor.php',
    'CommonITILCost' => $baseDir . '/src/CommonITILCost.php',
    'CommonITILObject' => $baseDir . '/src/CommonITILObject.php',
    'CommonITILObject_CommonITILObject' => $baseDir . '/src/CommonITILObject_CommonITILObject.php',
    'CommonITILRecurrent' => $baseDir . '/src/CommonITILRecurrent.php',
    'CommonITILRecurrentCron' => $baseDir . '/src/CommonITILRecurrentCron.php',
    'CommonITILSatisfaction' => $baseDir . '/src/CommonITILSatisfaction.php',
    'CommonITILTask' => $baseDir . '/src/CommonITILTask.php',
    'CommonITILValidation' => $baseDir . '/src/CommonITILValidation.php',
    'CommonITILValidationCron' => $baseDir . '/src/CommonITILValidationCron.php',
    'CommonImplicitTreeDropdown' => $baseDir . '/src/CommonImplicitTreeDropdown.php',
    'CommonItilObject_Item' => $baseDir . '/src/CommonItilObject_Item.php',
    'CommonTreeDropdown' => $baseDir . '/src/CommonTreeDropdown.php',
    'CommonType' => $baseDir . '/src/CommonType.php',
    'Complex\\Complex' => $vendorDir . '/markbaker/complex/classes/src/Complex.php',
    'Complex\\Exception' => $vendorDir . '/markbaker/complex/classes/src/Exception.php',
    'Complex\\Functions' => $vendorDir . '/markbaker/complex/classes/src/Functions.php',
    'Complex\\Operations' => $vendorDir . '/markbaker/complex/classes/src/Operations.php',
    'Composer\\InstalledVersions' => $vendorDir . '/composer/InstalledVersions.php',
    'Composer\\Pcre\\MatchAllResult' => $vendorDir . '/composer/pcre/src/MatchAllResult.php',
    'Composer\\Pcre\\MatchAllStrictGroupsResult' => $vendorDir . '/composer/pcre/src/MatchAllStrictGroupsResult.php',
    'Composer\\Pcre\\MatchAllWithOffsetsResult' => $vendorDir . '/composer/pcre/src/MatchAllWithOffsetsResult.php',
    'Composer\\Pcre\\MatchResult' => $vendorDir . '/composer/pcre/src/MatchResult.php',
    'Composer\\Pcre\\MatchStrictGroupsResult' => $vendorDir . '/composer/pcre/src/MatchStrictGroupsResult.php',
    'Composer\\Pcre\\MatchWithOffsetsResult' => $vendorDir . '/composer/pcre/src/MatchWithOffsetsResult.php',
    'Composer\\Pcre\\PHPStan\\InvalidRegexPatternRule' => $vendorDir . '/composer/pcre/src/PHPStan/InvalidRegexPatternRule.php',
    'Composer\\Pcre\\PHPStan\\PregMatchFlags' => $vendorDir . '/composer/pcre/src/PHPStan/PregMatchFlags.php',
    'Composer\\Pcre\\PHPStan\\PregMatchParameterOutTypeExtension' => $vendorDir . '/composer/pcre/src/PHPStan/PregMatchParameterOutTypeExtension.php',
    'Composer\\Pcre\\PHPStan\\PregMatchTypeSpecifyingExtension' => $vendorDir . '/composer/pcre/src/PHPStan/PregMatchTypeSpecifyingExtension.php',
    'Composer\\Pcre\\PHPStan\\PregReplaceCallbackClosureTypeExtension' => $vendorDir . '/composer/pcre/src/PHPStan/PregReplaceCallbackClosureTypeExtension.php',
    'Composer\\Pcre\\PHPStan\\UnsafeStrictGroupsCallRule' => $vendorDir . '/composer/pcre/src/PHPStan/UnsafeStrictGroupsCallRule.php',
    'Composer\\Pcre\\PcreException' => $vendorDir . '/composer/pcre/src/PcreException.php',
    'Composer\\Pcre\\Preg' => $vendorDir . '/composer/pcre/src/Preg.php',
    'Composer\\Pcre\\Regex' => $vendorDir . '/composer/pcre/src/Regex.php',
    'Composer\\Pcre\\ReplaceResult' => $vendorDir . '/composer/pcre/src/ReplaceResult.php',
    'Composer\\Pcre\\UnexpectedNullMatchException' => $vendorDir . '/composer/pcre/src/UnexpectedNullMatchException.php',
    'Computer' => $baseDir . '/src/Computer.php',
    'ComputerAntivirus' => $baseDir . '/src/ComputerAntivirus.php',
    'ComputerModel' => $baseDir . '/src/ComputerModel.php',
    'ComputerType' => $baseDir . '/src/ComputerType.php',
    'ComputerVirtualMachine' => $baseDir . '/src/ComputerVirtualMachine.php',
    'Config' => $baseDir . '/src/Config.php',
    'Consumable' => $baseDir . '/src/Consumable.php',
    'ConsumableItem' => $baseDir . '/src/ConsumableItem.php',
    'ConsumableItemType' => $baseDir . '/src/ConsumableItemType.php',
    'Contact' => $baseDir . '/src/Contact.php',
    'ContactType' => $baseDir . '/src/ContactType.php',
    'Contact_Supplier' => $baseDir . '/src/Contact_Supplier.php',
    'Contract' => $baseDir . '/src/Contract.php',
    'ContractCost' => $baseDir . '/src/ContractCost.php',
    'ContractType' => $baseDir . '/src/ContractType.php',
    'Contract_Item' => $baseDir . '/src/Contract_Item.php',
    'Contract_Supplier' => $baseDir . '/src/Contract_Supplier.php',
    'Contract_User' => $baseDir . '/src/Contract_User.php',
    'CronTask' => $baseDir . '/src/CronTask.php',
    'CronTaskLog' => $baseDir . '/src/CronTaskLog.php',
    'DASPRiD\\Enum\\AbstractEnum' => $vendorDir . '/dasprid/enum/src/AbstractEnum.php',
    'DASPRiD\\Enum\\EnumMap' => $vendorDir . '/dasprid/enum/src/EnumMap.php',
    'DASPRiD\\Enum\\Exception\\CloneNotSupportedException' => $vendorDir . '/dasprid/enum/src/Exception/CloneNotSupportedException.php',
    'DASPRiD\\Enum\\Exception\\ExceptionInterface' => $vendorDir . '/dasprid/enum/src/Exception/ExceptionInterface.php',
    'DASPRiD\\Enum\\Exception\\ExpectationException' => $vendorDir . '/dasprid/enum/src/Exception/ExpectationException.php',
    'DASPRiD\\Enum\\Exception\\IllegalArgumentException' => $vendorDir . '/dasprid/enum/src/Exception/IllegalArgumentException.php',
    'DASPRiD\\Enum\\Exception\\MismatchException' => $vendorDir . '/dasprid/enum/src/Exception/MismatchException.php',
    'DASPRiD\\Enum\\Exception\\SerializeNotSupportedException' => $vendorDir . '/dasprid/enum/src/Exception/SerializeNotSupportedException.php',
    'DASPRiD\\Enum\\Exception\\UnserializeNotSupportedException' => $vendorDir . '/dasprid/enum/src/Exception/UnserializeNotSupportedException.php',
    'DASPRiD\\Enum\\NullValue' => $vendorDir . '/dasprid/enum/src/NullValue.php',
    'DBConnection' => $baseDir . '/src/DBConnection.php',
    'DBmysql' => $baseDir . '/src/DBmysql.php',
    'DBmysqlIterator' => $baseDir . '/src/DBmysqlIterator.php',
    'DCRoom' => $baseDir . '/src/DCRoom.php',
    'Database' => $baseDir . '/src/Database.php',
    'DatabaseInstance' => $baseDir . '/src/DatabaseInstance.php',
    'DatabaseInstanceCategory' => $baseDir . '/src/DatabaseInstanceCategory.php',
    'DatabaseInstanceType' => $baseDir . '/src/DatabaseInstanceType.php',
    'Datacenter' => $baseDir . '/src/Datacenter.php',
    'Datamatrix' => $vendorDir . '/tecnickcom/tcpdf/include/barcodes/datamatrix.php',
    'DateError' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateError.php',
    'DateException' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateException.php',
    'DateInvalidOperationException' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateInvalidOperationException.php',
    'DateInvalidTimeZoneException' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateInvalidTimeZoneException.php',
    'DateMalformedIntervalStringException' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateMalformedIntervalStringException.php',
    'DateMalformedPeriodStringException' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateMalformedPeriodStringException.php',
    'DateMalformedStringException' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateMalformedStringException.php',
    'DateObjectError' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateObjectError.php',
    'DateRangeError' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/DateRangeError.php',
    'DbUtils' => $baseDir . '/src/DbUtils.php',
    'DefaultFilter' => $baseDir . '/src/DefaultFilter.php',
    'Defuse\\Crypto\\Core' => $vendorDir . '/defuse/php-encryption/src/Core.php',
    'Defuse\\Crypto\\Crypto' => $vendorDir . '/defuse/php-encryption/src/Crypto.php',
    'Defuse\\Crypto\\DerivedKeys' => $vendorDir . '/defuse/php-encryption/src/DerivedKeys.php',
    'Defuse\\Crypto\\Encoding' => $vendorDir . '/defuse/php-encryption/src/Encoding.php',
    'Defuse\\Crypto\\Exception\\BadFormatException' => $vendorDir . '/defuse/php-encryption/src/Exception/BadFormatException.php',
    'Defuse\\Crypto\\Exception\\CryptoException' => $vendorDir . '/defuse/php-encryption/src/Exception/CryptoException.php',
    'Defuse\\Crypto\\Exception\\EnvironmentIsBrokenException' => $vendorDir . '/defuse/php-encryption/src/Exception/EnvironmentIsBrokenException.php',
    'Defuse\\Crypto\\Exception\\IOException' => $vendorDir . '/defuse/php-encryption/src/Exception/IOException.php',
    'Defuse\\Crypto\\Exception\\WrongKeyOrModifiedCiphertextException' => $vendorDir . '/defuse/php-encryption/src/Exception/WrongKeyOrModifiedCiphertextException.php',
    'Defuse\\Crypto\\File' => $vendorDir . '/defuse/php-encryption/src/File.php',
    'Defuse\\Crypto\\Key' => $vendorDir . '/defuse/php-encryption/src/Key.php',
    'Defuse\\Crypto\\KeyOrPassword' => $vendorDir . '/defuse/php-encryption/src/KeyOrPassword.php',
    'Defuse\\Crypto\\KeyProtectedByPassword' => $vendorDir . '/defuse/php-encryption/src/KeyProtectedByPassword.php',
    'Defuse\\Crypto\\RuntimeTests' => $vendorDir . '/defuse/php-encryption/src/RuntimeTests.php',
    'Deprecated' => $vendorDir . '/symfony/polyfill-php84/Resources/stubs/Deprecated.php',
    'DeviceBattery' => $baseDir . '/src/DeviceBattery.php',
    'DeviceBatteryModel' => $baseDir . '/src/DeviceBatteryModel.php',
    'DeviceBatteryType' => $baseDir . '/src/DeviceBatteryType.php',
    'DeviceCamera' => $baseDir . '/src/DeviceCamera.php',
    'DeviceCameraModel' => $baseDir . '/src/DeviceCameraModel.php',
    'DeviceCase' => $baseDir . '/src/DeviceCase.php',
    'DeviceCaseModel' => $baseDir . '/src/DeviceCaseModel.php',
    'DeviceCaseType' => $baseDir . '/src/DeviceCaseType.php',
    'DeviceControl' => $baseDir . '/src/DeviceControl.php',
    'DeviceControlModel' => $baseDir . '/src/DeviceControlModel.php',
    'DeviceDrive' => $baseDir . '/src/DeviceDrive.php',
    'DeviceDriveModel' => $baseDir . '/src/DeviceDriveModel.php',
    'DeviceFirmware' => $baseDir . '/src/DeviceFirmware.php',
    'DeviceFirmwareModel' => $baseDir . '/src/DeviceFirmwareModel.php',
    'DeviceFirmwareType' => $baseDir . '/src/DeviceFirmwareType.php',
    'DeviceGeneric' => $baseDir . '/src/DeviceGeneric.php',
    'DeviceGenericModel' => $baseDir . '/src/DeviceGenericModel.php',
    'DeviceGenericType' => $baseDir . '/src/DeviceGenericType.php',
    'DeviceGraphicCard' => $baseDir . '/src/DeviceGraphicCard.php',
    'DeviceGraphicCardModel' => $baseDir . '/src/DeviceGraphicCardModel.php',
    'DeviceHardDrive' => $baseDir . '/src/DeviceHardDrive.php',
    'DeviceHardDriveModel' => $baseDir . '/src/DeviceHardDriveModel.php',
    'DeviceHardDriveType' => $baseDir . '/src/DeviceHardDriveType.php',
    'DeviceMemory' => $baseDir . '/src/DeviceMemory.php',
    'DeviceMemoryModel' => $baseDir . '/src/DeviceMemoryModel.php',
    'DeviceMemoryType' => $baseDir . '/src/DeviceMemoryType.php',
    'DeviceMotherboard' => $baseDir . '/src/DeviceMotherboard.php',
    'DeviceMotherboardModel' => $baseDir . '/src/DeviceMotherboardModel.php',
    'DeviceNetworkCard' => $baseDir . '/src/DeviceNetworkCard.php',
    'DeviceNetworkCardModel' => $baseDir . '/src/DeviceNetworkCardModel.php',
    'DevicePci' => $baseDir . '/src/DevicePci.php',
    'DevicePciModel' => $baseDir . '/src/DevicePciModel.php',
    'DevicePowerSupply' => $baseDir . '/src/DevicePowerSupply.php',
    'DevicePowerSupplyModel' => $baseDir . '/src/DevicePowerSupplyModel.php',
    'DeviceProcessor' => $baseDir . '/src/DeviceProcessor.php',
    'DeviceProcessorModel' => $baseDir . '/src/DeviceProcessorModel.php',
    'DeviceSensor' => $baseDir . '/src/DeviceSensor.php',
    'DeviceSensorModel' => $baseDir . '/src/DeviceSensorModel.php',
    'DeviceSensorType' => $baseDir . '/src/DeviceSensorType.php',
    'DeviceSimcard' => $baseDir . '/src/DeviceSimcard.php',
    'DeviceSimcardType' => $baseDir . '/src/DeviceSimcardType.php',
    'DeviceSoundCard' => $baseDir . '/src/DeviceSoundCard.php',
    'DeviceSoundCardModel' => $baseDir . '/src/DeviceSoundCardModel.php',
    'Dflydev\\DotAccessData\\Data' => $vendorDir . '/dflydev/dot-access-data/src/Data.php',
    'Dflydev\\DotAccessData\\DataInterface' => $vendorDir . '/dflydev/dot-access-data/src/DataInterface.php',
    'Dflydev\\DotAccessData\\Exception\\DataException' => $vendorDir . '/dflydev/dot-access-data/src/Exception/DataException.php',
    'Dflydev\\DotAccessData\\Exception\\InvalidPathException' => $vendorDir . '/dflydev/dot-access-data/src/Exception/InvalidPathException.php',
    'Dflydev\\DotAccessData\\Exception\\MissingPathException' => $vendorDir . '/dflydev/dot-access-data/src/Exception/MissingPathException.php',
    'Dflydev\\DotAccessData\\Util' => $vendorDir . '/dflydev/dot-access-data/src/Util.php',
    'DisplayPreference' => $baseDir . '/src/DisplayPreference.php',
    'Doctrine\\Common\\Lexer\\AbstractLexer' => $vendorDir . '/doctrine/lexer/src/AbstractLexer.php',
    'Doctrine\\Common\\Lexer\\Token' => $vendorDir . '/doctrine/lexer/src/Token.php',
    'Doctrine\\Deprecations\\Deprecation' => $vendorDir . '/doctrine/deprecations/src/Deprecation.php',
    'Doctrine\\Deprecations\\PHPUnit\\VerifyDeprecations' => $vendorDir . '/doctrine/deprecations/src/PHPUnit/VerifyDeprecations.php',
    'Document' => $baseDir . '/src/Document.php',
    'DocumentCategory' => $baseDir . '/src/DocumentCategory.php',
    'DocumentType' => $baseDir . '/src/DocumentType.php',
    'Document_Item' => $baseDir . '/src/Document_Item.php',
    'Domain' => $baseDir . '/src/Domain.php',
    'DomainRecord' => $baseDir . '/src/DomainRecord.php',
    'DomainRecordType' => $baseDir . '/src/DomainRecordType.php',
    'DomainRelation' => $baseDir . '/src/DomainRelation.php',
    'DomainType' => $baseDir . '/src/DomainType.php',
    'Domain_Item' => $baseDir . '/src/Domain_Item.php',
    'Dropdown' => $baseDir . '/src/Dropdown.php',
    'DropdownTranslation' => $baseDir . '/src/DropdownTranslation.php',
    'DropdownVisibility' => $baseDir . '/src/DropdownVisibility.php',
    'Egulias\\EmailValidator\\EmailLexer' => $vendorDir . '/egulias/email-validator/src/EmailLexer.php',
    'Egulias\\EmailValidator\\EmailParser' => $vendorDir . '/egulias/email-validator/src/EmailParser.php',
    'Egulias\\EmailValidator\\EmailValidator' => $vendorDir . '/egulias/email-validator/src/EmailValidator.php',
    'Egulias\\EmailValidator\\MessageIDParser' => $vendorDir . '/egulias/email-validator/src/MessageIDParser.php',
    'Egulias\\EmailValidator\\Parser' => $vendorDir . '/egulias/email-validator/src/Parser.php',
    'Egulias\\EmailValidator\\Parser\\Comment' => $vendorDir . '/egulias/email-validator/src/Parser/Comment.php',
    'Egulias\\EmailValidator\\Parser\\CommentStrategy\\CommentStrategy' => $vendorDir . '/egulias/email-validator/src/Parser/CommentStrategy/CommentStrategy.php',
    'Egulias\\EmailValidator\\Parser\\CommentStrategy\\DomainComment' => $vendorDir . '/egulias/email-validator/src/Parser/CommentStrategy/DomainComment.php',
    'Egulias\\EmailValidator\\Parser\\CommentStrategy\\LocalComment' => $vendorDir . '/egulias/email-validator/src/Parser/CommentStrategy/LocalComment.php',
    'Egulias\\EmailValidator\\Parser\\DomainLiteral' => $vendorDir . '/egulias/email-validator/src/Parser/DomainLiteral.php',
    'Egulias\\EmailValidator\\Parser\\DomainPart' => $vendorDir . '/egulias/email-validator/src/Parser/DomainPart.php',
    'Egulias\\EmailValidator\\Parser\\DoubleQuote' => $vendorDir . '/egulias/email-validator/src/Parser/DoubleQuote.php',
    'Egulias\\EmailValidator\\Parser\\FoldingWhiteSpace' => $vendorDir . '/egulias/email-validator/src/Parser/FoldingWhiteSpace.php',
    'Egulias\\EmailValidator\\Parser\\IDLeftPart' => $vendorDir . '/egulias/email-validator/src/Parser/IDLeftPart.php',
    'Egulias\\EmailValidator\\Parser\\IDRightPart' => $vendorDir . '/egulias/email-validator/src/Parser/IDRightPart.php',
    'Egulias\\EmailValidator\\Parser\\LocalPart' => $vendorDir . '/egulias/email-validator/src/Parser/LocalPart.php',
    'Egulias\\EmailValidator\\Parser\\PartParser' => $vendorDir . '/egulias/email-validator/src/Parser/PartParser.php',
    'Egulias\\EmailValidator\\Result\\InvalidEmail' => $vendorDir . '/egulias/email-validator/src/Result/InvalidEmail.php',
    'Egulias\\EmailValidator\\Result\\MultipleErrors' => $vendorDir . '/egulias/email-validator/src/Result/MultipleErrors.php',
    'Egulias\\EmailValidator\\Result\\Reason\\AtextAfterCFWS' => $vendorDir . '/egulias/email-validator/src/Result/Reason/AtextAfterCFWS.php',
    'Egulias\\EmailValidator\\Result\\Reason\\CRLFAtTheEnd' => $vendorDir . '/egulias/email-validator/src/Result/Reason/CRLFAtTheEnd.php',
    'Egulias\\EmailValidator\\Result\\Reason\\CRLFX2' => $vendorDir . '/egulias/email-validator/src/Result/Reason/CRLFX2.php',
    'Egulias\\EmailValidator\\Result\\Reason\\CRNoLF' => $vendorDir . '/egulias/email-validator/src/Result/Reason/CRNoLF.php',
    'Egulias\\EmailValidator\\Result\\Reason\\CharNotAllowed' => $vendorDir . '/egulias/email-validator/src/Result/Reason/CharNotAllowed.php',
    'Egulias\\EmailValidator\\Result\\Reason\\CommaInDomain' => $vendorDir . '/egulias/email-validator/src/Result/Reason/CommaInDomain.php',
    'Egulias\\EmailValidator\\Result\\Reason\\CommentsInIDRight' => $vendorDir . '/egulias/email-validator/src/Result/Reason/CommentsInIDRight.php',
    'Egulias\\EmailValidator\\Result\\Reason\\ConsecutiveAt' => $vendorDir . '/egulias/email-validator/src/Result/Reason/ConsecutiveAt.php',
    'Egulias\\EmailValidator\\Result\\Reason\\ConsecutiveDot' => $vendorDir . '/egulias/email-validator/src/Result/Reason/ConsecutiveDot.php',
    'Egulias\\EmailValidator\\Result\\Reason\\DetailedReason' => $vendorDir . '/egulias/email-validator/src/Result/Reason/DetailedReason.php',
    'Egulias\\EmailValidator\\Result\\Reason\\DomainAcceptsNoMail' => $vendorDir . '/egulias/email-validator/src/Result/Reason/DomainAcceptsNoMail.php',
    'Egulias\\EmailValidator\\Result\\Reason\\DomainHyphened' => $vendorDir . '/egulias/email-validator/src/Result/Reason/DomainHyphened.php',
    'Egulias\\EmailValidator\\Result\\Reason\\DomainTooLong' => $vendorDir . '/egulias/email-validator/src/Result/Reason/DomainTooLong.php',
    'Egulias\\EmailValidator\\Result\\Reason\\DotAtEnd' => $vendorDir . '/egulias/email-validator/src/Result/Reason/DotAtEnd.php',
    'Egulias\\EmailValidator\\Result\\Reason\\DotAtStart' => $vendorDir . '/egulias/email-validator/src/Result/Reason/DotAtStart.php',
    'Egulias\\EmailValidator\\Result\\Reason\\EmptyReason' => $vendorDir . '/egulias/email-validator/src/Result/Reason/EmptyReason.php',
    'Egulias\\EmailValidator\\Result\\Reason\\ExceptionFound' => $vendorDir . '/egulias/email-validator/src/Result/Reason/ExceptionFound.php',
    'Egulias\\EmailValidator\\Result\\Reason\\ExpectingATEXT' => $vendorDir . '/egulias/email-validator/src/Result/Reason/ExpectingATEXT.php',
    'Egulias\\EmailValidator\\Result\\Reason\\ExpectingCTEXT' => $vendorDir . '/egulias/email-validator/src/Result/Reason/ExpectingCTEXT.php',
    'Egulias\\EmailValidator\\Result\\Reason\\ExpectingDTEXT' => $vendorDir . '/egulias/email-validator/src/Result/Reason/ExpectingDTEXT.php',
    'Egulias\\EmailValidator\\Result\\Reason\\ExpectingDomainLiteralClose' => $vendorDir . '/egulias/email-validator/src/Result/Reason/ExpectingDomainLiteralClose.php',
    'Egulias\\EmailValidator\\Result\\Reason\\LabelTooLong' => $vendorDir . '/egulias/email-validator/src/Result/Reason/LabelTooLong.php',
    'Egulias\\EmailValidator\\Result\\Reason\\LocalOrReservedDomain' => $vendorDir . '/egulias/email-validator/src/Result/Reason/LocalOrReservedDomain.php',
    'Egulias\\EmailValidator\\Result\\Reason\\NoDNSRecord' => $vendorDir . '/egulias/email-validator/src/Result/Reason/NoDNSRecord.php',
    'Egulias\\EmailValidator\\Result\\Reason\\NoDomainPart' => $vendorDir . '/egulias/email-validator/src/Result/Reason/NoDomainPart.php',
    'Egulias\\EmailValidator\\Result\\Reason\\NoLocalPart' => $vendorDir . '/egulias/email-validator/src/Result/Reason/NoLocalPart.php',
    'Egulias\\EmailValidator\\Result\\Reason\\RFCWarnings' => $vendorDir . '/egulias/email-validator/src/Result/Reason/RFCWarnings.php',
    'Egulias\\EmailValidator\\Result\\Reason\\Reason' => $vendorDir . '/egulias/email-validator/src/Result/Reason/Reason.php',
    'Egulias\\EmailValidator\\Result\\Reason\\SpoofEmail' => $vendorDir . '/egulias/email-validator/src/Result/Reason/SpoofEmail.php',
    'Egulias\\EmailValidator\\Result\\Reason\\UnOpenedComment' => $vendorDir . '/egulias/email-validator/src/Result/Reason/UnOpenedComment.php',
    'Egulias\\EmailValidator\\Result\\Reason\\UnableToGetDNSRecord' => $vendorDir . '/egulias/email-validator/src/Result/Reason/UnableToGetDNSRecord.php',
    'Egulias\\EmailValidator\\Result\\Reason\\UnclosedComment' => $vendorDir . '/egulias/email-validator/src/Result/Reason/UnclosedComment.php',
    'Egulias\\EmailValidator\\Result\\Reason\\UnclosedQuotedString' => $vendorDir . '/egulias/email-validator/src/Result/Reason/UnclosedQuotedString.php',
    'Egulias\\EmailValidator\\Result\\Reason\\UnusualElements' => $vendorDir . '/egulias/email-validator/src/Result/Reason/UnusualElements.php',
    'Egulias\\EmailValidator\\Result\\Result' => $vendorDir . '/egulias/email-validator/src/Result/Result.php',
    'Egulias\\EmailValidator\\Result\\SpoofEmail' => $vendorDir . '/egulias/email-validator/src/Result/SpoofEmail.php',
    'Egulias\\EmailValidator\\Result\\ValidEmail' => $vendorDir . '/egulias/email-validator/src/Result/ValidEmail.php',
    'Egulias\\EmailValidator\\Validation\\DNSCheckValidation' => $vendorDir . '/egulias/email-validator/src/Validation/DNSCheckValidation.php',
    'Egulias\\EmailValidator\\Validation\\DNSGetRecordWrapper' => $vendorDir . '/egulias/email-validator/src/Validation/DNSGetRecordWrapper.php',
    'Egulias\\EmailValidator\\Validation\\DNSRecords' => $vendorDir . '/egulias/email-validator/src/Validation/DNSRecords.php',
    'Egulias\\EmailValidator\\Validation\\EmailValidation' => $vendorDir . '/egulias/email-validator/src/Validation/EmailValidation.php',
    'Egulias\\EmailValidator\\Validation\\Exception\\EmptyValidationList' => $vendorDir . '/egulias/email-validator/src/Validation/Exception/EmptyValidationList.php',
    'Egulias\\EmailValidator\\Validation\\Extra\\SpoofCheckValidation' => $vendorDir . '/egulias/email-validator/src/Validation/Extra/SpoofCheckValidation.php',
    'Egulias\\EmailValidator\\Validation\\MessageIDValidation' => $vendorDir . '/egulias/email-validator/src/Validation/MessageIDValidation.php',
    'Egulias\\EmailValidator\\Validation\\MultipleValidationWithAnd' => $vendorDir . '/egulias/email-validator/src/Validation/MultipleValidationWithAnd.php',
    'Egulias\\EmailValidator\\Validation\\NoRFCWarningsValidation' => $vendorDir . '/egulias/email-validator/src/Validation/NoRFCWarningsValidation.php',
    'Egulias\\EmailValidator\\Validation\\RFCValidation' => $vendorDir . '/egulias/email-validator/src/Validation/RFCValidation.php',
    'Egulias\\EmailValidator\\Warning\\AddressLiteral' => $vendorDir . '/egulias/email-validator/src/Warning/AddressLiteral.php',
    'Egulias\\EmailValidator\\Warning\\CFWSNearAt' => $vendorDir . '/egulias/email-validator/src/Warning/CFWSNearAt.php',
    'Egulias\\EmailValidator\\Warning\\CFWSWithFWS' => $vendorDir . '/egulias/email-validator/src/Warning/CFWSWithFWS.php',
    'Egulias\\EmailValidator\\Warning\\Comment' => $vendorDir . '/egulias/email-validator/src/Warning/Comment.php',
    'Egulias\\EmailValidator\\Warning\\DeprecatedComment' => $vendorDir . '/egulias/email-validator/src/Warning/DeprecatedComment.php',
    'Egulias\\EmailValidator\\Warning\\DomainLiteral' => $vendorDir . '/egulias/email-validator/src/Warning/DomainLiteral.php',
    'Egulias\\EmailValidator\\Warning\\EmailTooLong' => $vendorDir . '/egulias/email-validator/src/Warning/EmailTooLong.php',
    'Egulias\\EmailValidator\\Warning\\IPV6BadChar' => $vendorDir . '/egulias/email-validator/src/Warning/IPV6BadChar.php',
    'Egulias\\EmailValidator\\Warning\\IPV6ColonEnd' => $vendorDir . '/egulias/email-validator/src/Warning/IPV6ColonEnd.php',
    'Egulias\\EmailValidator\\Warning\\IPV6ColonStart' => $vendorDir . '/egulias/email-validator/src/Warning/IPV6ColonStart.php',
    'Egulias\\EmailValidator\\Warning\\IPV6Deprecated' => $vendorDir . '/egulias/email-validator/src/Warning/IPV6Deprecated.php',
    'Egulias\\EmailValidator\\Warning\\IPV6DoubleColon' => $vendorDir . '/egulias/email-validator/src/Warning/IPV6DoubleColon.php',
    'Egulias\\EmailValidator\\Warning\\IPV6GroupCount' => $vendorDir . '/egulias/email-validator/src/Warning/IPV6GroupCount.php',
    'Egulias\\EmailValidator\\Warning\\IPV6MaxGroups' => $vendorDir . '/egulias/email-validator/src/Warning/IPV6MaxGroups.php',
    'Egulias\\EmailValidator\\Warning\\LocalTooLong' => $vendorDir . '/egulias/email-validator/src/Warning/LocalTooLong.php',
    'Egulias\\EmailValidator\\Warning\\NoDNSMXRecord' => $vendorDir . '/egulias/email-validator/src/Warning/NoDNSMXRecord.php',
    'Egulias\\EmailValidator\\Warning\\ObsoleteDTEXT' => $vendorDir . '/egulias/email-validator/src/Warning/ObsoleteDTEXT.php',
    'Egulias\\EmailValidator\\Warning\\QuotedPart' => $vendorDir . '/egulias/email-validator/src/Warning/QuotedPart.php',
    'Egulias\\EmailValidator\\Warning\\QuotedString' => $vendorDir . '/egulias/email-validator/src/Warning/QuotedString.php',
    'Egulias\\EmailValidator\\Warning\\TLD' => $vendorDir . '/egulias/email-validator/src/Warning/TLD.php',
    'Egulias\\EmailValidator\\Warning\\Warning' => $vendorDir . '/egulias/email-validator/src/Warning/Warning.php',
    'Enclosure' => $baseDir . '/src/Enclosure.php',
    'EnclosureModel' => $baseDir . '/src/EnclosureModel.php',
    'Entity' => $baseDir . '/src/Entity.php',
    'Entity_KnowbaseItem' => $baseDir . '/src/Entity_KnowbaseItem.php',
    'Entity_RSSFeed' => $baseDir . '/src/Entity_RSSFeed.php',
    'Entity_Reminder' => $baseDir . '/src/Entity_Reminder.php',
    'ExtraVisibilityCriteria' => $baseDir . '/src/ExtraVisibilityCriteria.php',
    'FQDN' => $baseDir . '/src/FQDN.php',
    'FQDNLabel' => $baseDir . '/src/FQDNLabel.php',
    'FieldUnicity' => $baseDir . '/src/FieldUnicity.php',
    'Fieldblacklist' => $baseDir . '/src/Fieldblacklist.php',
    'Filesystem' => $baseDir . '/src/Filesystem.php',
    'Firebase\\JWT\\BeforeValidException' => $vendorDir . '/firebase/php-jwt/src/BeforeValidException.php',
    'Firebase\\JWT\\CachedKeySet' => $vendorDir . '/firebase/php-jwt/src/CachedKeySet.php',
    'Firebase\\JWT\\ExpiredException' => $vendorDir . '/firebase/php-jwt/src/ExpiredException.php',
    'Firebase\\JWT\\JWK' => $vendorDir . '/firebase/php-jwt/src/JWK.php',
    'Firebase\\JWT\\JWT' => $vendorDir . '/firebase/php-jwt/src/JWT.php',
    'Firebase\\JWT\\JWTExceptionWithPayloadInterface' => $vendorDir . '/firebase/php-jwt/src/JWTExceptionWithPayloadInterface.php',
    'Firebase\\JWT\\Key' => $vendorDir . '/firebase/php-jwt/src/Key.php',
    'Firebase\\JWT\\SignatureInvalidException' => $vendorDir . '/firebase/php-jwt/src/SignatureInvalidException.php',
    'GLPIKey' => $baseDir . '/src/GLPIKey.php',
    'GLPIMailer' => $baseDir . '/src/GLPIMailer.php',
    'GLPINetwork' => $baseDir . '/src/GLPINetwork.php',
    'GLPIPDF' => $baseDir . '/src/GLPIPDF.php',
    'GLPIUploadHandler' => $baseDir . '/src/GLPIUploadHandler.php',
    'Gettext\\Languages\\Category' => $vendorDir . '/gettext/languages/src/Category.php',
    'Gettext\\Languages\\CldrData' => $vendorDir . '/gettext/languages/src/CldrData.php',
    'Gettext\\Languages\\Exporter\\Exporter' => $vendorDir . '/gettext/languages/src/Exporter/Exporter.php',
    'Gettext\\Languages\\Exporter\\Html' => $vendorDir . '/gettext/languages/src/Exporter/Html.php',
    'Gettext\\Languages\\Exporter\\Json' => $vendorDir . '/gettext/languages/src/Exporter/Json.php',
    'Gettext\\Languages\\Exporter\\Php' => $vendorDir . '/gettext/languages/src/Exporter/Php.php',
    'Gettext\\Languages\\Exporter\\Po' => $vendorDir . '/gettext/languages/src/Exporter/Po.php',
    'Gettext\\Languages\\Exporter\\Prettyjson' => $vendorDir . '/gettext/languages/src/Exporter/Prettyjson.php',
    'Gettext\\Languages\\Exporter\\Ruby' => $vendorDir . '/gettext/languages/src/Exporter/Ruby.php',
    'Gettext\\Languages\\Exporter\\Xml' => $vendorDir . '/gettext/languages/src/Exporter/Xml.php',
    'Gettext\\Languages\\FormulaConverter' => $vendorDir . '/gettext/languages/src/FormulaConverter.php',
    'Gettext\\Languages\\Language' => $vendorDir . '/gettext/languages/src/Language.php',
    'Glpi\\Agent\\Communication\\AbstractRequest' => $baseDir . '/src/Glpi/Agent/Communication/AbstractRequest.php',
    'Glpi\\Agent\\Communication\\Headers\\Common' => $baseDir . '/src/Glpi/Agent/Communication/Headers/Common.php',
    'Glpi\\Altcha\\AltchaManager' => $baseDir . '/src/Glpi/Altcha/AltchaManager.php',
    'Glpi\\Altcha\\AltchaMode' => $baseDir . '/src/Glpi/Altcha/AltchaMode.php',
    'Glpi\\Altcha\\InvalidPayloadException' => $baseDir . '/src/Glpi/Altcha/InvalidPayloadException.php',
    'Glpi\\Api\\API' => $baseDir . '/src/Glpi/Api/API.php',
    'Glpi\\Api\\APIRest' => $baseDir . '/src/Glpi/Api/APIRest.php',
    'Glpi\\Api\\Deprecated\\CommonDeprecatedTrait' => $baseDir . '/src/Glpi/Api/Deprecated/CommonDeprecatedTrait.php',
    'Glpi\\Api\\Deprecated\\ComputerAntivirus' => $baseDir . '/src/Glpi/Api/Deprecated/ComputerAntivirus.php',
    'Glpi\\Api\\Deprecated\\ComputerVirtualMachine' => $baseDir . '/src/Glpi/Api/Deprecated/ComputerVirtualMachine.php',
    'Glpi\\Api\\Deprecated\\Computer_Item' => $baseDir . '/src/Glpi/Api/Deprecated/Computer_Item.php',
    'Glpi\\Api\\Deprecated\\Computer_SoftwareLicense' => $baseDir . '/src/Glpi/Api/Deprecated/Computer_SoftwareLicense.php',
    'Glpi\\Api\\Deprecated\\Computer_SoftwareVersion' => $baseDir . '/src/Glpi/Api/Deprecated/Computer_SoftwareVersion.php',
    'Glpi\\Api\\Deprecated\\DeprecatedInterface' => $baseDir . '/src/Glpi/Api/Deprecated/DeprecatedInterface.php',
    'Glpi\\Api\\Deprecated\\Netpoint' => $baseDir . '/src/Glpi/Api/Deprecated/Netpoint.php',
    'Glpi\\Api\\Deprecated\\Pdu_Plug' => $baseDir . '/src/Glpi/Api/Deprecated/Pdu_Plug.php',
    'Glpi\\Api\\Deprecated\\TicketFollowup' => $baseDir . '/src/Glpi/Api/Deprecated/TicketFollowup.php',
    'Glpi\\Api\\HL\\APIException' => $baseDir . '/src/Glpi/Api/HL/APIException.php',
    'Glpi\\Api\\HL\\Controller\\AbstractController' => $baseDir . '/src/Glpi/Api/HL/Controller/AbstractController.php',
    'Glpi\\Api\\HL\\Controller\\AdministrationController' => $baseDir . '/src/Glpi/Api/HL/Controller/AdministrationController.php',
    'Glpi\\Api\\HL\\Controller\\AssetController' => $baseDir . '/src/Glpi/Api/HL/Controller/AssetController.php',
    'Glpi\\Api\\HL\\Controller\\CRUDControllerTrait' => $baseDir . '/src/Glpi/Api/HL/Controller/CRUDControllerTrait.php',
    'Glpi\\Api\\HL\\Controller\\ComponentController' => $baseDir . '/src/Glpi/Api/HL/Controller/ComponentController.php',
    'Glpi\\Api\\HL\\Controller\\CoreController' => $baseDir . '/src/Glpi/Api/HL/Controller/CoreController.php',
    'Glpi\\Api\\HL\\Controller\\CustomAssetController' => $baseDir . '/src/Glpi/Api/HL/Controller/CustomAssetController.php',
    'Glpi\\Api\\HL\\Controller\\DashboardController' => $baseDir . '/src/Glpi/Api/HL/Controller/DashboardController.php',
    'Glpi\\Api\\HL\\Controller\\DropdownController' => $baseDir . '/src/Glpi/Api/HL/Controller/DropdownController.php',
    'Glpi\\Api\\HL\\Controller\\GraphQLController' => $baseDir . '/src/Glpi/Api/HL/Controller/GraphQLController.php',
    'Glpi\\Api\\HL\\Controller\\ITILController' => $baseDir . '/src/Glpi/Api/HL/Controller/ITILController.php',
    'Glpi\\Api\\HL\\Controller\\KnowbaseController' => $baseDir . '/src/Glpi/Api/HL/Controller/KnowbaseController.php',
    'Glpi\\Api\\HL\\Controller\\ManagementController' => $baseDir . '/src/Glpi/Api/HL/Controller/ManagementController.php',
    'Glpi\\Api\\HL\\Controller\\NotepadController' => $baseDir . '/src/Glpi/Api/HL/Controller/NotepadController.php',
    'Glpi\\Api\\HL\\Controller\\ProjectController' => $baseDir . '/src/Glpi/Api/HL/Controller/ProjectController.php',
    'Glpi\\Api\\HL\\Controller\\ReportController' => $baseDir . '/src/Glpi/Api/HL/Controller/ReportController.php',
    'Glpi\\Api\\HL\\Controller\\RuleController' => $baseDir . '/src/Glpi/Api/HL/Controller/RuleController.php',
    'Glpi\\Api\\HL\\Controller\\SetupController' => $baseDir . '/src/Glpi/Api/HL/Controller/SetupController.php',
    'Glpi\\Api\\HL\\Controller\\ToolController' => $baseDir . '/src/Glpi/Api/HL/Controller/ToolController.php',
    'Glpi\\Api\\HL\\Doc\\CreateRoute' => $baseDir . '/src/Glpi/Api/HL/Doc/CreateRoute.php',
    'Glpi\\Api\\HL\\Doc\\DeleteRoute' => $baseDir . '/src/Glpi/Api/HL/Doc/DeleteRoute.php',
    'Glpi\\Api\\HL\\Doc\\GetRoute' => $baseDir . '/src/Glpi/Api/HL/Doc/GetRoute.php',
    'Glpi\\Api\\HL\\Doc\\Parameter' => $baseDir . '/src/Glpi/Api/HL/Doc/Parameter.php',
    'Glpi\\Api\\HL\\Doc\\Response' => $baseDir . '/src/Glpi/Api/HL/Doc/Response.php',
    'Glpi\\Api\\HL\\Doc\\Route' => $baseDir . '/src/Glpi/Api/HL/Doc/Route.php',
    'Glpi\\Api\\HL\\Doc\\Schema' => $baseDir . '/src/Glpi/Api/HL/Doc/Schema.php',
    'Glpi\\Api\\HL\\Doc\\SchemaReference' => $baseDir . '/src/Glpi/Api/HL/Doc/SchemaReference.php',
    'Glpi\\Api\\HL\\Doc\\SearchRoute' => $baseDir . '/src/Glpi/Api/HL/Doc/SearchRoute.php',
    'Glpi\\Api\\HL\\Doc\\UpdateRoute' => $baseDir . '/src/Glpi/Api/HL/Doc/UpdateRoute.php',
    'Glpi\\Api\\HL\\GraphQL' => $baseDir . '/src/Glpi/Api/HL/GraphQL.php',
    'Glpi\\Api\\HL\\GraphQLGenerator' => $baseDir . '/src/Glpi/Api/HL/GraphQLGenerator.php',
    'Glpi\\Api\\HL\\Middleware\\AbstractMiddleware' => $baseDir . '/src/Glpi/Api/HL/Middleware/AbstractMiddleware.php',
    'Glpi\\Api\\HL\\Middleware\\AuthMiddlewareInterface' => $baseDir . '/src/Glpi/Api/HL/Middleware/AuthMiddlewareInterface.php',
    'Glpi\\Api\\HL\\Middleware\\CRUDRequestMiddleware' => $baseDir . '/src/Glpi/Api/HL/Middleware/CRUDRequestMiddleware.php',
    'Glpi\\Api\\HL\\Middleware\\CookieAuthMiddleware' => $baseDir . '/src/Glpi/Api/HL/Middleware/CookieAuthMiddleware.php',
    'Glpi\\Api\\HL\\Middleware\\DebugRequestMiddleware' => $baseDir . '/src/Glpi/Api/HL/Middleware/DebugRequestMiddleware.php',
    'Glpi\\Api\\HL\\Middleware\\DebugResponseMiddleware' => $baseDir . '/src/Glpi/Api/HL/Middleware/DebugResponseMiddleware.php',
    'Glpi\\Api\\HL\\Middleware\\IPRestrictionRequestMiddleware' => $baseDir . '/src/Glpi/Api/HL/Middleware/IPRestrictionRequestMiddleware.php',
    'Glpi\\Api\\HL\\Middleware\\InternalAuthMiddleware' => $baseDir . '/src/Glpi/Api/HL/Middleware/InternalAuthMiddleware.php',
    'Glpi\\Api\\HL\\Middleware\\MiddlewareInput' => $baseDir . '/src/Glpi/Api/HL/Middleware/MiddlewareInput.php',
    'Glpi\\Api\\HL\\Middleware\\OAuthRequestMiddleware' => $baseDir . '/src/Glpi/Api/HL/Middleware/OAuthRequestMiddleware.php',
    'Glpi\\Api\\HL\\Middleware\\RSQLRequestMiddleware' => $baseDir . '/src/Glpi/Api/HL/Middleware/RSQLRequestMiddleware.php',
    'Glpi\\Api\\HL\\Middleware\\RequestMiddlewareInterface' => $baseDir . '/src/Glpi/Api/HL/Middleware/RequestMiddlewareInterface.php',
    'Glpi\\Api\\HL\\Middleware\\ResponseMiddlewareInterface' => $baseDir . '/src/Glpi/Api/HL/Middleware/ResponseMiddlewareInterface.php',
    'Glpi\\Api\\HL\\Middleware\\ResultFormatterMiddleware' => $baseDir . '/src/Glpi/Api/HL/Middleware/ResultFormatterMiddleware.php',
    'Glpi\\Api\\HL\\Middleware\\SecurityResponseMiddleware' => $baseDir . '/src/Glpi/Api/HL/Middleware/SecurityResponseMiddleware.php',
    'Glpi\\Api\\HL\\OpenAPIGenerator' => $baseDir . '/src/Glpi/Api/HL/OpenAPIGenerator.php',
    'Glpi\\Api\\HL\\RSQL\\Error' => $baseDir . '/src/Glpi/Api/HL/RSQL/Error.php',
    'Glpi\\Api\\HL\\RSQL\\Lexer' => $baseDir . '/src/Glpi/Api/HL/RSQL/Lexer.php',
    'Glpi\\Api\\HL\\RSQL\\Parser' => $baseDir . '/src/Glpi/Api/HL/RSQL/Parser.php',
    'Glpi\\Api\\HL\\RSQL\\RSQLException' => $baseDir . '/src/Glpi/Api/HL/RSQL/RSQLException.php',
    'Glpi\\Api\\HL\\RSQL\\Result' => $baseDir . '/src/Glpi/Api/HL/RSQL/Result.php',
    'Glpi\\Api\\HL\\ResourceAccessor' => $baseDir . '/src/Glpi/Api/HL/ResourceAccessor.php',
    'Glpi\\Api\\HL\\RightConditionNotMetException' => $baseDir . '/src/Glpi/Api/HL/RightConditionNotMetException.php',
    'Glpi\\Api\\HL\\Route' => $baseDir . '/src/Glpi/Api/HL/Route.php',
    'Glpi\\Api\\HL\\RoutePath' => $baseDir . '/src/Glpi/Api/HL/RoutePath.php',
    'Glpi\\Api\\HL\\RouteVersion' => $baseDir . '/src/Glpi/Api/HL/RouteVersion.php',
    'Glpi\\Api\\HL\\Router' => $baseDir . '/src/Glpi/Api/HL/Router.php',
    'Glpi\\Api\\HL\\Search' => $baseDir . '/src/Glpi/Api/HL/Search.php',
    'Glpi\\Api\\HL\\Search\\RecordSet' => $baseDir . '/src/Glpi/Api/HL/Search/RecordSet.php',
    'Glpi\\Api\\HL\\Search\\SearchContext' => $baseDir . '/src/Glpi/Api/HL/Search/SearchContext.php',
    'Glpi\\Application\\Environment' => $baseDir . '/src/Glpi/Application/Environment.php',
    'Glpi\\Application\\ImportMapGenerator' => $baseDir . '/src/Glpi/Application/ImportMapGenerator.php',
    'Glpi\\Application\\ResourcesChecker' => $baseDir . '/src/Glpi/Application/ResourcesChecker.php',
    'Glpi\\Application\\SystemConfigurator' => $baseDir . '/src/Glpi/Application/SystemConfigurator.php',
    'Glpi\\Application\\View\\Extension\\ConfigExtension' => $baseDir . '/src/Glpi/Application/View/Extension/ConfigExtension.php',
    'Glpi\\Application\\View\\Extension\\DataHelpersExtension' => $baseDir . '/src/Glpi/Application/View/Extension/DataHelpersExtension.php',
    'Glpi\\Application\\View\\Extension\\DocumentExtension' => $baseDir . '/src/Glpi/Application/View/Extension/DocumentExtension.php',
    'Glpi\\Application\\View\\Extension\\FrontEndAssetsExtension' => $baseDir . '/src/Glpi/Application/View/Extension/FrontEndAssetsExtension.php',
    'Glpi\\Application\\View\\Extension\\HtmlExtension' => $baseDir . '/src/Glpi/Application/View/Extension/HtmlExtension.php',
    'Glpi\\Application\\View\\Extension\\I18nExtension' => $baseDir . '/src/Glpi/Application/View/Extension/I18nExtension.php',
    'Glpi\\Application\\View\\Extension\\IllustrationExtension' => $baseDir . '/src/Glpi/Application/View/Extension/IllustrationExtension.php',
    'Glpi\\Application\\View\\Extension\\ItemtypeExtension' => $baseDir . '/src/Glpi/Application/View/Extension/ItemtypeExtension.php',
    'Glpi\\Application\\View\\Extension\\PhpExtension' => $baseDir . '/src/Glpi/Application/View/Extension/PhpExtension.php',
    'Glpi\\Application\\View\\Extension\\PluginExtension' => $baseDir . '/src/Glpi/Application/View/Extension/PluginExtension.php',
    'Glpi\\Application\\View\\Extension\\RoutingExtension' => $baseDir . '/src/Glpi/Application/View/Extension/RoutingExtension.php',
    'Glpi\\Application\\View\\Extension\\SearchExtension' => $baseDir . '/src/Glpi/Application/View/Extension/SearchExtension.php',
    'Glpi\\Application\\View\\Extension\\SecurityExtension' => $baseDir . '/src/Glpi/Application/View/Extension/SecurityExtension.php',
    'Glpi\\Application\\View\\Extension\\SessionExtension' => $baseDir . '/src/Glpi/Application/View/Extension/SessionExtension.php',
    'Glpi\\Application\\View\\Extension\\TeamExtension' => $baseDir . '/src/Glpi/Application/View/Extension/TeamExtension.php',
    'Glpi\\Application\\View\\TemplateRenderer' => $baseDir . '/src/Glpi/Application/View/TemplateRenderer.php',
    'Glpi\\Asset\\Asset' => $baseDir . '/src/Glpi/Asset/Asset.php',
    'Glpi\\Asset\\AssetDefinition' => $baseDir . '/src/Glpi/Asset/AssetDefinition.php',
    'Glpi\\Asset\\AssetDefinitionManager' => $baseDir . '/src/Glpi/Asset/AssetDefinitionManager.php',
    'Glpi\\Asset\\AssetModel' => $baseDir . '/src/Glpi/Asset/AssetModel.php',
    'Glpi\\Asset\\AssetType' => $baseDir . '/src/Glpi/Asset/AssetType.php',
    'Glpi\\Asset\\Asset_PeripheralAsset' => $baseDir . '/src/Glpi/Asset/Asset_PeripheralAsset.php',
    'Glpi\\Asset\\Capacity' => $baseDir . '/src/Glpi/Asset/Capacity.php',
    'Glpi\\Asset\\CapacityConfig' => $baseDir . '/src/Glpi/Asset/CapacityConfig.php',
    'Glpi\\Asset\\Capacity\\AbstractCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/AbstractCapacity.php',
    'Glpi\\Asset\\Capacity\\AllowedInGlobalSearchCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/AllowedInGlobalSearchCapacity.php',
    'Glpi\\Asset\\Capacity\\CapacityInterface' => $baseDir . '/src/Glpi/Asset/Capacity/CapacityInterface.php',
    'Glpi\\Asset\\Capacity\\HasAntivirusCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasAntivirusCapacity.php',
    'Glpi\\Asset\\Capacity\\HasAppliancesCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasAppliancesCapacity.php',
    'Glpi\\Asset\\Capacity\\HasCertificatesCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasCertificatesCapacity.php',
    'Glpi\\Asset\\Capacity\\HasContractsCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasContractsCapacity.php',
    'Glpi\\Asset\\Capacity\\HasDatabaseInstanceCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasDatabaseInstanceCapacity.php',
    'Glpi\\Asset\\Capacity\\HasDevicesCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasDevicesCapacity.php',
    'Glpi\\Asset\\Capacity\\HasDocumentsCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasDocumentsCapacity.php',
    'Glpi\\Asset\\Capacity\\HasDomainsCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasDomainsCapacity.php',
    'Glpi\\Asset\\Capacity\\HasHistoryCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasHistoryCapacity.php',
    'Glpi\\Asset\\Capacity\\HasImpactCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasImpactCapacity.php',
    'Glpi\\Asset\\Capacity\\HasInfocomCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasInfocomCapacity.php',
    'Glpi\\Asset\\Capacity\\HasKnowbaseCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasKnowbaseCapacity.php',
    'Glpi\\Asset\\Capacity\\HasLinksCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasLinksCapacity.php',
    'Glpi\\Asset\\Capacity\\HasNetworkPortCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasNetworkPortCapacity.php',
    'Glpi\\Asset\\Capacity\\HasNotepadCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasNotepadCapacity.php',
    'Glpi\\Asset\\Capacity\\HasOperatingSystemCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasOperatingSystemCapacity.php',
    'Glpi\\Asset\\Capacity\\HasPeripheralAssetsCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasPeripheralAssetsCapacity.php',
    'Glpi\\Asset\\Capacity\\HasPlugCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasPlugCapacity.php',
    'Glpi\\Asset\\Capacity\\HasRemoteManagementCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasRemoteManagementCapacity.php',
    'Glpi\\Asset\\Capacity\\HasSocketCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasSocketCapacity.php',
    'Glpi\\Asset\\Capacity\\HasSoftwaresCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasSoftwaresCapacity.php',
    'Glpi\\Asset\\Capacity\\HasVirtualMachineCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasVirtualMachineCapacity.php',
    'Glpi\\Asset\\Capacity\\HasVolumesCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/HasVolumesCapacity.php',
    'Glpi\\Asset\\Capacity\\IsInventoriableCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/IsInventoriableCapacity.php',
    'Glpi\\Asset\\Capacity\\IsProjectAssetCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/IsProjectAssetCapacity.php',
    'Glpi\\Asset\\Capacity\\IsRackableCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/IsRackableCapacity.php',
    'Glpi\\Asset\\Capacity\\IsReservableCapacity' => $baseDir . '/src/Glpi/Asset/Capacity/IsReservableCapacity.php',
    'Glpi\\Asset\\CustomFieldDefinition' => $baseDir . '/src/Glpi/Asset/CustomFieldDefinition.php',
    'Glpi\\Asset\\CustomFieldOption\\AbstractOption' => $baseDir . '/src/Glpi/Asset/CustomFieldOption/AbstractOption.php',
    'Glpi\\Asset\\CustomFieldOption\\BooleanOption' => $baseDir . '/src/Glpi/Asset/CustomFieldOption/BooleanOption.php',
    'Glpi\\Asset\\CustomFieldOption\\NumberOption' => $baseDir . '/src/Glpi/Asset/CustomFieldOption/NumberOption.php',
    'Glpi\\Asset\\CustomFieldOption\\OptionInterface' => $baseDir . '/src/Glpi/Asset/CustomFieldOption/OptionInterface.php',
    'Glpi\\Asset\\CustomFieldOption\\ProfileRestrictOption' => $baseDir . '/src/Glpi/Asset/CustomFieldOption/ProfileRestrictOption.php',
    'Glpi\\Asset\\CustomFieldType\\AbstractType' => $baseDir . '/src/Glpi/Asset/CustomFieldType/AbstractType.php',
    'Glpi\\Asset\\CustomFieldType\\BooleanType' => $baseDir . '/src/Glpi/Asset/CustomFieldType/BooleanType.php',
    'Glpi\\Asset\\CustomFieldType\\DateTimeType' => $baseDir . '/src/Glpi/Asset/CustomFieldType/DateTimeType.php',
    'Glpi\\Asset\\CustomFieldType\\DateType' => $baseDir . '/src/Glpi/Asset/CustomFieldType/DateType.php',
    'Glpi\\Asset\\CustomFieldType\\DropdownType' => $baseDir . '/src/Glpi/Asset/CustomFieldType/DropdownType.php',
    'Glpi\\Asset\\CustomFieldType\\NumberType' => $baseDir . '/src/Glpi/Asset/CustomFieldType/NumberType.php',
    'Glpi\\Asset\\CustomFieldType\\StringType' => $baseDir . '/src/Glpi/Asset/CustomFieldType/StringType.php',
    'Glpi\\Asset\\CustomFieldType\\TextType' => $baseDir . '/src/Glpi/Asset/CustomFieldType/TextType.php',
    'Glpi\\Asset\\CustomFieldType\\TypeInterface' => $baseDir . '/src/Glpi/Asset/CustomFieldType/TypeInterface.php',
    'Glpi\\Asset\\CustomFieldType\\URLType' => $baseDir . '/src/Glpi/Asset/CustomFieldType/URLType.php',
    'Glpi\\Asset\\RuleDictionaryModel' => $baseDir . '/src/Glpi/Asset/RuleDictionaryModel.php',
    'Glpi\\Asset\\RuleDictionaryModelCollection' => $baseDir . '/src/Glpi/Asset/RuleDictionaryModelCollection.php',
    'Glpi\\Asset\\RuleDictionaryType' => $baseDir . '/src/Glpi/Asset/RuleDictionaryType.php',
    'Glpi\\Asset\\RuleDictionaryTypeCollection' => $baseDir . '/src/Glpi/Asset/RuleDictionaryTypeCollection.php',
    'Glpi\\Cache\\CacheManager' => $baseDir . '/src/Glpi/Cache/CacheManager.php',
    'Glpi\\Cache\\I18nCache' => $baseDir . '/src/Glpi/Cache/I18nCache.php',
    'Glpi\\Cache\\SimpleCache' => $baseDir . '/src/Glpi/Cache/SimpleCache.php',
    'Glpi\\CalDAV\\Backend\\Auth' => $baseDir . '/src/Glpi/CalDAV/Backend/Auth.php',
    'Glpi\\CalDAV\\Backend\\Calendar' => $baseDir . '/src/Glpi/CalDAV/Backend/Calendar.php',
    'Glpi\\CalDAV\\Backend\\Principal' => $baseDir . '/src/Glpi/CalDAV/Backend/Principal.php',
    'Glpi\\CalDAV\\Contracts\\CalDAVCompatibleItemInterface' => $baseDir . '/src/Glpi/CalDAV/Contracts/CalDAVCompatibleItemInterface.php',
    'Glpi\\CalDAV\\Node\\CalendarRoot' => $baseDir . '/src/Glpi/CalDAV/Node/CalendarRoot.php',
    'Glpi\\CalDAV\\Node\\Property' => $baseDir . '/src/Glpi/CalDAV/Node/Property.php',
    'Glpi\\CalDAV\\Plugin\\Acl' => $baseDir . '/src/Glpi/CalDAV/Plugin/Acl.php',
    'Glpi\\CalDAV\\Plugin\\Browser' => $baseDir . '/src/Glpi/CalDAV/Plugin/Browser.php',
    'Glpi\\CalDAV\\Plugin\\CalDAV' => $baseDir . '/src/Glpi/CalDAV/Plugin/CalDAV.php',
    'Glpi\\CalDAV\\Server' => $baseDir . '/src/Glpi/CalDAV/Server.php',
    'Glpi\\CalDAV\\Traits\\CalDAVPrincipalsTrait' => $baseDir . '/src/Glpi/CalDAV/Traits/CalDAVPrincipalsTrait.php',
    'Glpi\\CalDAV\\Traits\\CalDAVUriUtilTrait' => $baseDir . '/src/Glpi/CalDAV/Traits/CalDAVUriUtilTrait.php',
    'Glpi\\CalDAV\\Traits\\VobjectConverterTrait' => $baseDir . '/src/Glpi/CalDAV/Traits/VobjectConverterTrait.php',
    'Glpi\\Config\\ProxyExclusion' => $baseDir . '/src/Glpi/Config/ProxyExclusion.php',
    'Glpi\\Config\\ProxyExclusions' => $baseDir . '/src/Glpi/Config/ProxyExclusions.php',
    'Glpi\\Console\\AbstractCommand' => $baseDir . '/src/Glpi/Console/AbstractCommand.php',
    'Glpi\\Console\\Application' => $baseDir . '/src/Glpi/Console/Application.php',
    'Glpi\\Console\\Assets\\CleanSoftwareCommand' => $baseDir . '/src/Glpi/Console/Assets/CleanSoftwareCommand.php',
    'Glpi\\Console\\Assets\\PurgeSoftwareCommand' => $baseDir . '/src/Glpi/Console/Assets/PurgeSoftwareCommand.php',
    'Glpi\\Console\\Build\\CompileScssCommand' => $baseDir . '/src/Glpi/Console/Build/CompileScssCommand.php',
    'Glpi\\Console\\Build\\GenerateCodeManifestCommand' => $baseDir . '/src/Glpi/Console/Build/GenerateCodeManifestCommand.php',
    'Glpi\\Console\\Cache\\ClearCommand' => $baseDir . '/src/Glpi/Console/Cache/ClearCommand.php',
    'Glpi\\Console\\Cache\\ConfigureCommand' => $baseDir . '/src/Glpi/Console/Cache/ConfigureCommand.php',
    'Glpi\\Console\\Cache\\DebugCommand' => $baseDir . '/src/Glpi/Console/Cache/DebugCommand.php',
    'Glpi\\Console\\Cache\\SetNamespacePrefixCommand' => $baseDir . '/src/Glpi/Console/Cache/SetNamespacePrefixCommand.php',
    'Glpi\\Console\\CommandLoader' => $baseDir . '/src/Glpi/Console/CommandLoader.php',
    'Glpi\\Console\\Command\\ConfigurationCommandInterface' => $baseDir . '/src/Glpi/Console/Command/ConfigurationCommandInterface.php',
    'Glpi\\Console\\Command\\GlpiCommandInterface' => $baseDir . '/src/Glpi/Console/Command/GlpiCommandInterface.php',
    'Glpi\\Console\\Config\\SetCommand' => $baseDir . '/src/Glpi/Console/Config/SetCommand.php',
    'Glpi\\Console\\Database\\AbstractConfigureCommand' => $baseDir . '/src/Glpi/Console/Database/AbstractConfigureCommand.php',
    'Glpi\\Console\\Database\\CheckSchemaIntegrityCommand' => $baseDir . '/src/Glpi/Console/Database/CheckSchemaIntegrityCommand.php',
    'Glpi\\Console\\Database\\ConfigureCommand' => $baseDir . '/src/Glpi/Console/Database/ConfigureCommand.php',
    'Glpi\\Console\\Database\\EnableTimezonesCommand' => $baseDir . '/src/Glpi/Console/Database/EnableTimezonesCommand.php',
    'Glpi\\Console\\Database\\InstallCommand' => $baseDir . '/src/Glpi/Console/Database/InstallCommand.php',
    'Glpi\\Console\\Database\\UpdateCommand' => $baseDir . '/src/Glpi/Console/Database/UpdateCommand.php',
    'Glpi\\Console\\Diagnostic\\CheckDocumentsIntegrityCommand' => $baseDir . '/src/Glpi/Console/Diagnostic/CheckDocumentsIntegrityCommand.php',
    'Glpi\\Console\\Diagnostic\\CheckHtmlEncodingCommand' => $baseDir . '/src/Glpi/Console/Diagnostic/CheckHtmlEncodingCommand.php',
    'Glpi\\Console\\Diagnostic\\CheckSourceCodeIntegrityCommand' => $baseDir . '/src/Glpi/Console/Diagnostic/CheckSourceCodeIntegrityCommand.php',
    'Glpi\\Console\\Exception\\EarlyExitException' => $baseDir . '/src/Glpi/Console/Exception/EarlyExitException.php',
    'Glpi\\Console\\Ldap\\SynchronizeUsersCommand' => $baseDir . '/src/Glpi/Console/Ldap/SynchronizeUsersCommand.php',
    'Glpi\\Console\\Maintenance\\DisableMaintenanceModeCommand' => $baseDir . '/src/Glpi/Console/Maintenance/DisableMaintenanceModeCommand.php',
    'Glpi\\Console\\Maintenance\\EnableMaintenanceModeCommand' => $baseDir . '/src/Glpi/Console/Maintenance/EnableMaintenanceModeCommand.php',
    'Glpi\\Console\\Marketplace\\AbstractMarketplaceCommand' => $baseDir . '/src/Glpi/Console/Marketplace/AbstractMarketplaceCommand.php',
    'Glpi\\Console\\Marketplace\\DownloadCommand' => $baseDir . '/src/Glpi/Console/Marketplace/DownloadCommand.php',
    'Glpi\\Console\\Marketplace\\InfoCommand' => $baseDir . '/src/Glpi/Console/Marketplace/InfoCommand.php',
    'Glpi\\Console\\Marketplace\\SearchCommand' => $baseDir . '/src/Glpi/Console/Marketplace/SearchCommand.php',
    'Glpi\\Console\\Marketplace\\UpgradeCommand' => $baseDir . '/src/Glpi/Console/Marketplace/UpgradeCommand.php',
    'Glpi\\Console\\Migration\\AbstractPluginMigrationCommand' => $baseDir . '/src/Glpi/Console/Migration/AbstractPluginMigrationCommand.php',
    'Glpi\\Console\\Migration\\AbstractPluginToCoreCommand' => $baseDir . '/src/Glpi/Console/Migration/AbstractPluginToCoreCommand.php',
    'Glpi\\Console\\Migration\\AppliancesPluginToCoreCommand' => $baseDir . '/src/Glpi/Console/Migration/AppliancesPluginToCoreCommand.php',
    'Glpi\\Console\\Migration\\BuildMissingTimestampsCommand' => $baseDir . '/src/Glpi/Console/Migration/BuildMissingTimestampsCommand.php',
    'Glpi\\Console\\Migration\\DatabasesPluginToCoreCommand' => $baseDir . '/src/Glpi/Console/Migration/DatabasesPluginToCoreCommand.php',
    'Glpi\\Console\\Migration\\DomainsPluginToCoreCommand' => $baseDir . '/src/Glpi/Console/Migration/DomainsPluginToCoreCommand.php',
    'Glpi\\Console\\Migration\\DynamicRowFormatCommand' => $baseDir . '/src/Glpi/Console/Migration/DynamicRowFormatCommand.php',
    'Glpi\\Console\\Migration\\FormCreatorPluginToCoreCommand' => $baseDir . '/src/Glpi/Console/Migration/FormCreatorPluginToCoreCommand.php',
    'Glpi\\Console\\Migration\\GenericobjectPluginToCoreCommand' => $baseDir . '/src/Glpi/Console/Migration/GenericobjectPluginToCoreCommand.php',
    'Glpi\\Console\\Migration\\MigrateAllCommand' => $baseDir . '/src/Glpi/Console/Migration/MigrateAllCommand.php',
    'Glpi\\Console\\Migration\\MyIsamToInnoDbCommand' => $baseDir . '/src/Glpi/Console/Migration/MyIsamToInnoDbCommand.php',
    'Glpi\\Console\\Migration\\RacksPluginToCoreCommand' => $baseDir . '/src/Glpi/Console/Migration/RacksPluginToCoreCommand.php',
    'Glpi\\Console\\Migration\\TimestampsCommand' => $baseDir . '/src/Glpi/Console/Migration/TimestampsCommand.php',
    'Glpi\\Console\\Migration\\UnsignedKeysCommand' => $baseDir . '/src/Glpi/Console/Migration/UnsignedKeysCommand.php',
    'Glpi\\Console\\Migration\\Utf8mb4Command' => $baseDir . '/src/Glpi/Console/Migration/Utf8mb4Command.php',
    'Glpi\\Console\\Plugin\\AbstractPluginCommand' => $baseDir . '/src/Glpi/Console/Plugin/AbstractPluginCommand.php',
    'Glpi\\Console\\Plugin\\ActivateCommand' => $baseDir . '/src/Glpi/Console/Plugin/ActivateCommand.php',
    'Glpi\\Console\\Plugin\\DeactivateCommand' => $baseDir . '/src/Glpi/Console/Plugin/DeactivateCommand.php',
    'Glpi\\Console\\Plugin\\InstallCommand' => $baseDir . '/src/Glpi/Console/Plugin/InstallCommand.php',
    'Glpi\\Console\\Plugin\\ListCommand' => $baseDir . '/src/Glpi/Console/Plugin/ListCommand.php',
    'Glpi\\Console\\Plugin\\ResumeExecutionCommand' => $baseDir . '/src/Glpi/Console/Plugin/ResumeExecutionCommand.php',
    'Glpi\\Console\\Plugin\\SuspendExecutionCommand' => $baseDir . '/src/Glpi/Console/Plugin/SuspendExecutionCommand.php',
    'Glpi\\Console\\Plugin\\UninstallCommand' => $baseDir . '/src/Glpi/Console/Plugin/UninstallCommand.php',
    'Glpi\\Console\\Rules\\ProcessSoftwareCategoryRulesCommand' => $baseDir . '/src/Glpi/Console/Rules/ProcessSoftwareCategoryRulesCommand.php',
    'Glpi\\Console\\Rules\\ReplayDictionnaryRulesCommand' => $baseDir . '/src/Glpi/Console/Rules/ReplayDictionnaryRulesCommand.php',
    'Glpi\\Console\\Security\\ChangekeyCommand' => $baseDir . '/src/Glpi/Console/Security/ChangekeyCommand.php',
    'Glpi\\Console\\Security\\DisableTFACommand' => $baseDir . '/src/Glpi/Console/Security/DisableTFACommand.php',
    'Glpi\\Console\\System\\CheckRequirementsCommand' => $baseDir . '/src/Glpi/Console/System/CheckRequirementsCommand.php',
    'Glpi\\Console\\System\\CheckStatusCommand' => $baseDir . '/src/Glpi/Console/System/CheckStatusCommand.php',
    'Glpi\\Console\\System\\ListServicesCommand' => $baseDir . '/src/Glpi/Console/System/ListServicesCommand.php',
    'Glpi\\Console\\Task\\UnlockCommand' => $baseDir . '/src/Glpi/Console/Task/UnlockCommand.php',
    'Glpi\\Console\\Traits\\PluginMigrationTrait' => $baseDir . '/src/Glpi/Console/Traits/PluginMigrationTrait.php',
    'Glpi\\Console\\Traits\\TelemetryActivationTrait' => $baseDir . '/src/Glpi/Console/Traits/TelemetryActivationTrait.php',
    'Glpi\\Console\\User\\AbstractUserCommand' => $baseDir . '/src/Glpi/Console/User/AbstractUserCommand.php',
    'Glpi\\Console\\User\\CreateCommand' => $baseDir . '/src/Glpi/Console/User/CreateCommand.php',
    'Glpi\\Console\\User\\DeleteCommand' => $baseDir . '/src/Glpi/Console/User/DeleteCommand.php',
    'Glpi\\Console\\User\\DisableCommand' => $baseDir . '/src/Glpi/Console/User/DisableCommand.php',
    'Glpi\\Console\\User\\EnableCommand' => $baseDir . '/src/Glpi/Console/User/EnableCommand.php',
    'Glpi\\Console\\User\\GrantCommand' => $baseDir . '/src/Glpi/Console/User/GrantCommand.php',
    'Glpi\\Console\\User\\ResetPasswordCommand' => $baseDir . '/src/Glpi/Console/User/ResetPasswordCommand.php',
    'Glpi\\ContentTemplates\\ParametersPreset' => $baseDir . '/src/Glpi/ContentTemplates/ParametersPreset.php',
    'Glpi\\ContentTemplates\\Parameters\\AbstractParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/AbstractParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\AssetParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/AssetParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\ChangeParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/ChangeParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\CommonITILObjectParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/CommonITILObjectParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\DropdownParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/DropdownParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\EntityParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/EntityParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\GroupParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/GroupParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\ITILCategoryParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/ITILCategoryParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\KnowbaseItemParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/KnowbaseItemParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\LevelAgreementParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/LevelAgreementParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\LocationParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/LocationParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\OLAParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/OLAParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\ParametersTypes\\AbstractParameterType' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/ParametersTypes/AbstractParameterType.php',
    'Glpi\\ContentTemplates\\Parameters\\ParametersTypes\\ArrayParameter' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/ParametersTypes/ArrayParameter.php',
    'Glpi\\ContentTemplates\\Parameters\\ParametersTypes\\AttributeParameter' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/ParametersTypes/AttributeParameter.php',
    'Glpi\\ContentTemplates\\Parameters\\ParametersTypes\\ObjectParameter' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/ParametersTypes/ObjectParameter.php',
    'Glpi\\ContentTemplates\\Parameters\\ParametersTypes\\ParameterTypeInterface' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/ParametersTypes/ParameterTypeInterface.php',
    'Glpi\\ContentTemplates\\Parameters\\ProblemParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/ProblemParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\RequestTypeParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/RequestTypeParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\SLAParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/SLAParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\SupplierParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/SupplierParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\TemplatesParametersInterface' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/TemplatesParametersInterface.php',
    'Glpi\\ContentTemplates\\Parameters\\TicketParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/TicketParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\TreeDropdownParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/TreeDropdownParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\UserCategoryParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/UserCategoryParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\UserParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/UserParameters.php',
    'Glpi\\ContentTemplates\\Parameters\\UserTitleParameters' => $baseDir . '/src/Glpi/ContentTemplates/Parameters/UserTitleParameters.php',
    'Glpi\\ContentTemplates\\TemplateDocumentation' => $baseDir . '/src/Glpi/ContentTemplates/TemplateDocumentation.php',
    'Glpi\\ContentTemplates\\TemplateManager' => $baseDir . '/src/Glpi/ContentTemplates/TemplateManager.php',
    'Glpi\\Controller\\AbstractController' => $baseDir . '/src/Glpi/Controller/AbstractController.php',
    'Glpi\\Controller\\Altcha\\ChallengeController' => $baseDir . '/src/Glpi/Controller/Altcha/ChallengeController.php',
    'Glpi\\Controller\\ApiController' => $baseDir . '/src/Glpi/Controller/ApiController.php',
    'Glpi\\Controller\\ApiRestController' => $baseDir . '/src/Glpi/Controller/ApiRestController.php',
    'Glpi\\Controller\\CaldavController' => $baseDir . '/src/Glpi/Controller/CaldavController.php',
    'Glpi\\Controller\\CentralController' => $baseDir . '/src/Glpi/Controller/CentralController.php',
    'Glpi\\Controller\\Config\\Helpdesk\\AbstractTileController' => $baseDir . '/src/Glpi/Controller/Config/Helpdesk/AbstractTileController.php',
    'Glpi\\Controller\\Config\\Helpdesk\\AddTileController' => $baseDir . '/src/Glpi/Controller/Config/Helpdesk/AddTileController.php',
    'Glpi\\Controller\\Config\\Helpdesk\\CopyParentEntityController' => $baseDir . '/src/Glpi/Controller/Config/Helpdesk/CopyParentEntityController.php',
    'Glpi\\Controller\\Config\\Helpdesk\\DeleteTileController' => $baseDir . '/src/Glpi/Controller/Config/Helpdesk/DeleteTileController.php',
    'Glpi\\Controller\\Config\\Helpdesk\\SetTilesOrderController' => $baseDir . '/src/Glpi/Controller/Config/Helpdesk/SetTilesOrderController.php',
    'Glpi\\Controller\\Config\\Helpdesk\\ShowAddTileFormController' => $baseDir . '/src/Glpi/Controller/Config/Helpdesk/ShowAddTileFormController.php',
    'Glpi\\Controller\\Config\\Helpdesk\\ShowEditTileFormController' => $baseDir . '/src/Glpi/Controller/Config/Helpdesk/ShowEditTileFormController.php',
    'Glpi\\Controller\\Config\\Helpdesk\\UpdateTileController' => $baseDir . '/src/Glpi/Controller/Config/Helpdesk/UpdateTileController.php',
    'Glpi\\Controller\\DropdownFormController' => $baseDir . '/src/Glpi/Controller/DropdownFormController.php',
    'Glpi\\Controller\\ErrorController' => $baseDir . '/src/Glpi/Controller/ErrorController.php',
    'Glpi\\Controller\\Form\\AllowListDropdown\\CountUsersController' => $baseDir . '/src/Glpi/Controller/Form/AllowListDropdown/CountUsersController.php',
    'Glpi\\Controller\\Form\\Condition\\EditorController' => $baseDir . '/src/Glpi/Controller/Form/Condition/EditorController.php',
    'Glpi\\Controller\\Form\\Condition\\EngineController' => $baseDir . '/src/Glpi/Controller/Form/Condition/EngineController.php',
    'Glpi\\Controller\\Form\\DelegationController' => $baseDir . '/src/Glpi/Controller/Form/DelegationController.php',
    'Glpi\\Controller\\Form\\Destination\\AddDestinationController' => $baseDir . '/src/Glpi/Controller/Form/Destination/AddDestinationController.php',
    'Glpi\\Controller\\Form\\Destination\\GetDestinationFormController' => $baseDir . '/src/Glpi/Controller/Form/Destination/GetDestinationFormController.php',
    'Glpi\\Controller\\Form\\Destination\\PurgeDestinationController' => $baseDir . '/src/Glpi/Controller/Form/Destination/PurgeDestinationController.php',
    'Glpi\\Controller\\Form\\Destination\\UpdateDestinationController' => $baseDir . '/src/Glpi/Controller/Form/Destination/UpdateDestinationController.php',
    'Glpi\\Controller\\Form\\ExportController' => $baseDir . '/src/Glpi/Controller/Form/ExportController.php',
    'Glpi\\Controller\\Form\\Import\\Step1IndexController' => $baseDir . '/src/Glpi/Controller/Form/Import/Step1IndexController.php',
    'Glpi\\Controller\\Form\\Import\\Step2PreviewController' => $baseDir . '/src/Glpi/Controller/Form/Import/Step2PreviewController.php',
    'Glpi\\Controller\\Form\\Import\\Step3ResolveIssuesController' => $baseDir . '/src/Glpi/Controller/Form/Import/Step3ResolveIssuesController.php',
    'Glpi\\Controller\\Form\\Import\\Step4ExecuteController' => $baseDir . '/src/Glpi/Controller/Form/Import/Step4ExecuteController.php',
    'Glpi\\Controller\\Form\\QuestionActorsDropdownController' => $baseDir . '/src/Glpi/Controller/Form/QuestionActorsDropdownController.php',
    'Glpi\\Controller\\Form\\RendererController' => $baseDir . '/src/Glpi/Controller/Form/RendererController.php',
    'Glpi\\Controller\\Form\\SubmitAnswerController' => $baseDir . '/src/Glpi/Controller/Form/SubmitAnswerController.php',
    'Glpi\\Controller\\Form\\TagListController' => $baseDir . '/src/Glpi/Controller/Form/TagListController.php',
    'Glpi\\Controller\\Form\\Translation\\AddNewFormTranslationController' => $baseDir . '/src/Glpi/Controller/Form/Translation/AddNewFormTranslationController.php',
    'Glpi\\Controller\\Form\\Translation\\DeleteFormTranslationController' => $baseDir . '/src/Glpi/Controller/Form/Translation/DeleteFormTranslationController.php',
    'Glpi\\Controller\\Form\\Translation\\UpdateFormTranslationController' => $baseDir . '/src/Glpi/Controller/Form/Translation/UpdateFormTranslationController.php',
    'Glpi\\Controller\\Form\\Utils\\CanCheckAccessPolicies' => $baseDir . '/src/Glpi/Controller/Form/Utils/CanCheckAccessPolicies.php',
    'Glpi\\Controller\\Form\\ValidateAnswerController' => $baseDir . '/src/Glpi/Controller/Form/ValidateAnswerController.php',
    'Glpi\\Controller\\GenericAjaxCrudController' => $baseDir . '/src/Glpi/Controller/GenericAjaxCrudController.php',
    'Glpi\\Controller\\GenericFormController' => $baseDir . '/src/Glpi/Controller/GenericFormController.php',
    'Glpi\\Controller\\GenericListController' => $baseDir . '/src/Glpi/Controller/GenericListController.php',
    'Glpi\\Controller\\Helpdesk\\IndexController' => $baseDir . '/src/Glpi/Controller/Helpdesk/IndexController.php',
    'Glpi\\Controller\\Helpdesk\\SearchController' => $baseDir . '/src/Glpi/Controller/Helpdesk/SearchController.php',
    'Glpi\\Controller\\Helpdesk\\Translation\\AddNewHelpdeskTranslationController' => $baseDir . '/src/Glpi/Controller/Helpdesk/Translation/AddNewHelpdeskTranslationController.php',
    'Glpi\\Controller\\Helpdesk\\Translation\\DeleteHelpdeskTranslationController' => $baseDir . '/src/Glpi/Controller/Helpdesk/Translation/DeleteHelpdeskTranslationController.php',
    'Glpi\\Controller\\Helpdesk\\Translation\\UpdateHelpdeskTranslationController' => $baseDir . '/src/Glpi/Controller/Helpdesk/Translation/UpdateHelpdeskTranslationController.php',
    'Glpi\\Controller\\IndexController' => $baseDir . '/src/Glpi/Controller/IndexController.php',
    'Glpi\\Controller\\InstallController' => $baseDir . '/src/Glpi/Controller/InstallController.php',
    'Glpi\\Controller\\InventoryController' => $baseDir . '/src/Glpi/Controller/InventoryController.php',
    'Glpi\\Controller\\ItemType\\Form\\AuthMailFormController' => $baseDir . '/src/Glpi/Controller/ItemType/Form/AuthMailFormController.php',
    'Glpi\\Controller\\ItemType\\Form\\ContactFormController' => $baseDir . '/src/Glpi/Controller/ItemType/Form/ContactFormController.php',
    'Glpi\\Controller\\ItemType\\Form\\MailCollectorFormController' => $baseDir . '/src/Glpi/Controller/ItemType/Form/MailCollectorFormController.php',
    'Glpi\\Controller\\ItemType\\Form\\SavedSearchFormController' => $baseDir . '/src/Glpi/Controller/ItemType/Form/SavedSearchFormController.php',
    'Glpi\\Controller\\Knowbase\\KnowbaseItemController' => $baseDir . '/src/Glpi/Controller/Knowbase/KnowbaseItemController.php',
    'Glpi\\Controller\\LegacyFileLoadController' => $baseDir . '/src/Glpi/Controller/LegacyFileLoadController.php',
    'Glpi\\Controller\\MaintenanceController' => $baseDir . '/src/Glpi/Controller/MaintenanceController.php',
    'Glpi\\Controller\\Plugin\\LogoController' => $baseDir . '/src/Glpi/Controller/Plugin/LogoController.php',
    'Glpi\\Controller\\ProgressController' => $baseDir . '/src/Glpi/Controller/ProgressController.php',
    'Glpi\\Controller\\RequestException' => $baseDir . '/src/Glpi/Controller/RequestException.php',
    'Glpi\\Controller\\Security\\MFAController' => $baseDir . '/src/Glpi/Controller/Security/MFAController.php',
    'Glpi\\Controller\\ServiceCatalog\\IndexController' => $baseDir . '/src/Glpi/Controller/ServiceCatalog/IndexController.php',
    'Glpi\\Controller\\ServiceCatalog\\ItemsController' => $baseDir . '/src/Glpi/Controller/ServiceCatalog/ItemsController.php',
    'Glpi\\Controller\\Session\\ChangeEntityController' => $baseDir . '/src/Glpi/Controller/Session/ChangeEntityController.php',
    'Glpi\\Controller\\Session\\ChangeProfileController' => $baseDir . '/src/Glpi/Controller/Session/ChangeProfileController.php',
    'Glpi\\Controller\\StatusController' => $baseDir . '/src/Glpi/Controller/StatusController.php',
    'Glpi\\Controller\\Traits\\AsyncOperationProgressControllerTrait' => $baseDir . '/src/Glpi/Controller/Traits/AsyncOperationProgressControllerTrait.php',
    'Glpi\\Controller\\Translation\\AbstractTranslationController' => $baseDir . '/src/Glpi/Controller/Translation/AbstractTranslationController.php',
    'Glpi\\Controller\\UI\\Illustration\\CustomIllustrationController' => $baseDir . '/src/Glpi/Controller/UI/Illustration/CustomIllustrationController.php',
    'Glpi\\Controller\\UI\\Illustration\\CustomSceneController' => $baseDir . '/src/Glpi/Controller/UI/Illustration/CustomSceneController.php',
    'Glpi\\Controller\\UI\\Illustration\\SearchController' => $baseDir . '/src/Glpi/Controller/UI/Illustration/SearchController.php',
    'Glpi\\Controller\\UI\\Illustration\\UploadController' => $baseDir . '/src/Glpi/Controller/UI/Illustration/UploadController.php',
    'Glpi\\Controller\\WellKnownController' => $baseDir . '/src/Glpi/Controller/WellKnownController.php',
    'Glpi\\Csv\\CsvResponse' => $baseDir . '/src/Glpi/Csv/CsvResponse.php',
    'Glpi\\Csv\\ExportToCsvInterface' => $baseDir . '/src/Glpi/Csv/ExportToCsvInterface.php',
    'Glpi\\Csv\\ImpactCsvExport' => $baseDir . '/src/Glpi/Csv/ImpactCsvExport.php',
    'Glpi\\Csv\\LogCsvExport' => $baseDir . '/src/Glpi/Csv/LogCsvExport.php',
    'Glpi\\Csv\\PlanningCsv' => $baseDir . '/src/Glpi/Csv/PlanningCsv.php',
    'Glpi\\Csv\\PrinterLogCsvExport' => $baseDir . '/src/Glpi/Csv/PrinterLogCsvExport.php',
    'Glpi\\Csv\\PrinterLogCsvExportComparison' => $baseDir . '/src/Glpi/Csv/PrinterLogCsvExportComparison.php',
    'Glpi\\Csv\\StatCsvExport' => $baseDir . '/src/Glpi/Csv/StatCsvExport.php',
    'Glpi\\CustomObject\\AbstractDefinition' => $baseDir . '/src/Glpi/CustomObject/AbstractDefinition.php',
    'Glpi\\CustomObject\\AbstractDefinitionManager' => $baseDir . '/src/Glpi/CustomObject/AbstractDefinitionManager.php',
    'Glpi\\CustomObject\\CustomObjectTrait' => $baseDir . '/src/Glpi/CustomObject/CustomObjectTrait.php',
    'Glpi\\DBAL\\JsonFieldInterface' => $baseDir . '/src/Glpi/DBAL/JsonFieldInterface.php',
    'Glpi\\DBAL\\PrepareForCloneInterface' => $baseDir . '/src/Glpi/DBAL/PrepareForCloneInterface.php',
    'Glpi\\DBAL\\QueryExpression' => $baseDir . '/src/Glpi/DBAL/QueryExpression.php',
    'Glpi\\DBAL\\QueryFunction' => $baseDir . '/src/Glpi/DBAL/QueryFunction.php',
    'Glpi\\DBAL\\QueryParam' => $baseDir . '/src/Glpi/DBAL/QueryParam.php',
    'Glpi\\DBAL\\QuerySubQuery' => $baseDir . '/src/Glpi/DBAL/QuerySubQuery.php',
    'Glpi\\DBAL\\QueryUnion' => $baseDir . '/src/Glpi/DBAL/QueryUnion.php',
    'Glpi\\Dashboard\\Dashboard' => $baseDir . '/src/Glpi/Dashboard/Dashboard.php',
    'Glpi\\Dashboard\\FakeProvider' => $baseDir . '/src/Glpi/Dashboard/FakeProvider.php',
    'Glpi\\Dashboard\\Filter' => $baseDir . '/src/Glpi/Dashboard/Filter.php',
    'Glpi\\Dashboard\\Filters\\AbstractFilter' => $baseDir . '/src/Glpi/Dashboard/Filters/AbstractFilter.php',
    'Glpi\\Dashboard\\Filters\\AbstractGroupFilter' => $baseDir . '/src/Glpi/Dashboard/Filters/AbstractGroupFilter.php',
    'Glpi\\Dashboard\\Filters\\DatesFilter' => $baseDir . '/src/Glpi/Dashboard/Filters/DatesFilter.php',
    'Glpi\\Dashboard\\Filters\\DatesModFilter' => $baseDir . '/src/Glpi/Dashboard/Filters/DatesModFilter.php',
    'Glpi\\Dashboard\\Filters\\GroupRequesterFilter' => $baseDir . '/src/Glpi/Dashboard/Filters/GroupRequesterFilter.php',
    'Glpi\\Dashboard\\Filters\\GroupTechFilter' => $baseDir . '/src/Glpi/Dashboard/Filters/GroupTechFilter.php',
    'Glpi\\Dashboard\\Filters\\ItilCategoryFilter' => $baseDir . '/src/Glpi/Dashboard/Filters/ItilCategoryFilter.php',
    'Glpi\\Dashboard\\Filters\\LocationFilter' => $baseDir . '/src/Glpi/Dashboard/Filters/LocationFilter.php',
    'Glpi\\Dashboard\\Filters\\ManufacturerFilter' => $baseDir . '/src/Glpi/Dashboard/Filters/ManufacturerFilter.php',
    'Glpi\\Dashboard\\Filters\\RequestTypeFilter' => $baseDir . '/src/Glpi/Dashboard/Filters/RequestTypeFilter.php',
    'Glpi\\Dashboard\\Filters\\StateFilter' => $baseDir . '/src/Glpi/Dashboard/Filters/StateFilter.php',
    'Glpi\\Dashboard\\Filters\\TicketTypeFilter' => $baseDir . '/src/Glpi/Dashboard/Filters/TicketTypeFilter.php',
    'Glpi\\Dashboard\\Filters\\UserTechFilter' => $baseDir . '/src/Glpi/Dashboard/Filters/UserTechFilter.php',
    'Glpi\\Dashboard\\Grid' => $baseDir . '/src/Glpi/Dashboard/Grid.php',
    'Glpi\\Dashboard\\Item' => $baseDir . '/src/Glpi/Dashboard/Item.php',
    'Glpi\\Dashboard\\Palette' => $baseDir . '/src/Glpi/Dashboard/Palette.php',
    'Glpi\\Dashboard\\Provider' => $baseDir . '/src/Glpi/Dashboard/Provider.php',
    'Glpi\\Dashboard\\Right' => $baseDir . '/src/Glpi/Dashboard/Right.php',
    'Glpi\\Dashboard\\Widget' => $baseDir . '/src/Glpi/Dashboard/Widget.php',
    'Glpi\\Debug\\Profile' => $baseDir . '/src/Glpi/Debug/Profile.php',
    'Glpi\\Debug\\Profiler' => $baseDir . '/src/Glpi/Debug/Profiler.php',
    'Glpi\\Debug\\ProfilerSection' => $baseDir . '/src/Glpi/Debug/ProfilerSection.php',
    'Glpi\\DependencyInjection\\PluginContainer' => $baseDir . '/src/Glpi/DependencyInjection/PluginContainer.php',
    'Glpi\\DependencyInjection\\PublicService' => $baseDir . '/src/Glpi/DependencyInjection/PublicService.php',
    'Glpi\\Dropdown\\Dropdown' => $baseDir . '/src/Glpi/Dropdown/Dropdown.php',
    'Glpi\\Dropdown\\DropdownDefinition' => $baseDir . '/src/Glpi/Dropdown/DropdownDefinition.php',
    'Glpi\\Dropdown\\DropdownDefinitionManager' => $baseDir . '/src/Glpi/Dropdown/DropdownDefinitionManager.php',
    'Glpi\\Error\\ErrorDisplayHandler\\CliDisplayHandler' => $baseDir . '/src/Glpi/Error/ErrorDisplayHandler/CliDisplayHandler.php',
    'Glpi\\Error\\ErrorDisplayHandler\\ConsoleErrorDisplayHandler' => $baseDir . '/src/Glpi/Error/ErrorDisplayHandler/ConsoleErrorDisplayHandler.php',
    'Glpi\\Error\\ErrorDisplayHandler\\ErrorDisplayHandler' => $baseDir . '/src/Glpi/Error/ErrorDisplayHandler/ErrorDisplayHandler.php',
    'Glpi\\Error\\ErrorDisplayHandler\\HtmlErrorDisplayHandler' => $baseDir . '/src/Glpi/Error/ErrorDisplayHandler/HtmlErrorDisplayHandler.php',
    'Glpi\\Error\\ErrorHandler' => $baseDir . '/src/Glpi/Error/ErrorHandler.php',
    'Glpi\\Error\\ErrorUtils' => $baseDir . '/src/Glpi/Error/ErrorUtils.php',
    'Glpi\\Event' => $baseDir . '/src/Glpi/Event.php',
    'Glpi\\Exception\\AuthenticationFailedException' => $baseDir . '/src/Glpi/Exception/AuthenticationFailedException.php',
    'Glpi\\Exception\\Database\\StatementException' => $baseDir . '/src/Glpi/Exception/Database/StatementException.php',
    'Glpi\\Exception\\EmptyCurlContentException' => $baseDir . '/src/Glpi/Exception/EmptyCurlContentException.php',
    'Glpi\\Exception\\ForgetPasswordException' => $baseDir . '/src/Glpi/Exception/ForgetPasswordException.php',
    'Glpi\\Exception\\Http\\AccessDeniedHttpException' => $baseDir . '/src/Glpi/Exception/Http/AccessDeniedHttpException.php',
    'Glpi\\Exception\\Http\\BadRequestHttpException' => $baseDir . '/src/Glpi/Exception/Http/BadRequestHttpException.php',
    'Glpi\\Exception\\Http\\HttpException' => $baseDir . '/src/Glpi/Exception/Http/HttpException.php',
    'Glpi\\Exception\\Http\\HttpExceptionInterface' => $baseDir . '/src/Glpi/Exception/Http/HttpExceptionInterface.php',
    'Glpi\\Exception\\Http\\HttpExceptionTrait' => $baseDir . '/src/Glpi/Exception/Http/HttpExceptionTrait.php',
    'Glpi\\Exception\\Http\\NotFoundHttpException' => $baseDir . '/src/Glpi/Exception/Http/NotFoundHttpException.php',
    'Glpi\\Exception\\Http\\UnprocessableEntityHttpException' => $baseDir . '/src/Glpi/Exception/Http/UnprocessableEntityHttpException.php',
    'Glpi\\Exception\\ItemLinkException' => $baseDir . '/src/Glpi/Exception/ItemLinkException.php',
    'Glpi\\Exception\\OAuth2KeyException' => $baseDir . '/src/Glpi/Exception/OAuth2KeyException.php',
    'Glpi\\Exception\\PasswordTooWeakException' => $baseDir . '/src/Glpi/Exception/PasswordTooWeakException.php',
    'Glpi\\Exception\\RedirectException' => $baseDir . '/src/Glpi/Exception/RedirectException.php',
    'Glpi\\Exception\\SessionExpiredException' => $baseDir . '/src/Glpi/Exception/SessionExpiredException.php',
    'Glpi\\Exception\\TooManyResultsException' => $baseDir . '/src/Glpi/Exception/TooManyResultsException.php',
    'Glpi\\Features\\AssetImage' => $baseDir . '/src/Glpi/Features/AssetImage.php',
    'Glpi\\Features\\AssignableItem' => $baseDir . '/src/Glpi/Features/AssignableItem.php',
    'Glpi\\Features\\AssignableItemInterface' => $baseDir . '/src/Glpi/Features/AssignableItemInterface.php',
    'Glpi\\Features\\CacheableListInterface' => $baseDir . '/src/Glpi/Features/CacheableListInterface.php',
    'Glpi\\Features\\Clonable' => $baseDir . '/src/Glpi/Features/Clonable.php',
    'Glpi\\Features\\CloneMapper' => $baseDir . '/src/Glpi/Features/CloneMapper.php',
    'Glpi\\Features\\CloneWithoutNameSuffix' => $baseDir . '/src/Glpi/Features/CloneWithoutNameSuffix.php',
    'Glpi\\Features\\DCBreadcrumb' => $baseDir . '/src/Glpi/Features/DCBreadcrumb.php',
    'Glpi\\Features\\DCBreadcrumbInterface' => $baseDir . '/src/Glpi/Features/DCBreadcrumbInterface.php',
    'Glpi\\Features\\Inventoriable' => $baseDir . '/src/Glpi/Features/Inventoriable.php',
    'Glpi\\Features\\Kanban' => $baseDir . '/src/Glpi/Features/Kanban.php',
    'Glpi\\Features\\KanbanInterface' => $baseDir . '/src/Glpi/Features/KanbanInterface.php',
    'Glpi\\Features\\ParentStatus' => $baseDir . '/src/Glpi/Features/ParentStatus.php',
    'Glpi\\Features\\PlanningEvent' => $baseDir . '/src/Glpi/Features/PlanningEvent.php',
    'Glpi\\Features\\State' => $baseDir . '/src/Glpi/Features/State.php',
    'Glpi\\Features\\StateInterface' => $baseDir . '/src/Glpi/Features/StateInterface.php',
    'Glpi\\Features\\Teamwork' => $baseDir . '/src/Glpi/Features/Teamwork.php',
    'Glpi\\Features\\TeamworkInterface' => $baseDir . '/src/Glpi/Features/TeamworkInterface.php',
    'Glpi\\Features\\Timeline' => $baseDir . '/src/Glpi/Features/Timeline.php',
    'Glpi\\Features\\TreeBrowse' => $baseDir . '/src/Glpi/Features/TreeBrowse.php',
    'Glpi\\Features\\TreeBrowseInterface' => $baseDir . '/src/Glpi/Features/TreeBrowseInterface.php',
    'Glpi\\Features\\ZonableModelPicture' => $baseDir . '/src/Glpi/Features/ZonableModelPicture.php',
    'Glpi\\Form\\AccessControl\\AccessVote' => $baseDir . '/src/Glpi/Form/AccessControl/AccessVote.php',
    'Glpi\\Form\\AccessControl\\ControlType\\AllowList' => $baseDir . '/src/Glpi/Form/AccessControl/ControlType/AllowList.php',
    'Glpi\\Form\\AccessControl\\ControlType\\AllowListConfig' => $baseDir . '/src/Glpi/Form/AccessControl/ControlType/AllowListConfig.php',
    'Glpi\\Form\\AccessControl\\ControlType\\AllowListDropdown' => $baseDir . '/src/Glpi/Form/AccessControl/ControlType/AllowListDropdown.php',
    'Glpi\\Form\\AccessControl\\ControlType\\ControlTypeInterface' => $baseDir . '/src/Glpi/Form/AccessControl/ControlType/ControlTypeInterface.php',
    'Glpi\\Form\\AccessControl\\ControlType\\DirectAccess' => $baseDir . '/src/Glpi/Form/AccessControl/ControlType/DirectAccess.php',
    'Glpi\\Form\\AccessControl\\ControlType\\DirectAccessConfig' => $baseDir . '/src/Glpi/Form/AccessControl/ControlType/DirectAccessConfig.php',
    'Glpi\\Form\\AccessControl\\FormAccessControl' => $baseDir . '/src/Glpi/Form/AccessControl/FormAccessControl.php',
    'Glpi\\Form\\AccessControl\\FormAccessControlManager' => $baseDir . '/src/Glpi/Form/AccessControl/FormAccessControlManager.php',
    'Glpi\\Form\\AccessControl\\FormAccessParameters' => $baseDir . '/src/Glpi/Form/AccessControl/FormAccessParameters.php',
    'Glpi\\Form\\Answer' => $baseDir . '/src/Glpi/Form/Answer.php',
    'Glpi\\Form\\AnswersHandler\\AnswersHandler' => $baseDir . '/src/Glpi/Form/AnswersHandler/AnswersHandler.php',
    'Glpi\\Form\\AnswersSet' => $baseDir . '/src/Glpi/Form/AnswersSet.php',
    'Glpi\\Form\\BlockInterface' => $baseDir . '/src/Glpi/Form/BlockInterface.php',
    'Glpi\\Form\\Category' => $baseDir . '/src/Glpi/Form/Category.php',
    'Glpi\\Form\\Clone\\FormCloneHelper' => $baseDir . '/src/Glpi/Form/Clone/FormCloneHelper.php',
    'Glpi\\Form\\Clone\\MappedItemNotFoundException' => $baseDir . '/src/Glpi/Form/Clone/MappedItemNotFoundException.php',
    'Glpi\\Form\\Comment' => $baseDir . '/src/Glpi/Form/Comment.php',
    'Glpi\\Form\\Condition\\CommentData' => $baseDir . '/src/Glpi/Form/Condition/CommentData.php',
    'Glpi\\Form\\Condition\\ConditionData' => $baseDir . '/src/Glpi/Form/Condition/ConditionData.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\AbstractDateTimeConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/AbstractDateTimeConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\ActorConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/ActorConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\ArrayConditionHandlerTrait' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/ArrayConditionHandlerTrait.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\ConditionHandlerInterface' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/ConditionHandlerInterface.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\DateAndTimeConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/DateAndTimeConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\DateConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/DateConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\EmptyConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/EmptyConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\ItemAsTextConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/ItemAsTextConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\ItemConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/ItemConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\MultipleChoiceFromValuesConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/MultipleChoiceFromValuesConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\NumberConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/NumberConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\RegexConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/RegexConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\RequestTypeConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/RequestTypeConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\RichTextConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/RichTextConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\SingleChoiceFromValuesConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/SingleChoiceFromValuesConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\StringConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/StringConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\TimeConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/TimeConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\UrgencyConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/UrgencyConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\UserDevicesAsTextConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/UserDevicesAsTextConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\UserDevicesConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/UserDevicesConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionHandler\\VisibilityConditionHandler' => $baseDir . '/src/Glpi/Form/Condition/ConditionHandler/VisibilityConditionHandler.php',
    'Glpi\\Form\\Condition\\ConditionValueTransformerInterface' => $baseDir . '/src/Glpi/Form/Condition/ConditionValueTransformerInterface.php',
    'Glpi\\Form\\Condition\\ConditionableCreationInterface' => $baseDir . '/src/Glpi/Form/Condition/ConditionableCreationInterface.php',
    'Glpi\\Form\\Condition\\ConditionableCreationTrait' => $baseDir . '/src/Glpi/Form/Condition/ConditionableCreationTrait.php',
    'Glpi\\Form\\Condition\\ConditionableInterface' => $baseDir . '/src/Glpi/Form/Condition/ConditionableInterface.php',
    'Glpi\\Form\\Condition\\ConditionableTrait' => $baseDir . '/src/Glpi/Form/Condition/ConditionableTrait.php',
    'Glpi\\Form\\Condition\\ConditionableValidationTrait' => $baseDir . '/src/Glpi/Form/Condition/ConditionableValidationTrait.php',
    'Glpi\\Form\\Condition\\ConditionableVisibilityInterface' => $baseDir . '/src/Glpi/Form/Condition/ConditionableVisibilityInterface.php',
    'Glpi\\Form\\Condition\\ConditionableVisibilityTrait' => $baseDir . '/src/Glpi/Form/Condition/ConditionableVisibilityTrait.php',
    'Glpi\\Form\\Condition\\CreationStrategy' => $baseDir . '/src/Glpi/Form/Condition/CreationStrategy.php',
    'Glpi\\Form\\Condition\\EditorManager' => $baseDir . '/src/Glpi/Form/Condition/EditorManager.php',
    'Glpi\\Form\\Condition\\Engine' => $baseDir . '/src/Glpi/Form/Condition/Engine.php',
    'Glpi\\Form\\Condition\\EngineCreationOutput' => $baseDir . '/src/Glpi/Form/Condition/EngineCreationOutput.php',
    'Glpi\\Form\\Condition\\EngineInput' => $baseDir . '/src/Glpi/Form/Condition/EngineInput.php',
    'Glpi\\Form\\Condition\\EngineValidationOutput' => $baseDir . '/src/Glpi/Form/Condition/EngineValidationOutput.php',
    'Glpi\\Form\\Condition\\EngineVisibilityOutput' => $baseDir . '/src/Glpi/Form/Condition/EngineVisibilityOutput.php',
    'Glpi\\Form\\Condition\\FormData' => $baseDir . '/src/Glpi/Form/Condition/FormData.php',
    'Glpi\\Form\\Condition\\LogicOperator' => $baseDir . '/src/Glpi/Form/Condition/LogicOperator.php',
    'Glpi\\Form\\Condition\\QuestionData' => $baseDir . '/src/Glpi/Form/Condition/QuestionData.php',
    'Glpi\\Form\\Condition\\SectionData' => $baseDir . '/src/Glpi/Form/Condition/SectionData.php',
    'Glpi\\Form\\Condition\\StrategyInterface' => $baseDir . '/src/Glpi/Form/Condition/StrategyInterface.php',
    'Glpi\\Form\\Condition\\Type' => $baseDir . '/src/Glpi/Form/Condition/Type.php',
    'Glpi\\Form\\Condition\\UsedAsCriteriaInterface' => $baseDir . '/src/Glpi/Form/Condition/UsedAsCriteriaInterface.php',
    'Glpi\\Form\\Condition\\ValidationStrategy' => $baseDir . '/src/Glpi/Form/Condition/ValidationStrategy.php',
    'Glpi\\Form\\Condition\\ValueOperator' => $baseDir . '/src/Glpi/Form/Condition/ValueOperator.php',
    'Glpi\\Form\\Condition\\VisibilityStrategy' => $baseDir . '/src/Glpi/Form/Condition/VisibilityStrategy.php',
    'Glpi\\Form\\DelegationData' => $baseDir . '/src/Glpi/Form/DelegationData.php',
    'Glpi\\Form\\Destination\\AbstractCommonITILFormDestination' => $baseDir . '/src/Glpi/Form/Destination/AbstractCommonITILFormDestination.php',
    'Glpi\\Form\\Destination\\AbstractConfigField' => $baseDir . '/src/Glpi/Form/Destination/AbstractConfigField.php',
    'Glpi\\Form\\Destination\\AnswersSet_FormDestinationItem' => $baseDir . '/src/Glpi/Form/Destination/AnswersSet_FormDestinationItem.php',
    'Glpi\\Form\\Destination\\CommonITILField\\AbstractTextAreaField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/AbstractTextAreaField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\AssigneeField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/AssigneeField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\AssigneeFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/AssigneeFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\AssociatedItemsField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/AssociatedItemsField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\AssociatedItemsFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/AssociatedItemsFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\AssociatedItemsFieldStrategy' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/AssociatedItemsFieldStrategy.php',
    'Glpi\\Form\\Destination\\CommonITILField\\BackupPlanField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/BackupPlanField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\Category' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/Category.php',
    'Glpi\\Form\\Destination\\CommonITILField\\CausesField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/CausesField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\CheckListField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/CheckListField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ContentField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ContentField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ControlsListField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ControlsListField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\DeploymentPlanField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/DeploymentPlanField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\EntityField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/EntityField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\EntityFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/EntityFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\EntityFieldStrategy' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/EntityFieldStrategy.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ITILActorField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ITILActorField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ITILActorFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ITILActorFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ITILActorFieldStrategy' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ITILActorFieldStrategy.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ITILCategoryField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ITILCategoryField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ITILCategoryFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ITILCategoryFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ITILCategoryFieldStrategy' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ITILCategoryFieldStrategy.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ITILFollowupField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ITILFollowupField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ITILFollowupFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ITILFollowupFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ITILFollowupFieldStrategy' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ITILFollowupFieldStrategy.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ITILTaskField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ITILTaskField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ITILTaskFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ITILTaskFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ITILTaskFieldStrategy' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ITILTaskFieldStrategy.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ImpactsField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ImpactsField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\LinkedITILObjectsField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/LinkedITILObjectsField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\LinkedITILObjectsFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/LinkedITILObjectsFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\LinkedITILObjectsFieldStrategy' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/LinkedITILObjectsFieldStrategy.php',
    'Glpi\\Form\\Destination\\CommonITILField\\LinkedITILObjectsFieldStrategyConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/LinkedITILObjectsFieldStrategyConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\LocationField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/LocationField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\LocationFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/LocationFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\LocationFieldStrategy' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/LocationFieldStrategy.php',
    'Glpi\\Form\\Destination\\CommonITILField\\OLATTOField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/OLATTOField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\OLATTOFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/OLATTOFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\OLATTRField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/OLATTRField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\OLATTRFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/OLATTRFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ObserverField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ObserverField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ObserverFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ObserverFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\RequestSourceField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/RequestSourceField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\RequestSourceFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/RequestSourceFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\RequestSourceFieldStrategy' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/RequestSourceFieldStrategy.php',
    'Glpi\\Form\\Destination\\CommonITILField\\RequestTypeField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/RequestTypeField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\RequestTypeFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/RequestTypeFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\RequestTypeFieldStrategy' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/RequestTypeFieldStrategy.php',
    'Glpi\\Form\\Destination\\CommonITILField\\RequesterField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/RequesterField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\RequesterFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/RequesterFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\SLATTOField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/SLATTOField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\SLATTOFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/SLATTOFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\SLATTRField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/SLATTRField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\SLATTRFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/SLATTRFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\SLMField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/SLMField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\SLMFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/SLMFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\SLMFieldStrategy' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/SLMFieldStrategy.php',
    'Glpi\\Form\\Destination\\CommonITILField\\SimpleValueConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/SimpleValueConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\StatusField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/StatusField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\SymptomsField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/SymptomsField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\TemplateField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/TemplateField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\TemplateFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/TemplateFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\TemplateFieldStrategy' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/TemplateFieldStrategy.php',
    'Glpi\\Form\\Destination\\CommonITILField\\TitleField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/TitleField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\UrgencyField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/UrgencyField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\UrgencyFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/UrgencyFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\UrgencyFieldStrategy' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/UrgencyFieldStrategy.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ValidationField' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ValidationField.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ValidationFieldConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ValidationFieldConfig.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ValidationFieldStrategy' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ValidationFieldStrategy.php',
    'Glpi\\Form\\Destination\\CommonITILField\\ValidationFieldStrategyConfig' => $baseDir . '/src/Glpi/Form/Destination/CommonITILField/ValidationFieldStrategyConfig.php',
    'Glpi\\Form\\Destination\\ConfigFieldWithStrategiesInterface' => $baseDir . '/src/Glpi/Form/Destination/ConfigFieldWithStrategiesInterface.php',
    'Glpi\\Form\\Destination\\DestinationFieldInterface' => $baseDir . '/src/Glpi/Form/Destination/DestinationFieldInterface.php',
    'Glpi\\Form\\Destination\\FormDestination' => $baseDir . '/src/Glpi/Form/Destination/FormDestination.php',
    'Glpi\\Form\\Destination\\FormDestinationChange' => $baseDir . '/src/Glpi/Form/Destination/FormDestinationChange.php',
    'Glpi\\Form\\Destination\\FormDestinationInterface' => $baseDir . '/src/Glpi/Form/Destination/FormDestinationInterface.php',
    'Glpi\\Form\\Destination\\FormDestinationManager' => $baseDir . '/src/Glpi/Form/Destination/FormDestinationManager.php',
    'Glpi\\Form\\Destination\\FormDestinationProblem' => $baseDir . '/src/Glpi/Form/Destination/FormDestinationProblem.php',
    'Glpi\\Form\\Destination\\FormDestinationTicket' => $baseDir . '/src/Glpi/Form/Destination/FormDestinationTicket.php',
    'Glpi\\Form\\Destination\\HasFieldWithDestinationId' => $baseDir . '/src/Glpi/Form/Destination/HasFieldWithDestinationId.php',
    'Glpi\\Form\\Destination\\HasFieldWithQuestionId' => $baseDir . '/src/Glpi/Form/Destination/HasFieldWithQuestionId.php',
    'Glpi\\Form\\Destination\\HasFormTags' => $baseDir . '/src/Glpi/Form/Destination/HasFormTags.php',
    'Glpi\\Form\\Dropdown\\FormActorsDropdown' => $baseDir . '/src/Glpi/Form/Dropdown/FormActorsDropdown.php',
    'Glpi\\Form\\EndUserInputNameProvider' => $baseDir . '/src/Glpi/Form/EndUserInputNameProvider.php',
    'Glpi\\Form\\Export\\Context\\DatabaseMapper' => $baseDir . '/src/Glpi/Form/Export/Context/DatabaseMapper.php',
    'Glpi\\Form\\Export\\Result\\ExportResult' => $baseDir . '/src/Glpi/Form/Export/Result/ExportResult.php',
    'Glpi\\Form\\Export\\Result\\ImportError' => $baseDir . '/src/Glpi/Form/Export/Result/ImportError.php',
    'Glpi\\Form\\Export\\Result\\ImportResult' => $baseDir . '/src/Glpi/Form/Export/Result/ImportResult.php',
    'Glpi\\Form\\Export\\Result\\ImportResultIssues' => $baseDir . '/src/Glpi/Form/Export/Result/ImportResultIssues.php',
    'Glpi\\Form\\Export\\Result\\ImportResultPreview' => $baseDir . '/src/Glpi/Form/Export/Result/ImportResultPreview.php',
    'Glpi\\Form\\Export\\Serializer\\AbstractFormSerializer' => $baseDir . '/src/Glpi/Form/Export/Serializer/AbstractFormSerializer.php',
    'Glpi\\Form\\Export\\Serializer\\DynamicExportData' => $baseDir . '/src/Glpi/Form/Export/Serializer/DynamicExportData.php',
    'Glpi\\Form\\Export\\Serializer\\DynamicExportDataField' => $baseDir . '/src/Glpi/Form/Export/Serializer/DynamicExportDataField.php',
    'Glpi\\Form\\Export\\Serializer\\FormSerializer' => $baseDir . '/src/Glpi/Form/Export/Serializer/FormSerializer.php',
    'Glpi\\Form\\Export\\Specification\\AccesControlPolicyContentSpecification' => $baseDir . '/src/Glpi/Form/Export/Specification/AccesControlPolicyContentSpecification.php',
    'Glpi\\Form\\Export\\Specification\\CommentContentSpecification' => $baseDir . '/src/Glpi/Form/Export/Specification/CommentContentSpecification.php',
    'Glpi\\Form\\Export\\Specification\\ConditionDataSpecification' => $baseDir . '/src/Glpi/Form/Export/Specification/ConditionDataSpecification.php',
    'Glpi\\Form\\Export\\Specification\\ContentSpecificationInterface' => $baseDir . '/src/Glpi/Form/Export/Specification/ContentSpecificationInterface.php',
    'Glpi\\Form\\Export\\Specification\\CustomIllustrationContentSpecification' => $baseDir . '/src/Glpi/Form/Export/Specification/CustomIllustrationContentSpecification.php',
    'Glpi\\Form\\Export\\Specification\\CustomTypeRequirementSpecification' => $baseDir . '/src/Glpi/Form/Export/Specification/CustomTypeRequirementSpecification.php',
    'Glpi\\Form\\Export\\Specification\\DataRequirementSpecification' => $baseDir . '/src/Glpi/Form/Export/Specification/DataRequirementSpecification.php',
    'Glpi\\Form\\Export\\Specification\\DestinationContentSpecification' => $baseDir . '/src/Glpi/Form/Export/Specification/DestinationContentSpecification.php',
    'Glpi\\Form\\Export\\Specification\\ExportContentSpecification' => $baseDir . '/src/Glpi/Form/Export/Specification/ExportContentSpecification.php',
    'Glpi\\Form\\Export\\Specification\\FormContentSpecification' => $baseDir . '/src/Glpi/Form/Export/Specification/FormContentSpecification.php',
    'Glpi\\Form\\Export\\Specification\\PluginRequirementSpecification' => $baseDir . '/src/Glpi/Form/Export/Specification/PluginRequirementSpecification.php',
    'Glpi\\Form\\Export\\Specification\\QuestionContentSpecification' => $baseDir . '/src/Glpi/Form/Export/Specification/QuestionContentSpecification.php',
    'Glpi\\Form\\Export\\Specification\\SectionContentSpecification' => $baseDir . '/src/Glpi/Form/Export/Specification/SectionContentSpecification.php',
    'Glpi\\Form\\Export\\Specification\\TranslationContentSpecification' => $baseDir . '/src/Glpi/Form/Export/Specification/TranslationContentSpecification.php',
    'Glpi\\Form\\Form' => $baseDir . '/src/Glpi/Form/Form.php',
    'Glpi\\Form\\FormTranslation' => $baseDir . '/src/Glpi/Form/FormTranslation.php',
    'Glpi\\Form\\Migration\\ConditionHandlerDataConverterInterface' => $baseDir . '/src/Glpi/Form/Migration/ConditionHandlerDataConverterInterface.php',
    'Glpi\\Form\\Migration\\DestinationFieldConverterInterface' => $baseDir . '/src/Glpi/Form/Migration/DestinationFieldConverterInterface.php',
    'Glpi\\Form\\Migration\\FallbackToAnotherOperatorException' => $baseDir . '/src/Glpi/Form/Migration/FallbackToAnotherOperatorException.php',
    'Glpi\\Form\\Migration\\FormMigration' => $baseDir . '/src/Glpi/Form/Migration/FormMigration.php',
    'Glpi\\Form\\Migration\\FormQuestionDataConverterInterface' => $baseDir . '/src/Glpi/Form/Migration/FormQuestionDataConverterInterface.php',
    'Glpi\\Form\\Migration\\TagConversionTrait' => $baseDir . '/src/Glpi/Form/Migration/TagConversionTrait.php',
    'Glpi\\Form\\Migration\\TypesConversionMapper' => $baseDir . '/src/Glpi/Form/Migration/TypesConversionMapper.php',
    'Glpi\\Form\\Question' => $baseDir . '/src/Glpi/Form/Question.php',
    'Glpi\\Form\\QuestionType\\AbstractQuestionType' => $baseDir . '/src/Glpi/Form/QuestionType/AbstractQuestionType.php',
    'Glpi\\Form\\QuestionType\\AbstractQuestionTypeActors' => $baseDir . '/src/Glpi/Form/QuestionType/AbstractQuestionTypeActors.php',
    'Glpi\\Form\\QuestionType\\AbstractQuestionTypeSelectable' => $baseDir . '/src/Glpi/Form/QuestionType/AbstractQuestionTypeSelectable.php',
    'Glpi\\Form\\QuestionType\\AbstractQuestionTypeShortAnswer' => $baseDir . '/src/Glpi/Form/QuestionType/AbstractQuestionTypeShortAnswer.php',
    'Glpi\\Form\\QuestionType\\CustomMandatoryMessageInterface' => $baseDir . '/src/Glpi/Form/QuestionType/CustomMandatoryMessageInterface.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeActorsDefaultValueConfig' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeActorsDefaultValueConfig.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeActorsExtraDataConfig' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeActorsExtraDataConfig.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeAssignee' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeAssignee.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeCategory' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeCategory.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeCategoryInterface' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeCategoryInterface.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeCheckbox' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeCheckbox.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeDateTime' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeDateTime.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeDateTimeExtraDataConfig' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeDateTimeExtraDataConfig.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeDropdown' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeDropdown.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeDropdownExtraDataConfig' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeDropdownExtraDataConfig.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeEmail' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeEmail.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeFile' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeFile.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeInterface' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeInterface.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeItem' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeItem.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeItemDefaultValueConfig' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeItemDefaultValueConfig.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeItemDropdown' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeItemDropdown.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeItemDropdownExtraDataConfig' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeItemDropdownExtraDataConfig.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeItemExtraDataConfig' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeItemExtraDataConfig.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeLongText' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeLongText.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeNumber' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeNumber.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeObserver' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeObserver.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeRadio' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeRadio.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeRequestType' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeRequestType.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeRequester' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeRequester.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeSelectableExtraDataConfig' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeSelectableExtraDataConfig.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeShortText' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeShortText.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeUrgency' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeUrgency.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeUserDevice' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeUserDevice.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeUserDevicesConfig' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeUserDevicesConfig.php',
    'Glpi\\Form\\QuestionType\\QuestionTypeValidationInterface' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypeValidationInterface.php',
    'Glpi\\Form\\QuestionType\\QuestionTypesManager' => $baseDir . '/src/Glpi/Form/QuestionType/QuestionTypesManager.php',
    'Glpi\\Form\\QuestionType\\TranslationAwareQuestionType' => $baseDir . '/src/Glpi/Form/QuestionType/TranslationAwareQuestionType.php',
    'Glpi\\Form\\RenderLayout' => $baseDir . '/src/Glpi/Form/RenderLayout.php',
    'Glpi\\Form\\Section' => $baseDir . '/src/Glpi/Form/Section.php',
    'Glpi\\Form\\ServiceCatalog\\HomeSearchManager' => $baseDir . '/src/Glpi/Form/ServiceCatalog/HomeSearchManager.php',
    'Glpi\\Form\\ServiceCatalog\\ItemRequest' => $baseDir . '/src/Glpi/Form/ServiceCatalog/ItemRequest.php',
    'Glpi\\Form\\ServiceCatalog\\ItemRequestContext' => $baseDir . '/src/Glpi/Form/ServiceCatalog/ItemRequestContext.php',
    'Glpi\\Form\\ServiceCatalog\\Provider\\CategoryProvider' => $baseDir . '/src/Glpi/Form/ServiceCatalog/Provider/CategoryProvider.php',
    'Glpi\\Form\\ServiceCatalog\\Provider\\CompositeProviderInterface' => $baseDir . '/src/Glpi/Form/ServiceCatalog/Provider/CompositeProviderInterface.php',
    'Glpi\\Form\\ServiceCatalog\\Provider\\FormProvider' => $baseDir . '/src/Glpi/Form/ServiceCatalog/Provider/FormProvider.php',
    'Glpi\\Form\\ServiceCatalog\\Provider\\ItemProviderInterface' => $baseDir . '/src/Glpi/Form/ServiceCatalog/Provider/ItemProviderInterface.php',
    'Glpi\\Form\\ServiceCatalog\\Provider\\KnowbaseItemProvider' => $baseDir . '/src/Glpi/Form/ServiceCatalog/Provider/KnowbaseItemProvider.php',
    'Glpi\\Form\\ServiceCatalog\\Provider\\LeafProviderInterface' => $baseDir . '/src/Glpi/Form/ServiceCatalog/Provider/LeafProviderInterface.php',
    'Glpi\\Form\\ServiceCatalog\\ServiceCatalog' => $baseDir . '/src/Glpi/Form/ServiceCatalog/ServiceCatalog.php',
    'Glpi\\Form\\ServiceCatalog\\ServiceCatalogCompositeInterface' => $baseDir . '/src/Glpi/Form/ServiceCatalog/ServiceCatalogCompositeInterface.php',
    'Glpi\\Form\\ServiceCatalog\\ServiceCatalogItemInterface' => $baseDir . '/src/Glpi/Form/ServiceCatalog/ServiceCatalogItemInterface.php',
    'Glpi\\Form\\ServiceCatalog\\ServiceCatalogLeafInterface' => $baseDir . '/src/Glpi/Form/ServiceCatalog/ServiceCatalogLeafInterface.php',
    'Glpi\\Form\\ServiceCatalog\\ServiceCatalogManager' => $baseDir . '/src/Glpi/Form/ServiceCatalog/ServiceCatalogManager.php',
    'Glpi\\Form\\ServiceCatalog\\SortStrategy\\AbstractSortStrategy' => $baseDir . '/src/Glpi/Form/ServiceCatalog/SortStrategy/AbstractSortStrategy.php',
    'Glpi\\Form\\ServiceCatalog\\SortStrategy\\AlphabeticalSort' => $baseDir . '/src/Glpi/Form/ServiceCatalog/SortStrategy/AlphabeticalSort.php',
    'Glpi\\Form\\ServiceCatalog\\SortStrategy\\PopularitySort' => $baseDir . '/src/Glpi/Form/ServiceCatalog/SortStrategy/PopularitySort.php',
    'Glpi\\Form\\ServiceCatalog\\SortStrategy\\ReverseAlphabeticalSort' => $baseDir . '/src/Glpi/Form/ServiceCatalog/SortStrategy/ReverseAlphabeticalSort.php',
    'Glpi\\Form\\ServiceCatalog\\SortStrategy\\SortStrategyEnum' => $baseDir . '/src/Glpi/Form/ServiceCatalog/SortStrategy/SortStrategyEnum.php',
    'Glpi\\Form\\ServiceCatalog\\SortStrategy\\SortStrategyInterface' => $baseDir . '/src/Glpi/Form/ServiceCatalog/SortStrategy/SortStrategyInterface.php',
    'Glpi\\Form\\Tag\\AnswerTagProvider' => $baseDir . '/src/Glpi/Form/Tag/AnswerTagProvider.php',
    'Glpi\\Form\\Tag\\CommentDescriptionTagProvider' => $baseDir . '/src/Glpi/Form/Tag/CommentDescriptionTagProvider.php',
    'Glpi\\Form\\Tag\\CommentTitleTagProvider' => $baseDir . '/src/Glpi/Form/Tag/CommentTitleTagProvider.php',
    'Glpi\\Form\\Tag\\FormTagProvider' => $baseDir . '/src/Glpi/Form/Tag/FormTagProvider.php',
    'Glpi\\Form\\Tag\\FormTagsManager' => $baseDir . '/src/Glpi/Form/Tag/FormTagsManager.php',
    'Glpi\\Form\\Tag\\QuestionTagProvider' => $baseDir . '/src/Glpi/Form/Tag/QuestionTagProvider.php',
    'Glpi\\Form\\Tag\\SectionTagProvider' => $baseDir . '/src/Glpi/Form/Tag/SectionTagProvider.php',
    'Glpi\\Form\\Tag\\Tag' => $baseDir . '/src/Glpi/Form/Tag/Tag.php',
    'Glpi\\Form\\Tag\\TagProviderInterface' => $baseDir . '/src/Glpi/Form/Tag/TagProviderInterface.php',
    'Glpi\\Form\\Tag\\TagWithIdValueInterface' => $baseDir . '/src/Glpi/Form/Tag/TagWithIdValueInterface.php',
    'Glpi\\Form\\ValidationResult' => $baseDir . '/src/Glpi/Form/ValidationResult.php',
    'Glpi\\FuzzyMatcher\\FuzzyMatcher' => $baseDir . '/src/Glpi/FuzzyMatcher/FuzzyMatcher.php',
    'Glpi\\FuzzyMatcher\\FuzzyMatcherStrategyInterface' => $baseDir . '/src/Glpi/FuzzyMatcher/FuzzyMatcherStrategyInterface.php',
    'Glpi\\FuzzyMatcher\\PartialMatchStrategy' => $baseDir . '/src/Glpi/FuzzyMatcher/PartialMatchStrategy.php',
    'Glpi\\Helpdesk\\DefaultDataManager' => $baseDir . '/src/Glpi/Helpdesk/DefaultDataManager.php',
    'Glpi\\Helpdesk\\HelpdeskTranslation' => $baseDir . '/src/Glpi/Helpdesk/HelpdeskTranslation.php',
    'Glpi\\Helpdesk\\HomePageTabs' => $baseDir . '/src/Glpi/Helpdesk/HomePageTabs.php',
    'Glpi\\Helpdesk\\Tile\\ExternalPageTile' => $baseDir . '/src/Glpi/Helpdesk/Tile/ExternalPageTile.php',
    'Glpi\\Helpdesk\\Tile\\FormTile' => $baseDir . '/src/Glpi/Helpdesk/Tile/FormTile.php',
    'Glpi\\Helpdesk\\Tile\\GlpiPageTile' => $baseDir . '/src/Glpi/Helpdesk/Tile/GlpiPageTile.php',
    'Glpi\\Helpdesk\\Tile\\InvalidTileException' => $baseDir . '/src/Glpi/Helpdesk/Tile/InvalidTileException.php',
    'Glpi\\Helpdesk\\Tile\\Item_Tile' => $baseDir . '/src/Glpi/Helpdesk/Tile/Item_Tile.php',
    'Glpi\\Helpdesk\\Tile\\LinkableToTilesInterface' => $baseDir . '/src/Glpi/Helpdesk/Tile/LinkableToTilesInterface.php',
    'Glpi\\Helpdesk\\Tile\\TileInterface' => $baseDir . '/src/Glpi/Helpdesk/Tile/TileInterface.php',
    'Glpi\\Helpdesk\\Tile\\TilesManager' => $baseDir . '/src/Glpi/Helpdesk/Tile/TilesManager.php',
    'Glpi\\Http\\Firewall' => $baseDir . '/src/Glpi/Http/Firewall.php',
    'Glpi\\Http\\HeaderlessStreamedResponse' => $baseDir . '/src/Glpi/Http/HeaderlessStreamedResponse.php',
    'Glpi\\Http\\JSONResponse' => $baseDir . '/src/Glpi/Http/JSONResponse.php',
    'Glpi\\Http\\RedirectResponse' => $baseDir . '/src/Glpi/Http/RedirectResponse.php',
    'Glpi\\Http\\Request' => $baseDir . '/src/Glpi/Http/Request.php',
    'Glpi\\Http\\RequestRouterTrait' => $baseDir . '/src/Glpi/Http/RequestRouterTrait.php',
    'Glpi\\Http\\Response' => $baseDir . '/src/Glpi/Http/Response.php',
    'Glpi\\Http\\SessionManager' => $baseDir . '/src/Glpi/Http/SessionManager.php',
    'Glpi\\Inventory\\Asset\\Antivirus' => $baseDir . '/src/Glpi/Inventory/Asset/Antivirus.php',
    'Glpi\\Inventory\\Asset\\Battery' => $baseDir . '/src/Glpi/Inventory/Asset/Battery.php',
    'Glpi\\Inventory\\Asset\\Bios' => $baseDir . '/src/Glpi/Inventory/Asset/Bios.php',
    'Glpi\\Inventory\\Asset\\Camera' => $baseDir . '/src/Glpi/Inventory/Asset/Camera.php',
    'Glpi\\Inventory\\Asset\\Cartridge' => $baseDir . '/src/Glpi/Inventory/Asset/Cartridge.php',
    'Glpi\\Inventory\\Asset\\Controller' => $baseDir . '/src/Glpi/Inventory/Asset/Controller.php',
    'Glpi\\Inventory\\Asset\\DatabaseInstance' => $baseDir . '/src/Glpi/Inventory/Asset/DatabaseInstance.php',
    'Glpi\\Inventory\\Asset\\Device' => $baseDir . '/src/Glpi/Inventory/Asset/Device.php',
    'Glpi\\Inventory\\Asset\\Drive' => $baseDir . '/src/Glpi/Inventory/Asset/Drive.php',
    'Glpi\\Inventory\\Asset\\Environment' => $baseDir . '/src/Glpi/Inventory/Asset/Environment.php',
    'Glpi\\Inventory\\Asset\\Firmware' => $baseDir . '/src/Glpi/Inventory/Asset/Firmware.php',
    'Glpi\\Inventory\\Asset\\GraphicCard' => $baseDir . '/src/Glpi/Inventory/Asset/GraphicCard.php',
    'Glpi\\Inventory\\Asset\\HardDrive' => $baseDir . '/src/Glpi/Inventory/Asset/HardDrive.php',
    'Glpi\\Inventory\\Asset\\InventoryAsset' => $baseDir . '/src/Glpi/Inventory/Asset/InventoryAsset.php',
    'Glpi\\Inventory\\Asset\\InventoryNetworkPort' => $baseDir . '/src/Glpi/Inventory/Asset/InventoryNetworkPort.php',
    'Glpi\\Inventory\\Asset\\Memory' => $baseDir . '/src/Glpi/Inventory/Asset/Memory.php',
    'Glpi\\Inventory\\Asset\\Monitor' => $baseDir . '/src/Glpi/Inventory/Asset/Monitor.php',
    'Glpi\\Inventory\\Asset\\NetworkCard' => $baseDir . '/src/Glpi/Inventory/Asset/NetworkCard.php',
    'Glpi\\Inventory\\Asset\\NetworkPort' => $baseDir . '/src/Glpi/Inventory/Asset/NetworkPort.php',
    'Glpi\\Inventory\\Asset\\OperatingSystem' => $baseDir . '/src/Glpi/Inventory/Asset/OperatingSystem.php',
    'Glpi\\Inventory\\Asset\\Peripheral' => $baseDir . '/src/Glpi/Inventory/Asset/Peripheral.php',
    'Glpi\\Inventory\\Asset\\PowerSupply' => $baseDir . '/src/Glpi/Inventory/Asset/PowerSupply.php',
    'Glpi\\Inventory\\Asset\\Printer' => $baseDir . '/src/Glpi/Inventory/Asset/Printer.php',
    'Glpi\\Inventory\\Asset\\Process' => $baseDir . '/src/Glpi/Inventory/Asset/Process.php',
    'Glpi\\Inventory\\Asset\\Processor' => $baseDir . '/src/Glpi/Inventory/Asset/Processor.php',
    'Glpi\\Inventory\\Asset\\RemoteManagement' => $baseDir . '/src/Glpi/Inventory/Asset/RemoteManagement.php',
    'Glpi\\Inventory\\Asset\\Sensor' => $baseDir . '/src/Glpi/Inventory/Asset/Sensor.php',
    'Glpi\\Inventory\\Asset\\Simcard' => $baseDir . '/src/Glpi/Inventory/Asset/Simcard.php',
    'Glpi\\Inventory\\Asset\\Software' => $baseDir . '/src/Glpi/Inventory/Asset/Software.php',
    'Glpi\\Inventory\\Asset\\SoundCard' => $baseDir . '/src/Glpi/Inventory/Asset/SoundCard.php',
    'Glpi\\Inventory\\Asset\\VirtualMachine' => $baseDir . '/src/Glpi/Inventory/Asset/VirtualMachine.php',
    'Glpi\\Inventory\\Asset\\Volume' => $baseDir . '/src/Glpi/Inventory/Asset/Volume.php',
    'Glpi\\Inventory\\Conf' => $baseDir . '/src/Glpi/Inventory/Conf.php',
    'Glpi\\Inventory\\Inventory' => $baseDir . '/src/Glpi/Inventory/Inventory.php',
    'Glpi\\Inventory\\MainAsset\\Computer' => $baseDir . '/src/Glpi/Inventory/MainAsset/Computer.php',
    'Glpi\\Inventory\\MainAsset\\GenericAsset' => $baseDir . '/src/Glpi/Inventory/MainAsset/GenericAsset.php',
    'Glpi\\Inventory\\MainAsset\\GenericNetworkAsset' => $baseDir . '/src/Glpi/Inventory/MainAsset/GenericNetworkAsset.php',
    'Glpi\\Inventory\\MainAsset\\GenericPrinterAsset' => $baseDir . '/src/Glpi/Inventory/MainAsset/GenericPrinterAsset.php',
    'Glpi\\Inventory\\MainAsset\\Itemtype' => $baseDir . '/src/Glpi/Inventory/MainAsset/Itemtype.php',
    'Glpi\\Inventory\\MainAsset\\MainAsset' => $baseDir . '/src/Glpi/Inventory/MainAsset/MainAsset.php',
    'Glpi\\Inventory\\MainAsset\\NetworkEquipment' => $baseDir . '/src/Glpi/Inventory/MainAsset/NetworkEquipment.php',
    'Glpi\\Inventory\\MainAsset\\Phone' => $baseDir . '/src/Glpi/Inventory/MainAsset/Phone.php',
    'Glpi\\Inventory\\MainAsset\\Printer' => $baseDir . '/src/Glpi/Inventory/MainAsset/Printer.php',
    'Glpi\\Inventory\\MainAsset\\Unmanaged' => $baseDir . '/src/Glpi/Inventory/MainAsset/Unmanaged.php',
    'Glpi\\Inventory\\Request' => $baseDir . '/src/Glpi/Inventory/Request.php',
    'Glpi\\ItemTranslation\\CldrLanguage' => $baseDir . '/src/Glpi/ItemTranslation/CldrLanguage.php',
    'Glpi\\ItemTranslation\\Context\\ProvideTranslationsInterface' => $baseDir . '/src/Glpi/ItemTranslation/Context/ProvideTranslationsInterface.php',
    'Glpi\\ItemTranslation\\Context\\TranslationHandler' => $baseDir . '/src/Glpi/ItemTranslation/Context/TranslationHandler.php',
    'Glpi\\ItemTranslation\\ItemTranslation' => $baseDir . '/src/Glpi/ItemTranslation/ItemTranslation.php',
    'Glpi\\Kernel\\Kernel' => $baseDir . '/src/Glpi/Kernel/Kernel.php',
    'Glpi\\Kernel\\KernelListenerTrait' => $baseDir . '/src/Glpi/Kernel/KernelListenerTrait.php',
    'Glpi\\Kernel\\Listener\\ControllerListener\\CheckCsrfListener' => $baseDir . '/src/Glpi/Kernel/Listener/ControllerListener/CheckCsrfListener.php',
    'Glpi\\Kernel\\Listener\\ControllerListener\\FirewallStrategyListener' => $baseDir . '/src/Glpi/Kernel/Listener/ControllerListener/FirewallStrategyListener.php',
    'Glpi\\Kernel\\Listener\\ExceptionListener\\AccessErrorListener' => $baseDir . '/src/Glpi/Kernel/Listener/ExceptionListener/AccessErrorListener.php',
    'Glpi\\Kernel\\Listener\\ExceptionListener\\ProgressErrorListener' => $baseDir . '/src/Glpi/Kernel/Listener/ExceptionListener/ProgressErrorListener.php',
    'Glpi\\Kernel\\Listener\\ExceptionListener\\RedirectExceptionListener' => $baseDir . '/src/Glpi/Kernel/Listener/ExceptionListener/RedirectExceptionListener.php',
    'Glpi\\Kernel\\Listener\\PostBootListener\\BootPlugins' => $baseDir . '/src/Glpi/Kernel/Listener/PostBootListener/BootPlugins.php',
    'Glpi\\Kernel\\Listener\\PostBootListener\\CheckPluginsStates' => $baseDir . '/src/Glpi/Kernel/Listener/PostBootListener/CheckPluginsStates.php',
    'Glpi\\Kernel\\Listener\\PostBootListener\\CustomObjectsAutoloaderRegistration' => $baseDir . '/src/Glpi/Kernel/Listener/PostBootListener/CustomObjectsAutoloaderRegistration.php',
    'Glpi\\Kernel\\Listener\\PostBootListener\\CustomObjectsBoot' => $baseDir . '/src/Glpi/Kernel/Listener/PostBootListener/CustomObjectsBoot.php',
    'Glpi\\Kernel\\Listener\\PostBootListener\\InitializeCache' => $baseDir . '/src/Glpi/Kernel/Listener/PostBootListener/InitializeCache.php',
    'Glpi\\Kernel\\Listener\\PostBootListener\\InitializeDbConnection' => $baseDir . '/src/Glpi/Kernel/Listener/PostBootListener/InitializeDbConnection.php',
    'Glpi\\Kernel\\Listener\\PostBootListener\\InitializePlugins' => $baseDir . '/src/Glpi/Kernel/Listener/PostBootListener/InitializePlugins.php',
    'Glpi\\Kernel\\Listener\\PostBootListener\\LoadLanguage' => $baseDir . '/src/Glpi/Kernel/Listener/PostBootListener/LoadLanguage.php',
    'Glpi\\Kernel\\Listener\\PostBootListener\\LoadLegacyConfiguration' => $baseDir . '/src/Glpi/Kernel/Listener/PostBootListener/LoadLegacyConfiguration.php',
    'Glpi\\Kernel\\Listener\\PostBootListener\\ProfilerStart' => $baseDir . '/src/Glpi/Kernel/Listener/PostBootListener/ProfilerStart.php',
    'Glpi\\Kernel\\Listener\\PostBootListener\\SessionStart' => $baseDir . '/src/Glpi/Kernel/Listener/PostBootListener/SessionStart.php',
    'Glpi\\Kernel\\Listener\\PostBootListener\\SetDbSessionVars' => $baseDir . '/src/Glpi/Kernel/Listener/PostBootListener/SetDbSessionVars.php',
    'Glpi\\Kernel\\Listener\\RequestListener\\CatchInventoryAgentRequestListener' => $baseDir . '/src/Glpi/Kernel/Listener/RequestListener/CatchInventoryAgentRequestListener.php',
    'Glpi\\Kernel\\Listener\\RequestListener\\CheckDatabaseStatusListener' => $baseDir . '/src/Glpi/Kernel/Listener/RequestListener/CheckDatabaseStatusListener.php',
    'Glpi\\Kernel\\Listener\\RequestListener\\CheckMaintenanceListener' => $baseDir . '/src/Glpi/Kernel/Listener/RequestListener/CheckMaintenanceListener.php',
    'Glpi\\Kernel\\Listener\\RequestListener\\ErrorHandlerRequestListener' => $baseDir . '/src/Glpi/Kernel/Listener/RequestListener/ErrorHandlerRequestListener.php',
    'Glpi\\Kernel\\Listener\\RequestListener\\FlushBootErrors' => $baseDir . '/src/Glpi/Kernel/Listener/RequestListener/FlushBootErrors.php',
    'Glpi\\Kernel\\Listener\\RequestListener\\FrontEndAssetsListener' => $baseDir . '/src/Glpi/Kernel/Listener/RequestListener/FrontEndAssetsListener.php',
    'Glpi\\Kernel\\Listener\\RequestListener\\LegacyItemtypeRouteListener' => $baseDir . '/src/Glpi/Kernel/Listener/RequestListener/LegacyItemtypeRouteListener.php',
    'Glpi\\Kernel\\Listener\\RequestListener\\LegacyRouterListener' => $baseDir . '/src/Glpi/Kernel/Listener/RequestListener/LegacyRouterListener.php',
    'Glpi\\Kernel\\Listener\\RequestListener\\PluginsRouterListener' => $baseDir . '/src/Glpi/Kernel/Listener/RequestListener/PluginsRouterListener.php',
    'Glpi\\Kernel\\Listener\\RequestListener\\RedirectLegacyRouteListener' => $baseDir . '/src/Glpi/Kernel/Listener/RequestListener/RedirectLegacyRouteListener.php',
    'Glpi\\Kernel\\Listener\\RequestListener\\SessionCheckCookieListener' => $baseDir . '/src/Glpi/Kernel/Listener/RequestListener/SessionCheckCookieListener.php',
    'Glpi\\Kernel\\Listener\\RequestListener\\SessionVariables' => $baseDir . '/src/Glpi/Kernel/Listener/RequestListener/SessionVariables.php',
    'Glpi\\Kernel\\ListenersPriority' => $baseDir . '/src/Glpi/Kernel/ListenersPriority.php',
    'Glpi\\Kernel\\PostBootEvent' => $baseDir . '/src/Glpi/Kernel/PostBootEvent.php',
    'Glpi\\Log\\AbstractLogHandler' => $baseDir . '/src/Glpi/Log/AbstractLogHandler.php',
    'Glpi\\Log\\AbstractLogLineFormatter' => $baseDir . '/src/Glpi/Log/AbstractLogLineFormatter.php',
    'Glpi\\Log\\AccessLogHandler' => $baseDir . '/src/Glpi/Log/AccessLogHandler.php',
    'Glpi\\Log\\AccessLogLineFormatter' => $baseDir . '/src/Glpi/Log/AccessLogLineFormatter.php',
    'Glpi\\Log\\ErrorLogHandler' => $baseDir . '/src/Glpi/Log/ErrorLogHandler.php',
    'Glpi\\Log\\ErrorLogLineFormatter' => $baseDir . '/src/Glpi/Log/ErrorLogLineFormatter.php',
    'Glpi\\Mail\\Protocol\\ProtocolInterface' => $baseDir . '/src/Glpi/Mail/Protocol/ProtocolInterface.php',
    'Glpi\\Mail\\SMTP\\OauthConfig' => $baseDir . '/src/Glpi/Mail/SMTP/OauthConfig.php',
    'Glpi\\Mail\\SMTP\\OauthProvider\\Azure' => $baseDir . '/src/Glpi/Mail/SMTP/OauthProvider/Azure.php',
    'Glpi\\Mail\\SMTP\\OauthProvider\\Google' => $baseDir . '/src/Glpi/Mail/SMTP/OauthProvider/Google.php',
    'Glpi\\Mail\\SMTP\\OauthProvider\\ProviderInterface' => $baseDir . '/src/Glpi/Mail/SMTP/OauthProvider/ProviderInterface.php',
    'Glpi\\Marketplace\\Api\\Plugins' => $baseDir . '/src/Glpi/Marketplace/Api/Plugins.php',
    'Glpi\\Marketplace\\Controller' => $baseDir . '/src/Glpi/Marketplace/Controller.php',
    'Glpi\\Marketplace\\NotificationTargetController' => $baseDir . '/src/Glpi/Marketplace/NotificationTargetController.php',
    'Glpi\\Marketplace\\View' => $baseDir . '/src/Glpi/Marketplace/View.php',
    'Glpi\\Message\\MessageType' => $baseDir . '/src/Glpi/Message/MessageType.php',
    'Glpi\\Migration\\AbstractPluginMigration' => $baseDir . '/src/Glpi/Migration/AbstractPluginMigration.php',
    'Glpi\\Migration\\GenericobjectPluginMigration' => $baseDir . '/src/Glpi/Migration/GenericobjectPluginMigration.php',
    'Glpi\\Migration\\MigrationException' => $baseDir . '/src/Glpi/Migration/MigrationException.php',
    'Glpi\\Migration\\PluginMigrationResult' => $baseDir . '/src/Glpi/Migration/PluginMigrationResult.php',
    'Glpi\\OAuth\\AccessToken' => $baseDir . '/src/Glpi/OAuth/AccessToken.php',
    'Glpi\\OAuth\\AccessTokenRepository' => $baseDir . '/src/Glpi/OAuth/AccessTokenRepository.php',
    'Glpi\\OAuth\\AuthCode' => $baseDir . '/src/Glpi/OAuth/AuthCode.php',
    'Glpi\\OAuth\\AuthCodeRepository' => $baseDir . '/src/Glpi/OAuth/AuthCodeRepository.php',
    'Glpi\\OAuth\\Client' => $baseDir . '/src/Glpi/OAuth/Client.php',
    'Glpi\\OAuth\\ClientRepository' => $baseDir . '/src/Glpi/OAuth/ClientRepository.php',
    'Glpi\\OAuth\\RefreshToken' => $baseDir . '/src/Glpi/OAuth/RefreshToken.php',
    'Glpi\\OAuth\\RefreshTokenRepository' => $baseDir . '/src/Glpi/OAuth/RefreshTokenRepository.php',
    'Glpi\\OAuth\\Scope' => $baseDir . '/src/Glpi/OAuth/Scope.php',
    'Glpi\\OAuth\\ScopeRepository' => $baseDir . '/src/Glpi/OAuth/ScopeRepository.php',
    'Glpi\\OAuth\\Server' => $baseDir . '/src/Glpi/OAuth/Server.php',
    'Glpi\\OAuth\\User' => $baseDir . '/src/Glpi/OAuth/User.php',
    'Glpi\\OAuth\\UserRepository' => $baseDir . '/src/Glpi/OAuth/UserRepository.php',
    'Glpi\\Plugin\\HookManager' => $baseDir . '/src/Glpi/Plugin/HookManager.php',
    'Glpi\\Plugin\\Hooks' => $baseDir . '/src/Glpi/Plugin/Hooks.php',
    'Glpi\\Progress\\AbstractProgressIndicator' => $baseDir . '/src/Glpi/Progress/AbstractProgressIndicator.php',
    'Glpi\\Progress\\ConsoleProgressIndicator' => $baseDir . '/src/Glpi/Progress/ConsoleProgressIndicator.php',
    'Glpi\\Progress\\ProgressStorage' => $baseDir . '/src/Glpi/Progress/ProgressStorage.php',
    'Glpi\\Progress\\StoredProgressIndicator' => $baseDir . '/src/Glpi/Progress/StoredProgressIndicator.php',
    'Glpi\\RichText\\RichText' => $baseDir . '/src/Glpi/RichText/RichText.php',
    'Glpi\\RichText\\UserMention' => $baseDir . '/src/Glpi/RichText/UserMention.php',
    'Glpi\\Routing\\Attribute\\ItemtypeFormRoute' => $baseDir . '/src/Glpi/Routing/Attribute/ItemtypeFormRoute.php',
    'Glpi\\Routing\\Attribute\\ItemtypeListRoute' => $baseDir . '/src/Glpi/Routing/Attribute/ItemtypeListRoute.php',
    'Glpi\\Routing\\PluginRoutesLoader' => $baseDir . '/src/Glpi/Routing/PluginRoutesLoader.php',
    'Glpi\\Rules\\RulesManager' => $baseDir . '/src/Glpi/Rules/RulesManager.php',
    'Glpi\\Search\\CriteriaFilter' => $baseDir . '/src/Glpi/Search/CriteriaFilter.php',
    'Glpi\\Search\\DefaultSearchRequestInterface' => $baseDir . '/src/Glpi/Search/DefaultSearchRequestInterface.php',
    'Glpi\\Search\\FilterableInterface' => $baseDir . '/src/Glpi/Search/FilterableInterface.php',
    'Glpi\\Search\\FilterableTrait' => $baseDir . '/src/Glpi/Search/FilterableTrait.php',
    'Glpi\\Search\\Input\\QueryBuilder' => $baseDir . '/src/Glpi/Search/Input/QueryBuilder.php',
    'Glpi\\Search\\Input\\SearchInputInterface' => $baseDir . '/src/Glpi/Search/Input/SearchInputInterface.php',
    'Glpi\\Search\\Output\\AbstractSearchOutput' => $baseDir . '/src/Glpi/Search/Output/AbstractSearchOutput.php',
    'Glpi\\Search\\Output\\Csv' => $baseDir . '/src/Glpi/Search/Output/Csv.php',
    'Glpi\\Search\\Output\\ExportSearchOutput' => $baseDir . '/src/Glpi/Search/Output/ExportSearchOutput.php',
    'Glpi\\Search\\Output\\HTMLSearchOutput' => $baseDir . '/src/Glpi/Search/Output/HTMLSearchOutput.php',
    'Glpi\\Search\\Output\\MapSearchOutput' => $baseDir . '/src/Glpi/Search/Output/MapSearchOutput.php',
    'Glpi\\Search\\Output\\NamesListSearchOutput' => $baseDir . '/src/Glpi/Search/Output/NamesListSearchOutput.php',
    'Glpi\\Search\\Output\\Ods' => $baseDir . '/src/Glpi/Search/Output/Ods.php',
    'Glpi\\Search\\Output\\Pdf' => $baseDir . '/src/Glpi/Search/Output/Pdf.php',
    'Glpi\\Search\\Output\\Spreadsheet' => $baseDir . '/src/Glpi/Search/Output/Spreadsheet.php',
    'Glpi\\Search\\Output\\Tcpdf' => $baseDir . '/src/Glpi/Search/Output/Tcpdf.php',
    'Glpi\\Search\\Output\\Xlsx' => $baseDir . '/src/Glpi/Search/Output/Xlsx.php',
    'Glpi\\Search\\Provider\\SQLProvider' => $baseDir . '/src/Glpi/Search/Provider/SQLProvider.php',
    'Glpi\\Search\\Provider\\SearchProviderInterface' => $baseDir . '/src/Glpi/Search/Provider/SearchProviderInterface.php',
    'Glpi\\Search\\SearchEngine' => $baseDir . '/src/Glpi/Search/SearchEngine.php',
    'Glpi\\Search\\SearchOption' => $baseDir . '/src/Glpi/Search/SearchOption.php',
    'Glpi\\Security\\Attribute\\SecurityStrategy' => $baseDir . '/src/Glpi/Security/Attribute/SecurityStrategy.php',
    'Glpi\\Security\\PermissionManager' => $baseDir . '/src/Glpi/Security/PermissionManager.php',
    'Glpi\\Security\\TOTPManager' => $baseDir . '/src/Glpi/Security/TOTPManager.php',
    'Glpi\\Session\\SessionInfo' => $baseDir . '/src/Glpi/Session/SessionInfo.php',
    'Glpi\\Socket' => $baseDir . '/src/Glpi/Socket.php',
    'Glpi\\SocketModel' => $baseDir . '/src/Glpi/SocketModel.php',
    'Glpi\\Stat\\Data\\Graph\\StatDataSatisfaction' => $baseDir . '/src/Glpi/Stat/Data/Graph/StatDataSatisfaction.php',
    'Glpi\\Stat\\Data\\Graph\\StatDataSatisfactionSurvey' => $baseDir . '/src/Glpi/Stat/Data/Graph/StatDataSatisfactionSurvey.php',
    'Glpi\\Stat\\Data\\Graph\\StatDataTicketAverageTime' => $baseDir . '/src/Glpi/Stat/Data/Graph/StatDataTicketAverageTime.php',
    'Glpi\\Stat\\Data\\Graph\\StatDataTicketNumber' => $baseDir . '/src/Glpi/Stat/Data/Graph/StatDataTicketNumber.php',
    'Glpi\\Stat\\Data\\Location\\StatDataClosed' => $baseDir . '/src/Glpi/Stat/Data/Location/StatDataClosed.php',
    'Glpi\\Stat\\Data\\Location\\StatDataLate' => $baseDir . '/src/Glpi/Stat/Data/Location/StatDataLate.php',
    'Glpi\\Stat\\Data\\Location\\StatDataLocation' => $baseDir . '/src/Glpi/Stat/Data/Location/StatDataLocation.php',
    'Glpi\\Stat\\Data\\Location\\StatDataOpenSatisfaction' => $baseDir . '/src/Glpi/Stat/Data/Location/StatDataOpenSatisfaction.php',
    'Glpi\\Stat\\Data\\Location\\StatDataOpened' => $baseDir . '/src/Glpi/Stat/Data/Location/StatDataOpened.php',
    'Glpi\\Stat\\Data\\Location\\StatDataSolved' => $baseDir . '/src/Glpi/Stat/Data/Location/StatDataSolved.php',
    'Glpi\\Stat\\Data\\Sglobal\\StatDataAverageSatisfaction' => $baseDir . '/src/Glpi/Stat/Data/Sglobal/StatDataAverageSatisfaction.php',
    'Glpi\\Stat\\Data\\Sglobal\\StatDataSatisfaction' => $baseDir . '/src/Glpi/Stat/Data/Sglobal/StatDataSatisfaction.php',
    'Glpi\\Stat\\Data\\Sglobal\\StatDataTicketAverageTime' => $baseDir . '/src/Glpi/Stat/Data/Sglobal/StatDataTicketAverageTime.php',
    'Glpi\\Stat\\Data\\Sglobal\\StatDataTicketNumber' => $baseDir . '/src/Glpi/Stat/Data/Sglobal/StatDataTicketNumber.php',
    'Glpi\\Stat\\StatData' => $baseDir . '/src/Glpi/Stat/StatData.php',
    'Glpi\\Stat\\StatDataAlwaysDisplay' => $baseDir . '/src/Glpi/Stat/StatDataAlwaysDisplay.php',
    'Glpi\\System\\Diagnostic\\AbstractDatabaseChecker' => $baseDir . '/src/Glpi/System/Diagnostic/AbstractDatabaseChecker.php',
    'Glpi\\System\\Diagnostic\\DatabaseKeysChecker' => $baseDir . '/src/Glpi/System/Diagnostic/DatabaseKeysChecker.php',
    'Glpi\\System\\Diagnostic\\DatabaseSchemaConsistencyChecker' => $baseDir . '/src/Glpi/System/Diagnostic/DatabaseSchemaConsistencyChecker.php',
    'Glpi\\System\\Diagnostic\\DatabaseSchemaIntegrityChecker' => $baseDir . '/src/Glpi/System/Diagnostic/DatabaseSchemaIntegrityChecker.php',
    'Glpi\\System\\Diagnostic\\SourceCodeIntegrityChecker' => $baseDir . '/src/Glpi/System/Diagnostic/SourceCodeIntegrityChecker.php',
    'Glpi\\System\\Log\\LogParser' => $baseDir . '/src/Glpi/System/Log/LogParser.php',
    'Glpi\\System\\Log\\LogViewer' => $baseDir . '/src/Glpi/System/Log/LogViewer.php',
    'Glpi\\System\\Requirement\\AbstractRequirement' => $baseDir . '/src/Glpi/System/Requirement/AbstractRequirement.php',
    'Glpi\\System\\Requirement\\DatabaseTablesEngine' => $baseDir . '/src/Glpi/System/Requirement/DatabaseTablesEngine.php',
    'Glpi\\System\\Requirement\\DbConfiguration' => $baseDir . '/src/Glpi/System/Requirement/DbConfiguration.php',
    'Glpi\\System\\Requirement\\DbEngine' => $baseDir . '/src/Glpi/System/Requirement/DbEngine.php',
    'Glpi\\System\\Requirement\\DbTimezones' => $baseDir . '/src/Glpi/System/Requirement/DbTimezones.php',
    'Glpi\\System\\Requirement\\DirectoriesWriteAccess' => $baseDir . '/src/Glpi/System/Requirement/DirectoriesWriteAccess.php',
    'Glpi\\System\\Requirement\\DirectoryWriteAccess' => $baseDir . '/src/Glpi/System/Requirement/DirectoryWriteAccess.php',
    'Glpi\\System\\Requirement\\Extension' => $baseDir . '/src/Glpi/System/Requirement/Extension.php',
    'Glpi\\System\\Requirement\\ExtensionClass' => $baseDir . '/src/Glpi/System/Requirement/ExtensionClass.php',
    'Glpi\\System\\Requirement\\ExtensionConstant' => $baseDir . '/src/Glpi/System/Requirement/ExtensionConstant.php',
    'Glpi\\System\\Requirement\\ExtensionFunction' => $baseDir . '/src/Glpi/System/Requirement/ExtensionFunction.php',
    'Glpi\\System\\Requirement\\ExtensionGroup' => $baseDir . '/src/Glpi/System/Requirement/ExtensionGroup.php',
    'Glpi\\System\\Requirement\\IntegerSize' => $baseDir . '/src/Glpi/System/Requirement/IntegerSize.php',
    'Glpi\\System\\Requirement\\LogsWriteAccess' => $baseDir . '/src/Glpi/System/Requirement/LogsWriteAccess.php',
    'Glpi\\System\\Requirement\\MemoryLimit' => $baseDir . '/src/Glpi/System/Requirement/MemoryLimit.php',
    'Glpi\\System\\Requirement\\PhpSupportedVersion' => $baseDir . '/src/Glpi/System/Requirement/PhpSupportedVersion.php',
    'Glpi\\System\\Requirement\\PhpVersion' => $baseDir . '/src/Glpi/System/Requirement/PhpVersion.php',
    'Glpi\\System\\Requirement\\RequirementInterface' => $baseDir . '/src/Glpi/System/Requirement/RequirementInterface.php',
    'Glpi\\System\\Requirement\\SeLinux' => $baseDir . '/src/Glpi/System/Requirement/SeLinux.php',
    'Glpi\\System\\Requirement\\SessionsConfiguration' => $baseDir . '/src/Glpi/System/Requirement/SessionsConfiguration.php',
    'Glpi\\System\\Requirement\\SessionsSecurityConfiguration' => $baseDir . '/src/Glpi/System/Requirement/SessionsSecurityConfiguration.php',
    'Glpi\\System\\RequirementsList' => $baseDir . '/src/Glpi/System/RequirementsList.php',
    'Glpi\\System\\RequirementsManager' => $baseDir . '/src/Glpi/System/RequirementsManager.php',
    'Glpi\\System\\Status\\StatusChecker' => $baseDir . '/src/Glpi/System/Status/StatusChecker.php',
    'Glpi\\System\\Variables' => $baseDir . '/src/Glpi/System/Variables.php',
    'Glpi\\Team\\Team' => $baseDir . '/src/Glpi/Team/Team.php',
    'Glpi\\Toolbox\\ArrayNormalizer' => $baseDir . '/src/Glpi/Toolbox/ArrayNormalizer.php',
    'Glpi\\Toolbox\\ArrayPathAccessor' => $baseDir . '/src/Glpi/Toolbox/ArrayPathAccessor.php',
    'Glpi\\Toolbox\\DataExport' => $baseDir . '/src/Glpi/Toolbox/DataExport.php',
    'Glpi\\Toolbox\\DatabaseSchema' => $baseDir . '/src/Glpi/Toolbox/DatabaseSchema.php',
    'Glpi\\Toolbox\\Filesystem' => $baseDir . '/src/Glpi/Toolbox/Filesystem.php',
    'Glpi\\Toolbox\\FrontEnd' => $baseDir . '/src/Glpi/Toolbox/FrontEnd.php',
    'Glpi\\Toolbox\\MapperInterface' => $baseDir . '/src/Glpi/Toolbox/MapperInterface.php',
    'Glpi\\Toolbox\\MarkdownBuilder' => $baseDir . '/src/Glpi/Toolbox/MarkdownBuilder.php',
    'Glpi\\Toolbox\\MarkdownRenderer' => $baseDir . '/src/Glpi/Toolbox/MarkdownRenderer.php',
    'Glpi\\Toolbox\\SanitizedStringsDecoder' => $baseDir . '/src/Glpi/Toolbox/SanitizedStringsDecoder.php',
    'Glpi\\Toolbox\\Sanitizer' => $baseDir . '/src/Glpi/Toolbox/Sanitizer.php',
    'Glpi\\Toolbox\\SingletonTrait' => $baseDir . '/src/Glpi/Toolbox/SingletonTrait.php',
    'Glpi\\Toolbox\\URL' => $baseDir . '/src/Glpi/Toolbox/URL.php',
    'Glpi\\Toolbox\\UuidStore' => $baseDir . '/src/Glpi/Toolbox/UuidStore.php',
    'Glpi\\Toolbox\\VersionParser' => $baseDir . '/src/Glpi/Toolbox/VersionParser.php',
    'Glpi\\UI\\IllustrationManager' => $baseDir . '/src/Glpi/UI/IllustrationManager.php',
    'Glpi\\UI\\Theme' => $baseDir . '/src/Glpi/UI/Theme.php',
    'Glpi\\UI\\ThemeManager' => $baseDir . '/src/Glpi/UI/ThemeManager.php',
    'Glpi\\Urgency' => $baseDir . '/src/Glpi/Urgency.php',
    'GraphQL\\Deferred' => $vendorDir . '/webonyx/graphql-php/src/Deferred.php',
    'GraphQL\\Error\\ClientAware' => $vendorDir . '/webonyx/graphql-php/src/Error/ClientAware.php',
    'GraphQL\\Error\\CoercionError' => $vendorDir . '/webonyx/graphql-php/src/Error/CoercionError.php',
    'GraphQL\\Error\\DebugFlag' => $vendorDir . '/webonyx/graphql-php/src/Error/DebugFlag.php',
    'GraphQL\\Error\\Error' => $vendorDir . '/webonyx/graphql-php/src/Error/Error.php',
    'GraphQL\\Error\\FormattedError' => $vendorDir . '/webonyx/graphql-php/src/Error/FormattedError.php',
    'GraphQL\\Error\\InvariantViolation' => $vendorDir . '/webonyx/graphql-php/src/Error/InvariantViolation.php',
    'GraphQL\\Error\\ProvidesExtensions' => $vendorDir . '/webonyx/graphql-php/src/Error/ProvidesExtensions.php',
    'GraphQL\\Error\\SerializationError' => $vendorDir . '/webonyx/graphql-php/src/Error/SerializationError.php',
    'GraphQL\\Error\\SyntaxError' => $vendorDir . '/webonyx/graphql-php/src/Error/SyntaxError.php',
    'GraphQL\\Error\\UserError' => $vendorDir . '/webonyx/graphql-php/src/Error/UserError.php',
    'GraphQL\\Error\\Warning' => $vendorDir . '/webonyx/graphql-php/src/Error/Warning.php',
    'GraphQL\\Executor\\ExecutionContext' => $vendorDir . '/webonyx/graphql-php/src/Executor/ExecutionContext.php',
    'GraphQL\\Executor\\ExecutionResult' => $vendorDir . '/webonyx/graphql-php/src/Executor/ExecutionResult.php',
    'GraphQL\\Executor\\Executor' => $vendorDir . '/webonyx/graphql-php/src/Executor/Executor.php',
    'GraphQL\\Executor\\ExecutorImplementation' => $vendorDir . '/webonyx/graphql-php/src/Executor/ExecutorImplementation.php',
    'GraphQL\\Executor\\Promise\\Adapter\\AmpPromiseAdapter' => $vendorDir . '/webonyx/graphql-php/src/Executor/Promise/Adapter/AmpPromiseAdapter.php',
    'GraphQL\\Executor\\Promise\\Adapter\\ReactPromiseAdapter' => $vendorDir . '/webonyx/graphql-php/src/Executor/Promise/Adapter/ReactPromiseAdapter.php',
    'GraphQL\\Executor\\Promise\\Adapter\\SyncPromise' => $vendorDir . '/webonyx/graphql-php/src/Executor/Promise/Adapter/SyncPromise.php',
    'GraphQL\\Executor\\Promise\\Adapter\\SyncPromiseAdapter' => $vendorDir . '/webonyx/graphql-php/src/Executor/Promise/Adapter/SyncPromiseAdapter.php',
    'GraphQL\\Executor\\Promise\\Promise' => $vendorDir . '/webonyx/graphql-php/src/Executor/Promise/Promise.php',
    'GraphQL\\Executor\\Promise\\PromiseAdapter' => $vendorDir . '/webonyx/graphql-php/src/Executor/Promise/PromiseAdapter.php',
    'GraphQL\\Executor\\ReferenceExecutor' => $vendorDir . '/webonyx/graphql-php/src/Executor/ReferenceExecutor.php',
    'GraphQL\\Executor\\ScopedContext' => $vendorDir . '/webonyx/graphql-php/src/Executor/ScopedContext.php',
    'GraphQL\\Executor\\Values' => $vendorDir . '/webonyx/graphql-php/src/Executor/Values.php',
    'GraphQL\\GraphQL' => $vendorDir . '/webonyx/graphql-php/src/GraphQL.php',
    'GraphQL\\Language\\AST\\ArgumentNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/ArgumentNode.php',
    'GraphQL\\Language\\AST\\BooleanValueNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/BooleanValueNode.php',
    'GraphQL\\Language\\AST\\DefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/DefinitionNode.php',
    'GraphQL\\Language\\AST\\DirectiveDefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/DirectiveDefinitionNode.php',
    'GraphQL\\Language\\AST\\DirectiveNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/DirectiveNode.php',
    'GraphQL\\Language\\AST\\DocumentNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/DocumentNode.php',
    'GraphQL\\Language\\AST\\EnumTypeDefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/EnumTypeDefinitionNode.php',
    'GraphQL\\Language\\AST\\EnumTypeExtensionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/EnumTypeExtensionNode.php',
    'GraphQL\\Language\\AST\\EnumValueDefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/EnumValueDefinitionNode.php',
    'GraphQL\\Language\\AST\\EnumValueNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/EnumValueNode.php',
    'GraphQL\\Language\\AST\\ExecutableDefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/ExecutableDefinitionNode.php',
    'GraphQL\\Language\\AST\\FieldDefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/FieldDefinitionNode.php',
    'GraphQL\\Language\\AST\\FieldNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/FieldNode.php',
    'GraphQL\\Language\\AST\\FloatValueNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/FloatValueNode.php',
    'GraphQL\\Language\\AST\\FragmentDefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/FragmentDefinitionNode.php',
    'GraphQL\\Language\\AST\\FragmentSpreadNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/FragmentSpreadNode.php',
    'GraphQL\\Language\\AST\\HasSelectionSet' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/HasSelectionSet.php',
    'GraphQL\\Language\\AST\\InlineFragmentNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/InlineFragmentNode.php',
    'GraphQL\\Language\\AST\\InputObjectTypeDefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/InputObjectTypeDefinitionNode.php',
    'GraphQL\\Language\\AST\\InputObjectTypeExtensionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/InputObjectTypeExtensionNode.php',
    'GraphQL\\Language\\AST\\InputValueDefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/InputValueDefinitionNode.php',
    'GraphQL\\Language\\AST\\IntValueNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/IntValueNode.php',
    'GraphQL\\Language\\AST\\InterfaceTypeDefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/InterfaceTypeDefinitionNode.php',
    'GraphQL\\Language\\AST\\InterfaceTypeExtensionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/InterfaceTypeExtensionNode.php',
    'GraphQL\\Language\\AST\\ListTypeNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/ListTypeNode.php',
    'GraphQL\\Language\\AST\\ListValueNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/ListValueNode.php',
    'GraphQL\\Language\\AST\\Location' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/Location.php',
    'GraphQL\\Language\\AST\\NameNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/NameNode.php',
    'GraphQL\\Language\\AST\\NamedTypeNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/NamedTypeNode.php',
    'GraphQL\\Language\\AST\\Node' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/Node.php',
    'GraphQL\\Language\\AST\\NodeKind' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/NodeKind.php',
    'GraphQL\\Language\\AST\\NodeList' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/NodeList.php',
    'GraphQL\\Language\\AST\\NonNullTypeNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/NonNullTypeNode.php',
    'GraphQL\\Language\\AST\\NullValueNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/NullValueNode.php',
    'GraphQL\\Language\\AST\\ObjectFieldNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/ObjectFieldNode.php',
    'GraphQL\\Language\\AST\\ObjectTypeDefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/ObjectTypeDefinitionNode.php',
    'GraphQL\\Language\\AST\\ObjectTypeExtensionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/ObjectTypeExtensionNode.php',
    'GraphQL\\Language\\AST\\ObjectValueNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/ObjectValueNode.php',
    'GraphQL\\Language\\AST\\OperationDefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/OperationDefinitionNode.php',
    'GraphQL\\Language\\AST\\OperationTypeDefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/OperationTypeDefinitionNode.php',
    'GraphQL\\Language\\AST\\ScalarTypeDefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/ScalarTypeDefinitionNode.php',
    'GraphQL\\Language\\AST\\ScalarTypeExtensionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/ScalarTypeExtensionNode.php',
    'GraphQL\\Language\\AST\\SchemaDefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/SchemaDefinitionNode.php',
    'GraphQL\\Language\\AST\\SchemaExtensionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/SchemaExtensionNode.php',
    'GraphQL\\Language\\AST\\SelectionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/SelectionNode.php',
    'GraphQL\\Language\\AST\\SelectionSetNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/SelectionSetNode.php',
    'GraphQL\\Language\\AST\\StringValueNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/StringValueNode.php',
    'GraphQL\\Language\\AST\\TypeDefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/TypeDefinitionNode.php',
    'GraphQL\\Language\\AST\\TypeExtensionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/TypeExtensionNode.php',
    'GraphQL\\Language\\AST\\TypeNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/TypeNode.php',
    'GraphQL\\Language\\AST\\TypeSystemDefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/TypeSystemDefinitionNode.php',
    'GraphQL\\Language\\AST\\TypeSystemExtensionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/TypeSystemExtensionNode.php',
    'GraphQL\\Language\\AST\\UnionTypeDefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/UnionTypeDefinitionNode.php',
    'GraphQL\\Language\\AST\\UnionTypeExtensionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/UnionTypeExtensionNode.php',
    'GraphQL\\Language\\AST\\ValueNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/ValueNode.php',
    'GraphQL\\Language\\AST\\VariableDefinitionNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/VariableDefinitionNode.php',
    'GraphQL\\Language\\AST\\VariableNode' => $vendorDir . '/webonyx/graphql-php/src/Language/AST/VariableNode.php',
    'GraphQL\\Language\\BlockString' => $vendorDir . '/webonyx/graphql-php/src/Language/BlockString.php',
    'GraphQL\\Language\\DirectiveLocation' => $vendorDir . '/webonyx/graphql-php/src/Language/DirectiveLocation.php',
    'GraphQL\\Language\\Lexer' => $vendorDir . '/webonyx/graphql-php/src/Language/Lexer.php',
    'GraphQL\\Language\\Parser' => $vendorDir . '/webonyx/graphql-php/src/Language/Parser.php',
    'GraphQL\\Language\\Printer' => $vendorDir . '/webonyx/graphql-php/src/Language/Printer.php',
    'GraphQL\\Language\\Source' => $vendorDir . '/webonyx/graphql-php/src/Language/Source.php',
    'GraphQL\\Language\\SourceLocation' => $vendorDir . '/webonyx/graphql-php/src/Language/SourceLocation.php',
    'GraphQL\\Language\\Token' => $vendorDir . '/webonyx/graphql-php/src/Language/Token.php',
    'GraphQL\\Language\\Visitor' => $vendorDir . '/webonyx/graphql-php/src/Language/Visitor.php',
    'GraphQL\\Language\\VisitorOperation' => $vendorDir . '/webonyx/graphql-php/src/Language/VisitorOperation.php',
    'GraphQL\\Language\\VisitorRemoveNode' => $vendorDir . '/webonyx/graphql-php/src/Language/VisitorRemoveNode.php',
    'GraphQL\\Language\\VisitorSkipNode' => $vendorDir . '/webonyx/graphql-php/src/Language/VisitorSkipNode.php',
    'GraphQL\\Language\\VisitorStop' => $vendorDir . '/webonyx/graphql-php/src/Language/VisitorStop.php',
    'GraphQL\\Server\\Exception\\BatchedQueriesAreNotSupported' => $vendorDir . '/webonyx/graphql-php/src/Server/Exception/BatchedQueriesAreNotSupported.php',
    'GraphQL\\Server\\Exception\\CannotParseJsonBody' => $vendorDir . '/webonyx/graphql-php/src/Server/Exception/CannotParseJsonBody.php',
    'GraphQL\\Server\\Exception\\CannotParseVariables' => $vendorDir . '/webonyx/graphql-php/src/Server/Exception/CannotParseVariables.php',
    'GraphQL\\Server\\Exception\\CannotReadBody' => $vendorDir . '/webonyx/graphql-php/src/Server/Exception/CannotReadBody.php',
    'GraphQL\\Server\\Exception\\FailedToDetermineOperationType' => $vendorDir . '/webonyx/graphql-php/src/Server/Exception/FailedToDetermineOperationType.php',
    'GraphQL\\Server\\Exception\\GetMethodSupportsOnlyQueryOperation' => $vendorDir . '/webonyx/graphql-php/src/Server/Exception/GetMethodSupportsOnlyQueryOperation.php',
    'GraphQL\\Server\\Exception\\HttpMethodNotSupported' => $vendorDir . '/webonyx/graphql-php/src/Server/Exception/HttpMethodNotSupported.php',
    'GraphQL\\Server\\Exception\\InvalidOperationParameter' => $vendorDir . '/webonyx/graphql-php/src/Server/Exception/InvalidOperationParameter.php',
    'GraphQL\\Server\\Exception\\InvalidQueryIdParameter' => $vendorDir . '/webonyx/graphql-php/src/Server/Exception/InvalidQueryIdParameter.php',
    'GraphQL\\Server\\Exception\\InvalidQueryParameter' => $vendorDir . '/webonyx/graphql-php/src/Server/Exception/InvalidQueryParameter.php',
    'GraphQL\\Server\\Exception\\MissingContentTypeHeader' => $vendorDir . '/webonyx/graphql-php/src/Server/Exception/MissingContentTypeHeader.php',
    'GraphQL\\Server\\Exception\\MissingQueryOrQueryIdParameter' => $vendorDir . '/webonyx/graphql-php/src/Server/Exception/MissingQueryOrQueryIdParameter.php',
    'GraphQL\\Server\\Exception\\PersistedQueriesAreNotSupported' => $vendorDir . '/webonyx/graphql-php/src/Server/Exception/PersistedQueriesAreNotSupported.php',
    'GraphQL\\Server\\Exception\\UnexpectedContentType' => $vendorDir . '/webonyx/graphql-php/src/Server/Exception/UnexpectedContentType.php',
    'GraphQL\\Server\\Helper' => $vendorDir . '/webonyx/graphql-php/src/Server/Helper.php',
    'GraphQL\\Server\\OperationParams' => $vendorDir . '/webonyx/graphql-php/src/Server/OperationParams.php',
    'GraphQL\\Server\\RequestError' => $vendorDir . '/webonyx/graphql-php/src/Server/RequestError.php',
    'GraphQL\\Server\\ServerConfig' => $vendorDir . '/webonyx/graphql-php/src/Server/ServerConfig.php',
    'GraphQL\\Server\\StandardServer' => $vendorDir . '/webonyx/graphql-php/src/Server/StandardServer.php',
    'GraphQL\\Type\\Definition\\AbstractType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/AbstractType.php',
    'GraphQL\\Type\\Definition\\Argument' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/Argument.php',
    'GraphQL\\Type\\Definition\\BooleanType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/BooleanType.php',
    'GraphQL\\Type\\Definition\\CompositeType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/CompositeType.php',
    'GraphQL\\Type\\Definition\\CustomScalarType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/CustomScalarType.php',
    'GraphQL\\Type\\Definition\\Deprecated' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/Deprecated.php',
    'GraphQL\\Type\\Definition\\Description' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/Description.php',
    'GraphQL\\Type\\Definition\\Directive' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/Directive.php',
    'GraphQL\\Type\\Definition\\EnumType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/EnumType.php',
    'GraphQL\\Type\\Definition\\EnumValueDefinition' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/EnumValueDefinition.php',
    'GraphQL\\Type\\Definition\\FieldDefinition' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/FieldDefinition.php',
    'GraphQL\\Type\\Definition\\FloatType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/FloatType.php',
    'GraphQL\\Type\\Definition\\HasFieldsType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/HasFieldsType.php',
    'GraphQL\\Type\\Definition\\HasFieldsTypeImplementation' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/HasFieldsTypeImplementation.php',
    'GraphQL\\Type\\Definition\\IDType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/IDType.php',
    'GraphQL\\Type\\Definition\\ImplementingType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/ImplementingType.php',
    'GraphQL\\Type\\Definition\\ImplementingTypeImplementation' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/ImplementingTypeImplementation.php',
    'GraphQL\\Type\\Definition\\InputObjectField' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/InputObjectField.php',
    'GraphQL\\Type\\Definition\\InputObjectType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/InputObjectType.php',
    'GraphQL\\Type\\Definition\\InputType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/InputType.php',
    'GraphQL\\Type\\Definition\\IntType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/IntType.php',
    'GraphQL\\Type\\Definition\\InterfaceType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/InterfaceType.php',
    'GraphQL\\Type\\Definition\\LeafType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/LeafType.php',
    'GraphQL\\Type\\Definition\\ListOfType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/ListOfType.php',
    'GraphQL\\Type\\Definition\\NamedType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/NamedType.php',
    'GraphQL\\Type\\Definition\\NamedTypeImplementation' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/NamedTypeImplementation.php',
    'GraphQL\\Type\\Definition\\NonNull' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/NonNull.php',
    'GraphQL\\Type\\Definition\\NullableType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/NullableType.php',
    'GraphQL\\Type\\Definition\\ObjectType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/ObjectType.php',
    'GraphQL\\Type\\Definition\\OutputType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/OutputType.php',
    'GraphQL\\Type\\Definition\\PhpEnumType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/PhpEnumType.php',
    'GraphQL\\Type\\Definition\\QueryPlan' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/QueryPlan.php',
    'GraphQL\\Type\\Definition\\ResolveInfo' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/ResolveInfo.php',
    'GraphQL\\Type\\Definition\\ScalarType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/ScalarType.php',
    'GraphQL\\Type\\Definition\\StringType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/StringType.php',
    'GraphQL\\Type\\Definition\\Type' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/Type.php',
    'GraphQL\\Type\\Definition\\UnionType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/UnionType.php',
    'GraphQL\\Type\\Definition\\UnmodifiedType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/UnmodifiedType.php',
    'GraphQL\\Type\\Definition\\UnresolvedFieldDefinition' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/UnresolvedFieldDefinition.php',
    'GraphQL\\Type\\Definition\\WrappingType' => $vendorDir . '/webonyx/graphql-php/src/Type/Definition/WrappingType.php',
    'GraphQL\\Type\\Introspection' => $vendorDir . '/webonyx/graphql-php/src/Type/Introspection.php',
    'GraphQL\\Type\\Schema' => $vendorDir . '/webonyx/graphql-php/src/Type/Schema.php',
    'GraphQL\\Type\\SchemaConfig' => $vendorDir . '/webonyx/graphql-php/src/Type/SchemaConfig.php',
    'GraphQL\\Type\\SchemaValidationContext' => $vendorDir . '/webonyx/graphql-php/src/Type/SchemaValidationContext.php',
    'GraphQL\\Type\\TypeKind' => $vendorDir . '/webonyx/graphql-php/src/Type/TypeKind.php',
    'GraphQL\\Type\\Validation\\InputObjectCircularRefs' => $vendorDir . '/webonyx/graphql-php/src/Type/Validation/InputObjectCircularRefs.php',
    'GraphQL\\Utils\\AST' => $vendorDir . '/webonyx/graphql-php/src/Utils/AST.php',
    'GraphQL\\Utils\\ASTDefinitionBuilder' => $vendorDir . '/webonyx/graphql-php/src/Utils/ASTDefinitionBuilder.php',
    'GraphQL\\Utils\\BreakingChangesFinder' => $vendorDir . '/webonyx/graphql-php/src/Utils/BreakingChangesFinder.php',
    'GraphQL\\Utils\\BuildClientSchema' => $vendorDir . '/webonyx/graphql-php/src/Utils/BuildClientSchema.php',
    'GraphQL\\Utils\\BuildSchema' => $vendorDir . '/webonyx/graphql-php/src/Utils/BuildSchema.php',
    'GraphQL\\Utils\\InterfaceImplementations' => $vendorDir . '/webonyx/graphql-php/src/Utils/InterfaceImplementations.php',
    'GraphQL\\Utils\\LazyException' => $vendorDir . '/webonyx/graphql-php/src/Utils/LazyException.php',
    'GraphQL\\Utils\\LexicalDistance' => $vendorDir . '/webonyx/graphql-php/src/Utils/LexicalDistance.php',
    'GraphQL\\Utils\\MixedStore' => $vendorDir . '/webonyx/graphql-php/src/Utils/MixedStore.php',
    'GraphQL\\Utils\\PairSet' => $vendorDir . '/webonyx/graphql-php/src/Utils/PairSet.php',
    'GraphQL\\Utils\\PhpDoc' => $vendorDir . '/webonyx/graphql-php/src/Utils/PhpDoc.php',
    'GraphQL\\Utils\\SchemaExtender' => $vendorDir . '/webonyx/graphql-php/src/Utils/SchemaExtender.php',
    'GraphQL\\Utils\\SchemaPrinter' => $vendorDir . '/webonyx/graphql-php/src/Utils/SchemaPrinter.php',
    'GraphQL\\Utils\\TypeComparators' => $vendorDir . '/webonyx/graphql-php/src/Utils/TypeComparators.php',
    'GraphQL\\Utils\\TypeInfo' => $vendorDir . '/webonyx/graphql-php/src/Utils/TypeInfo.php',
    'GraphQL\\Utils\\Utils' => $vendorDir . '/webonyx/graphql-php/src/Utils/Utils.php',
    'GraphQL\\Utils\\Value' => $vendorDir . '/webonyx/graphql-php/src/Utils/Value.php',
    'GraphQL\\Validator\\DocumentValidator' => $vendorDir . '/webonyx/graphql-php/src/Validator/DocumentValidator.php',
    'GraphQL\\Validator\\QueryValidationContext' => $vendorDir . '/webonyx/graphql-php/src/Validator/QueryValidationContext.php',
    'GraphQL\\Validator\\Rules\\CustomValidationRule' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/CustomValidationRule.php',
    'GraphQL\\Validator\\Rules\\DisableIntrospection' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/DisableIntrospection.php',
    'GraphQL\\Validator\\Rules\\ExecutableDefinitions' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/ExecutableDefinitions.php',
    'GraphQL\\Validator\\Rules\\FieldsOnCorrectType' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/FieldsOnCorrectType.php',
    'GraphQL\\Validator\\Rules\\FragmentsOnCompositeTypes' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/FragmentsOnCompositeTypes.php',
    'GraphQL\\Validator\\Rules\\KnownArgumentNames' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/KnownArgumentNames.php',
    'GraphQL\\Validator\\Rules\\KnownArgumentNamesOnDirectives' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/KnownArgumentNamesOnDirectives.php',
    'GraphQL\\Validator\\Rules\\KnownDirectives' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/KnownDirectives.php',
    'GraphQL\\Validator\\Rules\\KnownFragmentNames' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/KnownFragmentNames.php',
    'GraphQL\\Validator\\Rules\\KnownTypeNames' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/KnownTypeNames.php',
    'GraphQL\\Validator\\Rules\\LoneAnonymousOperation' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/LoneAnonymousOperation.php',
    'GraphQL\\Validator\\Rules\\LoneSchemaDefinition' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/LoneSchemaDefinition.php',
    'GraphQL\\Validator\\Rules\\NoFragmentCycles' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/NoFragmentCycles.php',
    'GraphQL\\Validator\\Rules\\NoUndefinedVariables' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/NoUndefinedVariables.php',
    'GraphQL\\Validator\\Rules\\NoUnusedFragments' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/NoUnusedFragments.php',
    'GraphQL\\Validator\\Rules\\NoUnusedVariables' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/NoUnusedVariables.php',
    'GraphQL\\Validator\\Rules\\OneOfInputObjectsRule' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/OneOfInputObjectsRule.php',
    'GraphQL\\Validator\\Rules\\OverlappingFieldsCanBeMerged' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/OverlappingFieldsCanBeMerged.php',
    'GraphQL\\Validator\\Rules\\PossibleFragmentSpreads' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/PossibleFragmentSpreads.php',
    'GraphQL\\Validator\\Rules\\PossibleTypeExtensions' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/PossibleTypeExtensions.php',
    'GraphQL\\Validator\\Rules\\ProvidedRequiredArguments' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/ProvidedRequiredArguments.php',
    'GraphQL\\Validator\\Rules\\ProvidedRequiredArgumentsOnDirectives' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/ProvidedRequiredArgumentsOnDirectives.php',
    'GraphQL\\Validator\\Rules\\QueryComplexity' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/QueryComplexity.php',
    'GraphQL\\Validator\\Rules\\QueryDepth' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/QueryDepth.php',
    'GraphQL\\Validator\\Rules\\QuerySecurityRule' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/QuerySecurityRule.php',
    'GraphQL\\Validator\\Rules\\ScalarLeafs' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/ScalarLeafs.php',
    'GraphQL\\Validator\\Rules\\SingleFieldSubscription' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/SingleFieldSubscription.php',
    'GraphQL\\Validator\\Rules\\UniqueArgumentDefinitionNames' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/UniqueArgumentDefinitionNames.php',
    'GraphQL\\Validator\\Rules\\UniqueArgumentNames' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/UniqueArgumentNames.php',
    'GraphQL\\Validator\\Rules\\UniqueDirectiveNames' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/UniqueDirectiveNames.php',
    'GraphQL\\Validator\\Rules\\UniqueDirectivesPerLocation' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/UniqueDirectivesPerLocation.php',
    'GraphQL\\Validator\\Rules\\UniqueEnumValueNames' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/UniqueEnumValueNames.php',
    'GraphQL\\Validator\\Rules\\UniqueFieldDefinitionNames' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/UniqueFieldDefinitionNames.php',
    'GraphQL\\Validator\\Rules\\UniqueFragmentNames' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/UniqueFragmentNames.php',
    'GraphQL\\Validator\\Rules\\UniqueInputFieldNames' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/UniqueInputFieldNames.php',
    'GraphQL\\Validator\\Rules\\UniqueOperationNames' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/UniqueOperationNames.php',
    'GraphQL\\Validator\\Rules\\UniqueOperationTypes' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/UniqueOperationTypes.php',
    'GraphQL\\Validator\\Rules\\UniqueTypeNames' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/UniqueTypeNames.php',
    'GraphQL\\Validator\\Rules\\UniqueVariableNames' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/UniqueVariableNames.php',
    'GraphQL\\Validator\\Rules\\ValidationRule' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/ValidationRule.php',
    'GraphQL\\Validator\\Rules\\ValuesOfCorrectType' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/ValuesOfCorrectType.php',
    'GraphQL\\Validator\\Rules\\VariablesAreInputTypes' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/VariablesAreInputTypes.php',
    'GraphQL\\Validator\\Rules\\VariablesInAllowedPosition' => $vendorDir . '/webonyx/graphql-php/src/Validator/Rules/VariablesInAllowedPosition.php',
    'GraphQL\\Validator\\SDLValidationContext' => $vendorDir . '/webonyx/graphql-php/src/Validator/SDLValidationContext.php',
    'GraphQL\\Validator\\ValidationContext' => $vendorDir . '/webonyx/graphql-php/src/Validator/ValidationContext.php',
    'Group' => $baseDir . '/src/Group.php',
    'Group_Item' => $baseDir . '/src/Group_Item.php',
    'Group_KnowbaseItem' => $baseDir . '/src/Group_KnowbaseItem.php',
    'Group_Problem' => $baseDir . '/src/Group_Problem.php',
    'Group_RSSFeed' => $baseDir . '/src/Group_RSSFeed.php',
    'Group_Reminder' => $baseDir . '/src/Group_Reminder.php',
    'Group_Ticket' => $baseDir . '/src/Group_Ticket.php',
    'Group_User' => $baseDir . '/src/Group_User.php',
    'GuzzleHttp\\BodySummarizer' => $vendorDir . '/guzzlehttp/guzzle/src/BodySummarizer.php',
    'GuzzleHttp\\BodySummarizerInterface' => $vendorDir . '/guzzlehttp/guzzle/src/BodySummarizerInterface.php',
    'GuzzleHttp\\Client' => $vendorDir . '/guzzlehttp/guzzle/src/Client.php',
    'GuzzleHttp\\ClientInterface' => $vendorDir . '/guzzlehttp/guzzle/src/ClientInterface.php',
    'GuzzleHttp\\ClientTrait' => $vendorDir . '/guzzlehttp/guzzle/src/ClientTrait.php',
    'GuzzleHttp\\Cookie\\CookieJar' => $vendorDir . '/guzzlehttp/guzzle/src/Cookie/CookieJar.php',
    'GuzzleHttp\\Cookie\\CookieJarInterface' => $vendorDir . '/guzzlehttp/guzzle/src/Cookie/CookieJarInterface.php',
    'GuzzleHttp\\Cookie\\FileCookieJar' => $vendorDir . '/guzzlehttp/guzzle/src/Cookie/FileCookieJar.php',
    'GuzzleHttp\\Cookie\\SessionCookieJar' => $vendorDir . '/guzzlehttp/guzzle/src/Cookie/SessionCookieJar.php',
    'GuzzleHttp\\Cookie\\SetCookie' => $vendorDir . '/guzzlehttp/guzzle/src/Cookie/SetCookie.php',
    'GuzzleHttp\\Exception\\BadResponseException' => $vendorDir . '/guzzlehttp/guzzle/src/Exception/BadResponseException.php',
    'GuzzleHttp\\Exception\\ClientException' => $vendorDir . '/guzzlehttp/guzzle/src/Exception/ClientException.php',
    'GuzzleHttp\\Exception\\ConnectException' => $vendorDir . '/guzzlehttp/guzzle/src/Exception/ConnectException.php',
    'GuzzleHttp\\Exception\\GuzzleException' => $vendorDir . '/guzzlehttp/guzzle/src/Exception/GuzzleException.php',
    'GuzzleHttp\\Exception\\InvalidArgumentException' => $vendorDir . '/guzzlehttp/guzzle/src/Exception/InvalidArgumentException.php',
    'GuzzleHttp\\Exception\\RequestException' => $vendorDir . '/guzzlehttp/guzzle/src/Exception/RequestException.php',
    'GuzzleHttp\\Exception\\ServerException' => $vendorDir . '/guzzlehttp/guzzle/src/Exception/ServerException.php',
    'GuzzleHttp\\Exception\\TooManyRedirectsException' => $vendorDir . '/guzzlehttp/guzzle/src/Exception/TooManyRedirectsException.php',
    'GuzzleHttp\\Exception\\TransferException' => $vendorDir . '/guzzlehttp/guzzle/src/Exception/TransferException.php',
    'GuzzleHttp\\HandlerStack' => $vendorDir . '/guzzlehttp/guzzle/src/HandlerStack.php',
    'GuzzleHttp\\Handler\\CurlFactory' => $vendorDir . '/guzzlehttp/guzzle/src/Handler/CurlFactory.php',
    'GuzzleHttp\\Handler\\CurlFactoryInterface' => $vendorDir . '/guzzlehttp/guzzle/src/Handler/CurlFactoryInterface.php',
    'GuzzleHttp\\Handler\\CurlHandler' => $vendorDir . '/guzzlehttp/guzzle/src/Handler/CurlHandler.php',
    'GuzzleHttp\\Handler\\CurlMultiHandler' => $vendorDir . '/guzzlehttp/guzzle/src/Handler/CurlMultiHandler.php',
    'GuzzleHttp\\Handler\\EasyHandle' => $vendorDir . '/guzzlehttp/guzzle/src/Handler/EasyHandle.php',
    'GuzzleHttp\\Handler\\HeaderProcessor' => $vendorDir . '/guzzlehttp/guzzle/src/Handler/HeaderProcessor.php',
    'GuzzleHttp\\Handler\\MockHandler' => $vendorDir . '/guzzlehttp/guzzle/src/Handler/MockHandler.php',
    'GuzzleHttp\\Handler\\Proxy' => $vendorDir . '/guzzlehttp/guzzle/src/Handler/Proxy.php',
    'GuzzleHttp\\Handler\\StreamHandler' => $vendorDir . '/guzzlehttp/guzzle/src/Handler/StreamHandler.php',
    'GuzzleHttp\\MessageFormatter' => $vendorDir . '/guzzlehttp/guzzle/src/MessageFormatter.php',
    'GuzzleHttp\\MessageFormatterInterface' => $vendorDir . '/guzzlehttp/guzzle/src/MessageFormatterInterface.php',
    'GuzzleHttp\\Middleware' => $vendorDir . '/guzzlehttp/guzzle/src/Middleware.php',
    'GuzzleHttp\\Pool' => $vendorDir . '/guzzlehttp/guzzle/src/Pool.php',
    'GuzzleHttp\\PrepareBodyMiddleware' => $vendorDir . '/guzzlehttp/guzzle/src/PrepareBodyMiddleware.php',
    'GuzzleHttp\\Promise\\AggregateException' => $vendorDir . '/guzzlehttp/promises/src/AggregateException.php',
    'GuzzleHttp\\Promise\\CancellationException' => $vendorDir . '/guzzlehttp/promises/src/CancellationException.php',
    'GuzzleHttp\\Promise\\Coroutine' => $vendorDir . '/guzzlehttp/promises/src/Coroutine.php',
    'GuzzleHttp\\Promise\\Create' => $vendorDir . '/guzzlehttp/promises/src/Create.php',
    'GuzzleHttp\\Promise\\Each' => $vendorDir . '/guzzlehttp/promises/src/Each.php',
    'GuzzleHttp\\Promise\\EachPromise' => $vendorDir . '/guzzlehttp/promises/src/EachPromise.php',
    'GuzzleHttp\\Promise\\FulfilledPromise' => $vendorDir . '/guzzlehttp/promises/src/FulfilledPromise.php',
    'GuzzleHttp\\Promise\\Is' => $vendorDir . '/guzzlehttp/promises/src/Is.php',
    'GuzzleHttp\\Promise\\Promise' => $vendorDir . '/guzzlehttp/promises/src/Promise.php',
    'GuzzleHttp\\Promise\\PromiseInterface' => $vendorDir . '/guzzlehttp/promises/src/PromiseInterface.php',
    'GuzzleHttp\\Promise\\PromisorInterface' => $vendorDir . '/guzzlehttp/promises/src/PromisorInterface.php',
    'GuzzleHttp\\Promise\\RejectedPromise' => $vendorDir . '/guzzlehttp/promises/src/RejectedPromise.php',
    'GuzzleHttp\\Promise\\RejectionException' => $vendorDir . '/guzzlehttp/promises/src/RejectionException.php',
    'GuzzleHttp\\Promise\\TaskQueue' => $vendorDir . '/guzzlehttp/promises/src/TaskQueue.php',
    'GuzzleHttp\\Promise\\TaskQueueInterface' => $vendorDir . '/guzzlehttp/promises/src/TaskQueueInterface.php',
    'GuzzleHttp\\Promise\\Utils' => $vendorDir . '/guzzlehttp/promises/src/Utils.php',
    'GuzzleHttp\\Psr7\\AppendStream' => $vendorDir . '/guzzlehttp/psr7/src/AppendStream.php',
    'GuzzleHttp\\Psr7\\BufferStream' => $vendorDir . '/guzzlehttp/psr7/src/BufferStream.php',
    'GuzzleHttp\\Psr7\\CachingStream' => $vendorDir . '/guzzlehttp/psr7/src/CachingStream.php',
    'GuzzleHttp\\Psr7\\DroppingStream' => $vendorDir . '/guzzlehttp/psr7/src/DroppingStream.php',
    'GuzzleHttp\\Psr7\\Exception\\MalformedUriException' => $vendorDir . '/guzzlehttp/psr7/src/Exception/MalformedUriException.php',
    'GuzzleHttp\\Psr7\\FnStream' => $vendorDir . '/guzzlehttp/psr7/src/FnStream.php',
    'GuzzleHttp\\Psr7\\Header' => $vendorDir . '/guzzlehttp/psr7/src/Header.php',
    'GuzzleHttp\\Psr7\\HttpFactory' => $vendorDir . '/guzzlehttp/psr7/src/HttpFactory.php',
    'GuzzleHttp\\Psr7\\InflateStream' => $vendorDir . '/guzzlehttp/psr7/src/InflateStream.php',
    'GuzzleHttp\\Psr7\\LazyOpenStream' => $vendorDir . '/guzzlehttp/psr7/src/LazyOpenStream.php',
    'GuzzleHttp\\Psr7\\LimitStream' => $vendorDir . '/guzzlehttp/psr7/src/LimitStream.php',
    'GuzzleHttp\\Psr7\\Message' => $vendorDir . '/guzzlehttp/psr7/src/Message.php',
    'GuzzleHttp\\Psr7\\MessageTrait' => $vendorDir . '/guzzlehttp/psr7/src/MessageTrait.php',
    'GuzzleHttp\\Psr7\\MimeType' => $vendorDir . '/guzzlehttp/psr7/src/MimeType.php',
    'GuzzleHttp\\Psr7\\MultipartStream' => $vendorDir . '/guzzlehttp/psr7/src/MultipartStream.php',
    'GuzzleHttp\\Psr7\\NoSeekStream' => $vendorDir . '/guzzlehttp/psr7/src/NoSeekStream.php',
    'GuzzleHttp\\Psr7\\PumpStream' => $vendorDir . '/guzzlehttp/psr7/src/PumpStream.php',
    'GuzzleHttp\\Psr7\\Query' => $vendorDir . '/guzzlehttp/psr7/src/Query.php',
    'GuzzleHttp\\Psr7\\Request' => $vendorDir . '/guzzlehttp/psr7/src/Request.php',
    'GuzzleHttp\\Psr7\\Response' => $vendorDir . '/guzzlehttp/psr7/src/Response.php',
    'GuzzleHttp\\Psr7\\Rfc7230' => $vendorDir . '/guzzlehttp/psr7/src/Rfc7230.php',
    'GuzzleHttp\\Psr7\\ServerRequest' => $vendorDir . '/guzzlehttp/psr7/src/ServerRequest.php',
    'GuzzleHttp\\Psr7\\Stream' => $vendorDir . '/guzzlehttp/psr7/src/Stream.php',
    'GuzzleHttp\\Psr7\\StreamDecoratorTrait' => $vendorDir . '/guzzlehttp/psr7/src/StreamDecoratorTrait.php',
    'GuzzleHttp\\Psr7\\StreamWrapper' => $vendorDir . '/guzzlehttp/psr7/src/StreamWrapper.php',
    'GuzzleHttp\\Psr7\\UploadedFile' => $vendorDir . '/guzzlehttp/psr7/src/UploadedFile.php',
    'GuzzleHttp\\Psr7\\Uri' => $vendorDir . '/guzzlehttp/psr7/src/Uri.php',
    'GuzzleHttp\\Psr7\\UriComparator' => $vendorDir . '/guzzlehttp/psr7/src/UriComparator.php',
    'GuzzleHttp\\Psr7\\UriNormalizer' => $vendorDir . '/guzzlehttp/psr7/src/UriNormalizer.php',
    'GuzzleHttp\\Psr7\\UriResolver' => $vendorDir . '/guzzlehttp/psr7/src/UriResolver.php',
    'GuzzleHttp\\Psr7\\Utils' => $vendorDir . '/guzzlehttp/psr7/src/Utils.php',
    'GuzzleHttp\\RedirectMiddleware' => $vendorDir . '/guzzlehttp/guzzle/src/RedirectMiddleware.php',
    'GuzzleHttp\\RequestOptions' => $vendorDir . '/guzzlehttp/guzzle/src/RequestOptions.php',
    'GuzzleHttp\\RetryMiddleware' => $vendorDir . '/guzzlehttp/guzzle/src/RetryMiddleware.php',
    'GuzzleHttp\\TransferStats' => $vendorDir . '/guzzlehttp/guzzle/src/TransferStats.php',
    'GuzzleHttp\\Utils' => $vendorDir . '/guzzlehttp/guzzle/src/Utils.php',
    'HTMLCompositeTableInterface' => $baseDir . '/src/HTMLCompositeTableInterface.php',
    'HTMLTableBase' => $baseDir . '/src/HTMLTableBase.php',
    'HTMLTableCell' => $baseDir . '/src/HTMLTableCell.php',
    'HTMLTableCellFatherCoherentHeader' => $baseDir . '/src/HTMLTableCellFatherCoherentHeader.php',
    'HTMLTableCellFatherSameRow' => $baseDir . '/src/HTMLTableCellFatherSameRow.php',
    'HTMLTableCellWithoutFather' => $baseDir . '/src/HTMLTableCellWithoutFather.php',
    'HTMLTableEntity' => $baseDir . '/src/HTMLTableEntity.php',
    'HTMLTableGroup' => $baseDir . '/src/HTMLTableGroup.php',
    'HTMLTableHeader' => $baseDir . '/src/HTMLTableHeader.php',
    'HTMLTableMain' => $baseDir . '/src/HTMLTableMain.php',
    'HTMLTableRow' => $baseDir . '/src/HTMLTableRow.php',
    'HTMLTableSubHeader' => $baseDir . '/src/HTMLTableSubHeader.php',
    'HTMLTableSuperHeader' => $baseDir . '/src/HTMLTableSuperHeader.php',
    'HTMLTableUnknownHeader' => $baseDir . '/src/HTMLTableUnknownHeader.php',
    'HTMLTableUnknownHeaders' => $baseDir . '/src/HTMLTableUnknownHeaders.php',
    'HTMLTableUnknownHeadersOrder' => $baseDir . '/src/HTMLTableUnknownHeadersOrder.php',
    'Holiday' => $baseDir . '/src/Holiday.php',
    'Html' => $baseDir . '/src/Html.php',
    'Html2Text\\Html2Text' => $vendorDir . '/html2text/html2text/src/Html2Text.php',
    'IPAddress' => $baseDir . '/src/IPAddress.php',
    'IPAddress_IPNetwork' => $baseDir . '/src/IPAddress_IPNetwork.php',
    'IPNetmask' => $baseDir . '/src/IPNetmask.php',
    'IPNetwork' => $baseDir . '/src/IPNetwork.php',
    'IPNetwork_Vlan' => $baseDir . '/src/IPNetwork_Vlan.php',
    'ITILCategory' => $baseDir . '/src/ITILCategory.php',
    'ITILFollowup' => $baseDir . '/src/ITILFollowup.php',
    'ITILFollowupTemplate' => $baseDir . '/src/ITILFollowupTemplate.php',
    'ITILReminder' => $baseDir . '/src/ITILReminder.php',
    'ITILSolution' => $baseDir . '/src/ITILSolution.php',
    'ITILSubItemRights' => $baseDir . '/src/ITILSubItemRights.php',
    'ITILTemplate' => $baseDir . '/src/ITILTemplate.php',
    'ITILTemplateField' => $baseDir . '/src/ITILTemplateField.php',
    'ITILTemplateHiddenField' => $baseDir . '/src/ITILTemplateHiddenField.php',
    'ITILTemplateMandatoryField' => $baseDir . '/src/ITILTemplateMandatoryField.php',
    'ITILTemplatePredefinedField' => $baseDir . '/src/ITILTemplatePredefinedField.php',
    'ITILTemplateReadonlyField' => $baseDir . '/src/ITILTemplateReadonlyField.php',
    'ITILValidationTemplate' => $baseDir . '/src/ITILValidationTemplate.php',
    'ITILValidationTemplate_Target' => $baseDir . '/src/ITILValidationTemplate_Target.php',
    'ITIL_ValidationStep' => $baseDir . '/src/ITIL_ValidationStep.php',
    'ImageFormat' => $baseDir . '/src/ImageFormat.php',
    'ImageResolution' => $baseDir . '/src/ImageResolution.php',
    'Impact' => $baseDir . '/src/Impact.php',
    'ImpactCompound' => $baseDir . '/src/ImpactCompound.php',
    'ImpactContext' => $baseDir . '/src/ImpactContext.php',
    'ImpactItem' => $baseDir . '/src/ImpactItem.php',
    'ImpactRelation' => $baseDir . '/src/ImpactRelation.php',
    'Infocom' => $baseDir . '/src/Infocom.php',
    'InterfaceType' => $baseDir . '/src/InterfaceType.php',
    'ItemAntivirus' => $baseDir . '/src/ItemAntivirus.php',
    'ItemVirtualMachine' => $baseDir . '/src/ItemVirtualMachine.php',
    'Item_Cluster' => $baseDir . '/src/Item_Cluster.php',
    'Item_DeviceBattery' => $baseDir . '/src/Item_DeviceBattery.php',
    'Item_DeviceCamera' => $baseDir . '/src/Item_DeviceCamera.php',
    'Item_DeviceCamera_ImageFormat' => $baseDir . '/src/Item_DeviceCamera_ImageFormat.php',
    'Item_DeviceCamera_ImageResolution' => $baseDir . '/src/Item_DeviceCamera_ImageResolution.php',
    'Item_DeviceCase' => $baseDir . '/src/Item_DeviceCase.php',
    'Item_DeviceControl' => $baseDir . '/src/Item_DeviceControl.php',
    'Item_DeviceDrive' => $baseDir . '/src/Item_DeviceDrive.php',
    'Item_DeviceFirmware' => $baseDir . '/src/Item_DeviceFirmware.php',
    'Item_DeviceGeneric' => $baseDir . '/src/Item_DeviceGeneric.php',
    'Item_DeviceGraphicCard' => $baseDir . '/src/Item_DeviceGraphicCard.php',
    'Item_DeviceHardDrive' => $baseDir . '/src/Item_DeviceHardDrive.php',
    'Item_DeviceMemory' => $baseDir . '/src/Item_DeviceMemory.php',
    'Item_DeviceMotherboard' => $baseDir . '/src/Item_DeviceMotherboard.php',
    'Item_DeviceNetworkCard' => $baseDir . '/src/Item_DeviceNetworkCard.php',
    'Item_DevicePci' => $baseDir . '/src/Item_DevicePci.php',
    'Item_DevicePowerSupply' => $baseDir . '/src/Item_DevicePowerSupply.php',
    'Item_DeviceProcessor' => $baseDir . '/src/Item_DeviceProcessor.php',
    'Item_DeviceSensor' => $baseDir . '/src/Item_DeviceSensor.php',
    'Item_DeviceSimcard' => $baseDir . '/src/Item_DeviceSimcard.php',
    'Item_DeviceSoundCard' => $baseDir . '/src/Item_DeviceSoundCard.php',
    'Item_Devices' => $baseDir . '/src/Item_Devices.php',
    'Item_Disk' => $baseDir . '/src/Item_Disk.php',
    'Item_Enclosure' => $baseDir . '/src/Item_Enclosure.php',
    'Item_Environment' => $baseDir . '/src/Item_Environment.php',
    'Item_Kanban' => $baseDir . '/src/Item_Kanban.php',
    'Item_Line' => $baseDir . '/src/Item_Line.php',
    'Item_OperatingSystem' => $baseDir . '/src/Item_OperatingSystem.php',
    'Item_Plug' => $baseDir . '/src/Item_Plug.php',
    'Item_Problem' => $baseDir . '/src/Item_Problem.php',
    'Item_Process' => $baseDir . '/src/Item_Process.php',
    'Item_Project' => $baseDir . '/src/Item_Project.php',
    'Item_Rack' => $baseDir . '/src/Item_Rack.php',
    'Item_RemoteManagement' => $baseDir . '/src/Item_RemoteManagement.php',
    'Item_SoftwareLicense' => $baseDir . '/src/Item_SoftwareLicense.php',
    'Item_SoftwareVersion' => $baseDir . '/src/Item_SoftwareVersion.php',
    'Item_Ticket' => $baseDir . '/src/Item_Ticket.php',
    'Item_TicketRecurrent' => $baseDir . '/src/Item_TicketRecurrent.php',
    'Itil_Project' => $baseDir . '/src/Itil_Project.php',
    'Knowbase' => $baseDir . '/src/Knowbase.php',
    'KnowbaseItem' => $baseDir . '/src/KnowbaseItem.php',
    'KnowbaseItemCategory' => $baseDir . '/src/KnowbaseItemCategory.php',
    'KnowbaseItemTranslation' => $baseDir . '/src/KnowbaseItemTranslation.php',
    'KnowbaseItem_Comment' => $baseDir . '/src/KnowbaseItem_Comment.php',
    'KnowbaseItem_Item' => $baseDir . '/src/KnowbaseItem_Item.php',
    'KnowbaseItem_KnowbaseItemCategory' => $baseDir . '/src/KnowbaseItem_KnowbaseItemCategory.php',
    'KnowbaseItem_Profile' => $baseDir . '/src/KnowbaseItem_Profile.php',
    'KnowbaseItem_Revision' => $baseDir . '/src/KnowbaseItem_Revision.php',
    'KnowbaseItem_User' => $baseDir . '/src/KnowbaseItem_User.php',
    'Laminas\\I18n\\ConfigProvider' => $vendorDir . '/laminas/laminas-i18n/src/ConfigProvider.php',
    'Laminas\\I18n\\CountryCode' => $vendorDir . '/laminas/laminas-i18n/src/CountryCode.php',
    'Laminas\\I18n\\Exception\\ExceptionInterface' => $vendorDir . '/laminas/laminas-i18n/src/Exception/ExceptionInterface.php',
    'Laminas\\I18n\\Exception\\ExtensionNotLoadedException' => $vendorDir . '/laminas/laminas-i18n/src/Exception/ExtensionNotLoadedException.php',
    'Laminas\\I18n\\Exception\\InvalidArgumentException' => $vendorDir . '/laminas/laminas-i18n/src/Exception/InvalidArgumentException.php',
    'Laminas\\I18n\\Exception\\OutOfBoundsException' => $vendorDir . '/laminas/laminas-i18n/src/Exception/OutOfBoundsException.php',
    'Laminas\\I18n\\Exception\\ParseException' => $vendorDir . '/laminas/laminas-i18n/src/Exception/ParseException.php',
    'Laminas\\I18n\\Exception\\RangeException' => $vendorDir . '/laminas/laminas-i18n/src/Exception/RangeException.php',
    'Laminas\\I18n\\Exception\\RuntimeException' => $vendorDir . '/laminas/laminas-i18n/src/Exception/RuntimeException.php',
    'Laminas\\I18n\\Filter\\AbstractLocale' => $vendorDir . '/laminas/laminas-i18n/src/Filter/AbstractLocale.php',
    'Laminas\\I18n\\Filter\\Alnum' => $vendorDir . '/laminas/laminas-i18n/src/Filter/Alnum.php',
    'Laminas\\I18n\\Filter\\Alpha' => $vendorDir . '/laminas/laminas-i18n/src/Filter/Alpha.php',
    'Laminas\\I18n\\Filter\\NumberFormat' => $vendorDir . '/laminas/laminas-i18n/src/Filter/NumberFormat.php',
    'Laminas\\I18n\\Filter\\NumberParse' => $vendorDir . '/laminas/laminas-i18n/src/Filter/NumberParse.php',
    'Laminas\\I18n\\Geography\\CountryCodeListInterface' => $vendorDir . '/laminas/laminas-i18n/src/Geography/CountryCodeListInterface.php',
    'Laminas\\I18n\\Geography\\DefaultCountryCodeList' => $vendorDir . '/laminas/laminas-i18n/src/Geography/DefaultCountryCodeList.php',
    'Laminas\\I18n\\Module' => $vendorDir . '/laminas/laminas-i18n/src/Module.php',
    'Laminas\\I18n\\Translator\\LoaderPluginManager' => $vendorDir . '/laminas/laminas-i18n/src/Translator/LoaderPluginManager.php',
    'Laminas\\I18n\\Translator\\LoaderPluginManagerFactory' => $vendorDir . '/laminas/laminas-i18n/src/Translator/LoaderPluginManagerFactory.php',
    'Laminas\\I18n\\Translator\\Loader\\AbstractFileLoader' => $vendorDir . '/laminas/laminas-i18n/src/Translator/Loader/AbstractFileLoader.php',
    'Laminas\\I18n\\Translator\\Loader\\FileLoaderInterface' => $vendorDir . '/laminas/laminas-i18n/src/Translator/Loader/FileLoaderInterface.php',
    'Laminas\\I18n\\Translator\\Loader\\Gettext' => $vendorDir . '/laminas/laminas-i18n/src/Translator/Loader/Gettext.php',
    'Laminas\\I18n\\Translator\\Loader\\Ini' => $vendorDir . '/laminas/laminas-i18n/src/Translator/Loader/Ini.php',
    'Laminas\\I18n\\Translator\\Loader\\PhpArray' => $vendorDir . '/laminas/laminas-i18n/src/Translator/Loader/PhpArray.php',
    'Laminas\\I18n\\Translator\\Loader\\PhpMemoryArray' => $vendorDir . '/laminas/laminas-i18n/src/Translator/Loader/PhpMemoryArray.php',
    'Laminas\\I18n\\Translator\\Loader\\RemoteLoaderInterface' => $vendorDir . '/laminas/laminas-i18n/src/Translator/Loader/RemoteLoaderInterface.php',
    'Laminas\\I18n\\Translator\\Plural\\Parser' => $vendorDir . '/laminas/laminas-i18n/src/Translator/Plural/Parser.php',
    'Laminas\\I18n\\Translator\\Plural\\Rule' => $vendorDir . '/laminas/laminas-i18n/src/Translator/Plural/Rule.php',
    'Laminas\\I18n\\Translator\\Plural\\Symbol' => $vendorDir . '/laminas/laminas-i18n/src/Translator/Plural/Symbol.php',
    'Laminas\\I18n\\Translator\\TextDomain' => $vendorDir . '/laminas/laminas-i18n/src/Translator/TextDomain.php',
    'Laminas\\I18n\\Translator\\Translator' => $vendorDir . '/laminas/laminas-i18n/src/Translator/Translator.php',
    'Laminas\\I18n\\Translator\\TranslatorAwareInterface' => $vendorDir . '/laminas/laminas-i18n/src/Translator/TranslatorAwareInterface.php',
    'Laminas\\I18n\\Translator\\TranslatorAwareTrait' => $vendorDir . '/laminas/laminas-i18n/src/Translator/TranslatorAwareTrait.php',
    'Laminas\\I18n\\Translator\\TranslatorInterface' => $vendorDir . '/laminas/laminas-i18n/src/Translator/TranslatorInterface.php',
    'Laminas\\I18n\\Translator\\TranslatorServiceFactory' => $vendorDir . '/laminas/laminas-i18n/src/Translator/TranslatorServiceFactory.php',
    'Laminas\\I18n\\Validator\\Alnum' => $vendorDir . '/laminas/laminas-i18n/src/Validator/Alnum.php',
    'Laminas\\I18n\\Validator\\Alpha' => $vendorDir . '/laminas/laminas-i18n/src/Validator/Alpha.php',
    'Laminas\\I18n\\Validator\\CountryCode' => $vendorDir . '/laminas/laminas-i18n/src/Validator/CountryCode.php',
    'Laminas\\I18n\\Validator\\DateTime' => $vendorDir . '/laminas/laminas-i18n/src/Validator/DateTime.php',
    'Laminas\\I18n\\Validator\\IsFloat' => $vendorDir . '/laminas/laminas-i18n/src/Validator/IsFloat.php',
    'Laminas\\I18n\\Validator\\IsInt' => $vendorDir . '/laminas/laminas-i18n/src/Validator/IsInt.php',
    'Laminas\\I18n\\Validator\\PhoneNumber' => $vendorDir . '/laminas/laminas-i18n/src/Validator/PhoneNumber.php',
    'Laminas\\I18n\\Validator\\PostCode' => $vendorDir . '/laminas/laminas-i18n/src/Validator/PostCode.php',
    'Laminas\\I18n\\View\\HelperConfig' => $vendorDir . '/laminas/laminas-i18n/src/View/HelperConfig.php',
    'Laminas\\I18n\\View\\HelperTrait' => $vendorDir . '/laminas/laminas-i18n/src/View/HelperTrait.php',
    'Laminas\\I18n\\View\\Helper\\AbstractTranslatorHelper' => $vendorDir . '/laminas/laminas-i18n/src/View/Helper/AbstractTranslatorHelper.php',
    'Laminas\\I18n\\View\\Helper\\Container\\CountryCodeDataListFactory' => $vendorDir . '/laminas/laminas-i18n/src/View/Helper/Container/CountryCodeDataListFactory.php',
    'Laminas\\I18n\\View\\Helper\\CountryCodeDataList' => $vendorDir . '/laminas/laminas-i18n/src/View/Helper/CountryCodeDataList.php',
    'Laminas\\I18n\\View\\Helper\\CurrencyFormat' => $vendorDir . '/laminas/laminas-i18n/src/View/Helper/CurrencyFormat.php',
    'Laminas\\I18n\\View\\Helper\\DateFormat' => $vendorDir . '/laminas/laminas-i18n/src/View/Helper/DateFormat.php',
    'Laminas\\I18n\\View\\Helper\\NumberFormat' => $vendorDir . '/laminas/laminas-i18n/src/View/Helper/NumberFormat.php',
    'Laminas\\I18n\\View\\Helper\\Plural' => $vendorDir . '/laminas/laminas-i18n/src/View/Helper/Plural.php',
    'Laminas\\I18n\\View\\Helper\\Translate' => $vendorDir . '/laminas/laminas-i18n/src/View/Helper/Translate.php',
    'Laminas\\I18n\\View\\Helper\\TranslatePlural' => $vendorDir . '/laminas/laminas-i18n/src/View/Helper/TranslatePlural.php',
    'Laminas\\Loader\\AutoloaderFactory' => $vendorDir . '/laminas/laminas-loader/src/AutoloaderFactory.php',
    'Laminas\\Loader\\ClassMapAutoloader' => $vendorDir . '/laminas/laminas-loader/src/ClassMapAutoloader.php',
    'Laminas\\Loader\\Exception\\BadMethodCallException' => $vendorDir . '/laminas/laminas-loader/src/Exception/BadMethodCallException.php',
    'Laminas\\Loader\\Exception\\DomainException' => $vendorDir . '/laminas/laminas-loader/src/Exception/DomainException.php',
    'Laminas\\Loader\\Exception\\ExceptionInterface' => $vendorDir . '/laminas/laminas-loader/src/Exception/ExceptionInterface.php',
    'Laminas\\Loader\\Exception\\InvalidArgumentException' => $vendorDir . '/laminas/laminas-loader/src/Exception/InvalidArgumentException.php',
    'Laminas\\Loader\\Exception\\InvalidPathException' => $vendorDir . '/laminas/laminas-loader/src/Exception/InvalidPathException.php',
    'Laminas\\Loader\\Exception\\MissingResourceNamespaceException' => $vendorDir . '/laminas/laminas-loader/src/Exception/MissingResourceNamespaceException.php',
    'Laminas\\Loader\\Exception\\PluginLoaderException' => $vendorDir . '/laminas/laminas-loader/src/Exception/PluginLoaderException.php',
    'Laminas\\Loader\\Exception\\RuntimeException' => $vendorDir . '/laminas/laminas-loader/src/Exception/RuntimeException.php',
    'Laminas\\Loader\\Exception\\SecurityException' => $vendorDir . '/laminas/laminas-loader/src/Exception/SecurityException.php',
    'Laminas\\Loader\\ModuleAutoloader' => $vendorDir . '/laminas/laminas-loader/src/ModuleAutoloader.php',
    'Laminas\\Loader\\PluginClassLoader' => $vendorDir . '/laminas/laminas-loader/src/PluginClassLoader.php',
    'Laminas\\Loader\\PluginClassLocator' => $vendorDir . '/laminas/laminas-loader/src/PluginClassLocator.php',
    'Laminas\\Loader\\ShortNameLocator' => $vendorDir . '/laminas/laminas-loader/src/ShortNameLocator.php',
    'Laminas\\Loader\\SplAutoloader' => $vendorDir . '/laminas/laminas-loader/src/SplAutoloader.php',
    'Laminas\\Loader\\StandardAutoloader' => $vendorDir . '/laminas/laminas-loader/src/StandardAutoloader.php',
    'Laminas\\Mail\\Address' => $vendorDir . '/laminas/laminas-mail/src/Address.php',
    'Laminas\\Mail\\AddressList' => $vendorDir . '/laminas/laminas-mail/src/AddressList.php',
    'Laminas\\Mail\\Address\\AddressInterface' => $vendorDir . '/laminas/laminas-mail/src/Address/AddressInterface.php',
    'Laminas\\Mail\\ConfigProvider' => $vendorDir . '/laminas/laminas-mail/src/ConfigProvider.php',
    'Laminas\\Mail\\Exception\\BadMethodCallException' => $vendorDir . '/laminas/laminas-mail/src/Exception/BadMethodCallException.php',
    'Laminas\\Mail\\Exception\\DomainException' => $vendorDir . '/laminas/laminas-mail/src/Exception/DomainException.php',
    'Laminas\\Mail\\Exception\\ExceptionInterface' => $vendorDir . '/laminas/laminas-mail/src/Exception/ExceptionInterface.php',
    'Laminas\\Mail\\Exception\\InvalidArgumentException' => $vendorDir . '/laminas/laminas-mail/src/Exception/InvalidArgumentException.php',
    'Laminas\\Mail\\Exception\\OutOfBoundsException' => $vendorDir . '/laminas/laminas-mail/src/Exception/OutOfBoundsException.php',
    'Laminas\\Mail\\Exception\\RuntimeException' => $vendorDir . '/laminas/laminas-mail/src/Exception/RuntimeException.php',
    'Laminas\\Mail\\Header\\AbstractAddressList' => $vendorDir . '/laminas/laminas-mail/src/Header/AbstractAddressList.php',
    'Laminas\\Mail\\Header\\Bcc' => $vendorDir . '/laminas/laminas-mail/src/Header/Bcc.php',
    'Laminas\\Mail\\Header\\Cc' => $vendorDir . '/laminas/laminas-mail/src/Header/Cc.php',
    'Laminas\\Mail\\Header\\ContentDisposition' => $vendorDir . '/laminas/laminas-mail/src/Header/ContentDisposition.php',
    'Laminas\\Mail\\Header\\ContentTransferEncoding' => $vendorDir . '/laminas/laminas-mail/src/Header/ContentTransferEncoding.php',
    'Laminas\\Mail\\Header\\ContentType' => $vendorDir . '/laminas/laminas-mail/src/Header/ContentType.php',
    'Laminas\\Mail\\Header\\Date' => $vendorDir . '/laminas/laminas-mail/src/Header/Date.php',
    'Laminas\\Mail\\Header\\Exception\\BadMethodCallException' => $vendorDir . '/laminas/laminas-mail/src/Header/Exception/BadMethodCallException.php',
    'Laminas\\Mail\\Header\\Exception\\ExceptionInterface' => $vendorDir . '/laminas/laminas-mail/src/Header/Exception/ExceptionInterface.php',
    'Laminas\\Mail\\Header\\Exception\\InvalidArgumentException' => $vendorDir . '/laminas/laminas-mail/src/Header/Exception/InvalidArgumentException.php',
    'Laminas\\Mail\\Header\\Exception\\RuntimeException' => $vendorDir . '/laminas/laminas-mail/src/Header/Exception/RuntimeException.php',
    'Laminas\\Mail\\Header\\From' => $vendorDir . '/laminas/laminas-mail/src/Header/From.php',
    'Laminas\\Mail\\Header\\GenericHeader' => $vendorDir . '/laminas/laminas-mail/src/Header/GenericHeader.php',
    'Laminas\\Mail\\Header\\GenericMultiHeader' => $vendorDir . '/laminas/laminas-mail/src/Header/GenericMultiHeader.php',
    'Laminas\\Mail\\Header\\HeaderInterface' => $vendorDir . '/laminas/laminas-mail/src/Header/HeaderInterface.php',
    'Laminas\\Mail\\Header\\HeaderLoader' => $vendorDir . '/laminas/laminas-mail/src/Header/HeaderLoader.php',
    'Laminas\\Mail\\Header\\HeaderLocator' => $vendorDir . '/laminas/laminas-mail/src/Header/HeaderLocator.php',
    'Laminas\\Mail\\Header\\HeaderLocatorInterface' => $vendorDir . '/laminas/laminas-mail/src/Header/HeaderLocatorInterface.php',
    'Laminas\\Mail\\Header\\HeaderName' => $vendorDir . '/laminas/laminas-mail/src/Header/HeaderName.php',
    'Laminas\\Mail\\Header\\HeaderValue' => $vendorDir . '/laminas/laminas-mail/src/Header/HeaderValue.php',
    'Laminas\\Mail\\Header\\HeaderWrap' => $vendorDir . '/laminas/laminas-mail/src/Header/HeaderWrap.php',
    'Laminas\\Mail\\Header\\IdentificationField' => $vendorDir . '/laminas/laminas-mail/src/Header/IdentificationField.php',
    'Laminas\\Mail\\Header\\InReplyTo' => $vendorDir . '/laminas/laminas-mail/src/Header/InReplyTo.php',
    'Laminas\\Mail\\Header\\ListParser' => $vendorDir . '/laminas/laminas-mail/src/Header/ListParser.php',
    'Laminas\\Mail\\Header\\MessageId' => $vendorDir . '/laminas/laminas-mail/src/Header/MessageId.php',
    'Laminas\\Mail\\Header\\MimeVersion' => $vendorDir . '/laminas/laminas-mail/src/Header/MimeVersion.php',
    'Laminas\\Mail\\Header\\MultipleHeadersInterface' => $vendorDir . '/laminas/laminas-mail/src/Header/MultipleHeadersInterface.php',
    'Laminas\\Mail\\Header\\Received' => $vendorDir . '/laminas/laminas-mail/src/Header/Received.php',
    'Laminas\\Mail\\Header\\References' => $vendorDir . '/laminas/laminas-mail/src/Header/References.php',
    'Laminas\\Mail\\Header\\ReplyTo' => $vendorDir . '/laminas/laminas-mail/src/Header/ReplyTo.php',
    'Laminas\\Mail\\Header\\Sender' => $vendorDir . '/laminas/laminas-mail/src/Header/Sender.php',
    'Laminas\\Mail\\Header\\StructuredInterface' => $vendorDir . '/laminas/laminas-mail/src/Header/StructuredInterface.php',
    'Laminas\\Mail\\Header\\Subject' => $vendorDir . '/laminas/laminas-mail/src/Header/Subject.php',
    'Laminas\\Mail\\Header\\To' => $vendorDir . '/laminas/laminas-mail/src/Header/To.php',
    'Laminas\\Mail\\Header\\UnstructuredInterface' => $vendorDir . '/laminas/laminas-mail/src/Header/UnstructuredInterface.php',
    'Laminas\\Mail\\Headers' => $vendorDir . '/laminas/laminas-mail/src/Headers.php',
    'Laminas\\Mail\\Message' => $vendorDir . '/laminas/laminas-mail/src/Message.php',
    'Laminas\\Mail\\MessageFactory' => $vendorDir . '/laminas/laminas-mail/src/MessageFactory.php',
    'Laminas\\Mail\\Module' => $vendorDir . '/laminas/laminas-mail/src/Module.php',
    'Laminas\\Mail\\Protocol\\AbstractProtocol' => $vendorDir . '/laminas/laminas-mail/src/Protocol/AbstractProtocol.php',
    'Laminas\\Mail\\Protocol\\Exception\\ExceptionInterface' => $vendorDir . '/laminas/laminas-mail/src/Protocol/Exception/ExceptionInterface.php',
    'Laminas\\Mail\\Protocol\\Exception\\InvalidArgumentException' => $vendorDir . '/laminas/laminas-mail/src/Protocol/Exception/InvalidArgumentException.php',
    'Laminas\\Mail\\Protocol\\Exception\\RuntimeException' => $vendorDir . '/laminas/laminas-mail/src/Protocol/Exception/RuntimeException.php',
    'Laminas\\Mail\\Protocol\\Imap' => $vendorDir . '/laminas/laminas-mail/src/Protocol/Imap.php',
    'Laminas\\Mail\\Protocol\\Pop3' => $vendorDir . '/laminas/laminas-mail/src/Protocol/Pop3.php',
    'Laminas\\Mail\\Protocol\\Pop3\\Response' => $vendorDir . '/laminas/laminas-mail/src/Protocol/Pop3/Response.php',
    'Laminas\\Mail\\Protocol\\Pop3\\Xoauth2\\Microsoft' => $vendorDir . '/laminas/laminas-mail/src/Protocol/Pop3/Xoauth2/Microsoft.php',
    'Laminas\\Mail\\Protocol\\ProtocolTrait' => $vendorDir . '/laminas/laminas-mail/src/Protocol/ProtocolTrait.php',
    'Laminas\\Mail\\Protocol\\Smtp' => $vendorDir . '/laminas/laminas-mail/src/Protocol/Smtp.php',
    'Laminas\\Mail\\Protocol\\SmtpPluginManager' => $vendorDir . '/laminas/laminas-mail/src/Protocol/SmtpPluginManager.php',
    'Laminas\\Mail\\Protocol\\SmtpPluginManagerFactory' => $vendorDir . '/laminas/laminas-mail/src/Protocol/SmtpPluginManagerFactory.php',
    'Laminas\\Mail\\Protocol\\Smtp\\Auth\\Crammd5' => $vendorDir . '/laminas/laminas-mail/src/Protocol/Smtp/Auth/Crammd5.php',
    'Laminas\\Mail\\Protocol\\Smtp\\Auth\\Login' => $vendorDir . '/laminas/laminas-mail/src/Protocol/Smtp/Auth/Login.php',
    'Laminas\\Mail\\Protocol\\Smtp\\Auth\\Plain' => $vendorDir . '/laminas/laminas-mail/src/Protocol/Smtp/Auth/Plain.php',
    'Laminas\\Mail\\Protocol\\Smtp\\Auth\\Xoauth2' => $vendorDir . '/laminas/laminas-mail/src/Protocol/Smtp/Auth/Xoauth2.php',
    'Laminas\\Mail\\Protocol\\Xoauth2\\Xoauth2' => $vendorDir . '/laminas/laminas-mail/src/Protocol/Xoauth2/Xoauth2.php',
    'Laminas\\Mail\\Storage' => $vendorDir . '/laminas/laminas-mail/src/Storage.php',
    'Laminas\\Mail\\Storage\\AbstractStorage' => $vendorDir . '/laminas/laminas-mail/src/Storage/AbstractStorage.php',
    'Laminas\\Mail\\Storage\\Exception\\ExceptionInterface' => $vendorDir . '/laminas/laminas-mail/src/Storage/Exception/ExceptionInterface.php',
    'Laminas\\Mail\\Storage\\Exception\\InvalidArgumentException' => $vendorDir . '/laminas/laminas-mail/src/Storage/Exception/InvalidArgumentException.php',
    'Laminas\\Mail\\Storage\\Exception\\OutOfBoundsException' => $vendorDir . '/laminas/laminas-mail/src/Storage/Exception/OutOfBoundsException.php',
    'Laminas\\Mail\\Storage\\Exception\\RuntimeException' => $vendorDir . '/laminas/laminas-mail/src/Storage/Exception/RuntimeException.php',
    'Laminas\\Mail\\Storage\\Folder' => $vendorDir . '/laminas/laminas-mail/src/Storage/Folder.php',
    'Laminas\\Mail\\Storage\\Folder\\FolderInterface' => $vendorDir . '/laminas/laminas-mail/src/Storage/Folder/FolderInterface.php',
    'Laminas\\Mail\\Storage\\Folder\\Maildir' => $vendorDir . '/laminas/laminas-mail/src/Storage/Folder/Maildir.php',
    'Laminas\\Mail\\Storage\\Folder\\Mbox' => $vendorDir . '/laminas/laminas-mail/src/Storage/Folder/Mbox.php',
    'Laminas\\Mail\\Storage\\Imap' => $vendorDir . '/laminas/laminas-mail/src/Storage/Imap.php',
    'Laminas\\Mail\\Storage\\Maildir' => $vendorDir . '/laminas/laminas-mail/src/Storage/Maildir.php',
    'Laminas\\Mail\\Storage\\Mbox' => $vendorDir . '/laminas/laminas-mail/src/Storage/Mbox.php',
    'Laminas\\Mail\\Storage\\Message' => $vendorDir . '/laminas/laminas-mail/src/Storage/Message.php',
    'Laminas\\Mail\\Storage\\Message\\File' => $vendorDir . '/laminas/laminas-mail/src/Storage/Message/File.php',
    'Laminas\\Mail\\Storage\\Message\\MessageInterface' => $vendorDir . '/laminas/laminas-mail/src/Storage/Message/MessageInterface.php',
    'Laminas\\Mail\\Storage\\ParamsNormalizer' => $vendorDir . '/laminas/laminas-mail/src/Storage/ParamsNormalizer.php',
    'Laminas\\Mail\\Storage\\Part' => $vendorDir . '/laminas/laminas-mail/src/Storage/Part.php',
    'Laminas\\Mail\\Storage\\Part\\Exception\\ExceptionInterface' => $vendorDir . '/laminas/laminas-mail/src/Storage/Part/Exception/ExceptionInterface.php',
    'Laminas\\Mail\\Storage\\Part\\Exception\\InvalidArgumentException' => $vendorDir . '/laminas/laminas-mail/src/Storage/Part/Exception/InvalidArgumentException.php',
    'Laminas\\Mail\\Storage\\Part\\Exception\\RuntimeException' => $vendorDir . '/laminas/laminas-mail/src/Storage/Part/Exception/RuntimeException.php',
    'Laminas\\Mail\\Storage\\Part\\File' => $vendorDir . '/laminas/laminas-mail/src/Storage/Part/File.php',
    'Laminas\\Mail\\Storage\\Part\\PartInterface' => $vendorDir . '/laminas/laminas-mail/src/Storage/Part/PartInterface.php',
    'Laminas\\Mail\\Storage\\Pop3' => $vendorDir . '/laminas/laminas-mail/src/Storage/Pop3.php',
    'Laminas\\Mail\\Storage\\Writable\\Maildir' => $vendorDir . '/laminas/laminas-mail/src/Storage/Writable/Maildir.php',
    'Laminas\\Mail\\Storage\\Writable\\WritableInterface' => $vendorDir . '/laminas/laminas-mail/src/Storage/Writable/WritableInterface.php',
    'Laminas\\Mail\\Transport\\Envelope' => $vendorDir . '/laminas/laminas-mail/src/Transport/Envelope.php',
    'Laminas\\Mail\\Transport\\Exception\\DomainException' => $vendorDir . '/laminas/laminas-mail/src/Transport/Exception/DomainException.php',
    'Laminas\\Mail\\Transport\\Exception\\ExceptionInterface' => $vendorDir . '/laminas/laminas-mail/src/Transport/Exception/ExceptionInterface.php',
    'Laminas\\Mail\\Transport\\Exception\\InvalidArgumentException' => $vendorDir . '/laminas/laminas-mail/src/Transport/Exception/InvalidArgumentException.php',
    'Laminas\\Mail\\Transport\\Exception\\RuntimeException' => $vendorDir . '/laminas/laminas-mail/src/Transport/Exception/RuntimeException.php',
    'Laminas\\Mail\\Transport\\Factory' => $vendorDir . '/laminas/laminas-mail/src/Transport/Factory.php',
    'Laminas\\Mail\\Transport\\File' => $vendorDir . '/laminas/laminas-mail/src/Transport/File.php',
    'Laminas\\Mail\\Transport\\FileOptions' => $vendorDir . '/laminas/laminas-mail/src/Transport/FileOptions.php',
    'Laminas\\Mail\\Transport\\InMemory' => $vendorDir . '/laminas/laminas-mail/src/Transport/InMemory.php',
    'Laminas\\Mail\\Transport\\Sendmail' => $vendorDir . '/laminas/laminas-mail/src/Transport/Sendmail.php',
    'Laminas\\Mail\\Transport\\Smtp' => $vendorDir . '/laminas/laminas-mail/src/Transport/Smtp.php',
    'Laminas\\Mail\\Transport\\SmtpOptions' => $vendorDir . '/laminas/laminas-mail/src/Transport/SmtpOptions.php',
    'Laminas\\Mail\\Transport\\TransportInterface' => $vendorDir . '/laminas/laminas-mail/src/Transport/TransportInterface.php',
    'Laminas\\Mime\\Decode' => $vendorDir . '/laminas/laminas-mime/src/Decode.php',
    'Laminas\\Mime\\Exception\\ExceptionInterface' => $vendorDir . '/laminas/laminas-mime/src/Exception/ExceptionInterface.php',
    'Laminas\\Mime\\Exception\\InvalidArgumentException' => $vendorDir . '/laminas/laminas-mime/src/Exception/InvalidArgumentException.php',
    'Laminas\\Mime\\Exception\\RuntimeException' => $vendorDir . '/laminas/laminas-mime/src/Exception/RuntimeException.php',
    'Laminas\\Mime\\Message' => $vendorDir . '/laminas/laminas-mime/src/Message.php',
    'Laminas\\Mime\\Mime' => $vendorDir . '/laminas/laminas-mime/src/Mime.php',
    'Laminas\\Mime\\Part' => $vendorDir . '/laminas/laminas-mime/src/Part.php',
    'Laminas\\ServiceManager\\AbstractFactoryInterface' => $vendorDir . '/laminas/laminas-servicemanager/src/AbstractFactoryInterface.php',
    'Laminas\\ServiceManager\\AbstractFactory\\ConfigAbstractFactory' => $vendorDir . '/laminas/laminas-servicemanager/src/AbstractFactory/ConfigAbstractFactory.php',
    'Laminas\\ServiceManager\\AbstractFactory\\ReflectionBasedAbstractFactory' => $vendorDir . '/laminas/laminas-servicemanager/src/AbstractFactory/ReflectionBasedAbstractFactory.php',
    'Laminas\\ServiceManager\\AbstractPluginManager' => $vendorDir . '/laminas/laminas-servicemanager/src/AbstractPluginManager.php',
    'Laminas\\ServiceManager\\Config' => $vendorDir . '/laminas/laminas-servicemanager/src/Config.php',
    'Laminas\\ServiceManager\\ConfigInterface' => $vendorDir . '/laminas/laminas-servicemanager/src/ConfigInterface.php',
    'Laminas\\ServiceManager\\DelegatorFactoryInterface' => $vendorDir . '/laminas/laminas-servicemanager/src/DelegatorFactoryInterface.php',
    'Laminas\\ServiceManager\\Exception\\ContainerModificationsNotAllowedException' => $vendorDir . '/laminas/laminas-servicemanager/src/Exception/ContainerModificationsNotAllowedException.php',
    'Laminas\\ServiceManager\\Exception\\CyclicAliasException' => $vendorDir . '/laminas/laminas-servicemanager/src/Exception/CyclicAliasException.php',
    'Laminas\\ServiceManager\\Exception\\ExceptionInterface' => $vendorDir . '/laminas/laminas-servicemanager/src/Exception/ExceptionInterface.php',
    'Laminas\\ServiceManager\\Exception\\InvalidArgumentException' => $vendorDir . '/laminas/laminas-servicemanager/src/Exception/InvalidArgumentException.php',
    'Laminas\\ServiceManager\\Exception\\InvalidServiceException' => $vendorDir . '/laminas/laminas-servicemanager/src/Exception/InvalidServiceException.php',
    'Laminas\\ServiceManager\\Exception\\ServiceNotCreatedException' => $vendorDir . '/laminas/laminas-servicemanager/src/Exception/ServiceNotCreatedException.php',
    'Laminas\\ServiceManager\\Exception\\ServiceNotFoundException' => $vendorDir . '/laminas/laminas-servicemanager/src/Exception/ServiceNotFoundException.php',
    'Laminas\\ServiceManager\\FactoryInterface' => $vendorDir . '/laminas/laminas-servicemanager/src/FactoryInterface.php',
    'Laminas\\ServiceManager\\Factory\\AbstractFactoryInterface' => $vendorDir . '/laminas/laminas-servicemanager/src/Factory/AbstractFactoryInterface.php',
    'Laminas\\ServiceManager\\Factory\\DelegatorFactoryInterface' => $vendorDir . '/laminas/laminas-servicemanager/src/Factory/DelegatorFactoryInterface.php',
    'Laminas\\ServiceManager\\Factory\\FactoryInterface' => $vendorDir . '/laminas/laminas-servicemanager/src/Factory/FactoryInterface.php',
    'Laminas\\ServiceManager\\Factory\\InvokableFactory' => $vendorDir . '/laminas/laminas-servicemanager/src/Factory/InvokableFactory.php',
    'Laminas\\ServiceManager\\InitializerInterface' => $vendorDir . '/laminas/laminas-servicemanager/src/InitializerInterface.php',
    'Laminas\\ServiceManager\\Initializer\\InitializerInterface' => $vendorDir . '/laminas/laminas-servicemanager/src/Initializer/InitializerInterface.php',
    'Laminas\\ServiceManager\\PluginManagerInterface' => $vendorDir . '/laminas/laminas-servicemanager/src/PluginManagerInterface.php',
    'Laminas\\ServiceManager\\Proxy\\LazyServiceFactory' => $vendorDir . '/laminas/laminas-servicemanager/src/Proxy/LazyServiceFactory.php',
    'Laminas\\ServiceManager\\ServiceLocatorInterface' => $vendorDir . '/laminas/laminas-servicemanager/src/ServiceLocatorInterface.php',
    'Laminas\\ServiceManager\\ServiceManager' => $vendorDir . '/laminas/laminas-servicemanager/src/ServiceManager.php',
    'Laminas\\ServiceManager\\Test\\CommonPluginManagerTrait' => $vendorDir . '/laminas/laminas-servicemanager/src/Test/CommonPluginManagerTrait.php',
    'Laminas\\ServiceManager\\Tool\\ConfigDumper' => $vendorDir . '/laminas/laminas-servicemanager/src/Tool/ConfigDumper.php',
    'Laminas\\ServiceManager\\Tool\\ConfigDumperCommand' => $vendorDir . '/laminas/laminas-servicemanager/src/Tool/ConfigDumperCommand.php',
    'Laminas\\ServiceManager\\Tool\\FactoryCreator' => $vendorDir . '/laminas/laminas-servicemanager/src/Tool/FactoryCreator.php',
    'Laminas\\ServiceManager\\Tool\\FactoryCreatorCommand' => $vendorDir . '/laminas/laminas-servicemanager/src/Tool/FactoryCreatorCommand.php',
    'Laminas\\Stdlib\\AbstractOptions' => $vendorDir . '/laminas/laminas-stdlib/src/AbstractOptions.php',
    'Laminas\\Stdlib\\ArrayObject' => $vendorDir . '/laminas/laminas-stdlib/src/ArrayObject.php',
    'Laminas\\Stdlib\\ArraySerializableInterface' => $vendorDir . '/laminas/laminas-stdlib/src/ArraySerializableInterface.php',
    'Laminas\\Stdlib\\ArrayStack' => $vendorDir . '/laminas/laminas-stdlib/src/ArrayStack.php',
    'Laminas\\Stdlib\\ArrayUtils' => $vendorDir . '/laminas/laminas-stdlib/src/ArrayUtils.php',
    'Laminas\\Stdlib\\ArrayUtils\\MergeRemoveKey' => $vendorDir . '/laminas/laminas-stdlib/src/ArrayUtils/MergeRemoveKey.php',
    'Laminas\\Stdlib\\ArrayUtils\\MergeReplaceKey' => $vendorDir . '/laminas/laminas-stdlib/src/ArrayUtils/MergeReplaceKey.php',
    'Laminas\\Stdlib\\ArrayUtils\\MergeReplaceKeyInterface' => $vendorDir . '/laminas/laminas-stdlib/src/ArrayUtils/MergeReplaceKeyInterface.php',
    'Laminas\\Stdlib\\ConsoleHelper' => $vendorDir . '/laminas/laminas-stdlib/src/ConsoleHelper.php',
    'Laminas\\Stdlib\\DispatchableInterface' => $vendorDir . '/laminas/laminas-stdlib/src/DispatchableInterface.php',
    'Laminas\\Stdlib\\ErrorHandler' => $vendorDir . '/laminas/laminas-stdlib/src/ErrorHandler.php',
    'Laminas\\Stdlib\\Exception\\BadMethodCallException' => $vendorDir . '/laminas/laminas-stdlib/src/Exception/BadMethodCallException.php',
    'Laminas\\Stdlib\\Exception\\DomainException' => $vendorDir . '/laminas/laminas-stdlib/src/Exception/DomainException.php',
    'Laminas\\Stdlib\\Exception\\ExceptionInterface' => $vendorDir . '/laminas/laminas-stdlib/src/Exception/ExceptionInterface.php',
    'Laminas\\Stdlib\\Exception\\ExtensionNotLoadedException' => $vendorDir . '/laminas/laminas-stdlib/src/Exception/ExtensionNotLoadedException.php',
    'Laminas\\Stdlib\\Exception\\InvalidArgumentException' => $vendorDir . '/laminas/laminas-stdlib/src/Exception/InvalidArgumentException.php',
    'Laminas\\Stdlib\\Exception\\LogicException' => $vendorDir . '/laminas/laminas-stdlib/src/Exception/LogicException.php',
    'Laminas\\Stdlib\\Exception\\RuntimeException' => $vendorDir . '/laminas/laminas-stdlib/src/Exception/RuntimeException.php',
    'Laminas\\Stdlib\\FastPriorityQueue' => $vendorDir . '/laminas/laminas-stdlib/src/FastPriorityQueue.php',
    'Laminas\\Stdlib\\Glob' => $vendorDir . '/laminas/laminas-stdlib/src/Glob.php',
    'Laminas\\Stdlib\\Guard\\AllGuardsTrait' => $vendorDir . '/laminas/laminas-stdlib/src/Guard/AllGuardsTrait.php',
    'Laminas\\Stdlib\\Guard\\ArrayOrTraversableGuardTrait' => $vendorDir . '/laminas/laminas-stdlib/src/Guard/ArrayOrTraversableGuardTrait.php',
    'Laminas\\Stdlib\\Guard\\EmptyGuardTrait' => $vendorDir . '/laminas/laminas-stdlib/src/Guard/EmptyGuardTrait.php',
    'Laminas\\Stdlib\\Guard\\NullGuardTrait' => $vendorDir . '/laminas/laminas-stdlib/src/Guard/NullGuardTrait.php',
    'Laminas\\Stdlib\\InitializableInterface' => $vendorDir . '/laminas/laminas-stdlib/src/InitializableInterface.php',
    'Laminas\\Stdlib\\JsonSerializable' => $vendorDir . '/laminas/laminas-stdlib/src/JsonSerializable.php',
    'Laminas\\Stdlib\\Message' => $vendorDir . '/laminas/laminas-stdlib/src/Message.php',
    'Laminas\\Stdlib\\MessageInterface' => $vendorDir . '/laminas/laminas-stdlib/src/MessageInterface.php',
    'Laminas\\Stdlib\\ParameterObjectInterface' => $vendorDir . '/laminas/laminas-stdlib/src/ParameterObjectInterface.php',
    'Laminas\\Stdlib\\Parameters' => $vendorDir . '/laminas/laminas-stdlib/src/Parameters.php',
    'Laminas\\Stdlib\\ParametersInterface' => $vendorDir . '/laminas/laminas-stdlib/src/ParametersInterface.php',
    'Laminas\\Stdlib\\PriorityList' => $vendorDir . '/laminas/laminas-stdlib/src/PriorityList.php',
    'Laminas\\Stdlib\\PriorityQueue' => $vendorDir . '/laminas/laminas-stdlib/src/PriorityQueue.php',
    'Laminas\\Stdlib\\Request' => $vendorDir . '/laminas/laminas-stdlib/src/Request.php',
    'Laminas\\Stdlib\\RequestInterface' => $vendorDir . '/laminas/laminas-stdlib/src/RequestInterface.php',
    'Laminas\\Stdlib\\Response' => $vendorDir . '/laminas/laminas-stdlib/src/Response.php',
    'Laminas\\Stdlib\\ResponseInterface' => $vendorDir . '/laminas/laminas-stdlib/src/ResponseInterface.php',
    'Laminas\\Stdlib\\SplPriorityQueue' => $vendorDir . '/laminas/laminas-stdlib/src/SplPriorityQueue.php',
    'Laminas\\Stdlib\\SplQueue' => $vendorDir . '/laminas/laminas-stdlib/src/SplQueue.php',
    'Laminas\\Stdlib\\SplStack' => $vendorDir . '/laminas/laminas-stdlib/src/SplStack.php',
    'Laminas\\Stdlib\\StringUtils' => $vendorDir . '/laminas/laminas-stdlib/src/StringUtils.php',
    'Laminas\\Stdlib\\StringWrapper\\AbstractStringWrapper' => $vendorDir . '/laminas/laminas-stdlib/src/StringWrapper/AbstractStringWrapper.php',
    'Laminas\\Stdlib\\StringWrapper\\Iconv' => $vendorDir . '/laminas/laminas-stdlib/src/StringWrapper/Iconv.php',
    'Laminas\\Stdlib\\StringWrapper\\Intl' => $vendorDir . '/laminas/laminas-stdlib/src/StringWrapper/Intl.php',
    'Laminas\\Stdlib\\StringWrapper\\MbString' => $vendorDir . '/laminas/laminas-stdlib/src/StringWrapper/MbString.php',
    'Laminas\\Stdlib\\StringWrapper\\Native' => $vendorDir . '/laminas/laminas-stdlib/src/StringWrapper/Native.php',
    'Laminas\\Stdlib\\StringWrapper\\StringWrapperInterface' => $vendorDir . '/laminas/laminas-stdlib/src/StringWrapper/StringWrapperInterface.php',
    'Laminas\\Translator\\TranslatorInterface' => $vendorDir . '/laminas/laminas-translator/src/TranslatorInterface.php',
    'Laminas\\Validator\\AbstractValidator' => $vendorDir . '/laminas/laminas-validator/src/AbstractValidator.php',
    'Laminas\\Validator\\Barcode' => $vendorDir . '/laminas/laminas-validator/src/Barcode.php',
    'Laminas\\Validator\\Barcode\\AbstractAdapter' => $vendorDir . '/laminas/laminas-validator/src/Barcode/AbstractAdapter.php',
    'Laminas\\Validator\\Barcode\\AdapterInterface' => $vendorDir . '/laminas/laminas-validator/src/Barcode/AdapterInterface.php',
    'Laminas\\Validator\\Barcode\\Codabar' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Codabar.php',
    'Laminas\\Validator\\Barcode\\Code128' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Code128.php',
    'Laminas\\Validator\\Barcode\\Code25' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Code25.php',
    'Laminas\\Validator\\Barcode\\Code25interleaved' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Code25interleaved.php',
    'Laminas\\Validator\\Barcode\\Code39' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Code39.php',
    'Laminas\\Validator\\Barcode\\Code39ext' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Code39ext.php',
    'Laminas\\Validator\\Barcode\\Code93' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Code93.php',
    'Laminas\\Validator\\Barcode\\Code93ext' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Code93ext.php',
    'Laminas\\Validator\\Barcode\\Ean12' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Ean12.php',
    'Laminas\\Validator\\Barcode\\Ean13' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Ean13.php',
    'Laminas\\Validator\\Barcode\\Ean14' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Ean14.php',
    'Laminas\\Validator\\Barcode\\Ean18' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Ean18.php',
    'Laminas\\Validator\\Barcode\\Ean2' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Ean2.php',
    'Laminas\\Validator\\Barcode\\Ean5' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Ean5.php',
    'Laminas\\Validator\\Barcode\\Ean8' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Ean8.php',
    'Laminas\\Validator\\Barcode\\Gtin12' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Gtin12.php',
    'Laminas\\Validator\\Barcode\\Gtin13' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Gtin13.php',
    'Laminas\\Validator\\Barcode\\Gtin14' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Gtin14.php',
    'Laminas\\Validator\\Barcode\\Identcode' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Identcode.php',
    'Laminas\\Validator\\Barcode\\Intelligentmail' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Intelligentmail.php',
    'Laminas\\Validator\\Barcode\\Issn' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Issn.php',
    'Laminas\\Validator\\Barcode\\Itf14' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Itf14.php',
    'Laminas\\Validator\\Barcode\\Leitcode' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Leitcode.php',
    'Laminas\\Validator\\Barcode\\Planet' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Planet.php',
    'Laminas\\Validator\\Barcode\\Postnet' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Postnet.php',
    'Laminas\\Validator\\Barcode\\Royalmail' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Royalmail.php',
    'Laminas\\Validator\\Barcode\\Sscc' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Sscc.php',
    'Laminas\\Validator\\Barcode\\Upca' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Upca.php',
    'Laminas\\Validator\\Barcode\\Upce' => $vendorDir . '/laminas/laminas-validator/src/Barcode/Upce.php',
    'Laminas\\Validator\\Between' => $vendorDir . '/laminas/laminas-validator/src/Between.php',
    'Laminas\\Validator\\Bitwise' => $vendorDir . '/laminas/laminas-validator/src/Bitwise.php',
    'Laminas\\Validator\\BusinessIdentifierCode' => $vendorDir . '/laminas/laminas-validator/src/BusinessIdentifierCode.php',
    'Laminas\\Validator\\Callback' => $vendorDir . '/laminas/laminas-validator/src/Callback.php',
    'Laminas\\Validator\\ConfigProvider' => $vendorDir . '/laminas/laminas-validator/src/ConfigProvider.php',
    'Laminas\\Validator\\CreditCard' => $vendorDir . '/laminas/laminas-validator/src/CreditCard.php',
    'Laminas\\Validator\\Csrf' => $vendorDir . '/laminas/laminas-validator/src/Csrf.php',
    'Laminas\\Validator\\Date' => $vendorDir . '/laminas/laminas-validator/src/Date.php',
    'Laminas\\Validator\\DateComparison' => $vendorDir . '/laminas/laminas-validator/src/DateComparison.php',
    'Laminas\\Validator\\DateStep' => $vendorDir . '/laminas/laminas-validator/src/DateStep.php',
    'Laminas\\Validator\\Db\\AbstractDb' => $vendorDir . '/laminas/laminas-validator/src/Db/AbstractDb.php',
    'Laminas\\Validator\\Db\\NoRecordExists' => $vendorDir . '/laminas/laminas-validator/src/Db/NoRecordExists.php',
    'Laminas\\Validator\\Db\\RecordExists' => $vendorDir . '/laminas/laminas-validator/src/Db/RecordExists.php',
    'Laminas\\Validator\\Digits' => $vendorDir . '/laminas/laminas-validator/src/Digits.php',
    'Laminas\\Validator\\EmailAddress' => $vendorDir . '/laminas/laminas-validator/src/EmailAddress.php',
    'Laminas\\Validator\\Exception\\BadMethodCallException' => $vendorDir . '/laminas/laminas-validator/src/Exception/BadMethodCallException.php',
    'Laminas\\Validator\\Exception\\ExceptionInterface' => $vendorDir . '/laminas/laminas-validator/src/Exception/ExceptionInterface.php',
    'Laminas\\Validator\\Exception\\ExtensionNotLoadedException' => $vendorDir . '/laminas/laminas-validator/src/Exception/ExtensionNotLoadedException.php',
    'Laminas\\Validator\\Exception\\InvalidArgumentException' => $vendorDir . '/laminas/laminas-validator/src/Exception/InvalidArgumentException.php',
    'Laminas\\Validator\\Exception\\InvalidMagicMimeFileException' => $vendorDir . '/laminas/laminas-validator/src/Exception/InvalidMagicMimeFileException.php',
    'Laminas\\Validator\\Exception\\RuntimeException' => $vendorDir . '/laminas/laminas-validator/src/Exception/RuntimeException.php',
    'Laminas\\Validator\\Explode' => $vendorDir . '/laminas/laminas-validator/src/Explode.php',
    'Laminas\\Validator\\File\\Count' => $vendorDir . '/laminas/laminas-validator/src/File/Count.php',
    'Laminas\\Validator\\File\\Crc32' => $vendorDir . '/laminas/laminas-validator/src/File/Crc32.php',
    'Laminas\\Validator\\File\\ExcludeExtension' => $vendorDir . '/laminas/laminas-validator/src/File/ExcludeExtension.php',
    'Laminas\\Validator\\File\\ExcludeMimeType' => $vendorDir . '/laminas/laminas-validator/src/File/ExcludeMimeType.php',
    'Laminas\\Validator\\File\\Exists' => $vendorDir . '/laminas/laminas-validator/src/File/Exists.php',
    'Laminas\\Validator\\File\\Extension' => $vendorDir . '/laminas/laminas-validator/src/File/Extension.php',
    'Laminas\\Validator\\File\\FileInformationTrait' => $vendorDir . '/laminas/laminas-validator/src/File/FileInformationTrait.php',
    'Laminas\\Validator\\File\\FilesSize' => $vendorDir . '/laminas/laminas-validator/src/File/FilesSize.php',
    'Laminas\\Validator\\File\\Hash' => $vendorDir . '/laminas/laminas-validator/src/File/Hash.php',
    'Laminas\\Validator\\File\\ImageSize' => $vendorDir . '/laminas/laminas-validator/src/File/ImageSize.php',
    'Laminas\\Validator\\File\\IsCompressed' => $vendorDir . '/laminas/laminas-validator/src/File/IsCompressed.php',
    'Laminas\\Validator\\File\\IsImage' => $vendorDir . '/laminas/laminas-validator/src/File/IsImage.php',
    'Laminas\\Validator\\File\\Md5' => $vendorDir . '/laminas/laminas-validator/src/File/Md5.php',
    'Laminas\\Validator\\File\\MimeType' => $vendorDir . '/laminas/laminas-validator/src/File/MimeType.php',
    'Laminas\\Validator\\File\\NotExists' => $vendorDir . '/laminas/laminas-validator/src/File/NotExists.php',
    'Laminas\\Validator\\File\\Sha1' => $vendorDir . '/laminas/laminas-validator/src/File/Sha1.php',
    'Laminas\\Validator\\File\\Size' => $vendorDir . '/laminas/laminas-validator/src/File/Size.php',
    'Laminas\\Validator\\File\\Upload' => $vendorDir . '/laminas/laminas-validator/src/File/Upload.php',
    'Laminas\\Validator\\File\\UploadFile' => $vendorDir . '/laminas/laminas-validator/src/File/UploadFile.php',
    'Laminas\\Validator\\File\\WordCount' => $vendorDir . '/laminas/laminas-validator/src/File/WordCount.php',
    'Laminas\\Validator\\GpsPoint' => $vendorDir . '/laminas/laminas-validator/src/GpsPoint.php',
    'Laminas\\Validator\\GreaterThan' => $vendorDir . '/laminas/laminas-validator/src/GreaterThan.php',
    'Laminas\\Validator\\Hex' => $vendorDir . '/laminas/laminas-validator/src/Hex.php',
    'Laminas\\Validator\\HostWithPublicIPv4Address' => $vendorDir . '/laminas/laminas-validator/src/HostWithPublicIPv4Address.php',
    'Laminas\\Validator\\Hostname' => $vendorDir . '/laminas/laminas-validator/src/Hostname.php',
    'Laminas\\Validator\\Iban' => $vendorDir . '/laminas/laminas-validator/src/Iban.php',
    'Laminas\\Validator\\Identical' => $vendorDir . '/laminas/laminas-validator/src/Identical.php',
    'Laminas\\Validator\\InArray' => $vendorDir . '/laminas/laminas-validator/src/InArray.php',
    'Laminas\\Validator\\Ip' => $vendorDir . '/laminas/laminas-validator/src/Ip.php',
    'Laminas\\Validator\\IsArray' => $vendorDir . '/laminas/laminas-validator/src/IsArray.php',
    'Laminas\\Validator\\IsCountable' => $vendorDir . '/laminas/laminas-validator/src/IsCountable.php',
    'Laminas\\Validator\\IsInstanceOf' => $vendorDir . '/laminas/laminas-validator/src/IsInstanceOf.php',
    'Laminas\\Validator\\IsJsonString' => $vendorDir . '/laminas/laminas-validator/src/IsJsonString.php',
    'Laminas\\Validator\\Isbn' => $vendorDir . '/laminas/laminas-validator/src/Isbn.php',
    'Laminas\\Validator\\Isbn\\Isbn10' => $vendorDir . '/laminas/laminas-validator/src/Isbn/Isbn10.php',
    'Laminas\\Validator\\Isbn\\Isbn13' => $vendorDir . '/laminas/laminas-validator/src/Isbn/Isbn13.php',
    'Laminas\\Validator\\LessThan' => $vendorDir . '/laminas/laminas-validator/src/LessThan.php',
    'Laminas\\Validator\\Module' => $vendorDir . '/laminas/laminas-validator/src/Module.php',
    'Laminas\\Validator\\NotEmpty' => $vendorDir . '/laminas/laminas-validator/src/NotEmpty.php',
    'Laminas\\Validator\\NumberComparison' => $vendorDir . '/laminas/laminas-validator/src/NumberComparison.php',
    'Laminas\\Validator\\Regex' => $vendorDir . '/laminas/laminas-validator/src/Regex.php',
    'Laminas\\Validator\\Sitemap\\Changefreq' => $vendorDir . '/laminas/laminas-validator/src/Sitemap/Changefreq.php',
    'Laminas\\Validator\\Sitemap\\Lastmod' => $vendorDir . '/laminas/laminas-validator/src/Sitemap/Lastmod.php',
    'Laminas\\Validator\\Sitemap\\Loc' => $vendorDir . '/laminas/laminas-validator/src/Sitemap/Loc.php',
    'Laminas\\Validator\\Sitemap\\Priority' => $vendorDir . '/laminas/laminas-validator/src/Sitemap/Priority.php',
    'Laminas\\Validator\\StaticValidator' => $vendorDir . '/laminas/laminas-validator/src/StaticValidator.php',
    'Laminas\\Validator\\Step' => $vendorDir . '/laminas/laminas-validator/src/Step.php',
    'Laminas\\Validator\\StringLength' => $vendorDir . '/laminas/laminas-validator/src/StringLength.php',
    'Laminas\\Validator\\Timezone' => $vendorDir . '/laminas/laminas-validator/src/Timezone.php',
    'Laminas\\Validator\\Translator\\DummyTranslator' => $vendorDir . '/laminas/laminas-validator/src/Translator/DummyTranslator.php',
    'Laminas\\Validator\\Translator\\Translator' => $vendorDir . '/laminas/laminas-validator/src/Translator/Translator.php',
    'Laminas\\Validator\\Translator\\TranslatorAwareInterface' => $vendorDir . '/laminas/laminas-validator/src/Translator/TranslatorAwareInterface.php',
    'Laminas\\Validator\\Translator\\TranslatorFactory' => $vendorDir . '/laminas/laminas-validator/src/Translator/TranslatorFactory.php',
    'Laminas\\Validator\\Translator\\TranslatorInterface' => $vendorDir . '/laminas/laminas-validator/src/Translator/TranslatorInterface.php',
    'Laminas\\Validator\\UndisclosedPassword' => $vendorDir . '/laminas/laminas-validator/src/UndisclosedPassword.php',
    'Laminas\\Validator\\Uri' => $vendorDir . '/laminas/laminas-validator/src/Uri.php',
    'Laminas\\Validator\\Uuid' => $vendorDir . '/laminas/laminas-validator/src/Uuid.php',
    'Laminas\\Validator\\ValidatorChain' => $vendorDir . '/laminas/laminas-validator/src/ValidatorChain.php',
    'Laminas\\Validator\\ValidatorInterface' => $vendorDir . '/laminas/laminas-validator/src/ValidatorInterface.php',
    'Laminas\\Validator\\ValidatorPluginManager' => $vendorDir . '/laminas/laminas-validator/src/ValidatorPluginManager.php',
    'Laminas\\Validator\\ValidatorPluginManagerAwareInterface' => $vendorDir . '/laminas/laminas-validator/src/ValidatorPluginManagerAwareInterface.php',
    'Laminas\\Validator\\ValidatorPluginManagerFactory' => $vendorDir . '/laminas/laminas-validator/src/ValidatorPluginManagerFactory.php',
    'Laminas\\Validator\\ValidatorProviderInterface' => $vendorDir . '/laminas/laminas-validator/src/ValidatorProviderInterface.php',
    'Lcobucci\\Clock\\Clock' => $vendorDir . '/lcobucci/clock/src/Clock.php',
    'Lcobucci\\Clock\\FrozenClock' => $vendorDir . '/lcobucci/clock/src/FrozenClock.php',
    'Lcobucci\\Clock\\SystemClock' => $vendorDir . '/lcobucci/clock/src/SystemClock.php',
    'Lcobucci\\JWT\\Builder' => $vendorDir . '/lcobucci/jwt/src/Builder.php',
    'Lcobucci\\JWT\\ClaimsFormatter' => $vendorDir . '/lcobucci/jwt/src/ClaimsFormatter.php',
    'Lcobucci\\JWT\\Configuration' => $vendorDir . '/lcobucci/jwt/src/Configuration.php',
    'Lcobucci\\JWT\\Decoder' => $vendorDir . '/lcobucci/jwt/src/Decoder.php',
    'Lcobucci\\JWT\\Encoder' => $vendorDir . '/lcobucci/jwt/src/Encoder.php',
    'Lcobucci\\JWT\\Encoding\\CannotDecodeContent' => $vendorDir . '/lcobucci/jwt/src/Encoding/CannotDecodeContent.php',
    'Lcobucci\\JWT\\Encoding\\CannotEncodeContent' => $vendorDir . '/lcobucci/jwt/src/Encoding/CannotEncodeContent.php',
    'Lcobucci\\JWT\\Encoding\\ChainedFormatter' => $vendorDir . '/lcobucci/jwt/src/Encoding/ChainedFormatter.php',
    'Lcobucci\\JWT\\Encoding\\JoseEncoder' => $vendorDir . '/lcobucci/jwt/src/Encoding/JoseEncoder.php',
    'Lcobucci\\JWT\\Encoding\\MicrosecondBasedDateConversion' => $vendorDir . '/lcobucci/jwt/src/Encoding/MicrosecondBasedDateConversion.php',
    'Lcobucci\\JWT\\Encoding\\UnifyAudience' => $vendorDir . '/lcobucci/jwt/src/Encoding/UnifyAudience.php',
    'Lcobucci\\JWT\\Encoding\\UnixTimestampDates' => $vendorDir . '/lcobucci/jwt/src/Encoding/UnixTimestampDates.php',
    'Lcobucci\\JWT\\Exception' => $vendorDir . '/lcobucci/jwt/src/Exception.php',
    'Lcobucci\\JWT\\JwtFacade' => $vendorDir . '/lcobucci/jwt/src/JwtFacade.php',
    'Lcobucci\\JWT\\Parser' => $vendorDir . '/lcobucci/jwt/src/Parser.php',
    'Lcobucci\\JWT\\Signer' => $vendorDir . '/lcobucci/jwt/src/Signer.php',
    'Lcobucci\\JWT\\Signer\\Blake2b' => $vendorDir . '/lcobucci/jwt/src/Signer/Blake2b.php',
    'Lcobucci\\JWT\\Signer\\CannotSignPayload' => $vendorDir . '/lcobucci/jwt/src/Signer/CannotSignPayload.php',
    'Lcobucci\\JWT\\Signer\\Ecdsa' => $vendorDir . '/lcobucci/jwt/src/Signer/Ecdsa.php',
    'Lcobucci\\JWT\\Signer\\Ecdsa\\ConversionFailed' => $vendorDir . '/lcobucci/jwt/src/Signer/Ecdsa/ConversionFailed.php',
    'Lcobucci\\JWT\\Signer\\Ecdsa\\MultibyteStringConverter' => $vendorDir . '/lcobucci/jwt/src/Signer/Ecdsa/MultibyteStringConverter.php',
    'Lcobucci\\JWT\\Signer\\Ecdsa\\Sha256' => $vendorDir . '/lcobucci/jwt/src/Signer/Ecdsa/Sha256.php',
    'Lcobucci\\JWT\\Signer\\Ecdsa\\Sha384' => $vendorDir . '/lcobucci/jwt/src/Signer/Ecdsa/Sha384.php',
    'Lcobucci\\JWT\\Signer\\Ecdsa\\Sha512' => $vendorDir . '/lcobucci/jwt/src/Signer/Ecdsa/Sha512.php',
    'Lcobucci\\JWT\\Signer\\Ecdsa\\SignatureConverter' => $vendorDir . '/lcobucci/jwt/src/Signer/Ecdsa/SignatureConverter.php',
    'Lcobucci\\JWT\\Signer\\Eddsa' => $vendorDir . '/lcobucci/jwt/src/Signer/Eddsa.php',
    'Lcobucci\\JWT\\Signer\\Hmac' => $vendorDir . '/lcobucci/jwt/src/Signer/Hmac.php',
    'Lcobucci\\JWT\\Signer\\Hmac\\Sha256' => $vendorDir . '/lcobucci/jwt/src/Signer/Hmac/Sha256.php',
    'Lcobucci\\JWT\\Signer\\Hmac\\Sha384' => $vendorDir . '/lcobucci/jwt/src/Signer/Hmac/Sha384.php',
    'Lcobucci\\JWT\\Signer\\Hmac\\Sha512' => $vendorDir . '/lcobucci/jwt/src/Signer/Hmac/Sha512.php',
    'Lcobucci\\JWT\\Signer\\InvalidKeyProvided' => $vendorDir . '/lcobucci/jwt/src/Signer/InvalidKeyProvided.php',
    'Lcobucci\\JWT\\Signer\\Key' => $vendorDir . '/lcobucci/jwt/src/Signer/Key.php',
    'Lcobucci\\JWT\\Signer\\Key\\FileCouldNotBeRead' => $vendorDir . '/lcobucci/jwt/src/Signer/Key/FileCouldNotBeRead.php',
    'Lcobucci\\JWT\\Signer\\Key\\InMemory' => $vendorDir . '/lcobucci/jwt/src/Signer/Key/InMemory.php',
    'Lcobucci\\JWT\\Signer\\OpenSSL' => $vendorDir . '/lcobucci/jwt/src/Signer/OpenSSL.php',
    'Lcobucci\\JWT\\Signer\\Rsa' => $vendorDir . '/lcobucci/jwt/src/Signer/Rsa.php',
    'Lcobucci\\JWT\\Signer\\Rsa\\Sha256' => $vendorDir . '/lcobucci/jwt/src/Signer/Rsa/Sha256.php',
    'Lcobucci\\JWT\\Signer\\Rsa\\Sha384' => $vendorDir . '/lcobucci/jwt/src/Signer/Rsa/Sha384.php',
    'Lcobucci\\JWT\\Signer\\Rsa\\Sha512' => $vendorDir . '/lcobucci/jwt/src/Signer/Rsa/Sha512.php',
    'Lcobucci\\JWT\\SodiumBase64Polyfill' => $vendorDir . '/lcobucci/jwt/src/SodiumBase64Polyfill.php',
    'Lcobucci\\JWT\\Token' => $vendorDir . '/lcobucci/jwt/src/Token.php',
    'Lcobucci\\JWT\\Token\\Builder' => $vendorDir . '/lcobucci/jwt/src/Token/Builder.php',
    'Lcobucci\\JWT\\Token\\DataSet' => $vendorDir . '/lcobucci/jwt/src/Token/DataSet.php',
    'Lcobucci\\JWT\\Token\\InvalidTokenStructure' => $vendorDir . '/lcobucci/jwt/src/Token/InvalidTokenStructure.php',
    'Lcobucci\\JWT\\Token\\Parser' => $vendorDir . '/lcobucci/jwt/src/Token/Parser.php',
    'Lcobucci\\JWT\\Token\\Plain' => $vendorDir . '/lcobucci/jwt/src/Token/Plain.php',
    'Lcobucci\\JWT\\Token\\RegisteredClaimGiven' => $vendorDir . '/lcobucci/jwt/src/Token/RegisteredClaimGiven.php',
    'Lcobucci\\JWT\\Token\\RegisteredClaims' => $vendorDir . '/lcobucci/jwt/src/Token/RegisteredClaims.php',
    'Lcobucci\\JWT\\Token\\Signature' => $vendorDir . '/lcobucci/jwt/src/Token/Signature.php',
    'Lcobucci\\JWT\\Token\\UnsupportedHeaderFound' => $vendorDir . '/lcobucci/jwt/src/Token/UnsupportedHeaderFound.php',
    'Lcobucci\\JWT\\UnencryptedToken' => $vendorDir . '/lcobucci/jwt/src/UnencryptedToken.php',
    'Lcobucci\\JWT\\Validation\\Constraint' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint.php',
    'Lcobucci\\JWT\\Validation\\ConstraintViolation' => $vendorDir . '/lcobucci/jwt/src/Validation/ConstraintViolation.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\CannotValidateARegisteredClaim' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/CannotValidateARegisteredClaim.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\HasClaim' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/HasClaim.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\HasClaimWithValue' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/HasClaimWithValue.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\IdentifiedBy' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/IdentifiedBy.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\IssuedBy' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/IssuedBy.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\LeewayCannotBeNegative' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/LeewayCannotBeNegative.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\LooseValidAt' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/LooseValidAt.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\PermittedFor' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/PermittedFor.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\RelatedTo' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/RelatedTo.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\SignedWith' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/SignedWith.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\SignedWithOneInSet' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/SignedWithOneInSet.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\SignedWithUntilDate' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/SignedWithUntilDate.php',
    'Lcobucci\\JWT\\Validation\\Constraint\\StrictValidAt' => $vendorDir . '/lcobucci/jwt/src/Validation/Constraint/StrictValidAt.php',
    'Lcobucci\\JWT\\Validation\\NoConstraintsGiven' => $vendorDir . '/lcobucci/jwt/src/Validation/NoConstraintsGiven.php',
    'Lcobucci\\JWT\\Validation\\RequiredConstraintsViolated' => $vendorDir . '/lcobucci/jwt/src/Validation/RequiredConstraintsViolated.php',
    'Lcobucci\\JWT\\Validation\\SignedWith' => $vendorDir . '/lcobucci/jwt/src/Validation/SignedWith.php',
    'Lcobucci\\JWT\\Validation\\ValidAt' => $vendorDir . '/lcobucci/jwt/src/Validation/ValidAt.php',
    'Lcobucci\\JWT\\Validation\\Validator' => $vendorDir . '/lcobucci/jwt/src/Validation/Validator.php',
    'Lcobucci\\JWT\\Validator' => $vendorDir . '/lcobucci/jwt/src/Validator.php',
    'League\\CommonMark\\CommonMarkConverter' => $vendorDir . '/league/commonmark/src/CommonMarkConverter.php',
    'League\\CommonMark\\ConverterInterface' => $vendorDir . '/league/commonmark/src/ConverterInterface.php',
    'League\\CommonMark\\Delimiter\\Bracket' => $vendorDir . '/league/commonmark/src/Delimiter/Bracket.php',
    'League\\CommonMark\\Delimiter\\Delimiter' => $vendorDir . '/league/commonmark/src/Delimiter/Delimiter.php',
    'League\\CommonMark\\Delimiter\\DelimiterInterface' => $vendorDir . '/league/commonmark/src/Delimiter/DelimiterInterface.php',
    'League\\CommonMark\\Delimiter\\DelimiterParser' => $vendorDir . '/league/commonmark/src/Delimiter/DelimiterParser.php',
    'League\\CommonMark\\Delimiter\\DelimiterStack' => $vendorDir . '/league/commonmark/src/Delimiter/DelimiterStack.php',
    'League\\CommonMark\\Delimiter\\Processor\\CacheableDelimiterProcessorInterface' => $vendorDir . '/league/commonmark/src/Delimiter/Processor/CacheableDelimiterProcessorInterface.php',
    'League\\CommonMark\\Delimiter\\Processor\\DelimiterProcessorCollection' => $vendorDir . '/league/commonmark/src/Delimiter/Processor/DelimiterProcessorCollection.php',
    'League\\CommonMark\\Delimiter\\Processor\\DelimiterProcessorCollectionInterface' => $vendorDir . '/league/commonmark/src/Delimiter/Processor/DelimiterProcessorCollectionInterface.php',
    'League\\CommonMark\\Delimiter\\Processor\\DelimiterProcessorInterface' => $vendorDir . '/league/commonmark/src/Delimiter/Processor/DelimiterProcessorInterface.php',
    'League\\CommonMark\\Delimiter\\Processor\\StaggeredDelimiterProcessor' => $vendorDir . '/league/commonmark/src/Delimiter/Processor/StaggeredDelimiterProcessor.php',
    'League\\CommonMark\\Environment\\Environment' => $vendorDir . '/league/commonmark/src/Environment/Environment.php',
    'League\\CommonMark\\Environment\\EnvironmentAwareInterface' => $vendorDir . '/league/commonmark/src/Environment/EnvironmentAwareInterface.php',
    'League\\CommonMark\\Environment\\EnvironmentBuilderInterface' => $vendorDir . '/league/commonmark/src/Environment/EnvironmentBuilderInterface.php',
    'League\\CommonMark\\Environment\\EnvironmentInterface' => $vendorDir . '/league/commonmark/src/Environment/EnvironmentInterface.php',
    'League\\CommonMark\\Event\\AbstractEvent' => $vendorDir . '/league/commonmark/src/Event/AbstractEvent.php',
    'League\\CommonMark\\Event\\DocumentParsedEvent' => $vendorDir . '/league/commonmark/src/Event/DocumentParsedEvent.php',
    'League\\CommonMark\\Event\\DocumentPreParsedEvent' => $vendorDir . '/league/commonmark/src/Event/DocumentPreParsedEvent.php',
    'League\\CommonMark\\Event\\DocumentPreRenderEvent' => $vendorDir . '/league/commonmark/src/Event/DocumentPreRenderEvent.php',
    'League\\CommonMark\\Event\\DocumentRenderedEvent' => $vendorDir . '/league/commonmark/src/Event/DocumentRenderedEvent.php',
    'League\\CommonMark\\Event\\ListenerData' => $vendorDir . '/league/commonmark/src/Event/ListenerData.php',
    'League\\CommonMark\\Exception\\AlreadyInitializedException' => $vendorDir . '/league/commonmark/src/Exception/AlreadyInitializedException.php',
    'League\\CommonMark\\Exception\\CommonMarkException' => $vendorDir . '/league/commonmark/src/Exception/CommonMarkException.php',
    'League\\CommonMark\\Exception\\IOException' => $vendorDir . '/league/commonmark/src/Exception/IOException.php',
    'League\\CommonMark\\Exception\\InvalidArgumentException' => $vendorDir . '/league/commonmark/src/Exception/InvalidArgumentException.php',
    'League\\CommonMark\\Exception\\LogicException' => $vendorDir . '/league/commonmark/src/Exception/LogicException.php',
    'League\\CommonMark\\Exception\\MissingDependencyException' => $vendorDir . '/league/commonmark/src/Exception/MissingDependencyException.php',
    'League\\CommonMark\\Exception\\UnexpectedEncodingException' => $vendorDir . '/league/commonmark/src/Exception/UnexpectedEncodingException.php',
    'League\\CommonMark\\Extension\\Attributes\\AttributesExtension' => $vendorDir . '/league/commonmark/src/Extension/Attributes/AttributesExtension.php',
    'League\\CommonMark\\Extension\\Attributes\\Event\\AttributesListener' => $vendorDir . '/league/commonmark/src/Extension/Attributes/Event/AttributesListener.php',
    'League\\CommonMark\\Extension\\Attributes\\Node\\Attributes' => $vendorDir . '/league/commonmark/src/Extension/Attributes/Node/Attributes.php',
    'League\\CommonMark\\Extension\\Attributes\\Node\\AttributesInline' => $vendorDir . '/league/commonmark/src/Extension/Attributes/Node/AttributesInline.php',
    'League\\CommonMark\\Extension\\Attributes\\Parser\\AttributesBlockContinueParser' => $vendorDir . '/league/commonmark/src/Extension/Attributes/Parser/AttributesBlockContinueParser.php',
    'League\\CommonMark\\Extension\\Attributes\\Parser\\AttributesBlockStartParser' => $vendorDir . '/league/commonmark/src/Extension/Attributes/Parser/AttributesBlockStartParser.php',
    'League\\CommonMark\\Extension\\Attributes\\Parser\\AttributesInlineParser' => $vendorDir . '/league/commonmark/src/Extension/Attributes/Parser/AttributesInlineParser.php',
    'League\\CommonMark\\Extension\\Attributes\\Util\\AttributesHelper' => $vendorDir . '/league/commonmark/src/Extension/Attributes/Util/AttributesHelper.php',
    'League\\CommonMark\\Extension\\Autolink\\AutolinkExtension' => $vendorDir . '/league/commonmark/src/Extension/Autolink/AutolinkExtension.php',
    'League\\CommonMark\\Extension\\Autolink\\EmailAutolinkParser' => $vendorDir . '/league/commonmark/src/Extension/Autolink/EmailAutolinkParser.php',
    'League\\CommonMark\\Extension\\Autolink\\UrlAutolinkParser' => $vendorDir . '/league/commonmark/src/Extension/Autolink/UrlAutolinkParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\CommonMarkCoreExtension' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/CommonMarkCoreExtension.php',
    'League\\CommonMark\\Extension\\CommonMark\\Delimiter\\Processor\\EmphasisDelimiterProcessor' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Delimiter/Processor/EmphasisDelimiterProcessor.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Block\\BlockQuote' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Block/BlockQuote.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Block\\FencedCode' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Block/FencedCode.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Block\\Heading' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Block/Heading.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Block\\HtmlBlock' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Block/HtmlBlock.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Block\\IndentedCode' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Block/IndentedCode.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Block\\ListBlock' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Block/ListBlock.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Block\\ListData' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Block/ListData.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Block\\ListItem' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Block/ListItem.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Block\\ThematicBreak' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Block/ThematicBreak.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Inline\\AbstractWebResource' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Inline/AbstractWebResource.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Inline\\Code' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Inline/Code.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Inline\\Emphasis' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Inline/Emphasis.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Inline\\HtmlInline' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Inline/HtmlInline.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Inline\\Image' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Inline/Image.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Inline\\Link' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Inline/Link.php',
    'League\\CommonMark\\Extension\\CommonMark\\Node\\Inline\\Strong' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Node/Inline/Strong.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\BlockQuoteParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/BlockQuoteParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\BlockQuoteStartParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/BlockQuoteStartParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\FencedCodeParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/FencedCodeParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\FencedCodeStartParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/FencedCodeStartParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\HeadingParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/HeadingParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\HeadingStartParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/HeadingStartParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\HtmlBlockParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/HtmlBlockParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\HtmlBlockStartParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/HtmlBlockStartParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\IndentedCodeParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/IndentedCodeParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\IndentedCodeStartParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/IndentedCodeStartParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\ListBlockParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/ListBlockParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\ListBlockStartParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/ListBlockStartParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\ListItemParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/ListItemParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\ThematicBreakParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/ThematicBreakParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Block\\ThematicBreakStartParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Block/ThematicBreakStartParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Inline\\AutolinkParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Inline/AutolinkParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Inline\\BacktickParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Inline/BacktickParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Inline\\BangParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Inline/BangParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Inline\\CloseBracketParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Inline/CloseBracketParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Inline\\EntityParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Inline/EntityParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Inline\\EscapableParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Inline/EscapableParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Inline\\HtmlInlineParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Inline/HtmlInlineParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Parser\\Inline\\OpenBracketParser' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Parser/Inline/OpenBracketParser.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Block\\BlockQuoteRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Block/BlockQuoteRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Block\\FencedCodeRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Block/FencedCodeRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Block\\HeadingRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Block/HeadingRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Block\\HtmlBlockRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Block/HtmlBlockRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Block\\IndentedCodeRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Block/IndentedCodeRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Block\\ListBlockRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Block/ListBlockRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Block\\ListItemRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Block/ListItemRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Block\\ThematicBreakRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Block/ThematicBreakRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Inline\\CodeRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Inline/CodeRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Inline\\EmphasisRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Inline/EmphasisRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Inline\\HtmlInlineRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Inline/HtmlInlineRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Inline\\ImageRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Inline/ImageRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Inline\\LinkRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Inline/LinkRenderer.php',
    'League\\CommonMark\\Extension\\CommonMark\\Renderer\\Inline\\StrongRenderer' => $vendorDir . '/league/commonmark/src/Extension/CommonMark/Renderer/Inline/StrongRenderer.php',
    'League\\CommonMark\\Extension\\ConfigurableExtensionInterface' => $vendorDir . '/league/commonmark/src/Extension/ConfigurableExtensionInterface.php',
    'League\\CommonMark\\Extension\\DefaultAttributes\\ApplyDefaultAttributesProcessor' => $vendorDir . '/league/commonmark/src/Extension/DefaultAttributes/ApplyDefaultAttributesProcessor.php',
    'League\\CommonMark\\Extension\\DefaultAttributes\\DefaultAttributesExtension' => $vendorDir . '/league/commonmark/src/Extension/DefaultAttributes/DefaultAttributesExtension.php',
    'League\\CommonMark\\Extension\\DescriptionList\\DescriptionListExtension' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/DescriptionListExtension.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Event\\ConsecutiveDescriptionListMerger' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Event/ConsecutiveDescriptionListMerger.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Event\\LooseDescriptionHandler' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Event/LooseDescriptionHandler.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Node\\Description' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Node/Description.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Node\\DescriptionList' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Node/DescriptionList.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Node\\DescriptionTerm' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Node/DescriptionTerm.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Parser\\DescriptionContinueParser' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Parser/DescriptionContinueParser.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Parser\\DescriptionListContinueParser' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Parser/DescriptionListContinueParser.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Parser\\DescriptionStartParser' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Parser/DescriptionStartParser.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Parser\\DescriptionTermContinueParser' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Parser/DescriptionTermContinueParser.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Renderer\\DescriptionListRenderer' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Renderer/DescriptionListRenderer.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Renderer\\DescriptionRenderer' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Renderer/DescriptionRenderer.php',
    'League\\CommonMark\\Extension\\DescriptionList\\Renderer\\DescriptionTermRenderer' => $vendorDir . '/league/commonmark/src/Extension/DescriptionList/Renderer/DescriptionTermRenderer.php',
    'League\\CommonMark\\Extension\\DisallowedRawHtml\\DisallowedRawHtmlExtension' => $vendorDir . '/league/commonmark/src/Extension/DisallowedRawHtml/DisallowedRawHtmlExtension.php',
    'League\\CommonMark\\Extension\\DisallowedRawHtml\\DisallowedRawHtmlRenderer' => $vendorDir . '/league/commonmark/src/Extension/DisallowedRawHtml/DisallowedRawHtmlRenderer.php',
    'League\\CommonMark\\Extension\\Embed\\Bridge\\OscaroteroEmbedAdapter' => $vendorDir . '/league/commonmark/src/Extension/Embed/Bridge/OscaroteroEmbedAdapter.php',
    'League\\CommonMark\\Extension\\Embed\\DomainFilteringAdapter' => $vendorDir . '/league/commonmark/src/Extension/Embed/DomainFilteringAdapter.php',
    'League\\CommonMark\\Extension\\Embed\\Embed' => $vendorDir . '/league/commonmark/src/Extension/Embed/Embed.php',
    'League\\CommonMark\\Extension\\Embed\\EmbedAdapterInterface' => $vendorDir . '/league/commonmark/src/Extension/Embed/EmbedAdapterInterface.php',
    'League\\CommonMark\\Extension\\Embed\\EmbedExtension' => $vendorDir . '/league/commonmark/src/Extension/Embed/EmbedExtension.php',
    'League\\CommonMark\\Extension\\Embed\\EmbedParser' => $vendorDir . '/league/commonmark/src/Extension/Embed/EmbedParser.php',
    'League\\CommonMark\\Extension\\Embed\\EmbedProcessor' => $vendorDir . '/league/commonmark/src/Extension/Embed/EmbedProcessor.php',
    'League\\CommonMark\\Extension\\Embed\\EmbedRenderer' => $vendorDir . '/league/commonmark/src/Extension/Embed/EmbedRenderer.php',
    'League\\CommonMark\\Extension\\Embed\\EmbedStartParser' => $vendorDir . '/league/commonmark/src/Extension/Embed/EmbedStartParser.php',
    'League\\CommonMark\\Extension\\ExtensionInterface' => $vendorDir . '/league/commonmark/src/Extension/ExtensionInterface.php',
    'League\\CommonMark\\Extension\\ExternalLink\\ExternalLinkExtension' => $vendorDir . '/league/commonmark/src/Extension/ExternalLink/ExternalLinkExtension.php',
    'League\\CommonMark\\Extension\\ExternalLink\\ExternalLinkProcessor' => $vendorDir . '/league/commonmark/src/Extension/ExternalLink/ExternalLinkProcessor.php',
    'League\\CommonMark\\Extension\\Footnote\\Event\\AnonymousFootnotesListener' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Event/AnonymousFootnotesListener.php',
    'League\\CommonMark\\Extension\\Footnote\\Event\\FixOrphanedFootnotesAndRefsListener' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Event/FixOrphanedFootnotesAndRefsListener.php',
    'League\\CommonMark\\Extension\\Footnote\\Event\\GatherFootnotesListener' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Event/GatherFootnotesListener.php',
    'League\\CommonMark\\Extension\\Footnote\\Event\\NumberFootnotesListener' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Event/NumberFootnotesListener.php',
    'League\\CommonMark\\Extension\\Footnote\\FootnoteExtension' => $vendorDir . '/league/commonmark/src/Extension/Footnote/FootnoteExtension.php',
    'League\\CommonMark\\Extension\\Footnote\\Node\\Footnote' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Node/Footnote.php',
    'League\\CommonMark\\Extension\\Footnote\\Node\\FootnoteBackref' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Node/FootnoteBackref.php',
    'League\\CommonMark\\Extension\\Footnote\\Node\\FootnoteContainer' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Node/FootnoteContainer.php',
    'League\\CommonMark\\Extension\\Footnote\\Node\\FootnoteRef' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Node/FootnoteRef.php',
    'League\\CommonMark\\Extension\\Footnote\\Parser\\AnonymousFootnoteRefParser' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Parser/AnonymousFootnoteRefParser.php',
    'League\\CommonMark\\Extension\\Footnote\\Parser\\FootnoteParser' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Parser/FootnoteParser.php',
    'League\\CommonMark\\Extension\\Footnote\\Parser\\FootnoteRefParser' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Parser/FootnoteRefParser.php',
    'League\\CommonMark\\Extension\\Footnote\\Parser\\FootnoteStartParser' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Parser/FootnoteStartParser.php',
    'League\\CommonMark\\Extension\\Footnote\\Renderer\\FootnoteBackrefRenderer' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Renderer/FootnoteBackrefRenderer.php',
    'League\\CommonMark\\Extension\\Footnote\\Renderer\\FootnoteContainerRenderer' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Renderer/FootnoteContainerRenderer.php',
    'League\\CommonMark\\Extension\\Footnote\\Renderer\\FootnoteRefRenderer' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Renderer/FootnoteRefRenderer.php',
    'League\\CommonMark\\Extension\\Footnote\\Renderer\\FootnoteRenderer' => $vendorDir . '/league/commonmark/src/Extension/Footnote/Renderer/FootnoteRenderer.php',
    'League\\CommonMark\\Extension\\FrontMatter\\Data\\FrontMatterDataParserInterface' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/Data/FrontMatterDataParserInterface.php',
    'League\\CommonMark\\Extension\\FrontMatter\\Data\\LibYamlFrontMatterParser' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/Data/LibYamlFrontMatterParser.php',
    'League\\CommonMark\\Extension\\FrontMatter\\Data\\SymfonyYamlFrontMatterParser' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/Data/SymfonyYamlFrontMatterParser.php',
    'League\\CommonMark\\Extension\\FrontMatter\\Exception\\InvalidFrontMatterException' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/Exception/InvalidFrontMatterException.php',
    'League\\CommonMark\\Extension\\FrontMatter\\FrontMatterExtension' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/FrontMatterExtension.php',
    'League\\CommonMark\\Extension\\FrontMatter\\FrontMatterParser' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/FrontMatterParser.php',
    'League\\CommonMark\\Extension\\FrontMatter\\FrontMatterParserInterface' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/FrontMatterParserInterface.php',
    'League\\CommonMark\\Extension\\FrontMatter\\FrontMatterProviderInterface' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/FrontMatterProviderInterface.php',
    'League\\CommonMark\\Extension\\FrontMatter\\Input\\MarkdownInputWithFrontMatter' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/Input/MarkdownInputWithFrontMatter.php',
    'League\\CommonMark\\Extension\\FrontMatter\\Listener\\FrontMatterPostRenderListener' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/Listener/FrontMatterPostRenderListener.php',
    'League\\CommonMark\\Extension\\FrontMatter\\Listener\\FrontMatterPreParser' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/Listener/FrontMatterPreParser.php',
    'League\\CommonMark\\Extension\\FrontMatter\\Output\\RenderedContentWithFrontMatter' => $vendorDir . '/league/commonmark/src/Extension/FrontMatter/Output/RenderedContentWithFrontMatter.php',
    'League\\CommonMark\\Extension\\GithubFlavoredMarkdownExtension' => $vendorDir . '/league/commonmark/src/Extension/GithubFlavoredMarkdownExtension.php',
    'League\\CommonMark\\Extension\\HeadingPermalink\\HeadingPermalink' => $vendorDir . '/league/commonmark/src/Extension/HeadingPermalink/HeadingPermalink.php',
    'League\\CommonMark\\Extension\\HeadingPermalink\\HeadingPermalinkExtension' => $vendorDir . '/league/commonmark/src/Extension/HeadingPermalink/HeadingPermalinkExtension.php',
    'League\\CommonMark\\Extension\\HeadingPermalink\\HeadingPermalinkProcessor' => $vendorDir . '/league/commonmark/src/Extension/HeadingPermalink/HeadingPermalinkProcessor.php',
    'League\\CommonMark\\Extension\\HeadingPermalink\\HeadingPermalinkRenderer' => $vendorDir . '/league/commonmark/src/Extension/HeadingPermalink/HeadingPermalinkRenderer.php',
    'League\\CommonMark\\Extension\\InlinesOnly\\ChildRenderer' => $vendorDir . '/league/commonmark/src/Extension/InlinesOnly/ChildRenderer.php',
    'League\\CommonMark\\Extension\\InlinesOnly\\InlinesOnlyExtension' => $vendorDir . '/league/commonmark/src/Extension/InlinesOnly/InlinesOnlyExtension.php',
    'League\\CommonMark\\Extension\\Mention\\Generator\\CallbackGenerator' => $vendorDir . '/league/commonmark/src/Extension/Mention/Generator/CallbackGenerator.php',
    'League\\CommonMark\\Extension\\Mention\\Generator\\MentionGeneratorInterface' => $vendorDir . '/league/commonmark/src/Extension/Mention/Generator/MentionGeneratorInterface.php',
    'League\\CommonMark\\Extension\\Mention\\Generator\\StringTemplateLinkGenerator' => $vendorDir . '/league/commonmark/src/Extension/Mention/Generator/StringTemplateLinkGenerator.php',
    'League\\CommonMark\\Extension\\Mention\\Mention' => $vendorDir . '/league/commonmark/src/Extension/Mention/Mention.php',
    'League\\CommonMark\\Extension\\Mention\\MentionExtension' => $vendorDir . '/league/commonmark/src/Extension/Mention/MentionExtension.php',
    'League\\CommonMark\\Extension\\Mention\\MentionParser' => $vendorDir . '/league/commonmark/src/Extension/Mention/MentionParser.php',
    'League\\CommonMark\\Extension\\SmartPunct\\DashParser' => $vendorDir . '/league/commonmark/src/Extension/SmartPunct/DashParser.php',
    'League\\CommonMark\\Extension\\SmartPunct\\EllipsesParser' => $vendorDir . '/league/commonmark/src/Extension/SmartPunct/EllipsesParser.php',
    'League\\CommonMark\\Extension\\SmartPunct\\Quote' => $vendorDir . '/league/commonmark/src/Extension/SmartPunct/Quote.php',
    'League\\CommonMark\\Extension\\SmartPunct\\QuoteParser' => $vendorDir . '/league/commonmark/src/Extension/SmartPunct/QuoteParser.php',
    'League\\CommonMark\\Extension\\SmartPunct\\QuoteProcessor' => $vendorDir . '/league/commonmark/src/Extension/SmartPunct/QuoteProcessor.php',
    'League\\CommonMark\\Extension\\SmartPunct\\ReplaceUnpairedQuotesListener' => $vendorDir . '/league/commonmark/src/Extension/SmartPunct/ReplaceUnpairedQuotesListener.php',
    'League\\CommonMark\\Extension\\SmartPunct\\SmartPunctExtension' => $vendorDir . '/league/commonmark/src/Extension/SmartPunct/SmartPunctExtension.php',
    'League\\CommonMark\\Extension\\Strikethrough\\Strikethrough' => $vendorDir . '/league/commonmark/src/Extension/Strikethrough/Strikethrough.php',
    'League\\CommonMark\\Extension\\Strikethrough\\StrikethroughDelimiterProcessor' => $vendorDir . '/league/commonmark/src/Extension/Strikethrough/StrikethroughDelimiterProcessor.php',
    'League\\CommonMark\\Extension\\Strikethrough\\StrikethroughExtension' => $vendorDir . '/league/commonmark/src/Extension/Strikethrough/StrikethroughExtension.php',
    'League\\CommonMark\\Extension\\Strikethrough\\StrikethroughRenderer' => $vendorDir . '/league/commonmark/src/Extension/Strikethrough/StrikethroughRenderer.php',
    'League\\CommonMark\\Extension\\TableOfContents\\Node\\TableOfContents' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/Node/TableOfContents.php',
    'League\\CommonMark\\Extension\\TableOfContents\\Node\\TableOfContentsPlaceholder' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/Node/TableOfContentsPlaceholder.php',
    'League\\CommonMark\\Extension\\TableOfContents\\Normalizer\\AsIsNormalizerStrategy' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/Normalizer/AsIsNormalizerStrategy.php',
    'League\\CommonMark\\Extension\\TableOfContents\\Normalizer\\FlatNormalizerStrategy' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/Normalizer/FlatNormalizerStrategy.php',
    'League\\CommonMark\\Extension\\TableOfContents\\Normalizer\\NormalizerStrategyInterface' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/Normalizer/NormalizerStrategyInterface.php',
    'League\\CommonMark\\Extension\\TableOfContents\\Normalizer\\RelativeNormalizerStrategy' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/Normalizer/RelativeNormalizerStrategy.php',
    'League\\CommonMark\\Extension\\TableOfContents\\TableOfContentsBuilder' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/TableOfContentsBuilder.php',
    'League\\CommonMark\\Extension\\TableOfContents\\TableOfContentsExtension' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/TableOfContentsExtension.php',
    'League\\CommonMark\\Extension\\TableOfContents\\TableOfContentsGenerator' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/TableOfContentsGenerator.php',
    'League\\CommonMark\\Extension\\TableOfContents\\TableOfContentsGeneratorInterface' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/TableOfContentsGeneratorInterface.php',
    'League\\CommonMark\\Extension\\TableOfContents\\TableOfContentsPlaceholderParser' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/TableOfContentsPlaceholderParser.php',
    'League\\CommonMark\\Extension\\TableOfContents\\TableOfContentsPlaceholderRenderer' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/TableOfContentsPlaceholderRenderer.php',
    'League\\CommonMark\\Extension\\TableOfContents\\TableOfContentsRenderer' => $vendorDir . '/league/commonmark/src/Extension/TableOfContents/TableOfContentsRenderer.php',
    'League\\CommonMark\\Extension\\Table\\Table' => $vendorDir . '/league/commonmark/src/Extension/Table/Table.php',
    'League\\CommonMark\\Extension\\Table\\TableCell' => $vendorDir . '/league/commonmark/src/Extension/Table/TableCell.php',
    'League\\CommonMark\\Extension\\Table\\TableCellRenderer' => $vendorDir . '/league/commonmark/src/Extension/Table/TableCellRenderer.php',
    'League\\CommonMark\\Extension\\Table\\TableExtension' => $vendorDir . '/league/commonmark/src/Extension/Table/TableExtension.php',
    'League\\CommonMark\\Extension\\Table\\TableParser' => $vendorDir . '/league/commonmark/src/Extension/Table/TableParser.php',
    'League\\CommonMark\\Extension\\Table\\TableRenderer' => $vendorDir . '/league/commonmark/src/Extension/Table/TableRenderer.php',
    'League\\CommonMark\\Extension\\Table\\TableRow' => $vendorDir . '/league/commonmark/src/Extension/Table/TableRow.php',
    'League\\CommonMark\\Extension\\Table\\TableRowRenderer' => $vendorDir . '/league/commonmark/src/Extension/Table/TableRowRenderer.php',
    'League\\CommonMark\\Extension\\Table\\TableSection' => $vendorDir . '/league/commonmark/src/Extension/Table/TableSection.php',
    'League\\CommonMark\\Extension\\Table\\TableSectionRenderer' => $vendorDir . '/league/commonmark/src/Extension/Table/TableSectionRenderer.php',
    'League\\CommonMark\\Extension\\Table\\TableStartParser' => $vendorDir . '/league/commonmark/src/Extension/Table/TableStartParser.php',
    'League\\CommonMark\\Extension\\TaskList\\TaskListExtension' => $vendorDir . '/league/commonmark/src/Extension/TaskList/TaskListExtension.php',
    'League\\CommonMark\\Extension\\TaskList\\TaskListItemMarker' => $vendorDir . '/league/commonmark/src/Extension/TaskList/TaskListItemMarker.php',
    'League\\CommonMark\\Extension\\TaskList\\TaskListItemMarkerParser' => $vendorDir . '/league/commonmark/src/Extension/TaskList/TaskListItemMarkerParser.php',
    'League\\CommonMark\\Extension\\TaskList\\TaskListItemMarkerRenderer' => $vendorDir . '/league/commonmark/src/Extension/TaskList/TaskListItemMarkerRenderer.php',
    'League\\CommonMark\\GithubFlavoredMarkdownConverter' => $vendorDir . '/league/commonmark/src/GithubFlavoredMarkdownConverter.php',
    'League\\CommonMark\\Input\\MarkdownInput' => $vendorDir . '/league/commonmark/src/Input/MarkdownInput.php',
    'League\\CommonMark\\Input\\MarkdownInputInterface' => $vendorDir . '/league/commonmark/src/Input/MarkdownInputInterface.php',
    'League\\CommonMark\\MarkdownConverter' => $vendorDir . '/league/commonmark/src/MarkdownConverter.php',
    'League\\CommonMark\\MarkdownConverterInterface' => $vendorDir . '/league/commonmark/src/MarkdownConverterInterface.php',
    'League\\CommonMark\\Node\\Block\\AbstractBlock' => $vendorDir . '/league/commonmark/src/Node/Block/AbstractBlock.php',
    'League\\CommonMark\\Node\\Block\\Document' => $vendorDir . '/league/commonmark/src/Node/Block/Document.php',
    'League\\CommonMark\\Node\\Block\\Paragraph' => $vendorDir . '/league/commonmark/src/Node/Block/Paragraph.php',
    'League\\CommonMark\\Node\\Block\\TightBlockInterface' => $vendorDir . '/league/commonmark/src/Node/Block/TightBlockInterface.php',
    'League\\CommonMark\\Node\\Inline\\AbstractInline' => $vendorDir . '/league/commonmark/src/Node/Inline/AbstractInline.php',
    'League\\CommonMark\\Node\\Inline\\AbstractStringContainer' => $vendorDir . '/league/commonmark/src/Node/Inline/AbstractStringContainer.php',
    'League\\CommonMark\\Node\\Inline\\AdjacentTextMerger' => $vendorDir . '/league/commonmark/src/Node/Inline/AdjacentTextMerger.php',
    'League\\CommonMark\\Node\\Inline\\DelimitedInterface' => $vendorDir . '/league/commonmark/src/Node/Inline/DelimitedInterface.php',
    'League\\CommonMark\\Node\\Inline\\Newline' => $vendorDir . '/league/commonmark/src/Node/Inline/Newline.php',
    'League\\CommonMark\\Node\\Inline\\Text' => $vendorDir . '/league/commonmark/src/Node/Inline/Text.php',
    'League\\CommonMark\\Node\\Node' => $vendorDir . '/league/commonmark/src/Node/Node.php',
    'League\\CommonMark\\Node\\NodeIterator' => $vendorDir . '/league/commonmark/src/Node/NodeIterator.php',
    'League\\CommonMark\\Node\\NodeWalker' => $vendorDir . '/league/commonmark/src/Node/NodeWalker.php',
    'League\\CommonMark\\Node\\NodeWalkerEvent' => $vendorDir . '/league/commonmark/src/Node/NodeWalkerEvent.php',
    'League\\CommonMark\\Node\\Query' => $vendorDir . '/league/commonmark/src/Node/Query.php',
    'League\\CommonMark\\Node\\Query\\AndExpr' => $vendorDir . '/league/commonmark/src/Node/Query/AndExpr.php',
    'League\\CommonMark\\Node\\Query\\ExpressionInterface' => $vendorDir . '/league/commonmark/src/Node/Query/ExpressionInterface.php',
    'League\\CommonMark\\Node\\Query\\OrExpr' => $vendorDir . '/league/commonmark/src/Node/Query/OrExpr.php',
    'League\\CommonMark\\Node\\RawMarkupContainerInterface' => $vendorDir . '/league/commonmark/src/Node/RawMarkupContainerInterface.php',
    'League\\CommonMark\\Node\\StringContainerHelper' => $vendorDir . '/league/commonmark/src/Node/StringContainerHelper.php',
    'League\\CommonMark\\Node\\StringContainerInterface' => $vendorDir . '/league/commonmark/src/Node/StringContainerInterface.php',
    'League\\CommonMark\\Normalizer\\SlugNormalizer' => $vendorDir . '/league/commonmark/src/Normalizer/SlugNormalizer.php',
    'League\\CommonMark\\Normalizer\\TextNormalizer' => $vendorDir . '/league/commonmark/src/Normalizer/TextNormalizer.php',
    'League\\CommonMark\\Normalizer\\TextNormalizerInterface' => $vendorDir . '/league/commonmark/src/Normalizer/TextNormalizerInterface.php',
    'League\\CommonMark\\Normalizer\\UniqueSlugNormalizer' => $vendorDir . '/league/commonmark/src/Normalizer/UniqueSlugNormalizer.php',
    'League\\CommonMark\\Normalizer\\UniqueSlugNormalizerInterface' => $vendorDir . '/league/commonmark/src/Normalizer/UniqueSlugNormalizerInterface.php',
    'League\\CommonMark\\Output\\RenderedContent' => $vendorDir . '/league/commonmark/src/Output/RenderedContent.php',
    'League\\CommonMark\\Output\\RenderedContentInterface' => $vendorDir . '/league/commonmark/src/Output/RenderedContentInterface.php',
    'League\\CommonMark\\Parser\\Block\\AbstractBlockContinueParser' => $vendorDir . '/league/commonmark/src/Parser/Block/AbstractBlockContinueParser.php',
    'League\\CommonMark\\Parser\\Block\\BlockContinue' => $vendorDir . '/league/commonmark/src/Parser/Block/BlockContinue.php',
    'League\\CommonMark\\Parser\\Block\\BlockContinueParserInterface' => $vendorDir . '/league/commonmark/src/Parser/Block/BlockContinueParserInterface.php',
    'League\\CommonMark\\Parser\\Block\\BlockContinueParserWithInlinesInterface' => $vendorDir . '/league/commonmark/src/Parser/Block/BlockContinueParserWithInlinesInterface.php',
    'League\\CommonMark\\Parser\\Block\\BlockStart' => $vendorDir . '/league/commonmark/src/Parser/Block/BlockStart.php',
    'League\\CommonMark\\Parser\\Block\\BlockStartParserInterface' => $vendorDir . '/league/commonmark/src/Parser/Block/BlockStartParserInterface.php',
    'League\\CommonMark\\Parser\\Block\\DocumentBlockParser' => $vendorDir . '/league/commonmark/src/Parser/Block/DocumentBlockParser.php',
    'League\\CommonMark\\Parser\\Block\\ParagraphParser' => $vendorDir . '/league/commonmark/src/Parser/Block/ParagraphParser.php',
    'League\\CommonMark\\Parser\\Block\\SkipLinesStartingWithLettersParser' => $vendorDir . '/league/commonmark/src/Parser/Block/SkipLinesStartingWithLettersParser.php',
    'League\\CommonMark\\Parser\\Cursor' => $vendorDir . '/league/commonmark/src/Parser/Cursor.php',
    'League\\CommonMark\\Parser\\CursorState' => $vendorDir . '/league/commonmark/src/Parser/CursorState.php',
    'League\\CommonMark\\Parser\\InlineParserContext' => $vendorDir . '/league/commonmark/src/Parser/InlineParserContext.php',
    'League\\CommonMark\\Parser\\InlineParserEngine' => $vendorDir . '/league/commonmark/src/Parser/InlineParserEngine.php',
    'League\\CommonMark\\Parser\\InlineParserEngineInterface' => $vendorDir . '/league/commonmark/src/Parser/InlineParserEngineInterface.php',
    'League\\CommonMark\\Parser\\Inline\\InlineParserInterface' => $vendorDir . '/league/commonmark/src/Parser/Inline/InlineParserInterface.php',
    'League\\CommonMark\\Parser\\Inline\\InlineParserMatch' => $vendorDir . '/league/commonmark/src/Parser/Inline/InlineParserMatch.php',
    'League\\CommonMark\\Parser\\Inline\\NewlineParser' => $vendorDir . '/league/commonmark/src/Parser/Inline/NewlineParser.php',
    'League\\CommonMark\\Parser\\MarkdownParser' => $vendorDir . '/league/commonmark/src/Parser/MarkdownParser.php',
    'League\\CommonMark\\Parser\\MarkdownParserInterface' => $vendorDir . '/league/commonmark/src/Parser/MarkdownParserInterface.php',
    'League\\CommonMark\\Parser\\MarkdownParserState' => $vendorDir . '/league/commonmark/src/Parser/MarkdownParserState.php',
    'League\\CommonMark\\Parser\\MarkdownParserStateInterface' => $vendorDir . '/league/commonmark/src/Parser/MarkdownParserStateInterface.php',
    'League\\CommonMark\\Parser\\ParserLogicException' => $vendorDir . '/league/commonmark/src/Parser/ParserLogicException.php',
    'League\\CommonMark\\Reference\\MemoryLimitedReferenceMap' => $vendorDir . '/league/commonmark/src/Reference/MemoryLimitedReferenceMap.php',
    'League\\CommonMark\\Reference\\Reference' => $vendorDir . '/league/commonmark/src/Reference/Reference.php',
    'League\\CommonMark\\Reference\\ReferenceInterface' => $vendorDir . '/league/commonmark/src/Reference/ReferenceInterface.php',
    'League\\CommonMark\\Reference\\ReferenceMap' => $vendorDir . '/league/commonmark/src/Reference/ReferenceMap.php',
    'League\\CommonMark\\Reference\\ReferenceMapInterface' => $vendorDir . '/league/commonmark/src/Reference/ReferenceMapInterface.php',
    'League\\CommonMark\\Reference\\ReferenceParser' => $vendorDir . '/league/commonmark/src/Reference/ReferenceParser.php',
    'League\\CommonMark\\Reference\\ReferenceableInterface' => $vendorDir . '/league/commonmark/src/Reference/ReferenceableInterface.php',
    'League\\CommonMark\\Renderer\\Block\\DocumentRenderer' => $vendorDir . '/league/commonmark/src/Renderer/Block/DocumentRenderer.php',
    'League\\CommonMark\\Renderer\\Block\\ParagraphRenderer' => $vendorDir . '/league/commonmark/src/Renderer/Block/ParagraphRenderer.php',
    'League\\CommonMark\\Renderer\\ChildNodeRendererInterface' => $vendorDir . '/league/commonmark/src/Renderer/ChildNodeRendererInterface.php',
    'League\\CommonMark\\Renderer\\DocumentRendererInterface' => $vendorDir . '/league/commonmark/src/Renderer/DocumentRendererInterface.php',
    'League\\CommonMark\\Renderer\\HtmlDecorator' => $vendorDir . '/league/commonmark/src/Renderer/HtmlDecorator.php',
    'League\\CommonMark\\Renderer\\HtmlRenderer' => $vendorDir . '/league/commonmark/src/Renderer/HtmlRenderer.php',
    'League\\CommonMark\\Renderer\\Inline\\NewlineRenderer' => $vendorDir . '/league/commonmark/src/Renderer/Inline/NewlineRenderer.php',
    'League\\CommonMark\\Renderer\\Inline\\TextRenderer' => $vendorDir . '/league/commonmark/src/Renderer/Inline/TextRenderer.php',
    'League\\CommonMark\\Renderer\\MarkdownRendererInterface' => $vendorDir . '/league/commonmark/src/Renderer/MarkdownRendererInterface.php',
    'League\\CommonMark\\Renderer\\NoMatchingRendererException' => $vendorDir . '/league/commonmark/src/Renderer/NoMatchingRendererException.php',
    'League\\CommonMark\\Renderer\\NodeRendererInterface' => $vendorDir . '/league/commonmark/src/Renderer/NodeRendererInterface.php',
    'League\\CommonMark\\Util\\ArrayCollection' => $vendorDir . '/league/commonmark/src/Util/ArrayCollection.php',
    'League\\CommonMark\\Util\\Html5EntityDecoder' => $vendorDir . '/league/commonmark/src/Util/Html5EntityDecoder.php',
    'League\\CommonMark\\Util\\HtmlElement' => $vendorDir . '/league/commonmark/src/Util/HtmlElement.php',
    'League\\CommonMark\\Util\\HtmlFilter' => $vendorDir . '/league/commonmark/src/Util/HtmlFilter.php',
    'League\\CommonMark\\Util\\LinkParserHelper' => $vendorDir . '/league/commonmark/src/Util/LinkParserHelper.php',
    'League\\CommonMark\\Util\\PrioritizedList' => $vendorDir . '/league/commonmark/src/Util/PrioritizedList.php',
    'League\\CommonMark\\Util\\RegexHelper' => $vendorDir . '/league/commonmark/src/Util/RegexHelper.php',
    'League\\CommonMark\\Util\\SpecReader' => $vendorDir . '/league/commonmark/src/Util/SpecReader.php',
    'League\\CommonMark\\Util\\UrlEncoder' => $vendorDir . '/league/commonmark/src/Util/UrlEncoder.php',
    'League\\CommonMark\\Util\\Xml' => $vendorDir . '/league/commonmark/src/Util/Xml.php',
    'League\\CommonMark\\Xml\\FallbackNodeXmlRenderer' => $vendorDir . '/league/commonmark/src/Xml/FallbackNodeXmlRenderer.php',
    'League\\CommonMark\\Xml\\MarkdownToXmlConverter' => $vendorDir . '/league/commonmark/src/Xml/MarkdownToXmlConverter.php',
    'League\\CommonMark\\Xml\\XmlNodeRendererInterface' => $vendorDir . '/league/commonmark/src/Xml/XmlNodeRendererInterface.php',
    'League\\CommonMark\\Xml\\XmlRenderer' => $vendorDir . '/league/commonmark/src/Xml/XmlRenderer.php',
    'League\\Config\\Configuration' => $vendorDir . '/league/config/src/Configuration.php',
    'League\\Config\\ConfigurationAwareInterface' => $vendorDir . '/league/config/src/ConfigurationAwareInterface.php',
    'League\\Config\\ConfigurationBuilderInterface' => $vendorDir . '/league/config/src/ConfigurationBuilderInterface.php',
    'League\\Config\\ConfigurationInterface' => $vendorDir . '/league/config/src/ConfigurationInterface.php',
    'League\\Config\\ConfigurationProviderInterface' => $vendorDir . '/league/config/src/ConfigurationProviderInterface.php',
    'League\\Config\\Exception\\ConfigurationExceptionInterface' => $vendorDir . '/league/config/src/Exception/ConfigurationExceptionInterface.php',
    'League\\Config\\Exception\\InvalidConfigurationException' => $vendorDir . '/league/config/src/Exception/InvalidConfigurationException.php',
    'League\\Config\\Exception\\UnknownOptionException' => $vendorDir . '/league/config/src/Exception/UnknownOptionException.php',
    'League\\Config\\Exception\\ValidationException' => $vendorDir . '/league/config/src/Exception/ValidationException.php',
    'League\\Config\\MutableConfigurationInterface' => $vendorDir . '/league/config/src/MutableConfigurationInterface.php',
    'League\\Config\\ReadOnlyConfiguration' => $vendorDir . '/league/config/src/ReadOnlyConfiguration.php',
    'League\\Config\\SchemaBuilderInterface' => $vendorDir . '/league/config/src/SchemaBuilderInterface.php',
    'League\\Csv\\AbstractCsv' => $vendorDir . '/league/csv/src/AbstractCsv.php',
    'League\\Csv\\Bom' => $vendorDir . '/league/csv/src/Bom.php',
    'League\\Csv\\Buffer' => $vendorDir . '/league/csv/src/Buffer.php',
    'League\\Csv\\ByteSequence' => $vendorDir . '/league/csv/src/ByteSequence.php',
    'League\\Csv\\CallbackStreamFilter' => $vendorDir . '/league/csv/src/CallbackStreamFilter.php',
    'League\\Csv\\CannotInsertRecord' => $vendorDir . '/league/csv/src/CannotInsertRecord.php',
    'League\\Csv\\CharsetConverter' => $vendorDir . '/league/csv/src/CharsetConverter.php',
    'League\\Csv\\ColumnConsistency' => $vendorDir . '/league/csv/src/ColumnConsistency.php',
    'League\\Csv\\EncloseField' => $vendorDir . '/league/csv/src/EncloseField.php',
    'League\\Csv\\EscapeFormula' => $vendorDir . '/league/csv/src/EscapeFormula.php',
    'League\\Csv\\Exception' => $vendorDir . '/league/csv/src/Exception.php',
    'League\\Csv\\FragmentFinder' => $vendorDir . '/league/csv/src/FragmentFinder.php',
    'League\\Csv\\FragmentNotFound' => $vendorDir . '/league/csv/src/FragmentNotFound.php',
    'League\\Csv\\HTMLConverter' => $vendorDir . '/league/csv/src/HTMLConverter.php',
    'League\\Csv\\HttpHeaders' => $vendorDir . '/league/csv/src/HttpHeaders.php',
    'League\\Csv\\Info' => $vendorDir . '/league/csv/src/Info.php',
    'League\\Csv\\InvalidArgument' => $vendorDir . '/league/csv/src/InvalidArgument.php',
    'League\\Csv\\JsonConverter' => $vendorDir . '/league/csv/src/JsonConverter.php',
    'League\\Csv\\JsonFormat' => $vendorDir . '/league/csv/src/JsonFormat.php',
    'League\\Csv\\MapIterator' => $vendorDir . '/league/csv/src/MapIterator.php',
    'League\\Csv\\Query\\Constraint\\Column' => $vendorDir . '/league/csv/src/Query/Constraint/Column.php',
    'League\\Csv\\Query\\Constraint\\Comparison' => $vendorDir . '/league/csv/src/Query/Constraint/Comparison.php',
    'League\\Csv\\Query\\Constraint\\Criteria' => $vendorDir . '/league/csv/src/Query/Constraint/Criteria.php',
    'League\\Csv\\Query\\Constraint\\Offset' => $vendorDir . '/league/csv/src/Query/Constraint/Offset.php',
    'League\\Csv\\Query\\Constraint\\TwoColumns' => $vendorDir . '/league/csv/src/Query/Constraint/TwoColumns.php',
    'League\\Csv\\Query\\Limit' => $vendorDir . '/league/csv/src/Query/Limit.php',
    'League\\Csv\\Query\\Ordering\\Column' => $vendorDir . '/league/csv/src/Query/Ordering/Column.php',
    'League\\Csv\\Query\\Ordering\\MultiSort' => $vendorDir . '/league/csv/src/Query/Ordering/MultiSort.php',
    'League\\Csv\\Query\\Predicate' => $vendorDir . '/league/csv/src/Query/Predicate.php',
    'League\\Csv\\Query\\PredicateCombinator' => $vendorDir . '/league/csv/src/Query/PredicateCombinator.php',
    'League\\Csv\\Query\\QueryException' => $vendorDir . '/league/csv/src/Query/QueryException.php',
    'League\\Csv\\Query\\Row' => $vendorDir . '/league/csv/src/Query/Row.php',
    'League\\Csv\\Query\\Sort' => $vendorDir . '/league/csv/src/Query/Sort.php',
    'League\\Csv\\Query\\SortCombinator' => $vendorDir . '/league/csv/src/Query/SortCombinator.php',
    'League\\Csv\\RFC4180Field' => $vendorDir . '/league/csv/src/RFC4180Field.php',
    'League\\Csv\\RdbmsResult' => $vendorDir . '/league/csv/src/RdbmsResult.php',
    'League\\Csv\\Reader' => $vendorDir . '/league/csv/src/Reader.php',
    'League\\Csv\\ResultSet' => $vendorDir . '/league/csv/src/ResultSet.php',
    'League\\Csv\\Serializer\\AfterMapping' => $vendorDir . '/league/csv/src/Serializer/AfterMapping.php',
    'League\\Csv\\Serializer\\ArrayShape' => $vendorDir . '/league/csv/src/Serializer/ArrayShape.php',
    'League\\Csv\\Serializer\\CallbackCasting' => $vendorDir . '/league/csv/src/Serializer/CallbackCasting.php',
    'League\\Csv\\Serializer\\CastToArray' => $vendorDir . '/league/csv/src/Serializer/CastToArray.php',
    'League\\Csv\\Serializer\\CastToBool' => $vendorDir . '/league/csv/src/Serializer/CastToBool.php',
    'League\\Csv\\Serializer\\CastToDate' => $vendorDir . '/league/csv/src/Serializer/CastToDate.php',
    'League\\Csv\\Serializer\\CastToEnum' => $vendorDir . '/league/csv/src/Serializer/CastToEnum.php',
    'League\\Csv\\Serializer\\CastToFloat' => $vendorDir . '/league/csv/src/Serializer/CastToFloat.php',
    'League\\Csv\\Serializer\\CastToInt' => $vendorDir . '/league/csv/src/Serializer/CastToInt.php',
    'League\\Csv\\Serializer\\CastToString' => $vendorDir . '/league/csv/src/Serializer/CastToString.php',
    'League\\Csv\\Serializer\\DenormalizationFailed' => $vendorDir . '/league/csv/src/Serializer/DenormalizationFailed.php',
    'League\\Csv\\Serializer\\Denormalizer' => $vendorDir . '/league/csv/src/Serializer/Denormalizer.php',
    'League\\Csv\\Serializer\\MapCell' => $vendorDir . '/league/csv/src/Serializer/MapCell.php',
    'League\\Csv\\Serializer\\MapRecord' => $vendorDir . '/league/csv/src/Serializer/MapRecord.php',
    'League\\Csv\\Serializer\\MappingFailed' => $vendorDir . '/league/csv/src/Serializer/MappingFailed.php',
    'League\\Csv\\Serializer\\PropertySetter' => $vendorDir . '/league/csv/src/Serializer/PropertySetter.php',
    'League\\Csv\\Serializer\\SerializationFailed' => $vendorDir . '/league/csv/src/Serializer/SerializationFailed.php',
    'League\\Csv\\Serializer\\Type' => $vendorDir . '/league/csv/src/Serializer/Type.php',
    'League\\Csv\\Serializer\\TypeCasting' => $vendorDir . '/league/csv/src/Serializer/TypeCasting.php',
    'League\\Csv\\Serializer\\TypeCastingFailed' => $vendorDir . '/league/csv/src/Serializer/TypeCastingFailed.php',
    'League\\Csv\\Serializer\\TypeCastingInfo' => $vendorDir . '/league/csv/src/Serializer/TypeCastingInfo.php',
    'League\\Csv\\Serializer\\TypeCastingTargetType' => $vendorDir . '/league/csv/src/Serializer/TypeCastingTargetType.php',
    'League\\Csv\\Statement' => $vendorDir . '/league/csv/src/Statement.php',
    'League\\Csv\\Stream' => $vendorDir . '/league/csv/src/Stream.php',
    'League\\Csv\\StreamFilter' => $vendorDir . '/league/csv/src/StreamFilter.php',
    'League\\Csv\\SwapDelimiter' => $vendorDir . '/league/csv/src/SwapDelimiter.php',
    'League\\Csv\\SyntaxError' => $vendorDir . '/league/csv/src/SyntaxError.php',
    'League\\Csv\\TabularData' => $vendorDir . '/league/csv/src/TabularData.php',
    'League\\Csv\\TabularDataProvider' => $vendorDir . '/league/csv/src/TabularDataProvider.php',
    'League\\Csv\\TabularDataReader' => $vendorDir . '/league/csv/src/TabularDataReader.php',
    'League\\Csv\\TabularDataWriter' => $vendorDir . '/league/csv/src/TabularDataWriter.php',
    'League\\Csv\\UnableToProcessCsv' => $vendorDir . '/league/csv/src/UnableToProcessCsv.php',
    'League\\Csv\\UnavailableFeature' => $vendorDir . '/league/csv/src/UnavailableFeature.php',
    'League\\Csv\\UnavailableStream' => $vendorDir . '/league/csv/src/UnavailableStream.php',
    'League\\Csv\\Writer' => $vendorDir . '/league/csv/src/Writer.php',
    'League\\Csv\\XMLConverter' => $vendorDir . '/league/csv/src/XMLConverter.php',
    'League\\Event\\BufferedEventDispatcher' => $vendorDir . '/league/event/src/BufferedEventDispatcher.php',
    'League\\Event\\EventDispatcher' => $vendorDir . '/league/event/src/EventDispatcher.php',
    'League\\Event\\EventDispatcherAware' => $vendorDir . '/league/event/src/EventDispatcherAware.php',
    'League\\Event\\EventDispatcherAwareBehavior' => $vendorDir . '/league/event/src/EventDispatcherAwareBehavior.php',
    'League\\Event\\EventDispatchingListenerRegistry' => $vendorDir . '/league/event/src/EventDispatchingListenerRegistry.php',
    'League\\Event\\EventGenerator' => $vendorDir . '/league/event/src/EventGenerator.php',
    'League\\Event\\EventGeneratorBehavior' => $vendorDir . '/league/event/src/EventGeneratorBehavior.php',
    'League\\Event\\HasEventName' => $vendorDir . '/league/event/src/HasEventName.php',
    'League\\Event\\Listener' => $vendorDir . '/league/event/src/Listener.php',
    'League\\Event\\ListenerPriority' => $vendorDir . '/league/event/src/ListenerPriority.php',
    'League\\Event\\ListenerRegistry' => $vendorDir . '/league/event/src/ListenerRegistry.php',
    'League\\Event\\ListenerSubscriber' => $vendorDir . '/league/event/src/ListenerSubscriber.php',
    'League\\Event\\OneTimeListener' => $vendorDir . '/league/event/src/OneTimeListener.php',
    'League\\Event\\PrioritizedListenerRegistry' => $vendorDir . '/league/event/src/PrioritizedListenerRegistry.php',
    'League\\Event\\PrioritizedListenersForEvent' => $vendorDir . '/league/event/src/PrioritizedListenersForEvent.php',
    'League\\Event\\UnableToSubscribeListener' => $vendorDir . '/league/event/src/UnableToSubscribeListener.php',
    'League\\HTMLToMarkdown\\Coerce' => $vendorDir . '/league/html-to-markdown/src/Coerce.php',
    'League\\HTMLToMarkdown\\Configuration' => $vendorDir . '/league/html-to-markdown/src/Configuration.php',
    'League\\HTMLToMarkdown\\ConfigurationAwareInterface' => $vendorDir . '/league/html-to-markdown/src/ConfigurationAwareInterface.php',
    'League\\HTMLToMarkdown\\Converter\\BlockquoteConverter' => $vendorDir . '/league/html-to-markdown/src/Converter/BlockquoteConverter.php',
    'League\\HTMLToMarkdown\\Converter\\CodeConverter' => $vendorDir . '/league/html-to-markdown/src/Converter/CodeConverter.php',
    'League\\HTMLToMarkdown\\Converter\\CommentConverter' => $vendorDir . '/league/html-to-markdown/src/Converter/CommentConverter.php',
    'League\\HTMLToMarkdown\\Converter\\ConverterInterface' => $vendorDir . '/league/html-to-markdown/src/Converter/ConverterInterface.php',
    'League\\HTMLToMarkdown\\Converter\\DefaultConverter' => $vendorDir . '/league/html-to-markdown/src/Converter/DefaultConverter.php',
    'League\\HTMLToMarkdown\\Converter\\DivConverter' => $vendorDir . '/league/html-to-markdown/src/Converter/DivConverter.php',
    'League\\HTMLToMarkdown\\Converter\\EmphasisConverter' => $vendorDir . '/league/html-to-markdown/src/Converter/EmphasisConverter.php',
    'League\\HTMLToMarkdown\\Converter\\HardBreakConverter' => $vendorDir . '/league/html-to-markdown/src/Converter/HardBreakConverter.php',
    'League\\HTMLToMarkdown\\Converter\\HeaderConverter' => $vendorDir . '/league/html-to-markdown/src/Converter/HeaderConverter.php',
    'League\\HTMLToMarkdown\\Converter\\HorizontalRuleConverter' => $vendorDir . '/league/html-to-markdown/src/Converter/HorizontalRuleConverter.php',
    'League\\HTMLToMarkdown\\Converter\\ImageConverter' => $vendorDir . '/league/html-to-markdown/src/Converter/ImageConverter.php',
    'League\\HTMLToMarkdown\\Converter\\LinkConverter' => $vendorDir . '/league/html-to-markdown/src/Converter/LinkConverter.php',
    'League\\HTMLToMarkdown\\Converter\\ListBlockConverter' => $vendorDir . '/league/html-to-markdown/src/Converter/ListBlockConverter.php',
    'League\\HTMLToMarkdown\\Converter\\ListItemConverter' => $vendorDir . '/league/html-to-markdown/src/Converter/ListItemConverter.php',
    'League\\HTMLToMarkdown\\Converter\\ParagraphConverter' => $vendorDir . '/league/html-to-markdown/src/Converter/ParagraphConverter.php',
    'League\\HTMLToMarkdown\\Converter\\PreformattedConverter' => $vendorDir . '/league/html-to-markdown/src/Converter/PreformattedConverter.php',
    'League\\HTMLToMarkdown\\Converter\\TableConverter' => $vendorDir . '/league/html-to-markdown/src/Converter/TableConverter.php',
    'League\\HTMLToMarkdown\\Converter\\TextConverter' => $vendorDir . '/league/html-to-markdown/src/Converter/TextConverter.php',
    'League\\HTMLToMarkdown\\Element' => $vendorDir . '/league/html-to-markdown/src/Element.php',
    'League\\HTMLToMarkdown\\ElementInterface' => $vendorDir . '/league/html-to-markdown/src/ElementInterface.php',
    'League\\HTMLToMarkdown\\Environment' => $vendorDir . '/league/html-to-markdown/src/Environment.php',
    'League\\HTMLToMarkdown\\HtmlConverter' => $vendorDir . '/league/html-to-markdown/src/HtmlConverter.php',
    'League\\HTMLToMarkdown\\HtmlConverterInterface' => $vendorDir . '/league/html-to-markdown/src/HtmlConverterInterface.php',
    'League\\HTMLToMarkdown\\PreConverterInterface' => $vendorDir . '/league/html-to-markdown/src/PreConverterInterface.php',
    'League\\OAuth2\\Client\\Exception\\HostedDomainException' => $vendorDir . '/league/oauth2-google/src/Exception/HostedDomainException.php',
    'League\\OAuth2\\Client\\Grant\\AbstractGrant' => $vendorDir . '/league/oauth2-client/src/Grant/AbstractGrant.php',
    'League\\OAuth2\\Client\\Grant\\AuthorizationCode' => $vendorDir . '/league/oauth2-client/src/Grant/AuthorizationCode.php',
    'League\\OAuth2\\Client\\Grant\\ClientCredentials' => $vendorDir . '/league/oauth2-client/src/Grant/ClientCredentials.php',
    'League\\OAuth2\\Client\\Grant\\Exception\\InvalidGrantException' => $vendorDir . '/league/oauth2-client/src/Grant/Exception/InvalidGrantException.php',
    'League\\OAuth2\\Client\\Grant\\GrantFactory' => $vendorDir . '/league/oauth2-client/src/Grant/GrantFactory.php',
    'League\\OAuth2\\Client\\Grant\\Password' => $vendorDir . '/league/oauth2-client/src/Grant/Password.php',
    'League\\OAuth2\\Client\\Grant\\RefreshToken' => $vendorDir . '/league/oauth2-client/src/Grant/RefreshToken.php',
    'League\\OAuth2\\Client\\OptionProvider\\HttpBasicAuthOptionProvider' => $vendorDir . '/league/oauth2-client/src/OptionProvider/HttpBasicAuthOptionProvider.php',
    'League\\OAuth2\\Client\\OptionProvider\\OptionProviderInterface' => $vendorDir . '/league/oauth2-client/src/OptionProvider/OptionProviderInterface.php',
    'League\\OAuth2\\Client\\OptionProvider\\PostAuthOptionProvider' => $vendorDir . '/league/oauth2-client/src/OptionProvider/PostAuthOptionProvider.php',
    'League\\OAuth2\\Client\\Provider\\AbstractProvider' => $vendorDir . '/league/oauth2-client/src/Provider/AbstractProvider.php',
    'League\\OAuth2\\Client\\Provider\\Exception\\IdentityProviderException' => $vendorDir . '/league/oauth2-client/src/Provider/Exception/IdentityProviderException.php',
    'League\\OAuth2\\Client\\Provider\\GenericProvider' => $vendorDir . '/league/oauth2-client/src/Provider/GenericProvider.php',
    'League\\OAuth2\\Client\\Provider\\GenericResourceOwner' => $vendorDir . '/league/oauth2-client/src/Provider/GenericResourceOwner.php',
    'League\\OAuth2\\Client\\Provider\\Google' => $vendorDir . '/league/oauth2-google/src/Provider/Google.php',
    'League\\OAuth2\\Client\\Provider\\GoogleUser' => $vendorDir . '/league/oauth2-google/src/Provider/GoogleUser.php',
    'League\\OAuth2\\Client\\Provider\\ResourceOwnerInterface' => $vendorDir . '/league/oauth2-client/src/Provider/ResourceOwnerInterface.php',
    'League\\OAuth2\\Client\\Token\\AccessToken' => $vendorDir . '/league/oauth2-client/src/Token/AccessToken.php',
    'League\\OAuth2\\Client\\Token\\AccessTokenInterface' => $vendorDir . '/league/oauth2-client/src/Token/AccessTokenInterface.php',
    'League\\OAuth2\\Client\\Token\\ResourceOwnerAccessTokenInterface' => $vendorDir . '/league/oauth2-client/src/Token/ResourceOwnerAccessTokenInterface.php',
    'League\\OAuth2\\Client\\Token\\SettableRefreshTokenInterface' => $vendorDir . '/league/oauth2-client/src/Token/SettableRefreshTokenInterface.php',
    'League\\OAuth2\\Client\\Tool\\ArrayAccessorTrait' => $vendorDir . '/league/oauth2-client/src/Tool/ArrayAccessorTrait.php',
    'League\\OAuth2\\Client\\Tool\\BearerAuthorizationTrait' => $vendorDir . '/league/oauth2-client/src/Tool/BearerAuthorizationTrait.php',
    'League\\OAuth2\\Client\\Tool\\GuardedPropertyTrait' => $vendorDir . '/league/oauth2-client/src/Tool/GuardedPropertyTrait.php',
    'League\\OAuth2\\Client\\Tool\\MacAuthorizationTrait' => $vendorDir . '/league/oauth2-client/src/Tool/MacAuthorizationTrait.php',
    'League\\OAuth2\\Client\\Tool\\ProviderRedirectTrait' => $vendorDir . '/league/oauth2-client/src/Tool/ProviderRedirectTrait.php',
    'League\\OAuth2\\Client\\Tool\\QueryBuilderTrait' => $vendorDir . '/league/oauth2-client/src/Tool/QueryBuilderTrait.php',
    'League\\OAuth2\\Client\\Tool\\RequestFactory' => $vendorDir . '/league/oauth2-client/src/Tool/RequestFactory.php',
    'League\\OAuth2\\Client\\Tool\\RequiredParameterTrait' => $vendorDir . '/league/oauth2-client/src/Tool/RequiredParameterTrait.php',
    'League\\OAuth2\\Server\\AuthorizationServer' => $vendorDir . '/league/oauth2-server/src/AuthorizationServer.php',
    'League\\OAuth2\\Server\\AuthorizationValidators\\AuthorizationValidatorInterface' => $vendorDir . '/league/oauth2-server/src/AuthorizationValidators/AuthorizationValidatorInterface.php',
    'League\\OAuth2\\Server\\AuthorizationValidators\\BearerTokenValidator' => $vendorDir . '/league/oauth2-server/src/AuthorizationValidators/BearerTokenValidator.php',
    'League\\OAuth2\\Server\\CodeChallengeVerifiers\\CodeChallengeVerifierInterface' => $vendorDir . '/league/oauth2-server/src/CodeChallengeVerifiers/CodeChallengeVerifierInterface.php',
    'League\\OAuth2\\Server\\CodeChallengeVerifiers\\PlainVerifier' => $vendorDir . '/league/oauth2-server/src/CodeChallengeVerifiers/PlainVerifier.php',
    'League\\OAuth2\\Server\\CodeChallengeVerifiers\\S256Verifier' => $vendorDir . '/league/oauth2-server/src/CodeChallengeVerifiers/S256Verifier.php',
    'League\\OAuth2\\Server\\CryptKey' => $vendorDir . '/league/oauth2-server/src/CryptKey.php',
    'League\\OAuth2\\Server\\CryptKeyInterface' => $vendorDir . '/league/oauth2-server/src/CryptKeyInterface.php',
    'League\\OAuth2\\Server\\CryptTrait' => $vendorDir . '/league/oauth2-server/src/CryptTrait.php',
    'League\\OAuth2\\Server\\Entities\\AccessTokenEntityInterface' => $vendorDir . '/league/oauth2-server/src/Entities/AccessTokenEntityInterface.php',
    'League\\OAuth2\\Server\\Entities\\AuthCodeEntityInterface' => $vendorDir . '/league/oauth2-server/src/Entities/AuthCodeEntityInterface.php',
    'League\\OAuth2\\Server\\Entities\\ClientEntityInterface' => $vendorDir . '/league/oauth2-server/src/Entities/ClientEntityInterface.php',
    'League\\OAuth2\\Server\\Entities\\DeviceCodeEntityInterface' => $vendorDir . '/league/oauth2-server/src/Entities/DeviceCodeEntityInterface.php',
    'League\\OAuth2\\Server\\Entities\\RefreshTokenEntityInterface' => $vendorDir . '/league/oauth2-server/src/Entities/RefreshTokenEntityInterface.php',
    'League\\OAuth2\\Server\\Entities\\ScopeEntityInterface' => $vendorDir . '/league/oauth2-server/src/Entities/ScopeEntityInterface.php',
    'League\\OAuth2\\Server\\Entities\\TokenInterface' => $vendorDir . '/league/oauth2-server/src/Entities/TokenInterface.php',
    'League\\OAuth2\\Server\\Entities\\Traits\\AccessTokenTrait' => $vendorDir . '/league/oauth2-server/src/Entities/Traits/AccessTokenTrait.php',
    'League\\OAuth2\\Server\\Entities\\Traits\\AuthCodeTrait' => $vendorDir . '/league/oauth2-server/src/Entities/Traits/AuthCodeTrait.php',
    'League\\OAuth2\\Server\\Entities\\Traits\\ClientTrait' => $vendorDir . '/league/oauth2-server/src/Entities/Traits/ClientTrait.php',
    'League\\OAuth2\\Server\\Entities\\Traits\\DeviceCodeTrait' => $vendorDir . '/league/oauth2-server/src/Entities/Traits/DeviceCodeTrait.php',
    'League\\OAuth2\\Server\\Entities\\Traits\\EntityTrait' => $vendorDir . '/league/oauth2-server/src/Entities/Traits/EntityTrait.php',
    'League\\OAuth2\\Server\\Entities\\Traits\\RefreshTokenTrait' => $vendorDir . '/league/oauth2-server/src/Entities/Traits/RefreshTokenTrait.php',
    'League\\OAuth2\\Server\\Entities\\Traits\\ScopeTrait' => $vendorDir . '/league/oauth2-server/src/Entities/Traits/ScopeTrait.php',
    'League\\OAuth2\\Server\\Entities\\Traits\\TokenEntityTrait' => $vendorDir . '/league/oauth2-server/src/Entities/Traits/TokenEntityTrait.php',
    'League\\OAuth2\\Server\\Entities\\UserEntityInterface' => $vendorDir . '/league/oauth2-server/src/Entities/UserEntityInterface.php',
    'League\\OAuth2\\Server\\EventEmitting\\AbstractEvent' => $vendorDir . '/league/oauth2-server/src/EventEmitting/AbstractEvent.php',
    'League\\OAuth2\\Server\\EventEmitting\\EmitterAwareInterface' => $vendorDir . '/league/oauth2-server/src/EventEmitting/EmitterAwareInterface.php',
    'League\\OAuth2\\Server\\EventEmitting\\EmitterAwarePolyfill' => $vendorDir . '/league/oauth2-server/src/EventEmitting/EmitterAwarePolyfill.php',
    'League\\OAuth2\\Server\\EventEmitting\\EventEmitter' => $vendorDir . '/league/oauth2-server/src/EventEmitting/EventEmitter.php',
    'League\\OAuth2\\Server\\Exception\\OAuthServerException' => $vendorDir . '/league/oauth2-server/src/Exception/OAuthServerException.php',
    'League\\OAuth2\\Server\\Exception\\UniqueTokenIdentifierConstraintViolationException' => $vendorDir . '/league/oauth2-server/src/Exception/UniqueTokenIdentifierConstraintViolationException.php',
    'League\\OAuth2\\Server\\Grant\\AbstractAuthorizeGrant' => $vendorDir . '/league/oauth2-server/src/Grant/AbstractAuthorizeGrant.php',
    'League\\OAuth2\\Server\\Grant\\AbstractGrant' => $vendorDir . '/league/oauth2-server/src/Grant/AbstractGrant.php',
    'League\\OAuth2\\Server\\Grant\\AuthCodeGrant' => $vendorDir . '/league/oauth2-server/src/Grant/AuthCodeGrant.php',
    'League\\OAuth2\\Server\\Grant\\ClientCredentialsGrant' => $vendorDir . '/league/oauth2-server/src/Grant/ClientCredentialsGrant.php',
    'League\\OAuth2\\Server\\Grant\\DeviceCodeGrant' => $vendorDir . '/league/oauth2-server/src/Grant/DeviceCodeGrant.php',
    'League\\OAuth2\\Server\\Grant\\GrantTypeInterface' => $vendorDir . '/league/oauth2-server/src/Grant/GrantTypeInterface.php',
    'League\\OAuth2\\Server\\Grant\\ImplicitGrant' => $vendorDir . '/league/oauth2-server/src/Grant/ImplicitGrant.php',
    'League\\OAuth2\\Server\\Grant\\PasswordGrant' => $vendorDir . '/league/oauth2-server/src/Grant/PasswordGrant.php',
    'League\\OAuth2\\Server\\Grant\\RefreshTokenGrant' => $vendorDir . '/league/oauth2-server/src/Grant/RefreshTokenGrant.php',
    'League\\OAuth2\\Server\\Middleware\\AuthorizationServerMiddleware' => $vendorDir . '/league/oauth2-server/src/Middleware/AuthorizationServerMiddleware.php',
    'League\\OAuth2\\Server\\Middleware\\ResourceServerMiddleware' => $vendorDir . '/league/oauth2-server/src/Middleware/ResourceServerMiddleware.php',
    'League\\OAuth2\\Server\\RedirectUriValidators\\RedirectUriValidator' => $vendorDir . '/league/oauth2-server/src/RedirectUriValidators/RedirectUriValidator.php',
    'League\\OAuth2\\Server\\RedirectUriValidators\\RedirectUriValidatorInterface' => $vendorDir . '/league/oauth2-server/src/RedirectUriValidators/RedirectUriValidatorInterface.php',
    'League\\OAuth2\\Server\\Repositories\\AccessTokenRepositoryInterface' => $vendorDir . '/league/oauth2-server/src/Repositories/AccessTokenRepositoryInterface.php',
    'League\\OAuth2\\Server\\Repositories\\AuthCodeRepositoryInterface' => $vendorDir . '/league/oauth2-server/src/Repositories/AuthCodeRepositoryInterface.php',
    'League\\OAuth2\\Server\\Repositories\\ClientRepositoryInterface' => $vendorDir . '/league/oauth2-server/src/Repositories/ClientRepositoryInterface.php',
    'League\\OAuth2\\Server\\Repositories\\DeviceCodeRepositoryInterface' => $vendorDir . '/league/oauth2-server/src/Repositories/DeviceCodeRepositoryInterface.php',
    'League\\OAuth2\\Server\\Repositories\\RefreshTokenRepositoryInterface' => $vendorDir . '/league/oauth2-server/src/Repositories/RefreshTokenRepositoryInterface.php',
    'League\\OAuth2\\Server\\Repositories\\RepositoryInterface' => $vendorDir . '/league/oauth2-server/src/Repositories/RepositoryInterface.php',
    'League\\OAuth2\\Server\\Repositories\\ScopeRepositoryInterface' => $vendorDir . '/league/oauth2-server/src/Repositories/ScopeRepositoryInterface.php',
    'League\\OAuth2\\Server\\Repositories\\UserRepositoryInterface' => $vendorDir . '/league/oauth2-server/src/Repositories/UserRepositoryInterface.php',
    'League\\OAuth2\\Server\\RequestAccessTokenEvent' => $vendorDir . '/league/oauth2-server/src/RequestAccessTokenEvent.php',
    'League\\OAuth2\\Server\\RequestEvent' => $vendorDir . '/league/oauth2-server/src/RequestEvent.php',
    'League\\OAuth2\\Server\\RequestRefreshTokenEvent' => $vendorDir . '/league/oauth2-server/src/RequestRefreshTokenEvent.php',
    'League\\OAuth2\\Server\\RequestTypes\\AuthorizationRequest' => $vendorDir . '/league/oauth2-server/src/RequestTypes/AuthorizationRequest.php',
    'League\\OAuth2\\Server\\RequestTypes\\AuthorizationRequestInterface' => $vendorDir . '/league/oauth2-server/src/RequestTypes/AuthorizationRequestInterface.php',
    'League\\OAuth2\\Server\\ResourceServer' => $vendorDir . '/league/oauth2-server/src/ResourceServer.php',
    'League\\OAuth2\\Server\\ResponseTypes\\AbstractResponseType' => $vendorDir . '/league/oauth2-server/src/ResponseTypes/AbstractResponseType.php',
    'League\\OAuth2\\Server\\ResponseTypes\\BearerTokenResponse' => $vendorDir . '/league/oauth2-server/src/ResponseTypes/BearerTokenResponse.php',
    'League\\OAuth2\\Server\\ResponseTypes\\DeviceCodeResponse' => $vendorDir . '/league/oauth2-server/src/ResponseTypes/DeviceCodeResponse.php',
    'League\\OAuth2\\Server\\ResponseTypes\\RedirectResponse' => $vendorDir . '/league/oauth2-server/src/ResponseTypes/RedirectResponse.php',
    'League\\OAuth2\\Server\\ResponseTypes\\ResponseTypeInterface' => $vendorDir . '/league/oauth2-server/src/ResponseTypes/ResponseTypeInterface.php',
    'League\\Uri\\BaseUri' => $vendorDir . '/league/uri/BaseUri.php',
    'League\\Uri\\Contracts\\AuthorityInterface' => $vendorDir . '/league/uri-interfaces/Contracts/AuthorityInterface.php',
    'League\\Uri\\Contracts\\Conditionable' => $vendorDir . '/league/uri-interfaces/Contracts/Conditionable.php',
    'League\\Uri\\Contracts\\DataPathInterface' => $vendorDir . '/league/uri-interfaces/Contracts/DataPathInterface.php',
    'League\\Uri\\Contracts\\DomainHostInterface' => $vendorDir . '/league/uri-interfaces/Contracts/DomainHostInterface.php',
    'League\\Uri\\Contracts\\FragmentDirective' => $vendorDir . '/league/uri-interfaces/Contracts/FragmentDirective.php',
    'League\\Uri\\Contracts\\FragmentInterface' => $vendorDir . '/league/uri-interfaces/Contracts/FragmentInterface.php',
    'League\\Uri\\Contracts\\HostInterface' => $vendorDir . '/league/uri-interfaces/Contracts/HostInterface.php',
    'League\\Uri\\Contracts\\IpHostInterface' => $vendorDir . '/league/uri-interfaces/Contracts/IpHostInterface.php',
    'League\\Uri\\Contracts\\PathInterface' => $vendorDir . '/league/uri-interfaces/Contracts/PathInterface.php',
    'League\\Uri\\Contracts\\PortInterface' => $vendorDir . '/league/uri-interfaces/Contracts/PortInterface.php',
    'League\\Uri\\Contracts\\QueryInterface' => $vendorDir . '/league/uri-interfaces/Contracts/QueryInterface.php',
    'League\\Uri\\Contracts\\SegmentedPathInterface' => $vendorDir . '/league/uri-interfaces/Contracts/SegmentedPathInterface.php',
    'League\\Uri\\Contracts\\UriAccess' => $vendorDir . '/league/uri-interfaces/Contracts/UriAccess.php',
    'League\\Uri\\Contracts\\UriComponentInterface' => $vendorDir . '/league/uri-interfaces/Contracts/UriComponentInterface.php',
    'League\\Uri\\Contracts\\UriException' => $vendorDir . '/league/uri-interfaces/Contracts/UriException.php',
    'League\\Uri\\Contracts\\UriInterface' => $vendorDir . '/league/uri-interfaces/Contracts/UriInterface.php',
    'League\\Uri\\Contracts\\UserInfoInterface' => $vendorDir . '/league/uri-interfaces/Contracts/UserInfoInterface.php',
    'League\\Uri\\Encoder' => $vendorDir . '/league/uri-interfaces/Encoder.php',
    'League\\Uri\\Exceptions\\ConversionFailed' => $vendorDir . '/league/uri-interfaces/Exceptions/ConversionFailed.php',
    'League\\Uri\\Exceptions\\MissingFeature' => $vendorDir . '/league/uri-interfaces/Exceptions/MissingFeature.php',
    'League\\Uri\\Exceptions\\OffsetOutOfBounds' => $vendorDir . '/league/uri-interfaces/Exceptions/OffsetOutOfBounds.php',
    'League\\Uri\\Exceptions\\SyntaxError' => $vendorDir . '/league/uri-interfaces/Exceptions/SyntaxError.php',
    'League\\Uri\\FeatureDetection' => $vendorDir . '/league/uri-interfaces/FeatureDetection.php',
    'League\\Uri\\HostFormat' => $vendorDir . '/league/uri-interfaces/HostFormat.php',
    'League\\Uri\\HostRecord' => $vendorDir . '/league/uri-interfaces/HostRecord.php',
    'League\\Uri\\HostType' => $vendorDir . '/league/uri-interfaces/HostType.php',
    'League\\Uri\\Http' => $vendorDir . '/league/uri/Http.php',
    'League\\Uri\\HttpFactory' => $vendorDir . '/league/uri/HttpFactory.php',
    'League\\Uri\\IPv4\\BCMathCalculator' => $vendorDir . '/league/uri-interfaces/IPv4/BCMathCalculator.php',
    'League\\Uri\\IPv4\\Calculator' => $vendorDir . '/league/uri-interfaces/IPv4/Calculator.php',
    'League\\Uri\\IPv4\\Converter' => $vendorDir . '/league/uri-interfaces/IPv4/Converter.php',
    'League\\Uri\\IPv4\\GMPCalculator' => $vendorDir . '/league/uri-interfaces/IPv4/GMPCalculator.php',
    'League\\Uri\\IPv4\\NativeCalculator' => $vendorDir . '/league/uri-interfaces/IPv4/NativeCalculator.php',
    'League\\Uri\\IPv6\\Converter' => $vendorDir . '/league/uri-interfaces/IPv6/Converter.php',
    'League\\Uri\\Idna\\Converter' => $vendorDir . '/league/uri-interfaces/Idna/Converter.php',
    'League\\Uri\\Idna\\Error' => $vendorDir . '/league/uri-interfaces/Idna/Error.php',
    'League\\Uri\\Idna\\Option' => $vendorDir . '/league/uri-interfaces/Idna/Option.php',
    'League\\Uri\\Idna\\Result' => $vendorDir . '/league/uri-interfaces/Idna/Result.php',
    'League\\Uri\\KeyValuePair\\Converter' => $vendorDir . '/league/uri-interfaces/KeyValuePair/Converter.php',
    'League\\Uri\\QueryString' => $vendorDir . '/league/uri-interfaces/QueryString.php',
    'League\\Uri\\SchemeType' => $vendorDir . '/league/uri/SchemeType.php',
    'League\\Uri\\Uri' => $vendorDir . '/league/uri/Uri.php',
    'League\\Uri\\UriComparisonMode' => $vendorDir . '/league/uri-interfaces/UriComparisonMode.php',
    'League\\Uri\\UriInfo' => $vendorDir . '/league/uri/UriInfo.php',
    'League\\Uri\\UriResolver' => $vendorDir . '/league/uri/UriResolver.php',
    'League\\Uri\\UriScheme' => $vendorDir . '/league/uri/UriScheme.php',
    'League\\Uri\\UriString' => $vendorDir . '/league/uri-interfaces/UriString.php',
    'League\\Uri\\UriTemplate' => $vendorDir . '/league/uri/UriTemplate.php',
    'League\\Uri\\UriTemplate\\Expression' => $vendorDir . '/league/uri/UriTemplate/Expression.php',
    'League\\Uri\\UriTemplate\\Operator' => $vendorDir . '/league/uri/UriTemplate/Operator.php',
    'League\\Uri\\UriTemplate\\Template' => $vendorDir . '/league/uri/UriTemplate/Template.php',
    'League\\Uri\\UriTemplate\\TemplateCanNotBeExpanded' => $vendorDir . '/league/uri/UriTemplate/TemplateCanNotBeExpanded.php',
    'League\\Uri\\UriTemplate\\VarSpecifier' => $vendorDir . '/league/uri/UriTemplate/VarSpecifier.php',
    'League\\Uri\\UriTemplate\\VariableBag' => $vendorDir . '/league/uri/UriTemplate/VariableBag.php',
    'League\\Uri\\Urn' => $vendorDir . '/league/uri/Urn.php',
    'League\\Uri\\UrnComparisonMode' => $vendorDir . '/league/uri-interfaces/UrnComparisonMode.php',
    'LevelAgreement' => $baseDir . '/src/LevelAgreement.php',
    'LevelAgreementLevel' => $baseDir . '/src/LevelAgreementLevel.php',
    'Line' => $baseDir . '/src/Line.php',
    'LineOperator' => $baseDir . '/src/LineOperator.php',
    'LineType' => $baseDir . '/src/LineType.php',
    'Link' => $baseDir . '/src/Link.php',
    'Link_Itemtype' => $baseDir . '/src/Link_Itemtype.php',
    'LitEmoji\\LitEmoji' => $vendorDir . '/elvanto/litemoji/src/LitEmoji.php',
    'Location' => $baseDir . '/src/Location.php',
    'Lock' => $baseDir . '/src/Lock.php',
    'Lockedfield' => $baseDir . '/src/Lockedfield.php',
    'Log' => $baseDir . '/src/Log.php',
    'MailCollector' => $baseDir . '/src/MailCollector.php',
    'ManualLink' => $baseDir . '/src/ManualLink.php',
    'Manufacturer' => $baseDir . '/src/Manufacturer.php',
    'MapGeolocation' => $baseDir . '/src/MapGeolocation.php',
    'MassiveAction' => $baseDir . '/src/MassiveAction.php',
    'Masterminds\\HTML5' => $vendorDir . '/masterminds/html5/src/HTML5.php',
    'Masterminds\\HTML5\\Elements' => $vendorDir . '/masterminds/html5/src/HTML5/Elements.php',
    'Masterminds\\HTML5\\Entities' => $vendorDir . '/masterminds/html5/src/HTML5/Entities.php',
    'Masterminds\\HTML5\\Exception' => $vendorDir . '/masterminds/html5/src/HTML5/Exception.php',
    'Masterminds\\HTML5\\InstructionProcessor' => $vendorDir . '/masterminds/html5/src/HTML5/InstructionProcessor.php',
    'Masterminds\\HTML5\\Parser\\CharacterReference' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/CharacterReference.php',
    'Masterminds\\HTML5\\Parser\\DOMTreeBuilder' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/DOMTreeBuilder.php',
    'Masterminds\\HTML5\\Parser\\EventHandler' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/EventHandler.php',
    'Masterminds\\HTML5\\Parser\\FileInputStream' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/FileInputStream.php',
    'Masterminds\\HTML5\\Parser\\InputStream' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/InputStream.php',
    'Masterminds\\HTML5\\Parser\\ParseError' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/ParseError.php',
    'Masterminds\\HTML5\\Parser\\Scanner' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/Scanner.php',
    'Masterminds\\HTML5\\Parser\\StringInputStream' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/StringInputStream.php',
    'Masterminds\\HTML5\\Parser\\Tokenizer' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/Tokenizer.php',
    'Masterminds\\HTML5\\Parser\\TreeBuildingRules' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/TreeBuildingRules.php',
    'Masterminds\\HTML5\\Parser\\UTF8Utils' => $vendorDir . '/masterminds/html5/src/HTML5/Parser/UTF8Utils.php',
    'Masterminds\\HTML5\\Serializer\\HTML5Entities' => $vendorDir . '/masterminds/html5/src/HTML5/Serializer/HTML5Entities.php',
    'Masterminds\\HTML5\\Serializer\\OutputRules' => $vendorDir . '/masterminds/html5/src/HTML5/Serializer/OutputRules.php',
    'Masterminds\\HTML5\\Serializer\\RulesInterface' => $vendorDir . '/masterminds/html5/src/HTML5/Serializer/RulesInterface.php',
    'Masterminds\\HTML5\\Serializer\\Traverser' => $vendorDir . '/masterminds/html5/src/HTML5/Serializer/Traverser.php',
    'Matrix\\Builder' => $vendorDir . '/markbaker/matrix/classes/src/Builder.php',
    'Matrix\\Decomposition\\Decomposition' => $vendorDir . '/markbaker/matrix/classes/src/Decomposition/Decomposition.php',
    'Matrix\\Decomposition\\LU' => $vendorDir . '/markbaker/matrix/classes/src/Decomposition/LU.php',
    'Matrix\\Decomposition\\QR' => $vendorDir . '/markbaker/matrix/classes/src/Decomposition/QR.php',
    'Matrix\\Div0Exception' => $vendorDir . '/markbaker/matrix/classes/src/Div0Exception.php',
    'Matrix\\Exception' => $vendorDir . '/markbaker/matrix/classes/src/Exception.php',
    'Matrix\\Functions' => $vendorDir . '/markbaker/matrix/classes/src/Functions.php',
    'Matrix\\Matrix' => $vendorDir . '/markbaker/matrix/classes/src/Matrix.php',
    'Matrix\\Operations' => $vendorDir . '/markbaker/matrix/classes/src/Operations.php',
    'Matrix\\Operators\\Addition' => $vendorDir . '/markbaker/matrix/classes/src/Operators/Addition.php',
    'Matrix\\Operators\\DirectSum' => $vendorDir . '/markbaker/matrix/classes/src/Operators/DirectSum.php',
    'Matrix\\Operators\\Division' => $vendorDir . '/markbaker/matrix/classes/src/Operators/Division.php',
    'Matrix\\Operators\\Multiplication' => $vendorDir . '/markbaker/matrix/classes/src/Operators/Multiplication.php',
    'Matrix\\Operators\\Operator' => $vendorDir . '/markbaker/matrix/classes/src/Operators/Operator.php',
    'Matrix\\Operators\\Subtraction' => $vendorDir . '/markbaker/matrix/classes/src/Operators/Subtraction.php',
    'Mexitek\\PHPColors\\Color' => $vendorDir . '/mexitek/phpcolors/src/Mexitek/PHPColors/Color.php',
    'Migration' => $baseDir . '/src/Migration.php',
    'Monitor' => $baseDir . '/src/Monitor.php',
    'MonitorModel' => $baseDir . '/src/MonitorModel.php',
    'MonitorType' => $baseDir . '/src/MonitorType.php',
    'Monolog\\Attribute\\AsMonologProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Attribute/AsMonologProcessor.php',
    'Monolog\\Attribute\\WithMonologChannel' => $vendorDir . '/monolog/monolog/src/Monolog/Attribute/WithMonologChannel.php',
    'Monolog\\DateTimeImmutable' => $vendorDir . '/monolog/monolog/src/Monolog/DateTimeImmutable.php',
    'Monolog\\ErrorHandler' => $vendorDir . '/monolog/monolog/src/Monolog/ErrorHandler.php',
    'Monolog\\Formatter\\ChromePHPFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/ChromePHPFormatter.php',
    'Monolog\\Formatter\\ElasticaFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/ElasticaFormatter.php',
    'Monolog\\Formatter\\ElasticsearchFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/ElasticsearchFormatter.php',
    'Monolog\\Formatter\\FlowdockFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/FlowdockFormatter.php',
    'Monolog\\Formatter\\FluentdFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/FluentdFormatter.php',
    'Monolog\\Formatter\\FormatterInterface' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/FormatterInterface.php',
    'Monolog\\Formatter\\GelfMessageFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/GelfMessageFormatter.php',
    'Monolog\\Formatter\\GoogleCloudLoggingFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/GoogleCloudLoggingFormatter.php',
    'Monolog\\Formatter\\HtmlFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/HtmlFormatter.php',
    'Monolog\\Formatter\\JsonFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/JsonFormatter.php',
    'Monolog\\Formatter\\LineFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/LineFormatter.php',
    'Monolog\\Formatter\\LogglyFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/LogglyFormatter.php',
    'Monolog\\Formatter\\LogmaticFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/LogmaticFormatter.php',
    'Monolog\\Formatter\\LogstashFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/LogstashFormatter.php',
    'Monolog\\Formatter\\MongoDBFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/MongoDBFormatter.php',
    'Monolog\\Formatter\\NormalizerFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/NormalizerFormatter.php',
    'Monolog\\Formatter\\ScalarFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/ScalarFormatter.php',
    'Monolog\\Formatter\\SyslogFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/SyslogFormatter.php',
    'Monolog\\Formatter\\WildfireFormatter' => $vendorDir . '/monolog/monolog/src/Monolog/Formatter/WildfireFormatter.php',
    'Monolog\\Handler\\AbstractHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/AbstractHandler.php',
    'Monolog\\Handler\\AbstractProcessingHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/AbstractProcessingHandler.php',
    'Monolog\\Handler\\AbstractSyslogHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/AbstractSyslogHandler.php',
    'Monolog\\Handler\\AmqpHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/AmqpHandler.php',
    'Monolog\\Handler\\BrowserConsoleHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/BrowserConsoleHandler.php',
    'Monolog\\Handler\\BufferHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/BufferHandler.php',
    'Monolog\\Handler\\ChromePHPHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/ChromePHPHandler.php',
    'Monolog\\Handler\\CouchDBHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/CouchDBHandler.php',
    'Monolog\\Handler\\CubeHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/CubeHandler.php',
    'Monolog\\Handler\\Curl\\Util' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/Curl/Util.php',
    'Monolog\\Handler\\DeduplicationHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/DeduplicationHandler.php',
    'Monolog\\Handler\\DoctrineCouchDBHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/DoctrineCouchDBHandler.php',
    'Monolog\\Handler\\DynamoDbHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/DynamoDbHandler.php',
    'Monolog\\Handler\\ElasticaHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/ElasticaHandler.php',
    'Monolog\\Handler\\ElasticsearchHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/ElasticsearchHandler.php',
    'Monolog\\Handler\\ErrorLogHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/ErrorLogHandler.php',
    'Monolog\\Handler\\FallbackGroupHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FallbackGroupHandler.php',
    'Monolog\\Handler\\FilterHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FilterHandler.php',
    'Monolog\\Handler\\FingersCrossedHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FingersCrossedHandler.php',
    'Monolog\\Handler\\FingersCrossed\\ActivationStrategyInterface' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FingersCrossed/ActivationStrategyInterface.php',
    'Monolog\\Handler\\FingersCrossed\\ChannelLevelActivationStrategy' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FingersCrossed/ChannelLevelActivationStrategy.php',
    'Monolog\\Handler\\FingersCrossed\\ErrorLevelActivationStrategy' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FingersCrossed/ErrorLevelActivationStrategy.php',
    'Monolog\\Handler\\FirePHPHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FirePHPHandler.php',
    'Monolog\\Handler\\FleepHookHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FleepHookHandler.php',
    'Monolog\\Handler\\FlowdockHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FlowdockHandler.php',
    'Monolog\\Handler\\FormattableHandlerInterface' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FormattableHandlerInterface.php',
    'Monolog\\Handler\\FormattableHandlerTrait' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/FormattableHandlerTrait.php',
    'Monolog\\Handler\\GelfHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/GelfHandler.php',
    'Monolog\\Handler\\GroupHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/GroupHandler.php',
    'Monolog\\Handler\\Handler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/Handler.php',
    'Monolog\\Handler\\HandlerInterface' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/HandlerInterface.php',
    'Monolog\\Handler\\HandlerWrapper' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/HandlerWrapper.php',
    'Monolog\\Handler\\IFTTTHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/IFTTTHandler.php',
    'Monolog\\Handler\\InsightOpsHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/InsightOpsHandler.php',
    'Monolog\\Handler\\LogEntriesHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/LogEntriesHandler.php',
    'Monolog\\Handler\\LogglyHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/LogglyHandler.php',
    'Monolog\\Handler\\LogmaticHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/LogmaticHandler.php',
    'Monolog\\Handler\\MailHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/MailHandler.php',
    'Monolog\\Handler\\MandrillHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/MandrillHandler.php',
    'Monolog\\Handler\\MissingExtensionException' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/MissingExtensionException.php',
    'Monolog\\Handler\\MongoDBHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/MongoDBHandler.php',
    'Monolog\\Handler\\NativeMailerHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/NativeMailerHandler.php',
    'Monolog\\Handler\\NewRelicHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/NewRelicHandler.php',
    'Monolog\\Handler\\NoopHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/NoopHandler.php',
    'Monolog\\Handler\\NullHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/NullHandler.php',
    'Monolog\\Handler\\OverflowHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/OverflowHandler.php',
    'Monolog\\Handler\\PHPConsoleHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/PHPConsoleHandler.php',
    'Monolog\\Handler\\ProcessHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/ProcessHandler.php',
    'Monolog\\Handler\\ProcessableHandlerInterface' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/ProcessableHandlerInterface.php',
    'Monolog\\Handler\\ProcessableHandlerTrait' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/ProcessableHandlerTrait.php',
    'Monolog\\Handler\\PsrHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/PsrHandler.php',
    'Monolog\\Handler\\PushoverHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/PushoverHandler.php',
    'Monolog\\Handler\\RedisHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/RedisHandler.php',
    'Monolog\\Handler\\RedisPubSubHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/RedisPubSubHandler.php',
    'Monolog\\Handler\\RollbarHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/RollbarHandler.php',
    'Monolog\\Handler\\RotatingFileHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/RotatingFileHandler.php',
    'Monolog\\Handler\\SamplingHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SamplingHandler.php',
    'Monolog\\Handler\\SendGridHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SendGridHandler.php',
    'Monolog\\Handler\\SlackHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SlackHandler.php',
    'Monolog\\Handler\\SlackWebhookHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SlackWebhookHandler.php',
    'Monolog\\Handler\\Slack\\SlackRecord' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/Slack/SlackRecord.php',
    'Monolog\\Handler\\SocketHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SocketHandler.php',
    'Monolog\\Handler\\SqsHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SqsHandler.php',
    'Monolog\\Handler\\StreamHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/StreamHandler.php',
    'Monolog\\Handler\\SymfonyMailerHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SymfonyMailerHandler.php',
    'Monolog\\Handler\\SyslogHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SyslogHandler.php',
    'Monolog\\Handler\\SyslogUdpHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SyslogUdpHandler.php',
    'Monolog\\Handler\\SyslogUdp\\UdpSocket' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/SyslogUdp/UdpSocket.php',
    'Monolog\\Handler\\TelegramBotHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/TelegramBotHandler.php',
    'Monolog\\Handler\\TestHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/TestHandler.php',
    'Monolog\\Handler\\WebRequestRecognizerTrait' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/WebRequestRecognizerTrait.php',
    'Monolog\\Handler\\WhatFailureGroupHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/WhatFailureGroupHandler.php',
    'Monolog\\Handler\\ZendMonitorHandler' => $vendorDir . '/monolog/monolog/src/Monolog/Handler/ZendMonitorHandler.php',
    'Monolog\\JsonSerializableDateTimeImmutable' => $vendorDir . '/monolog/monolog/src/Monolog/JsonSerializableDateTimeImmutable.php',
    'Monolog\\Level' => $vendorDir . '/monolog/monolog/src/Monolog/Level.php',
    'Monolog\\LogRecord' => $vendorDir . '/monolog/monolog/src/Monolog/LogRecord.php',
    'Monolog\\Logger' => $vendorDir . '/monolog/monolog/src/Monolog/Logger.php',
    'Monolog\\Processor\\ClosureContextProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/ClosureContextProcessor.php',
    'Monolog\\Processor\\GitProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/GitProcessor.php',
    'Monolog\\Processor\\HostnameProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/HostnameProcessor.php',
    'Monolog\\Processor\\IntrospectionProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/IntrospectionProcessor.php',
    'Monolog\\Processor\\LoadAverageProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/LoadAverageProcessor.php',
    'Monolog\\Processor\\MemoryPeakUsageProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/MemoryPeakUsageProcessor.php',
    'Monolog\\Processor\\MemoryProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/MemoryProcessor.php',
    'Monolog\\Processor\\MemoryUsageProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/MemoryUsageProcessor.php',
    'Monolog\\Processor\\MercurialProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/MercurialProcessor.php',
    'Monolog\\Processor\\ProcessIdProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/ProcessIdProcessor.php',
    'Monolog\\Processor\\ProcessorInterface' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/ProcessorInterface.php',
    'Monolog\\Processor\\PsrLogMessageProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/PsrLogMessageProcessor.php',
    'Monolog\\Processor\\TagProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/TagProcessor.php',
    'Monolog\\Processor\\UidProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/UidProcessor.php',
    'Monolog\\Processor\\WebProcessor' => $vendorDir . '/monolog/monolog/src/Monolog/Processor/WebProcessor.php',
    'Monolog\\Registry' => $vendorDir . '/monolog/monolog/src/Monolog/Registry.php',
    'Monolog\\ResettableInterface' => $vendorDir . '/monolog/monolog/src/Monolog/ResettableInterface.php',
    'Monolog\\SignalHandler' => $vendorDir . '/monolog/monolog/src/Monolog/SignalHandler.php',
    'Monolog\\Test\\MonologTestCase' => $vendorDir . '/monolog/monolog/src/Monolog/Test/MonologTestCase.php',
    'Monolog\\Test\\TestCase' => $vendorDir . '/monolog/monolog/src/Monolog/Test/TestCase.php',
    'Monolog\\Utils' => $vendorDir . '/monolog/monolog/src/Monolog/Utils.php',
    'Nette\\ArgumentOutOfRangeException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\DeprecatedException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\DirectoryNotFoundException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\FileNotFoundException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\HtmlStringable' => $vendorDir . '/nette/utils/src/HtmlStringable.php',
    'Nette\\IOException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\InvalidArgumentException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\InvalidStateException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\Iterators\\CachingIterator' => $vendorDir . '/nette/utils/src/Iterators/CachingIterator.php',
    'Nette\\Iterators\\Mapper' => $vendorDir . '/nette/utils/src/Iterators/Mapper.php',
    'Nette\\Localization\\ITranslator' => $vendorDir . '/nette/utils/src/compatibility.php',
    'Nette\\Localization\\Translator' => $vendorDir . '/nette/utils/src/Translator.php',
    'Nette\\MemberAccessException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\NotImplementedException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\NotSupportedException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\OutOfRangeException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\Schema\\Context' => $vendorDir . '/nette/schema/src/Schema/Context.php',
    'Nette\\Schema\\DynamicParameter' => $vendorDir . '/nette/schema/src/Schema/DynamicParameter.php',
    'Nette\\Schema\\Elements\\AnyOf' => $vendorDir . '/nette/schema/src/Schema/Elements/AnyOf.php',
    'Nette\\Schema\\Elements\\Base' => $vendorDir . '/nette/schema/src/Schema/Elements/Base.php',
    'Nette\\Schema\\Elements\\Structure' => $vendorDir . '/nette/schema/src/Schema/Elements/Structure.php',
    'Nette\\Schema\\Elements\\Type' => $vendorDir . '/nette/schema/src/Schema/Elements/Type.php',
    'Nette\\Schema\\Expect' => $vendorDir . '/nette/schema/src/Schema/Expect.php',
    'Nette\\Schema\\Helpers' => $vendorDir . '/nette/schema/src/Schema/Helpers.php',
    'Nette\\Schema\\Message' => $vendorDir . '/nette/schema/src/Schema/Message.php',
    'Nette\\Schema\\Processor' => $vendorDir . '/nette/schema/src/Schema/Processor.php',
    'Nette\\Schema\\Schema' => $vendorDir . '/nette/schema/src/Schema/Schema.php',
    'Nette\\Schema\\ValidationException' => $vendorDir . '/nette/schema/src/Schema/ValidationException.php',
    'Nette\\ShouldNotHappenException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\SmartObject' => $vendorDir . '/nette/utils/src/SmartObject.php',
    'Nette\\StaticClass' => $vendorDir . '/nette/utils/src/StaticClass.php',
    'Nette\\UnexpectedValueException' => $vendorDir . '/nette/utils/src/exceptions.php',
    'Nette\\Utils\\ArrayHash' => $vendorDir . '/nette/utils/src/Utils/ArrayHash.php',
    'Nette\\Utils\\ArrayList' => $vendorDir . '/nette/utils/src/Utils/ArrayList.php',
    'Nette\\Utils\\Arrays' => $vendorDir . '/nette/utils/src/Utils/Arrays.php',
    'Nette\\Utils\\AssertionException' => $vendorDir . '/nette/utils/src/Utils/exceptions.php',
    'Nette\\Utils\\Callback' => $vendorDir . '/nette/utils/src/Utils/Callback.php',
    'Nette\\Utils\\DateTime' => $vendorDir . '/nette/utils/src/Utils/DateTime.php',
    'Nette\\Utils\\FileInfo' => $vendorDir . '/nette/utils/src/Utils/FileInfo.php',
    'Nette\\Utils\\FileSystem' => $vendorDir . '/nette/utils/src/Utils/FileSystem.php',
    'Nette\\Utils\\Finder' => $vendorDir . '/nette/utils/src/Utils/Finder.php',
    'Nette\\Utils\\Floats' => $vendorDir . '/nette/utils/src/Utils/Floats.php',
    'Nette\\Utils\\Helpers' => $vendorDir . '/nette/utils/src/Utils/Helpers.php',
    'Nette\\Utils\\Html' => $vendorDir . '/nette/utils/src/Utils/Html.php',
    'Nette\\Utils\\IHtmlString' => $vendorDir . '/nette/utils/src/compatibility.php',
    'Nette\\Utils\\Image' => $vendorDir . '/nette/utils/src/Utils/Image.php',
    'Nette\\Utils\\ImageColor' => $vendorDir . '/nette/utils/src/Utils/ImageColor.php',
    'Nette\\Utils\\ImageException' => $vendorDir . '/nette/utils/src/Utils/exceptions.php',
    'Nette\\Utils\\ImageType' => $vendorDir . '/nette/utils/src/Utils/ImageType.php',
    'Nette\\Utils\\Iterables' => $vendorDir . '/nette/utils/src/Utils/Iterables.php',
    'Nette\\Utils\\Json' => $vendorDir . '/nette/utils/src/Utils/Json.php',
    'Nette\\Utils\\JsonException' => $vendorDir . '/nette/utils/src/Utils/exceptions.php',
    'Nette\\Utils\\ObjectHelpers' => $vendorDir . '/nette/utils/src/Utils/ObjectHelpers.php',
    'Nette\\Utils\\Paginator' => $vendorDir . '/nette/utils/src/Utils/Paginator.php',
    'Nette\\Utils\\Random' => $vendorDir . '/nette/utils/src/Utils/Random.php',
    'Nette\\Utils\\Reflection' => $vendorDir . '/nette/utils/src/Utils/Reflection.php',
    'Nette\\Utils\\ReflectionMethod' => $vendorDir . '/nette/utils/src/Utils/ReflectionMethod.php',
    'Nette\\Utils\\RegexpException' => $vendorDir . '/nette/utils/src/Utils/exceptions.php',
    'Nette\\Utils\\Strings' => $vendorDir . '/nette/utils/src/Utils/Strings.php',
    'Nette\\Utils\\Type' => $vendorDir . '/nette/utils/src/Utils/Type.php',
    'Nette\\Utils\\UnknownImageFileException' => $vendorDir . '/nette/utils/src/Utils/exceptions.php',
    'Nette\\Utils\\Validators' => $vendorDir . '/nette/utils/src/Utils/Validators.php',
    'Network' => $baseDir . '/src/Network.php',
    'NetworkAlias' => $baseDir . '/src/NetworkAlias.php',
    'NetworkEquipment' => $baseDir . '/src/NetworkEquipment.php',
    'NetworkEquipmentModel' => $baseDir . '/src/NetworkEquipmentModel.php',
    'NetworkEquipmentModelStencil' => $baseDir . '/src/NetworkEquipmentModelStencil.php',
    'NetworkEquipmentType' => $baseDir . '/src/NetworkEquipmentType.php',
    'NetworkInterface' => $baseDir . '/src/NetworkInterface.php',
    'NetworkName' => $baseDir . '/src/NetworkName.php',
    'NetworkPort' => $baseDir . '/src/NetworkPort.php',
    'NetworkPortAggregate' => $baseDir . '/src/NetworkPortAggregate.php',
    'NetworkPortAlias' => $baseDir . '/src/NetworkPortAlias.php',
    'NetworkPortConnectionLog' => $baseDir . '/src/NetworkPortConnectionLog.php',
    'NetworkPortDialup' => $baseDir . '/src/NetworkPortDialup.php',
    'NetworkPortEthernet' => $baseDir . '/src/NetworkPortEthernet.php',
    'NetworkPortFiberchannel' => $baseDir . '/src/NetworkPortFiberchannel.php',
    'NetworkPortFiberchannelType' => $baseDir . '/src/NetworkPortFiberchannelType.php',
    'NetworkPortInstantiation' => $baseDir . '/src/NetworkPortInstantiation.php',
    'NetworkPortLocal' => $baseDir . '/src/NetworkPortLocal.php',
    'NetworkPortMetrics' => $baseDir . '/src/NetworkPortMetrics.php',
    'NetworkPortType' => $baseDir . '/src/NetworkPortType.php',
    'NetworkPortWifi' => $baseDir . '/src/NetworkPortWifi.php',
    'NetworkPort_NetworkPort' => $baseDir . '/src/NetworkPort_NetworkPort.php',
    'NetworkPort_Vlan' => $baseDir . '/src/NetworkPort_Vlan.php',
    'NotImportedEmail' => $baseDir . '/src/NotImportedEmail.php',
    'Notepad' => $baseDir . '/src/Notepad.php',
    'Notification' => $baseDir . '/src/Notification.php',
    'NotificationAjax' => $baseDir . '/src/NotificationAjax.php',
    'NotificationAjaxSetting' => $baseDir . '/src/NotificationAjaxSetting.php',
    'NotificationEvent' => $baseDir . '/src/NotificationEvent.php',
    'NotificationEventAbstract' => $baseDir . '/src/NotificationEventAbstract.php',
    'NotificationEventAjax' => $baseDir . '/src/NotificationEventAjax.php',
    'NotificationEventInterface' => $baseDir . '/src/NotificationEventInterface.php',
    'NotificationEventMailing' => $baseDir . '/src/NotificationEventMailing.php',
    'NotificationInterface' => $baseDir . '/src/NotificationInterface.php',
    'NotificationMailing' => $baseDir . '/src/NotificationMailing.php',
    'NotificationMailingSetting' => $baseDir . '/src/NotificationMailingSetting.php',
    'NotificationSetting' => $baseDir . '/src/NotificationSetting.php',
    'NotificationSettingConfig' => $baseDir . '/src/NotificationSettingConfig.php',
    'NotificationTarget' => $baseDir . '/src/NotificationTarget.php',
    'NotificationTargetCartridgeItem' => $baseDir . '/src/NotificationTargetCartridgeItem.php',
    'NotificationTargetCertificate' => $baseDir . '/src/NotificationTargetCertificate.php',
    'NotificationTargetChange' => $baseDir . '/src/NotificationTargetChange.php',
    'NotificationTargetCommonITILObject' => $baseDir . '/src/NotificationTargetCommonITILObject.php',
    'NotificationTargetConsumableItem' => $baseDir . '/src/NotificationTargetConsumableItem.php',
    'NotificationTargetContract' => $baseDir . '/src/NotificationTargetContract.php',
    'NotificationTargetCronTask' => $baseDir . '/src/NotificationTargetCronTask.php',
    'NotificationTargetDBConnection' => $baseDir . '/src/NotificationTargetDBConnection.php',
    'NotificationTargetDomain' => $baseDir . '/src/NotificationTargetDomain.php',
    'NotificationTargetFieldUnicity' => $baseDir . '/src/NotificationTargetFieldUnicity.php',
    'NotificationTargetInfocom' => $baseDir . '/src/NotificationTargetInfocom.php',
    'NotificationTargetKnowbaseItem' => $baseDir . '/src/NotificationTargetKnowbaseItem.php',
    'NotificationTargetMailCollector' => $baseDir . '/src/NotificationTargetMailCollector.php',
    'NotificationTargetObjectLock' => $baseDir . '/src/NotificationTargetObjectLock.php',
    'NotificationTargetPlanningRecall' => $baseDir . '/src/NotificationTargetPlanningRecall.php',
    'NotificationTargetProblem' => $baseDir . '/src/NotificationTargetProblem.php',
    'NotificationTargetProject' => $baseDir . '/src/NotificationTargetProject.php',
    'NotificationTargetProjectTask' => $baseDir . '/src/NotificationTargetProjectTask.php',
    'NotificationTargetReservation' => $baseDir . '/src/NotificationTargetReservation.php',
    'NotificationTargetSavedSearch_Alert' => $baseDir . '/src/NotificationTargetSavedSearch_Alert.php',
    'NotificationTargetSoftwareLicense' => $baseDir . '/src/NotificationTargetSoftwareLicense.php',
    'NotificationTargetTicket' => $baseDir . '/src/NotificationTargetTicket.php',
    'NotificationTargetUser' => $baseDir . '/src/NotificationTargetUser.php',
    'NotificationTemplate' => $baseDir . '/src/NotificationTemplate.php',
    'NotificationTemplateTranslation' => $baseDir . '/src/NotificationTemplateTranslation.php',
    'Notification_NotificationTemplate' => $baseDir . '/src/Notification_NotificationTemplate.php',
    'OAuthClient' => $baseDir . '/src/OAuthClient.php',
    'OLA' => $baseDir . '/src/OLA.php',
    'ObjectLock' => $baseDir . '/src/ObjectLock.php',
    'OlaLevel' => $baseDir . '/src/OlaLevel.php',
    'OlaLevelAction' => $baseDir . '/src/OlaLevelAction.php',
    'OlaLevelCriteria' => $baseDir . '/src/OlaLevelCriteria.php',
    'OlaLevel_Ticket' => $baseDir . '/src/OlaLevel_Ticket.php',
    'OperatingSystem' => $baseDir . '/src/OperatingSystem.php',
    'OperatingSystemArchitecture' => $baseDir . '/src/OperatingSystemArchitecture.php',
    'OperatingSystemEdition' => $baseDir . '/src/OperatingSystemEdition.php',
    'OperatingSystemKernel' => $baseDir . '/src/OperatingSystemKernel.php',
    'OperatingSystemKernelVersion' => $baseDir . '/src/OperatingSystemKernelVersion.php',
    'OperatingSystemServicePack' => $baseDir . '/src/OperatingSystemServicePack.php',
    'OperatingSystemVersion' => $baseDir . '/src/OperatingSystemVersion.php',
    'Override' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/Override.php',
    'PCIVendor' => $baseDir . '/src/PCIVendor.php',
    'PDF417' => $vendorDir . '/tecnickcom/tcpdf/include/barcodes/pdf417.php',
    'PDU' => $baseDir . '/src/PDU.php',
    'PDUModel' => $baseDir . '/src/PDUModel.php',
    'PDUType' => $baseDir . '/src/PDUType.php',
    'PDU_Rack' => $baseDir . '/src/PDU_Rack.php',
    'PHPStan\\PhpDocParser\\Ast\\AbstractNodeVisitor' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/AbstractNodeVisitor.php',
    'PHPStan\\PhpDocParser\\Ast\\Attribute' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Attribute.php',
    'PHPStan\\PhpDocParser\\Ast\\Comment' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Comment.php',
    'PHPStan\\PhpDocParser\\Ast\\ConstExpr\\ConstExprArrayItemNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/ConstExpr/ConstExprArrayItemNode.php',
    'PHPStan\\PhpDocParser\\Ast\\ConstExpr\\ConstExprArrayNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/ConstExpr/ConstExprArrayNode.php',
    'PHPStan\\PhpDocParser\\Ast\\ConstExpr\\ConstExprFalseNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/ConstExpr/ConstExprFalseNode.php',
    'PHPStan\\PhpDocParser\\Ast\\ConstExpr\\ConstExprFloatNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/ConstExpr/ConstExprFloatNode.php',
    'PHPStan\\PhpDocParser\\Ast\\ConstExpr\\ConstExprIntegerNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/ConstExpr/ConstExprIntegerNode.php',
    'PHPStan\\PhpDocParser\\Ast\\ConstExpr\\ConstExprNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/ConstExpr/ConstExprNode.php',
    'PHPStan\\PhpDocParser\\Ast\\ConstExpr\\ConstExprNullNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/ConstExpr/ConstExprNullNode.php',
    'PHPStan\\PhpDocParser\\Ast\\ConstExpr\\ConstExprStringNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/ConstExpr/ConstExprStringNode.php',
    'PHPStan\\PhpDocParser\\Ast\\ConstExpr\\ConstExprTrueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/ConstExpr/ConstExprTrueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\ConstExpr\\ConstFetchNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/ConstExpr/ConstFetchNode.php',
    'PHPStan\\PhpDocParser\\Ast\\ConstExpr\\DoctrineConstExprStringNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/ConstExpr/DoctrineConstExprStringNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Node' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Node.php',
    'PHPStan\\PhpDocParser\\Ast\\NodeAttributes' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/NodeAttributes.php',
    'PHPStan\\PhpDocParser\\Ast\\NodeTraverser' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/NodeTraverser.php',
    'PHPStan\\PhpDocParser\\Ast\\NodeVisitor' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/NodeVisitor.php',
    'PHPStan\\PhpDocParser\\Ast\\NodeVisitor\\CloningVisitor' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/NodeVisitor/CloningVisitor.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\AssertTagMethodValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/AssertTagMethodValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\AssertTagPropertyValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/AssertTagPropertyValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\AssertTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/AssertTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\DeprecatedTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/DeprecatedTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\Doctrine\\DoctrineAnnotation' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/Doctrine/DoctrineAnnotation.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\Doctrine\\DoctrineArgument' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/Doctrine/DoctrineArgument.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\Doctrine\\DoctrineArray' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/Doctrine/DoctrineArray.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\Doctrine\\DoctrineArrayItem' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/Doctrine/DoctrineArrayItem.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\Doctrine\\DoctrineTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/Doctrine/DoctrineTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\ExtendsTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/ExtendsTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\GenericTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/GenericTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\ImplementsTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/ImplementsTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\InvalidTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/InvalidTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\MethodTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/MethodTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\MethodTagValueParameterNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/MethodTagValueParameterNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\MixinTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/MixinTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\ParamClosureThisTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/ParamClosureThisTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\ParamImmediatelyInvokedCallableTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/ParamImmediatelyInvokedCallableTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\ParamLaterInvokedCallableTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/ParamLaterInvokedCallableTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\ParamOutTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/ParamOutTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\ParamTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/ParamTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\PhpDocChildNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/PhpDocChildNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\PhpDocNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/PhpDocNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\PhpDocTagNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/PhpDocTagNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\PhpDocTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/PhpDocTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\PhpDocTextNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/PhpDocTextNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\PropertyTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/PropertyTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\PureUnlessCallableIsImpureTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/PureUnlessCallableIsImpureTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\RequireExtendsTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/RequireExtendsTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\RequireImplementsTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/RequireImplementsTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\ReturnTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/ReturnTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\SealedTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/SealedTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\SelfOutTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/SelfOutTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\TemplateTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/TemplateTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\ThrowsTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/ThrowsTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\TypeAliasImportTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/TypeAliasImportTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\TypeAliasTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/TypeAliasTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\TypelessParamTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/TypelessParamTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\UsesTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/UsesTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\PhpDoc\\VarTagValueNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/PhpDoc/VarTagValueNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\ArrayShapeItemNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/ArrayShapeItemNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\ArrayShapeNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/ArrayShapeNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\ArrayShapeUnsealedTypeNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/ArrayShapeUnsealedTypeNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\ArrayTypeNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/ArrayTypeNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\CallableTypeNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/CallableTypeNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\CallableTypeParameterNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/CallableTypeParameterNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\ConditionalTypeForParameterNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/ConditionalTypeForParameterNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\ConditionalTypeNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/ConditionalTypeNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\ConstTypeNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/ConstTypeNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\GenericTypeNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/GenericTypeNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\IdentifierTypeNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/IdentifierTypeNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\IntersectionTypeNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/IntersectionTypeNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\InvalidTypeNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/InvalidTypeNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\NullableTypeNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/NullableTypeNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\ObjectShapeItemNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/ObjectShapeItemNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\ObjectShapeNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/ObjectShapeNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\OffsetAccessTypeNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/OffsetAccessTypeNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\ThisTypeNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/ThisTypeNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\TypeNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/TypeNode.php',
    'PHPStan\\PhpDocParser\\Ast\\Type\\UnionTypeNode' => $vendorDir . '/phpstan/phpdoc-parser/src/Ast/Type/UnionTypeNode.php',
    'PHPStan\\PhpDocParser\\Lexer\\Lexer' => $vendorDir . '/phpstan/phpdoc-parser/src/Lexer/Lexer.php',
    'PHPStan\\PhpDocParser\\ParserConfig' => $vendorDir . '/phpstan/phpdoc-parser/src/ParserConfig.php',
    'PHPStan\\PhpDocParser\\Parser\\ConstExprParser' => $vendorDir . '/phpstan/phpdoc-parser/src/Parser/ConstExprParser.php',
    'PHPStan\\PhpDocParser\\Parser\\ParserException' => $vendorDir . '/phpstan/phpdoc-parser/src/Parser/ParserException.php',
    'PHPStan\\PhpDocParser\\Parser\\PhpDocParser' => $vendorDir . '/phpstan/phpdoc-parser/src/Parser/PhpDocParser.php',
    'PHPStan\\PhpDocParser\\Parser\\StringUnescaper' => $vendorDir . '/phpstan/phpdoc-parser/src/Parser/StringUnescaper.php',
    'PHPStan\\PhpDocParser\\Parser\\TokenIterator' => $vendorDir . '/phpstan/phpdoc-parser/src/Parser/TokenIterator.php',
    'PHPStan\\PhpDocParser\\Parser\\TypeParser' => $vendorDir . '/phpstan/phpdoc-parser/src/Parser/TypeParser.php',
    'PHPStan\\PhpDocParser\\Printer\\DiffElem' => $vendorDir . '/phpstan/phpdoc-parser/src/Printer/DiffElem.php',
    'PHPStan\\PhpDocParser\\Printer\\Differ' => $vendorDir . '/phpstan/phpdoc-parser/src/Printer/Differ.php',
    'PHPStan\\PhpDocParser\\Printer\\Printer' => $vendorDir . '/phpstan/phpdoc-parser/src/Printer/Printer.php',
    'ParagonIE\\Sodium\\Compat' => $vendorDir . '/paragonie/sodium_compat/namespaced/Compat.php',
    'ParagonIE\\Sodium\\Core\\BLAKE2b' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/BLAKE2b.php',
    'ParagonIE\\Sodium\\Core\\ChaCha20' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/ChaCha20.php',
    'ParagonIE\\Sodium\\Core\\ChaCha20\\Ctx' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/ChaCha20/Ctx.php',
    'ParagonIE\\Sodium\\Core\\ChaCha20\\IetfCtx' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/ChaCha20/IetfCtx.php',
    'ParagonIE\\Sodium\\Core\\Curve25519' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/Curve25519.php',
    'ParagonIE\\Sodium\\Core\\Curve25519\\Fe' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/Curve25519/Fe.php',
    'ParagonIE\\Sodium\\Core\\Curve25519\\Ge\\Cached' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/Curve25519/Ge/Cached.php',
    'ParagonIE\\Sodium\\Core\\Curve25519\\Ge\\P1p1' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/Curve25519/Ge/P1p1.php',
    'ParagonIE\\Sodium\\Core\\Curve25519\\Ge\\P2' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/Curve25519/Ge/P2.php',
    'ParagonIE\\Sodium\\Core\\Curve25519\\Ge\\P3' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/Curve25519/Ge/P3.php',
    'ParagonIE\\Sodium\\Core\\Curve25519\\Ge\\Precomp' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/Curve25519/Ge/Precomp.php',
    'ParagonIE\\Sodium\\Core\\Curve25519\\H' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/Curve25519/H.php',
    'ParagonIE\\Sodium\\Core\\Ed25519' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/Ed25519.php',
    'ParagonIE\\Sodium\\Core\\HChaCha20' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/HChaCha20.php',
    'ParagonIE\\Sodium\\Core\\HSalsa20' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/HSalsa20.php',
    'ParagonIE\\Sodium\\Core\\Poly1305' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/Poly1305.php',
    'ParagonIE\\Sodium\\Core\\Poly1305\\State' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/Poly1305/State.php',
    'ParagonIE\\Sodium\\Core\\Salsa20' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/Salsa20.php',
    'ParagonIE\\Sodium\\Core\\SipHash' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/SipHash.php',
    'ParagonIE\\Sodium\\Core\\Util' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/Util.php',
    'ParagonIE\\Sodium\\Core\\X25519' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/X25519.php',
    'ParagonIE\\Sodium\\Core\\XChaCha20' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/XChaCha20.php',
    'ParagonIE\\Sodium\\Core\\Xsalsa20' => $vendorDir . '/paragonie/sodium_compat/namespaced/Core/Xsalsa20.php',
    'ParagonIE\\Sodium\\Crypto' => $vendorDir . '/paragonie/sodium_compat/namespaced/Crypto.php',
    'ParagonIE\\Sodium\\File' => $vendorDir . '/paragonie/sodium_compat/namespaced/File.php',
    'PassiveDCEquipment' => $baseDir . '/src/PassiveDCEquipment.php',
    'PassiveDCEquipmentModel' => $baseDir . '/src/PassiveDCEquipmentModel.php',
    'PassiveDCEquipmentType' => $baseDir . '/src/PassiveDCEquipmentType.php',
    'PasswordHistory' => $baseDir . '/src/PasswordHistory.php',
    'Pdu_Plug' => $baseDir . '/src/Pdu_Plug.php',
    'PendingReason' => $baseDir . '/src/PendingReason.php',
    'PendingReasonCron' => $baseDir . '/src/PendingReasonCron.php',
    'PendingReason_Item' => $baseDir . '/src/PendingReason_Item.php',
    'Peripheral' => $baseDir . '/src/Peripheral.php',
    'PeripheralModel' => $baseDir . '/src/PeripheralModel.php',
    'PeripheralType' => $baseDir . '/src/PeripheralType.php',
    'Phone' => $baseDir . '/src/Phone.php',
    'PhoneModel' => $baseDir . '/src/PhoneModel.php',
    'PhonePowerSupply' => $baseDir . '/src/PhonePowerSupply.php',
    'PhoneType' => $baseDir . '/src/PhoneType.php',
    'PhpLang\\ScopeExit' => $vendorDir . '/phplang/scope-exit/src/ScopeExit.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\ArrayEnabled' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/ArrayEnabled.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\BinaryComparison' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/BinaryComparison.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Calculation' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Calculation.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\CalculationBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/CalculationBase.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\CalculationLocale' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/CalculationLocale.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Category' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Category.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DAverage' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DAverage.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DCount' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DCount.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DCountA' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DCountA.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DGet' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DGet.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DMax' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DMax.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DMin' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DMin.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DProduct' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DProduct.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DStDev' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DStDev.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DStDevP' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DStDevP.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DSum' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DSum.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DVar' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DVar.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DVarP' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DVarP.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Database\\DatabaseAbstract' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Database/DatabaseAbstract.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Constants' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Constants.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Current' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Current.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Date' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Date.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\DateParts' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/DateParts.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\DateValue' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/DateValue.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Days' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Days.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Days360' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Days360.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Difference' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Difference.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Helpers' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Helpers.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Month' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Month.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\NetworkDays' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/NetworkDays.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Time' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Time.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\TimeParts' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/TimeParts.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\TimeValue' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/TimeValue.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\Week' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/Week.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\WorkDay' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/WorkDay.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\DateTimeExcel\\YearFrac' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/DateTimeExcel/YearFrac.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engine\\ArrayArgumentHelper' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engine/ArrayArgumentHelper.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engine\\ArrayArgumentProcessor' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engine/ArrayArgumentProcessor.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engine\\BranchPruner' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engine/BranchPruner.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engine\\CyclicReferenceStack' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engine/CyclicReferenceStack.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engine\\FormattedNumber' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engine/FormattedNumber.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engine\\Logger' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engine/Logger.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engine\\Operands\\Operand' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engine/Operands/Operand.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engine\\Operands\\StructuredReference' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engine/Operands/StructuredReference.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\BesselI' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/BesselI.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\BesselJ' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/BesselJ.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\BesselK' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/BesselK.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\BesselY' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/BesselY.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\BitWise' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/BitWise.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\Compare' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/Compare.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\Complex' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/Complex.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\ComplexFunctions' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/ComplexFunctions.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\ComplexOperations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/ComplexOperations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\Constants' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/Constants.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\ConvertBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/ConvertBase.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\ConvertBinary' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/ConvertBinary.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\ConvertDecimal' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/ConvertDecimal.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\ConvertHex' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/ConvertHex.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\ConvertOctal' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/ConvertOctal.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\ConvertUOM' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/ConvertUOM.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\EngineeringValidations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/EngineeringValidations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\Erf' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/Erf.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Engineering\\ErfC' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Engineering/ErfC.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Exception' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Exception.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\ExceptionHandler' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/ExceptionHandler.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Amortization' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Amortization.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\CashFlow\\CashFlowValidations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/CashFlow/CashFlowValidations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\CashFlow\\Constant\\Periodic' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/CashFlow/Constant/Periodic.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\CashFlow\\Constant\\Periodic\\Cumulative' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/CashFlow/Constant/Periodic/Cumulative.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\CashFlow\\Constant\\Periodic\\Interest' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/CashFlow/Constant/Periodic/Interest.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\CashFlow\\Constant\\Periodic\\InterestAndPrincipal' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/CashFlow/Constant/Periodic/InterestAndPrincipal.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\CashFlow\\Constant\\Periodic\\Payments' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/CashFlow/Constant/Periodic/Payments.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\CashFlow\\Single' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/CashFlow/Single.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\CashFlow\\Variable\\NonPeriodic' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/CashFlow/Variable/NonPeriodic.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\CashFlow\\Variable\\Periodic' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/CashFlow/Variable/Periodic.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Constants' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Constants.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Coupons' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Coupons.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Depreciation' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Depreciation.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Dollar' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Dollar.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\FinancialValidations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/FinancialValidations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Helpers' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Helpers.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\InterestRate' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/InterestRate.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Securities\\AccruedInterest' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Securities/AccruedInterest.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Securities\\Price' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Securities/Price.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Securities\\Rates' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Securities/Rates.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Securities\\SecurityValidations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Securities/SecurityValidations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\Securities\\Yields' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/Securities/Yields.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Financial\\TreasuryBill' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Financial/TreasuryBill.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\FormulaParser' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/FormulaParser.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\FormulaToken' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/FormulaToken.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\FunctionArray' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/FunctionArray.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Functions' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Functions.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Information\\ErrorValue' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Information/ErrorValue.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Information\\ExcelError' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Information/ExcelError.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Information\\Value' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Information/Value.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Internal\\ExcelArrayPseudoFunctions' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Internal/ExcelArrayPseudoFunctions.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Internal\\MakeMatrix' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Internal/MakeMatrix.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Internal\\WildcardMatch' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Internal/WildcardMatch.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Logical\\Boolean' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Logical/Boolean.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Logical\\Conditional' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Logical/Conditional.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Logical\\Operations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Logical/Operations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Address' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Address.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\ChooseRowsEtc' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/ChooseRowsEtc.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\ExcelMatch' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/ExcelMatch.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Filter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Filter.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Formula' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Formula.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\HLookup' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/HLookup.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Helpers' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Helpers.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Hstack' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Hstack.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Hyperlink' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Hyperlink.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Indirect' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Indirect.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Lookup' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Lookup.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\LookupBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/LookupBase.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\LookupRefValidations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/LookupRefValidations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Matrix' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Matrix.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Offset' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Offset.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\RowColumnInformation' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/RowColumnInformation.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Selection' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Selection.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Sort' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Sort.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\TorowTocol' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/TorowTocol.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Unique' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Unique.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\VLookup' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/VLookup.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\LookupRef\\Vstack' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/LookupRef/Vstack.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Absolute' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Absolute.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Angle' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Angle.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Arabic' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Arabic.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Base' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Base.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Ceiling' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Ceiling.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Combinations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Combinations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Exp' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Exp.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Factorial' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Factorial.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Floor' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Floor.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Gcd' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Gcd.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Helpers' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Helpers.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\IntClass' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/IntClass.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Lcm' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Lcm.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Logarithms' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Logarithms.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\MatrixFunctions' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/MatrixFunctions.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Operations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Operations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Random' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Random.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Roman' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Roman.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Round' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Round.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\SeriesSum' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/SeriesSum.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Sign' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Sign.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Sqrt' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Sqrt.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Subtotal' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Subtotal.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Sum' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Sum.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\SumSquares' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/SumSquares.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Trig\\Cosecant' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Trig/Cosecant.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Trig\\Cosine' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Trig/Cosine.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Trig\\Cotangent' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Trig/Cotangent.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Trig\\Secant' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Trig/Secant.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Trig\\Sine' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Trig/Sine.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Trig\\Tangent' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Trig/Tangent.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\MathTrig\\Trunc' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/MathTrig/Trunc.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\AggregateBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/AggregateBase.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Averages' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Averages.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Averages\\Mean' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Averages/Mean.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Conditional' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Conditional.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Confidence' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Confidence.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Counts' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Counts.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Deviations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Deviations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\Beta' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/Beta.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\Binomial' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/Binomial.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\ChiSquared' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/ChiSquared.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\DistributionValidations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/DistributionValidations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\Exponential' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/Exponential.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\F' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/F.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\Fisher' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/Fisher.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\Gamma' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/Gamma.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\GammaBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/GammaBase.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\HyperGeometric' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/HyperGeometric.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\LogNormal' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/LogNormal.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\NewtonRaphson' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/NewtonRaphson.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\Normal' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/Normal.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\Poisson' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/Poisson.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\StandardNormal' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/StandardNormal.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\StudentT' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/StudentT.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Distributions\\Weibull' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Distributions/Weibull.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\MaxMinBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/MaxMinBase.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Maximum' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Maximum.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Minimum' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Minimum.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Percentiles' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Percentiles.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Permutations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Permutations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Size' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Size.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\StandardDeviations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/StandardDeviations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Standardize' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Standardize.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\StatisticalValidations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/StatisticalValidations.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Trends' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Trends.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\VarianceBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/VarianceBase.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Statistical\\Variances' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Statistical/Variances.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\CaseConvert' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/CaseConvert.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\CharacterConvert' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/CharacterConvert.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\Concatenate' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/Concatenate.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\Extract' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/Extract.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\Format' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/Format.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\Helpers' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/Helpers.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\Replace' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/Replace.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\Search' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/Search.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\Text' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/Text.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\TextData\\Trim' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/TextData/Trim.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Token\\Stack' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Token/Stack.php',
    'PhpOffice\\PhpSpreadsheet\\Calculation\\Web\\Service' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Calculation/Web/Service.php',
    'PhpOffice\\PhpSpreadsheet\\CellReferenceHelper' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/CellReferenceHelper.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\AddressHelper' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/AddressHelper.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\AddressRange' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/AddressRange.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\AdvancedValueBinder' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/AdvancedValueBinder.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\Cell' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/Cell.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\CellAddress' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/CellAddress.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\CellRange' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/CellRange.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\ColumnRange' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/ColumnRange.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\Coordinate' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/Coordinate.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\DataType' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/DataType.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\DataValidation' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/DataValidation.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\DataValidator' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/DataValidator.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\DefaultValueBinder' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/DefaultValueBinder.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\Hyperlink' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/Hyperlink.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\IValueBinder' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/IValueBinder.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\IgnoredErrors' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/IgnoredErrors.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\RowRange' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/RowRange.php',
    'PhpOffice\\PhpSpreadsheet\\Cell\\StringValueBinder' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Cell/StringValueBinder.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Axis' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Axis.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\AxisText' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/AxisText.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Chart' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Chart.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\ChartColor' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/ChartColor.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\DataSeries' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/DataSeries.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\DataSeriesValues' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/DataSeriesValues.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Exception' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Exception.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\GridLines' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/GridLines.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Layout' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Layout.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Legend' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Legend.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\PlotArea' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/PlotArea.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Properties' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Properties.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Renderer\\IRenderer' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Renderer/IRenderer.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Renderer\\JpGraph' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Renderer/JpGraph.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Renderer\\JpGraphRendererBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Renderer/JpGraphRendererBase.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Renderer\\MtJpGraphRenderer' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Renderer/MtJpGraphRenderer.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\Title' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/Title.php',
    'PhpOffice\\PhpSpreadsheet\\Chart\\TrendLine' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Chart/TrendLine.php',
    'PhpOffice\\PhpSpreadsheet\\Collection\\Cells' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Collection/Cells.php',
    'PhpOffice\\PhpSpreadsheet\\Collection\\CellsFactory' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Collection/CellsFactory.php',
    'PhpOffice\\PhpSpreadsheet\\Collection\\Memory\\SimpleCache1' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Collection/Memory/SimpleCache1.php',
    'PhpOffice\\PhpSpreadsheet\\Collection\\Memory\\SimpleCache3' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Collection/Memory/SimpleCache3.php',
    'PhpOffice\\PhpSpreadsheet\\Comment' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Comment.php',
    'PhpOffice\\PhpSpreadsheet\\DefinedName' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/DefinedName.php',
    'PhpOffice\\PhpSpreadsheet\\Document\\Properties' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Document/Properties.php',
    'PhpOffice\\PhpSpreadsheet\\Document\\Security' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Document/Security.php',
    'PhpOffice\\PhpSpreadsheet\\Exception' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Exception.php',
    'PhpOffice\\PhpSpreadsheet\\HashTable' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/HashTable.php',
    'PhpOffice\\PhpSpreadsheet\\Helper\\Dimension' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Helper/Dimension.php',
    'PhpOffice\\PhpSpreadsheet\\Helper\\Downloader' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Helper/Downloader.php',
    'PhpOffice\\PhpSpreadsheet\\Helper\\Handler' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Helper/Handler.php',
    'PhpOffice\\PhpSpreadsheet\\Helper\\Html' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Helper/Html.php',
    'PhpOffice\\PhpSpreadsheet\\Helper\\Sample' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Helper/Sample.php',
    'PhpOffice\\PhpSpreadsheet\\Helper\\Size' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Helper/Size.php',
    'PhpOffice\\PhpSpreadsheet\\Helper\\TextGrid' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Helper/TextGrid.php',
    'PhpOffice\\PhpSpreadsheet\\IComparable' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/IComparable.php',
    'PhpOffice\\PhpSpreadsheet\\IOFactory' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/IOFactory.php',
    'PhpOffice\\PhpSpreadsheet\\NamedFormula' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/NamedFormula.php',
    'PhpOffice\\PhpSpreadsheet\\NamedRange' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/NamedRange.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\BaseReader' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/BaseReader.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Csv' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Csv.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Csv\\Delimiter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Csv/Delimiter.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\DefaultReadFilter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/DefaultReadFilter.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Exception' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Exception.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Gnumeric' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Gnumeric.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Gnumeric\\PageSetup' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Gnumeric/PageSetup.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Gnumeric\\Properties' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Gnumeric/Properties.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Gnumeric\\Styles' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Gnumeric/Styles.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Html' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Html.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\IReadFilter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/IReadFilter.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\IReader' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/IReader.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Ods' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Ods.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Ods\\AutoFilter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Ods/AutoFilter.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Ods\\BaseLoader' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Ods/BaseLoader.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Ods\\DefinedNames' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Ods/DefinedNames.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Ods\\FormulaTranslator' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Ods/FormulaTranslator.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Ods\\PageSettings' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Ods/PageSettings.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Ods\\Properties' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Ods/Properties.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Security\\XmlScanner' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Security/XmlScanner.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Slk' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Slk.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\XlsBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/XlsBase.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Biff5' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Biff5.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Biff8' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Biff8.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Color' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Color.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Color\\BIFF5' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Color/BIFF5.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Color\\BIFF8' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Color/BIFF8.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Color\\BuiltIn' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Color/BuiltIn.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\ConditionalFormatting' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/ConditionalFormatting.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\DataValidationHelper' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/DataValidationHelper.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\ErrorCode' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/ErrorCode.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Escher' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Escher.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\ListFunctions' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/ListFunctions.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\LoadSpreadsheet' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/LoadSpreadsheet.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\MD5' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/MD5.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Mappings' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Mappings.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\RC4' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/RC4.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Style\\Border' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Style/Border.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Style\\CellAlignment' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Style/CellAlignment.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Style\\CellFont' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Style/CellFont.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xls\\Style\\FillPattern' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xls/Style/FillPattern.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\AutoFilter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/AutoFilter.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\BaseParserClass' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/BaseParserClass.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\Chart' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/Chart.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\ColumnAndRowAttributes' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/ColumnAndRowAttributes.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\ConditionalStyles' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/ConditionalStyles.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\DataValidations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/DataValidations.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\Hyperlinks' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/Hyperlinks.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\Namespaces' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/Namespaces.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\PageSetup' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/PageSetup.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\Properties' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/Properties.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\SharedFormula' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/SharedFormula.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\SheetViewOptions' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/SheetViewOptions.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\SheetViews' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/SheetViews.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\Styles' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/Styles.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\TableReader' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/TableReader.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\Theme' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/Theme.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xlsx\\WorkbookView' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xlsx/WorkbookView.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\DataValidations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/DataValidations.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\PageSettings' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/PageSettings.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\Properties' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/Properties.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\Style' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/Style.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\Style\\Alignment' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/Style/Alignment.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\Style\\Border' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/Style/Border.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\Style\\Fill' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/Style/Fill.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\Style\\Font' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/Style/Font.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\Style\\NumberFormat' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/Style/NumberFormat.php',
    'PhpOffice\\PhpSpreadsheet\\Reader\\Xml\\Style\\StyleBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Reader/Xml/Style/StyleBase.php',
    'PhpOffice\\PhpSpreadsheet\\ReferenceHelper' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/ReferenceHelper.php',
    'PhpOffice\\PhpSpreadsheet\\RichText\\ITextElement' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/RichText/ITextElement.php',
    'PhpOffice\\PhpSpreadsheet\\RichText\\RichText' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/RichText/RichText.php',
    'PhpOffice\\PhpSpreadsheet\\RichText\\Run' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/RichText/Run.php',
    'PhpOffice\\PhpSpreadsheet\\RichText\\TextElement' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/RichText/TextElement.php',
    'PhpOffice\\PhpSpreadsheet\\Settings' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Settings.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\CodePage' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/CodePage.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Date' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Date.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Drawing' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Drawing.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Escher' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Escher.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Escher\\DgContainer' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Escher/DgContainer.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Escher\\DgContainer\\SpgrContainer' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Escher/DgContainer/SpgrContainer.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Escher\\DgContainer\\SpgrContainer\\SpContainer' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Escher/DgContainer/SpgrContainer/SpContainer.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Escher\\DggContainer' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Escher/DggContainer.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Escher\\DggContainer\\BstoreContainer' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Escher/DggContainer/BstoreContainer.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Escher\\DggContainer\\BstoreContainer\\BSE' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Escher/DggContainer/BstoreContainer/BSE.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Escher\\DggContainer\\BstoreContainer\\BSE\\Blip' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Escher/DggContainer/BstoreContainer/BSE/Blip.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\File' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/File.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Font' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Font.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\IntOrFloat' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/IntOrFloat.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\OLE' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/OLE.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\OLERead' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/OLERead.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\OLE\\ChainedBlockStream' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/OLE/ChainedBlockStream.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\OLE\\PPS' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/OLE/PPS.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\OLE\\PPS\\File' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/OLE/PPS/File.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\OLE\\PPS\\Root' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/OLE/PPS/Root.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\PasswordHasher' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/PasswordHasher.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\StringHelper' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/StringHelper.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\TimeZone' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/TimeZone.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Trend\\BestFit' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Trend/BestFit.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Trend\\ExponentialBestFit' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Trend/ExponentialBestFit.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Trend\\LinearBestFit' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Trend/LinearBestFit.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Trend\\LogarithmicBestFit' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Trend/LogarithmicBestFit.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Trend\\PolynomialBestFit' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Trend/PolynomialBestFit.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Trend\\PowerBestFit' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Trend/PowerBestFit.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Trend\\Trend' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Trend/Trend.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\XMLWriter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/XMLWriter.php',
    'PhpOffice\\PhpSpreadsheet\\Shared\\Xls' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Shared/Xls.php',
    'PhpOffice\\PhpSpreadsheet\\Spreadsheet' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Spreadsheet.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Alignment' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Alignment.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Border' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Border.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Borders' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Borders.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Color' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Color.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Conditional' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Conditional.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\CellMatcher' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/CellMatcher.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\CellStyleAssessor' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/CellStyleAssessor.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\ConditionalColorScale' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/ConditionalColorScale.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\ConditionalDataBar' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/ConditionalDataBar.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\ConditionalDataBarExtension' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/ConditionalDataBarExtension.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\ConditionalFormatValueObject' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/ConditionalFormatValueObject.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\ConditionalFormattingRuleExtension' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/ConditionalFormattingRuleExtension.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\ConditionalIconSet' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/ConditionalIconSet.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\IconSetValues' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/IconSetValues.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\StyleMerger' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/StyleMerger.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard\\Blanks' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard/Blanks.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard\\CellValue' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard/CellValue.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard\\DateValue' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard/DateValue.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard\\Duplicates' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard/Duplicates.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard\\Errors' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard/Errors.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard\\Expression' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard/Expression.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard\\TextValue' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard/TextValue.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard\\WizardAbstract' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard/WizardAbstract.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\ConditionalFormatting\\Wizard\\WizardInterface' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/ConditionalFormatting/Wizard/WizardInterface.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Fill' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Fill.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Font' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Font.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\BaseFormatter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/BaseFormatter.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\DateFormatter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/DateFormatter.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Formatter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Formatter.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\FractionFormatter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/FractionFormatter.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\NumberFormatter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/NumberFormatter.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\PercentageFormatter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/PercentageFormatter.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Accounting' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Accounting.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Currency' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Currency.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\CurrencyBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/CurrencyBase.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\CurrencyNegative' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/CurrencyNegative.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Date' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Date.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\DateTime' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/DateTime.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\DateTimeWizard' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/DateTimeWizard.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Duration' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Duration.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Locale' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Locale.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Number' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Number.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\NumberBase' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/NumberBase.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Percentage' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Percentage.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Scientific' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Scientific.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Time' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Time.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\NumberFormat\\Wizard\\Wizard' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/NumberFormat/Wizard/Wizard.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Protection' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Protection.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\RgbTint' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/RgbTint.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Style' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Style.php',
    'PhpOffice\\PhpSpreadsheet\\Style\\Supervisor' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Style/Supervisor.php',
    'PhpOffice\\PhpSpreadsheet\\Theme' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Theme.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\AutoFilter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/AutoFilter.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\AutoFilter\\Column' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/AutoFilter/Column.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\AutoFilter\\Column\\Rule' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/AutoFilter/Column/Rule.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\AutoFit' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/AutoFit.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\BaseDrawing' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/BaseDrawing.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\CellIterator' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/CellIterator.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Column' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Column.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\ColumnCellIterator' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/ColumnCellIterator.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\ColumnDimension' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/ColumnDimension.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\ColumnIterator' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/ColumnIterator.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Dimension' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Dimension.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Drawing' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Drawing.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Drawing\\Shadow' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Drawing/Shadow.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\HeaderFooter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/HeaderFooter.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\HeaderFooterDrawing' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/HeaderFooterDrawing.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Iterator' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Iterator.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\MemoryDrawing' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/MemoryDrawing.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\PageBreak' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/PageBreak.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\PageMargins' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/PageMargins.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\PageSetup' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/PageSetup.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Pane' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Pane.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\ProtectedRange' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/ProtectedRange.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Protection' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Protection.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Row' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Row.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\RowCellIterator' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/RowCellIterator.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\RowDimension' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/RowDimension.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\RowIterator' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/RowIterator.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\SheetView' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/SheetView.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Table' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Table.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Table\\Column' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Table/Column.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Table\\TableDxfsStyle' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Table/TableDxfsStyle.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Table\\TableStyle' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Table/TableStyle.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Validations' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Validations.php',
    'PhpOffice\\PhpSpreadsheet\\Worksheet\\Worksheet' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Worksheet/Worksheet.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\BaseWriter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/BaseWriter.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Csv' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Csv.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Exception' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Exception.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Html' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Html.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\IWriter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/IWriter.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\AutoFilters' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/AutoFilters.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\Cell\\Comment' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/Cell/Comment.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\Cell\\Style' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/Cell/Style.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\Content' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/Content.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\Formula' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/Formula.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\Meta' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/Meta.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\MetaInf' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/MetaInf.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\Mimetype' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/Mimetype.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\NamedExpressions' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/NamedExpressions.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\Settings' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/Settings.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\Styles' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/Styles.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\Thumbnails' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/Thumbnails.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Ods\\WriterPart' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Ods/WriterPart.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Pdf' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Pdf.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Pdf\\Dompdf' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Pdf/Dompdf.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Pdf\\Mpdf' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Pdf/Mpdf.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Pdf\\Tcpdf' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Pdf/Tcpdf.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\BIFFwriter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/BIFFwriter.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\CellDataValidation' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/CellDataValidation.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\ConditionalHelper' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/ConditionalHelper.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\ErrorCode' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/ErrorCode.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Escher' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Escher.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Font' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Font.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Parser' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Parser.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Style\\CellAlignment' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Style/CellAlignment.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Style\\CellBorder' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Style/CellBorder.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Style\\CellFill' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Style/CellFill.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Workbook' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Workbook.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Worksheet' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Worksheet.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xls\\Xf' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xls/Xf.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\AutoFilter' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/AutoFilter.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Chart' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Chart.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Comments' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Comments.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\ContentTypes' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/ContentTypes.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\DefinedNames' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/DefinedNames.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\DocProps' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/DocProps.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Drawing' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Drawing.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\FunctionPrefix' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/FunctionPrefix.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Metadata' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Metadata.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Rels' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Rels.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\RelsRibbon' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/RelsRibbon.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\RelsVBA' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/RelsVBA.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\StringTable' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/StringTable.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Style' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Style.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Table' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Table.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Theme' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Theme.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Workbook' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Workbook.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\Worksheet' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/Worksheet.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\Xlsx\\WriterPart' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/Xlsx/WriterPart.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\ZipStream0' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/ZipStream0.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\ZipStream2' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/ZipStream2.php',
    'PhpOffice\\PhpSpreadsheet\\Writer\\ZipStream3' => $vendorDir . '/phpoffice/phpspreadsheet/src/PhpSpreadsheet/Writer/ZipStream3.php',
    'Planning' => $baseDir . '/src/Planning.php',
    'PlanningEventCategory' => $baseDir . '/src/PlanningEventCategory.php',
    'PlanningExternalEvent' => $baseDir . '/src/PlanningExternalEvent.php',
    'PlanningExternalEventTemplate' => $baseDir . '/src/PlanningExternalEventTemplate.php',
    'PlanningRecall' => $baseDir . '/src/PlanningRecall.php',
    'Plug' => $baseDir . '/src/Plug.php',
    'Plugin' => $baseDir . '/src/Plugin.php',
    'Preference' => $baseDir . '/src/Preference.php',
    'Printer' => $baseDir . '/src/Printer.php',
    'PrinterLog' => $baseDir . '/src/PrinterLog.php',
    'PrinterModel' => $baseDir . '/src/PrinterModel.php',
    'PrinterType' => $baseDir . '/src/PrinterType.php',
    'Printer_CartridgeInfo' => $baseDir . '/src/Printer_CartridgeInfo.php',
    'Problem' => $baseDir . '/src/Problem.php',
    'ProblemCost' => $baseDir . '/src/ProblemCost.php',
    'ProblemTask' => $baseDir . '/src/ProblemTask.php',
    'ProblemTemplate' => $baseDir . '/src/ProblemTemplate.php',
    'ProblemTemplateHiddenField' => $baseDir . '/src/ProblemTemplateHiddenField.php',
    'ProblemTemplateMandatoryField' => $baseDir . '/src/ProblemTemplateMandatoryField.php',
    'ProblemTemplatePredefinedField' => $baseDir . '/src/ProblemTemplatePredefinedField.php',
    'ProblemTemplateReadonlyField' => $baseDir . '/src/ProblemTemplateReadonlyField.php',
    'Problem_Problem' => $baseDir . '/src/Problem_Problem.php',
    'Problem_Supplier' => $baseDir . '/src/Problem_Supplier.php',
    'Problem_Ticket' => $baseDir . '/src/Problem_Ticket.php',
    'Problem_User' => $baseDir . '/src/Problem_User.php',
    'Profile' => $baseDir . '/src/Profile.php',
    'ProfileRight' => $baseDir . '/src/ProfileRight.php',
    'Profile_RSSFeed' => $baseDir . '/src/Profile_RSSFeed.php',
    'Profile_Reminder' => $baseDir . '/src/Profile_Reminder.php',
    'Profile_User' => $baseDir . '/src/Profile_User.php',
    'Project' => $baseDir . '/src/Project.php',
    'ProjectCost' => $baseDir . '/src/ProjectCost.php',
    'ProjectState' => $baseDir . '/src/ProjectState.php',
    'ProjectTask' => $baseDir . '/src/ProjectTask.php',
    'ProjectTaskLink' => $baseDir . '/src/ProjectTaskLink.php',
    'ProjectTaskTeam' => $baseDir . '/src/ProjectTaskTeam.php',
    'ProjectTaskTeamDropdown' => $baseDir . '/src/ProjectTaskTeamDropdown.php',
    'ProjectTaskTemplate' => $baseDir . '/src/ProjectTaskTemplate.php',
    'ProjectTaskType' => $baseDir . '/src/ProjectTaskType.php',
    'ProjectTask_Ticket' => $baseDir . '/src/ProjectTask_Ticket.php',
    'ProjectTeam' => $baseDir . '/src/ProjectTeam.php',
    'ProjectType' => $baseDir . '/src/ProjectType.php',
    'Psr\\Cache\\CacheException' => $vendorDir . '/psr/cache/src/CacheException.php',
    'Psr\\Cache\\CacheItemInterface' => $vendorDir . '/psr/cache/src/CacheItemInterface.php',
    'Psr\\Cache\\CacheItemPoolInterface' => $vendorDir . '/psr/cache/src/CacheItemPoolInterface.php',
    'Psr\\Cache\\InvalidArgumentException' => $vendorDir . '/psr/cache/src/InvalidArgumentException.php',
    'Psr\\Clock\\ClockInterface' => $vendorDir . '/psr/clock/src/ClockInterface.php',
    'Psr\\Container\\ContainerExceptionInterface' => $vendorDir . '/psr/container/src/ContainerExceptionInterface.php',
    'Psr\\Container\\ContainerInterface' => $vendorDir . '/psr/container/src/ContainerInterface.php',
    'Psr\\Container\\NotFoundExceptionInterface' => $vendorDir . '/psr/container/src/NotFoundExceptionInterface.php',
    'Psr\\EventDispatcher\\EventDispatcherInterface' => $vendorDir . '/psr/event-dispatcher/src/EventDispatcherInterface.php',
    'Psr\\EventDispatcher\\ListenerProviderInterface' => $vendorDir . '/psr/event-dispatcher/src/ListenerProviderInterface.php',
    'Psr\\EventDispatcher\\StoppableEventInterface' => $vendorDir . '/psr/event-dispatcher/src/StoppableEventInterface.php',
    'Psr\\Http\\Client\\ClientExceptionInterface' => $vendorDir . '/psr/http-client/src/ClientExceptionInterface.php',
    'Psr\\Http\\Client\\ClientInterface' => $vendorDir . '/psr/http-client/src/ClientInterface.php',
    'Psr\\Http\\Client\\NetworkExceptionInterface' => $vendorDir . '/psr/http-client/src/NetworkExceptionInterface.php',
    'Psr\\Http\\Client\\RequestExceptionInterface' => $vendorDir . '/psr/http-client/src/RequestExceptionInterface.php',
    'Psr\\Http\\Message\\MessageInterface' => $vendorDir . '/psr/http-message/src/MessageInterface.php',
    'Psr\\Http\\Message\\RequestFactoryInterface' => $vendorDir . '/psr/http-factory/src/RequestFactoryInterface.php',
    'Psr\\Http\\Message\\RequestInterface' => $vendorDir . '/psr/http-message/src/RequestInterface.php',
    'Psr\\Http\\Message\\ResponseFactoryInterface' => $vendorDir . '/psr/http-factory/src/ResponseFactoryInterface.php',
    'Psr\\Http\\Message\\ResponseInterface' => $vendorDir . '/psr/http-message/src/ResponseInterface.php',
    'Psr\\Http\\Message\\ServerRequestFactoryInterface' => $vendorDir . '/psr/http-factory/src/ServerRequestFactoryInterface.php',
    'Psr\\Http\\Message\\ServerRequestInterface' => $vendorDir . '/psr/http-message/src/ServerRequestInterface.php',
    'Psr\\Http\\Message\\StreamFactoryInterface' => $vendorDir . '/psr/http-factory/src/StreamFactoryInterface.php',
    'Psr\\Http\\Message\\StreamInterface' => $vendorDir . '/psr/http-message/src/StreamInterface.php',
    'Psr\\Http\\Message\\UploadedFileFactoryInterface' => $vendorDir . '/psr/http-factory/src/UploadedFileFactoryInterface.php',
    'Psr\\Http\\Message\\UploadedFileInterface' => $vendorDir . '/psr/http-message/src/UploadedFileInterface.php',
    'Psr\\Http\\Message\\UriFactoryInterface' => $vendorDir . '/psr/http-factory/src/UriFactoryInterface.php',
    'Psr\\Http\\Message\\UriInterface' => $vendorDir . '/psr/http-message/src/UriInterface.php',
    'Psr\\Http\\Server\\MiddlewareInterface' => $vendorDir . '/psr/http-server-middleware/src/MiddlewareInterface.php',
    'Psr\\Http\\Server\\RequestHandlerInterface' => $vendorDir . '/psr/http-server-handler/src/RequestHandlerInterface.php',
    'Psr\\Log\\AbstractLogger' => $vendorDir . '/psr/log/src/AbstractLogger.php',
    'Psr\\Log\\InvalidArgumentException' => $vendorDir . '/psr/log/src/InvalidArgumentException.php',
    'Psr\\Log\\LogLevel' => $vendorDir . '/psr/log/src/LogLevel.php',
    'Psr\\Log\\LoggerAwareInterface' => $vendorDir . '/psr/log/src/LoggerAwareInterface.php',
    'Psr\\Log\\LoggerAwareTrait' => $vendorDir . '/psr/log/src/LoggerAwareTrait.php',
    'Psr\\Log\\LoggerInterface' => $vendorDir . '/psr/log/src/LoggerInterface.php',
    'Psr\\Log\\LoggerTrait' => $vendorDir . '/psr/log/src/LoggerTrait.php',
    'Psr\\Log\\NullLogger' => $vendorDir . '/psr/log/src/NullLogger.php',
    'Psr\\SimpleCache\\CacheException' => $vendorDir . '/psr/simple-cache/src/CacheException.php',
    'Psr\\SimpleCache\\CacheInterface' => $vendorDir . '/psr/simple-cache/src/CacheInterface.php',
    'Psr\\SimpleCache\\InvalidArgumentException' => $vendorDir . '/psr/simple-cache/src/InvalidArgumentException.php',
    'PurgeLogs' => $baseDir . '/src/PurgeLogs.php',
    'PurgeSoftwareTask' => $baseDir . '/src/PurgeSoftwareTask.php',
    'QRcode' => $vendorDir . '/tecnickcom/tcpdf/include/barcodes/qrcode.php',
    'QueryExpression' => $baseDir . '/src/QueryExpression.php',
    'QueryParam' => $baseDir . '/src/QueryParam.php',
    'QuerySubQuery' => $baseDir . '/src/QuerySubQuery.php',
    'QueryUnion' => $baseDir . '/src/QueryUnion.php',
    'QueuedNotification' => $baseDir . '/src/QueuedNotification.php',
    'QueuedWebhook' => $baseDir . '/src/QueuedWebhook.php',
    'RRule\\RRule' => $vendorDir . '/rlanvin/php-rrule/src/RRule.php',
    'RRule\\RRuleInterface' => $vendorDir . '/rlanvin/php-rrule/src/RRuleInterface.php',
    'RRule\\RRuleTrait' => $vendorDir . '/rlanvin/php-rrule/src/RRuleTrait.php',
    'RRule\\RSet' => $vendorDir . '/rlanvin/php-rrule/src/RSet.php',
    'RRule\\RfcParser' => $vendorDir . '/rlanvin/php-rrule/src/RfcParser.php',
    'RSSFeed' => $baseDir . '/src/RSSFeed.php',
    'RSSFeed_User' => $baseDir . '/src/RSSFeed_User.php',
    'Rack' => $baseDir . '/src/Rack.php',
    'RackModel' => $baseDir . '/src/RackModel.php',
    'RackType' => $baseDir . '/src/RackType.php',
    'Ramsey\\Collection\\AbstractArray' => $vendorDir . '/ramsey/collection/src/AbstractArray.php',
    'Ramsey\\Collection\\AbstractCollection' => $vendorDir . '/ramsey/collection/src/AbstractCollection.php',
    'Ramsey\\Collection\\AbstractSet' => $vendorDir . '/ramsey/collection/src/AbstractSet.php',
    'Ramsey\\Collection\\ArrayInterface' => $vendorDir . '/ramsey/collection/src/ArrayInterface.php',
    'Ramsey\\Collection\\Collection' => $vendorDir . '/ramsey/collection/src/Collection.php',
    'Ramsey\\Collection\\CollectionInterface' => $vendorDir . '/ramsey/collection/src/CollectionInterface.php',
    'Ramsey\\Collection\\DoubleEndedQueue' => $vendorDir . '/ramsey/collection/src/DoubleEndedQueue.php',
    'Ramsey\\Collection\\DoubleEndedQueueInterface' => $vendorDir . '/ramsey/collection/src/DoubleEndedQueueInterface.php',
    'Ramsey\\Collection\\Exception\\CollectionException' => $vendorDir . '/ramsey/collection/src/Exception/CollectionException.php',
    'Ramsey\\Collection\\Exception\\CollectionMismatchException' => $vendorDir . '/ramsey/collection/src/Exception/CollectionMismatchException.php',
    'Ramsey\\Collection\\Exception\\InvalidArgumentException' => $vendorDir . '/ramsey/collection/src/Exception/InvalidArgumentException.php',
    'Ramsey\\Collection\\Exception\\InvalidPropertyOrMethod' => $vendorDir . '/ramsey/collection/src/Exception/InvalidPropertyOrMethod.php',
    'Ramsey\\Collection\\Exception\\NoSuchElementException' => $vendorDir . '/ramsey/collection/src/Exception/NoSuchElementException.php',
    'Ramsey\\Collection\\Exception\\OutOfBoundsException' => $vendorDir . '/ramsey/collection/src/Exception/OutOfBoundsException.php',
    'Ramsey\\Collection\\Exception\\UnsupportedOperationException' => $vendorDir . '/ramsey/collection/src/Exception/UnsupportedOperationException.php',
    'Ramsey\\Collection\\GenericArray' => $vendorDir . '/ramsey/collection/src/GenericArray.php',
    'Ramsey\\Collection\\Map\\AbstractMap' => $vendorDir . '/ramsey/collection/src/Map/AbstractMap.php',
    'Ramsey\\Collection\\Map\\AbstractTypedMap' => $vendorDir . '/ramsey/collection/src/Map/AbstractTypedMap.php',
    'Ramsey\\Collection\\Map\\AssociativeArrayMap' => $vendorDir . '/ramsey/collection/src/Map/AssociativeArrayMap.php',
    'Ramsey\\Collection\\Map\\MapInterface' => $vendorDir . '/ramsey/collection/src/Map/MapInterface.php',
    'Ramsey\\Collection\\Map\\NamedParameterMap' => $vendorDir . '/ramsey/collection/src/Map/NamedParameterMap.php',
    'Ramsey\\Collection\\Map\\TypedMap' => $vendorDir . '/ramsey/collection/src/Map/TypedMap.php',
    'Ramsey\\Collection\\Map\\TypedMapInterface' => $vendorDir . '/ramsey/collection/src/Map/TypedMapInterface.php',
    'Ramsey\\Collection\\Queue' => $vendorDir . '/ramsey/collection/src/Queue.php',
    'Ramsey\\Collection\\QueueInterface' => $vendorDir . '/ramsey/collection/src/QueueInterface.php',
    'Ramsey\\Collection\\Set' => $vendorDir . '/ramsey/collection/src/Set.php',
    'Ramsey\\Collection\\Sort' => $vendorDir . '/ramsey/collection/src/Sort.php',
    'Ramsey\\Collection\\Tool\\TypeTrait' => $vendorDir . '/ramsey/collection/src/Tool/TypeTrait.php',
    'Ramsey\\Collection\\Tool\\ValueExtractorTrait' => $vendorDir . '/ramsey/collection/src/Tool/ValueExtractorTrait.php',
    'Ramsey\\Collection\\Tool\\ValueToStringTrait' => $vendorDir . '/ramsey/collection/src/Tool/ValueToStringTrait.php',
    'Ramsey\\Uuid\\BinaryUtils' => $vendorDir . '/ramsey/uuid/src/BinaryUtils.php',
    'Ramsey\\Uuid\\Builder\\BuilderCollection' => $vendorDir . '/ramsey/uuid/src/Builder/BuilderCollection.php',
    'Ramsey\\Uuid\\Builder\\DefaultUuidBuilder' => $vendorDir . '/ramsey/uuid/src/Builder/DefaultUuidBuilder.php',
    'Ramsey\\Uuid\\Builder\\DegradedUuidBuilder' => $vendorDir . '/ramsey/uuid/src/Builder/DegradedUuidBuilder.php',
    'Ramsey\\Uuid\\Builder\\FallbackBuilder' => $vendorDir . '/ramsey/uuid/src/Builder/FallbackBuilder.php',
    'Ramsey\\Uuid\\Builder\\UuidBuilderInterface' => $vendorDir . '/ramsey/uuid/src/Builder/UuidBuilderInterface.php',
    'Ramsey\\Uuid\\Codec\\CodecInterface' => $vendorDir . '/ramsey/uuid/src/Codec/CodecInterface.php',
    'Ramsey\\Uuid\\Codec\\GuidStringCodec' => $vendorDir . '/ramsey/uuid/src/Codec/GuidStringCodec.php',
    'Ramsey\\Uuid\\Codec\\OrderedTimeCodec' => $vendorDir . '/ramsey/uuid/src/Codec/OrderedTimeCodec.php',
    'Ramsey\\Uuid\\Codec\\StringCodec' => $vendorDir . '/ramsey/uuid/src/Codec/StringCodec.php',
    'Ramsey\\Uuid\\Codec\\TimestampFirstCombCodec' => $vendorDir . '/ramsey/uuid/src/Codec/TimestampFirstCombCodec.php',
    'Ramsey\\Uuid\\Codec\\TimestampLastCombCodec' => $vendorDir . '/ramsey/uuid/src/Codec/TimestampLastCombCodec.php',
    'Ramsey\\Uuid\\Converter\\NumberConverterInterface' => $vendorDir . '/ramsey/uuid/src/Converter/NumberConverterInterface.php',
    'Ramsey\\Uuid\\Converter\\Number\\BigNumberConverter' => $vendorDir . '/ramsey/uuid/src/Converter/Number/BigNumberConverter.php',
    'Ramsey\\Uuid\\Converter\\Number\\DegradedNumberConverter' => $vendorDir . '/ramsey/uuid/src/Converter/Number/DegradedNumberConverter.php',
    'Ramsey\\Uuid\\Converter\\Number\\GenericNumberConverter' => $vendorDir . '/ramsey/uuid/src/Converter/Number/GenericNumberConverter.php',
    'Ramsey\\Uuid\\Converter\\TimeConverterInterface' => $vendorDir . '/ramsey/uuid/src/Converter/TimeConverterInterface.php',
    'Ramsey\\Uuid\\Converter\\Time\\BigNumberTimeConverter' => $vendorDir . '/ramsey/uuid/src/Converter/Time/BigNumberTimeConverter.php',
    'Ramsey\\Uuid\\Converter\\Time\\DegradedTimeConverter' => $vendorDir . '/ramsey/uuid/src/Converter/Time/DegradedTimeConverter.php',
    'Ramsey\\Uuid\\Converter\\Time\\GenericTimeConverter' => $vendorDir . '/ramsey/uuid/src/Converter/Time/GenericTimeConverter.php',
    'Ramsey\\Uuid\\Converter\\Time\\PhpTimeConverter' => $vendorDir . '/ramsey/uuid/src/Converter/Time/PhpTimeConverter.php',
    'Ramsey\\Uuid\\Converter\\Time\\UnixTimeConverter' => $vendorDir . '/ramsey/uuid/src/Converter/Time/UnixTimeConverter.php',
    'Ramsey\\Uuid\\DegradedUuid' => $vendorDir . '/ramsey/uuid/src/DegradedUuid.php',
    'Ramsey\\Uuid\\DeprecatedUuidInterface' => $vendorDir . '/ramsey/uuid/src/DeprecatedUuidInterface.php',
    'Ramsey\\Uuid\\DeprecatedUuidMethodsTrait' => $vendorDir . '/ramsey/uuid/src/DeprecatedUuidMethodsTrait.php',
    'Ramsey\\Uuid\\Exception\\BuilderNotFoundException' => $vendorDir . '/ramsey/uuid/src/Exception/BuilderNotFoundException.php',
    'Ramsey\\Uuid\\Exception\\DateTimeException' => $vendorDir . '/ramsey/uuid/src/Exception/DateTimeException.php',
    'Ramsey\\Uuid\\Exception\\DceSecurityException' => $vendorDir . '/ramsey/uuid/src/Exception/DceSecurityException.php',
    'Ramsey\\Uuid\\Exception\\InvalidArgumentException' => $vendorDir . '/ramsey/uuid/src/Exception/InvalidArgumentException.php',
    'Ramsey\\Uuid\\Exception\\InvalidBytesException' => $vendorDir . '/ramsey/uuid/src/Exception/InvalidBytesException.php',
    'Ramsey\\Uuid\\Exception\\InvalidUuidStringException' => $vendorDir . '/ramsey/uuid/src/Exception/InvalidUuidStringException.php',
    'Ramsey\\Uuid\\Exception\\NameException' => $vendorDir . '/ramsey/uuid/src/Exception/NameException.php',
    'Ramsey\\Uuid\\Exception\\NodeException' => $vendorDir . '/ramsey/uuid/src/Exception/NodeException.php',
    'Ramsey\\Uuid\\Exception\\RandomSourceException' => $vendorDir . '/ramsey/uuid/src/Exception/RandomSourceException.php',
    'Ramsey\\Uuid\\Exception\\TimeSourceException' => $vendorDir . '/ramsey/uuid/src/Exception/TimeSourceException.php',
    'Ramsey\\Uuid\\Exception\\UnableToBuildUuidException' => $vendorDir . '/ramsey/uuid/src/Exception/UnableToBuildUuidException.php',
    'Ramsey\\Uuid\\Exception\\UnsupportedOperationException' => $vendorDir . '/ramsey/uuid/src/Exception/UnsupportedOperationException.php',
    'Ramsey\\Uuid\\Exception\\UuidExceptionInterface' => $vendorDir . '/ramsey/uuid/src/Exception/UuidExceptionInterface.php',
    'Ramsey\\Uuid\\FeatureSet' => $vendorDir . '/ramsey/uuid/src/FeatureSet.php',
    'Ramsey\\Uuid\\Fields\\FieldsInterface' => $vendorDir . '/ramsey/uuid/src/Fields/FieldsInterface.php',
    'Ramsey\\Uuid\\Fields\\SerializableFieldsTrait' => $vendorDir . '/ramsey/uuid/src/Fields/SerializableFieldsTrait.php',
    'Ramsey\\Uuid\\Generator\\CombGenerator' => $vendorDir . '/ramsey/uuid/src/Generator/CombGenerator.php',
    'Ramsey\\Uuid\\Generator\\DceSecurityGenerator' => $vendorDir . '/ramsey/uuid/src/Generator/DceSecurityGenerator.php',
    'Ramsey\\Uuid\\Generator\\DceSecurityGeneratorInterface' => $vendorDir . '/ramsey/uuid/src/Generator/DceSecurityGeneratorInterface.php',
    'Ramsey\\Uuid\\Generator\\DefaultNameGenerator' => $vendorDir . '/ramsey/uuid/src/Generator/DefaultNameGenerator.php',
    'Ramsey\\Uuid\\Generator\\DefaultTimeGenerator' => $vendorDir . '/ramsey/uuid/src/Generator/DefaultTimeGenerator.php',
    'Ramsey\\Uuid\\Generator\\NameGeneratorFactory' => $vendorDir . '/ramsey/uuid/src/Generator/NameGeneratorFactory.php',
    'Ramsey\\Uuid\\Generator\\NameGeneratorInterface' => $vendorDir . '/ramsey/uuid/src/Generator/NameGeneratorInterface.php',
    'Ramsey\\Uuid\\Generator\\PeclUuidNameGenerator' => $vendorDir . '/ramsey/uuid/src/Generator/PeclUuidNameGenerator.php',
    'Ramsey\\Uuid\\Generator\\PeclUuidRandomGenerator' => $vendorDir . '/ramsey/uuid/src/Generator/PeclUuidRandomGenerator.php',
    'Ramsey\\Uuid\\Generator\\PeclUuidTimeGenerator' => $vendorDir . '/ramsey/uuid/src/Generator/PeclUuidTimeGenerator.php',
    'Ramsey\\Uuid\\Generator\\RandomBytesGenerator' => $vendorDir . '/ramsey/uuid/src/Generator/RandomBytesGenerator.php',
    'Ramsey\\Uuid\\Generator\\RandomGeneratorFactory' => $vendorDir . '/ramsey/uuid/src/Generator/RandomGeneratorFactory.php',
    'Ramsey\\Uuid\\Generator\\RandomGeneratorInterface' => $vendorDir . '/ramsey/uuid/src/Generator/RandomGeneratorInterface.php',
    'Ramsey\\Uuid\\Generator\\RandomLibAdapter' => $vendorDir . '/ramsey/uuid/src/Generator/RandomLibAdapter.php',
    'Ramsey\\Uuid\\Generator\\TimeGeneratorFactory' => $vendorDir . '/ramsey/uuid/src/Generator/TimeGeneratorFactory.php',
    'Ramsey\\Uuid\\Generator\\TimeGeneratorInterface' => $vendorDir . '/ramsey/uuid/src/Generator/TimeGeneratorInterface.php',
    'Ramsey\\Uuid\\Generator\\UnixTimeGenerator' => $vendorDir . '/ramsey/uuid/src/Generator/UnixTimeGenerator.php',
    'Ramsey\\Uuid\\Guid\\Fields' => $vendorDir . '/ramsey/uuid/src/Guid/Fields.php',
    'Ramsey\\Uuid\\Guid\\Guid' => $vendorDir . '/ramsey/uuid/src/Guid/Guid.php',
    'Ramsey\\Uuid\\Guid\\GuidBuilder' => $vendorDir . '/ramsey/uuid/src/Guid/GuidBuilder.php',
    'Ramsey\\Uuid\\Lazy\\LazyUuidFromString' => $vendorDir . '/ramsey/uuid/src/Lazy/LazyUuidFromString.php',
    'Ramsey\\Uuid\\Math\\BrickMathCalculator' => $vendorDir . '/ramsey/uuid/src/Math/BrickMathCalculator.php',
    'Ramsey\\Uuid\\Math\\CalculatorInterface' => $vendorDir . '/ramsey/uuid/src/Math/CalculatorInterface.php',
    'Ramsey\\Uuid\\Math\\RoundingMode' => $vendorDir . '/ramsey/uuid/src/Math/RoundingMode.php',
    'Ramsey\\Uuid\\Nonstandard\\Fields' => $vendorDir . '/ramsey/uuid/src/Nonstandard/Fields.php',
    'Ramsey\\Uuid\\Nonstandard\\Uuid' => $vendorDir . '/ramsey/uuid/src/Nonstandard/Uuid.php',
    'Ramsey\\Uuid\\Nonstandard\\UuidBuilder' => $vendorDir . '/ramsey/uuid/src/Nonstandard/UuidBuilder.php',
    'Ramsey\\Uuid\\Nonstandard\\UuidV6' => $vendorDir . '/ramsey/uuid/src/Nonstandard/UuidV6.php',
    'Ramsey\\Uuid\\Provider\\DceSecurityProviderInterface' => $vendorDir . '/ramsey/uuid/src/Provider/DceSecurityProviderInterface.php',
    'Ramsey\\Uuid\\Provider\\Dce\\SystemDceSecurityProvider' => $vendorDir . '/ramsey/uuid/src/Provider/Dce/SystemDceSecurityProvider.php',
    'Ramsey\\Uuid\\Provider\\NodeProviderInterface' => $vendorDir . '/ramsey/uuid/src/Provider/NodeProviderInterface.php',
    'Ramsey\\Uuid\\Provider\\Node\\FallbackNodeProvider' => $vendorDir . '/ramsey/uuid/src/Provider/Node/FallbackNodeProvider.php',
    'Ramsey\\Uuid\\Provider\\Node\\NodeProviderCollection' => $vendorDir . '/ramsey/uuid/src/Provider/Node/NodeProviderCollection.php',
    'Ramsey\\Uuid\\Provider\\Node\\RandomNodeProvider' => $vendorDir . '/ramsey/uuid/src/Provider/Node/RandomNodeProvider.php',
    'Ramsey\\Uuid\\Provider\\Node\\StaticNodeProvider' => $vendorDir . '/ramsey/uuid/src/Provider/Node/StaticNodeProvider.php',
    'Ramsey\\Uuid\\Provider\\Node\\SystemNodeProvider' => $vendorDir . '/ramsey/uuid/src/Provider/Node/SystemNodeProvider.php',
    'Ramsey\\Uuid\\Provider\\TimeProviderInterface' => $vendorDir . '/ramsey/uuid/src/Provider/TimeProviderInterface.php',
    'Ramsey\\Uuid\\Provider\\Time\\FixedTimeProvider' => $vendorDir . '/ramsey/uuid/src/Provider/Time/FixedTimeProvider.php',
    'Ramsey\\Uuid\\Provider\\Time\\SystemTimeProvider' => $vendorDir . '/ramsey/uuid/src/Provider/Time/SystemTimeProvider.php',
    'Ramsey\\Uuid\\Rfc4122\\Fields' => $vendorDir . '/ramsey/uuid/src/Rfc4122/Fields.php',
    'Ramsey\\Uuid\\Rfc4122\\FieldsInterface' => $vendorDir . '/ramsey/uuid/src/Rfc4122/FieldsInterface.php',
    'Ramsey\\Uuid\\Rfc4122\\MaxTrait' => $vendorDir . '/ramsey/uuid/src/Rfc4122/MaxTrait.php',
    'Ramsey\\Uuid\\Rfc4122\\MaxUuid' => $vendorDir . '/ramsey/uuid/src/Rfc4122/MaxUuid.php',
    'Ramsey\\Uuid\\Rfc4122\\NilTrait' => $vendorDir . '/ramsey/uuid/src/Rfc4122/NilTrait.php',
    'Ramsey\\Uuid\\Rfc4122\\NilUuid' => $vendorDir . '/ramsey/uuid/src/Rfc4122/NilUuid.php',
    'Ramsey\\Uuid\\Rfc4122\\TimeTrait' => $vendorDir . '/ramsey/uuid/src/Rfc4122/TimeTrait.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidBuilder' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidBuilder.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidInterface' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidInterface.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidV1' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidV1.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidV2' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidV2.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidV3' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidV3.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidV4' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidV4.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidV5' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidV5.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidV6' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidV6.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidV7' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidV7.php',
    'Ramsey\\Uuid\\Rfc4122\\UuidV8' => $vendorDir . '/ramsey/uuid/src/Rfc4122/UuidV8.php',
    'Ramsey\\Uuid\\Rfc4122\\Validator' => $vendorDir . '/ramsey/uuid/src/Rfc4122/Validator.php',
    'Ramsey\\Uuid\\Rfc4122\\VariantTrait' => $vendorDir . '/ramsey/uuid/src/Rfc4122/VariantTrait.php',
    'Ramsey\\Uuid\\Rfc4122\\VersionTrait' => $vendorDir . '/ramsey/uuid/src/Rfc4122/VersionTrait.php',
    'Ramsey\\Uuid\\Type\\Decimal' => $vendorDir . '/ramsey/uuid/src/Type/Decimal.php',
    'Ramsey\\Uuid\\Type\\Hexadecimal' => $vendorDir . '/ramsey/uuid/src/Type/Hexadecimal.php',
    'Ramsey\\Uuid\\Type\\Integer' => $vendorDir . '/ramsey/uuid/src/Type/Integer.php',
    'Ramsey\\Uuid\\Type\\NumberInterface' => $vendorDir . '/ramsey/uuid/src/Type/NumberInterface.php',
    'Ramsey\\Uuid\\Type\\Time' => $vendorDir . '/ramsey/uuid/src/Type/Time.php',
    'Ramsey\\Uuid\\Type\\TypeInterface' => $vendorDir . '/ramsey/uuid/src/Type/TypeInterface.php',
    'Ramsey\\Uuid\\Uuid' => $vendorDir . '/ramsey/uuid/src/Uuid.php',
    'Ramsey\\Uuid\\UuidFactory' => $vendorDir . '/ramsey/uuid/src/UuidFactory.php',
    'Ramsey\\Uuid\\UuidFactoryInterface' => $vendorDir . '/ramsey/uuid/src/UuidFactoryInterface.php',
    'Ramsey\\Uuid\\UuidInterface' => $vendorDir . '/ramsey/uuid/src/UuidInterface.php',
    'Ramsey\\Uuid\\Validator\\GenericValidator' => $vendorDir . '/ramsey/uuid/src/Validator/GenericValidator.php',
    'Ramsey\\Uuid\\Validator\\ValidatorInterface' => $vendorDir . '/ramsey/uuid/src/Validator/ValidatorInterface.php',
    'RecurrentChange' => $baseDir . '/src/RecurrentChange.php',
    'ReflectionConstant' => $vendorDir . '/symfony/polyfill-php84/Resources/stubs/ReflectionConstant.php',
    'RefusedEquipment' => $baseDir . '/src/RefusedEquipment.php',
    'RegisteredID' => $baseDir . '/src/RegisteredID.php',
    'Reminder' => $baseDir . '/src/Reminder.php',
    'ReminderTranslation' => $baseDir . '/src/ReminderTranslation.php',
    'Reminder_User' => $baseDir . '/src/Reminder_User.php',
    'Report' => $baseDir . '/src/Report.php',
    'RequestType' => $baseDir . '/src/RequestType.php',
    'Reservation' => $baseDir . '/src/Reservation.php',
    'ReservationItem' => $baseDir . '/src/ReservationItem.php',
    'RobThree\\Auth\\Algorithm' => $vendorDir . '/robthree/twofactorauth/lib/Algorithm.php',
    'RobThree\\Auth\\Providers\\Qr\\BaconQrCodeProvider' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Qr/BaconQrCodeProvider.php',
    'RobThree\\Auth\\Providers\\Qr\\BaseHTTPQRCodeProvider' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Qr/BaseHTTPQRCodeProvider.php',
    'RobThree\\Auth\\Providers\\Qr\\EndroidQrCodeProvider' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Qr/EndroidQrCodeProvider.php',
    'RobThree\\Auth\\Providers\\Qr\\EndroidQrCodeWithLogoProvider' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Qr/EndroidQrCodeWithLogoProvider.php',
    'RobThree\\Auth\\Providers\\Qr\\GoogleChartsQrCodeProvider' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Qr/GoogleChartsQrCodeProvider.php',
    'RobThree\\Auth\\Providers\\Qr\\HandlesDataUri' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Qr/HandlesDataUri.php',
    'RobThree\\Auth\\Providers\\Qr\\IQRCodeProvider' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Qr/IQRCodeProvider.php',
    'RobThree\\Auth\\Providers\\Qr\\ImageChartsQRCodeProvider' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Qr/ImageChartsQRCodeProvider.php',
    'RobThree\\Auth\\Providers\\Qr\\QRException' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Qr/QRException.php',
    'RobThree\\Auth\\Providers\\Qr\\QRServerProvider' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Qr/QRServerProvider.php',
    'RobThree\\Auth\\Providers\\Qr\\QRicketProvider' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Qr/QRicketProvider.php',
    'RobThree\\Auth\\Providers\\Rng\\CSRNGProvider' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Rng/CSRNGProvider.php',
    'RobThree\\Auth\\Providers\\Rng\\IRNGProvider' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Rng/IRNGProvider.php',
    'RobThree\\Auth\\Providers\\Rng\\RNGException' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Rng/RNGException.php',
    'RobThree\\Auth\\Providers\\Time\\HttpTimeProvider' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Time/HttpTimeProvider.php',
    'RobThree\\Auth\\Providers\\Time\\ITimeProvider' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Time/ITimeProvider.php',
    'RobThree\\Auth\\Providers\\Time\\LocalMachineTimeProvider' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Time/LocalMachineTimeProvider.php',
    'RobThree\\Auth\\Providers\\Time\\NTPTimeProvider' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Time/NTPTimeProvider.php',
    'RobThree\\Auth\\Providers\\Time\\TimeException' => $vendorDir . '/robthree/twofactorauth/lib/Providers/Time/TimeException.php',
    'RobThree\\Auth\\TwoFactorAuth' => $vendorDir . '/robthree/twofactorauth/lib/TwoFactorAuth.php',
    'RobThree\\Auth\\TwoFactorAuthException' => $vendorDir . '/robthree/twofactorauth/lib/TwoFactorAuthException.php',
    'Rule' => $baseDir . '/src/Rule.php',
    'RuleAction' => $baseDir . '/src/RuleAction.php',
    'RuleAsset' => $baseDir . '/src/RuleAsset.php',
    'RuleAssetCollection' => $baseDir . '/src/RuleAssetCollection.php',
    'RuleChange' => $baseDir . '/src/RuleChange.php',
    'RuleChangeCollection' => $baseDir . '/src/RuleChangeCollection.php',
    'RuleCollection' => $baseDir . '/src/RuleCollection.php',
    'RuleCommonITILObject' => $baseDir . '/src/RuleCommonITILObject.php',
    'RuleCommonITILObjectCollection' => $baseDir . '/src/RuleCommonITILObjectCollection.php',
    'RuleCriteria' => $baseDir . '/src/RuleCriteria.php',
    'RuleDefineItemtype' => $baseDir . '/src/RuleDefineItemtype.php',
    'RuleDefineItemtypeCollection' => $baseDir . '/src/RuleDefineItemtypeCollection.php',
    'RuleDictionnaryComputerModel' => $baseDir . '/src/RuleDictionnaryComputerModel.php',
    'RuleDictionnaryComputerModelCollection' => $baseDir . '/src/RuleDictionnaryComputerModelCollection.php',
    'RuleDictionnaryComputerType' => $baseDir . '/src/RuleDictionnaryComputerType.php',
    'RuleDictionnaryComputerTypeCollection' => $baseDir . '/src/RuleDictionnaryComputerTypeCollection.php',
    'RuleDictionnaryDropdown' => $baseDir . '/src/RuleDictionnaryDropdown.php',
    'RuleDictionnaryDropdownCollection' => $baseDir . '/src/RuleDictionnaryDropdownCollection.php',
    'RuleDictionnaryManufacturer' => $baseDir . '/src/RuleDictionnaryManufacturer.php',
    'RuleDictionnaryManufacturerCollection' => $baseDir . '/src/RuleDictionnaryManufacturerCollection.php',
    'RuleDictionnaryMonitorModel' => $baseDir . '/src/RuleDictionnaryMonitorModel.php',
    'RuleDictionnaryMonitorModelCollection' => $baseDir . '/src/RuleDictionnaryMonitorModelCollection.php',
    'RuleDictionnaryMonitorType' => $baseDir . '/src/RuleDictionnaryMonitorType.php',
    'RuleDictionnaryMonitorTypeCollection' => $baseDir . '/src/RuleDictionnaryMonitorTypeCollection.php',
    'RuleDictionnaryNetworkEquipmentModel' => $baseDir . '/src/RuleDictionnaryNetworkEquipmentModel.php',
    'RuleDictionnaryNetworkEquipmentModelCollection' => $baseDir . '/src/RuleDictionnaryNetworkEquipmentModelCollection.php',
    'RuleDictionnaryNetworkEquipmentType' => $baseDir . '/src/RuleDictionnaryNetworkEquipmentType.php',
    'RuleDictionnaryNetworkEquipmentTypeCollection' => $baseDir . '/src/RuleDictionnaryNetworkEquipmentTypeCollection.php',
    'RuleDictionnaryOperatingSystem' => $baseDir . '/src/RuleDictionnaryOperatingSystem.php',
    'RuleDictionnaryOperatingSystemArchitecture' => $baseDir . '/src/RuleDictionnaryOperatingSystemArchitecture.php',
    'RuleDictionnaryOperatingSystemArchitectureCollection' => $baseDir . '/src/RuleDictionnaryOperatingSystemArchitectureCollection.php',
    'RuleDictionnaryOperatingSystemCollection' => $baseDir . '/src/RuleDictionnaryOperatingSystemCollection.php',
    'RuleDictionnaryOperatingSystemEdition' => $baseDir . '/src/RuleDictionnaryOperatingSystemEdition.php',
    'RuleDictionnaryOperatingSystemEditionCollection' => $baseDir . '/src/RuleDictionnaryOperatingSystemEditionCollection.php',
    'RuleDictionnaryOperatingSystemServicePack' => $baseDir . '/src/RuleDictionnaryOperatingSystemServicePack.php',
    'RuleDictionnaryOperatingSystemServicePackCollection' => $baseDir . '/src/RuleDictionnaryOperatingSystemServicePackCollection.php',
    'RuleDictionnaryOperatingSystemVersion' => $baseDir . '/src/RuleDictionnaryOperatingSystemVersion.php',
    'RuleDictionnaryOperatingSystemVersionCollection' => $baseDir . '/src/RuleDictionnaryOperatingSystemVersionCollection.php',
    'RuleDictionnaryPeripheralModel' => $baseDir . '/src/RuleDictionnaryPeripheralModel.php',
    'RuleDictionnaryPeripheralModelCollection' => $baseDir . '/src/RuleDictionnaryPeripheralModelCollection.php',
    'RuleDictionnaryPeripheralType' => $baseDir . '/src/RuleDictionnaryPeripheralType.php',
    'RuleDictionnaryPeripheralTypeCollection' => $baseDir . '/src/RuleDictionnaryPeripheralTypeCollection.php',
    'RuleDictionnaryPhoneModel' => $baseDir . '/src/RuleDictionnaryPhoneModel.php',
    'RuleDictionnaryPhoneModelCollection' => $baseDir . '/src/RuleDictionnaryPhoneModelCollection.php',
    'RuleDictionnaryPhoneType' => $baseDir . '/src/RuleDictionnaryPhoneType.php',
    'RuleDictionnaryPhoneTypeCollection' => $baseDir . '/src/RuleDictionnaryPhoneTypeCollection.php',
    'RuleDictionnaryPrinter' => $baseDir . '/src/RuleDictionnaryPrinter.php',
    'RuleDictionnaryPrinterCollection' => $baseDir . '/src/RuleDictionnaryPrinterCollection.php',
    'RuleDictionnaryPrinterModel' => $baseDir . '/src/RuleDictionnaryPrinterModel.php',
    'RuleDictionnaryPrinterModelCollection' => $baseDir . '/src/RuleDictionnaryPrinterModelCollection.php',
    'RuleDictionnaryPrinterType' => $baseDir . '/src/RuleDictionnaryPrinterType.php',
    'RuleDictionnaryPrinterTypeCollection' => $baseDir . '/src/RuleDictionnaryPrinterTypeCollection.php',
    'RuleDictionnarySoftware' => $baseDir . '/src/RuleDictionnarySoftware.php',
    'RuleDictionnarySoftwareCollection' => $baseDir . '/src/RuleDictionnarySoftwareCollection.php',
    'RuleImportAsset' => $baseDir . '/src/RuleImportAsset.php',
    'RuleImportAssetCollection' => $baseDir . '/src/RuleImportAssetCollection.php',
    'RuleImportEntity' => $baseDir . '/src/RuleImportEntity.php',
    'RuleImportEntityCollection' => $baseDir . '/src/RuleImportEntityCollection.php',
    'RuleLocation' => $baseDir . '/src/RuleLocation.php',
    'RuleLocationCollection' => $baseDir . '/src/RuleLocationCollection.php',
    'RuleMailCollector' => $baseDir . '/src/RuleMailCollector.php',
    'RuleMailCollectorCollection' => $baseDir . '/src/RuleMailCollectorCollection.php',
    'RuleMatchedLog' => $baseDir . '/src/RuleMatchedLog.php',
    'RuleProblem' => $baseDir . '/src/RuleProblem.php',
    'RuleProblemCollection' => $baseDir . '/src/RuleProblemCollection.php',
    'RuleRight' => $baseDir . '/src/RuleRight.php',
    'RuleRightCollection' => $baseDir . '/src/RuleRightCollection.php',
    'RuleRightParameter' => $baseDir . '/src/RuleRightParameter.php',
    'RuleSoftwareCategory' => $baseDir . '/src/RuleSoftwareCategory.php',
    'RuleSoftwareCategoryCollection' => $baseDir . '/src/RuleSoftwareCategoryCollection.php',
    'RuleTicket' => $baseDir . '/src/RuleTicket.php',
    'RuleTicketCollection' => $baseDir . '/src/RuleTicketCollection.php',
    'SLA' => $baseDir . '/src/SLA.php',
    'SLM' => $baseDir . '/src/SLM.php',
    'SNMPCredential' => $baseDir . '/src/SNMPCredential.php',
    'SQLite3Exception' => $vendorDir . '/symfony/polyfill-php83/Resources/stubs/SQLite3Exception.php',
    'Sabre\\CalDAV\\Backend\\AbstractBackend' => $vendorDir . '/sabre/dav/lib/CalDAV/Backend/AbstractBackend.php',
    'Sabre\\CalDAV\\Backend\\BackendInterface' => $vendorDir . '/sabre/dav/lib/CalDAV/Backend/BackendInterface.php',
    'Sabre\\CalDAV\\Backend\\NotificationSupport' => $vendorDir . '/sabre/dav/lib/CalDAV/Backend/NotificationSupport.php',
    'Sabre\\CalDAV\\Backend\\PDO' => $vendorDir . '/sabre/dav/lib/CalDAV/Backend/PDO.php',
    'Sabre\\CalDAV\\Backend\\SchedulingSupport' => $vendorDir . '/sabre/dav/lib/CalDAV/Backend/SchedulingSupport.php',
    'Sabre\\CalDAV\\Backend\\SharingSupport' => $vendorDir . '/sabre/dav/lib/CalDAV/Backend/SharingSupport.php',
    'Sabre\\CalDAV\\Backend\\SimplePDO' => $vendorDir . '/sabre/dav/lib/CalDAV/Backend/SimplePDO.php',
    'Sabre\\CalDAV\\Backend\\SubscriptionSupport' => $vendorDir . '/sabre/dav/lib/CalDAV/Backend/SubscriptionSupport.php',
    'Sabre\\CalDAV\\Backend\\SyncSupport' => $vendorDir . '/sabre/dav/lib/CalDAV/Backend/SyncSupport.php',
    'Sabre\\CalDAV\\Calendar' => $vendorDir . '/sabre/dav/lib/CalDAV/Calendar.php',
    'Sabre\\CalDAV\\CalendarHome' => $vendorDir . '/sabre/dav/lib/CalDAV/CalendarHome.php',
    'Sabre\\CalDAV\\CalendarObject' => $vendorDir . '/sabre/dav/lib/CalDAV/CalendarObject.php',
    'Sabre\\CalDAV\\CalendarQueryValidator' => $vendorDir . '/sabre/dav/lib/CalDAV/CalendarQueryValidator.php',
    'Sabre\\CalDAV\\CalendarRoot' => $vendorDir . '/sabre/dav/lib/CalDAV/CalendarRoot.php',
    'Sabre\\CalDAV\\Exception\\InvalidComponentType' => $vendorDir . '/sabre/dav/lib/CalDAV/Exception/InvalidComponentType.php',
    'Sabre\\CalDAV\\ICSExportPlugin' => $vendorDir . '/sabre/dav/lib/CalDAV/ICSExportPlugin.php',
    'Sabre\\CalDAV\\ICalendar' => $vendorDir . '/sabre/dav/lib/CalDAV/ICalendar.php',
    'Sabre\\CalDAV\\ICalendarObject' => $vendorDir . '/sabre/dav/lib/CalDAV/ICalendarObject.php',
    'Sabre\\CalDAV\\ICalendarObjectContainer' => $vendorDir . '/sabre/dav/lib/CalDAV/ICalendarObjectContainer.php',
    'Sabre\\CalDAV\\ISharedCalendar' => $vendorDir . '/sabre/dav/lib/CalDAV/ISharedCalendar.php',
    'Sabre\\CalDAV\\Notifications\\Collection' => $vendorDir . '/sabre/dav/lib/CalDAV/Notifications/Collection.php',
    'Sabre\\CalDAV\\Notifications\\ICollection' => $vendorDir . '/sabre/dav/lib/CalDAV/Notifications/ICollection.php',
    'Sabre\\CalDAV\\Notifications\\INode' => $vendorDir . '/sabre/dav/lib/CalDAV/Notifications/INode.php',
    'Sabre\\CalDAV\\Notifications\\Node' => $vendorDir . '/sabre/dav/lib/CalDAV/Notifications/Node.php',
    'Sabre\\CalDAV\\Notifications\\Plugin' => $vendorDir . '/sabre/dav/lib/CalDAV/Notifications/Plugin.php',
    'Sabre\\CalDAV\\Plugin' => $vendorDir . '/sabre/dav/lib/CalDAV/Plugin.php',
    'Sabre\\CalDAV\\Principal\\Collection' => $vendorDir . '/sabre/dav/lib/CalDAV/Principal/Collection.php',
    'Sabre\\CalDAV\\Principal\\IProxyRead' => $vendorDir . '/sabre/dav/lib/CalDAV/Principal/IProxyRead.php',
    'Sabre\\CalDAV\\Principal\\IProxyWrite' => $vendorDir . '/sabre/dav/lib/CalDAV/Principal/IProxyWrite.php',
    'Sabre\\CalDAV\\Principal\\ProxyRead' => $vendorDir . '/sabre/dav/lib/CalDAV/Principal/ProxyRead.php',
    'Sabre\\CalDAV\\Principal\\ProxyWrite' => $vendorDir . '/sabre/dav/lib/CalDAV/Principal/ProxyWrite.php',
    'Sabre\\CalDAV\\Principal\\User' => $vendorDir . '/sabre/dav/lib/CalDAV/Principal/User.php',
    'Sabre\\CalDAV\\Schedule\\IInbox' => $vendorDir . '/sabre/dav/lib/CalDAV/Schedule/IInbox.php',
    'Sabre\\CalDAV\\Schedule\\IMipPlugin' => $vendorDir . '/sabre/dav/lib/CalDAV/Schedule/IMipPlugin.php',
    'Sabre\\CalDAV\\Schedule\\IOutbox' => $vendorDir . '/sabre/dav/lib/CalDAV/Schedule/IOutbox.php',
    'Sabre\\CalDAV\\Schedule\\ISchedulingObject' => $vendorDir . '/sabre/dav/lib/CalDAV/Schedule/ISchedulingObject.php',
    'Sabre\\CalDAV\\Schedule\\Inbox' => $vendorDir . '/sabre/dav/lib/CalDAV/Schedule/Inbox.php',
    'Sabre\\CalDAV\\Schedule\\Outbox' => $vendorDir . '/sabre/dav/lib/CalDAV/Schedule/Outbox.php',
    'Sabre\\CalDAV\\Schedule\\Plugin' => $vendorDir . '/sabre/dav/lib/CalDAV/Schedule/Plugin.php',
    'Sabre\\CalDAV\\Schedule\\SchedulingObject' => $vendorDir . '/sabre/dav/lib/CalDAV/Schedule/SchedulingObject.php',
    'Sabre\\CalDAV\\SharedCalendar' => $vendorDir . '/sabre/dav/lib/CalDAV/SharedCalendar.php',
    'Sabre\\CalDAV\\SharingPlugin' => $vendorDir . '/sabre/dav/lib/CalDAV/SharingPlugin.php',
    'Sabre\\CalDAV\\Subscriptions\\ISubscription' => $vendorDir . '/sabre/dav/lib/CalDAV/Subscriptions/ISubscription.php',
    'Sabre\\CalDAV\\Subscriptions\\Plugin' => $vendorDir . '/sabre/dav/lib/CalDAV/Subscriptions/Plugin.php',
    'Sabre\\CalDAV\\Subscriptions\\Subscription' => $vendorDir . '/sabre/dav/lib/CalDAV/Subscriptions/Subscription.php',
    'Sabre\\CalDAV\\Xml\\Filter\\CalendarData' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Filter/CalendarData.php',
    'Sabre\\CalDAV\\Xml\\Filter\\CompFilter' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Filter/CompFilter.php',
    'Sabre\\CalDAV\\Xml\\Filter\\ParamFilter' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Filter/ParamFilter.php',
    'Sabre\\CalDAV\\Xml\\Filter\\PropFilter' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Filter/PropFilter.php',
    'Sabre\\CalDAV\\Xml\\Notification\\Invite' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Notification/Invite.php',
    'Sabre\\CalDAV\\Xml\\Notification\\InviteReply' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Notification/InviteReply.php',
    'Sabre\\CalDAV\\Xml\\Notification\\NotificationInterface' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Notification/NotificationInterface.php',
    'Sabre\\CalDAV\\Xml\\Notification\\SystemStatus' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Notification/SystemStatus.php',
    'Sabre\\CalDAV\\Xml\\Property\\AllowedSharingModes' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Property/AllowedSharingModes.php',
    'Sabre\\CalDAV\\Xml\\Property\\EmailAddressSet' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Property/EmailAddressSet.php',
    'Sabre\\CalDAV\\Xml\\Property\\Invite' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Property/Invite.php',
    'Sabre\\CalDAV\\Xml\\Property\\ScheduleCalendarTransp' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Property/ScheduleCalendarTransp.php',
    'Sabre\\CalDAV\\Xml\\Property\\SupportedCalendarComponentSet' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Property/SupportedCalendarComponentSet.php',
    'Sabre\\CalDAV\\Xml\\Property\\SupportedCalendarData' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Property/SupportedCalendarData.php',
    'Sabre\\CalDAV\\Xml\\Property\\SupportedCollationSet' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Property/SupportedCollationSet.php',
    'Sabre\\CalDAV\\Xml\\Request\\CalendarMultiGetReport' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Request/CalendarMultiGetReport.php',
    'Sabre\\CalDAV\\Xml\\Request\\CalendarQueryReport' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Request/CalendarQueryReport.php',
    'Sabre\\CalDAV\\Xml\\Request\\FreeBusyQueryReport' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Request/FreeBusyQueryReport.php',
    'Sabre\\CalDAV\\Xml\\Request\\InviteReply' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Request/InviteReply.php',
    'Sabre\\CalDAV\\Xml\\Request\\MkCalendar' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Request/MkCalendar.php',
    'Sabre\\CalDAV\\Xml\\Request\\Share' => $vendorDir . '/sabre/dav/lib/CalDAV/Xml/Request/Share.php',
    'Sabre\\CardDAV\\AddressBook' => $vendorDir . '/sabre/dav/lib/CardDAV/AddressBook.php',
    'Sabre\\CardDAV\\AddressBookHome' => $vendorDir . '/sabre/dav/lib/CardDAV/AddressBookHome.php',
    'Sabre\\CardDAV\\AddressBookRoot' => $vendorDir . '/sabre/dav/lib/CardDAV/AddressBookRoot.php',
    'Sabre\\CardDAV\\Backend\\AbstractBackend' => $vendorDir . '/sabre/dav/lib/CardDAV/Backend/AbstractBackend.php',
    'Sabre\\CardDAV\\Backend\\BackendInterface' => $vendorDir . '/sabre/dav/lib/CardDAV/Backend/BackendInterface.php',
    'Sabre\\CardDAV\\Backend\\PDO' => $vendorDir . '/sabre/dav/lib/CardDAV/Backend/PDO.php',
    'Sabre\\CardDAV\\Backend\\SyncSupport' => $vendorDir . '/sabre/dav/lib/CardDAV/Backend/SyncSupport.php',
    'Sabre\\CardDAV\\Card' => $vendorDir . '/sabre/dav/lib/CardDAV/Card.php',
    'Sabre\\CardDAV\\IAddressBook' => $vendorDir . '/sabre/dav/lib/CardDAV/IAddressBook.php',
    'Sabre\\CardDAV\\ICard' => $vendorDir . '/sabre/dav/lib/CardDAV/ICard.php',
    'Sabre\\CardDAV\\IDirectory' => $vendorDir . '/sabre/dav/lib/CardDAV/IDirectory.php',
    'Sabre\\CardDAV\\Plugin' => $vendorDir . '/sabre/dav/lib/CardDAV/Plugin.php',
    'Sabre\\CardDAV\\VCFExportPlugin' => $vendorDir . '/sabre/dav/lib/CardDAV/VCFExportPlugin.php',
    'Sabre\\CardDAV\\Xml\\Filter\\AddressData' => $vendorDir . '/sabre/dav/lib/CardDAV/Xml/Filter/AddressData.php',
    'Sabre\\CardDAV\\Xml\\Filter\\ParamFilter' => $vendorDir . '/sabre/dav/lib/CardDAV/Xml/Filter/ParamFilter.php',
    'Sabre\\CardDAV\\Xml\\Filter\\PropFilter' => $vendorDir . '/sabre/dav/lib/CardDAV/Xml/Filter/PropFilter.php',
    'Sabre\\CardDAV\\Xml\\Property\\SupportedAddressData' => $vendorDir . '/sabre/dav/lib/CardDAV/Xml/Property/SupportedAddressData.php',
    'Sabre\\CardDAV\\Xml\\Property\\SupportedCollationSet' => $vendorDir . '/sabre/dav/lib/CardDAV/Xml/Property/SupportedCollationSet.php',
    'Sabre\\CardDAV\\Xml\\Request\\AddressBookMultiGetReport' => $vendorDir . '/sabre/dav/lib/CardDAV/Xml/Request/AddressBookMultiGetReport.php',
    'Sabre\\CardDAV\\Xml\\Request\\AddressBookQueryReport' => $vendorDir . '/sabre/dav/lib/CardDAV/Xml/Request/AddressBookQueryReport.php',
    'Sabre\\DAVACL\\ACLTrait' => $vendorDir . '/sabre/dav/lib/DAVACL/ACLTrait.php',
    'Sabre\\DAVACL\\AbstractPrincipalCollection' => $vendorDir . '/sabre/dav/lib/DAVACL/AbstractPrincipalCollection.php',
    'Sabre\\DAVACL\\Exception\\AceConflict' => $vendorDir . '/sabre/dav/lib/DAVACL/Exception/AceConflict.php',
    'Sabre\\DAVACL\\Exception\\NeedPrivileges' => $vendorDir . '/sabre/dav/lib/DAVACL/Exception/NeedPrivileges.php',
    'Sabre\\DAVACL\\Exception\\NoAbstract' => $vendorDir . '/sabre/dav/lib/DAVACL/Exception/NoAbstract.php',
    'Sabre\\DAVACL\\Exception\\NotRecognizedPrincipal' => $vendorDir . '/sabre/dav/lib/DAVACL/Exception/NotRecognizedPrincipal.php',
    'Sabre\\DAVACL\\Exception\\NotSupportedPrivilege' => $vendorDir . '/sabre/dav/lib/DAVACL/Exception/NotSupportedPrivilege.php',
    'Sabre\\DAVACL\\FS\\Collection' => $vendorDir . '/sabre/dav/lib/DAVACL/FS/Collection.php',
    'Sabre\\DAVACL\\FS\\File' => $vendorDir . '/sabre/dav/lib/DAVACL/FS/File.php',
    'Sabre\\DAVACL\\FS\\HomeCollection' => $vendorDir . '/sabre/dav/lib/DAVACL/FS/HomeCollection.php',
    'Sabre\\DAVACL\\IACL' => $vendorDir . '/sabre/dav/lib/DAVACL/IACL.php',
    'Sabre\\DAVACL\\IPrincipal' => $vendorDir . '/sabre/dav/lib/DAVACL/IPrincipal.php',
    'Sabre\\DAVACL\\IPrincipalCollection' => $vendorDir . '/sabre/dav/lib/DAVACL/IPrincipalCollection.php',
    'Sabre\\DAVACL\\Plugin' => $vendorDir . '/sabre/dav/lib/DAVACL/Plugin.php',
    'Sabre\\DAVACL\\Principal' => $vendorDir . '/sabre/dav/lib/DAVACL/Principal.php',
    'Sabre\\DAVACL\\PrincipalBackend\\AbstractBackend' => $vendorDir . '/sabre/dav/lib/DAVACL/PrincipalBackend/AbstractBackend.php',
    'Sabre\\DAVACL\\PrincipalBackend\\BackendInterface' => $vendorDir . '/sabre/dav/lib/DAVACL/PrincipalBackend/BackendInterface.php',
    'Sabre\\DAVACL\\PrincipalBackend\\CreatePrincipalSupport' => $vendorDir . '/sabre/dav/lib/DAVACL/PrincipalBackend/CreatePrincipalSupport.php',
    'Sabre\\DAVACL\\PrincipalBackend\\PDO' => $vendorDir . '/sabre/dav/lib/DAVACL/PrincipalBackend/PDO.php',
    'Sabre\\DAVACL\\PrincipalCollection' => $vendorDir . '/sabre/dav/lib/DAVACL/PrincipalCollection.php',
    'Sabre\\DAVACL\\Xml\\Property\\Acl' => $vendorDir . '/sabre/dav/lib/DAVACL/Xml/Property/Acl.php',
    'Sabre\\DAVACL\\Xml\\Property\\AclRestrictions' => $vendorDir . '/sabre/dav/lib/DAVACL/Xml/Property/AclRestrictions.php',
    'Sabre\\DAVACL\\Xml\\Property\\CurrentUserPrivilegeSet' => $vendorDir . '/sabre/dav/lib/DAVACL/Xml/Property/CurrentUserPrivilegeSet.php',
    'Sabre\\DAVACL\\Xml\\Property\\Principal' => $vendorDir . '/sabre/dav/lib/DAVACL/Xml/Property/Principal.php',
    'Sabre\\DAVACL\\Xml\\Property\\SupportedPrivilegeSet' => $vendorDir . '/sabre/dav/lib/DAVACL/Xml/Property/SupportedPrivilegeSet.php',
    'Sabre\\DAVACL\\Xml\\Request\\AclPrincipalPropSetReport' => $vendorDir . '/sabre/dav/lib/DAVACL/Xml/Request/AclPrincipalPropSetReport.php',
    'Sabre\\DAVACL\\Xml\\Request\\ExpandPropertyReport' => $vendorDir . '/sabre/dav/lib/DAVACL/Xml/Request/ExpandPropertyReport.php',
    'Sabre\\DAVACL\\Xml\\Request\\PrincipalMatchReport' => $vendorDir . '/sabre/dav/lib/DAVACL/Xml/Request/PrincipalMatchReport.php',
    'Sabre\\DAVACL\\Xml\\Request\\PrincipalPropertySearchReport' => $vendorDir . '/sabre/dav/lib/DAVACL/Xml/Request/PrincipalPropertySearchReport.php',
    'Sabre\\DAVACL\\Xml\\Request\\PrincipalSearchPropertySetReport' => $vendorDir . '/sabre/dav/lib/DAVACL/Xml/Request/PrincipalSearchPropertySetReport.php',
    'Sabre\\DAV\\Auth\\Backend\\AbstractBasic' => $vendorDir . '/sabre/dav/lib/DAV/Auth/Backend/AbstractBasic.php',
    'Sabre\\DAV\\Auth\\Backend\\AbstractBearer' => $vendorDir . '/sabre/dav/lib/DAV/Auth/Backend/AbstractBearer.php',
    'Sabre\\DAV\\Auth\\Backend\\AbstractDigest' => $vendorDir . '/sabre/dav/lib/DAV/Auth/Backend/AbstractDigest.php',
    'Sabre\\DAV\\Auth\\Backend\\Apache' => $vendorDir . '/sabre/dav/lib/DAV/Auth/Backend/Apache.php',
    'Sabre\\DAV\\Auth\\Backend\\BackendInterface' => $vendorDir . '/sabre/dav/lib/DAV/Auth/Backend/BackendInterface.php',
    'Sabre\\DAV\\Auth\\Backend\\BasicCallBack' => $vendorDir . '/sabre/dav/lib/DAV/Auth/Backend/BasicCallBack.php',
    'Sabre\\DAV\\Auth\\Backend\\File' => $vendorDir . '/sabre/dav/lib/DAV/Auth/Backend/File.php',
    'Sabre\\DAV\\Auth\\Backend\\IMAP' => $vendorDir . '/sabre/dav/lib/DAV/Auth/Backend/IMAP.php',
    'Sabre\\DAV\\Auth\\Backend\\PDO' => $vendorDir . '/sabre/dav/lib/DAV/Auth/Backend/PDO.php',
    'Sabre\\DAV\\Auth\\Backend\\PDOBasicAuth' => $vendorDir . '/sabre/dav/lib/DAV/Auth/Backend/PDOBasicAuth.php',
    'Sabre\\DAV\\Auth\\Plugin' => $vendorDir . '/sabre/dav/lib/DAV/Auth/Plugin.php',
    'Sabre\\DAV\\Browser\\GuessContentType' => $vendorDir . '/sabre/dav/lib/DAV/Browser/GuessContentType.php',
    'Sabre\\DAV\\Browser\\HtmlOutput' => $vendorDir . '/sabre/dav/lib/DAV/Browser/HtmlOutput.php',
    'Sabre\\DAV\\Browser\\HtmlOutputHelper' => $vendorDir . '/sabre/dav/lib/DAV/Browser/HtmlOutputHelper.php',
    'Sabre\\DAV\\Browser\\MapGetToPropFind' => $vendorDir . '/sabre/dav/lib/DAV/Browser/MapGetToPropFind.php',
    'Sabre\\DAV\\Browser\\Plugin' => $vendorDir . '/sabre/dav/lib/DAV/Browser/Plugin.php',
    'Sabre\\DAV\\Browser\\PropFindAll' => $vendorDir . '/sabre/dav/lib/DAV/Browser/PropFindAll.php',
    'Sabre\\DAV\\Client' => $vendorDir . '/sabre/dav/lib/DAV/Client.php',
    'Sabre\\DAV\\Collection' => $vendorDir . '/sabre/dav/lib/DAV/Collection.php',
    'Sabre\\DAV\\CorePlugin' => $vendorDir . '/sabre/dav/lib/DAV/CorePlugin.php',
    'Sabre\\DAV\\Exception' => $vendorDir . '/sabre/dav/lib/DAV/Exception.php',
    'Sabre\\DAV\\Exception\\BadRequest' => $vendorDir . '/sabre/dav/lib/DAV/Exception/BadRequest.php',
    'Sabre\\DAV\\Exception\\Conflict' => $vendorDir . '/sabre/dav/lib/DAV/Exception/Conflict.php',
    'Sabre\\DAV\\Exception\\ConflictingLock' => $vendorDir . '/sabre/dav/lib/DAV/Exception/ConflictingLock.php',
    'Sabre\\DAV\\Exception\\Forbidden' => $vendorDir . '/sabre/dav/lib/DAV/Exception/Forbidden.php',
    'Sabre\\DAV\\Exception\\InsufficientStorage' => $vendorDir . '/sabre/dav/lib/DAV/Exception/InsufficientStorage.php',
    'Sabre\\DAV\\Exception\\InvalidResourceType' => $vendorDir . '/sabre/dav/lib/DAV/Exception/InvalidResourceType.php',
    'Sabre\\DAV\\Exception\\InvalidSyncToken' => $vendorDir . '/sabre/dav/lib/DAV/Exception/InvalidSyncToken.php',
    'Sabre\\DAV\\Exception\\LengthRequired' => $vendorDir . '/sabre/dav/lib/DAV/Exception/LengthRequired.php',
    'Sabre\\DAV\\Exception\\LockTokenMatchesRequestUri' => $vendorDir . '/sabre/dav/lib/DAV/Exception/LockTokenMatchesRequestUri.php',
    'Sabre\\DAV\\Exception\\Locked' => $vendorDir . '/sabre/dav/lib/DAV/Exception/Locked.php',
    'Sabre\\DAV\\Exception\\MethodNotAllowed' => $vendorDir . '/sabre/dav/lib/DAV/Exception/MethodNotAllowed.php',
    'Sabre\\DAV\\Exception\\NotAuthenticated' => $vendorDir . '/sabre/dav/lib/DAV/Exception/NotAuthenticated.php',
    'Sabre\\DAV\\Exception\\NotFound' => $vendorDir . '/sabre/dav/lib/DAV/Exception/NotFound.php',
    'Sabre\\DAV\\Exception\\NotImplemented' => $vendorDir . '/sabre/dav/lib/DAV/Exception/NotImplemented.php',
    'Sabre\\DAV\\Exception\\PaymentRequired' => $vendorDir . '/sabre/dav/lib/DAV/Exception/PaymentRequired.php',
    'Sabre\\DAV\\Exception\\PreconditionFailed' => $vendorDir . '/sabre/dav/lib/DAV/Exception/PreconditionFailed.php',
    'Sabre\\DAV\\Exception\\ReportNotSupported' => $vendorDir . '/sabre/dav/lib/DAV/Exception/ReportNotSupported.php',
    'Sabre\\DAV\\Exception\\RequestedRangeNotSatisfiable' => $vendorDir . '/sabre/dav/lib/DAV/Exception/RequestedRangeNotSatisfiable.php',
    'Sabre\\DAV\\Exception\\ServiceUnavailable' => $vendorDir . '/sabre/dav/lib/DAV/Exception/ServiceUnavailable.php',
    'Sabre\\DAV\\Exception\\TooManyMatches' => $vendorDir . '/sabre/dav/lib/DAV/Exception/TooManyMatches.php',
    'Sabre\\DAV\\Exception\\UnsupportedMediaType' => $vendorDir . '/sabre/dav/lib/DAV/Exception/UnsupportedMediaType.php',
    'Sabre\\DAV\\FSExt\\Directory' => $vendorDir . '/sabre/dav/lib/DAV/FSExt/Directory.php',
    'Sabre\\DAV\\FSExt\\File' => $vendorDir . '/sabre/dav/lib/DAV/FSExt/File.php',
    'Sabre\\DAV\\FS\\Directory' => $vendorDir . '/sabre/dav/lib/DAV/FS/Directory.php',
    'Sabre\\DAV\\FS\\File' => $vendorDir . '/sabre/dav/lib/DAV/FS/File.php',
    'Sabre\\DAV\\FS\\Node' => $vendorDir . '/sabre/dav/lib/DAV/FS/Node.php',
    'Sabre\\DAV\\File' => $vendorDir . '/sabre/dav/lib/DAV/File.php',
    'Sabre\\DAV\\ICollection' => $vendorDir . '/sabre/dav/lib/DAV/ICollection.php',
    'Sabre\\DAV\\ICopyTarget' => $vendorDir . '/sabre/dav/lib/DAV/ICopyTarget.php',
    'Sabre\\DAV\\IExtendedCollection' => $vendorDir . '/sabre/dav/lib/DAV/IExtendedCollection.php',
    'Sabre\\DAV\\IFile' => $vendorDir . '/sabre/dav/lib/DAV/IFile.php',
    'Sabre\\DAV\\IMoveTarget' => $vendorDir . '/sabre/dav/lib/DAV/IMoveTarget.php',
    'Sabre\\DAV\\IMultiGet' => $vendorDir . '/sabre/dav/lib/DAV/IMultiGet.php',
    'Sabre\\DAV\\INode' => $vendorDir . '/sabre/dav/lib/DAV/INode.php',
    'Sabre\\DAV\\INodeByPath' => $vendorDir . '/sabre/dav/lib/DAV/INodeByPath.php',
    'Sabre\\DAV\\IProperties' => $vendorDir . '/sabre/dav/lib/DAV/IProperties.php',
    'Sabre\\DAV\\IQuota' => $vendorDir . '/sabre/dav/lib/DAV/IQuota.php',
    'Sabre\\DAV\\Locks\\Backend\\AbstractBackend' => $vendorDir . '/sabre/dav/lib/DAV/Locks/Backend/AbstractBackend.php',
    'Sabre\\DAV\\Locks\\Backend\\BackendInterface' => $vendorDir . '/sabre/dav/lib/DAV/Locks/Backend/BackendInterface.php',
    'Sabre\\DAV\\Locks\\Backend\\File' => $vendorDir . '/sabre/dav/lib/DAV/Locks/Backend/File.php',
    'Sabre\\DAV\\Locks\\Backend\\PDO' => $vendorDir . '/sabre/dav/lib/DAV/Locks/Backend/PDO.php',
    'Sabre\\DAV\\Locks\\LockInfo' => $vendorDir . '/sabre/dav/lib/DAV/Locks/LockInfo.php',
    'Sabre\\DAV\\Locks\\Plugin' => $vendorDir . '/sabre/dav/lib/DAV/Locks/Plugin.php',
    'Sabre\\DAV\\MkCol' => $vendorDir . '/sabre/dav/lib/DAV/MkCol.php',
    'Sabre\\DAV\\Mount\\Plugin' => $vendorDir . '/sabre/dav/lib/DAV/Mount/Plugin.php',
    'Sabre\\DAV\\Node' => $vendorDir . '/sabre/dav/lib/DAV/Node.php',
    'Sabre\\DAV\\PartialUpdate\\IPatchSupport' => $vendorDir . '/sabre/dav/lib/DAV/PartialUpdate/IPatchSupport.php',
    'Sabre\\DAV\\PartialUpdate\\Plugin' => $vendorDir . '/sabre/dav/lib/DAV/PartialUpdate/Plugin.php',
    'Sabre\\DAV\\PropFind' => $vendorDir . '/sabre/dav/lib/DAV/PropFind.php',
    'Sabre\\DAV\\PropPatch' => $vendorDir . '/sabre/dav/lib/DAV/PropPatch.php',
    'Sabre\\DAV\\PropertyStorage\\Backend\\BackendInterface' => $vendorDir . '/sabre/dav/lib/DAV/PropertyStorage/Backend/BackendInterface.php',
    'Sabre\\DAV\\PropertyStorage\\Backend\\PDO' => $vendorDir . '/sabre/dav/lib/DAV/PropertyStorage/Backend/PDO.php',
    'Sabre\\DAV\\PropertyStorage\\Plugin' => $vendorDir . '/sabre/dav/lib/DAV/PropertyStorage/Plugin.php',
    'Sabre\\DAV\\Server' => $vendorDir . '/sabre/dav/lib/DAV/Server.php',
    'Sabre\\DAV\\ServerPlugin' => $vendorDir . '/sabre/dav/lib/DAV/ServerPlugin.php',
    'Sabre\\DAV\\Sharing\\ISharedNode' => $vendorDir . '/sabre/dav/lib/DAV/Sharing/ISharedNode.php',
    'Sabre\\DAV\\Sharing\\Plugin' => $vendorDir . '/sabre/dav/lib/DAV/Sharing/Plugin.php',
    'Sabre\\DAV\\SimpleCollection' => $vendorDir . '/sabre/dav/lib/DAV/SimpleCollection.php',
    'Sabre\\DAV\\SimpleFile' => $vendorDir . '/sabre/dav/lib/DAV/SimpleFile.php',
    'Sabre\\DAV\\StringUtil' => $vendorDir . '/sabre/dav/lib/DAV/StringUtil.php',
    'Sabre\\DAV\\Sync\\ISyncCollection' => $vendorDir . '/sabre/dav/lib/DAV/Sync/ISyncCollection.php',
    'Sabre\\DAV\\Sync\\Plugin' => $vendorDir . '/sabre/dav/lib/DAV/Sync/Plugin.php',
    'Sabre\\DAV\\TemporaryFileFilterPlugin' => $vendorDir . '/sabre/dav/lib/DAV/TemporaryFileFilterPlugin.php',
    'Sabre\\DAV\\Tree' => $vendorDir . '/sabre/dav/lib/DAV/Tree.php',
    'Sabre\\DAV\\UUIDUtil' => $vendorDir . '/sabre/dav/lib/DAV/UUIDUtil.php',
    'Sabre\\DAV\\Version' => $vendorDir . '/sabre/dav/lib/DAV/Version.php',
    'Sabre\\DAV\\Xml\\Element\\Prop' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Element/Prop.php',
    'Sabre\\DAV\\Xml\\Element\\Response' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Element/Response.php',
    'Sabre\\DAV\\Xml\\Element\\Sharee' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Element/Sharee.php',
    'Sabre\\DAV\\Xml\\Property\\Complex' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Property/Complex.php',
    'Sabre\\DAV\\Xml\\Property\\GetLastModified' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Property/GetLastModified.php',
    'Sabre\\DAV\\Xml\\Property\\Href' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Property/Href.php',
    'Sabre\\DAV\\Xml\\Property\\Invite' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Property/Invite.php',
    'Sabre\\DAV\\Xml\\Property\\LocalHref' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Property/LocalHref.php',
    'Sabre\\DAV\\Xml\\Property\\LockDiscovery' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Property/LockDiscovery.php',
    'Sabre\\DAV\\Xml\\Property\\ResourceType' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Property/ResourceType.php',
    'Sabre\\DAV\\Xml\\Property\\ShareAccess' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Property/ShareAccess.php',
    'Sabre\\DAV\\Xml\\Property\\SupportedLock' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Property/SupportedLock.php',
    'Sabre\\DAV\\Xml\\Property\\SupportedMethodSet' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Property/SupportedMethodSet.php',
    'Sabre\\DAV\\Xml\\Property\\SupportedReportSet' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Property/SupportedReportSet.php',
    'Sabre\\DAV\\Xml\\Request\\Lock' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Request/Lock.php',
    'Sabre\\DAV\\Xml\\Request\\MkCol' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Request/MkCol.php',
    'Sabre\\DAV\\Xml\\Request\\PropFind' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Request/PropFind.php',
    'Sabre\\DAV\\Xml\\Request\\PropPatch' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Request/PropPatch.php',
    'Sabre\\DAV\\Xml\\Request\\ShareResource' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Request/ShareResource.php',
    'Sabre\\DAV\\Xml\\Request\\SyncCollectionReport' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Request/SyncCollectionReport.php',
    'Sabre\\DAV\\Xml\\Response\\MultiStatus' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Response/MultiStatus.php',
    'Sabre\\DAV\\Xml\\Service' => $vendorDir . '/sabre/dav/lib/DAV/Xml/Service.php',
    'Sabre\\Event\\Emitter' => $vendorDir . '/sabre/event/lib/Emitter.php',
    'Sabre\\Event\\EmitterInterface' => $vendorDir . '/sabre/event/lib/EmitterInterface.php',
    'Sabre\\Event\\EmitterTrait' => $vendorDir . '/sabre/event/lib/EmitterTrait.php',
    'Sabre\\Event\\EventEmitter' => $vendorDir . '/sabre/event/lib/EventEmitter.php',
    'Sabre\\Event\\Loop\\Loop' => $vendorDir . '/sabre/event/lib/Loop/Loop.php',
    'Sabre\\Event\\Promise' => $vendorDir . '/sabre/event/lib/Promise.php',
    'Sabre\\Event\\PromiseAlreadyResolvedException' => $vendorDir . '/sabre/event/lib/PromiseAlreadyResolvedException.php',
    'Sabre\\Event\\Version' => $vendorDir . '/sabre/event/lib/Version.php',
    'Sabre\\Event\\WildcardEmitter' => $vendorDir . '/sabre/event/lib/WildcardEmitter.php',
    'Sabre\\Event\\WildcardEmitterTrait' => $vendorDir . '/sabre/event/lib/WildcardEmitterTrait.php',
    'Sabre\\HTTP\\Auth\\AWS' => $vendorDir . '/sabre/http/lib/Auth/AWS.php',
    'Sabre\\HTTP\\Auth\\AbstractAuth' => $vendorDir . '/sabre/http/lib/Auth/AbstractAuth.php',
    'Sabre\\HTTP\\Auth\\Basic' => $vendorDir . '/sabre/http/lib/Auth/Basic.php',
    'Sabre\\HTTP\\Auth\\Bearer' => $vendorDir . '/sabre/http/lib/Auth/Bearer.php',
    'Sabre\\HTTP\\Auth\\Digest' => $vendorDir . '/sabre/http/lib/Auth/Digest.php',
    'Sabre\\HTTP\\Client' => $vendorDir . '/sabre/http/lib/Client.php',
    'Sabre\\HTTP\\ClientException' => $vendorDir . '/sabre/http/lib/ClientException.php',
    'Sabre\\HTTP\\ClientHttpException' => $vendorDir . '/sabre/http/lib/ClientHttpException.php',
    'Sabre\\HTTP\\HttpException' => $vendorDir . '/sabre/http/lib/HttpException.php',
    'Sabre\\HTTP\\Message' => $vendorDir . '/sabre/http/lib/Message.php',
    'Sabre\\HTTP\\MessageDecoratorTrait' => $vendorDir . '/sabre/http/lib/MessageDecoratorTrait.php',
    'Sabre\\HTTP\\MessageInterface' => $vendorDir . '/sabre/http/lib/MessageInterface.php',
    'Sabre\\HTTP\\Request' => $vendorDir . '/sabre/http/lib/Request.php',
    'Sabre\\HTTP\\RequestDecorator' => $vendorDir . '/sabre/http/lib/RequestDecorator.php',
    'Sabre\\HTTP\\RequestInterface' => $vendorDir . '/sabre/http/lib/RequestInterface.php',
    'Sabre\\HTTP\\Response' => $vendorDir . '/sabre/http/lib/Response.php',
    'Sabre\\HTTP\\ResponseDecorator' => $vendorDir . '/sabre/http/lib/ResponseDecorator.php',
    'Sabre\\HTTP\\ResponseInterface' => $vendorDir . '/sabre/http/lib/ResponseInterface.php',
    'Sabre\\HTTP\\Sapi' => $vendorDir . '/sabre/http/lib/Sapi.php',
    'Sabre\\HTTP\\Version' => $vendorDir . '/sabre/http/lib/Version.php',
    'Sabre\\Uri\\InvalidUriException' => $vendorDir . '/sabre/uri/lib/InvalidUriException.php',
    'Sabre\\Uri\\Version' => $vendorDir . '/sabre/uri/lib/Version.php',
    'Sabre\\VObject\\BirthdayCalendarGenerator' => $vendorDir . '/sabre/vobject/lib/BirthdayCalendarGenerator.php',
    'Sabre\\VObject\\Cli' => $vendorDir . '/sabre/vobject/lib/Cli.php',
    'Sabre\\VObject\\Component' => $vendorDir . '/sabre/vobject/lib/Component.php',
    'Sabre\\VObject\\Component\\Available' => $vendorDir . '/sabre/vobject/lib/Component/Available.php',
    'Sabre\\VObject\\Component\\VAlarm' => $vendorDir . '/sabre/vobject/lib/Component/VAlarm.php',
    'Sabre\\VObject\\Component\\VAvailability' => $vendorDir . '/sabre/vobject/lib/Component/VAvailability.php',
    'Sabre\\VObject\\Component\\VCalendar' => $vendorDir . '/sabre/vobject/lib/Component/VCalendar.php',
    'Sabre\\VObject\\Component\\VCard' => $vendorDir . '/sabre/vobject/lib/Component/VCard.php',
    'Sabre\\VObject\\Component\\VEvent' => $vendorDir . '/sabre/vobject/lib/Component/VEvent.php',
    'Sabre\\VObject\\Component\\VFreeBusy' => $vendorDir . '/sabre/vobject/lib/Component/VFreeBusy.php',
    'Sabre\\VObject\\Component\\VJournal' => $vendorDir . '/sabre/vobject/lib/Component/VJournal.php',
    'Sabre\\VObject\\Component\\VTimeZone' => $vendorDir . '/sabre/vobject/lib/Component/VTimeZone.php',
    'Sabre\\VObject\\Component\\VTodo' => $vendorDir . '/sabre/vobject/lib/Component/VTodo.php',
    'Sabre\\VObject\\DateTimeParser' => $vendorDir . '/sabre/vobject/lib/DateTimeParser.php',
    'Sabre\\VObject\\Document' => $vendorDir . '/sabre/vobject/lib/Document.php',
    'Sabre\\VObject\\ElementList' => $vendorDir . '/sabre/vobject/lib/ElementList.php',
    'Sabre\\VObject\\EofException' => $vendorDir . '/sabre/vobject/lib/EofException.php',
    'Sabre\\VObject\\FreeBusyData' => $vendorDir . '/sabre/vobject/lib/FreeBusyData.php',
    'Sabre\\VObject\\FreeBusyGenerator' => $vendorDir . '/sabre/vobject/lib/FreeBusyGenerator.php',
    'Sabre\\VObject\\ITip\\Broker' => $vendorDir . '/sabre/vobject/lib/ITip/Broker.php',
    'Sabre\\VObject\\ITip\\ITipException' => $vendorDir . '/sabre/vobject/lib/ITip/ITipException.php',
    'Sabre\\VObject\\ITip\\Message' => $vendorDir . '/sabre/vobject/lib/ITip/Message.php',
    'Sabre\\VObject\\ITip\\SameOrganizerForAllComponentsException' => $vendorDir . '/sabre/vobject/lib/ITip/SameOrganizerForAllComponentsException.php',
    'Sabre\\VObject\\InvalidDataException' => $vendorDir . '/sabre/vobject/lib/InvalidDataException.php',
    'Sabre\\VObject\\Node' => $vendorDir . '/sabre/vobject/lib/Node.php',
    'Sabre\\VObject\\PHPUnitAssertions' => $vendorDir . '/sabre/vobject/lib/PHPUnitAssertions.php',
    'Sabre\\VObject\\Parameter' => $vendorDir . '/sabre/vobject/lib/Parameter.php',
    'Sabre\\VObject\\ParseException' => $vendorDir . '/sabre/vobject/lib/ParseException.php',
    'Sabre\\VObject\\Parser\\Json' => $vendorDir . '/sabre/vobject/lib/Parser/Json.php',
    'Sabre\\VObject\\Parser\\MimeDir' => $vendorDir . '/sabre/vobject/lib/Parser/MimeDir.php',
    'Sabre\\VObject\\Parser\\Parser' => $vendorDir . '/sabre/vobject/lib/Parser/Parser.php',
    'Sabre\\VObject\\Parser\\XML' => $vendorDir . '/sabre/vobject/lib/Parser/XML.php',
    'Sabre\\VObject\\Parser\\XML\\Element\\KeyValue' => $vendorDir . '/sabre/vobject/lib/Parser/XML/Element/KeyValue.php',
    'Sabre\\VObject\\Property' => $vendorDir . '/sabre/vobject/lib/Property.php',
    'Sabre\\VObject\\Property\\Binary' => $vendorDir . '/sabre/vobject/lib/Property/Binary.php',
    'Sabre\\VObject\\Property\\Boolean' => $vendorDir . '/sabre/vobject/lib/Property/Boolean.php',
    'Sabre\\VObject\\Property\\FlatText' => $vendorDir . '/sabre/vobject/lib/Property/FlatText.php',
    'Sabre\\VObject\\Property\\FloatValue' => $vendorDir . '/sabre/vobject/lib/Property/FloatValue.php',
    'Sabre\\VObject\\Property\\ICalendar\\CalAddress' => $vendorDir . '/sabre/vobject/lib/Property/ICalendar/CalAddress.php',
    'Sabre\\VObject\\Property\\ICalendar\\Date' => $vendorDir . '/sabre/vobject/lib/Property/ICalendar/Date.php',
    'Sabre\\VObject\\Property\\ICalendar\\DateTime' => $vendorDir . '/sabre/vobject/lib/Property/ICalendar/DateTime.php',
    'Sabre\\VObject\\Property\\ICalendar\\Duration' => $vendorDir . '/sabre/vobject/lib/Property/ICalendar/Duration.php',
    'Sabre\\VObject\\Property\\ICalendar\\Period' => $vendorDir . '/sabre/vobject/lib/Property/ICalendar/Period.php',
    'Sabre\\VObject\\Property\\ICalendar\\Recur' => $vendorDir . '/sabre/vobject/lib/Property/ICalendar/Recur.php',
    'Sabre\\VObject\\Property\\IntegerValue' => $vendorDir . '/sabre/vobject/lib/Property/IntegerValue.php',
    'Sabre\\VObject\\Property\\Text' => $vendorDir . '/sabre/vobject/lib/Property/Text.php',
    'Sabre\\VObject\\Property\\Time' => $vendorDir . '/sabre/vobject/lib/Property/Time.php',
    'Sabre\\VObject\\Property\\Unknown' => $vendorDir . '/sabre/vobject/lib/Property/Unknown.php',
    'Sabre\\VObject\\Property\\Uri' => $vendorDir . '/sabre/vobject/lib/Property/Uri.php',
    'Sabre\\VObject\\Property\\UtcOffset' => $vendorDir . '/sabre/vobject/lib/Property/UtcOffset.php',
    'Sabre\\VObject\\Property\\VCard\\Date' => $vendorDir . '/sabre/vobject/lib/Property/VCard/Date.php',
    'Sabre\\VObject\\Property\\VCard\\DateAndOrTime' => $vendorDir . '/sabre/vobject/lib/Property/VCard/DateAndOrTime.php',
    'Sabre\\VObject\\Property\\VCard\\DateTime' => $vendorDir . '/sabre/vobject/lib/Property/VCard/DateTime.php',
    'Sabre\\VObject\\Property\\VCard\\LanguageTag' => $vendorDir . '/sabre/vobject/lib/Property/VCard/LanguageTag.php',
    'Sabre\\VObject\\Property\\VCard\\PhoneNumber' => $vendorDir . '/sabre/vobject/lib/Property/VCard/PhoneNumber.php',
    'Sabre\\VObject\\Property\\VCard\\TimeStamp' => $vendorDir . '/sabre/vobject/lib/Property/VCard/TimeStamp.php',
    'Sabre\\VObject\\Reader' => $vendorDir . '/sabre/vobject/lib/Reader.php',
    'Sabre\\VObject\\Recur\\EventIterator' => $vendorDir . '/sabre/vobject/lib/Recur/EventIterator.php',
    'Sabre\\VObject\\Recur\\MaxInstancesExceededException' => $vendorDir . '/sabre/vobject/lib/Recur/MaxInstancesExceededException.php',
    'Sabre\\VObject\\Recur\\NoInstancesException' => $vendorDir . '/sabre/vobject/lib/Recur/NoInstancesException.php',
    'Sabre\\VObject\\Recur\\RDateIterator' => $vendorDir . '/sabre/vobject/lib/Recur/RDateIterator.php',
    'Sabre\\VObject\\Recur\\RRuleIterator' => $vendorDir . '/sabre/vobject/lib/Recur/RRuleIterator.php',
    'Sabre\\VObject\\Settings' => $vendorDir . '/sabre/vobject/lib/Settings.php',
    'Sabre\\VObject\\Splitter\\ICalendar' => $vendorDir . '/sabre/vobject/lib/Splitter/ICalendar.php',
    'Sabre\\VObject\\Splitter\\SplitterInterface' => $vendorDir . '/sabre/vobject/lib/Splitter/SplitterInterface.php',
    'Sabre\\VObject\\Splitter\\VCard' => $vendorDir . '/sabre/vobject/lib/Splitter/VCard.php',
    'Sabre\\VObject\\StringUtil' => $vendorDir . '/sabre/vobject/lib/StringUtil.php',
    'Sabre\\VObject\\TimeZoneUtil' => $vendorDir . '/sabre/vobject/lib/TimeZoneUtil.php',
    'Sabre\\VObject\\TimezoneGuesser\\FindFromOffset' => $vendorDir . '/sabre/vobject/lib/TimezoneGuesser/FindFromOffset.php',
    'Sabre\\VObject\\TimezoneGuesser\\FindFromTimezoneIdentifier' => $vendorDir . '/sabre/vobject/lib/TimezoneGuesser/FindFromTimezoneIdentifier.php',
    'Sabre\\VObject\\TimezoneGuesser\\FindFromTimezoneMap' => $vendorDir . '/sabre/vobject/lib/TimezoneGuesser/FindFromTimezoneMap.php',
    'Sabre\\VObject\\TimezoneGuesser\\GuessFromLicEntry' => $vendorDir . '/sabre/vobject/lib/TimezoneGuesser/GuessFromLicEntry.php',
    'Sabre\\VObject\\TimezoneGuesser\\GuessFromMsTzId' => $vendorDir . '/sabre/vobject/lib/TimezoneGuesser/GuessFromMsTzId.php',
    'Sabre\\VObject\\TimezoneGuesser\\TimezoneFinder' => $vendorDir . '/sabre/vobject/lib/TimezoneGuesser/TimezoneFinder.php',
    'Sabre\\VObject\\TimezoneGuesser\\TimezoneGuesser' => $vendorDir . '/sabre/vobject/lib/TimezoneGuesser/TimezoneGuesser.php',
    'Sabre\\VObject\\UUIDUtil' => $vendorDir . '/sabre/vobject/lib/UUIDUtil.php',
    'Sabre\\VObject\\VCardConverter' => $vendorDir . '/sabre/vobject/lib/VCardConverter.php',
    'Sabre\\VObject\\Version' => $vendorDir . '/sabre/vobject/lib/Version.php',
    'Sabre\\VObject\\Writer' => $vendorDir . '/sabre/vobject/lib/Writer.php',
    'Sabre\\Xml\\ContextStackTrait' => $vendorDir . '/sabre/xml/lib/ContextStackTrait.php',
    'Sabre\\Xml\\Element' => $vendorDir . '/sabre/xml/lib/Element.php',
    'Sabre\\Xml\\Element\\Base' => $vendorDir . '/sabre/xml/lib/Element/Base.php',
    'Sabre\\Xml\\Element\\Cdata' => $vendorDir . '/sabre/xml/lib/Element/Cdata.php',
    'Sabre\\Xml\\Element\\Elements' => $vendorDir . '/sabre/xml/lib/Element/Elements.php',
    'Sabre\\Xml\\Element\\KeyValue' => $vendorDir . '/sabre/xml/lib/Element/KeyValue.php',
    'Sabre\\Xml\\Element\\Uri' => $vendorDir . '/sabre/xml/lib/Element/Uri.php',
    'Sabre\\Xml\\Element\\XmlFragment' => $vendorDir . '/sabre/xml/lib/Element/XmlFragment.php',
    'Sabre\\Xml\\LibXMLException' => $vendorDir . '/sabre/xml/lib/LibXMLException.php',
    'Sabre\\Xml\\ParseException' => $vendorDir . '/sabre/xml/lib/ParseException.php',
    'Sabre\\Xml\\Reader' => $vendorDir . '/sabre/xml/lib/Reader.php',
    'Sabre\\Xml\\Service' => $vendorDir . '/sabre/xml/lib/Service.php',
    'Sabre\\Xml\\Version' => $vendorDir . '/sabre/xml/lib/Version.php',
    'Sabre\\Xml\\Writer' => $vendorDir . '/sabre/xml/lib/Writer.php',
    'Sabre\\Xml\\XmlDeserializable' => $vendorDir . '/sabre/xml/lib/XmlDeserializable.php',
    'Sabre\\Xml\\XmlSerializable' => $vendorDir . '/sabre/xml/lib/XmlSerializable.php',
    'Safe\\DateTime' => $vendorDir . '/thecodingmachine/safe/lib/DateTime.php',
    'Safe\\DateTimeImmutable' => $vendorDir . '/thecodingmachine/safe/lib/DateTimeImmutable.php',
    'Safe\\Exceptions\\ApacheException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/ApacheException.php',
    'Safe\\Exceptions\\ApcuException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/ApcuException.php',
    'Safe\\Exceptions\\ArrayException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/ArrayException.php',
    'Safe\\Exceptions\\Bzip2Exception' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/Bzip2Exception.php',
    'Safe\\Exceptions\\CalendarException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/CalendarException.php',
    'Safe\\Exceptions\\ClassobjException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/ClassobjException.php',
    'Safe\\Exceptions\\ComException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/ComException.php',
    'Safe\\Exceptions\\CubridException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/CubridException.php',
    'Safe\\Exceptions\\CurlException' => $vendorDir . '/thecodingmachine/safe/lib/Exceptions/CurlException.php',
    'Safe\\Exceptions\\DatetimeException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/DatetimeException.php',
    'Safe\\Exceptions\\DirException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/DirException.php',
    'Safe\\Exceptions\\EioException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/EioException.php',
    'Safe\\Exceptions\\ErrorfuncException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/ErrorfuncException.php',
    'Safe\\Exceptions\\ExecException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/ExecException.php',
    'Safe\\Exceptions\\FileinfoException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/FileinfoException.php',
    'Safe\\Exceptions\\FilesystemException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/FilesystemException.php',
    'Safe\\Exceptions\\FilterException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/FilterException.php',
    'Safe\\Exceptions\\FpmException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/FpmException.php',
    'Safe\\Exceptions\\FtpException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/FtpException.php',
    'Safe\\Exceptions\\FunchandException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/FunchandException.php',
    'Safe\\Exceptions\\GettextException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/GettextException.php',
    'Safe\\Exceptions\\GmpException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/GmpException.php',
    'Safe\\Exceptions\\GnupgException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/GnupgException.php',
    'Safe\\Exceptions\\HashException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/HashException.php',
    'Safe\\Exceptions\\IbaseException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/IbaseException.php',
    'Safe\\Exceptions\\IbmDb2Exception' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/IbmDb2Exception.php',
    'Safe\\Exceptions\\IconvException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/IconvException.php',
    'Safe\\Exceptions\\ImageException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/ImageException.php',
    'Safe\\Exceptions\\ImapException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/ImapException.php',
    'Safe\\Exceptions\\InfoException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/InfoException.php',
    'Safe\\Exceptions\\InotifyException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/InotifyException.php',
    'Safe\\Exceptions\\JsonException' => $vendorDir . '/thecodingmachine/safe/lib/Exceptions/JsonException.php',
    'Safe\\Exceptions\\LdapException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/LdapException.php',
    'Safe\\Exceptions\\LibxmlException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/LibxmlException.php',
    'Safe\\Exceptions\\LzfException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/LzfException.php',
    'Safe\\Exceptions\\MailparseException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/MailparseException.php',
    'Safe\\Exceptions\\MbstringException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/MbstringException.php',
    'Safe\\Exceptions\\MiscException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/MiscException.php',
    'Safe\\Exceptions\\MysqlException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/MysqlException.php',
    'Safe\\Exceptions\\MysqliException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/MysqliException.php',
    'Safe\\Exceptions\\NetworkException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/NetworkException.php',
    'Safe\\Exceptions\\Oci8Exception' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/Oci8Exception.php',
    'Safe\\Exceptions\\OpcacheException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/OpcacheException.php',
    'Safe\\Exceptions\\OpensslException' => $vendorDir . '/thecodingmachine/safe/lib/Exceptions/OpensslException.php',
    'Safe\\Exceptions\\OutcontrolException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/OutcontrolException.php',
    'Safe\\Exceptions\\PcntlException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/PcntlException.php',
    'Safe\\Exceptions\\PcreException' => $vendorDir . '/thecodingmachine/safe/lib/Exceptions/PcreException.php',
    'Safe\\Exceptions\\PgsqlException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/PgsqlException.php',
    'Safe\\Exceptions\\PosixException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/PosixException.php',
    'Safe\\Exceptions\\PsException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/PsException.php',
    'Safe\\Exceptions\\PspellException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/PspellException.php',
    'Safe\\Exceptions\\ReadlineException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/ReadlineException.php',
    'Safe\\Exceptions\\RnpException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/RnpException.php',
    'Safe\\Exceptions\\RpminfoException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/RpminfoException.php',
    'Safe\\Exceptions\\RrdException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/RrdException.php',
    'Safe\\Exceptions\\SafeExceptionInterface' => $vendorDir . '/thecodingmachine/safe/lib/Exceptions/SafeExceptionInterface.php',
    'Safe\\Exceptions\\SemException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/SemException.php',
    'Safe\\Exceptions\\SessionException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/SessionException.php',
    'Safe\\Exceptions\\ShmopException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/ShmopException.php',
    'Safe\\Exceptions\\SimplexmlException' => $vendorDir . '/thecodingmachine/safe/lib/Exceptions/SimplexmlException.php',
    'Safe\\Exceptions\\SocketsException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/SocketsException.php',
    'Safe\\Exceptions\\SodiumException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/SodiumException.php',
    'Safe\\Exceptions\\SolrException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/SolrException.php',
    'Safe\\Exceptions\\SplException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/SplException.php',
    'Safe\\Exceptions\\SqlsrvException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/SqlsrvException.php',
    'Safe\\Exceptions\\SsdeepException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/SsdeepException.php',
    'Safe\\Exceptions\\Ssh2Exception' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/Ssh2Exception.php',
    'Safe\\Exceptions\\StreamException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/StreamException.php',
    'Safe\\Exceptions\\StringsException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/StringsException.php',
    'Safe\\Exceptions\\SwooleException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/SwooleException.php',
    'Safe\\Exceptions\\UodbcException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/UodbcException.php',
    'Safe\\Exceptions\\UopzException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/UopzException.php',
    'Safe\\Exceptions\\UrlException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/UrlException.php',
    'Safe\\Exceptions\\VarException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/VarException.php',
    'Safe\\Exceptions\\XdiffException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/XdiffException.php',
    'Safe\\Exceptions\\XmlException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/XmlException.php',
    'Safe\\Exceptions\\XmlrpcException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/XmlrpcException.php',
    'Safe\\Exceptions\\YamlException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/YamlException.php',
    'Safe\\Exceptions\\YazException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/YazException.php',
    'Safe\\Exceptions\\ZipException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/ZipException.php',
    'Safe\\Exceptions\\ZlibException' => $vendorDir . '/thecodingmachine/safe/generated/Exceptions/ZlibException.php',
    'SavedSearch' => $baseDir . '/src/SavedSearch.php',
    'SavedSearch_Alert' => $baseDir . '/src/SavedSearch_Alert.php',
    'SavedSearch_User' => $baseDir . '/src/SavedSearch_User.php',
    'ScssPhp\\ScssPhp\\Ast\\AstNode' => $vendorDir . '/scssphp/scssphp/src/Ast/AstNode.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\CssAtRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/CssAtRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\CssComment' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/CssComment.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\CssDeclaration' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/CssDeclaration.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\CssImport' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/CssImport.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\CssKeyframeBlock' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/CssKeyframeBlock.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\CssMediaQuery' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/CssMediaQuery.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\CssMediaRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/CssMediaRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\CssNode' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/CssNode.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\CssParentNode' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/CssParentNode.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\CssStyleRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/CssStyleRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\CssStylesheet' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/CssStylesheet.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\CssSupportsRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/CssSupportsRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\CssValue' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/CssValue.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\IsInvisibleVisitor' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/IsInvisibleVisitor.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\MediaQueryMergeResult' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/MediaQueryMergeResult.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\MediaQuerySingletonMergeResult' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/MediaQuerySingletonMergeResult.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\ModifiableCssAtRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/ModifiableCssAtRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\ModifiableCssComment' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/ModifiableCssComment.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\ModifiableCssDeclaration' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/ModifiableCssDeclaration.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\ModifiableCssImport' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/ModifiableCssImport.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\ModifiableCssKeyframeBlock' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/ModifiableCssKeyframeBlock.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\ModifiableCssMediaRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/ModifiableCssMediaRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\ModifiableCssNode' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/ModifiableCssNode.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\ModifiableCssParentNode' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/ModifiableCssParentNode.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\ModifiableCssStyleRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/ModifiableCssStyleRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\ModifiableCssStylesheet' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/ModifiableCssStylesheet.php',
    'ScssPhp\\ScssPhp\\Ast\\Css\\ModifiableCssSupportsRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Css/ModifiableCssSupportsRule.php',
    'ScssPhp\\ScssPhp\\Ast\\FakeAstNode' => $vendorDir . '/scssphp/scssphp/src/Ast/FakeAstNode.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Argument' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Argument.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\ArgumentDeclaration' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/ArgumentDeclaration.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\ArgumentInvocation' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/ArgumentInvocation.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\AtRootQuery' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/AtRootQuery.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\CallableInvocation' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/CallableInvocation.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\ConfiguredVariable' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/ConfiguredVariable.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\BinaryOperationExpression' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/BinaryOperationExpression.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\BinaryOperator' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/BinaryOperator.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\BooleanExpression' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/BooleanExpression.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\ColorExpression' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/ColorExpression.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\FunctionExpression' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/FunctionExpression.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\IfExpression' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/IfExpression.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\InterpolatedFunctionExpression' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/InterpolatedFunctionExpression.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\IsCalculationSafeVisitor' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/IsCalculationSafeVisitor.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\ListExpression' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/ListExpression.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\MapExpression' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/MapExpression.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\NullExpression' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/NullExpression.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\NumberExpression' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/NumberExpression.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\ParenthesizedExpression' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/ParenthesizedExpression.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\SelectorExpression' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/SelectorExpression.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\StringExpression' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/StringExpression.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\SupportsExpression' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/SupportsExpression.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\UnaryOperationExpression' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/UnaryOperationExpression.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\UnaryOperator' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/UnaryOperator.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\ValueExpression' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/ValueExpression.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Expression\\VariableExpression' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Expression/VariableExpression.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Import' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Import.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Import\\DynamicImport' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Import/DynamicImport.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Import\\StaticImport' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Import/StaticImport.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Interpolation' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Interpolation.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\SassDeclaration' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/SassDeclaration.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\SassNode' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/SassNode.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\SassReference' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/SassReference.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\AtRootRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/AtRootRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\AtRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/AtRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\CallableDeclaration' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/CallableDeclaration.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\ContentBlock' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/ContentBlock.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\ContentRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/ContentRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\DebugRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/DebugRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\Declaration' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/Declaration.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\EachRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/EachRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\ElseClause' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/ElseClause.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\ErrorRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/ErrorRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\ExtendRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/ExtendRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\ForRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/ForRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\FunctionRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/FunctionRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\HasContentVisitor' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/HasContentVisitor.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\IfClause' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/IfClause.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\IfRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/IfRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\IfRuleClause' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/IfRuleClause.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\ImportRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/ImportRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\IncludeRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/IncludeRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\LoudComment' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/LoudComment.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\MediaRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/MediaRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\MixinRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/MixinRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\ParentStatement' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/ParentStatement.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\ReturnRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/ReturnRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\SilentComment' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/SilentComment.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\StyleRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/StyleRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\Stylesheet' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/Stylesheet.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\SupportsRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/SupportsRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\VariableDeclaration' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/VariableDeclaration.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\WarnRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/WarnRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\Statement\\WhileRule' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/Statement/WhileRule.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\SupportsCondition' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/SupportsCondition.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\SupportsCondition\\SupportsAnything' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/SupportsCondition/SupportsAnything.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\SupportsCondition\\SupportsDeclaration' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/SupportsCondition/SupportsDeclaration.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\SupportsCondition\\SupportsFunction' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/SupportsCondition/SupportsFunction.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\SupportsCondition\\SupportsInterpolation' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/SupportsCondition/SupportsInterpolation.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\SupportsCondition\\SupportsNegation' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/SupportsCondition/SupportsNegation.php',
    'ScssPhp\\ScssPhp\\Ast\\Sass\\SupportsCondition\\SupportsOperation' => $vendorDir . '/scssphp/scssphp/src/Ast/Sass/SupportsCondition/SupportsOperation.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\AttributeOperator' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/AttributeOperator.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\AttributeSelector' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/AttributeSelector.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\ClassSelector' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/ClassSelector.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\Combinator' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/Combinator.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\ComplexSelector' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/ComplexSelector.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\ComplexSelectorComponent' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/ComplexSelectorComponent.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\CompoundSelector' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/CompoundSelector.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\IDSelector' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/IDSelector.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\IsBogusVisitor' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/IsBogusVisitor.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\IsInvisibleVisitor' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/IsInvisibleVisitor.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\IsUselessVisitor' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/IsUselessVisitor.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\ParentSelector' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/ParentSelector.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\ParentSelectorVisitor' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/ParentSelectorVisitor.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\PlaceholderSelector' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/PlaceholderSelector.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\PseudoSelector' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/PseudoSelector.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\QualifiedName' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/QualifiedName.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\Selector' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/Selector.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\SelectorList' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/SelectorList.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\SimpleSelector' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/SimpleSelector.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\TypeSelector' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/TypeSelector.php',
    'ScssPhp\\ScssPhp\\Ast\\Selector\\UniversalSelector' => $vendorDir . '/scssphp/scssphp/src/Ast/Selector/UniversalSelector.php',
    'ScssPhp\\ScssPhp\\Collection\\Map' => $vendorDir . '/scssphp/scssphp/src/Collection/Map.php',
    'ScssPhp\\ScssPhp\\Colors' => $vendorDir . '/scssphp/scssphp/src/Colors.php',
    'ScssPhp\\ScssPhp\\CompilationResult' => $vendorDir . '/scssphp/scssphp/src/CompilationResult.php',
    'ScssPhp\\ScssPhp\\Compiler' => $vendorDir . '/scssphp/scssphp/src/Compiler.php',
    'ScssPhp\\ScssPhp\\Compiler\\LegacyValueVisitor' => $vendorDir . '/scssphp/scssphp/src/Compiler/LegacyValueVisitor.php',
    'ScssPhp\\ScssPhp\\Deprecation' => $vendorDir . '/scssphp/scssphp/src/Deprecation.php',
    'ScssPhp\\ScssPhp\\DeprecationStatus' => $vendorDir . '/scssphp/scssphp/src/DeprecationStatus.php',
    'ScssPhp\\ScssPhp\\Evaluation\\ArgumentResults' => $vendorDir . '/scssphp/scssphp/src/Evaluation/ArgumentResults.php',
    'ScssPhp\\ScssPhp\\Evaluation\\Environment' => $vendorDir . '/scssphp/scssphp/src/Evaluation/Environment.php',
    'ScssPhp\\ScssPhp\\Evaluation\\EvaluateResult' => $vendorDir . '/scssphp/scssphp/src/Evaluation/EvaluateResult.php',
    'ScssPhp\\ScssPhp\\Evaluation\\EvaluateVisitor' => $vendorDir . '/scssphp/scssphp/src/Evaluation/EvaluateVisitor.php',
    'ScssPhp\\ScssPhp\\Evaluation\\EvaluationContext' => $vendorDir . '/scssphp/scssphp/src/Evaluation/EvaluationContext.php',
    'ScssPhp\\ScssPhp\\Evaluation\\LoadedStylesheet' => $vendorDir . '/scssphp/scssphp/src/Evaluation/LoadedStylesheet.php',
    'ScssPhp\\ScssPhp\\Evaluation\\VisitorEvaluationContext' => $vendorDir . '/scssphp/scssphp/src/Evaluation/VisitorEvaluationContext.php',
    'ScssPhp\\ScssPhp\\Exception\\MultiSpanSassException' => $vendorDir . '/scssphp/scssphp/src/Exception/MultiSpanSassException.php',
    'ScssPhp\\ScssPhp\\Exception\\MultiSpanSassFormatException' => $vendorDir . '/scssphp/scssphp/src/Exception/MultiSpanSassFormatException.php',
    'ScssPhp\\ScssPhp\\Exception\\MultiSpanSassRuntimeException' => $vendorDir . '/scssphp/scssphp/src/Exception/MultiSpanSassRuntimeException.php',
    'ScssPhp\\ScssPhp\\Exception\\MultiSpanSassScriptException' => $vendorDir . '/scssphp/scssphp/src/Exception/MultiSpanSassScriptException.php',
    'ScssPhp\\ScssPhp\\Exception\\SassException' => $vendorDir . '/scssphp/scssphp/src/Exception/SassException.php',
    'ScssPhp\\ScssPhp\\Exception\\SassFormatException' => $vendorDir . '/scssphp/scssphp/src/Exception/SassFormatException.php',
    'ScssPhp\\ScssPhp\\Exception\\SassRuntimeException' => $vendorDir . '/scssphp/scssphp/src/Exception/SassRuntimeException.php',
    'ScssPhp\\ScssPhp\\Exception\\SassScriptException' => $vendorDir . '/scssphp/scssphp/src/Exception/SassScriptException.php',
    'ScssPhp\\ScssPhp\\Exception\\SimpleSassException' => $vendorDir . '/scssphp/scssphp/src/Exception/SimpleSassException.php',
    'ScssPhp\\ScssPhp\\Exception\\SimpleSassFormatException' => $vendorDir . '/scssphp/scssphp/src/Exception/SimpleSassFormatException.php',
    'ScssPhp\\ScssPhp\\Exception\\SimpleSassRuntimeException' => $vendorDir . '/scssphp/scssphp/src/Exception/SimpleSassRuntimeException.php',
    'ScssPhp\\ScssPhp\\Extend\\ComplexSelectorMap' => $vendorDir . '/scssphp/scssphp/src/Extend/ComplexSelectorMap.php',
    'ScssPhp\\ScssPhp\\Extend\\ConcreteExtensionStore' => $vendorDir . '/scssphp/scssphp/src/Extend/ConcreteExtensionStore.php',
    'ScssPhp\\ScssPhp\\Extend\\EmptyExtensionStore' => $vendorDir . '/scssphp/scssphp/src/Extend/EmptyExtensionStore.php',
    'ScssPhp\\ScssPhp\\Extend\\ExtendMode' => $vendorDir . '/scssphp/scssphp/src/Extend/ExtendMode.php',
    'ScssPhp\\ScssPhp\\Extend\\ExtendUtil' => $vendorDir . '/scssphp/scssphp/src/Extend/ExtendUtil.php',
    'ScssPhp\\ScssPhp\\Extend\\Extender' => $vendorDir . '/scssphp/scssphp/src/Extend/Extender.php',
    'ScssPhp\\ScssPhp\\Extend\\Extension' => $vendorDir . '/scssphp/scssphp/src/Extend/Extension.php',
    'ScssPhp\\ScssPhp\\Extend\\ExtensionStore' => $vendorDir . '/scssphp/scssphp/src/Extend/ExtensionStore.php',
    'ScssPhp\\ScssPhp\\Extend\\MergedExtension' => $vendorDir . '/scssphp/scssphp/src/Extend/MergedExtension.php',
    'ScssPhp\\ScssPhp\\Extend\\ObjectSet' => $vendorDir . '/scssphp/scssphp/src/Extend/ObjectSet.php',
    'ScssPhp\\ScssPhp\\Extend\\SimpleSelectorMap' => $vendorDir . '/scssphp/scssphp/src/Extend/SimpleSelectorMap.php',
    'ScssPhp\\ScssPhp\\Function\\ColorFunctions' => $vendorDir . '/scssphp/scssphp/src/Function/ColorFunctions.php',
    'ScssPhp\\ScssPhp\\Function\\FunctionRegistry' => $vendorDir . '/scssphp/scssphp/src/Function/FunctionRegistry.php',
    'ScssPhp\\ScssPhp\\Function\\ListFunctions' => $vendorDir . '/scssphp/scssphp/src/Function/ListFunctions.php',
    'ScssPhp\\ScssPhp\\Function\\MapFunctions' => $vendorDir . '/scssphp/scssphp/src/Function/MapFunctions.php',
    'ScssPhp\\ScssPhp\\Function\\MathFunctions' => $vendorDir . '/scssphp/scssphp/src/Function/MathFunctions.php',
    'ScssPhp\\ScssPhp\\Function\\MetaFunctions' => $vendorDir . '/scssphp/scssphp/src/Function/MetaFunctions.php',
    'ScssPhp\\ScssPhp\\Function\\SelectorFunctions' => $vendorDir . '/scssphp/scssphp/src/Function/SelectorFunctions.php',
    'ScssPhp\\ScssPhp\\Function\\StringFunctions' => $vendorDir . '/scssphp/scssphp/src/Function/StringFunctions.php',
    'ScssPhp\\ScssPhp\\Importer\\CanonicalizeContext' => $vendorDir . '/scssphp/scssphp/src/Importer/CanonicalizeContext.php',
    'ScssPhp\\ScssPhp\\Importer\\CanonicalizeResult' => $vendorDir . '/scssphp/scssphp/src/Importer/CanonicalizeResult.php',
    'ScssPhp\\ScssPhp\\Importer\\FilesystemImporter' => $vendorDir . '/scssphp/scssphp/src/Importer/FilesystemImporter.php',
    'ScssPhp\\ScssPhp\\Importer\\ImportCache' => $vendorDir . '/scssphp/scssphp/src/Importer/ImportCache.php',
    'ScssPhp\\ScssPhp\\Importer\\ImportContext' => $vendorDir . '/scssphp/scssphp/src/Importer/ImportContext.php',
    'ScssPhp\\ScssPhp\\Importer\\ImportUtil' => $vendorDir . '/scssphp/scssphp/src/Importer/ImportUtil.php',
    'ScssPhp\\ScssPhp\\Importer\\Importer' => $vendorDir . '/scssphp/scssphp/src/Importer/Importer.php',
    'ScssPhp\\ScssPhp\\Importer\\ImporterResult' => $vendorDir . '/scssphp/scssphp/src/Importer/ImporterResult.php',
    'ScssPhp\\ScssPhp\\Importer\\LegacyCallbackImporter' => $vendorDir . '/scssphp/scssphp/src/Importer/LegacyCallbackImporter.php',
    'ScssPhp\\ScssPhp\\Importer\\NoOpImporter' => $vendorDir . '/scssphp/scssphp/src/Importer/NoOpImporter.php',
    'ScssPhp\\ScssPhp\\Importer\\SpecialCacheValue' => $vendorDir . '/scssphp/scssphp/src/Importer/SpecialCacheValue.php',
    'ScssPhp\\ScssPhp\\Logger\\DeprecationProcessingLogger' => $vendorDir . '/scssphp/scssphp/src/Logger/DeprecationProcessingLogger.php',
    'ScssPhp\\ScssPhp\\Logger\\LoggerInterface' => $vendorDir . '/scssphp/scssphp/src/Logger/LoggerInterface.php',
    'ScssPhp\\ScssPhp\\Logger\\QuietLogger' => $vendorDir . '/scssphp/scssphp/src/Logger/QuietLogger.php',
    'ScssPhp\\ScssPhp\\Logger\\StreamLogger' => $vendorDir . '/scssphp/scssphp/src/Logger/StreamLogger.php',
    'ScssPhp\\ScssPhp\\Node' => $vendorDir . '/scssphp/scssphp/src/Node.php',
    'ScssPhp\\ScssPhp\\Node\\Number' => $vendorDir . '/scssphp/scssphp/src/Node/Number.php',
    'ScssPhp\\ScssPhp\\OutputStyle' => $vendorDir . '/scssphp/scssphp/src/OutputStyle.php',
    'ScssPhp\\ScssPhp\\Parser\\AtRootQueryParser' => $vendorDir . '/scssphp/scssphp/src/Parser/AtRootQueryParser.php',
    'ScssPhp\\ScssPhp\\Parser\\CssParser' => $vendorDir . '/scssphp/scssphp/src/Parser/CssParser.php',
    'ScssPhp\\ScssPhp\\Parser\\FormatException' => $vendorDir . '/scssphp/scssphp/src/Parser/FormatException.php',
    'ScssPhp\\ScssPhp\\Parser\\InterpolationBuffer' => $vendorDir . '/scssphp/scssphp/src/Parser/InterpolationBuffer.php',
    'ScssPhp\\ScssPhp\\Parser\\InterpolationMap' => $vendorDir . '/scssphp/scssphp/src/Parser/InterpolationMap.php',
    'ScssPhp\\ScssPhp\\Parser\\KeyframeSelectorParser' => $vendorDir . '/scssphp/scssphp/src/Parser/KeyframeSelectorParser.php',
    'ScssPhp\\ScssPhp\\Parser\\LineScanner' => $vendorDir . '/scssphp/scssphp/src/Parser/LineScanner.php',
    'ScssPhp\\ScssPhp\\Parser\\MediaQueryParser' => $vendorDir . '/scssphp/scssphp/src/Parser/MediaQueryParser.php',
    'ScssPhp\\ScssPhp\\Parser\\MultiSourceFormatException' => $vendorDir . '/scssphp/scssphp/src/Parser/MultiSourceFormatException.php',
    'ScssPhp\\ScssPhp\\Parser\\Parser' => $vendorDir . '/scssphp/scssphp/src/Parser/Parser.php',
    'ScssPhp\\ScssPhp\\Parser\\SassParser' => $vendorDir . '/scssphp/scssphp/src/Parser/SassParser.php',
    'ScssPhp\\ScssPhp\\Parser\\ScssParser' => $vendorDir . '/scssphp/scssphp/src/Parser/ScssParser.php',
    'ScssPhp\\ScssPhp\\Parser\\SelectorParser' => $vendorDir . '/scssphp/scssphp/src/Parser/SelectorParser.php',
    'ScssPhp\\ScssPhp\\Parser\\StringScanner' => $vendorDir . '/scssphp/scssphp/src/Parser/StringScanner.php',
    'ScssPhp\\ScssPhp\\Parser\\StylesheetParser' => $vendorDir . '/scssphp/scssphp/src/Parser/StylesheetParser.php',
    'ScssPhp\\ScssPhp\\SassCallable\\BuiltInCallable' => $vendorDir . '/scssphp/scssphp/src/SassCallable/BuiltInCallable.php',
    'ScssPhp\\ScssPhp\\SassCallable\\PlainCssCallable' => $vendorDir . '/scssphp/scssphp/src/SassCallable/PlainCssCallable.php',
    'ScssPhp\\ScssPhp\\SassCallable\\SassCallable' => $vendorDir . '/scssphp/scssphp/src/SassCallable/SassCallable.php',
    'ScssPhp\\ScssPhp\\SassCallable\\UserDefinedCallable' => $vendorDir . '/scssphp/scssphp/src/SassCallable/UserDefinedCallable.php',
    'ScssPhp\\ScssPhp\\Serializer\\SerializeResult' => $vendorDir . '/scssphp/scssphp/src/Serializer/SerializeResult.php',
    'ScssPhp\\ScssPhp\\Serializer\\SerializeVisitor' => $vendorDir . '/scssphp/scssphp/src/Serializer/SerializeVisitor.php',
    'ScssPhp\\ScssPhp\\Serializer\\Serializer' => $vendorDir . '/scssphp/scssphp/src/Serializer/Serializer.php',
    'ScssPhp\\ScssPhp\\Serializer\\SimpleStringBuffer' => $vendorDir . '/scssphp/scssphp/src/Serializer/SimpleStringBuffer.php',
    'ScssPhp\\ScssPhp\\Serializer\\SourceMapBuffer' => $vendorDir . '/scssphp/scssphp/src/Serializer/SourceMapBuffer.php',
    'ScssPhp\\ScssPhp\\Serializer\\StringBuffer' => $vendorDir . '/scssphp/scssphp/src/Serializer/StringBuffer.php',
    'ScssPhp\\ScssPhp\\Serializer\\TrackingSourceMapBuffer' => $vendorDir . '/scssphp/scssphp/src/Serializer/TrackingSourceMapBuffer.php',
    'ScssPhp\\ScssPhp\\SourceMap\\Base64' => $vendorDir . '/scssphp/scssphp/src/SourceMap/Base64.php',
    'ScssPhp\\ScssPhp\\SourceMap\\Base64VLQ' => $vendorDir . '/scssphp/scssphp/src/SourceMap/Base64VLQ.php',
    'ScssPhp\\ScssPhp\\SourceMap\\Builder\\Entry' => $vendorDir . '/scssphp/scssphp/src/SourceMap/Builder/Entry.php',
    'ScssPhp\\ScssPhp\\SourceMap\\SingleMapping' => $vendorDir . '/scssphp/scssphp/src/SourceMap/SingleMapping.php',
    'ScssPhp\\ScssPhp\\SourceMap\\TargetEntry' => $vendorDir . '/scssphp/scssphp/src/SourceMap/TargetEntry.php',
    'ScssPhp\\ScssPhp\\SourceMap\\TargetLineEntry' => $vendorDir . '/scssphp/scssphp/src/SourceMap/TargetLineEntry.php',
    'ScssPhp\\ScssPhp\\SourceSpan\\LazyFileSpan' => $vendorDir . '/scssphp/scssphp/src/SourceSpan/LazyFileSpan.php',
    'ScssPhp\\ScssPhp\\SourceSpan\\MultiSpan' => $vendorDir . '/scssphp/scssphp/src/SourceSpan/MultiSpan.php',
    'ScssPhp\\ScssPhp\\StackTrace\\Frame' => $vendorDir . '/scssphp/scssphp/src/StackTrace/Frame.php',
    'ScssPhp\\ScssPhp\\StackTrace\\Trace' => $vendorDir . '/scssphp/scssphp/src/StackTrace/Trace.php',
    'ScssPhp\\ScssPhp\\Syntax' => $vendorDir . '/scssphp/scssphp/src/Syntax.php',
    'ScssPhp\\ScssPhp\\Type' => $vendorDir . '/scssphp/scssphp/src/Type.php',
    'ScssPhp\\ScssPhp\\Util' => $vendorDir . '/scssphp/scssphp/src/Util.php',
    'ScssPhp\\ScssPhp\\Util\\ArrayUtil' => $vendorDir . '/scssphp/scssphp/src/Util/ArrayUtil.php',
    'ScssPhp\\ScssPhp\\Util\\AstUtil' => $vendorDir . '/scssphp/scssphp/src/Util/AstUtil.php',
    'ScssPhp\\ScssPhp\\Util\\Box' => $vendorDir . '/scssphp/scssphp/src/Util/Box.php',
    'ScssPhp\\ScssPhp\\Util\\Character' => $vendorDir . '/scssphp/scssphp/src/Util/Character.php',
    'ScssPhp\\ScssPhp\\Util\\Equatable' => $vendorDir . '/scssphp/scssphp/src/Util/Equatable.php',
    'ScssPhp\\ScssPhp\\Util\\EquatableUtil' => $vendorDir . '/scssphp/scssphp/src/Util/EquatableUtil.php',
    'ScssPhp\\ScssPhp\\Util\\ErrorUtil' => $vendorDir . '/scssphp/scssphp/src/Util/ErrorUtil.php',
    'ScssPhp\\ScssPhp\\Util\\IterableUtil' => $vendorDir . '/scssphp/scssphp/src/Util/IterableUtil.php',
    'ScssPhp\\ScssPhp\\Util\\ListUtil' => $vendorDir . '/scssphp/scssphp/src/Util/ListUtil.php',
    'ScssPhp\\ScssPhp\\Util\\LoggerUtil' => $vendorDir . '/scssphp/scssphp/src/Util/LoggerUtil.php',
    'ScssPhp\\ScssPhp\\Util\\MakeExpressionCalculationSafe' => $vendorDir . '/scssphp/scssphp/src/Util/MakeExpressionCalculationSafe.php',
    'ScssPhp\\ScssPhp\\Util\\ModifiableBox' => $vendorDir . '/scssphp/scssphp/src/Util/ModifiableBox.php',
    'ScssPhp\\ScssPhp\\Util\\NumberUtil' => $vendorDir . '/scssphp/scssphp/src/Util/NumberUtil.php',
    'ScssPhp\\ScssPhp\\Util\\ParserUtil' => $vendorDir . '/scssphp/scssphp/src/Util/ParserUtil.php',
    'ScssPhp\\ScssPhp\\Util\\Path' => $vendorDir . '/scssphp/scssphp/src/Util/Path.php',
    'ScssPhp\\ScssPhp\\Util\\SpanUtil' => $vendorDir . '/scssphp/scssphp/src/Util/SpanUtil.php',
    'ScssPhp\\ScssPhp\\Util\\StringUtil' => $vendorDir . '/scssphp/scssphp/src/Util/StringUtil.php',
    'ScssPhp\\ScssPhp\\Util\\UriUtil' => $vendorDir . '/scssphp/scssphp/src/Util/UriUtil.php',
    'ScssPhp\\ScssPhp\\ValueConverter' => $vendorDir . '/scssphp/scssphp/src/ValueConverter.php',
    'ScssPhp\\ScssPhp\\Value\\CalculationOperation' => $vendorDir . '/scssphp/scssphp/src/Value/CalculationOperation.php',
    'ScssPhp\\ScssPhp\\Value\\CalculationOperator' => $vendorDir . '/scssphp/scssphp/src/Value/CalculationOperator.php',
    'ScssPhp\\ScssPhp\\Value\\ColorFormat' => $vendorDir . '/scssphp/scssphp/src/Value/ColorFormat.php',
    'ScssPhp\\ScssPhp\\Value\\ColorFormatEnum' => $vendorDir . '/scssphp/scssphp/src/Value/ColorFormatEnum.php',
    'ScssPhp\\ScssPhp\\Value\\ComplexSassNumber' => $vendorDir . '/scssphp/scssphp/src/Value/ComplexSassNumber.php',
    'ScssPhp\\ScssPhp\\Value\\ListSeparator' => $vendorDir . '/scssphp/scssphp/src/Value/ListSeparator.php',
    'ScssPhp\\ScssPhp\\Value\\SassArgumentList' => $vendorDir . '/scssphp/scssphp/src/Value/SassArgumentList.php',
    'ScssPhp\\ScssPhp\\Value\\SassBoolean' => $vendorDir . '/scssphp/scssphp/src/Value/SassBoolean.php',
    'ScssPhp\\ScssPhp\\Value\\SassCalculation' => $vendorDir . '/scssphp/scssphp/src/Value/SassCalculation.php',
    'ScssPhp\\ScssPhp\\Value\\SassColor' => $vendorDir . '/scssphp/scssphp/src/Value/SassColor.php',
    'ScssPhp\\ScssPhp\\Value\\SassFunction' => $vendorDir . '/scssphp/scssphp/src/Value/SassFunction.php',
    'ScssPhp\\ScssPhp\\Value\\SassList' => $vendorDir . '/scssphp/scssphp/src/Value/SassList.php',
    'ScssPhp\\ScssPhp\\Value\\SassMap' => $vendorDir . '/scssphp/scssphp/src/Value/SassMap.php',
    'ScssPhp\\ScssPhp\\Value\\SassMixin' => $vendorDir . '/scssphp/scssphp/src/Value/SassMixin.php',
    'ScssPhp\\ScssPhp\\Value\\SassNull' => $vendorDir . '/scssphp/scssphp/src/Value/SassNull.php',
    'ScssPhp\\ScssPhp\\Value\\SassNumber' => $vendorDir . '/scssphp/scssphp/src/Value/SassNumber.php',
    'ScssPhp\\ScssPhp\\Value\\SassString' => $vendorDir . '/scssphp/scssphp/src/Value/SassString.php',
    'ScssPhp\\ScssPhp\\Value\\SingleUnitSassNumber' => $vendorDir . '/scssphp/scssphp/src/Value/SingleUnitSassNumber.php',
    'ScssPhp\\ScssPhp\\Value\\SpanColorFormat' => $vendorDir . '/scssphp/scssphp/src/Value/SpanColorFormat.php',
    'ScssPhp\\ScssPhp\\Value\\UnitlessSassNumber' => $vendorDir . '/scssphp/scssphp/src/Value/UnitlessSassNumber.php',
    'ScssPhp\\ScssPhp\\Value\\Value' => $vendorDir . '/scssphp/scssphp/src/Value/Value.php',
    'ScssPhp\\ScssPhp\\Version' => $vendorDir . '/scssphp/scssphp/src/Version.php',
    'ScssPhp\\ScssPhp\\Visitor\\AnySelectorVisitor' => $vendorDir . '/scssphp/scssphp/src/Visitor/AnySelectorVisitor.php',
    'ScssPhp\\ScssPhp\\Visitor\\CssVisitor' => $vendorDir . '/scssphp/scssphp/src/Visitor/CssVisitor.php',
    'ScssPhp\\ScssPhp\\Visitor\\EveryCssVisitor' => $vendorDir . '/scssphp/scssphp/src/Visitor/EveryCssVisitor.php',
    'ScssPhp\\ScssPhp\\Visitor\\ExpressionVisitor' => $vendorDir . '/scssphp/scssphp/src/Visitor/ExpressionVisitor.php',
    'ScssPhp\\ScssPhp\\Visitor\\ModifiableCssVisitor' => $vendorDir . '/scssphp/scssphp/src/Visitor/ModifiableCssVisitor.php',
    'ScssPhp\\ScssPhp\\Visitor\\ReplaceExpressionVisitor' => $vendorDir . '/scssphp/scssphp/src/Visitor/ReplaceExpressionVisitor.php',
    'ScssPhp\\ScssPhp\\Visitor\\SelectorSearchVisitor' => $vendorDir . '/scssphp/scssphp/src/Visitor/SelectorSearchVisitor.php',
    'ScssPhp\\ScssPhp\\Visitor\\SelectorVisitor' => $vendorDir . '/scssphp/scssphp/src/Visitor/SelectorVisitor.php',
    'ScssPhp\\ScssPhp\\Visitor\\StatementSearchVisitor' => $vendorDir . '/scssphp/scssphp/src/Visitor/StatementSearchVisitor.php',
    'ScssPhp\\ScssPhp\\Visitor\\StatementVisitor' => $vendorDir . '/scssphp/scssphp/src/Visitor/StatementVisitor.php',
    'ScssPhp\\ScssPhp\\Visitor\\ValueVisitor' => $vendorDir . '/scssphp/scssphp/src/Visitor/ValueVisitor.php',
    'ScssPhp\\ScssPhp\\Warn' => $vendorDir . '/scssphp/scssphp/src/Warn.php',
    'Search' => $baseDir . '/src/Search.php',
    'SebastianBergmann\\Diff\\Chunk' => $vendorDir . '/sebastian/diff/src/Chunk.php',
    'SebastianBergmann\\Diff\\ConfigurationException' => $vendorDir . '/sebastian/diff/src/Exception/ConfigurationException.php',
    'SebastianBergmann\\Diff\\Diff' => $vendorDir . '/sebastian/diff/src/Diff.php',
    'SebastianBergmann\\Diff\\Differ' => $vendorDir . '/sebastian/diff/src/Differ.php',
    'SebastianBergmann\\Diff\\Exception' => $vendorDir . '/sebastian/diff/src/Exception/Exception.php',
    'SebastianBergmann\\Diff\\InvalidArgumentException' => $vendorDir . '/sebastian/diff/src/Exception/InvalidArgumentException.php',
    'SebastianBergmann\\Diff\\Line' => $vendorDir . '/sebastian/diff/src/Line.php',
    'SebastianBergmann\\Diff\\LongestCommonSubsequenceCalculator' => $vendorDir . '/sebastian/diff/src/LongestCommonSubsequenceCalculator.php',
    'SebastianBergmann\\Diff\\MemoryEfficientLongestCommonSubsequenceCalculator' => $vendorDir . '/sebastian/diff/src/MemoryEfficientLongestCommonSubsequenceCalculator.php',
    'SebastianBergmann\\Diff\\Output\\AbstractChunkOutputBuilder' => $vendorDir . '/sebastian/diff/src/Output/AbstractChunkOutputBuilder.php',
    'SebastianBergmann\\Diff\\Output\\DiffOnlyOutputBuilder' => $vendorDir . '/sebastian/diff/src/Output/DiffOnlyOutputBuilder.php',
    'SebastianBergmann\\Diff\\Output\\DiffOutputBuilderInterface' => $vendorDir . '/sebastian/diff/src/Output/DiffOutputBuilderInterface.php',
    'SebastianBergmann\\Diff\\Output\\StrictUnifiedDiffOutputBuilder' => $vendorDir . '/sebastian/diff/src/Output/StrictUnifiedDiffOutputBuilder.php',
    'SebastianBergmann\\Diff\\Output\\UnifiedDiffOutputBuilder' => $vendorDir . '/sebastian/diff/src/Output/UnifiedDiffOutputBuilder.php',
    'SebastianBergmann\\Diff\\Parser' => $vendorDir . '/sebastian/diff/src/Parser.php',
    'SebastianBergmann\\Diff\\TimeEfficientLongestCommonSubsequenceCalculator' => $vendorDir . '/sebastian/diff/src/TimeEfficientLongestCommonSubsequenceCalculator.php',
    'Seld\\JsonLint\\DuplicateKeyException' => $vendorDir . '/seld/jsonlint/src/Seld/JsonLint/DuplicateKeyException.php',
    'Seld\\JsonLint\\JsonParser' => $vendorDir . '/seld/jsonlint/src/Seld/JsonLint/JsonParser.php',
    'Seld\\JsonLint\\Lexer' => $vendorDir . '/seld/jsonlint/src/Seld/JsonLint/Lexer.php',
    'Seld\\JsonLint\\ParsingException' => $vendorDir . '/seld/jsonlint/src/Seld/JsonLint/ParsingException.php',
    'Seld\\JsonLint\\Undefined' => $vendorDir . '/seld/jsonlint/src/Seld/JsonLint/Undefined.php',
    'Session' => $baseDir . '/src/Session.php',
    'ShareDashboardDropdown' => $baseDir . '/src/ShareDashboardDropdown.php',
    'SimplePie' => $vendorDir . '/simplepie/simplepie/library/SimplePie.php',
    'SimplePie\\Author' => $vendorDir . '/simplepie/simplepie/src/Author.php',
    'SimplePie\\Cache' => $vendorDir . '/simplepie/simplepie/src/Cache.php',
    'SimplePie\\Cache\\Base' => $vendorDir . '/simplepie/simplepie/src/Cache/Base.php',
    'SimplePie\\Cache\\BaseDataCache' => $vendorDir . '/simplepie/simplepie/src/Cache/BaseDataCache.php',
    'SimplePie\\Cache\\CallableNameFilter' => $vendorDir . '/simplepie/simplepie/src/Cache/CallableNameFilter.php',
    'SimplePie\\Cache\\DB' => $vendorDir . '/simplepie/simplepie/src/Cache/DB.php',
    'SimplePie\\Cache\\DataCache' => $vendorDir . '/simplepie/simplepie/src/Cache/DataCache.php',
    'SimplePie\\Cache\\File' => $vendorDir . '/simplepie/simplepie/src/Cache/File.php',
    'SimplePie\\Cache\\Memcache' => $vendorDir . '/simplepie/simplepie/src/Cache/Memcache.php',
    'SimplePie\\Cache\\Memcached' => $vendorDir . '/simplepie/simplepie/src/Cache/Memcached.php',
    'SimplePie\\Cache\\MySQL' => $vendorDir . '/simplepie/simplepie/src/Cache/MySQL.php',
    'SimplePie\\Cache\\NameFilter' => $vendorDir . '/simplepie/simplepie/src/Cache/NameFilter.php',
    'SimplePie\\Cache\\Psr16' => $vendorDir . '/simplepie/simplepie/src/Cache/Psr16.php',
    'SimplePie\\Cache\\Redis' => $vendorDir . '/simplepie/simplepie/src/Cache/Redis.php',
    'SimplePie\\Caption' => $vendorDir . '/simplepie/simplepie/src/Caption.php',
    'SimplePie\\Category' => $vendorDir . '/simplepie/simplepie/src/Category.php',
    'SimplePie\\Content\\Type\\Sniffer' => $vendorDir . '/simplepie/simplepie/src/Content/Type/Sniffer.php',
    'SimplePie\\Copyright' => $vendorDir . '/simplepie/simplepie/src/Copyright.php',
    'SimplePie\\Credit' => $vendorDir . '/simplepie/simplepie/src/Credit.php',
    'SimplePie\\Enclosure' => $vendorDir . '/simplepie/simplepie/src/Enclosure.php',
    'SimplePie\\Exception' => $vendorDir . '/simplepie/simplepie/src/Exception.php',
    'SimplePie\\File' => $vendorDir . '/simplepie/simplepie/src/File.php',
    'SimplePie\\Gzdecode' => $vendorDir . '/simplepie/simplepie/src/Gzdecode.php',
    'SimplePie\\HTTP\\Client' => $vendorDir . '/simplepie/simplepie/src/HTTP/Client.php',
    'SimplePie\\HTTP\\ClientException' => $vendorDir . '/simplepie/simplepie/src/HTTP/ClientException.php',
    'SimplePie\\HTTP\\FileClient' => $vendorDir . '/simplepie/simplepie/src/HTTP/FileClient.php',
    'SimplePie\\HTTP\\Parser' => $vendorDir . '/simplepie/simplepie/src/HTTP/Parser.php',
    'SimplePie\\HTTP\\Psr18Client' => $vendorDir . '/simplepie/simplepie/src/HTTP/Psr18Client.php',
    'SimplePie\\HTTP\\Psr7Response' => $vendorDir . '/simplepie/simplepie/src/HTTP/Psr7Response.php',
    'SimplePie\\HTTP\\RawTextResponse' => $vendorDir . '/simplepie/simplepie/src/HTTP/RawTextResponse.php',
    'SimplePie\\HTTP\\Response' => $vendorDir . '/simplepie/simplepie/src/HTTP/Response.php',
    'SimplePie\\IRI' => $vendorDir . '/simplepie/simplepie/src/IRI.php',
    'SimplePie\\Item' => $vendorDir . '/simplepie/simplepie/src/Item.php',
    'SimplePie\\Locator' => $vendorDir . '/simplepie/simplepie/src/Locator.php',
    'SimplePie\\Misc' => $vendorDir . '/simplepie/simplepie/src/Misc.php',
    'SimplePie\\Net\\IPv6' => $vendorDir . '/simplepie/simplepie/src/Net/IPv6.php',
    'SimplePie\\Parse\\Date' => $vendorDir . '/simplepie/simplepie/src/Parse/Date.php',
    'SimplePie\\Parser' => $vendorDir . '/simplepie/simplepie/src/Parser.php',
    'SimplePie\\Rating' => $vendorDir . '/simplepie/simplepie/src/Rating.php',
    'SimplePie\\Registry' => $vendorDir . '/simplepie/simplepie/src/Registry.php',
    'SimplePie\\RegistryAware' => $vendorDir . '/simplepie/simplepie/src/RegistryAware.php',
    'SimplePie\\Restriction' => $vendorDir . '/simplepie/simplepie/src/Restriction.php',
    'SimplePie\\Sanitize' => $vendorDir . '/simplepie/simplepie/src/Sanitize.php',
    'SimplePie\\SimplePie' => $vendorDir . '/simplepie/simplepie/src/SimplePie.php',
    'SimplePie\\Source' => $vendorDir . '/simplepie/simplepie/src/Source.php',
    'SimplePie\\XML\\Declaration\\Parser' => $vendorDir . '/simplepie/simplepie/src/XML/Declaration/Parser.php',
    'SimplePie_Author' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Author.php',
    'SimplePie_Cache' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Cache.php',
    'SimplePie_Cache_Base' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Cache/Base.php',
    'SimplePie_Cache_DB' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Cache/DB.php',
    'SimplePie_Cache_File' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Cache/File.php',
    'SimplePie_Cache_Memcache' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Cache/Memcache.php',
    'SimplePie_Cache_Memcached' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Cache/Memcached.php',
    'SimplePie_Cache_MySQL' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Cache/MySQL.php',
    'SimplePie_Cache_Redis' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Cache/Redis.php',
    'SimplePie_Caption' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Caption.php',
    'SimplePie_Category' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Category.php',
    'SimplePie_Content_Type_Sniffer' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Content/Type/Sniffer.php',
    'SimplePie_Copyright' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Copyright.php',
    'SimplePie_Core' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Core.php',
    'SimplePie_Credit' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Credit.php',
    'SimplePie_Decode_HTML_Entities' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Decode/HTML/Entities.php',
    'SimplePie_Enclosure' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Enclosure.php',
    'SimplePie_Exception' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Exception.php',
    'SimplePie_File' => $vendorDir . '/simplepie/simplepie/library/SimplePie/File.php',
    'SimplePie_HTTP_Parser' => $vendorDir . '/simplepie/simplepie/library/SimplePie/HTTP/Parser.php',
    'SimplePie_IRI' => $vendorDir . '/simplepie/simplepie/library/SimplePie/IRI.php',
    'SimplePie_Item' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Item.php',
    'SimplePie_Locator' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Locator.php',
    'SimplePie_Misc' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Misc.php',
    'SimplePie_Net_IPv6' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Net/IPv6.php',
    'SimplePie_Parse_Date' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Parse/Date.php',
    'SimplePie_Parser' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Parser.php',
    'SimplePie_Rating' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Rating.php',
    'SimplePie_Registry' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Registry.php',
    'SimplePie_Restriction' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Restriction.php',
    'SimplePie_Sanitize' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Sanitize.php',
    'SimplePie_Source' => $vendorDir . '/simplepie/simplepie/library/SimplePie/Source.php',
    'SimplePie_XML_Declaration_Parser' => $vendorDir . '/simplepie/simplepie/library/SimplePie/XML/Declaration/Parser.php',
    'SimplePie_gzdecode' => $vendorDir . '/simplepie/simplepie/library/SimplePie/gzdecode.php',
    'SingletonRuleList' => $baseDir . '/src/SingletonRuleList.php',
    'SlaLevel' => $baseDir . '/src/SlaLevel.php',
    'SlaLevelAction' => $baseDir . '/src/SlaLevelAction.php',
    'SlaLevelCriteria' => $baseDir . '/src/SlaLevelCriteria.php',
    'SlaLevel_Ticket' => $baseDir . '/src/SlaLevel_Ticket.php',
    'Software' => $baseDir . '/src/Software.php',
    'SoftwareCategory' => $baseDir . '/src/SoftwareCategory.php',
    'SoftwareLicense' => $baseDir . '/src/SoftwareLicense.php',
    'SoftwareLicenseType' => $baseDir . '/src/SoftwareLicenseType.php',
    'SoftwareLicense_User' => $baseDir . '/src/SoftwareLicense_User.php',
    'SoftwareVersion' => $baseDir . '/src/SoftwareVersion.php',
    'SolutionTemplate' => $baseDir . '/src/SolutionTemplate.php',
    'SolutionType' => $baseDir . '/src/SolutionType.php',
    'SourceSpan\\ConcreteFileSpan' => $vendorDir . '/scssphp/source-span/src/ConcreteFileSpan.php',
    'SourceSpan\\FileLocation' => $vendorDir . '/scssphp/source-span/src/FileLocation.php',
    'SourceSpan\\FileSpan' => $vendorDir . '/scssphp/source-span/src/FileSpan.php',
    'SourceSpan\\Highlighter\\AsciiGlyph' => $vendorDir . '/scssphp/source-span/src/Highlighter/AsciiGlyph.php',
    'SourceSpan\\Highlighter\\Highlight' => $vendorDir . '/scssphp/source-span/src/Highlighter/Highlight.php',
    'SourceSpan\\Highlighter\\Highlighter' => $vendorDir . '/scssphp/source-span/src/Highlighter/Highlighter.php',
    'SourceSpan\\Highlighter\\Line' => $vendorDir . '/scssphp/source-span/src/Highlighter/Line.php',
    'SourceSpan\\SimpleSourceLocation' => $vendorDir . '/scssphp/source-span/src/SimpleSourceLocation.php',
    'SourceSpan\\SimpleSourceSpan' => $vendorDir . '/scssphp/source-span/src/SimpleSourceSpan.php',
    'SourceSpan\\SimpleSourceSpanWithContext' => $vendorDir . '/scssphp/source-span/src/SimpleSourceSpanWithContext.php',
    'SourceSpan\\SourceFile' => $vendorDir . '/scssphp/source-span/src/SourceFile.php',
    'SourceSpan\\SourceLocation' => $vendorDir . '/scssphp/source-span/src/SourceLocation.php',
    'SourceSpan\\SourceLocationMixin' => $vendorDir . '/scssphp/source-span/src/SourceLocationMixin.php',
    'SourceSpan\\SourceSpan' => $vendorDir . '/scssphp/source-span/src/SourceSpan.php',
    'SourceSpan\\SourceSpanMixin' => $vendorDir . '/scssphp/source-span/src/SourceSpanMixin.php',
    'SourceSpan\\SourceSpanWithContext' => $vendorDir . '/scssphp/source-span/src/SourceSpanWithContext.php',
    'SourceSpan\\Util' => $vendorDir . '/scssphp/source-span/src/Util.php',
    'SsoVariable' => $baseDir . '/src/SsoVariable.php',
    'Stat' => $baseDir . '/src/Stat.php',
    'State' => $baseDir . '/src/State.php',
    'Stencil' => $baseDir . '/src/Stencil.php',
    'Supplier' => $baseDir . '/src/Supplier.php',
    'SupplierType' => $baseDir . '/src/SupplierType.php',
    'Supplier_Ticket' => $baseDir . '/src/Supplier_Ticket.php',
    'Swaggest\\JsonDiff\\Exception' => $vendorDir . '/swaggest/json-diff/src/Exception.php',
    'Swaggest\\JsonDiff\\InvalidFieldTypeException' => $vendorDir . '/swaggest/json-diff/src/InvalidFieldTypeException.php',
    'Swaggest\\JsonDiff\\JsonDiff' => $vendorDir . '/swaggest/json-diff/src/JsonDiff.php',
    'Swaggest\\JsonDiff\\JsonHash' => $vendorDir . '/swaggest/json-diff/src/JsonHash.php',
    'Swaggest\\JsonDiff\\JsonMergePatch' => $vendorDir . '/swaggest/json-diff/src/JsonMergePatch.php',
    'Swaggest\\JsonDiff\\JsonPatch' => $vendorDir . '/swaggest/json-diff/src/JsonPatch.php',
    'Swaggest\\JsonDiff\\JsonPatch\\Add' => $vendorDir . '/swaggest/json-diff/src/JsonPatch/Add.php',
    'Swaggest\\JsonDiff\\JsonPatch\\Copy' => $vendorDir . '/swaggest/json-diff/src/JsonPatch/Copy.php',
    'Swaggest\\JsonDiff\\JsonPatch\\Move' => $vendorDir . '/swaggest/json-diff/src/JsonPatch/Move.php',
    'Swaggest\\JsonDiff\\JsonPatch\\OpPath' => $vendorDir . '/swaggest/json-diff/src/JsonPatch/OpPath.php',
    'Swaggest\\JsonDiff\\JsonPatch\\OpPathFrom' => $vendorDir . '/swaggest/json-diff/src/JsonPatch/OpPathFrom.php',
    'Swaggest\\JsonDiff\\JsonPatch\\OpPathValue' => $vendorDir . '/swaggest/json-diff/src/JsonPatch/OpPathValue.php',
    'Swaggest\\JsonDiff\\JsonPatch\\Remove' => $vendorDir . '/swaggest/json-diff/src/JsonPatch/Remove.php',
    'Swaggest\\JsonDiff\\JsonPatch\\Replace' => $vendorDir . '/swaggest/json-diff/src/JsonPatch/Replace.php',
    'Swaggest\\JsonDiff\\JsonPatch\\Test' => $vendorDir . '/swaggest/json-diff/src/JsonPatch/Test.php',
    'Swaggest\\JsonDiff\\JsonPointer' => $vendorDir . '/swaggest/json-diff/src/JsonPointer.php',
    'Swaggest\\JsonDiff\\JsonPointerException' => $vendorDir . '/swaggest/json-diff/src/JsonPointerException.php',
    'Swaggest\\JsonDiff\\JsonValueReplace' => $vendorDir . '/swaggest/json-diff/src/JsonValueReplace.php',
    'Swaggest\\JsonDiff\\MissingFieldException' => $vendorDir . '/swaggest/json-diff/src/MissingFieldException.php',
    'Swaggest\\JsonDiff\\ModifiedPathDiff' => $vendorDir . '/swaggest/json-diff/src/ModifiedPathDiff.php',
    'Swaggest\\JsonDiff\\PatchTestOperationFailedException' => $vendorDir . '/swaggest/json-diff/src/PatchTestOperationFailedException.php',
    'Swaggest\\JsonDiff\\PathException' => $vendorDir . '/swaggest/json-diff/src/PathException.php',
    'Swaggest\\JsonDiff\\UnknownOperationException' => $vendorDir . '/swaggest/json-diff/src/UnknownOperationException.php',
    'Swaggest\\JsonSchema\\Constraint\\Constraint' => $vendorDir . '/swaggest/json-schema/src/Constraint/Constraint.php',
    'Swaggest\\JsonSchema\\Constraint\\Content' => $vendorDir . '/swaggest/json-schema/src/Constraint/Content.php',
    'Swaggest\\JsonSchema\\Constraint\\Format' => $vendorDir . '/swaggest/json-schema/src/Constraint/Format.php',
    'Swaggest\\JsonSchema\\Constraint\\Format\\IdnHostname' => $vendorDir . '/swaggest/json-schema/src/Constraint/Format/IdnHostname.php',
    'Swaggest\\JsonSchema\\Constraint\\Format\\Iri' => $vendorDir . '/swaggest/json-schema/src/Constraint/Format/Iri.php',
    'Swaggest\\JsonSchema\\Constraint\\Format\\Uri' => $vendorDir . '/swaggest/json-schema/src/Constraint/Format/Uri.php',
    'Swaggest\\JsonSchema\\Constraint\\Properties' => $vendorDir . '/swaggest/json-schema/src/Constraint/Properties.php',
    'Swaggest\\JsonSchema\\Constraint\\Ref' => $vendorDir . '/swaggest/json-schema/src/Constraint/Ref.php',
    'Swaggest\\JsonSchema\\Constraint\\Type' => $vendorDir . '/swaggest/json-schema/src/Constraint/Type.php',
    'Swaggest\\JsonSchema\\Constraint\\UniqueItems' => $vendorDir . '/swaggest/json-schema/src/Constraint/UniqueItems.php',
    'Swaggest\\JsonSchema\\Context' => $vendorDir . '/swaggest/json-schema/src/Context.php',
    'Swaggest\\JsonSchema\\DataPreProcessor' => $vendorDir . '/swaggest/json-schema/src/DataPreProcessor.php',
    'Swaggest\\JsonSchema\\Exception' => $vendorDir . '/swaggest/json-schema/src/Exception.php',
    'Swaggest\\JsonSchema\\Exception\\ArrayException' => $vendorDir . '/swaggest/json-schema/src/Exception/ArrayException.php',
    'Swaggest\\JsonSchema\\Exception\\ConstException' => $vendorDir . '/swaggest/json-schema/src/Exception/ConstException.php',
    'Swaggest\\JsonSchema\\Exception\\ContentException' => $vendorDir . '/swaggest/json-schema/src/Exception/ContentException.php',
    'Swaggest\\JsonSchema\\Exception\\EnumException' => $vendorDir . '/swaggest/json-schema/src/Exception/EnumException.php',
    'Swaggest\\JsonSchema\\Exception\\Error' => $vendorDir . '/swaggest/json-schema/src/Exception/Error.php',
    'Swaggest\\JsonSchema\\Exception\\LogicException' => $vendorDir . '/swaggest/json-schema/src/Exception/LogicException.php',
    'Swaggest\\JsonSchema\\Exception\\NumericException' => $vendorDir . '/swaggest/json-schema/src/Exception/NumericException.php',
    'Swaggest\\JsonSchema\\Exception\\ObjectException' => $vendorDir . '/swaggest/json-schema/src/Exception/ObjectException.php',
    'Swaggest\\JsonSchema\\Exception\\StringException' => $vendorDir . '/swaggest/json-schema/src/Exception/StringException.php',
    'Swaggest\\JsonSchema\\Exception\\TypeException' => $vendorDir . '/swaggest/json-schema/src/Exception/TypeException.php',
    'Swaggest\\JsonSchema\\HasDefault' => $vendorDir . '/swaggest/json-schema/src/HasDefault.php',
    'Swaggest\\JsonSchema\\Helper' => $vendorDir . '/swaggest/json-schema/src/Helper.php',
    'Swaggest\\JsonSchema\\InvalidRef' => $vendorDir . '/swaggest/json-schema/src/InvalidRef.php',
    'Swaggest\\JsonSchema\\InvalidValue' => $vendorDir . '/swaggest/json-schema/src/InvalidValue.php',
    'Swaggest\\JsonSchema\\JsonSchema' => $vendorDir . '/swaggest/json-schema/src/JsonSchema.php',
    'Swaggest\\JsonSchema\\MagicMap' => $vendorDir . '/swaggest/json-schema/src/MagicMap.php',
    'Swaggest\\JsonSchema\\MagicMapTrait' => $vendorDir . '/swaggest/json-schema/src/MagicMapTrait.php',
    'Swaggest\\JsonSchema\\Meta\\AbstractMeta' => $vendorDir . '/swaggest/json-schema/src/Meta/AbstractMeta.php',
    'Swaggest\\JsonSchema\\Meta\\MetaHolder' => $vendorDir . '/swaggest/json-schema/src/Meta/MetaHolder.php',
    'Swaggest\\JsonSchema\\NameMirror' => $vendorDir . '/swaggest/json-schema/src/NameMirror.php',
    'Swaggest\\JsonSchema\\Path\\PointerUtil' => $vendorDir . '/swaggest/json-schema/src/Path/PointerUtil.php',
    'Swaggest\\JsonSchema\\RefResolver' => $vendorDir . '/swaggest/json-schema/src/RefResolver.php',
    'Swaggest\\JsonSchema\\RemoteRefProvider' => $vendorDir . '/swaggest/json-schema/src/RemoteRefProvider.php',
    'Swaggest\\JsonSchema\\RemoteRef\\BasicFetcher' => $vendorDir . '/swaggest/json-schema/src/RemoteRef/BasicFetcher.php',
    'Swaggest\\JsonSchema\\RemoteRef\\Preloaded' => $vendorDir . '/swaggest/json-schema/src/RemoteRef/Preloaded.php',
    'Swaggest\\JsonSchema\\Schema' => $vendorDir . '/swaggest/json-schema/src/Schema.php',
    'Swaggest\\JsonSchema\\SchemaContract' => $vendorDir . '/swaggest/json-schema/src/SchemaContract.php',
    'Swaggest\\JsonSchema\\SchemaExporter' => $vendorDir . '/swaggest/json-schema/src/SchemaExporter.php',
    'Swaggest\\JsonSchema\\Structure\\ClassStructure' => $vendorDir . '/swaggest/json-schema/src/Structure/ClassStructure.php',
    'Swaggest\\JsonSchema\\Structure\\ClassStructureContract' => $vendorDir . '/swaggest/json-schema/src/Structure/ClassStructureContract.php',
    'Swaggest\\JsonSchema\\Structure\\ClassStructureTrait' => $vendorDir . '/swaggest/json-schema/src/Structure/ClassStructureTrait.php',
    'Swaggest\\JsonSchema\\Structure\\Composition' => $vendorDir . '/swaggest/json-schema/src/Structure/Composition.php',
    'Swaggest\\JsonSchema\\Structure\\Egg' => $vendorDir . '/swaggest/json-schema/src/Structure/Egg.php',
    'Swaggest\\JsonSchema\\Structure\\Nested' => $vendorDir . '/swaggest/json-schema/src/Structure/Nested.php',
    'Swaggest\\JsonSchema\\Structure\\ObjectItem' => $vendorDir . '/swaggest/json-schema/src/Structure/ObjectItem.php',
    'Swaggest\\JsonSchema\\Structure\\ObjectItemContract' => $vendorDir . '/swaggest/json-schema/src/Structure/ObjectItemContract.php',
    'Swaggest\\JsonSchema\\Structure\\ObjectItemTrait' => $vendorDir . '/swaggest/json-schema/src/Structure/ObjectItemTrait.php',
    'Swaggest\\JsonSchema\\Structure\\WithResolvedValue' => $vendorDir . '/swaggest/json-schema/src/Structure/WithResolvedValue.php',
    'Swaggest\\JsonSchema\\Wrapper' => $vendorDir . '/swaggest/json-schema/src/Wrapper.php',
    'Symfony\\Bundle\\FrameworkBundle\\CacheWarmer\\AbstractPhpFileCacheWarmer' => $vendorDir . '/symfony/framework-bundle/CacheWarmer/AbstractPhpFileCacheWarmer.php',
    'Symfony\\Bundle\\FrameworkBundle\\CacheWarmer\\AnnotationsCacheWarmer' => $vendorDir . '/symfony/framework-bundle/CacheWarmer/AnnotationsCacheWarmer.php',
    'Symfony\\Bundle\\FrameworkBundle\\CacheWarmer\\CachePoolClearerCacheWarmer' => $vendorDir . '/symfony/framework-bundle/CacheWarmer/CachePoolClearerCacheWarmer.php',
    'Symfony\\Bundle\\FrameworkBundle\\CacheWarmer\\ConfigBuilderCacheWarmer' => $vendorDir . '/symfony/framework-bundle/CacheWarmer/ConfigBuilderCacheWarmer.php',
    'Symfony\\Bundle\\FrameworkBundle\\CacheWarmer\\RouterCacheWarmer' => $vendorDir . '/symfony/framework-bundle/CacheWarmer/RouterCacheWarmer.php',
    'Symfony\\Bundle\\FrameworkBundle\\CacheWarmer\\SerializerCacheWarmer' => $vendorDir . '/symfony/framework-bundle/CacheWarmer/SerializerCacheWarmer.php',
    'Symfony\\Bundle\\FrameworkBundle\\CacheWarmer\\TranslationsCacheWarmer' => $vendorDir . '/symfony/framework-bundle/CacheWarmer/TranslationsCacheWarmer.php',
    'Symfony\\Bundle\\FrameworkBundle\\CacheWarmer\\ValidatorCacheWarmer' => $vendorDir . '/symfony/framework-bundle/CacheWarmer/ValidatorCacheWarmer.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\AboutCommand' => $vendorDir . '/symfony/framework-bundle/Command/AboutCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\AbstractConfigCommand' => $vendorDir . '/symfony/framework-bundle/Command/AbstractConfigCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\AssetsInstallCommand' => $vendorDir . '/symfony/framework-bundle/Command/AssetsInstallCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\BuildDebugContainerTrait' => $vendorDir . '/symfony/framework-bundle/Command/BuildDebugContainerTrait.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\CacheClearCommand' => $vendorDir . '/symfony/framework-bundle/Command/CacheClearCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\CachePoolClearCommand' => $vendorDir . '/symfony/framework-bundle/Command/CachePoolClearCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\CachePoolDeleteCommand' => $vendorDir . '/symfony/framework-bundle/Command/CachePoolDeleteCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\CachePoolInvalidateTagsCommand' => $vendorDir . '/symfony/framework-bundle/Command/CachePoolInvalidateTagsCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\CachePoolListCommand' => $vendorDir . '/symfony/framework-bundle/Command/CachePoolListCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\CachePoolPruneCommand' => $vendorDir . '/symfony/framework-bundle/Command/CachePoolPruneCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\CacheWarmupCommand' => $vendorDir . '/symfony/framework-bundle/Command/CacheWarmupCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\ConfigDebugCommand' => $vendorDir . '/symfony/framework-bundle/Command/ConfigDebugCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\ConfigDumpReferenceCommand' => $vendorDir . '/symfony/framework-bundle/Command/ConfigDumpReferenceCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\ContainerDebugCommand' => $vendorDir . '/symfony/framework-bundle/Command/ContainerDebugCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\ContainerLintCommand' => $vendorDir . '/symfony/framework-bundle/Command/ContainerLintCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\DebugAutowiringCommand' => $vendorDir . '/symfony/framework-bundle/Command/DebugAutowiringCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\EventDispatcherDebugCommand' => $vendorDir . '/symfony/framework-bundle/Command/EventDispatcherDebugCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\RouterDebugCommand' => $vendorDir . '/symfony/framework-bundle/Command/RouterDebugCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\RouterMatchCommand' => $vendorDir . '/symfony/framework-bundle/Command/RouterMatchCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\SecretsDecryptToLocalCommand' => $vendorDir . '/symfony/framework-bundle/Command/SecretsDecryptToLocalCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\SecretsEncryptFromLocalCommand' => $vendorDir . '/symfony/framework-bundle/Command/SecretsEncryptFromLocalCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\SecretsGenerateKeysCommand' => $vendorDir . '/symfony/framework-bundle/Command/SecretsGenerateKeysCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\SecretsListCommand' => $vendorDir . '/symfony/framework-bundle/Command/SecretsListCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\SecretsRemoveCommand' => $vendorDir . '/symfony/framework-bundle/Command/SecretsRemoveCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\SecretsSetCommand' => $vendorDir . '/symfony/framework-bundle/Command/SecretsSetCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\TranslationDebugCommand' => $vendorDir . '/symfony/framework-bundle/Command/TranslationDebugCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\TranslationUpdateCommand' => $vendorDir . '/symfony/framework-bundle/Command/TranslationUpdateCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\WorkflowDumpCommand' => $vendorDir . '/symfony/framework-bundle/Command/WorkflowDumpCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\XliffLintCommand' => $vendorDir . '/symfony/framework-bundle/Command/XliffLintCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Command\\YamlLintCommand' => $vendorDir . '/symfony/framework-bundle/Command/YamlLintCommand.php',
    'Symfony\\Bundle\\FrameworkBundle\\Console\\Application' => $vendorDir . '/symfony/framework-bundle/Console/Application.php',
    'Symfony\\Bundle\\FrameworkBundle\\Console\\Descriptor\\Descriptor' => $vendorDir . '/symfony/framework-bundle/Console/Descriptor/Descriptor.php',
    'Symfony\\Bundle\\FrameworkBundle\\Console\\Descriptor\\JsonDescriptor' => $vendorDir . '/symfony/framework-bundle/Console/Descriptor/JsonDescriptor.php',
    'Symfony\\Bundle\\FrameworkBundle\\Console\\Descriptor\\MarkdownDescriptor' => $vendorDir . '/symfony/framework-bundle/Console/Descriptor/MarkdownDescriptor.php',
    'Symfony\\Bundle\\FrameworkBundle\\Console\\Descriptor\\TextDescriptor' => $vendorDir . '/symfony/framework-bundle/Console/Descriptor/TextDescriptor.php',
    'Symfony\\Bundle\\FrameworkBundle\\Console\\Descriptor\\XmlDescriptor' => $vendorDir . '/symfony/framework-bundle/Console/Descriptor/XmlDescriptor.php',
    'Symfony\\Bundle\\FrameworkBundle\\Console\\Helper\\DescriptorHelper' => $vendorDir . '/symfony/framework-bundle/Console/Helper/DescriptorHelper.php',
    'Symfony\\Bundle\\FrameworkBundle\\Controller\\AbstractController' => $vendorDir . '/symfony/framework-bundle/Controller/AbstractController.php',
    'Symfony\\Bundle\\FrameworkBundle\\Controller\\ControllerResolver' => $vendorDir . '/symfony/framework-bundle/Controller/ControllerResolver.php',
    'Symfony\\Bundle\\FrameworkBundle\\Controller\\RedirectController' => $vendorDir . '/symfony/framework-bundle/Controller/RedirectController.php',
    'Symfony\\Bundle\\FrameworkBundle\\Controller\\TemplateController' => $vendorDir . '/symfony/framework-bundle/Controller/TemplateController.php',
    'Symfony\\Bundle\\FrameworkBundle\\DataCollector\\AbstractDataCollector' => $vendorDir . '/symfony/framework-bundle/DataCollector/AbstractDataCollector.php',
    'Symfony\\Bundle\\FrameworkBundle\\DataCollector\\RouterDataCollector' => $vendorDir . '/symfony/framework-bundle/DataCollector/RouterDataCollector.php',
    'Symfony\\Bundle\\FrameworkBundle\\DataCollector\\TemplateAwareDataCollectorInterface' => $vendorDir . '/symfony/framework-bundle/DataCollector/TemplateAwareDataCollectorInterface.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\AddAnnotationsCachedReaderPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/AddAnnotationsCachedReaderPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\AddDebugLogProcessorPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/AddDebugLogProcessorPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\AddExpressionLanguageProvidersPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/AddExpressionLanguageProvidersPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\AssetsContextPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/AssetsContextPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\ContainerBuilderDebugDumpPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/ContainerBuilderDebugDumpPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\DataCollectorTranslatorPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/DataCollectorTranslatorPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\EnableLoggerDebugModePass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/EnableLoggerDebugModePass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\ErrorLoggerCompilerPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/ErrorLoggerCompilerPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\LoggingTranslatorPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/LoggingTranslatorPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\ProfilerPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/ProfilerPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\RemoveUnusedSessionMarshallingHandlerPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/RemoveUnusedSessionMarshallingHandlerPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\TestServiceContainerRealRefPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/TestServiceContainerRealRefPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\TestServiceContainerWeakRefPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/TestServiceContainerWeakRefPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\TranslationUpdateCommandPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/TranslationUpdateCommandPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\UnusedTagsPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/UnusedTagsPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Compiler\\WorkflowGuardListenerPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Compiler/WorkflowGuardListenerPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\Configuration' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/Configuration.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\FrameworkExtension' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/FrameworkExtension.php',
    'Symfony\\Bundle\\FrameworkBundle\\DependencyInjection\\VirtualRequestStackPass' => $vendorDir . '/symfony/framework-bundle/DependencyInjection/VirtualRequestStackPass.php',
    'Symfony\\Bundle\\FrameworkBundle\\EventListener\\ConsoleProfilerListener' => $vendorDir . '/symfony/framework-bundle/EventListener/ConsoleProfilerListener.php',
    'Symfony\\Bundle\\FrameworkBundle\\EventListener\\SuggestMissingPackageSubscriber' => $vendorDir . '/symfony/framework-bundle/EventListener/SuggestMissingPackageSubscriber.php',
    'Symfony\\Bundle\\FrameworkBundle\\FrameworkBundle' => $vendorDir . '/symfony/framework-bundle/FrameworkBundle.php',
    'Symfony\\Bundle\\FrameworkBundle\\HttpCache\\HttpCache' => $vendorDir . '/symfony/framework-bundle/HttpCache/HttpCache.php',
    'Symfony\\Bundle\\FrameworkBundle\\KernelBrowser' => $vendorDir . '/symfony/framework-bundle/KernelBrowser.php',
    'Symfony\\Bundle\\FrameworkBundle\\Kernel\\MicroKernelTrait' => $vendorDir . '/symfony/framework-bundle/Kernel/MicroKernelTrait.php',
    'Symfony\\Bundle\\FrameworkBundle\\Routing\\AnnotatedRouteControllerLoader' => $vendorDir . '/symfony/framework-bundle/Routing/AnnotatedRouteControllerLoader.php',
    'Symfony\\Bundle\\FrameworkBundle\\Routing\\AttributeRouteControllerLoader' => $vendorDir . '/symfony/framework-bundle/Routing/AttributeRouteControllerLoader.php',
    'Symfony\\Bundle\\FrameworkBundle\\Routing\\Attribute\\AsRoutingConditionService' => $vendorDir . '/symfony/framework-bundle/Routing/Attribute/AsRoutingConditionService.php',
    'Symfony\\Bundle\\FrameworkBundle\\Routing\\DelegatingLoader' => $vendorDir . '/symfony/framework-bundle/Routing/DelegatingLoader.php',
    'Symfony\\Bundle\\FrameworkBundle\\Routing\\RedirectableCompiledUrlMatcher' => $vendorDir . '/symfony/framework-bundle/Routing/RedirectableCompiledUrlMatcher.php',
    'Symfony\\Bundle\\FrameworkBundle\\Routing\\RouteLoaderInterface' => $vendorDir . '/symfony/framework-bundle/Routing/RouteLoaderInterface.php',
    'Symfony\\Bundle\\FrameworkBundle\\Routing\\Router' => $vendorDir . '/symfony/framework-bundle/Routing/Router.php',
    'Symfony\\Bundle\\FrameworkBundle\\Secrets\\AbstractVault' => $vendorDir . '/symfony/framework-bundle/Secrets/AbstractVault.php',
    'Symfony\\Bundle\\FrameworkBundle\\Secrets\\DotenvVault' => $vendorDir . '/symfony/framework-bundle/Secrets/DotenvVault.php',
    'Symfony\\Bundle\\FrameworkBundle\\Secrets\\SodiumVault' => $vendorDir . '/symfony/framework-bundle/Secrets/SodiumVault.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\BrowserKitAssertionsTrait' => $vendorDir . '/symfony/framework-bundle/Test/BrowserKitAssertionsTrait.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\DomCrawlerAssertionsTrait' => $vendorDir . '/symfony/framework-bundle/Test/DomCrawlerAssertionsTrait.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\HttpClientAssertionsTrait' => $vendorDir . '/symfony/framework-bundle/Test/HttpClientAssertionsTrait.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\KernelTestCase' => $vendorDir . '/symfony/framework-bundle/Test/KernelTestCase.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\MailerAssertionsTrait' => $vendorDir . '/symfony/framework-bundle/Test/MailerAssertionsTrait.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\NotificationAssertionsTrait' => $vendorDir . '/symfony/framework-bundle/Test/NotificationAssertionsTrait.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\TestBrowserToken' => $vendorDir . '/symfony/framework-bundle/Test/TestBrowserToken.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\TestContainer' => $vendorDir . '/symfony/framework-bundle/Test/TestContainer.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\WebTestAssertionsTrait' => $vendorDir . '/symfony/framework-bundle/Test/WebTestAssertionsTrait.php',
    'Symfony\\Bundle\\FrameworkBundle\\Test\\WebTestCase' => $vendorDir . '/symfony/framework-bundle/Test/WebTestCase.php',
    'Symfony\\Bundle\\FrameworkBundle\\Translation\\Translator' => $vendorDir . '/symfony/framework-bundle/Translation/Translator.php',
    'Symfony\\Component\\Cache\\Adapter\\AbstractAdapter' => $vendorDir . '/symfony/cache/Adapter/AbstractAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\AbstractTagAwareAdapter' => $vendorDir . '/symfony/cache/Adapter/AbstractTagAwareAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\AdapterInterface' => $vendorDir . '/symfony/cache/Adapter/AdapterInterface.php',
    'Symfony\\Component\\Cache\\Adapter\\ApcuAdapter' => $vendorDir . '/symfony/cache/Adapter/ApcuAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\ArrayAdapter' => $vendorDir . '/symfony/cache/Adapter/ArrayAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\ChainAdapter' => $vendorDir . '/symfony/cache/Adapter/ChainAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\CouchbaseBucketAdapter' => $vendorDir . '/symfony/cache/Adapter/CouchbaseBucketAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\CouchbaseCollectionAdapter' => $vendorDir . '/symfony/cache/Adapter/CouchbaseCollectionAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\DoctrineDbalAdapter' => $vendorDir . '/symfony/cache/Adapter/DoctrineDbalAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\FilesystemAdapter' => $vendorDir . '/symfony/cache/Adapter/FilesystemAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\FilesystemTagAwareAdapter' => $vendorDir . '/symfony/cache/Adapter/FilesystemTagAwareAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\MemcachedAdapter' => $vendorDir . '/symfony/cache/Adapter/MemcachedAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\NullAdapter' => $vendorDir . '/symfony/cache/Adapter/NullAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\ParameterNormalizer' => $vendorDir . '/symfony/cache/Adapter/ParameterNormalizer.php',
    'Symfony\\Component\\Cache\\Adapter\\PdoAdapter' => $vendorDir . '/symfony/cache/Adapter/PdoAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\PhpArrayAdapter' => $vendorDir . '/symfony/cache/Adapter/PhpArrayAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\PhpFilesAdapter' => $vendorDir . '/symfony/cache/Adapter/PhpFilesAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\ProxyAdapter' => $vendorDir . '/symfony/cache/Adapter/ProxyAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\Psr16Adapter' => $vendorDir . '/symfony/cache/Adapter/Psr16Adapter.php',
    'Symfony\\Component\\Cache\\Adapter\\RedisAdapter' => $vendorDir . '/symfony/cache/Adapter/RedisAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\RedisTagAwareAdapter' => $vendorDir . '/symfony/cache/Adapter/RedisTagAwareAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\TagAwareAdapter' => $vendorDir . '/symfony/cache/Adapter/TagAwareAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\TagAwareAdapterInterface' => $vendorDir . '/symfony/cache/Adapter/TagAwareAdapterInterface.php',
    'Symfony\\Component\\Cache\\Adapter\\TraceableAdapter' => $vendorDir . '/symfony/cache/Adapter/TraceableAdapter.php',
    'Symfony\\Component\\Cache\\Adapter\\TraceableTagAwareAdapter' => $vendorDir . '/symfony/cache/Adapter/TraceableTagAwareAdapter.php',
    'Symfony\\Component\\Cache\\CacheItem' => $vendorDir . '/symfony/cache/CacheItem.php',
    'Symfony\\Component\\Cache\\DataCollector\\CacheDataCollector' => $vendorDir . '/symfony/cache/DataCollector/CacheDataCollector.php',
    'Symfony\\Component\\Cache\\DependencyInjection\\CacheCollectorPass' => $vendorDir . '/symfony/cache/DependencyInjection/CacheCollectorPass.php',
    'Symfony\\Component\\Cache\\DependencyInjection\\CachePoolClearerPass' => $vendorDir . '/symfony/cache/DependencyInjection/CachePoolClearerPass.php',
    'Symfony\\Component\\Cache\\DependencyInjection\\CachePoolPass' => $vendorDir . '/symfony/cache/DependencyInjection/CachePoolPass.php',
    'Symfony\\Component\\Cache\\DependencyInjection\\CachePoolPrunerPass' => $vendorDir . '/symfony/cache/DependencyInjection/CachePoolPrunerPass.php',
    'Symfony\\Component\\Cache\\Exception\\CacheException' => $vendorDir . '/symfony/cache/Exception/CacheException.php',
    'Symfony\\Component\\Cache\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/cache/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Cache\\Exception\\LogicException' => $vendorDir . '/symfony/cache/Exception/LogicException.php',
    'Symfony\\Component\\Cache\\LockRegistry' => $vendorDir . '/symfony/cache/LockRegistry.php',
    'Symfony\\Component\\Cache\\Marshaller\\DefaultMarshaller' => $vendorDir . '/symfony/cache/Marshaller/DefaultMarshaller.php',
    'Symfony\\Component\\Cache\\Marshaller\\DeflateMarshaller' => $vendorDir . '/symfony/cache/Marshaller/DeflateMarshaller.php',
    'Symfony\\Component\\Cache\\Marshaller\\MarshallerInterface' => $vendorDir . '/symfony/cache/Marshaller/MarshallerInterface.php',
    'Symfony\\Component\\Cache\\Marshaller\\SodiumMarshaller' => $vendorDir . '/symfony/cache/Marshaller/SodiumMarshaller.php',
    'Symfony\\Component\\Cache\\Marshaller\\TagAwareMarshaller' => $vendorDir . '/symfony/cache/Marshaller/TagAwareMarshaller.php',
    'Symfony\\Component\\Cache\\Messenger\\EarlyExpirationDispatcher' => $vendorDir . '/symfony/cache/Messenger/EarlyExpirationDispatcher.php',
    'Symfony\\Component\\Cache\\Messenger\\EarlyExpirationHandler' => $vendorDir . '/symfony/cache/Messenger/EarlyExpirationHandler.php',
    'Symfony\\Component\\Cache\\Messenger\\EarlyExpirationMessage' => $vendorDir . '/symfony/cache/Messenger/EarlyExpirationMessage.php',
    'Symfony\\Component\\Cache\\PruneableInterface' => $vendorDir . '/symfony/cache/PruneableInterface.php',
    'Symfony\\Component\\Cache\\Psr16Cache' => $vendorDir . '/symfony/cache/Psr16Cache.php',
    'Symfony\\Component\\Cache\\ResettableInterface' => $vendorDir . '/symfony/cache/ResettableInterface.php',
    'Symfony\\Component\\Cache\\Traits\\AbstractAdapterTrait' => $vendorDir . '/symfony/cache/Traits/AbstractAdapterTrait.php',
    'Symfony\\Component\\Cache\\Traits\\CachedValueInterface' => $vendorDir . '/symfony/cache/Traits/CachedValueInterface.php',
    'Symfony\\Component\\Cache\\Traits\\ContractsTrait' => $vendorDir . '/symfony/cache/Traits/ContractsTrait.php',
    'Symfony\\Component\\Cache\\Traits\\FilesystemCommonTrait' => $vendorDir . '/symfony/cache/Traits/FilesystemCommonTrait.php',
    'Symfony\\Component\\Cache\\Traits\\FilesystemTrait' => $vendorDir . '/symfony/cache/Traits/FilesystemTrait.php',
    'Symfony\\Component\\Cache\\Traits\\ProxyTrait' => $vendorDir . '/symfony/cache/Traits/ProxyTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Redis5Proxy' => $vendorDir . '/symfony/cache/Traits/Redis5Proxy.php',
    'Symfony\\Component\\Cache\\Traits\\Redis61ProxyTrait' => $vendorDir . '/symfony/cache/Traits/Redis61ProxyTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Redis62ProxyTrait' => $vendorDir . '/symfony/cache/Traits/Redis62ProxyTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Redis63ProxyTrait' => $vendorDir . '/symfony/cache/Traits/Redis63ProxyTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Redis6Proxy' => $vendorDir . '/symfony/cache/Traits/Redis6Proxy.php',
    'Symfony\\Component\\Cache\\Traits\\RedisCluster5Proxy' => $vendorDir . '/symfony/cache/Traits/RedisCluster5Proxy.php',
    'Symfony\\Component\\Cache\\Traits\\RedisCluster61ProxyTrait' => $vendorDir . '/symfony/cache/Traits/RedisCluster61ProxyTrait.php',
    'Symfony\\Component\\Cache\\Traits\\RedisCluster62ProxyTrait' => $vendorDir . '/symfony/cache/Traits/RedisCluster62ProxyTrait.php',
    'Symfony\\Component\\Cache\\Traits\\RedisCluster63ProxyTrait' => $vendorDir . '/symfony/cache/Traits/RedisCluster63ProxyTrait.php',
    'Symfony\\Component\\Cache\\Traits\\RedisCluster6Proxy' => $vendorDir . '/symfony/cache/Traits/RedisCluster6Proxy.php',
    'Symfony\\Component\\Cache\\Traits\\RedisClusterNodeProxy' => $vendorDir . '/symfony/cache/Traits/RedisClusterNodeProxy.php',
    'Symfony\\Component\\Cache\\Traits\\RedisClusterProxy' => $vendorDir . '/symfony/cache/Traits/RedisClusterProxy.php',
    'Symfony\\Component\\Cache\\Traits\\RedisProxy' => $vendorDir . '/symfony/cache/Traits/RedisProxy.php',
    'Symfony\\Component\\Cache\\Traits\\RedisTrait' => $vendorDir . '/symfony/cache/Traits/RedisTrait.php',
    'Symfony\\Component\\Cache\\Traits\\RelayProxy' => $vendorDir . '/symfony/cache/Traits/RelayProxy.php',
    'Symfony\\Component\\Cache\\Traits\\RelayProxyTrait' => $vendorDir . '/symfony/cache/Traits/RelayProxyTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\BgsaveTrait' => $vendorDir . '/symfony/cache/Traits/Relay/BgsaveTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\CopyTrait' => $vendorDir . '/symfony/cache/Traits/Relay/CopyTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\FtTrait' => $vendorDir . '/symfony/cache/Traits/Relay/FtTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\GeosearchTrait' => $vendorDir . '/symfony/cache/Traits/Relay/GeosearchTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\GetWithMetaTrait' => $vendorDir . '/symfony/cache/Traits/Relay/GetWithMetaTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\GetrangeTrait' => $vendorDir . '/symfony/cache/Traits/Relay/GetrangeTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\HsetTrait' => $vendorDir . '/symfony/cache/Traits/Relay/HsetTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\IsTrackedTrait' => $vendorDir . '/symfony/cache/Traits/Relay/IsTrackedTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\MoveTrait' => $vendorDir . '/symfony/cache/Traits/Relay/MoveTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\NullableReturnTrait' => $vendorDir . '/symfony/cache/Traits/Relay/NullableReturnTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\PfcountTrait' => $vendorDir . '/symfony/cache/Traits/Relay/PfcountTrait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\Relay11Trait' => $vendorDir . '/symfony/cache/Traits/Relay/Relay11Trait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\Relay121Trait' => $vendorDir . '/symfony/cache/Traits/Relay/Relay121Trait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\Relay12Trait' => $vendorDir . '/symfony/cache/Traits/Relay/Relay12Trait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\Relay20Trait' => $vendorDir . '/symfony/cache/Traits/Relay/Relay20Trait.php',
    'Symfony\\Component\\Cache\\Traits\\Relay\\SwapdbTrait' => $vendorDir . '/symfony/cache/Traits/Relay/SwapdbTrait.php',
    'Symfony\\Component\\Config\\Builder\\ClassBuilder' => $vendorDir . '/symfony/config/Builder/ClassBuilder.php',
    'Symfony\\Component\\Config\\Builder\\ConfigBuilderGenerator' => $vendorDir . '/symfony/config/Builder/ConfigBuilderGenerator.php',
    'Symfony\\Component\\Config\\Builder\\ConfigBuilderGeneratorInterface' => $vendorDir . '/symfony/config/Builder/ConfigBuilderGeneratorInterface.php',
    'Symfony\\Component\\Config\\Builder\\ConfigBuilderInterface' => $vendorDir . '/symfony/config/Builder/ConfigBuilderInterface.php',
    'Symfony\\Component\\Config\\Builder\\Method' => $vendorDir . '/symfony/config/Builder/Method.php',
    'Symfony\\Component\\Config\\Builder\\Property' => $vendorDir . '/symfony/config/Builder/Property.php',
    'Symfony\\Component\\Config\\ConfigCache' => $vendorDir . '/symfony/config/ConfigCache.php',
    'Symfony\\Component\\Config\\ConfigCacheFactory' => $vendorDir . '/symfony/config/ConfigCacheFactory.php',
    'Symfony\\Component\\Config\\ConfigCacheFactoryInterface' => $vendorDir . '/symfony/config/ConfigCacheFactoryInterface.php',
    'Symfony\\Component\\Config\\ConfigCacheInterface' => $vendorDir . '/symfony/config/ConfigCacheInterface.php',
    'Symfony\\Component\\Config\\Definition\\ArrayNode' => $vendorDir . '/symfony/config/Definition/ArrayNode.php',
    'Symfony\\Component\\Config\\Definition\\BaseNode' => $vendorDir . '/symfony/config/Definition/BaseNode.php',
    'Symfony\\Component\\Config\\Definition\\BooleanNode' => $vendorDir . '/symfony/config/Definition/BooleanNode.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\ArrayNodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/ArrayNodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\BooleanNodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/BooleanNodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\BuilderAwareInterface' => $vendorDir . '/symfony/config/Definition/Builder/BuilderAwareInterface.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\EnumNodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/EnumNodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\ExprBuilder' => $vendorDir . '/symfony/config/Definition/Builder/ExprBuilder.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\FloatNodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/FloatNodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\IntegerNodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/IntegerNodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\MergeBuilder' => $vendorDir . '/symfony/config/Definition/Builder/MergeBuilder.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\NodeBuilder' => $vendorDir . '/symfony/config/Definition/Builder/NodeBuilder.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\NodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/NodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\NodeParentInterface' => $vendorDir . '/symfony/config/Definition/Builder/NodeParentInterface.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\NormalizationBuilder' => $vendorDir . '/symfony/config/Definition/Builder/NormalizationBuilder.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\NumericNodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/NumericNodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\ParentNodeDefinitionInterface' => $vendorDir . '/symfony/config/Definition/Builder/ParentNodeDefinitionInterface.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\ScalarNodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/ScalarNodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\TreeBuilder' => $vendorDir . '/symfony/config/Definition/Builder/TreeBuilder.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\ValidationBuilder' => $vendorDir . '/symfony/config/Definition/Builder/ValidationBuilder.php',
    'Symfony\\Component\\Config\\Definition\\Builder\\VariableNodeDefinition' => $vendorDir . '/symfony/config/Definition/Builder/VariableNodeDefinition.php',
    'Symfony\\Component\\Config\\Definition\\ConfigurableInterface' => $vendorDir . '/symfony/config/Definition/ConfigurableInterface.php',
    'Symfony\\Component\\Config\\Definition\\Configuration' => $vendorDir . '/symfony/config/Definition/Configuration.php',
    'Symfony\\Component\\Config\\Definition\\ConfigurationInterface' => $vendorDir . '/symfony/config/Definition/ConfigurationInterface.php',
    'Symfony\\Component\\Config\\Definition\\Configurator\\DefinitionConfigurator' => $vendorDir . '/symfony/config/Definition/Configurator/DefinitionConfigurator.php',
    'Symfony\\Component\\Config\\Definition\\Dumper\\XmlReferenceDumper' => $vendorDir . '/symfony/config/Definition/Dumper/XmlReferenceDumper.php',
    'Symfony\\Component\\Config\\Definition\\Dumper\\YamlReferenceDumper' => $vendorDir . '/symfony/config/Definition/Dumper/YamlReferenceDumper.php',
    'Symfony\\Component\\Config\\Definition\\EnumNode' => $vendorDir . '/symfony/config/Definition/EnumNode.php',
    'Symfony\\Component\\Config\\Definition\\Exception\\DuplicateKeyException' => $vendorDir . '/symfony/config/Definition/Exception/DuplicateKeyException.php',
    'Symfony\\Component\\Config\\Definition\\Exception\\Exception' => $vendorDir . '/symfony/config/Definition/Exception/Exception.php',
    'Symfony\\Component\\Config\\Definition\\Exception\\ForbiddenOverwriteException' => $vendorDir . '/symfony/config/Definition/Exception/ForbiddenOverwriteException.php',
    'Symfony\\Component\\Config\\Definition\\Exception\\InvalidConfigurationException' => $vendorDir . '/symfony/config/Definition/Exception/InvalidConfigurationException.php',
    'Symfony\\Component\\Config\\Definition\\Exception\\InvalidDefinitionException' => $vendorDir . '/symfony/config/Definition/Exception/InvalidDefinitionException.php',
    'Symfony\\Component\\Config\\Definition\\Exception\\InvalidTypeException' => $vendorDir . '/symfony/config/Definition/Exception/InvalidTypeException.php',
    'Symfony\\Component\\Config\\Definition\\Exception\\UnsetKeyException' => $vendorDir . '/symfony/config/Definition/Exception/UnsetKeyException.php',
    'Symfony\\Component\\Config\\Definition\\FloatNode' => $vendorDir . '/symfony/config/Definition/FloatNode.php',
    'Symfony\\Component\\Config\\Definition\\IntegerNode' => $vendorDir . '/symfony/config/Definition/IntegerNode.php',
    'Symfony\\Component\\Config\\Definition\\Loader\\DefinitionFileLoader' => $vendorDir . '/symfony/config/Definition/Loader/DefinitionFileLoader.php',
    'Symfony\\Component\\Config\\Definition\\NodeInterface' => $vendorDir . '/symfony/config/Definition/NodeInterface.php',
    'Symfony\\Component\\Config\\Definition\\NumericNode' => $vendorDir . '/symfony/config/Definition/NumericNode.php',
    'Symfony\\Component\\Config\\Definition\\Processor' => $vendorDir . '/symfony/config/Definition/Processor.php',
    'Symfony\\Component\\Config\\Definition\\PrototypeNodeInterface' => $vendorDir . '/symfony/config/Definition/PrototypeNodeInterface.php',
    'Symfony\\Component\\Config\\Definition\\PrototypedArrayNode' => $vendorDir . '/symfony/config/Definition/PrototypedArrayNode.php',
    'Symfony\\Component\\Config\\Definition\\ScalarNode' => $vendorDir . '/symfony/config/Definition/ScalarNode.php',
    'Symfony\\Component\\Config\\Definition\\VariableNode' => $vendorDir . '/symfony/config/Definition/VariableNode.php',
    'Symfony\\Component\\Config\\Exception\\FileLoaderImportCircularReferenceException' => $vendorDir . '/symfony/config/Exception/FileLoaderImportCircularReferenceException.php',
    'Symfony\\Component\\Config\\Exception\\FileLocatorFileNotFoundException' => $vendorDir . '/symfony/config/Exception/FileLocatorFileNotFoundException.php',
    'Symfony\\Component\\Config\\Exception\\LoaderLoadException' => $vendorDir . '/symfony/config/Exception/LoaderLoadException.php',
    'Symfony\\Component\\Config\\FileLocator' => $vendorDir . '/symfony/config/FileLocator.php',
    'Symfony\\Component\\Config\\FileLocatorInterface' => $vendorDir . '/symfony/config/FileLocatorInterface.php',
    'Symfony\\Component\\Config\\Loader\\DelegatingLoader' => $vendorDir . '/symfony/config/Loader/DelegatingLoader.php',
    'Symfony\\Component\\Config\\Loader\\DirectoryAwareLoaderInterface' => $vendorDir . '/symfony/config/Loader/DirectoryAwareLoaderInterface.php',
    'Symfony\\Component\\Config\\Loader\\FileLoader' => $vendorDir . '/symfony/config/Loader/FileLoader.php',
    'Symfony\\Component\\Config\\Loader\\GlobFileLoader' => $vendorDir . '/symfony/config/Loader/GlobFileLoader.php',
    'Symfony\\Component\\Config\\Loader\\Loader' => $vendorDir . '/symfony/config/Loader/Loader.php',
    'Symfony\\Component\\Config\\Loader\\LoaderInterface' => $vendorDir . '/symfony/config/Loader/LoaderInterface.php',
    'Symfony\\Component\\Config\\Loader\\LoaderResolver' => $vendorDir . '/symfony/config/Loader/LoaderResolver.php',
    'Symfony\\Component\\Config\\Loader\\LoaderResolverInterface' => $vendorDir . '/symfony/config/Loader/LoaderResolverInterface.php',
    'Symfony\\Component\\Config\\Loader\\ParamConfigurator' => $vendorDir . '/symfony/config/Loader/ParamConfigurator.php',
    'Symfony\\Component\\Config\\ResourceCheckerConfigCache' => $vendorDir . '/symfony/config/ResourceCheckerConfigCache.php',
    'Symfony\\Component\\Config\\ResourceCheckerConfigCacheFactory' => $vendorDir . '/symfony/config/ResourceCheckerConfigCacheFactory.php',
    'Symfony\\Component\\Config\\ResourceCheckerInterface' => $vendorDir . '/symfony/config/ResourceCheckerInterface.php',
    'Symfony\\Component\\Config\\Resource\\ClassExistenceResource' => $vendorDir . '/symfony/config/Resource/ClassExistenceResource.php',
    'Symfony\\Component\\Config\\Resource\\ComposerResource' => $vendorDir . '/symfony/config/Resource/ComposerResource.php',
    'Symfony\\Component\\Config\\Resource\\DirectoryResource' => $vendorDir . '/symfony/config/Resource/DirectoryResource.php',
    'Symfony\\Component\\Config\\Resource\\FileExistenceResource' => $vendorDir . '/symfony/config/Resource/FileExistenceResource.php',
    'Symfony\\Component\\Config\\Resource\\FileResource' => $vendorDir . '/symfony/config/Resource/FileResource.php',
    'Symfony\\Component\\Config\\Resource\\GlobResource' => $vendorDir . '/symfony/config/Resource/GlobResource.php',
    'Symfony\\Component\\Config\\Resource\\ReflectionClassResource' => $vendorDir . '/symfony/config/Resource/ReflectionClassResource.php',
    'Symfony\\Component\\Config\\Resource\\ResourceInterface' => $vendorDir . '/symfony/config/Resource/ResourceInterface.php',
    'Symfony\\Component\\Config\\Resource\\SelfCheckingResourceChecker' => $vendorDir . '/symfony/config/Resource/SelfCheckingResourceChecker.php',
    'Symfony\\Component\\Config\\Resource\\SelfCheckingResourceInterface' => $vendorDir . '/symfony/config/Resource/SelfCheckingResourceInterface.php',
    'Symfony\\Component\\Config\\Util\\Exception\\InvalidXmlException' => $vendorDir . '/symfony/config/Util/Exception/InvalidXmlException.php',
    'Symfony\\Component\\Config\\Util\\Exception\\XmlParsingException' => $vendorDir . '/symfony/config/Util/Exception/XmlParsingException.php',
    'Symfony\\Component\\Config\\Util\\XmlUtils' => $vendorDir . '/symfony/config/Util/XmlUtils.php',
    'Symfony\\Component\\Console\\Application' => $vendorDir . '/symfony/console/Application.php',
    'Symfony\\Component\\Console\\Attribute\\AsCommand' => $vendorDir . '/symfony/console/Attribute/AsCommand.php',
    'Symfony\\Component\\Console\\CI\\GithubActionReporter' => $vendorDir . '/symfony/console/CI/GithubActionReporter.php',
    'Symfony\\Component\\Console\\Color' => $vendorDir . '/symfony/console/Color.php',
    'Symfony\\Component\\Console\\CommandLoader\\CommandLoaderInterface' => $vendorDir . '/symfony/console/CommandLoader/CommandLoaderInterface.php',
    'Symfony\\Component\\Console\\CommandLoader\\ContainerCommandLoader' => $vendorDir . '/symfony/console/CommandLoader/ContainerCommandLoader.php',
    'Symfony\\Component\\Console\\CommandLoader\\FactoryCommandLoader' => $vendorDir . '/symfony/console/CommandLoader/FactoryCommandLoader.php',
    'Symfony\\Component\\Console\\Command\\Command' => $vendorDir . '/symfony/console/Command/Command.php',
    'Symfony\\Component\\Console\\Command\\CompleteCommand' => $vendorDir . '/symfony/console/Command/CompleteCommand.php',
    'Symfony\\Component\\Console\\Command\\DumpCompletionCommand' => $vendorDir . '/symfony/console/Command/DumpCompletionCommand.php',
    'Symfony\\Component\\Console\\Command\\HelpCommand' => $vendorDir . '/symfony/console/Command/HelpCommand.php',
    'Symfony\\Component\\Console\\Command\\LazyCommand' => $vendorDir . '/symfony/console/Command/LazyCommand.php',
    'Symfony\\Component\\Console\\Command\\ListCommand' => $vendorDir . '/symfony/console/Command/ListCommand.php',
    'Symfony\\Component\\Console\\Command\\LockableTrait' => $vendorDir . '/symfony/console/Command/LockableTrait.php',
    'Symfony\\Component\\Console\\Command\\SignalableCommandInterface' => $vendorDir . '/symfony/console/Command/SignalableCommandInterface.php',
    'Symfony\\Component\\Console\\Command\\TraceableCommand' => $vendorDir . '/symfony/console/Command/TraceableCommand.php',
    'Symfony\\Component\\Console\\Completion\\CompletionInput' => $vendorDir . '/symfony/console/Completion/CompletionInput.php',
    'Symfony\\Component\\Console\\Completion\\CompletionSuggestions' => $vendorDir . '/symfony/console/Completion/CompletionSuggestions.php',
    'Symfony\\Component\\Console\\Completion\\Output\\BashCompletionOutput' => $vendorDir . '/symfony/console/Completion/Output/BashCompletionOutput.php',
    'Symfony\\Component\\Console\\Completion\\Output\\CompletionOutputInterface' => $vendorDir . '/symfony/console/Completion/Output/CompletionOutputInterface.php',
    'Symfony\\Component\\Console\\Completion\\Output\\FishCompletionOutput' => $vendorDir . '/symfony/console/Completion/Output/FishCompletionOutput.php',
    'Symfony\\Component\\Console\\Completion\\Output\\ZshCompletionOutput' => $vendorDir . '/symfony/console/Completion/Output/ZshCompletionOutput.php',
    'Symfony\\Component\\Console\\Completion\\Suggestion' => $vendorDir . '/symfony/console/Completion/Suggestion.php',
    'Symfony\\Component\\Console\\ConsoleEvents' => $vendorDir . '/symfony/console/ConsoleEvents.php',
    'Symfony\\Component\\Console\\Cursor' => $vendorDir . '/symfony/console/Cursor.php',
    'Symfony\\Component\\Console\\DataCollector\\CommandDataCollector' => $vendorDir . '/symfony/console/DataCollector/CommandDataCollector.php',
    'Symfony\\Component\\Console\\Debug\\CliRequest' => $vendorDir . '/symfony/console/Debug/CliRequest.php',
    'Symfony\\Component\\Console\\DependencyInjection\\AddConsoleCommandPass' => $vendorDir . '/symfony/console/DependencyInjection/AddConsoleCommandPass.php',
    'Symfony\\Component\\Console\\Descriptor\\ApplicationDescription' => $vendorDir . '/symfony/console/Descriptor/ApplicationDescription.php',
    'Symfony\\Component\\Console\\Descriptor\\Descriptor' => $vendorDir . '/symfony/console/Descriptor/Descriptor.php',
    'Symfony\\Component\\Console\\Descriptor\\DescriptorInterface' => $vendorDir . '/symfony/console/Descriptor/DescriptorInterface.php',
    'Symfony\\Component\\Console\\Descriptor\\JsonDescriptor' => $vendorDir . '/symfony/console/Descriptor/JsonDescriptor.php',
    'Symfony\\Component\\Console\\Descriptor\\MarkdownDescriptor' => $vendorDir . '/symfony/console/Descriptor/MarkdownDescriptor.php',
    'Symfony\\Component\\Console\\Descriptor\\ReStructuredTextDescriptor' => $vendorDir . '/symfony/console/Descriptor/ReStructuredTextDescriptor.php',
    'Symfony\\Component\\Console\\Descriptor\\TextDescriptor' => $vendorDir . '/symfony/console/Descriptor/TextDescriptor.php',
    'Symfony\\Component\\Console\\Descriptor\\XmlDescriptor' => $vendorDir . '/symfony/console/Descriptor/XmlDescriptor.php',
    'Symfony\\Component\\Console\\EventListener\\ErrorListener' => $vendorDir . '/symfony/console/EventListener/ErrorListener.php',
    'Symfony\\Component\\Console\\Event\\ConsoleCommandEvent' => $vendorDir . '/symfony/console/Event/ConsoleCommandEvent.php',
    'Symfony\\Component\\Console\\Event\\ConsoleErrorEvent' => $vendorDir . '/symfony/console/Event/ConsoleErrorEvent.php',
    'Symfony\\Component\\Console\\Event\\ConsoleEvent' => $vendorDir . '/symfony/console/Event/ConsoleEvent.php',
    'Symfony\\Component\\Console\\Event\\ConsoleSignalEvent' => $vendorDir . '/symfony/console/Event/ConsoleSignalEvent.php',
    'Symfony\\Component\\Console\\Event\\ConsoleTerminateEvent' => $vendorDir . '/symfony/console/Event/ConsoleTerminateEvent.php',
    'Symfony\\Component\\Console\\Exception\\CommandNotFoundException' => $vendorDir . '/symfony/console/Exception/CommandNotFoundException.php',
    'Symfony\\Component\\Console\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/console/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Console\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/console/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Console\\Exception\\InvalidOptionException' => $vendorDir . '/symfony/console/Exception/InvalidOptionException.php',
    'Symfony\\Component\\Console\\Exception\\LogicException' => $vendorDir . '/symfony/console/Exception/LogicException.php',
    'Symfony\\Component\\Console\\Exception\\MissingInputException' => $vendorDir . '/symfony/console/Exception/MissingInputException.php',
    'Symfony\\Component\\Console\\Exception\\NamespaceNotFoundException' => $vendorDir . '/symfony/console/Exception/NamespaceNotFoundException.php',
    'Symfony\\Component\\Console\\Exception\\RunCommandFailedException' => $vendorDir . '/symfony/console/Exception/RunCommandFailedException.php',
    'Symfony\\Component\\Console\\Exception\\RuntimeException' => $vendorDir . '/symfony/console/Exception/RuntimeException.php',
    'Symfony\\Component\\Console\\Formatter\\NullOutputFormatter' => $vendorDir . '/symfony/console/Formatter/NullOutputFormatter.php',
    'Symfony\\Component\\Console\\Formatter\\NullOutputFormatterStyle' => $vendorDir . '/symfony/console/Formatter/NullOutputFormatterStyle.php',
    'Symfony\\Component\\Console\\Formatter\\OutputFormatter' => $vendorDir . '/symfony/console/Formatter/OutputFormatter.php',
    'Symfony\\Component\\Console\\Formatter\\OutputFormatterInterface' => $vendorDir . '/symfony/console/Formatter/OutputFormatterInterface.php',
    'Symfony\\Component\\Console\\Formatter\\OutputFormatterStyle' => $vendorDir . '/symfony/console/Formatter/OutputFormatterStyle.php',
    'Symfony\\Component\\Console\\Formatter\\OutputFormatterStyleInterface' => $vendorDir . '/symfony/console/Formatter/OutputFormatterStyleInterface.php',
    'Symfony\\Component\\Console\\Formatter\\OutputFormatterStyleStack' => $vendorDir . '/symfony/console/Formatter/OutputFormatterStyleStack.php',
    'Symfony\\Component\\Console\\Formatter\\WrappableOutputFormatterInterface' => $vendorDir . '/symfony/console/Formatter/WrappableOutputFormatterInterface.php',
    'Symfony\\Component\\Console\\Helper\\DebugFormatterHelper' => $vendorDir . '/symfony/console/Helper/DebugFormatterHelper.php',
    'Symfony\\Component\\Console\\Helper\\DescriptorHelper' => $vendorDir . '/symfony/console/Helper/DescriptorHelper.php',
    'Symfony\\Component\\Console\\Helper\\Dumper' => $vendorDir . '/symfony/console/Helper/Dumper.php',
    'Symfony\\Component\\Console\\Helper\\FormatterHelper' => $vendorDir . '/symfony/console/Helper/FormatterHelper.php',
    'Symfony\\Component\\Console\\Helper\\Helper' => $vendorDir . '/symfony/console/Helper/Helper.php',
    'Symfony\\Component\\Console\\Helper\\HelperInterface' => $vendorDir . '/symfony/console/Helper/HelperInterface.php',
    'Symfony\\Component\\Console\\Helper\\HelperSet' => $vendorDir . '/symfony/console/Helper/HelperSet.php',
    'Symfony\\Component\\Console\\Helper\\InputAwareHelper' => $vendorDir . '/symfony/console/Helper/InputAwareHelper.php',
    'Symfony\\Component\\Console\\Helper\\OutputWrapper' => $vendorDir . '/symfony/console/Helper/OutputWrapper.php',
    'Symfony\\Component\\Console\\Helper\\ProcessHelper' => $vendorDir . '/symfony/console/Helper/ProcessHelper.php',
    'Symfony\\Component\\Console\\Helper\\ProgressBar' => $vendorDir . '/symfony/console/Helper/ProgressBar.php',
    'Symfony\\Component\\Console\\Helper\\ProgressIndicator' => $vendorDir . '/symfony/console/Helper/ProgressIndicator.php',
    'Symfony\\Component\\Console\\Helper\\QuestionHelper' => $vendorDir . '/symfony/console/Helper/QuestionHelper.php',
    'Symfony\\Component\\Console\\Helper\\SymfonyQuestionHelper' => $vendorDir . '/symfony/console/Helper/SymfonyQuestionHelper.php',
    'Symfony\\Component\\Console\\Helper\\Table' => $vendorDir . '/symfony/console/Helper/Table.php',
    'Symfony\\Component\\Console\\Helper\\TableCell' => $vendorDir . '/symfony/console/Helper/TableCell.php',
    'Symfony\\Component\\Console\\Helper\\TableCellStyle' => $vendorDir . '/symfony/console/Helper/TableCellStyle.php',
    'Symfony\\Component\\Console\\Helper\\TableRows' => $vendorDir . '/symfony/console/Helper/TableRows.php',
    'Symfony\\Component\\Console\\Helper\\TableSeparator' => $vendorDir . '/symfony/console/Helper/TableSeparator.php',
    'Symfony\\Component\\Console\\Helper\\TableStyle' => $vendorDir . '/symfony/console/Helper/TableStyle.php',
    'Symfony\\Component\\Console\\Helper\\TerminalInputHelper' => $vendorDir . '/symfony/console/Helper/TerminalInputHelper.php',
    'Symfony\\Component\\Console\\Input\\ArgvInput' => $vendorDir . '/symfony/console/Input/ArgvInput.php',
    'Symfony\\Component\\Console\\Input\\ArrayInput' => $vendorDir . '/symfony/console/Input/ArrayInput.php',
    'Symfony\\Component\\Console\\Input\\Input' => $vendorDir . '/symfony/console/Input/Input.php',
    'Symfony\\Component\\Console\\Input\\InputArgument' => $vendorDir . '/symfony/console/Input/InputArgument.php',
    'Symfony\\Component\\Console\\Input\\InputAwareInterface' => $vendorDir . '/symfony/console/Input/InputAwareInterface.php',
    'Symfony\\Component\\Console\\Input\\InputDefinition' => $vendorDir . '/symfony/console/Input/InputDefinition.php',
    'Symfony\\Component\\Console\\Input\\InputInterface' => $vendorDir . '/symfony/console/Input/InputInterface.php',
    'Symfony\\Component\\Console\\Input\\InputOption' => $vendorDir . '/symfony/console/Input/InputOption.php',
    'Symfony\\Component\\Console\\Input\\StreamableInputInterface' => $vendorDir . '/symfony/console/Input/StreamableInputInterface.php',
    'Symfony\\Component\\Console\\Input\\StringInput' => $vendorDir . '/symfony/console/Input/StringInput.php',
    'Symfony\\Component\\Console\\Logger\\ConsoleLogger' => $vendorDir . '/symfony/console/Logger/ConsoleLogger.php',
    'Symfony\\Component\\Console\\Messenger\\RunCommandContext' => $vendorDir . '/symfony/console/Messenger/RunCommandContext.php',
    'Symfony\\Component\\Console\\Messenger\\RunCommandMessage' => $vendorDir . '/symfony/console/Messenger/RunCommandMessage.php',
    'Symfony\\Component\\Console\\Messenger\\RunCommandMessageHandler' => $vendorDir . '/symfony/console/Messenger/RunCommandMessageHandler.php',
    'Symfony\\Component\\Console\\Output\\AnsiColorMode' => $vendorDir . '/symfony/console/Output/AnsiColorMode.php',
    'Symfony\\Component\\Console\\Output\\BufferedOutput' => $vendorDir . '/symfony/console/Output/BufferedOutput.php',
    'Symfony\\Component\\Console\\Output\\ConsoleOutput' => $vendorDir . '/symfony/console/Output/ConsoleOutput.php',
    'Symfony\\Component\\Console\\Output\\ConsoleOutputInterface' => $vendorDir . '/symfony/console/Output/ConsoleOutputInterface.php',
    'Symfony\\Component\\Console\\Output\\ConsoleSectionOutput' => $vendorDir . '/symfony/console/Output/ConsoleSectionOutput.php',
    'Symfony\\Component\\Console\\Output\\NullOutput' => $vendorDir . '/symfony/console/Output/NullOutput.php',
    'Symfony\\Component\\Console\\Output\\Output' => $vendorDir . '/symfony/console/Output/Output.php',
    'Symfony\\Component\\Console\\Output\\OutputInterface' => $vendorDir . '/symfony/console/Output/OutputInterface.php',
    'Symfony\\Component\\Console\\Output\\StreamOutput' => $vendorDir . '/symfony/console/Output/StreamOutput.php',
    'Symfony\\Component\\Console\\Output\\TrimmedBufferOutput' => $vendorDir . '/symfony/console/Output/TrimmedBufferOutput.php',
    'Symfony\\Component\\Console\\Question\\ChoiceQuestion' => $vendorDir . '/symfony/console/Question/ChoiceQuestion.php',
    'Symfony\\Component\\Console\\Question\\ConfirmationQuestion' => $vendorDir . '/symfony/console/Question/ConfirmationQuestion.php',
    'Symfony\\Component\\Console\\Question\\Question' => $vendorDir . '/symfony/console/Question/Question.php',
    'Symfony\\Component\\Console\\SignalRegistry\\SignalMap' => $vendorDir . '/symfony/console/SignalRegistry/SignalMap.php',
    'Symfony\\Component\\Console\\SignalRegistry\\SignalRegistry' => $vendorDir . '/symfony/console/SignalRegistry/SignalRegistry.php',
    'Symfony\\Component\\Console\\SingleCommandApplication' => $vendorDir . '/symfony/console/SingleCommandApplication.php',
    'Symfony\\Component\\Console\\Style\\OutputStyle' => $vendorDir . '/symfony/console/Style/OutputStyle.php',
    'Symfony\\Component\\Console\\Style\\StyleInterface' => $vendorDir . '/symfony/console/Style/StyleInterface.php',
    'Symfony\\Component\\Console\\Style\\SymfonyStyle' => $vendorDir . '/symfony/console/Style/SymfonyStyle.php',
    'Symfony\\Component\\Console\\Terminal' => $vendorDir . '/symfony/console/Terminal.php',
    'Symfony\\Component\\Console\\Tester\\ApplicationTester' => $vendorDir . '/symfony/console/Tester/ApplicationTester.php',
    'Symfony\\Component\\Console\\Tester\\CommandCompletionTester' => $vendorDir . '/symfony/console/Tester/CommandCompletionTester.php',
    'Symfony\\Component\\Console\\Tester\\CommandTester' => $vendorDir . '/symfony/console/Tester/CommandTester.php',
    'Symfony\\Component\\Console\\Tester\\Constraint\\CommandIsSuccessful' => $vendorDir . '/symfony/console/Tester/Constraint/CommandIsSuccessful.php',
    'Symfony\\Component\\Console\\Tester\\TesterTrait' => $vendorDir . '/symfony/console/Tester/TesterTrait.php',
    'Symfony\\Component\\CssSelector\\CssSelectorConverter' => $vendorDir . '/symfony/css-selector/CssSelectorConverter.php',
    'Symfony\\Component\\CssSelector\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/css-selector/Exception/ExceptionInterface.php',
    'Symfony\\Component\\CssSelector\\Exception\\ExpressionErrorException' => $vendorDir . '/symfony/css-selector/Exception/ExpressionErrorException.php',
    'Symfony\\Component\\CssSelector\\Exception\\InternalErrorException' => $vendorDir . '/symfony/css-selector/Exception/InternalErrorException.php',
    'Symfony\\Component\\CssSelector\\Exception\\ParseException' => $vendorDir . '/symfony/css-selector/Exception/ParseException.php',
    'Symfony\\Component\\CssSelector\\Exception\\SyntaxErrorException' => $vendorDir . '/symfony/css-selector/Exception/SyntaxErrorException.php',
    'Symfony\\Component\\CssSelector\\Node\\AbstractNode' => $vendorDir . '/symfony/css-selector/Node/AbstractNode.php',
    'Symfony\\Component\\CssSelector\\Node\\AttributeNode' => $vendorDir . '/symfony/css-selector/Node/AttributeNode.php',
    'Symfony\\Component\\CssSelector\\Node\\ClassNode' => $vendorDir . '/symfony/css-selector/Node/ClassNode.php',
    'Symfony\\Component\\CssSelector\\Node\\CombinedSelectorNode' => $vendorDir . '/symfony/css-selector/Node/CombinedSelectorNode.php',
    'Symfony\\Component\\CssSelector\\Node\\ElementNode' => $vendorDir . '/symfony/css-selector/Node/ElementNode.php',
    'Symfony\\Component\\CssSelector\\Node\\FunctionNode' => $vendorDir . '/symfony/css-selector/Node/FunctionNode.php',
    'Symfony\\Component\\CssSelector\\Node\\HashNode' => $vendorDir . '/symfony/css-selector/Node/HashNode.php',
    'Symfony\\Component\\CssSelector\\Node\\NegationNode' => $vendorDir . '/symfony/css-selector/Node/NegationNode.php',
    'Symfony\\Component\\CssSelector\\Node\\NodeInterface' => $vendorDir . '/symfony/css-selector/Node/NodeInterface.php',
    'Symfony\\Component\\CssSelector\\Node\\PseudoNode' => $vendorDir . '/symfony/css-selector/Node/PseudoNode.php',
    'Symfony\\Component\\CssSelector\\Node\\SelectorNode' => $vendorDir . '/symfony/css-selector/Node/SelectorNode.php',
    'Symfony\\Component\\CssSelector\\Node\\Specificity' => $vendorDir . '/symfony/css-selector/Node/Specificity.php',
    'Symfony\\Component\\CssSelector\\Parser\\Handler\\CommentHandler' => $vendorDir . '/symfony/css-selector/Parser/Handler/CommentHandler.php',
    'Symfony\\Component\\CssSelector\\Parser\\Handler\\HandlerInterface' => $vendorDir . '/symfony/css-selector/Parser/Handler/HandlerInterface.php',
    'Symfony\\Component\\CssSelector\\Parser\\Handler\\HashHandler' => $vendorDir . '/symfony/css-selector/Parser/Handler/HashHandler.php',
    'Symfony\\Component\\CssSelector\\Parser\\Handler\\IdentifierHandler' => $vendorDir . '/symfony/css-selector/Parser/Handler/IdentifierHandler.php',
    'Symfony\\Component\\CssSelector\\Parser\\Handler\\NumberHandler' => $vendorDir . '/symfony/css-selector/Parser/Handler/NumberHandler.php',
    'Symfony\\Component\\CssSelector\\Parser\\Handler\\StringHandler' => $vendorDir . '/symfony/css-selector/Parser/Handler/StringHandler.php',
    'Symfony\\Component\\CssSelector\\Parser\\Handler\\WhitespaceHandler' => $vendorDir . '/symfony/css-selector/Parser/Handler/WhitespaceHandler.php',
    'Symfony\\Component\\CssSelector\\Parser\\Parser' => $vendorDir . '/symfony/css-selector/Parser/Parser.php',
    'Symfony\\Component\\CssSelector\\Parser\\ParserInterface' => $vendorDir . '/symfony/css-selector/Parser/ParserInterface.php',
    'Symfony\\Component\\CssSelector\\Parser\\Reader' => $vendorDir . '/symfony/css-selector/Parser/Reader.php',
    'Symfony\\Component\\CssSelector\\Parser\\Shortcut\\ClassParser' => $vendorDir . '/symfony/css-selector/Parser/Shortcut/ClassParser.php',
    'Symfony\\Component\\CssSelector\\Parser\\Shortcut\\ElementParser' => $vendorDir . '/symfony/css-selector/Parser/Shortcut/ElementParser.php',
    'Symfony\\Component\\CssSelector\\Parser\\Shortcut\\EmptyStringParser' => $vendorDir . '/symfony/css-selector/Parser/Shortcut/EmptyStringParser.php',
    'Symfony\\Component\\CssSelector\\Parser\\Shortcut\\HashParser' => $vendorDir . '/symfony/css-selector/Parser/Shortcut/HashParser.php',
    'Symfony\\Component\\CssSelector\\Parser\\Token' => $vendorDir . '/symfony/css-selector/Parser/Token.php',
    'Symfony\\Component\\CssSelector\\Parser\\TokenStream' => $vendorDir . '/symfony/css-selector/Parser/TokenStream.php',
    'Symfony\\Component\\CssSelector\\Parser\\Tokenizer\\Tokenizer' => $vendorDir . '/symfony/css-selector/Parser/Tokenizer/Tokenizer.php',
    'Symfony\\Component\\CssSelector\\Parser\\Tokenizer\\TokenizerEscaping' => $vendorDir . '/symfony/css-selector/Parser/Tokenizer/TokenizerEscaping.php',
    'Symfony\\Component\\CssSelector\\Parser\\Tokenizer\\TokenizerPatterns' => $vendorDir . '/symfony/css-selector/Parser/Tokenizer/TokenizerPatterns.php',
    'Symfony\\Component\\CssSelector\\XPath\\Extension\\AbstractExtension' => $vendorDir . '/symfony/css-selector/XPath/Extension/AbstractExtension.php',
    'Symfony\\Component\\CssSelector\\XPath\\Extension\\AttributeMatchingExtension' => $vendorDir . '/symfony/css-selector/XPath/Extension/AttributeMatchingExtension.php',
    'Symfony\\Component\\CssSelector\\XPath\\Extension\\CombinationExtension' => $vendorDir . '/symfony/css-selector/XPath/Extension/CombinationExtension.php',
    'Symfony\\Component\\CssSelector\\XPath\\Extension\\ExtensionInterface' => $vendorDir . '/symfony/css-selector/XPath/Extension/ExtensionInterface.php',
    'Symfony\\Component\\CssSelector\\XPath\\Extension\\FunctionExtension' => $vendorDir . '/symfony/css-selector/XPath/Extension/FunctionExtension.php',
    'Symfony\\Component\\CssSelector\\XPath\\Extension\\HtmlExtension' => $vendorDir . '/symfony/css-selector/XPath/Extension/HtmlExtension.php',
    'Symfony\\Component\\CssSelector\\XPath\\Extension\\NodeExtension' => $vendorDir . '/symfony/css-selector/XPath/Extension/NodeExtension.php',
    'Symfony\\Component\\CssSelector\\XPath\\Extension\\PseudoClassExtension' => $vendorDir . '/symfony/css-selector/XPath/Extension/PseudoClassExtension.php',
    'Symfony\\Component\\CssSelector\\XPath\\Translator' => $vendorDir . '/symfony/css-selector/XPath/Translator.php',
    'Symfony\\Component\\CssSelector\\XPath\\TranslatorInterface' => $vendorDir . '/symfony/css-selector/XPath/TranslatorInterface.php',
    'Symfony\\Component\\CssSelector\\XPath\\XPathExpr' => $vendorDir . '/symfony/css-selector/XPath/XPathExpr.php',
    'Symfony\\Component\\DependencyInjection\\Alias' => $vendorDir . '/symfony/dependency-injection/Alias.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\AbstractArgument' => $vendorDir . '/symfony/dependency-injection/Argument/AbstractArgument.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\ArgumentInterface' => $vendorDir . '/symfony/dependency-injection/Argument/ArgumentInterface.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\BoundArgument' => $vendorDir . '/symfony/dependency-injection/Argument/BoundArgument.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\IteratorArgument' => $vendorDir . '/symfony/dependency-injection/Argument/IteratorArgument.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\LazyClosure' => $vendorDir . '/symfony/dependency-injection/Argument/LazyClosure.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\ReferenceSetArgumentTrait' => $vendorDir . '/symfony/dependency-injection/Argument/ReferenceSetArgumentTrait.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\RewindableGenerator' => $vendorDir . '/symfony/dependency-injection/Argument/RewindableGenerator.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\ServiceClosureArgument' => $vendorDir . '/symfony/dependency-injection/Argument/ServiceClosureArgument.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\ServiceLocator' => $vendorDir . '/symfony/dependency-injection/Argument/ServiceLocator.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\ServiceLocatorArgument' => $vendorDir . '/symfony/dependency-injection/Argument/ServiceLocatorArgument.php',
    'Symfony\\Component\\DependencyInjection\\Argument\\TaggedIteratorArgument' => $vendorDir . '/symfony/dependency-injection/Argument/TaggedIteratorArgument.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AsAlias' => $vendorDir . '/symfony/dependency-injection/Attribute/AsAlias.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AsDecorator' => $vendorDir . '/symfony/dependency-injection/Attribute/AsDecorator.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AsTaggedItem' => $vendorDir . '/symfony/dependency-injection/Attribute/AsTaggedItem.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\Autoconfigure' => $vendorDir . '/symfony/dependency-injection/Attribute/Autoconfigure.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AutoconfigureTag' => $vendorDir . '/symfony/dependency-injection/Attribute/AutoconfigureTag.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\Autowire' => $vendorDir . '/symfony/dependency-injection/Attribute/Autowire.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AutowireCallable' => $vendorDir . '/symfony/dependency-injection/Attribute/AutowireCallable.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AutowireDecorated' => $vendorDir . '/symfony/dependency-injection/Attribute/AutowireDecorated.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AutowireIterator' => $vendorDir . '/symfony/dependency-injection/Attribute/AutowireIterator.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AutowireLocator' => $vendorDir . '/symfony/dependency-injection/Attribute/AutowireLocator.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\AutowireServiceClosure' => $vendorDir . '/symfony/dependency-injection/Attribute/AutowireServiceClosure.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\Exclude' => $vendorDir . '/symfony/dependency-injection/Attribute/Exclude.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\MapDecorated' => $vendorDir . '/symfony/dependency-injection/Attribute/MapDecorated.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\TaggedIterator' => $vendorDir . '/symfony/dependency-injection/Attribute/TaggedIterator.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\TaggedLocator' => $vendorDir . '/symfony/dependency-injection/Attribute/TaggedLocator.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\Target' => $vendorDir . '/symfony/dependency-injection/Attribute/Target.php',
    'Symfony\\Component\\DependencyInjection\\Attribute\\When' => $vendorDir . '/symfony/dependency-injection/Attribute/When.php',
    'Symfony\\Component\\DependencyInjection\\ChildDefinition' => $vendorDir . '/symfony/dependency-injection/ChildDefinition.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\AbstractRecursivePass' => $vendorDir . '/symfony/dependency-injection/Compiler/AbstractRecursivePass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\AliasDeprecatedPublicServicesPass' => $vendorDir . '/symfony/dependency-injection/Compiler/AliasDeprecatedPublicServicesPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\AnalyzeServiceReferencesPass' => $vendorDir . '/symfony/dependency-injection/Compiler/AnalyzeServiceReferencesPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\AttributeAutoconfigurationPass' => $vendorDir . '/symfony/dependency-injection/Compiler/AttributeAutoconfigurationPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\AutoAliasServicePass' => $vendorDir . '/symfony/dependency-injection/Compiler/AutoAliasServicePass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\AutowireAsDecoratorPass' => $vendorDir . '/symfony/dependency-injection/Compiler/AutowireAsDecoratorPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\AutowirePass' => $vendorDir . '/symfony/dependency-injection/Compiler/AutowirePass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\AutowireRequiredMethodsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/AutowireRequiredMethodsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\AutowireRequiredPropertiesPass' => $vendorDir . '/symfony/dependency-injection/Compiler/AutowireRequiredPropertiesPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\CheckArgumentsValidityPass' => $vendorDir . '/symfony/dependency-injection/Compiler/CheckArgumentsValidityPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\CheckCircularReferencesPass' => $vendorDir . '/symfony/dependency-injection/Compiler/CheckCircularReferencesPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\CheckDefinitionValidityPass' => $vendorDir . '/symfony/dependency-injection/Compiler/CheckDefinitionValidityPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\CheckExceptionOnInvalidReferenceBehaviorPass' => $vendorDir . '/symfony/dependency-injection/Compiler/CheckExceptionOnInvalidReferenceBehaviorPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\CheckReferenceValidityPass' => $vendorDir . '/symfony/dependency-injection/Compiler/CheckReferenceValidityPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\CheckTypeDeclarationsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/CheckTypeDeclarationsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\Compiler' => $vendorDir . '/symfony/dependency-injection/Compiler/Compiler.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\CompilerPassInterface' => $vendorDir . '/symfony/dependency-injection/Compiler/CompilerPassInterface.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\DecoratorServicePass' => $vendorDir . '/symfony/dependency-injection/Compiler/DecoratorServicePass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\DefinitionErrorExceptionPass' => $vendorDir . '/symfony/dependency-injection/Compiler/DefinitionErrorExceptionPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ExtensionCompilerPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ExtensionCompilerPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\InlineServiceDefinitionsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/InlineServiceDefinitionsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\MergeExtensionConfigurationPass' => $vendorDir . '/symfony/dependency-injection/Compiler/MergeExtensionConfigurationPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\PassConfig' => $vendorDir . '/symfony/dependency-injection/Compiler/PassConfig.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\PriorityTaggedServiceTrait' => $vendorDir . '/symfony/dependency-injection/Compiler/PriorityTaggedServiceTrait.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\RegisterAutoconfigureAttributesPass' => $vendorDir . '/symfony/dependency-injection/Compiler/RegisterAutoconfigureAttributesPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\RegisterEnvVarProcessorsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/RegisterEnvVarProcessorsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\RegisterReverseContainerPass' => $vendorDir . '/symfony/dependency-injection/Compiler/RegisterReverseContainerPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\RegisterServiceSubscribersPass' => $vendorDir . '/symfony/dependency-injection/Compiler/RegisterServiceSubscribersPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\RemoveAbstractDefinitionsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/RemoveAbstractDefinitionsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\RemoveBuildParametersPass' => $vendorDir . '/symfony/dependency-injection/Compiler/RemoveBuildParametersPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\RemovePrivateAliasesPass' => $vendorDir . '/symfony/dependency-injection/Compiler/RemovePrivateAliasesPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\RemoveUnusedDefinitionsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/RemoveUnusedDefinitionsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ReplaceAliasByActualDefinitionPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ReplaceAliasByActualDefinitionPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveBindingsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveBindingsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveChildDefinitionsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveChildDefinitionsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveClassPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveClassPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveDecoratorStackPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveDecoratorStackPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveEnvPlaceholdersPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveEnvPlaceholdersPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveFactoryClassPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveFactoryClassPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveHotPathPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveHotPathPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveInstanceofConditionalsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveInstanceofConditionalsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveInvalidReferencesPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveInvalidReferencesPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveNamedArgumentsPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveNamedArgumentsPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveNoPreloadPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveNoPreloadPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveParameterPlaceHoldersPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveParameterPlaceHoldersPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveReferencesToAliasesPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveReferencesToAliasesPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveServiceSubscribersPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveServiceSubscribersPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ResolveTaggedIteratorArgumentPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ResolveTaggedIteratorArgumentPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ServiceLocatorTagPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ServiceLocatorTagPass.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ServiceReferenceGraph' => $vendorDir . '/symfony/dependency-injection/Compiler/ServiceReferenceGraph.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ServiceReferenceGraphEdge' => $vendorDir . '/symfony/dependency-injection/Compiler/ServiceReferenceGraphEdge.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ServiceReferenceGraphNode' => $vendorDir . '/symfony/dependency-injection/Compiler/ServiceReferenceGraphNode.php',
    'Symfony\\Component\\DependencyInjection\\Compiler\\ValidateEnvPlaceholdersPass' => $vendorDir . '/symfony/dependency-injection/Compiler/ValidateEnvPlaceholdersPass.php',
    'Symfony\\Component\\DependencyInjection\\Config\\ContainerParametersResource' => $vendorDir . '/symfony/dependency-injection/Config/ContainerParametersResource.php',
    'Symfony\\Component\\DependencyInjection\\Config\\ContainerParametersResourceChecker' => $vendorDir . '/symfony/dependency-injection/Config/ContainerParametersResourceChecker.php',
    'Symfony\\Component\\DependencyInjection\\Container' => $vendorDir . '/symfony/dependency-injection/Container.php',
    'Symfony\\Component\\DependencyInjection\\ContainerAwareInterface' => $vendorDir . '/symfony/dependency-injection/ContainerAwareInterface.php',
    'Symfony\\Component\\DependencyInjection\\ContainerAwareTrait' => $vendorDir . '/symfony/dependency-injection/ContainerAwareTrait.php',
    'Symfony\\Component\\DependencyInjection\\ContainerBuilder' => $vendorDir . '/symfony/dependency-injection/ContainerBuilder.php',
    'Symfony\\Component\\DependencyInjection\\ContainerInterface' => $vendorDir . '/symfony/dependency-injection/ContainerInterface.php',
    'Symfony\\Component\\DependencyInjection\\Definition' => $vendorDir . '/symfony/dependency-injection/Definition.php',
    'Symfony\\Component\\DependencyInjection\\Dumper\\Dumper' => $vendorDir . '/symfony/dependency-injection/Dumper/Dumper.php',
    'Symfony\\Component\\DependencyInjection\\Dumper\\DumperInterface' => $vendorDir . '/symfony/dependency-injection/Dumper/DumperInterface.php',
    'Symfony\\Component\\DependencyInjection\\Dumper\\GraphvizDumper' => $vendorDir . '/symfony/dependency-injection/Dumper/GraphvizDumper.php',
    'Symfony\\Component\\DependencyInjection\\Dumper\\PhpDumper' => $vendorDir . '/symfony/dependency-injection/Dumper/PhpDumper.php',
    'Symfony\\Component\\DependencyInjection\\Dumper\\Preloader' => $vendorDir . '/symfony/dependency-injection/Dumper/Preloader.php',
    'Symfony\\Component\\DependencyInjection\\Dumper\\XmlDumper' => $vendorDir . '/symfony/dependency-injection/Dumper/XmlDumper.php',
    'Symfony\\Component\\DependencyInjection\\Dumper\\YamlDumper' => $vendorDir . '/symfony/dependency-injection/Dumper/YamlDumper.php',
    'Symfony\\Component\\DependencyInjection\\EnvVarLoaderInterface' => $vendorDir . '/symfony/dependency-injection/EnvVarLoaderInterface.php',
    'Symfony\\Component\\DependencyInjection\\EnvVarProcessor' => $vendorDir . '/symfony/dependency-injection/EnvVarProcessor.php',
    'Symfony\\Component\\DependencyInjection\\EnvVarProcessorInterface' => $vendorDir . '/symfony/dependency-injection/EnvVarProcessorInterface.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\AutowiringFailedException' => $vendorDir . '/symfony/dependency-injection/Exception/AutowiringFailedException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\BadMethodCallException' => $vendorDir . '/symfony/dependency-injection/Exception/BadMethodCallException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\EnvNotFoundException' => $vendorDir . '/symfony/dependency-injection/Exception/EnvNotFoundException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\EnvParameterException' => $vendorDir . '/symfony/dependency-injection/Exception/EnvParameterException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/dependency-injection/Exception/ExceptionInterface.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/dependency-injection/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\InvalidParameterTypeException' => $vendorDir . '/symfony/dependency-injection/Exception/InvalidParameterTypeException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\LogicException' => $vendorDir . '/symfony/dependency-injection/Exception/LogicException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\OutOfBoundsException' => $vendorDir . '/symfony/dependency-injection/Exception/OutOfBoundsException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\ParameterCircularReferenceException' => $vendorDir . '/symfony/dependency-injection/Exception/ParameterCircularReferenceException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\ParameterNotFoundException' => $vendorDir . '/symfony/dependency-injection/Exception/ParameterNotFoundException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\RuntimeException' => $vendorDir . '/symfony/dependency-injection/Exception/RuntimeException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\ServiceCircularReferenceException' => $vendorDir . '/symfony/dependency-injection/Exception/ServiceCircularReferenceException.php',
    'Symfony\\Component\\DependencyInjection\\Exception\\ServiceNotFoundException' => $vendorDir . '/symfony/dependency-injection/Exception/ServiceNotFoundException.php',
    'Symfony\\Component\\DependencyInjection\\ExpressionLanguage' => $vendorDir . '/symfony/dependency-injection/ExpressionLanguage.php',
    'Symfony\\Component\\DependencyInjection\\ExpressionLanguageProvider' => $vendorDir . '/symfony/dependency-injection/ExpressionLanguageProvider.php',
    'Symfony\\Component\\DependencyInjection\\Extension\\AbstractExtension' => $vendorDir . '/symfony/dependency-injection/Extension/AbstractExtension.php',
    'Symfony\\Component\\DependencyInjection\\Extension\\ConfigurableExtensionInterface' => $vendorDir . '/symfony/dependency-injection/Extension/ConfigurableExtensionInterface.php',
    'Symfony\\Component\\DependencyInjection\\Extension\\ConfigurationExtensionInterface' => $vendorDir . '/symfony/dependency-injection/Extension/ConfigurationExtensionInterface.php',
    'Symfony\\Component\\DependencyInjection\\Extension\\Extension' => $vendorDir . '/symfony/dependency-injection/Extension/Extension.php',
    'Symfony\\Component\\DependencyInjection\\Extension\\ExtensionInterface' => $vendorDir . '/symfony/dependency-injection/Extension/ExtensionInterface.php',
    'Symfony\\Component\\DependencyInjection\\Extension\\ExtensionTrait' => $vendorDir . '/symfony/dependency-injection/Extension/ExtensionTrait.php',
    'Symfony\\Component\\DependencyInjection\\Extension\\PrependExtensionInterface' => $vendorDir . '/symfony/dependency-injection/Extension/PrependExtensionInterface.php',
    'Symfony\\Component\\DependencyInjection\\LazyProxy\\Instantiator\\InstantiatorInterface' => $vendorDir . '/symfony/dependency-injection/LazyProxy/Instantiator/InstantiatorInterface.php',
    'Symfony\\Component\\DependencyInjection\\LazyProxy\\Instantiator\\LazyServiceInstantiator' => $vendorDir . '/symfony/dependency-injection/LazyProxy/Instantiator/LazyServiceInstantiator.php',
    'Symfony\\Component\\DependencyInjection\\LazyProxy\\Instantiator\\RealServiceInstantiator' => $vendorDir . '/symfony/dependency-injection/LazyProxy/Instantiator/RealServiceInstantiator.php',
    'Symfony\\Component\\DependencyInjection\\LazyProxy\\PhpDumper\\DumperInterface' => $vendorDir . '/symfony/dependency-injection/LazyProxy/PhpDumper/DumperInterface.php',
    'Symfony\\Component\\DependencyInjection\\LazyProxy\\PhpDumper\\LazyServiceDumper' => $vendorDir . '/symfony/dependency-injection/LazyProxy/PhpDumper/LazyServiceDumper.php',
    'Symfony\\Component\\DependencyInjection\\LazyProxy\\PhpDumper\\NullDumper' => $vendorDir . '/symfony/dependency-injection/LazyProxy/PhpDumper/NullDumper.php',
    'Symfony\\Component\\DependencyInjection\\LazyProxy\\ProxyHelper' => $vendorDir . '/symfony/dependency-injection/LazyProxy/ProxyHelper.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\ClosureLoader' => $vendorDir . '/symfony/dependency-injection/Loader/ClosureLoader.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\AbstractConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/AbstractConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\AbstractServiceConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/AbstractServiceConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\AliasConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/AliasConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\ClosureReferenceConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/ClosureReferenceConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\ContainerConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/ContainerConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\DefaultsConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/DefaultsConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\EnvConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/EnvConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\FromCallableConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/FromCallableConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\InlineServiceConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/InlineServiceConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\InstanceofConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/InstanceofConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\ParametersConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/ParametersConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\PrototypeConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/PrototypeConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\ReferenceConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/ReferenceConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\ServiceConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/ServiceConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\ServicesConfigurator' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/ServicesConfigurator.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\AbstractTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/AbstractTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\ArgumentTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/ArgumentTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\AutoconfigureTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/AutoconfigureTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\AutowireTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/AutowireTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\BindTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/BindTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\CallTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/CallTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\ClassTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/ClassTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\ConfiguratorTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/ConfiguratorTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\ConstructorTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/ConstructorTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\DecorateTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/DecorateTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\DeprecateTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/DeprecateTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\FactoryTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/FactoryTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\FileTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/FileTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\FromCallableTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/FromCallableTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\LazyTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/LazyTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\ParentTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/ParentTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\PropertyTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/PropertyTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\PublicTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/PublicTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\ShareTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/ShareTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\SyntheticTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/SyntheticTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\Configurator\\Traits\\TagTrait' => $vendorDir . '/symfony/dependency-injection/Loader/Configurator/Traits/TagTrait.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\DirectoryLoader' => $vendorDir . '/symfony/dependency-injection/Loader/DirectoryLoader.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\FileLoader' => $vendorDir . '/symfony/dependency-injection/Loader/FileLoader.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\GlobFileLoader' => $vendorDir . '/symfony/dependency-injection/Loader/GlobFileLoader.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\IniFileLoader' => $vendorDir . '/symfony/dependency-injection/Loader/IniFileLoader.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\PhpFileLoader' => $vendorDir . '/symfony/dependency-injection/Loader/PhpFileLoader.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\XmlFileLoader' => $vendorDir . '/symfony/dependency-injection/Loader/XmlFileLoader.php',
    'Symfony\\Component\\DependencyInjection\\Loader\\YamlFileLoader' => $vendorDir . '/symfony/dependency-injection/Loader/YamlFileLoader.php',
    'Symfony\\Component\\DependencyInjection\\Parameter' => $vendorDir . '/symfony/dependency-injection/Parameter.php',
    'Symfony\\Component\\DependencyInjection\\ParameterBag\\ContainerBag' => $vendorDir . '/symfony/dependency-injection/ParameterBag/ContainerBag.php',
    'Symfony\\Component\\DependencyInjection\\ParameterBag\\ContainerBagInterface' => $vendorDir . '/symfony/dependency-injection/ParameterBag/ContainerBagInterface.php',
    'Symfony\\Component\\DependencyInjection\\ParameterBag\\EnvPlaceholderParameterBag' => $vendorDir . '/symfony/dependency-injection/ParameterBag/EnvPlaceholderParameterBag.php',
    'Symfony\\Component\\DependencyInjection\\ParameterBag\\FrozenParameterBag' => $vendorDir . '/symfony/dependency-injection/ParameterBag/FrozenParameterBag.php',
    'Symfony\\Component\\DependencyInjection\\ParameterBag\\ParameterBag' => $vendorDir . '/symfony/dependency-injection/ParameterBag/ParameterBag.php',
    'Symfony\\Component\\DependencyInjection\\ParameterBag\\ParameterBagInterface' => $vendorDir . '/symfony/dependency-injection/ParameterBag/ParameterBagInterface.php',
    'Symfony\\Component\\DependencyInjection\\Reference' => $vendorDir . '/symfony/dependency-injection/Reference.php',
    'Symfony\\Component\\DependencyInjection\\ReverseContainer' => $vendorDir . '/symfony/dependency-injection/ReverseContainer.php',
    'Symfony\\Component\\DependencyInjection\\ServiceLocator' => $vendorDir . '/symfony/dependency-injection/ServiceLocator.php',
    'Symfony\\Component\\DependencyInjection\\TaggedContainerInterface' => $vendorDir . '/symfony/dependency-injection/TaggedContainerInterface.php',
    'Symfony\\Component\\DependencyInjection\\TypedReference' => $vendorDir . '/symfony/dependency-injection/TypedReference.php',
    'Symfony\\Component\\DependencyInjection\\Variable' => $vendorDir . '/symfony/dependency-injection/Variable.php',
    'Symfony\\Component\\DomCrawler\\AbstractUriElement' => $vendorDir . '/symfony/dom-crawler/AbstractUriElement.php',
    'Symfony\\Component\\DomCrawler\\Crawler' => $vendorDir . '/symfony/dom-crawler/Crawler.php',
    'Symfony\\Component\\DomCrawler\\Field\\ChoiceFormField' => $vendorDir . '/symfony/dom-crawler/Field/ChoiceFormField.php',
    'Symfony\\Component\\DomCrawler\\Field\\FileFormField' => $vendorDir . '/symfony/dom-crawler/Field/FileFormField.php',
    'Symfony\\Component\\DomCrawler\\Field\\FormField' => $vendorDir . '/symfony/dom-crawler/Field/FormField.php',
    'Symfony\\Component\\DomCrawler\\Field\\InputFormField' => $vendorDir . '/symfony/dom-crawler/Field/InputFormField.php',
    'Symfony\\Component\\DomCrawler\\Field\\TextareaFormField' => $vendorDir . '/symfony/dom-crawler/Field/TextareaFormField.php',
    'Symfony\\Component\\DomCrawler\\Form' => $vendorDir . '/symfony/dom-crawler/Form.php',
    'Symfony\\Component\\DomCrawler\\FormFieldRegistry' => $vendorDir . '/symfony/dom-crawler/FormFieldRegistry.php',
    'Symfony\\Component\\DomCrawler\\Image' => $vendorDir . '/symfony/dom-crawler/Image.php',
    'Symfony\\Component\\DomCrawler\\Link' => $vendorDir . '/symfony/dom-crawler/Link.php',
    'Symfony\\Component\\DomCrawler\\Test\\Constraint\\CrawlerAnySelectorTextContains' => $vendorDir . '/symfony/dom-crawler/Test/Constraint/CrawlerAnySelectorTextContains.php',
    'Symfony\\Component\\DomCrawler\\Test\\Constraint\\CrawlerAnySelectorTextSame' => $vendorDir . '/symfony/dom-crawler/Test/Constraint/CrawlerAnySelectorTextSame.php',
    'Symfony\\Component\\DomCrawler\\Test\\Constraint\\CrawlerSelectorAttributeValueSame' => $vendorDir . '/symfony/dom-crawler/Test/Constraint/CrawlerSelectorAttributeValueSame.php',
    'Symfony\\Component\\DomCrawler\\Test\\Constraint\\CrawlerSelectorCount' => $vendorDir . '/symfony/dom-crawler/Test/Constraint/CrawlerSelectorCount.php',
    'Symfony\\Component\\DomCrawler\\Test\\Constraint\\CrawlerSelectorExists' => $vendorDir . '/symfony/dom-crawler/Test/Constraint/CrawlerSelectorExists.php',
    'Symfony\\Component\\DomCrawler\\Test\\Constraint\\CrawlerSelectorTextContains' => $vendorDir . '/symfony/dom-crawler/Test/Constraint/CrawlerSelectorTextContains.php',
    'Symfony\\Component\\DomCrawler\\Test\\Constraint\\CrawlerSelectorTextSame' => $vendorDir . '/symfony/dom-crawler/Test/Constraint/CrawlerSelectorTextSame.php',
    'Symfony\\Component\\DomCrawler\\UriResolver' => $vendorDir . '/symfony/dom-crawler/UriResolver.php',
    'Symfony\\Component\\ErrorHandler\\BufferingLogger' => $vendorDir . '/symfony/error-handler/BufferingLogger.php',
    'Symfony\\Component\\ErrorHandler\\Debug' => $vendorDir . '/symfony/error-handler/Debug.php',
    'Symfony\\Component\\ErrorHandler\\DebugClassLoader' => $vendorDir . '/symfony/error-handler/DebugClassLoader.php',
    'Symfony\\Component\\ErrorHandler\\ErrorEnhancer\\ClassNotFoundErrorEnhancer' => $vendorDir . '/symfony/error-handler/ErrorEnhancer/ClassNotFoundErrorEnhancer.php',
    'Symfony\\Component\\ErrorHandler\\ErrorEnhancer\\ErrorEnhancerInterface' => $vendorDir . '/symfony/error-handler/ErrorEnhancer/ErrorEnhancerInterface.php',
    'Symfony\\Component\\ErrorHandler\\ErrorEnhancer\\UndefinedFunctionErrorEnhancer' => $vendorDir . '/symfony/error-handler/ErrorEnhancer/UndefinedFunctionErrorEnhancer.php',
    'Symfony\\Component\\ErrorHandler\\ErrorEnhancer\\UndefinedMethodErrorEnhancer' => $vendorDir . '/symfony/error-handler/ErrorEnhancer/UndefinedMethodErrorEnhancer.php',
    'Symfony\\Component\\ErrorHandler\\ErrorHandler' => $vendorDir . '/symfony/error-handler/ErrorHandler.php',
    'Symfony\\Component\\ErrorHandler\\ErrorRenderer\\CliErrorRenderer' => $vendorDir . '/symfony/error-handler/ErrorRenderer/CliErrorRenderer.php',
    'Symfony\\Component\\ErrorHandler\\ErrorRenderer\\ErrorRendererInterface' => $vendorDir . '/symfony/error-handler/ErrorRenderer/ErrorRendererInterface.php',
    'Symfony\\Component\\ErrorHandler\\ErrorRenderer\\FileLinkFormatter' => $vendorDir . '/symfony/error-handler/ErrorRenderer/FileLinkFormatter.php',
    'Symfony\\Component\\ErrorHandler\\ErrorRenderer\\HtmlErrorRenderer' => $vendorDir . '/symfony/error-handler/ErrorRenderer/HtmlErrorRenderer.php',
    'Symfony\\Component\\ErrorHandler\\ErrorRenderer\\SerializerErrorRenderer' => $vendorDir . '/symfony/error-handler/ErrorRenderer/SerializerErrorRenderer.php',
    'Symfony\\Component\\ErrorHandler\\Error\\ClassNotFoundError' => $vendorDir . '/symfony/error-handler/Error/ClassNotFoundError.php',
    'Symfony\\Component\\ErrorHandler\\Error\\FatalError' => $vendorDir . '/symfony/error-handler/Error/FatalError.php',
    'Symfony\\Component\\ErrorHandler\\Error\\OutOfMemoryError' => $vendorDir . '/symfony/error-handler/Error/OutOfMemoryError.php',
    'Symfony\\Component\\ErrorHandler\\Error\\UndefinedFunctionError' => $vendorDir . '/symfony/error-handler/Error/UndefinedFunctionError.php',
    'Symfony\\Component\\ErrorHandler\\Error\\UndefinedMethodError' => $vendorDir . '/symfony/error-handler/Error/UndefinedMethodError.php',
    'Symfony\\Component\\ErrorHandler\\Exception\\FlattenException' => $vendorDir . '/symfony/error-handler/Exception/FlattenException.php',
    'Symfony\\Component\\ErrorHandler\\Exception\\SilencedErrorContext' => $vendorDir . '/symfony/error-handler/Exception/SilencedErrorContext.php',
    'Symfony\\Component\\ErrorHandler\\Internal\\TentativeTypes' => $vendorDir . '/symfony/error-handler/Internal/TentativeTypes.php',
    'Symfony\\Component\\ErrorHandler\\ThrowableUtils' => $vendorDir . '/symfony/error-handler/ThrowableUtils.php',
    'Symfony\\Component\\EventDispatcher\\Attribute\\AsEventListener' => $vendorDir . '/symfony/event-dispatcher/Attribute/AsEventListener.php',
    'Symfony\\Component\\EventDispatcher\\Debug\\TraceableEventDispatcher' => $vendorDir . '/symfony/event-dispatcher/Debug/TraceableEventDispatcher.php',
    'Symfony\\Component\\EventDispatcher\\Debug\\WrappedListener' => $vendorDir . '/symfony/event-dispatcher/Debug/WrappedListener.php',
    'Symfony\\Component\\EventDispatcher\\DependencyInjection\\AddEventAliasesPass' => $vendorDir . '/symfony/event-dispatcher/DependencyInjection/AddEventAliasesPass.php',
    'Symfony\\Component\\EventDispatcher\\DependencyInjection\\RegisterListenersPass' => $vendorDir . '/symfony/event-dispatcher/DependencyInjection/RegisterListenersPass.php',
    'Symfony\\Component\\EventDispatcher\\EventDispatcher' => $vendorDir . '/symfony/event-dispatcher/EventDispatcher.php',
    'Symfony\\Component\\EventDispatcher\\EventDispatcherInterface' => $vendorDir . '/symfony/event-dispatcher/EventDispatcherInterface.php',
    'Symfony\\Component\\EventDispatcher\\EventSubscriberInterface' => $vendorDir . '/symfony/event-dispatcher/EventSubscriberInterface.php',
    'Symfony\\Component\\EventDispatcher\\GenericEvent' => $vendorDir . '/symfony/event-dispatcher/GenericEvent.php',
    'Symfony\\Component\\EventDispatcher\\ImmutableEventDispatcher' => $vendorDir . '/symfony/event-dispatcher/ImmutableEventDispatcher.php',
    'Symfony\\Component\\Filesystem\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/filesystem/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Filesystem\\Exception\\FileNotFoundException' => $vendorDir . '/symfony/filesystem/Exception/FileNotFoundException.php',
    'Symfony\\Component\\Filesystem\\Exception\\IOException' => $vendorDir . '/symfony/filesystem/Exception/IOException.php',
    'Symfony\\Component\\Filesystem\\Exception\\IOExceptionInterface' => $vendorDir . '/symfony/filesystem/Exception/IOExceptionInterface.php',
    'Symfony\\Component\\Filesystem\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/filesystem/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Filesystem\\Exception\\RuntimeException' => $vendorDir . '/symfony/filesystem/Exception/RuntimeException.php',
    'Symfony\\Component\\Filesystem\\Filesystem' => $vendorDir . '/symfony/filesystem/Filesystem.php',
    'Symfony\\Component\\Filesystem\\Path' => $vendorDir . '/symfony/filesystem/Path.php',
    'Symfony\\Component\\Finder\\Comparator\\Comparator' => $vendorDir . '/symfony/finder/Comparator/Comparator.php',
    'Symfony\\Component\\Finder\\Comparator\\DateComparator' => $vendorDir . '/symfony/finder/Comparator/DateComparator.php',
    'Symfony\\Component\\Finder\\Comparator\\NumberComparator' => $vendorDir . '/symfony/finder/Comparator/NumberComparator.php',
    'Symfony\\Component\\Finder\\Exception\\AccessDeniedException' => $vendorDir . '/symfony/finder/Exception/AccessDeniedException.php',
    'Symfony\\Component\\Finder\\Exception\\DirectoryNotFoundException' => $vendorDir . '/symfony/finder/Exception/DirectoryNotFoundException.php',
    'Symfony\\Component\\Finder\\Finder' => $vendorDir . '/symfony/finder/Finder.php',
    'Symfony\\Component\\Finder\\Gitignore' => $vendorDir . '/symfony/finder/Gitignore.php',
    'Symfony\\Component\\Finder\\Glob' => $vendorDir . '/symfony/finder/Glob.php',
    'Symfony\\Component\\Finder\\Iterator\\CustomFilterIterator' => $vendorDir . '/symfony/finder/Iterator/CustomFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\DateRangeFilterIterator' => $vendorDir . '/symfony/finder/Iterator/DateRangeFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\DepthRangeFilterIterator' => $vendorDir . '/symfony/finder/Iterator/DepthRangeFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\ExcludeDirectoryFilterIterator' => $vendorDir . '/symfony/finder/Iterator/ExcludeDirectoryFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\FileTypeFilterIterator' => $vendorDir . '/symfony/finder/Iterator/FileTypeFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\FilecontentFilterIterator' => $vendorDir . '/symfony/finder/Iterator/FilecontentFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\FilenameFilterIterator' => $vendorDir . '/symfony/finder/Iterator/FilenameFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\LazyIterator' => $vendorDir . '/symfony/finder/Iterator/LazyIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\MultiplePcreFilterIterator' => $vendorDir . '/symfony/finder/Iterator/MultiplePcreFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\PathFilterIterator' => $vendorDir . '/symfony/finder/Iterator/PathFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\RecursiveDirectoryIterator' => $vendorDir . '/symfony/finder/Iterator/RecursiveDirectoryIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\SizeRangeFilterIterator' => $vendorDir . '/symfony/finder/Iterator/SizeRangeFilterIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\SortableIterator' => $vendorDir . '/symfony/finder/Iterator/SortableIterator.php',
    'Symfony\\Component\\Finder\\Iterator\\VcsIgnoredFilterIterator' => $vendorDir . '/symfony/finder/Iterator/VcsIgnoredFilterIterator.php',
    'Symfony\\Component\\Finder\\SplFileInfo' => $vendorDir . '/symfony/finder/SplFileInfo.php',
    'Symfony\\Component\\HtmlSanitizer\\HtmlSanitizer' => $vendorDir . '/symfony/html-sanitizer/HtmlSanitizer.php',
    'Symfony\\Component\\HtmlSanitizer\\HtmlSanitizerConfig' => $vendorDir . '/symfony/html-sanitizer/HtmlSanitizerConfig.php',
    'Symfony\\Component\\HtmlSanitizer\\HtmlSanitizerInterface' => $vendorDir . '/symfony/html-sanitizer/HtmlSanitizerInterface.php',
    'Symfony\\Component\\HtmlSanitizer\\Parser\\MastermindsParser' => $vendorDir . '/symfony/html-sanitizer/Parser/MastermindsParser.php',
    'Symfony\\Component\\HtmlSanitizer\\Parser\\ParserInterface' => $vendorDir . '/symfony/html-sanitizer/Parser/ParserInterface.php',
    'Symfony\\Component\\HtmlSanitizer\\Reference\\W3CReference' => $vendorDir . '/symfony/html-sanitizer/Reference/W3CReference.php',
    'Symfony\\Component\\HtmlSanitizer\\TextSanitizer\\StringSanitizer' => $vendorDir . '/symfony/html-sanitizer/TextSanitizer/StringSanitizer.php',
    'Symfony\\Component\\HtmlSanitizer\\TextSanitizer\\UrlSanitizer' => $vendorDir . '/symfony/html-sanitizer/TextSanitizer/UrlSanitizer.php',
    'Symfony\\Component\\HtmlSanitizer\\Visitor\\AttributeSanitizer\\AttributeSanitizerInterface' => $vendorDir . '/symfony/html-sanitizer/Visitor/AttributeSanitizer/AttributeSanitizerInterface.php',
    'Symfony\\Component\\HtmlSanitizer\\Visitor\\AttributeSanitizer\\UrlAttributeSanitizer' => $vendorDir . '/symfony/html-sanitizer/Visitor/AttributeSanitizer/UrlAttributeSanitizer.php',
    'Symfony\\Component\\HtmlSanitizer\\Visitor\\DomVisitor' => $vendorDir . '/symfony/html-sanitizer/Visitor/DomVisitor.php',
    'Symfony\\Component\\HtmlSanitizer\\Visitor\\Model\\Cursor' => $vendorDir . '/symfony/html-sanitizer/Visitor/Model/Cursor.php',
    'Symfony\\Component\\HtmlSanitizer\\Visitor\\Node\\BlockedNode' => $vendorDir . '/symfony/html-sanitizer/Visitor/Node/BlockedNode.php',
    'Symfony\\Component\\HtmlSanitizer\\Visitor\\Node\\DocumentNode' => $vendorDir . '/symfony/html-sanitizer/Visitor/Node/DocumentNode.php',
    'Symfony\\Component\\HtmlSanitizer\\Visitor\\Node\\Node' => $vendorDir . '/symfony/html-sanitizer/Visitor/Node/Node.php',
    'Symfony\\Component\\HtmlSanitizer\\Visitor\\Node\\NodeInterface' => $vendorDir . '/symfony/html-sanitizer/Visitor/Node/NodeInterface.php',
    'Symfony\\Component\\HtmlSanitizer\\Visitor\\Node\\TextNode' => $vendorDir . '/symfony/html-sanitizer/Visitor/Node/TextNode.php',
    'Symfony\\Component\\HttpFoundation\\AcceptHeader' => $vendorDir . '/symfony/http-foundation/AcceptHeader.php',
    'Symfony\\Component\\HttpFoundation\\AcceptHeaderItem' => $vendorDir . '/symfony/http-foundation/AcceptHeaderItem.php',
    'Symfony\\Component\\HttpFoundation\\BinaryFileResponse' => $vendorDir . '/symfony/http-foundation/BinaryFileResponse.php',
    'Symfony\\Component\\HttpFoundation\\ChainRequestMatcher' => $vendorDir . '/symfony/http-foundation/ChainRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\Cookie' => $vendorDir . '/symfony/http-foundation/Cookie.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\BadRequestException' => $vendorDir . '/symfony/http-foundation/Exception/BadRequestException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\ConflictingHeadersException' => $vendorDir . '/symfony/http-foundation/Exception/ConflictingHeadersException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\JsonException' => $vendorDir . '/symfony/http-foundation/Exception/JsonException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\RequestExceptionInterface' => $vendorDir . '/symfony/http-foundation/Exception/RequestExceptionInterface.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\SessionNotFoundException' => $vendorDir . '/symfony/http-foundation/Exception/SessionNotFoundException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\SuspiciousOperationException' => $vendorDir . '/symfony/http-foundation/Exception/SuspiciousOperationException.php',
    'Symfony\\Component\\HttpFoundation\\Exception\\UnexpectedValueException' => $vendorDir . '/symfony/http-foundation/Exception/UnexpectedValueException.php',
    'Symfony\\Component\\HttpFoundation\\ExpressionRequestMatcher' => $vendorDir . '/symfony/http-foundation/ExpressionRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\FileBag' => $vendorDir . '/symfony/http-foundation/FileBag.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\AccessDeniedException' => $vendorDir . '/symfony/http-foundation/File/Exception/AccessDeniedException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\CannotWriteFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/CannotWriteFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\ExtensionFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/ExtensionFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\FileException' => $vendorDir . '/symfony/http-foundation/File/Exception/FileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\FileNotFoundException' => $vendorDir . '/symfony/http-foundation/File/Exception/FileNotFoundException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\FormSizeFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/FormSizeFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\IniSizeFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/IniSizeFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\NoFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/NoFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\NoTmpDirFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/NoTmpDirFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\PartialFileException' => $vendorDir . '/symfony/http-foundation/File/Exception/PartialFileException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\UnexpectedTypeException' => $vendorDir . '/symfony/http-foundation/File/Exception/UnexpectedTypeException.php',
    'Symfony\\Component\\HttpFoundation\\File\\Exception\\UploadException' => $vendorDir . '/symfony/http-foundation/File/Exception/UploadException.php',
    'Symfony\\Component\\HttpFoundation\\File\\File' => $vendorDir . '/symfony/http-foundation/File/File.php',
    'Symfony\\Component\\HttpFoundation\\File\\Stream' => $vendorDir . '/symfony/http-foundation/File/Stream.php',
    'Symfony\\Component\\HttpFoundation\\File\\UploadedFile' => $vendorDir . '/symfony/http-foundation/File/UploadedFile.php',
    'Symfony\\Component\\HttpFoundation\\HeaderBag' => $vendorDir . '/symfony/http-foundation/HeaderBag.php',
    'Symfony\\Component\\HttpFoundation\\HeaderUtils' => $vendorDir . '/symfony/http-foundation/HeaderUtils.php',
    'Symfony\\Component\\HttpFoundation\\InputBag' => $vendorDir . '/symfony/http-foundation/InputBag.php',
    'Symfony\\Component\\HttpFoundation\\IpUtils' => $vendorDir . '/symfony/http-foundation/IpUtils.php',
    'Symfony\\Component\\HttpFoundation\\JsonResponse' => $vendorDir . '/symfony/http-foundation/JsonResponse.php',
    'Symfony\\Component\\HttpFoundation\\ParameterBag' => $vendorDir . '/symfony/http-foundation/ParameterBag.php',
    'Symfony\\Component\\HttpFoundation\\RateLimiter\\AbstractRequestRateLimiter' => $vendorDir . '/symfony/http-foundation/RateLimiter/AbstractRequestRateLimiter.php',
    'Symfony\\Component\\HttpFoundation\\RateLimiter\\PeekableRequestRateLimiterInterface' => $vendorDir . '/symfony/http-foundation/RateLimiter/PeekableRequestRateLimiterInterface.php',
    'Symfony\\Component\\HttpFoundation\\RateLimiter\\RequestRateLimiterInterface' => $vendorDir . '/symfony/http-foundation/RateLimiter/RequestRateLimiterInterface.php',
    'Symfony\\Component\\HttpFoundation\\RedirectResponse' => $vendorDir . '/symfony/http-foundation/RedirectResponse.php',
    'Symfony\\Component\\HttpFoundation\\Request' => $vendorDir . '/symfony/http-foundation/Request.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcherInterface' => $vendorDir . '/symfony/http-foundation/RequestMatcherInterface.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\AttributesRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/AttributesRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\ExpressionRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/ExpressionRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\HostRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/HostRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\IpsRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/IpsRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\IsJsonRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/IsJsonRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\MethodRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/MethodRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\PathRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/PathRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\PortRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/PortRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestMatcher\\SchemeRequestMatcher' => $vendorDir . '/symfony/http-foundation/RequestMatcher/SchemeRequestMatcher.php',
    'Symfony\\Component\\HttpFoundation\\RequestStack' => $vendorDir . '/symfony/http-foundation/RequestStack.php',
    'Symfony\\Component\\HttpFoundation\\Response' => $vendorDir . '/symfony/http-foundation/Response.php',
    'Symfony\\Component\\HttpFoundation\\ResponseHeaderBag' => $vendorDir . '/symfony/http-foundation/ResponseHeaderBag.php',
    'Symfony\\Component\\HttpFoundation\\ServerBag' => $vendorDir . '/symfony/http-foundation/ServerBag.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Attribute\\AttributeBag' => $vendorDir . '/symfony/http-foundation/Session/Attribute/AttributeBag.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Attribute\\AttributeBagInterface' => $vendorDir . '/symfony/http-foundation/Session/Attribute/AttributeBagInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\FlashBagAwareSessionInterface' => $vendorDir . '/symfony/http-foundation/Session/FlashBagAwareSessionInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Flash\\AutoExpireFlashBag' => $vendorDir . '/symfony/http-foundation/Session/Flash/AutoExpireFlashBag.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Flash\\FlashBag' => $vendorDir . '/symfony/http-foundation/Session/Flash/FlashBag.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Flash\\FlashBagInterface' => $vendorDir . '/symfony/http-foundation/Session/Flash/FlashBagInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Session' => $vendorDir . '/symfony/http-foundation/Session/Session.php',
    'Symfony\\Component\\HttpFoundation\\Session\\SessionBagInterface' => $vendorDir . '/symfony/http-foundation/Session/SessionBagInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\SessionBagProxy' => $vendorDir . '/symfony/http-foundation/Session/SessionBagProxy.php',
    'Symfony\\Component\\HttpFoundation\\Session\\SessionFactory' => $vendorDir . '/symfony/http-foundation/Session/SessionFactory.php',
    'Symfony\\Component\\HttpFoundation\\Session\\SessionFactoryInterface' => $vendorDir . '/symfony/http-foundation/Session/SessionFactoryInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\SessionInterface' => $vendorDir . '/symfony/http-foundation/Session/SessionInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\SessionUtils' => $vendorDir . '/symfony/http-foundation/Session/SessionUtils.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\AbstractSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/AbstractSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\IdentityMarshaller' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/IdentityMarshaller.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\MarshallingSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/MarshallingSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\MemcachedSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/MemcachedSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\MigratingSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/MigratingSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\MongoDbSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/MongoDbSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\NativeFileSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/NativeFileSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\NullSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/NullSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\PdoSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/PdoSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\RedisSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/RedisSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\SessionHandlerFactory' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/SessionHandlerFactory.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Handler\\StrictSessionHandler' => $vendorDir . '/symfony/http-foundation/Session/Storage/Handler/StrictSessionHandler.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\MetadataBag' => $vendorDir . '/symfony/http-foundation/Session/Storage/MetadataBag.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\MockArraySessionStorage' => $vendorDir . '/symfony/http-foundation/Session/Storage/MockArraySessionStorage.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\MockFileSessionStorage' => $vendorDir . '/symfony/http-foundation/Session/Storage/MockFileSessionStorage.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\MockFileSessionStorageFactory' => $vendorDir . '/symfony/http-foundation/Session/Storage/MockFileSessionStorageFactory.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\NativeSessionStorage' => $vendorDir . '/symfony/http-foundation/Session/Storage/NativeSessionStorage.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\NativeSessionStorageFactory' => $vendorDir . '/symfony/http-foundation/Session/Storage/NativeSessionStorageFactory.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\PhpBridgeSessionStorage' => $vendorDir . '/symfony/http-foundation/Session/Storage/PhpBridgeSessionStorage.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\PhpBridgeSessionStorageFactory' => $vendorDir . '/symfony/http-foundation/Session/Storage/PhpBridgeSessionStorageFactory.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Proxy\\AbstractProxy' => $vendorDir . '/symfony/http-foundation/Session/Storage/Proxy/AbstractProxy.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\Proxy\\SessionHandlerProxy' => $vendorDir . '/symfony/http-foundation/Session/Storage/Proxy/SessionHandlerProxy.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\SessionStorageFactoryInterface' => $vendorDir . '/symfony/http-foundation/Session/Storage/SessionStorageFactoryInterface.php',
    'Symfony\\Component\\HttpFoundation\\Session\\Storage\\SessionStorageInterface' => $vendorDir . '/symfony/http-foundation/Session/Storage/SessionStorageInterface.php',
    'Symfony\\Component\\HttpFoundation\\StreamedJsonResponse' => $vendorDir . '/symfony/http-foundation/StreamedJsonResponse.php',
    'Symfony\\Component\\HttpFoundation\\StreamedResponse' => $vendorDir . '/symfony/http-foundation/StreamedResponse.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\RequestAttributeValueSame' => $vendorDir . '/symfony/http-foundation/Test/Constraint/RequestAttributeValueSame.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseCookieValueSame' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseCookieValueSame.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseFormatSame' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseFormatSame.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseHasCookie' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseHasCookie.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseHasHeader' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseHasHeader.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseHeaderLocationSame' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseHeaderLocationSame.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseHeaderSame' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseHeaderSame.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseIsRedirected' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseIsRedirected.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseIsSuccessful' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseIsSuccessful.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseIsUnprocessable' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseIsUnprocessable.php',
    'Symfony\\Component\\HttpFoundation\\Test\\Constraint\\ResponseStatusCodeSame' => $vendorDir . '/symfony/http-foundation/Test/Constraint/ResponseStatusCodeSame.php',
    'Symfony\\Component\\HttpFoundation\\UriSigner' => $vendorDir . '/symfony/http-foundation/UriSigner.php',
    'Symfony\\Component\\HttpFoundation\\UrlHelper' => $vendorDir . '/symfony/http-foundation/UrlHelper.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\AsController' => $vendorDir . '/symfony/http-kernel/Attribute/AsController.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\AsTargetedValueResolver' => $vendorDir . '/symfony/http-kernel/Attribute/AsTargetedValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\Cache' => $vendorDir . '/symfony/http-kernel/Attribute/Cache.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\MapDateTime' => $vendorDir . '/symfony/http-kernel/Attribute/MapDateTime.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\MapQueryParameter' => $vendorDir . '/symfony/http-kernel/Attribute/MapQueryParameter.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\MapQueryString' => $vendorDir . '/symfony/http-kernel/Attribute/MapQueryString.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\MapRequestPayload' => $vendorDir . '/symfony/http-kernel/Attribute/MapRequestPayload.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\ValueResolver' => $vendorDir . '/symfony/http-kernel/Attribute/ValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\WithHttpStatus' => $vendorDir . '/symfony/http-kernel/Attribute/WithHttpStatus.php',
    'Symfony\\Component\\HttpKernel\\Attribute\\WithLogLevel' => $vendorDir . '/symfony/http-kernel/Attribute/WithLogLevel.php',
    'Symfony\\Component\\HttpKernel\\Bundle\\AbstractBundle' => $vendorDir . '/symfony/http-kernel/Bundle/AbstractBundle.php',
    'Symfony\\Component\\HttpKernel\\Bundle\\Bundle' => $vendorDir . '/symfony/http-kernel/Bundle/Bundle.php',
    'Symfony\\Component\\HttpKernel\\Bundle\\BundleExtension' => $vendorDir . '/symfony/http-kernel/Bundle/BundleExtension.php',
    'Symfony\\Component\\HttpKernel\\Bundle\\BundleInterface' => $vendorDir . '/symfony/http-kernel/Bundle/BundleInterface.php',
    'Symfony\\Component\\HttpKernel\\CacheClearer\\CacheClearerInterface' => $vendorDir . '/symfony/http-kernel/CacheClearer/CacheClearerInterface.php',
    'Symfony\\Component\\HttpKernel\\CacheClearer\\ChainCacheClearer' => $vendorDir . '/symfony/http-kernel/CacheClearer/ChainCacheClearer.php',
    'Symfony\\Component\\HttpKernel\\CacheClearer\\Psr6CacheClearer' => $vendorDir . '/symfony/http-kernel/CacheClearer/Psr6CacheClearer.php',
    'Symfony\\Component\\HttpKernel\\CacheWarmer\\CacheWarmer' => $vendorDir . '/symfony/http-kernel/CacheWarmer/CacheWarmer.php',
    'Symfony\\Component\\HttpKernel\\CacheWarmer\\CacheWarmerAggregate' => $vendorDir . '/symfony/http-kernel/CacheWarmer/CacheWarmerAggregate.php',
    'Symfony\\Component\\HttpKernel\\CacheWarmer\\CacheWarmerInterface' => $vendorDir . '/symfony/http-kernel/CacheWarmer/CacheWarmerInterface.php',
    'Symfony\\Component\\HttpKernel\\CacheWarmer\\WarmableInterface' => $vendorDir . '/symfony/http-kernel/CacheWarmer/WarmableInterface.php',
    'Symfony\\Component\\HttpKernel\\Config\\FileLocator' => $vendorDir . '/symfony/http-kernel/Config/FileLocator.php',
    'Symfony\\Component\\HttpKernel\\ControllerMetadata\\ArgumentMetadata' => $vendorDir . '/symfony/http-kernel/ControllerMetadata/ArgumentMetadata.php',
    'Symfony\\Component\\HttpKernel\\ControllerMetadata\\ArgumentMetadataFactory' => $vendorDir . '/symfony/http-kernel/ControllerMetadata/ArgumentMetadataFactory.php',
    'Symfony\\Component\\HttpKernel\\ControllerMetadata\\ArgumentMetadataFactoryInterface' => $vendorDir . '/symfony/http-kernel/ControllerMetadata/ArgumentMetadataFactoryInterface.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolverInterface' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolverInterface.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\BackedEnumValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/BackedEnumValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\DateTimeValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/DateTimeValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\DefaultValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/DefaultValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\NotTaggedControllerValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/NotTaggedControllerValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\QueryParameterValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/QueryParameterValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\RequestAttributeValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/RequestAttributeValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\RequestPayloadValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/RequestPayloadValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\RequestValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/RequestValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\ServiceValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/ServiceValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\SessionValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/SessionValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\TraceableValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/TraceableValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\UidValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/UidValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentResolver\\VariadicValueResolver' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentResolver/VariadicValueResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ArgumentValueResolverInterface' => $vendorDir . '/symfony/http-kernel/Controller/ArgumentValueResolverInterface.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ContainerControllerResolver' => $vendorDir . '/symfony/http-kernel/Controller/ContainerControllerResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ControllerReference' => $vendorDir . '/symfony/http-kernel/Controller/ControllerReference.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ControllerResolver' => $vendorDir . '/symfony/http-kernel/Controller/ControllerResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ControllerResolverInterface' => $vendorDir . '/symfony/http-kernel/Controller/ControllerResolverInterface.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ErrorController' => $vendorDir . '/symfony/http-kernel/Controller/ErrorController.php',
    'Symfony\\Component\\HttpKernel\\Controller\\TraceableArgumentResolver' => $vendorDir . '/symfony/http-kernel/Controller/TraceableArgumentResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\TraceableControllerResolver' => $vendorDir . '/symfony/http-kernel/Controller/TraceableControllerResolver.php',
    'Symfony\\Component\\HttpKernel\\Controller\\ValueResolverInterface' => $vendorDir . '/symfony/http-kernel/Controller/ValueResolverInterface.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\AjaxDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/AjaxDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\ConfigDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/ConfigDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\DataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/DataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\DataCollectorInterface' => $vendorDir . '/symfony/http-kernel/DataCollector/DataCollectorInterface.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\DumpDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/DumpDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\EventDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/EventDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\ExceptionDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/ExceptionDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\LateDataCollectorInterface' => $vendorDir . '/symfony/http-kernel/DataCollector/LateDataCollectorInterface.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\LoggerDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/LoggerDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\MemoryDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/MemoryDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\RequestDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/RequestDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\RouterDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/RouterDataCollector.php',
    'Symfony\\Component\\HttpKernel\\DataCollector\\TimeDataCollector' => $vendorDir . '/symfony/http-kernel/DataCollector/TimeDataCollector.php',
    'Symfony\\Component\\HttpKernel\\Debug\\ErrorHandlerConfigurator' => $vendorDir . '/symfony/http-kernel/Debug/ErrorHandlerConfigurator.php',
    'Symfony\\Component\\HttpKernel\\Debug\\FileLinkFormatter' => $vendorDir . '/symfony/http-kernel/Debug/FileLinkFormatter.php',
    'Symfony\\Component\\HttpKernel\\Debug\\TraceableEventDispatcher' => $vendorDir . '/symfony/http-kernel/Debug/TraceableEventDispatcher.php',
    'Symfony\\Component\\HttpKernel\\Debug\\VirtualRequestStack' => $vendorDir . '/symfony/http-kernel/Debug/VirtualRequestStack.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\AddAnnotatedClassesToCachePass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/AddAnnotatedClassesToCachePass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\ConfigurableExtension' => $vendorDir . '/symfony/http-kernel/DependencyInjection/ConfigurableExtension.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\ControllerArgumentValueResolverPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/ControllerArgumentValueResolverPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\Extension' => $vendorDir . '/symfony/http-kernel/DependencyInjection/Extension.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\FragmentRendererPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/FragmentRendererPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\LazyLoadingFragmentHandler' => $vendorDir . '/symfony/http-kernel/DependencyInjection/LazyLoadingFragmentHandler.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\LoggerPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/LoggerPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\MergeExtensionConfigurationPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/MergeExtensionConfigurationPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\RegisterControllerArgumentLocatorsPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/RegisterControllerArgumentLocatorsPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\RegisterLocaleAwareServicesPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/RegisterLocaleAwareServicesPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\RemoveEmptyControllerArgumentLocatorsPass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/RemoveEmptyControllerArgumentLocatorsPass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\ResettableServicePass' => $vendorDir . '/symfony/http-kernel/DependencyInjection/ResettableServicePass.php',
    'Symfony\\Component\\HttpKernel\\DependencyInjection\\ServicesResetter' => $vendorDir . '/symfony/http-kernel/DependencyInjection/ServicesResetter.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\AbstractSessionListener' => $vendorDir . '/symfony/http-kernel/EventListener/AbstractSessionListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\AddRequestFormatsListener' => $vendorDir . '/symfony/http-kernel/EventListener/AddRequestFormatsListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\CacheAttributeListener' => $vendorDir . '/symfony/http-kernel/EventListener/CacheAttributeListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\DebugHandlersListener' => $vendorDir . '/symfony/http-kernel/EventListener/DebugHandlersListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\DisallowRobotsIndexingListener' => $vendorDir . '/symfony/http-kernel/EventListener/DisallowRobotsIndexingListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\DumpListener' => $vendorDir . '/symfony/http-kernel/EventListener/DumpListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\ErrorListener' => $vendorDir . '/symfony/http-kernel/EventListener/ErrorListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\FragmentListener' => $vendorDir . '/symfony/http-kernel/EventListener/FragmentListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\LocaleAwareListener' => $vendorDir . '/symfony/http-kernel/EventListener/LocaleAwareListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\LocaleListener' => $vendorDir . '/symfony/http-kernel/EventListener/LocaleListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\ProfilerListener' => $vendorDir . '/symfony/http-kernel/EventListener/ProfilerListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\ResponseListener' => $vendorDir . '/symfony/http-kernel/EventListener/ResponseListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\RouterListener' => $vendorDir . '/symfony/http-kernel/EventListener/RouterListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\SessionListener' => $vendorDir . '/symfony/http-kernel/EventListener/SessionListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\StreamedResponseListener' => $vendorDir . '/symfony/http-kernel/EventListener/StreamedResponseListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\SurrogateListener' => $vendorDir . '/symfony/http-kernel/EventListener/SurrogateListener.php',
    'Symfony\\Component\\HttpKernel\\EventListener\\ValidateRequestListener' => $vendorDir . '/symfony/http-kernel/EventListener/ValidateRequestListener.php',
    'Symfony\\Component\\HttpKernel\\Event\\ControllerArgumentsEvent' => $vendorDir . '/symfony/http-kernel/Event/ControllerArgumentsEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\ControllerEvent' => $vendorDir . '/symfony/http-kernel/Event/ControllerEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\ExceptionEvent' => $vendorDir . '/symfony/http-kernel/Event/ExceptionEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\FinishRequestEvent' => $vendorDir . '/symfony/http-kernel/Event/FinishRequestEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\KernelEvent' => $vendorDir . '/symfony/http-kernel/Event/KernelEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\RequestEvent' => $vendorDir . '/symfony/http-kernel/Event/RequestEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\ResponseEvent' => $vendorDir . '/symfony/http-kernel/Event/ResponseEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\TerminateEvent' => $vendorDir . '/symfony/http-kernel/Event/TerminateEvent.php',
    'Symfony\\Component\\HttpKernel\\Event\\ViewEvent' => $vendorDir . '/symfony/http-kernel/Event/ViewEvent.php',
    'Symfony\\Component\\HttpKernel\\Exception\\AccessDeniedHttpException' => $vendorDir . '/symfony/http-kernel/Exception/AccessDeniedHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\BadRequestHttpException' => $vendorDir . '/symfony/http-kernel/Exception/BadRequestHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\ConflictHttpException' => $vendorDir . '/symfony/http-kernel/Exception/ConflictHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\ControllerDoesNotReturnResponseException' => $vendorDir . '/symfony/http-kernel/Exception/ControllerDoesNotReturnResponseException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\GoneHttpException' => $vendorDir . '/symfony/http-kernel/Exception/GoneHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\HttpException' => $vendorDir . '/symfony/http-kernel/Exception/HttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\HttpExceptionInterface' => $vendorDir . '/symfony/http-kernel/Exception/HttpExceptionInterface.php',
    'Symfony\\Component\\HttpKernel\\Exception\\InvalidMetadataException' => $vendorDir . '/symfony/http-kernel/Exception/InvalidMetadataException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\LengthRequiredHttpException' => $vendorDir . '/symfony/http-kernel/Exception/LengthRequiredHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\LockedHttpException' => $vendorDir . '/symfony/http-kernel/Exception/LockedHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\MethodNotAllowedHttpException' => $vendorDir . '/symfony/http-kernel/Exception/MethodNotAllowedHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\NotAcceptableHttpException' => $vendorDir . '/symfony/http-kernel/Exception/NotAcceptableHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\NotFoundHttpException' => $vendorDir . '/symfony/http-kernel/Exception/NotFoundHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\PreconditionFailedHttpException' => $vendorDir . '/symfony/http-kernel/Exception/PreconditionFailedHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\PreconditionRequiredHttpException' => $vendorDir . '/symfony/http-kernel/Exception/PreconditionRequiredHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\ResolverNotFoundException' => $vendorDir . '/symfony/http-kernel/Exception/ResolverNotFoundException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\ServiceUnavailableHttpException' => $vendorDir . '/symfony/http-kernel/Exception/ServiceUnavailableHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\TooManyRequestsHttpException' => $vendorDir . '/symfony/http-kernel/Exception/TooManyRequestsHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\UnauthorizedHttpException' => $vendorDir . '/symfony/http-kernel/Exception/UnauthorizedHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\UnexpectedSessionUsageException' => $vendorDir . '/symfony/http-kernel/Exception/UnexpectedSessionUsageException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\UnprocessableEntityHttpException' => $vendorDir . '/symfony/http-kernel/Exception/UnprocessableEntityHttpException.php',
    'Symfony\\Component\\HttpKernel\\Exception\\UnsupportedMediaTypeHttpException' => $vendorDir . '/symfony/http-kernel/Exception/UnsupportedMediaTypeHttpException.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\AbstractSurrogateFragmentRenderer' => $vendorDir . '/symfony/http-kernel/Fragment/AbstractSurrogateFragmentRenderer.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\EsiFragmentRenderer' => $vendorDir . '/symfony/http-kernel/Fragment/EsiFragmentRenderer.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\FragmentHandler' => $vendorDir . '/symfony/http-kernel/Fragment/FragmentHandler.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\FragmentRendererInterface' => $vendorDir . '/symfony/http-kernel/Fragment/FragmentRendererInterface.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\FragmentUriGenerator' => $vendorDir . '/symfony/http-kernel/Fragment/FragmentUriGenerator.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\FragmentUriGeneratorInterface' => $vendorDir . '/symfony/http-kernel/Fragment/FragmentUriGeneratorInterface.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\HIncludeFragmentRenderer' => $vendorDir . '/symfony/http-kernel/Fragment/HIncludeFragmentRenderer.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\InlineFragmentRenderer' => $vendorDir . '/symfony/http-kernel/Fragment/InlineFragmentRenderer.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\RoutableFragmentRenderer' => $vendorDir . '/symfony/http-kernel/Fragment/RoutableFragmentRenderer.php',
    'Symfony\\Component\\HttpKernel\\Fragment\\SsiFragmentRenderer' => $vendorDir . '/symfony/http-kernel/Fragment/SsiFragmentRenderer.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\AbstractSurrogate' => $vendorDir . '/symfony/http-kernel/HttpCache/AbstractSurrogate.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\CacheWasLockedException' => $vendorDir . '/symfony/http-kernel/HttpCache/CacheWasLockedException.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\Esi' => $vendorDir . '/symfony/http-kernel/HttpCache/Esi.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\HttpCache' => $vendorDir . '/symfony/http-kernel/HttpCache/HttpCache.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\ResponseCacheStrategy' => $vendorDir . '/symfony/http-kernel/HttpCache/ResponseCacheStrategy.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\ResponseCacheStrategyInterface' => $vendorDir . '/symfony/http-kernel/HttpCache/ResponseCacheStrategyInterface.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\Ssi' => $vendorDir . '/symfony/http-kernel/HttpCache/Ssi.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\Store' => $vendorDir . '/symfony/http-kernel/HttpCache/Store.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\StoreInterface' => $vendorDir . '/symfony/http-kernel/HttpCache/StoreInterface.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\SubRequestHandler' => $vendorDir . '/symfony/http-kernel/HttpCache/SubRequestHandler.php',
    'Symfony\\Component\\HttpKernel\\HttpCache\\SurrogateInterface' => $vendorDir . '/symfony/http-kernel/HttpCache/SurrogateInterface.php',
    'Symfony\\Component\\HttpKernel\\HttpClientKernel' => $vendorDir . '/symfony/http-kernel/HttpClientKernel.php',
    'Symfony\\Component\\HttpKernel\\HttpKernel' => $vendorDir . '/symfony/http-kernel/HttpKernel.php',
    'Symfony\\Component\\HttpKernel\\HttpKernelBrowser' => $vendorDir . '/symfony/http-kernel/HttpKernelBrowser.php',
    'Symfony\\Component\\HttpKernel\\HttpKernelInterface' => $vendorDir . '/symfony/http-kernel/HttpKernelInterface.php',
    'Symfony\\Component\\HttpKernel\\Kernel' => $vendorDir . '/symfony/http-kernel/Kernel.php',
    'Symfony\\Component\\HttpKernel\\KernelEvents' => $vendorDir . '/symfony/http-kernel/KernelEvents.php',
    'Symfony\\Component\\HttpKernel\\KernelInterface' => $vendorDir . '/symfony/http-kernel/KernelInterface.php',
    'Symfony\\Component\\HttpKernel\\Log\\DebugLoggerConfigurator' => $vendorDir . '/symfony/http-kernel/Log/DebugLoggerConfigurator.php',
    'Symfony\\Component\\HttpKernel\\Log\\DebugLoggerInterface' => $vendorDir . '/symfony/http-kernel/Log/DebugLoggerInterface.php',
    'Symfony\\Component\\HttpKernel\\Log\\Logger' => $vendorDir . '/symfony/http-kernel/Log/Logger.php',
    'Symfony\\Component\\HttpKernel\\Profiler\\FileProfilerStorage' => $vendorDir . '/symfony/http-kernel/Profiler/FileProfilerStorage.php',
    'Symfony\\Component\\HttpKernel\\Profiler\\Profile' => $vendorDir . '/symfony/http-kernel/Profiler/Profile.php',
    'Symfony\\Component\\HttpKernel\\Profiler\\Profiler' => $vendorDir . '/symfony/http-kernel/Profiler/Profiler.php',
    'Symfony\\Component\\HttpKernel\\Profiler\\ProfilerStorageInterface' => $vendorDir . '/symfony/http-kernel/Profiler/ProfilerStorageInterface.php',
    'Symfony\\Component\\HttpKernel\\RebootableInterface' => $vendorDir . '/symfony/http-kernel/RebootableInterface.php',
    'Symfony\\Component\\HttpKernel\\TerminableInterface' => $vendorDir . '/symfony/http-kernel/TerminableInterface.php',
    'Symfony\\Component\\HttpKernel\\UriSigner' => $vendorDir . '/symfony/http-kernel/UriSigner.php',
    'Symfony\\Component\\Mailer\\Command\\MailerTestCommand' => $vendorDir . '/symfony/mailer/Command/MailerTestCommand.php',
    'Symfony\\Component\\Mailer\\DataCollector\\MessageDataCollector' => $vendorDir . '/symfony/mailer/DataCollector/MessageDataCollector.php',
    'Symfony\\Component\\Mailer\\DelayedEnvelope' => $vendorDir . '/symfony/mailer/DelayedEnvelope.php',
    'Symfony\\Component\\Mailer\\Envelope' => $vendorDir . '/symfony/mailer/Envelope.php',
    'Symfony\\Component\\Mailer\\EventListener\\EnvelopeListener' => $vendorDir . '/symfony/mailer/EventListener/EnvelopeListener.php',
    'Symfony\\Component\\Mailer\\EventListener\\MessageListener' => $vendorDir . '/symfony/mailer/EventListener/MessageListener.php',
    'Symfony\\Component\\Mailer\\EventListener\\MessageLoggerListener' => $vendorDir . '/symfony/mailer/EventListener/MessageLoggerListener.php',
    'Symfony\\Component\\Mailer\\EventListener\\MessengerTransportListener' => $vendorDir . '/symfony/mailer/EventListener/MessengerTransportListener.php',
    'Symfony\\Component\\Mailer\\Event\\FailedMessageEvent' => $vendorDir . '/symfony/mailer/Event/FailedMessageEvent.php',
    'Symfony\\Component\\Mailer\\Event\\MessageEvent' => $vendorDir . '/symfony/mailer/Event/MessageEvent.php',
    'Symfony\\Component\\Mailer\\Event\\MessageEvents' => $vendorDir . '/symfony/mailer/Event/MessageEvents.php',
    'Symfony\\Component\\Mailer\\Event\\SentMessageEvent' => $vendorDir . '/symfony/mailer/Event/SentMessageEvent.php',
    'Symfony\\Component\\Mailer\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/mailer/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Mailer\\Exception\\HttpTransportException' => $vendorDir . '/symfony/mailer/Exception/HttpTransportException.php',
    'Symfony\\Component\\Mailer\\Exception\\IncompleteDsnException' => $vendorDir . '/symfony/mailer/Exception/IncompleteDsnException.php',
    'Symfony\\Component\\Mailer\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/mailer/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Mailer\\Exception\\LogicException' => $vendorDir . '/symfony/mailer/Exception/LogicException.php',
    'Symfony\\Component\\Mailer\\Exception\\RuntimeException' => $vendorDir . '/symfony/mailer/Exception/RuntimeException.php',
    'Symfony\\Component\\Mailer\\Exception\\TransportException' => $vendorDir . '/symfony/mailer/Exception/TransportException.php',
    'Symfony\\Component\\Mailer\\Exception\\TransportExceptionInterface' => $vendorDir . '/symfony/mailer/Exception/TransportExceptionInterface.php',
    'Symfony\\Component\\Mailer\\Exception\\UnexpectedResponseException' => $vendorDir . '/symfony/mailer/Exception/UnexpectedResponseException.php',
    'Symfony\\Component\\Mailer\\Exception\\UnsupportedSchemeException' => $vendorDir . '/symfony/mailer/Exception/UnsupportedSchemeException.php',
    'Symfony\\Component\\Mailer\\Header\\MetadataHeader' => $vendorDir . '/symfony/mailer/Header/MetadataHeader.php',
    'Symfony\\Component\\Mailer\\Header\\TagHeader' => $vendorDir . '/symfony/mailer/Header/TagHeader.php',
    'Symfony\\Component\\Mailer\\Mailer' => $vendorDir . '/symfony/mailer/Mailer.php',
    'Symfony\\Component\\Mailer\\MailerInterface' => $vendorDir . '/symfony/mailer/MailerInterface.php',
    'Symfony\\Component\\Mailer\\Messenger\\MessageHandler' => $vendorDir . '/symfony/mailer/Messenger/MessageHandler.php',
    'Symfony\\Component\\Mailer\\Messenger\\SendEmailMessage' => $vendorDir . '/symfony/mailer/Messenger/SendEmailMessage.php',
    'Symfony\\Component\\Mailer\\SentMessage' => $vendorDir . '/symfony/mailer/SentMessage.php',
    'Symfony\\Component\\Mailer\\Test\\Constraint\\EmailCount' => $vendorDir . '/symfony/mailer/Test/Constraint/EmailCount.php',
    'Symfony\\Component\\Mailer\\Test\\Constraint\\EmailIsQueued' => $vendorDir . '/symfony/mailer/Test/Constraint/EmailIsQueued.php',
    'Symfony\\Component\\Mailer\\Test\\TransportFactoryTestCase' => $vendorDir . '/symfony/mailer/Test/TransportFactoryTestCase.php',
    'Symfony\\Component\\Mailer\\Transport' => $vendorDir . '/symfony/mailer/Transport.php',
    'Symfony\\Component\\Mailer\\Transport\\AbstractApiTransport' => $vendorDir . '/symfony/mailer/Transport/AbstractApiTransport.php',
    'Symfony\\Component\\Mailer\\Transport\\AbstractHttpTransport' => $vendorDir . '/symfony/mailer/Transport/AbstractHttpTransport.php',
    'Symfony\\Component\\Mailer\\Transport\\AbstractTransport' => $vendorDir . '/symfony/mailer/Transport/AbstractTransport.php',
    'Symfony\\Component\\Mailer\\Transport\\AbstractTransportFactory' => $vendorDir . '/symfony/mailer/Transport/AbstractTransportFactory.php',
    'Symfony\\Component\\Mailer\\Transport\\Dsn' => $vendorDir . '/symfony/mailer/Transport/Dsn.php',
    'Symfony\\Component\\Mailer\\Transport\\FailoverTransport' => $vendorDir . '/symfony/mailer/Transport/FailoverTransport.php',
    'Symfony\\Component\\Mailer\\Transport\\NativeTransportFactory' => $vendorDir . '/symfony/mailer/Transport/NativeTransportFactory.php',
    'Symfony\\Component\\Mailer\\Transport\\NullTransport' => $vendorDir . '/symfony/mailer/Transport/NullTransport.php',
    'Symfony\\Component\\Mailer\\Transport\\NullTransportFactory' => $vendorDir . '/symfony/mailer/Transport/NullTransportFactory.php',
    'Symfony\\Component\\Mailer\\Transport\\RoundRobinTransport' => $vendorDir . '/symfony/mailer/Transport/RoundRobinTransport.php',
    'Symfony\\Component\\Mailer\\Transport\\SendmailTransport' => $vendorDir . '/symfony/mailer/Transport/SendmailTransport.php',
    'Symfony\\Component\\Mailer\\Transport\\SendmailTransportFactory' => $vendorDir . '/symfony/mailer/Transport/SendmailTransportFactory.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\Auth\\AuthenticatorInterface' => $vendorDir . '/symfony/mailer/Transport/Smtp/Auth/AuthenticatorInterface.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\Auth\\CramMd5Authenticator' => $vendorDir . '/symfony/mailer/Transport/Smtp/Auth/CramMd5Authenticator.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\Auth\\LoginAuthenticator' => $vendorDir . '/symfony/mailer/Transport/Smtp/Auth/LoginAuthenticator.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\Auth\\PlainAuthenticator' => $vendorDir . '/symfony/mailer/Transport/Smtp/Auth/PlainAuthenticator.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\Auth\\XOAuth2Authenticator' => $vendorDir . '/symfony/mailer/Transport/Smtp/Auth/XOAuth2Authenticator.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\EsmtpTransport' => $vendorDir . '/symfony/mailer/Transport/Smtp/EsmtpTransport.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\EsmtpTransportFactory' => $vendorDir . '/symfony/mailer/Transport/Smtp/EsmtpTransportFactory.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\SmtpTransport' => $vendorDir . '/symfony/mailer/Transport/Smtp/SmtpTransport.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\Stream\\AbstractStream' => $vendorDir . '/symfony/mailer/Transport/Smtp/Stream/AbstractStream.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\Stream\\ProcessStream' => $vendorDir . '/symfony/mailer/Transport/Smtp/Stream/ProcessStream.php',
    'Symfony\\Component\\Mailer\\Transport\\Smtp\\Stream\\SocketStream' => $vendorDir . '/symfony/mailer/Transport/Smtp/Stream/SocketStream.php',
    'Symfony\\Component\\Mailer\\Transport\\TransportFactoryInterface' => $vendorDir . '/symfony/mailer/Transport/TransportFactoryInterface.php',
    'Symfony\\Component\\Mailer\\Transport\\TransportInterface' => $vendorDir . '/symfony/mailer/Transport/TransportInterface.php',
    'Symfony\\Component\\Mailer\\Transport\\Transports' => $vendorDir . '/symfony/mailer/Transport/Transports.php',
    'Symfony\\Component\\Mime\\Address' => $vendorDir . '/symfony/mime/Address.php',
    'Symfony\\Component\\Mime\\BodyRendererInterface' => $vendorDir . '/symfony/mime/BodyRendererInterface.php',
    'Symfony\\Component\\Mime\\CharacterStream' => $vendorDir . '/symfony/mime/CharacterStream.php',
    'Symfony\\Component\\Mime\\Crypto\\DkimOptions' => $vendorDir . '/symfony/mime/Crypto/DkimOptions.php',
    'Symfony\\Component\\Mime\\Crypto\\DkimSigner' => $vendorDir . '/symfony/mime/Crypto/DkimSigner.php',
    'Symfony\\Component\\Mime\\Crypto\\SMime' => $vendorDir . '/symfony/mime/Crypto/SMime.php',
    'Symfony\\Component\\Mime\\Crypto\\SMimeEncrypter' => $vendorDir . '/symfony/mime/Crypto/SMimeEncrypter.php',
    'Symfony\\Component\\Mime\\Crypto\\SMimeSigner' => $vendorDir . '/symfony/mime/Crypto/SMimeSigner.php',
    'Symfony\\Component\\Mime\\DependencyInjection\\AddMimeTypeGuesserPass' => $vendorDir . '/symfony/mime/DependencyInjection/AddMimeTypeGuesserPass.php',
    'Symfony\\Component\\Mime\\DraftEmail' => $vendorDir . '/symfony/mime/DraftEmail.php',
    'Symfony\\Component\\Mime\\Email' => $vendorDir . '/symfony/mime/Email.php',
    'Symfony\\Component\\Mime\\Encoder\\AddressEncoderInterface' => $vendorDir . '/symfony/mime/Encoder/AddressEncoderInterface.php',
    'Symfony\\Component\\Mime\\Encoder\\Base64ContentEncoder' => $vendorDir . '/symfony/mime/Encoder/Base64ContentEncoder.php',
    'Symfony\\Component\\Mime\\Encoder\\Base64Encoder' => $vendorDir . '/symfony/mime/Encoder/Base64Encoder.php',
    'Symfony\\Component\\Mime\\Encoder\\Base64MimeHeaderEncoder' => $vendorDir . '/symfony/mime/Encoder/Base64MimeHeaderEncoder.php',
    'Symfony\\Component\\Mime\\Encoder\\ContentEncoderInterface' => $vendorDir . '/symfony/mime/Encoder/ContentEncoderInterface.php',
    'Symfony\\Component\\Mime\\Encoder\\EightBitContentEncoder' => $vendorDir . '/symfony/mime/Encoder/EightBitContentEncoder.php',
    'Symfony\\Component\\Mime\\Encoder\\EncoderInterface' => $vendorDir . '/symfony/mime/Encoder/EncoderInterface.php',
    'Symfony\\Component\\Mime\\Encoder\\IdnAddressEncoder' => $vendorDir . '/symfony/mime/Encoder/IdnAddressEncoder.php',
    'Symfony\\Component\\Mime\\Encoder\\MimeHeaderEncoderInterface' => $vendorDir . '/symfony/mime/Encoder/MimeHeaderEncoderInterface.php',
    'Symfony\\Component\\Mime\\Encoder\\QpContentEncoder' => $vendorDir . '/symfony/mime/Encoder/QpContentEncoder.php',
    'Symfony\\Component\\Mime\\Encoder\\QpEncoder' => $vendorDir . '/symfony/mime/Encoder/QpEncoder.php',
    'Symfony\\Component\\Mime\\Encoder\\QpMimeHeaderEncoder' => $vendorDir . '/symfony/mime/Encoder/QpMimeHeaderEncoder.php',
    'Symfony\\Component\\Mime\\Encoder\\Rfc2231Encoder' => $vendorDir . '/symfony/mime/Encoder/Rfc2231Encoder.php',
    'Symfony\\Component\\Mime\\Exception\\AddressEncoderException' => $vendorDir . '/symfony/mime/Exception/AddressEncoderException.php',
    'Symfony\\Component\\Mime\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/mime/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Mime\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/mime/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Mime\\Exception\\LogicException' => $vendorDir . '/symfony/mime/Exception/LogicException.php',
    'Symfony\\Component\\Mime\\Exception\\RfcComplianceException' => $vendorDir . '/symfony/mime/Exception/RfcComplianceException.php',
    'Symfony\\Component\\Mime\\Exception\\RuntimeException' => $vendorDir . '/symfony/mime/Exception/RuntimeException.php',
    'Symfony\\Component\\Mime\\FileBinaryMimeTypeGuesser' => $vendorDir . '/symfony/mime/FileBinaryMimeTypeGuesser.php',
    'Symfony\\Component\\Mime\\FileinfoMimeTypeGuesser' => $vendorDir . '/symfony/mime/FileinfoMimeTypeGuesser.php',
    'Symfony\\Component\\Mime\\Header\\AbstractHeader' => $vendorDir . '/symfony/mime/Header/AbstractHeader.php',
    'Symfony\\Component\\Mime\\Header\\DateHeader' => $vendorDir . '/symfony/mime/Header/DateHeader.php',
    'Symfony\\Component\\Mime\\Header\\HeaderInterface' => $vendorDir . '/symfony/mime/Header/HeaderInterface.php',
    'Symfony\\Component\\Mime\\Header\\Headers' => $vendorDir . '/symfony/mime/Header/Headers.php',
    'Symfony\\Component\\Mime\\Header\\IdentificationHeader' => $vendorDir . '/symfony/mime/Header/IdentificationHeader.php',
    'Symfony\\Component\\Mime\\Header\\MailboxHeader' => $vendorDir . '/symfony/mime/Header/MailboxHeader.php',
    'Symfony\\Component\\Mime\\Header\\MailboxListHeader' => $vendorDir . '/symfony/mime/Header/MailboxListHeader.php',
    'Symfony\\Component\\Mime\\Header\\ParameterizedHeader' => $vendorDir . '/symfony/mime/Header/ParameterizedHeader.php',
    'Symfony\\Component\\Mime\\Header\\PathHeader' => $vendorDir . '/symfony/mime/Header/PathHeader.php',
    'Symfony\\Component\\Mime\\Header\\UnstructuredHeader' => $vendorDir . '/symfony/mime/Header/UnstructuredHeader.php',
    'Symfony\\Component\\Mime\\HtmlToTextConverter\\DefaultHtmlToTextConverter' => $vendorDir . '/symfony/mime/HtmlToTextConverter/DefaultHtmlToTextConverter.php',
    'Symfony\\Component\\Mime\\HtmlToTextConverter\\HtmlToTextConverterInterface' => $vendorDir . '/symfony/mime/HtmlToTextConverter/HtmlToTextConverterInterface.php',
    'Symfony\\Component\\Mime\\HtmlToTextConverter\\LeagueHtmlToMarkdownConverter' => $vendorDir . '/symfony/mime/HtmlToTextConverter/LeagueHtmlToMarkdownConverter.php',
    'Symfony\\Component\\Mime\\Message' => $vendorDir . '/symfony/mime/Message.php',
    'Symfony\\Component\\Mime\\MessageConverter' => $vendorDir . '/symfony/mime/MessageConverter.php',
    'Symfony\\Component\\Mime\\MimeTypeGuesserInterface' => $vendorDir . '/symfony/mime/MimeTypeGuesserInterface.php',
    'Symfony\\Component\\Mime\\MimeTypes' => $vendorDir . '/symfony/mime/MimeTypes.php',
    'Symfony\\Component\\Mime\\MimeTypesInterface' => $vendorDir . '/symfony/mime/MimeTypesInterface.php',
    'Symfony\\Component\\Mime\\Part\\AbstractMultipartPart' => $vendorDir . '/symfony/mime/Part/AbstractMultipartPart.php',
    'Symfony\\Component\\Mime\\Part\\AbstractPart' => $vendorDir . '/symfony/mime/Part/AbstractPart.php',
    'Symfony\\Component\\Mime\\Part\\DataPart' => $vendorDir . '/symfony/mime/Part/DataPart.php',
    'Symfony\\Component\\Mime\\Part\\File' => $vendorDir . '/symfony/mime/Part/File.php',
    'Symfony\\Component\\Mime\\Part\\MessagePart' => $vendorDir . '/symfony/mime/Part/MessagePart.php',
    'Symfony\\Component\\Mime\\Part\\Multipart\\AlternativePart' => $vendorDir . '/symfony/mime/Part/Multipart/AlternativePart.php',
    'Symfony\\Component\\Mime\\Part\\Multipart\\DigestPart' => $vendorDir . '/symfony/mime/Part/Multipart/DigestPart.php',
    'Symfony\\Component\\Mime\\Part\\Multipart\\FormDataPart' => $vendorDir . '/symfony/mime/Part/Multipart/FormDataPart.php',
    'Symfony\\Component\\Mime\\Part\\Multipart\\MixedPart' => $vendorDir . '/symfony/mime/Part/Multipart/MixedPart.php',
    'Symfony\\Component\\Mime\\Part\\Multipart\\RelatedPart' => $vendorDir . '/symfony/mime/Part/Multipart/RelatedPart.php',
    'Symfony\\Component\\Mime\\Part\\SMimePart' => $vendorDir . '/symfony/mime/Part/SMimePart.php',
    'Symfony\\Component\\Mime\\Part\\TextPart' => $vendorDir . '/symfony/mime/Part/TextPart.php',
    'Symfony\\Component\\Mime\\RawMessage' => $vendorDir . '/symfony/mime/RawMessage.php',
    'Symfony\\Component\\Mime\\Test\\Constraint\\EmailAddressContains' => $vendorDir . '/symfony/mime/Test/Constraint/EmailAddressContains.php',
    'Symfony\\Component\\Mime\\Test\\Constraint\\EmailAttachmentCount' => $vendorDir . '/symfony/mime/Test/Constraint/EmailAttachmentCount.php',
    'Symfony\\Component\\Mime\\Test\\Constraint\\EmailHasHeader' => $vendorDir . '/symfony/mime/Test/Constraint/EmailHasHeader.php',
    'Symfony\\Component\\Mime\\Test\\Constraint\\EmailHeaderSame' => $vendorDir . '/symfony/mime/Test/Constraint/EmailHeaderSame.php',
    'Symfony\\Component\\Mime\\Test\\Constraint\\EmailHtmlBodyContains' => $vendorDir . '/symfony/mime/Test/Constraint/EmailHtmlBodyContains.php',
    'Symfony\\Component\\Mime\\Test\\Constraint\\EmailSubjectContains' => $vendorDir . '/symfony/mime/Test/Constraint/EmailSubjectContains.php',
    'Symfony\\Component\\Mime\\Test\\Constraint\\EmailTextBodyContains' => $vendorDir . '/symfony/mime/Test/Constraint/EmailTextBodyContains.php',
    'Symfony\\Component\\PropertyAccess\\Exception\\AccessException' => $vendorDir . '/symfony/property-access/Exception/AccessException.php',
    'Symfony\\Component\\PropertyAccess\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/property-access/Exception/ExceptionInterface.php',
    'Symfony\\Component\\PropertyAccess\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/property-access/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\PropertyAccess\\Exception\\InvalidPropertyPathException' => $vendorDir . '/symfony/property-access/Exception/InvalidPropertyPathException.php',
    'Symfony\\Component\\PropertyAccess\\Exception\\NoSuchIndexException' => $vendorDir . '/symfony/property-access/Exception/NoSuchIndexException.php',
    'Symfony\\Component\\PropertyAccess\\Exception\\NoSuchPropertyException' => $vendorDir . '/symfony/property-access/Exception/NoSuchPropertyException.php',
    'Symfony\\Component\\PropertyAccess\\Exception\\OutOfBoundsException' => $vendorDir . '/symfony/property-access/Exception/OutOfBoundsException.php',
    'Symfony\\Component\\PropertyAccess\\Exception\\RuntimeException' => $vendorDir . '/symfony/property-access/Exception/RuntimeException.php',
    'Symfony\\Component\\PropertyAccess\\Exception\\UnexpectedTypeException' => $vendorDir . '/symfony/property-access/Exception/UnexpectedTypeException.php',
    'Symfony\\Component\\PropertyAccess\\Exception\\UninitializedPropertyException' => $vendorDir . '/symfony/property-access/Exception/UninitializedPropertyException.php',
    'Symfony\\Component\\PropertyAccess\\PropertyAccess' => $vendorDir . '/symfony/property-access/PropertyAccess.php',
    'Symfony\\Component\\PropertyAccess\\PropertyAccessor' => $vendorDir . '/symfony/property-access/PropertyAccessor.php',
    'Symfony\\Component\\PropertyAccess\\PropertyAccessorBuilder' => $vendorDir . '/symfony/property-access/PropertyAccessorBuilder.php',
    'Symfony\\Component\\PropertyAccess\\PropertyAccessorInterface' => $vendorDir . '/symfony/property-access/PropertyAccessorInterface.php',
    'Symfony\\Component\\PropertyAccess\\PropertyPath' => $vendorDir . '/symfony/property-access/PropertyPath.php',
    'Symfony\\Component\\PropertyAccess\\PropertyPathBuilder' => $vendorDir . '/symfony/property-access/PropertyPathBuilder.php',
    'Symfony\\Component\\PropertyAccess\\PropertyPathInterface' => $vendorDir . '/symfony/property-access/PropertyPathInterface.php',
    'Symfony\\Component\\PropertyAccess\\PropertyPathIterator' => $vendorDir . '/symfony/property-access/PropertyPathIterator.php',
    'Symfony\\Component\\PropertyAccess\\PropertyPathIteratorInterface' => $vendorDir . '/symfony/property-access/PropertyPathIteratorInterface.php',
    'Symfony\\Component\\PropertyInfo\\DependencyInjection\\PropertyInfoConstructorPass' => $vendorDir . '/symfony/property-info/DependencyInjection/PropertyInfoConstructorPass.php',
    'Symfony\\Component\\PropertyInfo\\DependencyInjection\\PropertyInfoPass' => $vendorDir . '/symfony/property-info/DependencyInjection/PropertyInfoPass.php',
    'Symfony\\Component\\PropertyInfo\\Extractor\\ConstructorArgumentTypeExtractorInterface' => $vendorDir . '/symfony/property-info/Extractor/ConstructorArgumentTypeExtractorInterface.php',
    'Symfony\\Component\\PropertyInfo\\Extractor\\ConstructorExtractor' => $vendorDir . '/symfony/property-info/Extractor/ConstructorExtractor.php',
    'Symfony\\Component\\PropertyInfo\\Extractor\\PhpDocExtractor' => $vendorDir . '/symfony/property-info/Extractor/PhpDocExtractor.php',
    'Symfony\\Component\\PropertyInfo\\Extractor\\PhpStanExtractor' => $vendorDir . '/symfony/property-info/Extractor/PhpStanExtractor.php',
    'Symfony\\Component\\PropertyInfo\\Extractor\\ReflectionExtractor' => $vendorDir . '/symfony/property-info/Extractor/ReflectionExtractor.php',
    'Symfony\\Component\\PropertyInfo\\Extractor\\SerializerExtractor' => $vendorDir . '/symfony/property-info/Extractor/SerializerExtractor.php',
    'Symfony\\Component\\PropertyInfo\\PhpStan\\NameScope' => $vendorDir . '/symfony/property-info/PhpStan/NameScope.php',
    'Symfony\\Component\\PropertyInfo\\PhpStan\\NameScopeFactory' => $vendorDir . '/symfony/property-info/PhpStan/NameScopeFactory.php',
    'Symfony\\Component\\PropertyInfo\\PropertyAccessExtractorInterface' => $vendorDir . '/symfony/property-info/PropertyAccessExtractorInterface.php',
    'Symfony\\Component\\PropertyInfo\\PropertyDescriptionExtractorInterface' => $vendorDir . '/symfony/property-info/PropertyDescriptionExtractorInterface.php',
    'Symfony\\Component\\PropertyInfo\\PropertyInfoCacheExtractor' => $vendorDir . '/symfony/property-info/PropertyInfoCacheExtractor.php',
    'Symfony\\Component\\PropertyInfo\\PropertyInfoExtractor' => $vendorDir . '/symfony/property-info/PropertyInfoExtractor.php',
    'Symfony\\Component\\PropertyInfo\\PropertyInfoExtractorInterface' => $vendorDir . '/symfony/property-info/PropertyInfoExtractorInterface.php',
    'Symfony\\Component\\PropertyInfo\\PropertyInitializableExtractorInterface' => $vendorDir . '/symfony/property-info/PropertyInitializableExtractorInterface.php',
    'Symfony\\Component\\PropertyInfo\\PropertyListExtractorInterface' => $vendorDir . '/symfony/property-info/PropertyListExtractorInterface.php',
    'Symfony\\Component\\PropertyInfo\\PropertyReadInfo' => $vendorDir . '/symfony/property-info/PropertyReadInfo.php',
    'Symfony\\Component\\PropertyInfo\\PropertyReadInfoExtractorInterface' => $vendorDir . '/symfony/property-info/PropertyReadInfoExtractorInterface.php',
    'Symfony\\Component\\PropertyInfo\\PropertyTypeExtractorInterface' => $vendorDir . '/symfony/property-info/PropertyTypeExtractorInterface.php',
    'Symfony\\Component\\PropertyInfo\\PropertyWriteInfo' => $vendorDir . '/symfony/property-info/PropertyWriteInfo.php',
    'Symfony\\Component\\PropertyInfo\\PropertyWriteInfoExtractorInterface' => $vendorDir . '/symfony/property-info/PropertyWriteInfoExtractorInterface.php',
    'Symfony\\Component\\PropertyInfo\\Type' => $vendorDir . '/symfony/property-info/Type.php',
    'Symfony\\Component\\PropertyInfo\\Util\\PhpDocTypeHelper' => $vendorDir . '/symfony/property-info/Util/PhpDocTypeHelper.php',
    'Symfony\\Component\\PropertyInfo\\Util\\PhpStanTypeHelper' => $vendorDir . '/symfony/property-info/Util/PhpStanTypeHelper.php',
    'Symfony\\Component\\Routing\\Alias' => $vendorDir . '/symfony/routing/Alias.php',
    'Symfony\\Component\\Routing\\Annotation\\Route' => $vendorDir . '/symfony/routing/Annotation/Route.php',
    'Symfony\\Component\\Routing\\Attribute\\Route' => $vendorDir . '/symfony/routing/Attribute/Route.php',
    'Symfony\\Component\\Routing\\CompiledRoute' => $vendorDir . '/symfony/routing/CompiledRoute.php',
    'Symfony\\Component\\Routing\\DependencyInjection\\AddExpressionLanguageProvidersPass' => $vendorDir . '/symfony/routing/DependencyInjection/AddExpressionLanguageProvidersPass.php',
    'Symfony\\Component\\Routing\\DependencyInjection\\RoutingResolverPass' => $vendorDir . '/symfony/routing/DependencyInjection/RoutingResolverPass.php',
    'Symfony\\Component\\Routing\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/routing/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Routing\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/routing/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Routing\\Exception\\InvalidParameterException' => $vendorDir . '/symfony/routing/Exception/InvalidParameterException.php',
    'Symfony\\Component\\Routing\\Exception\\MethodNotAllowedException' => $vendorDir . '/symfony/routing/Exception/MethodNotAllowedException.php',
    'Symfony\\Component\\Routing\\Exception\\MissingMandatoryParametersException' => $vendorDir . '/symfony/routing/Exception/MissingMandatoryParametersException.php',
    'Symfony\\Component\\Routing\\Exception\\NoConfigurationException' => $vendorDir . '/symfony/routing/Exception/NoConfigurationException.php',
    'Symfony\\Component\\Routing\\Exception\\ResourceNotFoundException' => $vendorDir . '/symfony/routing/Exception/ResourceNotFoundException.php',
    'Symfony\\Component\\Routing\\Exception\\RouteCircularReferenceException' => $vendorDir . '/symfony/routing/Exception/RouteCircularReferenceException.php',
    'Symfony\\Component\\Routing\\Exception\\RouteNotFoundException' => $vendorDir . '/symfony/routing/Exception/RouteNotFoundException.php',
    'Symfony\\Component\\Routing\\Exception\\RuntimeException' => $vendorDir . '/symfony/routing/Exception/RuntimeException.php',
    'Symfony\\Component\\Routing\\Generator\\CompiledUrlGenerator' => $vendorDir . '/symfony/routing/Generator/CompiledUrlGenerator.php',
    'Symfony\\Component\\Routing\\Generator\\ConfigurableRequirementsInterface' => $vendorDir . '/symfony/routing/Generator/ConfigurableRequirementsInterface.php',
    'Symfony\\Component\\Routing\\Generator\\Dumper\\CompiledUrlGeneratorDumper' => $vendorDir . '/symfony/routing/Generator/Dumper/CompiledUrlGeneratorDumper.php',
    'Symfony\\Component\\Routing\\Generator\\Dumper\\GeneratorDumper' => $vendorDir . '/symfony/routing/Generator/Dumper/GeneratorDumper.php',
    'Symfony\\Component\\Routing\\Generator\\Dumper\\GeneratorDumperInterface' => $vendorDir . '/symfony/routing/Generator/Dumper/GeneratorDumperInterface.php',
    'Symfony\\Component\\Routing\\Generator\\UrlGenerator' => $vendorDir . '/symfony/routing/Generator/UrlGenerator.php',
    'Symfony\\Component\\Routing\\Generator\\UrlGeneratorInterface' => $vendorDir . '/symfony/routing/Generator/UrlGeneratorInterface.php',
    'Symfony\\Component\\Routing\\Loader\\AnnotationClassLoader' => $vendorDir . '/symfony/routing/Loader/AnnotationClassLoader.php',
    'Symfony\\Component\\Routing\\Loader\\AnnotationDirectoryLoader' => $vendorDir . '/symfony/routing/Loader/AnnotationDirectoryLoader.php',
    'Symfony\\Component\\Routing\\Loader\\AnnotationFileLoader' => $vendorDir . '/symfony/routing/Loader/AnnotationFileLoader.php',
    'Symfony\\Component\\Routing\\Loader\\AttributeClassLoader' => $vendorDir . '/symfony/routing/Loader/AttributeClassLoader.php',
    'Symfony\\Component\\Routing\\Loader\\AttributeDirectoryLoader' => $vendorDir . '/symfony/routing/Loader/AttributeDirectoryLoader.php',
    'Symfony\\Component\\Routing\\Loader\\AttributeFileLoader' => $vendorDir . '/symfony/routing/Loader/AttributeFileLoader.php',
    'Symfony\\Component\\Routing\\Loader\\ClosureLoader' => $vendorDir . '/symfony/routing/Loader/ClosureLoader.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\AliasConfigurator' => $vendorDir . '/symfony/routing/Loader/Configurator/AliasConfigurator.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\CollectionConfigurator' => $vendorDir . '/symfony/routing/Loader/Configurator/CollectionConfigurator.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\ImportConfigurator' => $vendorDir . '/symfony/routing/Loader/Configurator/ImportConfigurator.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\RouteConfigurator' => $vendorDir . '/symfony/routing/Loader/Configurator/RouteConfigurator.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\RoutingConfigurator' => $vendorDir . '/symfony/routing/Loader/Configurator/RoutingConfigurator.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\Traits\\AddTrait' => $vendorDir . '/symfony/routing/Loader/Configurator/Traits/AddTrait.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\Traits\\HostTrait' => $vendorDir . '/symfony/routing/Loader/Configurator/Traits/HostTrait.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\Traits\\LocalizedRouteTrait' => $vendorDir . '/symfony/routing/Loader/Configurator/Traits/LocalizedRouteTrait.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\Traits\\PrefixTrait' => $vendorDir . '/symfony/routing/Loader/Configurator/Traits/PrefixTrait.php',
    'Symfony\\Component\\Routing\\Loader\\Configurator\\Traits\\RouteTrait' => $vendorDir . '/symfony/routing/Loader/Configurator/Traits/RouteTrait.php',
    'Symfony\\Component\\Routing\\Loader\\ContainerLoader' => $vendorDir . '/symfony/routing/Loader/ContainerLoader.php',
    'Symfony\\Component\\Routing\\Loader\\DirectoryLoader' => $vendorDir . '/symfony/routing/Loader/DirectoryLoader.php',
    'Symfony\\Component\\Routing\\Loader\\GlobFileLoader' => $vendorDir . '/symfony/routing/Loader/GlobFileLoader.php',
    'Symfony\\Component\\Routing\\Loader\\ObjectLoader' => $vendorDir . '/symfony/routing/Loader/ObjectLoader.php',
    'Symfony\\Component\\Routing\\Loader\\PhpFileLoader' => $vendorDir . '/symfony/routing/Loader/PhpFileLoader.php',
    'Symfony\\Component\\Routing\\Loader\\Psr4DirectoryLoader' => $vendorDir . '/symfony/routing/Loader/Psr4DirectoryLoader.php',
    'Symfony\\Component\\Routing\\Loader\\XmlFileLoader' => $vendorDir . '/symfony/routing/Loader/XmlFileLoader.php',
    'Symfony\\Component\\Routing\\Loader\\YamlFileLoader' => $vendorDir . '/symfony/routing/Loader/YamlFileLoader.php',
    'Symfony\\Component\\Routing\\Matcher\\CompiledUrlMatcher' => $vendorDir . '/symfony/routing/Matcher/CompiledUrlMatcher.php',
    'Symfony\\Component\\Routing\\Matcher\\Dumper\\CompiledUrlMatcherDumper' => $vendorDir . '/symfony/routing/Matcher/Dumper/CompiledUrlMatcherDumper.php',
    'Symfony\\Component\\Routing\\Matcher\\Dumper\\CompiledUrlMatcherTrait' => $vendorDir . '/symfony/routing/Matcher/Dumper/CompiledUrlMatcherTrait.php',
    'Symfony\\Component\\Routing\\Matcher\\Dumper\\MatcherDumper' => $vendorDir . '/symfony/routing/Matcher/Dumper/MatcherDumper.php',
    'Symfony\\Component\\Routing\\Matcher\\Dumper\\MatcherDumperInterface' => $vendorDir . '/symfony/routing/Matcher/Dumper/MatcherDumperInterface.php',
    'Symfony\\Component\\Routing\\Matcher\\Dumper\\StaticPrefixCollection' => $vendorDir . '/symfony/routing/Matcher/Dumper/StaticPrefixCollection.php',
    'Symfony\\Component\\Routing\\Matcher\\ExpressionLanguageProvider' => $vendorDir . '/symfony/routing/Matcher/ExpressionLanguageProvider.php',
    'Symfony\\Component\\Routing\\Matcher\\RedirectableUrlMatcher' => $vendorDir . '/symfony/routing/Matcher/RedirectableUrlMatcher.php',
    'Symfony\\Component\\Routing\\Matcher\\RedirectableUrlMatcherInterface' => $vendorDir . '/symfony/routing/Matcher/RedirectableUrlMatcherInterface.php',
    'Symfony\\Component\\Routing\\Matcher\\RequestMatcherInterface' => $vendorDir . '/symfony/routing/Matcher/RequestMatcherInterface.php',
    'Symfony\\Component\\Routing\\Matcher\\TraceableUrlMatcher' => $vendorDir . '/symfony/routing/Matcher/TraceableUrlMatcher.php',
    'Symfony\\Component\\Routing\\Matcher\\UrlMatcher' => $vendorDir . '/symfony/routing/Matcher/UrlMatcher.php',
    'Symfony\\Component\\Routing\\Matcher\\UrlMatcherInterface' => $vendorDir . '/symfony/routing/Matcher/UrlMatcherInterface.php',
    'Symfony\\Component\\Routing\\RequestContext' => $vendorDir . '/symfony/routing/RequestContext.php',
    'Symfony\\Component\\Routing\\RequestContextAwareInterface' => $vendorDir . '/symfony/routing/RequestContextAwareInterface.php',
    'Symfony\\Component\\Routing\\Requirement\\EnumRequirement' => $vendorDir . '/symfony/routing/Requirement/EnumRequirement.php',
    'Symfony\\Component\\Routing\\Requirement\\Requirement' => $vendorDir . '/symfony/routing/Requirement/Requirement.php',
    'Symfony\\Component\\Routing\\Route' => $vendorDir . '/symfony/routing/Route.php',
    'Symfony\\Component\\Routing\\RouteCollection' => $vendorDir . '/symfony/routing/RouteCollection.php',
    'Symfony\\Component\\Routing\\RouteCompiler' => $vendorDir . '/symfony/routing/RouteCompiler.php',
    'Symfony\\Component\\Routing\\RouteCompilerInterface' => $vendorDir . '/symfony/routing/RouteCompilerInterface.php',
    'Symfony\\Component\\Routing\\Router' => $vendorDir . '/symfony/routing/Router.php',
    'Symfony\\Component\\Routing\\RouterInterface' => $vendorDir . '/symfony/routing/RouterInterface.php',
    'Symfony\\Component\\Serializer\\Annotation\\Context' => $vendorDir . '/symfony/serializer/Annotation/Context.php',
    'Symfony\\Component\\Serializer\\Annotation\\DiscriminatorMap' => $vendorDir . '/symfony/serializer/Annotation/DiscriminatorMap.php',
    'Symfony\\Component\\Serializer\\Annotation\\Groups' => $vendorDir . '/symfony/serializer/Annotation/Groups.php',
    'Symfony\\Component\\Serializer\\Annotation\\Ignore' => $vendorDir . '/symfony/serializer/Annotation/Ignore.php',
    'Symfony\\Component\\Serializer\\Annotation\\MaxDepth' => $vendorDir . '/symfony/serializer/Annotation/MaxDepth.php',
    'Symfony\\Component\\Serializer\\Annotation\\SerializedName' => $vendorDir . '/symfony/serializer/Annotation/SerializedName.php',
    'Symfony\\Component\\Serializer\\Annotation\\SerializedPath' => $vendorDir . '/symfony/serializer/Annotation/SerializedPath.php',
    'Symfony\\Component\\Serializer\\Attribute\\Context' => $vendorDir . '/symfony/serializer/Attribute/Context.php',
    'Symfony\\Component\\Serializer\\Attribute\\DiscriminatorMap' => $vendorDir . '/symfony/serializer/Attribute/DiscriminatorMap.php',
    'Symfony\\Component\\Serializer\\Attribute\\Groups' => $vendorDir . '/symfony/serializer/Attribute/Groups.php',
    'Symfony\\Component\\Serializer\\Attribute\\Ignore' => $vendorDir . '/symfony/serializer/Attribute/Ignore.php',
    'Symfony\\Component\\Serializer\\Attribute\\MaxDepth' => $vendorDir . '/symfony/serializer/Attribute/MaxDepth.php',
    'Symfony\\Component\\Serializer\\Attribute\\SerializedName' => $vendorDir . '/symfony/serializer/Attribute/SerializedName.php',
    'Symfony\\Component\\Serializer\\Attribute\\SerializedPath' => $vendorDir . '/symfony/serializer/Attribute/SerializedPath.php',
    'Symfony\\Component\\Serializer\\CacheWarmer\\CompiledClassMetadataCacheWarmer' => $vendorDir . '/symfony/serializer/CacheWarmer/CompiledClassMetadataCacheWarmer.php',
    'Symfony\\Component\\Serializer\\Command\\DebugCommand' => $vendorDir . '/symfony/serializer/Command/DebugCommand.php',
    'Symfony\\Component\\Serializer\\Context\\ContextBuilderInterface' => $vendorDir . '/symfony/serializer/Context/ContextBuilderInterface.php',
    'Symfony\\Component\\Serializer\\Context\\ContextBuilderTrait' => $vendorDir . '/symfony/serializer/Context/ContextBuilderTrait.php',
    'Symfony\\Component\\Serializer\\Context\\Encoder\\CsvEncoderContextBuilder' => $vendorDir . '/symfony/serializer/Context/Encoder/CsvEncoderContextBuilder.php',
    'Symfony\\Component\\Serializer\\Context\\Encoder\\JsonEncoderContextBuilder' => $vendorDir . '/symfony/serializer/Context/Encoder/JsonEncoderContextBuilder.php',
    'Symfony\\Component\\Serializer\\Context\\Encoder\\XmlEncoderContextBuilder' => $vendorDir . '/symfony/serializer/Context/Encoder/XmlEncoderContextBuilder.php',
    'Symfony\\Component\\Serializer\\Context\\Encoder\\YamlEncoderContextBuilder' => $vendorDir . '/symfony/serializer/Context/Encoder/YamlEncoderContextBuilder.php',
    'Symfony\\Component\\Serializer\\Context\\Normalizer\\AbstractNormalizerContextBuilder' => $vendorDir . '/symfony/serializer/Context/Normalizer/AbstractNormalizerContextBuilder.php',
    'Symfony\\Component\\Serializer\\Context\\Normalizer\\AbstractObjectNormalizerContextBuilder' => $vendorDir . '/symfony/serializer/Context/Normalizer/AbstractObjectNormalizerContextBuilder.php',
    'Symfony\\Component\\Serializer\\Context\\Normalizer\\BackedEnumNormalizerContextBuilder' => $vendorDir . '/symfony/serializer/Context/Normalizer/BackedEnumNormalizerContextBuilder.php',
    'Symfony\\Component\\Serializer\\Context\\Normalizer\\ConstraintViolationListNormalizerContextBuilder' => $vendorDir . '/symfony/serializer/Context/Normalizer/ConstraintViolationListNormalizerContextBuilder.php',
    'Symfony\\Component\\Serializer\\Context\\Normalizer\\DateIntervalNormalizerContextBuilder' => $vendorDir . '/symfony/serializer/Context/Normalizer/DateIntervalNormalizerContextBuilder.php',
    'Symfony\\Component\\Serializer\\Context\\Normalizer\\DateTimeNormalizerContextBuilder' => $vendorDir . '/symfony/serializer/Context/Normalizer/DateTimeNormalizerContextBuilder.php',
    'Symfony\\Component\\Serializer\\Context\\Normalizer\\FormErrorNormalizerContextBuilder' => $vendorDir . '/symfony/serializer/Context/Normalizer/FormErrorNormalizerContextBuilder.php',
    'Symfony\\Component\\Serializer\\Context\\Normalizer\\GetSetMethodNormalizerContextBuilder' => $vendorDir . '/symfony/serializer/Context/Normalizer/GetSetMethodNormalizerContextBuilder.php',
    'Symfony\\Component\\Serializer\\Context\\Normalizer\\JsonSerializableNormalizerContextBuilder' => $vendorDir . '/symfony/serializer/Context/Normalizer/JsonSerializableNormalizerContextBuilder.php',
    'Symfony\\Component\\Serializer\\Context\\Normalizer\\ObjectNormalizerContextBuilder' => $vendorDir . '/symfony/serializer/Context/Normalizer/ObjectNormalizerContextBuilder.php',
    'Symfony\\Component\\Serializer\\Context\\Normalizer\\ProblemNormalizerContextBuilder' => $vendorDir . '/symfony/serializer/Context/Normalizer/ProblemNormalizerContextBuilder.php',
    'Symfony\\Component\\Serializer\\Context\\Normalizer\\PropertyNormalizerContextBuilder' => $vendorDir . '/symfony/serializer/Context/Normalizer/PropertyNormalizerContextBuilder.php',
    'Symfony\\Component\\Serializer\\Context\\Normalizer\\UidNormalizerContextBuilder' => $vendorDir . '/symfony/serializer/Context/Normalizer/UidNormalizerContextBuilder.php',
    'Symfony\\Component\\Serializer\\Context\\Normalizer\\UnwrappingDenormalizerContextBuilder' => $vendorDir . '/symfony/serializer/Context/Normalizer/UnwrappingDenormalizerContextBuilder.php',
    'Symfony\\Component\\Serializer\\Context\\SerializerContextBuilder' => $vendorDir . '/symfony/serializer/Context/SerializerContextBuilder.php',
    'Symfony\\Component\\Serializer\\DataCollector\\SerializerDataCollector' => $vendorDir . '/symfony/serializer/DataCollector/SerializerDataCollector.php',
    'Symfony\\Component\\Serializer\\Debug\\TraceableEncoder' => $vendorDir . '/symfony/serializer/Debug/TraceableEncoder.php',
    'Symfony\\Component\\Serializer\\Debug\\TraceableNormalizer' => $vendorDir . '/symfony/serializer/Debug/TraceableNormalizer.php',
    'Symfony\\Component\\Serializer\\Debug\\TraceableSerializer' => $vendorDir . '/symfony/serializer/Debug/TraceableSerializer.php',
    'Symfony\\Component\\Serializer\\DependencyInjection\\SerializerPass' => $vendorDir . '/symfony/serializer/DependencyInjection/SerializerPass.php',
    'Symfony\\Component\\Serializer\\Encoder\\ChainDecoder' => $vendorDir . '/symfony/serializer/Encoder/ChainDecoder.php',
    'Symfony\\Component\\Serializer\\Encoder\\ChainEncoder' => $vendorDir . '/symfony/serializer/Encoder/ChainEncoder.php',
    'Symfony\\Component\\Serializer\\Encoder\\ContextAwareDecoderInterface' => $vendorDir . '/symfony/serializer/Encoder/ContextAwareDecoderInterface.php',
    'Symfony\\Component\\Serializer\\Encoder\\ContextAwareEncoderInterface' => $vendorDir . '/symfony/serializer/Encoder/ContextAwareEncoderInterface.php',
    'Symfony\\Component\\Serializer\\Encoder\\CsvEncoder' => $vendorDir . '/symfony/serializer/Encoder/CsvEncoder.php',
    'Symfony\\Component\\Serializer\\Encoder\\DecoderInterface' => $vendorDir . '/symfony/serializer/Encoder/DecoderInterface.php',
    'Symfony\\Component\\Serializer\\Encoder\\EncoderInterface' => $vendorDir . '/symfony/serializer/Encoder/EncoderInterface.php',
    'Symfony\\Component\\Serializer\\Encoder\\JsonDecode' => $vendorDir . '/symfony/serializer/Encoder/JsonDecode.php',
    'Symfony\\Component\\Serializer\\Encoder\\JsonEncode' => $vendorDir . '/symfony/serializer/Encoder/JsonEncode.php',
    'Symfony\\Component\\Serializer\\Encoder\\JsonEncoder' => $vendorDir . '/symfony/serializer/Encoder/JsonEncoder.php',
    'Symfony\\Component\\Serializer\\Encoder\\NormalizationAwareInterface' => $vendorDir . '/symfony/serializer/Encoder/NormalizationAwareInterface.php',
    'Symfony\\Component\\Serializer\\Encoder\\XmlEncoder' => $vendorDir . '/symfony/serializer/Encoder/XmlEncoder.php',
    'Symfony\\Component\\Serializer\\Encoder\\YamlEncoder' => $vendorDir . '/symfony/serializer/Encoder/YamlEncoder.php',
    'Symfony\\Component\\Serializer\\Exception\\BadMethodCallException' => $vendorDir . '/symfony/serializer/Exception/BadMethodCallException.php',
    'Symfony\\Component\\Serializer\\Exception\\CircularReferenceException' => $vendorDir . '/symfony/serializer/Exception/CircularReferenceException.php',
    'Symfony\\Component\\Serializer\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/serializer/Exception/ExceptionInterface.php',
    'Symfony\\Component\\Serializer\\Exception\\ExtraAttributesException' => $vendorDir . '/symfony/serializer/Exception/ExtraAttributesException.php',
    'Symfony\\Component\\Serializer\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/serializer/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\Serializer\\Exception\\LogicException' => $vendorDir . '/symfony/serializer/Exception/LogicException.php',
    'Symfony\\Component\\Serializer\\Exception\\MappingException' => $vendorDir . '/symfony/serializer/Exception/MappingException.php',
    'Symfony\\Component\\Serializer\\Exception\\MissingConstructorArgumentsException' => $vendorDir . '/symfony/serializer/Exception/MissingConstructorArgumentsException.php',
    'Symfony\\Component\\Serializer\\Exception\\NotEncodableValueException' => $vendorDir . '/symfony/serializer/Exception/NotEncodableValueException.php',
    'Symfony\\Component\\Serializer\\Exception\\NotNormalizableValueException' => $vendorDir . '/symfony/serializer/Exception/NotNormalizableValueException.php',
    'Symfony\\Component\\Serializer\\Exception\\PartialDenormalizationException' => $vendorDir . '/symfony/serializer/Exception/PartialDenormalizationException.php',
    'Symfony\\Component\\Serializer\\Exception\\RuntimeException' => $vendorDir . '/symfony/serializer/Exception/RuntimeException.php',
    'Symfony\\Component\\Serializer\\Exception\\UnexpectedValueException' => $vendorDir . '/symfony/serializer/Exception/UnexpectedValueException.php',
    'Symfony\\Component\\Serializer\\Exception\\UnsupportedException' => $vendorDir . '/symfony/serializer/Exception/UnsupportedException.php',
    'Symfony\\Component\\Serializer\\Exception\\UnsupportedFormatException' => $vendorDir . '/symfony/serializer/Exception/UnsupportedFormatException.php',
    'Symfony\\Component\\Serializer\\Extractor\\ObjectPropertyListExtractor' => $vendorDir . '/symfony/serializer/Extractor/ObjectPropertyListExtractor.php',
    'Symfony\\Component\\Serializer\\Extractor\\ObjectPropertyListExtractorInterface' => $vendorDir . '/symfony/serializer/Extractor/ObjectPropertyListExtractorInterface.php',
    'Symfony\\Component\\Serializer\\Mapping\\AttributeMetadata' => $vendorDir . '/symfony/serializer/Mapping/AttributeMetadata.php',
    'Symfony\\Component\\Serializer\\Mapping\\AttributeMetadataInterface' => $vendorDir . '/symfony/serializer/Mapping/AttributeMetadataInterface.php',
    'Symfony\\Component\\Serializer\\Mapping\\ClassDiscriminatorFromClassMetadata' => $vendorDir . '/symfony/serializer/Mapping/ClassDiscriminatorFromClassMetadata.php',
    'Symfony\\Component\\Serializer\\Mapping\\ClassDiscriminatorMapping' => $vendorDir . '/symfony/serializer/Mapping/ClassDiscriminatorMapping.php',
    'Symfony\\Component\\Serializer\\Mapping\\ClassDiscriminatorResolverInterface' => $vendorDir . '/symfony/serializer/Mapping/ClassDiscriminatorResolverInterface.php',
    'Symfony\\Component\\Serializer\\Mapping\\ClassMetadata' => $vendorDir . '/symfony/serializer/Mapping/ClassMetadata.php',
    'Symfony\\Component\\Serializer\\Mapping\\ClassMetadataInterface' => $vendorDir . '/symfony/serializer/Mapping/ClassMetadataInterface.php',
    'Symfony\\Component\\Serializer\\Mapping\\Factory\\CacheClassMetadataFactory' => $vendorDir . '/symfony/serializer/Mapping/Factory/CacheClassMetadataFactory.php',
    'Symfony\\Component\\Serializer\\Mapping\\Factory\\ClassMetadataFactory' => $vendorDir . '/symfony/serializer/Mapping/Factory/ClassMetadataFactory.php',
    'Symfony\\Component\\Serializer\\Mapping\\Factory\\ClassMetadataFactoryCompiler' => $vendorDir . '/symfony/serializer/Mapping/Factory/ClassMetadataFactoryCompiler.php',
    'Symfony\\Component\\Serializer\\Mapping\\Factory\\ClassMetadataFactoryInterface' => $vendorDir . '/symfony/serializer/Mapping/Factory/ClassMetadataFactoryInterface.php',
    'Symfony\\Component\\Serializer\\Mapping\\Factory\\ClassResolverTrait' => $vendorDir . '/symfony/serializer/Mapping/Factory/ClassResolverTrait.php',
    'Symfony\\Component\\Serializer\\Mapping\\Factory\\CompiledClassMetadataFactory' => $vendorDir . '/symfony/serializer/Mapping/Factory/CompiledClassMetadataFactory.php',
    'Symfony\\Component\\Serializer\\Mapping\\Loader\\AccessorCollisionResolverTrait' => $vendorDir . '/symfony/serializer/Mapping/Loader/AccessorCollisionResolverTrait.php',
    'Symfony\\Component\\Serializer\\Mapping\\Loader\\AnnotationLoader' => $vendorDir . '/symfony/serializer/Mapping/Loader/AnnotationLoader.php',
    'Symfony\\Component\\Serializer\\Mapping\\Loader\\AttributeLoader' => $vendorDir . '/symfony/serializer/Mapping/Loader/AttributeLoader.php',
    'Symfony\\Component\\Serializer\\Mapping\\Loader\\FileLoader' => $vendorDir . '/symfony/serializer/Mapping/Loader/FileLoader.php',
    'Symfony\\Component\\Serializer\\Mapping\\Loader\\LoaderChain' => $vendorDir . '/symfony/serializer/Mapping/Loader/LoaderChain.php',
    'Symfony\\Component\\Serializer\\Mapping\\Loader\\LoaderInterface' => $vendorDir . '/symfony/serializer/Mapping/Loader/LoaderInterface.php',
    'Symfony\\Component\\Serializer\\Mapping\\Loader\\XmlFileLoader' => $vendorDir . '/symfony/serializer/Mapping/Loader/XmlFileLoader.php',
    'Symfony\\Component\\Serializer\\Mapping\\Loader\\YamlFileLoader' => $vendorDir . '/symfony/serializer/Mapping/Loader/YamlFileLoader.php',
    'Symfony\\Component\\Serializer\\NameConverter\\AdvancedNameConverterInterface' => $vendorDir . '/symfony/serializer/NameConverter/AdvancedNameConverterInterface.php',
    'Symfony\\Component\\Serializer\\NameConverter\\CamelCaseToSnakeCaseNameConverter' => $vendorDir . '/symfony/serializer/NameConverter/CamelCaseToSnakeCaseNameConverter.php',
    'Symfony\\Component\\Serializer\\NameConverter\\MetadataAwareNameConverter' => $vendorDir . '/symfony/serializer/NameConverter/MetadataAwareNameConverter.php',
    'Symfony\\Component\\Serializer\\NameConverter\\NameConverterInterface' => $vendorDir . '/symfony/serializer/NameConverter/NameConverterInterface.php',
    'Symfony\\Component\\Serializer\\Normalizer\\AbstractNormalizer' => $vendorDir . '/symfony/serializer/Normalizer/AbstractNormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\AbstractObjectNormalizer' => $vendorDir . '/symfony/serializer/Normalizer/AbstractObjectNormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\ArrayDenormalizer' => $vendorDir . '/symfony/serializer/Normalizer/ArrayDenormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\BackedEnumNormalizer' => $vendorDir . '/symfony/serializer/Normalizer/BackedEnumNormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\CacheableSupportsMethodInterface' => $vendorDir . '/symfony/serializer/Normalizer/CacheableSupportsMethodInterface.php',
    'Symfony\\Component\\Serializer\\Normalizer\\ConstraintViolationListNormalizer' => $vendorDir . '/symfony/serializer/Normalizer/ConstraintViolationListNormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\ContextAwareDenormalizerInterface' => $vendorDir . '/symfony/serializer/Normalizer/ContextAwareDenormalizerInterface.php',
    'Symfony\\Component\\Serializer\\Normalizer\\ContextAwareNormalizerInterface' => $vendorDir . '/symfony/serializer/Normalizer/ContextAwareNormalizerInterface.php',
    'Symfony\\Component\\Serializer\\Normalizer\\CustomNormalizer' => $vendorDir . '/symfony/serializer/Normalizer/CustomNormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\DataUriNormalizer' => $vendorDir . '/symfony/serializer/Normalizer/DataUriNormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\DateIntervalNormalizer' => $vendorDir . '/symfony/serializer/Normalizer/DateIntervalNormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\DateTimeNormalizer' => $vendorDir . '/symfony/serializer/Normalizer/DateTimeNormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\DateTimeZoneNormalizer' => $vendorDir . '/symfony/serializer/Normalizer/DateTimeZoneNormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\DenormalizableInterface' => $vendorDir . '/symfony/serializer/Normalizer/DenormalizableInterface.php',
    'Symfony\\Component\\Serializer\\Normalizer\\DenormalizerAwareInterface' => $vendorDir . '/symfony/serializer/Normalizer/DenormalizerAwareInterface.php',
    'Symfony\\Component\\Serializer\\Normalizer\\DenormalizerAwareTrait' => $vendorDir . '/symfony/serializer/Normalizer/DenormalizerAwareTrait.php',
    'Symfony\\Component\\Serializer\\Normalizer\\DenormalizerInterface' => $vendorDir . '/symfony/serializer/Normalizer/DenormalizerInterface.php',
    'Symfony\\Component\\Serializer\\Normalizer\\FormErrorNormalizer' => $vendorDir . '/symfony/serializer/Normalizer/FormErrorNormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\GetSetMethodNormalizer' => $vendorDir . '/symfony/serializer/Normalizer/GetSetMethodNormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\JsonSerializableNormalizer' => $vendorDir . '/symfony/serializer/Normalizer/JsonSerializableNormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\MimeMessageNormalizer' => $vendorDir . '/symfony/serializer/Normalizer/MimeMessageNormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\NormalizableInterface' => $vendorDir . '/symfony/serializer/Normalizer/NormalizableInterface.php',
    'Symfony\\Component\\Serializer\\Normalizer\\NormalizerAwareInterface' => $vendorDir . '/symfony/serializer/Normalizer/NormalizerAwareInterface.php',
    'Symfony\\Component\\Serializer\\Normalizer\\NormalizerAwareTrait' => $vendorDir . '/symfony/serializer/Normalizer/NormalizerAwareTrait.php',
    'Symfony\\Component\\Serializer\\Normalizer\\NormalizerInterface' => $vendorDir . '/symfony/serializer/Normalizer/NormalizerInterface.php',
    'Symfony\\Component\\Serializer\\Normalizer\\ObjectNormalizer' => $vendorDir . '/symfony/serializer/Normalizer/ObjectNormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\ObjectToPopulateTrait' => $vendorDir . '/symfony/serializer/Normalizer/ObjectToPopulateTrait.php',
    'Symfony\\Component\\Serializer\\Normalizer\\ProblemNormalizer' => $vendorDir . '/symfony/serializer/Normalizer/ProblemNormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\PropertyNormalizer' => $vendorDir . '/symfony/serializer/Normalizer/PropertyNormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\TranslatableNormalizer' => $vendorDir . '/symfony/serializer/Normalizer/TranslatableNormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\UidNormalizer' => $vendorDir . '/symfony/serializer/Normalizer/UidNormalizer.php',
    'Symfony\\Component\\Serializer\\Normalizer\\UnwrappingDenormalizer' => $vendorDir . '/symfony/serializer/Normalizer/UnwrappingDenormalizer.php',
    'Symfony\\Component\\Serializer\\Serializer' => $vendorDir . '/symfony/serializer/Serializer.php',
    'Symfony\\Component\\Serializer\\SerializerAwareInterface' => $vendorDir . '/symfony/serializer/SerializerAwareInterface.php',
    'Symfony\\Component\\Serializer\\SerializerAwareTrait' => $vendorDir . '/symfony/serializer/SerializerAwareTrait.php',
    'Symfony\\Component\\Serializer\\SerializerInterface' => $vendorDir . '/symfony/serializer/SerializerInterface.php',
    'Symfony\\Component\\String\\AbstractString' => $vendorDir . '/symfony/string/AbstractString.php',
    'Symfony\\Component\\String\\AbstractUnicodeString' => $vendorDir . '/symfony/string/AbstractUnicodeString.php',
    'Symfony\\Component\\String\\ByteString' => $vendorDir . '/symfony/string/ByteString.php',
    'Symfony\\Component\\String\\CodePointString' => $vendorDir . '/symfony/string/CodePointString.php',
    'Symfony\\Component\\String\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/string/Exception/ExceptionInterface.php',
    'Symfony\\Component\\String\\Exception\\InvalidArgumentException' => $vendorDir . '/symfony/string/Exception/InvalidArgumentException.php',
    'Symfony\\Component\\String\\Exception\\RuntimeException' => $vendorDir . '/symfony/string/Exception/RuntimeException.php',
    'Symfony\\Component\\String\\Inflector\\EnglishInflector' => $vendorDir . '/symfony/string/Inflector/EnglishInflector.php',
    'Symfony\\Component\\String\\Inflector\\FrenchInflector' => $vendorDir . '/symfony/string/Inflector/FrenchInflector.php',
    'Symfony\\Component\\String\\Inflector\\InflectorInterface' => $vendorDir . '/symfony/string/Inflector/InflectorInterface.php',
    'Symfony\\Component\\String\\Inflector\\SpanishInflector' => $vendorDir . '/symfony/string/Inflector/SpanishInflector.php',
    'Symfony\\Component\\String\\LazyString' => $vendorDir . '/symfony/string/LazyString.php',
    'Symfony\\Component\\String\\Slugger\\AsciiSlugger' => $vendorDir . '/symfony/string/Slugger/AsciiSlugger.php',
    'Symfony\\Component\\String\\Slugger\\SluggerInterface' => $vendorDir . '/symfony/string/Slugger/SluggerInterface.php',
    'Symfony\\Component\\String\\TruncateMode' => $vendorDir . '/symfony/string/TruncateMode.php',
    'Symfony\\Component\\String\\UnicodeString' => $vendorDir . '/symfony/string/UnicodeString.php',
    'Symfony\\Component\\VarDumper\\Caster\\AmqpCaster' => $vendorDir . '/symfony/var-dumper/Caster/AmqpCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ArgsStub' => $vendorDir . '/symfony/var-dumper/Caster/ArgsStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\Caster' => $vendorDir . '/symfony/var-dumper/Caster/Caster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ClassStub' => $vendorDir . '/symfony/var-dumper/Caster/ClassStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\ConstStub' => $vendorDir . '/symfony/var-dumper/Caster/ConstStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\CutArrayStub' => $vendorDir . '/symfony/var-dumper/Caster/CutArrayStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\CutStub' => $vendorDir . '/symfony/var-dumper/Caster/CutStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\DOMCaster' => $vendorDir . '/symfony/var-dumper/Caster/DOMCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\DateCaster' => $vendorDir . '/symfony/var-dumper/Caster/DateCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\DoctrineCaster' => $vendorDir . '/symfony/var-dumper/Caster/DoctrineCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\DsCaster' => $vendorDir . '/symfony/var-dumper/Caster/DsCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\DsPairStub' => $vendorDir . '/symfony/var-dumper/Caster/DsPairStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\EnumStub' => $vendorDir . '/symfony/var-dumper/Caster/EnumStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\ExceptionCaster' => $vendorDir . '/symfony/var-dumper/Caster/ExceptionCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\FFICaster' => $vendorDir . '/symfony/var-dumper/Caster/FFICaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\FiberCaster' => $vendorDir . '/symfony/var-dumper/Caster/FiberCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\FrameStub' => $vendorDir . '/symfony/var-dumper/Caster/FrameStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\GmpCaster' => $vendorDir . '/symfony/var-dumper/Caster/GmpCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ImagineCaster' => $vendorDir . '/symfony/var-dumper/Caster/ImagineCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ImgStub' => $vendorDir . '/symfony/var-dumper/Caster/ImgStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\IntlCaster' => $vendorDir . '/symfony/var-dumper/Caster/IntlCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\LinkStub' => $vendorDir . '/symfony/var-dumper/Caster/LinkStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\MemcachedCaster' => $vendorDir . '/symfony/var-dumper/Caster/MemcachedCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\MysqliCaster' => $vendorDir . '/symfony/var-dumper/Caster/MysqliCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\PdoCaster' => $vendorDir . '/symfony/var-dumper/Caster/PdoCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\PgSqlCaster' => $vendorDir . '/symfony/var-dumper/Caster/PgSqlCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ProxyManagerCaster' => $vendorDir . '/symfony/var-dumper/Caster/ProxyManagerCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\RdKafkaCaster' => $vendorDir . '/symfony/var-dumper/Caster/RdKafkaCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\RedisCaster' => $vendorDir . '/symfony/var-dumper/Caster/RedisCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ReflectionCaster' => $vendorDir . '/symfony/var-dumper/Caster/ReflectionCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ResourceCaster' => $vendorDir . '/symfony/var-dumper/Caster/ResourceCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\ScalarStub' => $vendorDir . '/symfony/var-dumper/Caster/ScalarStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\SplCaster' => $vendorDir . '/symfony/var-dumper/Caster/SplCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\StubCaster' => $vendorDir . '/symfony/var-dumper/Caster/StubCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\SymfonyCaster' => $vendorDir . '/symfony/var-dumper/Caster/SymfonyCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\TraceStub' => $vendorDir . '/symfony/var-dumper/Caster/TraceStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\UninitializedStub' => $vendorDir . '/symfony/var-dumper/Caster/UninitializedStub.php',
    'Symfony\\Component\\VarDumper\\Caster\\UuidCaster' => $vendorDir . '/symfony/var-dumper/Caster/UuidCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\XmlReaderCaster' => $vendorDir . '/symfony/var-dumper/Caster/XmlReaderCaster.php',
    'Symfony\\Component\\VarDumper\\Caster\\XmlResourceCaster' => $vendorDir . '/symfony/var-dumper/Caster/XmlResourceCaster.php',
    'Symfony\\Component\\VarDumper\\Cloner\\AbstractCloner' => $vendorDir . '/symfony/var-dumper/Cloner/AbstractCloner.php',
    'Symfony\\Component\\VarDumper\\Cloner\\ClonerInterface' => $vendorDir . '/symfony/var-dumper/Cloner/ClonerInterface.php',
    'Symfony\\Component\\VarDumper\\Cloner\\Cursor' => $vendorDir . '/symfony/var-dumper/Cloner/Cursor.php',
    'Symfony\\Component\\VarDumper\\Cloner\\Data' => $vendorDir . '/symfony/var-dumper/Cloner/Data.php',
    'Symfony\\Component\\VarDumper\\Cloner\\DumperInterface' => $vendorDir . '/symfony/var-dumper/Cloner/DumperInterface.php',
    'Symfony\\Component\\VarDumper\\Cloner\\Stub' => $vendorDir . '/symfony/var-dumper/Cloner/Stub.php',
    'Symfony\\Component\\VarDumper\\Cloner\\VarCloner' => $vendorDir . '/symfony/var-dumper/Cloner/VarCloner.php',
    'Symfony\\Component\\VarDumper\\Command\\Descriptor\\CliDescriptor' => $vendorDir . '/symfony/var-dumper/Command/Descriptor/CliDescriptor.php',
    'Symfony\\Component\\VarDumper\\Command\\Descriptor\\DumpDescriptorInterface' => $vendorDir . '/symfony/var-dumper/Command/Descriptor/DumpDescriptorInterface.php',
    'Symfony\\Component\\VarDumper\\Command\\Descriptor\\HtmlDescriptor' => $vendorDir . '/symfony/var-dumper/Command/Descriptor/HtmlDescriptor.php',
    'Symfony\\Component\\VarDumper\\Command\\ServerDumpCommand' => $vendorDir . '/symfony/var-dumper/Command/ServerDumpCommand.php',
    'Symfony\\Component\\VarDumper\\Dumper\\AbstractDumper' => $vendorDir . '/symfony/var-dumper/Dumper/AbstractDumper.php',
    'Symfony\\Component\\VarDumper\\Dumper\\CliDumper' => $vendorDir . '/symfony/var-dumper/Dumper/CliDumper.php',
    'Symfony\\Component\\VarDumper\\Dumper\\ContextProvider\\CliContextProvider' => $vendorDir . '/symfony/var-dumper/Dumper/ContextProvider/CliContextProvider.php',
    'Symfony\\Component\\VarDumper\\Dumper\\ContextProvider\\ContextProviderInterface' => $vendorDir . '/symfony/var-dumper/Dumper/ContextProvider/ContextProviderInterface.php',
    'Symfony\\Component\\VarDumper\\Dumper\\ContextProvider\\RequestContextProvider' => $vendorDir . '/symfony/var-dumper/Dumper/ContextProvider/RequestContextProvider.php',
    'Symfony\\Component\\VarDumper\\Dumper\\ContextProvider\\SourceContextProvider' => $vendorDir . '/symfony/var-dumper/Dumper/ContextProvider/SourceContextProvider.php',
    'Symfony\\Component\\VarDumper\\Dumper\\ContextualizedDumper' => $vendorDir . '/symfony/var-dumper/Dumper/ContextualizedDumper.php',
    'Symfony\\Component\\VarDumper\\Dumper\\DataDumperInterface' => $vendorDir . '/symfony/var-dumper/Dumper/DataDumperInterface.php',
    'Symfony\\Component\\VarDumper\\Dumper\\HtmlDumper' => $vendorDir . '/symfony/var-dumper/Dumper/HtmlDumper.php',
    'Symfony\\Component\\VarDumper\\Dumper\\ServerDumper' => $vendorDir . '/symfony/var-dumper/Dumper/ServerDumper.php',
    'Symfony\\Component\\VarDumper\\Exception\\ThrowingCasterException' => $vendorDir . '/symfony/var-dumper/Exception/ThrowingCasterException.php',
    'Symfony\\Component\\VarDumper\\Server\\Connection' => $vendorDir . '/symfony/var-dumper/Server/Connection.php',
    'Symfony\\Component\\VarDumper\\Server\\DumpServer' => $vendorDir . '/symfony/var-dumper/Server/DumpServer.php',
    'Symfony\\Component\\VarDumper\\Test\\VarDumperTestTrait' => $vendorDir . '/symfony/var-dumper/Test/VarDumperTestTrait.php',
    'Symfony\\Component\\VarDumper\\VarDumper' => $vendorDir . '/symfony/var-dumper/VarDumper.php',
    'Symfony\\Component\\VarExporter\\Exception\\ClassNotFoundException' => $vendorDir . '/symfony/var-exporter/Exception/ClassNotFoundException.php',
    'Symfony\\Component\\VarExporter\\Exception\\ExceptionInterface' => $vendorDir . '/symfony/var-exporter/Exception/ExceptionInterface.php',
    'Symfony\\Component\\VarExporter\\Exception\\LogicException' => $vendorDir . '/symfony/var-exporter/Exception/LogicException.php',
    'Symfony\\Component\\VarExporter\\Exception\\NotInstantiableTypeException' => $vendorDir . '/symfony/var-exporter/Exception/NotInstantiableTypeException.php',
    'Symfony\\Component\\VarExporter\\Hydrator' => $vendorDir . '/symfony/var-exporter/Hydrator.php',
    'Symfony\\Component\\VarExporter\\Instantiator' => $vendorDir . '/symfony/var-exporter/Instantiator.php',
    'Symfony\\Component\\VarExporter\\Internal\\Exporter' => $vendorDir . '/symfony/var-exporter/Internal/Exporter.php',
    'Symfony\\Component\\VarExporter\\Internal\\Hydrator' => $vendorDir . '/symfony/var-exporter/Internal/Hydrator.php',
    'Symfony\\Component\\VarExporter\\Internal\\LazyDecoratorTrait' => $vendorDir . '/symfony/var-exporter/Internal/LazyDecoratorTrait.php',
    'Symfony\\Component\\VarExporter\\Internal\\LazyObjectRegistry' => $vendorDir . '/symfony/var-exporter/Internal/LazyObjectRegistry.php',
    'Symfony\\Component\\VarExporter\\Internal\\LazyObjectState' => $vendorDir . '/symfony/var-exporter/Internal/LazyObjectState.php',
    'Symfony\\Component\\VarExporter\\Internal\\LazyObjectTrait' => $vendorDir . '/symfony/var-exporter/Internal/LazyObjectTrait.php',
    'Symfony\\Component\\VarExporter\\Internal\\NamedClosure' => $vendorDir . '/symfony/var-exporter/Internal/NamedClosure.php',
    'Symfony\\Component\\VarExporter\\Internal\\Reference' => $vendorDir . '/symfony/var-exporter/Internal/Reference.php',
    'Symfony\\Component\\VarExporter\\Internal\\Registry' => $vendorDir . '/symfony/var-exporter/Internal/Registry.php',
    'Symfony\\Component\\VarExporter\\Internal\\Values' => $vendorDir . '/symfony/var-exporter/Internal/Values.php',
    'Symfony\\Component\\VarExporter\\LazyGhostTrait' => $vendorDir . '/symfony/var-exporter/LazyGhostTrait.php',
    'Symfony\\Component\\VarExporter\\LazyObjectInterface' => $vendorDir . '/symfony/var-exporter/LazyObjectInterface.php',
    'Symfony\\Component\\VarExporter\\LazyProxyTrait' => $vendorDir . '/symfony/var-exporter/LazyProxyTrait.php',
    'Symfony\\Component\\VarExporter\\ProxyHelper' => $vendorDir . '/symfony/var-exporter/ProxyHelper.php',
    'Symfony\\Component\\VarExporter\\VarExporter' => $vendorDir . '/symfony/var-exporter/VarExporter.php',
    'Symfony\\Contracts\\Cache\\CacheInterface' => $vendorDir . '/symfony/cache-contracts/CacheInterface.php',
    'Symfony\\Contracts\\Cache\\CacheTrait' => $vendorDir . '/symfony/cache-contracts/CacheTrait.php',
    'Symfony\\Contracts\\Cache\\CallbackInterface' => $vendorDir . '/symfony/cache-contracts/CallbackInterface.php',
    'Symfony\\Contracts\\Cache\\ItemInterface' => $vendorDir . '/symfony/cache-contracts/ItemInterface.php',
    'Symfony\\Contracts\\Cache\\NamespacedPoolInterface' => $vendorDir . '/symfony/cache-contracts/NamespacedPoolInterface.php',
    'Symfony\\Contracts\\Cache\\TagAwareCacheInterface' => $vendorDir . '/symfony/cache-contracts/TagAwareCacheInterface.php',
    'Symfony\\Contracts\\EventDispatcher\\Event' => $vendorDir . '/symfony/event-dispatcher-contracts/Event.php',
    'Symfony\\Contracts\\EventDispatcher\\EventDispatcherInterface' => $vendorDir . '/symfony/event-dispatcher-contracts/EventDispatcherInterface.php',
    'Symfony\\Contracts\\Service\\Attribute\\Required' => $vendorDir . '/symfony/service-contracts/Attribute/Required.php',
    'Symfony\\Contracts\\Service\\Attribute\\SubscribedService' => $vendorDir . '/symfony/service-contracts/Attribute/SubscribedService.php',
    'Symfony\\Contracts\\Service\\ResetInterface' => $vendorDir . '/symfony/service-contracts/ResetInterface.php',
    'Symfony\\Contracts\\Service\\ServiceCollectionInterface' => $vendorDir . '/symfony/service-contracts/ServiceCollectionInterface.php',
    'Symfony\\Contracts\\Service\\ServiceLocatorTrait' => $vendorDir . '/symfony/service-contracts/ServiceLocatorTrait.php',
    'Symfony\\Contracts\\Service\\ServiceMethodsSubscriberTrait' => $vendorDir . '/symfony/service-contracts/ServiceMethodsSubscriberTrait.php',
    'Symfony\\Contracts\\Service\\ServiceProviderInterface' => $vendorDir . '/symfony/service-contracts/ServiceProviderInterface.php',
    'Symfony\\Contracts\\Service\\ServiceSubscriberInterface' => $vendorDir . '/symfony/service-contracts/ServiceSubscriberInterface.php',
    'Symfony\\Contracts\\Service\\ServiceSubscriberTrait' => $vendorDir . '/symfony/service-contracts/ServiceSubscriberTrait.php',
    'Symfony\\Contracts\\Translation\\LocaleAwareInterface' => $vendorDir . '/symfony/translation-contracts/LocaleAwareInterface.php',
    'Symfony\\Contracts\\Translation\\TranslatableInterface' => $vendorDir . '/symfony/translation-contracts/TranslatableInterface.php',
    'Symfony\\Contracts\\Translation\\TranslatorInterface' => $vendorDir . '/symfony/translation-contracts/TranslatorInterface.php',
    'Symfony\\Contracts\\Translation\\TranslatorTrait' => $vendorDir . '/symfony/translation-contracts/TranslatorTrait.php',
    'Symfony\\Polyfill\\Ctype\\Ctype' => $vendorDir . '/symfony/polyfill-ctype/Ctype.php',
    'Symfony\\Polyfill\\Iconv\\Iconv' => $vendorDir . '/symfony/polyfill-iconv/Iconv.php',
    'Symfony\\Polyfill\\Mbstring\\Mbstring' => $vendorDir . '/symfony/polyfill-mbstring/Mbstring.php',
    'Symfony\\Polyfill\\Php83\\Php83' => $vendorDir . '/symfony/polyfill-php83/Php83.php',
    'Symfony\\Polyfill\\Php84\\Php84' => $vendorDir . '/symfony/polyfill-php84/Php84.php',
    'TCPDF' => $vendorDir . '/tecnickcom/tcpdf/tcpdf.php',
    'TCPDF2DBarcode' => $vendorDir . '/tecnickcom/tcpdf/tcpdf_barcodes_2d.php',
    'TCPDFBarcode' => $vendorDir . '/tecnickcom/tcpdf/tcpdf_barcodes_1d.php',
    'TCPDF_COLORS' => $vendorDir . '/tecnickcom/tcpdf/include/tcpdf_colors.php',
    'TCPDF_FILTERS' => $vendorDir . '/tecnickcom/tcpdf/include/tcpdf_filters.php',
    'TCPDF_FONTS' => $vendorDir . '/tecnickcom/tcpdf/include/tcpdf_fonts.php',
    'TCPDF_FONT_DATA' => $vendorDir . '/tecnickcom/tcpdf/include/tcpdf_font_data.php',
    'TCPDF_IMAGES' => $vendorDir . '/tecnickcom/tcpdf/include/tcpdf_images.php',
    'TCPDF_STATIC' => $vendorDir . '/tecnickcom/tcpdf/include/tcpdf_static.php',
    'TaskCategory' => $baseDir . '/src/TaskCategory.php',
    'TaskTemplate' => $baseDir . '/src/TaskTemplate.php',
    'Telemetry' => $baseDir . '/src/Telemetry.php',
    'TheNetworg\\OAuth2\\Client\\Grant\\JwtBearer' => $vendorDir . '/thenetworg/oauth2-azure/src/Grant/JwtBearer.php',
    'TheNetworg\\OAuth2\\Client\\Provider\\Azure' => $vendorDir . '/thenetworg/oauth2-azure/src/Provider/Azure.php',
    'TheNetworg\\OAuth2\\Client\\Provider\\AzureResourceOwner' => $vendorDir . '/thenetworg/oauth2-azure/src/Provider/AzureResourceOwner.php',
    'TheNetworg\\OAuth2\\Client\\Token\\AccessToken' => $vendorDir . '/thenetworg/oauth2-azure/src/Token/AccessToken.php',
    'Ticket' => $baseDir . '/src/Ticket.php',
    'TicketCost' => $baseDir . '/src/TicketCost.php',
    'TicketRecurrent' => $baseDir . '/src/TicketRecurrent.php',
    'TicketSatisfaction' => $baseDir . '/src/TicketSatisfaction.php',
    'TicketTask' => $baseDir . '/src/TicketTask.php',
    'TicketTemplate' => $baseDir . '/src/TicketTemplate.php',
    'TicketTemplateHiddenField' => $baseDir . '/src/TicketTemplateHiddenField.php',
    'TicketTemplateMandatoryField' => $baseDir . '/src/TicketTemplateMandatoryField.php',
    'TicketTemplatePredefinedField' => $baseDir . '/src/TicketTemplatePredefinedField.php',
    'TicketTemplateReadonlyField' => $baseDir . '/src/TicketTemplateReadonlyField.php',
    'TicketValidation' => $baseDir . '/src/TicketValidation.php',
    'TicketValidationStep' => $baseDir . '/src/TicketValidationStep.php',
    'Ticket_Contract' => $baseDir . '/src/Ticket_Contract.php',
    'Ticket_Ticket' => $baseDir . '/src/Ticket_Ticket.php',
    'Ticket_User' => $baseDir . '/src/Ticket_User.php',
    'Timer' => $baseDir . '/src/Timer.php',
    'Toolbox' => $baseDir . '/src/Toolbox.php',
    'Transfer' => $baseDir . '/src/Transfer.php',
    'Twig\\AbstractTwigCallable' => $vendorDir . '/twig/twig/src/AbstractTwigCallable.php',
    'Twig\\Attribute\\AsTwigFilter' => $vendorDir . '/twig/twig/src/Attribute/AsTwigFilter.php',
    'Twig\\Attribute\\AsTwigFunction' => $vendorDir . '/twig/twig/src/Attribute/AsTwigFunction.php',
    'Twig\\Attribute\\AsTwigTest' => $vendorDir . '/twig/twig/src/Attribute/AsTwigTest.php',
    'Twig\\Attribute\\FirstClassTwigCallableReady' => $vendorDir . '/twig/twig/src/Attribute/FirstClassTwigCallableReady.php',
    'Twig\\Attribute\\YieldReady' => $vendorDir . '/twig/twig/src/Attribute/YieldReady.php',
    'Twig\\Cache\\CacheInterface' => $vendorDir . '/twig/twig/src/Cache/CacheInterface.php',
    'Twig\\Cache\\ChainCache' => $vendorDir . '/twig/twig/src/Cache/ChainCache.php',
    'Twig\\Cache\\FilesystemCache' => $vendorDir . '/twig/twig/src/Cache/FilesystemCache.php',
    'Twig\\Cache\\NullCache' => $vendorDir . '/twig/twig/src/Cache/NullCache.php',
    'Twig\\Cache\\ReadOnlyFilesystemCache' => $vendorDir . '/twig/twig/src/Cache/ReadOnlyFilesystemCache.php',
    'Twig\\Cache\\RemovableCacheInterface' => $vendorDir . '/twig/twig/src/Cache/RemovableCacheInterface.php',
    'Twig\\Compiler' => $vendorDir . '/twig/twig/src/Compiler.php',
    'Twig\\DeprecatedCallableInfo' => $vendorDir . '/twig/twig/src/DeprecatedCallableInfo.php',
    'Twig\\Environment' => $vendorDir . '/twig/twig/src/Environment.php',
    'Twig\\Error\\Error' => $vendorDir . '/twig/twig/src/Error/Error.php',
    'Twig\\Error\\LoaderError' => $vendorDir . '/twig/twig/src/Error/LoaderError.php',
    'Twig\\Error\\RuntimeError' => $vendorDir . '/twig/twig/src/Error/RuntimeError.php',
    'Twig\\Error\\SyntaxError' => $vendorDir . '/twig/twig/src/Error/SyntaxError.php',
    'Twig\\ExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser.php',
    'Twig\\ExpressionParser\\AbstractExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/AbstractExpressionParser.php',
    'Twig\\ExpressionParser\\ExpressionParserDescriptionInterface' => $vendorDir . '/twig/twig/src/ExpressionParser/ExpressionParserDescriptionInterface.php',
    'Twig\\ExpressionParser\\ExpressionParserInterface' => $vendorDir . '/twig/twig/src/ExpressionParser/ExpressionParserInterface.php',
    'Twig\\ExpressionParser\\ExpressionParserType' => $vendorDir . '/twig/twig/src/ExpressionParser/ExpressionParserType.php',
    'Twig\\ExpressionParser\\ExpressionParsers' => $vendorDir . '/twig/twig/src/ExpressionParser/ExpressionParsers.php',
    'Twig\\ExpressionParser\\InfixAssociativity' => $vendorDir . '/twig/twig/src/ExpressionParser/InfixAssociativity.php',
    'Twig\\ExpressionParser\\InfixExpressionParserInterface' => $vendorDir . '/twig/twig/src/ExpressionParser/InfixExpressionParserInterface.php',
    'Twig\\ExpressionParser\\Infix\\ArgumentsTrait' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/ArgumentsTrait.php',
    'Twig\\ExpressionParser\\Infix\\ArrowExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/ArrowExpressionParser.php',
    'Twig\\ExpressionParser\\Infix\\AssignmentExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/AssignmentExpressionParser.php',
    'Twig\\ExpressionParser\\Infix\\BinaryOperatorExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/BinaryOperatorExpressionParser.php',
    'Twig\\ExpressionParser\\Infix\\ConditionalTernaryExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/ConditionalTernaryExpressionParser.php',
    'Twig\\ExpressionParser\\Infix\\DotExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/DotExpressionParser.php',
    'Twig\\ExpressionParser\\Infix\\FilterExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/FilterExpressionParser.php',
    'Twig\\ExpressionParser\\Infix\\FunctionExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/FunctionExpressionParser.php',
    'Twig\\ExpressionParser\\Infix\\IsExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/IsExpressionParser.php',
    'Twig\\ExpressionParser\\Infix\\IsNotExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/IsNotExpressionParser.php',
    'Twig\\ExpressionParser\\Infix\\SquareBracketExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Infix/SquareBracketExpressionParser.php',
    'Twig\\ExpressionParser\\PrecedenceChange' => $vendorDir . '/twig/twig/src/ExpressionParser/PrecedenceChange.php',
    'Twig\\ExpressionParser\\PrefixExpressionParserInterface' => $vendorDir . '/twig/twig/src/ExpressionParser/PrefixExpressionParserInterface.php',
    'Twig\\ExpressionParser\\Prefix\\GroupingExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Prefix/GroupingExpressionParser.php',
    'Twig\\ExpressionParser\\Prefix\\LiteralExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Prefix/LiteralExpressionParser.php',
    'Twig\\ExpressionParser\\Prefix\\UnaryOperatorExpressionParser' => $vendorDir . '/twig/twig/src/ExpressionParser/Prefix/UnaryOperatorExpressionParser.php',
    'Twig\\ExtensionSet' => $vendorDir . '/twig/twig/src/ExtensionSet.php',
    'Twig\\Extension\\AbstractExtension' => $vendorDir . '/twig/twig/src/Extension/AbstractExtension.php',
    'Twig\\Extension\\AttributeExtension' => $vendorDir . '/twig/twig/src/Extension/AttributeExtension.php',
    'Twig\\Extension\\CoreExtension' => $vendorDir . '/twig/twig/src/Extension/CoreExtension.php',
    'Twig\\Extension\\DebugExtension' => $vendorDir . '/twig/twig/src/Extension/DebugExtension.php',
    'Twig\\Extension\\EscaperExtension' => $vendorDir . '/twig/twig/src/Extension/EscaperExtension.php',
    'Twig\\Extension\\ExtensionInterface' => $vendorDir . '/twig/twig/src/Extension/ExtensionInterface.php',
    'Twig\\Extension\\GlobalsInterface' => $vendorDir . '/twig/twig/src/Extension/GlobalsInterface.php',
    'Twig\\Extension\\LastModifiedExtensionInterface' => $vendorDir . '/twig/twig/src/Extension/LastModifiedExtensionInterface.php',
    'Twig\\Extension\\OptimizerExtension' => $vendorDir . '/twig/twig/src/Extension/OptimizerExtension.php',
    'Twig\\Extension\\ProfilerExtension' => $vendorDir . '/twig/twig/src/Extension/ProfilerExtension.php',
    'Twig\\Extension\\RuntimeExtensionInterface' => $vendorDir . '/twig/twig/src/Extension/RuntimeExtensionInterface.php',
    'Twig\\Extension\\SandboxExtension' => $vendorDir . '/twig/twig/src/Extension/SandboxExtension.php',
    'Twig\\Extension\\StagingExtension' => $vendorDir . '/twig/twig/src/Extension/StagingExtension.php',
    'Twig\\Extension\\StringLoaderExtension' => $vendorDir . '/twig/twig/src/Extension/StringLoaderExtension.php',
    'Twig\\Extension\\YieldNotReadyExtension' => $vendorDir . '/twig/twig/src/Extension/YieldNotReadyExtension.php',
    'Twig\\Extra\\Markdown\\DefaultMarkdown' => $vendorDir . '/twig/markdown-extra/DefaultMarkdown.php',
    'Twig\\Extra\\Markdown\\ErusevMarkdown' => $vendorDir . '/twig/markdown-extra/ErusevMarkdown.php',
    'Twig\\Extra\\Markdown\\LeagueMarkdown' => $vendorDir . '/twig/markdown-extra/LeagueMarkdown.php',
    'Twig\\Extra\\Markdown\\MarkdownExtension' => $vendorDir . '/twig/markdown-extra/MarkdownExtension.php',
    'Twig\\Extra\\Markdown\\MarkdownInterface' => $vendorDir . '/twig/markdown-extra/MarkdownInterface.php',
    'Twig\\Extra\\Markdown\\MarkdownRuntime' => $vendorDir . '/twig/markdown-extra/MarkdownRuntime.php',
    'Twig\\Extra\\Markdown\\MichelfMarkdown' => $vendorDir . '/twig/markdown-extra/MichelfMarkdown.php',
    'Twig\\Extra\\String\\StringExtension' => $vendorDir . '/twig/string-extra/StringExtension.php',
    'Twig\\FileExtensionEscapingStrategy' => $vendorDir . '/twig/twig/src/FileExtensionEscapingStrategy.php',
    'Twig\\Lexer' => $vendorDir . '/twig/twig/src/Lexer.php',
    'Twig\\Loader\\ArrayLoader' => $vendorDir . '/twig/twig/src/Loader/ArrayLoader.php',
    'Twig\\Loader\\ChainLoader' => $vendorDir . '/twig/twig/src/Loader/ChainLoader.php',
    'Twig\\Loader\\FilesystemLoader' => $vendorDir . '/twig/twig/src/Loader/FilesystemLoader.php',
    'Twig\\Loader\\LoaderInterface' => $vendorDir . '/twig/twig/src/Loader/LoaderInterface.php',
    'Twig\\Markup' => $vendorDir . '/twig/twig/src/Markup.php',
    'Twig\\NodeTraverser' => $vendorDir . '/twig/twig/src/NodeTraverser.php',
    'Twig\\NodeVisitor\\AbstractNodeVisitor' => $vendorDir . '/twig/twig/src/NodeVisitor/AbstractNodeVisitor.php',
    'Twig\\NodeVisitor\\EscaperNodeVisitor' => $vendorDir . '/twig/twig/src/NodeVisitor/EscaperNodeVisitor.php',
    'Twig\\NodeVisitor\\NodeVisitorInterface' => $vendorDir . '/twig/twig/src/NodeVisitor/NodeVisitorInterface.php',
    'Twig\\NodeVisitor\\OptimizerNodeVisitor' => $vendorDir . '/twig/twig/src/NodeVisitor/OptimizerNodeVisitor.php',
    'Twig\\NodeVisitor\\SafeAnalysisNodeVisitor' => $vendorDir . '/twig/twig/src/NodeVisitor/SafeAnalysisNodeVisitor.php',
    'Twig\\NodeVisitor\\SandboxNodeVisitor' => $vendorDir . '/twig/twig/src/NodeVisitor/SandboxNodeVisitor.php',
    'Twig\\NodeVisitor\\YieldNotReadyNodeVisitor' => $vendorDir . '/twig/twig/src/NodeVisitor/YieldNotReadyNodeVisitor.php',
    'Twig\\Node\\AutoEscapeNode' => $vendorDir . '/twig/twig/src/Node/AutoEscapeNode.php',
    'Twig\\Node\\BlockNode' => $vendorDir . '/twig/twig/src/Node/BlockNode.php',
    'Twig\\Node\\BlockReferenceNode' => $vendorDir . '/twig/twig/src/Node/BlockReferenceNode.php',
    'Twig\\Node\\BodyNode' => $vendorDir . '/twig/twig/src/Node/BodyNode.php',
    'Twig\\Node\\CaptureNode' => $vendorDir . '/twig/twig/src/Node/CaptureNode.php',
    'Twig\\Node\\CheckSecurityCallNode' => $vendorDir . '/twig/twig/src/Node/CheckSecurityCallNode.php',
    'Twig\\Node\\CheckSecurityNode' => $vendorDir . '/twig/twig/src/Node/CheckSecurityNode.php',
    'Twig\\Node\\CheckToStringNode' => $vendorDir . '/twig/twig/src/Node/CheckToStringNode.php',
    'Twig\\Node\\DeprecatedNode' => $vendorDir . '/twig/twig/src/Node/DeprecatedNode.php',
    'Twig\\Node\\DoNode' => $vendorDir . '/twig/twig/src/Node/DoNode.php',
    'Twig\\Node\\EmbedNode' => $vendorDir . '/twig/twig/src/Node/EmbedNode.php',
    'Twig\\Node\\EmptyNode' => $vendorDir . '/twig/twig/src/Node/EmptyNode.php',
    'Twig\\Node\\Expression\\AbstractExpression' => $vendorDir . '/twig/twig/src/Node/Expression/AbstractExpression.php',
    'Twig\\Node\\Expression\\ArrayExpression' => $vendorDir . '/twig/twig/src/Node/Expression/ArrayExpression.php',
    'Twig\\Node\\Expression\\ArrowFunctionExpression' => $vendorDir . '/twig/twig/src/Node/Expression/ArrowFunctionExpression.php',
    'Twig\\Node\\Expression\\AssignNameExpression' => $vendorDir . '/twig/twig/src/Node/Expression/AssignNameExpression.php',
    'Twig\\Node\\Expression\\Binary\\AbstractBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/AbstractBinary.php',
    'Twig\\Node\\Expression\\Binary\\AddBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/AddBinary.php',
    'Twig\\Node\\Expression\\Binary\\AndBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/AndBinary.php',
    'Twig\\Node\\Expression\\Binary\\BinaryInterface' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/BinaryInterface.php',
    'Twig\\Node\\Expression\\Binary\\BitwiseAndBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/BitwiseAndBinary.php',
    'Twig\\Node\\Expression\\Binary\\BitwiseOrBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/BitwiseOrBinary.php',
    'Twig\\Node\\Expression\\Binary\\BitwiseXorBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/BitwiseXorBinary.php',
    'Twig\\Node\\Expression\\Binary\\ConcatBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/ConcatBinary.php',
    'Twig\\Node\\Expression\\Binary\\DivBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/DivBinary.php',
    'Twig\\Node\\Expression\\Binary\\ElvisBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/ElvisBinary.php',
    'Twig\\Node\\Expression\\Binary\\EndsWithBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/EndsWithBinary.php',
    'Twig\\Node\\Expression\\Binary\\EqualBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/EqualBinary.php',
    'Twig\\Node\\Expression\\Binary\\FloorDivBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/FloorDivBinary.php',
    'Twig\\Node\\Expression\\Binary\\GreaterBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/GreaterBinary.php',
    'Twig\\Node\\Expression\\Binary\\GreaterEqualBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/GreaterEqualBinary.php',
    'Twig\\Node\\Expression\\Binary\\HasEveryBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/HasEveryBinary.php',
    'Twig\\Node\\Expression\\Binary\\HasSomeBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/HasSomeBinary.php',
    'Twig\\Node\\Expression\\Binary\\InBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/InBinary.php',
    'Twig\\Node\\Expression\\Binary\\LessBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/LessBinary.php',
    'Twig\\Node\\Expression\\Binary\\LessEqualBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/LessEqualBinary.php',
    'Twig\\Node\\Expression\\Binary\\MatchesBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/MatchesBinary.php',
    'Twig\\Node\\Expression\\Binary\\ModBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/ModBinary.php',
    'Twig\\Node\\Expression\\Binary\\MulBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/MulBinary.php',
    'Twig\\Node\\Expression\\Binary\\NotEqualBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/NotEqualBinary.php',
    'Twig\\Node\\Expression\\Binary\\NotInBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/NotInBinary.php',
    'Twig\\Node\\Expression\\Binary\\NotSameAsBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/NotSameAsBinary.php',
    'Twig\\Node\\Expression\\Binary\\NullCoalesceBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/NullCoalesceBinary.php',
    'Twig\\Node\\Expression\\Binary\\ObjectDestructuringSetBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/ObjectDestructuringSetBinary.php',
    'Twig\\Node\\Expression\\Binary\\OrBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/OrBinary.php',
    'Twig\\Node\\Expression\\Binary\\PowerBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/PowerBinary.php',
    'Twig\\Node\\Expression\\Binary\\RangeBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/RangeBinary.php',
    'Twig\\Node\\Expression\\Binary\\SameAsBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/SameAsBinary.php',
    'Twig\\Node\\Expression\\Binary\\SequenceDestructuringSetBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/SequenceDestructuringSetBinary.php',
    'Twig\\Node\\Expression\\Binary\\SetBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/SetBinary.php',
    'Twig\\Node\\Expression\\Binary\\SpaceshipBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/SpaceshipBinary.php',
    'Twig\\Node\\Expression\\Binary\\StartsWithBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/StartsWithBinary.php',
    'Twig\\Node\\Expression\\Binary\\SubBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/SubBinary.php',
    'Twig\\Node\\Expression\\Binary\\XorBinary' => $vendorDir . '/twig/twig/src/Node/Expression/Binary/XorBinary.php',
    'Twig\\Node\\Expression\\BlockReferenceExpression' => $vendorDir . '/twig/twig/src/Node/Expression/BlockReferenceExpression.php',
    'Twig\\Node\\Expression\\CallExpression' => $vendorDir . '/twig/twig/src/Node/Expression/CallExpression.php',
    'Twig\\Node\\Expression\\ConditionalExpression' => $vendorDir . '/twig/twig/src/Node/Expression/ConditionalExpression.php',
    'Twig\\Node\\Expression\\ConstantExpression' => $vendorDir . '/twig/twig/src/Node/Expression/ConstantExpression.php',
    'Twig\\Node\\Expression\\EmptyExpression' => $vendorDir . '/twig/twig/src/Node/Expression/EmptyExpression.php',
    'Twig\\Node\\Expression\\FilterExpression' => $vendorDir . '/twig/twig/src/Node/Expression/FilterExpression.php',
    'Twig\\Node\\Expression\\Filter\\DefaultFilter' => $vendorDir . '/twig/twig/src/Node/Expression/Filter/DefaultFilter.php',
    'Twig\\Node\\Expression\\Filter\\RawFilter' => $vendorDir . '/twig/twig/src/Node/Expression/Filter/RawFilter.php',
    'Twig\\Node\\Expression\\FunctionExpression' => $vendorDir . '/twig/twig/src/Node/Expression/FunctionExpression.php',
    'Twig\\Node\\Expression\\FunctionNode\\EnumCasesFunction' => $vendorDir . '/twig/twig/src/Node/Expression/FunctionNode/EnumCasesFunction.php',
    'Twig\\Node\\Expression\\FunctionNode\\EnumFunction' => $vendorDir . '/twig/twig/src/Node/Expression/FunctionNode/EnumFunction.php',
    'Twig\\Node\\Expression\\GetAttrExpression' => $vendorDir . '/twig/twig/src/Node/Expression/GetAttrExpression.php',
    'Twig\\Node\\Expression\\InlinePrint' => $vendorDir . '/twig/twig/src/Node/Expression/InlinePrint.php',
    'Twig\\Node\\Expression\\ListExpression' => $vendorDir . '/twig/twig/src/Node/Expression/ListExpression.php',
    'Twig\\Node\\Expression\\MacroReferenceExpression' => $vendorDir . '/twig/twig/src/Node/Expression/MacroReferenceExpression.php',
    'Twig\\Node\\Expression\\MethodCallExpression' => $vendorDir . '/twig/twig/src/Node/Expression/MethodCallExpression.php',
    'Twig\\Node\\Expression\\NameExpression' => $vendorDir . '/twig/twig/src/Node/Expression/NameExpression.php',
    'Twig\\Node\\Expression\\NullCoalesceExpression' => $vendorDir . '/twig/twig/src/Node/Expression/NullCoalesceExpression.php',
    'Twig\\Node\\Expression\\OperatorEscapeInterface' => $vendorDir . '/twig/twig/src/Node/Expression/OperatorEscapeInterface.php',
    'Twig\\Node\\Expression\\ParentExpression' => $vendorDir . '/twig/twig/src/Node/Expression/ParentExpression.php',
    'Twig\\Node\\Expression\\ReturnArrayInterface' => $vendorDir . '/twig/twig/src/Node/Expression/ReturnArrayInterface.php',
    'Twig\\Node\\Expression\\ReturnBoolInterface' => $vendorDir . '/twig/twig/src/Node/Expression/ReturnBoolInterface.php',
    'Twig\\Node\\Expression\\ReturnNumberInterface' => $vendorDir . '/twig/twig/src/Node/Expression/ReturnNumberInterface.php',
    'Twig\\Node\\Expression\\ReturnPrimitiveTypeInterface' => $vendorDir . '/twig/twig/src/Node/Expression/ReturnPrimitiveTypeInterface.php',
    'Twig\\Node\\Expression\\ReturnStringInterface' => $vendorDir . '/twig/twig/src/Node/Expression/ReturnStringInterface.php',
    'Twig\\Node\\Expression\\SupportDefinedTestDeprecationTrait' => $vendorDir . '/twig/twig/src/Node/Expression/SupportDefinedTestDeprecationTrait.php',
    'Twig\\Node\\Expression\\SupportDefinedTestInterface' => $vendorDir . '/twig/twig/src/Node/Expression/SupportDefinedTestInterface.php',
    'Twig\\Node\\Expression\\SupportDefinedTestTrait' => $vendorDir . '/twig/twig/src/Node/Expression/SupportDefinedTestTrait.php',
    'Twig\\Node\\Expression\\TempNameExpression' => $vendorDir . '/twig/twig/src/Node/Expression/TempNameExpression.php',
    'Twig\\Node\\Expression\\Ternary\\ConditionalTernary' => $vendorDir . '/twig/twig/src/Node/Expression/Ternary/ConditionalTernary.php',
    'Twig\\Node\\Expression\\TestExpression' => $vendorDir . '/twig/twig/src/Node/Expression/TestExpression.php',
    'Twig\\Node\\Expression\\Test\\ConstantTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/ConstantTest.php',
    'Twig\\Node\\Expression\\Test\\DefinedTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/DefinedTest.php',
    'Twig\\Node\\Expression\\Test\\DivisiblebyTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/DivisiblebyTest.php',
    'Twig\\Node\\Expression\\Test\\EvenTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/EvenTest.php',
    'Twig\\Node\\Expression\\Test\\NullTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/NullTest.php',
    'Twig\\Node\\Expression\\Test\\OddTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/OddTest.php',
    'Twig\\Node\\Expression\\Test\\SameasTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/SameasTest.php',
    'Twig\\Node\\Expression\\Test\\TrueTest' => $vendorDir . '/twig/twig/src/Node/Expression/Test/TrueTest.php',
    'Twig\\Node\\Expression\\Unary\\AbstractUnary' => $vendorDir . '/twig/twig/src/Node/Expression/Unary/AbstractUnary.php',
    'Twig\\Node\\Expression\\Unary\\NegUnary' => $vendorDir . '/twig/twig/src/Node/Expression/Unary/NegUnary.php',
    'Twig\\Node\\Expression\\Unary\\NotUnary' => $vendorDir . '/twig/twig/src/Node/Expression/Unary/NotUnary.php',
    'Twig\\Node\\Expression\\Unary\\PosUnary' => $vendorDir . '/twig/twig/src/Node/Expression/Unary/PosUnary.php',
    'Twig\\Node\\Expression\\Unary\\SpreadUnary' => $vendorDir . '/twig/twig/src/Node/Expression/Unary/SpreadUnary.php',
    'Twig\\Node\\Expression\\Unary\\StringCastUnary' => $vendorDir . '/twig/twig/src/Node/Expression/Unary/StringCastUnary.php',
    'Twig\\Node\\Expression\\Unary\\UnaryInterface' => $vendorDir . '/twig/twig/src/Node/Expression/Unary/UnaryInterface.php',
    'Twig\\Node\\Expression\\Variable\\AssignContextVariable' => $vendorDir . '/twig/twig/src/Node/Expression/Variable/AssignContextVariable.php',
    'Twig\\Node\\Expression\\Variable\\AssignTemplateVariable' => $vendorDir . '/twig/twig/src/Node/Expression/Variable/AssignTemplateVariable.php',
    'Twig\\Node\\Expression\\Variable\\ContextVariable' => $vendorDir . '/twig/twig/src/Node/Expression/Variable/ContextVariable.php',
    'Twig\\Node\\Expression\\Variable\\LocalVariable' => $vendorDir . '/twig/twig/src/Node/Expression/Variable/LocalVariable.php',
    'Twig\\Node\\Expression\\Variable\\TemplateVariable' => $vendorDir . '/twig/twig/src/Node/Expression/Variable/TemplateVariable.php',
    'Twig\\Node\\Expression\\VariadicExpression' => $vendorDir . '/twig/twig/src/Node/Expression/VariadicExpression.php',
    'Twig\\Node\\FlushNode' => $vendorDir . '/twig/twig/src/Node/FlushNode.php',
    'Twig\\Node\\ForElseNode' => $vendorDir . '/twig/twig/src/Node/ForElseNode.php',
    'Twig\\Node\\ForLoopNode' => $vendorDir . '/twig/twig/src/Node/ForLoopNode.php',
    'Twig\\Node\\ForNode' => $vendorDir . '/twig/twig/src/Node/ForNode.php',
    'Twig\\Node\\IfNode' => $vendorDir . '/twig/twig/src/Node/IfNode.php',
    'Twig\\Node\\ImportNode' => $vendorDir . '/twig/twig/src/Node/ImportNode.php',
    'Twig\\Node\\IncludeNode' => $vendorDir . '/twig/twig/src/Node/IncludeNode.php',
    'Twig\\Node\\MacroNode' => $vendorDir . '/twig/twig/src/Node/MacroNode.php',
    'Twig\\Node\\ModuleNode' => $vendorDir . '/twig/twig/src/Node/ModuleNode.php',
    'Twig\\Node\\NameDeprecation' => $vendorDir . '/twig/twig/src/Node/NameDeprecation.php',
    'Twig\\Node\\Node' => $vendorDir . '/twig/twig/src/Node/Node.php',
    'Twig\\Node\\NodeCaptureInterface' => $vendorDir . '/twig/twig/src/Node/NodeCaptureInterface.php',
    'Twig\\Node\\NodeOutputInterface' => $vendorDir . '/twig/twig/src/Node/NodeOutputInterface.php',
    'Twig\\Node\\Nodes' => $vendorDir . '/twig/twig/src/Node/Nodes.php',
    'Twig\\Node\\PrintNode' => $vendorDir . '/twig/twig/src/Node/PrintNode.php',
    'Twig\\Node\\SandboxNode' => $vendorDir . '/twig/twig/src/Node/SandboxNode.php',
    'Twig\\Node\\SetNode' => $vendorDir . '/twig/twig/src/Node/SetNode.php',
    'Twig\\Node\\TextNode' => $vendorDir . '/twig/twig/src/Node/TextNode.php',
    'Twig\\Node\\TypesNode' => $vendorDir . '/twig/twig/src/Node/TypesNode.php',
    'Twig\\Node\\WithNode' => $vendorDir . '/twig/twig/src/Node/WithNode.php',
    'Twig\\OperatorPrecedenceChange' => $vendorDir . '/twig/twig/src/OperatorPrecedenceChange.php',
    'Twig\\Parser' => $vendorDir . '/twig/twig/src/Parser.php',
    'Twig\\Profiler\\Dumper\\BaseDumper' => $vendorDir . '/twig/twig/src/Profiler/Dumper/BaseDumper.php',
    'Twig\\Profiler\\Dumper\\BlackfireDumper' => $vendorDir . '/twig/twig/src/Profiler/Dumper/BlackfireDumper.php',
    'Twig\\Profiler\\Dumper\\HtmlDumper' => $vendorDir . '/twig/twig/src/Profiler/Dumper/HtmlDumper.php',
    'Twig\\Profiler\\Dumper\\TextDumper' => $vendorDir . '/twig/twig/src/Profiler/Dumper/TextDumper.php',
    'Twig\\Profiler\\NodeVisitor\\ProfilerNodeVisitor' => $vendorDir . '/twig/twig/src/Profiler/NodeVisitor/ProfilerNodeVisitor.php',
    'Twig\\Profiler\\Node\\EnterProfileNode' => $vendorDir . '/twig/twig/src/Profiler/Node/EnterProfileNode.php',
    'Twig\\Profiler\\Node\\LeaveProfileNode' => $vendorDir . '/twig/twig/src/Profiler/Node/LeaveProfileNode.php',
    'Twig\\Profiler\\Profile' => $vendorDir . '/twig/twig/src/Profiler/Profile.php',
    'Twig\\RuntimeLoader\\ContainerRuntimeLoader' => $vendorDir . '/twig/twig/src/RuntimeLoader/ContainerRuntimeLoader.php',
    'Twig\\RuntimeLoader\\FactoryRuntimeLoader' => $vendorDir . '/twig/twig/src/RuntimeLoader/FactoryRuntimeLoader.php',
    'Twig\\RuntimeLoader\\RuntimeLoaderInterface' => $vendorDir . '/twig/twig/src/RuntimeLoader/RuntimeLoaderInterface.php',
    'Twig\\Runtime\\EscaperRuntime' => $vendorDir . '/twig/twig/src/Runtime/EscaperRuntime.php',
    'Twig\\Sandbox\\SecurityError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityError.php',
    'Twig\\Sandbox\\SecurityNotAllowedFilterError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityNotAllowedFilterError.php',
    'Twig\\Sandbox\\SecurityNotAllowedFunctionError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityNotAllowedFunctionError.php',
    'Twig\\Sandbox\\SecurityNotAllowedMethodError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityNotAllowedMethodError.php',
    'Twig\\Sandbox\\SecurityNotAllowedPropertyError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityNotAllowedPropertyError.php',
    'Twig\\Sandbox\\SecurityNotAllowedTagError' => $vendorDir . '/twig/twig/src/Sandbox/SecurityNotAllowedTagError.php',
    'Twig\\Sandbox\\SecurityPolicy' => $vendorDir . '/twig/twig/src/Sandbox/SecurityPolicy.php',
    'Twig\\Sandbox\\SecurityPolicyInterface' => $vendorDir . '/twig/twig/src/Sandbox/SecurityPolicyInterface.php',
    'Twig\\Sandbox\\SourcePolicyInterface' => $vendorDir . '/twig/twig/src/Sandbox/SourcePolicyInterface.php',
    'Twig\\Source' => $vendorDir . '/twig/twig/src/Source.php',
    'Twig\\Template' => $vendorDir . '/twig/twig/src/Template.php',
    'Twig\\TemplateWrapper' => $vendorDir . '/twig/twig/src/TemplateWrapper.php',
    'Twig\\Test\\IntegrationTestCase' => $vendorDir . '/twig/twig/src/Test/IntegrationTestCase.php',
    'Twig\\Test\\NodeTestCase' => $vendorDir . '/twig/twig/src/Test/NodeTestCase.php',
    'Twig\\Token' => $vendorDir . '/twig/twig/src/Token.php',
    'Twig\\TokenParser\\AbstractTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/AbstractTokenParser.php',
    'Twig\\TokenParser\\ApplyTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/ApplyTokenParser.php',
    'Twig\\TokenParser\\AutoEscapeTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/AutoEscapeTokenParser.php',
    'Twig\\TokenParser\\BlockTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/BlockTokenParser.php',
    'Twig\\TokenParser\\DeprecatedTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/DeprecatedTokenParser.php',
    'Twig\\TokenParser\\DoTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/DoTokenParser.php',
    'Twig\\TokenParser\\EmbedTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/EmbedTokenParser.php',
    'Twig\\TokenParser\\ExtendsTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/ExtendsTokenParser.php',
    'Twig\\TokenParser\\FlushTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/FlushTokenParser.php',
    'Twig\\TokenParser\\ForTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/ForTokenParser.php',
    'Twig\\TokenParser\\FromTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/FromTokenParser.php',
    'Twig\\TokenParser\\GuardTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/GuardTokenParser.php',
    'Twig\\TokenParser\\IfTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/IfTokenParser.php',
    'Twig\\TokenParser\\ImportTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/ImportTokenParser.php',
    'Twig\\TokenParser\\IncludeTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/IncludeTokenParser.php',
    'Twig\\TokenParser\\MacroTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/MacroTokenParser.php',
    'Twig\\TokenParser\\SandboxTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/SandboxTokenParser.php',
    'Twig\\TokenParser\\SetTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/SetTokenParser.php',
    'Twig\\TokenParser\\TokenParserInterface' => $vendorDir . '/twig/twig/src/TokenParser/TokenParserInterface.php',
    'Twig\\TokenParser\\TypesTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/TypesTokenParser.php',
    'Twig\\TokenParser\\UseTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/UseTokenParser.php',
    'Twig\\TokenParser\\WithTokenParser' => $vendorDir . '/twig/twig/src/TokenParser/WithTokenParser.php',
    'Twig\\TokenStream' => $vendorDir . '/twig/twig/src/TokenStream.php',
    'Twig\\TwigCallableInterface' => $vendorDir . '/twig/twig/src/TwigCallableInterface.php',
    'Twig\\TwigFilter' => $vendorDir . '/twig/twig/src/TwigFilter.php',
    'Twig\\TwigFunction' => $vendorDir . '/twig/twig/src/TwigFunction.php',
    'Twig\\TwigTest' => $vendorDir . '/twig/twig/src/TwigTest.php',
    'Twig\\Util\\CallableArgumentsExtractor' => $vendorDir . '/twig/twig/src/Util/CallableArgumentsExtractor.php',
    'Twig\\Util\\DeprecationCollector' => $vendorDir . '/twig/twig/src/Util/DeprecationCollector.php',
    'Twig\\Util\\ReflectionCallable' => $vendorDir . '/twig/twig/src/Util/ReflectionCallable.php',
    'Twig\\Util\\TemplateDirIterator' => $vendorDir . '/twig/twig/src/Util/TemplateDirIterator.php',
    'USBVendor' => $baseDir . '/src/USBVendor.php',
    'Unmanaged' => $baseDir . '/src/Unmanaged.php',
    'Update' => $baseDir . '/src/Update.php',
    'UploadHandler' => $baseDir . '/src/UploadHandler.php',
    'User' => $baseDir . '/src/User.php',
    'UserCategory' => $baseDir . '/src/UserCategory.php',
    'UserEmail' => $baseDir . '/src/UserEmail.php',
    'UserTitle' => $baseDir . '/src/UserTitle.php',
    'VObject' => $baseDir . '/src/VObject.php',
    'ValidationStep' => $baseDir . '/src/ValidationStep.php',
    'ValidatorSubstitute' => $baseDir . '/src/ValidatorSubstitute.php',
    'VirtualMachineState' => $baseDir . '/src/VirtualMachineState.php',
    'VirtualMachineSystem' => $baseDir . '/src/VirtualMachineSystem.php',
    'VirtualMachineType' => $baseDir . '/src/VirtualMachineType.php',
    'Vlan' => $baseDir . '/src/Vlan.php',
    'Webhook' => $baseDir . '/src/Webhook.php',
    'WebhookCategory' => $baseDir . '/src/WebhookCategory.php',
    'Webmozart\\Assert\\Assert' => $vendorDir . '/webmozart/assert/src/Assert.php',
    'Webmozart\\Assert\\InvalidArgumentException' => $vendorDir . '/webmozart/assert/src/InvalidArgumentException.php',
    'Webmozart\\Assert\\Mixin' => $vendorDir . '/webmozart/assert/src/Mixin.php',
    'WifiNetwork' => $baseDir . '/src/WifiNetwork.php',
    'XHProf' => $baseDir . '/src/XHProf.php',
    'ZipStream\\CentralDirectoryFileHeader' => $vendorDir . '/maennchen/zipstream-php/src/CentralDirectoryFileHeader.php',
    'ZipStream\\CompressionMethod' => $vendorDir . '/maennchen/zipstream-php/src/CompressionMethod.php',
    'ZipStream\\DataDescriptor' => $vendorDir . '/maennchen/zipstream-php/src/DataDescriptor.php',
    'ZipStream\\EndOfCentralDirectory' => $vendorDir . '/maennchen/zipstream-php/src/EndOfCentralDirectory.php',
    'ZipStream\\Exception' => $vendorDir . '/maennchen/zipstream-php/src/Exception.php',
    'ZipStream\\Exception\\DosTimeOverflowException' => $vendorDir . '/maennchen/zipstream-php/src/Exception/DosTimeOverflowException.php',
    'ZipStream\\Exception\\FileNotFoundException' => $vendorDir . '/maennchen/zipstream-php/src/Exception/FileNotFoundException.php',
    'ZipStream\\Exception\\FileNotReadableException' => $vendorDir . '/maennchen/zipstream-php/src/Exception/FileNotReadableException.php',
    'ZipStream\\Exception\\FileSizeIncorrectException' => $vendorDir . '/maennchen/zipstream-php/src/Exception/FileSizeIncorrectException.php',
    'ZipStream\\Exception\\OverflowException' => $vendorDir . '/maennchen/zipstream-php/src/Exception/OverflowException.php',
    'ZipStream\\Exception\\ResourceActionException' => $vendorDir . '/maennchen/zipstream-php/src/Exception/ResourceActionException.php',
    'ZipStream\\Exception\\SimulationFileUnknownException' => $vendorDir . '/maennchen/zipstream-php/src/Exception/SimulationFileUnknownException.php',
    'ZipStream\\Exception\\StreamNotReadableException' => $vendorDir . '/maennchen/zipstream-php/src/Exception/StreamNotReadableException.php',
    'ZipStream\\Exception\\StreamNotSeekableException' => $vendorDir . '/maennchen/zipstream-php/src/Exception/StreamNotSeekableException.php',
    'ZipStream\\File' => $vendorDir . '/maennchen/zipstream-php/src/File.php',
    'ZipStream\\GeneralPurposeBitFlag' => $vendorDir . '/maennchen/zipstream-php/src/GeneralPurposeBitFlag.php',
    'ZipStream\\LocalFileHeader' => $vendorDir . '/maennchen/zipstream-php/src/LocalFileHeader.php',
    'ZipStream\\OperationMode' => $vendorDir . '/maennchen/zipstream-php/src/OperationMode.php',
    'ZipStream\\PackField' => $vendorDir . '/maennchen/zipstream-php/src/PackField.php',
    'ZipStream\\Time' => $vendorDir . '/maennchen/zipstream-php/src/Time.php',
    'ZipStream\\Version' => $vendorDir . '/maennchen/zipstream-php/src/Version.php',
    'ZipStream\\Zip64\\DataDescriptor' => $vendorDir . '/maennchen/zipstream-php/src/Zip64/DataDescriptor.php',
    'ZipStream\\Zip64\\EndOfCentralDirectory' => $vendorDir . '/maennchen/zipstream-php/src/Zip64/EndOfCentralDirectory.php',
    'ZipStream\\Zip64\\EndOfCentralDirectoryLocator' => $vendorDir . '/maennchen/zipstream-php/src/Zip64/EndOfCentralDirectoryLocator.php',
    'ZipStream\\Zip64\\ExtendedInformationExtraField' => $vendorDir . '/maennchen/zipstream-php/src/Zip64/ExtendedInformationExtraField.php',
    'ZipStream\\ZipStream' => $vendorDir . '/maennchen/zipstream-php/src/ZipStream.php',
    'ZipStream\\Zs\\ExtendedInformationExtraField' => $vendorDir . '/maennchen/zipstream-php/src/Zs/ExtendedInformationExtraField.php',
    'donatj\\UserAgent\\Browsers' => $vendorDir . '/donatj/phpuseragentparser/src/UserAgent/Browsers.php',
    'donatj\\UserAgent\\Platforms' => $vendorDir . '/donatj/phpuseragentparser/src/UserAgent/Platforms.php',
    'donatj\\UserAgent\\UserAgent' => $vendorDir . '/donatj/phpuseragentparser/src/UserAgent/UserAgent.php',
    'donatj\\UserAgent\\UserAgentInterface' => $vendorDir . '/donatj/phpuseragentparser/src/UserAgent/UserAgentInterface.php',
    'donatj\\UserAgent\\UserAgentParser' => $vendorDir . '/donatj/phpuseragentparser/src/UserAgent/UserAgentParser.php',
    'phpCAS' => $vendorDir . '/apereo/phpcas/source/CAS.php',
    'phpDocumentor\\Reflection\\DocBlock' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock.php',
    'phpDocumentor\\Reflection\\DocBlockFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlockFactory.php',
    'phpDocumentor\\Reflection\\DocBlockFactoryInterface' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlockFactoryInterface.php',
    'phpDocumentor\\Reflection\\DocBlock\\Description' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Description.php',
    'phpDocumentor\\Reflection\\DocBlock\\DescriptionFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/DescriptionFactory.php',
    'phpDocumentor\\Reflection\\DocBlock\\ExampleFinder' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/ExampleFinder.php',
    'phpDocumentor\\Reflection\\DocBlock\\Serializer' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Serializer.php',
    'phpDocumentor\\Reflection\\DocBlock\\StandardTagFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/StandardTagFactory.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tag' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tag.php',
    'phpDocumentor\\Reflection\\DocBlock\\TagFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/TagFactory.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Author' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Author.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\BaseTag' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/BaseTag.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Covers' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Covers.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Deprecated' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Deprecated.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Example' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Example.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Extends_' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Extends_.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Factory\\AbstractPHPStanFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Factory/AbstractPHPStanFactory.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Factory\\ExtendsFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Factory/ExtendsFactory.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Factory\\Factory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Factory/Factory.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Factory\\ImplementsFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Factory/ImplementsFactory.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Factory\\MethodFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Factory/MethodFactory.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Factory\\MethodParameterFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Factory/MethodParameterFactory.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Factory\\PHPStanFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Factory/PHPStanFactory.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Factory\\ParamFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Factory/ParamFactory.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Factory\\PropertyFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Factory/PropertyFactory.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Factory\\PropertyReadFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Factory/PropertyReadFactory.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Factory\\PropertyWriteFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Factory/PropertyWriteFactory.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Factory\\ReturnFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Factory/ReturnFactory.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Factory\\StaticMethod' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Factory/StaticMethod.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Factory\\TemplateExtendsFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Factory/TemplateExtendsFactory.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Factory\\TemplateFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Factory/TemplateFactory.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Factory\\TemplateImplementsFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Factory/TemplateImplementsFactory.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Factory\\VarFactory' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Factory/VarFactory.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Formatter' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Formatter.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Formatter\\AlignFormatter' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Formatter/AlignFormatter.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Formatter\\PassthroughFormatter' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Formatter/PassthroughFormatter.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Generic' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Generic.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Implements_' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Implements_.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\InvalidTag' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/InvalidTag.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Link' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Link.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Method' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Method.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\MethodParameter' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/MethodParameter.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Mixin' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Mixin.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Param' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Param.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Property' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Property.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\PropertyRead' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/PropertyRead.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\PropertyWrite' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/PropertyWrite.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Reference\\Fqsen' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Reference/Fqsen.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Reference\\Reference' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Reference/Reference.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Reference\\Url' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Reference/Url.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Return_' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Return_.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\See' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/See.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Since' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Since.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Source' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Source.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\TagWithType' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/TagWithType.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Template' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Template.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\TemplateCovariant' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/TemplateCovariant.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\TemplateExtends' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/TemplateExtends.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\TemplateImplements' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/TemplateImplements.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Throws' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Throws.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Uses' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Uses.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Var_' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Var_.php',
    'phpDocumentor\\Reflection\\DocBlock\\Tags\\Version' => $vendorDir . '/phpdocumentor/reflection-docblock/src/DocBlock/Tags/Version.php',
    'phpDocumentor\\Reflection\\Element' => $vendorDir . '/phpdocumentor/reflection-common/src/Element.php',
    'phpDocumentor\\Reflection\\Exception\\PcreException' => $vendorDir . '/phpdocumentor/reflection-docblock/src/Exception/PcreException.php',
    'phpDocumentor\\Reflection\\File' => $vendorDir . '/phpdocumentor/reflection-common/src/File.php',
    'phpDocumentor\\Reflection\\Fqsen' => $vendorDir . '/phpdocumentor/reflection-common/src/Fqsen.php',
    'phpDocumentor\\Reflection\\FqsenResolver' => $vendorDir . '/phpdocumentor/type-resolver/src/FqsenResolver.php',
    'phpDocumentor\\Reflection\\Location' => $vendorDir . '/phpdocumentor/reflection-common/src/Location.php',
    'phpDocumentor\\Reflection\\Project' => $vendorDir . '/phpdocumentor/reflection-common/src/Project.php',
    'phpDocumentor\\Reflection\\ProjectFactory' => $vendorDir . '/phpdocumentor/reflection-common/src/ProjectFactory.php',
    'phpDocumentor\\Reflection\\PseudoType' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoType.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\ArrayShape' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/ArrayShape.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\ArrayShapeItem' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/ArrayShapeItem.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\CallableString' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/CallableString.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\Conditional' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/Conditional.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\ConditionalForParameter' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/ConditionalForParameter.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\ConstExpression' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/ConstExpression.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\False_' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/False_.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\FloatValue' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/FloatValue.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\HtmlEscapedString' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/HtmlEscapedString.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\IntMask' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/IntMask.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\IntMaskOf' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/IntMaskOf.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\IntegerRange' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/IntegerRange.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\IntegerValue' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/IntegerValue.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\KeyOf' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/KeyOf.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\ListShape' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/ListShape.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\ListShapeItem' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/ListShapeItem.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\List_' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/List_.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\LiteralString' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/LiteralString.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\LowercaseString' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/LowercaseString.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\NegativeInteger' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/NegativeInteger.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\NonEmptyArray' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/NonEmptyArray.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\NonEmptyList' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/NonEmptyList.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\NonEmptyLowercaseString' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/NonEmptyLowercaseString.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\NonEmptyString' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/NonEmptyString.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\NumericString' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/NumericString.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\Numeric_' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/Numeric_.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\ObjectShape' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/ObjectShape.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\ObjectShapeItem' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/ObjectShapeItem.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\OffsetAccess' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/OffsetAccess.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\PositiveInteger' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/PositiveInteger.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\ShapeItem' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/ShapeItem.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\StringValue' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/StringValue.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\TraitString' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/TraitString.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\True_' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/True_.php',
    'phpDocumentor\\Reflection\\PseudoTypes\\ValueOf' => $vendorDir . '/phpdocumentor/type-resolver/src/PseudoTypes/ValueOf.php',
    'phpDocumentor\\Reflection\\Type' => $vendorDir . '/phpdocumentor/type-resolver/src/Type.php',
    'phpDocumentor\\Reflection\\TypeResolver' => $vendorDir . '/phpdocumentor/type-resolver/src/TypeResolver.php',
    'phpDocumentor\\Reflection\\Types\\AbstractList' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/AbstractList.php',
    'phpDocumentor\\Reflection\\Types\\AggregatedType' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/AggregatedType.php',
    'phpDocumentor\\Reflection\\Types\\ArrayKey' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/ArrayKey.php',
    'phpDocumentor\\Reflection\\Types\\Array_' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Array_.php',
    'phpDocumentor\\Reflection\\Types\\Boolean' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Boolean.php',
    'phpDocumentor\\Reflection\\Types\\CallableParameter' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/CallableParameter.php',
    'phpDocumentor\\Reflection\\Types\\Callable_' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Callable_.php',
    'phpDocumentor\\Reflection\\Types\\ClassString' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/ClassString.php',
    'phpDocumentor\\Reflection\\Types\\Collection' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Collection.php',
    'phpDocumentor\\Reflection\\Types\\Compound' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Compound.php',
    'phpDocumentor\\Reflection\\Types\\Context' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Context.php',
    'phpDocumentor\\Reflection\\Types\\ContextFactory' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/ContextFactory.php',
    'phpDocumentor\\Reflection\\Types\\Expression' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Expression.php',
    'phpDocumentor\\Reflection\\Types\\Float_' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Float_.php',
    'phpDocumentor\\Reflection\\Types\\Integer' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Integer.php',
    'phpDocumentor\\Reflection\\Types\\InterfaceString' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/InterfaceString.php',
    'phpDocumentor\\Reflection\\Types\\Intersection' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Intersection.php',
    'phpDocumentor\\Reflection\\Types\\Iterable_' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Iterable_.php',
    'phpDocumentor\\Reflection\\Types\\Mixed_' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Mixed_.php',
    'phpDocumentor\\Reflection\\Types\\Never_' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Never_.php',
    'phpDocumentor\\Reflection\\Types\\Null_' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Null_.php',
    'phpDocumentor\\Reflection\\Types\\Nullable' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Nullable.php',
    'phpDocumentor\\Reflection\\Types\\Object_' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Object_.php',
    'phpDocumentor\\Reflection\\Types\\Parent_' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Parent_.php',
    'phpDocumentor\\Reflection\\Types\\Resource_' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Resource_.php',
    'phpDocumentor\\Reflection\\Types\\Scalar' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Scalar.php',
    'phpDocumentor\\Reflection\\Types\\Self_' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Self_.php',
    'phpDocumentor\\Reflection\\Types\\Static_' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Static_.php',
    'phpDocumentor\\Reflection\\Types\\String_' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/String_.php',
    'phpDocumentor\\Reflection\\Types\\This' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/This.php',
    'phpDocumentor\\Reflection\\Types\\Void_' => $vendorDir . '/phpdocumentor/type-resolver/src/Types/Void_.php',
    'phpDocumentor\\Reflection\\Utils' => $vendorDir . '/phpdocumentor/reflection-docblock/src/Utils.php',
    'wapmorgan\\UnifiedArchive\\Abilities' => $vendorDir . '/wapmorgan/unified-archive/src/Abilities.php',
    'wapmorgan\\UnifiedArchive\\Archive7z' => $vendorDir . '/wapmorgan/unified-archive/src/Archive7z.php',
    'wapmorgan\\UnifiedArchive\\ArchiveEntry' => $vendorDir . '/wapmorgan/unified-archive/src/ArchiveEntry.php',
    'wapmorgan\\UnifiedArchive\\ArchiveInformation' => $vendorDir . '/wapmorgan/unified-archive/src/ArchiveInformation.php',
    'wapmorgan\\UnifiedArchive\\Commands\\AddFileCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/AddFileCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\BaseArchiveCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/BaseArchiveCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\BaseCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/BaseCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\BaseFileCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/BaseFileCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\CommentCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/CommentCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\CreateCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/CreateCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\DeleteFileCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/DeleteFileCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\DetailsCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/DetailsCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\DriversCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/DriversCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\ExtractArchiveCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/ExtractArchiveCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\ExtractFileCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/ExtractFileCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\ExtractFilesCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/ExtractFilesCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\FoldersCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/FoldersCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\FormatCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/FormatCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\FormatsCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/FormatsCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\InfoCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/InfoCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\ListCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/ListCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\PrintCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/PrintCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\TableCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/TableCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\TestCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/TestCommand.php',
    'wapmorgan\\UnifiedArchive\\Commands\\TypesCommand' => $vendorDir . '/wapmorgan/unified-archive/src/Commands/TypesCommand.php',
    'wapmorgan\\UnifiedArchive\\Drivers\\AlchemyZippy' => $vendorDir . '/wapmorgan/unified-archive/src/Drivers/AlchemyZippy.php',
    'wapmorgan\\UnifiedArchive\\Drivers\\Basic\\BasicDriver' => $vendorDir . '/wapmorgan/unified-archive/src/Drivers/Basic/BasicDriver.php',
    'wapmorgan\\UnifiedArchive\\Drivers\\Basic\\BasicExtensionDriver' => $vendorDir . '/wapmorgan/unified-archive/src/Drivers/Basic/BasicExtensionDriver.php',
    'wapmorgan\\UnifiedArchive\\Drivers\\Basic\\BasicPureDriver' => $vendorDir . '/wapmorgan/unified-archive/src/Drivers/Basic/BasicPureDriver.php',
    'wapmorgan\\UnifiedArchive\\Drivers\\Basic\\BasicUtilityDriver' => $vendorDir . '/wapmorgan/unified-archive/src/Drivers/Basic/BasicUtilityDriver.php',
    'wapmorgan\\UnifiedArchive\\Drivers\\Cab' => $vendorDir . '/wapmorgan/unified-archive/src/Drivers/Cab.php',
    'wapmorgan\\UnifiedArchive\\Drivers\\Iso' => $vendorDir . '/wapmorgan/unified-archive/src/Drivers/Iso.php',
    'wapmorgan\\UnifiedArchive\\Drivers\\NelexaZip' => $vendorDir . '/wapmorgan/unified-archive/src/Drivers/NelexaZip.php',
    'wapmorgan\\UnifiedArchive\\Drivers\\OneFile\\Bzip' => $vendorDir . '/wapmorgan/unified-archive/src/Drivers/OneFile/Bzip.php',
    'wapmorgan\\UnifiedArchive\\Drivers\\OneFile\\Gzip' => $vendorDir . '/wapmorgan/unified-archive/src/Drivers/OneFile/Gzip.php',
    'wapmorgan\\UnifiedArchive\\Drivers\\OneFile\\Lzma' => $vendorDir . '/wapmorgan/unified-archive/src/Drivers/OneFile/Lzma.php',
    'wapmorgan\\UnifiedArchive\\Drivers\\OneFile\\OneFileDriver' => $vendorDir . '/wapmorgan/unified-archive/src/Drivers/OneFile/OneFileDriver.php',
    'wapmorgan\\UnifiedArchive\\Drivers\\Rar' => $vendorDir . '/wapmorgan/unified-archive/src/Drivers/Rar.php',
    'wapmorgan\\UnifiedArchive\\Drivers\\SevenZip' => $vendorDir . '/wapmorgan/unified-archive/src/Drivers/SevenZip.php',
    'wapmorgan\\UnifiedArchive\\Drivers\\SplitbrainPhpArchive' => $vendorDir . '/wapmorgan/unified-archive/src/Drivers/SplitbrainPhpArchive.php',
    'wapmorgan\\UnifiedArchive\\Drivers\\TarByPear' => $vendorDir . '/wapmorgan/unified-archive/src/Drivers/TarByPear.php',
    'wapmorgan\\UnifiedArchive\\Drivers\\TarByPhar' => $vendorDir . '/wapmorgan/unified-archive/src/Drivers/TarByPhar.php',
    'wapmorgan\\UnifiedArchive\\Drivers\\Zip' => $vendorDir . '/wapmorgan/unified-archive/src/Drivers/Zip.php',
    'wapmorgan\\UnifiedArchive\\Exceptions\\ArchiveCreationException' => $vendorDir . '/wapmorgan/unified-archive/src/Exceptions/ArchiveCreationException.php',
    'wapmorgan\\UnifiedArchive\\Exceptions\\ArchiveExtractionException' => $vendorDir . '/wapmorgan/unified-archive/src/Exceptions/ArchiveExtractionException.php',
    'wapmorgan\\UnifiedArchive\\Exceptions\\ArchiveModificationException' => $vendorDir . '/wapmorgan/unified-archive/src/Exceptions/ArchiveModificationException.php',
    'wapmorgan\\UnifiedArchive\\Exceptions\\EmptyFileListException' => $vendorDir . '/wapmorgan/unified-archive/src/Exceptions/EmptyFileListException.php',
    'wapmorgan\\UnifiedArchive\\Exceptions\\FileAlreadyExistsException' => $vendorDir . '/wapmorgan/unified-archive/src/Exceptions/FileAlreadyExistsException.php',
    'wapmorgan\\UnifiedArchive\\Exceptions\\NonExistentArchiveFileException' => $vendorDir . '/wapmorgan/unified-archive/src/Exceptions/NonExistentArchiveFileException.php',
    'wapmorgan\\UnifiedArchive\\Exceptions\\UnsupportedArchiveException' => $vendorDir . '/wapmorgan/unified-archive/src/Exceptions/UnsupportedArchiveException.php',
    'wapmorgan\\UnifiedArchive\\Exceptions\\UnsupportedOperationException' => $vendorDir . '/wapmorgan/unified-archive/src/Exceptions/UnsupportedOperationException.php',
    'wapmorgan\\UnifiedArchive\\Formats' => $vendorDir . '/wapmorgan/unified-archive/src/Formats.php',
    'wapmorgan\\UnifiedArchive\\LzwStreamWrapper' => $vendorDir . '/wapmorgan/unified-archive/src/LzwStreamWrapper.php',
    'wapmorgan\\UnifiedArchive\\PclZipInterface' => $vendorDir . '/wapmorgan/unified-archive/src/PclZipInterface.php',
    'wapmorgan\\UnifiedArchive\\UnifiedArchive' => $vendorDir . '/wapmorgan/unified-archive/src/UnifiedArchive.php',
    '©' => $vendorDir . '/symfony/cache/Traits/ValueWrapper.php',
);
