/******/ (() => { // webpackBootstrap
/******/ 	var __webpack_modules__ = ({

/***/ 1101
(__unused_webpack_module, __webpack_exports__, __webpack_require__) {

"use strict";
__webpack_require__.r(__webpack_exports__);
// extracted by mini-css-extract-plugin


/***/ },

/***/ 1100
(__unused_webpack_module, __webpack_exports__, __webpack_require__) {

"use strict";
__webpack_require__.r(__webpack_exports__);
// extracted by mini-css-extract-plugin


/***/ },

/***/ 1102
(__unused_webpack_module, __webpack_exports__, __webpack_require__) {

"use strict";
__webpack_require__.r(__webpack_exports__);
// extracted by mini-css-extract-plugin


/***/ },

/***/ 1103
(__unused_webpack_module, __webpack_exports__, __webpack_require__) {

"use strict";
__webpack_require__.r(__webpack_exports__);
// extracted by mini-css-extract-plugin


/***/ },

/***/ 1104
(__unused_webpack_module, __webpack_exports__, __webpack_require__) {

"use strict";
__webpack_require__.r(__webpack_exports__);
// extracted by mini-css-extract-plugin


/***/ },

/***/ 1105
(__unused_webpack_module, __webpack_exports__, __webpack_require__) {

"use strict";
__webpack_require__.r(__webpack_exports__);
// extracted by mini-css-extract-plugin


/***/ },

/***/ 1106
(__unused_webpack_module, __webpack_exports__, __webpack_require__) {

"use strict";
__webpack_require__.r(__webpack_exports__);
// extracted by mini-css-extract-plugin


/***/ },

/***/ 1085
(module) {

module.exports = "/*!\nFullCalendar Bootstrap Plugin v4.4.2\nDocs & License: https://fullcalendar.io/\n(c) 2019 Adam Shaw\n*/\n\n(function (global, factory) {\n    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@fullcalendar/core')) :\n    typeof define === 'function' && define.amd ? define(['exports', '@fullcalendar/core'], factory) :\n    (global = global || self, factory(global.FullCalendarBootstrap = {}, global.FullCalendar));\n}(this, function (exports, core) { 'use strict';\n\n    /*! *****************************************************************************\r\n    Copyright (c) Microsoft Corporation.\r\n\r\n    Permission to use, copy, modify, and/or distribute this software for any\r\n    purpose with or without fee is hereby granted.\r\n\r\n    THE SOFTWARE IS PROVIDED \"AS IS\" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH\r\n    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY\r\n    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,\r\n    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM\r\n    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR\r\n    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR\r\n    PERFORMANCE OF THIS SOFTWARE.\r\n    ***************************************************************************** */\r\n    /* global Reflect, Promise */\r\n\r\n    var extendStatics = function(d, b) {\r\n        extendStatics = Object.setPrototypeOf ||\r\n            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||\r\n            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };\r\n        return extendStatics(d, b);\r\n    };\r\n\r\n    function __extends(d, b) {\r\n        extendStatics(d, b);\r\n        function __() { this.constructor = d; }\r\n        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());\r\n    }\n\n    var BootstrapTheme = /** @class */ (function (_super) {\n        __extends(BootstrapTheme, _super);\n        function BootstrapTheme() {\n            return _super !== null && _super.apply(this, arguments) || this;\n        }\n        return BootstrapTheme;\n    }(core.Theme));\n    BootstrapTheme.prototype.classes = {\n        widget: 'fc-bootstrap',\n        tableGrid: 'table-bordered',\n        tableList: 'table',\n        tableListHeading: 'table-active',\n        buttonGroup: 'btn-group',\n        button: 'btn btn-primary',\n        buttonActive: 'active',\n        today: 'alert alert-info',\n        popover: 'card card-primary',\n        popoverHeader: 'card-header',\n        popoverContent: 'card-body',\n        // day grid\n        // for left/right border color when border is inset from edges (all-day in timeGrid view)\n        // avoid `table` class b/c don't want margins/padding/structure. only border color.\n        headerRow: 'table-bordered',\n        dayRow: 'table-bordered',\n        // list view\n        listView: 'card card-primary'\n    };\n    BootstrapTheme.prototype.baseIconClass = 'fa';\n    BootstrapTheme.prototype.iconClasses = {\n        close: 'fa-times',\n        prev: 'fa-chevron-left',\n        next: 'fa-chevron-right',\n        prevYear: 'fa-angle-double-left',\n        nextYear: 'fa-angle-double-right'\n    };\n    BootstrapTheme.prototype.iconOverrideOption = 'bootstrapFontAwesome';\n    BootstrapTheme.prototype.iconOverrideCustomButtonOption = 'bootstrapFontAwesome';\n    BootstrapTheme.prototype.iconOverridePrefix = 'fa-';\n    var main = core.createPlugin({\n        themeClasses: {\n            bootstrap: BootstrapTheme\n        }\n    });\n\n    exports.BootstrapTheme = BootstrapTheme;\n    exports.default = main;\n\n    Object.defineProperty(exports, '__esModule', { value: true });\n\n}));\n"

/***/ },

/***/ 1081
(module) {

module.exports = "/*!\nFullCalendar Core Package v4.4.2\nDocs & License: https://fullcalendar.io/\n(c) 2019 Adam Shaw\n*/\n\n(function (global, factory) {\n    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports) :\n    typeof define === 'function' && define.amd ? define(['exports'], factory) :\n    (global = global || self, factory(global.FullCalendar = {}));\n}(this, function (exports) { 'use strict';\n\n    // Creating\n    // ----------------------------------------------------------------------------------------------------------------\n    var elementPropHash = {\n        className: true,\n        colSpan: true,\n        rowSpan: true\n    };\n    var containerTagHash = {\n        '<tr': 'tbody',\n        '<td': 'tr'\n    };\n    function createElement(tagName, attrs, content) {\n        var el = document.createElement(tagName);\n        if (attrs) {\n            for (var attrName in attrs) {\n                if (attrName === 'style') {\n                    applyStyle(el, attrs[attrName]);\n                }\n                else if (elementPropHash[attrName]) {\n                    el[attrName] = attrs[attrName];\n                }\n                else {\n                    el.setAttribute(attrName, attrs[attrName]);\n                }\n            }\n        }\n        if (typeof content === 'string') {\n            el.innerHTML = content; // shortcut. no need to process HTML in any way\n        }\n        else if (content != null) {\n            appendToElement(el, content);\n        }\n        return el;\n    }\n    function htmlToElement(html) {\n        html = html.trim();\n        var container = document.createElement(computeContainerTag(html));\n        container.innerHTML = html;\n        return container.firstChild;\n    }\n    function htmlToElements(html) {\n        return Array.prototype.slice.call(htmlToNodeList(html));\n    }\n    function htmlToNodeList(html) {\n        html = html.trim();\n        var container = document.createElement(computeContainerTag(html));\n        container.innerHTML = html;\n        return container.childNodes;\n    }\n    // assumes html already trimmed and tag names are lowercase\n    function computeContainerTag(html) {\n        return containerTagHash[html.substr(0, 3) // faster than using regex\n        ] || 'div';\n    }\n    function appendToElement(el, content) {\n        var childNodes = normalizeContent(content);\n        for (var i = 0; i < childNodes.length; i++) {\n            el.appendChild(childNodes[i]);\n        }\n    }\n    function prependToElement(parent, content) {\n        var newEls = normalizeContent(content);\n        var afterEl = parent.firstChild || null; // if no firstChild, will append to end, but that's okay, b/c there were no children\n        for (var i = 0; i < newEls.length; i++) {\n            parent.insertBefore(newEls[i], afterEl);\n        }\n    }\n    function insertAfterElement(refEl, content) {\n        var newEls = normalizeContent(content);\n        var afterEl = refEl.nextSibling || null;\n        for (var i = 0; i < newEls.length; i++) {\n            refEl.parentNode.insertBefore(newEls[i], afterEl);\n        }\n    }\n    function normalizeContent(content) {\n        var els;\n        if (typeof content === 'string') {\n            els = htmlToElements(content);\n        }\n        else if (content instanceof Node) {\n            els = [content];\n        }\n        else { // Node[] or NodeList\n            els = Array.prototype.slice.call(content);\n        }\n        return els;\n    }\n    function removeElement(el) {\n        if (el.parentNode) {\n            el.parentNode.removeChild(el);\n        }\n    }\n    // Querying\n    // ----------------------------------------------------------------------------------------------------------------\n    // from https://developer.mozilla.org/en-US/docs/Web/API/Element/closest\n    var matchesMethod = Element.prototype.matches ||\n        Element.prototype.matchesSelector ||\n        Element.prototype.msMatchesSelector;\n    var closestMethod = Element.prototype.closest || function (selector) {\n        // polyfill\n        var el = this;\n        if (!document.documentElement.contains(el)) {\n            return null;\n        }\n        do {\n            if (elementMatches(el, selector)) {\n                return el;\n            }\n            el = el.parentElement || el.parentNode;\n        } while (el !== null && el.nodeType === 1);\n        return null;\n    };\n    function elementClosest(el, selector) {\n        return closestMethod.call(el, selector);\n    }\n    function elementMatches(el, selector) {\n        return matchesMethod.call(el, selector);\n    }\n    // accepts multiple subject els\n    // returns a real array. good for methods like forEach\n    function findElements(container, selector) {\n        var containers = container instanceof HTMLElement ? [container] : container;\n        var allMatches = [];\n        for (var i = 0; i < containers.length; i++) {\n            var matches = containers[i].querySelectorAll(selector);\n            for (var j = 0; j < matches.length; j++) {\n                allMatches.push(matches[j]);\n            }\n        }\n        return allMatches;\n    }\n    // accepts multiple subject els\n    // only queries direct child elements\n    function findChildren(parent, selector) {\n        var parents = parent instanceof HTMLElement ? [parent] : parent;\n        var allMatches = [];\n        for (var i = 0; i < parents.length; i++) {\n            var childNodes = parents[i].children; // only ever elements\n            for (var j = 0; j < childNodes.length; j++) {\n                var childNode = childNodes[j];\n                if (!selector || elementMatches(childNode, selector)) {\n                    allMatches.push(childNode);\n                }\n            }\n        }\n        return allMatches;\n    }\n    // Attributes\n    // ----------------------------------------------------------------------------------------------------------------\n    function forceClassName(el, className, bool) {\n        if (bool) {\n            el.classList.add(className);\n        }\n        else {\n            el.classList.remove(className);\n        }\n    }\n    // Style\n    // ----------------------------------------------------------------------------------------------------------------\n    var PIXEL_PROP_RE = /(top|left|right|bottom|width|height)$/i;\n    function applyStyle(el, props) {\n        for (var propName in props) {\n            applyStyleProp(el, propName, props[propName]);\n        }\n    }\n    function applyStyleProp(el, name, val) {\n        if (val == null) {\n            el.style[name] = '';\n        }\n        else if (typeof val === 'number' && PIXEL_PROP_RE.test(name)) {\n            el.style[name] = val + 'px';\n        }\n        else {\n            el.style[name] = val;\n        }\n    }\n\n    function pointInsideRect(point, rect) {\n        return point.left >= rect.left &&\n            point.left < rect.right &&\n            point.top >= rect.top &&\n            point.top < rect.bottom;\n    }\n    // Returns a new rectangle that is the intersection of the two rectangles. If they don't intersect, returns false\n    function intersectRects(rect1, rect2) {\n        var res = {\n            left: Math.max(rect1.left, rect2.left),\n            right: Math.min(rect1.right, rect2.right),\n            top: Math.max(rect1.top, rect2.top),\n            bottom: Math.min(rect1.bottom, rect2.bottom)\n        };\n        if (res.left < res.right && res.top < res.bottom) {\n            return res;\n        }\n        return false;\n    }\n    function translateRect(rect, deltaX, deltaY) {\n        return {\n            left: rect.left + deltaX,\n            right: rect.right + deltaX,\n            top: rect.top + deltaY,\n            bottom: rect.bottom + deltaY\n        };\n    }\n    // Returns a new point that will have been moved to reside within the given rectangle\n    function constrainPoint(point, rect) {\n        return {\n            left: Math.min(Math.max(point.left, rect.left), rect.right),\n            top: Math.min(Math.max(point.top, rect.top), rect.bottom)\n        };\n    }\n    // Returns a point that is the center of the given rectangle\n    function getRectCenter(rect) {\n        return {\n            left: (rect.left + rect.right) / 2,\n            top: (rect.top + rect.bottom) / 2\n        };\n    }\n    // Subtracts point2's coordinates from point1's coordinates, returning a delta\n    function diffPoints(point1, point2) {\n        return {\n            left: point1.left - point2.left,\n            top: point1.top - point2.top\n        };\n    }\n\n    // Logic for determining if, when the element is right-to-left, the scrollbar appears on the left side\n    var isRtlScrollbarOnLeft = null;\n    function getIsRtlScrollbarOnLeft() {\n        if (isRtlScrollbarOnLeft === null) {\n            isRtlScrollbarOnLeft = computeIsRtlScrollbarOnLeft();\n        }\n        return isRtlScrollbarOnLeft;\n    }\n    function computeIsRtlScrollbarOnLeft() {\n        var outerEl = createElement('div', {\n            style: {\n                position: 'absolute',\n                top: -1000,\n                left: 0,\n                border: 0,\n                padding: 0,\n                overflow: 'scroll',\n                direction: 'rtl'\n            }\n        }, '<div></div>');\n        document.body.appendChild(outerEl);\n        var innerEl = outerEl.firstChild;\n        var res = innerEl.getBoundingClientRect().left > outerEl.getBoundingClientRect().left;\n        removeElement(outerEl);\n        return res;\n    }\n    // The scrollbar width computations in computeEdges are sometimes flawed when it comes to\n    // retina displays, rounding, and IE11. Massage them into a usable value.\n    function sanitizeScrollbarWidth(width) {\n        width = Math.max(0, width); // no negatives\n        width = Math.round(width);\n        return width;\n    }\n\n    function computeEdges(el, getPadding) {\n        if (getPadding === void 0) { getPadding = false; }\n        var computedStyle = window.getComputedStyle(el);\n        var borderLeft = parseInt(computedStyle.borderLeftWidth, 10) || 0;\n        var borderRight = parseInt(computedStyle.borderRightWidth, 10) || 0;\n        var borderTop = parseInt(computedStyle.borderTopWidth, 10) || 0;\n        var borderBottom = parseInt(computedStyle.borderBottomWidth, 10) || 0;\n        // must use offset(Width|Height) because compatible with client(Width|Height)\n        var scrollbarLeftRight = sanitizeScrollbarWidth(el.offsetWidth - el.clientWidth - borderLeft - borderRight);\n        var scrollbarBottom = sanitizeScrollbarWidth(el.offsetHeight - el.clientHeight - borderTop - borderBottom);\n        var res = {\n            borderLeft: borderLeft,\n            borderRight: borderRight,\n            borderTop: borderTop,\n            borderBottom: borderBottom,\n            scrollbarBottom: scrollbarBottom,\n            scrollbarLeft: 0,\n            scrollbarRight: 0\n        };\n        if (getIsRtlScrollbarOnLeft() && computedStyle.direction === 'rtl') { // is the scrollbar on the left side?\n            res.scrollbarLeft = scrollbarLeftRight;\n        }\n        else {\n            res.scrollbarRight = scrollbarLeftRight;\n        }\n        if (getPadding) {\n            res.paddingLeft = parseInt(computedStyle.paddingLeft, 10) || 0;\n            res.paddingRight = parseInt(computedStyle.paddingRight, 10) || 0;\n            res.paddingTop = parseInt(computedStyle.paddingTop, 10) || 0;\n            res.paddingBottom = parseInt(computedStyle.paddingBottom, 10) || 0;\n        }\n        return res;\n    }\n    function computeInnerRect(el, goWithinPadding) {\n        if (goWithinPadding === void 0) { goWithinPadding = false; }\n        var outerRect = computeRect(el);\n        var edges = computeEdges(el, goWithinPadding);\n        var res = {\n            left: outerRect.left + edges.borderLeft + edges.scrollbarLeft,\n            right: outerRect.right - edges.borderRight - edges.scrollbarRight,\n            top: outerRect.top + edges.borderTop,\n            bottom: outerRect.bottom - edges.borderBottom - edges.scrollbarBottom\n        };\n        if (goWithinPadding) {\n            res.left += edges.paddingLeft;\n            res.right -= edges.paddingRight;\n            res.top += edges.paddingTop;\n            res.bottom -= edges.paddingBottom;\n        }\n        return res;\n    }\n    function computeRect(el) {\n        var rect = el.getBoundingClientRect();\n        return {\n            left: rect.left + window.pageXOffset,\n            top: rect.top + window.pageYOffset,\n            right: rect.right + window.pageXOffset,\n            bottom: rect.bottom + window.pageYOffset\n        };\n    }\n    function computeViewportRect() {\n        return {\n            left: window.pageXOffset,\n            right: window.pageXOffset + document.documentElement.clientWidth,\n            top: window.pageYOffset,\n            bottom: window.pageYOffset + document.documentElement.clientHeight\n        };\n    }\n    function computeHeightAndMargins(el) {\n        return el.getBoundingClientRect().height + computeVMargins(el);\n    }\n    function computeVMargins(el) {\n        var computed = window.getComputedStyle(el);\n        return parseInt(computed.marginTop, 10) +\n            parseInt(computed.marginBottom, 10);\n    }\n    // does not return window\n    function getClippingParents(el) {\n        var parents = [];\n        while (el instanceof HTMLElement) { // will stop when gets to document or null\n            var computedStyle = window.getComputedStyle(el);\n            if (computedStyle.position === 'fixed') {\n                break;\n            }\n            if ((/(auto|scroll)/).test(computedStyle.overflow + computedStyle.overflowY + computedStyle.overflowX)) {\n                parents.push(el);\n            }\n            el = el.parentNode;\n        }\n        return parents;\n    }\n    function computeClippingRect(el) {\n        return getClippingParents(el)\n            .map(function (el) {\n            return computeInnerRect(el);\n        })\n            .concat(computeViewportRect())\n            .reduce(function (rect0, rect1) {\n            return intersectRects(rect0, rect1) || rect1; // should always intersect\n        });\n    }\n\n    // Stops a mouse/touch event from doing it's native browser action\n    function preventDefault(ev) {\n        ev.preventDefault();\n    }\n    // Event Delegation\n    // ----------------------------------------------------------------------------------------------------------------\n    function listenBySelector(container, eventType, selector, handler) {\n        function realHandler(ev) {\n            var matchedChild = elementClosest(ev.target, selector);\n            if (matchedChild) {\n                handler.call(matchedChild, ev, matchedChild);\n            }\n        }\n        container.addEventListener(eventType, realHandler);\n        return function () {\n            container.removeEventListener(eventType, realHandler);\n        };\n    }\n    function listenToHoverBySelector(container, selector, onMouseEnter, onMouseLeave) {\n        var currentMatchedChild;\n        return listenBySelector(container, 'mouseover', selector, function (ev, matchedChild) {\n            if (matchedChild !== currentMatchedChild) {\n                currentMatchedChild = matchedChild;\n                onMouseEnter(ev, matchedChild);\n                var realOnMouseLeave_1 = function (ev) {\n                    currentMatchedChild = null;\n                    onMouseLeave(ev, matchedChild);\n                    matchedChild.removeEventListener('mouseleave', realOnMouseLeave_1);\n                };\n                // listen to the next mouseleave, and then unattach\n                matchedChild.addEventListener('mouseleave', realOnMouseLeave_1);\n            }\n        });\n    }\n    // Animation\n    // ----------------------------------------------------------------------------------------------------------------\n    var transitionEventNames = [\n        'webkitTransitionEnd',\n        'otransitionend',\n        'oTransitionEnd',\n        'msTransitionEnd',\n        'transitionend'\n    ];\n    // triggered only when the next single subsequent transition finishes\n    function whenTransitionDone(el, callback) {\n        var realCallback = function (ev) {\n            callback(ev);\n            transitionEventNames.forEach(function (eventName) {\n                el.removeEventListener(eventName, realCallback);\n            });\n        };\n        transitionEventNames.forEach(function (eventName) {\n            el.addEventListener(eventName, realCallback); // cross-browser way to determine when the transition finishes\n        });\n    }\n\n    var DAY_IDS = ['sun', 'mon', 'tue', 'wed', 'thu', 'fri', 'sat'];\n    // Adding\n    function addWeeks(m, n) {\n        var a = dateToUtcArray(m);\n        a[2] += n * 7;\n        return arrayToUtcDate(a);\n    }\n    function addDays(m, n) {\n        var a = dateToUtcArray(m);\n        a[2] += n;\n        return arrayToUtcDate(a);\n    }\n    function addMs(m, n) {\n        var a = dateToUtcArray(m);\n        a[6] += n;\n        return arrayToUtcDate(a);\n    }\n    // Diffing (all return floats)\n    function diffWeeks(m0, m1) {\n        return diffDays(m0, m1) / 7;\n    }\n    function diffDays(m0, m1) {\n        return (m1.valueOf() - m0.valueOf()) / (1000 * 60 * 60 * 24);\n    }\n    function diffHours(m0, m1) {\n        return (m1.valueOf() - m0.valueOf()) / (1000 * 60 * 60);\n    }\n    function diffMinutes(m0, m1) {\n        return (m1.valueOf() - m0.valueOf()) / (1000 * 60);\n    }\n    function diffSeconds(m0, m1) {\n        return (m1.valueOf() - m0.valueOf()) / 1000;\n    }\n    function diffDayAndTime(m0, m1) {\n        var m0day = startOfDay(m0);\n        var m1day = startOfDay(m1);\n        return {\n            years: 0,\n            months: 0,\n            days: Math.round(diffDays(m0day, m1day)),\n            milliseconds: (m1.valueOf() - m1day.valueOf()) - (m0.valueOf() - m0day.valueOf())\n        };\n    }\n    // Diffing Whole Units\n    function diffWholeWeeks(m0, m1) {\n        var d = diffWholeDays(m0, m1);\n        if (d !== null && d % 7 === 0) {\n            return d / 7;\n        }\n        return null;\n    }\n    function diffWholeDays(m0, m1) {\n        if (timeAsMs(m0) === timeAsMs(m1)) {\n            return Math.round(diffDays(m0, m1));\n        }\n        return null;\n    }\n    // Start-Of\n    function startOfDay(m) {\n        return arrayToUtcDate([\n            m.getUTCFullYear(),\n            m.getUTCMonth(),\n            m.getUTCDate()\n        ]);\n    }\n    function startOfHour(m) {\n        return arrayToUtcDate([\n            m.getUTCFullYear(),\n            m.getUTCMonth(),\n            m.getUTCDate(),\n            m.getUTCHours()\n        ]);\n    }\n    function startOfMinute(m) {\n        return arrayToUtcDate([\n            m.getUTCFullYear(),\n            m.getUTCMonth(),\n            m.getUTCDate(),\n            m.getUTCHours(),\n            m.getUTCMinutes()\n        ]);\n    }\n    function startOfSecond(m) {\n        return arrayToUtcDate([\n            m.getUTCFullYear(),\n            m.getUTCMonth(),\n            m.getUTCDate(),\n            m.getUTCHours(),\n            m.getUTCMinutes(),\n            m.getUTCSeconds()\n        ]);\n    }\n    // Week Computation\n    function weekOfYear(marker, dow, doy) {\n        var y = marker.getUTCFullYear();\n        var w = weekOfGivenYear(marker, y, dow, doy);\n        if (w < 1) {\n            return weekOfGivenYear(marker, y - 1, dow, doy);\n        }\n        var nextW = weekOfGivenYear(marker, y + 1, dow, doy);\n        if (nextW >= 1) {\n            return Math.min(w, nextW);\n        }\n        return w;\n    }\n    function weekOfGivenYear(marker, year, dow, doy) {\n        var firstWeekStart = arrayToUtcDate([year, 0, 1 + firstWeekOffset(year, dow, doy)]);\n        var dayStart = startOfDay(marker);\n        var days = Math.round(diffDays(firstWeekStart, dayStart));\n        return Math.floor(days / 7) + 1; // zero-indexed\n    }\n    // start-of-first-week - start-of-year\n    function firstWeekOffset(year, dow, doy) {\n        // first-week day -- which january is always in the first week (4 for iso, 1 for other)\n        var fwd = 7 + dow - doy;\n        // first-week day local weekday -- which local weekday is fwd\n        var fwdlw = (7 + arrayToUtcDate([year, 0, fwd]).getUTCDay() - dow) % 7;\n        return -fwdlw + fwd - 1;\n    }\n    // Array Conversion\n    function dateToLocalArray(date) {\n        return [\n            date.getFullYear(),\n            date.getMonth(),\n            date.getDate(),\n            date.getHours(),\n            date.getMinutes(),\n            date.getSeconds(),\n            date.getMilliseconds()\n        ];\n    }\n    function arrayToLocalDate(a) {\n        return new Date(a[0], a[1] || 0, a[2] == null ? 1 : a[2], // day of month\n        a[3] || 0, a[4] || 0, a[5] || 0);\n    }\n    function dateToUtcArray(date) {\n        return [\n            date.getUTCFullYear(),\n            date.getUTCMonth(),\n            date.getUTCDate(),\n            date.getUTCHours(),\n            date.getUTCMinutes(),\n            date.getUTCSeconds(),\n            date.getUTCMilliseconds()\n        ];\n    }\n    function arrayToUtcDate(a) {\n        // according to web standards (and Safari), a month index is required.\n        // massage if only given a year.\n        if (a.length === 1) {\n            a = a.concat([0]);\n        }\n        return new Date(Date.UTC.apply(Date, a));\n    }\n    // Other Utils\n    function isValidDate(m) {\n        return !isNaN(m.valueOf());\n    }\n    function timeAsMs(m) {\n        return m.getUTCHours() * 1000 * 60 * 60 +\n            m.getUTCMinutes() * 1000 * 60 +\n            m.getUTCSeconds() * 1000 +\n            m.getUTCMilliseconds();\n    }\n\n    var INTERNAL_UNITS = ['years', 'months', 'days', 'milliseconds'];\n    var PARSE_RE = /^(-?)(?:(\\d+)\\.)?(\\d+):(\\d\\d)(?::(\\d\\d)(?:\\.(\\d\\d\\d))?)?/;\n    // Parsing and Creation\n    function createDuration(input, unit) {\n        var _a;\n        if (typeof input === 'string') {\n            return parseString(input);\n        }\n        else if (typeof input === 'object' && input) { // non-null object\n            return normalizeObject(input);\n        }\n        else if (typeof input === 'number') {\n            return normalizeObject((_a = {}, _a[unit || 'milliseconds'] = input, _a));\n        }\n        else {\n            return null;\n        }\n    }\n    function parseString(s) {\n        var m = PARSE_RE.exec(s);\n        if (m) {\n            var sign = m[1] ? -1 : 1;\n            return {\n                years: 0,\n                months: 0,\n                days: sign * (m[2] ? parseInt(m[2], 10) : 0),\n                milliseconds: sign * ((m[3] ? parseInt(m[3], 10) : 0) * 60 * 60 * 1000 + // hours\n                    (m[4] ? parseInt(m[4], 10) : 0) * 60 * 1000 + // minutes\n                    (m[5] ? parseInt(m[5], 10) : 0) * 1000 + // seconds\n                    (m[6] ? parseInt(m[6], 10) : 0) // ms\n                )\n            };\n        }\n        return null;\n    }\n    function normalizeObject(obj) {\n        return {\n            years: obj.years || obj.year || 0,\n            months: obj.months || obj.month || 0,\n            days: (obj.days || obj.day || 0) +\n                getWeeksFromInput(obj) * 7,\n            milliseconds: (obj.hours || obj.hour || 0) * 60 * 60 * 1000 + // hours\n                (obj.minutes || obj.minute || 0) * 60 * 1000 + // minutes\n                (obj.seconds || obj.second || 0) * 1000 + // seconds\n                (obj.milliseconds || obj.millisecond || obj.ms || 0) // ms\n        };\n    }\n    function getWeeksFromInput(obj) {\n        return obj.weeks || obj.week || 0;\n    }\n    // Equality\n    function durationsEqual(d0, d1) {\n        return d0.years === d1.years &&\n            d0.months === d1.months &&\n            d0.days === d1.days &&\n            d0.milliseconds === d1.milliseconds;\n    }\n    function isSingleDay(dur) {\n        return dur.years === 0 && dur.months === 0 && dur.days === 1 && dur.milliseconds === 0;\n    }\n    // Simple Math\n    function addDurations(d0, d1) {\n        return {\n            years: d0.years + d1.years,\n            months: d0.months + d1.months,\n            days: d0.days + d1.days,\n            milliseconds: d0.milliseconds + d1.milliseconds\n        };\n    }\n    function subtractDurations(d1, d0) {\n        return {\n            years: d1.years - d0.years,\n            months: d1.months - d0.months,\n            days: d1.days - d0.days,\n            milliseconds: d1.milliseconds - d0.milliseconds\n        };\n    }\n    function multiplyDuration(d, n) {\n        return {\n            years: d.years * n,\n            months: d.months * n,\n            days: d.days * n,\n            milliseconds: d.milliseconds * n\n        };\n    }\n    // Conversions\n    // \"Rough\" because they are based on average-case Gregorian months/years\n    function asRoughYears(dur) {\n        return asRoughDays(dur) / 365;\n    }\n    function asRoughMonths(dur) {\n        return asRoughDays(dur) / 30;\n    }\n    function asRoughDays(dur) {\n        return asRoughMs(dur) / 864e5;\n    }\n    function asRoughMinutes(dur) {\n        return asRoughMs(dur) / (1000 * 60);\n    }\n    function asRoughSeconds(dur) {\n        return asRoughMs(dur) / 1000;\n    }\n    function asRoughMs(dur) {\n        return dur.years * (365 * 864e5) +\n            dur.months * (30 * 864e5) +\n            dur.days * 864e5 +\n            dur.milliseconds;\n    }\n    // Advanced Math\n    function wholeDivideDurations(numerator, denominator) {\n        var res = null;\n        for (var i = 0; i < INTERNAL_UNITS.length; i++) {\n            var unit = INTERNAL_UNITS[i];\n            if (denominator[unit]) {\n                var localRes = numerator[unit] / denominator[unit];\n                if (!isInt(localRes) || (res !== null && res !== localRes)) {\n                    return null;\n                }\n                res = localRes;\n            }\n            else if (numerator[unit]) {\n                // needs to divide by something but can't!\n                return null;\n            }\n        }\n        return res;\n    }\n    function greatestDurationDenominator(dur, dontReturnWeeks) {\n        var ms = dur.milliseconds;\n        if (ms) {\n            if (ms % 1000 !== 0) {\n                return { unit: 'millisecond', value: ms };\n            }\n            if (ms % (1000 * 60) !== 0) {\n                return { unit: 'second', value: ms / 1000 };\n            }\n            if (ms % (1000 * 60 * 60) !== 0) {\n                return { unit: 'minute', value: ms / (1000 * 60) };\n            }\n            if (ms) {\n                return { unit: 'hour', value: ms / (1000 * 60 * 60) };\n            }\n        }\n        if (dur.days) {\n            if (!dontReturnWeeks && dur.days % 7 === 0) {\n                return { unit: 'week', value: dur.days / 7 };\n            }\n            return { unit: 'day', value: dur.days };\n        }\n        if (dur.months) {\n            return { unit: 'month', value: dur.months };\n        }\n        if (dur.years) {\n            return { unit: 'year', value: dur.years };\n        }\n        return { unit: 'millisecond', value: 0 };\n    }\n\n    /* FullCalendar-specific DOM Utilities\n    ----------------------------------------------------------------------------------------------------------------------*/\n    // Given the scrollbar widths of some other container, create borders/margins on rowEls in order to match the left\n    // and right space that was offset by the scrollbars. A 1-pixel border first, then margin beyond that.\n    function compensateScroll(rowEl, scrollbarWidths) {\n        if (scrollbarWidths.left) {\n            applyStyle(rowEl, {\n                borderLeftWidth: 1,\n                marginLeft: scrollbarWidths.left - 1\n            });\n        }\n        if (scrollbarWidths.right) {\n            applyStyle(rowEl, {\n                borderRightWidth: 1,\n                marginRight: scrollbarWidths.right - 1\n            });\n        }\n    }\n    // Undoes compensateScroll and restores all borders/margins\n    function uncompensateScroll(rowEl) {\n        applyStyle(rowEl, {\n            marginLeft: '',\n            marginRight: '',\n            borderLeftWidth: '',\n            borderRightWidth: ''\n        });\n    }\n    // Make the mouse cursor express that an event is not allowed in the current area\n    function disableCursor() {\n        document.body.classList.add('fc-not-allowed');\n    }\n    // Returns the mouse cursor to its original look\n    function enableCursor() {\n        document.body.classList.remove('fc-not-allowed');\n    }\n    // Given a total available height to fill, have `els` (essentially child rows) expand to accomodate.\n    // By default, all elements that are shorter than the recommended height are expanded uniformly, not considering\n    // any other els that are already too tall. if `shouldRedistribute` is on, it considers these tall rows and\n    // reduces the available height.\n    function distributeHeight(els, availableHeight, shouldRedistribute) {\n        // *FLOORING NOTE*: we floor in certain places because zoom can give inaccurate floating-point dimensions,\n        // and it is better to be shorter than taller, to avoid creating unnecessary scrollbars.\n        var minOffset1 = Math.floor(availableHeight / els.length); // for non-last element\n        var minOffset2 = Math.floor(availableHeight - minOffset1 * (els.length - 1)); // for last element *FLOORING NOTE*\n        var flexEls = []; // elements that are allowed to expand. array of DOM nodes\n        var flexOffsets = []; // amount of vertical space it takes up\n        var flexHeights = []; // actual css height\n        var usedHeight = 0;\n        undistributeHeight(els); // give all elements their natural height\n        // find elements that are below the recommended height (expandable).\n        // important to query for heights in a single first pass (to avoid reflow oscillation).\n        els.forEach(function (el, i) {\n            var minOffset = i === els.length - 1 ? minOffset2 : minOffset1;\n            var naturalHeight = el.getBoundingClientRect().height;\n            var naturalOffset = naturalHeight + computeVMargins(el);\n            if (naturalOffset < minOffset) {\n                flexEls.push(el);\n                flexOffsets.push(naturalOffset);\n                flexHeights.push(naturalHeight);\n            }\n            else {\n                // this element stretches past recommended height (non-expandable). mark the space as occupied.\n                usedHeight += naturalOffset;\n            }\n        });\n        // readjust the recommended height to only consider the height available to non-maxed-out rows.\n        if (shouldRedistribute) {\n            availableHeight -= usedHeight;\n            minOffset1 = Math.floor(availableHeight / flexEls.length);\n            minOffset2 = Math.floor(availableHeight - minOffset1 * (flexEls.length - 1)); // *FLOORING NOTE*\n        }\n        // assign heights to all expandable elements\n        flexEls.forEach(function (el, i) {\n            var minOffset = i === flexEls.length - 1 ? minOffset2 : minOffset1;\n            var naturalOffset = flexOffsets[i];\n            var naturalHeight = flexHeights[i];\n            var newHeight = minOffset - (naturalOffset - naturalHeight); // subtract the margin/padding\n            if (naturalOffset < minOffset) { // we check this again because redistribution might have changed things\n                el.style.height = newHeight + 'px';\n            }\n        });\n    }\n    // Undoes distrubuteHeight, restoring all els to their natural height\n    function undistributeHeight(els) {\n        els.forEach(function (el) {\n            el.style.height = '';\n        });\n    }\n    // Given `els`, a set of <td> cells, find the cell with the largest natural width and set the widths of all the\n    // cells to be that width.\n    // PREREQUISITE: if you want a cell to take up width, it needs to have a single inner element w/ display:inline\n    function matchCellWidths(els) {\n        var maxInnerWidth = 0;\n        els.forEach(function (el) {\n            var innerEl = el.firstChild; // hopefully an element\n            if (innerEl instanceof HTMLElement) {\n                var innerWidth_1 = innerEl.getBoundingClientRect().width;\n                if (innerWidth_1 > maxInnerWidth) {\n                    maxInnerWidth = innerWidth_1;\n                }\n            }\n        });\n        maxInnerWidth++; // sometimes not accurate of width the text needs to stay on one line. insurance\n        els.forEach(function (el) {\n            el.style.width = maxInnerWidth + 'px';\n        });\n        return maxInnerWidth;\n    }\n    // Given one element that resides inside another,\n    // Subtracts the height of the inner element from the outer element.\n    function subtractInnerElHeight(outerEl, innerEl) {\n        // effin' IE8/9/10/11 sometimes returns 0 for dimensions. this weird hack was the only thing that worked\n        var reflowStyleProps = {\n            position: 'relative',\n            left: -1 // ensure reflow in case the el was already relative. negative is less likely to cause new scroll\n        };\n        applyStyle(outerEl, reflowStyleProps);\n        applyStyle(innerEl, reflowStyleProps);\n        var diff = // grab the dimensions\n         outerEl.getBoundingClientRect().height -\n            innerEl.getBoundingClientRect().height;\n        // undo hack\n        var resetStyleProps = { position: '', left: '' };\n        applyStyle(outerEl, resetStyleProps);\n        applyStyle(innerEl, resetStyleProps);\n        return diff;\n    }\n    /* Selection\n    ----------------------------------------------------------------------------------------------------------------------*/\n    function preventSelection(el) {\n        el.classList.add('fc-unselectable');\n        el.addEventListener('selectstart', preventDefault);\n    }\n    function allowSelection(el) {\n        el.classList.remove('fc-unselectable');\n        el.removeEventListener('selectstart', preventDefault);\n    }\n    /* Context Menu\n    ----------------------------------------------------------------------------------------------------------------------*/\n    function preventContextMenu(el) {\n        el.addEventListener('contextmenu', preventDefault);\n    }\n    function allowContextMenu(el) {\n        el.removeEventListener('contextmenu', preventDefault);\n    }\n    /* Object Ordering by Field\n    ----------------------------------------------------------------------------------------------------------------------*/\n    function parseFieldSpecs(input) {\n        var specs = [];\n        var tokens = [];\n        var i;\n        var token;\n        if (typeof input === 'string') {\n            tokens = input.split(/\\s*,\\s*/);\n        }\n        else if (typeof input === 'function') {\n            tokens = [input];\n        }\n        else if (Array.isArray(input)) {\n            tokens = input;\n        }\n        for (i = 0; i < tokens.length; i++) {\n            token = tokens[i];\n            if (typeof token === 'string') {\n                specs.push(token.charAt(0) === '-' ?\n                    { field: token.substring(1), order: -1 } :\n                    { field: token, order: 1 });\n            }\n            else if (typeof token === 'function') {\n                specs.push({ func: token });\n            }\n        }\n        return specs;\n    }\n    function compareByFieldSpecs(obj0, obj1, fieldSpecs) {\n        var i;\n        var cmp;\n        for (i = 0; i < fieldSpecs.length; i++) {\n            cmp = compareByFieldSpec(obj0, obj1, fieldSpecs[i]);\n            if (cmp) {\n                return cmp;\n            }\n        }\n        return 0;\n    }\n    function compareByFieldSpec(obj0, obj1, fieldSpec) {\n        if (fieldSpec.func) {\n            return fieldSpec.func(obj0, obj1);\n        }\n        return flexibleCompare(obj0[fieldSpec.field], obj1[fieldSpec.field])\n            * (fieldSpec.order || 1);\n    }\n    function flexibleCompare(a, b) {\n        if (!a && !b) {\n            return 0;\n        }\n        if (b == null) {\n            return -1;\n        }\n        if (a == null) {\n            return 1;\n        }\n        if (typeof a === 'string' || typeof b === 'string') {\n            return String(a).localeCompare(String(b));\n        }\n        return a - b;\n    }\n    /* String Utilities\n    ----------------------------------------------------------------------------------------------------------------------*/\n    function capitaliseFirstLetter(str) {\n        return str.charAt(0).toUpperCase() + str.slice(1);\n    }\n    function padStart(val, len) {\n        var s = String(val);\n        return '000'.substr(0, len - s.length) + s;\n    }\n    /* Number Utilities\n    ----------------------------------------------------------------------------------------------------------------------*/\n    function compareNumbers(a, b) {\n        return a - b;\n    }\n    function isInt(n) {\n        return n % 1 === 0;\n    }\n    /* Weird Utilities\n    ----------------------------------------------------------------------------------------------------------------------*/\n    function applyAll(functions, thisObj, args) {\n        if (typeof functions === 'function') { // supplied a single function\n            functions = [functions];\n        }\n        if (functions) {\n            var i = void 0;\n            var ret = void 0;\n            for (i = 0; i < functions.length; i++) {\n                ret = functions[i].apply(thisObj, args) || ret;\n            }\n            return ret;\n        }\n    }\n    function firstDefined() {\n        var args = [];\n        for (var _i = 0; _i < arguments.length; _i++) {\n            args[_i] = arguments[_i];\n        }\n        for (var i = 0; i < args.length; i++) {\n            if (args[i] !== undefined) {\n                return args[i];\n            }\n        }\n    }\n    // Returns a function, that, as long as it continues to be invoked, will not\n    // be triggered. The function will be called after it stops being called for\n    // N milliseconds. If `immediate` is passed, trigger the function on the\n    // leading edge, instead of the trailing.\n    // https://github.com/jashkenas/underscore/blob/1.6.0/underscore.js#L714\n    function debounce(func, wait) {\n        var timeout;\n        var args;\n        var context;\n        var timestamp;\n        var result;\n        var later = function () {\n            var last = new Date().valueOf() - timestamp;\n            if (last < wait) {\n                timeout = setTimeout(later, wait - last);\n            }\n            else {\n                timeout = null;\n                result = func.apply(context, args);\n                context = args = null;\n            }\n        };\n        return function () {\n            context = this;\n            args = arguments;\n            timestamp = new Date().valueOf();\n            if (!timeout) {\n                timeout = setTimeout(later, wait);\n            }\n            return result;\n        };\n    }\n    // Number and Boolean are only types that defaults or not computed for\n    // TODO: write more comments\n    function refineProps(rawProps, processors, defaults, leftoverProps) {\n        if (defaults === void 0) { defaults = {}; }\n        var refined = {};\n        for (var key in processors) {\n            var processor = processors[key];\n            if (rawProps[key] !== undefined) {\n                // found\n                if (processor === Function) {\n                    refined[key] = typeof rawProps[key] === 'function' ? rawProps[key] : null;\n                }\n                else if (processor) { // a refining function?\n                    refined[key] = processor(rawProps[key]);\n                }\n                else {\n                    refined[key] = rawProps[key];\n                }\n            }\n            else if (defaults[key] !== undefined) {\n                // there's an explicit default\n                refined[key] = defaults[key];\n            }\n            else {\n                // must compute a default\n                if (processor === String) {\n                    refined[key] = ''; // empty string is default for String\n                }\n                else if (!processor || processor === Number || processor === Boolean || processor === Function) {\n                    refined[key] = null; // assign null for other non-custom processor funcs\n                }\n                else {\n                    refined[key] = processor(null); // run the custom processor func\n                }\n            }\n        }\n        if (leftoverProps) {\n            for (var key in rawProps) {\n                if (processors[key] === undefined) {\n                    leftoverProps[key] = rawProps[key];\n                }\n            }\n        }\n        return refined;\n    }\n    /* Date stuff that doesn't belong in datelib core\n    ----------------------------------------------------------------------------------------------------------------------*/\n    // given a timed range, computes an all-day range that has the same exact duration,\n    // but whose start time is aligned with the start of the day.\n    function computeAlignedDayRange(timedRange) {\n        var dayCnt = Math.floor(diffDays(timedRange.start, timedRange.end)) || 1;\n        var start = startOfDay(timedRange.start);\n        var end = addDays(start, dayCnt);\n        return { start: start, end: end };\n    }\n    // given a timed range, computes an all-day range based on how for the end date bleeds into the next day\n    // TODO: give nextDayThreshold a default arg\n    function computeVisibleDayRange(timedRange, nextDayThreshold) {\n        if (nextDayThreshold === void 0) { nextDayThreshold = createDuration(0); }\n        var startDay = null;\n        var endDay = null;\n        if (timedRange.end) {\n            endDay = startOfDay(timedRange.end);\n            var endTimeMS = timedRange.end.valueOf() - endDay.valueOf(); // # of milliseconds into `endDay`\n            // If the end time is actually inclusively part of the next day and is equal to or\n            // beyond the next day threshold, adjust the end to be the exclusive end of `endDay`.\n            // Otherwise, leaving it as inclusive will cause it to exclude `endDay`.\n            if (endTimeMS && endTimeMS >= asRoughMs(nextDayThreshold)) {\n                endDay = addDays(endDay, 1);\n            }\n        }\n        if (timedRange.start) {\n            startDay = startOfDay(timedRange.start); // the beginning of the day the range starts\n            // If end is within `startDay` but not past nextDayThreshold, assign the default duration of one day.\n            if (endDay && endDay <= startDay) {\n                endDay = addDays(startDay, 1);\n            }\n        }\n        return { start: startDay, end: endDay };\n    }\n    // spans from one day into another?\n    function isMultiDayRange(range) {\n        var visibleRange = computeVisibleDayRange(range);\n        return diffDays(visibleRange.start, visibleRange.end) > 1;\n    }\n    function diffDates(date0, date1, dateEnv, largeUnit) {\n        if (largeUnit === 'year') {\n            return createDuration(dateEnv.diffWholeYears(date0, date1), 'year');\n        }\n        else if (largeUnit === 'month') {\n            return createDuration(dateEnv.diffWholeMonths(date0, date1), 'month');\n        }\n        else {\n            return diffDayAndTime(date0, date1); // returns a duration\n        }\n    }\n\n    /*! *****************************************************************************\r\n    Copyright (c) Microsoft Corporation.\r\n\r\n    Permission to use, copy, modify, and/or distribute this software for any\r\n    purpose with or without fee is hereby granted.\r\n\r\n    THE SOFTWARE IS PROVIDED \"AS IS\" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH\r\n    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY\r\n    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,\r\n    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM\r\n    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR\r\n    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR\r\n    PERFORMANCE OF THIS SOFTWARE.\r\n    ***************************************************************************** */\r\n    /* global Reflect, Promise */\r\n\r\n    var extendStatics = function(d, b) {\r\n        extendStatics = Object.setPrototypeOf ||\r\n            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||\r\n            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };\r\n        return extendStatics(d, b);\r\n    };\r\n\r\n    function __extends(d, b) {\r\n        extendStatics(d, b);\r\n        function __() { this.constructor = d; }\r\n        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());\r\n    }\r\n\r\n    var __assign = function() {\r\n        __assign = Object.assign || function __assign(t) {\r\n            for (var s, i = 1, n = arguments.length; i < n; i++) {\r\n                s = arguments[i];\r\n                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];\r\n            }\r\n            return t;\r\n        };\r\n        return __assign.apply(this, arguments);\r\n    };\n\n    function parseRecurring(eventInput, allDayDefault, dateEnv, recurringTypes, leftovers) {\n        for (var i = 0; i < recurringTypes.length; i++) {\n            var localLeftovers = {};\n            var parsed = recurringTypes[i].parse(eventInput, localLeftovers, dateEnv);\n            if (parsed) {\n                var allDay = localLeftovers.allDay;\n                delete localLeftovers.allDay; // remove from leftovers\n                if (allDay == null) {\n                    allDay = allDayDefault;\n                    if (allDay == null) {\n                        allDay = parsed.allDayGuess;\n                        if (allDay == null) {\n                            allDay = false;\n                        }\n                    }\n                }\n                __assign(leftovers, localLeftovers);\n                return {\n                    allDay: allDay,\n                    duration: parsed.duration,\n                    typeData: parsed.typeData,\n                    typeId: i\n                };\n            }\n        }\n        return null;\n    }\n    /*\n    Event MUST have a recurringDef\n    */\n    function expandRecurringRanges(eventDef, duration, framingRange, dateEnv, recurringTypes) {\n        var typeDef = recurringTypes[eventDef.recurringDef.typeId];\n        var markers = typeDef.expand(eventDef.recurringDef.typeData, {\n            start: dateEnv.subtract(framingRange.start, duration),\n            end: framingRange.end\n        }, dateEnv);\n        // the recurrence plugins don't guarantee that all-day events are start-of-day, so we have to\n        if (eventDef.allDay) {\n            markers = markers.map(startOfDay);\n        }\n        return markers;\n    }\n\n    var hasOwnProperty = Object.prototype.hasOwnProperty;\n    // Merges an array of objects into a single object.\n    // The second argument allows for an array of property names who's object values will be merged together.\n    function mergeProps(propObjs, complexProps) {\n        var dest = {};\n        var i;\n        var name;\n        var complexObjs;\n        var j;\n        var val;\n        var props;\n        if (complexProps) {\n            for (i = 0; i < complexProps.length; i++) {\n                name = complexProps[i];\n                complexObjs = [];\n                // collect the trailing object values, stopping when a non-object is discovered\n                for (j = propObjs.length - 1; j >= 0; j--) {\n                    val = propObjs[j][name];\n                    if (typeof val === 'object' && val) { // non-null object\n                        complexObjs.unshift(val);\n                    }\n                    else if (val !== undefined) {\n                        dest[name] = val; // if there were no objects, this value will be used\n                        break;\n                    }\n                }\n                // if the trailing values were objects, use the merged value\n                if (complexObjs.length) {\n                    dest[name] = mergeProps(complexObjs);\n                }\n            }\n        }\n        // copy values into the destination, going from last to first\n        for (i = propObjs.length - 1; i >= 0; i--) {\n            props = propObjs[i];\n            for (name in props) {\n                if (!(name in dest)) { // if already assigned by previous props or complex props, don't reassign\n                    dest[name] = props[name];\n                }\n            }\n        }\n        return dest;\n    }\n    function filterHash(hash, func) {\n        var filtered = {};\n        for (var key in hash) {\n            if (func(hash[key], key)) {\n                filtered[key] = hash[key];\n            }\n        }\n        return filtered;\n    }\n    function mapHash(hash, func) {\n        var newHash = {};\n        for (var key in hash) {\n            newHash[key] = func(hash[key], key);\n        }\n        return newHash;\n    }\n    function arrayToHash(a) {\n        var hash = {};\n        for (var _i = 0, a_1 = a; _i < a_1.length; _i++) {\n            var item = a_1[_i];\n            hash[item] = true;\n        }\n        return hash;\n    }\n    function hashValuesToArray(obj) {\n        var a = [];\n        for (var key in obj) {\n            a.push(obj[key]);\n        }\n        return a;\n    }\n    function isPropsEqual(obj0, obj1) {\n        for (var key in obj0) {\n            if (hasOwnProperty.call(obj0, key)) {\n                if (!(key in obj1)) {\n                    return false;\n                }\n            }\n        }\n        for (var key in obj1) {\n            if (hasOwnProperty.call(obj1, key)) {\n                if (obj0[key] !== obj1[key]) {\n                    return false;\n                }\n            }\n        }\n        return true;\n    }\n\n    function parseEvents(rawEvents, sourceId, calendar, allowOpenRange) {\n        var eventStore = createEmptyEventStore();\n        for (var _i = 0, rawEvents_1 = rawEvents; _i < rawEvents_1.length; _i++) {\n            var rawEvent = rawEvents_1[_i];\n            var tuple = parseEvent(rawEvent, sourceId, calendar, allowOpenRange);\n            if (tuple) {\n                eventTupleToStore(tuple, eventStore);\n            }\n        }\n        return eventStore;\n    }\n    function eventTupleToStore(tuple, eventStore) {\n        if (eventStore === void 0) { eventStore = createEmptyEventStore(); }\n        eventStore.defs[tuple.def.defId] = tuple.def;\n        if (tuple.instance) {\n            eventStore.instances[tuple.instance.instanceId] = tuple.instance;\n        }\n        return eventStore;\n    }\n    function expandRecurring(eventStore, framingRange, calendar) {\n        var dateEnv = calendar.dateEnv;\n        var defs = eventStore.defs, instances = eventStore.instances;\n        // remove existing recurring instances\n        instances = filterHash(instances, function (instance) {\n            return !defs[instance.defId].recurringDef;\n        });\n        for (var defId in defs) {\n            var def = defs[defId];\n            if (def.recurringDef) {\n                var duration = def.recurringDef.duration;\n                if (!duration) {\n                    duration = def.allDay ?\n                        calendar.defaultAllDayEventDuration :\n                        calendar.defaultTimedEventDuration;\n                }\n                var starts = expandRecurringRanges(def, duration, framingRange, calendar.dateEnv, calendar.pluginSystem.hooks.recurringTypes);\n                for (var _i = 0, starts_1 = starts; _i < starts_1.length; _i++) {\n                    var start = starts_1[_i];\n                    var instance = createEventInstance(defId, {\n                        start: start,\n                        end: dateEnv.add(start, duration)\n                    });\n                    instances[instance.instanceId] = instance;\n                }\n            }\n        }\n        return { defs: defs, instances: instances };\n    }\n    // retrieves events that have the same groupId as the instance specified by `instanceId`\n    // or they are the same as the instance.\n    // why might instanceId not be in the store? an event from another calendar?\n    function getRelevantEvents(eventStore, instanceId) {\n        var instance = eventStore.instances[instanceId];\n        if (instance) {\n            var def_1 = eventStore.defs[instance.defId];\n            // get events/instances with same group\n            var newStore = filterEventStoreDefs(eventStore, function (lookDef) {\n                return isEventDefsGrouped(def_1, lookDef);\n            });\n            // add the original\n            // TODO: wish we could use eventTupleToStore or something like it\n            newStore.defs[def_1.defId] = def_1;\n            newStore.instances[instance.instanceId] = instance;\n            return newStore;\n        }\n        return createEmptyEventStore();\n    }\n    function isEventDefsGrouped(def0, def1) {\n        return Boolean(def0.groupId && def0.groupId === def1.groupId);\n    }\n    function transformRawEvents(rawEvents, eventSource, calendar) {\n        var calEachTransform = calendar.opt('eventDataTransform');\n        var sourceEachTransform = eventSource ? eventSource.eventDataTransform : null;\n        if (sourceEachTransform) {\n            rawEvents = transformEachRawEvent(rawEvents, sourceEachTransform);\n        }\n        if (calEachTransform) {\n            rawEvents = transformEachRawEvent(rawEvents, calEachTransform);\n        }\n        return rawEvents;\n    }\n    function transformEachRawEvent(rawEvents, func) {\n        var refinedEvents;\n        if (!func) {\n            refinedEvents = rawEvents;\n        }\n        else {\n            refinedEvents = [];\n            for (var _i = 0, rawEvents_2 = rawEvents; _i < rawEvents_2.length; _i++) {\n                var rawEvent = rawEvents_2[_i];\n                var refinedEvent = func(rawEvent);\n                if (refinedEvent) {\n                    refinedEvents.push(refinedEvent);\n                }\n                else if (refinedEvent == null) {\n                    refinedEvents.push(rawEvent);\n                } // if a different falsy value, do nothing\n            }\n        }\n        return refinedEvents;\n    }\n    function createEmptyEventStore() {\n        return { defs: {}, instances: {} };\n    }\n    function mergeEventStores(store0, store1) {\n        return {\n            defs: __assign({}, store0.defs, store1.defs),\n            instances: __assign({}, store0.instances, store1.instances)\n        };\n    }\n    function filterEventStoreDefs(eventStore, filterFunc) {\n        var defs = filterHash(eventStore.defs, filterFunc);\n        var instances = filterHash(eventStore.instances, function (instance) {\n            return defs[instance.defId]; // still exists?\n        });\n        return { defs: defs, instances: instances };\n    }\n\n    function parseRange(input, dateEnv) {\n        var start = null;\n        var end = null;\n        if (input.start) {\n            start = dateEnv.createMarker(input.start);\n        }\n        if (input.end) {\n            end = dateEnv.createMarker(input.end);\n        }\n        if (!start && !end) {\n            return null;\n        }\n        if (start && end && end < start) {\n            return null;\n        }\n        return { start: start, end: end };\n    }\n    // SIDE-EFFECT: will mutate ranges.\n    // Will return a new array result.\n    function invertRanges(ranges, constraintRange) {\n        var invertedRanges = [];\n        var start = constraintRange.start; // the end of the previous range. the start of the new range\n        var i;\n        var dateRange;\n        // ranges need to be in order. required for our date-walking algorithm\n        ranges.sort(compareRanges);\n        for (i = 0; i < ranges.length; i++) {\n            dateRange = ranges[i];\n            // add the span of time before the event (if there is any)\n            if (dateRange.start > start) { // compare millisecond time (skip any ambig logic)\n                invertedRanges.push({ start: start, end: dateRange.start });\n            }\n            if (dateRange.end > start) {\n                start = dateRange.end;\n            }\n        }\n        // add the span of time after the last event (if there is any)\n        if (start < constraintRange.end) { // compare millisecond time (skip any ambig logic)\n            invertedRanges.push({ start: start, end: constraintRange.end });\n        }\n        return invertedRanges;\n    }\n    function compareRanges(range0, range1) {\n        return range0.start.valueOf() - range1.start.valueOf(); // earlier ranges go first\n    }\n    function intersectRanges(range0, range1) {\n        var start = range0.start;\n        var end = range0.end;\n        var newRange = null;\n        if (range1.start !== null) {\n            if (start === null) {\n                start = range1.start;\n            }\n            else {\n                start = new Date(Math.max(start.valueOf(), range1.start.valueOf()));\n            }\n        }\n        if (range1.end != null) {\n            if (end === null) {\n                end = range1.end;\n            }\n            else {\n                end = new Date(Math.min(end.valueOf(), range1.end.valueOf()));\n            }\n        }\n        if (start === null || end === null || start < end) {\n            newRange = { start: start, end: end };\n        }\n        return newRange;\n    }\n    function rangesEqual(range0, range1) {\n        return (range0.start === null ? null : range0.start.valueOf()) === (range1.start === null ? null : range1.start.valueOf()) &&\n            (range0.end === null ? null : range0.end.valueOf()) === (range1.end === null ? null : range1.end.valueOf());\n    }\n    function rangesIntersect(range0, range1) {\n        return (range0.end === null || range1.start === null || range0.end > range1.start) &&\n            (range0.start === null || range1.end === null || range0.start < range1.end);\n    }\n    function rangeContainsRange(outerRange, innerRange) {\n        return (outerRange.start === null || (innerRange.start !== null && innerRange.start >= outerRange.start)) &&\n            (outerRange.end === null || (innerRange.end !== null && innerRange.end <= outerRange.end));\n    }\n    function rangeContainsMarker(range, date) {\n        return (range.start === null || date >= range.start) &&\n            (range.end === null || date < range.end);\n    }\n    // If the given date is not within the given range, move it inside.\n    // (If it's past the end, make it one millisecond before the end).\n    function constrainMarkerToRange(date, range) {\n        if (range.start != null && date < range.start) {\n            return range.start;\n        }\n        if (range.end != null && date >= range.end) {\n            return new Date(range.end.valueOf() - 1);\n        }\n        return date;\n    }\n\n    function removeExact(array, exactVal) {\n        var removeCnt = 0;\n        var i = 0;\n        while (i < array.length) {\n            if (array[i] === exactVal) {\n                array.splice(i, 1);\n                removeCnt++;\n            }\n            else {\n                i++;\n            }\n        }\n        return removeCnt;\n    }\n    function isArraysEqual(a0, a1) {\n        var len = a0.length;\n        var i;\n        if (len !== a1.length) { // not array? or not same length?\n            return false;\n        }\n        for (i = 0; i < len; i++) {\n            if (a0[i] !== a1[i]) {\n                return false;\n            }\n        }\n        return true;\n    }\n\n    function memoize(workerFunc) {\n        var args;\n        var res;\n        return function () {\n            if (!args || !isArraysEqual(args, arguments)) {\n                args = arguments;\n                res = workerFunc.apply(this, arguments);\n            }\n            return res;\n        };\n    }\n    /*\n    always executes the workerFunc, but if the result is equal to the previous result,\n    return the previous result instead.\n    */\n    function memoizeOutput(workerFunc, equalityFunc) {\n        var cachedRes = null;\n        return function () {\n            var newRes = workerFunc.apply(this, arguments);\n            if (cachedRes === null || !(cachedRes === newRes || equalityFunc(cachedRes, newRes))) {\n                cachedRes = newRes;\n            }\n            return cachedRes;\n        };\n    }\n\n    var EXTENDED_SETTINGS_AND_SEVERITIES = {\n        week: 3,\n        separator: 0,\n        omitZeroMinute: 0,\n        meridiem: 0,\n        omitCommas: 0\n    };\n    var STANDARD_DATE_PROP_SEVERITIES = {\n        timeZoneName: 7,\n        era: 6,\n        year: 5,\n        month: 4,\n        day: 2,\n        weekday: 2,\n        hour: 1,\n        minute: 1,\n        second: 1\n    };\n    var MERIDIEM_RE = /\\s*([ap])\\.?m\\.?/i; // eats up leading spaces too\n    var COMMA_RE = /,/g; // we need re for globalness\n    var MULTI_SPACE_RE = /\\s+/g;\n    var LTR_RE = /\\u200e/g; // control character\n    var UTC_RE = /UTC|GMT/;\n    var NativeFormatter = /** @class */ (function () {\n        function NativeFormatter(formatSettings) {\n            var standardDateProps = {};\n            var extendedSettings = {};\n            var severity = 0;\n            for (var name_1 in formatSettings) {\n                if (name_1 in EXTENDED_SETTINGS_AND_SEVERITIES) {\n                    extendedSettings[name_1] = formatSettings[name_1];\n                    severity = Math.max(EXTENDED_SETTINGS_AND_SEVERITIES[name_1], severity);\n                }\n                else {\n                    standardDateProps[name_1] = formatSettings[name_1];\n                    if (name_1 in STANDARD_DATE_PROP_SEVERITIES) {\n                        severity = Math.max(STANDARD_DATE_PROP_SEVERITIES[name_1], severity);\n                    }\n                }\n            }\n            this.standardDateProps = standardDateProps;\n            this.extendedSettings = extendedSettings;\n            this.severity = severity;\n            this.buildFormattingFunc = memoize(buildFormattingFunc);\n        }\n        NativeFormatter.prototype.format = function (date, context) {\n            return this.buildFormattingFunc(this.standardDateProps, this.extendedSettings, context)(date);\n        };\n        NativeFormatter.prototype.formatRange = function (start, end, context) {\n            var _a = this, standardDateProps = _a.standardDateProps, extendedSettings = _a.extendedSettings;\n            var diffSeverity = computeMarkerDiffSeverity(start.marker, end.marker, context.calendarSystem);\n            if (!diffSeverity) {\n                return this.format(start, context);\n            }\n            var biggestUnitForPartial = diffSeverity;\n            if (biggestUnitForPartial > 1 && // the two dates are different in a way that's larger scale than time\n                (standardDateProps.year === 'numeric' || standardDateProps.year === '2-digit') &&\n                (standardDateProps.month === 'numeric' || standardDateProps.month === '2-digit') &&\n                (standardDateProps.day === 'numeric' || standardDateProps.day === '2-digit')) {\n                biggestUnitForPartial = 1; // make it look like the dates are only different in terms of time\n            }\n            var full0 = this.format(start, context);\n            var full1 = this.format(end, context);\n            if (full0 === full1) {\n                return full0;\n            }\n            var partialDateProps = computePartialFormattingOptions(standardDateProps, biggestUnitForPartial);\n            var partialFormattingFunc = buildFormattingFunc(partialDateProps, extendedSettings, context);\n            var partial0 = partialFormattingFunc(start);\n            var partial1 = partialFormattingFunc(end);\n            var insertion = findCommonInsertion(full0, partial0, full1, partial1);\n            var separator = extendedSettings.separator || '';\n            if (insertion) {\n                return insertion.before + partial0 + separator + partial1 + insertion.after;\n            }\n            return full0 + separator + full1;\n        };\n        NativeFormatter.prototype.getLargestUnit = function () {\n            switch (this.severity) {\n                case 7:\n                case 6:\n                case 5:\n                    return 'year';\n                case 4:\n                    return 'month';\n                case 3:\n                    return 'week';\n                default:\n                    return 'day';\n            }\n        };\n        return NativeFormatter;\n    }());\n    function buildFormattingFunc(standardDateProps, extendedSettings, context) {\n        var standardDatePropCnt = Object.keys(standardDateProps).length;\n        if (standardDatePropCnt === 1 && standardDateProps.timeZoneName === 'short') {\n            return function (date) {\n                return formatTimeZoneOffset(date.timeZoneOffset);\n            };\n        }\n        if (standardDatePropCnt === 0 && extendedSettings.week) {\n            return function (date) {\n                return formatWeekNumber(context.computeWeekNumber(date.marker), context.weekLabel, context.locale, extendedSettings.week);\n            };\n        }\n        return buildNativeFormattingFunc(standardDateProps, extendedSettings, context);\n    }\n    function buildNativeFormattingFunc(standardDateProps, extendedSettings, context) {\n        standardDateProps = __assign({}, standardDateProps); // copy\n        extendedSettings = __assign({}, extendedSettings); // copy\n        sanitizeSettings(standardDateProps, extendedSettings);\n        standardDateProps.timeZone = 'UTC'; // we leverage the only guaranteed timeZone for our UTC markers\n        var normalFormat = new Intl.DateTimeFormat(context.locale.codes, standardDateProps);\n        var zeroFormat; // needed?\n        if (extendedSettings.omitZeroMinute) {\n            var zeroProps = __assign({}, standardDateProps);\n            delete zeroProps.minute; // seconds and ms were already considered in sanitizeSettings\n            zeroFormat = new Intl.DateTimeFormat(context.locale.codes, zeroProps);\n        }\n        return function (date) {\n            var marker = date.marker;\n            var format;\n            if (zeroFormat && !marker.getUTCMinutes()) {\n                format = zeroFormat;\n            }\n            else {\n                format = normalFormat;\n            }\n            var s = format.format(marker);\n            return postProcess(s, date, standardDateProps, extendedSettings, context);\n        };\n    }\n    function sanitizeSettings(standardDateProps, extendedSettings) {\n        // deal with a browser inconsistency where formatting the timezone\n        // requires that the hour/minute be present.\n        if (standardDateProps.timeZoneName) {\n            if (!standardDateProps.hour) {\n                standardDateProps.hour = '2-digit';\n            }\n            if (!standardDateProps.minute) {\n                standardDateProps.minute = '2-digit';\n            }\n        }\n        // only support short timezone names\n        if (standardDateProps.timeZoneName === 'long') {\n            standardDateProps.timeZoneName = 'short';\n        }\n        // if requesting to display seconds, MUST display minutes\n        if (extendedSettings.omitZeroMinute && (standardDateProps.second || standardDateProps.millisecond)) {\n            delete extendedSettings.omitZeroMinute;\n        }\n    }\n    function postProcess(s, date, standardDateProps, extendedSettings, context) {\n        s = s.replace(LTR_RE, ''); // remove left-to-right control chars. do first. good for other regexes\n        if (standardDateProps.timeZoneName === 'short') {\n            s = injectTzoStr(s, (context.timeZone === 'UTC' || date.timeZoneOffset == null) ?\n                'UTC' : // important to normalize for IE, which does \"GMT\"\n                formatTimeZoneOffset(date.timeZoneOffset));\n        }\n        if (extendedSettings.omitCommas) {\n            s = s.replace(COMMA_RE, '').trim();\n        }\n        if (extendedSettings.omitZeroMinute) {\n            s = s.replace(':00', ''); // zeroFormat doesn't always achieve this\n        }\n        // ^ do anything that might create adjacent spaces before this point,\n        // because MERIDIEM_RE likes to eat up loading spaces\n        if (extendedSettings.meridiem === false) {\n            s = s.replace(MERIDIEM_RE, '').trim();\n        }\n        else if (extendedSettings.meridiem === 'narrow') { // a/p\n            s = s.replace(MERIDIEM_RE, function (m0, m1) {\n                return m1.toLocaleLowerCase();\n            });\n        }\n        else if (extendedSettings.meridiem === 'short') { // am/pm\n            s = s.replace(MERIDIEM_RE, function (m0, m1) {\n                return m1.toLocaleLowerCase() + 'm';\n            });\n        }\n        else if (extendedSettings.meridiem === 'lowercase') { // other meridiem transformers already converted to lowercase\n            s = s.replace(MERIDIEM_RE, function (m0) {\n                return m0.toLocaleLowerCase();\n            });\n        }\n        s = s.replace(MULTI_SPACE_RE, ' ');\n        s = s.trim();\n        return s;\n    }\n    function injectTzoStr(s, tzoStr) {\n        var replaced = false;\n        s = s.replace(UTC_RE, function () {\n            replaced = true;\n            return tzoStr;\n        });\n        // IE11 doesn't include UTC/GMT in the original string, so append to end\n        if (!replaced) {\n            s += ' ' + tzoStr;\n        }\n        return s;\n    }\n    function formatWeekNumber(num, weekLabel, locale, display) {\n        var parts = [];\n        if (display === 'narrow') {\n            parts.push(weekLabel);\n        }\n        else if (display === 'short') {\n            parts.push(weekLabel, ' ');\n        }\n        // otherwise, considered 'numeric'\n        parts.push(locale.simpleNumberFormat.format(num));\n        if (locale.options.isRtl) { // TODO: use control characters instead?\n            parts.reverse();\n        }\n        return parts.join('');\n    }\n    // Range Formatting Utils\n    // 0 = exactly the same\n    // 1 = different by time\n    // and bigger\n    function computeMarkerDiffSeverity(d0, d1, ca) {\n        if (ca.getMarkerYear(d0) !== ca.getMarkerYear(d1)) {\n            return 5;\n        }\n        if (ca.getMarkerMonth(d0) !== ca.getMarkerMonth(d1)) {\n            return 4;\n        }\n        if (ca.getMarkerDay(d0) !== ca.getMarkerDay(d1)) {\n            return 2;\n        }\n        if (timeAsMs(d0) !== timeAsMs(d1)) {\n            return 1;\n        }\n        return 0;\n    }\n    function computePartialFormattingOptions(options, biggestUnit) {\n        var partialOptions = {};\n        for (var name_2 in options) {\n            if (!(name_2 in STANDARD_DATE_PROP_SEVERITIES) || // not a date part prop (like timeZone)\n                STANDARD_DATE_PROP_SEVERITIES[name_2] <= biggestUnit) {\n                partialOptions[name_2] = options[name_2];\n            }\n        }\n        return partialOptions;\n    }\n    function findCommonInsertion(full0, partial0, full1, partial1) {\n        var i0 = 0;\n        while (i0 < full0.length) {\n            var found0 = full0.indexOf(partial0, i0);\n            if (found0 === -1) {\n                break;\n            }\n            var before0 = full0.substr(0, found0);\n            i0 = found0 + partial0.length;\n            var after0 = full0.substr(i0);\n            var i1 = 0;\n            while (i1 < full1.length) {\n                var found1 = full1.indexOf(partial1, i1);\n                if (found1 === -1) {\n                    break;\n                }\n                var before1 = full1.substr(0, found1);\n                i1 = found1 + partial1.length;\n                var after1 = full1.substr(i1);\n                if (before0 === before1 && after0 === after1) {\n                    return {\n                        before: before0,\n                        after: after0\n                    };\n                }\n            }\n        }\n        return null;\n    }\n\n    /*\n    TODO: fix the terminology of \"formatter\" vs \"formatting func\"\n    */\n    /*\n    At the time of instantiation, this object does not know which cmd-formatting system it will use.\n    It receives this at the time of formatting, as a setting.\n    */\n    var CmdFormatter = /** @class */ (function () {\n        function CmdFormatter(cmdStr, separator) {\n            this.cmdStr = cmdStr;\n            this.separator = separator;\n        }\n        CmdFormatter.prototype.format = function (date, context) {\n            return context.cmdFormatter(this.cmdStr, createVerboseFormattingArg(date, null, context, this.separator));\n        };\n        CmdFormatter.prototype.formatRange = function (start, end, context) {\n            return context.cmdFormatter(this.cmdStr, createVerboseFormattingArg(start, end, context, this.separator));\n        };\n        return CmdFormatter;\n    }());\n\n    var FuncFormatter = /** @class */ (function () {\n        function FuncFormatter(func) {\n            this.func = func;\n        }\n        FuncFormatter.prototype.format = function (date, context) {\n            return this.func(createVerboseFormattingArg(date, null, context));\n        };\n        FuncFormatter.prototype.formatRange = function (start, end, context) {\n            return this.func(createVerboseFormattingArg(start, end, context));\n        };\n        return FuncFormatter;\n    }());\n\n    // Formatter Object Creation\n    function createFormatter(input, defaultSeparator) {\n        if (typeof input === 'object' && input) { // non-null object\n            if (typeof defaultSeparator === 'string') {\n                input = __assign({ separator: defaultSeparator }, input);\n            }\n            return new NativeFormatter(input);\n        }\n        else if (typeof input === 'string') {\n            return new CmdFormatter(input, defaultSeparator);\n        }\n        else if (typeof input === 'function') {\n            return new FuncFormatter(input);\n        }\n    }\n    // String Utils\n    // timeZoneOffset is in minutes\n    function buildIsoString(marker, timeZoneOffset, stripZeroTime) {\n        if (stripZeroTime === void 0) { stripZeroTime = false; }\n        var s = marker.toISOString();\n        s = s.replace('.000', '');\n        if (stripZeroTime) {\n            s = s.replace('T00:00:00Z', '');\n        }\n        if (s.length > 10) { // time part wasn't stripped, can add timezone info\n            if (timeZoneOffset == null) {\n                s = s.replace('Z', '');\n            }\n            else if (timeZoneOffset !== 0) {\n                s = s.replace('Z', formatTimeZoneOffset(timeZoneOffset, true));\n            }\n            // otherwise, its UTC-0 and we want to keep the Z\n        }\n        return s;\n    }\n    function formatIsoTimeString(marker) {\n        return padStart(marker.getUTCHours(), 2) + ':' +\n            padStart(marker.getUTCMinutes(), 2) + ':' +\n            padStart(marker.getUTCSeconds(), 2);\n    }\n    function formatTimeZoneOffset(minutes, doIso) {\n        if (doIso === void 0) { doIso = false; }\n        var sign = minutes < 0 ? '-' : '+';\n        var abs = Math.abs(minutes);\n        var hours = Math.floor(abs / 60);\n        var mins = Math.round(abs % 60);\n        if (doIso) {\n            return sign + padStart(hours, 2) + ':' + padStart(mins, 2);\n        }\n        else {\n            return 'GMT' + sign + hours + (mins ? ':' + padStart(mins, 2) : '');\n        }\n    }\n    // Arg Utils\n    function createVerboseFormattingArg(start, end, context, separator) {\n        var startInfo = expandZonedMarker(start, context.calendarSystem);\n        var endInfo = end ? expandZonedMarker(end, context.calendarSystem) : null;\n        return {\n            date: startInfo,\n            start: startInfo,\n            end: endInfo,\n            timeZone: context.timeZone,\n            localeCodes: context.locale.codes,\n            separator: separator\n        };\n    }\n    function expandZonedMarker(dateInfo, calendarSystem) {\n        var a = calendarSystem.markerToArray(dateInfo.marker);\n        return {\n            marker: dateInfo.marker,\n            timeZoneOffset: dateInfo.timeZoneOffset,\n            array: a,\n            year: a[0],\n            month: a[1],\n            day: a[2],\n            hour: a[3],\n            minute: a[4],\n            second: a[5],\n            millisecond: a[6]\n        };\n    }\n\n    var EventSourceApi = /** @class */ (function () {\n        function EventSourceApi(calendar, internalEventSource) {\n            this.calendar = calendar;\n            this.internalEventSource = internalEventSource;\n        }\n        EventSourceApi.prototype.remove = function () {\n            this.calendar.dispatch({\n                type: 'REMOVE_EVENT_SOURCE',\n                sourceId: this.internalEventSource.sourceId\n            });\n        };\n        EventSourceApi.prototype.refetch = function () {\n            this.calendar.dispatch({\n                type: 'FETCH_EVENT_SOURCES',\n                sourceIds: [this.internalEventSource.sourceId]\n            });\n        };\n        Object.defineProperty(EventSourceApi.prototype, \"id\", {\n            get: function () {\n                return this.internalEventSource.publicId;\n            },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventSourceApi.prototype, \"url\", {\n            // only relevant to json-feed event sources\n            get: function () {\n                return this.internalEventSource.meta.url;\n            },\n            enumerable: true,\n            configurable: true\n        });\n        return EventSourceApi;\n    }());\n\n    var EventApi = /** @class */ (function () {\n        function EventApi(calendar, def, instance) {\n            this._calendar = calendar;\n            this._def = def;\n            this._instance = instance || null;\n        }\n        /*\n        TODO: make event struct more responsible for this\n        */\n        EventApi.prototype.setProp = function (name, val) {\n            var _a, _b;\n            if (name in DATE_PROPS) ;\n            else if (name in NON_DATE_PROPS) {\n                if (typeof NON_DATE_PROPS[name] === 'function') {\n                    val = NON_DATE_PROPS[name](val);\n                }\n                this.mutate({\n                    standardProps: (_a = {}, _a[name] = val, _a)\n                });\n            }\n            else if (name in UNSCOPED_EVENT_UI_PROPS) {\n                var ui = void 0;\n                if (typeof UNSCOPED_EVENT_UI_PROPS[name] === 'function') {\n                    val = UNSCOPED_EVENT_UI_PROPS[name](val);\n                }\n                if (name === 'color') {\n                    ui = { backgroundColor: val, borderColor: val };\n                }\n                else if (name === 'editable') {\n                    ui = { startEditable: val, durationEditable: val };\n                }\n                else {\n                    ui = (_b = {}, _b[name] = val, _b);\n                }\n                this.mutate({\n                    standardProps: { ui: ui }\n                });\n            }\n        };\n        EventApi.prototype.setExtendedProp = function (name, val) {\n            var _a;\n            this.mutate({\n                extendedProps: (_a = {}, _a[name] = val, _a)\n            });\n        };\n        EventApi.prototype.setStart = function (startInput, options) {\n            if (options === void 0) { options = {}; }\n            var dateEnv = this._calendar.dateEnv;\n            var start = dateEnv.createMarker(startInput);\n            if (start && this._instance) { // TODO: warning if parsed bad\n                var instanceRange = this._instance.range;\n                var startDelta = diffDates(instanceRange.start, start, dateEnv, options.granularity); // what if parsed bad!?\n                if (options.maintainDuration) {\n                    this.mutate({ datesDelta: startDelta });\n                }\n                else {\n                    this.mutate({ startDelta: startDelta });\n                }\n            }\n        };\n        EventApi.prototype.setEnd = function (endInput, options) {\n            if (options === void 0) { options = {}; }\n            var dateEnv = this._calendar.dateEnv;\n            var end;\n            if (endInput != null) {\n                end = dateEnv.createMarker(endInput);\n                if (!end) {\n                    return; // TODO: warning if parsed bad\n                }\n            }\n            if (this._instance) {\n                if (end) {\n                    var endDelta = diffDates(this._instance.range.end, end, dateEnv, options.granularity);\n                    this.mutate({ endDelta: endDelta });\n                }\n                else {\n                    this.mutate({ standardProps: { hasEnd: false } });\n                }\n            }\n        };\n        EventApi.prototype.setDates = function (startInput, endInput, options) {\n            if (options === void 0) { options = {}; }\n            var dateEnv = this._calendar.dateEnv;\n            var standardProps = { allDay: options.allDay };\n            var start = dateEnv.createMarker(startInput);\n            var end;\n            if (!start) {\n                return; // TODO: warning if parsed bad\n            }\n            if (endInput != null) {\n                end = dateEnv.createMarker(endInput);\n                if (!end) { // TODO: warning if parsed bad\n                    return;\n                }\n            }\n            if (this._instance) {\n                var instanceRange = this._instance.range;\n                // when computing the diff for an event being converted to all-day,\n                // compute diff off of the all-day values the way event-mutation does.\n                if (options.allDay === true) {\n                    instanceRange = computeAlignedDayRange(instanceRange);\n                }\n                var startDelta = diffDates(instanceRange.start, start, dateEnv, options.granularity);\n                if (end) {\n                    var endDelta = diffDates(instanceRange.end, end, dateEnv, options.granularity);\n                    if (durationsEqual(startDelta, endDelta)) {\n                        this.mutate({ datesDelta: startDelta, standardProps: standardProps });\n                    }\n                    else {\n                        this.mutate({ startDelta: startDelta, endDelta: endDelta, standardProps: standardProps });\n                    }\n                }\n                else { // means \"clear the end\"\n                    standardProps.hasEnd = false;\n                    this.mutate({ datesDelta: startDelta, standardProps: standardProps });\n                }\n            }\n        };\n        EventApi.prototype.moveStart = function (deltaInput) {\n            var delta = createDuration(deltaInput);\n            if (delta) { // TODO: warning if parsed bad\n                this.mutate({ startDelta: delta });\n            }\n        };\n        EventApi.prototype.moveEnd = function (deltaInput) {\n            var delta = createDuration(deltaInput);\n            if (delta) { // TODO: warning if parsed bad\n                this.mutate({ endDelta: delta });\n            }\n        };\n        EventApi.prototype.moveDates = function (deltaInput) {\n            var delta = createDuration(deltaInput);\n            if (delta) { // TODO: warning if parsed bad\n                this.mutate({ datesDelta: delta });\n            }\n        };\n        EventApi.prototype.setAllDay = function (allDay, options) {\n            if (options === void 0) { options = {}; }\n            var standardProps = { allDay: allDay };\n            var maintainDuration = options.maintainDuration;\n            if (maintainDuration == null) {\n                maintainDuration = this._calendar.opt('allDayMaintainDuration');\n            }\n            if (this._def.allDay !== allDay) {\n                standardProps.hasEnd = maintainDuration;\n            }\n            this.mutate({ standardProps: standardProps });\n        };\n        EventApi.prototype.formatRange = function (formatInput) {\n            var dateEnv = this._calendar.dateEnv;\n            var instance = this._instance;\n            var formatter = createFormatter(formatInput, this._calendar.opt('defaultRangeSeparator'));\n            if (this._def.hasEnd) {\n                return dateEnv.formatRange(instance.range.start, instance.range.end, formatter, {\n                    forcedStartTzo: instance.forcedStartTzo,\n                    forcedEndTzo: instance.forcedEndTzo\n                });\n            }\n            else {\n                return dateEnv.format(instance.range.start, formatter, {\n                    forcedTzo: instance.forcedStartTzo\n                });\n            }\n        };\n        EventApi.prototype.mutate = function (mutation) {\n            var def = this._def;\n            var instance = this._instance;\n            if (instance) {\n                this._calendar.dispatch({\n                    type: 'MUTATE_EVENTS',\n                    instanceId: instance.instanceId,\n                    mutation: mutation,\n                    fromApi: true\n                });\n                var eventStore = this._calendar.state.eventStore;\n                this._def = eventStore.defs[def.defId];\n                this._instance = eventStore.instances[instance.instanceId];\n            }\n        };\n        EventApi.prototype.remove = function () {\n            this._calendar.dispatch({\n                type: 'REMOVE_EVENT_DEF',\n                defId: this._def.defId\n            });\n        };\n        Object.defineProperty(EventApi.prototype, \"source\", {\n            get: function () {\n                var sourceId = this._def.sourceId;\n                if (sourceId) {\n                    return new EventSourceApi(this._calendar, this._calendar.state.eventSources[sourceId]);\n                }\n                return null;\n            },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventApi.prototype, \"start\", {\n            get: function () {\n                return this._instance ?\n                    this._calendar.dateEnv.toDate(this._instance.range.start) :\n                    null;\n            },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventApi.prototype, \"end\", {\n            get: function () {\n                return (this._instance && this._def.hasEnd) ?\n                    this._calendar.dateEnv.toDate(this._instance.range.end) :\n                    null;\n            },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventApi.prototype, \"id\", {\n            // computable props that all access the def\n            // TODO: find a TypeScript-compatible way to do this at scale\n            get: function () { return this._def.publicId; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventApi.prototype, \"groupId\", {\n            get: function () { return this._def.groupId; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventApi.prototype, \"allDay\", {\n            get: function () { return this._def.allDay; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventApi.prototype, \"title\", {\n            get: function () { return this._def.title; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventApi.prototype, \"url\", {\n            get: function () { return this._def.url; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventApi.prototype, \"rendering\", {\n            get: function () { return this._def.rendering; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventApi.prototype, \"startEditable\", {\n            get: function () { return this._def.ui.startEditable; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventApi.prototype, \"durationEditable\", {\n            get: function () { return this._def.ui.durationEditable; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventApi.prototype, \"constraint\", {\n            get: function () { return this._def.ui.constraints[0] || null; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventApi.prototype, \"overlap\", {\n            get: function () { return this._def.ui.overlap; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventApi.prototype, \"allow\", {\n            get: function () { return this._def.ui.allows[0] || null; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventApi.prototype, \"backgroundColor\", {\n            get: function () { return this._def.ui.backgroundColor; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventApi.prototype, \"borderColor\", {\n            get: function () { return this._def.ui.borderColor; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventApi.prototype, \"textColor\", {\n            get: function () { return this._def.ui.textColor; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventApi.prototype, \"classNames\", {\n            // NOTE: user can't modify these because Object.freeze was called in event-def parsing\n            get: function () { return this._def.ui.classNames; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(EventApi.prototype, \"extendedProps\", {\n            get: function () { return this._def.extendedProps; },\n            enumerable: true,\n            configurable: true\n        });\n        return EventApi;\n    }());\n\n    /*\n    Specifying nextDayThreshold signals that all-day ranges should be sliced.\n    */\n    function sliceEventStore(eventStore, eventUiBases, framingRange, nextDayThreshold) {\n        var inverseBgByGroupId = {};\n        var inverseBgByDefId = {};\n        var defByGroupId = {};\n        var bgRanges = [];\n        var fgRanges = [];\n        var eventUis = compileEventUis(eventStore.defs, eventUiBases);\n        for (var defId in eventStore.defs) {\n            var def = eventStore.defs[defId];\n            if (def.rendering === 'inverse-background') {\n                if (def.groupId) {\n                    inverseBgByGroupId[def.groupId] = [];\n                    if (!defByGroupId[def.groupId]) {\n                        defByGroupId[def.groupId] = def;\n                    }\n                }\n                else {\n                    inverseBgByDefId[defId] = [];\n                }\n            }\n        }\n        for (var instanceId in eventStore.instances) {\n            var instance = eventStore.instances[instanceId];\n            var def = eventStore.defs[instance.defId];\n            var ui = eventUis[def.defId];\n            var origRange = instance.range;\n            var normalRange = (!def.allDay && nextDayThreshold) ?\n                computeVisibleDayRange(origRange, nextDayThreshold) :\n                origRange;\n            var slicedRange = intersectRanges(normalRange, framingRange);\n            if (slicedRange) {\n                if (def.rendering === 'inverse-background') {\n                    if (def.groupId) {\n                        inverseBgByGroupId[def.groupId].push(slicedRange);\n                    }\n                    else {\n                        inverseBgByDefId[instance.defId].push(slicedRange);\n                    }\n                }\n                else {\n                    (def.rendering === 'background' ? bgRanges : fgRanges).push({\n                        def: def,\n                        ui: ui,\n                        instance: instance,\n                        range: slicedRange,\n                        isStart: normalRange.start && normalRange.start.valueOf() === slicedRange.start.valueOf(),\n                        isEnd: normalRange.end && normalRange.end.valueOf() === slicedRange.end.valueOf()\n                    });\n                }\n            }\n        }\n        for (var groupId in inverseBgByGroupId) { // BY GROUP\n            var ranges = inverseBgByGroupId[groupId];\n            var invertedRanges = invertRanges(ranges, framingRange);\n            for (var _i = 0, invertedRanges_1 = invertedRanges; _i < invertedRanges_1.length; _i++) {\n                var invertedRange = invertedRanges_1[_i];\n                var def = defByGroupId[groupId];\n                var ui = eventUis[def.defId];\n                bgRanges.push({\n                    def: def,\n                    ui: ui,\n                    instance: null,\n                    range: invertedRange,\n                    isStart: false,\n                    isEnd: false\n                });\n            }\n        }\n        for (var defId in inverseBgByDefId) {\n            var ranges = inverseBgByDefId[defId];\n            var invertedRanges = invertRanges(ranges, framingRange);\n            for (var _a = 0, invertedRanges_2 = invertedRanges; _a < invertedRanges_2.length; _a++) {\n                var invertedRange = invertedRanges_2[_a];\n                bgRanges.push({\n                    def: eventStore.defs[defId],\n                    ui: eventUis[defId],\n                    instance: null,\n                    range: invertedRange,\n                    isStart: false,\n                    isEnd: false\n                });\n            }\n        }\n        return { bg: bgRanges, fg: fgRanges };\n    }\n    function hasBgRendering(def) {\n        return def.rendering === 'background' || def.rendering === 'inverse-background';\n    }\n    function filterSegsViaEls(context, segs, isMirror) {\n        var calendar = context.calendar, view = context.view;\n        if (calendar.hasPublicHandlers('eventRender')) {\n            segs = segs.filter(function (seg) {\n                var custom = calendar.publiclyTrigger('eventRender', [\n                    {\n                        event: new EventApi(calendar, seg.eventRange.def, seg.eventRange.instance),\n                        isMirror: isMirror,\n                        isStart: seg.isStart,\n                        isEnd: seg.isEnd,\n                        // TODO: include seg.range once all components consistently generate it\n                        el: seg.el,\n                        view: view\n                    }\n                ]);\n                if (custom === false) { // means don't render at all\n                    return false;\n                }\n                else if (custom && custom !== true) {\n                    seg.el = custom;\n                }\n                return true;\n            });\n        }\n        for (var _i = 0, segs_1 = segs; _i < segs_1.length; _i++) {\n            var seg = segs_1[_i];\n            setElSeg(seg.el, seg);\n        }\n        return segs;\n    }\n    function setElSeg(el, seg) {\n        el.fcSeg = seg;\n    }\n    function getElSeg(el) {\n        return el.fcSeg || null;\n    }\n    // event ui computation\n    function compileEventUis(eventDefs, eventUiBases) {\n        return mapHash(eventDefs, function (eventDef) {\n            return compileEventUi(eventDef, eventUiBases);\n        });\n    }\n    function compileEventUi(eventDef, eventUiBases) {\n        var uis = [];\n        if (eventUiBases['']) {\n            uis.push(eventUiBases['']);\n        }\n        if (eventUiBases[eventDef.defId]) {\n            uis.push(eventUiBases[eventDef.defId]);\n        }\n        uis.push(eventDef.ui);\n        return combineEventUis(uis);\n    }\n    // triggers\n    function triggerRenderedSegs(context, segs, isMirrors) {\n        var calendar = context.calendar, view = context.view;\n        if (calendar.hasPublicHandlers('eventPositioned')) {\n            for (var _i = 0, segs_2 = segs; _i < segs_2.length; _i++) {\n                var seg = segs_2[_i];\n                calendar.publiclyTriggerAfterSizing('eventPositioned', [\n                    {\n                        event: new EventApi(calendar, seg.eventRange.def, seg.eventRange.instance),\n                        isMirror: isMirrors,\n                        isStart: seg.isStart,\n                        isEnd: seg.isEnd,\n                        el: seg.el,\n                        view: view\n                    }\n                ]);\n            }\n        }\n        if (!calendar.state.eventSourceLoadingLevel) { // avoid initial empty state while pending\n            calendar.afterSizingTriggers._eventsPositioned = [null]; // fire once\n        }\n    }\n    function triggerWillRemoveSegs(context, segs, isMirrors) {\n        var calendar = context.calendar, view = context.view;\n        for (var _i = 0, segs_3 = segs; _i < segs_3.length; _i++) {\n            var seg = segs_3[_i];\n            calendar.trigger('eventElRemove', seg.el);\n        }\n        if (calendar.hasPublicHandlers('eventDestroy')) {\n            for (var _a = 0, segs_4 = segs; _a < segs_4.length; _a++) {\n                var seg = segs_4[_a];\n                calendar.publiclyTrigger('eventDestroy', [\n                    {\n                        event: new EventApi(calendar, seg.eventRange.def, seg.eventRange.instance),\n                        isMirror: isMirrors,\n                        el: seg.el,\n                        view: view\n                    }\n                ]);\n            }\n        }\n    }\n    // is-interactable\n    function computeEventDraggable(context, eventDef, eventUi) {\n        var calendar = context.calendar, view = context.view;\n        var transformers = calendar.pluginSystem.hooks.isDraggableTransformers;\n        var val = eventUi.startEditable;\n        for (var _i = 0, transformers_1 = transformers; _i < transformers_1.length; _i++) {\n            var transformer = transformers_1[_i];\n            val = transformer(val, eventDef, eventUi, view);\n        }\n        return val;\n    }\n    function computeEventStartResizable(context, eventDef, eventUi) {\n        return eventUi.durationEditable && context.options.eventResizableFromStart;\n    }\n    function computeEventEndResizable(context, eventDef, eventUi) {\n        return eventUi.durationEditable;\n    }\n\n    // applies the mutation to ALL defs/instances within the event store\n    function applyMutationToEventStore(eventStore, eventConfigBase, mutation, calendar) {\n        var eventConfigs = compileEventUis(eventStore.defs, eventConfigBase);\n        var dest = createEmptyEventStore();\n        for (var defId in eventStore.defs) {\n            var def = eventStore.defs[defId];\n            dest.defs[defId] = applyMutationToEventDef(def, eventConfigs[defId], mutation, calendar.pluginSystem.hooks.eventDefMutationAppliers, calendar);\n        }\n        for (var instanceId in eventStore.instances) {\n            var instance = eventStore.instances[instanceId];\n            var def = dest.defs[instance.defId]; // important to grab the newly modified def\n            dest.instances[instanceId] = applyMutationToEventInstance(instance, def, eventConfigs[instance.defId], mutation, calendar);\n        }\n        return dest;\n    }\n    function applyMutationToEventDef(eventDef, eventConfig, mutation, appliers, calendar) {\n        var standardProps = mutation.standardProps || {};\n        // if hasEnd has not been specified, guess a good value based on deltas.\n        // if duration will change, there's no way the default duration will persist,\n        // and thus, we need to mark the event as having a real end\n        if (standardProps.hasEnd == null &&\n            eventConfig.durationEditable &&\n            (mutation.startDelta || mutation.endDelta)) {\n            standardProps.hasEnd = true; // TODO: is this mutation okay?\n        }\n        var copy = __assign({}, eventDef, standardProps, { ui: __assign({}, eventDef.ui, standardProps.ui) });\n        if (mutation.extendedProps) {\n            copy.extendedProps = __assign({}, copy.extendedProps, mutation.extendedProps);\n        }\n        for (var _i = 0, appliers_1 = appliers; _i < appliers_1.length; _i++) {\n            var applier = appliers_1[_i];\n            applier(copy, mutation, calendar);\n        }\n        if (!copy.hasEnd && calendar.opt('forceEventDuration')) {\n            copy.hasEnd = true;\n        }\n        return copy;\n    }\n    function applyMutationToEventInstance(eventInstance, eventDef, // must first be modified by applyMutationToEventDef\n    eventConfig, mutation, calendar) {\n        var dateEnv = calendar.dateEnv;\n        var forceAllDay = mutation.standardProps && mutation.standardProps.allDay === true;\n        var clearEnd = mutation.standardProps && mutation.standardProps.hasEnd === false;\n        var copy = __assign({}, eventInstance);\n        if (forceAllDay) {\n            copy.range = computeAlignedDayRange(copy.range);\n        }\n        if (mutation.datesDelta && eventConfig.startEditable) {\n            copy.range = {\n                start: dateEnv.add(copy.range.start, mutation.datesDelta),\n                end: dateEnv.add(copy.range.end, mutation.datesDelta)\n            };\n        }\n        if (mutation.startDelta && eventConfig.durationEditable) {\n            copy.range = {\n                start: dateEnv.add(copy.range.start, mutation.startDelta),\n                end: copy.range.end\n            };\n        }\n        if (mutation.endDelta && eventConfig.durationEditable) {\n            copy.range = {\n                start: copy.range.start,\n                end: dateEnv.add(copy.range.end, mutation.endDelta)\n            };\n        }\n        if (clearEnd) {\n            copy.range = {\n                start: copy.range.start,\n                end: calendar.getDefaultEventEnd(eventDef.allDay, copy.range.start)\n            };\n        }\n        // in case event was all-day but the supplied deltas were not\n        // better util for this?\n        if (eventDef.allDay) {\n            copy.range = {\n                start: startOfDay(copy.range.start),\n                end: startOfDay(copy.range.end)\n            };\n        }\n        // handle invalid durations\n        if (copy.range.end < copy.range.start) {\n            copy.range.end = calendar.getDefaultEventEnd(eventDef.allDay, copy.range.start);\n        }\n        return copy;\n    }\n\n    function reduceEventStore (eventStore, action, eventSources, dateProfile, calendar) {\n        switch (action.type) {\n            case 'RECEIVE_EVENTS': // raw\n                return receiveRawEvents(eventStore, eventSources[action.sourceId], action.fetchId, action.fetchRange, action.rawEvents, calendar);\n            case 'ADD_EVENTS': // already parsed, but not expanded\n                return addEvent(eventStore, action.eventStore, // new ones\n                dateProfile ? dateProfile.activeRange : null, calendar);\n            case 'MERGE_EVENTS': // already parsed and expanded\n                return mergeEventStores(eventStore, action.eventStore);\n            case 'PREV': // TODO: how do we track all actions that affect dateProfile :(\n            case 'NEXT':\n            case 'SET_DATE':\n            case 'SET_VIEW_TYPE':\n                if (dateProfile) {\n                    return expandRecurring(eventStore, dateProfile.activeRange, calendar);\n                }\n                else {\n                    return eventStore;\n                }\n            case 'CHANGE_TIMEZONE':\n                return rezoneDates(eventStore, action.oldDateEnv, calendar.dateEnv);\n            case 'MUTATE_EVENTS':\n                return applyMutationToRelated(eventStore, action.instanceId, action.mutation, action.fromApi, calendar);\n            case 'REMOVE_EVENT_INSTANCES':\n                return excludeInstances(eventStore, action.instances);\n            case 'REMOVE_EVENT_DEF':\n                return filterEventStoreDefs(eventStore, function (eventDef) {\n                    return eventDef.defId !== action.defId;\n                });\n            case 'REMOVE_EVENT_SOURCE':\n                return excludeEventsBySourceId(eventStore, action.sourceId);\n            case 'REMOVE_ALL_EVENT_SOURCES':\n                return filterEventStoreDefs(eventStore, function (eventDef) {\n                    return !eventDef.sourceId; // only keep events with no source id\n                });\n            case 'REMOVE_ALL_EVENTS':\n                return createEmptyEventStore();\n            case 'RESET_EVENTS':\n                return {\n                    defs: eventStore.defs,\n                    instances: eventStore.instances\n                };\n            default:\n                return eventStore;\n        }\n    }\n    function receiveRawEvents(eventStore, eventSource, fetchId, fetchRange, rawEvents, calendar) {\n        if (eventSource && // not already removed\n            fetchId === eventSource.latestFetchId // TODO: wish this logic was always in event-sources\n        ) {\n            var subset = parseEvents(transformRawEvents(rawEvents, eventSource, calendar), eventSource.sourceId, calendar);\n            if (fetchRange) {\n                subset = expandRecurring(subset, fetchRange, calendar);\n            }\n            return mergeEventStores(excludeEventsBySourceId(eventStore, eventSource.sourceId), subset);\n        }\n        return eventStore;\n    }\n    function addEvent(eventStore, subset, expandRange, calendar) {\n        if (expandRange) {\n            subset = expandRecurring(subset, expandRange, calendar);\n        }\n        return mergeEventStores(eventStore, subset);\n    }\n    function rezoneDates(eventStore, oldDateEnv, newDateEnv) {\n        var defs = eventStore.defs;\n        var instances = mapHash(eventStore.instances, function (instance) {\n            var def = defs[instance.defId];\n            if (def.allDay || def.recurringDef) {\n                return instance; // isn't dependent on timezone\n            }\n            else {\n                return __assign({}, instance, { range: {\n                        start: newDateEnv.createMarker(oldDateEnv.toDate(instance.range.start, instance.forcedStartTzo)),\n                        end: newDateEnv.createMarker(oldDateEnv.toDate(instance.range.end, instance.forcedEndTzo))\n                    }, forcedStartTzo: newDateEnv.canComputeOffset ? null : instance.forcedStartTzo, forcedEndTzo: newDateEnv.canComputeOffset ? null : instance.forcedEndTzo });\n            }\n        });\n        return { defs: defs, instances: instances };\n    }\n    function applyMutationToRelated(eventStore, instanceId, mutation, fromApi, calendar) {\n        var relevant = getRelevantEvents(eventStore, instanceId);\n        var eventConfigBase = fromApi ?\n            { '': {\n                    startEditable: true,\n                    durationEditable: true,\n                    constraints: [],\n                    overlap: null,\n                    allows: [],\n                    backgroundColor: '',\n                    borderColor: '',\n                    textColor: '',\n                    classNames: []\n                } } :\n            calendar.eventUiBases;\n        relevant = applyMutationToEventStore(relevant, eventConfigBase, mutation, calendar);\n        return mergeEventStores(eventStore, relevant);\n    }\n    function excludeEventsBySourceId(eventStore, sourceId) {\n        return filterEventStoreDefs(eventStore, function (eventDef) {\n            return eventDef.sourceId !== sourceId;\n        });\n    }\n    // QUESTION: why not just return instances? do a general object-property-exclusion util\n    function excludeInstances(eventStore, removals) {\n        return {\n            defs: eventStore.defs,\n            instances: filterHash(eventStore.instances, function (instance) {\n                return !removals[instance.instanceId];\n            })\n        };\n    }\n\n    // high-level segmenting-aware tester functions\n    // ------------------------------------------------------------------------------------------------------------------------\n    function isInteractionValid(interaction, calendar) {\n        return isNewPropsValid({ eventDrag: interaction }, calendar); // HACK: the eventDrag props is used for ALL interactions\n    }\n    function isDateSelectionValid(dateSelection, calendar) {\n        return isNewPropsValid({ dateSelection: dateSelection }, calendar);\n    }\n    function isNewPropsValid(newProps, calendar) {\n        var view = calendar.view;\n        var props = __assign({ businessHours: view ? view.props.businessHours : createEmptyEventStore(), dateSelection: '', eventStore: calendar.state.eventStore, eventUiBases: calendar.eventUiBases, eventSelection: '', eventDrag: null, eventResize: null }, newProps);\n        return (calendar.pluginSystem.hooks.isPropsValid || isPropsValid)(props, calendar);\n    }\n    function isPropsValid(state, calendar, dateSpanMeta, filterConfig) {\n        if (dateSpanMeta === void 0) { dateSpanMeta = {}; }\n        if (state.eventDrag && !isInteractionPropsValid(state, calendar, dateSpanMeta, filterConfig)) {\n            return false;\n        }\n        if (state.dateSelection && !isDateSelectionPropsValid(state, calendar, dateSpanMeta, filterConfig)) {\n            return false;\n        }\n        return true;\n    }\n    // Moving Event Validation\n    // ------------------------------------------------------------------------------------------------------------------------\n    function isInteractionPropsValid(state, calendar, dateSpanMeta, filterConfig) {\n        var interaction = state.eventDrag; // HACK: the eventDrag props is used for ALL interactions\n        var subjectEventStore = interaction.mutatedEvents;\n        var subjectDefs = subjectEventStore.defs;\n        var subjectInstances = subjectEventStore.instances;\n        var subjectConfigs = compileEventUis(subjectDefs, interaction.isEvent ?\n            state.eventUiBases :\n            { '': calendar.selectionConfig } // if not a real event, validate as a selection\n        );\n        if (filterConfig) {\n            subjectConfigs = mapHash(subjectConfigs, filterConfig);\n        }\n        var otherEventStore = excludeInstances(state.eventStore, interaction.affectedEvents.instances); // exclude the subject events. TODO: exclude defs too?\n        var otherDefs = otherEventStore.defs;\n        var otherInstances = otherEventStore.instances;\n        var otherConfigs = compileEventUis(otherDefs, state.eventUiBases);\n        for (var subjectInstanceId in subjectInstances) {\n            var subjectInstance = subjectInstances[subjectInstanceId];\n            var subjectRange = subjectInstance.range;\n            var subjectConfig = subjectConfigs[subjectInstance.defId];\n            var subjectDef = subjectDefs[subjectInstance.defId];\n            // constraint\n            if (!allConstraintsPass(subjectConfig.constraints, subjectRange, otherEventStore, state.businessHours, calendar)) {\n                return false;\n            }\n            // overlap\n            var overlapFunc = calendar.opt('eventOverlap');\n            if (typeof overlapFunc !== 'function') {\n                overlapFunc = null;\n            }\n            for (var otherInstanceId in otherInstances) {\n                var otherInstance = otherInstances[otherInstanceId];\n                // intersect! evaluate\n                if (rangesIntersect(subjectRange, otherInstance.range)) {\n                    var otherOverlap = otherConfigs[otherInstance.defId].overlap;\n                    // consider the other event's overlap. only do this if the subject event is a \"real\" event\n                    if (otherOverlap === false && interaction.isEvent) {\n                        return false;\n                    }\n                    if (subjectConfig.overlap === false) {\n                        return false;\n                    }\n                    if (overlapFunc && !overlapFunc(new EventApi(calendar, otherDefs[otherInstance.defId], otherInstance), // still event\n                    new EventApi(calendar, subjectDef, subjectInstance) // moving event\n                    )) {\n                        return false;\n                    }\n                }\n            }\n            // allow (a function)\n            var calendarEventStore = calendar.state.eventStore; // need global-to-calendar, not local to component (splittable)state\n            for (var _i = 0, _a = subjectConfig.allows; _i < _a.length; _i++) {\n                var subjectAllow = _a[_i];\n                var subjectDateSpan = __assign({}, dateSpanMeta, { range: subjectInstance.range, allDay: subjectDef.allDay });\n                var origDef = calendarEventStore.defs[subjectDef.defId];\n                var origInstance = calendarEventStore.instances[subjectInstanceId];\n                var eventApi = void 0;\n                if (origDef) { // was previously in the calendar\n                    eventApi = new EventApi(calendar, origDef, origInstance);\n                }\n                else { // was an external event\n                    eventApi = new EventApi(calendar, subjectDef); // no instance, because had no dates\n                }\n                if (!subjectAllow(calendar.buildDateSpanApi(subjectDateSpan), eventApi)) {\n                    return false;\n                }\n            }\n        }\n        return true;\n    }\n    // Date Selection Validation\n    // ------------------------------------------------------------------------------------------------------------------------\n    function isDateSelectionPropsValid(state, calendar, dateSpanMeta, filterConfig) {\n        var relevantEventStore = state.eventStore;\n        var relevantDefs = relevantEventStore.defs;\n        var relevantInstances = relevantEventStore.instances;\n        var selection = state.dateSelection;\n        var selectionRange = selection.range;\n        var selectionConfig = calendar.selectionConfig;\n        if (filterConfig) {\n            selectionConfig = filterConfig(selectionConfig);\n        }\n        // constraint\n        if (!allConstraintsPass(selectionConfig.constraints, selectionRange, relevantEventStore, state.businessHours, calendar)) {\n            return false;\n        }\n        // overlap\n        var overlapFunc = calendar.opt('selectOverlap');\n        if (typeof overlapFunc !== 'function') {\n            overlapFunc = null;\n        }\n        for (var relevantInstanceId in relevantInstances) {\n            var relevantInstance = relevantInstances[relevantInstanceId];\n            // intersect! evaluate\n            if (rangesIntersect(selectionRange, relevantInstance.range)) {\n                if (selectionConfig.overlap === false) {\n                    return false;\n                }\n                if (overlapFunc && !overlapFunc(new EventApi(calendar, relevantDefs[relevantInstance.defId], relevantInstance))) {\n                    return false;\n                }\n            }\n        }\n        // allow (a function)\n        for (var _i = 0, _a = selectionConfig.allows; _i < _a.length; _i++) {\n            var selectionAllow = _a[_i];\n            var fullDateSpan = __assign({}, dateSpanMeta, selection);\n            if (!selectionAllow(calendar.buildDateSpanApi(fullDateSpan), null)) {\n                return false;\n            }\n        }\n        return true;\n    }\n    // Constraint Utils\n    // ------------------------------------------------------------------------------------------------------------------------\n    function allConstraintsPass(constraints, subjectRange, otherEventStore, businessHoursUnexpanded, calendar) {\n        for (var _i = 0, constraints_1 = constraints; _i < constraints_1.length; _i++) {\n            var constraint = constraints_1[_i];\n            if (!anyRangesContainRange(constraintToRanges(constraint, subjectRange, otherEventStore, businessHoursUnexpanded, calendar), subjectRange)) {\n                return false;\n            }\n        }\n        return true;\n    }\n    function constraintToRanges(constraint, subjectRange, // for expanding a recurring constraint, or expanding business hours\n    otherEventStore, // for if constraint is an even group ID\n    businessHoursUnexpanded, // for if constraint is 'businessHours'\n    calendar // for expanding businesshours\n    ) {\n        if (constraint === 'businessHours') {\n            return eventStoreToRanges(expandRecurring(businessHoursUnexpanded, subjectRange, calendar));\n        }\n        else if (typeof constraint === 'string') { // an group ID\n            return eventStoreToRanges(filterEventStoreDefs(otherEventStore, function (eventDef) {\n                return eventDef.groupId === constraint;\n            }));\n        }\n        else if (typeof constraint === 'object' && constraint) { // non-null object\n            return eventStoreToRanges(expandRecurring(constraint, subjectRange, calendar));\n        }\n        return []; // if it's false\n    }\n    // TODO: move to event-store file?\n    function eventStoreToRanges(eventStore) {\n        var instances = eventStore.instances;\n        var ranges = [];\n        for (var instanceId in instances) {\n            ranges.push(instances[instanceId].range);\n        }\n        return ranges;\n    }\n    // TODO: move to geom file?\n    function anyRangesContainRange(outerRanges, innerRange) {\n        for (var _i = 0, outerRanges_1 = outerRanges; _i < outerRanges_1.length; _i++) {\n            var outerRange = outerRanges_1[_i];\n            if (rangeContainsRange(outerRange, innerRange)) {\n                return true;\n            }\n        }\n        return false;\n    }\n    // Parsing\n    // ------------------------------------------------------------------------------------------------------------------------\n    function normalizeConstraint(input, calendar) {\n        if (Array.isArray(input)) {\n            return parseEvents(input, '', calendar, true); // allowOpenRange=true\n        }\n        else if (typeof input === 'object' && input) { // non-null object\n            return parseEvents([input], '', calendar, true); // allowOpenRange=true\n        }\n        else if (input != null) {\n            return String(input);\n        }\n        else {\n            return null;\n        }\n    }\n\n    function htmlEscape(s) {\n        return (s + '').replace(/&/g, '&amp;')\n            .replace(/</g, '&lt;')\n            .replace(/>/g, '&gt;')\n            .replace(/'/g, '&#039;')\n            .replace(/\"/g, '&quot;')\n            .replace(/\\n/g, '<br />');\n    }\n    // Given a hash of CSS properties, returns a string of CSS.\n    // Uses property names as-is (no camel-case conversion). Will not make statements for null/undefined values.\n    function cssToStr(cssProps) {\n        var statements = [];\n        for (var name_1 in cssProps) {\n            var val = cssProps[name_1];\n            if (val != null && val !== '') {\n                statements.push(name_1 + ':' + val);\n            }\n        }\n        return statements.join(';');\n    }\n    // Given an object hash of HTML attribute names to values,\n    // generates a string that can be injected between < > in HTML\n    function attrsToStr(attrs) {\n        var parts = [];\n        for (var name_2 in attrs) {\n            var val = attrs[name_2];\n            if (val != null) {\n                parts.push(name_2 + '=\"' + htmlEscape(val) + '\"');\n            }\n        }\n        return parts.join(' ');\n    }\n    function parseClassName(raw) {\n        if (Array.isArray(raw)) {\n            return raw;\n        }\n        else if (typeof raw === 'string') {\n            return raw.split(/\\s+/);\n        }\n        else {\n            return [];\n        }\n    }\n\n    var UNSCOPED_EVENT_UI_PROPS = {\n        editable: Boolean,\n        startEditable: Boolean,\n        durationEditable: Boolean,\n        constraint: null,\n        overlap: null,\n        allow: null,\n        className: parseClassName,\n        classNames: parseClassName,\n        color: String,\n        backgroundColor: String,\n        borderColor: String,\n        textColor: String\n    };\n    function processUnscopedUiProps(rawProps, calendar, leftovers) {\n        var props = refineProps(rawProps, UNSCOPED_EVENT_UI_PROPS, {}, leftovers);\n        var constraint = normalizeConstraint(props.constraint, calendar);\n        return {\n            startEditable: props.startEditable != null ? props.startEditable : props.editable,\n            durationEditable: props.durationEditable != null ? props.durationEditable : props.editable,\n            constraints: constraint != null ? [constraint] : [],\n            overlap: props.overlap,\n            allows: props.allow != null ? [props.allow] : [],\n            backgroundColor: props.backgroundColor || props.color,\n            borderColor: props.borderColor || props.color,\n            textColor: props.textColor,\n            classNames: props.classNames.concat(props.className)\n        };\n    }\n    function processScopedUiProps(prefix, rawScoped, calendar, leftovers) {\n        var rawUnscoped = {};\n        var wasFound = {};\n        for (var key in UNSCOPED_EVENT_UI_PROPS) {\n            var scopedKey = prefix + capitaliseFirstLetter(key);\n            rawUnscoped[key] = rawScoped[scopedKey];\n            wasFound[scopedKey] = true;\n        }\n        if (prefix === 'event') {\n            rawUnscoped.editable = rawScoped.editable; // special case. there is no 'eventEditable', just 'editable'\n        }\n        if (leftovers) {\n            for (var key in rawScoped) {\n                if (!wasFound[key]) {\n                    leftovers[key] = rawScoped[key];\n                }\n            }\n        }\n        return processUnscopedUiProps(rawUnscoped, calendar);\n    }\n    var EMPTY_EVENT_UI = {\n        startEditable: null,\n        durationEditable: null,\n        constraints: [],\n        overlap: null,\n        allows: [],\n        backgroundColor: '',\n        borderColor: '',\n        textColor: '',\n        classNames: []\n    };\n    // prevent against problems with <2 args!\n    function combineEventUis(uis) {\n        return uis.reduce(combineTwoEventUis, EMPTY_EVENT_UI);\n    }\n    function combineTwoEventUis(item0, item1) {\n        return {\n            startEditable: item1.startEditable != null ? item1.startEditable : item0.startEditable,\n            durationEditable: item1.durationEditable != null ? item1.durationEditable : item0.durationEditable,\n            constraints: item0.constraints.concat(item1.constraints),\n            overlap: typeof item1.overlap === 'boolean' ? item1.overlap : item0.overlap,\n            allows: item0.allows.concat(item1.allows),\n            backgroundColor: item1.backgroundColor || item0.backgroundColor,\n            borderColor: item1.borderColor || item0.borderColor,\n            textColor: item1.textColor || item0.textColor,\n            classNames: item0.classNames.concat(item1.classNames)\n        };\n    }\n\n    var NON_DATE_PROPS = {\n        id: String,\n        groupId: String,\n        title: String,\n        url: String,\n        rendering: String,\n        extendedProps: null\n    };\n    var DATE_PROPS = {\n        start: null,\n        date: null,\n        end: null,\n        allDay: null\n    };\n    var uid = 0;\n    function parseEvent(raw, sourceId, calendar, allowOpenRange) {\n        var allDayDefault = computeIsAllDayDefault(sourceId, calendar);\n        var leftovers0 = {};\n        var recurringRes = parseRecurring(raw, // raw, but with single-event stuff stripped out\n        allDayDefault, calendar.dateEnv, calendar.pluginSystem.hooks.recurringTypes, leftovers0 // will populate with non-recurring props\n        );\n        if (recurringRes) {\n            var def = parseEventDef(leftovers0, sourceId, recurringRes.allDay, Boolean(recurringRes.duration), calendar);\n            def.recurringDef = {\n                typeId: recurringRes.typeId,\n                typeData: recurringRes.typeData,\n                duration: recurringRes.duration\n            };\n            return { def: def, instance: null };\n        }\n        else {\n            var leftovers1 = {};\n            var singleRes = parseSingle(raw, allDayDefault, calendar, leftovers1, allowOpenRange);\n            if (singleRes) {\n                var def = parseEventDef(leftovers1, sourceId, singleRes.allDay, singleRes.hasEnd, calendar);\n                var instance = createEventInstance(def.defId, singleRes.range, singleRes.forcedStartTzo, singleRes.forcedEndTzo);\n                return { def: def, instance: instance };\n            }\n        }\n        return null;\n    }\n    /*\n    Will NOT populate extendedProps with the leftover properties.\n    Will NOT populate date-related props.\n    The EventNonDateInput has been normalized (id => publicId, etc).\n    */\n    function parseEventDef(raw, sourceId, allDay, hasEnd, calendar) {\n        var leftovers = {};\n        var def = pluckNonDateProps(raw, calendar, leftovers);\n        def.defId = String(uid++);\n        def.sourceId = sourceId;\n        def.allDay = allDay;\n        def.hasEnd = hasEnd;\n        for (var _i = 0, _a = calendar.pluginSystem.hooks.eventDefParsers; _i < _a.length; _i++) {\n            var eventDefParser = _a[_i];\n            var newLeftovers = {};\n            eventDefParser(def, leftovers, newLeftovers);\n            leftovers = newLeftovers;\n        }\n        def.extendedProps = __assign(leftovers, def.extendedProps || {});\n        // help out EventApi from having user modify props\n        Object.freeze(def.ui.classNames);\n        Object.freeze(def.extendedProps);\n        return def;\n    }\n    function createEventInstance(defId, range, forcedStartTzo, forcedEndTzo) {\n        return {\n            instanceId: String(uid++),\n            defId: defId,\n            range: range,\n            forcedStartTzo: forcedStartTzo == null ? null : forcedStartTzo,\n            forcedEndTzo: forcedEndTzo == null ? null : forcedEndTzo\n        };\n    }\n    function parseSingle(raw, allDayDefault, calendar, leftovers, allowOpenRange) {\n        var props = pluckDateProps(raw, leftovers);\n        var allDay = props.allDay;\n        var startMeta;\n        var startMarker = null;\n        var hasEnd = false;\n        var endMeta;\n        var endMarker = null;\n        startMeta = calendar.dateEnv.createMarkerMeta(props.start);\n        if (startMeta) {\n            startMarker = startMeta.marker;\n        }\n        else if (!allowOpenRange) {\n            return null;\n        }\n        if (props.end != null) {\n            endMeta = calendar.dateEnv.createMarkerMeta(props.end);\n        }\n        if (allDay == null) {\n            if (allDayDefault != null) {\n                allDay = allDayDefault;\n            }\n            else {\n                // fall back to the date props LAST\n                allDay = (!startMeta || startMeta.isTimeUnspecified) &&\n                    (!endMeta || endMeta.isTimeUnspecified);\n            }\n        }\n        if (allDay && startMarker) {\n            startMarker = startOfDay(startMarker);\n        }\n        if (endMeta) {\n            endMarker = endMeta.marker;\n            if (allDay) {\n                endMarker = startOfDay(endMarker);\n            }\n            if (startMarker && endMarker <= startMarker) {\n                endMarker = null;\n            }\n        }\n        if (endMarker) {\n            hasEnd = true;\n        }\n        else if (!allowOpenRange) {\n            hasEnd = calendar.opt('forceEventDuration') || false;\n            endMarker = calendar.dateEnv.add(startMarker, allDay ?\n                calendar.defaultAllDayEventDuration :\n                calendar.defaultTimedEventDuration);\n        }\n        return {\n            allDay: allDay,\n            hasEnd: hasEnd,\n            range: { start: startMarker, end: endMarker },\n            forcedStartTzo: startMeta ? startMeta.forcedTzo : null,\n            forcedEndTzo: endMeta ? endMeta.forcedTzo : null\n        };\n    }\n    function pluckDateProps(raw, leftovers) {\n        var props = refineProps(raw, DATE_PROPS, {}, leftovers);\n        props.start = (props.start !== null) ? props.start : props.date;\n        delete props.date;\n        return props;\n    }\n    function pluckNonDateProps(raw, calendar, leftovers) {\n        var preLeftovers = {};\n        var props = refineProps(raw, NON_DATE_PROPS, {}, preLeftovers);\n        var ui = processUnscopedUiProps(preLeftovers, calendar, leftovers);\n        props.publicId = props.id;\n        delete props.id;\n        props.ui = ui;\n        return props;\n    }\n    function computeIsAllDayDefault(sourceId, calendar) {\n        var res = null;\n        if (sourceId) {\n            var source = calendar.state.eventSources[sourceId];\n            res = source.allDayDefault;\n        }\n        if (res == null) {\n            res = calendar.opt('allDayDefault');\n        }\n        return res;\n    }\n\n    var DEF_DEFAULTS = {\n        startTime: '09:00',\n        endTime: '17:00',\n        daysOfWeek: [1, 2, 3, 4, 5],\n        rendering: 'inverse-background',\n        classNames: 'fc-nonbusiness',\n        groupId: '_businessHours' // so multiple defs get grouped\n    };\n    /*\n    TODO: pass around as EventDefHash!!!\n    */\n    function parseBusinessHours(input, calendar) {\n        return parseEvents(refineInputs(input), '', calendar);\n    }\n    function refineInputs(input) {\n        var rawDefs;\n        if (input === true) {\n            rawDefs = [{}]; // will get DEF_DEFAULTS verbatim\n        }\n        else if (Array.isArray(input)) {\n            // if specifying an array, every sub-definition NEEDS a day-of-week\n            rawDefs = input.filter(function (rawDef) {\n                return rawDef.daysOfWeek;\n            });\n        }\n        else if (typeof input === 'object' && input) { // non-null object\n            rawDefs = [input];\n        }\n        else { // is probably false\n            rawDefs = [];\n        }\n        rawDefs = rawDefs.map(function (rawDef) {\n            return __assign({}, DEF_DEFAULTS, rawDef);\n        });\n        return rawDefs;\n    }\n\n    function memoizeRendering(renderFunc, unrenderFunc, dependencies) {\n        if (dependencies === void 0) { dependencies = []; }\n        var dependents = [];\n        var thisContext;\n        var prevArgs;\n        function unrender() {\n            if (prevArgs) {\n                for (var _i = 0, dependents_1 = dependents; _i < dependents_1.length; _i++) {\n                    var dependent = dependents_1[_i];\n                    dependent.unrender();\n                }\n                if (unrenderFunc) {\n                    unrenderFunc.apply(thisContext, prevArgs);\n                }\n                prevArgs = null;\n            }\n        }\n        function res() {\n            if (!prevArgs || !isArraysEqual(prevArgs, arguments)) {\n                unrender();\n                thisContext = this;\n                prevArgs = arguments;\n                renderFunc.apply(this, arguments);\n            }\n        }\n        res.dependents = dependents;\n        res.unrender = unrender;\n        for (var _i = 0, dependencies_1 = dependencies; _i < dependencies_1.length; _i++) {\n            var dependency = dependencies_1[_i];\n            dependency.dependents.push(res);\n        }\n        return res;\n    }\n\n    var EMPTY_EVENT_STORE = createEmptyEventStore(); // for purecomponents. TODO: keep elsewhere\n    var Splitter = /** @class */ (function () {\n        function Splitter() {\n            this.getKeysForEventDefs = memoize(this._getKeysForEventDefs);\n            this.splitDateSelection = memoize(this._splitDateSpan);\n            this.splitEventStore = memoize(this._splitEventStore);\n            this.splitIndividualUi = memoize(this._splitIndividualUi);\n            this.splitEventDrag = memoize(this._splitInteraction);\n            this.splitEventResize = memoize(this._splitInteraction);\n            this.eventUiBuilders = {}; // TODO: typescript protection\n        }\n        Splitter.prototype.splitProps = function (props) {\n            var _this = this;\n            var keyInfos = this.getKeyInfo(props);\n            var defKeys = this.getKeysForEventDefs(props.eventStore);\n            var dateSelections = this.splitDateSelection(props.dateSelection);\n            var individualUi = this.splitIndividualUi(props.eventUiBases, defKeys); // the individual *bases*\n            var eventStores = this.splitEventStore(props.eventStore, defKeys);\n            var eventDrags = this.splitEventDrag(props.eventDrag);\n            var eventResizes = this.splitEventResize(props.eventResize);\n            var splitProps = {};\n            this.eventUiBuilders = mapHash(keyInfos, function (info, key) {\n                return _this.eventUiBuilders[key] || memoize(buildEventUiForKey);\n            });\n            for (var key in keyInfos) {\n                var keyInfo = keyInfos[key];\n                var eventStore = eventStores[key] || EMPTY_EVENT_STORE;\n                var buildEventUi = this.eventUiBuilders[key];\n                splitProps[key] = {\n                    businessHours: keyInfo.businessHours || props.businessHours,\n                    dateSelection: dateSelections[key] || null,\n                    eventStore: eventStore,\n                    eventUiBases: buildEventUi(props.eventUiBases[''], keyInfo.ui, individualUi[key]),\n                    eventSelection: eventStore.instances[props.eventSelection] ? props.eventSelection : '',\n                    eventDrag: eventDrags[key] || null,\n                    eventResize: eventResizes[key] || null\n                };\n            }\n            return splitProps;\n        };\n        Splitter.prototype._splitDateSpan = function (dateSpan) {\n            var dateSpans = {};\n            if (dateSpan) {\n                var keys = this.getKeysForDateSpan(dateSpan);\n                for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {\n                    var key = keys_1[_i];\n                    dateSpans[key] = dateSpan;\n                }\n            }\n            return dateSpans;\n        };\n        Splitter.prototype._getKeysForEventDefs = function (eventStore) {\n            var _this = this;\n            return mapHash(eventStore.defs, function (eventDef) {\n                return _this.getKeysForEventDef(eventDef);\n            });\n        };\n        Splitter.prototype._splitEventStore = function (eventStore, defKeys) {\n            var defs = eventStore.defs, instances = eventStore.instances;\n            var splitStores = {};\n            for (var defId in defs) {\n                for (var _i = 0, _a = defKeys[defId]; _i < _a.length; _i++) {\n                    var key = _a[_i];\n                    if (!splitStores[key]) {\n                        splitStores[key] = createEmptyEventStore();\n                    }\n                    splitStores[key].defs[defId] = defs[defId];\n                }\n            }\n            for (var instanceId in instances) {\n                var instance = instances[instanceId];\n                for (var _b = 0, _c = defKeys[instance.defId]; _b < _c.length; _b++) {\n                    var key = _c[_b];\n                    if (splitStores[key]) { // must have already been created\n                        splitStores[key].instances[instanceId] = instance;\n                    }\n                }\n            }\n            return splitStores;\n        };\n        Splitter.prototype._splitIndividualUi = function (eventUiBases, defKeys) {\n            var splitHashes = {};\n            for (var defId in eventUiBases) {\n                if (defId) { // not the '' key\n                    for (var _i = 0, _a = defKeys[defId]; _i < _a.length; _i++) {\n                        var key = _a[_i];\n                        if (!splitHashes[key]) {\n                            splitHashes[key] = {};\n                        }\n                        splitHashes[key][defId] = eventUiBases[defId];\n                    }\n                }\n            }\n            return splitHashes;\n        };\n        Splitter.prototype._splitInteraction = function (interaction) {\n            var splitStates = {};\n            if (interaction) {\n                var affectedStores_1 = this._splitEventStore(interaction.affectedEvents, this._getKeysForEventDefs(interaction.affectedEvents) // can't use cached. might be events from other calendar\n                );\n                // can't rely on defKeys because event data is mutated\n                var mutatedKeysByDefId = this._getKeysForEventDefs(interaction.mutatedEvents);\n                var mutatedStores_1 = this._splitEventStore(interaction.mutatedEvents, mutatedKeysByDefId);\n                var populate = function (key) {\n                    if (!splitStates[key]) {\n                        splitStates[key] = {\n                            affectedEvents: affectedStores_1[key] || EMPTY_EVENT_STORE,\n                            mutatedEvents: mutatedStores_1[key] || EMPTY_EVENT_STORE,\n                            isEvent: interaction.isEvent,\n                            origSeg: interaction.origSeg\n                        };\n                    }\n                };\n                for (var key in affectedStores_1) {\n                    populate(key);\n                }\n                for (var key in mutatedStores_1) {\n                    populate(key);\n                }\n            }\n            return splitStates;\n        };\n        return Splitter;\n    }());\n    function buildEventUiForKey(allUi, eventUiForKey, individualUi) {\n        var baseParts = [];\n        if (allUi) {\n            baseParts.push(allUi);\n        }\n        if (eventUiForKey) {\n            baseParts.push(eventUiForKey);\n        }\n        var stuff = {\n            '': combineEventUis(baseParts)\n        };\n        if (individualUi) {\n            __assign(stuff, individualUi);\n        }\n        return stuff;\n    }\n\n    // Generates HTML for an anchor to another view into the calendar.\n    // Will either generate an <a> tag or a non-clickable <span> tag, depending on enabled settings.\n    // `gotoOptions` can either be a DateMarker, or an object with the form:\n    // { date, type, forceOff }\n    // `type` is a view-type like \"day\" or \"week\". default value is \"day\".\n    // `attrs` and `innerHtml` are use to generate the rest of the HTML tag.\n    function buildGotoAnchorHtml(allOptions, dateEnv, gotoOptions, attrs, innerHtml) {\n        var date;\n        var type;\n        var forceOff;\n        var finalOptions;\n        if (gotoOptions instanceof Date) {\n            date = gotoOptions; // a single date-like input\n        }\n        else {\n            date = gotoOptions.date;\n            type = gotoOptions.type;\n            forceOff = gotoOptions.forceOff;\n        }\n        finalOptions = {\n            date: dateEnv.formatIso(date, { omitTime: true }),\n            type: type || 'day'\n        };\n        if (typeof attrs === 'string') {\n            innerHtml = attrs;\n            attrs = null;\n        }\n        attrs = attrs ? ' ' + attrsToStr(attrs) : ''; // will have a leading space\n        innerHtml = innerHtml || '';\n        if (!forceOff && allOptions.navLinks) {\n            return '<a' + attrs +\n                ' data-goto=\"' + htmlEscape(JSON.stringify(finalOptions)) + '\">' +\n                innerHtml +\n                '</a>';\n        }\n        else {\n            return '<span' + attrs + '>' +\n                innerHtml +\n                '</span>';\n        }\n    }\n    function getAllDayHtml(allOptions) {\n        return allOptions.allDayHtml || htmlEscape(allOptions.allDayText);\n    }\n    // Computes HTML classNames for a single-day element\n    function getDayClasses(date, dateProfile, context, noThemeHighlight) {\n        var calendar = context.calendar, options = context.options, theme = context.theme, dateEnv = context.dateEnv;\n        var classes = [];\n        var todayStart;\n        var todayEnd;\n        if (!rangeContainsMarker(dateProfile.activeRange, date)) {\n            classes.push('fc-disabled-day');\n        }\n        else {\n            classes.push('fc-' + DAY_IDS[date.getUTCDay()]);\n            if (options.monthMode &&\n                dateEnv.getMonth(date) !== dateEnv.getMonth(dateProfile.currentRange.start)) {\n                classes.push('fc-other-month');\n            }\n            todayStart = startOfDay(calendar.getNow());\n            todayEnd = addDays(todayStart, 1);\n            if (date < todayStart) {\n                classes.push('fc-past');\n            }\n            else if (date >= todayEnd) {\n                classes.push('fc-future');\n            }\n            else {\n                classes.push('fc-today');\n                if (noThemeHighlight !== true) {\n                    classes.push(theme.getClass('today'));\n                }\n            }\n        }\n        return classes;\n    }\n\n    // given a function that resolves a result asynchronously.\n    // the function can either call passed-in success and failure callbacks,\n    // or it can return a promise.\n    // if you need to pass additional params to func, bind them first.\n    function unpromisify(func, success, failure) {\n        // guard against success/failure callbacks being called more than once\n        // and guard against a promise AND callback being used together.\n        var isResolved = false;\n        var wrappedSuccess = function () {\n            if (!isResolved) {\n                isResolved = true;\n                success.apply(this, arguments);\n            }\n        };\n        var wrappedFailure = function () {\n            if (!isResolved) {\n                isResolved = true;\n                if (failure) {\n                    failure.apply(this, arguments);\n                }\n            }\n        };\n        var res = func(wrappedSuccess, wrappedFailure);\n        if (res && typeof res.then === 'function') {\n            res.then(wrappedSuccess, wrappedFailure);\n        }\n    }\n\n    var Mixin = /** @class */ (function () {\n        function Mixin() {\n        }\n        // mix into a CLASS\n        Mixin.mixInto = function (destClass) {\n            this.mixIntoObj(destClass.prototype);\n        };\n        // mix into ANY object\n        Mixin.mixIntoObj = function (destObj) {\n            var _this = this;\n            Object.getOwnPropertyNames(this.prototype).forEach(function (name) {\n                if (!destObj[name]) { // if destination doesn't already define it\n                    destObj[name] = _this.prototype[name];\n                }\n            });\n        };\n        /*\n        will override existing methods\n        TODO: remove! not used anymore\n        */\n        Mixin.mixOver = function (destClass) {\n            var _this = this;\n            Object.getOwnPropertyNames(this.prototype).forEach(function (name) {\n                destClass.prototype[name] = _this.prototype[name];\n            });\n        };\n        return Mixin;\n    }());\n\n    /*\n    USAGE:\n      import { default as EmitterMixin, EmitterInterface } from './EmitterMixin'\n    in class:\n      on: EmitterInterface['on']\n      one: EmitterInterface['one']\n      off: EmitterInterface['off']\n      trigger: EmitterInterface['trigger']\n      triggerWith: EmitterInterface['triggerWith']\n      hasHandlers: EmitterInterface['hasHandlers']\n    after class:\n      EmitterMixin.mixInto(TheClass)\n    */\n    var EmitterMixin = /** @class */ (function (_super) {\n        __extends(EmitterMixin, _super);\n        function EmitterMixin() {\n            return _super !== null && _super.apply(this, arguments) || this;\n        }\n        EmitterMixin.prototype.on = function (type, handler) {\n            addToHash(this._handlers || (this._handlers = {}), type, handler);\n            return this; // for chaining\n        };\n        // todo: add comments\n        EmitterMixin.prototype.one = function (type, handler) {\n            addToHash(this._oneHandlers || (this._oneHandlers = {}), type, handler);\n            return this; // for chaining\n        };\n        EmitterMixin.prototype.off = function (type, handler) {\n            if (this._handlers) {\n                removeFromHash(this._handlers, type, handler);\n            }\n            if (this._oneHandlers) {\n                removeFromHash(this._oneHandlers, type, handler);\n            }\n            return this; // for chaining\n        };\n        EmitterMixin.prototype.trigger = function (type) {\n            var args = [];\n            for (var _i = 1; _i < arguments.length; _i++) {\n                args[_i - 1] = arguments[_i];\n            }\n            this.triggerWith(type, this, args);\n            return this; // for chaining\n        };\n        EmitterMixin.prototype.triggerWith = function (type, context, args) {\n            if (this._handlers) {\n                applyAll(this._handlers[type], context, args);\n            }\n            if (this._oneHandlers) {\n                applyAll(this._oneHandlers[type], context, args);\n                delete this._oneHandlers[type]; // will never fire again\n            }\n            return this; // for chaining\n        };\n        EmitterMixin.prototype.hasHandlers = function (type) {\n            return (this._handlers && this._handlers[type] && this._handlers[type].length) ||\n                (this._oneHandlers && this._oneHandlers[type] && this._oneHandlers[type].length);\n        };\n        return EmitterMixin;\n    }(Mixin));\n    function addToHash(hash, type, handler) {\n        (hash[type] || (hash[type] = []))\n            .push(handler);\n    }\n    function removeFromHash(hash, type, handler) {\n        if (handler) {\n            if (hash[type]) {\n                hash[type] = hash[type].filter(function (func) {\n                    return func !== handler;\n                });\n            }\n        }\n        else {\n            delete hash[type]; // remove all handler funcs for this type\n        }\n    }\n\n    /*\n    Records offset information for a set of elements, relative to an origin element.\n    Can record the left/right OR the top/bottom OR both.\n    Provides methods for querying the cache by position.\n    */\n    var PositionCache = /** @class */ (function () {\n        function PositionCache(originEl, els, isHorizontal, isVertical) {\n            this.originEl = originEl;\n            this.els = els;\n            this.isHorizontal = isHorizontal;\n            this.isVertical = isVertical;\n        }\n        // Queries the els for coordinates and stores them.\n        // Call this method before using and of the get* methods below.\n        PositionCache.prototype.build = function () {\n            var originEl = this.originEl;\n            var originClientRect = this.originClientRect =\n                originEl.getBoundingClientRect(); // relative to viewport top-left\n            if (this.isHorizontal) {\n                this.buildElHorizontals(originClientRect.left);\n            }\n            if (this.isVertical) {\n                this.buildElVerticals(originClientRect.top);\n            }\n        };\n        // Populates the left/right internal coordinate arrays\n        PositionCache.prototype.buildElHorizontals = function (originClientLeft) {\n            var lefts = [];\n            var rights = [];\n            for (var _i = 0, _a = this.els; _i < _a.length; _i++) {\n                var el = _a[_i];\n                var rect = el.getBoundingClientRect();\n                lefts.push(rect.left - originClientLeft);\n                rights.push(rect.right - originClientLeft);\n            }\n            this.lefts = lefts;\n            this.rights = rights;\n        };\n        // Populates the top/bottom internal coordinate arrays\n        PositionCache.prototype.buildElVerticals = function (originClientTop) {\n            var tops = [];\n            var bottoms = [];\n            for (var _i = 0, _a = this.els; _i < _a.length; _i++) {\n                var el = _a[_i];\n                var rect = el.getBoundingClientRect();\n                tops.push(rect.top - originClientTop);\n                bottoms.push(rect.bottom - originClientTop);\n            }\n            this.tops = tops;\n            this.bottoms = bottoms;\n        };\n        // Given a left offset (from document left), returns the index of the el that it horizontally intersects.\n        // If no intersection is made, returns undefined.\n        PositionCache.prototype.leftToIndex = function (leftPosition) {\n            var lefts = this.lefts;\n            var rights = this.rights;\n            var len = lefts.length;\n            var i;\n            for (i = 0; i < len; i++) {\n                if (leftPosition >= lefts[i] && leftPosition < rights[i]) {\n                    return i;\n                }\n            }\n        };\n        // Given a top offset (from document top), returns the index of the el that it vertically intersects.\n        // If no intersection is made, returns undefined.\n        PositionCache.prototype.topToIndex = function (topPosition) {\n            var tops = this.tops;\n            var bottoms = this.bottoms;\n            var len = tops.length;\n            var i;\n            for (i = 0; i < len; i++) {\n                if (topPosition >= tops[i] && topPosition < bottoms[i]) {\n                    return i;\n                }\n            }\n        };\n        // Gets the width of the element at the given index\n        PositionCache.prototype.getWidth = function (leftIndex) {\n            return this.rights[leftIndex] - this.lefts[leftIndex];\n        };\n        // Gets the height of the element at the given index\n        PositionCache.prototype.getHeight = function (topIndex) {\n            return this.bottoms[topIndex] - this.tops[topIndex];\n        };\n        return PositionCache;\n    }());\n\n    /*\n    An object for getting/setting scroll-related information for an element.\n    Internally, this is done very differently for window versus DOM element,\n    so this object serves as a common interface.\n    */\n    var ScrollController = /** @class */ (function () {\n        function ScrollController() {\n        }\n        ScrollController.prototype.getMaxScrollTop = function () {\n            return this.getScrollHeight() - this.getClientHeight();\n        };\n        ScrollController.prototype.getMaxScrollLeft = function () {\n            return this.getScrollWidth() - this.getClientWidth();\n        };\n        ScrollController.prototype.canScrollVertically = function () {\n            return this.getMaxScrollTop() > 0;\n        };\n        ScrollController.prototype.canScrollHorizontally = function () {\n            return this.getMaxScrollLeft() > 0;\n        };\n        ScrollController.prototype.canScrollUp = function () {\n            return this.getScrollTop() > 0;\n        };\n        ScrollController.prototype.canScrollDown = function () {\n            return this.getScrollTop() < this.getMaxScrollTop();\n        };\n        ScrollController.prototype.canScrollLeft = function () {\n            return this.getScrollLeft() > 0;\n        };\n        ScrollController.prototype.canScrollRight = function () {\n            return this.getScrollLeft() < this.getMaxScrollLeft();\n        };\n        return ScrollController;\n    }());\n    var ElementScrollController = /** @class */ (function (_super) {\n        __extends(ElementScrollController, _super);\n        function ElementScrollController(el) {\n            var _this = _super.call(this) || this;\n            _this.el = el;\n            return _this;\n        }\n        ElementScrollController.prototype.getScrollTop = function () {\n            return this.el.scrollTop;\n        };\n        ElementScrollController.prototype.getScrollLeft = function () {\n            return this.el.scrollLeft;\n        };\n        ElementScrollController.prototype.setScrollTop = function (top) {\n            this.el.scrollTop = top;\n        };\n        ElementScrollController.prototype.setScrollLeft = function (left) {\n            this.el.scrollLeft = left;\n        };\n        ElementScrollController.prototype.getScrollWidth = function () {\n            return this.el.scrollWidth;\n        };\n        ElementScrollController.prototype.getScrollHeight = function () {\n            return this.el.scrollHeight;\n        };\n        ElementScrollController.prototype.getClientHeight = function () {\n            return this.el.clientHeight;\n        };\n        ElementScrollController.prototype.getClientWidth = function () {\n            return this.el.clientWidth;\n        };\n        return ElementScrollController;\n    }(ScrollController));\n    var WindowScrollController = /** @class */ (function (_super) {\n        __extends(WindowScrollController, _super);\n        function WindowScrollController() {\n            return _super !== null && _super.apply(this, arguments) || this;\n        }\n        WindowScrollController.prototype.getScrollTop = function () {\n            return window.pageYOffset;\n        };\n        WindowScrollController.prototype.getScrollLeft = function () {\n            return window.pageXOffset;\n        };\n        WindowScrollController.prototype.setScrollTop = function (n) {\n            window.scroll(window.pageXOffset, n);\n        };\n        WindowScrollController.prototype.setScrollLeft = function (n) {\n            window.scroll(n, window.pageYOffset);\n        };\n        WindowScrollController.prototype.getScrollWidth = function () {\n            return document.documentElement.scrollWidth;\n        };\n        WindowScrollController.prototype.getScrollHeight = function () {\n            return document.documentElement.scrollHeight;\n        };\n        WindowScrollController.prototype.getClientHeight = function () {\n            return document.documentElement.clientHeight;\n        };\n        WindowScrollController.prototype.getClientWidth = function () {\n            return document.documentElement.clientWidth;\n        };\n        return WindowScrollController;\n    }(ScrollController));\n\n    /*\n    Embodies a div that has potential scrollbars\n    */\n    var ScrollComponent = /** @class */ (function (_super) {\n        __extends(ScrollComponent, _super);\n        function ScrollComponent(overflowX, overflowY) {\n            var _this = _super.call(this, createElement('div', {\n                className: 'fc-scroller'\n            })) || this;\n            _this.overflowX = overflowX;\n            _this.overflowY = overflowY;\n            _this.applyOverflow();\n            return _this;\n        }\n        // sets to natural height, unlocks overflow\n        ScrollComponent.prototype.clear = function () {\n            this.setHeight('auto');\n            this.applyOverflow();\n        };\n        ScrollComponent.prototype.destroy = function () {\n            removeElement(this.el);\n        };\n        // Overflow\n        // -----------------------------------------------------------------------------------------------------------------\n        ScrollComponent.prototype.applyOverflow = function () {\n            applyStyle(this.el, {\n                overflowX: this.overflowX,\n                overflowY: this.overflowY\n            });\n        };\n        // Causes any 'auto' overflow values to resolves to 'scroll' or 'hidden'.\n        // Useful for preserving scrollbar widths regardless of future resizes.\n        // Can pass in scrollbarWidths for optimization.\n        ScrollComponent.prototype.lockOverflow = function (scrollbarWidths) {\n            var overflowX = this.overflowX;\n            var overflowY = this.overflowY;\n            scrollbarWidths = scrollbarWidths || this.getScrollbarWidths();\n            if (overflowX === 'auto') {\n                overflowX = (scrollbarWidths.bottom || // horizontal scrollbars?\n                    this.canScrollHorizontally() // OR scrolling pane with massless scrollbars?\n                ) ? 'scroll' : 'hidden';\n            }\n            if (overflowY === 'auto') {\n                overflowY = (scrollbarWidths.left || scrollbarWidths.right || // horizontal scrollbars?\n                    this.canScrollVertically() // OR scrolling pane with massless scrollbars?\n                ) ? 'scroll' : 'hidden';\n            }\n            applyStyle(this.el, { overflowX: overflowX, overflowY: overflowY });\n        };\n        ScrollComponent.prototype.setHeight = function (height) {\n            applyStyleProp(this.el, 'height', height);\n        };\n        ScrollComponent.prototype.getScrollbarWidths = function () {\n            var edges = computeEdges(this.el);\n            return {\n                left: edges.scrollbarLeft,\n                right: edges.scrollbarRight,\n                bottom: edges.scrollbarBottom\n            };\n        };\n        return ScrollComponent;\n    }(ElementScrollController));\n\n    var Theme = /** @class */ (function () {\n        function Theme(calendarOptions) {\n            this.calendarOptions = calendarOptions;\n            this.processIconOverride();\n        }\n        Theme.prototype.processIconOverride = function () {\n            if (this.iconOverrideOption) {\n                this.setIconOverride(this.calendarOptions[this.iconOverrideOption]);\n            }\n        };\n        Theme.prototype.setIconOverride = function (iconOverrideHash) {\n            var iconClassesCopy;\n            var buttonName;\n            if (typeof iconOverrideHash === 'object' && iconOverrideHash) { // non-null object\n                iconClassesCopy = __assign({}, this.iconClasses);\n                for (buttonName in iconOverrideHash) {\n                    iconClassesCopy[buttonName] = this.applyIconOverridePrefix(iconOverrideHash[buttonName]);\n                }\n                this.iconClasses = iconClassesCopy;\n            }\n            else if (iconOverrideHash === false) {\n                this.iconClasses = {};\n            }\n        };\n        Theme.prototype.applyIconOverridePrefix = function (className) {\n            var prefix = this.iconOverridePrefix;\n            if (prefix && className.indexOf(prefix) !== 0) { // if not already present\n                className = prefix + className;\n            }\n            return className;\n        };\n        Theme.prototype.getClass = function (key) {\n            return this.classes[key] || '';\n        };\n        Theme.prototype.getIconClass = function (buttonName) {\n            var className = this.iconClasses[buttonName];\n            if (className) {\n                return this.baseIconClass + ' ' + className;\n            }\n            return '';\n        };\n        Theme.prototype.getCustomButtonIconClass = function (customButtonProps) {\n            var className;\n            if (this.iconOverrideCustomButtonOption) {\n                className = customButtonProps[this.iconOverrideCustomButtonOption];\n                if (className) {\n                    return this.baseIconClass + ' ' + this.applyIconOverridePrefix(className);\n                }\n            }\n            return '';\n        };\n        return Theme;\n    }());\n    Theme.prototype.classes = {};\n    Theme.prototype.iconClasses = {};\n    Theme.prototype.baseIconClass = '';\n    Theme.prototype.iconOverridePrefix = '';\n\n    var guid = 0;\n    var ComponentContext = /** @class */ (function () {\n        function ComponentContext(calendar, theme, dateEnv, options, view) {\n            this.calendar = calendar;\n            this.theme = theme;\n            this.dateEnv = dateEnv;\n            this.options = options;\n            this.view = view;\n            this.isRtl = options.dir === 'rtl';\n            this.eventOrderSpecs = parseFieldSpecs(options.eventOrder);\n            this.nextDayThreshold = createDuration(options.nextDayThreshold);\n        }\n        ComponentContext.prototype.extend = function (options, view) {\n            return new ComponentContext(this.calendar, this.theme, this.dateEnv, options || this.options, view || this.view);\n        };\n        return ComponentContext;\n    }());\n    var Component = /** @class */ (function () {\n        function Component() {\n            this.everRendered = false;\n            this.uid = String(guid++);\n        }\n        Component.addEqualityFuncs = function (newFuncs) {\n            this.prototype.equalityFuncs = __assign({}, this.prototype.equalityFuncs, newFuncs);\n        };\n        Component.prototype.receiveProps = function (props, context) {\n            this.receiveContext(context);\n            var _a = recycleProps(this.props || {}, props, this.equalityFuncs), anyChanges = _a.anyChanges, comboProps = _a.comboProps;\n            this.props = comboProps;\n            if (anyChanges) {\n                if (this.everRendered) {\n                    this.beforeUpdate();\n                }\n                this.render(comboProps, context);\n                if (this.everRendered) {\n                    this.afterUpdate();\n                }\n            }\n            this.everRendered = true;\n        };\n        Component.prototype.receiveContext = function (context) {\n            var oldContext = this.context;\n            this.context = context;\n            if (!oldContext) {\n                this.firstContext(context);\n            }\n        };\n        Component.prototype.render = function (props, context) {\n        };\n        Component.prototype.firstContext = function (context) {\n        };\n        Component.prototype.beforeUpdate = function () {\n        };\n        Component.prototype.afterUpdate = function () {\n        };\n        // after destroy is called, this component won't ever be used again\n        Component.prototype.destroy = function () {\n        };\n        return Component;\n    }());\n    Component.prototype.equalityFuncs = {};\n    /*\n    Reuses old values when equal. If anything is unequal, returns newProps as-is.\n    Great for PureComponent, but won't be feasible with React, so just eliminate and use React's DOM diffing.\n    */\n    function recycleProps(oldProps, newProps, equalityFuncs) {\n        var comboProps = {}; // some old, some new\n        var anyChanges = false;\n        for (var key in newProps) {\n            if (key in oldProps && (oldProps[key] === newProps[key] ||\n                (equalityFuncs[key] && equalityFuncs[key](oldProps[key], newProps[key])))) {\n                // equal to old? use old prop\n                comboProps[key] = oldProps[key];\n            }\n            else {\n                comboProps[key] = newProps[key];\n                anyChanges = true;\n            }\n        }\n        for (var key in oldProps) {\n            if (!(key in newProps)) {\n                anyChanges = true;\n                break;\n            }\n        }\n        return { anyChanges: anyChanges, comboProps: comboProps };\n    }\n\n    /*\n    PURPOSES:\n    - hook up to fg, fill, and mirror renderers\n    - interface for dragging and hits\n    */\n    var DateComponent = /** @class */ (function (_super) {\n        __extends(DateComponent, _super);\n        function DateComponent(el) {\n            var _this = _super.call(this) || this;\n            _this.el = el;\n            return _this;\n        }\n        DateComponent.prototype.destroy = function () {\n            _super.prototype.destroy.call(this);\n            removeElement(this.el);\n        };\n        // Hit System\n        // -----------------------------------------------------------------------------------------------------------------\n        DateComponent.prototype.buildPositionCaches = function () {\n        };\n        DateComponent.prototype.queryHit = function (positionLeft, positionTop, elWidth, elHeight) {\n            return null; // this should be abstract\n        };\n        // Validation\n        // -----------------------------------------------------------------------------------------------------------------\n        DateComponent.prototype.isInteractionValid = function (interaction) {\n            var calendar = this.context.calendar;\n            var dateProfile = this.props.dateProfile; // HACK\n            var instances = interaction.mutatedEvents.instances;\n            if (dateProfile) { // HACK for DayTile\n                for (var instanceId in instances) {\n                    if (!rangeContainsRange(dateProfile.validRange, instances[instanceId].range)) {\n                        return false;\n                    }\n                }\n            }\n            return isInteractionValid(interaction, calendar);\n        };\n        DateComponent.prototype.isDateSelectionValid = function (selection) {\n            var calendar = this.context.calendar;\n            var dateProfile = this.props.dateProfile; // HACK\n            if (dateProfile && // HACK for DayTile\n                !rangeContainsRange(dateProfile.validRange, selection.range)) {\n                return false;\n            }\n            return isDateSelectionValid(selection, calendar);\n        };\n        // Pointer Interaction Utils\n        // -----------------------------------------------------------------------------------------------------------------\n        DateComponent.prototype.isValidSegDownEl = function (el) {\n            return !this.props.eventDrag && // HACK\n                !this.props.eventResize && // HACK\n                !elementClosest(el, '.fc-mirror') &&\n                (this.isPopover() || !this.isInPopover(el));\n            // ^above line ensures we don't detect a seg interaction within a nested component.\n            // it's a HACK because it only supports a popover as the nested component.\n        };\n        DateComponent.prototype.isValidDateDownEl = function (el) {\n            var segEl = elementClosest(el, this.fgSegSelector);\n            return (!segEl || segEl.classList.contains('fc-mirror')) &&\n                !elementClosest(el, '.fc-more') && // a \"more..\" link\n                !elementClosest(el, 'a[data-goto]') && // a clickable nav link\n                !this.isInPopover(el);\n        };\n        DateComponent.prototype.isPopover = function () {\n            return this.el.classList.contains('fc-popover');\n        };\n        DateComponent.prototype.isInPopover = function (el) {\n            return Boolean(elementClosest(el, '.fc-popover'));\n        };\n        return DateComponent;\n    }(Component));\n    DateComponent.prototype.fgSegSelector = '.fc-event-container > *';\n    DateComponent.prototype.bgSegSelector = '.fc-bgevent:not(.fc-nonbusiness)';\n\n    var uid$1 = 0;\n    function createPlugin(input) {\n        return {\n            id: String(uid$1++),\n            deps: input.deps || [],\n            reducers: input.reducers || [],\n            eventDefParsers: input.eventDefParsers || [],\n            isDraggableTransformers: input.isDraggableTransformers || [],\n            eventDragMutationMassagers: input.eventDragMutationMassagers || [],\n            eventDefMutationAppliers: input.eventDefMutationAppliers || [],\n            dateSelectionTransformers: input.dateSelectionTransformers || [],\n            datePointTransforms: input.datePointTransforms || [],\n            dateSpanTransforms: input.dateSpanTransforms || [],\n            views: input.views || {},\n            viewPropsTransformers: input.viewPropsTransformers || [],\n            isPropsValid: input.isPropsValid || null,\n            externalDefTransforms: input.externalDefTransforms || [],\n            eventResizeJoinTransforms: input.eventResizeJoinTransforms || [],\n            viewContainerModifiers: input.viewContainerModifiers || [],\n            eventDropTransformers: input.eventDropTransformers || [],\n            componentInteractions: input.componentInteractions || [],\n            calendarInteractions: input.calendarInteractions || [],\n            themeClasses: input.themeClasses || {},\n            eventSourceDefs: input.eventSourceDefs || [],\n            cmdFormatter: input.cmdFormatter,\n            recurringTypes: input.recurringTypes || [],\n            namedTimeZonedImpl: input.namedTimeZonedImpl,\n            defaultView: input.defaultView || '',\n            elementDraggingImpl: input.elementDraggingImpl,\n            optionChangeHandlers: input.optionChangeHandlers || {}\n        };\n    }\n    var PluginSystem = /** @class */ (function () {\n        function PluginSystem() {\n            this.hooks = {\n                reducers: [],\n                eventDefParsers: [],\n                isDraggableTransformers: [],\n                eventDragMutationMassagers: [],\n                eventDefMutationAppliers: [],\n                dateSelectionTransformers: [],\n                datePointTransforms: [],\n                dateSpanTransforms: [],\n                views: {},\n                viewPropsTransformers: [],\n                isPropsValid: null,\n                externalDefTransforms: [],\n                eventResizeJoinTransforms: [],\n                viewContainerModifiers: [],\n                eventDropTransformers: [],\n                componentInteractions: [],\n                calendarInteractions: [],\n                themeClasses: {},\n                eventSourceDefs: [],\n                cmdFormatter: null,\n                recurringTypes: [],\n                namedTimeZonedImpl: null,\n                defaultView: '',\n                elementDraggingImpl: null,\n                optionChangeHandlers: {}\n            };\n            this.addedHash = {};\n        }\n        PluginSystem.prototype.add = function (plugin) {\n            if (!this.addedHash[plugin.id]) {\n                this.addedHash[plugin.id] = true;\n                for (var _i = 0, _a = plugin.deps; _i < _a.length; _i++) {\n                    var dep = _a[_i];\n                    this.add(dep);\n                }\n                this.hooks = combineHooks(this.hooks, plugin);\n            }\n        };\n        return PluginSystem;\n    }());\n    function combineHooks(hooks0, hooks1) {\n        return {\n            reducers: hooks0.reducers.concat(hooks1.reducers),\n            eventDefParsers: hooks0.eventDefParsers.concat(hooks1.eventDefParsers),\n            isDraggableTransformers: hooks0.isDraggableTransformers.concat(hooks1.isDraggableTransformers),\n            eventDragMutationMassagers: hooks0.eventDragMutationMassagers.concat(hooks1.eventDragMutationMassagers),\n            eventDefMutationAppliers: hooks0.eventDefMutationAppliers.concat(hooks1.eventDefMutationAppliers),\n            dateSelectionTransformers: hooks0.dateSelectionTransformers.concat(hooks1.dateSelectionTransformers),\n            datePointTransforms: hooks0.datePointTransforms.concat(hooks1.datePointTransforms),\n            dateSpanTransforms: hooks0.dateSpanTransforms.concat(hooks1.dateSpanTransforms),\n            views: __assign({}, hooks0.views, hooks1.views),\n            viewPropsTransformers: hooks0.viewPropsTransformers.concat(hooks1.viewPropsTransformers),\n            isPropsValid: hooks1.isPropsValid || hooks0.isPropsValid,\n            externalDefTransforms: hooks0.externalDefTransforms.concat(hooks1.externalDefTransforms),\n            eventResizeJoinTransforms: hooks0.eventResizeJoinTransforms.concat(hooks1.eventResizeJoinTransforms),\n            viewContainerModifiers: hooks0.viewContainerModifiers.concat(hooks1.viewContainerModifiers),\n            eventDropTransformers: hooks0.eventDropTransformers.concat(hooks1.eventDropTransformers),\n            calendarInteractions: hooks0.calendarInteractions.concat(hooks1.calendarInteractions),\n            componentInteractions: hooks0.componentInteractions.concat(hooks1.componentInteractions),\n            themeClasses: __assign({}, hooks0.themeClasses, hooks1.themeClasses),\n            eventSourceDefs: hooks0.eventSourceDefs.concat(hooks1.eventSourceDefs),\n            cmdFormatter: hooks1.cmdFormatter || hooks0.cmdFormatter,\n            recurringTypes: hooks0.recurringTypes.concat(hooks1.recurringTypes),\n            namedTimeZonedImpl: hooks1.namedTimeZonedImpl || hooks0.namedTimeZonedImpl,\n            defaultView: hooks0.defaultView || hooks1.defaultView,\n            elementDraggingImpl: hooks0.elementDraggingImpl || hooks1.elementDraggingImpl,\n            optionChangeHandlers: __assign({}, hooks0.optionChangeHandlers, hooks1.optionChangeHandlers)\n        };\n    }\n\n    var eventSourceDef = {\n        ignoreRange: true,\n        parseMeta: function (raw) {\n            if (Array.isArray(raw)) { // short form\n                return raw;\n            }\n            else if (Array.isArray(raw.events)) {\n                return raw.events;\n            }\n            return null;\n        },\n        fetch: function (arg, success) {\n            success({\n                rawEvents: arg.eventSource.meta\n            });\n        }\n    };\n    var ArrayEventSourcePlugin = createPlugin({\n        eventSourceDefs: [eventSourceDef]\n    });\n\n    var eventSourceDef$1 = {\n        parseMeta: function (raw) {\n            if (typeof raw === 'function') { // short form\n                return raw;\n            }\n            else if (typeof raw.events === 'function') {\n                return raw.events;\n            }\n            return null;\n        },\n        fetch: function (arg, success, failure) {\n            var dateEnv = arg.calendar.dateEnv;\n            var func = arg.eventSource.meta;\n            unpromisify(func.bind(null, {\n                start: dateEnv.toDate(arg.range.start),\n                end: dateEnv.toDate(arg.range.end),\n                startStr: dateEnv.formatIso(arg.range.start),\n                endStr: dateEnv.formatIso(arg.range.end),\n                timeZone: dateEnv.timeZone\n            }), function (rawEvents) {\n                success({ rawEvents: rawEvents }); // needs an object response\n            }, failure // send errorObj directly to failure callback\n            );\n        }\n    };\n    var FuncEventSourcePlugin = createPlugin({\n        eventSourceDefs: [eventSourceDef$1]\n    });\n\n    function requestJson(method, url, params, successCallback, failureCallback) {\n        method = method.toUpperCase();\n        var body = null;\n        if (method === 'GET') {\n            url = injectQueryStringParams(url, params);\n        }\n        else {\n            body = encodeParams(params);\n        }\n        var xhr = new XMLHttpRequest();\n        xhr.open(method, url, true);\n        if (method !== 'GET') {\n            xhr.setRequestHeader('Content-Type', 'application/x-www-form-urlencoded');\n        }\n        xhr.onload = function () {\n            if (xhr.status >= 200 && xhr.status < 400) {\n                try {\n                    var res = JSON.parse(xhr.responseText);\n                    successCallback(res, xhr);\n                }\n                catch (err) {\n                    failureCallback('Failure parsing JSON', xhr);\n                }\n            }\n            else {\n                failureCallback('Request failed', xhr);\n            }\n        };\n        xhr.onerror = function () {\n            failureCallback('Request failed', xhr);\n        };\n        xhr.send(body);\n    }\n    function injectQueryStringParams(url, params) {\n        return url +\n            (url.indexOf('?') === -1 ? '?' : '&') +\n            encodeParams(params);\n    }\n    function encodeParams(params) {\n        var parts = [];\n        for (var key in params) {\n            parts.push(encodeURIComponent(key) + '=' + encodeURIComponent(params[key]));\n        }\n        return parts.join('&');\n    }\n\n    var eventSourceDef$2 = {\n        parseMeta: function (raw) {\n            if (typeof raw === 'string') { // short form\n                raw = { url: raw };\n            }\n            else if (!raw || typeof raw !== 'object' || !raw.url) {\n                return null;\n            }\n            return {\n                url: raw.url,\n                method: (raw.method || 'GET').toUpperCase(),\n                extraParams: raw.extraParams,\n                startParam: raw.startParam,\n                endParam: raw.endParam,\n                timeZoneParam: raw.timeZoneParam\n            };\n        },\n        fetch: function (arg, success, failure) {\n            var meta = arg.eventSource.meta;\n            var requestParams = buildRequestParams(meta, arg.range, arg.calendar);\n            requestJson(meta.method, meta.url, requestParams, function (rawEvents, xhr) {\n                success({ rawEvents: rawEvents, xhr: xhr });\n            }, function (errorMessage, xhr) {\n                failure({ message: errorMessage, xhr: xhr });\n            });\n        }\n    };\n    var JsonFeedEventSourcePlugin = createPlugin({\n        eventSourceDefs: [eventSourceDef$2]\n    });\n    function buildRequestParams(meta, range, calendar) {\n        var dateEnv = calendar.dateEnv;\n        var startParam;\n        var endParam;\n        var timeZoneParam;\n        var customRequestParams;\n        var params = {};\n        startParam = meta.startParam;\n        if (startParam == null) {\n            startParam = calendar.opt('startParam');\n        }\n        endParam = meta.endParam;\n        if (endParam == null) {\n            endParam = calendar.opt('endParam');\n        }\n        timeZoneParam = meta.timeZoneParam;\n        if (timeZoneParam == null) {\n            timeZoneParam = calendar.opt('timeZoneParam');\n        }\n        // retrieve any outbound GET/POST data from the options\n        if (typeof meta.extraParams === 'function') {\n            // supplied as a function that returns a key/value object\n            customRequestParams = meta.extraParams();\n        }\n        else {\n            // probably supplied as a straight key/value object\n            customRequestParams = meta.extraParams || {};\n        }\n        __assign(params, customRequestParams);\n        params[startParam] = dateEnv.formatIso(range.start);\n        params[endParam] = dateEnv.formatIso(range.end);\n        if (dateEnv.timeZone !== 'local') {\n            params[timeZoneParam] = dateEnv.timeZone;\n        }\n        return params;\n    }\n\n    var recurring = {\n        parse: function (rawEvent, leftoverProps, dateEnv) {\n            var createMarker = dateEnv.createMarker.bind(dateEnv);\n            var processors = {\n                daysOfWeek: null,\n                startTime: createDuration,\n                endTime: createDuration,\n                startRecur: createMarker,\n                endRecur: createMarker\n            };\n            var props = refineProps(rawEvent, processors, {}, leftoverProps);\n            var anyValid = false;\n            for (var propName in props) {\n                if (props[propName] != null) {\n                    anyValid = true;\n                    break;\n                }\n            }\n            if (anyValid) {\n                var duration = null;\n                if ('duration' in leftoverProps) {\n                    duration = createDuration(leftoverProps.duration);\n                    delete leftoverProps.duration;\n                }\n                if (!duration && props.startTime && props.endTime) {\n                    duration = subtractDurations(props.endTime, props.startTime);\n                }\n                return {\n                    allDayGuess: Boolean(!props.startTime && !props.endTime),\n                    duration: duration,\n                    typeData: props // doesn't need endTime anymore but oh well\n                };\n            }\n            return null;\n        },\n        expand: function (typeData, framingRange, dateEnv) {\n            var clippedFramingRange = intersectRanges(framingRange, { start: typeData.startRecur, end: typeData.endRecur });\n            if (clippedFramingRange) {\n                return expandRanges(typeData.daysOfWeek, typeData.startTime, clippedFramingRange, dateEnv);\n            }\n            else {\n                return [];\n            }\n        }\n    };\n    var SimpleRecurrencePlugin = createPlugin({\n        recurringTypes: [recurring]\n    });\n    function expandRanges(daysOfWeek, startTime, framingRange, dateEnv) {\n        var dowHash = daysOfWeek ? arrayToHash(daysOfWeek) : null;\n        var dayMarker = startOfDay(framingRange.start);\n        var endMarker = framingRange.end;\n        var instanceStarts = [];\n        while (dayMarker < endMarker) {\n            var instanceStart \n            // if everyday, or this particular day-of-week\n            = void 0;\n            // if everyday, or this particular day-of-week\n            if (!dowHash || dowHash[dayMarker.getUTCDay()]) {\n                if (startTime) {\n                    instanceStart = dateEnv.add(dayMarker, startTime);\n                }\n                else {\n                    instanceStart = dayMarker;\n                }\n                instanceStarts.push(instanceStart);\n            }\n            dayMarker = addDays(dayMarker, 1);\n        }\n        return instanceStarts;\n    }\n\n    var DefaultOptionChangeHandlers = createPlugin({\n        optionChangeHandlers: {\n            events: function (events, calendar, deepEqual) {\n                handleEventSources([events], calendar, deepEqual);\n            },\n            eventSources: handleEventSources,\n            plugins: handlePlugins\n        }\n    });\n    function handleEventSources(inputs, calendar, deepEqual) {\n        var unfoundSources = hashValuesToArray(calendar.state.eventSources);\n        var newInputs = [];\n        for (var _i = 0, inputs_1 = inputs; _i < inputs_1.length; _i++) {\n            var input = inputs_1[_i];\n            var inputFound = false;\n            for (var i = 0; i < unfoundSources.length; i++) {\n                if (deepEqual(unfoundSources[i]._raw, input)) {\n                    unfoundSources.splice(i, 1); // delete\n                    inputFound = true;\n                    break;\n                }\n            }\n            if (!inputFound) {\n                newInputs.push(input);\n            }\n        }\n        for (var _a = 0, unfoundSources_1 = unfoundSources; _a < unfoundSources_1.length; _a++) {\n            var unfoundSource = unfoundSources_1[_a];\n            calendar.dispatch({\n                type: 'REMOVE_EVENT_SOURCE',\n                sourceId: unfoundSource.sourceId\n            });\n        }\n        for (var _b = 0, newInputs_1 = newInputs; _b < newInputs_1.length; _b++) {\n            var newInput = newInputs_1[_b];\n            calendar.addEventSource(newInput);\n        }\n    }\n    // shortcoming: won't remove plugins\n    function handlePlugins(inputs, calendar) {\n        calendar.addPluginInputs(inputs); // will gracefully handle duplicates\n    }\n\n    var config = {}; // TODO: make these options\n    var globalDefaults = {\n        defaultRangeSeparator: ' - ',\n        titleRangeSeparator: ' \\u2013 ',\n        defaultTimedEventDuration: '01:00:00',\n        defaultAllDayEventDuration: { day: 1 },\n        forceEventDuration: false,\n        nextDayThreshold: '00:00:00',\n        // display\n        columnHeader: true,\n        defaultView: '',\n        aspectRatio: 1.35,\n        header: {\n            left: 'title',\n            center: '',\n            right: 'today prev,next'\n        },\n        weekends: true,\n        weekNumbers: false,\n        weekNumberCalculation: 'local',\n        editable: false,\n        // nowIndicator: false,\n        scrollTime: '06:00:00',\n        minTime: '00:00:00',\n        maxTime: '24:00:00',\n        showNonCurrentDates: true,\n        // event ajax\n        lazyFetching: true,\n        startParam: 'start',\n        endParam: 'end',\n        timeZoneParam: 'timeZone',\n        timeZone: 'local',\n        // allDayDefault: undefined,\n        // locale\n        locales: [],\n        locale: '',\n        // dir: will get this from the default locale\n        // buttonIcons: null,\n        // allows setting a min-height to the event segment to prevent short events overlapping each other\n        timeGridEventMinHeight: 0,\n        themeSystem: 'standard',\n        // eventResizableFromStart: false,\n        dragRevertDuration: 500,\n        dragScroll: true,\n        allDayMaintainDuration: false,\n        // selectable: false,\n        unselectAuto: true,\n        // selectMinDistance: 0,\n        dropAccept: '*',\n        eventOrder: 'start,-duration,allDay,title',\n        // ^ if start tie, longer events go before shorter. final tie-breaker is title text\n        // rerenderDelay: null,\n        eventLimit: false,\n        eventLimitClick: 'popover',\n        dayPopoverFormat: { month: 'long', day: 'numeric', year: 'numeric' },\n        handleWindowResize: true,\n        windowResizeDelay: 100,\n        longPressDelay: 1000,\n        eventDragMinDistance: 5 // only applies to mouse\n    };\n    var rtlDefaults = {\n        header: {\n            left: 'next,prev today',\n            center: '',\n            right: 'title'\n        },\n        buttonIcons: {\n            // TODO: make RTL support the responibility of the theme\n            prev: 'fc-icon-chevron-right',\n            next: 'fc-icon-chevron-left',\n            prevYear: 'fc-icon-chevrons-right',\n            nextYear: 'fc-icon-chevrons-left'\n        }\n    };\n    var complexOptions = [\n        'header',\n        'footer',\n        'buttonText',\n        'buttonIcons'\n    ];\n    // Merges an array of option objects into a single object\n    function mergeOptions(optionObjs) {\n        return mergeProps(optionObjs, complexOptions);\n    }\n    // TODO: move this stuff to a \"plugin\"-related file...\n    var INTERNAL_PLUGINS = [\n        ArrayEventSourcePlugin,\n        FuncEventSourcePlugin,\n        JsonFeedEventSourcePlugin,\n        SimpleRecurrencePlugin,\n        DefaultOptionChangeHandlers\n    ];\n    function refinePluginDefs(pluginInputs) {\n        var plugins = [];\n        for (var _i = 0, pluginInputs_1 = pluginInputs; _i < pluginInputs_1.length; _i++) {\n            var pluginInput = pluginInputs_1[_i];\n            if (typeof pluginInput === 'string') {\n                var globalName = 'FullCalendar' + capitaliseFirstLetter(pluginInput);\n                if (!window[globalName]) {\n                    console.warn('Plugin file not loaded for ' + pluginInput);\n                }\n                else {\n                    plugins.push(window[globalName].default); // is an ES6 module\n                }\n            }\n            else {\n                plugins.push(pluginInput);\n            }\n        }\n        return INTERNAL_PLUGINS.concat(plugins);\n    }\n\n    var RAW_EN_LOCALE = {\n        code: 'en',\n        week: {\n            dow: 0,\n            doy: 4 // 4 days need to be within the year to be considered the first week\n        },\n        dir: 'ltr',\n        buttonText: {\n            prev: 'prev',\n            next: 'next',\n            prevYear: 'prev year',\n            nextYear: 'next year',\n            year: 'year',\n            today: 'today',\n            month: 'month',\n            week: 'week',\n            day: 'day',\n            list: 'list'\n        },\n        weekLabel: 'W',\n        allDayText: 'all-day',\n        eventLimitText: 'more',\n        noEventsMessage: 'No events to display'\n    };\n    function parseRawLocales(explicitRawLocales) {\n        var defaultCode = explicitRawLocales.length > 0 ? explicitRawLocales[0].code : 'en';\n        var globalArray = window['FullCalendarLocalesAll'] || []; // from locales-all.js\n        var globalObject = window['FullCalendarLocales'] || {}; // from locales/*.js. keys are meaningless\n        var allRawLocales = globalArray.concat(// globalArray is low prio\n        hashValuesToArray(globalObject), // medium prio\n        explicitRawLocales // highest prio\n        );\n        var rawLocaleMap = {\n            en: RAW_EN_LOCALE // necessary?\n        };\n        for (var _i = 0, allRawLocales_1 = allRawLocales; _i < allRawLocales_1.length; _i++) {\n            var rawLocale = allRawLocales_1[_i];\n            rawLocaleMap[rawLocale.code] = rawLocale;\n        }\n        return {\n            map: rawLocaleMap,\n            defaultCode: defaultCode\n        };\n    }\n    function buildLocale(inputSingular, available) {\n        if (typeof inputSingular === 'object' && !Array.isArray(inputSingular)) {\n            return parseLocale(inputSingular.code, [inputSingular.code], inputSingular);\n        }\n        else {\n            return queryLocale(inputSingular, available);\n        }\n    }\n    function queryLocale(codeArg, available) {\n        var codes = [].concat(codeArg || []); // will convert to array\n        var raw = queryRawLocale(codes, available) || RAW_EN_LOCALE;\n        return parseLocale(codeArg, codes, raw);\n    }\n    function queryRawLocale(codes, available) {\n        for (var i = 0; i < codes.length; i++) {\n            var parts = codes[i].toLocaleLowerCase().split('-');\n            for (var j = parts.length; j > 0; j--) {\n                var simpleId = parts.slice(0, j).join('-');\n                if (available[simpleId]) {\n                    return available[simpleId];\n                }\n            }\n        }\n        return null;\n    }\n    function parseLocale(codeArg, codes, raw) {\n        var merged = mergeProps([RAW_EN_LOCALE, raw], ['buttonText']);\n        delete merged.code; // don't want this part of the options\n        var week = merged.week;\n        delete merged.week;\n        return {\n            codeArg: codeArg,\n            codes: codes,\n            week: week,\n            simpleNumberFormat: new Intl.NumberFormat(codeArg),\n            options: merged\n        };\n    }\n\n    var OptionsManager = /** @class */ (function () {\n        function OptionsManager(overrides) {\n            this.overrides = __assign({}, overrides); // make a copy\n            this.dynamicOverrides = {};\n            this.compute();\n        }\n        OptionsManager.prototype.mutate = function (updates, removals, isDynamic) {\n            if (!Object.keys(updates).length && !removals.length) {\n                return;\n            }\n            var overrideHash = isDynamic ? this.dynamicOverrides : this.overrides;\n            __assign(overrideHash, updates);\n            for (var _i = 0, removals_1 = removals; _i < removals_1.length; _i++) {\n                var propName = removals_1[_i];\n                delete overrideHash[propName];\n            }\n            this.compute();\n        };\n        // Computes the flattened options hash for the calendar and assigns to `this.options`.\n        // Assumes this.overrides and this.dynamicOverrides have already been initialized.\n        OptionsManager.prototype.compute = function () {\n            // TODO: not a very efficient system\n            var locales = firstDefined(// explicit locale option given?\n            this.dynamicOverrides.locales, this.overrides.locales, globalDefaults.locales);\n            var locale = firstDefined(// explicit locales option given?\n            this.dynamicOverrides.locale, this.overrides.locale, globalDefaults.locale);\n            var available = parseRawLocales(locales);\n            var localeDefaults = buildLocale(locale || available.defaultCode, available.map).options;\n            var dir = firstDefined(// based on options computed so far, is direction RTL?\n            this.dynamicOverrides.dir, this.overrides.dir, localeDefaults.dir);\n            var dirDefaults = dir === 'rtl' ? rtlDefaults : {};\n            this.dirDefaults = dirDefaults;\n            this.localeDefaults = localeDefaults;\n            this.computed = mergeOptions([\n                globalDefaults,\n                dirDefaults,\n                localeDefaults,\n                this.overrides,\n                this.dynamicOverrides\n            ]);\n        };\n        return OptionsManager;\n    }());\n\n    var calendarSystemClassMap = {};\n    function registerCalendarSystem(name, theClass) {\n        calendarSystemClassMap[name] = theClass;\n    }\n    function createCalendarSystem(name) {\n        return new calendarSystemClassMap[name]();\n    }\n    var GregorianCalendarSystem = /** @class */ (function () {\n        function GregorianCalendarSystem() {\n        }\n        GregorianCalendarSystem.prototype.getMarkerYear = function (d) {\n            return d.getUTCFullYear();\n        };\n        GregorianCalendarSystem.prototype.getMarkerMonth = function (d) {\n            return d.getUTCMonth();\n        };\n        GregorianCalendarSystem.prototype.getMarkerDay = function (d) {\n            return d.getUTCDate();\n        };\n        GregorianCalendarSystem.prototype.arrayToMarker = function (arr) {\n            return arrayToUtcDate(arr);\n        };\n        GregorianCalendarSystem.prototype.markerToArray = function (marker) {\n            return dateToUtcArray(marker);\n        };\n        return GregorianCalendarSystem;\n    }());\n    registerCalendarSystem('gregory', GregorianCalendarSystem);\n\n    var ISO_RE = /^\\s*(\\d{4})(-(\\d{2})(-(\\d{2})([T ](\\d{2}):(\\d{2})(:(\\d{2})(\\.(\\d+))?)?(Z|(([-+])(\\d{2})(:?(\\d{2}))?))?)?)?)?$/;\n    function parse(str) {\n        var m = ISO_RE.exec(str);\n        if (m) {\n            var marker = new Date(Date.UTC(Number(m[1]), m[3] ? Number(m[3]) - 1 : 0, Number(m[5] || 1), Number(m[7] || 0), Number(m[8] || 0), Number(m[10] || 0), m[12] ? Number('0.' + m[12]) * 1000 : 0));\n            if (isValidDate(marker)) {\n                var timeZoneOffset = null;\n                if (m[13]) {\n                    timeZoneOffset = (m[15] === '-' ? -1 : 1) * (Number(m[16] || 0) * 60 +\n                        Number(m[18] || 0));\n                }\n                return {\n                    marker: marker,\n                    isTimeUnspecified: !m[6],\n                    timeZoneOffset: timeZoneOffset\n                };\n            }\n        }\n        return null;\n    }\n\n    var DateEnv = /** @class */ (function () {\n        function DateEnv(settings) {\n            var timeZone = this.timeZone = settings.timeZone;\n            var isNamedTimeZone = timeZone !== 'local' && timeZone !== 'UTC';\n            if (settings.namedTimeZoneImpl && isNamedTimeZone) {\n                this.namedTimeZoneImpl = new settings.namedTimeZoneImpl(timeZone);\n            }\n            this.canComputeOffset = Boolean(!isNamedTimeZone || this.namedTimeZoneImpl);\n            this.calendarSystem = createCalendarSystem(settings.calendarSystem);\n            this.locale = settings.locale;\n            this.weekDow = settings.locale.week.dow;\n            this.weekDoy = settings.locale.week.doy;\n            if (settings.weekNumberCalculation === 'ISO') {\n                this.weekDow = 1;\n                this.weekDoy = 4;\n            }\n            if (typeof settings.firstDay === 'number') {\n                this.weekDow = settings.firstDay;\n            }\n            if (typeof settings.weekNumberCalculation === 'function') {\n                this.weekNumberFunc = settings.weekNumberCalculation;\n            }\n            this.weekLabel = settings.weekLabel != null ? settings.weekLabel : settings.locale.options.weekLabel;\n            this.cmdFormatter = settings.cmdFormatter;\n        }\n        // Creating / Parsing\n        DateEnv.prototype.createMarker = function (input) {\n            var meta = this.createMarkerMeta(input);\n            if (meta === null) {\n                return null;\n            }\n            return meta.marker;\n        };\n        DateEnv.prototype.createNowMarker = function () {\n            if (this.canComputeOffset) {\n                return this.timestampToMarker(new Date().valueOf());\n            }\n            else {\n                // if we can't compute the current date val for a timezone,\n                // better to give the current local date vals than UTC\n                return arrayToUtcDate(dateToLocalArray(new Date()));\n            }\n        };\n        DateEnv.prototype.createMarkerMeta = function (input) {\n            if (typeof input === 'string') {\n                return this.parse(input);\n            }\n            var marker = null;\n            if (typeof input === 'number') {\n                marker = this.timestampToMarker(input);\n            }\n            else if (input instanceof Date) {\n                input = input.valueOf();\n                if (!isNaN(input)) {\n                    marker = this.timestampToMarker(input);\n                }\n            }\n            else if (Array.isArray(input)) {\n                marker = arrayToUtcDate(input);\n            }\n            if (marker === null || !isValidDate(marker)) {\n                return null;\n            }\n            return { marker: marker, isTimeUnspecified: false, forcedTzo: null };\n        };\n        DateEnv.prototype.parse = function (s) {\n            var parts = parse(s);\n            if (parts === null) {\n                return null;\n            }\n            var marker = parts.marker;\n            var forcedTzo = null;\n            if (parts.timeZoneOffset !== null) {\n                if (this.canComputeOffset) {\n                    marker = this.timestampToMarker(marker.valueOf() - parts.timeZoneOffset * 60 * 1000);\n                }\n                else {\n                    forcedTzo = parts.timeZoneOffset;\n                }\n            }\n            return { marker: marker, isTimeUnspecified: parts.isTimeUnspecified, forcedTzo: forcedTzo };\n        };\n        // Accessors\n        DateEnv.prototype.getYear = function (marker) {\n            return this.calendarSystem.getMarkerYear(marker);\n        };\n        DateEnv.prototype.getMonth = function (marker) {\n            return this.calendarSystem.getMarkerMonth(marker);\n        };\n        // Adding / Subtracting\n        DateEnv.prototype.add = function (marker, dur) {\n            var a = this.calendarSystem.markerToArray(marker);\n            a[0] += dur.years;\n            a[1] += dur.months;\n            a[2] += dur.days;\n            a[6] += dur.milliseconds;\n            return this.calendarSystem.arrayToMarker(a);\n        };\n        DateEnv.prototype.subtract = function (marker, dur) {\n            var a = this.calendarSystem.markerToArray(marker);\n            a[0] -= dur.years;\n            a[1] -= dur.months;\n            a[2] -= dur.days;\n            a[6] -= dur.milliseconds;\n            return this.calendarSystem.arrayToMarker(a);\n        };\n        DateEnv.prototype.addYears = function (marker, n) {\n            var a = this.calendarSystem.markerToArray(marker);\n            a[0] += n;\n            return this.calendarSystem.arrayToMarker(a);\n        };\n        DateEnv.prototype.addMonths = function (marker, n) {\n            var a = this.calendarSystem.markerToArray(marker);\n            a[1] += n;\n            return this.calendarSystem.arrayToMarker(a);\n        };\n        // Diffing Whole Units\n        DateEnv.prototype.diffWholeYears = function (m0, m1) {\n            var calendarSystem = this.calendarSystem;\n            if (timeAsMs(m0) === timeAsMs(m1) &&\n                calendarSystem.getMarkerDay(m0) === calendarSystem.getMarkerDay(m1) &&\n                calendarSystem.getMarkerMonth(m0) === calendarSystem.getMarkerMonth(m1)) {\n                return calendarSystem.getMarkerYear(m1) - calendarSystem.getMarkerYear(m0);\n            }\n            return null;\n        };\n        DateEnv.prototype.diffWholeMonths = function (m0, m1) {\n            var calendarSystem = this.calendarSystem;\n            if (timeAsMs(m0) === timeAsMs(m1) &&\n                calendarSystem.getMarkerDay(m0) === calendarSystem.getMarkerDay(m1)) {\n                return (calendarSystem.getMarkerMonth(m1) - calendarSystem.getMarkerMonth(m0)) +\n                    (calendarSystem.getMarkerYear(m1) - calendarSystem.getMarkerYear(m0)) * 12;\n            }\n            return null;\n        };\n        // Range / Duration\n        DateEnv.prototype.greatestWholeUnit = function (m0, m1) {\n            var n = this.diffWholeYears(m0, m1);\n            if (n !== null) {\n                return { unit: 'year', value: n };\n            }\n            n = this.diffWholeMonths(m0, m1);\n            if (n !== null) {\n                return { unit: 'month', value: n };\n            }\n            n = diffWholeWeeks(m0, m1);\n            if (n !== null) {\n                return { unit: 'week', value: n };\n            }\n            n = diffWholeDays(m0, m1);\n            if (n !== null) {\n                return { unit: 'day', value: n };\n            }\n            n = diffHours(m0, m1);\n            if (isInt(n)) {\n                return { unit: 'hour', value: n };\n            }\n            n = diffMinutes(m0, m1);\n            if (isInt(n)) {\n                return { unit: 'minute', value: n };\n            }\n            n = diffSeconds(m0, m1);\n            if (isInt(n)) {\n                return { unit: 'second', value: n };\n            }\n            return { unit: 'millisecond', value: m1.valueOf() - m0.valueOf() };\n        };\n        DateEnv.prototype.countDurationsBetween = function (m0, m1, d) {\n            // TODO: can use greatestWholeUnit\n            var diff;\n            if (d.years) {\n                diff = this.diffWholeYears(m0, m1);\n                if (diff !== null) {\n                    return diff / asRoughYears(d);\n                }\n            }\n            if (d.months) {\n                diff = this.diffWholeMonths(m0, m1);\n                if (diff !== null) {\n                    return diff / asRoughMonths(d);\n                }\n            }\n            if (d.days) {\n                diff = diffWholeDays(m0, m1);\n                if (diff !== null) {\n                    return diff / asRoughDays(d);\n                }\n            }\n            return (m1.valueOf() - m0.valueOf()) / asRoughMs(d);\n        };\n        // Start-Of\n        DateEnv.prototype.startOf = function (m, unit) {\n            if (unit === 'year') {\n                return this.startOfYear(m);\n            }\n            else if (unit === 'month') {\n                return this.startOfMonth(m);\n            }\n            else if (unit === 'week') {\n                return this.startOfWeek(m);\n            }\n            else if (unit === 'day') {\n                return startOfDay(m);\n            }\n            else if (unit === 'hour') {\n                return startOfHour(m);\n            }\n            else if (unit === 'minute') {\n                return startOfMinute(m);\n            }\n            else if (unit === 'second') {\n                return startOfSecond(m);\n            }\n        };\n        DateEnv.prototype.startOfYear = function (m) {\n            return this.calendarSystem.arrayToMarker([\n                this.calendarSystem.getMarkerYear(m)\n            ]);\n        };\n        DateEnv.prototype.startOfMonth = function (m) {\n            return this.calendarSystem.arrayToMarker([\n                this.calendarSystem.getMarkerYear(m),\n                this.calendarSystem.getMarkerMonth(m)\n            ]);\n        };\n        DateEnv.prototype.startOfWeek = function (m) {\n            return this.calendarSystem.arrayToMarker([\n                this.calendarSystem.getMarkerYear(m),\n                this.calendarSystem.getMarkerMonth(m),\n                m.getUTCDate() - ((m.getUTCDay() - this.weekDow + 7) % 7)\n            ]);\n        };\n        // Week Number\n        DateEnv.prototype.computeWeekNumber = function (marker) {\n            if (this.weekNumberFunc) {\n                return this.weekNumberFunc(this.toDate(marker));\n            }\n            else {\n                return weekOfYear(marker, this.weekDow, this.weekDoy);\n            }\n        };\n        // TODO: choke on timeZoneName: long\n        DateEnv.prototype.format = function (marker, formatter, dateOptions) {\n            if (dateOptions === void 0) { dateOptions = {}; }\n            return formatter.format({\n                marker: marker,\n                timeZoneOffset: dateOptions.forcedTzo != null ?\n                    dateOptions.forcedTzo :\n                    this.offsetForMarker(marker)\n            }, this);\n        };\n        DateEnv.prototype.formatRange = function (start, end, formatter, dateOptions) {\n            if (dateOptions === void 0) { dateOptions = {}; }\n            if (dateOptions.isEndExclusive) {\n                end = addMs(end, -1);\n            }\n            return formatter.formatRange({\n                marker: start,\n                timeZoneOffset: dateOptions.forcedStartTzo != null ?\n                    dateOptions.forcedStartTzo :\n                    this.offsetForMarker(start)\n            }, {\n                marker: end,\n                timeZoneOffset: dateOptions.forcedEndTzo != null ?\n                    dateOptions.forcedEndTzo :\n                    this.offsetForMarker(end)\n            }, this);\n        };\n        DateEnv.prototype.formatIso = function (marker, extraOptions) {\n            if (extraOptions === void 0) { extraOptions = {}; }\n            var timeZoneOffset = null;\n            if (!extraOptions.omitTimeZoneOffset) {\n                if (extraOptions.forcedTzo != null) {\n                    timeZoneOffset = extraOptions.forcedTzo;\n                }\n                else {\n                    timeZoneOffset = this.offsetForMarker(marker);\n                }\n            }\n            return buildIsoString(marker, timeZoneOffset, extraOptions.omitTime);\n        };\n        // TimeZone\n        DateEnv.prototype.timestampToMarker = function (ms) {\n            if (this.timeZone === 'local') {\n                return arrayToUtcDate(dateToLocalArray(new Date(ms)));\n            }\n            else if (this.timeZone === 'UTC' || !this.namedTimeZoneImpl) {\n                return new Date(ms);\n            }\n            else {\n                return arrayToUtcDate(this.namedTimeZoneImpl.timestampToArray(ms));\n            }\n        };\n        DateEnv.prototype.offsetForMarker = function (m) {\n            if (this.timeZone === 'local') {\n                return -arrayToLocalDate(dateToUtcArray(m)).getTimezoneOffset(); // convert \"inverse\" offset to \"normal\" offset\n            }\n            else if (this.timeZone === 'UTC') {\n                return 0;\n            }\n            else if (this.namedTimeZoneImpl) {\n                return this.namedTimeZoneImpl.offsetForArray(dateToUtcArray(m));\n            }\n            return null;\n        };\n        // Conversion\n        DateEnv.prototype.toDate = function (m, forcedTzo) {\n            if (this.timeZone === 'local') {\n                return arrayToLocalDate(dateToUtcArray(m));\n            }\n            else if (this.timeZone === 'UTC') {\n                return new Date(m.valueOf()); // make sure it's a copy\n            }\n            else if (!this.namedTimeZoneImpl) {\n                return new Date(m.valueOf() - (forcedTzo || 0));\n            }\n            else {\n                return new Date(m.valueOf() -\n                    this.namedTimeZoneImpl.offsetForArray(dateToUtcArray(m)) * 1000 * 60 // convert minutes -> ms\n                );\n            }\n        };\n        return DateEnv;\n    }());\n\n    var SIMPLE_SOURCE_PROPS = {\n        id: String,\n        allDayDefault: Boolean,\n        eventDataTransform: Function,\n        success: Function,\n        failure: Function\n    };\n    var uid$2 = 0;\n    function doesSourceNeedRange(eventSource, calendar) {\n        var defs = calendar.pluginSystem.hooks.eventSourceDefs;\n        return !defs[eventSource.sourceDefId].ignoreRange;\n    }\n    function parseEventSource(raw, calendar) {\n        var defs = calendar.pluginSystem.hooks.eventSourceDefs;\n        for (var i = defs.length - 1; i >= 0; i--) { // later-added plugins take precedence\n            var def = defs[i];\n            var meta = def.parseMeta(raw);\n            if (meta) {\n                var res = parseEventSourceProps(typeof raw === 'object' ? raw : {}, meta, i, calendar);\n                res._raw = raw;\n                return res;\n            }\n        }\n        return null;\n    }\n    function parseEventSourceProps(raw, meta, sourceDefId, calendar) {\n        var leftovers0 = {};\n        var props = refineProps(raw, SIMPLE_SOURCE_PROPS, {}, leftovers0);\n        var leftovers1 = {};\n        var ui = processUnscopedUiProps(leftovers0, calendar, leftovers1);\n        props.isFetching = false;\n        props.latestFetchId = '';\n        props.fetchRange = null;\n        props.publicId = String(raw.id || '');\n        props.sourceId = String(uid$2++);\n        props.sourceDefId = sourceDefId;\n        props.meta = meta;\n        props.ui = ui;\n        props.extendedProps = leftovers1;\n        return props;\n    }\n\n    function reduceEventSources (eventSources, action, dateProfile, calendar) {\n        switch (action.type) {\n            case 'ADD_EVENT_SOURCES': // already parsed\n                return addSources(eventSources, action.sources, dateProfile ? dateProfile.activeRange : null, calendar);\n            case 'REMOVE_EVENT_SOURCE':\n                return removeSource(eventSources, action.sourceId);\n            case 'PREV': // TODO: how do we track all actions that affect dateProfile :(\n            case 'NEXT':\n            case 'SET_DATE':\n            case 'SET_VIEW_TYPE':\n                if (dateProfile) {\n                    return fetchDirtySources(eventSources, dateProfile.activeRange, calendar);\n                }\n                else {\n                    return eventSources;\n                }\n            case 'FETCH_EVENT_SOURCES':\n            case 'CHANGE_TIMEZONE':\n                return fetchSourcesByIds(eventSources, action.sourceIds ?\n                    arrayToHash(action.sourceIds) :\n                    excludeStaticSources(eventSources, calendar), dateProfile ? dateProfile.activeRange : null, calendar);\n            case 'RECEIVE_EVENTS':\n            case 'RECEIVE_EVENT_ERROR':\n                return receiveResponse(eventSources, action.sourceId, action.fetchId, action.fetchRange);\n            case 'REMOVE_ALL_EVENT_SOURCES':\n                return {};\n            default:\n                return eventSources;\n        }\n    }\n    var uid$3 = 0;\n    function addSources(eventSourceHash, sources, fetchRange, calendar) {\n        var hash = {};\n        for (var _i = 0, sources_1 = sources; _i < sources_1.length; _i++) {\n            var source = sources_1[_i];\n            hash[source.sourceId] = source;\n        }\n        if (fetchRange) {\n            hash = fetchDirtySources(hash, fetchRange, calendar);\n        }\n        return __assign({}, eventSourceHash, hash);\n    }\n    function removeSource(eventSourceHash, sourceId) {\n        return filterHash(eventSourceHash, function (eventSource) {\n            return eventSource.sourceId !== sourceId;\n        });\n    }\n    function fetchDirtySources(sourceHash, fetchRange, calendar) {\n        return fetchSourcesByIds(sourceHash, filterHash(sourceHash, function (eventSource) {\n            return isSourceDirty(eventSource, fetchRange, calendar);\n        }), fetchRange, calendar);\n    }\n    function isSourceDirty(eventSource, fetchRange, calendar) {\n        if (!doesSourceNeedRange(eventSource, calendar)) {\n            return !eventSource.latestFetchId;\n        }\n        else {\n            return !calendar.opt('lazyFetching') ||\n                !eventSource.fetchRange ||\n                eventSource.isFetching || // always cancel outdated in-progress fetches\n                fetchRange.start < eventSource.fetchRange.start ||\n                fetchRange.end > eventSource.fetchRange.end;\n        }\n    }\n    function fetchSourcesByIds(prevSources, sourceIdHash, fetchRange, calendar) {\n        var nextSources = {};\n        for (var sourceId in prevSources) {\n            var source = prevSources[sourceId];\n            if (sourceIdHash[sourceId]) {\n                nextSources[sourceId] = fetchSource(source, fetchRange, calendar);\n            }\n            else {\n                nextSources[sourceId] = source;\n            }\n        }\n        return nextSources;\n    }\n    function fetchSource(eventSource, fetchRange, calendar) {\n        var sourceDef = calendar.pluginSystem.hooks.eventSourceDefs[eventSource.sourceDefId];\n        var fetchId = String(uid$3++);\n        sourceDef.fetch({\n            eventSource: eventSource,\n            calendar: calendar,\n            range: fetchRange\n        }, function (res) {\n            var rawEvents = res.rawEvents;\n            var calSuccess = calendar.opt('eventSourceSuccess');\n            var calSuccessRes;\n            var sourceSuccessRes;\n            if (eventSource.success) {\n                sourceSuccessRes = eventSource.success(rawEvents, res.xhr);\n            }\n            if (calSuccess) {\n                calSuccessRes = calSuccess(rawEvents, res.xhr);\n            }\n            rawEvents = sourceSuccessRes || calSuccessRes || rawEvents;\n            calendar.dispatch({\n                type: 'RECEIVE_EVENTS',\n                sourceId: eventSource.sourceId,\n                fetchId: fetchId,\n                fetchRange: fetchRange,\n                rawEvents: rawEvents\n            });\n        }, function (error) {\n            var callFailure = calendar.opt('eventSourceFailure');\n            console.warn(error.message, error);\n            if (eventSource.failure) {\n                eventSource.failure(error);\n            }\n            if (callFailure) {\n                callFailure(error);\n            }\n            calendar.dispatch({\n                type: 'RECEIVE_EVENT_ERROR',\n                sourceId: eventSource.sourceId,\n                fetchId: fetchId,\n                fetchRange: fetchRange,\n                error: error\n            });\n        });\n        return __assign({}, eventSource, { isFetching: true, latestFetchId: fetchId });\n    }\n    function receiveResponse(sourceHash, sourceId, fetchId, fetchRange) {\n        var _a;\n        var eventSource = sourceHash[sourceId];\n        if (eventSource && // not already removed\n            fetchId === eventSource.latestFetchId) {\n            return __assign({}, sourceHash, (_a = {}, _a[sourceId] = __assign({}, eventSource, { isFetching: false, fetchRange: fetchRange // also serves as a marker that at least one fetch has completed\n             }), _a));\n        }\n        return sourceHash;\n    }\n    function excludeStaticSources(eventSources, calendar) {\n        return filterHash(eventSources, function (eventSource) {\n            return doesSourceNeedRange(eventSource, calendar);\n        });\n    }\n\n    var DateProfileGenerator = /** @class */ (function () {\n        function DateProfileGenerator(viewSpec, calendar) {\n            this.viewSpec = viewSpec;\n            this.options = viewSpec.options;\n            this.dateEnv = calendar.dateEnv;\n            this.calendar = calendar;\n            this.initHiddenDays();\n        }\n        /* Date Range Computation\n        ------------------------------------------------------------------------------------------------------------------*/\n        // Builds a structure with info about what the dates/ranges will be for the \"prev\" view.\n        DateProfileGenerator.prototype.buildPrev = function (currentDateProfile, currentDate) {\n            var dateEnv = this.dateEnv;\n            var prevDate = dateEnv.subtract(dateEnv.startOf(currentDate, currentDateProfile.currentRangeUnit), // important for start-of-month\n            currentDateProfile.dateIncrement);\n            return this.build(prevDate, -1);\n        };\n        // Builds a structure with info about what the dates/ranges will be for the \"next\" view.\n        DateProfileGenerator.prototype.buildNext = function (currentDateProfile, currentDate) {\n            var dateEnv = this.dateEnv;\n            var nextDate = dateEnv.add(dateEnv.startOf(currentDate, currentDateProfile.currentRangeUnit), // important for start-of-month\n            currentDateProfile.dateIncrement);\n            return this.build(nextDate, 1);\n        };\n        // Builds a structure holding dates/ranges for rendering around the given date.\n        // Optional direction param indicates whether the date is being incremented/decremented\n        // from its previous value. decremented = -1, incremented = 1 (default).\n        DateProfileGenerator.prototype.build = function (currentDate, direction, forceToValid) {\n            if (forceToValid === void 0) { forceToValid = false; }\n            var validRange;\n            var minTime = null;\n            var maxTime = null;\n            var currentInfo;\n            var isRangeAllDay;\n            var renderRange;\n            var activeRange;\n            var isValid;\n            validRange = this.buildValidRange();\n            validRange = this.trimHiddenDays(validRange);\n            if (forceToValid) {\n                currentDate = constrainMarkerToRange(currentDate, validRange);\n            }\n            currentInfo = this.buildCurrentRangeInfo(currentDate, direction);\n            isRangeAllDay = /^(year|month|week|day)$/.test(currentInfo.unit);\n            renderRange = this.buildRenderRange(this.trimHiddenDays(currentInfo.range), currentInfo.unit, isRangeAllDay);\n            renderRange = this.trimHiddenDays(renderRange);\n            activeRange = renderRange;\n            if (!this.options.showNonCurrentDates) {\n                activeRange = intersectRanges(activeRange, currentInfo.range);\n            }\n            minTime = createDuration(this.options.minTime);\n            maxTime = createDuration(this.options.maxTime);\n            activeRange = this.adjustActiveRange(activeRange, minTime, maxTime);\n            activeRange = intersectRanges(activeRange, validRange); // might return null\n            // it's invalid if the originally requested date is not contained,\n            // or if the range is completely outside of the valid range.\n            isValid = rangesIntersect(currentInfo.range, validRange);\n            return {\n                // constraint for where prev/next operations can go and where events can be dragged/resized to.\n                // an object with optional start and end properties.\n                validRange: validRange,\n                // range the view is formally responsible for.\n                // for example, a month view might have 1st-31st, excluding padded dates\n                currentRange: currentInfo.range,\n                // name of largest unit being displayed, like \"month\" or \"week\"\n                currentRangeUnit: currentInfo.unit,\n                isRangeAllDay: isRangeAllDay,\n                // dates that display events and accept drag-n-drop\n                // will be `null` if no dates accept events\n                activeRange: activeRange,\n                // date range with a rendered skeleton\n                // includes not-active days that need some sort of DOM\n                renderRange: renderRange,\n                // Duration object that denotes the first visible time of any given day\n                minTime: minTime,\n                // Duration object that denotes the exclusive visible end time of any given day\n                maxTime: maxTime,\n                isValid: isValid,\n                // how far the current date will move for a prev/next operation\n                dateIncrement: this.buildDateIncrement(currentInfo.duration)\n                // pass a fallback (might be null) ^\n            };\n        };\n        // Builds an object with optional start/end properties.\n        // Indicates the minimum/maximum dates to display.\n        // not responsible for trimming hidden days.\n        DateProfileGenerator.prototype.buildValidRange = function () {\n            return this.getRangeOption('validRange', this.calendar.getNow()) ||\n                { start: null, end: null }; // completely open-ended\n        };\n        // Builds a structure with info about the \"current\" range, the range that is\n        // highlighted as being the current month for example.\n        // See build() for a description of `direction`.\n        // Guaranteed to have `range` and `unit` properties. `duration` is optional.\n        DateProfileGenerator.prototype.buildCurrentRangeInfo = function (date, direction) {\n            var _a = this, viewSpec = _a.viewSpec, dateEnv = _a.dateEnv;\n            var duration = null;\n            var unit = null;\n            var range = null;\n            var dayCount;\n            if (viewSpec.duration) {\n                duration = viewSpec.duration;\n                unit = viewSpec.durationUnit;\n                range = this.buildRangeFromDuration(date, direction, duration, unit);\n            }\n            else if ((dayCount = this.options.dayCount)) {\n                unit = 'day';\n                range = this.buildRangeFromDayCount(date, direction, dayCount);\n            }\n            else if ((range = this.buildCustomVisibleRange(date))) {\n                unit = dateEnv.greatestWholeUnit(range.start, range.end).unit;\n            }\n            else {\n                duration = this.getFallbackDuration();\n                unit = greatestDurationDenominator(duration).unit;\n                range = this.buildRangeFromDuration(date, direction, duration, unit);\n            }\n            return { duration: duration, unit: unit, range: range };\n        };\n        DateProfileGenerator.prototype.getFallbackDuration = function () {\n            return createDuration({ day: 1 });\n        };\n        // Returns a new activeRange to have time values (un-ambiguate)\n        // minTime or maxTime causes the range to expand.\n        DateProfileGenerator.prototype.adjustActiveRange = function (range, minTime, maxTime) {\n            var dateEnv = this.dateEnv;\n            var start = range.start;\n            var end = range.end;\n            if (this.viewSpec.class.prototype.usesMinMaxTime) {\n                // expand active range if minTime is negative (why not when positive?)\n                if (asRoughDays(minTime) < 0) {\n                    start = startOfDay(start); // necessary?\n                    start = dateEnv.add(start, minTime);\n                }\n                // expand active range if maxTime is beyond one day (why not when positive?)\n                if (asRoughDays(maxTime) > 1) {\n                    end = startOfDay(end); // necessary?\n                    end = addDays(end, -1);\n                    end = dateEnv.add(end, maxTime);\n                }\n            }\n            return { start: start, end: end };\n        };\n        // Builds the \"current\" range when it is specified as an explicit duration.\n        // `unit` is the already-computed greatestDurationDenominator unit of duration.\n        DateProfileGenerator.prototype.buildRangeFromDuration = function (date, direction, duration, unit) {\n            var dateEnv = this.dateEnv;\n            var alignment = this.options.dateAlignment;\n            var dateIncrementInput;\n            var dateIncrementDuration;\n            var start;\n            var end;\n            var res;\n            // compute what the alignment should be\n            if (!alignment) {\n                dateIncrementInput = this.options.dateIncrement;\n                if (dateIncrementInput) {\n                    dateIncrementDuration = createDuration(dateIncrementInput);\n                    // use the smaller of the two units\n                    if (asRoughMs(dateIncrementDuration) < asRoughMs(duration)) {\n                        alignment = greatestDurationDenominator(dateIncrementDuration, !getWeeksFromInput(dateIncrementInput)).unit;\n                    }\n                    else {\n                        alignment = unit;\n                    }\n                }\n                else {\n                    alignment = unit;\n                }\n            }\n            // if the view displays a single day or smaller\n            if (asRoughDays(duration) <= 1) {\n                if (this.isHiddenDay(start)) {\n                    start = this.skipHiddenDays(start, direction);\n                    start = startOfDay(start);\n                }\n            }\n            function computeRes() {\n                start = dateEnv.startOf(date, alignment);\n                end = dateEnv.add(start, duration);\n                res = { start: start, end: end };\n            }\n            computeRes();\n            // if range is completely enveloped by hidden days, go past the hidden days\n            if (!this.trimHiddenDays(res)) {\n                date = this.skipHiddenDays(date, direction);\n                computeRes();\n            }\n            return res;\n        };\n        // Builds the \"current\" range when a dayCount is specified.\n        DateProfileGenerator.prototype.buildRangeFromDayCount = function (date, direction, dayCount) {\n            var dateEnv = this.dateEnv;\n            var customAlignment = this.options.dateAlignment;\n            var runningCount = 0;\n            var start = date;\n            var end;\n            if (customAlignment) {\n                start = dateEnv.startOf(start, customAlignment);\n            }\n            start = startOfDay(start);\n            start = this.skipHiddenDays(start, direction);\n            end = start;\n            do {\n                end = addDays(end, 1);\n                if (!this.isHiddenDay(end)) {\n                    runningCount++;\n                }\n            } while (runningCount < dayCount);\n            return { start: start, end: end };\n        };\n        // Builds a normalized range object for the \"visible\" range,\n        // which is a way to define the currentRange and activeRange at the same time.\n        DateProfileGenerator.prototype.buildCustomVisibleRange = function (date) {\n            var dateEnv = this.dateEnv;\n            var visibleRange = this.getRangeOption('visibleRange', dateEnv.toDate(date));\n            if (visibleRange && (visibleRange.start == null || visibleRange.end == null)) {\n                return null;\n            }\n            return visibleRange;\n        };\n        // Computes the range that will represent the element/cells for *rendering*,\n        // but which may have voided days/times.\n        // not responsible for trimming hidden days.\n        DateProfileGenerator.prototype.buildRenderRange = function (currentRange, currentRangeUnit, isRangeAllDay) {\n            return currentRange;\n        };\n        // Compute the duration value that should be added/substracted to the current date\n        // when a prev/next operation happens.\n        DateProfileGenerator.prototype.buildDateIncrement = function (fallback) {\n            var dateIncrementInput = this.options.dateIncrement;\n            var customAlignment;\n            if (dateIncrementInput) {\n                return createDuration(dateIncrementInput);\n            }\n            else if ((customAlignment = this.options.dateAlignment)) {\n                return createDuration(1, customAlignment);\n            }\n            else if (fallback) {\n                return fallback;\n            }\n            else {\n                return createDuration({ days: 1 });\n            }\n        };\n        // Arguments after name will be forwarded to a hypothetical function value\n        // WARNING: passed-in arguments will be given to generator functions as-is and can cause side-effects.\n        // Always clone your objects if you fear mutation.\n        DateProfileGenerator.prototype.getRangeOption = function (name) {\n            var otherArgs = [];\n            for (var _i = 1; _i < arguments.length; _i++) {\n                otherArgs[_i - 1] = arguments[_i];\n            }\n            var val = this.options[name];\n            if (typeof val === 'function') {\n                val = val.apply(null, otherArgs);\n            }\n            if (val) {\n                val = parseRange(val, this.dateEnv);\n            }\n            if (val) {\n                val = computeVisibleDayRange(val);\n            }\n            return val;\n        };\n        /* Hidden Days\n        ------------------------------------------------------------------------------------------------------------------*/\n        // Initializes internal variables related to calculating hidden days-of-week\n        DateProfileGenerator.prototype.initHiddenDays = function () {\n            var hiddenDays = this.options.hiddenDays || []; // array of day-of-week indices that are hidden\n            var isHiddenDayHash = []; // is the day-of-week hidden? (hash with day-of-week-index -> bool)\n            var dayCnt = 0;\n            var i;\n            if (this.options.weekends === false) {\n                hiddenDays.push(0, 6); // 0=sunday, 6=saturday\n            }\n            for (i = 0; i < 7; i++) {\n                if (!(isHiddenDayHash[i] = hiddenDays.indexOf(i) !== -1)) {\n                    dayCnt++;\n                }\n            }\n            if (!dayCnt) {\n                throw new Error('invalid hiddenDays'); // all days were hidden? bad.\n            }\n            this.isHiddenDayHash = isHiddenDayHash;\n        };\n        // Remove days from the beginning and end of the range that are computed as hidden.\n        // If the whole range is trimmed off, returns null\n        DateProfileGenerator.prototype.trimHiddenDays = function (range) {\n            var start = range.start;\n            var end = range.end;\n            if (start) {\n                start = this.skipHiddenDays(start);\n            }\n            if (end) {\n                end = this.skipHiddenDays(end, -1, true);\n            }\n            if (start == null || end == null || start < end) {\n                return { start: start, end: end };\n            }\n            return null;\n        };\n        // Is the current day hidden?\n        // `day` is a day-of-week index (0-6), or a Date (used for UTC)\n        DateProfileGenerator.prototype.isHiddenDay = function (day) {\n            if (day instanceof Date) {\n                day = day.getUTCDay();\n            }\n            return this.isHiddenDayHash[day];\n        };\n        // Incrementing the current day until it is no longer a hidden day, returning a copy.\n        // DOES NOT CONSIDER validRange!\n        // If the initial value of `date` is not a hidden day, don't do anything.\n        // Pass `isExclusive` as `true` if you are dealing with an end date.\n        // `inc` defaults to `1` (increment one day forward each time)\n        DateProfileGenerator.prototype.skipHiddenDays = function (date, inc, isExclusive) {\n            if (inc === void 0) { inc = 1; }\n            if (isExclusive === void 0) { isExclusive = false; }\n            while (this.isHiddenDayHash[(date.getUTCDay() + (isExclusive ? inc : 0) + 7) % 7]) {\n                date = addDays(date, inc);\n            }\n            return date;\n        };\n        return DateProfileGenerator;\n    }());\n    // TODO: find a way to avoid comparing DateProfiles. it's tedious\n    function isDateProfilesEqual(p0, p1) {\n        return rangesEqual(p0.validRange, p1.validRange) &&\n            rangesEqual(p0.activeRange, p1.activeRange) &&\n            rangesEqual(p0.renderRange, p1.renderRange) &&\n            durationsEqual(p0.minTime, p1.minTime) &&\n            durationsEqual(p0.maxTime, p1.maxTime);\n        /*\n        TODO: compare more?\n          currentRange: DateRange\n          currentRangeUnit: string\n          isRangeAllDay: boolean\n          isValid: boolean\n          dateIncrement: Duration\n        */\n    }\n\n    function reduce (state, action, calendar) {\n        var viewType = reduceViewType(state.viewType, action);\n        var dateProfile = reduceDateProfile(state.dateProfile, action, state.currentDate, viewType, calendar);\n        var eventSources = reduceEventSources(state.eventSources, action, dateProfile, calendar);\n        var nextState = __assign({}, state, { viewType: viewType,\n            dateProfile: dateProfile, currentDate: reduceCurrentDate(state.currentDate, action, dateProfile), eventSources: eventSources, eventStore: reduceEventStore(state.eventStore, action, eventSources, dateProfile, calendar), dateSelection: reduceDateSelection(state.dateSelection, action, calendar), eventSelection: reduceSelectedEvent(state.eventSelection, action), eventDrag: reduceEventDrag(state.eventDrag, action, eventSources, calendar), eventResize: reduceEventResize(state.eventResize, action, eventSources, calendar), eventSourceLoadingLevel: computeLoadingLevel(eventSources), loadingLevel: computeLoadingLevel(eventSources) });\n        for (var _i = 0, _a = calendar.pluginSystem.hooks.reducers; _i < _a.length; _i++) {\n            var reducerFunc = _a[_i];\n            nextState = reducerFunc(nextState, action, calendar);\n        }\n        // console.log(action.type, nextState)\n        return nextState;\n    }\n    function reduceViewType(currentViewType, action) {\n        switch (action.type) {\n            case 'SET_VIEW_TYPE':\n                return action.viewType;\n            default:\n                return currentViewType;\n        }\n    }\n    function reduceDateProfile(currentDateProfile, action, currentDate, viewType, calendar) {\n        var newDateProfile;\n        switch (action.type) {\n            case 'PREV':\n                newDateProfile = calendar.dateProfileGenerators[viewType].buildPrev(currentDateProfile, currentDate);\n                break;\n            case 'NEXT':\n                newDateProfile = calendar.dateProfileGenerators[viewType].buildNext(currentDateProfile, currentDate);\n                break;\n            case 'SET_DATE':\n                if (!currentDateProfile.activeRange ||\n                    !rangeContainsMarker(currentDateProfile.currentRange, action.dateMarker)) {\n                    newDateProfile = calendar.dateProfileGenerators[viewType].build(action.dateMarker, undefined, true // forceToValid\n                    );\n                }\n                break;\n            case 'SET_VIEW_TYPE':\n                var generator = calendar.dateProfileGenerators[viewType];\n                if (!generator) {\n                    throw new Error(viewType ?\n                        'The FullCalendar view \"' + viewType + '\" does not exist. Make sure your plugins are loaded correctly.' :\n                        'No available FullCalendar view plugins.');\n                }\n                newDateProfile = generator.build(action.dateMarker || currentDate, undefined, true // forceToValid\n                );\n                break;\n        }\n        if (newDateProfile &&\n            newDateProfile.isValid &&\n            !(currentDateProfile && isDateProfilesEqual(currentDateProfile, newDateProfile))) {\n            return newDateProfile;\n        }\n        else {\n            return currentDateProfile;\n        }\n    }\n    function reduceCurrentDate(currentDate, action, dateProfile) {\n        switch (action.type) {\n            case 'PREV':\n            case 'NEXT':\n                if (!rangeContainsMarker(dateProfile.currentRange, currentDate)) {\n                    return dateProfile.currentRange.start;\n                }\n                else {\n                    return currentDate;\n                }\n            case 'SET_DATE':\n            case 'SET_VIEW_TYPE':\n                var newDate = action.dateMarker || currentDate;\n                if (dateProfile.activeRange && !rangeContainsMarker(dateProfile.activeRange, newDate)) {\n                    return dateProfile.currentRange.start;\n                }\n                else {\n                    return newDate;\n                }\n            default:\n                return currentDate;\n        }\n    }\n    function reduceDateSelection(currentSelection, action, calendar) {\n        switch (action.type) {\n            case 'SELECT_DATES':\n                return action.selection;\n            case 'UNSELECT_DATES':\n                return null;\n            default:\n                return currentSelection;\n        }\n    }\n    function reduceSelectedEvent(currentInstanceId, action) {\n        switch (action.type) {\n            case 'SELECT_EVENT':\n                return action.eventInstanceId;\n            case 'UNSELECT_EVENT':\n                return '';\n            default:\n                return currentInstanceId;\n        }\n    }\n    function reduceEventDrag(currentDrag, action, sources, calendar) {\n        switch (action.type) {\n            case 'SET_EVENT_DRAG':\n                var newDrag = action.state;\n                return {\n                    affectedEvents: newDrag.affectedEvents,\n                    mutatedEvents: newDrag.mutatedEvents,\n                    isEvent: newDrag.isEvent,\n                    origSeg: newDrag.origSeg\n                };\n            case 'UNSET_EVENT_DRAG':\n                return null;\n            default:\n                return currentDrag;\n        }\n    }\n    function reduceEventResize(currentResize, action, sources, calendar) {\n        switch (action.type) {\n            case 'SET_EVENT_RESIZE':\n                var newResize = action.state;\n                return {\n                    affectedEvents: newResize.affectedEvents,\n                    mutatedEvents: newResize.mutatedEvents,\n                    isEvent: newResize.isEvent,\n                    origSeg: newResize.origSeg\n                };\n            case 'UNSET_EVENT_RESIZE':\n                return null;\n            default:\n                return currentResize;\n        }\n    }\n    function computeLoadingLevel(eventSources) {\n        var cnt = 0;\n        for (var sourceId in eventSources) {\n            if (eventSources[sourceId].isFetching) {\n                cnt++;\n            }\n        }\n        return cnt;\n    }\n\n    var STANDARD_PROPS = {\n        start: null,\n        end: null,\n        allDay: Boolean\n    };\n    function parseDateSpan(raw, dateEnv, defaultDuration) {\n        var span = parseOpenDateSpan(raw, dateEnv);\n        var range = span.range;\n        if (!range.start) {\n            return null;\n        }\n        if (!range.end) {\n            if (defaultDuration == null) {\n                return null;\n            }\n            else {\n                range.end = dateEnv.add(range.start, defaultDuration);\n            }\n        }\n        return span;\n    }\n    /*\n    TODO: somehow combine with parseRange?\n    Will return null if the start/end props were present but parsed invalidly.\n    */\n    function parseOpenDateSpan(raw, dateEnv) {\n        var leftovers = {};\n        var standardProps = refineProps(raw, STANDARD_PROPS, {}, leftovers);\n        var startMeta = standardProps.start ? dateEnv.createMarkerMeta(standardProps.start) : null;\n        var endMeta = standardProps.end ? dateEnv.createMarkerMeta(standardProps.end) : null;\n        var allDay = standardProps.allDay;\n        if (allDay == null) {\n            allDay = (startMeta && startMeta.isTimeUnspecified) &&\n                (!endMeta || endMeta.isTimeUnspecified);\n        }\n        // use this leftover object as the selection object\n        leftovers.range = {\n            start: startMeta ? startMeta.marker : null,\n            end: endMeta ? endMeta.marker : null\n        };\n        leftovers.allDay = allDay;\n        return leftovers;\n    }\n    function isDateSpansEqual(span0, span1) {\n        return rangesEqual(span0.range, span1.range) &&\n            span0.allDay === span1.allDay &&\n            isSpanPropsEqual(span0, span1);\n    }\n    // the NON-DATE-RELATED props\n    function isSpanPropsEqual(span0, span1) {\n        for (var propName in span1) {\n            if (propName !== 'range' && propName !== 'allDay') {\n                if (span0[propName] !== span1[propName]) {\n                    return false;\n                }\n            }\n        }\n        // are there any props that span0 has that span1 DOESN'T have?\n        // both have range/allDay, so no need to special-case.\n        for (var propName in span0) {\n            if (!(propName in span1)) {\n                return false;\n            }\n        }\n        return true;\n    }\n    function buildDateSpanApi(span, dateEnv) {\n        return {\n            start: dateEnv.toDate(span.range.start),\n            end: dateEnv.toDate(span.range.end),\n            startStr: dateEnv.formatIso(span.range.start, { omitTime: span.allDay }),\n            endStr: dateEnv.formatIso(span.range.end, { omitTime: span.allDay }),\n            allDay: span.allDay\n        };\n    }\n    function buildDatePointApi(span, dateEnv) {\n        return {\n            date: dateEnv.toDate(span.range.start),\n            dateStr: dateEnv.formatIso(span.range.start, { omitTime: span.allDay }),\n            allDay: span.allDay\n        };\n    }\n    function fabricateEventRange(dateSpan, eventUiBases, calendar) {\n        var def = parseEventDef({ editable: false }, '', // sourceId\n        dateSpan.allDay, true, // hasEnd\n        calendar);\n        return {\n            def: def,\n            ui: compileEventUi(def, eventUiBases),\n            instance: createEventInstance(def.defId, dateSpan.range),\n            range: dateSpan.range,\n            isStart: true,\n            isEnd: true\n        };\n    }\n\n    function compileViewDefs(defaultConfigs, overrideConfigs) {\n        var hash = {};\n        var viewType;\n        for (viewType in defaultConfigs) {\n            ensureViewDef(viewType, hash, defaultConfigs, overrideConfigs);\n        }\n        for (viewType in overrideConfigs) {\n            ensureViewDef(viewType, hash, defaultConfigs, overrideConfigs);\n        }\n        return hash;\n    }\n    function ensureViewDef(viewType, hash, defaultConfigs, overrideConfigs) {\n        if (hash[viewType]) {\n            return hash[viewType];\n        }\n        var viewDef = buildViewDef(viewType, hash, defaultConfigs, overrideConfigs);\n        if (viewDef) {\n            hash[viewType] = viewDef;\n        }\n        return viewDef;\n    }\n    function buildViewDef(viewType, hash, defaultConfigs, overrideConfigs) {\n        var defaultConfig = defaultConfigs[viewType];\n        var overrideConfig = overrideConfigs[viewType];\n        var queryProp = function (name) {\n            return (defaultConfig && defaultConfig[name] !== null) ? defaultConfig[name] :\n                ((overrideConfig && overrideConfig[name] !== null) ? overrideConfig[name] : null);\n        };\n        var theClass = queryProp('class');\n        var superType = queryProp('superType');\n        if (!superType && theClass) {\n            superType =\n                findViewNameBySubclass(theClass, overrideConfigs) ||\n                    findViewNameBySubclass(theClass, defaultConfigs);\n        }\n        var superDef = null;\n        if (superType) {\n            if (superType === viewType) {\n                throw new Error('Can\\'t have a custom view type that references itself');\n            }\n            superDef = ensureViewDef(superType, hash, defaultConfigs, overrideConfigs);\n        }\n        if (!theClass && superDef) {\n            theClass = superDef.class;\n        }\n        if (!theClass) {\n            return null; // don't throw a warning, might be settings for a single-unit view\n        }\n        return {\n            type: viewType,\n            class: theClass,\n            defaults: __assign({}, (superDef ? superDef.defaults : {}), (defaultConfig ? defaultConfig.options : {})),\n            overrides: __assign({}, (superDef ? superDef.overrides : {}), (overrideConfig ? overrideConfig.options : {}))\n        };\n    }\n    function findViewNameBySubclass(viewSubclass, configs) {\n        var superProto = Object.getPrototypeOf(viewSubclass.prototype);\n        for (var viewType in configs) {\n            var parsed = configs[viewType];\n            // need DIRECT subclass, so instanceof won't do it\n            if (parsed.class && parsed.class.prototype === superProto) {\n                return viewType;\n            }\n        }\n        return '';\n    }\n\n    function parseViewConfigs(inputs) {\n        return mapHash(inputs, parseViewConfig);\n    }\n    var VIEW_DEF_PROPS = {\n        type: String,\n        class: null\n    };\n    function parseViewConfig(input) {\n        if (typeof input === 'function') {\n            input = { class: input };\n        }\n        var options = {};\n        var props = refineProps(input, VIEW_DEF_PROPS, {}, options);\n        return {\n            superType: props.type,\n            class: props.class,\n            options: options\n        };\n    }\n\n    function buildViewSpecs(defaultInputs, optionsManager) {\n        var defaultConfigs = parseViewConfigs(defaultInputs);\n        var overrideConfigs = parseViewConfigs(optionsManager.overrides.views);\n        var viewDefs = compileViewDefs(defaultConfigs, overrideConfigs);\n        return mapHash(viewDefs, function (viewDef) {\n            return buildViewSpec(viewDef, overrideConfigs, optionsManager);\n        });\n    }\n    function buildViewSpec(viewDef, overrideConfigs, optionsManager) {\n        var durationInput = viewDef.overrides.duration ||\n            viewDef.defaults.duration ||\n            optionsManager.dynamicOverrides.duration ||\n            optionsManager.overrides.duration;\n        var duration = null;\n        var durationUnit = '';\n        var singleUnit = '';\n        var singleUnitOverrides = {};\n        if (durationInput) {\n            duration = createDuration(durationInput);\n            if (duration) { // valid?\n                var denom = greatestDurationDenominator(duration, !getWeeksFromInput(durationInput));\n                durationUnit = denom.unit;\n                if (denom.value === 1) {\n                    singleUnit = durationUnit;\n                    singleUnitOverrides = overrideConfigs[durationUnit] ? overrideConfigs[durationUnit].options : {};\n                }\n            }\n        }\n        var queryButtonText = function (options) {\n            var buttonTextMap = options.buttonText || {};\n            var buttonTextKey = viewDef.defaults.buttonTextKey;\n            if (buttonTextKey != null && buttonTextMap[buttonTextKey] != null) {\n                return buttonTextMap[buttonTextKey];\n            }\n            if (buttonTextMap[viewDef.type] != null) {\n                return buttonTextMap[viewDef.type];\n            }\n            if (buttonTextMap[singleUnit] != null) {\n                return buttonTextMap[singleUnit];\n            }\n        };\n        return {\n            type: viewDef.type,\n            class: viewDef.class,\n            duration: duration,\n            durationUnit: durationUnit,\n            singleUnit: singleUnit,\n            options: __assign({}, globalDefaults, viewDef.defaults, optionsManager.dirDefaults, optionsManager.localeDefaults, optionsManager.overrides, singleUnitOverrides, viewDef.overrides, optionsManager.dynamicOverrides),\n            buttonTextOverride: queryButtonText(optionsManager.dynamicOverrides) ||\n                queryButtonText(optionsManager.overrides) || // constructor-specified buttonText lookup hash takes precedence\n                viewDef.overrides.buttonText,\n            buttonTextDefault: queryButtonText(optionsManager.localeDefaults) ||\n                queryButtonText(optionsManager.dirDefaults) ||\n                viewDef.defaults.buttonText ||\n                queryButtonText(globalDefaults) ||\n                viewDef.type // fall back to given view name\n        };\n    }\n\n    var Toolbar = /** @class */ (function (_super) {\n        __extends(Toolbar, _super);\n        function Toolbar(extraClassName) {\n            var _this = _super.call(this) || this;\n            _this._renderLayout = memoizeRendering(_this.renderLayout, _this.unrenderLayout);\n            _this._updateTitle = memoizeRendering(_this.updateTitle, null, [_this._renderLayout]);\n            _this._updateActiveButton = memoizeRendering(_this.updateActiveButton, null, [_this._renderLayout]);\n            _this._updateToday = memoizeRendering(_this.updateToday, null, [_this._renderLayout]);\n            _this._updatePrev = memoizeRendering(_this.updatePrev, null, [_this._renderLayout]);\n            _this._updateNext = memoizeRendering(_this.updateNext, null, [_this._renderLayout]);\n            _this.el = createElement('div', { className: 'fc-toolbar ' + extraClassName });\n            return _this;\n        }\n        Toolbar.prototype.destroy = function () {\n            _super.prototype.destroy.call(this);\n            this._renderLayout.unrender(); // should unrender everything else\n            removeElement(this.el);\n        };\n        Toolbar.prototype.render = function (props) {\n            this._renderLayout(props.layout);\n            this._updateTitle(props.title);\n            this._updateActiveButton(props.activeButton);\n            this._updateToday(props.isTodayEnabled);\n            this._updatePrev(props.isPrevEnabled);\n            this._updateNext(props.isNextEnabled);\n        };\n        Toolbar.prototype.renderLayout = function (layout) {\n            var el = this.el;\n            this.viewsWithButtons = [];\n            appendToElement(el, this.renderSection('left', layout.left));\n            appendToElement(el, this.renderSection('center', layout.center));\n            appendToElement(el, this.renderSection('right', layout.right));\n        };\n        Toolbar.prototype.unrenderLayout = function () {\n            this.el.innerHTML = '';\n        };\n        Toolbar.prototype.renderSection = function (position, buttonStr) {\n            var _this = this;\n            var _a = this.context, theme = _a.theme, calendar = _a.calendar;\n            var optionsManager = calendar.optionsManager;\n            var viewSpecs = calendar.viewSpecs;\n            var sectionEl = createElement('div', { className: 'fc-' + position });\n            var calendarCustomButtons = optionsManager.computed.customButtons || {};\n            var calendarButtonTextOverrides = optionsManager.overrides.buttonText || {};\n            var calendarButtonText = optionsManager.computed.buttonText || {};\n            if (buttonStr) {\n                buttonStr.split(' ').forEach(function (buttonGroupStr, i) {\n                    var groupChildren = [];\n                    var isOnlyButtons = true;\n                    var groupEl;\n                    buttonGroupStr.split(',').forEach(function (buttonName, j) {\n                        var customButtonProps;\n                        var viewSpec;\n                        var buttonClick;\n                        var buttonIcon; // only one of these will be set\n                        var buttonText; // \"\n                        var buttonInnerHtml;\n                        var buttonClasses;\n                        var buttonEl;\n                        var buttonAriaAttr;\n                        if (buttonName === 'title') {\n                            groupChildren.push(htmlToElement('<h2>&nbsp;</h2>')); // we always want it to take up height\n                            isOnlyButtons = false;\n                        }\n                        else {\n                            if ((customButtonProps = calendarCustomButtons[buttonName])) {\n                                buttonClick = function (ev) {\n                                    if (customButtonProps.click) {\n                                        customButtonProps.click.call(buttonEl, ev);\n                                    }\n                                };\n                                (buttonIcon = theme.getCustomButtonIconClass(customButtonProps)) ||\n                                    (buttonIcon = theme.getIconClass(buttonName)) ||\n                                    (buttonText = customButtonProps.text);\n                            }\n                            else if ((viewSpec = viewSpecs[buttonName])) {\n                                _this.viewsWithButtons.push(buttonName);\n                                buttonClick = function () {\n                                    calendar.changeView(buttonName);\n                                };\n                                (buttonText = viewSpec.buttonTextOverride) ||\n                                    (buttonIcon = theme.getIconClass(buttonName)) ||\n                                    (buttonText = viewSpec.buttonTextDefault);\n                            }\n                            else if (calendar[buttonName]) { // a calendar method\n                                buttonClick = function () {\n                                    calendar[buttonName]();\n                                };\n                                (buttonText = calendarButtonTextOverrides[buttonName]) ||\n                                    (buttonIcon = theme.getIconClass(buttonName)) ||\n                                    (buttonText = calendarButtonText[buttonName]);\n                                //            ^ everything else is considered default\n                            }\n                            if (buttonClick) {\n                                buttonClasses = [\n                                    'fc-' + buttonName + '-button',\n                                    theme.getClass('button')\n                                ];\n                                if (buttonText) {\n                                    buttonInnerHtml = htmlEscape(buttonText);\n                                    buttonAriaAttr = '';\n                                }\n                                else if (buttonIcon) {\n                                    buttonInnerHtml = \"<span class='\" + buttonIcon + \"'></span>\";\n                                    buttonAriaAttr = ' aria-label=\"' + buttonName + '\"';\n                                }\n                                buttonEl = htmlToElement(// type=\"button\" so that it doesn't submit a form\n                                '<button type=\"button\" class=\"' + buttonClasses.join(' ') + '\"' +\n                                    buttonAriaAttr +\n                                    '>' + buttonInnerHtml + '</button>');\n                                buttonEl.addEventListener('click', buttonClick);\n                                groupChildren.push(buttonEl);\n                            }\n                        }\n                    });\n                    if (groupChildren.length > 1) {\n                        groupEl = document.createElement('div');\n                        var buttonGroupClassName = theme.getClass('buttonGroup');\n                        if (isOnlyButtons && buttonGroupClassName) {\n                            groupEl.classList.add(buttonGroupClassName);\n                        }\n                        appendToElement(groupEl, groupChildren);\n                        sectionEl.appendChild(groupEl);\n                    }\n                    else {\n                        appendToElement(sectionEl, groupChildren); // 1 or 0 children\n                    }\n                });\n            }\n            return sectionEl;\n        };\n        Toolbar.prototype.updateToday = function (isTodayEnabled) {\n            this.toggleButtonEnabled('today', isTodayEnabled);\n        };\n        Toolbar.prototype.updatePrev = function (isPrevEnabled) {\n            this.toggleButtonEnabled('prev', isPrevEnabled);\n        };\n        Toolbar.prototype.updateNext = function (isNextEnabled) {\n            this.toggleButtonEnabled('next', isNextEnabled);\n        };\n        Toolbar.prototype.updateTitle = function (text) {\n            findElements(this.el, 'h2').forEach(function (titleEl) {\n                titleEl.innerText = text;\n            });\n        };\n        Toolbar.prototype.updateActiveButton = function (buttonName) {\n            var theme = this.context.theme;\n            var className = theme.getClass('buttonActive');\n            findElements(this.el, 'button').forEach(function (buttonEl) {\n                if (buttonName && buttonEl.classList.contains('fc-' + buttonName + '-button')) {\n                    buttonEl.classList.add(className);\n                }\n                else {\n                    buttonEl.classList.remove(className);\n                }\n            });\n        };\n        Toolbar.prototype.toggleButtonEnabled = function (buttonName, bool) {\n            findElements(this.el, '.fc-' + buttonName + '-button').forEach(function (buttonEl) {\n                buttonEl.disabled = !bool;\n            });\n        };\n        return Toolbar;\n    }(Component));\n\n    var CalendarComponent = /** @class */ (function (_super) {\n        __extends(CalendarComponent, _super);\n        function CalendarComponent(el) {\n            var _this = _super.call(this) || this;\n            _this.elClassNames = [];\n            _this.renderSkeleton = memoizeRendering(_this._renderSkeleton, _this._unrenderSkeleton);\n            _this.renderToolbars = memoizeRendering(_this._renderToolbars, _this._unrenderToolbars, [_this.renderSkeleton]);\n            _this.buildComponentContext = memoize(buildComponentContext);\n            _this.buildViewPropTransformers = memoize(buildViewPropTransformers);\n            _this.el = el;\n            _this.computeTitle = memoize(computeTitle);\n            _this.parseBusinessHours = memoize(function (input) {\n                return parseBusinessHours(input, _this.context.calendar);\n            });\n            return _this;\n        }\n        CalendarComponent.prototype.render = function (props, context) {\n            this.freezeHeight();\n            var title = this.computeTitle(props.dateProfile, props.viewSpec.options);\n            this.renderSkeleton(context);\n            this.renderToolbars(props.viewSpec, props.dateProfile, props.currentDate, title);\n            this.renderView(props, title);\n            this.updateSize();\n            this.thawHeight();\n        };\n        CalendarComponent.prototype.destroy = function () {\n            if (this.header) {\n                this.header.destroy();\n            }\n            if (this.footer) {\n                this.footer.destroy();\n            }\n            this.renderSkeleton.unrender(); // will call destroyView\n            _super.prototype.destroy.call(this);\n        };\n        CalendarComponent.prototype._renderSkeleton = function (context) {\n            this.updateElClassNames(context);\n            prependToElement(this.el, this.contentEl = createElement('div', { className: 'fc-view-container' }));\n            var calendar = context.calendar;\n            for (var _i = 0, _a = calendar.pluginSystem.hooks.viewContainerModifiers; _i < _a.length; _i++) {\n                var modifyViewContainer = _a[_i];\n                modifyViewContainer(this.contentEl, calendar);\n            }\n        };\n        CalendarComponent.prototype._unrenderSkeleton = function () {\n            // weird to have this here\n            if (this.view) {\n                this.savedScroll = this.view.queryScroll();\n                this.view.destroy();\n                this.view = null;\n            }\n            removeElement(this.contentEl);\n            this.removeElClassNames();\n        };\n        CalendarComponent.prototype.removeElClassNames = function () {\n            var classList = this.el.classList;\n            for (var _i = 0, _a = this.elClassNames; _i < _a.length; _i++) {\n                var className = _a[_i];\n                classList.remove(className);\n            }\n            this.elClassNames = [];\n        };\n        CalendarComponent.prototype.updateElClassNames = function (context) {\n            this.removeElClassNames();\n            var theme = context.theme, options = context.options;\n            this.elClassNames = [\n                'fc',\n                'fc-' + options.dir,\n                theme.getClass('widget')\n            ];\n            var classList = this.el.classList;\n            for (var _i = 0, _a = this.elClassNames; _i < _a.length; _i++) {\n                var className = _a[_i];\n                classList.add(className);\n            }\n        };\n        CalendarComponent.prototype._renderToolbars = function (viewSpec, dateProfile, currentDate, title) {\n            var _a = this, context = _a.context, header = _a.header, footer = _a.footer;\n            var options = context.options, calendar = context.calendar;\n            var headerLayout = options.header;\n            var footerLayout = options.footer;\n            var dateProfileGenerator = this.props.dateProfileGenerator;\n            var now = calendar.getNow();\n            var todayInfo = dateProfileGenerator.build(now);\n            var prevInfo = dateProfileGenerator.buildPrev(dateProfile, currentDate);\n            var nextInfo = dateProfileGenerator.buildNext(dateProfile, currentDate);\n            var toolbarProps = {\n                title: title,\n                activeButton: viewSpec.type,\n                isTodayEnabled: todayInfo.isValid && !rangeContainsMarker(dateProfile.currentRange, now),\n                isPrevEnabled: prevInfo.isValid,\n                isNextEnabled: nextInfo.isValid\n            };\n            if (headerLayout) {\n                if (!header) {\n                    header = this.header = new Toolbar('fc-header-toolbar');\n                    prependToElement(this.el, header.el);\n                }\n                header.receiveProps(__assign({ layout: headerLayout }, toolbarProps), context);\n            }\n            else if (header) {\n                header.destroy();\n                header = this.header = null;\n            }\n            if (footerLayout) {\n                if (!footer) {\n                    footer = this.footer = new Toolbar('fc-footer-toolbar');\n                    appendToElement(this.el, footer.el);\n                }\n                footer.receiveProps(__assign({ layout: footerLayout }, toolbarProps), context);\n            }\n            else if (footer) {\n                footer.destroy();\n                footer = this.footer = null;\n            }\n        };\n        CalendarComponent.prototype._unrenderToolbars = function () {\n            if (this.header) {\n                this.header.destroy();\n                this.header = null;\n            }\n            if (this.footer) {\n                this.footer.destroy();\n                this.footer = null;\n            }\n        };\n        CalendarComponent.prototype.renderView = function (props, title) {\n            var view = this.view;\n            var _a = this.context, calendar = _a.calendar, options = _a.options;\n            var viewSpec = props.viewSpec, dateProfileGenerator = props.dateProfileGenerator;\n            if (!view || view.viewSpec !== viewSpec) {\n                if (view) {\n                    view.destroy();\n                }\n                view = this.view = new viewSpec['class'](viewSpec, this.contentEl);\n                if (this.savedScroll) {\n                    view.addScroll(this.savedScroll, true);\n                    this.savedScroll = null;\n                }\n            }\n            view.title = title; // for the API\n            var viewProps = {\n                dateProfileGenerator: dateProfileGenerator,\n                dateProfile: props.dateProfile,\n                businessHours: this.parseBusinessHours(viewSpec.options.businessHours),\n                eventStore: props.eventStore,\n                eventUiBases: props.eventUiBases,\n                dateSelection: props.dateSelection,\n                eventSelection: props.eventSelection,\n                eventDrag: props.eventDrag,\n                eventResize: props.eventResize\n            };\n            var transformers = this.buildViewPropTransformers(calendar.pluginSystem.hooks.viewPropsTransformers);\n            for (var _i = 0, transformers_1 = transformers; _i < transformers_1.length; _i++) {\n                var transformer = transformers_1[_i];\n                __assign(viewProps, transformer.transform(viewProps, viewSpec, props, options));\n            }\n            view.receiveProps(viewProps, this.buildComponentContext(this.context, viewSpec, view));\n        };\n        // Sizing\n        // -----------------------------------------------------------------------------------------------------------------\n        CalendarComponent.prototype.updateSize = function (isResize) {\n            if (isResize === void 0) { isResize = false; }\n            var view = this.view;\n            if (!view) {\n                return; // why?\n            }\n            if (isResize || this.isHeightAuto == null) {\n                this.computeHeightVars();\n            }\n            view.updateSize(isResize, this.viewHeight, this.isHeightAuto);\n            view.updateNowIndicator(); // we need to guarantee this will run after updateSize\n            view.popScroll(isResize);\n        };\n        CalendarComponent.prototype.computeHeightVars = function () {\n            var calendar = this.context.calendar; // yuck. need to handle dynamic options\n            var heightInput = calendar.opt('height');\n            var contentHeightInput = calendar.opt('contentHeight');\n            this.isHeightAuto = heightInput === 'auto' || contentHeightInput === 'auto';\n            if (typeof contentHeightInput === 'number') { // exists and not 'auto'\n                this.viewHeight = contentHeightInput;\n            }\n            else if (typeof contentHeightInput === 'function') { // exists and is a function\n                this.viewHeight = contentHeightInput();\n            }\n            else if (typeof heightInput === 'number') { // exists and not 'auto'\n                this.viewHeight = heightInput - this.queryToolbarsHeight();\n            }\n            else if (typeof heightInput === 'function') { // exists and is a function\n                this.viewHeight = heightInput() - this.queryToolbarsHeight();\n            }\n            else if (heightInput === 'parent') { // set to height of parent element\n                var parentEl = this.el.parentNode;\n                this.viewHeight = parentEl.getBoundingClientRect().height - this.queryToolbarsHeight();\n            }\n            else {\n                this.viewHeight = Math.round(this.contentEl.getBoundingClientRect().width /\n                    Math.max(calendar.opt('aspectRatio'), .5));\n            }\n        };\n        CalendarComponent.prototype.queryToolbarsHeight = function () {\n            var height = 0;\n            if (this.header) {\n                height += computeHeightAndMargins(this.header.el);\n            }\n            if (this.footer) {\n                height += computeHeightAndMargins(this.footer.el);\n            }\n            return height;\n        };\n        // Height \"Freezing\"\n        // -----------------------------------------------------------------------------------------------------------------\n        CalendarComponent.prototype.freezeHeight = function () {\n            applyStyle(this.el, {\n                height: this.el.getBoundingClientRect().height,\n                overflow: 'hidden'\n            });\n        };\n        CalendarComponent.prototype.thawHeight = function () {\n            applyStyle(this.el, {\n                height: '',\n                overflow: ''\n            });\n        };\n        return CalendarComponent;\n    }(Component));\n    // Title and Date Formatting\n    // -----------------------------------------------------------------------------------------------------------------\n    // Computes what the title at the top of the calendar should be for this view\n    function computeTitle(dateProfile, viewOptions) {\n        var range;\n        // for views that span a large unit of time, show the proper interval, ignoring stray days before and after\n        if (/^(year|month)$/.test(dateProfile.currentRangeUnit)) {\n            range = dateProfile.currentRange;\n        }\n        else { // for day units or smaller, use the actual day range\n            range = dateProfile.activeRange;\n        }\n        return this.context.dateEnv.formatRange(range.start, range.end, createFormatter(viewOptions.titleFormat || computeTitleFormat(dateProfile), viewOptions.titleRangeSeparator), { isEndExclusive: dateProfile.isRangeAllDay });\n    }\n    // Generates the format string that should be used to generate the title for the current date range.\n    // Attempts to compute the most appropriate format if not explicitly specified with `titleFormat`.\n    function computeTitleFormat(dateProfile) {\n        var currentRangeUnit = dateProfile.currentRangeUnit;\n        if (currentRangeUnit === 'year') {\n            return { year: 'numeric' };\n        }\n        else if (currentRangeUnit === 'month') {\n            return { year: 'numeric', month: 'long' }; // like \"September 2014\"\n        }\n        else {\n            var days = diffWholeDays(dateProfile.currentRange.start, dateProfile.currentRange.end);\n            if (days !== null && days > 1) {\n                // multi-day range. shorter, like \"Sep 9 - 10 2014\"\n                return { year: 'numeric', month: 'short', day: 'numeric' };\n            }\n            else {\n                // one day. longer, like \"September 9 2014\"\n                return { year: 'numeric', month: 'long', day: 'numeric' };\n            }\n        }\n    }\n    // build a context scoped to the view\n    function buildComponentContext(context, viewSpec, view) {\n        return context.extend(viewSpec.options, view);\n    }\n    // Plugin\n    // -----------------------------------------------------------------------------------------------------------------\n    function buildViewPropTransformers(theClasses) {\n        return theClasses.map(function (theClass) {\n            return new theClass();\n        });\n    }\n\n    var Interaction = /** @class */ (function () {\n        function Interaction(settings) {\n            this.component = settings.component;\n        }\n        Interaction.prototype.destroy = function () {\n        };\n        return Interaction;\n    }());\n    function parseInteractionSettings(component, input) {\n        return {\n            component: component,\n            el: input.el,\n            useEventCenter: input.useEventCenter != null ? input.useEventCenter : true\n        };\n    }\n    function interactionSettingsToStore(settings) {\n        var _a;\n        return _a = {},\n            _a[settings.component.uid] = settings,\n            _a;\n    }\n    // global state\n    var interactionSettingsStore = {};\n\n    /*\n    Detects when the user clicks on an event within a DateComponent\n    */\n    var EventClicking = /** @class */ (function (_super) {\n        __extends(EventClicking, _super);\n        function EventClicking(settings) {\n            var _this = _super.call(this, settings) || this;\n            _this.handleSegClick = function (ev, segEl) {\n                var component = _this.component;\n                var _a = component.context, calendar = _a.calendar, view = _a.view;\n                var seg = getElSeg(segEl);\n                if (seg && // might be the <div> surrounding the more link\n                    component.isValidSegDownEl(ev.target)) {\n                    // our way to simulate a link click for elements that can't be <a> tags\n                    // grab before trigger fired in case trigger trashes DOM thru rerendering\n                    var hasUrlContainer = elementClosest(ev.target, '.fc-has-url');\n                    var url = hasUrlContainer ? hasUrlContainer.querySelector('a[href]').href : '';\n                    calendar.publiclyTrigger('eventClick', [\n                        {\n                            el: segEl,\n                            event: new EventApi(component.context.calendar, seg.eventRange.def, seg.eventRange.instance),\n                            jsEvent: ev,\n                            view: view\n                        }\n                    ]);\n                    if (url && !ev.defaultPrevented) {\n                        window.location.href = url;\n                    }\n                }\n            };\n            var component = settings.component;\n            _this.destroy = listenBySelector(component.el, 'click', component.fgSegSelector + ',' + component.bgSegSelector, _this.handleSegClick);\n            return _this;\n        }\n        return EventClicking;\n    }(Interaction));\n\n    /*\n    Triggers events and adds/removes core classNames when the user's pointer\n    enters/leaves event-elements of a component.\n    */\n    var EventHovering = /** @class */ (function (_super) {\n        __extends(EventHovering, _super);\n        function EventHovering(settings) {\n            var _this = _super.call(this, settings) || this;\n            // for simulating an eventMouseLeave when the event el is destroyed while mouse is over it\n            _this.handleEventElRemove = function (el) {\n                if (el === _this.currentSegEl) {\n                    _this.handleSegLeave(null, _this.currentSegEl);\n                }\n            };\n            _this.handleSegEnter = function (ev, segEl) {\n                if (getElSeg(segEl)) { // TODO: better way to make sure not hovering over more+ link or its wrapper\n                    segEl.classList.add('fc-allow-mouse-resize');\n                    _this.currentSegEl = segEl;\n                    _this.triggerEvent('eventMouseEnter', ev, segEl);\n                }\n            };\n            _this.handleSegLeave = function (ev, segEl) {\n                if (_this.currentSegEl) {\n                    segEl.classList.remove('fc-allow-mouse-resize');\n                    _this.currentSegEl = null;\n                    _this.triggerEvent('eventMouseLeave', ev, segEl);\n                }\n            };\n            var component = settings.component;\n            _this.removeHoverListeners = listenToHoverBySelector(component.el, component.fgSegSelector + ',' + component.bgSegSelector, _this.handleSegEnter, _this.handleSegLeave);\n            // how to make sure component already has context?\n            component.context.calendar.on('eventElRemove', _this.handleEventElRemove);\n            return _this;\n        }\n        EventHovering.prototype.destroy = function () {\n            this.removeHoverListeners();\n            this.component.context.calendar.off('eventElRemove', this.handleEventElRemove);\n        };\n        EventHovering.prototype.triggerEvent = function (publicEvName, ev, segEl) {\n            var component = this.component;\n            var _a = component.context, calendar = _a.calendar, view = _a.view;\n            var seg = getElSeg(segEl);\n            if (!ev || component.isValidSegDownEl(ev.target)) {\n                calendar.publiclyTrigger(publicEvName, [\n                    {\n                        el: segEl,\n                        event: new EventApi(calendar, seg.eventRange.def, seg.eventRange.instance),\n                        jsEvent: ev,\n                        view: view\n                    }\n                ]);\n            }\n        };\n        return EventHovering;\n    }(Interaction));\n\n    var StandardTheme = /** @class */ (function (_super) {\n        __extends(StandardTheme, _super);\n        function StandardTheme() {\n            return _super !== null && _super.apply(this, arguments) || this;\n        }\n        return StandardTheme;\n    }(Theme));\n    StandardTheme.prototype.classes = {\n        widget: 'fc-unthemed',\n        widgetHeader: 'fc-widget-header',\n        widgetContent: 'fc-widget-content',\n        buttonGroup: 'fc-button-group',\n        button: 'fc-button fc-button-primary',\n        buttonActive: 'fc-button-active',\n        popoverHeader: 'fc-widget-header',\n        popoverContent: 'fc-widget-content',\n        // day grid\n        headerRow: 'fc-widget-header',\n        dayRow: 'fc-widget-content',\n        // list view\n        listView: 'fc-widget-content'\n    };\n    StandardTheme.prototype.baseIconClass = 'fc-icon';\n    StandardTheme.prototype.iconClasses = {\n        close: 'fc-icon-x',\n        prev: 'fc-icon-chevron-left',\n        next: 'fc-icon-chevron-right',\n        prevYear: 'fc-icon-chevrons-left',\n        nextYear: 'fc-icon-chevrons-right'\n    };\n    StandardTheme.prototype.iconOverrideOption = 'buttonIcons';\n    StandardTheme.prototype.iconOverrideCustomButtonOption = 'icon';\n    StandardTheme.prototype.iconOverridePrefix = 'fc-icon-';\n\n    var Calendar = /** @class */ (function () {\n        function Calendar(el, overrides) {\n            var _this = this;\n            this.buildComponentContext = memoize(buildComponentContext$1);\n            this.parseRawLocales = memoize(parseRawLocales);\n            this.buildLocale = memoize(buildLocale);\n            this.buildDateEnv = memoize(buildDateEnv);\n            this.buildTheme = memoize(buildTheme);\n            this.buildEventUiSingleBase = memoize(this._buildEventUiSingleBase);\n            this.buildSelectionConfig = memoize(this._buildSelectionConfig);\n            this.buildEventUiBySource = memoizeOutput(buildEventUiBySource, isPropsEqual);\n            this.buildEventUiBases = memoize(buildEventUiBases);\n            this.interactionsStore = {};\n            this.actionQueue = [];\n            this.isReducing = false;\n            // isDisplaying: boolean = false // installed in DOM? accepting renders?\n            this.needsRerender = false; // needs a render?\n            this.isRendering = false; // currently in the executeRender function?\n            this.renderingPauseDepth = 0;\n            this.buildDelayedRerender = memoize(buildDelayedRerender);\n            this.afterSizingTriggers = {};\n            this.isViewUpdated = false;\n            this.isDatesUpdated = false;\n            this.isEventsUpdated = false;\n            this.el = el;\n            this.optionsManager = new OptionsManager(overrides || {});\n            this.pluginSystem = new PluginSystem();\n            // only do once. don't do in handleOptions. because can't remove plugins\n            this.addPluginInputs(this.optionsManager.computed.plugins || []);\n            this.handleOptions(this.optionsManager.computed);\n            this.publiclyTrigger('_init'); // for tests\n            this.hydrate();\n            this.calendarInteractions = this.pluginSystem.hooks.calendarInteractions\n                .map(function (calendarInteractionClass) {\n                return new calendarInteractionClass(_this);\n            });\n        }\n        Calendar.prototype.addPluginInputs = function (pluginInputs) {\n            var pluginDefs = refinePluginDefs(pluginInputs);\n            for (var _i = 0, pluginDefs_1 = pluginDefs; _i < pluginDefs_1.length; _i++) {\n                var pluginDef = pluginDefs_1[_i];\n                this.pluginSystem.add(pluginDef);\n            }\n        };\n        Object.defineProperty(Calendar.prototype, \"view\", {\n            // public API\n            get: function () {\n                return this.component ? this.component.view : null;\n            },\n            enumerable: true,\n            configurable: true\n        });\n        // Public API for rendering\n        // -----------------------------------------------------------------------------------------------------------------\n        Calendar.prototype.render = function () {\n            if (!this.component) {\n                this.component = new CalendarComponent(this.el);\n                this.renderableEventStore = createEmptyEventStore();\n                this.bindHandlers();\n                this.executeRender();\n            }\n            else {\n                this.requestRerender();\n            }\n        };\n        Calendar.prototype.destroy = function () {\n            if (this.component) {\n                this.unbindHandlers();\n                this.component.destroy(); // don't null-out. in case API needs access\n                this.component = null; // umm ???\n                for (var _i = 0, _a = this.calendarInteractions; _i < _a.length; _i++) {\n                    var interaction = _a[_i];\n                    interaction.destroy();\n                }\n                this.publiclyTrigger('_destroyed');\n            }\n        };\n        // Handlers\n        // -----------------------------------------------------------------------------------------------------------------\n        Calendar.prototype.bindHandlers = function () {\n            var _this = this;\n            // event delegation for nav links\n            this.removeNavLinkListener = listenBySelector(this.el, 'click', 'a[data-goto]', function (ev, anchorEl) {\n                var gotoOptions = anchorEl.getAttribute('data-goto');\n                gotoOptions = gotoOptions ? JSON.parse(gotoOptions) : {};\n                var dateEnv = _this.dateEnv;\n                var dateMarker = dateEnv.createMarker(gotoOptions.date);\n                var viewType = gotoOptions.type;\n                // property like \"navLinkDayClick\". might be a string or a function\n                var customAction = _this.viewOpt('navLink' + capitaliseFirstLetter(viewType) + 'Click');\n                if (typeof customAction === 'function') {\n                    customAction(dateEnv.toDate(dateMarker), ev);\n                }\n                else {\n                    if (typeof customAction === 'string') {\n                        viewType = customAction;\n                    }\n                    _this.zoomTo(dateMarker, viewType);\n                }\n            });\n            if (this.opt('handleWindowResize')) {\n                window.addEventListener('resize', this.windowResizeProxy = debounce(// prevents rapid calls\n                this.windowResize.bind(this), this.opt('windowResizeDelay')));\n            }\n        };\n        Calendar.prototype.unbindHandlers = function () {\n            this.removeNavLinkListener();\n            if (this.windowResizeProxy) {\n                window.removeEventListener('resize', this.windowResizeProxy);\n                this.windowResizeProxy = null;\n            }\n        };\n        // Dispatcher\n        // -----------------------------------------------------------------------------------------------------------------\n        Calendar.prototype.hydrate = function () {\n            var _this = this;\n            this.state = this.buildInitialState();\n            var rawSources = this.opt('eventSources') || [];\n            var singleRawSource = this.opt('events');\n            var sources = []; // parsed\n            if (singleRawSource) {\n                rawSources.unshift(singleRawSource);\n            }\n            for (var _i = 0, rawSources_1 = rawSources; _i < rawSources_1.length; _i++) {\n                var rawSource = rawSources_1[_i];\n                var source = parseEventSource(rawSource, this);\n                if (source) {\n                    sources.push(source);\n                }\n            }\n            this.batchRendering(function () {\n                _this.dispatch({ type: 'INIT' }); // pass in sources here?\n                _this.dispatch({ type: 'ADD_EVENT_SOURCES', sources: sources });\n                _this.dispatch({\n                    type: 'SET_VIEW_TYPE',\n                    viewType: _this.opt('defaultView') || _this.pluginSystem.hooks.defaultView\n                });\n            });\n        };\n        Calendar.prototype.buildInitialState = function () {\n            return {\n                viewType: null,\n                loadingLevel: 0,\n                eventSourceLoadingLevel: 0,\n                currentDate: this.getInitialDate(),\n                dateProfile: null,\n                eventSources: {},\n                eventStore: createEmptyEventStore(),\n                dateSelection: null,\n                eventSelection: '',\n                eventDrag: null,\n                eventResize: null\n            };\n        };\n        Calendar.prototype.dispatch = function (action) {\n            this.actionQueue.push(action);\n            if (!this.isReducing) {\n                this.isReducing = true;\n                var oldState = this.state;\n                while (this.actionQueue.length) {\n                    this.state = this.reduce(this.state, this.actionQueue.shift(), this);\n                }\n                var newState = this.state;\n                this.isReducing = false;\n                if (!oldState.loadingLevel && newState.loadingLevel) {\n                    this.publiclyTrigger('loading', [true]);\n                }\n                else if (oldState.loadingLevel && !newState.loadingLevel) {\n                    this.publiclyTrigger('loading', [false]);\n                }\n                var view = this.component && this.component.view;\n                if (oldState.eventStore !== newState.eventStore) {\n                    if (oldState.eventStore) {\n                        this.isEventsUpdated = true;\n                    }\n                }\n                if (oldState.dateProfile !== newState.dateProfile) {\n                    if (oldState.dateProfile && view) { // why would view be null!?\n                        this.publiclyTrigger('datesDestroy', [\n                            {\n                                view: view,\n                                el: view.el\n                            }\n                        ]);\n                    }\n                    this.isDatesUpdated = true;\n                }\n                if (oldState.viewType !== newState.viewType) {\n                    if (oldState.viewType && view) { // why would view be null!?\n                        this.publiclyTrigger('viewSkeletonDestroy', [\n                            {\n                                view: view,\n                                el: view.el\n                            }\n                        ]);\n                    }\n                    this.isViewUpdated = true;\n                }\n                this.requestRerender();\n            }\n        };\n        Calendar.prototype.reduce = function (state, action, calendar) {\n            return reduce(state, action, calendar);\n        };\n        // Render Queue\n        // -----------------------------------------------------------------------------------------------------------------\n        Calendar.prototype.requestRerender = function () {\n            this.needsRerender = true;\n            this.delayedRerender(); // will call a debounced-version of tryRerender\n        };\n        Calendar.prototype.tryRerender = function () {\n            if (this.component && // must be accepting renders\n                this.needsRerender && // indicates that a rerender was requested\n                !this.renderingPauseDepth && // not paused\n                !this.isRendering // not currently in the render loop\n            ) {\n                this.executeRender();\n            }\n        };\n        Calendar.prototype.batchRendering = function (func) {\n            this.renderingPauseDepth++;\n            func();\n            this.renderingPauseDepth--;\n            if (this.needsRerender) {\n                this.requestRerender();\n            }\n        };\n        // Rendering\n        // -----------------------------------------------------------------------------------------------------------------\n        Calendar.prototype.executeRender = function () {\n            // clear these BEFORE the render so that new values will accumulate during render\n            this.needsRerender = false;\n            this.isRendering = true;\n            this.renderComponent();\n            this.isRendering = false;\n            // received a rerender request while rendering\n            if (this.needsRerender) {\n                this.delayedRerender();\n            }\n        };\n        /*\n        don't call this directly. use executeRender instead\n        */\n        Calendar.prototype.renderComponent = function () {\n            var _a = this, state = _a.state, component = _a.component;\n            var viewType = state.viewType;\n            var viewSpec = this.viewSpecs[viewType];\n            if (!viewSpec) {\n                throw new Error(\"View type \\\"\" + viewType + \"\\\" is not valid\");\n            }\n            // if event sources are still loading and progressive rendering hasn't been enabled,\n            // keep rendering the last fully loaded set of events\n            var renderableEventStore = this.renderableEventStore =\n                (state.eventSourceLoadingLevel && !this.opt('progressiveEventRendering')) ?\n                    this.renderableEventStore :\n                    state.eventStore;\n            var eventUiSingleBase = this.buildEventUiSingleBase(viewSpec.options);\n            var eventUiBySource = this.buildEventUiBySource(state.eventSources);\n            var eventUiBases = this.eventUiBases = this.buildEventUiBases(renderableEventStore.defs, eventUiSingleBase, eventUiBySource);\n            component.receiveProps(__assign({}, state, { viewSpec: viewSpec, dateProfileGenerator: this.dateProfileGenerators[viewType], dateProfile: state.dateProfile, eventStore: renderableEventStore, eventUiBases: eventUiBases, dateSelection: state.dateSelection, eventSelection: state.eventSelection, eventDrag: state.eventDrag, eventResize: state.eventResize }), this.buildComponentContext(this.theme, this.dateEnv, this.optionsManager.computed));\n            if (this.isViewUpdated) {\n                this.isViewUpdated = false;\n                this.publiclyTrigger('viewSkeletonRender', [\n                    {\n                        view: component.view,\n                        el: component.view.el\n                    }\n                ]);\n            }\n            if (this.isDatesUpdated) {\n                this.isDatesUpdated = false;\n                this.publiclyTrigger('datesRender', [\n                    {\n                        view: component.view,\n                        el: component.view.el\n                    }\n                ]);\n            }\n            if (this.isEventsUpdated) {\n                this.isEventsUpdated = false;\n            }\n            this.releaseAfterSizingTriggers();\n        };\n        // Options\n        // -----------------------------------------------------------------------------------------------------------------\n        Calendar.prototype.setOption = function (name, val) {\n            var _a;\n            this.mutateOptions((_a = {}, _a[name] = val, _a), [], true);\n        };\n        Calendar.prototype.getOption = function (name) {\n            return this.optionsManager.computed[name];\n        };\n        Calendar.prototype.opt = function (name) {\n            return this.optionsManager.computed[name];\n        };\n        Calendar.prototype.viewOpt = function (name) {\n            return this.viewOpts()[name];\n        };\n        Calendar.prototype.viewOpts = function () {\n            return this.viewSpecs[this.state.viewType].options;\n        };\n        /*\n        handles option changes (like a diff)\n        */\n        Calendar.prototype.mutateOptions = function (updates, removals, isDynamic, deepEqual) {\n            var _this = this;\n            var changeHandlers = this.pluginSystem.hooks.optionChangeHandlers;\n            var normalUpdates = {};\n            var specialUpdates = {};\n            var oldDateEnv = this.dateEnv; // do this before handleOptions\n            var isTimeZoneDirty = false;\n            var isSizeDirty = false;\n            var anyDifficultOptions = Boolean(removals.length);\n            for (var name_1 in updates) {\n                if (changeHandlers[name_1]) {\n                    specialUpdates[name_1] = updates[name_1];\n                }\n                else {\n                    normalUpdates[name_1] = updates[name_1];\n                }\n            }\n            for (var name_2 in normalUpdates) {\n                if (/^(height|contentHeight|aspectRatio)$/.test(name_2)) {\n                    isSizeDirty = true;\n                }\n                else if (/^(defaultDate|defaultView)$/.test(name_2)) ;\n                else {\n                    anyDifficultOptions = true;\n                    if (name_2 === 'timeZone') {\n                        isTimeZoneDirty = true;\n                    }\n                }\n            }\n            this.optionsManager.mutate(normalUpdates, removals, isDynamic);\n            if (anyDifficultOptions) {\n                this.handleOptions(this.optionsManager.computed);\n            }\n            this.batchRendering(function () {\n                if (anyDifficultOptions) {\n                    if (isTimeZoneDirty) {\n                        _this.dispatch({\n                            type: 'CHANGE_TIMEZONE',\n                            oldDateEnv: oldDateEnv\n                        });\n                    }\n                    /* HACK\n                    has the same effect as calling this.requestRerender()\n                    but recomputes the state's dateProfile\n                    */\n                    _this.dispatch({\n                        type: 'SET_VIEW_TYPE',\n                        viewType: _this.state.viewType\n                    });\n                }\n                else if (isSizeDirty) {\n                    _this.updateSize();\n                }\n                // special updates\n                if (deepEqual) {\n                    for (var name_3 in specialUpdates) {\n                        changeHandlers[name_3](specialUpdates[name_3], _this, deepEqual);\n                    }\n                }\n            });\n        };\n        /*\n        rebuilds things based off of a complete set of refined options\n        */\n        Calendar.prototype.handleOptions = function (options) {\n            var _this = this;\n            var pluginHooks = this.pluginSystem.hooks;\n            this.defaultAllDayEventDuration = createDuration(options.defaultAllDayEventDuration);\n            this.defaultTimedEventDuration = createDuration(options.defaultTimedEventDuration);\n            this.delayedRerender = this.buildDelayedRerender(options.rerenderDelay);\n            this.theme = this.buildTheme(options);\n            var available = this.parseRawLocales(options.locales);\n            this.availableRawLocales = available.map;\n            var locale = this.buildLocale(options.locale || available.defaultCode, available.map);\n            this.dateEnv = this.buildDateEnv(locale, options.timeZone, pluginHooks.namedTimeZonedImpl, options.firstDay, options.weekNumberCalculation, options.weekLabel, pluginHooks.cmdFormatter);\n            this.selectionConfig = this.buildSelectionConfig(options); // needs dateEnv. do after :(\n            // ineffecient to do every time?\n            this.viewSpecs = buildViewSpecs(pluginHooks.views, this.optionsManager);\n            // ineffecient to do every time?\n            this.dateProfileGenerators = mapHash(this.viewSpecs, function (viewSpec) {\n                return new viewSpec.class.prototype.dateProfileGeneratorClass(viewSpec, _this);\n            });\n        };\n        Calendar.prototype.getAvailableLocaleCodes = function () {\n            return Object.keys(this.availableRawLocales);\n        };\n        Calendar.prototype._buildSelectionConfig = function (rawOpts) {\n            return processScopedUiProps('select', rawOpts, this);\n        };\n        Calendar.prototype._buildEventUiSingleBase = function (rawOpts) {\n            if (rawOpts.editable) { // so 'editable' affected events\n                rawOpts = __assign({}, rawOpts, { eventEditable: true });\n            }\n            return processScopedUiProps('event', rawOpts, this);\n        };\n        // Trigger\n        // -----------------------------------------------------------------------------------------------------------------\n        Calendar.prototype.hasPublicHandlers = function (name) {\n            return this.hasHandlers(name) ||\n                this.opt(name); // handler specified in options\n        };\n        Calendar.prototype.publiclyTrigger = function (name, args) {\n            var optHandler = this.opt(name);\n            this.triggerWith(name, this, args);\n            if (optHandler) {\n                return optHandler.apply(this, args);\n            }\n        };\n        Calendar.prototype.publiclyTriggerAfterSizing = function (name, args) {\n            var afterSizingTriggers = this.afterSizingTriggers;\n            (afterSizingTriggers[name] || (afterSizingTriggers[name] = [])).push(args);\n        };\n        Calendar.prototype.releaseAfterSizingTriggers = function () {\n            var afterSizingTriggers = this.afterSizingTriggers;\n            for (var name_4 in afterSizingTriggers) {\n                for (var _i = 0, _a = afterSizingTriggers[name_4]; _i < _a.length; _i++) {\n                    var args = _a[_i];\n                    this.publiclyTrigger(name_4, args);\n                }\n            }\n            this.afterSizingTriggers = {};\n        };\n        // View\n        // -----------------------------------------------------------------------------------------------------------------\n        // Returns a boolean about whether the view is okay to instantiate at some point\n        Calendar.prototype.isValidViewType = function (viewType) {\n            return Boolean(this.viewSpecs[viewType]);\n        };\n        Calendar.prototype.changeView = function (viewType, dateOrRange) {\n            var dateMarker = null;\n            if (dateOrRange) {\n                if (dateOrRange.start && dateOrRange.end) { // a range\n                    this.optionsManager.mutate({ visibleRange: dateOrRange }, []); // will not rerender\n                    this.handleOptions(this.optionsManager.computed); // ...but yuck\n                }\n                else { // a date\n                    dateMarker = this.dateEnv.createMarker(dateOrRange); // just like gotoDate\n                }\n            }\n            this.unselect();\n            this.dispatch({\n                type: 'SET_VIEW_TYPE',\n                viewType: viewType,\n                dateMarker: dateMarker\n            });\n        };\n        // Forces navigation to a view for the given date.\n        // `viewType` can be a specific view name or a generic one like \"week\" or \"day\".\n        // needs to change\n        Calendar.prototype.zoomTo = function (dateMarker, viewType) {\n            var spec;\n            viewType = viewType || 'day'; // day is default zoom\n            spec = this.viewSpecs[viewType] ||\n                this.getUnitViewSpec(viewType);\n            this.unselect();\n            if (spec) {\n                this.dispatch({\n                    type: 'SET_VIEW_TYPE',\n                    viewType: spec.type,\n                    dateMarker: dateMarker\n                });\n            }\n            else {\n                this.dispatch({\n                    type: 'SET_DATE',\n                    dateMarker: dateMarker\n                });\n            }\n        };\n        // Given a duration singular unit, like \"week\" or \"day\", finds a matching view spec.\n        // Preference is given to views that have corresponding buttons.\n        Calendar.prototype.getUnitViewSpec = function (unit) {\n            var component = this.component;\n            var viewTypes = [];\n            var i;\n            var spec;\n            // put views that have buttons first. there will be duplicates, but oh\n            if (component.header) {\n                viewTypes.push.apply(viewTypes, component.header.viewsWithButtons);\n            }\n            if (component.footer) {\n                viewTypes.push.apply(viewTypes, component.footer.viewsWithButtons);\n            }\n            for (var viewType in this.viewSpecs) {\n                viewTypes.push(viewType);\n            }\n            for (i = 0; i < viewTypes.length; i++) {\n                spec = this.viewSpecs[viewTypes[i]];\n                if (spec) {\n                    if (spec.singleUnit === unit) {\n                        return spec;\n                    }\n                }\n            }\n        };\n        // Current Date\n        // -----------------------------------------------------------------------------------------------------------------\n        Calendar.prototype.getInitialDate = function () {\n            var defaultDateInput = this.opt('defaultDate');\n            // compute the initial ambig-timezone date\n            if (defaultDateInput != null) {\n                return this.dateEnv.createMarker(defaultDateInput);\n            }\n            else {\n                return this.getNow(); // getNow already returns unzoned\n            }\n        };\n        Calendar.prototype.prev = function () {\n            this.unselect();\n            this.dispatch({ type: 'PREV' });\n        };\n        Calendar.prototype.next = function () {\n            this.unselect();\n            this.dispatch({ type: 'NEXT' });\n        };\n        Calendar.prototype.prevYear = function () {\n            this.unselect();\n            this.dispatch({\n                type: 'SET_DATE',\n                dateMarker: this.dateEnv.addYears(this.state.currentDate, -1)\n            });\n        };\n        Calendar.prototype.nextYear = function () {\n            this.unselect();\n            this.dispatch({\n                type: 'SET_DATE',\n                dateMarker: this.dateEnv.addYears(this.state.currentDate, 1)\n            });\n        };\n        Calendar.prototype.today = function () {\n            this.unselect();\n            this.dispatch({\n                type: 'SET_DATE',\n                dateMarker: this.getNow()\n            });\n        };\n        Calendar.prototype.gotoDate = function (zonedDateInput) {\n            this.unselect();\n            this.dispatch({\n                type: 'SET_DATE',\n                dateMarker: this.dateEnv.createMarker(zonedDateInput)\n            });\n        };\n        Calendar.prototype.incrementDate = function (deltaInput) {\n            var delta = createDuration(deltaInput);\n            if (delta) { // else, warn about invalid input?\n                this.unselect();\n                this.dispatch({\n                    type: 'SET_DATE',\n                    dateMarker: this.dateEnv.add(this.state.currentDate, delta)\n                });\n            }\n        };\n        // for external API\n        Calendar.prototype.getDate = function () {\n            return this.dateEnv.toDate(this.state.currentDate);\n        };\n        // Date Formatting Utils\n        // -----------------------------------------------------------------------------------------------------------------\n        Calendar.prototype.formatDate = function (d, formatter) {\n            var dateEnv = this.dateEnv;\n            return dateEnv.format(dateEnv.createMarker(d), createFormatter(formatter));\n        };\n        // `settings` is for formatter AND isEndExclusive\n        Calendar.prototype.formatRange = function (d0, d1, settings) {\n            var dateEnv = this.dateEnv;\n            return dateEnv.formatRange(dateEnv.createMarker(d0), dateEnv.createMarker(d1), createFormatter(settings, this.opt('defaultRangeSeparator')), settings);\n        };\n        Calendar.prototype.formatIso = function (d, omitTime) {\n            var dateEnv = this.dateEnv;\n            return dateEnv.formatIso(dateEnv.createMarker(d), { omitTime: omitTime });\n        };\n        // Sizing\n        // -----------------------------------------------------------------------------------------------------------------\n        Calendar.prototype.windowResize = function (ev) {\n            if (!this.isHandlingWindowResize &&\n                this.component && // why?\n                ev.target === window // not a jqui resize event\n            ) {\n                this.isHandlingWindowResize = true;\n                this.updateSize();\n                this.publiclyTrigger('windowResize', [this.view]);\n                this.isHandlingWindowResize = false;\n            }\n        };\n        Calendar.prototype.updateSize = function () {\n            if (this.component) { // when?\n                this.component.updateSize(true);\n            }\n        };\n        // Component Registration\n        // -----------------------------------------------------------------------------------------------------------------\n        Calendar.prototype.registerInteractiveComponent = function (component, settingsInput) {\n            var settings = parseInteractionSettings(component, settingsInput);\n            var DEFAULT_INTERACTIONS = [\n                EventClicking,\n                EventHovering\n            ];\n            var interactionClasses = DEFAULT_INTERACTIONS.concat(this.pluginSystem.hooks.componentInteractions);\n            var interactions = interactionClasses.map(function (interactionClass) {\n                return new interactionClass(settings);\n            });\n            this.interactionsStore[component.uid] = interactions;\n            interactionSettingsStore[component.uid] = settings;\n        };\n        Calendar.prototype.unregisterInteractiveComponent = function (component) {\n            for (var _i = 0, _a = this.interactionsStore[component.uid]; _i < _a.length; _i++) {\n                var listener = _a[_i];\n                listener.destroy();\n            }\n            delete this.interactionsStore[component.uid];\n            delete interactionSettingsStore[component.uid];\n        };\n        // Date Selection / Event Selection / DayClick\n        // -----------------------------------------------------------------------------------------------------------------\n        // this public method receives start/end dates in any format, with any timezone\n        // NOTE: args were changed from v3\n        Calendar.prototype.select = function (dateOrObj, endDate) {\n            var selectionInput;\n            if (endDate == null) {\n                if (dateOrObj.start != null) {\n                    selectionInput = dateOrObj;\n                }\n                else {\n                    selectionInput = {\n                        start: dateOrObj,\n                        end: null\n                    };\n                }\n            }\n            else {\n                selectionInput = {\n                    start: dateOrObj,\n                    end: endDate\n                };\n            }\n            var selection = parseDateSpan(selectionInput, this.dateEnv, createDuration({ days: 1 }) // TODO: cache this?\n            );\n            if (selection) { // throw parse error otherwise?\n                this.dispatch({ type: 'SELECT_DATES', selection: selection });\n                this.triggerDateSelect(selection);\n            }\n        };\n        // public method\n        Calendar.prototype.unselect = function (pev) {\n            if (this.state.dateSelection) {\n                this.dispatch({ type: 'UNSELECT_DATES' });\n                this.triggerDateUnselect(pev);\n            }\n        };\n        Calendar.prototype.triggerDateSelect = function (selection, pev) {\n            var arg = __assign({}, this.buildDateSpanApi(selection), { jsEvent: pev ? pev.origEvent : null, view: this.view });\n            this.publiclyTrigger('select', [arg]);\n        };\n        Calendar.prototype.triggerDateUnselect = function (pev) {\n            this.publiclyTrigger('unselect', [\n                {\n                    jsEvent: pev ? pev.origEvent : null,\n                    view: this.view\n                }\n            ]);\n        };\n        // TODO: receive pev?\n        Calendar.prototype.triggerDateClick = function (dateSpan, dayEl, view, ev) {\n            var arg = __assign({}, this.buildDatePointApi(dateSpan), { dayEl: dayEl, jsEvent: ev, // Is this always a mouse event? See #4655\n                view: view });\n            this.publiclyTrigger('dateClick', [arg]);\n        };\n        Calendar.prototype.buildDatePointApi = function (dateSpan) {\n            var props = {};\n            for (var _i = 0, _a = this.pluginSystem.hooks.datePointTransforms; _i < _a.length; _i++) {\n                var transform = _a[_i];\n                __assign(props, transform(dateSpan, this));\n            }\n            __assign(props, buildDatePointApi(dateSpan, this.dateEnv));\n            return props;\n        };\n        Calendar.prototype.buildDateSpanApi = function (dateSpan) {\n            var props = {};\n            for (var _i = 0, _a = this.pluginSystem.hooks.dateSpanTransforms; _i < _a.length; _i++) {\n                var transform = _a[_i];\n                __assign(props, transform(dateSpan, this));\n            }\n            __assign(props, buildDateSpanApi(dateSpan, this.dateEnv));\n            return props;\n        };\n        // Date Utils\n        // -----------------------------------------------------------------------------------------------------------------\n        // Returns a DateMarker for the current date, as defined by the client's computer or from the `now` option\n        Calendar.prototype.getNow = function () {\n            var now = this.opt('now');\n            if (typeof now === 'function') {\n                now = now();\n            }\n            if (now == null) {\n                return this.dateEnv.createNowMarker();\n            }\n            return this.dateEnv.createMarker(now);\n        };\n        // Event-Date Utilities\n        // -----------------------------------------------------------------------------------------------------------------\n        // Given an event's allDay status and start date, return what its fallback end date should be.\n        // TODO: rename to computeDefaultEventEnd\n        Calendar.prototype.getDefaultEventEnd = function (allDay, marker) {\n            var end = marker;\n            if (allDay) {\n                end = startOfDay(end);\n                end = this.dateEnv.add(end, this.defaultAllDayEventDuration);\n            }\n            else {\n                end = this.dateEnv.add(end, this.defaultTimedEventDuration);\n            }\n            return end;\n        };\n        // Public Events API\n        // -----------------------------------------------------------------------------------------------------------------\n        Calendar.prototype.addEvent = function (eventInput, sourceInput) {\n            if (eventInput instanceof EventApi) {\n                var def = eventInput._def;\n                var instance = eventInput._instance;\n                // not already present? don't want to add an old snapshot\n                if (!this.state.eventStore.defs[def.defId]) {\n                    this.dispatch({\n                        type: 'ADD_EVENTS',\n                        eventStore: eventTupleToStore({ def: def, instance: instance }) // TODO: better util for two args?\n                    });\n                }\n                return eventInput;\n            }\n            var sourceId;\n            if (sourceInput instanceof EventSourceApi) {\n                sourceId = sourceInput.internalEventSource.sourceId;\n            }\n            else if (sourceInput != null) {\n                var sourceApi = this.getEventSourceById(sourceInput); // TODO: use an internal function\n                if (!sourceApi) {\n                    console.warn('Could not find an event source with ID \"' + sourceInput + '\"'); // TODO: test\n                    return null;\n                }\n                else {\n                    sourceId = sourceApi.internalEventSource.sourceId;\n                }\n            }\n            var tuple = parseEvent(eventInput, sourceId, this);\n            if (tuple) {\n                this.dispatch({\n                    type: 'ADD_EVENTS',\n                    eventStore: eventTupleToStore(tuple)\n                });\n                return new EventApi(this, tuple.def, tuple.def.recurringDef ? null : tuple.instance);\n            }\n            return null;\n        };\n        // TODO: optimize\n        Calendar.prototype.getEventById = function (id) {\n            var _a = this.state.eventStore, defs = _a.defs, instances = _a.instances;\n            id = String(id);\n            for (var defId in defs) {\n                var def = defs[defId];\n                if (def.publicId === id) {\n                    if (def.recurringDef) {\n                        return new EventApi(this, def, null);\n                    }\n                    else {\n                        for (var instanceId in instances) {\n                            var instance = instances[instanceId];\n                            if (instance.defId === def.defId) {\n                                return new EventApi(this, def, instance);\n                            }\n                        }\n                    }\n                }\n            }\n            return null;\n        };\n        Calendar.prototype.getEvents = function () {\n            var _a = this.state.eventStore, defs = _a.defs, instances = _a.instances;\n            var eventApis = [];\n            for (var id in instances) {\n                var instance = instances[id];\n                var def = defs[instance.defId];\n                eventApis.push(new EventApi(this, def, instance));\n            }\n            return eventApis;\n        };\n        Calendar.prototype.removeAllEvents = function () {\n            this.dispatch({ type: 'REMOVE_ALL_EVENTS' });\n        };\n        Calendar.prototype.rerenderEvents = function () {\n            this.dispatch({ type: 'RESET_EVENTS' });\n        };\n        // Public Event Sources API\n        // -----------------------------------------------------------------------------------------------------------------\n        Calendar.prototype.getEventSources = function () {\n            var sourceHash = this.state.eventSources;\n            var sourceApis = [];\n            for (var internalId in sourceHash) {\n                sourceApis.push(new EventSourceApi(this, sourceHash[internalId]));\n            }\n            return sourceApis;\n        };\n        Calendar.prototype.getEventSourceById = function (id) {\n            var sourceHash = this.state.eventSources;\n            id = String(id);\n            for (var sourceId in sourceHash) {\n                if (sourceHash[sourceId].publicId === id) {\n                    return new EventSourceApi(this, sourceHash[sourceId]);\n                }\n            }\n            return null;\n        };\n        Calendar.prototype.addEventSource = function (sourceInput) {\n            if (sourceInput instanceof EventSourceApi) {\n                // not already present? don't want to add an old snapshot\n                if (!this.state.eventSources[sourceInput.internalEventSource.sourceId]) {\n                    this.dispatch({\n                        type: 'ADD_EVENT_SOURCES',\n                        sources: [sourceInput.internalEventSource]\n                    });\n                }\n                return sourceInput;\n            }\n            var eventSource = parseEventSource(sourceInput, this);\n            if (eventSource) { // TODO: error otherwise?\n                this.dispatch({ type: 'ADD_EVENT_SOURCES', sources: [eventSource] });\n                return new EventSourceApi(this, eventSource);\n            }\n            return null;\n        };\n        Calendar.prototype.removeAllEventSources = function () {\n            this.dispatch({ type: 'REMOVE_ALL_EVENT_SOURCES' });\n        };\n        Calendar.prototype.refetchEvents = function () {\n            this.dispatch({ type: 'FETCH_EVENT_SOURCES' });\n        };\n        // Scroll\n        // -----------------------------------------------------------------------------------------------------------------\n        Calendar.prototype.scrollToTime = function (timeInput) {\n            var duration = createDuration(timeInput);\n            if (duration) {\n                this.component.view.scrollToDuration(duration);\n            }\n        };\n        return Calendar;\n    }());\n    EmitterMixin.mixInto(Calendar);\n    // for memoizers\n    // -----------------------------------------------------------------------------------------------------------------\n    function buildComponentContext$1(theme, dateEnv, options) {\n        return new ComponentContext(this, theme, dateEnv, options, null);\n    }\n    function buildDateEnv(locale, timeZone, namedTimeZoneImpl, firstDay, weekNumberCalculation, weekLabel, cmdFormatter) {\n        return new DateEnv({\n            calendarSystem: 'gregory',\n            timeZone: timeZone,\n            namedTimeZoneImpl: namedTimeZoneImpl,\n            locale: locale,\n            weekNumberCalculation: weekNumberCalculation,\n            firstDay: firstDay,\n            weekLabel: weekLabel,\n            cmdFormatter: cmdFormatter\n        });\n    }\n    function buildTheme(calendarOptions) {\n        var themeClass = this.pluginSystem.hooks.themeClasses[calendarOptions.themeSystem] || StandardTheme;\n        return new themeClass(calendarOptions);\n    }\n    function buildDelayedRerender(wait) {\n        var func = this.tryRerender.bind(this);\n        if (wait != null) {\n            func = debounce(func, wait);\n        }\n        return func;\n    }\n    function buildEventUiBySource(eventSources) {\n        return mapHash(eventSources, function (eventSource) {\n            return eventSource.ui;\n        });\n    }\n    function buildEventUiBases(eventDefs, eventUiSingleBase, eventUiBySource) {\n        var eventUiBases = { '': eventUiSingleBase };\n        for (var defId in eventDefs) {\n            var def = eventDefs[defId];\n            if (def.sourceId && eventUiBySource[def.sourceId]) {\n                eventUiBases[defId] = eventUiBySource[def.sourceId];\n            }\n        }\n        return eventUiBases;\n    }\n\n    var View = /** @class */ (function (_super) {\n        __extends(View, _super);\n        function View(viewSpec, parentEl) {\n            var _this = _super.call(this, createElement('div', { className: 'fc-view fc-' + viewSpec.type + '-view' })) || this;\n            _this.renderDatesMem = memoizeRendering(_this.renderDatesWrap, _this.unrenderDatesWrap);\n            _this.renderBusinessHoursMem = memoizeRendering(_this.renderBusinessHours, _this.unrenderBusinessHours, [_this.renderDatesMem]);\n            _this.renderDateSelectionMem = memoizeRendering(_this.renderDateSelectionWrap, _this.unrenderDateSelectionWrap, [_this.renderDatesMem]);\n            _this.renderEventsMem = memoizeRendering(_this.renderEvents, _this.unrenderEvents, [_this.renderDatesMem]);\n            _this.renderEventSelectionMem = memoizeRendering(_this.renderEventSelectionWrap, _this.unrenderEventSelectionWrap, [_this.renderEventsMem]);\n            _this.renderEventDragMem = memoizeRendering(_this.renderEventDragWrap, _this.unrenderEventDragWrap, [_this.renderDatesMem]);\n            _this.renderEventResizeMem = memoizeRendering(_this.renderEventResizeWrap, _this.unrenderEventResizeWrap, [_this.renderDatesMem]);\n            _this.viewSpec = viewSpec;\n            _this.type = viewSpec.type;\n            parentEl.appendChild(_this.el);\n            _this.initialize();\n            return _this;\n        }\n        View.prototype.initialize = function () {\n        };\n        Object.defineProperty(View.prototype, \"activeStart\", {\n            // Date Setting/Unsetting\n            // -----------------------------------------------------------------------------------------------------------------\n            get: function () {\n                return this.context.dateEnv.toDate(this.props.dateProfile.activeRange.start);\n            },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(View.prototype, \"activeEnd\", {\n            get: function () {\n                return this.context.dateEnv.toDate(this.props.dateProfile.activeRange.end);\n            },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(View.prototype, \"currentStart\", {\n            get: function () {\n                return this.context.dateEnv.toDate(this.props.dateProfile.currentRange.start);\n            },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(View.prototype, \"currentEnd\", {\n            get: function () {\n                return this.context.dateEnv.toDate(this.props.dateProfile.currentRange.end);\n            },\n            enumerable: true,\n            configurable: true\n        });\n        // General Rendering\n        // -----------------------------------------------------------------------------------------------------------------\n        View.prototype.render = function (props, context) {\n            this.renderDatesMem(props.dateProfile);\n            this.renderBusinessHoursMem(props.businessHours);\n            this.renderDateSelectionMem(props.dateSelection);\n            this.renderEventsMem(props.eventStore);\n            this.renderEventSelectionMem(props.eventSelection);\n            this.renderEventDragMem(props.eventDrag);\n            this.renderEventResizeMem(props.eventResize);\n        };\n        View.prototype.beforeUpdate = function () {\n            this.addScroll(this.queryScroll());\n        };\n        View.prototype.destroy = function () {\n            _super.prototype.destroy.call(this);\n            this.renderDatesMem.unrender(); // should unrender everything else\n        };\n        // Sizing\n        // -----------------------------------------------------------------------------------------------------------------\n        View.prototype.updateSize = function (isResize, viewHeight, isAuto) {\n            var calendar = this.context.calendar;\n            if (isResize) {\n                this.addScroll(this.queryScroll()); // NOTE: same code as in beforeUpdate\n            }\n            if (isResize || // HACKS...\n                calendar.isViewUpdated ||\n                calendar.isDatesUpdated ||\n                calendar.isEventsUpdated) {\n                // sort of the catch-all sizing\n                // anything that might cause dimension changes\n                this.updateBaseSize(isResize, viewHeight, isAuto);\n            }\n            // NOTE: popScroll is called by CalendarComponent\n        };\n        View.prototype.updateBaseSize = function (isResize, viewHeight, isAuto) {\n        };\n        // Date Rendering\n        // -----------------------------------------------------------------------------------------------------------------\n        View.prototype.renderDatesWrap = function (dateProfile) {\n            this.renderDates(dateProfile);\n            this.addScroll({\n                duration: createDuration(this.context.options.scrollTime)\n            });\n        };\n        View.prototype.unrenderDatesWrap = function () {\n            this.stopNowIndicator();\n            this.unrenderDates();\n        };\n        View.prototype.renderDates = function (dateProfile) { };\n        View.prototype.unrenderDates = function () { };\n        // Business Hours\n        // -----------------------------------------------------------------------------------------------------------------\n        View.prototype.renderBusinessHours = function (businessHours) { };\n        View.prototype.unrenderBusinessHours = function () { };\n        // Date Selection\n        // -----------------------------------------------------------------------------------------------------------------\n        View.prototype.renderDateSelectionWrap = function (selection) {\n            if (selection) {\n                this.renderDateSelection(selection);\n            }\n        };\n        View.prototype.unrenderDateSelectionWrap = function (selection) {\n            if (selection) {\n                this.unrenderDateSelection(selection);\n            }\n        };\n        View.prototype.renderDateSelection = function (selection) { };\n        View.prototype.unrenderDateSelection = function (selection) { };\n        // Event Rendering\n        // -----------------------------------------------------------------------------------------------------------------\n        View.prototype.renderEvents = function (eventStore) { };\n        View.prototype.unrenderEvents = function () { };\n        // util for subclasses\n        View.prototype.sliceEvents = function (eventStore, allDay) {\n            var props = this.props;\n            return sliceEventStore(eventStore, props.eventUiBases, props.dateProfile.activeRange, allDay ? this.context.nextDayThreshold : null).fg;\n        };\n        // Event Selection\n        // -----------------------------------------------------------------------------------------------------------------\n        View.prototype.renderEventSelectionWrap = function (instanceId) {\n            if (instanceId) {\n                this.renderEventSelection(instanceId);\n            }\n        };\n        View.prototype.unrenderEventSelectionWrap = function (instanceId) {\n            if (instanceId) {\n                this.unrenderEventSelection(instanceId);\n            }\n        };\n        View.prototype.renderEventSelection = function (instanceId) { };\n        View.prototype.unrenderEventSelection = function (instanceId) { };\n        // Event Drag\n        // -----------------------------------------------------------------------------------------------------------------\n        View.prototype.renderEventDragWrap = function (state) {\n            if (state) {\n                this.renderEventDrag(state);\n            }\n        };\n        View.prototype.unrenderEventDragWrap = function (state) {\n            if (state) {\n                this.unrenderEventDrag(state);\n            }\n        };\n        View.prototype.renderEventDrag = function (state) { };\n        View.prototype.unrenderEventDrag = function (state) { };\n        // Event Resize\n        // -----------------------------------------------------------------------------------------------------------------\n        View.prototype.renderEventResizeWrap = function (state) {\n            if (state) {\n                this.renderEventResize(state);\n            }\n        };\n        View.prototype.unrenderEventResizeWrap = function (state) {\n            if (state) {\n                this.unrenderEventResize(state);\n            }\n        };\n        View.prototype.renderEventResize = function (state) { };\n        View.prototype.unrenderEventResize = function (state) { };\n        /* Now Indicator\n        ------------------------------------------------------------------------------------------------------------------*/\n        // Immediately render the current time indicator and begins re-rendering it at an interval,\n        // which is defined by this.getNowIndicatorUnit().\n        // TODO: somehow do this for the current whole day's background too\n        // USAGE: must be called manually from subclasses' render methods! don't need to call stopNowIndicator tho\n        View.prototype.startNowIndicator = function (dateProfile, dateProfileGenerator) {\n            var _this = this;\n            var _a = this.context, calendar = _a.calendar, dateEnv = _a.dateEnv, options = _a.options;\n            var unit;\n            var update;\n            var delay; // ms wait value\n            if (options.nowIndicator && !this.initialNowDate) {\n                unit = this.getNowIndicatorUnit(dateProfile, dateProfileGenerator);\n                if (unit) {\n                    update = this.updateNowIndicator.bind(this);\n                    this.initialNowDate = calendar.getNow();\n                    this.initialNowQueriedMs = new Date().valueOf();\n                    // wait until the beginning of the next interval\n                    delay = dateEnv.add(dateEnv.startOf(this.initialNowDate, unit), createDuration(1, unit)).valueOf() - this.initialNowDate.valueOf();\n                    // TODO: maybe always use setTimeout, waiting until start of next unit\n                    this.nowIndicatorTimeoutID = setTimeout(function () {\n                        _this.nowIndicatorTimeoutID = null;\n                        update();\n                        if (unit === 'second') {\n                            delay = 1000; // every second\n                        }\n                        else {\n                            delay = 1000 * 60; // otherwise, every minute\n                        }\n                        _this.nowIndicatorIntervalID = setInterval(update, delay); // update every interval\n                    }, delay);\n                }\n                // rendering will be initiated in updateSize\n            }\n        };\n        // rerenders the now indicator, computing the new current time from the amount of time that has passed\n        // since the initial getNow call.\n        View.prototype.updateNowIndicator = function () {\n            if (this.props.dateProfile && // a way to determine if dates were rendered yet\n                this.initialNowDate // activated before?\n            ) {\n                this.unrenderNowIndicator(); // won't unrender if unnecessary\n                this.renderNowIndicator(addMs(this.initialNowDate, new Date().valueOf() - this.initialNowQueriedMs));\n                this.isNowIndicatorRendered = true;\n            }\n        };\n        // Immediately unrenders the view's current time indicator and stops any re-rendering timers.\n        // Won't cause side effects if indicator isn't rendered.\n        View.prototype.stopNowIndicator = function () {\n            if (this.nowIndicatorTimeoutID) {\n                clearTimeout(this.nowIndicatorTimeoutID);\n                this.nowIndicatorTimeoutID = null;\n            }\n            if (this.nowIndicatorIntervalID) {\n                clearInterval(this.nowIndicatorIntervalID);\n                this.nowIndicatorIntervalID = null;\n            }\n            if (this.isNowIndicatorRendered) {\n                this.unrenderNowIndicator();\n                this.isNowIndicatorRendered = false;\n            }\n        };\n        View.prototype.getNowIndicatorUnit = function (dateProfile, dateProfileGenerator) {\n            // subclasses should implement\n        };\n        // Renders a current time indicator at the given datetime\n        View.prototype.renderNowIndicator = function (date) {\n            // SUBCLASSES MUST PASS TO CHILDREN!\n        };\n        // Undoes the rendering actions from renderNowIndicator\n        View.prototype.unrenderNowIndicator = function () {\n            // SUBCLASSES MUST PASS TO CHILDREN!\n        };\n        /* Scroller\n        ------------------------------------------------------------------------------------------------------------------*/\n        View.prototype.addScroll = function (scroll, isForced) {\n            if (isForced) {\n                scroll.isForced = isForced;\n            }\n            __assign(this.queuedScroll || (this.queuedScroll = {}), scroll);\n        };\n        View.prototype.popScroll = function (isResize) {\n            this.applyQueuedScroll(isResize);\n            this.queuedScroll = null;\n        };\n        View.prototype.applyQueuedScroll = function (isResize) {\n            if (this.queuedScroll) {\n                this.applyScroll(this.queuedScroll, isResize);\n            }\n        };\n        View.prototype.queryScroll = function () {\n            var scroll = {};\n            if (this.props.dateProfile) { // dates rendered yet?\n                __assign(scroll, this.queryDateScroll());\n            }\n            return scroll;\n        };\n        View.prototype.applyScroll = function (scroll, isResize) {\n            var duration = scroll.duration, isForced = scroll.isForced;\n            if (duration != null && !isForced) {\n                delete scroll.duration;\n                if (this.props.dateProfile) { // dates rendered yet?\n                    __assign(scroll, this.computeDateScroll(duration));\n                }\n            }\n            if (this.props.dateProfile) { // dates rendered yet?\n                this.applyDateScroll(scroll);\n            }\n        };\n        View.prototype.computeDateScroll = function (duration) {\n            return {}; // subclasses must implement\n        };\n        View.prototype.queryDateScroll = function () {\n            return {}; // subclasses must implement\n        };\n        View.prototype.applyDateScroll = function (scroll) {\n            // subclasses must implement\n        };\n        // for API\n        View.prototype.scrollToDuration = function (duration) {\n            this.applyScroll({ duration: duration }, false);\n        };\n        return View;\n    }(DateComponent));\n    EmitterMixin.mixInto(View);\n    View.prototype.usesMinMaxTime = false;\n    View.prototype.dateProfileGeneratorClass = DateProfileGenerator;\n\n    var FgEventRenderer = /** @class */ (function () {\n        function FgEventRenderer() {\n            this.segs = [];\n            this.isSizeDirty = false;\n        }\n        FgEventRenderer.prototype.renderSegs = function (context, segs, mirrorInfo) {\n            this.context = context;\n            this.rangeUpdated(); // called too frequently :(\n            // render an `.el` on each seg\n            // returns a subset of the segs. segs that were actually rendered\n            segs = this.renderSegEls(segs, mirrorInfo);\n            this.segs = segs;\n            this.attachSegs(segs, mirrorInfo);\n            this.isSizeDirty = true;\n            triggerRenderedSegs(this.context, this.segs, Boolean(mirrorInfo));\n        };\n        FgEventRenderer.prototype.unrender = function (context, _segs, mirrorInfo) {\n            triggerWillRemoveSegs(this.context, this.segs, Boolean(mirrorInfo));\n            this.detachSegs(this.segs);\n            this.segs = [];\n        };\n        // Updates values that rely on options and also relate to range\n        FgEventRenderer.prototype.rangeUpdated = function () {\n            var options = this.context.options;\n            var displayEventTime;\n            var displayEventEnd;\n            this.eventTimeFormat = createFormatter(options.eventTimeFormat || this.computeEventTimeFormat(), options.defaultRangeSeparator);\n            displayEventTime = options.displayEventTime;\n            if (displayEventTime == null) {\n                displayEventTime = this.computeDisplayEventTime(); // might be based off of range\n            }\n            displayEventEnd = options.displayEventEnd;\n            if (displayEventEnd == null) {\n                displayEventEnd = this.computeDisplayEventEnd(); // might be based off of range\n            }\n            this.displayEventTime = displayEventTime;\n            this.displayEventEnd = displayEventEnd;\n        };\n        // Renders and assigns an `el` property for each foreground event segment.\n        // Only returns segments that successfully rendered.\n        FgEventRenderer.prototype.renderSegEls = function (segs, mirrorInfo) {\n            var html = '';\n            var i;\n            if (segs.length) { // don't build an empty html string\n                // build a large concatenation of event segment HTML\n                for (i = 0; i < segs.length; i++) {\n                    html += this.renderSegHtml(segs[i], mirrorInfo);\n                }\n                // Grab individual elements from the combined HTML string. Use each as the default rendering.\n                // Then, compute the 'el' for each segment. An el might be null if the eventRender callback returned false.\n                htmlToElements(html).forEach(function (el, i) {\n                    var seg = segs[i];\n                    if (el) {\n                        seg.el = el;\n                    }\n                });\n                segs = filterSegsViaEls(this.context, segs, Boolean(mirrorInfo));\n            }\n            return segs;\n        };\n        // Generic utility for generating the HTML classNames for an event segment's element\n        FgEventRenderer.prototype.getSegClasses = function (seg, isDraggable, isResizable, mirrorInfo) {\n            var classes = [\n                'fc-event',\n                seg.isStart ? 'fc-start' : 'fc-not-start',\n                seg.isEnd ? 'fc-end' : 'fc-not-end'\n            ].concat(seg.eventRange.ui.classNames);\n            if (isDraggable) {\n                classes.push('fc-draggable');\n            }\n            if (isResizable) {\n                classes.push('fc-resizable');\n            }\n            if (mirrorInfo) {\n                classes.push('fc-mirror');\n                if (mirrorInfo.isDragging) {\n                    classes.push('fc-dragging');\n                }\n                if (mirrorInfo.isResizing) {\n                    classes.push('fc-resizing');\n                }\n            }\n            return classes;\n        };\n        // Compute the text that should be displayed on an event's element.\n        // `range` can be the Event object itself, or something range-like, with at least a `start`.\n        // If event times are disabled, or the event has no time, will return a blank string.\n        // If not specified, formatter will default to the eventTimeFormat setting,\n        // and displayEnd will default to the displayEventEnd setting.\n        FgEventRenderer.prototype.getTimeText = function (eventRange, formatter, displayEnd) {\n            var def = eventRange.def, instance = eventRange.instance;\n            return this._getTimeText(instance.range.start, def.hasEnd ? instance.range.end : null, def.allDay, formatter, displayEnd, instance.forcedStartTzo, instance.forcedEndTzo);\n        };\n        FgEventRenderer.prototype._getTimeText = function (start, end, allDay, formatter, displayEnd, forcedStartTzo, forcedEndTzo) {\n            var dateEnv = this.context.dateEnv;\n            if (formatter == null) {\n                formatter = this.eventTimeFormat;\n            }\n            if (displayEnd == null) {\n                displayEnd = this.displayEventEnd;\n            }\n            if (this.displayEventTime && !allDay) {\n                if (displayEnd && end) {\n                    return dateEnv.formatRange(start, end, formatter, {\n                        forcedStartTzo: forcedStartTzo,\n                        forcedEndTzo: forcedEndTzo\n                    });\n                }\n                else {\n                    return dateEnv.format(start, formatter, {\n                        forcedTzo: forcedStartTzo\n                    });\n                }\n            }\n            return '';\n        };\n        FgEventRenderer.prototype.computeEventTimeFormat = function () {\n            return {\n                hour: 'numeric',\n                minute: '2-digit',\n                omitZeroMinute: true\n            };\n        };\n        FgEventRenderer.prototype.computeDisplayEventTime = function () {\n            return true;\n        };\n        FgEventRenderer.prototype.computeDisplayEventEnd = function () {\n            return true;\n        };\n        // Utility for generating event skin-related CSS properties\n        FgEventRenderer.prototype.getSkinCss = function (ui) {\n            return {\n                'background-color': ui.backgroundColor,\n                'border-color': ui.borderColor,\n                color: ui.textColor\n            };\n        };\n        FgEventRenderer.prototype.sortEventSegs = function (segs) {\n            var specs = this.context.eventOrderSpecs;\n            var objs = segs.map(buildSegCompareObj);\n            objs.sort(function (obj0, obj1) {\n                return compareByFieldSpecs(obj0, obj1, specs);\n            });\n            return objs.map(function (c) {\n                return c._seg;\n            });\n        };\n        FgEventRenderer.prototype.computeSizes = function (force) {\n            if (force || this.isSizeDirty) {\n                this.computeSegSizes(this.segs);\n            }\n        };\n        FgEventRenderer.prototype.assignSizes = function (force) {\n            if (force || this.isSizeDirty) {\n                this.assignSegSizes(this.segs);\n                this.isSizeDirty = false;\n            }\n        };\n        FgEventRenderer.prototype.computeSegSizes = function (segs) {\n        };\n        FgEventRenderer.prototype.assignSegSizes = function (segs) {\n        };\n        // Manipulation on rendered segs\n        FgEventRenderer.prototype.hideByHash = function (hash) {\n            if (hash) {\n                for (var _i = 0, _a = this.segs; _i < _a.length; _i++) {\n                    var seg = _a[_i];\n                    if (hash[seg.eventRange.instance.instanceId]) {\n                        seg.el.style.visibility = 'hidden';\n                    }\n                }\n            }\n        };\n        FgEventRenderer.prototype.showByHash = function (hash) {\n            if (hash) {\n                for (var _i = 0, _a = this.segs; _i < _a.length; _i++) {\n                    var seg = _a[_i];\n                    if (hash[seg.eventRange.instance.instanceId]) {\n                        seg.el.style.visibility = '';\n                    }\n                }\n            }\n        };\n        FgEventRenderer.prototype.selectByInstanceId = function (instanceId) {\n            if (instanceId) {\n                for (var _i = 0, _a = this.segs; _i < _a.length; _i++) {\n                    var seg = _a[_i];\n                    var eventInstance = seg.eventRange.instance;\n                    if (eventInstance && eventInstance.instanceId === instanceId &&\n                        seg.el // necessary?\n                    ) {\n                        seg.el.classList.add('fc-selected');\n                    }\n                }\n            }\n        };\n        FgEventRenderer.prototype.unselectByInstanceId = function (instanceId) {\n            if (instanceId) {\n                for (var _i = 0, _a = this.segs; _i < _a.length; _i++) {\n                    var seg = _a[_i];\n                    if (seg.el) { // necessary?\n                        seg.el.classList.remove('fc-selected');\n                    }\n                }\n            }\n        };\n        return FgEventRenderer;\n    }());\n    // returns a object with all primitive props that can be compared\n    function buildSegCompareObj(seg) {\n        var eventDef = seg.eventRange.def;\n        var range = seg.eventRange.instance.range;\n        var start = range.start ? range.start.valueOf() : 0; // TODO: better support for open-range events\n        var end = range.end ? range.end.valueOf() : 0; // \"\n        return __assign({}, eventDef.extendedProps, eventDef, { id: eventDef.publicId, start: start,\n            end: end, duration: end - start, allDay: Number(eventDef.allDay), _seg: seg // for later retrieval\n         });\n    }\n\n    /*\n    TODO: when refactoring this class, make a new FillRenderer instance for each `type`\n    */\n    var FillRenderer = /** @class */ (function () {\n        function FillRenderer() {\n            this.fillSegTag = 'div';\n            this.dirtySizeFlags = {};\n            this.containerElsByType = {};\n            this.segsByType = {};\n        }\n        FillRenderer.prototype.getSegsByType = function (type) {\n            return this.segsByType[type] || [];\n        };\n        FillRenderer.prototype.renderSegs = function (type, context, segs) {\n            var _a;\n            this.context = context;\n            var renderedSegs = this.renderSegEls(type, segs); // assignes `.el` to each seg. returns successfully rendered segs\n            var containerEls = this.attachSegs(type, renderedSegs);\n            if (containerEls) {\n                (_a = (this.containerElsByType[type] || (this.containerElsByType[type] = []))).push.apply(_a, containerEls);\n            }\n            this.segsByType[type] = renderedSegs;\n            if (type === 'bgEvent') {\n                triggerRenderedSegs(context, renderedSegs, false); // isMirror=false\n            }\n            this.dirtySizeFlags[type] = true;\n        };\n        // Unrenders a specific type of fill that is currently rendered on the grid\n        FillRenderer.prototype.unrender = function (type, context) {\n            var segs = this.segsByType[type];\n            if (segs) {\n                if (type === 'bgEvent') {\n                    triggerWillRemoveSegs(context, segs, false); // isMirror=false\n                }\n                this.detachSegs(type, segs);\n            }\n        };\n        // Renders and assigns an `el` property for each fill segment. Generic enough to work with different types.\n        // Only returns segments that successfully rendered.\n        FillRenderer.prototype.renderSegEls = function (type, segs) {\n            var _this = this;\n            var html = '';\n            var i;\n            if (segs.length) {\n                // build a large concatenation of segment HTML\n                for (i = 0; i < segs.length; i++) {\n                    html += this.renderSegHtml(type, segs[i]);\n                }\n                // Grab individual elements from the combined HTML string. Use each as the default rendering.\n                // Then, compute the 'el' for each segment.\n                htmlToElements(html).forEach(function (el, i) {\n                    var seg = segs[i];\n                    if (el) {\n                        seg.el = el;\n                    }\n                });\n                if (type === 'bgEvent') {\n                    segs = filterSegsViaEls(this.context, segs, false // isMirror. background events can never be mirror elements\n                    );\n                }\n                // correct element type? (would be bad if a non-TD were inserted into a table for example)\n                segs = segs.filter(function (seg) {\n                    return elementMatches(seg.el, _this.fillSegTag);\n                });\n            }\n            return segs;\n        };\n        // Builds the HTML needed for one fill segment. Generic enough to work with different types.\n        FillRenderer.prototype.renderSegHtml = function (type, seg) {\n            var css = null;\n            var classNames = [];\n            if (type !== 'highlight' && type !== 'businessHours') {\n                css = {\n                    'background-color': seg.eventRange.ui.backgroundColor\n                };\n            }\n            if (type !== 'highlight') {\n                classNames = classNames.concat(seg.eventRange.ui.classNames);\n            }\n            if (type === 'businessHours') {\n                classNames.push('fc-bgevent');\n            }\n            else {\n                classNames.push('fc-' + type.toLowerCase());\n            }\n            return '<' + this.fillSegTag +\n                (classNames.length ? ' class=\"' + classNames.join(' ') + '\"' : '') +\n                (css ? ' style=\"' + cssToStr(css) + '\"' : '') +\n                '></' + this.fillSegTag + '>';\n        };\n        FillRenderer.prototype.detachSegs = function (type, segs) {\n            var containerEls = this.containerElsByType[type];\n            if (containerEls) {\n                containerEls.forEach(removeElement);\n                delete this.containerElsByType[type];\n            }\n        };\n        FillRenderer.prototype.computeSizes = function (force) {\n            for (var type in this.segsByType) {\n                if (force || this.dirtySizeFlags[type]) {\n                    this.computeSegSizes(this.segsByType[type]);\n                }\n            }\n        };\n        FillRenderer.prototype.assignSizes = function (force) {\n            for (var type in this.segsByType) {\n                if (force || this.dirtySizeFlags[type]) {\n                    this.assignSegSizes(this.segsByType[type]);\n                }\n            }\n            this.dirtySizeFlags = {};\n        };\n        FillRenderer.prototype.computeSegSizes = function (segs) {\n        };\n        FillRenderer.prototype.assignSegSizes = function (segs) {\n        };\n        return FillRenderer;\n    }());\n\n    var NamedTimeZoneImpl = /** @class */ (function () {\n        function NamedTimeZoneImpl(timeZoneName) {\n            this.timeZoneName = timeZoneName;\n        }\n        return NamedTimeZoneImpl;\n    }());\n\n    /*\n    An abstraction for a dragging interaction originating on an event.\n    Does higher-level things than PointerDragger, such as possibly:\n    - a \"mirror\" that moves with the pointer\n    - a minimum number of pixels or other criteria for a true drag to begin\n\n    subclasses must emit:\n    - pointerdown\n    - dragstart\n    - dragmove\n    - pointerup\n    - dragend\n    */\n    var ElementDragging = /** @class */ (function () {\n        function ElementDragging(el) {\n            this.emitter = new EmitterMixin();\n        }\n        ElementDragging.prototype.destroy = function () {\n        };\n        ElementDragging.prototype.setMirrorIsVisible = function (bool) {\n            // optional if subclass doesn't want to support a mirror\n        };\n        ElementDragging.prototype.setMirrorNeedsRevert = function (bool) {\n            // optional if subclass doesn't want to support a mirror\n        };\n        ElementDragging.prototype.setAutoScrollEnabled = function (bool) {\n            // optional\n        };\n        return ElementDragging;\n    }());\n\n    function formatDate(dateInput, settings) {\n        if (settings === void 0) { settings = {}; }\n        var dateEnv = buildDateEnv$1(settings);\n        var formatter = createFormatter(settings);\n        var dateMeta = dateEnv.createMarkerMeta(dateInput);\n        if (!dateMeta) { // TODO: warning?\n            return '';\n        }\n        return dateEnv.format(dateMeta.marker, formatter, {\n            forcedTzo: dateMeta.forcedTzo\n        });\n    }\n    function formatRange(startInput, endInput, settings // mixture of env and formatter settings\n    ) {\n        var dateEnv = buildDateEnv$1(typeof settings === 'object' && settings ? settings : {}); // pass in if non-null object\n        var formatter = createFormatter(settings, globalDefaults.defaultRangeSeparator);\n        var startMeta = dateEnv.createMarkerMeta(startInput);\n        var endMeta = dateEnv.createMarkerMeta(endInput);\n        if (!startMeta || !endMeta) { // TODO: warning?\n            return '';\n        }\n        return dateEnv.formatRange(startMeta.marker, endMeta.marker, formatter, {\n            forcedStartTzo: startMeta.forcedTzo,\n            forcedEndTzo: endMeta.forcedTzo,\n            isEndExclusive: settings.isEndExclusive\n        });\n    }\n    // TODO: more DRY and optimized\n    function buildDateEnv$1(settings) {\n        var locale = buildLocale(settings.locale || 'en', parseRawLocales([]).map); // TODO: don't hardcode 'en' everywhere\n        // ensure required settings\n        settings = __assign({ timeZone: globalDefaults.timeZone, calendarSystem: 'gregory' }, settings, { locale: locale });\n        return new DateEnv(settings);\n    }\n\n    var DRAG_META_PROPS = {\n        startTime: createDuration,\n        duration: createDuration,\n        create: Boolean,\n        sourceId: String\n    };\n    var DRAG_META_DEFAULTS = {\n        create: true\n    };\n    function parseDragMeta(raw) {\n        var leftoverProps = {};\n        var refined = refineProps(raw, DRAG_META_PROPS, DRAG_META_DEFAULTS, leftoverProps);\n        refined.leftoverProps = leftoverProps;\n        return refined;\n    }\n\n    // Computes a default column header formatting string if `colFormat` is not explicitly defined\n    function computeFallbackHeaderFormat(datesRepDistinctDays, dayCnt) {\n        // if more than one week row, or if there are a lot of columns with not much space,\n        // put just the day numbers will be in each cell\n        if (!datesRepDistinctDays || dayCnt > 10) {\n            return { weekday: 'short' }; // \"Sat\"\n        }\n        else if (dayCnt > 1) {\n            return { weekday: 'short', month: 'numeric', day: 'numeric', omitCommas: true }; // \"Sat 11/12\"\n        }\n        else {\n            return { weekday: 'long' }; // \"Saturday\"\n        }\n    }\n    function renderDateCell(dateMarker, dateProfile, datesRepDistinctDays, colCnt, colHeadFormat, context, colspan, otherAttrs) {\n        var dateEnv = context.dateEnv, theme = context.theme, options = context.options;\n        var isDateValid = rangeContainsMarker(dateProfile.activeRange, dateMarker); // TODO: called too frequently. cache somehow.\n        var classNames = [\n            'fc-day-header',\n            theme.getClass('widgetHeader')\n        ];\n        var innerHtml;\n        if (typeof options.columnHeaderHtml === 'function') {\n            innerHtml = options.columnHeaderHtml(dateEnv.toDate(dateMarker));\n        }\n        else if (typeof options.columnHeaderText === 'function') {\n            innerHtml = htmlEscape(options.columnHeaderText(dateEnv.toDate(dateMarker)));\n        }\n        else {\n            innerHtml = htmlEscape(dateEnv.format(dateMarker, colHeadFormat));\n        }\n        // if only one row of days, the classNames on the header can represent the specific days beneath\n        if (datesRepDistinctDays) {\n            classNames = classNames.concat(\n            // includes the day-of-week class\n            // noThemeHighlight=true (don't highlight the header)\n            getDayClasses(dateMarker, dateProfile, context, true));\n        }\n        else {\n            classNames.push('fc-' + DAY_IDS[dateMarker.getUTCDay()]); // only add the day-of-week class\n        }\n        return '' +\n            '<th class=\"' + classNames.join(' ') + '\"' +\n            ((isDateValid && datesRepDistinctDays) ?\n                ' data-date=\"' + dateEnv.formatIso(dateMarker, { omitTime: true }) + '\"' :\n                '') +\n            (colspan > 1 ?\n                ' colspan=\"' + colspan + '\"' :\n                '') +\n            (otherAttrs ?\n                ' ' + otherAttrs :\n                '') +\n            '>' +\n            (isDateValid ?\n                // don't make a link if the heading could represent multiple days, or if there's only one day (forceOff)\n                buildGotoAnchorHtml(options, dateEnv, { date: dateMarker, forceOff: !datesRepDistinctDays || colCnt === 1 }, innerHtml) :\n                // if not valid, display text, but no link\n                innerHtml) +\n            '</th>';\n    }\n\n    var DayHeader = /** @class */ (function (_super) {\n        __extends(DayHeader, _super);\n        function DayHeader(parentEl) {\n            var _this = _super.call(this) || this;\n            _this.renderSkeleton = memoizeRendering(_this._renderSkeleton, _this._unrenderSkeleton);\n            _this.parentEl = parentEl;\n            return _this;\n        }\n        DayHeader.prototype.render = function (props, context) {\n            var dates = props.dates, datesRepDistinctDays = props.datesRepDistinctDays;\n            var parts = [];\n            this.renderSkeleton(context);\n            if (props.renderIntroHtml) {\n                parts.push(props.renderIntroHtml());\n            }\n            var colHeadFormat = createFormatter(context.options.columnHeaderFormat ||\n                computeFallbackHeaderFormat(datesRepDistinctDays, dates.length));\n            for (var _i = 0, dates_1 = dates; _i < dates_1.length; _i++) {\n                var date = dates_1[_i];\n                parts.push(renderDateCell(date, props.dateProfile, datesRepDistinctDays, dates.length, colHeadFormat, context));\n            }\n            if (context.isRtl) {\n                parts.reverse();\n            }\n            this.thead.innerHTML = '<tr>' + parts.join('') + '</tr>';\n        };\n        DayHeader.prototype.destroy = function () {\n            _super.prototype.destroy.call(this);\n            this.renderSkeleton.unrender();\n        };\n        DayHeader.prototype._renderSkeleton = function (context) {\n            var theme = context.theme;\n            var parentEl = this.parentEl;\n            parentEl.innerHTML = ''; // because might be nbsp\n            parentEl.appendChild(this.el = htmlToElement('<div class=\"fc-row ' + theme.getClass('headerRow') + '\">' +\n                '<table class=\"' + theme.getClass('tableGrid') + '\">' +\n                '<thead></thead>' +\n                '</table>' +\n                '</div>'));\n            this.thead = this.el.querySelector('thead');\n        };\n        DayHeader.prototype._unrenderSkeleton = function () {\n            removeElement(this.el);\n        };\n        return DayHeader;\n    }(Component));\n\n    var DaySeries = /** @class */ (function () {\n        function DaySeries(range, dateProfileGenerator) {\n            var date = range.start;\n            var end = range.end;\n            var indices = [];\n            var dates = [];\n            var dayIndex = -1;\n            while (date < end) { // loop each day from start to end\n                if (dateProfileGenerator.isHiddenDay(date)) {\n                    indices.push(dayIndex + 0.5); // mark that it's between indices\n                }\n                else {\n                    dayIndex++;\n                    indices.push(dayIndex);\n                    dates.push(date);\n                }\n                date = addDays(date, 1);\n            }\n            this.dates = dates;\n            this.indices = indices;\n            this.cnt = dates.length;\n        }\n        DaySeries.prototype.sliceRange = function (range) {\n            var firstIndex = this.getDateDayIndex(range.start); // inclusive first index\n            var lastIndex = this.getDateDayIndex(addDays(range.end, -1)); // inclusive last index\n            var clippedFirstIndex = Math.max(0, firstIndex);\n            var clippedLastIndex = Math.min(this.cnt - 1, lastIndex);\n            // deal with in-between indices\n            clippedFirstIndex = Math.ceil(clippedFirstIndex); // in-between starts round to next cell\n            clippedLastIndex = Math.floor(clippedLastIndex); // in-between ends round to prev cell\n            if (clippedFirstIndex <= clippedLastIndex) {\n                return {\n                    firstIndex: clippedFirstIndex,\n                    lastIndex: clippedLastIndex,\n                    isStart: firstIndex === clippedFirstIndex,\n                    isEnd: lastIndex === clippedLastIndex\n                };\n            }\n            else {\n                return null;\n            }\n        };\n        // Given a date, returns its chronolocial cell-index from the first cell of the grid.\n        // If the date lies between cells (because of hiddenDays), returns a floating-point value between offsets.\n        // If before the first offset, returns a negative number.\n        // If after the last offset, returns an offset past the last cell offset.\n        // Only works for *start* dates of cells. Will not work for exclusive end dates for cells.\n        DaySeries.prototype.getDateDayIndex = function (date) {\n            var indices = this.indices;\n            var dayOffset = Math.floor(diffDays(this.dates[0], date));\n            if (dayOffset < 0) {\n                return indices[0] - 1;\n            }\n            else if (dayOffset >= indices.length) {\n                return indices[indices.length - 1] + 1;\n            }\n            else {\n                return indices[dayOffset];\n            }\n        };\n        return DaySeries;\n    }());\n\n    var DayTable = /** @class */ (function () {\n        function DayTable(daySeries, breakOnWeeks) {\n            var dates = daySeries.dates;\n            var daysPerRow;\n            var firstDay;\n            var rowCnt;\n            if (breakOnWeeks) {\n                // count columns until the day-of-week repeats\n                firstDay = dates[0].getUTCDay();\n                for (daysPerRow = 1; daysPerRow < dates.length; daysPerRow++) {\n                    if (dates[daysPerRow].getUTCDay() === firstDay) {\n                        break;\n                    }\n                }\n                rowCnt = Math.ceil(dates.length / daysPerRow);\n            }\n            else {\n                rowCnt = 1;\n                daysPerRow = dates.length;\n            }\n            this.rowCnt = rowCnt;\n            this.colCnt = daysPerRow;\n            this.daySeries = daySeries;\n            this.cells = this.buildCells();\n            this.headerDates = this.buildHeaderDates();\n        }\n        DayTable.prototype.buildCells = function () {\n            var rows = [];\n            for (var row = 0; row < this.rowCnt; row++) {\n                var cells = [];\n                for (var col = 0; col < this.colCnt; col++) {\n                    cells.push(this.buildCell(row, col));\n                }\n                rows.push(cells);\n            }\n            return rows;\n        };\n        DayTable.prototype.buildCell = function (row, col) {\n            return {\n                date: this.daySeries.dates[row * this.colCnt + col]\n            };\n        };\n        DayTable.prototype.buildHeaderDates = function () {\n            var dates = [];\n            for (var col = 0; col < this.colCnt; col++) {\n                dates.push(this.cells[0][col].date);\n            }\n            return dates;\n        };\n        DayTable.prototype.sliceRange = function (range) {\n            var colCnt = this.colCnt;\n            var seriesSeg = this.daySeries.sliceRange(range);\n            var segs = [];\n            if (seriesSeg) {\n                var firstIndex = seriesSeg.firstIndex, lastIndex = seriesSeg.lastIndex;\n                var index = firstIndex;\n                while (index <= lastIndex) {\n                    var row = Math.floor(index / colCnt);\n                    var nextIndex = Math.min((row + 1) * colCnt, lastIndex + 1);\n                    segs.push({\n                        row: row,\n                        firstCol: index % colCnt,\n                        lastCol: (nextIndex - 1) % colCnt,\n                        isStart: seriesSeg.isStart && index === firstIndex,\n                        isEnd: seriesSeg.isEnd && (nextIndex - 1) === lastIndex\n                    });\n                    index = nextIndex;\n                }\n            }\n            return segs;\n        };\n        return DayTable;\n    }());\n\n    var Slicer = /** @class */ (function () {\n        function Slicer() {\n            this.sliceBusinessHours = memoize(this._sliceBusinessHours);\n            this.sliceDateSelection = memoize(this._sliceDateSpan);\n            this.sliceEventStore = memoize(this._sliceEventStore);\n            this.sliceEventDrag = memoize(this._sliceInteraction);\n            this.sliceEventResize = memoize(this._sliceInteraction);\n        }\n        Slicer.prototype.sliceProps = function (props, dateProfile, nextDayThreshold, calendar, component) {\n            var extraArgs = [];\n            for (var _i = 5; _i < arguments.length; _i++) {\n                extraArgs[_i - 5] = arguments[_i];\n            }\n            var eventUiBases = props.eventUiBases;\n            var eventSegs = this.sliceEventStore.apply(this, [props.eventStore, eventUiBases, dateProfile, nextDayThreshold, component].concat(extraArgs));\n            return {\n                dateSelectionSegs: this.sliceDateSelection.apply(this, [props.dateSelection, eventUiBases, component].concat(extraArgs)),\n                businessHourSegs: this.sliceBusinessHours.apply(this, [props.businessHours, dateProfile, nextDayThreshold, calendar, component].concat(extraArgs)),\n                fgEventSegs: eventSegs.fg,\n                bgEventSegs: eventSegs.bg,\n                eventDrag: this.sliceEventDrag.apply(this, [props.eventDrag, eventUiBases, dateProfile, nextDayThreshold, component].concat(extraArgs)),\n                eventResize: this.sliceEventResize.apply(this, [props.eventResize, eventUiBases, dateProfile, nextDayThreshold, component].concat(extraArgs)),\n                eventSelection: props.eventSelection\n            }; // TODO: give interactionSegs?\n        };\n        Slicer.prototype.sliceNowDate = function (// does not memoize\n        date, component) {\n            var extraArgs = [];\n            for (var _i = 2; _i < arguments.length; _i++) {\n                extraArgs[_i - 2] = arguments[_i];\n            }\n            return this._sliceDateSpan.apply(this, [{ range: { start: date, end: addMs(date, 1) }, allDay: false },\n                {},\n                component].concat(extraArgs));\n        };\n        Slicer.prototype._sliceBusinessHours = function (businessHours, dateProfile, nextDayThreshold, calendar, component) {\n            var extraArgs = [];\n            for (var _i = 5; _i < arguments.length; _i++) {\n                extraArgs[_i - 5] = arguments[_i];\n            }\n            if (!businessHours) {\n                return [];\n            }\n            return this._sliceEventStore.apply(this, [expandRecurring(businessHours, computeActiveRange(dateProfile, Boolean(nextDayThreshold)), calendar),\n                {},\n                dateProfile,\n                nextDayThreshold,\n                component].concat(extraArgs)).bg;\n        };\n        Slicer.prototype._sliceEventStore = function (eventStore, eventUiBases, dateProfile, nextDayThreshold, component) {\n            var extraArgs = [];\n            for (var _i = 5; _i < arguments.length; _i++) {\n                extraArgs[_i - 5] = arguments[_i];\n            }\n            if (eventStore) {\n                var rangeRes = sliceEventStore(eventStore, eventUiBases, computeActiveRange(dateProfile, Boolean(nextDayThreshold)), nextDayThreshold);\n                return {\n                    bg: this.sliceEventRanges(rangeRes.bg, component, extraArgs),\n                    fg: this.sliceEventRanges(rangeRes.fg, component, extraArgs)\n                };\n            }\n            else {\n                return { bg: [], fg: [] };\n            }\n        };\n        Slicer.prototype._sliceInteraction = function (interaction, eventUiBases, dateProfile, nextDayThreshold, component) {\n            var extraArgs = [];\n            for (var _i = 5; _i < arguments.length; _i++) {\n                extraArgs[_i - 5] = arguments[_i];\n            }\n            if (!interaction) {\n                return null;\n            }\n            var rangeRes = sliceEventStore(interaction.mutatedEvents, eventUiBases, computeActiveRange(dateProfile, Boolean(nextDayThreshold)), nextDayThreshold);\n            return {\n                segs: this.sliceEventRanges(rangeRes.fg, component, extraArgs),\n                affectedInstances: interaction.affectedEvents.instances,\n                isEvent: interaction.isEvent,\n                sourceSeg: interaction.origSeg\n            };\n        };\n        Slicer.prototype._sliceDateSpan = function (dateSpan, eventUiBases, component) {\n            var extraArgs = [];\n            for (var _i = 3; _i < arguments.length; _i++) {\n                extraArgs[_i - 3] = arguments[_i];\n            }\n            if (!dateSpan) {\n                return [];\n            }\n            var eventRange = fabricateEventRange(dateSpan, eventUiBases, component.context.calendar);\n            var segs = this.sliceRange.apply(this, [dateSpan.range].concat(extraArgs));\n            for (var _a = 0, segs_1 = segs; _a < segs_1.length; _a++) {\n                var seg = segs_1[_a];\n                seg.component = component;\n                seg.eventRange = eventRange;\n            }\n            return segs;\n        };\n        /*\n        \"complete\" seg means it has component and eventRange\n        */\n        Slicer.prototype.sliceEventRanges = function (eventRanges, component, // TODO: kill\n        extraArgs) {\n            var segs = [];\n            for (var _i = 0, eventRanges_1 = eventRanges; _i < eventRanges_1.length; _i++) {\n                var eventRange = eventRanges_1[_i];\n                segs.push.apply(segs, this.sliceEventRange(eventRange, component, extraArgs));\n            }\n            return segs;\n        };\n        /*\n        \"complete\" seg means it has component and eventRange\n        */\n        Slicer.prototype.sliceEventRange = function (eventRange, component, // TODO: kill\n        extraArgs) {\n            var segs = this.sliceRange.apply(this, [eventRange.range].concat(extraArgs));\n            for (var _i = 0, segs_2 = segs; _i < segs_2.length; _i++) {\n                var seg = segs_2[_i];\n                seg.component = component;\n                seg.eventRange = eventRange;\n                seg.isStart = eventRange.isStart && seg.isStart;\n                seg.isEnd = eventRange.isEnd && seg.isEnd;\n            }\n            return segs;\n        };\n        return Slicer;\n    }());\n    /*\n    for incorporating minTime/maxTime if appropriate\n    TODO: should be part of DateProfile!\n    TimelineDateProfile already does this btw\n    */\n    function computeActiveRange(dateProfile, isComponentAllDay) {\n        var range = dateProfile.activeRange;\n        if (isComponentAllDay) {\n            return range;\n        }\n        return {\n            start: addMs(range.start, dateProfile.minTime.milliseconds),\n            end: addMs(range.end, dateProfile.maxTime.milliseconds - 864e5) // 864e5 = ms in a day\n        };\n    }\n\n    // exports\n    // --------------------------------------------------------------------------------------------------\n    var version = '4.4.2';\n\n    exports.Calendar = Calendar;\n    exports.Component = Component;\n    exports.ComponentContext = ComponentContext;\n    exports.DateComponent = DateComponent;\n    exports.DateEnv = DateEnv;\n    exports.DateProfileGenerator = DateProfileGenerator;\n    exports.DayHeader = DayHeader;\n    exports.DaySeries = DaySeries;\n    exports.DayTable = DayTable;\n    exports.ElementDragging = ElementDragging;\n    exports.ElementScrollController = ElementScrollController;\n    exports.EmitterMixin = EmitterMixin;\n    exports.EventApi = EventApi;\n    exports.FgEventRenderer = FgEventRenderer;\n    exports.FillRenderer = FillRenderer;\n    exports.Interaction = Interaction;\n    exports.Mixin = Mixin;\n    exports.NamedTimeZoneImpl = NamedTimeZoneImpl;\n    exports.PositionCache = PositionCache;\n    exports.ScrollComponent = ScrollComponent;\n    exports.ScrollController = ScrollController;\n    exports.Slicer = Slicer;\n    exports.Splitter = Splitter;\n    exports.Theme = Theme;\n    exports.View = View;\n    exports.WindowScrollController = WindowScrollController;\n    exports.addDays = addDays;\n    exports.addDurations = addDurations;\n    exports.addMs = addMs;\n    exports.addWeeks = addWeeks;\n    exports.allowContextMenu = allowContextMenu;\n    exports.allowSelection = allowSelection;\n    exports.appendToElement = appendToElement;\n    exports.applyAll = applyAll;\n    exports.applyMutationToEventStore = applyMutationToEventStore;\n    exports.applyStyle = applyStyle;\n    exports.applyStyleProp = applyStyleProp;\n    exports.asRoughMinutes = asRoughMinutes;\n    exports.asRoughMs = asRoughMs;\n    exports.asRoughSeconds = asRoughSeconds;\n    exports.buildGotoAnchorHtml = buildGotoAnchorHtml;\n    exports.buildSegCompareObj = buildSegCompareObj;\n    exports.capitaliseFirstLetter = capitaliseFirstLetter;\n    exports.combineEventUis = combineEventUis;\n    exports.compareByFieldSpec = compareByFieldSpec;\n    exports.compareByFieldSpecs = compareByFieldSpecs;\n    exports.compareNumbers = compareNumbers;\n    exports.compensateScroll = compensateScroll;\n    exports.computeClippingRect = computeClippingRect;\n    exports.computeEdges = computeEdges;\n    exports.computeEventDraggable = computeEventDraggable;\n    exports.computeEventEndResizable = computeEventEndResizable;\n    exports.computeEventStartResizable = computeEventStartResizable;\n    exports.computeFallbackHeaderFormat = computeFallbackHeaderFormat;\n    exports.computeHeightAndMargins = computeHeightAndMargins;\n    exports.computeInnerRect = computeInnerRect;\n    exports.computeRect = computeRect;\n    exports.computeVisibleDayRange = computeVisibleDayRange;\n    exports.config = config;\n    exports.constrainPoint = constrainPoint;\n    exports.createDuration = createDuration;\n    exports.createElement = createElement;\n    exports.createEmptyEventStore = createEmptyEventStore;\n    exports.createEventInstance = createEventInstance;\n    exports.createFormatter = createFormatter;\n    exports.createPlugin = createPlugin;\n    exports.cssToStr = cssToStr;\n    exports.debounce = debounce;\n    exports.diffDates = diffDates;\n    exports.diffDayAndTime = diffDayAndTime;\n    exports.diffDays = diffDays;\n    exports.diffPoints = diffPoints;\n    exports.diffWeeks = diffWeeks;\n    exports.diffWholeDays = diffWholeDays;\n    exports.diffWholeWeeks = diffWholeWeeks;\n    exports.disableCursor = disableCursor;\n    exports.distributeHeight = distributeHeight;\n    exports.elementClosest = elementClosest;\n    exports.elementMatches = elementMatches;\n    exports.enableCursor = enableCursor;\n    exports.eventTupleToStore = eventTupleToStore;\n    exports.filterEventStoreDefs = filterEventStoreDefs;\n    exports.filterHash = filterHash;\n    exports.findChildren = findChildren;\n    exports.findElements = findElements;\n    exports.flexibleCompare = flexibleCompare;\n    exports.forceClassName = forceClassName;\n    exports.formatDate = formatDate;\n    exports.formatIsoTimeString = formatIsoTimeString;\n    exports.formatRange = formatRange;\n    exports.getAllDayHtml = getAllDayHtml;\n    exports.getClippingParents = getClippingParents;\n    exports.getDayClasses = getDayClasses;\n    exports.getElSeg = getElSeg;\n    exports.getRectCenter = getRectCenter;\n    exports.getRelevantEvents = getRelevantEvents;\n    exports.globalDefaults = globalDefaults;\n    exports.greatestDurationDenominator = greatestDurationDenominator;\n    exports.hasBgRendering = hasBgRendering;\n    exports.htmlEscape = htmlEscape;\n    exports.htmlToElement = htmlToElement;\n    exports.insertAfterElement = insertAfterElement;\n    exports.interactionSettingsStore = interactionSettingsStore;\n    exports.interactionSettingsToStore = interactionSettingsToStore;\n    exports.intersectRanges = intersectRanges;\n    exports.intersectRects = intersectRects;\n    exports.isArraysEqual = isArraysEqual;\n    exports.isDateSpansEqual = isDateSpansEqual;\n    exports.isInt = isInt;\n    exports.isInteractionValid = isInteractionValid;\n    exports.isMultiDayRange = isMultiDayRange;\n    exports.isPropsEqual = isPropsEqual;\n    exports.isPropsValid = isPropsValid;\n    exports.isSingleDay = isSingleDay;\n    exports.isValidDate = isValidDate;\n    exports.listenBySelector = listenBySelector;\n    exports.mapHash = mapHash;\n    exports.matchCellWidths = matchCellWidths;\n    exports.memoize = memoize;\n    exports.memoizeOutput = memoizeOutput;\n    exports.memoizeRendering = memoizeRendering;\n    exports.mergeEventStores = mergeEventStores;\n    exports.multiplyDuration = multiplyDuration;\n    exports.padStart = padStart;\n    exports.parseBusinessHours = parseBusinessHours;\n    exports.parseDragMeta = parseDragMeta;\n    exports.parseEventDef = parseEventDef;\n    exports.parseFieldSpecs = parseFieldSpecs;\n    exports.parseMarker = parse;\n    exports.pointInsideRect = pointInsideRect;\n    exports.prependToElement = prependToElement;\n    exports.preventContextMenu = preventContextMenu;\n    exports.preventDefault = preventDefault;\n    exports.preventSelection = preventSelection;\n    exports.processScopedUiProps = processScopedUiProps;\n    exports.rangeContainsMarker = rangeContainsMarker;\n    exports.rangeContainsRange = rangeContainsRange;\n    exports.rangesEqual = rangesEqual;\n    exports.rangesIntersect = rangesIntersect;\n    exports.refineProps = refineProps;\n    exports.removeElement = removeElement;\n    exports.removeExact = removeExact;\n    exports.renderDateCell = renderDateCell;\n    exports.requestJson = requestJson;\n    exports.sliceEventStore = sliceEventStore;\n    exports.startOfDay = startOfDay;\n    exports.subtractInnerElHeight = subtractInnerElHeight;\n    exports.translateRect = translateRect;\n    exports.uncompensateScroll = uncompensateScroll;\n    exports.undistributeHeight = undistributeHeight;\n    exports.unpromisify = unpromisify;\n    exports.version = version;\n    exports.whenTransitionDone = whenTransitionDone;\n    exports.wholeDivideDurations = wholeDivideDurations;\n\n    Object.defineProperty(exports, '__esModule', { value: true });\n\n}));\n"

/***/ },

/***/ 1083
(module) {

module.exports = "/*!\nFullCalendar Day Grid Plugin v4.4.2\nDocs & License: https://fullcalendar.io/\n(c) 2019 Adam Shaw\n*/\n\n(function (global, factory) {\n    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@fullcalendar/core')) :\n    typeof define === 'function' && define.amd ? define(['exports', '@fullcalendar/core'], factory) :\n    (global = global || self, factory(global.FullCalendarDayGrid = {}, global.FullCalendar));\n}(this, function (exports, core) { 'use strict';\n\n    /*! *****************************************************************************\r\n    Copyright (c) Microsoft Corporation.\r\n\r\n    Permission to use, copy, modify, and/or distribute this software for any\r\n    purpose with or without fee is hereby granted.\r\n\r\n    THE SOFTWARE IS PROVIDED \"AS IS\" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH\r\n    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY\r\n    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,\r\n    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM\r\n    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR\r\n    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR\r\n    PERFORMANCE OF THIS SOFTWARE.\r\n    ***************************************************************************** */\r\n    /* global Reflect, Promise */\r\n\r\n    var extendStatics = function(d, b) {\r\n        extendStatics = Object.setPrototypeOf ||\r\n            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||\r\n            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };\r\n        return extendStatics(d, b);\r\n    };\r\n\r\n    function __extends(d, b) {\r\n        extendStatics(d, b);\r\n        function __() { this.constructor = d; }\r\n        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());\r\n    }\r\n\r\n    var __assign = function() {\r\n        __assign = Object.assign || function __assign(t) {\r\n            for (var s, i = 1, n = arguments.length; i < n; i++) {\r\n                s = arguments[i];\r\n                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];\r\n            }\r\n            return t;\r\n        };\r\n        return __assign.apply(this, arguments);\r\n    };\n\n    var DayGridDateProfileGenerator = /** @class */ (function (_super) {\n        __extends(DayGridDateProfileGenerator, _super);\n        function DayGridDateProfileGenerator() {\n            return _super !== null && _super.apply(this, arguments) || this;\n        }\n        // Computes the date range that will be rendered.\n        DayGridDateProfileGenerator.prototype.buildRenderRange = function (currentRange, currentRangeUnit, isRangeAllDay) {\n            var dateEnv = this.dateEnv;\n            var renderRange = _super.prototype.buildRenderRange.call(this, currentRange, currentRangeUnit, isRangeAllDay);\n            var start = renderRange.start;\n            var end = renderRange.end;\n            var endOfWeek;\n            // year and month views should be aligned with weeks. this is already done for week\n            if (/^(year|month)$/.test(currentRangeUnit)) {\n                start = dateEnv.startOfWeek(start);\n                // make end-of-week if not already\n                endOfWeek = dateEnv.startOfWeek(end);\n                if (endOfWeek.valueOf() !== end.valueOf()) {\n                    end = core.addWeeks(endOfWeek, 1);\n                }\n            }\n            // ensure 6 weeks\n            if (this.options.monthMode &&\n                this.options.fixedWeekCount) {\n                var rowCnt = Math.ceil(// could be partial weeks due to hiddenDays\n                core.diffWeeks(start, end));\n                end = core.addWeeks(end, 6 - rowCnt);\n            }\n            return { start: start, end: end };\n        };\n        return DayGridDateProfileGenerator;\n    }(core.DateProfileGenerator));\n\n    /* A rectangular panel that is absolutely positioned over other content\n    ------------------------------------------------------------------------------------------------------------------------\n    Options:\n      - className (string)\n      - content (HTML string, element, or element array)\n      - parentEl\n      - top\n      - left\n      - right (the x coord of where the right edge should be. not a \"CSS\" right)\n      - autoHide (boolean)\n      - show (callback)\n      - hide (callback)\n    */\n    var Popover = /** @class */ (function () {\n        function Popover(options) {\n            var _this = this;\n            this.isHidden = true;\n            this.margin = 10; // the space required between the popover and the edges of the scroll container\n            // Triggered when the user clicks *anywhere* in the document, for the autoHide feature\n            this.documentMousedown = function (ev) {\n                // only hide the popover if the click happened outside the popover\n                if (_this.el && !_this.el.contains(ev.target)) {\n                    _this.hide();\n                }\n            };\n            this.options = options;\n        }\n        // Shows the popover on the specified position. Renders it if not already\n        Popover.prototype.show = function () {\n            if (this.isHidden) {\n                if (!this.el) {\n                    this.render();\n                }\n                this.el.style.display = '';\n                this.position();\n                this.isHidden = false;\n                this.trigger('show');\n            }\n        };\n        // Hides the popover, through CSS, but does not remove it from the DOM\n        Popover.prototype.hide = function () {\n            if (!this.isHidden) {\n                this.el.style.display = 'none';\n                this.isHidden = true;\n                this.trigger('hide');\n            }\n        };\n        // Creates `this.el` and renders content inside of it\n        Popover.prototype.render = function () {\n            var _this = this;\n            var options = this.options;\n            var el = this.el = core.createElement('div', {\n                className: 'fc-popover ' + (options.className || ''),\n                style: {\n                    top: '0',\n                    left: '0'\n                }\n            });\n            if (typeof options.content === 'function') {\n                options.content(el);\n            }\n            options.parentEl.appendChild(el);\n            // when a click happens on anything inside with a 'fc-close' className, hide the popover\n            core.listenBySelector(el, 'click', '.fc-close', function (ev) {\n                _this.hide();\n            });\n            if (options.autoHide) {\n                document.addEventListener('mousedown', this.documentMousedown);\n            }\n        };\n        // Hides and unregisters any handlers\n        Popover.prototype.destroy = function () {\n            this.hide();\n            if (this.el) {\n                core.removeElement(this.el);\n                this.el = null;\n            }\n            document.removeEventListener('mousedown', this.documentMousedown);\n        };\n        // Positions the popover optimally, using the top/left/right options\n        Popover.prototype.position = function () {\n            var options = this.options;\n            var el = this.el;\n            var elDims = el.getBoundingClientRect(); // only used for width,height\n            var origin = core.computeRect(el.offsetParent);\n            var clippingRect = core.computeClippingRect(options.parentEl);\n            var top; // the \"position\" (not \"offset\") values for the popover\n            var left; //\n            // compute top and left\n            top = options.top || 0;\n            if (options.left !== undefined) {\n                left = options.left;\n            }\n            else if (options.right !== undefined) {\n                left = options.right - elDims.width; // derive the left value from the right value\n            }\n            else {\n                left = 0;\n            }\n            // constrain to the view port. if constrained by two edges, give precedence to top/left\n            top = Math.min(top, clippingRect.bottom - elDims.height - this.margin);\n            top = Math.max(top, clippingRect.top + this.margin);\n            left = Math.min(left, clippingRect.right - elDims.width - this.margin);\n            left = Math.max(left, clippingRect.left + this.margin);\n            core.applyStyle(el, {\n                top: top - origin.top,\n                left: left - origin.left\n            });\n        };\n        // Triggers a callback. Calls a function in the option hash of the same name.\n        // Arguments beyond the first `name` are forwarded on.\n        // TODO: better code reuse for this. Repeat code\n        // can kill this???\n        Popover.prototype.trigger = function (name) {\n            if (this.options[name]) {\n                this.options[name].apply(this, Array.prototype.slice.call(arguments, 1));\n            }\n        };\n        return Popover;\n    }());\n\n    /* Event-rendering methods for the DayGrid class\n    ----------------------------------------------------------------------------------------------------------------------*/\n    // \"Simple\" is bad a name. has nothing to do with SimpleDayGrid\n    var SimpleDayGridEventRenderer = /** @class */ (function (_super) {\n        __extends(SimpleDayGridEventRenderer, _super);\n        function SimpleDayGridEventRenderer() {\n            return _super !== null && _super.apply(this, arguments) || this;\n        }\n        // Builds the HTML to be used for the default element for an individual segment\n        SimpleDayGridEventRenderer.prototype.renderSegHtml = function (seg, mirrorInfo) {\n            var context = this.context;\n            var eventRange = seg.eventRange;\n            var eventDef = eventRange.def;\n            var eventUi = eventRange.ui;\n            var allDay = eventDef.allDay;\n            var isDraggable = core.computeEventDraggable(context, eventDef, eventUi);\n            var isResizableFromStart = allDay && seg.isStart && core.computeEventStartResizable(context, eventDef, eventUi);\n            var isResizableFromEnd = allDay && seg.isEnd && core.computeEventEndResizable(context, eventDef, eventUi);\n            var classes = this.getSegClasses(seg, isDraggable, isResizableFromStart || isResizableFromEnd, mirrorInfo);\n            var skinCss = core.cssToStr(this.getSkinCss(eventUi));\n            var timeHtml = '';\n            var timeText;\n            var titleHtml;\n            classes.unshift('fc-day-grid-event', 'fc-h-event');\n            // Only display a timed events time if it is the starting segment\n            if (seg.isStart) {\n                timeText = this.getTimeText(eventRange);\n                if (timeText) {\n                    timeHtml = '<span class=\"fc-time\">' + core.htmlEscape(timeText) + '</span>';\n                }\n            }\n            titleHtml =\n                '<span class=\"fc-title\">' +\n                    (core.htmlEscape(eventDef.title || '') || '&nbsp;') + // we always want one line of height\n                    '</span>';\n            return '<a class=\"' + classes.join(' ') + '\"' +\n                (eventDef.url ?\n                    ' href=\"' + core.htmlEscape(eventDef.url) + '\"' :\n                    '') +\n                (skinCss ?\n                    ' style=\"' + skinCss + '\"' :\n                    '') +\n                '>' +\n                '<div class=\"fc-content\">' +\n                (context.options.dir === 'rtl' ?\n                    titleHtml + ' ' + timeHtml : // put a natural space in between\n                    timeHtml + ' ' + titleHtml //\n                ) +\n                '</div>' +\n                (isResizableFromStart ?\n                    '<div class=\"fc-resizer fc-start-resizer\"></div>' :\n                    '') +\n                (isResizableFromEnd ?\n                    '<div class=\"fc-resizer fc-end-resizer\"></div>' :\n                    '') +\n                '</a>';\n        };\n        // Computes a default event time formatting string if `eventTimeFormat` is not explicitly defined\n        SimpleDayGridEventRenderer.prototype.computeEventTimeFormat = function () {\n            return {\n                hour: 'numeric',\n                minute: '2-digit',\n                omitZeroMinute: true,\n                meridiem: 'narrow'\n            };\n        };\n        SimpleDayGridEventRenderer.prototype.computeDisplayEventEnd = function () {\n            return false; // TODO: somehow consider the originating DayGrid's column count\n        };\n        return SimpleDayGridEventRenderer;\n    }(core.FgEventRenderer));\n\n    /* Event-rendering methods for the DayGrid class\n    ----------------------------------------------------------------------------------------------------------------------*/\n    var DayGridEventRenderer = /** @class */ (function (_super) {\n        __extends(DayGridEventRenderer, _super);\n        function DayGridEventRenderer(dayGrid) {\n            var _this = _super.call(this) || this;\n            _this.dayGrid = dayGrid;\n            return _this;\n        }\n        // Renders the given foreground event segments onto the grid\n        DayGridEventRenderer.prototype.attachSegs = function (segs, mirrorInfo) {\n            var rowStructs = this.rowStructs = this.renderSegRows(segs);\n            // append to each row's content skeleton\n            this.dayGrid.rowEls.forEach(function (rowNode, i) {\n                rowNode.querySelector('.fc-content-skeleton > table').appendChild(rowStructs[i].tbodyEl);\n            });\n            // removes the \"more..\" events popover\n            if (!mirrorInfo) {\n                this.dayGrid.removeSegPopover();\n            }\n        };\n        // Unrenders all currently rendered foreground event segments\n        DayGridEventRenderer.prototype.detachSegs = function () {\n            var rowStructs = this.rowStructs || [];\n            var rowStruct;\n            while ((rowStruct = rowStructs.pop())) {\n                core.removeElement(rowStruct.tbodyEl);\n            }\n            this.rowStructs = null;\n        };\n        // Uses the given events array to generate <tbody> elements that should be appended to each row's content skeleton.\n        // Returns an array of rowStruct objects (see the bottom of `renderSegRow`).\n        // PRECONDITION: each segment shoud already have a rendered and assigned `.el`\n        DayGridEventRenderer.prototype.renderSegRows = function (segs) {\n            var rowStructs = [];\n            var segRows;\n            var row;\n            segRows = this.groupSegRows(segs); // group into nested arrays\n            // iterate each row of segment groupings\n            for (row = 0; row < segRows.length; row++) {\n                rowStructs.push(this.renderSegRow(row, segRows[row]));\n            }\n            return rowStructs;\n        };\n        // Given a row # and an array of segments all in the same row, render a <tbody> element, a skeleton that contains\n        // the segments. Returns object with a bunch of internal data about how the render was calculated.\n        // NOTE: modifies rowSegs\n        DayGridEventRenderer.prototype.renderSegRow = function (row, rowSegs) {\n            var isRtl = this.context.isRtl;\n            var dayGrid = this.dayGrid;\n            var colCnt = dayGrid.colCnt;\n            var segLevels = this.buildSegLevels(rowSegs); // group into sub-arrays of levels\n            var levelCnt = Math.max(1, segLevels.length); // ensure at least one level\n            var tbody = document.createElement('tbody');\n            var segMatrix = []; // lookup for which segments are rendered into which level+col cells\n            var cellMatrix = []; // lookup for all <td> elements of the level+col matrix\n            var loneCellMatrix = []; // lookup for <td> elements that only take up a single column\n            var i;\n            var levelSegs;\n            var col;\n            var tr;\n            var j;\n            var seg;\n            var td;\n            // populates empty cells from the current column (`col`) to `endCol`\n            function emptyCellsUntil(endCol) {\n                while (col < endCol) {\n                    // try to grab a cell from the level above and extend its rowspan. otherwise, create a fresh cell\n                    td = (loneCellMatrix[i - 1] || [])[col];\n                    if (td) {\n                        td.rowSpan = (td.rowSpan || 1) + 1;\n                    }\n                    else {\n                        td = document.createElement('td');\n                        tr.appendChild(td);\n                    }\n                    cellMatrix[i][col] = td;\n                    loneCellMatrix[i][col] = td;\n                    col++;\n                }\n            }\n            for (i = 0; i < levelCnt; i++) { // iterate through all levels\n                levelSegs = segLevels[i];\n                col = 0;\n                tr = document.createElement('tr');\n                segMatrix.push([]);\n                cellMatrix.push([]);\n                loneCellMatrix.push([]);\n                // levelCnt might be 1 even though there are no actual levels. protect against this.\n                // this single empty row is useful for styling.\n                if (levelSegs) {\n                    for (j = 0; j < levelSegs.length; j++) { // iterate through segments in level\n                        seg = levelSegs[j];\n                        var leftCol = isRtl ? (colCnt - 1 - seg.lastCol) : seg.firstCol;\n                        var rightCol = isRtl ? (colCnt - 1 - seg.firstCol) : seg.lastCol;\n                        emptyCellsUntil(leftCol);\n                        // create a container that occupies or more columns. append the event element.\n                        td = core.createElement('td', { className: 'fc-event-container' }, seg.el);\n                        if (leftCol !== rightCol) {\n                            td.colSpan = rightCol - leftCol + 1;\n                        }\n                        else { // a single-column segment\n                            loneCellMatrix[i][col] = td;\n                        }\n                        while (col <= rightCol) {\n                            cellMatrix[i][col] = td;\n                            segMatrix[i][col] = seg;\n                            col++;\n                        }\n                        tr.appendChild(td);\n                    }\n                }\n                emptyCellsUntil(colCnt); // finish off the row\n                var introHtml = dayGrid.renderProps.renderIntroHtml();\n                if (introHtml) {\n                    if (isRtl) {\n                        core.appendToElement(tr, introHtml);\n                    }\n                    else {\n                        core.prependToElement(tr, introHtml);\n                    }\n                }\n                tbody.appendChild(tr);\n            }\n            return {\n                row: row,\n                tbodyEl: tbody,\n                cellMatrix: cellMatrix,\n                segMatrix: segMatrix,\n                segLevels: segLevels,\n                segs: rowSegs\n            };\n        };\n        // Stacks a flat array of segments, which are all assumed to be in the same row, into subarrays of vertical levels.\n        // NOTE: modifies segs\n        DayGridEventRenderer.prototype.buildSegLevels = function (segs) {\n            var isRtl = this.context.isRtl;\n            var colCnt = this.dayGrid.colCnt;\n            var levels = [];\n            var i;\n            var seg;\n            var j;\n            // Give preference to elements with certain criteria, so they have\n            // a chance to be closer to the top.\n            segs = this.sortEventSegs(segs);\n            for (i = 0; i < segs.length; i++) {\n                seg = segs[i];\n                // loop through levels, starting with the topmost, until the segment doesn't collide with other segments\n                for (j = 0; j < levels.length; j++) {\n                    if (!isDaySegCollision(seg, levels[j])) {\n                        break;\n                    }\n                }\n                // `j` now holds the desired subrow index\n                seg.level = j;\n                seg.leftCol = isRtl ? (colCnt - 1 - seg.lastCol) : seg.firstCol; // for sorting only\n                seg.rightCol = isRtl ? (colCnt - 1 - seg.firstCol) : seg.lastCol // for sorting only\n                ;\n                (levels[j] || (levels[j] = [])).push(seg);\n            }\n            // order segments left-to-right. very important if calendar is RTL\n            for (j = 0; j < levels.length; j++) {\n                levels[j].sort(compareDaySegCols);\n            }\n            return levels;\n        };\n        // Given a flat array of segments, return an array of sub-arrays, grouped by each segment's row\n        DayGridEventRenderer.prototype.groupSegRows = function (segs) {\n            var segRows = [];\n            var i;\n            for (i = 0; i < this.dayGrid.rowCnt; i++) {\n                segRows.push([]);\n            }\n            for (i = 0; i < segs.length; i++) {\n                segRows[segs[i].row].push(segs[i]);\n            }\n            return segRows;\n        };\n        // Computes a default `displayEventEnd` value if one is not expliclty defined\n        DayGridEventRenderer.prototype.computeDisplayEventEnd = function () {\n            return this.dayGrid.colCnt === 1; // we'll likely have space if there's only one day\n        };\n        return DayGridEventRenderer;\n    }(SimpleDayGridEventRenderer));\n    // Computes whether two segments' columns collide. They are assumed to be in the same row.\n    function isDaySegCollision(seg, otherSegs) {\n        var i;\n        var otherSeg;\n        for (i = 0; i < otherSegs.length; i++) {\n            otherSeg = otherSegs[i];\n            if (otherSeg.firstCol <= seg.lastCol &&\n                otherSeg.lastCol >= seg.firstCol) {\n                return true;\n            }\n        }\n        return false;\n    }\n    // A cmp function for determining the leftmost event\n    function compareDaySegCols(a, b) {\n        return a.leftCol - b.leftCol;\n    }\n\n    var DayGridMirrorRenderer = /** @class */ (function (_super) {\n        __extends(DayGridMirrorRenderer, _super);\n        function DayGridMirrorRenderer() {\n            return _super !== null && _super.apply(this, arguments) || this;\n        }\n        DayGridMirrorRenderer.prototype.attachSegs = function (segs, mirrorInfo) {\n            var sourceSeg = mirrorInfo.sourceSeg;\n            var rowStructs = this.rowStructs = this.renderSegRows(segs);\n            // inject each new event skeleton into each associated row\n            this.dayGrid.rowEls.forEach(function (rowNode, row) {\n                var skeletonEl = core.htmlToElement('<div class=\"fc-mirror-skeleton\"><table></table></div>'); // will be absolutely positioned\n                var skeletonTopEl;\n                var skeletonTop;\n                // If there is an original segment, match the top position. Otherwise, put it at the row's top level\n                if (sourceSeg && sourceSeg.row === row) {\n                    skeletonTopEl = sourceSeg.el;\n                }\n                else {\n                    skeletonTopEl = rowNode.querySelector('.fc-content-skeleton tbody');\n                    if (!skeletonTopEl) { // when no events\n                        skeletonTopEl = rowNode.querySelector('.fc-content-skeleton table');\n                    }\n                }\n                skeletonTop = skeletonTopEl.getBoundingClientRect().top -\n                    rowNode.getBoundingClientRect().top; // the offsetParent origin\n                skeletonEl.style.top = skeletonTop + 'px';\n                skeletonEl.querySelector('table').appendChild(rowStructs[row].tbodyEl);\n                rowNode.appendChild(skeletonEl);\n            });\n        };\n        return DayGridMirrorRenderer;\n    }(DayGridEventRenderer));\n\n    var EMPTY_CELL_HTML = '<td style=\"pointer-events:none\"></td>';\n    var DayGridFillRenderer = /** @class */ (function (_super) {\n        __extends(DayGridFillRenderer, _super);\n        function DayGridFillRenderer(dayGrid) {\n            var _this = _super.call(this) || this;\n            _this.fillSegTag = 'td'; // override the default tag name\n            _this.dayGrid = dayGrid;\n            return _this;\n        }\n        DayGridFillRenderer.prototype.renderSegs = function (type, context, segs) {\n            // don't render timed background events\n            if (type === 'bgEvent') {\n                segs = segs.filter(function (seg) {\n                    return seg.eventRange.def.allDay;\n                });\n            }\n            _super.prototype.renderSegs.call(this, type, context, segs);\n        };\n        DayGridFillRenderer.prototype.attachSegs = function (type, segs) {\n            var els = [];\n            var i;\n            var seg;\n            var skeletonEl;\n            for (i = 0; i < segs.length; i++) {\n                seg = segs[i];\n                skeletonEl = this.renderFillRow(type, seg);\n                this.dayGrid.rowEls[seg.row].appendChild(skeletonEl);\n                els.push(skeletonEl);\n            }\n            return els;\n        };\n        // Generates the HTML needed for one row of a fill. Requires the seg's el to be rendered.\n        DayGridFillRenderer.prototype.renderFillRow = function (type, seg) {\n            var dayGrid = this.dayGrid;\n            var isRtl = this.context.isRtl;\n            var colCnt = dayGrid.colCnt;\n            var leftCol = isRtl ? (colCnt - 1 - seg.lastCol) : seg.firstCol;\n            var rightCol = isRtl ? (colCnt - 1 - seg.firstCol) : seg.lastCol;\n            var startCol = leftCol;\n            var endCol = rightCol + 1;\n            var className;\n            var skeletonEl;\n            var trEl;\n            if (type === 'businessHours') {\n                className = 'bgevent';\n            }\n            else {\n                className = type.toLowerCase();\n            }\n            skeletonEl = core.htmlToElement('<div class=\"fc-' + className + '-skeleton\">' +\n                '<table><tr></tr></table>' +\n                '</div>');\n            trEl = skeletonEl.getElementsByTagName('tr')[0];\n            if (startCol > 0) {\n                core.appendToElement(trEl, \n                // will create (startCol + 1) td's\n                new Array(startCol + 1).join(EMPTY_CELL_HTML));\n            }\n            seg.el.colSpan = endCol - startCol;\n            trEl.appendChild(seg.el);\n            if (endCol < colCnt) {\n                core.appendToElement(trEl, \n                // will create (colCnt - endCol) td's\n                new Array(colCnt - endCol + 1).join(EMPTY_CELL_HTML));\n            }\n            var introHtml = dayGrid.renderProps.renderIntroHtml();\n            if (introHtml) {\n                if (isRtl) {\n                    core.appendToElement(trEl, introHtml);\n                }\n                else {\n                    core.prependToElement(trEl, introHtml);\n                }\n            }\n            return skeletonEl;\n        };\n        return DayGridFillRenderer;\n    }(core.FillRenderer));\n\n    var DayTile = /** @class */ (function (_super) {\n        __extends(DayTile, _super);\n        function DayTile(el) {\n            var _this = _super.call(this, el) || this;\n            var eventRenderer = _this.eventRenderer = new DayTileEventRenderer(_this);\n            var renderFrame = _this.renderFrame = core.memoizeRendering(_this._renderFrame);\n            _this.renderFgEvents = core.memoizeRendering(eventRenderer.renderSegs.bind(eventRenderer), eventRenderer.unrender.bind(eventRenderer), [renderFrame]);\n            _this.renderEventSelection = core.memoizeRendering(eventRenderer.selectByInstanceId.bind(eventRenderer), eventRenderer.unselectByInstanceId.bind(eventRenderer), [_this.renderFgEvents]);\n            _this.renderEventDrag = core.memoizeRendering(eventRenderer.hideByHash.bind(eventRenderer), eventRenderer.showByHash.bind(eventRenderer), [renderFrame]);\n            _this.renderEventResize = core.memoizeRendering(eventRenderer.hideByHash.bind(eventRenderer), eventRenderer.showByHash.bind(eventRenderer), [renderFrame]);\n            return _this;\n        }\n        DayTile.prototype.firstContext = function (context) {\n            context.calendar.registerInteractiveComponent(this, {\n                el: this.el,\n                useEventCenter: false\n            });\n        };\n        DayTile.prototype.render = function (props, context) {\n            this.renderFrame(props.date);\n            this.renderFgEvents(context, props.fgSegs);\n            this.renderEventSelection(props.eventSelection);\n            this.renderEventDrag(props.eventDragInstances);\n            this.renderEventResize(props.eventResizeInstances);\n        };\n        DayTile.prototype.destroy = function () {\n            _super.prototype.destroy.call(this);\n            this.renderFrame.unrender(); // should unrender everything else\n            this.context.calendar.unregisterInteractiveComponent(this);\n        };\n        DayTile.prototype._renderFrame = function (date) {\n            var _a = this.context, theme = _a.theme, dateEnv = _a.dateEnv, options = _a.options;\n            var title = dateEnv.format(date, core.createFormatter(options.dayPopoverFormat) // TODO: cache\n            );\n            this.el.innerHTML =\n                '<div class=\"fc-header ' + theme.getClass('popoverHeader') + '\">' +\n                    '<span class=\"fc-title\">' +\n                    core.htmlEscape(title) +\n                    '</span>' +\n                    '<span class=\"fc-close ' + theme.getIconClass('close') + '\"></span>' +\n                    '</div>' +\n                    '<div class=\"fc-body ' + theme.getClass('popoverContent') + '\">' +\n                    '<div class=\"fc-event-container\"></div>' +\n                    '</div>';\n            this.segContainerEl = this.el.querySelector('.fc-event-container');\n        };\n        DayTile.prototype.queryHit = function (positionLeft, positionTop, elWidth, elHeight) {\n            var date = this.props.date; // HACK\n            if (positionLeft < elWidth && positionTop < elHeight) {\n                return {\n                    component: this,\n                    dateSpan: {\n                        allDay: true,\n                        range: { start: date, end: core.addDays(date, 1) }\n                    },\n                    dayEl: this.el,\n                    rect: {\n                        left: 0,\n                        top: 0,\n                        right: elWidth,\n                        bottom: elHeight\n                    },\n                    layer: 1\n                };\n            }\n        };\n        return DayTile;\n    }(core.DateComponent));\n    var DayTileEventRenderer = /** @class */ (function (_super) {\n        __extends(DayTileEventRenderer, _super);\n        function DayTileEventRenderer(dayTile) {\n            var _this = _super.call(this) || this;\n            _this.dayTile = dayTile;\n            return _this;\n        }\n        DayTileEventRenderer.prototype.attachSegs = function (segs) {\n            for (var _i = 0, segs_1 = segs; _i < segs_1.length; _i++) {\n                var seg = segs_1[_i];\n                this.dayTile.segContainerEl.appendChild(seg.el);\n            }\n        };\n        DayTileEventRenderer.prototype.detachSegs = function (segs) {\n            for (var _i = 0, segs_2 = segs; _i < segs_2.length; _i++) {\n                var seg = segs_2[_i];\n                core.removeElement(seg.el);\n            }\n        };\n        return DayTileEventRenderer;\n    }(SimpleDayGridEventRenderer));\n\n    var DayBgRow = /** @class */ (function () {\n        function DayBgRow(context) {\n            this.context = context;\n        }\n        DayBgRow.prototype.renderHtml = function (props) {\n            var parts = [];\n            if (props.renderIntroHtml) {\n                parts.push(props.renderIntroHtml());\n            }\n            for (var _i = 0, _a = props.cells; _i < _a.length; _i++) {\n                var cell = _a[_i];\n                parts.push(renderCellHtml(cell.date, props.dateProfile, this.context, cell.htmlAttrs));\n            }\n            if (!props.cells.length) {\n                parts.push('<td class=\"fc-day ' + this.context.theme.getClass('widgetContent') + '\"></td>');\n            }\n            if (this.context.options.dir === 'rtl') {\n                parts.reverse();\n            }\n            return '<tr>' + parts.join('') + '</tr>';\n        };\n        return DayBgRow;\n    }());\n    function renderCellHtml(date, dateProfile, context, otherAttrs) {\n        var dateEnv = context.dateEnv, theme = context.theme;\n        var isDateValid = core.rangeContainsMarker(dateProfile.activeRange, date); // TODO: called too frequently. cache somehow.\n        var classes = core.getDayClasses(date, dateProfile, context);\n        classes.unshift('fc-day', theme.getClass('widgetContent'));\n        return '<td class=\"' + classes.join(' ') + '\"' +\n            (isDateValid ?\n                ' data-date=\"' + dateEnv.formatIso(date, { omitTime: true }) + '\"' :\n                '') +\n            (otherAttrs ?\n                ' ' + otherAttrs :\n                '') +\n            '></td>';\n    }\n\n    var DAY_NUM_FORMAT = core.createFormatter({ day: 'numeric' });\n    var WEEK_NUM_FORMAT = core.createFormatter({ week: 'numeric' });\n    var DayGrid = /** @class */ (function (_super) {\n        __extends(DayGrid, _super);\n        function DayGrid(el, renderProps) {\n            var _this = _super.call(this, el) || this;\n            _this.bottomCoordPadding = 0; // hack for extending the hit area for the last row of the coordinate grid\n            _this.isCellSizesDirty = false;\n            _this.renderProps = renderProps;\n            var eventRenderer = _this.eventRenderer = new DayGridEventRenderer(_this);\n            var fillRenderer = _this.fillRenderer = new DayGridFillRenderer(_this);\n            _this.mirrorRenderer = new DayGridMirrorRenderer(_this);\n            var renderCells = _this.renderCells = core.memoizeRendering(_this._renderCells, _this._unrenderCells);\n            _this.renderBusinessHours = core.memoizeRendering(fillRenderer.renderSegs.bind(fillRenderer, 'businessHours'), fillRenderer.unrender.bind(fillRenderer, 'businessHours'), [renderCells]);\n            _this.renderDateSelection = core.memoizeRendering(fillRenderer.renderSegs.bind(fillRenderer, 'highlight'), fillRenderer.unrender.bind(fillRenderer, 'highlight'), [renderCells]);\n            _this.renderBgEvents = core.memoizeRendering(fillRenderer.renderSegs.bind(fillRenderer, 'bgEvent'), fillRenderer.unrender.bind(fillRenderer, 'bgEvent'), [renderCells]);\n            _this.renderFgEvents = core.memoizeRendering(eventRenderer.renderSegs.bind(eventRenderer), eventRenderer.unrender.bind(eventRenderer), [renderCells]);\n            _this.renderEventSelection = core.memoizeRendering(eventRenderer.selectByInstanceId.bind(eventRenderer), eventRenderer.unselectByInstanceId.bind(eventRenderer), [_this.renderFgEvents]);\n            _this.renderEventDrag = core.memoizeRendering(_this._renderEventDrag, _this._unrenderEventDrag, [renderCells]);\n            _this.renderEventResize = core.memoizeRendering(_this._renderEventResize, _this._unrenderEventResize, [renderCells]);\n            return _this;\n        }\n        DayGrid.prototype.render = function (props, context) {\n            var cells = props.cells;\n            this.rowCnt = cells.length;\n            this.colCnt = cells[0].length;\n            this.renderCells(cells, props.isRigid);\n            this.renderBusinessHours(context, props.businessHourSegs);\n            this.renderDateSelection(context, props.dateSelectionSegs);\n            this.renderBgEvents(context, props.bgEventSegs);\n            this.renderFgEvents(context, props.fgEventSegs);\n            this.renderEventSelection(props.eventSelection);\n            this.renderEventDrag(props.eventDrag);\n            this.renderEventResize(props.eventResize);\n            if (this.segPopoverTile) {\n                this.updateSegPopoverTile();\n            }\n        };\n        DayGrid.prototype.destroy = function () {\n            _super.prototype.destroy.call(this);\n            this.renderCells.unrender(); // will unrender everything else\n        };\n        DayGrid.prototype.getCellRange = function (row, col) {\n            var start = this.props.cells[row][col].date;\n            var end = core.addDays(start, 1);\n            return { start: start, end: end };\n        };\n        DayGrid.prototype.updateSegPopoverTile = function (date, segs) {\n            var ownProps = this.props;\n            this.segPopoverTile.receiveProps({\n                date: date || this.segPopoverTile.props.date,\n                fgSegs: segs || this.segPopoverTile.props.fgSegs,\n                eventSelection: ownProps.eventSelection,\n                eventDragInstances: ownProps.eventDrag ? ownProps.eventDrag.affectedInstances : null,\n                eventResizeInstances: ownProps.eventResize ? ownProps.eventResize.affectedInstances : null\n            }, this.context);\n        };\n        /* Date Rendering\n        ------------------------------------------------------------------------------------------------------------------*/\n        DayGrid.prototype._renderCells = function (cells, isRigid) {\n            var _a = this.context, calendar = _a.calendar, view = _a.view, isRtl = _a.isRtl, dateEnv = _a.dateEnv;\n            var _b = this, rowCnt = _b.rowCnt, colCnt = _b.colCnt;\n            var html = '';\n            var row;\n            var col;\n            for (row = 0; row < rowCnt; row++) {\n                html += this.renderDayRowHtml(row, isRigid);\n            }\n            this.el.innerHTML = html;\n            this.rowEls = core.findElements(this.el, '.fc-row');\n            this.cellEls = core.findElements(this.el, '.fc-day, .fc-disabled-day');\n            if (isRtl) {\n                this.cellEls.reverse();\n            }\n            this.rowPositions = new core.PositionCache(this.el, this.rowEls, false, true // vertical\n            );\n            this.colPositions = new core.PositionCache(this.el, this.cellEls.slice(0, colCnt), // only the first row\n            true, false // horizontal\n            );\n            // trigger dayRender with each cell's element\n            for (row = 0; row < rowCnt; row++) {\n                for (col = 0; col < colCnt; col++) {\n                    calendar.publiclyTrigger('dayRender', [\n                        {\n                            date: dateEnv.toDate(cells[row][col].date),\n                            el: this.getCellEl(row, col),\n                            view: view\n                        }\n                    ]);\n                }\n            }\n            this.isCellSizesDirty = true;\n        };\n        DayGrid.prototype._unrenderCells = function () {\n            this.removeSegPopover();\n        };\n        // Generates the HTML for a single row, which is a div that wraps a table.\n        // `row` is the row number.\n        DayGrid.prototype.renderDayRowHtml = function (row, isRigid) {\n            var theme = this.context.theme;\n            var classes = ['fc-row', 'fc-week', theme.getClass('dayRow')];\n            if (isRigid) {\n                classes.push('fc-rigid');\n            }\n            var bgRow = new DayBgRow(this.context);\n            return '' +\n                '<div class=\"' + classes.join(' ') + '\">' +\n                '<div class=\"fc-bg\">' +\n                '<table class=\"' + theme.getClass('tableGrid') + '\">' +\n                bgRow.renderHtml({\n                    cells: this.props.cells[row],\n                    dateProfile: this.props.dateProfile,\n                    renderIntroHtml: this.renderProps.renderBgIntroHtml\n                }) +\n                '</table>' +\n                '</div>' +\n                '<div class=\"fc-content-skeleton\">' +\n                '<table>' +\n                (this.getIsNumbersVisible() ?\n                    '<thead>' +\n                        this.renderNumberTrHtml(row) +\n                        '</thead>' :\n                    '') +\n                '</table>' +\n                '</div>' +\n                '</div>';\n        };\n        DayGrid.prototype.getIsNumbersVisible = function () {\n            return this.getIsDayNumbersVisible() ||\n                this.renderProps.cellWeekNumbersVisible ||\n                this.renderProps.colWeekNumbersVisible;\n        };\n        DayGrid.prototype.getIsDayNumbersVisible = function () {\n            return this.rowCnt > 1;\n        };\n        /* Grid Number Rendering\n        ------------------------------------------------------------------------------------------------------------------*/\n        DayGrid.prototype.renderNumberTrHtml = function (row) {\n            var isRtl = this.context.isRtl;\n            var intro = this.renderProps.renderNumberIntroHtml(row, this);\n            return '' +\n                '<tr>' +\n                (isRtl ? '' : intro) +\n                this.renderNumberCellsHtml(row) +\n                (isRtl ? intro : '') +\n                '</tr>';\n        };\n        DayGrid.prototype.renderNumberCellsHtml = function (row) {\n            var htmls = [];\n            var col;\n            var date;\n            for (col = 0; col < this.colCnt; col++) {\n                date = this.props.cells[row][col].date;\n                htmls.push(this.renderNumberCellHtml(date));\n            }\n            if (this.context.isRtl) {\n                htmls.reverse();\n            }\n            return htmls.join('');\n        };\n        // Generates the HTML for the <td>s of the \"number\" row in the DayGrid's content skeleton.\n        // The number row will only exist if either day numbers or week numbers are turned on.\n        DayGrid.prototype.renderNumberCellHtml = function (date) {\n            var _a = this.context, dateEnv = _a.dateEnv, options = _a.options;\n            var html = '';\n            var isDateValid = core.rangeContainsMarker(this.props.dateProfile.activeRange, date); // TODO: called too frequently. cache somehow.\n            var isDayNumberVisible = this.getIsDayNumbersVisible() && isDateValid;\n            var classes;\n            var weekCalcFirstDow;\n            if (!isDayNumberVisible && !this.renderProps.cellWeekNumbersVisible) {\n                // no numbers in day cell (week number must be along the side)\n                return '<td></td>'; //  will create an empty space above events :(\n            }\n            classes = core.getDayClasses(date, this.props.dateProfile, this.context);\n            classes.unshift('fc-day-top');\n            if (this.renderProps.cellWeekNumbersVisible) {\n                weekCalcFirstDow = dateEnv.weekDow;\n            }\n            html += '<td class=\"' + classes.join(' ') + '\"' +\n                (isDateValid ?\n                    ' data-date=\"' + dateEnv.formatIso(date, { omitTime: true }) + '\"' :\n                    '') +\n                '>';\n            if (this.renderProps.cellWeekNumbersVisible && (date.getUTCDay() === weekCalcFirstDow)) {\n                html += core.buildGotoAnchorHtml(options, dateEnv, { date: date, type: 'week' }, { 'class': 'fc-week-number' }, dateEnv.format(date, WEEK_NUM_FORMAT) // inner HTML\n                );\n            }\n            if (isDayNumberVisible) {\n                html += core.buildGotoAnchorHtml(options, dateEnv, date, { 'class': 'fc-day-number' }, dateEnv.format(date, DAY_NUM_FORMAT) // inner HTML\n                );\n            }\n            html += '</td>';\n            return html;\n        };\n        /* Sizing\n        ------------------------------------------------------------------------------------------------------------------*/\n        DayGrid.prototype.updateSize = function (isResize) {\n            var calendar = this.context.calendar;\n            var _a = this, fillRenderer = _a.fillRenderer, eventRenderer = _a.eventRenderer, mirrorRenderer = _a.mirrorRenderer;\n            if (isResize ||\n                this.isCellSizesDirty ||\n                calendar.isEventsUpdated // hack\n            ) {\n                this.buildPositionCaches();\n                this.isCellSizesDirty = false;\n            }\n            fillRenderer.computeSizes(isResize);\n            eventRenderer.computeSizes(isResize);\n            mirrorRenderer.computeSizes(isResize);\n            fillRenderer.assignSizes(isResize);\n            eventRenderer.assignSizes(isResize);\n            mirrorRenderer.assignSizes(isResize);\n        };\n        DayGrid.prototype.buildPositionCaches = function () {\n            this.buildColPositions();\n            this.buildRowPositions();\n        };\n        DayGrid.prototype.buildColPositions = function () {\n            this.colPositions.build();\n        };\n        DayGrid.prototype.buildRowPositions = function () {\n            this.rowPositions.build();\n            this.rowPositions.bottoms[this.rowCnt - 1] += this.bottomCoordPadding; // hack\n        };\n        /* Hit System\n        ------------------------------------------------------------------------------------------------------------------*/\n        DayGrid.prototype.positionToHit = function (leftPosition, topPosition) {\n            var _a = this, colPositions = _a.colPositions, rowPositions = _a.rowPositions;\n            var col = colPositions.leftToIndex(leftPosition);\n            var row = rowPositions.topToIndex(topPosition);\n            if (row != null && col != null) {\n                return {\n                    row: row,\n                    col: col,\n                    dateSpan: {\n                        range: this.getCellRange(row, col),\n                        allDay: true\n                    },\n                    dayEl: this.getCellEl(row, col),\n                    relativeRect: {\n                        left: colPositions.lefts[col],\n                        right: colPositions.rights[col],\n                        top: rowPositions.tops[row],\n                        bottom: rowPositions.bottoms[row]\n                    }\n                };\n            }\n        };\n        /* Cell System\n        ------------------------------------------------------------------------------------------------------------------*/\n        // FYI: the first column is the leftmost column, regardless of date\n        DayGrid.prototype.getCellEl = function (row, col) {\n            return this.cellEls[row * this.colCnt + col];\n        };\n        /* Event Drag Visualization\n        ------------------------------------------------------------------------------------------------------------------*/\n        DayGrid.prototype._renderEventDrag = function (state) {\n            if (state) {\n                this.eventRenderer.hideByHash(state.affectedInstances);\n                this.fillRenderer.renderSegs('highlight', this.context, state.segs);\n            }\n        };\n        DayGrid.prototype._unrenderEventDrag = function (state) {\n            if (state) {\n                this.eventRenderer.showByHash(state.affectedInstances);\n                this.fillRenderer.unrender('highlight', this.context);\n            }\n        };\n        /* Event Resize Visualization\n        ------------------------------------------------------------------------------------------------------------------*/\n        DayGrid.prototype._renderEventResize = function (state) {\n            if (state) {\n                this.eventRenderer.hideByHash(state.affectedInstances);\n                this.fillRenderer.renderSegs('highlight', this.context, state.segs);\n                this.mirrorRenderer.renderSegs(this.context, state.segs, { isResizing: true, sourceSeg: state.sourceSeg });\n            }\n        };\n        DayGrid.prototype._unrenderEventResize = function (state) {\n            if (state) {\n                this.eventRenderer.showByHash(state.affectedInstances);\n                this.fillRenderer.unrender('highlight', this.context);\n                this.mirrorRenderer.unrender(this.context, state.segs, { isResizing: true, sourceSeg: state.sourceSeg });\n            }\n        };\n        /* More+ Link Popover\n        ------------------------------------------------------------------------------------------------------------------*/\n        DayGrid.prototype.removeSegPopover = function () {\n            if (this.segPopover) {\n                this.segPopover.hide(); // in handler, will call segPopover's removeElement\n            }\n        };\n        // Limits the number of \"levels\" (vertically stacking layers of events) for each row of the grid.\n        // `levelLimit` can be false (don't limit), a number, or true (should be computed).\n        DayGrid.prototype.limitRows = function (levelLimit) {\n            var rowStructs = this.eventRenderer.rowStructs || [];\n            var row; // row #\n            var rowLevelLimit;\n            for (row = 0; row < rowStructs.length; row++) {\n                this.unlimitRow(row);\n                if (!levelLimit) {\n                    rowLevelLimit = false;\n                }\n                else if (typeof levelLimit === 'number') {\n                    rowLevelLimit = levelLimit;\n                }\n                else {\n                    rowLevelLimit = this.computeRowLevelLimit(row);\n                }\n                if (rowLevelLimit !== false) {\n                    this.limitRow(row, rowLevelLimit);\n                }\n            }\n        };\n        // Computes the number of levels a row will accomodate without going outside its bounds.\n        // Assumes the row is \"rigid\" (maintains a constant height regardless of what is inside).\n        // `row` is the row number.\n        DayGrid.prototype.computeRowLevelLimit = function (row) {\n            var rowEl = this.rowEls[row]; // the containing \"fake\" row div\n            var rowBottom = rowEl.getBoundingClientRect().bottom; // relative to viewport!\n            var trEls = core.findChildren(this.eventRenderer.rowStructs[row].tbodyEl);\n            var i;\n            var trEl;\n            // Reveal one level <tr> at a time and stop when we find one out of bounds\n            for (i = 0; i < trEls.length; i++) {\n                trEl = trEls[i];\n                trEl.classList.remove('fc-limited'); // reset to original state (reveal)\n                if (trEl.getBoundingClientRect().bottom > rowBottom) {\n                    return i;\n                }\n            }\n            return false; // should not limit at all\n        };\n        // Limits the given grid row to the maximum number of levels and injects \"more\" links if necessary.\n        // `row` is the row number.\n        // `levelLimit` is a number for the maximum (inclusive) number of levels allowed.\n        DayGrid.prototype.limitRow = function (row, levelLimit) {\n            var _this = this;\n            var colCnt = this.colCnt;\n            var isRtl = this.context.isRtl;\n            var rowStruct = this.eventRenderer.rowStructs[row];\n            var moreNodes = []; // array of \"more\" <a> links and <td> DOM nodes\n            var col = 0; // col #, left-to-right (not chronologically)\n            var levelSegs; // array of segment objects in the last allowable level, ordered left-to-right\n            var cellMatrix; // a matrix (by level, then column) of all <td> elements in the row\n            var limitedNodes; // array of temporarily hidden level <tr> and segment <td> DOM nodes\n            var i;\n            var seg;\n            var segsBelow; // array of segment objects below `seg` in the current `col`\n            var totalSegsBelow; // total number of segments below `seg` in any of the columns `seg` occupies\n            var colSegsBelow; // array of segment arrays, below seg, one for each column (offset from segs's first column)\n            var td;\n            var rowSpan;\n            var segMoreNodes; // array of \"more\" <td> cells that will stand-in for the current seg's cell\n            var j;\n            var moreTd;\n            var moreWrap;\n            var moreLink;\n            // Iterates through empty level cells and places \"more\" links inside if need be\n            var emptyCellsUntil = function (endCol) {\n                while (col < endCol) {\n                    segsBelow = _this.getCellSegs(row, col, levelLimit);\n                    if (segsBelow.length) {\n                        td = cellMatrix[levelLimit - 1][col];\n                        moreLink = _this.renderMoreLink(row, col, segsBelow);\n                        moreWrap = core.createElement('div', null, moreLink);\n                        td.appendChild(moreWrap);\n                        moreNodes.push(moreWrap);\n                    }\n                    col++;\n                }\n            };\n            if (levelLimit && levelLimit < rowStruct.segLevels.length) { // is it actually over the limit?\n                levelSegs = rowStruct.segLevels[levelLimit - 1];\n                cellMatrix = rowStruct.cellMatrix;\n                limitedNodes = core.findChildren(rowStruct.tbodyEl).slice(levelLimit); // get level <tr> elements past the limit\n                limitedNodes.forEach(function (node) {\n                    node.classList.add('fc-limited'); // hide elements and get a simple DOM-nodes array\n                });\n                // iterate though segments in the last allowable level\n                for (i = 0; i < levelSegs.length; i++) {\n                    seg = levelSegs[i];\n                    var leftCol = isRtl ? (colCnt - 1 - seg.lastCol) : seg.firstCol;\n                    var rightCol = isRtl ? (colCnt - 1 - seg.firstCol) : seg.lastCol;\n                    emptyCellsUntil(leftCol); // process empty cells before the segment\n                    // determine *all* segments below `seg` that occupy the same columns\n                    colSegsBelow = [];\n                    totalSegsBelow = 0;\n                    while (col <= rightCol) {\n                        segsBelow = this.getCellSegs(row, col, levelLimit);\n                        colSegsBelow.push(segsBelow);\n                        totalSegsBelow += segsBelow.length;\n                        col++;\n                    }\n                    if (totalSegsBelow) { // do we need to replace this segment with one or many \"more\" links?\n                        td = cellMatrix[levelLimit - 1][leftCol]; // the segment's parent cell\n                        rowSpan = td.rowSpan || 1;\n                        segMoreNodes = [];\n                        // make a replacement <td> for each column the segment occupies. will be one for each colspan\n                        for (j = 0; j < colSegsBelow.length; j++) {\n                            moreTd = core.createElement('td', { className: 'fc-more-cell', rowSpan: rowSpan });\n                            segsBelow = colSegsBelow[j];\n                            moreLink = this.renderMoreLink(row, leftCol + j, [seg].concat(segsBelow) // count seg as hidden too\n                            );\n                            moreWrap = core.createElement('div', null, moreLink);\n                            moreTd.appendChild(moreWrap);\n                            segMoreNodes.push(moreTd);\n                            moreNodes.push(moreTd);\n                        }\n                        td.classList.add('fc-limited');\n                        core.insertAfterElement(td, segMoreNodes);\n                        limitedNodes.push(td);\n                    }\n                }\n                emptyCellsUntil(this.colCnt); // finish off the level\n                rowStruct.moreEls = moreNodes; // for easy undoing later\n                rowStruct.limitedEls = limitedNodes; // for easy undoing later\n            }\n        };\n        // Reveals all levels and removes all \"more\"-related elements for a grid's row.\n        // `row` is a row number.\n        DayGrid.prototype.unlimitRow = function (row) {\n            var rowStruct = this.eventRenderer.rowStructs[row];\n            if (rowStruct.moreEls) {\n                rowStruct.moreEls.forEach(core.removeElement);\n                rowStruct.moreEls = null;\n            }\n            if (rowStruct.limitedEls) {\n                rowStruct.limitedEls.forEach(function (limitedEl) {\n                    limitedEl.classList.remove('fc-limited');\n                });\n                rowStruct.limitedEls = null;\n            }\n        };\n        // Renders an <a> element that represents hidden event element for a cell.\n        // Responsible for attaching click handler as well.\n        DayGrid.prototype.renderMoreLink = function (row, col, hiddenSegs) {\n            var _this = this;\n            var _a = this.context, calendar = _a.calendar, view = _a.view, dateEnv = _a.dateEnv, options = _a.options, isRtl = _a.isRtl;\n            var a = core.createElement('a', { className: 'fc-more' });\n            a.innerText = this.getMoreLinkText(hiddenSegs.length);\n            a.addEventListener('click', function (ev) {\n                var clickOption = options.eventLimitClick;\n                var _col = isRtl ? _this.colCnt - col - 1 : col; // HACK: props.cells has different dir system?\n                var date = _this.props.cells[row][_col].date;\n                var moreEl = ev.currentTarget;\n                var dayEl = _this.getCellEl(row, col);\n                var allSegs = _this.getCellSegs(row, col);\n                // rescope the segments to be within the cell's date\n                var reslicedAllSegs = _this.resliceDaySegs(allSegs, date);\n                var reslicedHiddenSegs = _this.resliceDaySegs(hiddenSegs, date);\n                if (typeof clickOption === 'function') {\n                    // the returned value can be an atomic option\n                    clickOption = calendar.publiclyTrigger('eventLimitClick', [\n                        {\n                            date: dateEnv.toDate(date),\n                            allDay: true,\n                            dayEl: dayEl,\n                            moreEl: moreEl,\n                            segs: reslicedAllSegs,\n                            hiddenSegs: reslicedHiddenSegs,\n                            jsEvent: ev,\n                            view: view\n                        }\n                    ]);\n                }\n                if (clickOption === 'popover') {\n                    _this.showSegPopover(row, col, moreEl, reslicedAllSegs);\n                }\n                else if (typeof clickOption === 'string') { // a view name\n                    calendar.zoomTo(date, clickOption);\n                }\n            });\n            return a;\n        };\n        // Reveals the popover that displays all events within a cell\n        DayGrid.prototype.showSegPopover = function (row, col, moreLink, segs) {\n            var _this = this;\n            var _a = this.context, calendar = _a.calendar, view = _a.view, theme = _a.theme, isRtl = _a.isRtl;\n            var _col = isRtl ? this.colCnt - col - 1 : col; // HACK: props.cells has different dir system?\n            var moreWrap = moreLink.parentNode; // the <div> wrapper around the <a>\n            var topEl; // the element we want to match the top coordinate of\n            var options;\n            if (this.rowCnt === 1) {\n                topEl = view.el; // will cause the popover to cover any sort of header\n            }\n            else {\n                topEl = this.rowEls[row]; // will align with top of row\n            }\n            options = {\n                className: 'fc-more-popover ' + theme.getClass('popover'),\n                parentEl: view.el,\n                top: core.computeRect(topEl).top,\n                autoHide: true,\n                content: function (el) {\n                    _this.segPopoverTile = new DayTile(el);\n                    _this.updateSegPopoverTile(_this.props.cells[row][_col].date, segs);\n                },\n                hide: function () {\n                    _this.segPopoverTile.destroy();\n                    _this.segPopoverTile = null;\n                    _this.segPopover.destroy();\n                    _this.segPopover = null;\n                }\n            };\n            // Determine horizontal coordinate.\n            // We use the moreWrap instead of the <td> to avoid border confusion.\n            if (isRtl) {\n                options.right = core.computeRect(moreWrap).right + 1; // +1 to be over cell border\n            }\n            else {\n                options.left = core.computeRect(moreWrap).left - 1; // -1 to be over cell border\n            }\n            this.segPopover = new Popover(options);\n            this.segPopover.show();\n            calendar.releaseAfterSizingTriggers(); // hack for eventPositioned\n        };\n        // Given the events within an array of segment objects, reslice them to be in a single day\n        DayGrid.prototype.resliceDaySegs = function (segs, dayDate) {\n            var dayStart = dayDate;\n            var dayEnd = core.addDays(dayStart, 1);\n            var dayRange = { start: dayStart, end: dayEnd };\n            var newSegs = [];\n            for (var _i = 0, segs_1 = segs; _i < segs_1.length; _i++) {\n                var seg = segs_1[_i];\n                var eventRange = seg.eventRange;\n                var origRange = eventRange.range;\n                var slicedRange = core.intersectRanges(origRange, dayRange);\n                if (slicedRange) {\n                    newSegs.push(__assign({}, seg, { eventRange: {\n                            def: eventRange.def,\n                            ui: __assign({}, eventRange.ui, { durationEditable: false }),\n                            instance: eventRange.instance,\n                            range: slicedRange\n                        }, isStart: seg.isStart && slicedRange.start.valueOf() === origRange.start.valueOf(), isEnd: seg.isEnd && slicedRange.end.valueOf() === origRange.end.valueOf() }));\n                }\n            }\n            return newSegs;\n        };\n        // Generates the text that should be inside a \"more\" link, given the number of events it represents\n        DayGrid.prototype.getMoreLinkText = function (num) {\n            var opt = this.context.options.eventLimitText;\n            if (typeof opt === 'function') {\n                return opt(num);\n            }\n            else {\n                return '+' + num + ' ' + opt;\n            }\n        };\n        // Returns segments within a given cell.\n        // If `startLevel` is specified, returns only events including and below that level. Otherwise returns all segs.\n        DayGrid.prototype.getCellSegs = function (row, col, startLevel) {\n            var segMatrix = this.eventRenderer.rowStructs[row].segMatrix;\n            var level = startLevel || 0;\n            var segs = [];\n            var seg;\n            while (level < segMatrix.length) {\n                seg = segMatrix[level][col];\n                if (seg) {\n                    segs.push(seg);\n                }\n                level++;\n            }\n            return segs;\n        };\n        return DayGrid;\n    }(core.DateComponent));\n\n    var WEEK_NUM_FORMAT$1 = core.createFormatter({ week: 'numeric' });\n    /* An abstract class for the daygrid views, as well as month view. Renders one or more rows of day cells.\n    ----------------------------------------------------------------------------------------------------------------------*/\n    // It is a manager for a DayGrid subcomponent, which does most of the heavy lifting.\n    // It is responsible for managing width/height.\n    var AbstractDayGridView = /** @class */ (function (_super) {\n        __extends(AbstractDayGridView, _super);\n        function AbstractDayGridView() {\n            var _this = _super !== null && _super.apply(this, arguments) || this;\n            _this.processOptions = core.memoize(_this._processOptions);\n            _this.renderSkeleton = core.memoizeRendering(_this._renderSkeleton, _this._unrenderSkeleton);\n            /* Header Rendering\n            ------------------------------------------------------------------------------------------------------------------*/\n            // Generates the HTML that will go before the day-of week header cells\n            _this.renderHeadIntroHtml = function () {\n                var _a = _this.context, theme = _a.theme, options = _a.options;\n                if (_this.colWeekNumbersVisible) {\n                    return '' +\n                        '<th class=\"fc-week-number ' + theme.getClass('widgetHeader') + '\" ' + _this.weekNumberStyleAttr() + '>' +\n                        '<span>' + // needed for matchCellWidths\n                        core.htmlEscape(options.weekLabel) +\n                        '</span>' +\n                        '</th>';\n                }\n                return '';\n            };\n            /* Day Grid Rendering\n            ------------------------------------------------------------------------------------------------------------------*/\n            // Generates the HTML that will go before content-skeleton cells that display the day/week numbers\n            _this.renderDayGridNumberIntroHtml = function (row, dayGrid) {\n                var _a = _this.context, options = _a.options, dateEnv = _a.dateEnv;\n                var weekStart = dayGrid.props.cells[row][0].date;\n                if (_this.colWeekNumbersVisible) {\n                    return '' +\n                        '<td class=\"fc-week-number\" ' + _this.weekNumberStyleAttr() + '>' +\n                        core.buildGotoAnchorHtml(// aside from link, important for matchCellWidths\n                        options, dateEnv, { date: weekStart, type: 'week', forceOff: dayGrid.colCnt === 1 }, dateEnv.format(weekStart, WEEK_NUM_FORMAT$1) // inner HTML\n                        ) +\n                        '</td>';\n                }\n                return '';\n            };\n            // Generates the HTML that goes before the day bg cells for each day-row\n            _this.renderDayGridBgIntroHtml = function () {\n                var theme = _this.context.theme;\n                if (_this.colWeekNumbersVisible) {\n                    return '<td class=\"fc-week-number ' + theme.getClass('widgetContent') + '\" ' + _this.weekNumberStyleAttr() + '></td>';\n                }\n                return '';\n            };\n            // Generates the HTML that goes before every other type of row generated by DayGrid.\n            // Affects mirror-skeleton and highlight-skeleton rows.\n            _this.renderDayGridIntroHtml = function () {\n                if (_this.colWeekNumbersVisible) {\n                    return '<td class=\"fc-week-number\" ' + _this.weekNumberStyleAttr() + '></td>';\n                }\n                return '';\n            };\n            return _this;\n        }\n        AbstractDayGridView.prototype._processOptions = function (options) {\n            if (options.weekNumbers) {\n                if (options.weekNumbersWithinDays) {\n                    this.cellWeekNumbersVisible = true;\n                    this.colWeekNumbersVisible = false;\n                }\n                else {\n                    this.cellWeekNumbersVisible = false;\n                    this.colWeekNumbersVisible = true;\n                }\n            }\n            else {\n                this.colWeekNumbersVisible = false;\n                this.cellWeekNumbersVisible = false;\n            }\n        };\n        AbstractDayGridView.prototype.render = function (props, context) {\n            _super.prototype.render.call(this, props, context);\n            this.processOptions(context.options);\n            this.renderSkeleton(context);\n        };\n        AbstractDayGridView.prototype.destroy = function () {\n            _super.prototype.destroy.call(this);\n            this.renderSkeleton.unrender();\n        };\n        AbstractDayGridView.prototype._renderSkeleton = function (context) {\n            this.el.classList.add('fc-dayGrid-view');\n            this.el.innerHTML = this.renderSkeletonHtml();\n            this.scroller = new core.ScrollComponent('hidden', // overflow x\n            'auto' // overflow y\n            );\n            var dayGridContainerEl = this.scroller.el;\n            this.el.querySelector('.fc-body > tr > td').appendChild(dayGridContainerEl);\n            dayGridContainerEl.classList.add('fc-day-grid-container');\n            var dayGridEl = core.createElement('div', { className: 'fc-day-grid' });\n            dayGridContainerEl.appendChild(dayGridEl);\n            this.dayGrid = new DayGrid(dayGridEl, {\n                renderNumberIntroHtml: this.renderDayGridNumberIntroHtml,\n                renderBgIntroHtml: this.renderDayGridBgIntroHtml,\n                renderIntroHtml: this.renderDayGridIntroHtml,\n                colWeekNumbersVisible: this.colWeekNumbersVisible,\n                cellWeekNumbersVisible: this.cellWeekNumbersVisible\n            });\n        };\n        AbstractDayGridView.prototype._unrenderSkeleton = function () {\n            this.el.classList.remove('fc-dayGrid-view');\n            this.dayGrid.destroy();\n            this.scroller.destroy();\n        };\n        // Builds the HTML skeleton for the view.\n        // The day-grid component will render inside of a container defined by this HTML.\n        AbstractDayGridView.prototype.renderSkeletonHtml = function () {\n            var _a = this.context, theme = _a.theme, options = _a.options;\n            return '' +\n                '<table class=\"' + theme.getClass('tableGrid') + '\">' +\n                (options.columnHeader ?\n                    '<thead class=\"fc-head\">' +\n                        '<tr>' +\n                        '<td class=\"fc-head-container ' + theme.getClass('widgetHeader') + '\">&nbsp;</td>' +\n                        '</tr>' +\n                        '</thead>' :\n                    '') +\n                '<tbody class=\"fc-body\">' +\n                '<tr>' +\n                '<td class=\"' + theme.getClass('widgetContent') + '\"></td>' +\n                '</tr>' +\n                '</tbody>' +\n                '</table>';\n        };\n        // Generates an HTML attribute string for setting the width of the week number column, if it is known\n        AbstractDayGridView.prototype.weekNumberStyleAttr = function () {\n            if (this.weekNumberWidth != null) {\n                return 'style=\"width:' + this.weekNumberWidth + 'px\"';\n            }\n            return '';\n        };\n        // Determines whether each row should have a constant height\n        AbstractDayGridView.prototype.hasRigidRows = function () {\n            var eventLimit = this.context.options.eventLimit;\n            return eventLimit && typeof eventLimit !== 'number';\n        };\n        /* Dimensions\n        ------------------------------------------------------------------------------------------------------------------*/\n        AbstractDayGridView.prototype.updateSize = function (isResize, viewHeight, isAuto) {\n            _super.prototype.updateSize.call(this, isResize, viewHeight, isAuto); // will call updateBaseSize. important that executes first\n            this.dayGrid.updateSize(isResize);\n        };\n        // Refreshes the horizontal dimensions of the view\n        AbstractDayGridView.prototype.updateBaseSize = function (isResize, viewHeight, isAuto) {\n            var dayGrid = this.dayGrid;\n            var eventLimit = this.context.options.eventLimit;\n            var headRowEl = this.header ? this.header.el : null; // HACK\n            var scrollerHeight;\n            var scrollbarWidths;\n            // hack to give the view some height prior to dayGrid's columns being rendered\n            // TODO: separate setting height from scroller VS dayGrid.\n            if (!dayGrid.rowEls) {\n                if (!isAuto) {\n                    scrollerHeight = this.computeScrollerHeight(viewHeight);\n                    this.scroller.setHeight(scrollerHeight);\n                }\n                return;\n            }\n            if (this.colWeekNumbersVisible) {\n                // Make sure all week number cells running down the side have the same width.\n                this.weekNumberWidth = core.matchCellWidths(core.findElements(this.el, '.fc-week-number'));\n            }\n            // reset all heights to be natural\n            this.scroller.clear();\n            if (headRowEl) {\n                core.uncompensateScroll(headRowEl);\n            }\n            dayGrid.removeSegPopover(); // kill the \"more\" popover if displayed\n            // is the event limit a constant level number?\n            if (eventLimit && typeof eventLimit === 'number') {\n                dayGrid.limitRows(eventLimit); // limit the levels first so the height can redistribute after\n            }\n            // distribute the height to the rows\n            // (viewHeight is a \"recommended\" value if isAuto)\n            scrollerHeight = this.computeScrollerHeight(viewHeight);\n            this.setGridHeight(scrollerHeight, isAuto);\n            // is the event limit dynamically calculated?\n            if (eventLimit && typeof eventLimit !== 'number') {\n                dayGrid.limitRows(eventLimit); // limit the levels after the grid's row heights have been set\n            }\n            if (!isAuto) { // should we force dimensions of the scroll container?\n                this.scroller.setHeight(scrollerHeight);\n                scrollbarWidths = this.scroller.getScrollbarWidths();\n                if (scrollbarWidths.left || scrollbarWidths.right) { // using scrollbars?\n                    if (headRowEl) {\n                        core.compensateScroll(headRowEl, scrollbarWidths);\n                    }\n                    // doing the scrollbar compensation might have created text overflow which created more height. redo\n                    scrollerHeight = this.computeScrollerHeight(viewHeight);\n                    this.scroller.setHeight(scrollerHeight);\n                }\n                // guarantees the same scrollbar widths\n                this.scroller.lockOverflow(scrollbarWidths);\n            }\n        };\n        // given a desired total height of the view, returns what the height of the scroller should be\n        AbstractDayGridView.prototype.computeScrollerHeight = function (viewHeight) {\n            return viewHeight -\n                core.subtractInnerElHeight(this.el, this.scroller.el); // everything that's NOT the scroller\n        };\n        // Sets the height of just the DayGrid component in this view\n        AbstractDayGridView.prototype.setGridHeight = function (height, isAuto) {\n            if (this.context.options.monthMode) {\n                // if auto, make the height of each row the height that it would be if there were 6 weeks\n                if (isAuto) {\n                    height *= this.dayGrid.rowCnt / 6;\n                }\n                core.distributeHeight(this.dayGrid.rowEls, height, !isAuto); // if auto, don't compensate for height-hogging rows\n            }\n            else {\n                if (isAuto) {\n                    core.undistributeHeight(this.dayGrid.rowEls); // let the rows be their natural height with no expanding\n                }\n                else {\n                    core.distributeHeight(this.dayGrid.rowEls, height, true); // true = compensate for height-hogging rows\n                }\n            }\n        };\n        /* Scroll\n        ------------------------------------------------------------------------------------------------------------------*/\n        AbstractDayGridView.prototype.computeDateScroll = function (duration) {\n            return { top: 0 };\n        };\n        AbstractDayGridView.prototype.queryDateScroll = function () {\n            return { top: this.scroller.getScrollTop() };\n        };\n        AbstractDayGridView.prototype.applyDateScroll = function (scroll) {\n            if (scroll.top !== undefined) {\n                this.scroller.setScrollTop(scroll.top);\n            }\n        };\n        return AbstractDayGridView;\n    }(core.View));\n    AbstractDayGridView.prototype.dateProfileGeneratorClass = DayGridDateProfileGenerator;\n\n    var SimpleDayGrid = /** @class */ (function (_super) {\n        __extends(SimpleDayGrid, _super);\n        function SimpleDayGrid(dayGrid) {\n            var _this = _super.call(this, dayGrid.el) || this;\n            _this.slicer = new DayGridSlicer();\n            _this.dayGrid = dayGrid;\n            return _this;\n        }\n        SimpleDayGrid.prototype.firstContext = function (context) {\n            context.calendar.registerInteractiveComponent(this, { el: this.dayGrid.el });\n        };\n        SimpleDayGrid.prototype.destroy = function () {\n            _super.prototype.destroy.call(this);\n            this.context.calendar.unregisterInteractiveComponent(this);\n        };\n        SimpleDayGrid.prototype.render = function (props, context) {\n            var dayGrid = this.dayGrid;\n            var dateProfile = props.dateProfile, dayTable = props.dayTable;\n            dayGrid.receiveContext(context); // hack because context is used in sliceProps\n            dayGrid.receiveProps(__assign({}, this.slicer.sliceProps(props, dateProfile, props.nextDayThreshold, context.calendar, dayGrid, dayTable), { dateProfile: dateProfile, cells: dayTable.cells, isRigid: props.isRigid }), context);\n        };\n        SimpleDayGrid.prototype.buildPositionCaches = function () {\n            this.dayGrid.buildPositionCaches();\n        };\n        SimpleDayGrid.prototype.queryHit = function (positionLeft, positionTop) {\n            var rawHit = this.dayGrid.positionToHit(positionLeft, positionTop);\n            if (rawHit) {\n                return {\n                    component: this.dayGrid,\n                    dateSpan: rawHit.dateSpan,\n                    dayEl: rawHit.dayEl,\n                    rect: {\n                        left: rawHit.relativeRect.left,\n                        right: rawHit.relativeRect.right,\n                        top: rawHit.relativeRect.top,\n                        bottom: rawHit.relativeRect.bottom\n                    },\n                    layer: 0\n                };\n            }\n        };\n        return SimpleDayGrid;\n    }(core.DateComponent));\n    var DayGridSlicer = /** @class */ (function (_super) {\n        __extends(DayGridSlicer, _super);\n        function DayGridSlicer() {\n            return _super !== null && _super.apply(this, arguments) || this;\n        }\n        DayGridSlicer.prototype.sliceRange = function (dateRange, dayTable) {\n            return dayTable.sliceRange(dateRange);\n        };\n        return DayGridSlicer;\n    }(core.Slicer));\n\n    var DayGridView = /** @class */ (function (_super) {\n        __extends(DayGridView, _super);\n        function DayGridView() {\n            var _this = _super !== null && _super.apply(this, arguments) || this;\n            _this.buildDayTable = core.memoize(buildDayTable);\n            return _this;\n        }\n        DayGridView.prototype.render = function (props, context) {\n            _super.prototype.render.call(this, props, context); // will call _renderSkeleton/_unrenderSkeleton\n            var dateProfile = this.props.dateProfile;\n            var dayTable = this.dayTable =\n                this.buildDayTable(dateProfile, props.dateProfileGenerator);\n            if (this.header) {\n                this.header.receiveProps({\n                    dateProfile: dateProfile,\n                    dates: dayTable.headerDates,\n                    datesRepDistinctDays: dayTable.rowCnt === 1,\n                    renderIntroHtml: this.renderHeadIntroHtml\n                }, context);\n            }\n            this.simpleDayGrid.receiveProps({\n                dateProfile: dateProfile,\n                dayTable: dayTable,\n                businessHours: props.businessHours,\n                dateSelection: props.dateSelection,\n                eventStore: props.eventStore,\n                eventUiBases: props.eventUiBases,\n                eventSelection: props.eventSelection,\n                eventDrag: props.eventDrag,\n                eventResize: props.eventResize,\n                isRigid: this.hasRigidRows(),\n                nextDayThreshold: this.context.nextDayThreshold\n            }, context);\n        };\n        DayGridView.prototype._renderSkeleton = function (context) {\n            _super.prototype._renderSkeleton.call(this, context);\n            if (context.options.columnHeader) {\n                this.header = new core.DayHeader(this.el.querySelector('.fc-head-container'));\n            }\n            this.simpleDayGrid = new SimpleDayGrid(this.dayGrid);\n        };\n        DayGridView.prototype._unrenderSkeleton = function () {\n            _super.prototype._unrenderSkeleton.call(this);\n            if (this.header) {\n                this.header.destroy();\n            }\n            this.simpleDayGrid.destroy();\n        };\n        return DayGridView;\n    }(AbstractDayGridView));\n    function buildDayTable(dateProfile, dateProfileGenerator) {\n        var daySeries = new core.DaySeries(dateProfile.renderRange, dateProfileGenerator);\n        return new core.DayTable(daySeries, /year|month|week/.test(dateProfile.currentRangeUnit));\n    }\n\n    var main = core.createPlugin({\n        defaultView: 'dayGridMonth',\n        views: {\n            dayGrid: DayGridView,\n            dayGridDay: {\n                type: 'dayGrid',\n                duration: { days: 1 }\n            },\n            dayGridWeek: {\n                type: 'dayGrid',\n                duration: { weeks: 1 }\n            },\n            dayGridMonth: {\n                type: 'dayGrid',\n                duration: { months: 1 },\n                monthMode: true,\n                fixedWeekCount: true\n            }\n        }\n    });\n\n    exports.AbstractDayGridView = AbstractDayGridView;\n    exports.DayBgRow = DayBgRow;\n    exports.DayGrid = DayGrid;\n    exports.DayGridSlicer = DayGridSlicer;\n    exports.DayGridView = DayGridView;\n    exports.SimpleDayGrid = SimpleDayGrid;\n    exports.buildBasicDayTable = buildDayTable;\n    exports.default = main;\n\n    Object.defineProperty(exports, '__esModule', { value: true });\n\n}));\n"

/***/ },

/***/ 1087
(module) {

module.exports = "/*!\nFullCalendar Interaction Plugin v4.4.2\nDocs & License: https://fullcalendar.io/\n(c) 2019 Adam Shaw\n*/\n\n(function (global, factory) {\n    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@fullcalendar/core')) :\n    typeof define === 'function' && define.amd ? define(['exports', '@fullcalendar/core'], factory) :\n    (global = global || self, factory(global.FullCalendarInteraction = {}, global.FullCalendar));\n}(this, function (exports, core) { 'use strict';\n\n    /*! *****************************************************************************\r\n    Copyright (c) Microsoft Corporation.\r\n\r\n    Permission to use, copy, modify, and/or distribute this software for any\r\n    purpose with or without fee is hereby granted.\r\n\r\n    THE SOFTWARE IS PROVIDED \"AS IS\" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH\r\n    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY\r\n    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,\r\n    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM\r\n    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR\r\n    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR\r\n    PERFORMANCE OF THIS SOFTWARE.\r\n    ***************************************************************************** */\r\n    /* global Reflect, Promise */\r\n\r\n    var extendStatics = function(d, b) {\r\n        extendStatics = Object.setPrototypeOf ||\r\n            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||\r\n            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };\r\n        return extendStatics(d, b);\r\n    };\r\n\r\n    function __extends(d, b) {\r\n        extendStatics(d, b);\r\n        function __() { this.constructor = d; }\r\n        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());\r\n    }\r\n\r\n    var __assign = function() {\r\n        __assign = Object.assign || function __assign(t) {\r\n            for (var s, i = 1, n = arguments.length; i < n; i++) {\r\n                s = arguments[i];\r\n                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];\r\n            }\r\n            return t;\r\n        };\r\n        return __assign.apply(this, arguments);\r\n    };\n\n    core.config.touchMouseIgnoreWait = 500;\n    var ignoreMouseDepth = 0;\n    var listenerCnt = 0;\n    var isWindowTouchMoveCancelled = false;\n    /*\n    Uses a \"pointer\" abstraction, which monitors UI events for both mouse and touch.\n    Tracks when the pointer \"drags\" on a certain element, meaning down+move+up.\n\n    Also, tracks if there was touch-scrolling.\n    Also, can prevent touch-scrolling from happening.\n    Also, can fire pointermove events when scrolling happens underneath, even when no real pointer movement.\n\n    emits:\n    - pointerdown\n    - pointermove\n    - pointerup\n    */\n    var PointerDragging = /** @class */ (function () {\n        function PointerDragging(containerEl) {\n            var _this = this;\n            this.subjectEl = null;\n            this.downEl = null;\n            // options that can be directly assigned by caller\n            this.selector = ''; // will cause subjectEl in all emitted events to be this element\n            this.handleSelector = '';\n            this.shouldIgnoreMove = false;\n            this.shouldWatchScroll = true; // for simulating pointermove on scroll\n            // internal states\n            this.isDragging = false;\n            this.isTouchDragging = false;\n            this.wasTouchScroll = false;\n            // Mouse\n            // ----------------------------------------------------------------------------------------------------\n            this.handleMouseDown = function (ev) {\n                if (!_this.shouldIgnoreMouse() &&\n                    isPrimaryMouseButton(ev) &&\n                    _this.tryStart(ev)) {\n                    var pev = _this.createEventFromMouse(ev, true);\n                    _this.emitter.trigger('pointerdown', pev);\n                    _this.initScrollWatch(pev);\n                    if (!_this.shouldIgnoreMove) {\n                        document.addEventListener('mousemove', _this.handleMouseMove);\n                    }\n                    document.addEventListener('mouseup', _this.handleMouseUp);\n                }\n            };\n            this.handleMouseMove = function (ev) {\n                var pev = _this.createEventFromMouse(ev);\n                _this.recordCoords(pev);\n                _this.emitter.trigger('pointermove', pev);\n            };\n            this.handleMouseUp = function (ev) {\n                document.removeEventListener('mousemove', _this.handleMouseMove);\n                document.removeEventListener('mouseup', _this.handleMouseUp);\n                _this.emitter.trigger('pointerup', _this.createEventFromMouse(ev));\n                _this.cleanup(); // call last so that pointerup has access to props\n            };\n            // Touch\n            // ----------------------------------------------------------------------------------------------------\n            this.handleTouchStart = function (ev) {\n                if (_this.tryStart(ev)) {\n                    _this.isTouchDragging = true;\n                    var pev = _this.createEventFromTouch(ev, true);\n                    _this.emitter.trigger('pointerdown', pev);\n                    _this.initScrollWatch(pev);\n                    // unlike mouse, need to attach to target, not document\n                    // https://stackoverflow.com/a/45760014\n                    var target = ev.target;\n                    if (!_this.shouldIgnoreMove) {\n                        target.addEventListener('touchmove', _this.handleTouchMove);\n                    }\n                    target.addEventListener('touchend', _this.handleTouchEnd);\n                    target.addEventListener('touchcancel', _this.handleTouchEnd); // treat it as a touch end\n                    // attach a handler to get called when ANY scroll action happens on the page.\n                    // this was impossible to do with normal on/off because 'scroll' doesn't bubble.\n                    // http://stackoverflow.com/a/32954565/96342\n                    window.addEventListener('scroll', _this.handleTouchScroll, true // useCapture\n                    );\n                }\n            };\n            this.handleTouchMove = function (ev) {\n                var pev = _this.createEventFromTouch(ev);\n                _this.recordCoords(pev);\n                _this.emitter.trigger('pointermove', pev);\n            };\n            this.handleTouchEnd = function (ev) {\n                if (_this.isDragging) { // done to guard against touchend followed by touchcancel\n                    var target = ev.target;\n                    target.removeEventListener('touchmove', _this.handleTouchMove);\n                    target.removeEventListener('touchend', _this.handleTouchEnd);\n                    target.removeEventListener('touchcancel', _this.handleTouchEnd);\n                    window.removeEventListener('scroll', _this.handleTouchScroll, true); // useCaptured=true\n                    _this.emitter.trigger('pointerup', _this.createEventFromTouch(ev));\n                    _this.cleanup(); // call last so that pointerup has access to props\n                    _this.isTouchDragging = false;\n                    startIgnoringMouse();\n                }\n            };\n            this.handleTouchScroll = function () {\n                _this.wasTouchScroll = true;\n            };\n            this.handleScroll = function (ev) {\n                if (!_this.shouldIgnoreMove) {\n                    var pageX = (window.pageXOffset - _this.prevScrollX) + _this.prevPageX;\n                    var pageY = (window.pageYOffset - _this.prevScrollY) + _this.prevPageY;\n                    _this.emitter.trigger('pointermove', {\n                        origEvent: ev,\n                        isTouch: _this.isTouchDragging,\n                        subjectEl: _this.subjectEl,\n                        pageX: pageX,\n                        pageY: pageY,\n                        deltaX: pageX - _this.origPageX,\n                        deltaY: pageY - _this.origPageY\n                    });\n                }\n            };\n            this.containerEl = containerEl;\n            this.emitter = new core.EmitterMixin();\n            containerEl.addEventListener('mousedown', this.handleMouseDown);\n            containerEl.addEventListener('touchstart', this.handleTouchStart, { passive: true });\n            listenerCreated();\n        }\n        PointerDragging.prototype.destroy = function () {\n            this.containerEl.removeEventListener('mousedown', this.handleMouseDown);\n            this.containerEl.removeEventListener('touchstart', this.handleTouchStart, { passive: true });\n            listenerDestroyed();\n        };\n        PointerDragging.prototype.tryStart = function (ev) {\n            var subjectEl = this.querySubjectEl(ev);\n            var downEl = ev.target;\n            if (subjectEl &&\n                (!this.handleSelector || core.elementClosest(downEl, this.handleSelector))) {\n                this.subjectEl = subjectEl;\n                this.downEl = downEl;\n                this.isDragging = true; // do this first so cancelTouchScroll will work\n                this.wasTouchScroll = false;\n                return true;\n            }\n            return false;\n        };\n        PointerDragging.prototype.cleanup = function () {\n            isWindowTouchMoveCancelled = false;\n            this.isDragging = false;\n            this.subjectEl = null;\n            this.downEl = null;\n            // keep wasTouchScroll around for later access\n            this.destroyScrollWatch();\n        };\n        PointerDragging.prototype.querySubjectEl = function (ev) {\n            if (this.selector) {\n                return core.elementClosest(ev.target, this.selector);\n            }\n            else {\n                return this.containerEl;\n            }\n        };\n        PointerDragging.prototype.shouldIgnoreMouse = function () {\n            return ignoreMouseDepth || this.isTouchDragging;\n        };\n        // can be called by user of this class, to cancel touch-based scrolling for the current drag\n        PointerDragging.prototype.cancelTouchScroll = function () {\n            if (this.isDragging) {\n                isWindowTouchMoveCancelled = true;\n            }\n        };\n        // Scrolling that simulates pointermoves\n        // ----------------------------------------------------------------------------------------------------\n        PointerDragging.prototype.initScrollWatch = function (ev) {\n            if (this.shouldWatchScroll) {\n                this.recordCoords(ev);\n                window.addEventListener('scroll', this.handleScroll, true); // useCapture=true\n            }\n        };\n        PointerDragging.prototype.recordCoords = function (ev) {\n            if (this.shouldWatchScroll) {\n                this.prevPageX = ev.pageX;\n                this.prevPageY = ev.pageY;\n                this.prevScrollX = window.pageXOffset;\n                this.prevScrollY = window.pageYOffset;\n            }\n        };\n        PointerDragging.prototype.destroyScrollWatch = function () {\n            if (this.shouldWatchScroll) {\n                window.removeEventListener('scroll', this.handleScroll, true); // useCaptured=true\n            }\n        };\n        // Event Normalization\n        // ----------------------------------------------------------------------------------------------------\n        PointerDragging.prototype.createEventFromMouse = function (ev, isFirst) {\n            var deltaX = 0;\n            var deltaY = 0;\n            // TODO: repeat code\n            if (isFirst) {\n                this.origPageX = ev.pageX;\n                this.origPageY = ev.pageY;\n            }\n            else {\n                deltaX = ev.pageX - this.origPageX;\n                deltaY = ev.pageY - this.origPageY;\n            }\n            return {\n                origEvent: ev,\n                isTouch: false,\n                subjectEl: this.subjectEl,\n                pageX: ev.pageX,\n                pageY: ev.pageY,\n                deltaX: deltaX,\n                deltaY: deltaY\n            };\n        };\n        PointerDragging.prototype.createEventFromTouch = function (ev, isFirst) {\n            var touches = ev.touches;\n            var pageX;\n            var pageY;\n            var deltaX = 0;\n            var deltaY = 0;\n            // if touch coords available, prefer,\n            // because FF would give bad ev.pageX ev.pageY\n            if (touches && touches.length) {\n                pageX = touches[0].pageX;\n                pageY = touches[0].pageY;\n            }\n            else {\n                pageX = ev.pageX;\n                pageY = ev.pageY;\n            }\n            // TODO: repeat code\n            if (isFirst) {\n                this.origPageX = pageX;\n                this.origPageY = pageY;\n            }\n            else {\n                deltaX = pageX - this.origPageX;\n                deltaY = pageY - this.origPageY;\n            }\n            return {\n                origEvent: ev,\n                isTouch: true,\n                subjectEl: this.subjectEl,\n                pageX: pageX,\n                pageY: pageY,\n                deltaX: deltaX,\n                deltaY: deltaY\n            };\n        };\n        return PointerDragging;\n    }());\n    // Returns a boolean whether this was a left mouse click and no ctrl key (which means right click on Mac)\n    function isPrimaryMouseButton(ev) {\n        return ev.button === 0 && !ev.ctrlKey;\n    }\n    // Ignoring fake mouse events generated by touch\n    // ----------------------------------------------------------------------------------------------------\n    function startIgnoringMouse() {\n        ignoreMouseDepth++;\n        setTimeout(function () {\n            ignoreMouseDepth--;\n        }, core.config.touchMouseIgnoreWait);\n    }\n    // We want to attach touchmove as early as possible for Safari\n    // ----------------------------------------------------------------------------------------------------\n    function listenerCreated() {\n        if (!(listenerCnt++)) {\n            window.addEventListener('touchmove', onWindowTouchMove, { passive: false });\n        }\n    }\n    function listenerDestroyed() {\n        if (!(--listenerCnt)) {\n            window.removeEventListener('touchmove', onWindowTouchMove, { passive: false });\n        }\n    }\n    function onWindowTouchMove(ev) {\n        if (isWindowTouchMoveCancelled) {\n            ev.preventDefault();\n        }\n    }\n\n    /*\n    An effect in which an element follows the movement of a pointer across the screen.\n    The moving element is a clone of some other element.\n    Must call start + handleMove + stop.\n    */\n    var ElementMirror = /** @class */ (function () {\n        function ElementMirror() {\n            this.isVisible = false; // must be explicitly enabled\n            this.sourceEl = null;\n            this.mirrorEl = null;\n            this.sourceElRect = null; // screen coords relative to viewport\n            // options that can be set directly by caller\n            this.parentNode = document.body;\n            this.zIndex = 9999;\n            this.revertDuration = 0;\n        }\n        ElementMirror.prototype.start = function (sourceEl, pageX, pageY) {\n            this.sourceEl = sourceEl;\n            this.sourceElRect = this.sourceEl.getBoundingClientRect();\n            this.origScreenX = pageX - window.pageXOffset;\n            this.origScreenY = pageY - window.pageYOffset;\n            this.deltaX = 0;\n            this.deltaY = 0;\n            this.updateElPosition();\n        };\n        ElementMirror.prototype.handleMove = function (pageX, pageY) {\n            this.deltaX = (pageX - window.pageXOffset) - this.origScreenX;\n            this.deltaY = (pageY - window.pageYOffset) - this.origScreenY;\n            this.updateElPosition();\n        };\n        // can be called before start\n        ElementMirror.prototype.setIsVisible = function (bool) {\n            if (bool) {\n                if (!this.isVisible) {\n                    if (this.mirrorEl) {\n                        this.mirrorEl.style.display = '';\n                    }\n                    this.isVisible = bool; // needs to happen before updateElPosition\n                    this.updateElPosition(); // because was not updating the position while invisible\n                }\n            }\n            else {\n                if (this.isVisible) {\n                    if (this.mirrorEl) {\n                        this.mirrorEl.style.display = 'none';\n                    }\n                    this.isVisible = bool;\n                }\n            }\n        };\n        // always async\n        ElementMirror.prototype.stop = function (needsRevertAnimation, callback) {\n            var _this = this;\n            var done = function () {\n                _this.cleanup();\n                callback();\n            };\n            if (needsRevertAnimation &&\n                this.mirrorEl &&\n                this.isVisible &&\n                this.revertDuration && // if 0, transition won't work\n                (this.deltaX || this.deltaY) // if same coords, transition won't work\n            ) {\n                this.doRevertAnimation(done, this.revertDuration);\n            }\n            else {\n                setTimeout(done, 0);\n            }\n        };\n        ElementMirror.prototype.doRevertAnimation = function (callback, revertDuration) {\n            var mirrorEl = this.mirrorEl;\n            var finalSourceElRect = this.sourceEl.getBoundingClientRect(); // because autoscrolling might have happened\n            mirrorEl.style.transition =\n                'top ' + revertDuration + 'ms,' +\n                    'left ' + revertDuration + 'ms';\n            core.applyStyle(mirrorEl, {\n                left: finalSourceElRect.left,\n                top: finalSourceElRect.top\n            });\n            core.whenTransitionDone(mirrorEl, function () {\n                mirrorEl.style.transition = '';\n                callback();\n            });\n        };\n        ElementMirror.prototype.cleanup = function () {\n            if (this.mirrorEl) {\n                core.removeElement(this.mirrorEl);\n                this.mirrorEl = null;\n            }\n            this.sourceEl = null;\n        };\n        ElementMirror.prototype.updateElPosition = function () {\n            if (this.sourceEl && this.isVisible) {\n                core.applyStyle(this.getMirrorEl(), {\n                    left: this.sourceElRect.left + this.deltaX,\n                    top: this.sourceElRect.top + this.deltaY\n                });\n            }\n        };\n        ElementMirror.prototype.getMirrorEl = function () {\n            var sourceElRect = this.sourceElRect;\n            var mirrorEl = this.mirrorEl;\n            if (!mirrorEl) {\n                mirrorEl = this.mirrorEl = this.sourceEl.cloneNode(true); // cloneChildren=true\n                // we don't want long taps or any mouse interaction causing selection/menus.\n                // would use preventSelection(), but that prevents selectstart, causing problems.\n                mirrorEl.classList.add('fc-unselectable');\n                mirrorEl.classList.add('fc-dragging');\n                core.applyStyle(mirrorEl, {\n                    position: 'fixed',\n                    zIndex: this.zIndex,\n                    visibility: '',\n                    boxSizing: 'border-box',\n                    width: sourceElRect.right - sourceElRect.left,\n                    height: sourceElRect.bottom - sourceElRect.top,\n                    right: 'auto',\n                    bottom: 'auto',\n                    margin: 0\n                });\n                this.parentNode.appendChild(mirrorEl);\n            }\n            return mirrorEl;\n        };\n        return ElementMirror;\n    }());\n\n    /*\n    Is a cache for a given element's scroll information (all the info that ScrollController stores)\n    in addition the \"client rectangle\" of the element.. the area within the scrollbars.\n\n    The cache can be in one of two modes:\n    - doesListening:false - ignores when the container is scrolled by someone else\n    - doesListening:true - watch for scrolling and update the cache\n    */\n    var ScrollGeomCache = /** @class */ (function (_super) {\n        __extends(ScrollGeomCache, _super);\n        function ScrollGeomCache(scrollController, doesListening) {\n            var _this = _super.call(this) || this;\n            _this.handleScroll = function () {\n                _this.scrollTop = _this.scrollController.getScrollTop();\n                _this.scrollLeft = _this.scrollController.getScrollLeft();\n                _this.handleScrollChange();\n            };\n            _this.scrollController = scrollController;\n            _this.doesListening = doesListening;\n            _this.scrollTop = _this.origScrollTop = scrollController.getScrollTop();\n            _this.scrollLeft = _this.origScrollLeft = scrollController.getScrollLeft();\n            _this.scrollWidth = scrollController.getScrollWidth();\n            _this.scrollHeight = scrollController.getScrollHeight();\n            _this.clientWidth = scrollController.getClientWidth();\n            _this.clientHeight = scrollController.getClientHeight();\n            _this.clientRect = _this.computeClientRect(); // do last in case it needs cached values\n            if (_this.doesListening) {\n                _this.getEventTarget().addEventListener('scroll', _this.handleScroll);\n            }\n            return _this;\n        }\n        ScrollGeomCache.prototype.destroy = function () {\n            if (this.doesListening) {\n                this.getEventTarget().removeEventListener('scroll', this.handleScroll);\n            }\n        };\n        ScrollGeomCache.prototype.getScrollTop = function () {\n            return this.scrollTop;\n        };\n        ScrollGeomCache.prototype.getScrollLeft = function () {\n            return this.scrollLeft;\n        };\n        ScrollGeomCache.prototype.setScrollTop = function (top) {\n            this.scrollController.setScrollTop(top);\n            if (!this.doesListening) {\n                // we are not relying on the element to normalize out-of-bounds scroll values\n                // so we need to sanitize ourselves\n                this.scrollTop = Math.max(Math.min(top, this.getMaxScrollTop()), 0);\n                this.handleScrollChange();\n            }\n        };\n        ScrollGeomCache.prototype.setScrollLeft = function (top) {\n            this.scrollController.setScrollLeft(top);\n            if (!this.doesListening) {\n                // we are not relying on the element to normalize out-of-bounds scroll values\n                // so we need to sanitize ourselves\n                this.scrollLeft = Math.max(Math.min(top, this.getMaxScrollLeft()), 0);\n                this.handleScrollChange();\n            }\n        };\n        ScrollGeomCache.prototype.getClientWidth = function () {\n            return this.clientWidth;\n        };\n        ScrollGeomCache.prototype.getClientHeight = function () {\n            return this.clientHeight;\n        };\n        ScrollGeomCache.prototype.getScrollWidth = function () {\n            return this.scrollWidth;\n        };\n        ScrollGeomCache.prototype.getScrollHeight = function () {\n            return this.scrollHeight;\n        };\n        ScrollGeomCache.prototype.handleScrollChange = function () {\n        };\n        return ScrollGeomCache;\n    }(core.ScrollController));\n    var ElementScrollGeomCache = /** @class */ (function (_super) {\n        __extends(ElementScrollGeomCache, _super);\n        function ElementScrollGeomCache(el, doesListening) {\n            return _super.call(this, new core.ElementScrollController(el), doesListening) || this;\n        }\n        ElementScrollGeomCache.prototype.getEventTarget = function () {\n            return this.scrollController.el;\n        };\n        ElementScrollGeomCache.prototype.computeClientRect = function () {\n            return core.computeInnerRect(this.scrollController.el);\n        };\n        return ElementScrollGeomCache;\n    }(ScrollGeomCache));\n    var WindowScrollGeomCache = /** @class */ (function (_super) {\n        __extends(WindowScrollGeomCache, _super);\n        function WindowScrollGeomCache(doesListening) {\n            return _super.call(this, new core.WindowScrollController(), doesListening) || this;\n        }\n        WindowScrollGeomCache.prototype.getEventTarget = function () {\n            return window;\n        };\n        WindowScrollGeomCache.prototype.computeClientRect = function () {\n            return {\n                left: this.scrollLeft,\n                right: this.scrollLeft + this.clientWidth,\n                top: this.scrollTop,\n                bottom: this.scrollTop + this.clientHeight\n            };\n        };\n        // the window is the only scroll object that changes it's rectangle relative\n        // to the document's topleft as it scrolls\n        WindowScrollGeomCache.prototype.handleScrollChange = function () {\n            this.clientRect = this.computeClientRect();\n        };\n        return WindowScrollGeomCache;\n    }(ScrollGeomCache));\n\n    // If available we are using native \"performance\" API instead of \"Date\"\n    // Read more about it on MDN:\n    // https://developer.mozilla.org/en-US/docs/Web/API/Performance\n    var getTime = typeof performance === 'function' ? performance.now : Date.now;\n    /*\n    For a pointer interaction, automatically scrolls certain scroll containers when the pointer\n    approaches the edge.\n\n    The caller must call start + handleMove + stop.\n    */\n    var AutoScroller = /** @class */ (function () {\n        function AutoScroller() {\n            var _this = this;\n            // options that can be set by caller\n            this.isEnabled = true;\n            this.scrollQuery = [window, '.fc-scroller'];\n            this.edgeThreshold = 50; // pixels\n            this.maxVelocity = 300; // pixels per second\n            // internal state\n            this.pointerScreenX = null;\n            this.pointerScreenY = null;\n            this.isAnimating = false;\n            this.scrollCaches = null;\n            // protect against the initial pointerdown being too close to an edge and starting the scroll\n            this.everMovedUp = false;\n            this.everMovedDown = false;\n            this.everMovedLeft = false;\n            this.everMovedRight = false;\n            this.animate = function () {\n                if (_this.isAnimating) { // wasn't cancelled between animation calls\n                    var edge = _this.computeBestEdge(_this.pointerScreenX + window.pageXOffset, _this.pointerScreenY + window.pageYOffset);\n                    if (edge) {\n                        var now = getTime();\n                        _this.handleSide(edge, (now - _this.msSinceRequest) / 1000);\n                        _this.requestAnimation(now);\n                    }\n                    else {\n                        _this.isAnimating = false; // will stop animation\n                    }\n                }\n            };\n        }\n        AutoScroller.prototype.start = function (pageX, pageY) {\n            if (this.isEnabled) {\n                this.scrollCaches = this.buildCaches();\n                this.pointerScreenX = null;\n                this.pointerScreenY = null;\n                this.everMovedUp = false;\n                this.everMovedDown = false;\n                this.everMovedLeft = false;\n                this.everMovedRight = false;\n                this.handleMove(pageX, pageY);\n            }\n        };\n        AutoScroller.prototype.handleMove = function (pageX, pageY) {\n            if (this.isEnabled) {\n                var pointerScreenX = pageX - window.pageXOffset;\n                var pointerScreenY = pageY - window.pageYOffset;\n                var yDelta = this.pointerScreenY === null ? 0 : pointerScreenY - this.pointerScreenY;\n                var xDelta = this.pointerScreenX === null ? 0 : pointerScreenX - this.pointerScreenX;\n                if (yDelta < 0) {\n                    this.everMovedUp = true;\n                }\n                else if (yDelta > 0) {\n                    this.everMovedDown = true;\n                }\n                if (xDelta < 0) {\n                    this.everMovedLeft = true;\n                }\n                else if (xDelta > 0) {\n                    this.everMovedRight = true;\n                }\n                this.pointerScreenX = pointerScreenX;\n                this.pointerScreenY = pointerScreenY;\n                if (!this.isAnimating) {\n                    this.isAnimating = true;\n                    this.requestAnimation(getTime());\n                }\n            }\n        };\n        AutoScroller.prototype.stop = function () {\n            if (this.isEnabled) {\n                this.isAnimating = false; // will stop animation\n                for (var _i = 0, _a = this.scrollCaches; _i < _a.length; _i++) {\n                    var scrollCache = _a[_i];\n                    scrollCache.destroy();\n                }\n                this.scrollCaches = null;\n            }\n        };\n        AutoScroller.prototype.requestAnimation = function (now) {\n            this.msSinceRequest = now;\n            requestAnimationFrame(this.animate);\n        };\n        AutoScroller.prototype.handleSide = function (edge, seconds) {\n            var scrollCache = edge.scrollCache;\n            var edgeThreshold = this.edgeThreshold;\n            var invDistance = edgeThreshold - edge.distance;\n            var velocity = // the closer to the edge, the faster we scroll\n             (invDistance * invDistance) / (edgeThreshold * edgeThreshold) * // quadratic\n                this.maxVelocity * seconds;\n            var sign = 1;\n            switch (edge.name) {\n                case 'left':\n                    sign = -1;\n                // falls through\n                case 'right':\n                    scrollCache.setScrollLeft(scrollCache.getScrollLeft() + velocity * sign);\n                    break;\n                case 'top':\n                    sign = -1;\n                // falls through\n                case 'bottom':\n                    scrollCache.setScrollTop(scrollCache.getScrollTop() + velocity * sign);\n                    break;\n            }\n        };\n        // left/top are relative to document topleft\n        AutoScroller.prototype.computeBestEdge = function (left, top) {\n            var edgeThreshold = this.edgeThreshold;\n            var bestSide = null;\n            for (var _i = 0, _a = this.scrollCaches; _i < _a.length; _i++) {\n                var scrollCache = _a[_i];\n                var rect = scrollCache.clientRect;\n                var leftDist = left - rect.left;\n                var rightDist = rect.right - left;\n                var topDist = top - rect.top;\n                var bottomDist = rect.bottom - top;\n                // completely within the rect?\n                if (leftDist >= 0 && rightDist >= 0 && topDist >= 0 && bottomDist >= 0) {\n                    if (topDist <= edgeThreshold && this.everMovedUp && scrollCache.canScrollUp() &&\n                        (!bestSide || bestSide.distance > topDist)) {\n                        bestSide = { scrollCache: scrollCache, name: 'top', distance: topDist };\n                    }\n                    if (bottomDist <= edgeThreshold && this.everMovedDown && scrollCache.canScrollDown() &&\n                        (!bestSide || bestSide.distance > bottomDist)) {\n                        bestSide = { scrollCache: scrollCache, name: 'bottom', distance: bottomDist };\n                    }\n                    if (leftDist <= edgeThreshold && this.everMovedLeft && scrollCache.canScrollLeft() &&\n                        (!bestSide || bestSide.distance > leftDist)) {\n                        bestSide = { scrollCache: scrollCache, name: 'left', distance: leftDist };\n                    }\n                    if (rightDist <= edgeThreshold && this.everMovedRight && scrollCache.canScrollRight() &&\n                        (!bestSide || bestSide.distance > rightDist)) {\n                        bestSide = { scrollCache: scrollCache, name: 'right', distance: rightDist };\n                    }\n                }\n            }\n            return bestSide;\n        };\n        AutoScroller.prototype.buildCaches = function () {\n            return this.queryScrollEls().map(function (el) {\n                if (el === window) {\n                    return new WindowScrollGeomCache(false); // false = don't listen to user-generated scrolls\n                }\n                else {\n                    return new ElementScrollGeomCache(el, false); // false = don't listen to user-generated scrolls\n                }\n            });\n        };\n        AutoScroller.prototype.queryScrollEls = function () {\n            var els = [];\n            for (var _i = 0, _a = this.scrollQuery; _i < _a.length; _i++) {\n                var query = _a[_i];\n                if (typeof query === 'object') {\n                    els.push(query);\n                }\n                else {\n                    els.push.apply(els, Array.prototype.slice.call(document.querySelectorAll(query)));\n                }\n            }\n            return els;\n        };\n        return AutoScroller;\n    }());\n\n    /*\n    Monitors dragging on an element. Has a number of high-level features:\n    - minimum distance required before dragging\n    - minimum wait time (\"delay\") before dragging\n    - a mirror element that follows the pointer\n    */\n    var FeaturefulElementDragging = /** @class */ (function (_super) {\n        __extends(FeaturefulElementDragging, _super);\n        function FeaturefulElementDragging(containerEl) {\n            var _this = _super.call(this, containerEl) || this;\n            // options that can be directly set by caller\n            // the caller can also set the PointerDragging's options as well\n            _this.delay = null;\n            _this.minDistance = 0;\n            _this.touchScrollAllowed = true; // prevents drag from starting and blocks scrolling during drag\n            _this.mirrorNeedsRevert = false;\n            _this.isInteracting = false; // is the user validly moving the pointer? lasts until pointerup\n            _this.isDragging = false; // is it INTENTFULLY dragging? lasts until after revert animation\n            _this.isDelayEnded = false;\n            _this.isDistanceSurpassed = false;\n            _this.delayTimeoutId = null;\n            _this.onPointerDown = function (ev) {\n                if (!_this.isDragging) { // so new drag doesn't happen while revert animation is going\n                    _this.isInteracting = true;\n                    _this.isDelayEnded = false;\n                    _this.isDistanceSurpassed = false;\n                    core.preventSelection(document.body);\n                    core.preventContextMenu(document.body);\n                    // prevent links from being visited if there's an eventual drag.\n                    // also prevents selection in older browsers (maybe?).\n                    // not necessary for touch, besides, browser would complain about passiveness.\n                    if (!ev.isTouch) {\n                        ev.origEvent.preventDefault();\n                    }\n                    _this.emitter.trigger('pointerdown', ev);\n                    if (!_this.pointer.shouldIgnoreMove) {\n                        // actions related to initiating dragstart+dragmove+dragend...\n                        _this.mirror.setIsVisible(false); // reset. caller must set-visible\n                        _this.mirror.start(ev.subjectEl, ev.pageX, ev.pageY); // must happen on first pointer down\n                        _this.startDelay(ev);\n                        if (!_this.minDistance) {\n                            _this.handleDistanceSurpassed(ev);\n                        }\n                    }\n                }\n            };\n            _this.onPointerMove = function (ev) {\n                if (_this.isInteracting) { // if false, still waiting for previous drag's revert\n                    _this.emitter.trigger('pointermove', ev);\n                    if (!_this.isDistanceSurpassed) {\n                        var minDistance = _this.minDistance;\n                        var distanceSq = void 0; // current distance from the origin, squared\n                        var deltaX = ev.deltaX, deltaY = ev.deltaY;\n                        distanceSq = deltaX * deltaX + deltaY * deltaY;\n                        if (distanceSq >= minDistance * minDistance) { // use pythagorean theorem\n                            _this.handleDistanceSurpassed(ev);\n                        }\n                    }\n                    if (_this.isDragging) {\n                        // a real pointer move? (not one simulated by scrolling)\n                        if (ev.origEvent.type !== 'scroll') {\n                            _this.mirror.handleMove(ev.pageX, ev.pageY);\n                            _this.autoScroller.handleMove(ev.pageX, ev.pageY);\n                        }\n                        _this.emitter.trigger('dragmove', ev);\n                    }\n                }\n            };\n            _this.onPointerUp = function (ev) {\n                if (_this.isInteracting) { // if false, still waiting for previous drag's revert\n                    _this.isInteracting = false;\n                    core.allowSelection(document.body);\n                    core.allowContextMenu(document.body);\n                    _this.emitter.trigger('pointerup', ev); // can potentially set mirrorNeedsRevert\n                    if (_this.isDragging) {\n                        _this.autoScroller.stop();\n                        _this.tryStopDrag(ev); // which will stop the mirror\n                    }\n                    if (_this.delayTimeoutId) {\n                        clearTimeout(_this.delayTimeoutId);\n                        _this.delayTimeoutId = null;\n                    }\n                }\n            };\n            var pointer = _this.pointer = new PointerDragging(containerEl);\n            pointer.emitter.on('pointerdown', _this.onPointerDown);\n            pointer.emitter.on('pointermove', _this.onPointerMove);\n            pointer.emitter.on('pointerup', _this.onPointerUp);\n            _this.mirror = new ElementMirror();\n            _this.autoScroller = new AutoScroller();\n            return _this;\n        }\n        FeaturefulElementDragging.prototype.destroy = function () {\n            this.pointer.destroy();\n        };\n        FeaturefulElementDragging.prototype.startDelay = function (ev) {\n            var _this = this;\n            if (typeof this.delay === 'number') {\n                this.delayTimeoutId = setTimeout(function () {\n                    _this.delayTimeoutId = null;\n                    _this.handleDelayEnd(ev);\n                }, this.delay); // not assignable to number!\n            }\n            else {\n                this.handleDelayEnd(ev);\n            }\n        };\n        FeaturefulElementDragging.prototype.handleDelayEnd = function (ev) {\n            this.isDelayEnded = true;\n            this.tryStartDrag(ev);\n        };\n        FeaturefulElementDragging.prototype.handleDistanceSurpassed = function (ev) {\n            this.isDistanceSurpassed = true;\n            this.tryStartDrag(ev);\n        };\n        FeaturefulElementDragging.prototype.tryStartDrag = function (ev) {\n            if (this.isDelayEnded && this.isDistanceSurpassed) {\n                if (!this.pointer.wasTouchScroll || this.touchScrollAllowed) {\n                    this.isDragging = true;\n                    this.mirrorNeedsRevert = false;\n                    this.autoScroller.start(ev.pageX, ev.pageY);\n                    this.emitter.trigger('dragstart', ev);\n                    if (this.touchScrollAllowed === false) {\n                        this.pointer.cancelTouchScroll();\n                    }\n                }\n            }\n        };\n        FeaturefulElementDragging.prototype.tryStopDrag = function (ev) {\n            // .stop() is ALWAYS asynchronous, which we NEED because we want all pointerup events\n            // that come from the document to fire beforehand. much more convenient this way.\n            this.mirror.stop(this.mirrorNeedsRevert, this.stopDrag.bind(this, ev) // bound with args\n            );\n        };\n        FeaturefulElementDragging.prototype.stopDrag = function (ev) {\n            this.isDragging = false;\n            this.emitter.trigger('dragend', ev);\n        };\n        // fill in the implementations...\n        FeaturefulElementDragging.prototype.setIgnoreMove = function (bool) {\n            this.pointer.shouldIgnoreMove = bool;\n        };\n        FeaturefulElementDragging.prototype.setMirrorIsVisible = function (bool) {\n            this.mirror.setIsVisible(bool);\n        };\n        FeaturefulElementDragging.prototype.setMirrorNeedsRevert = function (bool) {\n            this.mirrorNeedsRevert = bool;\n        };\n        FeaturefulElementDragging.prototype.setAutoScrollEnabled = function (bool) {\n            this.autoScroller.isEnabled = bool;\n        };\n        return FeaturefulElementDragging;\n    }(core.ElementDragging));\n\n    /*\n    When this class is instantiated, it records the offset of an element (relative to the document topleft),\n    and continues to monitor scrolling, updating the cached coordinates if it needs to.\n    Does not access the DOM after instantiation, so highly performant.\n\n    Also keeps track of all scrolling/overflow:hidden containers that are parents of the given element\n    and an determine if a given point is inside the combined clipping rectangle.\n    */\n    var OffsetTracker = /** @class */ (function () {\n        function OffsetTracker(el) {\n            this.origRect = core.computeRect(el);\n            // will work fine for divs that have overflow:hidden\n            this.scrollCaches = core.getClippingParents(el).map(function (el) {\n                return new ElementScrollGeomCache(el, true); // listen=true\n            });\n        }\n        OffsetTracker.prototype.destroy = function () {\n            for (var _i = 0, _a = this.scrollCaches; _i < _a.length; _i++) {\n                var scrollCache = _a[_i];\n                scrollCache.destroy();\n            }\n        };\n        OffsetTracker.prototype.computeLeft = function () {\n            var left = this.origRect.left;\n            for (var _i = 0, _a = this.scrollCaches; _i < _a.length; _i++) {\n                var scrollCache = _a[_i];\n                left += scrollCache.origScrollLeft - scrollCache.getScrollLeft();\n            }\n            return left;\n        };\n        OffsetTracker.prototype.computeTop = function () {\n            var top = this.origRect.top;\n            for (var _i = 0, _a = this.scrollCaches; _i < _a.length; _i++) {\n                var scrollCache = _a[_i];\n                top += scrollCache.origScrollTop - scrollCache.getScrollTop();\n            }\n            return top;\n        };\n        OffsetTracker.prototype.isWithinClipping = function (pageX, pageY) {\n            var point = { left: pageX, top: pageY };\n            for (var _i = 0, _a = this.scrollCaches; _i < _a.length; _i++) {\n                var scrollCache = _a[_i];\n                if (!isIgnoredClipping(scrollCache.getEventTarget()) &&\n                    !core.pointInsideRect(point, scrollCache.clientRect)) {\n                    return false;\n                }\n            }\n            return true;\n        };\n        return OffsetTracker;\n    }());\n    // certain clipping containers should never constrain interactions, like <html> and <body>\n    // https://github.com/fullcalendar/fullcalendar/issues/3615\n    function isIgnoredClipping(node) {\n        var tagName = node.tagName;\n        return tagName === 'HTML' || tagName === 'BODY';\n    }\n\n    /*\n    Tracks movement over multiple droppable areas (aka \"hits\")\n    that exist in one or more DateComponents.\n    Relies on an existing draggable.\n\n    emits:\n    - pointerdown\n    - dragstart\n    - hitchange - fires initially, even if not over a hit\n    - pointerup\n    - (hitchange - again, to null, if ended over a hit)\n    - dragend\n    */\n    var HitDragging = /** @class */ (function () {\n        function HitDragging(dragging, droppableStore) {\n            var _this = this;\n            // options that can be set by caller\n            this.useSubjectCenter = false;\n            this.requireInitial = true; // if doesn't start out on a hit, won't emit any events\n            this.initialHit = null;\n            this.movingHit = null;\n            this.finalHit = null; // won't ever be populated if shouldIgnoreMove\n            this.handlePointerDown = function (ev) {\n                var dragging = _this.dragging;\n                _this.initialHit = null;\n                _this.movingHit = null;\n                _this.finalHit = null;\n                _this.prepareHits();\n                _this.processFirstCoord(ev);\n                if (_this.initialHit || !_this.requireInitial) {\n                    dragging.setIgnoreMove(false);\n                    _this.emitter.trigger('pointerdown', ev); // TODO: fire this before computing processFirstCoord, so listeners can cancel. this gets fired by almost every handler :(\n                }\n                else {\n                    dragging.setIgnoreMove(true);\n                }\n            };\n            this.handleDragStart = function (ev) {\n                _this.emitter.trigger('dragstart', ev);\n                _this.handleMove(ev, true); // force = fire even if initially null\n            };\n            this.handleDragMove = function (ev) {\n                _this.emitter.trigger('dragmove', ev);\n                _this.handleMove(ev);\n            };\n            this.handlePointerUp = function (ev) {\n                _this.releaseHits();\n                _this.emitter.trigger('pointerup', ev);\n            };\n            this.handleDragEnd = function (ev) {\n                if (_this.movingHit) {\n                    _this.emitter.trigger('hitupdate', null, true, ev);\n                }\n                _this.finalHit = _this.movingHit;\n                _this.movingHit = null;\n                _this.emitter.trigger('dragend', ev);\n            };\n            this.droppableStore = droppableStore;\n            dragging.emitter.on('pointerdown', this.handlePointerDown);\n            dragging.emitter.on('dragstart', this.handleDragStart);\n            dragging.emitter.on('dragmove', this.handleDragMove);\n            dragging.emitter.on('pointerup', this.handlePointerUp);\n            dragging.emitter.on('dragend', this.handleDragEnd);\n            this.dragging = dragging;\n            this.emitter = new core.EmitterMixin();\n        }\n        // sets initialHit\n        // sets coordAdjust\n        HitDragging.prototype.processFirstCoord = function (ev) {\n            var origPoint = { left: ev.pageX, top: ev.pageY };\n            var adjustedPoint = origPoint;\n            var subjectEl = ev.subjectEl;\n            var subjectRect;\n            if (subjectEl !== document) {\n                subjectRect = core.computeRect(subjectEl);\n                adjustedPoint = core.constrainPoint(adjustedPoint, subjectRect);\n            }\n            var initialHit = this.initialHit = this.queryHitForOffset(adjustedPoint.left, adjustedPoint.top);\n            if (initialHit) {\n                if (this.useSubjectCenter && subjectRect) {\n                    var slicedSubjectRect = core.intersectRects(subjectRect, initialHit.rect);\n                    if (slicedSubjectRect) {\n                        adjustedPoint = core.getRectCenter(slicedSubjectRect);\n                    }\n                }\n                this.coordAdjust = core.diffPoints(adjustedPoint, origPoint);\n            }\n            else {\n                this.coordAdjust = { left: 0, top: 0 };\n            }\n        };\n        HitDragging.prototype.handleMove = function (ev, forceHandle) {\n            var hit = this.queryHitForOffset(ev.pageX + this.coordAdjust.left, ev.pageY + this.coordAdjust.top);\n            if (forceHandle || !isHitsEqual(this.movingHit, hit)) {\n                this.movingHit = hit;\n                this.emitter.trigger('hitupdate', hit, false, ev);\n            }\n        };\n        HitDragging.prototype.prepareHits = function () {\n            this.offsetTrackers = core.mapHash(this.droppableStore, function (interactionSettings) {\n                interactionSettings.component.buildPositionCaches();\n                return new OffsetTracker(interactionSettings.el);\n            });\n        };\n        HitDragging.prototype.releaseHits = function () {\n            var offsetTrackers = this.offsetTrackers;\n            for (var id in offsetTrackers) {\n                offsetTrackers[id].destroy();\n            }\n            this.offsetTrackers = {};\n        };\n        HitDragging.prototype.queryHitForOffset = function (offsetLeft, offsetTop) {\n            var _a = this, droppableStore = _a.droppableStore, offsetTrackers = _a.offsetTrackers;\n            var bestHit = null;\n            for (var id in droppableStore) {\n                var component = droppableStore[id].component;\n                var offsetTracker = offsetTrackers[id];\n                if (offsetTracker.isWithinClipping(offsetLeft, offsetTop)) {\n                    var originLeft = offsetTracker.computeLeft();\n                    var originTop = offsetTracker.computeTop();\n                    var positionLeft = offsetLeft - originLeft;\n                    var positionTop = offsetTop - originTop;\n                    var origRect = offsetTracker.origRect;\n                    var width = origRect.right - origRect.left;\n                    var height = origRect.bottom - origRect.top;\n                    if (\n                    // must be within the element's bounds\n                    positionLeft >= 0 && positionLeft < width &&\n                        positionTop >= 0 && positionTop < height) {\n                        var hit = component.queryHit(positionLeft, positionTop, width, height);\n                        if (hit &&\n                            (\n                            // make sure the hit is within activeRange, meaning it's not a deal cell\n                            !component.props.dateProfile || // hack for DayTile\n                                core.rangeContainsRange(component.props.dateProfile.activeRange, hit.dateSpan.range)) &&\n                            (!bestHit || hit.layer > bestHit.layer)) {\n                            // TODO: better way to re-orient rectangle\n                            hit.rect.left += originLeft;\n                            hit.rect.right += originLeft;\n                            hit.rect.top += originTop;\n                            hit.rect.bottom += originTop;\n                            bestHit = hit;\n                        }\n                    }\n                }\n            }\n            return bestHit;\n        };\n        return HitDragging;\n    }());\n    function isHitsEqual(hit0, hit1) {\n        if (!hit0 && !hit1) {\n            return true;\n        }\n        if (Boolean(hit0) !== Boolean(hit1)) {\n            return false;\n        }\n        return core.isDateSpansEqual(hit0.dateSpan, hit1.dateSpan);\n    }\n\n    /*\n    Monitors when the user clicks on a specific date/time of a component.\n    A pointerdown+pointerup on the same \"hit\" constitutes a click.\n    */\n    var DateClicking = /** @class */ (function (_super) {\n        __extends(DateClicking, _super);\n        function DateClicking(settings) {\n            var _this = _super.call(this, settings) || this;\n            _this.handlePointerDown = function (ev) {\n                var dragging = _this.dragging;\n                // do this in pointerdown (not dragend) because DOM might be mutated by the time dragend is fired\n                dragging.setIgnoreMove(!_this.component.isValidDateDownEl(dragging.pointer.downEl));\n            };\n            // won't even fire if moving was ignored\n            _this.handleDragEnd = function (ev) {\n                var component = _this.component;\n                var _a = component.context, calendar = _a.calendar, view = _a.view;\n                var pointer = _this.dragging.pointer;\n                if (!pointer.wasTouchScroll) {\n                    var _b = _this.hitDragging, initialHit = _b.initialHit, finalHit = _b.finalHit;\n                    if (initialHit && finalHit && isHitsEqual(initialHit, finalHit)) {\n                        calendar.triggerDateClick(initialHit.dateSpan, initialHit.dayEl, view, ev.origEvent);\n                    }\n                }\n            };\n            var component = settings.component;\n            // we DO want to watch pointer moves because otherwise finalHit won't get populated\n            _this.dragging = new FeaturefulElementDragging(component.el);\n            _this.dragging.autoScroller.isEnabled = false;\n            var hitDragging = _this.hitDragging = new HitDragging(_this.dragging, core.interactionSettingsToStore(settings));\n            hitDragging.emitter.on('pointerdown', _this.handlePointerDown);\n            hitDragging.emitter.on('dragend', _this.handleDragEnd);\n            return _this;\n        }\n        DateClicking.prototype.destroy = function () {\n            this.dragging.destroy();\n        };\n        return DateClicking;\n    }(core.Interaction));\n\n    /*\n    Tracks when the user selects a portion of time of a component,\n    constituted by a drag over date cells, with a possible delay at the beginning of the drag.\n    */\n    var DateSelecting = /** @class */ (function (_super) {\n        __extends(DateSelecting, _super);\n        function DateSelecting(settings) {\n            var _this = _super.call(this, settings) || this;\n            _this.dragSelection = null;\n            _this.handlePointerDown = function (ev) {\n                var _a = _this, component = _a.component, dragging = _a.dragging;\n                var options = component.context.options;\n                var canSelect = options.selectable &&\n                    component.isValidDateDownEl(ev.origEvent.target);\n                // don't bother to watch expensive moves if component won't do selection\n                dragging.setIgnoreMove(!canSelect);\n                // if touch, require user to hold down\n                dragging.delay = ev.isTouch ? getComponentTouchDelay(component) : null;\n            };\n            _this.handleDragStart = function (ev) {\n                _this.component.context.calendar.unselect(ev); // unselect previous selections\n            };\n            _this.handleHitUpdate = function (hit, isFinal) {\n                var calendar = _this.component.context.calendar;\n                var dragSelection = null;\n                var isInvalid = false;\n                if (hit) {\n                    dragSelection = joinHitsIntoSelection(_this.hitDragging.initialHit, hit, calendar.pluginSystem.hooks.dateSelectionTransformers);\n                    if (!dragSelection || !_this.component.isDateSelectionValid(dragSelection)) {\n                        isInvalid = true;\n                        dragSelection = null;\n                    }\n                }\n                if (dragSelection) {\n                    calendar.dispatch({ type: 'SELECT_DATES', selection: dragSelection });\n                }\n                else if (!isFinal) { // only unselect if moved away while dragging\n                    calendar.dispatch({ type: 'UNSELECT_DATES' });\n                }\n                if (!isInvalid) {\n                    core.enableCursor();\n                }\n                else {\n                    core.disableCursor();\n                }\n                if (!isFinal) {\n                    _this.dragSelection = dragSelection; // only clear if moved away from all hits while dragging\n                }\n            };\n            _this.handlePointerUp = function (pev) {\n                if (_this.dragSelection) {\n                    // selection is already rendered, so just need to report selection\n                    _this.component.context.calendar.triggerDateSelect(_this.dragSelection, pev);\n                    _this.dragSelection = null;\n                }\n            };\n            var component = settings.component;\n            var options = component.context.options;\n            var dragging = _this.dragging = new FeaturefulElementDragging(component.el);\n            dragging.touchScrollAllowed = false;\n            dragging.minDistance = options.selectMinDistance || 0;\n            dragging.autoScroller.isEnabled = options.dragScroll;\n            var hitDragging = _this.hitDragging = new HitDragging(_this.dragging, core.interactionSettingsToStore(settings));\n            hitDragging.emitter.on('pointerdown', _this.handlePointerDown);\n            hitDragging.emitter.on('dragstart', _this.handleDragStart);\n            hitDragging.emitter.on('hitupdate', _this.handleHitUpdate);\n            hitDragging.emitter.on('pointerup', _this.handlePointerUp);\n            return _this;\n        }\n        DateSelecting.prototype.destroy = function () {\n            this.dragging.destroy();\n        };\n        return DateSelecting;\n    }(core.Interaction));\n    function getComponentTouchDelay(component) {\n        var options = component.context.options;\n        var delay = options.selectLongPressDelay;\n        if (delay == null) {\n            delay = options.longPressDelay;\n        }\n        return delay;\n    }\n    function joinHitsIntoSelection(hit0, hit1, dateSelectionTransformers) {\n        var dateSpan0 = hit0.dateSpan;\n        var dateSpan1 = hit1.dateSpan;\n        var ms = [\n            dateSpan0.range.start,\n            dateSpan0.range.end,\n            dateSpan1.range.start,\n            dateSpan1.range.end\n        ];\n        ms.sort(core.compareNumbers);\n        var props = {};\n        for (var _i = 0, dateSelectionTransformers_1 = dateSelectionTransformers; _i < dateSelectionTransformers_1.length; _i++) {\n            var transformer = dateSelectionTransformers_1[_i];\n            var res = transformer(hit0, hit1);\n            if (res === false) {\n                return null;\n            }\n            else if (res) {\n                __assign(props, res);\n            }\n        }\n        props.range = { start: ms[0], end: ms[3] };\n        props.allDay = dateSpan0.allDay;\n        return props;\n    }\n\n    var EventDragging = /** @class */ (function (_super) {\n        __extends(EventDragging, _super);\n        function EventDragging(settings) {\n            var _this = _super.call(this, settings) || this;\n            // internal state\n            _this.subjectSeg = null; // the seg being selected/dragged\n            _this.isDragging = false;\n            _this.eventRange = null;\n            _this.relevantEvents = null; // the events being dragged\n            _this.receivingCalendar = null;\n            _this.validMutation = null;\n            _this.mutatedRelevantEvents = null;\n            _this.handlePointerDown = function (ev) {\n                var origTarget = ev.origEvent.target;\n                var _a = _this, component = _a.component, dragging = _a.dragging;\n                var mirror = dragging.mirror;\n                var options = component.context.options;\n                var initialCalendar = component.context.calendar;\n                var subjectSeg = _this.subjectSeg = core.getElSeg(ev.subjectEl);\n                var eventRange = _this.eventRange = subjectSeg.eventRange;\n                var eventInstanceId = eventRange.instance.instanceId;\n                _this.relevantEvents = core.getRelevantEvents(initialCalendar.state.eventStore, eventInstanceId);\n                dragging.minDistance = ev.isTouch ? 0 : options.eventDragMinDistance;\n                dragging.delay =\n                    // only do a touch delay if touch and this event hasn't been selected yet\n                    (ev.isTouch && eventInstanceId !== component.props.eventSelection) ?\n                        getComponentTouchDelay$1(component) :\n                        null;\n                mirror.parentNode = initialCalendar.el;\n                mirror.revertDuration = options.dragRevertDuration;\n                var isValid = component.isValidSegDownEl(origTarget) &&\n                    !core.elementClosest(origTarget, '.fc-resizer'); // NOT on a resizer\n                dragging.setIgnoreMove(!isValid);\n                // disable dragging for elements that are resizable (ie, selectable)\n                // but are not draggable\n                _this.isDragging = isValid &&\n                    ev.subjectEl.classList.contains('fc-draggable');\n            };\n            _this.handleDragStart = function (ev) {\n                var context = _this.component.context;\n                var initialCalendar = context.calendar;\n                var eventRange = _this.eventRange;\n                var eventInstanceId = eventRange.instance.instanceId;\n                if (ev.isTouch) {\n                    // need to select a different event?\n                    if (eventInstanceId !== _this.component.props.eventSelection) {\n                        initialCalendar.dispatch({ type: 'SELECT_EVENT', eventInstanceId: eventInstanceId });\n                    }\n                }\n                else {\n                    // if now using mouse, but was previous touch interaction, clear selected event\n                    initialCalendar.dispatch({ type: 'UNSELECT_EVENT' });\n                }\n                if (_this.isDragging) {\n                    initialCalendar.unselect(ev); // unselect *date* selection\n                    initialCalendar.publiclyTrigger('eventDragStart', [\n                        {\n                            el: _this.subjectSeg.el,\n                            event: new core.EventApi(initialCalendar, eventRange.def, eventRange.instance),\n                            jsEvent: ev.origEvent,\n                            view: context.view\n                        }\n                    ]);\n                }\n            };\n            _this.handleHitUpdate = function (hit, isFinal) {\n                if (!_this.isDragging) {\n                    return;\n                }\n                var relevantEvents = _this.relevantEvents;\n                var initialHit = _this.hitDragging.initialHit;\n                var initialCalendar = _this.component.context.calendar;\n                // states based on new hit\n                var receivingCalendar = null;\n                var mutation = null;\n                var mutatedRelevantEvents = null;\n                var isInvalid = false;\n                var interaction = {\n                    affectedEvents: relevantEvents,\n                    mutatedEvents: core.createEmptyEventStore(),\n                    isEvent: true,\n                    origSeg: _this.subjectSeg\n                };\n                if (hit) {\n                    var receivingComponent = hit.component;\n                    receivingCalendar = receivingComponent.context.calendar;\n                    var receivingOptions = receivingComponent.context.options;\n                    if (initialCalendar === receivingCalendar ||\n                        receivingOptions.editable && receivingOptions.droppable) {\n                        mutation = computeEventMutation(initialHit, hit, receivingCalendar.pluginSystem.hooks.eventDragMutationMassagers);\n                        if (mutation) {\n                            mutatedRelevantEvents = core.applyMutationToEventStore(relevantEvents, receivingCalendar.eventUiBases, mutation, receivingCalendar);\n                            interaction.mutatedEvents = mutatedRelevantEvents;\n                            if (!receivingComponent.isInteractionValid(interaction)) {\n                                isInvalid = true;\n                                mutation = null;\n                                mutatedRelevantEvents = null;\n                                interaction.mutatedEvents = core.createEmptyEventStore();\n                            }\n                        }\n                    }\n                    else {\n                        receivingCalendar = null;\n                    }\n                }\n                _this.displayDrag(receivingCalendar, interaction);\n                if (!isInvalid) {\n                    core.enableCursor();\n                }\n                else {\n                    core.disableCursor();\n                }\n                if (!isFinal) {\n                    if (initialCalendar === receivingCalendar && // TODO: write test for this\n                        isHitsEqual(initialHit, hit)) {\n                        mutation = null;\n                    }\n                    _this.dragging.setMirrorNeedsRevert(!mutation);\n                    // render the mirror if no already-rendered mirror\n                    // TODO: wish we could somehow wait for dispatch to guarantee render\n                    _this.dragging.setMirrorIsVisible(!hit || !document.querySelector('.fc-mirror'));\n                    // assign states based on new hit\n                    _this.receivingCalendar = receivingCalendar;\n                    _this.validMutation = mutation;\n                    _this.mutatedRelevantEvents = mutatedRelevantEvents;\n                }\n            };\n            _this.handlePointerUp = function () {\n                if (!_this.isDragging) {\n                    _this.cleanup(); // because handleDragEnd won't fire\n                }\n            };\n            _this.handleDragEnd = function (ev) {\n                if (_this.isDragging) {\n                    var context = _this.component.context;\n                    var initialCalendar_1 = context.calendar;\n                    var initialView = context.view;\n                    var _a = _this, receivingCalendar = _a.receivingCalendar, validMutation = _a.validMutation;\n                    var eventDef = _this.eventRange.def;\n                    var eventInstance = _this.eventRange.instance;\n                    var eventApi = new core.EventApi(initialCalendar_1, eventDef, eventInstance);\n                    var relevantEvents_1 = _this.relevantEvents;\n                    var mutatedRelevantEvents = _this.mutatedRelevantEvents;\n                    var finalHit = _this.hitDragging.finalHit;\n                    _this.clearDrag(); // must happen after revert animation\n                    initialCalendar_1.publiclyTrigger('eventDragStop', [\n                        {\n                            el: _this.subjectSeg.el,\n                            event: eventApi,\n                            jsEvent: ev.origEvent,\n                            view: initialView\n                        }\n                    ]);\n                    if (validMutation) {\n                        // dropped within same calendar\n                        if (receivingCalendar === initialCalendar_1) {\n                            initialCalendar_1.dispatch({\n                                type: 'MERGE_EVENTS',\n                                eventStore: mutatedRelevantEvents\n                            });\n                            var transformed = {};\n                            for (var _i = 0, _b = initialCalendar_1.pluginSystem.hooks.eventDropTransformers; _i < _b.length; _i++) {\n                                var transformer = _b[_i];\n                                __assign(transformed, transformer(validMutation, initialCalendar_1));\n                            }\n                            var eventDropArg = __assign({}, transformed, { el: ev.subjectEl, delta: validMutation.datesDelta, oldEvent: eventApi, event: new core.EventApi(// the data AFTER the mutation\n                                initialCalendar_1, mutatedRelevantEvents.defs[eventDef.defId], eventInstance ? mutatedRelevantEvents.instances[eventInstance.instanceId] : null), revert: function () {\n                                    initialCalendar_1.dispatch({\n                                        type: 'MERGE_EVENTS',\n                                        eventStore: relevantEvents_1\n                                    });\n                                }, jsEvent: ev.origEvent, view: initialView });\n                            initialCalendar_1.publiclyTrigger('eventDrop', [eventDropArg]);\n                            // dropped in different calendar\n                        }\n                        else if (receivingCalendar) {\n                            initialCalendar_1.publiclyTrigger('eventLeave', [\n                                {\n                                    draggedEl: ev.subjectEl,\n                                    event: eventApi,\n                                    view: initialView\n                                }\n                            ]);\n                            initialCalendar_1.dispatch({\n                                type: 'REMOVE_EVENT_INSTANCES',\n                                instances: _this.mutatedRelevantEvents.instances\n                            });\n                            receivingCalendar.dispatch({\n                                type: 'MERGE_EVENTS',\n                                eventStore: _this.mutatedRelevantEvents\n                            });\n                            if (ev.isTouch) {\n                                receivingCalendar.dispatch({\n                                    type: 'SELECT_EVENT',\n                                    eventInstanceId: eventInstance.instanceId\n                                });\n                            }\n                            var dropArg = __assign({}, receivingCalendar.buildDatePointApi(finalHit.dateSpan), { draggedEl: ev.subjectEl, jsEvent: ev.origEvent, view: finalHit.component // should this be finalHit.component.view? See #4644\n                             });\n                            receivingCalendar.publiclyTrigger('drop', [dropArg]);\n                            receivingCalendar.publiclyTrigger('eventReceive', [\n                                {\n                                    draggedEl: ev.subjectEl,\n                                    event: new core.EventApi(// the data AFTER the mutation\n                                    receivingCalendar, mutatedRelevantEvents.defs[eventDef.defId], mutatedRelevantEvents.instances[eventInstance.instanceId]),\n                                    view: finalHit.component // should this be finalHit.component.view? See #4644\n                                }\n                            ]);\n                        }\n                    }\n                    else {\n                        initialCalendar_1.publiclyTrigger('_noEventDrop');\n                    }\n                }\n                _this.cleanup();\n            };\n            var component = _this.component;\n            var options = component.context.options;\n            var dragging = _this.dragging = new FeaturefulElementDragging(component.el);\n            dragging.pointer.selector = EventDragging.SELECTOR;\n            dragging.touchScrollAllowed = false;\n            dragging.autoScroller.isEnabled = options.dragScroll;\n            var hitDragging = _this.hitDragging = new HitDragging(_this.dragging, core.interactionSettingsStore);\n            hitDragging.useSubjectCenter = settings.useEventCenter;\n            hitDragging.emitter.on('pointerdown', _this.handlePointerDown);\n            hitDragging.emitter.on('dragstart', _this.handleDragStart);\n            hitDragging.emitter.on('hitupdate', _this.handleHitUpdate);\n            hitDragging.emitter.on('pointerup', _this.handlePointerUp);\n            hitDragging.emitter.on('dragend', _this.handleDragEnd);\n            return _this;\n        }\n        EventDragging.prototype.destroy = function () {\n            this.dragging.destroy();\n        };\n        // render a drag state on the next receivingCalendar\n        EventDragging.prototype.displayDrag = function (nextCalendar, state) {\n            var initialCalendar = this.component.context.calendar;\n            var prevCalendar = this.receivingCalendar;\n            // does the previous calendar need to be cleared?\n            if (prevCalendar && prevCalendar !== nextCalendar) {\n                // does the initial calendar need to be cleared?\n                // if so, don't clear all the way. we still need to to hide the affectedEvents\n                if (prevCalendar === initialCalendar) {\n                    prevCalendar.dispatch({\n                        type: 'SET_EVENT_DRAG',\n                        state: {\n                            affectedEvents: state.affectedEvents,\n                            mutatedEvents: core.createEmptyEventStore(),\n                            isEvent: true,\n                            origSeg: state.origSeg\n                        }\n                    });\n                    // completely clear the old calendar if it wasn't the initial\n                }\n                else {\n                    prevCalendar.dispatch({ type: 'UNSET_EVENT_DRAG' });\n                }\n            }\n            if (nextCalendar) {\n                nextCalendar.dispatch({ type: 'SET_EVENT_DRAG', state: state });\n            }\n        };\n        EventDragging.prototype.clearDrag = function () {\n            var initialCalendar = this.component.context.calendar;\n            var receivingCalendar = this.receivingCalendar;\n            if (receivingCalendar) {\n                receivingCalendar.dispatch({ type: 'UNSET_EVENT_DRAG' });\n            }\n            // the initial calendar might have an dummy drag state from displayDrag\n            if (initialCalendar !== receivingCalendar) {\n                initialCalendar.dispatch({ type: 'UNSET_EVENT_DRAG' });\n            }\n        };\n        EventDragging.prototype.cleanup = function () {\n            this.subjectSeg = null;\n            this.isDragging = false;\n            this.eventRange = null;\n            this.relevantEvents = null;\n            this.receivingCalendar = null;\n            this.validMutation = null;\n            this.mutatedRelevantEvents = null;\n        };\n        EventDragging.SELECTOR = '.fc-draggable, .fc-resizable'; // TODO: test this in IE11\n        return EventDragging;\n    }(core.Interaction));\n    function computeEventMutation(hit0, hit1, massagers) {\n        var dateSpan0 = hit0.dateSpan;\n        var dateSpan1 = hit1.dateSpan;\n        var date0 = dateSpan0.range.start;\n        var date1 = dateSpan1.range.start;\n        var standardProps = {};\n        if (dateSpan0.allDay !== dateSpan1.allDay) {\n            standardProps.allDay = dateSpan1.allDay;\n            standardProps.hasEnd = hit1.component.context.options.allDayMaintainDuration;\n            if (dateSpan1.allDay) {\n                // means date1 is already start-of-day,\n                // but date0 needs to be converted\n                date0 = core.startOfDay(date0);\n            }\n        }\n        var delta = core.diffDates(date0, date1, hit0.component.context.dateEnv, hit0.component === hit1.component ?\n            hit0.component.largeUnit :\n            null);\n        if (delta.milliseconds) { // has hours/minutes/seconds\n            standardProps.allDay = false;\n        }\n        var mutation = {\n            datesDelta: delta,\n            standardProps: standardProps\n        };\n        for (var _i = 0, massagers_1 = massagers; _i < massagers_1.length; _i++) {\n            var massager = massagers_1[_i];\n            massager(mutation, hit0, hit1);\n        }\n        return mutation;\n    }\n    function getComponentTouchDelay$1(component) {\n        var options = component.context.options;\n        var delay = options.eventLongPressDelay;\n        if (delay == null) {\n            delay = options.longPressDelay;\n        }\n        return delay;\n    }\n\n    var EventDragging$1 = /** @class */ (function (_super) {\n        __extends(EventDragging, _super);\n        function EventDragging(settings) {\n            var _this = _super.call(this, settings) || this;\n            // internal state\n            _this.draggingSeg = null; // TODO: rename to resizingSeg? subjectSeg?\n            _this.eventRange = null;\n            _this.relevantEvents = null;\n            _this.validMutation = null;\n            _this.mutatedRelevantEvents = null;\n            _this.handlePointerDown = function (ev) {\n                var component = _this.component;\n                var seg = _this.querySeg(ev);\n                var eventRange = _this.eventRange = seg.eventRange;\n                _this.dragging.minDistance = component.context.options.eventDragMinDistance;\n                // if touch, need to be working with a selected event\n                _this.dragging.setIgnoreMove(!_this.component.isValidSegDownEl(ev.origEvent.target) ||\n                    (ev.isTouch && _this.component.props.eventSelection !== eventRange.instance.instanceId));\n            };\n            _this.handleDragStart = function (ev) {\n                var _a = _this.component.context, calendar = _a.calendar, view = _a.view;\n                var eventRange = _this.eventRange;\n                _this.relevantEvents = core.getRelevantEvents(calendar.state.eventStore, _this.eventRange.instance.instanceId);\n                _this.draggingSeg = _this.querySeg(ev);\n                calendar.unselect();\n                calendar.publiclyTrigger('eventResizeStart', [\n                    {\n                        el: _this.draggingSeg.el,\n                        event: new core.EventApi(calendar, eventRange.def, eventRange.instance),\n                        jsEvent: ev.origEvent,\n                        view: view\n                    }\n                ]);\n            };\n            _this.handleHitUpdate = function (hit, isFinal, ev) {\n                var calendar = _this.component.context.calendar;\n                var relevantEvents = _this.relevantEvents;\n                var initialHit = _this.hitDragging.initialHit;\n                var eventInstance = _this.eventRange.instance;\n                var mutation = null;\n                var mutatedRelevantEvents = null;\n                var isInvalid = false;\n                var interaction = {\n                    affectedEvents: relevantEvents,\n                    mutatedEvents: core.createEmptyEventStore(),\n                    isEvent: true,\n                    origSeg: _this.draggingSeg\n                };\n                if (hit) {\n                    mutation = computeMutation(initialHit, hit, ev.subjectEl.classList.contains('fc-start-resizer'), eventInstance.range, calendar.pluginSystem.hooks.eventResizeJoinTransforms);\n                }\n                if (mutation) {\n                    mutatedRelevantEvents = core.applyMutationToEventStore(relevantEvents, calendar.eventUiBases, mutation, calendar);\n                    interaction.mutatedEvents = mutatedRelevantEvents;\n                    if (!_this.component.isInteractionValid(interaction)) {\n                        isInvalid = true;\n                        mutation = null;\n                        mutatedRelevantEvents = null;\n                        interaction.mutatedEvents = null;\n                    }\n                }\n                if (mutatedRelevantEvents) {\n                    calendar.dispatch({\n                        type: 'SET_EVENT_RESIZE',\n                        state: interaction\n                    });\n                }\n                else {\n                    calendar.dispatch({ type: 'UNSET_EVENT_RESIZE' });\n                }\n                if (!isInvalid) {\n                    core.enableCursor();\n                }\n                else {\n                    core.disableCursor();\n                }\n                if (!isFinal) {\n                    if (mutation && isHitsEqual(initialHit, hit)) {\n                        mutation = null;\n                    }\n                    _this.validMutation = mutation;\n                    _this.mutatedRelevantEvents = mutatedRelevantEvents;\n                }\n            };\n            _this.handleDragEnd = function (ev) {\n                var _a = _this.component.context, calendar = _a.calendar, view = _a.view;\n                var eventDef = _this.eventRange.def;\n                var eventInstance = _this.eventRange.instance;\n                var eventApi = new core.EventApi(calendar, eventDef, eventInstance);\n                var relevantEvents = _this.relevantEvents;\n                var mutatedRelevantEvents = _this.mutatedRelevantEvents;\n                calendar.publiclyTrigger('eventResizeStop', [\n                    {\n                        el: _this.draggingSeg.el,\n                        event: eventApi,\n                        jsEvent: ev.origEvent,\n                        view: view\n                    }\n                ]);\n                if (_this.validMutation) {\n                    calendar.dispatch({\n                        type: 'MERGE_EVENTS',\n                        eventStore: mutatedRelevantEvents\n                    });\n                    calendar.publiclyTrigger('eventResize', [\n                        {\n                            el: _this.draggingSeg.el,\n                            startDelta: _this.validMutation.startDelta || core.createDuration(0),\n                            endDelta: _this.validMutation.endDelta || core.createDuration(0),\n                            prevEvent: eventApi,\n                            event: new core.EventApi(// the data AFTER the mutation\n                            calendar, mutatedRelevantEvents.defs[eventDef.defId], eventInstance ? mutatedRelevantEvents.instances[eventInstance.instanceId] : null),\n                            revert: function () {\n                                calendar.dispatch({\n                                    type: 'MERGE_EVENTS',\n                                    eventStore: relevantEvents\n                                });\n                            },\n                            jsEvent: ev.origEvent,\n                            view: view\n                        }\n                    ]);\n                }\n                else {\n                    calendar.publiclyTrigger('_noEventResize');\n                }\n                // reset all internal state\n                _this.draggingSeg = null;\n                _this.relevantEvents = null;\n                _this.validMutation = null;\n                // okay to keep eventInstance around. useful to set it in handlePointerDown\n            };\n            var component = settings.component;\n            var dragging = _this.dragging = new FeaturefulElementDragging(component.el);\n            dragging.pointer.selector = '.fc-resizer';\n            dragging.touchScrollAllowed = false;\n            dragging.autoScroller.isEnabled = component.context.options.dragScroll;\n            var hitDragging = _this.hitDragging = new HitDragging(_this.dragging, core.interactionSettingsToStore(settings));\n            hitDragging.emitter.on('pointerdown', _this.handlePointerDown);\n            hitDragging.emitter.on('dragstart', _this.handleDragStart);\n            hitDragging.emitter.on('hitupdate', _this.handleHitUpdate);\n            hitDragging.emitter.on('dragend', _this.handleDragEnd);\n            return _this;\n        }\n        EventDragging.prototype.destroy = function () {\n            this.dragging.destroy();\n        };\n        EventDragging.prototype.querySeg = function (ev) {\n            return core.getElSeg(core.elementClosest(ev.subjectEl, this.component.fgSegSelector));\n        };\n        return EventDragging;\n    }(core.Interaction));\n    function computeMutation(hit0, hit1, isFromStart, instanceRange, transforms) {\n        var dateEnv = hit0.component.context.dateEnv;\n        var date0 = hit0.dateSpan.range.start;\n        var date1 = hit1.dateSpan.range.start;\n        var delta = core.diffDates(date0, date1, dateEnv, hit0.component.largeUnit);\n        var props = {};\n        for (var _i = 0, transforms_1 = transforms; _i < transforms_1.length; _i++) {\n            var transform = transforms_1[_i];\n            var res = transform(hit0, hit1);\n            if (res === false) {\n                return null;\n            }\n            else if (res) {\n                __assign(props, res);\n            }\n        }\n        if (isFromStart) {\n            if (dateEnv.add(instanceRange.start, delta) < instanceRange.end) {\n                props.startDelta = delta;\n                return props;\n            }\n        }\n        else {\n            if (dateEnv.add(instanceRange.end, delta) > instanceRange.start) {\n                props.endDelta = delta;\n                return props;\n            }\n        }\n        return null;\n    }\n\n    var UnselectAuto = /** @class */ (function () {\n        function UnselectAuto(calendar) {\n            var _this = this;\n            this.isRecentPointerDateSelect = false; // wish we could use a selector to detect date selection, but uses hit system\n            this.onSelect = function (selectInfo) {\n                if (selectInfo.jsEvent) {\n                    _this.isRecentPointerDateSelect = true;\n                }\n            };\n            this.onDocumentPointerUp = function (pev) {\n                var _a = _this, calendar = _a.calendar, documentPointer = _a.documentPointer;\n                var state = calendar.state;\n                // touch-scrolling should never unfocus any type of selection\n                if (!documentPointer.wasTouchScroll) {\n                    if (state.dateSelection && // an existing date selection?\n                        !_this.isRecentPointerDateSelect // a new pointer-initiated date selection since last onDocumentPointerUp?\n                    ) {\n                        var unselectAuto = calendar.viewOpt('unselectAuto');\n                        var unselectCancel = calendar.viewOpt('unselectCancel');\n                        if (unselectAuto && (!unselectAuto || !core.elementClosest(documentPointer.downEl, unselectCancel))) {\n                            calendar.unselect(pev);\n                        }\n                    }\n                    if (state.eventSelection && // an existing event selected?\n                        !core.elementClosest(documentPointer.downEl, EventDragging.SELECTOR) // interaction DIDN'T start on an event\n                    ) {\n                        calendar.dispatch({ type: 'UNSELECT_EVENT' });\n                    }\n                }\n                _this.isRecentPointerDateSelect = false;\n            };\n            this.calendar = calendar;\n            var documentPointer = this.documentPointer = new PointerDragging(document);\n            documentPointer.shouldIgnoreMove = true;\n            documentPointer.shouldWatchScroll = false;\n            documentPointer.emitter.on('pointerup', this.onDocumentPointerUp);\n            /*\n            TODO: better way to know about whether there was a selection with the pointer\n            */\n            calendar.on('select', this.onSelect);\n        }\n        UnselectAuto.prototype.destroy = function () {\n            this.calendar.off('select', this.onSelect);\n            this.documentPointer.destroy();\n        };\n        return UnselectAuto;\n    }());\n\n    /*\n    Given an already instantiated draggable object for one-or-more elements,\n    Interprets any dragging as an attempt to drag an events that lives outside\n    of a calendar onto a calendar.\n    */\n    var ExternalElementDragging = /** @class */ (function () {\n        function ExternalElementDragging(dragging, suppliedDragMeta) {\n            var _this = this;\n            this.receivingCalendar = null;\n            this.droppableEvent = null; // will exist for all drags, even if create:false\n            this.suppliedDragMeta = null;\n            this.dragMeta = null;\n            this.handleDragStart = function (ev) {\n                _this.dragMeta = _this.buildDragMeta(ev.subjectEl);\n            };\n            this.handleHitUpdate = function (hit, isFinal, ev) {\n                var dragging = _this.hitDragging.dragging;\n                var receivingCalendar = null;\n                var droppableEvent = null;\n                var isInvalid = false;\n                var interaction = {\n                    affectedEvents: core.createEmptyEventStore(),\n                    mutatedEvents: core.createEmptyEventStore(),\n                    isEvent: _this.dragMeta.create,\n                    origSeg: null\n                };\n                if (hit) {\n                    receivingCalendar = hit.component.context.calendar;\n                    if (_this.canDropElOnCalendar(ev.subjectEl, receivingCalendar)) {\n                        droppableEvent = computeEventForDateSpan(hit.dateSpan, _this.dragMeta, receivingCalendar);\n                        interaction.mutatedEvents = core.eventTupleToStore(droppableEvent);\n                        isInvalid = !core.isInteractionValid(interaction, receivingCalendar);\n                        if (isInvalid) {\n                            interaction.mutatedEvents = core.createEmptyEventStore();\n                            droppableEvent = null;\n                        }\n                    }\n                }\n                _this.displayDrag(receivingCalendar, interaction);\n                // show mirror if no already-rendered mirror element OR if we are shutting down the mirror (?)\n                // TODO: wish we could somehow wait for dispatch to guarantee render\n                dragging.setMirrorIsVisible(isFinal || !droppableEvent || !document.querySelector('.fc-mirror'));\n                if (!isInvalid) {\n                    core.enableCursor();\n                }\n                else {\n                    core.disableCursor();\n                }\n                if (!isFinal) {\n                    dragging.setMirrorNeedsRevert(!droppableEvent);\n                    _this.receivingCalendar = receivingCalendar;\n                    _this.droppableEvent = droppableEvent;\n                }\n            };\n            this.handleDragEnd = function (pev) {\n                var _a = _this, receivingCalendar = _a.receivingCalendar, droppableEvent = _a.droppableEvent;\n                _this.clearDrag();\n                if (receivingCalendar && droppableEvent) {\n                    var finalHit = _this.hitDragging.finalHit;\n                    var finalView = finalHit.component.context.view;\n                    var dragMeta = _this.dragMeta;\n                    var arg = __assign({}, receivingCalendar.buildDatePointApi(finalHit.dateSpan), { draggedEl: pev.subjectEl, jsEvent: pev.origEvent, view: finalView });\n                    receivingCalendar.publiclyTrigger('drop', [arg]);\n                    if (dragMeta.create) {\n                        receivingCalendar.dispatch({\n                            type: 'MERGE_EVENTS',\n                            eventStore: core.eventTupleToStore(droppableEvent)\n                        });\n                        if (pev.isTouch) {\n                            receivingCalendar.dispatch({\n                                type: 'SELECT_EVENT',\n                                eventInstanceId: droppableEvent.instance.instanceId\n                            });\n                        }\n                        // signal that an external event landed\n                        receivingCalendar.publiclyTrigger('eventReceive', [\n                            {\n                                draggedEl: pev.subjectEl,\n                                event: new core.EventApi(receivingCalendar, droppableEvent.def, droppableEvent.instance),\n                                view: finalView\n                            }\n                        ]);\n                    }\n                }\n                _this.receivingCalendar = null;\n                _this.droppableEvent = null;\n            };\n            var hitDragging = this.hitDragging = new HitDragging(dragging, core.interactionSettingsStore);\n            hitDragging.requireInitial = false; // will start outside of a component\n            hitDragging.emitter.on('dragstart', this.handleDragStart);\n            hitDragging.emitter.on('hitupdate', this.handleHitUpdate);\n            hitDragging.emitter.on('dragend', this.handleDragEnd);\n            this.suppliedDragMeta = suppliedDragMeta;\n        }\n        ExternalElementDragging.prototype.buildDragMeta = function (subjectEl) {\n            if (typeof this.suppliedDragMeta === 'object') {\n                return core.parseDragMeta(this.suppliedDragMeta);\n            }\n            else if (typeof this.suppliedDragMeta === 'function') {\n                return core.parseDragMeta(this.suppliedDragMeta(subjectEl));\n            }\n            else {\n                return getDragMetaFromEl(subjectEl);\n            }\n        };\n        ExternalElementDragging.prototype.displayDrag = function (nextCalendar, state) {\n            var prevCalendar = this.receivingCalendar;\n            if (prevCalendar && prevCalendar !== nextCalendar) {\n                prevCalendar.dispatch({ type: 'UNSET_EVENT_DRAG' });\n            }\n            if (nextCalendar) {\n                nextCalendar.dispatch({ type: 'SET_EVENT_DRAG', state: state });\n            }\n        };\n        ExternalElementDragging.prototype.clearDrag = function () {\n            if (this.receivingCalendar) {\n                this.receivingCalendar.dispatch({ type: 'UNSET_EVENT_DRAG' });\n            }\n        };\n        ExternalElementDragging.prototype.canDropElOnCalendar = function (el, receivingCalendar) {\n            var dropAccept = receivingCalendar.opt('dropAccept');\n            if (typeof dropAccept === 'function') {\n                return dropAccept(el);\n            }\n            else if (typeof dropAccept === 'string' && dropAccept) {\n                return Boolean(core.elementMatches(el, dropAccept));\n            }\n            return true;\n        };\n        return ExternalElementDragging;\n    }());\n    // Utils for computing event store from the DragMeta\n    // ----------------------------------------------------------------------------------------------------\n    function computeEventForDateSpan(dateSpan, dragMeta, calendar) {\n        var defProps = __assign({}, dragMeta.leftoverProps);\n        for (var _i = 0, _a = calendar.pluginSystem.hooks.externalDefTransforms; _i < _a.length; _i++) {\n            var transform = _a[_i];\n            __assign(defProps, transform(dateSpan, dragMeta));\n        }\n        var def = core.parseEventDef(defProps, dragMeta.sourceId, dateSpan.allDay, calendar.opt('forceEventDuration') || Boolean(dragMeta.duration), // hasEnd\n        calendar);\n        var start = dateSpan.range.start;\n        // only rely on time info if drop zone is all-day,\n        // otherwise, we already know the time\n        if (dateSpan.allDay && dragMeta.startTime) {\n            start = calendar.dateEnv.add(start, dragMeta.startTime);\n        }\n        var end = dragMeta.duration ?\n            calendar.dateEnv.add(start, dragMeta.duration) :\n            calendar.getDefaultEventEnd(dateSpan.allDay, start);\n        var instance = core.createEventInstance(def.defId, { start: start, end: end });\n        return { def: def, instance: instance };\n    }\n    // Utils for extracting data from element\n    // ----------------------------------------------------------------------------------------------------\n    function getDragMetaFromEl(el) {\n        var str = getEmbeddedElData(el, 'event');\n        var obj = str ?\n            JSON.parse(str) :\n            { create: false }; // if no embedded data, assume no event creation\n        return core.parseDragMeta(obj);\n    }\n    core.config.dataAttrPrefix = '';\n    function getEmbeddedElData(el, name) {\n        var prefix = core.config.dataAttrPrefix;\n        var prefixedName = (prefix ? prefix + '-' : '') + name;\n        return el.getAttribute('data-' + prefixedName) || '';\n    }\n\n    /*\n    Makes an element (that is *external* to any calendar) draggable.\n    Can pass in data that determines how an event will be created when dropped onto a calendar.\n    Leverages FullCalendar's internal drag-n-drop functionality WITHOUT a third-party drag system.\n    */\n    var ExternalDraggable = /** @class */ (function () {\n        function ExternalDraggable(el, settings) {\n            var _this = this;\n            if (settings === void 0) { settings = {}; }\n            this.handlePointerDown = function (ev) {\n                var dragging = _this.dragging;\n                var _a = _this.settings, minDistance = _a.minDistance, longPressDelay = _a.longPressDelay;\n                dragging.minDistance =\n                    minDistance != null ?\n                        minDistance :\n                        (ev.isTouch ? 0 : core.globalDefaults.eventDragMinDistance);\n                dragging.delay =\n                    ev.isTouch ? // TODO: eventually read eventLongPressDelay instead vvv\n                        (longPressDelay != null ? longPressDelay : core.globalDefaults.longPressDelay) :\n                        0;\n            };\n            this.handleDragStart = function (ev) {\n                if (ev.isTouch &&\n                    _this.dragging.delay &&\n                    ev.subjectEl.classList.contains('fc-event')) {\n                    _this.dragging.mirror.getMirrorEl().classList.add('fc-selected');\n                }\n            };\n            this.settings = settings;\n            var dragging = this.dragging = new FeaturefulElementDragging(el);\n            dragging.touchScrollAllowed = false;\n            if (settings.itemSelector != null) {\n                dragging.pointer.selector = settings.itemSelector;\n            }\n            if (settings.appendTo != null) {\n                dragging.mirror.parentNode = settings.appendTo; // TODO: write tests\n            }\n            dragging.emitter.on('pointerdown', this.handlePointerDown);\n            dragging.emitter.on('dragstart', this.handleDragStart);\n            new ExternalElementDragging(dragging, settings.eventData);\n        }\n        ExternalDraggable.prototype.destroy = function () {\n            this.dragging.destroy();\n        };\n        return ExternalDraggable;\n    }());\n\n    /*\n    Detects when a *THIRD-PARTY* drag-n-drop system interacts with elements.\n    The third-party system is responsible for drawing the visuals effects of the drag.\n    This class simply monitors for pointer movements and fires events.\n    It also has the ability to hide the moving element (the \"mirror\") during the drag.\n    */\n    var InferredElementDragging = /** @class */ (function (_super) {\n        __extends(InferredElementDragging, _super);\n        function InferredElementDragging(containerEl) {\n            var _this = _super.call(this, containerEl) || this;\n            _this.shouldIgnoreMove = false;\n            _this.mirrorSelector = '';\n            _this.currentMirrorEl = null;\n            _this.handlePointerDown = function (ev) {\n                _this.emitter.trigger('pointerdown', ev);\n                if (!_this.shouldIgnoreMove) {\n                    // fire dragstart right away. does not support delay or min-distance\n                    _this.emitter.trigger('dragstart', ev);\n                }\n            };\n            _this.handlePointerMove = function (ev) {\n                if (!_this.shouldIgnoreMove) {\n                    _this.emitter.trigger('dragmove', ev);\n                }\n            };\n            _this.handlePointerUp = function (ev) {\n                _this.emitter.trigger('pointerup', ev);\n                if (!_this.shouldIgnoreMove) {\n                    // fire dragend right away. does not support a revert animation\n                    _this.emitter.trigger('dragend', ev);\n                }\n            };\n            var pointer = _this.pointer = new PointerDragging(containerEl);\n            pointer.emitter.on('pointerdown', _this.handlePointerDown);\n            pointer.emitter.on('pointermove', _this.handlePointerMove);\n            pointer.emitter.on('pointerup', _this.handlePointerUp);\n            return _this;\n        }\n        InferredElementDragging.prototype.destroy = function () {\n            this.pointer.destroy();\n        };\n        InferredElementDragging.prototype.setIgnoreMove = function (bool) {\n            this.shouldIgnoreMove = bool;\n        };\n        InferredElementDragging.prototype.setMirrorIsVisible = function (bool) {\n            if (bool) {\n                // restore a previously hidden element.\n                // use the reference in case the selector class has already been removed.\n                if (this.currentMirrorEl) {\n                    this.currentMirrorEl.style.visibility = '';\n                    this.currentMirrorEl = null;\n                }\n            }\n            else {\n                var mirrorEl = this.mirrorSelector ?\n                    document.querySelector(this.mirrorSelector) :\n                    null;\n                if (mirrorEl) {\n                    this.currentMirrorEl = mirrorEl;\n                    mirrorEl.style.visibility = 'hidden';\n                }\n            }\n        };\n        return InferredElementDragging;\n    }(core.ElementDragging));\n\n    /*\n    Bridges third-party drag-n-drop systems with FullCalendar.\n    Must be instantiated and destroyed by caller.\n    */\n    var ThirdPartyDraggable = /** @class */ (function () {\n        function ThirdPartyDraggable(containerOrSettings, settings) {\n            var containerEl = document;\n            if (\n            // wish we could just test instanceof EventTarget, but doesn't work in IE11\n            containerOrSettings === document ||\n                containerOrSettings instanceof Element) {\n                containerEl = containerOrSettings;\n                settings = settings || {};\n            }\n            else {\n                settings = (containerOrSettings || {});\n            }\n            var dragging = this.dragging = new InferredElementDragging(containerEl);\n            if (typeof settings.itemSelector === 'string') {\n                dragging.pointer.selector = settings.itemSelector;\n            }\n            else if (containerEl === document) {\n                dragging.pointer.selector = '[data-event]';\n            }\n            if (typeof settings.mirrorSelector === 'string') {\n                dragging.mirrorSelector = settings.mirrorSelector;\n            }\n            new ExternalElementDragging(dragging, settings.eventData);\n        }\n        ThirdPartyDraggable.prototype.destroy = function () {\n            this.dragging.destroy();\n        };\n        return ThirdPartyDraggable;\n    }());\n\n    var main = core.createPlugin({\n        componentInteractions: [DateClicking, DateSelecting, EventDragging, EventDragging$1],\n        calendarInteractions: [UnselectAuto],\n        elementDraggingImpl: FeaturefulElementDragging\n    });\n\n    exports.Draggable = ExternalDraggable;\n    exports.FeaturefulElementDragging = FeaturefulElementDragging;\n    exports.PointerDragging = PointerDragging;\n    exports.ThirdPartyDraggable = ThirdPartyDraggable;\n    exports.default = main;\n\n    Object.defineProperty(exports, '__esModule', { value: true });\n\n}));\n"

/***/ },

/***/ 1089
(module) {

module.exports = "/*!\nFullCalendar List View Plugin v4.4.2\nDocs & License: https://fullcalendar.io/\n(c) 2019 Adam Shaw\n*/\n\n(function (global, factory) {\n    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@fullcalendar/core')) :\n    typeof define === 'function' && define.amd ? define(['exports', '@fullcalendar/core'], factory) :\n    (global = global || self, factory(global.FullCalendarList = {}, global.FullCalendar));\n}(this, function (exports, core) { 'use strict';\n\n    /*! *****************************************************************************\r\n    Copyright (c) Microsoft Corporation.\r\n\r\n    Permission to use, copy, modify, and/or distribute this software for any\r\n    purpose with or without fee is hereby granted.\r\n\r\n    THE SOFTWARE IS PROVIDED \"AS IS\" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH\r\n    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY\r\n    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,\r\n    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM\r\n    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR\r\n    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR\r\n    PERFORMANCE OF THIS SOFTWARE.\r\n    ***************************************************************************** */\r\n    /* global Reflect, Promise */\r\n\r\n    var extendStatics = function(d, b) {\r\n        extendStatics = Object.setPrototypeOf ||\r\n            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||\r\n            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };\r\n        return extendStatics(d, b);\r\n    };\r\n\r\n    function __extends(d, b) {\r\n        extendStatics(d, b);\r\n        function __() { this.constructor = d; }\r\n        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());\r\n    }\n\n    var ListEventRenderer = /** @class */ (function (_super) {\n        __extends(ListEventRenderer, _super);\n        function ListEventRenderer(listView) {\n            var _this = _super.call(this) || this;\n            _this.listView = listView;\n            return _this;\n        }\n        ListEventRenderer.prototype.attachSegs = function (segs) {\n            if (!segs.length) {\n                this.listView.renderEmptyMessage();\n            }\n            else {\n                this.listView.renderSegList(segs);\n            }\n        };\n        ListEventRenderer.prototype.detachSegs = function () {\n        };\n        // generates the HTML for a single event row\n        ListEventRenderer.prototype.renderSegHtml = function (seg) {\n            var _a = this.context, theme = _a.theme, options = _a.options;\n            var eventRange = seg.eventRange;\n            var eventDef = eventRange.def;\n            var eventInstance = eventRange.instance;\n            var eventUi = eventRange.ui;\n            var url = eventDef.url;\n            var classes = ['fc-list-item'].concat(eventUi.classNames);\n            var bgColor = eventUi.backgroundColor;\n            var timeHtml;\n            if (eventDef.allDay) {\n                timeHtml = core.getAllDayHtml(options);\n            }\n            else if (core.isMultiDayRange(eventRange.range)) {\n                if (seg.isStart) {\n                    timeHtml = core.htmlEscape(this._getTimeText(eventInstance.range.start, seg.end, false // allDay\n                    ));\n                }\n                else if (seg.isEnd) {\n                    timeHtml = core.htmlEscape(this._getTimeText(seg.start, eventInstance.range.end, false // allDay\n                    ));\n                }\n                else { // inner segment that lasts the whole day\n                    timeHtml = core.getAllDayHtml(options);\n                }\n            }\n            else {\n                // Display the normal time text for the *event's* times\n                timeHtml = core.htmlEscape(this.getTimeText(eventRange));\n            }\n            if (url) {\n                classes.push('fc-has-url');\n            }\n            return '<tr class=\"' + classes.join(' ') + '\">' +\n                (this.displayEventTime ?\n                    '<td class=\"fc-list-item-time ' + theme.getClass('widgetContent') + '\">' +\n                        (timeHtml || '') +\n                        '</td>' :\n                    '') +\n                '<td class=\"fc-list-item-marker ' + theme.getClass('widgetContent') + '\">' +\n                '<span class=\"fc-event-dot\"' +\n                (bgColor ?\n                    ' style=\"background-color:' + bgColor + '\"' :\n                    '') +\n                '></span>' +\n                '</td>' +\n                '<td class=\"fc-list-item-title ' + theme.getClass('widgetContent') + '\">' +\n                '<a' + (url ? ' href=\"' + core.htmlEscape(url) + '\"' : '') + '>' +\n                core.htmlEscape(eventDef.title || '') +\n                '</a>' +\n                '</td>' +\n                '</tr>';\n        };\n        // like \"4:00am\"\n        ListEventRenderer.prototype.computeEventTimeFormat = function () {\n            return {\n                hour: 'numeric',\n                minute: '2-digit',\n                meridiem: 'short'\n            };\n        };\n        return ListEventRenderer;\n    }(core.FgEventRenderer));\n\n    /*\n    Responsible for the scroller, and forwarding event-related actions into the \"grid\".\n    */\n    var ListView = /** @class */ (function (_super) {\n        __extends(ListView, _super);\n        function ListView(viewSpec, parentEl) {\n            var _this = _super.call(this, viewSpec, parentEl) || this;\n            _this.computeDateVars = core.memoize(computeDateVars);\n            _this.eventStoreToSegs = core.memoize(_this._eventStoreToSegs);\n            _this.renderSkeleton = core.memoizeRendering(_this._renderSkeleton, _this._unrenderSkeleton);\n            var eventRenderer = _this.eventRenderer = new ListEventRenderer(_this);\n            _this.renderContent = core.memoizeRendering(eventRenderer.renderSegs.bind(eventRenderer), eventRenderer.unrender.bind(eventRenderer), [_this.renderSkeleton]);\n            return _this;\n        }\n        ListView.prototype.firstContext = function (context) {\n            context.calendar.registerInteractiveComponent(this, {\n                el: this.el\n                // TODO: make aware that it doesn't do Hits\n            });\n        };\n        ListView.prototype.render = function (props, context) {\n            _super.prototype.render.call(this, props, context);\n            var _a = this.computeDateVars(props.dateProfile), dayDates = _a.dayDates, dayRanges = _a.dayRanges;\n            this.dayDates = dayDates;\n            this.renderSkeleton(context);\n            this.renderContent(context, this.eventStoreToSegs(props.eventStore, props.eventUiBases, dayRanges));\n        };\n        ListView.prototype.destroy = function () {\n            _super.prototype.destroy.call(this);\n            this.renderSkeleton.unrender();\n            this.renderContent.unrender();\n            this.context.calendar.unregisterInteractiveComponent(this);\n        };\n        ListView.prototype._renderSkeleton = function (context) {\n            var theme = context.theme;\n            this.el.classList.add('fc-list-view');\n            var listViewClassNames = (theme.getClass('listView') || '').split(' '); // wish we didn't have to do this\n            for (var _i = 0, listViewClassNames_1 = listViewClassNames; _i < listViewClassNames_1.length; _i++) {\n                var listViewClassName = listViewClassNames_1[_i];\n                if (listViewClassName) { // in case input was empty string\n                    this.el.classList.add(listViewClassName);\n                }\n            }\n            this.scroller = new core.ScrollComponent('hidden', // overflow x\n            'auto' // overflow y\n            );\n            this.el.appendChild(this.scroller.el);\n            this.contentEl = this.scroller.el; // shortcut\n        };\n        ListView.prototype._unrenderSkeleton = function () {\n            // TODO: remove classNames\n            this.scroller.destroy(); // will remove the Grid too\n        };\n        ListView.prototype.updateSize = function (isResize, viewHeight, isAuto) {\n            _super.prototype.updateSize.call(this, isResize, viewHeight, isAuto);\n            this.eventRenderer.computeSizes(isResize);\n            this.eventRenderer.assignSizes(isResize);\n            this.scroller.clear(); // sets height to 'auto' and clears overflow\n            if (!isAuto) {\n                this.scroller.setHeight(this.computeScrollerHeight(viewHeight));\n            }\n        };\n        ListView.prototype.computeScrollerHeight = function (viewHeight) {\n            return viewHeight -\n                core.subtractInnerElHeight(this.el, this.scroller.el); // everything that's NOT the scroller\n        };\n        ListView.prototype._eventStoreToSegs = function (eventStore, eventUiBases, dayRanges) {\n            return this.eventRangesToSegs(core.sliceEventStore(eventStore, eventUiBases, this.props.dateProfile.activeRange, this.context.nextDayThreshold).fg, dayRanges);\n        };\n        ListView.prototype.eventRangesToSegs = function (eventRanges, dayRanges) {\n            var segs = [];\n            for (var _i = 0, eventRanges_1 = eventRanges; _i < eventRanges_1.length; _i++) {\n                var eventRange = eventRanges_1[_i];\n                segs.push.apply(segs, this.eventRangeToSegs(eventRange, dayRanges));\n            }\n            return segs;\n        };\n        ListView.prototype.eventRangeToSegs = function (eventRange, dayRanges) {\n            var _a = this.context, dateEnv = _a.dateEnv, nextDayThreshold = _a.nextDayThreshold;\n            var range = eventRange.range;\n            var allDay = eventRange.def.allDay;\n            var dayIndex;\n            var segRange;\n            var seg;\n            var segs = [];\n            for (dayIndex = 0; dayIndex < dayRanges.length; dayIndex++) {\n                segRange = core.intersectRanges(range, dayRanges[dayIndex]);\n                if (segRange) {\n                    seg = {\n                        component: this,\n                        eventRange: eventRange,\n                        start: segRange.start,\n                        end: segRange.end,\n                        isStart: eventRange.isStart && segRange.start.valueOf() === range.start.valueOf(),\n                        isEnd: eventRange.isEnd && segRange.end.valueOf() === range.end.valueOf(),\n                        dayIndex: dayIndex\n                    };\n                    segs.push(seg);\n                    // detect when range won't go fully into the next day,\n                    // and mutate the latest seg to the be the end.\n                    if (!seg.isEnd && !allDay &&\n                        dayIndex + 1 < dayRanges.length &&\n                        range.end <\n                            dateEnv.add(dayRanges[dayIndex + 1].start, nextDayThreshold)) {\n                        seg.end = range.end;\n                        seg.isEnd = true;\n                        break;\n                    }\n                }\n            }\n            return segs;\n        };\n        ListView.prototype.renderEmptyMessage = function () {\n            this.contentEl.innerHTML =\n                '<div class=\"fc-list-empty-wrap2\">' + // TODO: try less wraps\n                    '<div class=\"fc-list-empty-wrap1\">' +\n                    '<div class=\"fc-list-empty\">' +\n                    core.htmlEscape(this.context.options.noEventsMessage) +\n                    '</div>' +\n                    '</div>' +\n                    '</div>';\n        };\n        // called by ListEventRenderer\n        ListView.prototype.renderSegList = function (allSegs) {\n            var theme = this.context.theme;\n            var segsByDay = this.groupSegsByDay(allSegs); // sparse array\n            var dayIndex;\n            var daySegs;\n            var i;\n            var tableEl = core.htmlToElement('<table class=\"fc-list-table ' + theme.getClass('tableList') + '\"><tbody></tbody></table>');\n            var tbodyEl = tableEl.querySelector('tbody');\n            for (dayIndex = 0; dayIndex < segsByDay.length; dayIndex++) {\n                daySegs = segsByDay[dayIndex];\n                if (daySegs) { // sparse array, so might be undefined\n                    // append a day header\n                    tbodyEl.appendChild(this.buildDayHeaderRow(this.dayDates[dayIndex]));\n                    daySegs = this.eventRenderer.sortEventSegs(daySegs);\n                    for (i = 0; i < daySegs.length; i++) {\n                        tbodyEl.appendChild(daySegs[i].el); // append event row\n                    }\n                }\n            }\n            this.contentEl.innerHTML = '';\n            this.contentEl.appendChild(tableEl);\n        };\n        // Returns a sparse array of arrays, segs grouped by their dayIndex\n        ListView.prototype.groupSegsByDay = function (segs) {\n            var segsByDay = []; // sparse array\n            var i;\n            var seg;\n            for (i = 0; i < segs.length; i++) {\n                seg = segs[i];\n                (segsByDay[seg.dayIndex] || (segsByDay[seg.dayIndex] = []))\n                    .push(seg);\n            }\n            return segsByDay;\n        };\n        // generates the HTML for the day headers that live amongst the event rows\n        ListView.prototype.buildDayHeaderRow = function (dayDate) {\n            var _a = this.context, theme = _a.theme, dateEnv = _a.dateEnv, options = _a.options;\n            var mainFormat = core.createFormatter(options.listDayFormat); // TODO: cache\n            var altFormat = core.createFormatter(options.listDayAltFormat); // TODO: cache\n            return core.createElement('tr', {\n                className: 'fc-list-heading',\n                'data-date': dateEnv.formatIso(dayDate, { omitTime: true })\n            }, '<td class=\"' + (theme.getClass('tableListHeading') ||\n                theme.getClass('widgetHeader')) + '\" colspan=\"3\">' +\n                (mainFormat ?\n                    core.buildGotoAnchorHtml(options, dateEnv, dayDate, { 'class': 'fc-list-heading-main' }, core.htmlEscape(dateEnv.format(dayDate, mainFormat)) // inner HTML\n                    ) :\n                    '') +\n                (altFormat ?\n                    core.buildGotoAnchorHtml(options, dateEnv, dayDate, { 'class': 'fc-list-heading-alt' }, core.htmlEscape(dateEnv.format(dayDate, altFormat)) // inner HTML\n                    ) :\n                    '') +\n                '</td>');\n        };\n        return ListView;\n    }(core.View));\n    ListView.prototype.fgSegSelector = '.fc-list-item'; // which elements accept event actions\n    function computeDateVars(dateProfile) {\n        var dayStart = core.startOfDay(dateProfile.renderRange.start);\n        var viewEnd = dateProfile.renderRange.end;\n        var dayDates = [];\n        var dayRanges = [];\n        while (dayStart < viewEnd) {\n            dayDates.push(dayStart);\n            dayRanges.push({\n                start: dayStart,\n                end: core.addDays(dayStart, 1)\n            });\n            dayStart = core.addDays(dayStart, 1);\n        }\n        return { dayDates: dayDates, dayRanges: dayRanges };\n    }\n\n    var main = core.createPlugin({\n        views: {\n            list: {\n                class: ListView,\n                buttonTextKey: 'list',\n                listDayFormat: { month: 'long', day: 'numeric', year: 'numeric' } // like \"January 1, 2016\"\n            },\n            listDay: {\n                type: 'list',\n                duration: { days: 1 },\n                listDayFormat: { weekday: 'long' } // day-of-week is all we need. full date is probably in header\n            },\n            listWeek: {\n                type: 'list',\n                duration: { weeks: 1 },\n                listDayFormat: { weekday: 'long' },\n                listDayAltFormat: { month: 'long', day: 'numeric', year: 'numeric' }\n            },\n            listMonth: {\n                type: 'list',\n                duration: { month: 1 },\n                listDayAltFormat: { weekday: 'long' } // day-of-week is nice-to-have\n            },\n            listYear: {\n                type: 'list',\n                duration: { year: 1 },\n                listDayAltFormat: { weekday: 'long' } // day-of-week is nice-to-have\n            }\n        }\n    });\n\n    exports.ListView = ListView;\n    exports.default = main;\n\n    Object.defineProperty(exports, '__esModule', { value: true });\n\n}));\n"

/***/ },

/***/ 1095
(module) {

module.exports = "/*!\nFullCalendar Resources Common Plugin v4.4.2\nDocs & License: https://fullcalendar.io/scheduler\n(c) 2019 Adam Shaw\n*/\n\n(function (global, factory) {\n    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@fullcalendar/core')) :\n    typeof define === 'function' && define.amd ? define(['exports', '@fullcalendar/core'], factory) :\n    (global = global || self, factory(global.FullCalendarResourceCommon = {}, global.FullCalendar));\n}(this, function (exports, core) { 'use strict';\n\n    /*! *****************************************************************************\r\n    Copyright (c) Microsoft Corporation.\r\n\r\n    Permission to use, copy, modify, and/or distribute this software for any\r\n    purpose with or without fee is hereby granted.\r\n\r\n    THE SOFTWARE IS PROVIDED \"AS IS\" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH\r\n    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY\r\n    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,\r\n    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM\r\n    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR\r\n    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR\r\n    PERFORMANCE OF THIS SOFTWARE.\r\n    ***************************************************************************** */\r\n    /* global Reflect, Promise */\r\n\r\n    var extendStatics = function(d, b) {\r\n        extendStatics = Object.setPrototypeOf ||\r\n            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||\r\n            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };\r\n        return extendStatics(d, b);\r\n    };\r\n\r\n    function __extends(d, b) {\r\n        extendStatics(d, b);\r\n        function __() { this.constructor = d; }\r\n        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());\r\n    }\r\n\r\n    var __assign = function() {\r\n        __assign = Object.assign || function __assign(t) {\r\n            for (var s, i = 1, n = arguments.length; i < n; i++) {\r\n                s = arguments[i];\r\n                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];\r\n            }\r\n            return t;\r\n        };\r\n        return __assign.apply(this, arguments);\r\n    };\n\n    function massageEventDragMutation(eventMutation, hit0, hit1) {\n        var resource0 = hit0.dateSpan.resourceId;\n        var resource1 = hit1.dateSpan.resourceId;\n        if (resource0 && resource1 &&\n            resource0 !== resource1) {\n            eventMutation.resourceMutation = {\n                matchResourceId: resource0,\n                setResourceId: resource1\n            };\n        }\n    }\n    /*\n    TODO: all this would be much easier if we were using a hash!\n    */\n    function applyEventDefMutation(eventDef, mutation, calendar) {\n        var resourceMutation = mutation.resourceMutation;\n        if (resourceMutation && computeResourceEditable(eventDef, calendar)) {\n            var index = eventDef.resourceIds.indexOf(resourceMutation.matchResourceId);\n            if (index !== -1) {\n                var resourceIds = eventDef.resourceIds.slice(); // copy\n                resourceIds.splice(index, 1); // remove\n                if (resourceIds.indexOf(resourceMutation.setResourceId) === -1) { // not already in there\n                    resourceIds.push(resourceMutation.setResourceId); // add\n                }\n                eventDef.resourceIds = resourceIds;\n            }\n        }\n    }\n    /*\n    HACK\n    TODO: use EventUi system instead of this\n    */\n    function computeResourceEditable(eventDef, calendar) {\n        var resourceEditable = eventDef.resourceEditable;\n        if (resourceEditable == null) {\n            var source = eventDef.sourceId && calendar.state.eventSources[eventDef.sourceId];\n            if (source) {\n                resourceEditable = source.extendedProps.resourceEditable; // used the Source::extendedProps hack\n            }\n            if (resourceEditable == null) {\n                resourceEditable = calendar.opt('eventResourceEditable');\n                if (resourceEditable == null) {\n                    resourceEditable = calendar.opt('editable'); // TODO: use defaults system instead\n                }\n            }\n        }\n        return resourceEditable;\n    }\n    function transformEventDrop(mutation, calendar) {\n        var resourceMutation = mutation.resourceMutation;\n        if (resourceMutation) {\n            return {\n                oldResource: calendar.getResourceById(resourceMutation.matchResourceId),\n                newResource: calendar.getResourceById(resourceMutation.setResourceId)\n            };\n        }\n        else {\n            return {\n                oldResource: null,\n                newResource: null\n            };\n        }\n    }\n\n    var ResourceDataAdder = /** @class */ (function () {\n        function ResourceDataAdder() {\n            this.filterResources = core.memoize(filterResources);\n        }\n        ResourceDataAdder.prototype.transform = function (viewProps, viewSpec, calendarProps, allOptions) {\n            if (viewSpec.class.needsResourceData) {\n                return {\n                    resourceStore: this.filterResources(calendarProps.resourceStore, allOptions.filterResourcesWithEvents, calendarProps.eventStore, calendarProps.dateProfile.activeRange),\n                    resourceEntityExpansions: calendarProps.resourceEntityExpansions\n                };\n            }\n        };\n        return ResourceDataAdder;\n    }());\n    function filterResources(resourceStore, doFilterResourcesWithEvents, eventStore, activeRange) {\n        if (doFilterResourcesWithEvents) {\n            var instancesInRange = filterEventInstancesInRange(eventStore.instances, activeRange);\n            var hasEvents_1 = computeHasEvents(instancesInRange, eventStore.defs);\n            __assign(hasEvents_1, computeAncestorHasEvents(hasEvents_1, resourceStore));\n            return core.filterHash(resourceStore, function (resource, resourceId) {\n                return hasEvents_1[resourceId];\n            });\n        }\n        else {\n            return resourceStore;\n        }\n    }\n    function filterEventInstancesInRange(eventInstances, activeRange) {\n        return core.filterHash(eventInstances, function (eventInstance) {\n            return core.rangesIntersect(eventInstance.range, activeRange);\n        });\n    }\n    function computeHasEvents(eventInstances, eventDefs) {\n        var hasEvents = {};\n        for (var instanceId in eventInstances) {\n            var instance = eventInstances[instanceId];\n            for (var _i = 0, _a = eventDefs[instance.defId].resourceIds; _i < _a.length; _i++) {\n                var resourceId = _a[_i];\n                hasEvents[resourceId] = true;\n            }\n        }\n        return hasEvents;\n    }\n    /*\n    mark resources as having events if any of their ancestors have them\n    NOTE: resourceStore might not have all the resources that hasEvents{} has keyed\n    */\n    function computeAncestorHasEvents(hasEvents, resourceStore) {\n        var res = {};\n        for (var resourceId in hasEvents) {\n            var resource = void 0;\n            while ((resource = resourceStore[resourceId])) {\n                resourceId = resource.parentId; // now functioning as the parentId\n                if (resourceId) {\n                    res[resourceId] = true;\n                }\n                else {\n                    break;\n                }\n            }\n        }\n        return res;\n    }\n    // for when non-resource view should be given EventUi info (for event coloring/constraints based off of resource data)\n    var ResourceEventConfigAdder = /** @class */ (function () {\n        function ResourceEventConfigAdder() {\n            this.buildResourceEventUis = core.memoizeOutput(buildResourceEventUis, core.isPropsEqual);\n            this.injectResourceEventUis = core.memoize(injectResourceEventUis);\n        }\n        ResourceEventConfigAdder.prototype.transform = function (viewProps, viewSpec, calendarProps) {\n            if (!viewSpec.class.needsResourceData) { // is a non-resource view?\n                return {\n                    eventUiBases: this.injectResourceEventUis(viewProps.eventUiBases, viewProps.eventStore.defs, this.buildResourceEventUis(calendarProps.resourceStore))\n                };\n            }\n        };\n        return ResourceEventConfigAdder;\n    }());\n    function buildResourceEventUis(resourceStore) {\n        return core.mapHash(resourceStore, function (resource) {\n            return resource.ui;\n        });\n    }\n    function injectResourceEventUis(eventUiBases, eventDefs, resourceEventUis) {\n        return core.mapHash(eventUiBases, function (eventUi, defId) {\n            if (defId) { // not the '' key\n                return injectResourceEventUi(eventUi, eventDefs[defId], resourceEventUis);\n            }\n            else {\n                return eventUi;\n            }\n        });\n    }\n    function injectResourceEventUi(origEventUi, eventDef, resourceEventUis) {\n        var parts = [];\n        // first resource takes precedence, which fights with the ordering of combineEventUis, thus the unshifts\n        for (var _i = 0, _a = eventDef.resourceIds; _i < _a.length; _i++) {\n            var resourceId = _a[_i];\n            if (resourceEventUis[resourceId]) {\n                parts.unshift(resourceEventUis[resourceId]);\n            }\n        }\n        parts.unshift(origEventUi);\n        return core.combineEventUis(parts);\n    }\n    // for making sure events that have editable resources are always draggable in resource views\n    function transformIsDraggable(val, eventDef, eventUi, view) {\n        if (!val) {\n            if (view.viewSpec.class.needsResourceData) {\n                if (computeResourceEditable(eventDef, view.context.calendar)) { // yuck\n                    return true;\n                }\n            }\n        }\n        return val;\n    }\n\n    var RESOURCE_SOURCE_PROPS = {\n        id: String\n    };\n    var defs = [];\n    var uid = 0;\n    function registerResourceSourceDef(def) {\n        defs.push(def);\n    }\n    function getResourceSourceDef(id) {\n        return defs[id];\n    }\n    function doesSourceIgnoreRange(source) {\n        return Boolean(defs[source.sourceDefId].ignoreRange);\n    }\n    function parseResourceSource(input) {\n        for (var i = defs.length - 1; i >= 0; i--) { // later-added plugins take precedence\n            var def = defs[i];\n            var meta = def.parseMeta(input);\n            if (meta) {\n                var res = parseResourceSourceProps((typeof input === 'object' && input) ? input : {}, meta, i);\n                res._raw = input;\n                return res;\n            }\n        }\n        return null;\n    }\n    function parseResourceSourceProps(input, meta, sourceDefId) {\n        var props = core.refineProps(input, RESOURCE_SOURCE_PROPS);\n        props.sourceId = String(uid++);\n        props.sourceDefId = sourceDefId;\n        props.meta = meta;\n        props.publicId = props.id;\n        props.isFetching = false;\n        props.latestFetchId = '';\n        props.fetchRange = null;\n        delete props.id;\n        return props;\n    }\n\n    function reduceResourceSource (source, action, dateProfile, calendar) {\n        switch (action.type) {\n            case 'INIT':\n                return createSource(calendar.opt('resources'), calendar);\n            case 'RESET_RESOURCE_SOURCE':\n                return createSource(action.resourceSourceInput, calendar, true);\n            case 'PREV': // TODO: how do we track all actions that affect dateProfile :(\n            case 'NEXT':\n            case 'SET_DATE':\n            case 'SET_VIEW_TYPE':\n                return handleRange(source, dateProfile.activeRange, calendar);\n            case 'RECEIVE_RESOURCES':\n            case 'RECEIVE_RESOURCE_ERROR':\n                return receiveResponse(source, action.fetchId, action.fetchRange);\n            case 'REFETCH_RESOURCES':\n                return fetchSource(source, dateProfile.activeRange, calendar);\n            default:\n                return source;\n        }\n    }\n    var uid$1 = 0;\n    function createSource(input, calendar, forceFetch) {\n        if (input) {\n            var source = parseResourceSource(input);\n            if (forceFetch || !calendar.opt('refetchResourcesOnNavigate')) { // because assumes handleRange will do it later\n                source = fetchSource(source, null, calendar);\n            }\n            return source;\n        }\n        return null;\n    }\n    function handleRange(source, activeRange, calendar) {\n        if (calendar.opt('refetchResourcesOnNavigate') &&\n            !doesSourceIgnoreRange(source) &&\n            (!source.fetchRange || !core.rangesEqual(source.fetchRange, activeRange))) {\n            return fetchSource(source, activeRange, calendar);\n        }\n        else {\n            return source;\n        }\n    }\n    function fetchSource(source, fetchRange, calendar) {\n        var sourceDef = getResourceSourceDef(source.sourceDefId);\n        var fetchId = String(uid$1++);\n        sourceDef.fetch({\n            resourceSource: source,\n            calendar: calendar,\n            range: fetchRange\n        }, function (res) {\n            // HACK\n            // do before calling dispatch in case dispatch renders synchronously\n            calendar.afterSizingTriggers._resourcesRendered = [null]; // fire once\n            calendar.dispatch({\n                type: 'RECEIVE_RESOURCES',\n                fetchId: fetchId,\n                fetchRange: fetchRange,\n                rawResources: res.rawResources\n            });\n        }, function (error) {\n            calendar.dispatch({\n                type: 'RECEIVE_RESOURCE_ERROR',\n                fetchId: fetchId,\n                fetchRange: fetchRange,\n                error: error\n            });\n        });\n        return __assign({}, source, { isFetching: true, latestFetchId: fetchId });\n    }\n    function receiveResponse(source, fetchId, fetchRange) {\n        if (fetchId === source.latestFetchId) {\n            return __assign({}, source, { isFetching: false, fetchRange: fetchRange });\n        }\n        return source;\n    }\n\n    var RESOURCE_PROPS = {\n        id: String,\n        title: String,\n        parentId: String,\n        businessHours: null,\n        children: null,\n        extendedProps: null\n    };\n    var PRIVATE_ID_PREFIX = '_fc:';\n    var uid$2 = 0;\n    /*\n    needs a full store so that it can populate children too\n    */\n    function parseResource(input, parentId, store, calendar) {\n        if (parentId === void 0) { parentId = ''; }\n        var leftovers0 = {};\n        var props = core.refineProps(input, RESOURCE_PROPS, {}, leftovers0);\n        var leftovers1 = {};\n        var ui = core.processScopedUiProps('event', leftovers0, calendar, leftovers1);\n        if (!props.id) {\n            props.id = PRIVATE_ID_PREFIX + (uid$2++);\n        }\n        if (!props.parentId) { // give precedence to the parentId property\n            props.parentId = parentId;\n        }\n        props.businessHours = props.businessHours ? core.parseBusinessHours(props.businessHours, calendar) : null;\n        props.ui = ui;\n        props.extendedProps = __assign({}, leftovers1, props.extendedProps);\n        // help out ResourceApi from having user modify props\n        Object.freeze(ui.classNames);\n        Object.freeze(props.extendedProps);\n        if (store[props.id]) ;\n        else {\n            store[props.id] = props;\n            if (props.children) {\n                for (var _i = 0, _a = props.children; _i < _a.length; _i++) {\n                    var childInput = _a[_i];\n                    parseResource(childInput, props.id, store, calendar);\n                }\n                delete props.children;\n            }\n        }\n        return props;\n    }\n    /*\n    TODO: use this in more places\n    */\n    function getPublicId(id) {\n        if (id.indexOf(PRIVATE_ID_PREFIX) === 0) {\n            return '';\n        }\n        return id;\n    }\n\n    function reduceResourceStore (store, action, source, calendar) {\n        switch (action.type) {\n            case 'INIT':\n                return {};\n            case 'RECEIVE_RESOURCES':\n                return receiveRawResources(store, action.rawResources, action.fetchId, source, calendar);\n            case 'ADD_RESOURCE':\n                return addResource(store, action.resourceHash);\n            case 'REMOVE_RESOURCE':\n                return removeResource(store, action.resourceId);\n            case 'SET_RESOURCE_PROP':\n                return setResourceProp(store, action.resourceId, action.propName, action.propValue);\n            case 'RESET_RESOURCES':\n                // must make the calendar think each resource is a new object :/\n                return core.mapHash(store, function (resource) {\n                    return __assign({}, resource);\n                });\n            default:\n                return store;\n        }\n    }\n    function receiveRawResources(existingStore, inputs, fetchId, source, calendar) {\n        if (source.latestFetchId === fetchId) {\n            var nextStore = {};\n            for (var _i = 0, inputs_1 = inputs; _i < inputs_1.length; _i++) {\n                var input = inputs_1[_i];\n                parseResource(input, '', nextStore, calendar);\n            }\n            return nextStore;\n        }\n        else {\n            return existingStore;\n        }\n    }\n    function addResource(existingStore, additions) {\n        // TODO: warn about duplicate IDs\n        return __assign({}, existingStore, additions);\n    }\n    function removeResource(existingStore, resourceId) {\n        var newStore = __assign({}, existingStore);\n        delete newStore[resourceId];\n        // promote children\n        for (var childResourceId in newStore) { // a child, *maybe* but probably not\n            if (newStore[childResourceId].parentId === resourceId) {\n                newStore[childResourceId] = __assign({}, newStore[childResourceId], { parentId: '' });\n            }\n        }\n        return newStore;\n    }\n    function setResourceProp(existingStore, resourceId, name, value) {\n        var _a, _b;\n        var existingResource = existingStore[resourceId];\n        // TODO: sanitization\n        if (existingResource) {\n            return __assign({}, existingStore, (_a = {}, _a[resourceId] = __assign({}, existingResource, (_b = {}, _b[name] = value, _b)), _a));\n        }\n        else {\n            return existingStore;\n        }\n    }\n\n    function reduceResourceEntityExpansions(expansions, action) {\n        var _a;\n        switch (action.type) {\n            case 'INIT':\n                return {};\n            case 'SET_RESOURCE_ENTITY_EXPANDED':\n                return __assign({}, expansions, (_a = {}, _a[action.id] = action.isExpanded, _a));\n            default:\n                return expansions;\n        }\n    }\n\n    function resourcesReducers (state, action, calendar) {\n        var resourceSource = reduceResourceSource(state.resourceSource, action, state.dateProfile, calendar);\n        var resourceStore = reduceResourceStore(state.resourceStore, action, resourceSource, calendar);\n        var resourceEntityExpansions = reduceResourceEntityExpansions(state.resourceEntityExpansions, action);\n        var loadingLevel = state.loadingLevel + ((resourceSource && resourceSource.isFetching) ? 1 : 0);\n        return __assign({}, state, { resourceSource: resourceSource,\n            resourceStore: resourceStore,\n            resourceEntityExpansions: resourceEntityExpansions,\n            loadingLevel: loadingLevel });\n    }\n\n    var RESOURCE_RELATED_PROPS = {\n        resourceId: String,\n        resourceIds: function (items) {\n            return (items || []).map(function (item) {\n                return String(item);\n            });\n        },\n        resourceEditable: Boolean\n    };\n    function parseEventDef(def, props, leftovers) {\n        var resourceRelatedProps = core.refineProps(props, RESOURCE_RELATED_PROPS, {}, leftovers);\n        var resourceIds = resourceRelatedProps.resourceIds;\n        if (resourceRelatedProps.resourceId) {\n            resourceIds.push(resourceRelatedProps.resourceId);\n        }\n        def.resourceIds = resourceIds;\n        def.resourceEditable = resourceRelatedProps.resourceEditable;\n    }\n\n    function transformDateSelectionJoin(hit0, hit1) {\n        var resourceId0 = hit0.dateSpan.resourceId;\n        var resourceId1 = hit1.dateSpan.resourceId;\n        if (resourceId0 && resourceId1) {\n            if (hit0.component.allowAcrossResources === false &&\n                resourceId0 !== resourceId1) {\n                return false;\n            }\n            else {\n                return { resourceId: resourceId0 };\n            }\n        }\n    }\n\n    var ResourceApi = /** @class */ (function () {\n        function ResourceApi(calendar, rawResource) {\n            this._calendar = calendar;\n            this._resource = rawResource;\n        }\n        ResourceApi.prototype.setProp = function (name, value) {\n            this._calendar.dispatch({\n                type: 'SET_RESOURCE_PROP',\n                resourceId: this._resource.id,\n                propName: name,\n                propValue: value\n            });\n        };\n        ResourceApi.prototype.remove = function () {\n            this._calendar.dispatch({\n                type: 'REMOVE_RESOURCE',\n                resourceId: this._resource.id\n            });\n        };\n        ResourceApi.prototype.getParent = function () {\n            var calendar = this._calendar;\n            var parentId = this._resource.parentId;\n            if (parentId) {\n                return new ResourceApi(calendar, calendar.state.resourceSource[parentId]);\n            }\n            else {\n                return null;\n            }\n        };\n        ResourceApi.prototype.getChildren = function () {\n            var thisResourceId = this._resource.id;\n            var calendar = this._calendar;\n            var resourceStore = calendar.state.resourceStore;\n            var childApis = [];\n            for (var resourceId in resourceStore) {\n                if (resourceStore[resourceId].parentId === thisResourceId) {\n                    childApis.push(new ResourceApi(calendar, resourceStore[resourceId]));\n                }\n            }\n            return childApis;\n        };\n        /*\n        this is really inefficient!\n        TODO: make EventApi::resourceIds a hash or keep an index in the Calendar's state\n        */\n        ResourceApi.prototype.getEvents = function () {\n            var thisResourceId = this._resource.id;\n            var calendar = this._calendar;\n            var _a = calendar.state.eventStore, defs = _a.defs, instances = _a.instances;\n            var eventApis = [];\n            for (var instanceId in instances) {\n                var instance = instances[instanceId];\n                var def = defs[instance.defId];\n                if (def.resourceIds.indexOf(thisResourceId) !== -1) { // inefficient!!!\n                    eventApis.push(new core.EventApi(calendar, def, instance));\n                }\n            }\n            return eventApis;\n        };\n        Object.defineProperty(ResourceApi.prototype, \"id\", {\n            get: function () { return this._resource.id; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(ResourceApi.prototype, \"title\", {\n            get: function () { return this._resource.title; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(ResourceApi.prototype, \"eventConstraint\", {\n            get: function () { return this._resource.ui.constraints[0] || null; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(ResourceApi.prototype, \"eventOverlap\", {\n            get: function () { return this._resource.ui.overlap; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(ResourceApi.prototype, \"eventAllow\", {\n            get: function () { return this._resource.ui.allows[0] || null; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(ResourceApi.prototype, \"eventBackgroundColor\", {\n            get: function () { return this._resource.ui.backgroundColor; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(ResourceApi.prototype, \"eventBorderColor\", {\n            get: function () { return this._resource.ui.borderColor; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(ResourceApi.prototype, \"eventTextColor\", {\n            get: function () { return this._resource.ui.textColor; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(ResourceApi.prototype, \"eventClassNames\", {\n            // NOTE: user can't modify these because Object.freeze was called in event-def parsing\n            get: function () { return this._resource.ui.classNames; },\n            enumerable: true,\n            configurable: true\n        });\n        Object.defineProperty(ResourceApi.prototype, \"extendedProps\", {\n            get: function () { return this._resource.extendedProps; },\n            enumerable: true,\n            configurable: true\n        });\n        return ResourceApi;\n    }());\n\n    core.Calendar.prototype.addResource = function (input, scrollTo) {\n        var _a;\n        if (scrollTo === void 0) { scrollTo = true; }\n        var resourceHash;\n        var resource;\n        if (input instanceof ResourceApi) {\n            resource = input._resource;\n            resourceHash = (_a = {}, _a[resource.id] = resource, _a);\n        }\n        else {\n            resourceHash = {};\n            resource = parseResource(input, '', resourceHash, this);\n        }\n        // HACK\n        if (scrollTo) {\n            this.component.view.addScroll({ forcedRowId: resource.id });\n        }\n        this.dispatch({\n            type: 'ADD_RESOURCE',\n            resourceHash: resourceHash\n        });\n        return new ResourceApi(this, resource);\n    };\n    core.Calendar.prototype.getResourceById = function (id) {\n        id = String(id);\n        if (this.state.resourceStore) { // guard against calendar with no resource functionality\n            var rawResource = this.state.resourceStore[id];\n            if (rawResource) {\n                return new ResourceApi(this, rawResource);\n            }\n        }\n        return null;\n    };\n    core.Calendar.prototype.getResources = function () {\n        var resourceStore = this.state.resourceStore;\n        var resourceApis = [];\n        if (resourceStore) { // guard against calendar with no resource functionality\n            for (var resourceId in resourceStore) {\n                resourceApis.push(new ResourceApi(this, resourceStore[resourceId]));\n            }\n        }\n        return resourceApis;\n    };\n    core.Calendar.prototype.getTopLevelResources = function () {\n        var resourceStore = this.state.resourceStore;\n        var resourceApis = [];\n        if (resourceStore) { // guard against calendar with no resource functionality\n            for (var resourceId in resourceStore) {\n                if (!resourceStore[resourceId].parentId) {\n                    resourceApis.push(new ResourceApi(this, resourceStore[resourceId]));\n                }\n            }\n        }\n        return resourceApis;\n    };\n    core.Calendar.prototype.rerenderResources = function () {\n        this.dispatch({\n            type: 'RESET_RESOURCES'\n        });\n    };\n    core.Calendar.prototype.refetchResources = function () {\n        this.dispatch({\n            type: 'REFETCH_RESOURCES'\n        });\n    };\n    function transformDatePoint(dateSpan, calendar) {\n        return dateSpan.resourceId ?\n            { resource: calendar.getResourceById(dateSpan.resourceId) } :\n            {};\n    }\n    function transformDateSpan(dateSpan, calendar) {\n        return dateSpan.resourceId ?\n            { resource: calendar.getResourceById(dateSpan.resourceId) } :\n            {};\n    }\n\n    /*\n    splits things BASED OFF OF which resources they are associated with.\n    creates a '' entry which is when something has NO resource.\n    */\n    var ResourceSplitter = /** @class */ (function (_super) {\n        __extends(ResourceSplitter, _super);\n        function ResourceSplitter() {\n            return _super !== null && _super.apply(this, arguments) || this;\n        }\n        ResourceSplitter.prototype.getKeyInfo = function (props) {\n            return __assign({ '': {} }, props.resourceStore // already has `ui` and `businessHours` keys!\n            );\n        };\n        ResourceSplitter.prototype.getKeysForDateSpan = function (dateSpan) {\n            return [dateSpan.resourceId || ''];\n        };\n        ResourceSplitter.prototype.getKeysForEventDef = function (eventDef) {\n            var resourceIds = eventDef.resourceIds;\n            if (!resourceIds.length) {\n                return [''];\n            }\n            return resourceIds;\n        };\n        return ResourceSplitter;\n    }(core.Splitter));\n\n    function isPropsValidWithResources(props, calendar) {\n        var splitter = new ResourceSplitter();\n        var sets = splitter.splitProps(__assign({}, props, { resourceStore: calendar.state.resourceStore }));\n        for (var resourceId in sets) {\n            var props_1 = sets[resourceId];\n            // merge in event data from the non-resource segment\n            if (resourceId && sets['']) { // current segment is not the non-resource one, and there IS a non-resource one\n                props_1 = __assign({}, props_1, { eventStore: core.mergeEventStores(sets[''].eventStore, props_1.eventStore), eventUiBases: __assign({}, sets[''].eventUiBases, props_1.eventUiBases) });\n            }\n            if (!core.isPropsValid(props_1, calendar, { resourceId: resourceId }, filterConfig.bind(null, resourceId))) {\n                return false;\n            }\n        }\n        return true;\n    }\n    function filterConfig(resourceId, config) {\n        return __assign({}, config, { constraints: filterConstraints(resourceId, config.constraints) });\n    }\n    function filterConstraints(resourceId, constraints) {\n        return constraints.map(function (constraint) {\n            var defs = constraint.defs;\n            if (defs) { // we are dealing with an EventStore\n                // if any of the events define constraints to resources that are NOT this resource,\n                // then this resource is unconditionally prohibited, which is what a `false` value does.\n                for (var defId in defs) {\n                    var resourceIds = defs[defId].resourceIds;\n                    if (resourceIds.length && resourceIds.indexOf(resourceId) === -1) { // TODO: use a hash?!!! (for other reasons too)\n                        return false;\n                    }\n                }\n            }\n            return constraint;\n        });\n    }\n\n    function transformExternalDef(dateSpan) {\n        return dateSpan.resourceId ?\n            { resourceId: dateSpan.resourceId } :\n            {};\n    }\n\n    function transformEventResizeJoin(hit0, hit1) {\n        var component = hit0.component;\n        if (component.allowAcrossResources === false &&\n            hit0.dateSpan.resourceId !== hit1.dateSpan.resourceId) {\n            return false;\n        }\n    }\n\n    core.EventApi.prototype.getResources = function () {\n        var calendar = this._calendar;\n        return this._def.resourceIds.map(function (resourceId) {\n            return calendar.getResourceById(resourceId);\n        });\n    };\n    core.EventApi.prototype.setResources = function (resources) {\n        var resourceIds = [];\n        // massage resources -> resourceIds\n        for (var _i = 0, resources_1 = resources; _i < resources_1.length; _i++) {\n            var resource = resources_1[_i];\n            var resourceId = null;\n            if (typeof resource === 'string') {\n                resourceId = resource;\n            }\n            else if (typeof resource === 'number') {\n                resourceId = String(resource);\n            }\n            else if (resource instanceof ResourceApi) {\n                resourceId = resource.id; // guaranteed to always have an ID. hmmm\n            }\n            else {\n                console.warn('unknown resource type: ' + resource);\n            }\n            if (resourceId) {\n                resourceIds.push(resourceId);\n            }\n        }\n        this.mutate({\n            standardProps: {\n                resourceIds: resourceIds\n            }\n        });\n    };\n\n    var RELEASE_DATE = '2020-05-28'; // for Scheduler\n    var UPGRADE_WINDOW = 365 + 7; // days. 1 week leeway, for tz shift reasons too\n    var LICENSE_INFO_URL = 'http://fullcalendar.io/scheduler/license/';\n    var PRESET_LICENSE_KEYS = [\n        'GPL-My-Project-Is-Open-Source',\n        'CC-Attribution-NonCommercial-NoDerivatives'\n    ];\n    var CSS = {\n        position: 'absolute',\n        'z-index': 99999,\n        bottom: '1px',\n        left: '1px',\n        background: '#eee',\n        'border-color': '#ddd',\n        'border-style': 'solid',\n        'border-width': '1px 1px 0 0',\n        padding: '2px 4px',\n        'font-size': '12px',\n        'border-top-right-radius': '3px'\n    };\n    function injectLicenseWarning(containerEl, calendar) {\n        var key = calendar.opt('schedulerLicenseKey');\n        if (!isImmuneUrl(window.location.href) && !isValidKey(key)) {\n            core.appendToElement(containerEl, '<div class=\"fc-license-message\" style=\"' + core.htmlEscape(core.cssToStr(CSS)) + '\">' +\n                'Please use a valid license key. <a href=\"' + LICENSE_INFO_URL + '\">More Info</a>' +\n                '</div>');\n        }\n    }\n    /*\n    This decryption is not meant to be bulletproof. Just a way to remind about an upgrade.\n    */\n    function isValidKey(key) {\n        if (PRESET_LICENSE_KEYS.indexOf(key) !== -1) {\n            return true;\n        }\n        var parts = (key || '').match(/^(\\d+)\\-fcs\\-(\\d+)$/);\n        if (parts && (parts[1].length === 10)) {\n            var purchaseDate = new Date(parseInt(parts[2], 10) * 1000);\n            var releaseDate = new Date(core.config.mockSchedulerReleaseDate || RELEASE_DATE);\n            if (core.isValidDate(releaseDate)) { // token won't be replaced in dev mode\n                var minPurchaseDate = core.addDays(releaseDate, -UPGRADE_WINDOW);\n                if (minPurchaseDate < purchaseDate) {\n                    return true;\n                }\n            }\n        }\n        return false;\n    }\n    function isImmuneUrl(url) {\n        return /\\w+\\:\\/\\/fullcalendar\\.io\\/|\\/examples\\/[\\w-]+\\.html$/.test(url);\n    }\n\n    var optionChangeHandlers = {\n        resources: handleResources\n    };\n    function handleResources(newSourceInput, calendar, deepEqual) {\n        var oldSourceInput = calendar.state.resourceSource._raw;\n        if (!deepEqual(oldSourceInput, newSourceInput)) {\n            calendar.dispatch({\n                type: 'RESET_RESOURCE_SOURCE',\n                resourceSourceInput: newSourceInput\n            });\n        }\n    }\n\n    registerResourceSourceDef({\n        ignoreRange: true,\n        parseMeta: function (raw) {\n            if (Array.isArray(raw)) {\n                return raw;\n            }\n            else if (Array.isArray(raw.resources)) {\n                return raw.resources;\n            }\n            return null;\n        },\n        fetch: function (arg, successCallback) {\n            successCallback({\n                rawResources: arg.resourceSource.meta\n            });\n        }\n    });\n\n    registerResourceSourceDef({\n        parseMeta: function (raw) {\n            if (typeof raw === 'function') {\n                return raw;\n            }\n            else if (typeof raw.resources === 'function') {\n                return raw.resources;\n            }\n            return null;\n        },\n        fetch: function (arg, success, failure) {\n            var dateEnv = arg.calendar.dateEnv;\n            var func = arg.resourceSource.meta;\n            var publicArg = {};\n            if (arg.range) {\n                publicArg = {\n                    start: dateEnv.toDate(arg.range.start),\n                    end: dateEnv.toDate(arg.range.end),\n                    startStr: dateEnv.formatIso(arg.range.start),\n                    endStr: dateEnv.formatIso(arg.range.end),\n                    timeZone: dateEnv.timeZone\n                };\n            }\n            // TODO: make more dry with EventSourceFunc\n            // TODO: accept a response?\n            core.unpromisify(func.bind(null, publicArg), function (rawResources) {\n                success({ rawResources: rawResources }); // needs an object response\n            }, failure // send errorObj directly to failure callback\n            );\n        }\n    });\n\n    registerResourceSourceDef({\n        parseMeta: function (raw) {\n            if (typeof raw === 'string') {\n                raw = { url: raw };\n            }\n            else if (!raw || typeof raw !== 'object' || !raw.url) {\n                return null;\n            }\n            return {\n                url: raw.url,\n                method: (raw.method || 'GET').toUpperCase(),\n                extraParams: raw.extraParams\n            };\n        },\n        fetch: function (arg, successCallback, failureCallback) {\n            var meta = arg.resourceSource.meta;\n            var requestParams = buildRequestParams(meta, arg.range, arg.calendar);\n            core.requestJson(meta.method, meta.url, requestParams, function (rawResources, xhr) {\n                successCallback({ rawResources: rawResources, xhr: xhr });\n            }, function (message, xhr) {\n                failureCallback({ message: message, xhr: xhr });\n            });\n        }\n    });\n    // TODO: somehow consolidate with event json feed\n    function buildRequestParams(meta, range, calendar) {\n        var dateEnv = calendar.dateEnv;\n        var startParam;\n        var endParam;\n        var timeZoneParam;\n        var customRequestParams;\n        var params = {};\n        if (range) {\n            // startParam = meta.startParam\n            // if (startParam == null) {\n            startParam = calendar.opt('startParam');\n            // }\n            // endParam = meta.endParam\n            // if (endParam == null) {\n            endParam = calendar.opt('endParam');\n            // }\n            // timeZoneParam = meta.timeZoneParam\n            // if (timeZoneParam == null) {\n            timeZoneParam = calendar.opt('timeZoneParam');\n            // }\n            params[startParam] = dateEnv.formatIso(range.start);\n            params[endParam] = dateEnv.formatIso(range.end);\n            if (dateEnv.timeZone !== 'local') {\n                params[timeZoneParam] = dateEnv.timeZone;\n            }\n        }\n        // retrieve any outbound GET/POST data from the options\n        if (typeof meta.extraParams === 'function') {\n            // supplied as a function that returns a key/value object\n            customRequestParams = meta.extraParams();\n        }\n        else {\n            // probably supplied as a straight key/value object\n            customRequestParams = meta.extraParams || {};\n        }\n        __assign(params, customRequestParams);\n        return params;\n    }\n\n    function buildResourceTextFunc(resourceTextSetting, calendar) {\n        if (typeof resourceTextSetting === 'function') {\n            return function (resource) {\n                return resourceTextSetting(new ResourceApi(calendar, resource));\n            };\n        }\n        else {\n            return function (resource) {\n                return resource.title || getPublicId(resource.id);\n            };\n        }\n    }\n\n    var ResourceDayHeader = /** @class */ (function (_super) {\n        __extends(ResourceDayHeader, _super);\n        function ResourceDayHeader(parentEl) {\n            var _this = _super.call(this) || this;\n            _this.processOptions = core.memoize(_this._processOptions);\n            _this.parentEl = parentEl;\n            return _this;\n        }\n        ResourceDayHeader.prototype._processOptions = function (options, calendar) {\n            this.datesAboveResources = options.datesAboveResources;\n            this.resourceTextFunc = buildResourceTextFunc(options.resourceText, calendar);\n        };\n        ResourceDayHeader.prototype.render = function (props, context) {\n            var options = context.options, calendar = context.calendar, theme = context.theme;\n            this.processOptions(options, calendar);\n            this.parentEl.innerHTML = ''; // because might be nbsp\n            this.parentEl.appendChild(this.el = core.htmlToElement('<div class=\"fc-row ' + theme.getClass('headerRow') + '\">' +\n                '<table class=\"' + theme.getClass('tableGrid') + '\">' +\n                '<thead></thead>' +\n                '</table>' +\n                '</div>'));\n            this.thead = this.el.querySelector('thead');\n            var html;\n            this.dateFormat = core.createFormatter(options.columnHeaderFormat ||\n                core.computeFallbackHeaderFormat(props.datesRepDistinctDays, props.dates.length));\n            if (props.dates.length === 1) {\n                html = this.renderResourceRow(props.resources);\n            }\n            else {\n                if (this.datesAboveResources) {\n                    html = this.renderDayAndResourceRows(props.dates, props.resources);\n                }\n                else {\n                    html = this.renderResourceAndDayRows(props.resources, props.dates);\n                }\n            }\n            this.thead.innerHTML = html;\n            this.processResourceEls(props.resources);\n        };\n        ResourceDayHeader.prototype.destroy = function () {\n            core.removeElement(this.el);\n        };\n        ResourceDayHeader.prototype.renderResourceRow = function (resources) {\n            var _this = this;\n            var cellHtmls = resources.map(function (resource) {\n                return _this.renderResourceCell(resource, 1);\n            });\n            return this.buildTr(cellHtmls);\n        };\n        ResourceDayHeader.prototype.renderDayAndResourceRows = function (dates, resources) {\n            var dateHtmls = [];\n            var resourceHtmls = [];\n            for (var _i = 0, dates_1 = dates; _i < dates_1.length; _i++) {\n                var date = dates_1[_i];\n                dateHtmls.push(this.renderDateCell(date, resources.length));\n                for (var _a = 0, resources_1 = resources; _a < resources_1.length; _a++) {\n                    var resource = resources_1[_a];\n                    resourceHtmls.push(this.renderResourceCell(resource, 1, date));\n                }\n            }\n            return this.buildTr(dateHtmls) +\n                this.buildTr(resourceHtmls);\n        };\n        ResourceDayHeader.prototype.renderResourceAndDayRows = function (resources, dates) {\n            var resourceHtmls = [];\n            var dateHtmls = [];\n            for (var _i = 0, resources_2 = resources; _i < resources_2.length; _i++) {\n                var resource = resources_2[_i];\n                resourceHtmls.push(this.renderResourceCell(resource, dates.length));\n                for (var _a = 0, dates_2 = dates; _a < dates_2.length; _a++) {\n                    var date = dates_2[_a];\n                    dateHtmls.push(this.renderDateCell(date, 1, resource));\n                }\n            }\n            return this.buildTr(resourceHtmls) +\n                this.buildTr(dateHtmls);\n        };\n        // Cell Rendering Utils\n        // ----------------------------------------------------------------------------------------------\n        // a cell with the resource name. might be associated with a specific day\n        ResourceDayHeader.prototype.renderResourceCell = function (resource, colspan, date) {\n            var dateEnv = this.context.dateEnv;\n            return '<th class=\"fc-resource-cell\"' +\n                ' data-resource-id=\"' + resource.id + '\"' +\n                (date ?\n                    ' data-date=\"' + dateEnv.formatIso(date, { omitTime: true }) + '\"' :\n                    '') +\n                (colspan > 1 ?\n                    ' colspan=\"' + colspan + '\"' :\n                    '') +\n                '>' +\n                core.htmlEscape(this.resourceTextFunc(resource)) +\n                '</th>';\n        };\n        // a cell with date text. might have a resource associated with it\n        ResourceDayHeader.prototype.renderDateCell = function (date, colspan, resource) {\n            var props = this.props;\n            return core.renderDateCell(date, props.dateProfile, props.datesRepDistinctDays, props.dates.length * props.resources.length, this.dateFormat, this.context, colspan, resource ? 'data-resource-id=\"' + resource.id + '\"' : '');\n        };\n        ResourceDayHeader.prototype.buildTr = function (cellHtmls) {\n            if (!cellHtmls.length) {\n                cellHtmls = ['<td>&nbsp;</td>'];\n            }\n            if (this.props.renderIntroHtml) {\n                cellHtmls = [this.props.renderIntroHtml()].concat(cellHtmls);\n            }\n            if (this.context.isRtl) {\n                cellHtmls.reverse();\n            }\n            return '<tr>' +\n                cellHtmls.join('') +\n                '</tr>';\n        };\n        // Post-rendering\n        // ----------------------------------------------------------------------------------------------\n        // given a container with already rendered resource cells\n        ResourceDayHeader.prototype.processResourceEls = function (resources) {\n            var _a = this.context, calendar = _a.calendar, isRtl = _a.isRtl, view = _a.view;\n            core.findElements(this.thead, '.fc-resource-cell').forEach(function (node, col) {\n                col = col % resources.length;\n                if (isRtl) {\n                    col = resources.length - 1 - col;\n                }\n                var resource = resources[col];\n                calendar.publiclyTrigger('resourceRender', [\n                    {\n                        resource: new ResourceApi(calendar, resource),\n                        el: node,\n                        view: view\n                    }\n                ]);\n            });\n        };\n        return ResourceDayHeader;\n    }(core.Component));\n\n    var AbstractResourceDayTable = /** @class */ (function () {\n        function AbstractResourceDayTable(dayTable, resources) {\n            this.dayTable = dayTable;\n            this.resources = resources;\n            this.resourceIndex = new ResourceIndex(resources);\n            this.rowCnt = dayTable.rowCnt;\n            this.colCnt = dayTable.colCnt * resources.length;\n            this.cells = this.buildCells();\n        }\n        AbstractResourceDayTable.prototype.buildCells = function () {\n            var _a = this, rowCnt = _a.rowCnt, dayTable = _a.dayTable, resources = _a.resources;\n            var rows = [];\n            for (var row = 0; row < rowCnt; row++) {\n                var rowCells = [];\n                for (var dateCol = 0; dateCol < dayTable.colCnt; dateCol++) {\n                    for (var resourceCol = 0; resourceCol < resources.length; resourceCol++) {\n                        var resource = resources[resourceCol];\n                        var htmlAttrs = 'data-resource-id=\"' + resource.id + '\"';\n                        rowCells[this.computeCol(dateCol, resourceCol)] = {\n                            date: dayTable.cells[row][dateCol].date,\n                            resource: resource,\n                            htmlAttrs: htmlAttrs\n                        };\n                    }\n                }\n                rows.push(rowCells);\n            }\n            return rows;\n        };\n        return AbstractResourceDayTable;\n    }());\n    /*\n    resources over dates\n    */\n    var ResourceDayTable = /** @class */ (function (_super) {\n        __extends(ResourceDayTable, _super);\n        function ResourceDayTable() {\n            return _super !== null && _super.apply(this, arguments) || this;\n        }\n        ResourceDayTable.prototype.computeCol = function (dateI, resourceI) {\n            return resourceI * this.dayTable.colCnt + dateI;\n        };\n        /*\n        all date ranges are intact\n        */\n        ResourceDayTable.prototype.computeColRanges = function (dateStartI, dateEndI, resourceI) {\n            return [\n                {\n                    firstCol: this.computeCol(dateStartI, resourceI),\n                    lastCol: this.computeCol(dateEndI, resourceI),\n                    isStart: true,\n                    isEnd: true\n                }\n            ];\n        };\n        return ResourceDayTable;\n    }(AbstractResourceDayTable));\n    /*\n    dates over resources\n    */\n    var DayResourceTable = /** @class */ (function (_super) {\n        __extends(DayResourceTable, _super);\n        function DayResourceTable() {\n            return _super !== null && _super.apply(this, arguments) || this;\n        }\n        DayResourceTable.prototype.computeCol = function (dateI, resourceI) {\n            return dateI * this.resources.length + resourceI;\n        };\n        /*\n        every single day is broken up\n        */\n        DayResourceTable.prototype.computeColRanges = function (dateStartI, dateEndI, resourceI) {\n            var segs = [];\n            for (var i = dateStartI; i <= dateEndI; i++) {\n                var col = this.computeCol(i, resourceI);\n                segs.push({\n                    firstCol: col,\n                    lastCol: col,\n                    isStart: i === dateStartI,\n                    isEnd: i === dateEndI\n                });\n            }\n            return segs;\n        };\n        return DayResourceTable;\n    }(AbstractResourceDayTable));\n    var ResourceIndex = /** @class */ (function () {\n        function ResourceIndex(resources) {\n            var indicesById = {};\n            var ids = [];\n            for (var i = 0; i < resources.length; i++) {\n                var id = resources[i].id;\n                ids.push(id);\n                indicesById[id] = i;\n            }\n            this.ids = ids;\n            this.indicesById = indicesById;\n            this.length = resources.length;\n        }\n        return ResourceIndex;\n    }());\n    /*\n    TODO: just use ResourceHash somehow? could then use the generic ResourceSplitter\n    */\n    var VResourceSplitter = /** @class */ (function (_super) {\n        __extends(VResourceSplitter, _super);\n        function VResourceSplitter() {\n            return _super !== null && _super.apply(this, arguments) || this;\n        }\n        VResourceSplitter.prototype.getKeyInfo = function (props) {\n            var resourceDayTable = props.resourceDayTable;\n            var hash = core.mapHash(resourceDayTable.resourceIndex.indicesById, function (i) {\n                return resourceDayTable.resources[i]; // has `ui` AND `businessHours` keys!\n            }); // :(\n            hash[''] = {};\n            return hash;\n        };\n        VResourceSplitter.prototype.getKeysForDateSpan = function (dateSpan) {\n            return [dateSpan.resourceId || ''];\n        };\n        VResourceSplitter.prototype.getKeysForEventDef = function (eventDef) {\n            var resourceIds = eventDef.resourceIds;\n            if (!resourceIds.length) {\n                return [''];\n            }\n            return resourceIds;\n        };\n        return VResourceSplitter;\n    }(core.Splitter));\n    // joiner\n    var NO_SEGS = []; // for memoizing\n    var VResourceJoiner = /** @class */ (function () {\n        function VResourceJoiner() {\n            this.joinDateSelection = core.memoize(this.joinSegs);\n            this.joinBusinessHours = core.memoize(this.joinSegs);\n            this.joinFgEvents = core.memoize(this.joinSegs);\n            this.joinBgEvents = core.memoize(this.joinSegs);\n            this.joinEventDrags = core.memoize(this.joinInteractions);\n            this.joinEventResizes = core.memoize(this.joinInteractions);\n        }\n        /*\n        propSets also has a '' key for things with no resource\n        */\n        VResourceJoiner.prototype.joinProps = function (propSets, resourceDayTable) {\n            var dateSelectionSets = [];\n            var businessHoursSets = [];\n            var fgEventSets = [];\n            var bgEventSets = [];\n            var eventDrags = [];\n            var eventResizes = [];\n            var eventSelection = '';\n            var keys = resourceDayTable.resourceIndex.ids.concat(['']); // add in the all-resource key\n            for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {\n                var key = keys_1[_i];\n                var props = propSets[key];\n                dateSelectionSets.push(props.dateSelectionSegs);\n                businessHoursSets.push(key ? props.businessHourSegs : NO_SEGS); // don't include redundant all-resource businesshours\n                fgEventSets.push(key ? props.fgEventSegs : NO_SEGS); // don't include fg all-resource segs\n                bgEventSets.push(props.bgEventSegs);\n                eventDrags.push(props.eventDrag);\n                eventResizes.push(props.eventResize);\n                eventSelection = eventSelection || props.eventSelection;\n            }\n            return {\n                dateSelectionSegs: this.joinDateSelection.apply(this, [resourceDayTable].concat(dateSelectionSets)),\n                businessHourSegs: this.joinBusinessHours.apply(this, [resourceDayTable].concat(businessHoursSets)),\n                fgEventSegs: this.joinFgEvents.apply(this, [resourceDayTable].concat(fgEventSets)),\n                bgEventSegs: this.joinBgEvents.apply(this, [resourceDayTable].concat(bgEventSets)),\n                eventDrag: this.joinEventDrags.apply(this, [resourceDayTable].concat(eventDrags)),\n                eventResize: this.joinEventResizes.apply(this, [resourceDayTable].concat(eventResizes)),\n                eventSelection: eventSelection\n            };\n        };\n        VResourceJoiner.prototype.joinSegs = function (resourceDayTable) {\n            var segGroups = [];\n            for (var _i = 1; _i < arguments.length; _i++) {\n                segGroups[_i - 1] = arguments[_i];\n            }\n            var resourceCnt = resourceDayTable.resources.length;\n            var transformedSegs = [];\n            for (var i = 0; i < resourceCnt; i++) {\n                for (var _a = 0, _b = segGroups[i]; _a < _b.length; _a++) {\n                    var seg = _b[_a];\n                    transformedSegs.push.apply(transformedSegs, this.transformSeg(seg, resourceDayTable, i));\n                }\n                for (var _c = 0, _d = segGroups[resourceCnt]; _c < _d.length; _c++) { // one beyond. the all-resource\n                    var seg = _d[_c];\n                    transformedSegs.push.apply(// one beyond. the all-resource\n                    transformedSegs, this.transformSeg(seg, resourceDayTable, i));\n                }\n            }\n            return transformedSegs;\n        };\n        /*\n        for expanding non-resource segs to all resources.\n        only for public use.\n        no memoizing.\n        */\n        VResourceJoiner.prototype.expandSegs = function (resourceDayTable, segs) {\n            var resourceCnt = resourceDayTable.resources.length;\n            var transformedSegs = [];\n            for (var i = 0; i < resourceCnt; i++) {\n                for (var _i = 0, segs_1 = segs; _i < segs_1.length; _i++) {\n                    var seg = segs_1[_i];\n                    transformedSegs.push.apply(transformedSegs, this.transformSeg(seg, resourceDayTable, i));\n                }\n            }\n            return transformedSegs;\n        };\n        VResourceJoiner.prototype.joinInteractions = function (resourceDayTable) {\n            var interactions = [];\n            for (var _i = 1; _i < arguments.length; _i++) {\n                interactions[_i - 1] = arguments[_i];\n            }\n            var resourceCnt = resourceDayTable.resources.length;\n            var affectedInstances = {};\n            var transformedSegs = [];\n            var isEvent = false;\n            var sourceSeg = null;\n            for (var i = 0; i < resourceCnt; i++) {\n                var interaction = interactions[i];\n                if (interaction) {\n                    for (var _a = 0, _b = interaction.segs; _a < _b.length; _a++) {\n                        var seg = _b[_a];\n                        transformedSegs.push.apply(transformedSegs, this.transformSeg(seg, resourceDayTable, i) // TODO: templateify Interaction::segs\n                        );\n                    }\n                    __assign(affectedInstances, interaction.affectedInstances);\n                    isEvent = isEvent || interaction.isEvent;\n                    sourceSeg = sourceSeg || interaction.sourceSeg;\n                }\n                if (interactions[resourceCnt]) { // one beyond. the all-resource\n                    for (var _c = 0, _d = interactions[resourceCnt].segs; _c < _d.length; _c++) {\n                        var seg = _d[_c];\n                        transformedSegs.push.apply(transformedSegs, this.transformSeg(seg, resourceDayTable, i) // TODO: templateify Interaction::segs\n                        );\n                    }\n                }\n            }\n            return {\n                affectedInstances: affectedInstances,\n                segs: transformedSegs,\n                isEvent: isEvent,\n                sourceSeg: sourceSeg\n            };\n        };\n        return VResourceJoiner;\n    }());\n\n    /*\n    doesn't accept grouping\n    */\n    function flattenResources(resourceStore, orderSpecs) {\n        return buildRowNodes(resourceStore, [], orderSpecs, false, {}, true)\n            .map(function (node) {\n            return node.resource;\n        });\n    }\n    function buildRowNodes(resourceStore, groupSpecs, orderSpecs, isVGrouping, expansions, expansionDefault) {\n        var complexNodes = buildHierarchy(resourceStore, isVGrouping ? -1 : 1, groupSpecs, orderSpecs);\n        var flatNodes = [];\n        flattenNodes(complexNodes, flatNodes, isVGrouping, [], 0, expansions, expansionDefault);\n        return flatNodes;\n    }\n    function flattenNodes(complexNodes, res, isVGrouping, rowSpans, depth, expansions, expansionDefault) {\n        for (var i = 0; i < complexNodes.length; i++) {\n            var complexNode = complexNodes[i];\n            var group = complexNode.group;\n            if (group) {\n                if (isVGrouping) {\n                    var firstRowIndex = res.length;\n                    var rowSpanIndex = rowSpans.length;\n                    flattenNodes(complexNode.children, res, isVGrouping, rowSpans.concat(0), depth, expansions, expansionDefault);\n                    if (firstRowIndex < res.length) {\n                        var firstRow = res[firstRowIndex];\n                        var firstRowSpans = firstRow.rowSpans = firstRow.rowSpans.slice();\n                        firstRowSpans[rowSpanIndex] = res.length - firstRowIndex;\n                    }\n                }\n                else {\n                    var id = group.spec.field + ':' + group.value;\n                    var isExpanded = expansions[id] != null ? expansions[id] : expansionDefault;\n                    res.push({ id: id, group: group, isExpanded: isExpanded });\n                    if (isExpanded) {\n                        flattenNodes(complexNode.children, res, isVGrouping, rowSpans, depth + 1, expansions, expansionDefault);\n                    }\n                }\n            }\n            else if (complexNode.resource) {\n                var id = complexNode.resource.id;\n                var isExpanded = expansions[id] != null ? expansions[id] : expansionDefault;\n                res.push({\n                    id: id,\n                    rowSpans: rowSpans,\n                    depth: depth,\n                    isExpanded: isExpanded,\n                    hasChildren: Boolean(complexNode.children.length),\n                    resource: complexNode.resource,\n                    resourceFields: complexNode.resourceFields\n                });\n                if (isExpanded) {\n                    flattenNodes(complexNode.children, res, isVGrouping, rowSpans, depth + 1, expansions, expansionDefault);\n                }\n            }\n        }\n    }\n    function buildHierarchy(resourceStore, maxDepth, groupSpecs, orderSpecs) {\n        var resourceNodes = buildResourceNodes(resourceStore, orderSpecs);\n        var builtNodes = [];\n        for (var resourceId in resourceNodes) {\n            var resourceNode = resourceNodes[resourceId];\n            if (!resourceNode.resource.parentId) {\n                insertResourceNode(resourceNode, builtNodes, groupSpecs, 0, maxDepth, orderSpecs);\n            }\n        }\n        return builtNodes;\n    }\n    function buildResourceNodes(resourceStore, orderSpecs) {\n        var nodeHash = {};\n        for (var resourceId in resourceStore) {\n            var resource = resourceStore[resourceId];\n            nodeHash[resourceId] = {\n                resource: resource,\n                resourceFields: buildResourceFields(resource),\n                children: []\n            };\n        }\n        for (var resourceId in resourceStore) {\n            var resource = resourceStore[resourceId];\n            if (resource.parentId) {\n                var parentNode = nodeHash[resource.parentId];\n                if (parentNode) {\n                    insertResourceNodeInSiblings(nodeHash[resourceId], parentNode.children, orderSpecs);\n                }\n            }\n        }\n        return nodeHash;\n    }\n    function insertResourceNode(resourceNode, nodes, groupSpecs, depth, maxDepth, orderSpecs) {\n        if (groupSpecs.length && (maxDepth === -1 || depth <= maxDepth)) {\n            var groupNode = ensureGroupNodes(resourceNode, nodes, groupSpecs[0]);\n            insertResourceNode(resourceNode, groupNode.children, groupSpecs.slice(1), depth + 1, maxDepth, orderSpecs);\n        }\n        else {\n            insertResourceNodeInSiblings(resourceNode, nodes, orderSpecs);\n        }\n    }\n    function ensureGroupNodes(resourceNode, nodes, groupSpec) {\n        var groupValue = resourceNode.resourceFields[groupSpec.field];\n        var groupNode;\n        var newGroupIndex;\n        // find an existing group that matches, or determine the position for a new group\n        if (groupSpec.order) {\n            for (newGroupIndex = 0; newGroupIndex < nodes.length; newGroupIndex++) {\n                var node = nodes[newGroupIndex];\n                if (node.group) {\n                    var cmp = core.flexibleCompare(groupValue, node.group.value) * groupSpec.order;\n                    if (cmp === 0) {\n                        groupNode = node;\n                        break;\n                    }\n                    else if (cmp < 0) {\n                        break;\n                    }\n                }\n            }\n        }\n        else { // the groups are unordered\n            for (newGroupIndex = 0; newGroupIndex < nodes.length; newGroupIndex++) {\n                var node = nodes[newGroupIndex];\n                if (node.group && groupValue === node.group.value) {\n                    groupNode = node;\n                    break;\n                }\n            }\n        }\n        if (!groupNode) {\n            groupNode = {\n                group: {\n                    value: groupValue,\n                    spec: groupSpec\n                },\n                children: []\n            };\n            nodes.splice(newGroupIndex, 0, groupNode);\n        }\n        return groupNode;\n    }\n    function insertResourceNodeInSiblings(resourceNode, siblings, orderSpecs) {\n        var i;\n        for (i = 0; i < siblings.length; i++) {\n            var cmp = core.compareByFieldSpecs(siblings[i].resourceFields, resourceNode.resourceFields, orderSpecs);\n            if (cmp > 0) { // went 1 past. insert at i\n                break;\n            }\n        }\n        siblings.splice(i, 0, resourceNode);\n    }\n    function buildResourceFields(resource) {\n        var obj = __assign({}, resource.extendedProps, resource.ui, resource);\n        delete obj.ui;\n        delete obj.extendedProps;\n        return obj;\n    }\n    function isGroupsEqual(group0, group1) {\n        return group0.spec === group1.spec && group0.value === group1.value;\n    }\n\n    var main = core.createPlugin({\n        reducers: [resourcesReducers],\n        eventDefParsers: [parseEventDef],\n        isDraggableTransformers: [transformIsDraggable],\n        eventDragMutationMassagers: [massageEventDragMutation],\n        eventDefMutationAppliers: [applyEventDefMutation],\n        dateSelectionTransformers: [transformDateSelectionJoin],\n        datePointTransforms: [transformDatePoint],\n        dateSpanTransforms: [transformDateSpan],\n        viewPropsTransformers: [ResourceDataAdder, ResourceEventConfigAdder],\n        isPropsValid: isPropsValidWithResources,\n        externalDefTransforms: [transformExternalDef],\n        eventResizeJoinTransforms: [transformEventResizeJoin],\n        viewContainerModifiers: [injectLicenseWarning],\n        eventDropTransformers: [transformEventDrop],\n        optionChangeHandlers: optionChangeHandlers\n    });\n\n    exports.AbstractResourceDayTable = AbstractResourceDayTable;\n    exports.DayResourceTable = DayResourceTable;\n    exports.ResourceApi = ResourceApi;\n    exports.ResourceDayHeader = ResourceDayHeader;\n    exports.ResourceDayTable = ResourceDayTable;\n    exports.ResourceSplitter = ResourceSplitter;\n    exports.VResourceJoiner = VResourceJoiner;\n    exports.VResourceSplitter = VResourceSplitter;\n    exports.buildResourceFields = buildResourceFields;\n    exports.buildResourceTextFunc = buildResourceTextFunc;\n    exports.buildRowNodes = buildRowNodes;\n    exports.default = main;\n    exports.flattenResources = flattenResources;\n    exports.isGroupsEqual = isGroupsEqual;\n\n    Object.defineProperty(exports, '__esModule', { value: true });\n\n}));\n"

/***/ },

/***/ 1097
(module) {

module.exports = "/*!\nFullCalendar Resource Timeline Plugin v4.4.2\nDocs & License: https://fullcalendar.io/scheduler\n(c) 2019 Adam Shaw\n*/\n\n(function (global, factory) {\n    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@fullcalendar/core'), require('@fullcalendar/timeline'), require('@fullcalendar/resource-common')) :\n    typeof define === 'function' && define.amd ? define(['exports', '@fullcalendar/core', '@fullcalendar/timeline', '@fullcalendar/resource-common'], factory) :\n    (global = global || self, factory(global.FullCalendarResourceTimeline = {}, global.FullCalendar, global.FullCalendarTimeline, global.FullCalendarResourceCommon));\n}(this, function (exports, core, TimelinePlugin, ResourceCommonPlugin) { 'use strict';\n\n    var TimelinePlugin__default = 'default' in TimelinePlugin ? TimelinePlugin['default'] : TimelinePlugin;\n    var ResourceCommonPlugin__default = 'default' in ResourceCommonPlugin ? ResourceCommonPlugin['default'] : ResourceCommonPlugin;\n\n    /*! *****************************************************************************\r\n    Copyright (c) Microsoft Corporation.\r\n\r\n    Permission to use, copy, modify, and/or distribute this software for any\r\n    purpose with or without fee is hereby granted.\r\n\r\n    THE SOFTWARE IS PROVIDED \"AS IS\" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH\r\n    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY\r\n    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,\r\n    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM\r\n    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR\r\n    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR\r\n    PERFORMANCE OF THIS SOFTWARE.\r\n    ***************************************************************************** */\r\n    /* global Reflect, Promise */\r\n\r\n    var extendStatics = function(d, b) {\r\n        extendStatics = Object.setPrototypeOf ||\r\n            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||\r\n            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };\r\n        return extendStatics(d, b);\r\n    };\r\n\r\n    function __extends(d, b) {\r\n        extendStatics(d, b);\r\n        function __() { this.constructor = d; }\r\n        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());\r\n    }\r\n\r\n    var __assign = function() {\r\n        __assign = Object.assign || function __assign(t) {\r\n            for (var s, i = 1, n = arguments.length; i < n; i++) {\r\n                s = arguments[i];\r\n                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];\r\n            }\r\n            return t;\r\n        };\r\n        return __assign.apply(this, arguments);\r\n    };\n\n    var Row = /** @class */ (function (_super) {\n        __extends(Row, _super);\n        function Row(spreadsheetParent, spreadsheetNextSibling, timeAxisParent, timeAxisNextSibling) {\n            var _this = _super.call(this) || this;\n            _this.isSizeDirty = false;\n            spreadsheetParent.insertBefore(_this.spreadsheetTr = document.createElement('tr'), spreadsheetNextSibling);\n            timeAxisParent.insertBefore(_this.timeAxisTr = document.createElement('tr'), timeAxisNextSibling);\n            return _this;\n        }\n        Row.prototype.destroy = function () {\n            core.removeElement(this.spreadsheetTr);\n            core.removeElement(this.timeAxisTr);\n            _super.prototype.destroy.call(this);\n        };\n        Row.prototype.updateSize = function (isResize) {\n            this.isSizeDirty = false;\n        };\n        return Row;\n    }(core.Component));\n\n    function updateExpanderIcon(el, isExpanded) {\n        var classList = el.classList;\n        if (isExpanded) {\n            classList.remove('fc-icon-plus-square');\n            classList.add('fc-icon-minus-square');\n        }\n        else {\n            classList.remove('fc-icon-minus-square');\n            classList.add('fc-icon-plus-square');\n        }\n    }\n    function clearExpanderIcon(el) {\n        var classList = el.classList;\n        classList.remove('fc-icon-minus-square');\n        classList.remove('fc-icon-plus-square');\n    }\n    function updateTrResourceId(tr, resourceId) {\n        tr.setAttribute('data-resource-id', resourceId);\n    }\n\n    var GroupRow = /** @class */ (function (_super) {\n        __extends(GroupRow, _super);\n        function GroupRow() {\n            var _this = _super !== null && _super.apply(this, arguments) || this;\n            _this._renderCells = core.memoizeRendering(_this.renderCells, _this.unrenderCells);\n            _this._updateExpanderIcon = core.memoizeRendering(_this.updateExpanderIcon, null, [_this._renderCells]);\n            _this.onExpanderClick = function (ev) {\n                var props = _this.props;\n                _this.context.calendar.dispatch({\n                    type: 'SET_RESOURCE_ENTITY_EXPANDED',\n                    id: props.id,\n                    isExpanded: !props.isExpanded\n                });\n            };\n            return _this;\n        }\n        GroupRow.prototype.render = function (props) {\n            this._renderCells(props.group, props.spreadsheetColCnt);\n            this._updateExpanderIcon(props.isExpanded);\n            this.isSizeDirty = true;\n        };\n        GroupRow.prototype.destroy = function () {\n            _super.prototype.destroy.call(this);\n            this._renderCells.unrender(); // should unrender everything else\n        };\n        GroupRow.prototype.renderCells = function (group, spreadsheetColCnt) {\n            var spreadsheetContentEl = this.renderSpreadsheetContent(group);\n            this.spreadsheetTr.appendChild(core.createElement('td', {\n                className: 'fc-divider',\n                colSpan: spreadsheetColCnt // span across all columns\n            }, this.spreadsheetHeightEl = core.createElement('div', null, spreadsheetContentEl)) // needed by setTrInnerHeight\n            );\n            this.expanderIconEl = spreadsheetContentEl.querySelector('.fc-icon');\n            this.expanderIconEl.parentElement.addEventListener('click', this.onExpanderClick);\n            // insert a single cell, with a single empty <div>.\n            // there will be no content\n            this.timeAxisTr.appendChild(core.createElement('td', { className: 'fc-divider' }, this.timeAxisHeightEl = document.createElement('div')));\n        };\n        GroupRow.prototype.unrenderCells = function () {\n            this.spreadsheetTr.innerHTML = '';\n            this.timeAxisTr.innerHTML = '';\n        };\n        /*\n        Renders the content wrapper element that will be inserted into this row's TD cell.\n        */\n        GroupRow.prototype.renderSpreadsheetContent = function (group) {\n            var text = this.renderCellText(group);\n            var contentEl = core.htmlToElement('<div class=\"fc-cell-content\">' +\n                '<span class=\"fc-expander\">' +\n                '<span class=\"fc-icon\"></span>' +\n                '</span>' +\n                '<span class=\"fc-cell-text\">' +\n                (text ? core.htmlEscape(text) : '&nbsp;') +\n                '</span>' +\n                '</div>');\n            var filter = group.spec.render;\n            if (typeof filter === 'function') {\n                contentEl = filter(contentEl, group.value) || contentEl;\n            }\n            return contentEl;\n        };\n        GroupRow.prototype.renderCellText = function (group) {\n            var text = group.value || ''; // might be null/undefined if an ad-hoc grouping\n            var filter = group.spec.text;\n            if (typeof filter === 'function') {\n                text = filter(text) || text;\n            }\n            return text;\n        };\n        GroupRow.prototype.getHeightEls = function () {\n            return [this.spreadsheetHeightEl, this.timeAxisHeightEl];\n        };\n        GroupRow.prototype.updateExpanderIcon = function (isExpanded) {\n            updateExpanderIcon(this.expanderIconEl, isExpanded);\n        };\n        return GroupRow;\n    }(Row));\n    GroupRow.addEqualityFuncs({\n        group: ResourceCommonPlugin.isGroupsEqual // HACK for ResourceTimelineView::renderRows\n    });\n\n    var SpreadsheetRow = /** @class */ (function (_super) {\n        __extends(SpreadsheetRow, _super);\n        function SpreadsheetRow(tr) {\n            var _this = _super.call(this) || this;\n            _this._renderRow = core.memoizeRendering(_this.renderRow, _this.unrenderRow);\n            _this._updateTrResourceId = core.memoizeRendering(updateTrResourceId, null, [_this._renderRow]);\n            _this._updateExpanderIcon = core.memoizeRendering(_this.updateExpanderIcon, null, [_this._renderRow]);\n            _this.onExpanderClick = function (ev) {\n                var props = _this.props;\n                _this.context.calendar.dispatch({\n                    type: 'SET_RESOURCE_ENTITY_EXPANDED',\n                    id: props.id,\n                    isExpanded: !props.isExpanded\n                });\n            };\n            _this.tr = tr;\n            return _this;\n        }\n        SpreadsheetRow.prototype.render = function (props) {\n            this._renderRow(props.resource, props.rowSpans, props.depth, props.colSpecs);\n            this._updateTrResourceId(this.tr, props.resource.id); // TODO: only use public ID?\n            this._updateExpanderIcon(props.hasChildren, props.isExpanded);\n        };\n        SpreadsheetRow.prototype.destroy = function () {\n            _super.prototype.destroy.call(this);\n            this._renderRow.unrender(); // should unrender everything else\n        };\n        SpreadsheetRow.prototype.renderRow = function (resource, rowSpans, depth, colSpecs) {\n            var _a = this.context, calendar = _a.calendar, view = _a.view, theme = _a.theme;\n            var tr = this.tr;\n            var resourceFields = ResourceCommonPlugin.buildResourceFields(resource); // slightly inefficient. already done up the call stack\n            var mainTd;\n            for (var i = 0; i < colSpecs.length; i++) {\n                var colSpec = colSpecs[i];\n                var rowSpan = rowSpans[i];\n                if (rowSpan === 0) { // not responsible for group-based rows. VRowGroup is\n                    continue;\n                }\n                else if (rowSpan == null) {\n                    rowSpan = 1;\n                }\n                var text = void 0;\n                if (colSpec.field) {\n                    text = resourceFields[colSpec.field];\n                }\n                else {\n                    text = ResourceCommonPlugin.buildResourceTextFunc(colSpec.text, calendar)(resource);\n                }\n                var contentEl = core.htmlToElement('<div class=\"fc-cell-content\">' +\n                    (colSpec.isMain ? renderIconHtml(depth) : '') +\n                    '<span class=\"fc-cell-text\">' +\n                    (text ? core.htmlEscape(text) : '&nbsp;') +\n                    '</span>' +\n                    '</div>');\n                if (typeof colSpec.render === 'function') { // a filter function for the element\n                    contentEl = colSpec.render(new ResourceCommonPlugin.ResourceApi(calendar, resource), contentEl) || contentEl;\n                }\n                if (rowSpan > 1) {\n                    contentEl.classList.add('fc-sticky');\n                }\n                var td = core.createElement('td', {\n                    className: theme.getClass('widgetContent'),\n                    rowspan: rowSpan\n                }, contentEl);\n                // the first cell of the row needs to have an inner div for setTrInnerHeight\n                if (colSpec.isMain) {\n                    td.appendChild(this.heightEl = core.createElement('div', null, td.childNodes) // inner wrap\n                    );\n                    mainTd = td;\n                }\n                tr.appendChild(td);\n            }\n            this.expanderIconEl = tr.querySelector('.fc-expander-space .fc-icon');\n            // wait until very end\n            calendar.publiclyTrigger('resourceRender', [\n                {\n                    resource: new ResourceCommonPlugin.ResourceApi(calendar, resource),\n                    el: mainTd,\n                    view: view\n                }\n            ]);\n        };\n        SpreadsheetRow.prototype.unrenderRow = function () {\n            this.tr.innerHTML = '';\n        };\n        SpreadsheetRow.prototype.updateExpanderIcon = function (hasChildren, isExpanded) {\n            var expanderIconEl = this.expanderIconEl;\n            var expanderEl = expanderIconEl.parentElement;\n            if (expanderIconEl &&\n                expanderEl // why would this be null?? was the case in IE11\n            ) {\n                if (hasChildren) {\n                    expanderEl.addEventListener('click', this.onExpanderClick);\n                    expanderEl.classList.add('fc-expander');\n                    updateExpanderIcon(expanderIconEl, isExpanded);\n                }\n                else {\n                    expanderEl.removeEventListener('click', this.onExpanderClick);\n                    expanderEl.classList.remove('fc-expander');\n                    clearExpanderIcon(expanderIconEl);\n                }\n            }\n        };\n        return SpreadsheetRow;\n    }(core.Component));\n    /*\n    Renders the HTML responsible for the subrow expander area,\n    as well as the space before it (used to align expanders of similar depths)\n    */\n    function renderIconHtml(depth) {\n        var html = '';\n        for (var i = 0; i < depth; i++) {\n            html += '<span class=\"fc-icon\"></span>';\n        }\n        html +=\n            '<span class=\"fc-expander-space\">' +\n                '<span class=\"fc-icon\"></span>' +\n                '</span>';\n        return html;\n    }\n\n    var ResourceRow = /** @class */ (function (_super) {\n        __extends(ResourceRow, _super);\n        function ResourceRow(a, b, c, d, timeAxis) {\n            var _this = _super.call(this, a, b, c, d) || this;\n            _this.renderSkeleton = core.memoizeRendering(_this._renderSkeleton, _this._unrenderSkeleton);\n            _this.updateTrResourceId = core.memoizeRendering(updateTrResourceId);\n            _this.timeAxis = timeAxis;\n            return _this;\n        }\n        ResourceRow.prototype.render = function (props, context) {\n            this.renderSkeleton(context);\n            // spreadsheetRow handles calling updateTrResourceId for spreadsheetTr\n            this.spreadsheetRow.receiveProps({\n                colSpecs: props.colSpecs,\n                id: props.id,\n                rowSpans: props.rowSpans,\n                depth: props.depth,\n                isExpanded: props.isExpanded,\n                hasChildren: props.hasChildren,\n                resource: props.resource\n            }, context);\n            this.updateTrResourceId(this.timeAxisTr, props.resource.id);\n            this.lane.receiveProps({\n                dateProfile: props.dateProfile,\n                nextDayThreshold: props.nextDayThreshold,\n                businessHours: props.businessHours,\n                eventStore: props.eventStore,\n                eventUiBases: props.eventUiBases,\n                dateSelection: props.dateSelection,\n                eventSelection: props.eventSelection,\n                eventDrag: props.eventDrag,\n                eventResize: props.eventResize\n            }, context);\n            this.isSizeDirty = true;\n        };\n        ResourceRow.prototype.destroy = function () {\n            this.renderSkeleton.unrender();\n            _super.prototype.destroy.call(this);\n        };\n        ResourceRow.prototype._renderSkeleton = function (context) {\n            this.timeAxisTr.appendChild(this.cellEl = core.createElement('td', { className: context.theme.getClass('widgetContent') }, this.innerContainerEl = document.createElement('div')));\n            this.spreadsheetRow = new SpreadsheetRow(this.spreadsheetTr);\n            this.lane = new TimelinePlugin.TimelineLane(this.innerContainerEl, this.innerContainerEl, this.timeAxis);\n        };\n        ResourceRow.prototype._unrenderSkeleton = function () {\n            this.spreadsheetRow.destroy();\n            this.lane.destroy();\n            core.removeElement(this.cellEl);\n        };\n        ResourceRow.prototype.updateSize = function (isResize) {\n            _super.prototype.updateSize.call(this, isResize);\n            this.lane.updateSize(isResize);\n        };\n        ResourceRow.prototype.getHeightEls = function () {\n            return [this.spreadsheetRow.heightEl, this.innerContainerEl];\n        };\n        return ResourceRow;\n    }(Row));\n    ResourceRow.addEqualityFuncs({\n        rowSpans: core.isArraysEqual // HACK for isSizeDirty, ResourceTimelineView::renderRows\n    });\n\n    var COL_MIN_WIDTH = 30;\n    var SpreadsheetHeader = /** @class */ (function (_super) {\n        __extends(SpreadsheetHeader, _super);\n        function SpreadsheetHeader(parentEl) {\n            var _this = _super.call(this) || this;\n            _this.resizables = [];\n            _this.colWidths = [];\n            _this.emitter = new core.EmitterMixin();\n            _this.renderSkeleton = core.memoizeRendering(_this._renderSkeleton, _this._unrenderSkeleton);\n            _this.parentEl = parentEl;\n            return _this;\n        }\n        SpreadsheetHeader.prototype.render = function (props, context) {\n            this.renderSkeleton(context);\n            var theme = context.theme;\n            var colSpecs = props.colSpecs;\n            var html = '<colgroup>' + props.colTags + '</colgroup>' +\n                '<tbody>';\n            if (props.superHeaderText) {\n                html +=\n                    '<tr class=\"fc-super\">' +\n                        '<th class=\"' + theme.getClass('widgetHeader') + '\" colspan=\"' + colSpecs.length + '\">' +\n                        '<div class=\"fc-cell-content\">' +\n                        '<span class=\"fc-cell-text\">' +\n                        core.htmlEscape(props.superHeaderText) +\n                        '</span>' +\n                        '</div>' +\n                        '</th>' +\n                        '</tr>';\n            }\n            html += '<tr>';\n            for (var i = 0; i < colSpecs.length; i++) {\n                var o = colSpecs[i];\n                var isLast = i === (colSpecs.length - 1);\n                html +=\n                    \"<th class=\\\"\" + theme.getClass('widgetHeader') + \"\\\">\" +\n                        '<div>' +\n                        '<div class=\"fc-cell-content\">' +\n                        (o.isMain ?\n                            '<span class=\"fc-expander-space\">' +\n                                '<span class=\"fc-icon\"></span>' +\n                                '</span>' :\n                            '') +\n                        '<span class=\"fc-cell-text\">' +\n                        core.htmlEscape(o.labelText || '') + // what about normalizing this value ahead of time?\n                        '</span>' +\n                        '</div>' +\n                        (!isLast ? '<div class=\"fc-col-resizer\"></div>' : '') +\n                        '</div>' +\n                        '</th>';\n            }\n            html += '</tr>';\n            html += '</tbody>';\n            this.tableEl.innerHTML = html;\n            this.thEls = Array.prototype.slice.call(this.tableEl.querySelectorAll('th'));\n            this.colEls = Array.prototype.slice.call(this.tableEl.querySelectorAll('col'));\n            this.resizerEls = Array.prototype.slice.call(this.tableEl.querySelectorAll('.fc-col-resizer'));\n            this.initColResizing();\n        };\n        SpreadsheetHeader.prototype.destroy = function () {\n            for (var _i = 0, _a = this.resizables; _i < _a.length; _i++) {\n                var resizable = _a[_i];\n                resizable.destroy();\n            }\n            this.renderSkeleton.unrender();\n            _super.prototype.destroy.call(this);\n        };\n        SpreadsheetHeader.prototype._renderSkeleton = function (context) {\n            this.parentEl.appendChild(this.tableEl = core.createElement('table', {\n                className: context.theme.getClass('tableGrid')\n            }));\n        };\n        SpreadsheetHeader.prototype._unrenderSkeleton = function () {\n            core.removeElement(this.tableEl);\n        };\n        SpreadsheetHeader.prototype.initColResizing = function () {\n            var _this = this;\n            var _a = this.context, calendar = _a.calendar, isRtl = _a.isRtl;\n            var ElementDraggingImpl = calendar.pluginSystem.hooks.elementDraggingImpl;\n            if (ElementDraggingImpl) {\n                this.resizables = this.resizerEls.map(function (handleEl, colIndex) {\n                    var dragging = new ElementDraggingImpl(handleEl);\n                    var startWidth;\n                    dragging.emitter.on('dragstart', function () {\n                        startWidth = _this.colWidths[colIndex];\n                        if (typeof startWidth !== 'number') {\n                            startWidth = _this.thEls[colIndex].getBoundingClientRect().width;\n                        }\n                    });\n                    dragging.emitter.on('dragmove', function (pev) {\n                        _this.colWidths[colIndex] = Math.max(startWidth + pev.deltaX * (isRtl ? -1 : 1), COL_MIN_WIDTH);\n                        _this.emitter.trigger('colwidthchange', _this.colWidths);\n                    });\n                    dragging.setAutoScrollEnabled(false); // because gets weird with auto-scrolling time area\n                    return dragging;\n                });\n            }\n        };\n        return SpreadsheetHeader;\n    }(core.Component));\n\n    var Spreadsheet = /** @class */ (function (_super) {\n        __extends(Spreadsheet, _super);\n        function Spreadsheet(headParentEl, bodyParentEl) {\n            var _this = _super.call(this) || this;\n            _this.renderSkeleton = core.memoizeRendering(_this._renderSkeleton, _this._unrenderSkeleton);\n            _this.renderCells = core.memoizeRendering(_this._renderCells, _this._unrenderCells, [_this.renderSkeleton]);\n            _this.layout = new TimelinePlugin.HeaderBodyLayout(headParentEl, bodyParentEl, 'clipped-scroll');\n            return _this;\n        }\n        Spreadsheet.prototype.render = function (props, context) {\n            this.renderSkeleton(context);\n            this.renderCells(props.superHeaderText, props.colSpecs);\n        };\n        Spreadsheet.prototype.destroy = function () {\n            this.renderCells.unrender();\n            this.renderSkeleton.unrender();\n            this.layout.destroy();\n            _super.prototype.destroy.call(this);\n        };\n        Spreadsheet.prototype._renderSkeleton = function (context) {\n            var _this = this;\n            var bodyEnhancedScroller = this.layout.bodyScroller.enhancedScroll;\n            bodyEnhancedScroller.canvas.contentEl\n                .appendChild(this.bodyContainerEl = core.createElement('div', { className: 'fc-rows' }, '<table>' +\n                '<colgroup />' +\n                '<tbody />' +\n                '</table>'));\n            this.bodyColGroup = this.bodyContainerEl.querySelector('colgroup');\n            this.bodyTbody = this.bodyContainerEl.querySelector('tbody');\n            var headerEnhancedScroller = this.layout.headerScroller.enhancedScroll;\n            this.header = new SpreadsheetHeader(headerEnhancedScroller.canvas.contentEl);\n            this.header.emitter.on('colwidthchange', function (colWidths) {\n                _this.applyColWidths(colWidths);\n            });\n        };\n        Spreadsheet.prototype._unrenderSkeleton = function () {\n            this.header.destroy();\n        };\n        Spreadsheet.prototype._renderCells = function (superHeaderText, colSpecs) {\n            var colTags = this.renderColTags(colSpecs);\n            this.header.receiveProps({\n                superHeaderText: superHeaderText,\n                colSpecs: colSpecs,\n                colTags: colTags\n            }, this.context);\n            this.bodyColGroup.innerHTML = colTags;\n            this.bodyColEls = Array.prototype.slice.call(this.bodyColGroup.querySelectorAll('col'));\n            this.applyColWidths(colSpecs.map(function (colSpec) { return colSpec.width; }));\n        };\n        Spreadsheet.prototype._unrenderCells = function () {\n            this.bodyColGroup.innerHTML = '';\n        };\n        Spreadsheet.prototype.renderColTags = function (colSpecs) {\n            var html = '';\n            for (var _i = 0, colSpecs_1 = colSpecs; _i < colSpecs_1.length; _i++) {\n                var o = colSpecs_1[_i];\n                if (o.isMain) {\n                    html += '<col class=\"fc-main-col\"/>';\n                }\n                else {\n                    html += '<col/>';\n                }\n            }\n            return html;\n        };\n        Spreadsheet.prototype.updateSize = function (isResize, totalHeight, isAuto) {\n            this.layout.setHeight(totalHeight, isAuto);\n        };\n        Spreadsheet.prototype.applyColWidths = function (colWidths) {\n            var _this = this;\n            colWidths.forEach(function (colWidth, colIndex) {\n                var headEl = _this.header.colEls[colIndex]; // bad to access child\n                var bodyEl = _this.bodyColEls[colIndex];\n                var styleVal;\n                if (typeof colWidth === 'number') {\n                    styleVal = colWidth + 'px';\n                }\n                else if (typeof colWidth == null) {\n                    styleVal = '';\n                }\n                headEl.style.width = bodyEl.style.width = styleVal;\n            });\n        };\n        return Spreadsheet;\n    }(core.Component));\n\n    var MIN_RESOURCE_AREA_WIDTH = 30; // definitely bigger than scrollbars\n    var ResourceTimelineView = /** @class */ (function (_super) {\n        __extends(ResourceTimelineView, _super);\n        function ResourceTimelineView() {\n            var _this = _super !== null && _super.apply(this, arguments) || this;\n            _this.isStickyScrollDirty = false;\n            _this.rowNodes = [];\n            _this.rowComponents = [];\n            _this.rowComponentsById = {};\n            _this.resourceAreaWidthDraggings = [];\n            _this.splitter = new ResourceCommonPlugin.ResourceSplitter(); // doesn't let it do businessHours tho\n            _this.renderSkeleton = core.memoizeRendering(_this._renderSkeleton, _this._unrenderSkeleton);\n            _this.hasResourceBusinessHours = core.memoize(hasResourceBusinessHours);\n            _this.buildRowNodes = core.memoize(ResourceCommonPlugin.buildRowNodes);\n            _this.hasNesting = core.memoize(hasNesting);\n            _this.updateHasNesting = core.memoizeRendering(_this._updateHasNesting);\n            _this.startInteractive = core.memoizeRendering(_this._startInteractive, _this._stopInteractive);\n            return _this;\n        }\n        ResourceTimelineView.prototype._startInteractive = function (timeAxisEl) {\n            this.context.calendar.registerInteractiveComponent(this, { el: timeAxisEl });\n        };\n        ResourceTimelineView.prototype._stopInteractive = function () {\n            this.context.calendar.unregisterInteractiveComponent(this);\n        };\n        ResourceTimelineView.prototype.render = function (props, context) {\n            _super.prototype.render.call(this, props, context);\n            this.renderSkeleton(context);\n            var splitProps = this.splitter.splitProps(props);\n            var hasResourceBusinessHours = this.hasResourceBusinessHours(props.resourceStore);\n            this.spreadsheet.receiveProps({\n                superHeaderText: this.superHeaderText,\n                colSpecs: this.colSpecs\n            }, context);\n            this.timeAxis.receiveProps({\n                dateProfileGenerator: props.dateProfileGenerator,\n                dateProfile: props.dateProfile\n            }, context);\n            this.startInteractive(this.timeAxis.slats.el);\n            // for all-resource bg events / selections / business-hours\n            this.lane.receiveProps(__assign({}, splitProps[''], { dateProfile: props.dateProfile, nextDayThreshold: context.nextDayThreshold, businessHours: hasResourceBusinessHours ? null : props.businessHours }), context);\n            var newRowNodes = this.buildRowNodes(props.resourceStore, this.groupSpecs, this.orderSpecs, this.isVGrouping, props.resourceEntityExpansions, context.options.resourcesInitiallyExpanded);\n            this.updateHasNesting(this.hasNesting(newRowNodes));\n            this.diffRows(newRowNodes);\n            this.updateRowProps(props.dateProfile, hasResourceBusinessHours ? props.businessHours : null, // CONFUSING, comment\n            splitProps);\n            this.startNowIndicator(props.dateProfile, props.dateProfileGenerator);\n        };\n        ResourceTimelineView.prototype._renderSkeleton = function (context) {\n            var options = context.options, calendar = context.calendar;\n            var allColSpecs = options.resourceColumns || [];\n            var labelText = options.resourceLabelText; // TODO: view.override\n            var defaultLabelText = 'Resources'; // TODO: view.defaults\n            var superHeaderText = null;\n            if (!allColSpecs.length) {\n                allColSpecs.push({\n                    labelText: labelText || defaultLabelText,\n                    text: ResourceCommonPlugin.buildResourceTextFunc(options.resourceText, calendar)\n                });\n            }\n            else {\n                superHeaderText = labelText;\n            }\n            var plainColSpecs = [];\n            var groupColSpecs = [];\n            var groupSpecs = [];\n            var isVGrouping = false;\n            var isHGrouping = false;\n            for (var _i = 0, allColSpecs_1 = allColSpecs; _i < allColSpecs_1.length; _i++) {\n                var colSpec = allColSpecs_1[_i];\n                if (colSpec.group) {\n                    groupColSpecs.push(colSpec);\n                }\n                else {\n                    plainColSpecs.push(colSpec);\n                }\n            }\n            plainColSpecs[0].isMain = true;\n            if (groupColSpecs.length) {\n                groupSpecs = groupColSpecs;\n                isVGrouping = true;\n            }\n            else {\n                var hGroupField = options.resourceGroupField;\n                if (hGroupField) {\n                    isHGrouping = true;\n                    groupSpecs.push({\n                        field: hGroupField,\n                        text: options.resourceGroupText,\n                        render: options.resourceGroupRender\n                    });\n                }\n            }\n            var allOrderSpecs = core.parseFieldSpecs(options.resourceOrder);\n            var plainOrderSpecs = [];\n            for (var _a = 0, allOrderSpecs_1 = allOrderSpecs; _a < allOrderSpecs_1.length; _a++) {\n                var orderSpec = allOrderSpecs_1[_a];\n                var isGroup = false;\n                for (var _b = 0, groupSpecs_1 = groupSpecs; _b < groupSpecs_1.length; _b++) {\n                    var groupSpec = groupSpecs_1[_b];\n                    if (groupSpec.field === orderSpec.field) {\n                        groupSpec.order = orderSpec.order; // -1, 0, 1\n                        isGroup = true;\n                        break;\n                    }\n                }\n                if (!isGroup) {\n                    plainOrderSpecs.push(orderSpec);\n                }\n            }\n            this.superHeaderText = superHeaderText;\n            this.isVGrouping = isVGrouping;\n            this.isHGrouping = isHGrouping;\n            this.groupSpecs = groupSpecs;\n            this.colSpecs = groupColSpecs.concat(plainColSpecs);\n            this.orderSpecs = plainOrderSpecs;\n            // START RENDERING...\n            this.el.classList.add('fc-timeline');\n            if (options.eventOverlap === false) {\n                this.el.classList.add('fc-no-overlap');\n            }\n            this.el.innerHTML = this.renderSkeletonHtml();\n            this.resourceAreaHeadEl = this.el.querySelector('thead .fc-resource-area');\n            this.setResourceAreaWidth(options.resourceAreaWidth);\n            this.initResourceAreaWidthDragging();\n            this.miscHeight = this.el.getBoundingClientRect().height;\n            this.spreadsheet = new Spreadsheet(this.resourceAreaHeadEl, this.el.querySelector('tbody .fc-resource-area'));\n            this.timeAxis = new TimelinePlugin.TimeAxis(this.el.querySelector('thead .fc-time-area'), this.el.querySelector('tbody .fc-time-area'));\n            var timeAxisRowContainer = core.createElement('div', { className: 'fc-rows' }, '<table><tbody /></table>');\n            this.timeAxis.layout.bodyScroller.enhancedScroll.canvas.contentEl.appendChild(timeAxisRowContainer);\n            this.timeAxisTbody = timeAxisRowContainer.querySelector('tbody');\n            this.lane = new TimelinePlugin.TimelineLane(null, this.timeAxis.layout.bodyScroller.enhancedScroll.canvas.bgEl, this.timeAxis);\n            this.bodyScrollJoiner = new TimelinePlugin.ScrollJoiner('vertical', [\n                this.spreadsheet.layout.bodyScroller,\n                this.timeAxis.layout.bodyScroller\n            ]);\n            // after scrolljoiner\n            this.spreadsheetBodyStickyScroller = new TimelinePlugin.StickyScroller(this.spreadsheet.layout.bodyScroller.enhancedScroll, context.isRtl, true // isVertical\n            );\n        };\n        ResourceTimelineView.prototype._unrenderSkeleton = function (context) {\n            this.startInteractive.unrender(); // \"unrender\" bad name\n            this.destroyRows(); // wierd to call this here\n            this.spreadsheet.destroy();\n            this.timeAxis.destroy();\n            this.lane.destroy();\n            this.spreadsheetBodyStickyScroller.destroy();\n            this.el.classList.remove('fc-timeline');\n            this.el.classList.remove('fc-no-overlap');\n        };\n        ResourceTimelineView.prototype.renderSkeletonHtml = function () {\n            var theme = this.context.theme;\n            return \"<table class=\\\"\" + theme.getClass('tableGrid') + \"\\\"> <thead class=\\\"fc-head\\\"> <tr> <td class=\\\"fc-resource-area \" + theme.getClass('widgetHeader') + \"\\\"></td> <td class=\\\"fc-divider fc-col-resizer \" + theme.getClass('widgetHeader') + \"\\\"></td> <td class=\\\"fc-time-area \" + theme.getClass('widgetHeader') + \"\\\"></td> </tr> </thead> <tbody class=\\\"fc-body\\\"> <tr> <td class=\\\"fc-resource-area \" + theme.getClass('widgetContent') + \"\\\"></td> <td class=\\\"fc-divider fc-col-resizer \" + theme.getClass('widgetHeader') + \"\\\"></td> <td class=\\\"fc-time-area \" + theme.getClass('widgetContent') + \"\\\"></td> </tr> </tbody> </table>\";\n        };\n        ResourceTimelineView.prototype._updateHasNesting = function (isNesting) {\n            var classList = this.el.classList;\n            if (isNesting) {\n                classList.remove('fc-flat');\n            }\n            else {\n                classList.add('fc-flat');\n            }\n        };\n        ResourceTimelineView.prototype.diffRows = function (newNodes) {\n            var oldNodes = this.rowNodes;\n            var oldLen = oldNodes.length;\n            var oldIndexHash = {}; // id -> index\n            var oldI = 0;\n            var newI = 0;\n            for (oldI = 0; oldI < oldLen; oldI++) {\n                oldIndexHash[oldNodes[oldI].id] = oldI;\n            }\n            // iterate new nodes\n            for (oldI = 0, newI = 0; newI < newNodes.length; newI++) {\n                var newNode = newNodes[newI];\n                var oldIFound = oldIndexHash[newNode.id];\n                if (oldIFound != null && oldIFound >= oldI) {\n                    this.removeRows(newI, oldIFound - oldI, oldNodes); // won't do anything if same index\n                    oldI = oldIFound + 1;\n                }\n                else {\n                    this.addRow(newI, newNode);\n                }\n            }\n            // old rows that weren't found need to be removed\n            this.removeRows(newI, oldLen - oldI, oldNodes); // won't do anything if same index\n            this.rowNodes = newNodes;\n        };\n        /*\n        rowComponents is the in-progress result\n        */\n        ResourceTimelineView.prototype.addRow = function (index, rowNode) {\n            var _a = this, rowComponents = _a.rowComponents, rowComponentsById = _a.rowComponentsById;\n            var nextComponent = rowComponents[index];\n            var newComponent = this.buildChildComponent(rowNode, this.spreadsheet.bodyTbody, nextComponent ? nextComponent.spreadsheetTr : null, this.timeAxisTbody, nextComponent ? nextComponent.timeAxisTr : null);\n            rowComponents.splice(index, 0, newComponent);\n            rowComponentsById[rowNode.id] = newComponent;\n        };\n        ResourceTimelineView.prototype.removeRows = function (startIndex, len, oldRowNodes) {\n            if (len) {\n                var _a = this, rowComponents = _a.rowComponents, rowComponentsById = _a.rowComponentsById;\n                for (var i = 0; i < len; i++) {\n                    var rowComponent = rowComponents[startIndex + i];\n                    rowComponent.destroy();\n                    delete rowComponentsById[oldRowNodes[i].id];\n                }\n                rowComponents.splice(startIndex, len);\n            }\n        };\n        ResourceTimelineView.prototype.buildChildComponent = function (node, spreadsheetTbody, spreadsheetNext, timeAxisTbody, timeAxisNext) {\n            if (node.group) {\n                return new GroupRow(spreadsheetTbody, spreadsheetNext, timeAxisTbody, timeAxisNext);\n            }\n            else if (node.resource) {\n                return new ResourceRow(spreadsheetTbody, spreadsheetNext, timeAxisTbody, timeAxisNext, this.timeAxis);\n            }\n        };\n        ResourceTimelineView.prototype.updateRowProps = function (dateProfile, fallbackBusinessHours, splitProps) {\n            var _a = this, rowNodes = _a.rowNodes, rowComponents = _a.rowComponents, context = _a.context;\n            for (var i = 0; i < rowNodes.length; i++) {\n                var rowNode = rowNodes[i];\n                var rowComponent = rowComponents[i];\n                if (rowNode.group) {\n                    rowComponent.receiveProps({\n                        spreadsheetColCnt: this.colSpecs.length,\n                        id: rowNode.id,\n                        isExpanded: rowNode.isExpanded,\n                        group: rowNode.group\n                    }, context);\n                }\n                else {\n                    var resource = rowNode.resource;\n                    rowComponent.receiveProps(__assign({}, splitProps[resource.id], { dateProfile: dateProfile, nextDayThreshold: context.nextDayThreshold, businessHours: resource.businessHours || fallbackBusinessHours, colSpecs: this.colSpecs, id: rowNode.id, rowSpans: rowNode.rowSpans, depth: rowNode.depth, isExpanded: rowNode.isExpanded, hasChildren: rowNode.hasChildren, resource: rowNode.resource }), context);\n                }\n            }\n        };\n        ResourceTimelineView.prototype.updateSize = function (isResize, viewHeight, isAuto) {\n            // FYI: this ordering is really important\n            var calendar = this.context.calendar;\n            var isBaseSizing = isResize || calendar.isViewUpdated || calendar.isDatesUpdated || calendar.isEventsUpdated;\n            if (isBaseSizing) {\n                this.syncHeadHeights();\n            }\n            // TODO: don't always call these (but guarding behind isBaseSizing was unreliable)\n            this.timeAxis.updateSize(isResize, viewHeight - this.miscHeight, isAuto);\n            this.spreadsheet.updateSize(isResize, viewHeight - this.miscHeight, isAuto);\n            var rowSizingCnt = this.updateRowSizes(isResize);\n            this.lane.updateSize(isResize); // is efficient. uses flags\n            if (isBaseSizing || rowSizingCnt) {\n                this.bodyScrollJoiner.update();\n                this.timeAxis.layout.scrollJoiner.update(); // hack\n                this.rowPositions = new core.PositionCache(this.timeAxis.slats.el, this.rowComponents.map(function (rowComponent) {\n                    return rowComponent.timeAxisTr;\n                }), false, // isHorizontal\n                true // isVertical\n                );\n                this.rowPositions.build();\n                this.isStickyScrollDirty = true;\n            }\n        };\n        ResourceTimelineView.prototype.syncHeadHeights = function () {\n            var spreadsheetHeadEl = this.spreadsheet.header.tableEl;\n            var timeAxisHeadEl = this.timeAxis.header.tableEl;\n            spreadsheetHeadEl.style.height = '';\n            timeAxisHeadEl.style.height = '';\n            var max = Math.max(spreadsheetHeadEl.getBoundingClientRect().height, timeAxisHeadEl.getBoundingClientRect().height);\n            spreadsheetHeadEl.style.height =\n                timeAxisHeadEl.style.height = max + 'px';\n        };\n        ResourceTimelineView.prototype.updateRowSizes = function (isResize) {\n            var dirtyRowComponents = this.rowComponents;\n            if (!isResize) {\n                dirtyRowComponents = dirtyRowComponents.filter(function (rowComponent) {\n                    return rowComponent.isSizeDirty;\n                });\n            }\n            var elArrays = dirtyRowComponents.map(function (rowComponent) {\n                return rowComponent.getHeightEls();\n            });\n            // reset to natural heights\n            for (var _i = 0, elArrays_1 = elArrays; _i < elArrays_1.length; _i++) {\n                var elArray = elArrays_1[_i];\n                for (var _a = 0, elArray_1 = elArray; _a < elArray_1.length; _a++) {\n                    var el = elArray_1[_a];\n                    el.style.height = '';\n                }\n            }\n            // let rows update their contents' heights\n            for (var _b = 0, dirtyRowComponents_1 = dirtyRowComponents; _b < dirtyRowComponents_1.length; _b++) {\n                var rowComponent = dirtyRowComponents_1[_b];\n                rowComponent.updateSize(isResize); // will reset isSizeDirty\n            }\n            var maxHeights = elArrays.map(function (elArray) {\n                var maxHeight = null;\n                for (var _i = 0, elArray_2 = elArray; _i < elArray_2.length; _i++) {\n                    var el = elArray_2[_i];\n                    var height = el.getBoundingClientRect().height;\n                    if (maxHeight === null || height > maxHeight) {\n                        maxHeight = height;\n                    }\n                }\n                return maxHeight;\n            });\n            for (var i = 0; i < elArrays.length; i++) {\n                for (var _c = 0, _d = elArrays[i]; _c < _d.length; _c++) {\n                    var el = _d[_c];\n                    el.style.height = maxHeights[i] + 'px';\n                }\n            }\n            return dirtyRowComponents.length;\n        };\n        ResourceTimelineView.prototype.destroyRows = function () {\n            for (var _i = 0, _a = this.rowComponents; _i < _a.length; _i++) {\n                var rowComponent = _a[_i];\n                rowComponent.destroy();\n            }\n            this.rowNodes = [];\n            this.rowComponents = [];\n        };\n        ResourceTimelineView.prototype.destroy = function () {\n            for (var _i = 0, _a = this.resourceAreaWidthDraggings; _i < _a.length; _i++) {\n                var resourceAreaWidthDragging = _a[_i];\n                resourceAreaWidthDragging.destroy();\n            }\n            this.renderSkeleton.unrender(); // will call destroyRows\n            _super.prototype.destroy.call(this);\n        };\n        // Now Indicator\n        // ------------------------------------------------------------------------------------------\n        ResourceTimelineView.prototype.getNowIndicatorUnit = function (dateProfile, dateProfileGenerator) {\n            return this.timeAxis.getNowIndicatorUnit(dateProfile, dateProfileGenerator);\n        };\n        ResourceTimelineView.prototype.renderNowIndicator = function (date) {\n            this.timeAxis.renderNowIndicator(date);\n        };\n        ResourceTimelineView.prototype.unrenderNowIndicator = function () {\n            this.timeAxis.unrenderNowIndicator();\n        };\n        // Scrolling\n        // ------------------------------------------------------------------------------------------------------------------\n        // this is useful for scrolling prev/next dates while resource is scrolled down\n        ResourceTimelineView.prototype.queryScroll = function () {\n            var scroll = _super.prototype.queryScroll.call(this);\n            if (this.props.resourceStore) {\n                __assign(scroll, this.queryResourceScroll());\n            }\n            return scroll;\n        };\n        ResourceTimelineView.prototype.applyScroll = function (scroll, isResize) {\n            _super.prototype.applyScroll.call(this, scroll, isResize);\n            if (this.props.resourceStore) {\n                this.applyResourceScroll(scroll);\n            }\n            // avoid updating stickyscroll too often\n            if (isResize || this.isStickyScrollDirty) {\n                this.isStickyScrollDirty = false;\n                this.spreadsheetBodyStickyScroller.updateSize();\n                this.timeAxis.updateStickyScrollers();\n            }\n        };\n        ResourceTimelineView.prototype.computeDateScroll = function (duration) {\n            return this.timeAxis.computeDateScroll(duration);\n        };\n        ResourceTimelineView.prototype.queryDateScroll = function () {\n            return this.timeAxis.queryDateScroll();\n        };\n        ResourceTimelineView.prototype.applyDateScroll = function (scroll) {\n            this.timeAxis.applyDateScroll(scroll);\n        };\n        ResourceTimelineView.prototype.queryResourceScroll = function () {\n            var _a = this, rowComponents = _a.rowComponents, rowNodes = _a.rowNodes;\n            var scroll = {};\n            var scrollerTop = this.timeAxis.layout.bodyScroller.el.getBoundingClientRect().top; // fixed position\n            for (var i = 0; i < rowComponents.length; i++) {\n                var rowComponent = rowComponents[i];\n                var rowNode = rowNodes[i];\n                var el = rowComponent.timeAxisTr;\n                var elBottom = el.getBoundingClientRect().bottom; // fixed position\n                if (elBottom > scrollerTop) {\n                    scroll.rowId = rowNode.id;\n                    scroll.bottom = elBottom - scrollerTop;\n                    break;\n                }\n            }\n            // TODO: what about left scroll state for spreadsheet area?\n            return scroll;\n        };\n        ResourceTimelineView.prototype.applyResourceScroll = function (scroll) {\n            var rowId = scroll.forcedRowId || scroll.rowId;\n            if (rowId) {\n                var rowComponent = this.rowComponentsById[rowId];\n                if (rowComponent) {\n                    var el = rowComponent.timeAxisTr;\n                    if (el) {\n                        var innerTop = this.timeAxis.layout.bodyScroller.enhancedScroll.canvas.el.getBoundingClientRect().top;\n                        var rowRect = el.getBoundingClientRect();\n                        var scrollTop = (scroll.forcedRowId ?\n                            rowRect.top : // just use top edge\n                            rowRect.bottom - scroll.bottom) - // pixels from bottom edge\n                            innerTop;\n                        this.timeAxis.layout.bodyScroller.enhancedScroll.setScrollTop(scrollTop);\n                        this.spreadsheet.layout.bodyScroller.enhancedScroll.setScrollTop(scrollTop);\n                    }\n                }\n            }\n        };\n        // TODO: scrollToResource\n        // Hit System\n        // ------------------------------------------------------------------------------------------\n        ResourceTimelineView.prototype.buildPositionCaches = function () {\n            this.timeAxis.slats.updateSize();\n            this.rowPositions.build();\n        };\n        ResourceTimelineView.prototype.queryHit = function (positionLeft, positionTop) {\n            var rowPositions = this.rowPositions;\n            var slats = this.timeAxis.slats;\n            var rowIndex = rowPositions.topToIndex(positionTop);\n            if (rowIndex != null) {\n                var resource = this.rowNodes[rowIndex].resource;\n                if (resource) { // not a group\n                    var slatHit = slats.positionToHit(positionLeft);\n                    if (slatHit) {\n                        return {\n                            component: this,\n                            dateSpan: {\n                                range: slatHit.dateSpan.range,\n                                allDay: slatHit.dateSpan.allDay,\n                                resourceId: resource.id\n                            },\n                            rect: {\n                                left: slatHit.left,\n                                right: slatHit.right,\n                                top: rowPositions.tops[rowIndex],\n                                bottom: rowPositions.bottoms[rowIndex]\n                            },\n                            dayEl: slatHit.dayEl,\n                            layer: 0\n                        };\n                    }\n                }\n            }\n        };\n        // Resource Area\n        // ------------------------------------------------------------------------------------------------------------------\n        ResourceTimelineView.prototype.setResourceAreaWidth = function (widthVal) {\n            this.resourceAreaWidth = widthVal;\n            core.applyStyleProp(this.resourceAreaHeadEl, 'width', widthVal || '');\n        };\n        ResourceTimelineView.prototype.initResourceAreaWidthDragging = function () {\n            var _this = this;\n            var _a = this.context, calendar = _a.calendar, isRtl = _a.isRtl;\n            var resourceAreaDividerEls = Array.prototype.slice.call(this.el.querySelectorAll('.fc-col-resizer'));\n            var ElementDraggingImpl = calendar.pluginSystem.hooks.elementDraggingImpl;\n            if (ElementDraggingImpl) {\n                this.resourceAreaWidthDraggings = resourceAreaDividerEls.map(function (el) {\n                    var dragging = new ElementDraggingImpl(el);\n                    var dragStartWidth;\n                    var viewWidth;\n                    dragging.emitter.on('dragstart', function () {\n                        dragStartWidth = _this.resourceAreaWidth;\n                        if (typeof dragStartWidth !== 'number') {\n                            dragStartWidth = _this.resourceAreaHeadEl.getBoundingClientRect().width;\n                        }\n                        viewWidth = _this.el.getBoundingClientRect().width;\n                    });\n                    dragging.emitter.on('dragmove', function (pev) {\n                        var newWidth = dragStartWidth + pev.deltaX * (isRtl ? -1 : 1);\n                        newWidth = Math.max(newWidth, MIN_RESOURCE_AREA_WIDTH);\n                        newWidth = Math.min(newWidth, viewWidth - MIN_RESOURCE_AREA_WIDTH);\n                        _this.setResourceAreaWidth(newWidth);\n                    });\n                    dragging.setAutoScrollEnabled(false); // because gets weird with auto-scrolling time area\n                    return dragging;\n                });\n            }\n        };\n        ResourceTimelineView.needsResourceData = true; // for ResourceViewProps\n        return ResourceTimelineView;\n    }(core.View));\n    function hasResourceBusinessHours(resourceStore) {\n        for (var resourceId in resourceStore) {\n            var resource = resourceStore[resourceId];\n            if (resource.businessHours) {\n                return true;\n            }\n        }\n        return false;\n    }\n    function hasNesting(nodes) {\n        for (var _i = 0, nodes_1 = nodes; _i < nodes_1.length; _i++) {\n            var node = nodes_1[_i];\n            if (node.group) {\n                return true;\n            }\n            else if (node.resource) {\n                if (node.hasChildren) {\n                    return true;\n                }\n            }\n        }\n        return false;\n    }\n\n    var main = core.createPlugin({\n        deps: [ResourceCommonPlugin__default, TimelinePlugin__default],\n        defaultView: 'resourceTimelineDay',\n        views: {\n            resourceTimeline: {\n                class: ResourceTimelineView,\n                resourceAreaWidth: '30%',\n                resourcesInitiallyExpanded: true,\n                eventResizableFromStart: true // TODO: not DRY with this same setting in the main timeline config\n            },\n            resourceTimelineDay: {\n                type: 'resourceTimeline',\n                duration: { days: 1 }\n            },\n            resourceTimelineWeek: {\n                type: 'resourceTimeline',\n                duration: { weeks: 1 }\n            },\n            resourceTimelineMonth: {\n                type: 'resourceTimeline',\n                duration: { months: 1 }\n            },\n            resourceTimelineYear: {\n                type: 'resourceTimeline',\n                duration: { years: 1 }\n            }\n        }\n    });\n\n    exports.ResourceTimelineView = ResourceTimelineView;\n    exports.default = main;\n\n    Object.defineProperty(exports, '__esModule', { value: true });\n\n}));\n"

/***/ },

/***/ 1099
(module) {

module.exports = "/*!\nFullCalendar RRule Plugin v4.4.2\nDocs & License: https://fullcalendar.io/\n(c) 2019 Adam Shaw\n*/\n\n(function (global, factory) {\n    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('rrule'), require('@fullcalendar/core')) :\n    typeof define === 'function' && define.amd ? define(['exports', 'rrule', '@fullcalendar/core'], factory) :\n    (global = global || self, factory(global.FullCalendarRrule = {}, global.rrule, global.FullCalendar));\n}(this, function (exports, rrule, core) { 'use strict';\n\n    /*! *****************************************************************************\r\n    Copyright (c) Microsoft Corporation.\r\n\r\n    Permission to use, copy, modify, and/or distribute this software for any\r\n    purpose with or without fee is hereby granted.\r\n\r\n    THE SOFTWARE IS PROVIDED \"AS IS\" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH\r\n    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY\r\n    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,\r\n    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM\r\n    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR\r\n    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR\r\n    PERFORMANCE OF THIS SOFTWARE.\r\n    ***************************************************************************** */\r\n\r\n    var __assign = function() {\r\n        __assign = Object.assign || function __assign(t) {\r\n            for (var s, i = 1, n = arguments.length; i < n; i++) {\r\n                s = arguments[i];\r\n                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];\r\n            }\r\n            return t;\r\n        };\r\n        return __assign.apply(this, arguments);\r\n    };\n\n    var EVENT_DEF_PROPS = {\n        rrule: null,\n        duration: core.createDuration\n    };\n    var recurring = {\n        parse: function (rawEvent, leftoverProps, dateEnv) {\n            if (rawEvent.rrule != null) {\n                var props = core.refineProps(rawEvent, EVENT_DEF_PROPS, {}, leftoverProps);\n                var parsed = parseRRule(props.rrule, dateEnv);\n                if (parsed) {\n                    return {\n                        typeData: parsed.rrule,\n                        allDayGuess: parsed.allDayGuess,\n                        duration: props.duration\n                    };\n                }\n            }\n            return null;\n        },\n        expand: function (rrule, framingRange) {\n            // we WANT an inclusive start and in exclusive end, but the js rrule lib will only do either BOTH\n            // inclusive or BOTH exclusive, which is stupid: https://github.com/jakubroztocil/rrule/issues/84\n            // Workaround: make inclusive, which will generate extra occurences, and then trim.\n            return rrule.between(framingRange.start, framingRange.end, true)\n                .filter(function (date) {\n                return date.valueOf() < framingRange.end.valueOf();\n            });\n        }\n    };\n    var main = core.createPlugin({\n        recurringTypes: [recurring]\n    });\n    function parseRRule(input, dateEnv) {\n        var allDayGuess = null;\n        var rrule$1;\n        if (typeof input === 'string') {\n            rrule$1 = rrule.rrulestr(input);\n        }\n        else if (typeof input === 'object' && input) { // non-null object\n            var refined = __assign({}, input); // copy\n            if (typeof refined.dtstart === 'string') {\n                var dtstartMeta = dateEnv.createMarkerMeta(refined.dtstart);\n                if (dtstartMeta) {\n                    refined.dtstart = dtstartMeta.marker;\n                    allDayGuess = dtstartMeta.isTimeUnspecified;\n                }\n                else {\n                    delete refined.dtstart;\n                }\n            }\n            if (typeof refined.until === 'string') {\n                refined.until = dateEnv.createMarker(refined.until);\n            }\n            if (refined.freq != null) {\n                refined.freq = convertConstant(refined.freq);\n            }\n            if (refined.wkst != null) {\n                refined.wkst = convertConstant(refined.wkst);\n            }\n            else {\n                refined.wkst = (dateEnv.weekDow - 1 + 7) % 7; // convert Sunday-first to Monday-first\n            }\n            if (refined.byweekday != null) {\n                refined.byweekday = convertConstants(refined.byweekday); // the plural version\n            }\n            rrule$1 = new rrule.RRule(refined);\n        }\n        if (rrule$1) {\n            return { rrule: rrule$1, allDayGuess: allDayGuess };\n        }\n        return null;\n    }\n    function convertConstants(input) {\n        if (Array.isArray(input)) {\n            return input.map(convertConstant);\n        }\n        return convertConstant(input);\n    }\n    function convertConstant(input) {\n        if (typeof input === 'string') {\n            return rrule.RRule[input.toUpperCase()];\n        }\n        return input;\n    }\n\n    exports.default = main;\n\n    Object.defineProperty(exports, '__esModule', { value: true });\n\n}));\n"

/***/ },

/***/ 1091
(module) {

module.exports = "/*!\nFullCalendar Time Grid Plugin v4.4.2\nDocs & License: https://fullcalendar.io/\n(c) 2019 Adam Shaw\n*/\n\n(function (global, factory) {\n    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@fullcalendar/core'), require('@fullcalendar/daygrid')) :\n    typeof define === 'function' && define.amd ? define(['exports', '@fullcalendar/core', '@fullcalendar/daygrid'], factory) :\n    (global = global || self, factory(global.FullCalendarTimeGrid = {}, global.FullCalendar, global.FullCalendarDayGrid));\n}(this, function (exports, core, daygrid) { 'use strict';\n\n    /*! *****************************************************************************\r\n    Copyright (c) Microsoft Corporation.\r\n\r\n    Permission to use, copy, modify, and/or distribute this software for any\r\n    purpose with or without fee is hereby granted.\r\n\r\n    THE SOFTWARE IS PROVIDED \"AS IS\" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH\r\n    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY\r\n    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,\r\n    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM\r\n    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR\r\n    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR\r\n    PERFORMANCE OF THIS SOFTWARE.\r\n    ***************************************************************************** */\r\n    /* global Reflect, Promise */\r\n\r\n    var extendStatics = function(d, b) {\r\n        extendStatics = Object.setPrototypeOf ||\r\n            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||\r\n            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };\r\n        return extendStatics(d, b);\r\n    };\r\n\r\n    function __extends(d, b) {\r\n        extendStatics(d, b);\r\n        function __() { this.constructor = d; }\r\n        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());\r\n    }\r\n\r\n    var __assign = function() {\r\n        __assign = Object.assign || function __assign(t) {\r\n            for (var s, i = 1, n = arguments.length; i < n; i++) {\r\n                s = arguments[i];\r\n                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];\r\n            }\r\n            return t;\r\n        };\r\n        return __assign.apply(this, arguments);\r\n    };\n\n    /*\n    Only handles foreground segs.\n    Does not own rendering. Use for low-level util methods by TimeGrid.\n    */\n    var TimeGridEventRenderer = /** @class */ (function (_super) {\n        __extends(TimeGridEventRenderer, _super);\n        function TimeGridEventRenderer(timeGrid) {\n            var _this = _super.call(this) || this;\n            _this.timeGrid = timeGrid;\n            return _this;\n        }\n        TimeGridEventRenderer.prototype.renderSegs = function (context, segs, mirrorInfo) {\n            _super.prototype.renderSegs.call(this, context, segs, mirrorInfo);\n            // TODO: dont do every time. memoize\n            this.fullTimeFormat = core.createFormatter({\n                hour: 'numeric',\n                minute: '2-digit',\n                separator: this.context.options.defaultRangeSeparator\n            });\n        };\n        // Given an array of foreground segments, render a DOM element for each, computes position,\n        // and attaches to the column inner-container elements.\n        TimeGridEventRenderer.prototype.attachSegs = function (segs, mirrorInfo) {\n            var segsByCol = this.timeGrid.groupSegsByCol(segs);\n            // order the segs within each column\n            // TODO: have groupSegsByCol do this?\n            for (var col = 0; col < segsByCol.length; col++) {\n                segsByCol[col] = this.sortEventSegs(segsByCol[col]);\n            }\n            this.segsByCol = segsByCol;\n            this.timeGrid.attachSegsByCol(segsByCol, this.timeGrid.fgContainerEls);\n        };\n        TimeGridEventRenderer.prototype.detachSegs = function (segs) {\n            segs.forEach(function (seg) {\n                core.removeElement(seg.el);\n            });\n            this.segsByCol = null;\n        };\n        TimeGridEventRenderer.prototype.computeSegSizes = function (allSegs) {\n            var _a = this, timeGrid = _a.timeGrid, segsByCol = _a.segsByCol;\n            var colCnt = timeGrid.colCnt;\n            timeGrid.computeSegVerticals(allSegs); // horizontals relies on this\n            if (segsByCol) {\n                for (var col = 0; col < colCnt; col++) {\n                    this.computeSegHorizontals(segsByCol[col]); // compute horizontal coordinates, z-index's, and reorder the array\n                }\n            }\n        };\n        TimeGridEventRenderer.prototype.assignSegSizes = function (allSegs) {\n            var _a = this, timeGrid = _a.timeGrid, segsByCol = _a.segsByCol;\n            var colCnt = timeGrid.colCnt;\n            timeGrid.assignSegVerticals(allSegs); // horizontals relies on this\n            if (segsByCol) {\n                for (var col = 0; col < colCnt; col++) {\n                    this.assignSegCss(segsByCol[col]);\n                }\n            }\n        };\n        // Computes a default event time formatting string if `eventTimeFormat` is not explicitly defined\n        TimeGridEventRenderer.prototype.computeEventTimeFormat = function () {\n            return {\n                hour: 'numeric',\n                minute: '2-digit',\n                meridiem: false\n            };\n        };\n        // Computes a default `displayEventEnd` value if one is not expliclty defined\n        TimeGridEventRenderer.prototype.computeDisplayEventEnd = function () {\n            return true;\n        };\n        // Renders the HTML for a single event segment's default rendering\n        TimeGridEventRenderer.prototype.renderSegHtml = function (seg, mirrorInfo) {\n            var eventRange = seg.eventRange;\n            var eventDef = eventRange.def;\n            var eventUi = eventRange.ui;\n            var allDay = eventDef.allDay;\n            var isDraggable = core.computeEventDraggable(this.context, eventDef, eventUi);\n            var isResizableFromStart = seg.isStart && core.computeEventStartResizable(this.context, eventDef, eventUi);\n            var isResizableFromEnd = seg.isEnd && core.computeEventEndResizable(this.context, eventDef, eventUi);\n            var classes = this.getSegClasses(seg, isDraggable, isResizableFromStart || isResizableFromEnd, mirrorInfo);\n            var skinCss = core.cssToStr(this.getSkinCss(eventUi));\n            var timeText;\n            var fullTimeText; // more verbose time text. for the print stylesheet\n            var startTimeText; // just the start time text\n            classes.unshift('fc-time-grid-event');\n            // if the event appears to span more than one day...\n            if (core.isMultiDayRange(eventRange.range)) {\n                // Don't display time text on segments that run entirely through a day.\n                // That would appear as midnight-midnight and would look dumb.\n                // Otherwise, display the time text for the *segment's* times (like 6pm-midnight or midnight-10am)\n                if (seg.isStart || seg.isEnd) {\n                    var unzonedStart = seg.start;\n                    var unzonedEnd = seg.end;\n                    timeText = this._getTimeText(unzonedStart, unzonedEnd, allDay); // TODO: give the timezones\n                    fullTimeText = this._getTimeText(unzonedStart, unzonedEnd, allDay, this.fullTimeFormat);\n                    startTimeText = this._getTimeText(unzonedStart, unzonedEnd, allDay, null, false); // displayEnd=false\n                }\n            }\n            else {\n                // Display the normal time text for the *event's* times\n                timeText = this.getTimeText(eventRange);\n                fullTimeText = this.getTimeText(eventRange, this.fullTimeFormat);\n                startTimeText = this.getTimeText(eventRange, null, false); // displayEnd=false\n            }\n            return '<a class=\"' + classes.join(' ') + '\"' +\n                (eventDef.url ?\n                    ' href=\"' + core.htmlEscape(eventDef.url) + '\"' :\n                    '') +\n                (skinCss ?\n                    ' style=\"' + skinCss + '\"' :\n                    '') +\n                '>' +\n                '<div class=\"fc-content\">' +\n                (timeText ?\n                    '<div class=\"fc-time\"' +\n                        ' data-start=\"' + core.htmlEscape(startTimeText) + '\"' +\n                        ' data-full=\"' + core.htmlEscape(fullTimeText) + '\"' +\n                        '>' +\n                        '<span>' + core.htmlEscape(timeText) + '</span>' +\n                        '</div>' :\n                    '') +\n                (eventDef.title ?\n                    '<div class=\"fc-title\">' +\n                        core.htmlEscape(eventDef.title) +\n                        '</div>' :\n                    '') +\n                '</div>' +\n                /* TODO: write CSS for this\n                (isResizableFromStart ?\n                  '<div class=\"fc-resizer fc-start-resizer\"></div>' :\n                  ''\n                  ) +\n                */\n                (isResizableFromEnd ?\n                    '<div class=\"fc-resizer fc-end-resizer\"></div>' :\n                    '') +\n                '</a>';\n        };\n        // Given an array of segments that are all in the same column, sets the backwardCoord and forwardCoord on each.\n        // Assumed the segs are already ordered.\n        // NOTE: Also reorders the given array by date!\n        TimeGridEventRenderer.prototype.computeSegHorizontals = function (segs) {\n            var levels;\n            var level0;\n            var i;\n            levels = buildSlotSegLevels(segs);\n            computeForwardSlotSegs(levels);\n            if ((level0 = levels[0])) {\n                for (i = 0; i < level0.length; i++) {\n                    computeSlotSegPressures(level0[i]);\n                }\n                for (i = 0; i < level0.length; i++) {\n                    this.computeSegForwardBack(level0[i], 0, 0);\n                }\n            }\n        };\n        // Calculate seg.forwardCoord and seg.backwardCoord for the segment, where both values range\n        // from 0 to 1. If the calendar is left-to-right, the seg.backwardCoord maps to \"left\" and\n        // seg.forwardCoord maps to \"right\" (via percentage). Vice-versa if the calendar is right-to-left.\n        //\n        // The segment might be part of a \"series\", which means consecutive segments with the same pressure\n        // who's width is unknown until an edge has been hit. `seriesBackwardPressure` is the number of\n        // segments behind this one in the current series, and `seriesBackwardCoord` is the starting\n        // coordinate of the first segment in the series.\n        TimeGridEventRenderer.prototype.computeSegForwardBack = function (seg, seriesBackwardPressure, seriesBackwardCoord) {\n            var forwardSegs = seg.forwardSegs;\n            var i;\n            if (seg.forwardCoord === undefined) { // not already computed\n                if (!forwardSegs.length) {\n                    // if there are no forward segments, this segment should butt up against the edge\n                    seg.forwardCoord = 1;\n                }\n                else {\n                    // sort highest pressure first\n                    this.sortForwardSegs(forwardSegs);\n                    // this segment's forwardCoord will be calculated from the backwardCoord of the\n                    // highest-pressure forward segment.\n                    this.computeSegForwardBack(forwardSegs[0], seriesBackwardPressure + 1, seriesBackwardCoord);\n                    seg.forwardCoord = forwardSegs[0].backwardCoord;\n                }\n                // calculate the backwardCoord from the forwardCoord. consider the series\n                seg.backwardCoord = seg.forwardCoord -\n                    (seg.forwardCoord - seriesBackwardCoord) / // available width for series\n                        (seriesBackwardPressure + 1); // # of segments in the series\n                // use this segment's coordinates to computed the coordinates of the less-pressurized\n                // forward segments\n                for (i = 0; i < forwardSegs.length; i++) {\n                    this.computeSegForwardBack(forwardSegs[i], 0, seg.forwardCoord);\n                }\n            }\n        };\n        TimeGridEventRenderer.prototype.sortForwardSegs = function (forwardSegs) {\n            var objs = forwardSegs.map(buildTimeGridSegCompareObj);\n            var specs = [\n                // put higher-pressure first\n                { field: 'forwardPressure', order: -1 },\n                // put segments that are closer to initial edge first (and favor ones with no coords yet)\n                { field: 'backwardCoord', order: 1 }\n            ].concat(this.context.eventOrderSpecs);\n            objs.sort(function (obj0, obj1) {\n                return core.compareByFieldSpecs(obj0, obj1, specs);\n            });\n            return objs.map(function (c) {\n                return c._seg;\n            });\n        };\n        // Given foreground event segments that have already had their position coordinates computed,\n        // assigns position-related CSS values to their elements.\n        TimeGridEventRenderer.prototype.assignSegCss = function (segs) {\n            for (var _i = 0, segs_1 = segs; _i < segs_1.length; _i++) {\n                var seg = segs_1[_i];\n                core.applyStyle(seg.el, this.generateSegCss(seg));\n                if (seg.level > 0) {\n                    seg.el.classList.add('fc-time-grid-event-inset');\n                }\n                // if the event is short that the title will be cut off,\n                // attach a className that condenses the title into the time area.\n                if (seg.eventRange.def.title && seg.bottom - seg.top < 30) {\n                    seg.el.classList.add('fc-short'); // TODO: \"condensed\" is a better name\n                }\n            }\n        };\n        // Generates an object with CSS properties/values that should be applied to an event segment element.\n        // Contains important positioning-related properties that should be applied to any event element, customized or not.\n        TimeGridEventRenderer.prototype.generateSegCss = function (seg) {\n            var shouldOverlap = this.context.options.slotEventOverlap;\n            var backwardCoord = seg.backwardCoord; // the left side if LTR. the right side if RTL. floating-point\n            var forwardCoord = seg.forwardCoord; // the right side if LTR. the left side if RTL. floating-point\n            var props = this.timeGrid.generateSegVerticalCss(seg); // get top/bottom first\n            var isRtl = this.context.isRtl;\n            var left; // amount of space from left edge, a fraction of the total width\n            var right; // amount of space from right edge, a fraction of the total width\n            if (shouldOverlap) {\n                // double the width, but don't go beyond the maximum forward coordinate (1.0)\n                forwardCoord = Math.min(1, backwardCoord + (forwardCoord - backwardCoord) * 2);\n            }\n            if (isRtl) {\n                left = 1 - forwardCoord;\n                right = backwardCoord;\n            }\n            else {\n                left = backwardCoord;\n                right = 1 - forwardCoord;\n            }\n            props.zIndex = seg.level + 1; // convert from 0-base to 1-based\n            props.left = left * 100 + '%';\n            props.right = right * 100 + '%';\n            if (shouldOverlap && seg.forwardPressure) {\n                // add padding to the edge so that forward stacked events don't cover the resizer's icon\n                props[isRtl ? 'marginLeft' : 'marginRight'] = 10 * 2; // 10 is a guesstimate of the icon's width\n            }\n            return props;\n        };\n        return TimeGridEventRenderer;\n    }(core.FgEventRenderer));\n    // Builds an array of segments \"levels\". The first level will be the leftmost tier of segments if the calendar is\n    // left-to-right, or the rightmost if the calendar is right-to-left. Assumes the segments are already ordered by date.\n    function buildSlotSegLevels(segs) {\n        var levels = [];\n        var i;\n        var seg;\n        var j;\n        for (i = 0; i < segs.length; i++) {\n            seg = segs[i];\n            // go through all the levels and stop on the first level where there are no collisions\n            for (j = 0; j < levels.length; j++) {\n                if (!computeSlotSegCollisions(seg, levels[j]).length) {\n                    break;\n                }\n            }\n            seg.level = j;\n            (levels[j] || (levels[j] = [])).push(seg);\n        }\n        return levels;\n    }\n    // For every segment, figure out the other segments that are in subsequent\n    // levels that also occupy the same vertical space. Accumulate in seg.forwardSegs\n    function computeForwardSlotSegs(levels) {\n        var i;\n        var level;\n        var j;\n        var seg;\n        var k;\n        for (i = 0; i < levels.length; i++) {\n            level = levels[i];\n            for (j = 0; j < level.length; j++) {\n                seg = level[j];\n                seg.forwardSegs = [];\n                for (k = i + 1; k < levels.length; k++) {\n                    computeSlotSegCollisions(seg, levels[k], seg.forwardSegs);\n                }\n            }\n        }\n    }\n    // Figure out which path forward (via seg.forwardSegs) results in the longest path until\n    // the furthest edge is reached. The number of segments in this path will be seg.forwardPressure\n    function computeSlotSegPressures(seg) {\n        var forwardSegs = seg.forwardSegs;\n        var forwardPressure = 0;\n        var i;\n        var forwardSeg;\n        if (seg.forwardPressure === undefined) { // not already computed\n            for (i = 0; i < forwardSegs.length; i++) {\n                forwardSeg = forwardSegs[i];\n                // figure out the child's maximum forward path\n                computeSlotSegPressures(forwardSeg);\n                // either use the existing maximum, or use the child's forward pressure\n                // plus one (for the forwardSeg itself)\n                forwardPressure = Math.max(forwardPressure, 1 + forwardSeg.forwardPressure);\n            }\n            seg.forwardPressure = forwardPressure;\n        }\n    }\n    // Find all the segments in `otherSegs` that vertically collide with `seg`.\n    // Append into an optionally-supplied `results` array and return.\n    function computeSlotSegCollisions(seg, otherSegs, results) {\n        if (results === void 0) { results = []; }\n        for (var i = 0; i < otherSegs.length; i++) {\n            if (isSlotSegCollision(seg, otherSegs[i])) {\n                results.push(otherSegs[i]);\n            }\n        }\n        return results;\n    }\n    // Do these segments occupy the same vertical space?\n    function isSlotSegCollision(seg1, seg2) {\n        return seg1.bottom > seg2.top && seg1.top < seg2.bottom;\n    }\n    function buildTimeGridSegCompareObj(seg) {\n        var obj = core.buildSegCompareObj(seg);\n        obj.forwardPressure = seg.forwardPressure;\n        obj.backwardCoord = seg.backwardCoord;\n        return obj;\n    }\n\n    var TimeGridMirrorRenderer = /** @class */ (function (_super) {\n        __extends(TimeGridMirrorRenderer, _super);\n        function TimeGridMirrorRenderer() {\n            return _super !== null && _super.apply(this, arguments) || this;\n        }\n        TimeGridMirrorRenderer.prototype.attachSegs = function (segs, mirrorInfo) {\n            this.segsByCol = this.timeGrid.groupSegsByCol(segs);\n            this.timeGrid.attachSegsByCol(this.segsByCol, this.timeGrid.mirrorContainerEls);\n            this.sourceSeg = mirrorInfo.sourceSeg;\n        };\n        TimeGridMirrorRenderer.prototype.generateSegCss = function (seg) {\n            var props = _super.prototype.generateSegCss.call(this, seg);\n            var sourceSeg = this.sourceSeg;\n            if (sourceSeg && sourceSeg.col === seg.col) {\n                var sourceSegProps = _super.prototype.generateSegCss.call(this, sourceSeg);\n                props.left = sourceSegProps.left;\n                props.right = sourceSegProps.right;\n                props.marginLeft = sourceSegProps.marginLeft;\n                props.marginRight = sourceSegProps.marginRight;\n            }\n            return props;\n        };\n        return TimeGridMirrorRenderer;\n    }(TimeGridEventRenderer));\n\n    var TimeGridFillRenderer = /** @class */ (function (_super) {\n        __extends(TimeGridFillRenderer, _super);\n        function TimeGridFillRenderer(timeGrid) {\n            var _this = _super.call(this) || this;\n            _this.timeGrid = timeGrid;\n            return _this;\n        }\n        TimeGridFillRenderer.prototype.attachSegs = function (type, segs) {\n            var timeGrid = this.timeGrid;\n            var containerEls;\n            // TODO: more efficient lookup\n            if (type === 'bgEvent') {\n                containerEls = timeGrid.bgContainerEls;\n            }\n            else if (type === 'businessHours') {\n                containerEls = timeGrid.businessContainerEls;\n            }\n            else if (type === 'highlight') {\n                containerEls = timeGrid.highlightContainerEls;\n            }\n            timeGrid.attachSegsByCol(timeGrid.groupSegsByCol(segs), containerEls);\n            return segs.map(function (seg) {\n                return seg.el;\n            });\n        };\n        TimeGridFillRenderer.prototype.computeSegSizes = function (segs) {\n            this.timeGrid.computeSegVerticals(segs);\n        };\n        TimeGridFillRenderer.prototype.assignSegSizes = function (segs) {\n            this.timeGrid.assignSegVerticals(segs);\n        };\n        return TimeGridFillRenderer;\n    }(core.FillRenderer));\n\n    /* A component that renders one or more columns of vertical time slots\n    ----------------------------------------------------------------------------------------------------------------------*/\n    // potential nice values for the slot-duration and interval-duration\n    // from largest to smallest\n    var AGENDA_STOCK_SUB_DURATIONS = [\n        { hours: 1 },\n        { minutes: 30 },\n        { minutes: 15 },\n        { seconds: 30 },\n        { seconds: 15 }\n    ];\n    var TimeGrid = /** @class */ (function (_super) {\n        __extends(TimeGrid, _super);\n        function TimeGrid(el, renderProps) {\n            var _this = _super.call(this, el) || this;\n            _this.isSlatSizesDirty = false;\n            _this.isColSizesDirty = false;\n            _this.processOptions = core.memoize(_this._processOptions);\n            _this.renderSkeleton = core.memoizeRendering(_this._renderSkeleton);\n            _this.renderSlats = core.memoizeRendering(_this._renderSlats, null, [_this.renderSkeleton]);\n            _this.renderColumns = core.memoizeRendering(_this._renderColumns, _this._unrenderColumns, [_this.renderSkeleton]);\n            _this.renderProps = renderProps;\n            var renderColumns = _this.renderColumns;\n            var eventRenderer = _this.eventRenderer = new TimeGridEventRenderer(_this);\n            var fillRenderer = _this.fillRenderer = new TimeGridFillRenderer(_this);\n            _this.mirrorRenderer = new TimeGridMirrorRenderer(_this);\n            _this.renderBusinessHours = core.memoizeRendering(fillRenderer.renderSegs.bind(fillRenderer, 'businessHours'), fillRenderer.unrender.bind(fillRenderer, 'businessHours'), [renderColumns]);\n            _this.renderDateSelection = core.memoizeRendering(_this._renderDateSelection, _this._unrenderDateSelection, [renderColumns]);\n            _this.renderFgEvents = core.memoizeRendering(eventRenderer.renderSegs.bind(eventRenderer), eventRenderer.unrender.bind(eventRenderer), [renderColumns]);\n            _this.renderBgEvents = core.memoizeRendering(fillRenderer.renderSegs.bind(fillRenderer, 'bgEvent'), fillRenderer.unrender.bind(fillRenderer, 'bgEvent'), [renderColumns]);\n            _this.renderEventSelection = core.memoizeRendering(eventRenderer.selectByInstanceId.bind(eventRenderer), eventRenderer.unselectByInstanceId.bind(eventRenderer), [_this.renderFgEvents]);\n            _this.renderEventDrag = core.memoizeRendering(_this._renderEventDrag, _this._unrenderEventDrag, [renderColumns]);\n            _this.renderEventResize = core.memoizeRendering(_this._renderEventResize, _this._unrenderEventResize, [renderColumns]);\n            return _this;\n        }\n        /* Options\n        ------------------------------------------------------------------------------------------------------------------*/\n        // Parses various options into properties of this object\n        // MUST have context already set\n        TimeGrid.prototype._processOptions = function (options) {\n            var slotDuration = options.slotDuration, snapDuration = options.snapDuration;\n            var snapsPerSlot;\n            var input;\n            slotDuration = core.createDuration(slotDuration);\n            snapDuration = snapDuration ? core.createDuration(snapDuration) : slotDuration;\n            snapsPerSlot = core.wholeDivideDurations(slotDuration, snapDuration);\n            if (snapsPerSlot === null) {\n                snapDuration = slotDuration;\n                snapsPerSlot = 1;\n                // TODO: say warning?\n            }\n            this.slotDuration = slotDuration;\n            this.snapDuration = snapDuration;\n            this.snapsPerSlot = snapsPerSlot;\n            // might be an array value (for TimelineView).\n            // if so, getting the most granular entry (the last one probably).\n            input = options.slotLabelFormat;\n            if (Array.isArray(input)) {\n                input = input[input.length - 1];\n            }\n            this.labelFormat = core.createFormatter(input || {\n                hour: 'numeric',\n                minute: '2-digit',\n                omitZeroMinute: true,\n                meridiem: 'short'\n            });\n            input = options.slotLabelInterval;\n            this.labelInterval = input ?\n                core.createDuration(input) :\n                this.computeLabelInterval(slotDuration);\n        };\n        // Computes an automatic value for slotLabelInterval\n        TimeGrid.prototype.computeLabelInterval = function (slotDuration) {\n            var i;\n            var labelInterval;\n            var slotsPerLabel;\n            // find the smallest stock label interval that results in more than one slots-per-label\n            for (i = AGENDA_STOCK_SUB_DURATIONS.length - 1; i >= 0; i--) {\n                labelInterval = core.createDuration(AGENDA_STOCK_SUB_DURATIONS[i]);\n                slotsPerLabel = core.wholeDivideDurations(labelInterval, slotDuration);\n                if (slotsPerLabel !== null && slotsPerLabel > 1) {\n                    return labelInterval;\n                }\n            }\n            return slotDuration; // fall back\n        };\n        /* Rendering\n        ------------------------------------------------------------------------------------------------------------------*/\n        TimeGrid.prototype.render = function (props, context) {\n            this.processOptions(context.options);\n            var cells = props.cells;\n            this.colCnt = cells.length;\n            this.renderSkeleton(context.theme);\n            this.renderSlats(props.dateProfile);\n            this.renderColumns(props.cells, props.dateProfile);\n            this.renderBusinessHours(context, props.businessHourSegs);\n            this.renderDateSelection(props.dateSelectionSegs);\n            this.renderFgEvents(context, props.fgEventSegs);\n            this.renderBgEvents(context, props.bgEventSegs);\n            this.renderEventSelection(props.eventSelection);\n            this.renderEventDrag(props.eventDrag);\n            this.renderEventResize(props.eventResize);\n        };\n        TimeGrid.prototype.destroy = function () {\n            _super.prototype.destroy.call(this);\n            // should unrender everything else too\n            this.renderSlats.unrender();\n            this.renderColumns.unrender();\n            this.renderSkeleton.unrender();\n        };\n        TimeGrid.prototype.updateSize = function (isResize) {\n            var _a = this, fillRenderer = _a.fillRenderer, eventRenderer = _a.eventRenderer, mirrorRenderer = _a.mirrorRenderer;\n            if (isResize || this.isSlatSizesDirty) {\n                this.buildSlatPositions();\n                this.isSlatSizesDirty = false;\n            }\n            if (isResize || this.isColSizesDirty) {\n                this.buildColPositions();\n                this.isColSizesDirty = false;\n            }\n            fillRenderer.computeSizes(isResize);\n            eventRenderer.computeSizes(isResize);\n            mirrorRenderer.computeSizes(isResize);\n            fillRenderer.assignSizes(isResize);\n            eventRenderer.assignSizes(isResize);\n            mirrorRenderer.assignSizes(isResize);\n        };\n        TimeGrid.prototype._renderSkeleton = function (theme) {\n            var el = this.el;\n            el.innerHTML =\n                '<div class=\"fc-bg\"></div>' +\n                    '<div class=\"fc-slats\"></div>' +\n                    '<hr class=\"fc-divider ' + theme.getClass('widgetHeader') + '\" style=\"display:none\" />';\n            this.rootBgContainerEl = el.querySelector('.fc-bg');\n            this.slatContainerEl = el.querySelector('.fc-slats');\n            this.bottomRuleEl = el.querySelector('.fc-divider');\n        };\n        TimeGrid.prototype._renderSlats = function (dateProfile) {\n            var theme = this.context.theme;\n            this.slatContainerEl.innerHTML =\n                '<table class=\"' + theme.getClass('tableGrid') + '\">' +\n                    this.renderSlatRowHtml(dateProfile) +\n                    '</table>';\n            this.slatEls = core.findElements(this.slatContainerEl, 'tr');\n            this.slatPositions = new core.PositionCache(this.el, this.slatEls, false, true // vertical\n            );\n            this.isSlatSizesDirty = true;\n        };\n        // Generates the HTML for the horizontal \"slats\" that run width-wise. Has a time axis on a side. Depends on RTL.\n        TimeGrid.prototype.renderSlatRowHtml = function (dateProfile) {\n            var _a = this.context, dateEnv = _a.dateEnv, theme = _a.theme, isRtl = _a.isRtl;\n            var html = '';\n            var dayStart = core.startOfDay(dateProfile.renderRange.start);\n            var slotTime = dateProfile.minTime;\n            var slotIterator = core.createDuration(0);\n            var slotDate; // will be on the view's first day, but we only care about its time\n            var isLabeled;\n            var axisHtml;\n            // Calculate the time for each slot\n            while (core.asRoughMs(slotTime) < core.asRoughMs(dateProfile.maxTime)) {\n                slotDate = dateEnv.add(dayStart, slotTime);\n                isLabeled = core.wholeDivideDurations(slotIterator, this.labelInterval) !== null;\n                axisHtml =\n                    '<td class=\"fc-axis fc-time ' + theme.getClass('widgetContent') + '\">' +\n                        (isLabeled ?\n                            '<span>' + // for matchCellWidths\n                                core.htmlEscape(dateEnv.format(slotDate, this.labelFormat)) +\n                                '</span>' :\n                            '') +\n                        '</td>';\n                html +=\n                    '<tr data-time=\"' + core.formatIsoTimeString(slotDate) + '\"' +\n                        (isLabeled ? '' : ' class=\"fc-minor\"') +\n                        '>' +\n                        (!isRtl ? axisHtml : '') +\n                        '<td class=\"' + theme.getClass('widgetContent') + '\"></td>' +\n                        (isRtl ? axisHtml : '') +\n                        '</tr>';\n                slotTime = core.addDurations(slotTime, this.slotDuration);\n                slotIterator = core.addDurations(slotIterator, this.slotDuration);\n            }\n            return html;\n        };\n        TimeGrid.prototype._renderColumns = function (cells, dateProfile) {\n            var _a = this.context, calendar = _a.calendar, view = _a.view, isRtl = _a.isRtl, theme = _a.theme, dateEnv = _a.dateEnv;\n            var bgRow = new daygrid.DayBgRow(this.context);\n            this.rootBgContainerEl.innerHTML =\n                '<table class=\"' + theme.getClass('tableGrid') + '\">' +\n                    bgRow.renderHtml({\n                        cells: cells,\n                        dateProfile: dateProfile,\n                        renderIntroHtml: this.renderProps.renderBgIntroHtml\n                    }) +\n                    '</table>';\n            this.colEls = core.findElements(this.el, '.fc-day, .fc-disabled-day');\n            for (var col = 0; col < this.colCnt; col++) {\n                calendar.publiclyTrigger('dayRender', [\n                    {\n                        date: dateEnv.toDate(cells[col].date),\n                        el: this.colEls[col],\n                        view: view\n                    }\n                ]);\n            }\n            if (isRtl) {\n                this.colEls.reverse();\n            }\n            this.colPositions = new core.PositionCache(this.el, this.colEls, true, // horizontal\n            false);\n            this.renderContentSkeleton();\n            this.isColSizesDirty = true;\n        };\n        TimeGrid.prototype._unrenderColumns = function () {\n            this.unrenderContentSkeleton();\n        };\n        /* Content Skeleton\n        ------------------------------------------------------------------------------------------------------------------*/\n        // Renders the DOM that the view's content will live in\n        TimeGrid.prototype.renderContentSkeleton = function () {\n            var isRtl = this.context.isRtl;\n            var parts = [];\n            var skeletonEl;\n            parts.push(this.renderProps.renderIntroHtml());\n            for (var i = 0; i < this.colCnt; i++) {\n                parts.push('<td>' +\n                    '<div class=\"fc-content-col\">' +\n                    '<div class=\"fc-event-container fc-mirror-container\"></div>' +\n                    '<div class=\"fc-event-container\"></div>' +\n                    '<div class=\"fc-highlight-container\"></div>' +\n                    '<div class=\"fc-bgevent-container\"></div>' +\n                    '<div class=\"fc-business-container\"></div>' +\n                    '</div>' +\n                    '</td>');\n            }\n            if (isRtl) {\n                parts.reverse();\n            }\n            skeletonEl = this.contentSkeletonEl = core.htmlToElement('<div class=\"fc-content-skeleton\">' +\n                '<table>' +\n                '<tr>' + parts.join('') + '</tr>' +\n                '</table>' +\n                '</div>');\n            this.colContainerEls = core.findElements(skeletonEl, '.fc-content-col');\n            this.mirrorContainerEls = core.findElements(skeletonEl, '.fc-mirror-container');\n            this.fgContainerEls = core.findElements(skeletonEl, '.fc-event-container:not(.fc-mirror-container)');\n            this.bgContainerEls = core.findElements(skeletonEl, '.fc-bgevent-container');\n            this.highlightContainerEls = core.findElements(skeletonEl, '.fc-highlight-container');\n            this.businessContainerEls = core.findElements(skeletonEl, '.fc-business-container');\n            if (isRtl) {\n                this.colContainerEls.reverse();\n                this.mirrorContainerEls.reverse();\n                this.fgContainerEls.reverse();\n                this.bgContainerEls.reverse();\n                this.highlightContainerEls.reverse();\n                this.businessContainerEls.reverse();\n            }\n            this.el.appendChild(skeletonEl);\n        };\n        TimeGrid.prototype.unrenderContentSkeleton = function () {\n            core.removeElement(this.contentSkeletonEl);\n        };\n        // Given a flat array of segments, return an array of sub-arrays, grouped by each segment's col\n        TimeGrid.prototype.groupSegsByCol = function (segs) {\n            var segsByCol = [];\n            var i;\n            for (i = 0; i < this.colCnt; i++) {\n                segsByCol.push([]);\n            }\n            for (i = 0; i < segs.length; i++) {\n                segsByCol[segs[i].col].push(segs[i]);\n            }\n            return segsByCol;\n        };\n        // Given segments grouped by column, insert the segments' elements into a parallel array of container\n        // elements, each living within a column.\n        TimeGrid.prototype.attachSegsByCol = function (segsByCol, containerEls) {\n            var col;\n            var segs;\n            var i;\n            for (col = 0; col < this.colCnt; col++) { // iterate each column grouping\n                segs = segsByCol[col];\n                for (i = 0; i < segs.length; i++) {\n                    containerEls[col].appendChild(segs[i].el);\n                }\n            }\n        };\n        /* Now Indicator\n        ------------------------------------------------------------------------------------------------------------------*/\n        TimeGrid.prototype.getNowIndicatorUnit = function () {\n            return 'minute'; // will refresh on the minute\n        };\n        TimeGrid.prototype.renderNowIndicator = function (segs, date) {\n            // HACK: if date columns not ready for some reason (scheduler)\n            if (!this.colContainerEls) {\n                return;\n            }\n            var top = this.computeDateTop(date);\n            var nodes = [];\n            var i;\n            // render lines within the columns\n            for (i = 0; i < segs.length; i++) {\n                var lineEl = core.createElement('div', { className: 'fc-now-indicator fc-now-indicator-line' });\n                lineEl.style.top = top + 'px';\n                this.colContainerEls[segs[i].col].appendChild(lineEl);\n                nodes.push(lineEl);\n            }\n            // render an arrow over the axis\n            if (segs.length > 0) { // is the current time in view?\n                var arrowEl = core.createElement('div', { className: 'fc-now-indicator fc-now-indicator-arrow' });\n                arrowEl.style.top = top + 'px';\n                this.contentSkeletonEl.appendChild(arrowEl);\n                nodes.push(arrowEl);\n            }\n            this.nowIndicatorEls = nodes;\n        };\n        TimeGrid.prototype.unrenderNowIndicator = function () {\n            if (this.nowIndicatorEls) {\n                this.nowIndicatorEls.forEach(core.removeElement);\n                this.nowIndicatorEls = null;\n            }\n        };\n        /* Coordinates\n        ------------------------------------------------------------------------------------------------------------------*/\n        TimeGrid.prototype.getTotalSlatHeight = function () {\n            return this.slatContainerEl.getBoundingClientRect().height;\n        };\n        // Computes the top coordinate, relative to the bounds of the grid, of the given date.\n        // A `startOfDayDate` must be given for avoiding ambiguity over how to treat midnight.\n        TimeGrid.prototype.computeDateTop = function (when, startOfDayDate) {\n            if (!startOfDayDate) {\n                startOfDayDate = core.startOfDay(when);\n            }\n            return this.computeTimeTop(core.createDuration(when.valueOf() - startOfDayDate.valueOf()));\n        };\n        // Computes the top coordinate, relative to the bounds of the grid, of the given time (a Duration).\n        TimeGrid.prototype.computeTimeTop = function (duration) {\n            var len = this.slatEls.length;\n            var dateProfile = this.props.dateProfile;\n            var slatCoverage = (duration.milliseconds - core.asRoughMs(dateProfile.minTime)) / core.asRoughMs(this.slotDuration); // floating-point value of # of slots covered\n            var slatIndex;\n            var slatRemainder;\n            // compute a floating-point number for how many slats should be progressed through.\n            // from 0 to number of slats (inclusive)\n            // constrained because minTime/maxTime might be customized.\n            slatCoverage = Math.max(0, slatCoverage);\n            slatCoverage = Math.min(len, slatCoverage);\n            // an integer index of the furthest whole slat\n            // from 0 to number slats (*exclusive*, so len-1)\n            slatIndex = Math.floor(slatCoverage);\n            slatIndex = Math.min(slatIndex, len - 1);\n            // how much further through the slatIndex slat (from 0.0-1.0) must be covered in addition.\n            // could be 1.0 if slatCoverage is covering *all* the slots\n            slatRemainder = slatCoverage - slatIndex;\n            return this.slatPositions.tops[slatIndex] +\n                this.slatPositions.getHeight(slatIndex) * slatRemainder;\n        };\n        // For each segment in an array, computes and assigns its top and bottom properties\n        TimeGrid.prototype.computeSegVerticals = function (segs) {\n            var options = this.context.options;\n            var eventMinHeight = options.timeGridEventMinHeight;\n            var i;\n            var seg;\n            var dayDate;\n            for (i = 0; i < segs.length; i++) {\n                seg = segs[i];\n                dayDate = this.props.cells[seg.col].date;\n                seg.top = this.computeDateTop(seg.start, dayDate);\n                seg.bottom = Math.max(seg.top + eventMinHeight, this.computeDateTop(seg.end, dayDate));\n            }\n        };\n        // Given segments that already have their top/bottom properties computed, applies those values to\n        // the segments' elements.\n        TimeGrid.prototype.assignSegVerticals = function (segs) {\n            var i;\n            var seg;\n            for (i = 0; i < segs.length; i++) {\n                seg = segs[i];\n                core.applyStyle(seg.el, this.generateSegVerticalCss(seg));\n            }\n        };\n        // Generates an object with CSS properties for the top/bottom coordinates of a segment element\n        TimeGrid.prototype.generateSegVerticalCss = function (seg) {\n            return {\n                top: seg.top,\n                bottom: -seg.bottom // flipped because needs to be space beyond bottom edge of event container\n            };\n        };\n        /* Sizing\n        ------------------------------------------------------------------------------------------------------------------*/\n        TimeGrid.prototype.buildPositionCaches = function () {\n            this.buildColPositions();\n            this.buildSlatPositions();\n        };\n        TimeGrid.prototype.buildColPositions = function () {\n            this.colPositions.build();\n        };\n        TimeGrid.prototype.buildSlatPositions = function () {\n            this.slatPositions.build();\n        };\n        /* Hit System\n        ------------------------------------------------------------------------------------------------------------------*/\n        TimeGrid.prototype.positionToHit = function (positionLeft, positionTop) {\n            var dateEnv = this.context.dateEnv;\n            var _a = this, snapsPerSlot = _a.snapsPerSlot, slatPositions = _a.slatPositions, colPositions = _a.colPositions;\n            var colIndex = colPositions.leftToIndex(positionLeft);\n            var slatIndex = slatPositions.topToIndex(positionTop);\n            if (colIndex != null && slatIndex != null) {\n                var slatTop = slatPositions.tops[slatIndex];\n                var slatHeight = slatPositions.getHeight(slatIndex);\n                var partial = (positionTop - slatTop) / slatHeight; // floating point number between 0 and 1\n                var localSnapIndex = Math.floor(partial * snapsPerSlot); // the snap # relative to start of slat\n                var snapIndex = slatIndex * snapsPerSlot + localSnapIndex;\n                var dayDate = this.props.cells[colIndex].date;\n                var time = core.addDurations(this.props.dateProfile.minTime, core.multiplyDuration(this.snapDuration, snapIndex));\n                var start = dateEnv.add(dayDate, time);\n                var end = dateEnv.add(start, this.snapDuration);\n                return {\n                    col: colIndex,\n                    dateSpan: {\n                        range: { start: start, end: end },\n                        allDay: false\n                    },\n                    dayEl: this.colEls[colIndex],\n                    relativeRect: {\n                        left: colPositions.lefts[colIndex],\n                        right: colPositions.rights[colIndex],\n                        top: slatTop,\n                        bottom: slatTop + slatHeight\n                    }\n                };\n            }\n        };\n        /* Event Drag Visualization\n        ------------------------------------------------------------------------------------------------------------------*/\n        TimeGrid.prototype._renderEventDrag = function (state) {\n            if (state) {\n                this.eventRenderer.hideByHash(state.affectedInstances);\n                if (state.isEvent) {\n                    this.mirrorRenderer.renderSegs(this.context, state.segs, { isDragging: true, sourceSeg: state.sourceSeg });\n                }\n                else {\n                    this.fillRenderer.renderSegs('highlight', this.context, state.segs);\n                }\n            }\n        };\n        TimeGrid.prototype._unrenderEventDrag = function (state) {\n            if (state) {\n                this.eventRenderer.showByHash(state.affectedInstances);\n                if (state.isEvent) {\n                    this.mirrorRenderer.unrender(this.context, state.segs, { isDragging: true, sourceSeg: state.sourceSeg });\n                }\n                else {\n                    this.fillRenderer.unrender('highlight', this.context);\n                }\n            }\n        };\n        /* Event Resize Visualization\n        ------------------------------------------------------------------------------------------------------------------*/\n        TimeGrid.prototype._renderEventResize = function (state) {\n            if (state) {\n                this.eventRenderer.hideByHash(state.affectedInstances);\n                this.mirrorRenderer.renderSegs(this.context, state.segs, { isResizing: true, sourceSeg: state.sourceSeg });\n            }\n        };\n        TimeGrid.prototype._unrenderEventResize = function (state) {\n            if (state) {\n                this.eventRenderer.showByHash(state.affectedInstances);\n                this.mirrorRenderer.unrender(this.context, state.segs, { isResizing: true, sourceSeg: state.sourceSeg });\n            }\n        };\n        /* Selection\n        ------------------------------------------------------------------------------------------------------------------*/\n        // Renders a visual indication of a selection. Overrides the default, which was to simply render a highlight.\n        TimeGrid.prototype._renderDateSelection = function (segs) {\n            if (segs) {\n                if (this.context.options.selectMirror) {\n                    this.mirrorRenderer.renderSegs(this.context, segs, { isSelecting: true });\n                }\n                else {\n                    this.fillRenderer.renderSegs('highlight', this.context, segs);\n                }\n            }\n        };\n        TimeGrid.prototype._unrenderDateSelection = function (segs) {\n            if (segs) {\n                if (this.context.options.selectMirror) {\n                    this.mirrorRenderer.unrender(this.context, segs, { isSelecting: true });\n                }\n                else {\n                    this.fillRenderer.unrender('highlight', this.context);\n                }\n            }\n        };\n        return TimeGrid;\n    }(core.DateComponent));\n\n    var AllDaySplitter = /** @class */ (function (_super) {\n        __extends(AllDaySplitter, _super);\n        function AllDaySplitter() {\n            return _super !== null && _super.apply(this, arguments) || this;\n        }\n        AllDaySplitter.prototype.getKeyInfo = function () {\n            return {\n                allDay: {},\n                timed: {}\n            };\n        };\n        AllDaySplitter.prototype.getKeysForDateSpan = function (dateSpan) {\n            if (dateSpan.allDay) {\n                return ['allDay'];\n            }\n            else {\n                return ['timed'];\n            }\n        };\n        AllDaySplitter.prototype.getKeysForEventDef = function (eventDef) {\n            if (!eventDef.allDay) {\n                return ['timed'];\n            }\n            else if (core.hasBgRendering(eventDef)) {\n                return ['timed', 'allDay'];\n            }\n            else {\n                return ['allDay'];\n            }\n        };\n        return AllDaySplitter;\n    }(core.Splitter));\n\n    var TIMEGRID_ALL_DAY_EVENT_LIMIT = 5;\n    var WEEK_HEADER_FORMAT = core.createFormatter({ week: 'short' });\n    /* An abstract class for all timegrid-related views. Displays one more columns with time slots running vertically.\n    ----------------------------------------------------------------------------------------------------------------------*/\n    // Is a manager for the TimeGrid subcomponent and possibly the DayGrid subcomponent (if allDaySlot is on).\n    // Responsible for managing width/height.\n    var AbstractTimeGridView = /** @class */ (function (_super) {\n        __extends(AbstractTimeGridView, _super);\n        function AbstractTimeGridView() {\n            var _this = _super !== null && _super.apply(this, arguments) || this;\n            _this.splitter = new AllDaySplitter();\n            _this.renderSkeleton = core.memoizeRendering(_this._renderSkeleton, _this._unrenderSkeleton);\n            /* Header Render Methods\n            ------------------------------------------------------------------------------------------------------------------*/\n            // Generates the HTML that will go before the day-of week header cells\n            _this.renderHeadIntroHtml = function () {\n                var _a = _this.context, theme = _a.theme, dateEnv = _a.dateEnv, options = _a.options;\n                var range = _this.props.dateProfile.renderRange;\n                var dayCnt = core.diffDays(range.start, range.end);\n                var weekText;\n                if (options.weekNumbers) {\n                    weekText = dateEnv.format(range.start, WEEK_HEADER_FORMAT);\n                    return '' +\n                        '<th class=\"fc-axis fc-week-number ' + theme.getClass('widgetHeader') + '\" ' + _this.axisStyleAttr() + '>' +\n                        core.buildGotoAnchorHtml(// aside from link, important for matchCellWidths\n                        options, dateEnv, { date: range.start, type: 'week', forceOff: dayCnt > 1 }, core.htmlEscape(weekText) // inner HTML\n                        ) +\n                        '</th>';\n                }\n                else {\n                    return '<th class=\"fc-axis ' + theme.getClass('widgetHeader') + '\" ' + _this.axisStyleAttr() + '></th>';\n                }\n            };\n            /* Time Grid Render Methods\n            ------------------------------------------------------------------------------------------------------------------*/\n            // Generates the HTML that goes before the bg of the TimeGrid slot area. Long vertical column.\n            _this.renderTimeGridBgIntroHtml = function () {\n                var theme = _this.context.theme;\n                return '<td class=\"fc-axis ' + theme.getClass('widgetContent') + '\" ' + _this.axisStyleAttr() + '></td>';\n            };\n            // Generates the HTML that goes before all other types of cells.\n            // Affects content-skeleton, mirror-skeleton, highlight-skeleton for both the time-grid and day-grid.\n            _this.renderTimeGridIntroHtml = function () {\n                return '<td class=\"fc-axis\" ' + _this.axisStyleAttr() + '></td>';\n            };\n            /* Day Grid Render Methods\n            ------------------------------------------------------------------------------------------------------------------*/\n            // Generates the HTML that goes before the all-day cells\n            _this.renderDayGridBgIntroHtml = function () {\n                var _a = _this.context, theme = _a.theme, options = _a.options;\n                return '' +\n                    '<td class=\"fc-axis ' + theme.getClass('widgetContent') + '\" ' + _this.axisStyleAttr() + '>' +\n                    '<span>' + // needed for matchCellWidths\n                    core.getAllDayHtml(options) +\n                    '</span>' +\n                    '</td>';\n            };\n            // Generates the HTML that goes before all other types of cells.\n            // Affects content-skeleton, mirror-skeleton, highlight-skeleton for both the time-grid and day-grid.\n            _this.renderDayGridIntroHtml = function () {\n                return '<td class=\"fc-axis\" ' + _this.axisStyleAttr() + '></td>';\n            };\n            return _this;\n        }\n        AbstractTimeGridView.prototype.render = function (props, context) {\n            _super.prototype.render.call(this, props, context);\n            this.renderSkeleton(context);\n        };\n        AbstractTimeGridView.prototype.destroy = function () {\n            _super.prototype.destroy.call(this);\n            this.renderSkeleton.unrender();\n        };\n        AbstractTimeGridView.prototype._renderSkeleton = function (context) {\n            this.el.classList.add('fc-timeGrid-view');\n            this.el.innerHTML = this.renderSkeletonHtml();\n            this.scroller = new core.ScrollComponent('hidden', // overflow x\n            'auto' // overflow y\n            );\n            var timeGridWrapEl = this.scroller.el;\n            this.el.querySelector('.fc-body > tr > td').appendChild(timeGridWrapEl);\n            timeGridWrapEl.classList.add('fc-time-grid-container');\n            var timeGridEl = core.createElement('div', { className: 'fc-time-grid' });\n            timeGridWrapEl.appendChild(timeGridEl);\n            this.timeGrid = new TimeGrid(timeGridEl, {\n                renderBgIntroHtml: this.renderTimeGridBgIntroHtml,\n                renderIntroHtml: this.renderTimeGridIntroHtml\n            });\n            if (context.options.allDaySlot) { // should we display the \"all-day\" area?\n                this.dayGrid = new daygrid.DayGrid(// the all-day subcomponent of this view\n                this.el.querySelector('.fc-day-grid'), {\n                    renderNumberIntroHtml: this.renderDayGridIntroHtml,\n                    renderBgIntroHtml: this.renderDayGridBgIntroHtml,\n                    renderIntroHtml: this.renderDayGridIntroHtml,\n                    colWeekNumbersVisible: false,\n                    cellWeekNumbersVisible: false\n                });\n                // have the day-grid extend it's coordinate area over the <hr> dividing the two grids\n                var dividerEl = this.el.querySelector('.fc-divider');\n                this.dayGrid.bottomCoordPadding = dividerEl.getBoundingClientRect().height;\n            }\n        };\n        AbstractTimeGridView.prototype._unrenderSkeleton = function () {\n            this.el.classList.remove('fc-timeGrid-view');\n            this.timeGrid.destroy();\n            if (this.dayGrid) {\n                this.dayGrid.destroy();\n            }\n            this.scroller.destroy();\n        };\n        /* Rendering\n        ------------------------------------------------------------------------------------------------------------------*/\n        // Builds the HTML skeleton for the view.\n        // The day-grid and time-grid components will render inside containers defined by this HTML.\n        AbstractTimeGridView.prototype.renderSkeletonHtml = function () {\n            var _a = this.context, theme = _a.theme, options = _a.options;\n            return '' +\n                '<table class=\"' + theme.getClass('tableGrid') + '\">' +\n                (options.columnHeader ?\n                    '<thead class=\"fc-head\">' +\n                        '<tr>' +\n                        '<td class=\"fc-head-container ' + theme.getClass('widgetHeader') + '\">&nbsp;</td>' +\n                        '</tr>' +\n                        '</thead>' :\n                    '') +\n                '<tbody class=\"fc-body\">' +\n                '<tr>' +\n                '<td class=\"' + theme.getClass('widgetContent') + '\">' +\n                (options.allDaySlot ?\n                    '<div class=\"fc-day-grid\"></div>' +\n                        '<hr class=\"fc-divider ' + theme.getClass('widgetHeader') + '\" />' :\n                    '') +\n                '</td>' +\n                '</tr>' +\n                '</tbody>' +\n                '</table>';\n        };\n        /* Now Indicator\n        ------------------------------------------------------------------------------------------------------------------*/\n        AbstractTimeGridView.prototype.getNowIndicatorUnit = function () {\n            return this.timeGrid.getNowIndicatorUnit();\n        };\n        // subclasses should implement\n        // renderNowIndicator(date: DateMarker) {\n        // }\n        AbstractTimeGridView.prototype.unrenderNowIndicator = function () {\n            this.timeGrid.unrenderNowIndicator();\n        };\n        /* Dimensions\n        ------------------------------------------------------------------------------------------------------------------*/\n        AbstractTimeGridView.prototype.updateSize = function (isResize, viewHeight, isAuto) {\n            _super.prototype.updateSize.call(this, isResize, viewHeight, isAuto); // will call updateBaseSize. important that executes first\n            this.timeGrid.updateSize(isResize);\n            if (this.dayGrid) {\n                this.dayGrid.updateSize(isResize);\n            }\n        };\n        // Adjusts the vertical dimensions of the view to the specified values\n        AbstractTimeGridView.prototype.updateBaseSize = function (isResize, viewHeight, isAuto) {\n            var _this = this;\n            var eventLimit;\n            var scrollerHeight;\n            var scrollbarWidths;\n            // make all axis cells line up\n            this.axisWidth = core.matchCellWidths(core.findElements(this.el, '.fc-axis'));\n            // hack to give the view some height prior to timeGrid's columns being rendered\n            // TODO: separate setting height from scroller VS timeGrid.\n            if (!this.timeGrid.colEls) {\n                if (!isAuto) {\n                    scrollerHeight = this.computeScrollerHeight(viewHeight);\n                    this.scroller.setHeight(scrollerHeight);\n                }\n                return;\n            }\n            // set of fake row elements that must compensate when scroller has scrollbars\n            var noScrollRowEls = core.findElements(this.el, '.fc-row').filter(function (node) {\n                return !_this.scroller.el.contains(node);\n            });\n            // reset all dimensions back to the original state\n            this.timeGrid.bottomRuleEl.style.display = 'none'; // will be shown later if this <hr> is necessary\n            this.scroller.clear(); // sets height to 'auto' and clears overflow\n            noScrollRowEls.forEach(core.uncompensateScroll);\n            // limit number of events in the all-day area\n            if (this.dayGrid) {\n                this.dayGrid.removeSegPopover(); // kill the \"more\" popover if displayed\n                eventLimit = this.context.options.eventLimit;\n                if (eventLimit && typeof eventLimit !== 'number') {\n                    eventLimit = TIMEGRID_ALL_DAY_EVENT_LIMIT; // make sure \"auto\" goes to a real number\n                }\n                if (eventLimit) {\n                    this.dayGrid.limitRows(eventLimit);\n                }\n            }\n            if (!isAuto) { // should we force dimensions of the scroll container?\n                scrollerHeight = this.computeScrollerHeight(viewHeight);\n                this.scroller.setHeight(scrollerHeight);\n                scrollbarWidths = this.scroller.getScrollbarWidths();\n                if (scrollbarWidths.left || scrollbarWidths.right) { // using scrollbars?\n                    // make the all-day and header rows lines up\n                    noScrollRowEls.forEach(function (rowEl) {\n                        core.compensateScroll(rowEl, scrollbarWidths);\n                    });\n                    // the scrollbar compensation might have changed text flow, which might affect height, so recalculate\n                    // and reapply the desired height to the scroller.\n                    scrollerHeight = this.computeScrollerHeight(viewHeight);\n                    this.scroller.setHeight(scrollerHeight);\n                }\n                // guarantees the same scrollbar widths\n                this.scroller.lockOverflow(scrollbarWidths);\n                // if there's any space below the slats, show the horizontal rule.\n                // this won't cause any new overflow, because lockOverflow already called.\n                if (this.timeGrid.getTotalSlatHeight() < scrollerHeight) {\n                    this.timeGrid.bottomRuleEl.style.display = '';\n                }\n            }\n        };\n        // given a desired total height of the view, returns what the height of the scroller should be\n        AbstractTimeGridView.prototype.computeScrollerHeight = function (viewHeight) {\n            return viewHeight -\n                core.subtractInnerElHeight(this.el, this.scroller.el); // everything that's NOT the scroller\n        };\n        /* Scroll\n        ------------------------------------------------------------------------------------------------------------------*/\n        // Computes the initial pre-configured scroll state prior to allowing the user to change it\n        AbstractTimeGridView.prototype.computeDateScroll = function (duration) {\n            var top = this.timeGrid.computeTimeTop(duration);\n            // zoom can give weird floating-point values. rather scroll a little bit further\n            top = Math.ceil(top);\n            if (top) {\n                top++; // to overcome top border that slots beyond the first have. looks better\n            }\n            return { top: top };\n        };\n        AbstractTimeGridView.prototype.queryDateScroll = function () {\n            return { top: this.scroller.getScrollTop() };\n        };\n        AbstractTimeGridView.prototype.applyDateScroll = function (scroll) {\n            if (scroll.top !== undefined) {\n                this.scroller.setScrollTop(scroll.top);\n            }\n        };\n        // Generates an HTML attribute string for setting the width of the axis, if it is known\n        AbstractTimeGridView.prototype.axisStyleAttr = function () {\n            if (this.axisWidth != null) {\n                return 'style=\"width:' + this.axisWidth + 'px\"';\n            }\n            return '';\n        };\n        return AbstractTimeGridView;\n    }(core.View));\n    AbstractTimeGridView.prototype.usesMinMaxTime = true; // indicates that minTime/maxTime affects rendering\n\n    var SimpleTimeGrid = /** @class */ (function (_super) {\n        __extends(SimpleTimeGrid, _super);\n        function SimpleTimeGrid(timeGrid) {\n            var _this = _super.call(this, timeGrid.el) || this;\n            _this.buildDayRanges = core.memoize(buildDayRanges);\n            _this.slicer = new TimeGridSlicer();\n            _this.timeGrid = timeGrid;\n            return _this;\n        }\n        SimpleTimeGrid.prototype.firstContext = function (context) {\n            context.calendar.registerInteractiveComponent(this, {\n                el: this.timeGrid.el\n            });\n        };\n        SimpleTimeGrid.prototype.destroy = function () {\n            _super.prototype.destroy.call(this);\n            this.context.calendar.unregisterInteractiveComponent(this);\n        };\n        SimpleTimeGrid.prototype.render = function (props, context) {\n            var dateEnv = this.context.dateEnv;\n            var dateProfile = props.dateProfile, dayTable = props.dayTable;\n            var dayRanges = this.dayRanges = this.buildDayRanges(dayTable, dateProfile, dateEnv);\n            var timeGrid = this.timeGrid;\n            timeGrid.receiveContext(context); // hack because context is used in sliceProps\n            timeGrid.receiveProps(__assign({}, this.slicer.sliceProps(props, dateProfile, null, context.calendar, timeGrid, dayRanges), { dateProfile: dateProfile, cells: dayTable.cells[0] }), context);\n        };\n        SimpleTimeGrid.prototype.renderNowIndicator = function (date) {\n            this.timeGrid.renderNowIndicator(this.slicer.sliceNowDate(date, this.timeGrid, this.dayRanges), date);\n        };\n        SimpleTimeGrid.prototype.buildPositionCaches = function () {\n            this.timeGrid.buildPositionCaches();\n        };\n        SimpleTimeGrid.prototype.queryHit = function (positionLeft, positionTop) {\n            var rawHit = this.timeGrid.positionToHit(positionLeft, positionTop);\n            if (rawHit) {\n                return {\n                    component: this.timeGrid,\n                    dateSpan: rawHit.dateSpan,\n                    dayEl: rawHit.dayEl,\n                    rect: {\n                        left: rawHit.relativeRect.left,\n                        right: rawHit.relativeRect.right,\n                        top: rawHit.relativeRect.top,\n                        bottom: rawHit.relativeRect.bottom\n                    },\n                    layer: 0\n                };\n            }\n        };\n        return SimpleTimeGrid;\n    }(core.DateComponent));\n    function buildDayRanges(dayTable, dateProfile, dateEnv) {\n        var ranges = [];\n        for (var _i = 0, _a = dayTable.headerDates; _i < _a.length; _i++) {\n            var date = _a[_i];\n            ranges.push({\n                start: dateEnv.add(date, dateProfile.minTime),\n                end: dateEnv.add(date, dateProfile.maxTime)\n            });\n        }\n        return ranges;\n    }\n    var TimeGridSlicer = /** @class */ (function (_super) {\n        __extends(TimeGridSlicer, _super);\n        function TimeGridSlicer() {\n            return _super !== null && _super.apply(this, arguments) || this;\n        }\n        TimeGridSlicer.prototype.sliceRange = function (range, dayRanges) {\n            var segs = [];\n            for (var col = 0; col < dayRanges.length; col++) {\n                var segRange = core.intersectRanges(range, dayRanges[col]);\n                if (segRange) {\n                    segs.push({\n                        start: segRange.start,\n                        end: segRange.end,\n                        isStart: segRange.start.valueOf() === range.start.valueOf(),\n                        isEnd: segRange.end.valueOf() === range.end.valueOf(),\n                        col: col\n                    });\n                }\n            }\n            return segs;\n        };\n        return TimeGridSlicer;\n    }(core.Slicer));\n\n    var TimeGridView = /** @class */ (function (_super) {\n        __extends(TimeGridView, _super);\n        function TimeGridView() {\n            var _this = _super !== null && _super.apply(this, arguments) || this;\n            _this.buildDayTable = core.memoize(buildDayTable);\n            return _this;\n        }\n        TimeGridView.prototype.render = function (props, context) {\n            _super.prototype.render.call(this, props, context); // for flags for updateSize. also _renderSkeleton/_unrenderSkeleton\n            var _a = this.props, dateProfile = _a.dateProfile, dateProfileGenerator = _a.dateProfileGenerator;\n            var nextDayThreshold = context.nextDayThreshold;\n            var dayTable = this.buildDayTable(dateProfile, dateProfileGenerator);\n            var splitProps = this.splitter.splitProps(props);\n            if (this.header) {\n                this.header.receiveProps({\n                    dateProfile: dateProfile,\n                    dates: dayTable.headerDates,\n                    datesRepDistinctDays: true,\n                    renderIntroHtml: this.renderHeadIntroHtml\n                }, context);\n            }\n            this.simpleTimeGrid.receiveProps(__assign({}, splitProps['timed'], { dateProfile: dateProfile,\n                dayTable: dayTable }), context);\n            if (this.simpleDayGrid) {\n                this.simpleDayGrid.receiveProps(__assign({}, splitProps['allDay'], { dateProfile: dateProfile,\n                    dayTable: dayTable,\n                    nextDayThreshold: nextDayThreshold, isRigid: false }), context);\n            }\n            this.startNowIndicator(dateProfile, dateProfileGenerator);\n        };\n        TimeGridView.prototype._renderSkeleton = function (context) {\n            _super.prototype._renderSkeleton.call(this, context);\n            if (context.options.columnHeader) {\n                this.header = new core.DayHeader(this.el.querySelector('.fc-head-container'));\n            }\n            this.simpleTimeGrid = new SimpleTimeGrid(this.timeGrid);\n            if (this.dayGrid) {\n                this.simpleDayGrid = new daygrid.SimpleDayGrid(this.dayGrid);\n            }\n        };\n        TimeGridView.prototype._unrenderSkeleton = function () {\n            _super.prototype._unrenderSkeleton.call(this);\n            if (this.header) {\n                this.header.destroy();\n            }\n            this.simpleTimeGrid.destroy();\n            if (this.simpleDayGrid) {\n                this.simpleDayGrid.destroy();\n            }\n        };\n        TimeGridView.prototype.renderNowIndicator = function (date) {\n            this.simpleTimeGrid.renderNowIndicator(date);\n        };\n        return TimeGridView;\n    }(AbstractTimeGridView));\n    function buildDayTable(dateProfile, dateProfileGenerator) {\n        var daySeries = new core.DaySeries(dateProfile.renderRange, dateProfileGenerator);\n        return new core.DayTable(daySeries, false);\n    }\n\n    var main = core.createPlugin({\n        defaultView: 'timeGridWeek',\n        views: {\n            timeGrid: {\n                class: TimeGridView,\n                allDaySlot: true,\n                slotDuration: '00:30:00',\n                slotEventOverlap: true // a bad name. confused with overlap/constraint system\n            },\n            timeGridDay: {\n                type: 'timeGrid',\n                duration: { days: 1 }\n            },\n            timeGridWeek: {\n                type: 'timeGrid',\n                duration: { weeks: 1 }\n            }\n        }\n    });\n\n    exports.AbstractTimeGridView = AbstractTimeGridView;\n    exports.TimeGrid = TimeGrid;\n    exports.TimeGridSlicer = TimeGridSlicer;\n    exports.TimeGridView = TimeGridView;\n    exports.buildDayRanges = buildDayRanges;\n    exports.buildDayTable = buildDayTable;\n    exports.default = main;\n\n    Object.defineProperty(exports, '__esModule', { value: true });\n\n}));\n"

/***/ },

/***/ 1093
(module) {

module.exports = "/*!\nFullCalendar Timeline Plugin v4.4.3\nDocs & License: https://fullcalendar.io/scheduler\n(c) 2019 Adam Shaw\n*/\n\n(function (global, factory) {\n    typeof exports === 'object' && typeof module !== 'undefined' ? factory(exports, require('@fullcalendar/core')) :\n    typeof define === 'function' && define.amd ? define(['exports', '@fullcalendar/core'], factory) :\n    (global = global || self, factory(global.FullCalendarTimeline = {}, global.FullCalendar));\n}(this, function (exports, core) { 'use strict';\n\n    /*! *****************************************************************************\n    Copyright (c) Microsoft Corporation.\n\n    Permission to use, copy, modify, and/or distribute this software for any\n    purpose with or without fee is hereby granted.\n\n    THE SOFTWARE IS PROVIDED \"AS IS\" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH\n    REGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY\n    AND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,\n    INDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM\n    LOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR\n    OTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR\n    PERFORMANCE OF THIS SOFTWARE.\n    ***************************************************************************** */\n    /* global Reflect, Promise */\n\n    var extendStatics = function(d, b) {\n        extendStatics = Object.setPrototypeOf ||\n            ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||\n            function (d, b) { for (var p in b) if (b.hasOwnProperty(p)) d[p] = b[p]; };\n        return extendStatics(d, b);\n    };\n\n    function __extends(d, b) {\n        extendStatics(d, b);\n        function __() { this.constructor = d; }\n        d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());\n    }\n\n    var __assign = function() {\n        __assign = Object.assign || function __assign(t) {\n            for (var s, i = 1, n = arguments.length; i < n; i++) {\n                s = arguments[i];\n                for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];\n            }\n            return t;\n        };\n        return __assign.apply(this, arguments);\n    };\n\n    /*\n    A rectangular area of content that lives within a Scroller.\n    Can have \"gutters\", areas of dead spacing around the perimeter.\n    Also very useful for forcing a width, which a Scroller cannot do alone.\n    Has a content area that lives above a background area.\n    */\n    var ScrollerCanvas = /** @class */ (function () {\n        function ScrollerCanvas() {\n            this.gutters = {};\n            this.el = core.htmlToElement(\"<div class=\\\"fc-scroller-canvas\\\"> <div class=\\\"fc-content\\\"></div> <div class=\\\"fc-bg\\\"></div> </div>\");\n            this.contentEl = this.el.querySelector('.fc-content');\n            this.bgEl = this.el.querySelector('.fc-bg');\n        }\n        /*\n        If falsy, resets all the gutters to 0\n        */\n        ScrollerCanvas.prototype.setGutters = function (gutters) {\n            if (!gutters) {\n                this.gutters = {};\n            }\n            else {\n                __assign(this.gutters, gutters);\n            }\n            this.updateSize();\n        };\n        ScrollerCanvas.prototype.setWidth = function (width) {\n            this.width = width;\n            this.updateSize();\n        };\n        ScrollerCanvas.prototype.setMinWidth = function (minWidth) {\n            this.minWidth = minWidth;\n            this.updateSize();\n        };\n        ScrollerCanvas.prototype.clearWidth = function () {\n            this.width = null;\n            this.minWidth = null;\n            this.updateSize();\n        };\n        ScrollerCanvas.prototype.updateSize = function () {\n            var _a = this, gutters = _a.gutters, el = _a.el;\n            // is border-box (width includes padding)\n            core.forceClassName(el, 'fc-gutter-left', gutters.left);\n            core.forceClassName(el, 'fc-gutter-right', gutters.right);\n            core.forceClassName(el, 'fc-gutter-top', gutters.top);\n            core.forceClassName(el, 'fc-gutter-bottom', gutters.bottom);\n            core.applyStyle(el, {\n                paddingLeft: gutters.left || '',\n                paddingRight: gutters.right || '',\n                paddingTop: gutters.top || '',\n                paddingBottom: gutters.bottom || '',\n                width: (this.width != null) ?\n                    this.width + (gutters.left || 0) + (gutters.right || 0) :\n                    '',\n                minWidth: (this.minWidth != null) ?\n                    this.minWidth + (gutters.left || 0) + (gutters.right || 0) :\n                    ''\n            });\n            core.applyStyle(this.bgEl, {\n                left: gutters.left || '',\n                right: gutters.right || '',\n                top: gutters.top || '',\n                bottom: gutters.bottom || ''\n            });\n        };\n        return ScrollerCanvas;\n    }());\n\n    var EnhancedScroller = /** @class */ (function (_super) {\n        __extends(EnhancedScroller, _super);\n        function EnhancedScroller(overflowX, overflowY) {\n            var _this = _super.call(this, overflowX, overflowY) || this;\n            // Scroll Events\n            // ----------------------------------------------------------------------------------------------\n            _this.reportScroll = function () {\n                if (!_this.isScrolling) {\n                    _this.reportScrollStart();\n                }\n                _this.trigger('scroll');\n                _this.isMoving = true;\n                _this.requestMovingEnd();\n            };\n            _this.reportScrollStart = function () {\n                if (!_this.isScrolling) {\n                    _this.isScrolling = true;\n                    _this.trigger('scrollStart', _this.isTouching); // created in constructor\n                }\n            };\n            // Touch Events\n            // ----------------------------------------------------------------------------------------------\n            // will fire *before* the scroll event is fired\n            _this.reportTouchStart = function () {\n                _this.isTouching = true;\n            };\n            _this.reportTouchEnd = function () {\n                if (_this.isTouching) {\n                    _this.isTouching = false;\n                    // if touch scrolling was re-enabled during a recent touch scroll\n                    // then unbind the handlers that are preventing it from happening.\n                    if (_this.isTouchScrollEnabled) {\n                        _this.unbindPreventTouchScroll(); // won't do anything if not bound\n                    }\n                    // if the user ended their touch, and the scroll area wasn't moving,\n                    // we consider this to be the end of the scroll.\n                    if (!_this.isMoving) {\n                        _this.reportScrollEnd(); // won't fire if already ended\n                    }\n                }\n            };\n            _this.isScrolling = false;\n            _this.isTouching = false;\n            _this.isMoving = false;\n            _this.isTouchScrollEnabled = true;\n            _this.requestMovingEnd = core.debounce(_this.reportMovingEnd, 500);\n            _this.canvas = new ScrollerCanvas();\n            _this.el.appendChild(_this.canvas.el);\n            _this.applyOverflow();\n            _this.bindHandlers();\n            return _this;\n        }\n        EnhancedScroller.prototype.destroy = function () {\n            _super.prototype.destroy.call(this);\n            this.unbindHandlers();\n        };\n        // Touch scroll prevention\n        // ----------------------------------------------------------------------------------------------\n        EnhancedScroller.prototype.disableTouchScroll = function () {\n            this.isTouchScrollEnabled = false;\n            this.bindPreventTouchScroll(); // will be unbound in enableTouchScroll or reportTouchEnd\n        };\n        EnhancedScroller.prototype.enableTouchScroll = function () {\n            this.isTouchScrollEnabled = true;\n            // only immediately unbind if a touch event is NOT in progress.\n            // otherwise, it will be handled by reportTouchEnd.\n            if (!this.isTouching) {\n                this.unbindPreventTouchScroll();\n            }\n        };\n        EnhancedScroller.prototype.bindPreventTouchScroll = function () {\n            if (!this.preventTouchScrollHandler) {\n                this.el.addEventListener('touchmove', (this.preventTouchScrollHandler = core.preventDefault));\n            }\n        };\n        EnhancedScroller.prototype.unbindPreventTouchScroll = function () {\n            if (this.preventTouchScrollHandler) {\n                this.el.removeEventListener('touchmove', this.preventTouchScrollHandler);\n                this.preventTouchScrollHandler = null;\n            }\n        };\n        // Handlers\n        // ----------------------------------------------------------------------------------------------\n        EnhancedScroller.prototype.bindHandlers = function () {\n            this.el.addEventListener('scroll', this.reportScroll);\n            this.el.addEventListener('touchstart', this.reportTouchStart, { passive: true });\n            this.el.addEventListener('touchend', this.reportTouchEnd);\n        };\n        EnhancedScroller.prototype.unbindHandlers = function () {\n            this.el.removeEventListener('scroll', this.reportScroll);\n            this.el.removeEventListener('touchstart', this.reportTouchStart, { passive: true });\n            this.el.removeEventListener('touchend', this.reportTouchEnd);\n        };\n        EnhancedScroller.prototype.reportMovingEnd = function () {\n            this.isMoving = false;\n            // only end the scroll if not currently touching.\n            // if touching, the scrolling will end later, on touchend.\n            if (!this.isTouching) {\n                this.reportScrollEnd();\n            }\n        };\n        EnhancedScroller.prototype.reportScrollEnd = function () {\n            if (this.isScrolling) {\n                this.trigger('scrollEnd');\n                this.isScrolling = false;\n            }\n        };\n        // Horizontal Scroll Normalization\n        // ----------------------------------------------------------------------------------------------\n        // http://stackoverflow.com/questions/24276619/better-way-to-get-the-viewport-of-a-scrollable-div-in-rtl-mode/24394376#24394376\n        // TODO: move all this to util functions\n        /*\n        If RTL, and scrolled to the left, returns NEGATIVE value (like Firefox)\n        */\n        EnhancedScroller.prototype.getScrollLeft = function () {\n            var el = this.el;\n            var direction = window.getComputedStyle(el).direction;\n            var val = el.scrollLeft;\n            if (direction === 'rtl') {\n                switch (getRtlScrollSystem()) {\n                    case 'positive':\n                        val = (val + el.clientWidth) - el.scrollWidth;\n                        break;\n                    case 'reverse':\n                        val = -val;\n                        break;\n                }\n            }\n            return val;\n        };\n        /*\n        Accepts a NEGATIVE value for when scrolled in RTL\n        */\n        EnhancedScroller.prototype.setScrollLeft = function (val) {\n            var el = this.el;\n            var direction = window.getComputedStyle(el).direction;\n            if (direction === 'rtl') {\n                switch (getRtlScrollSystem()) {\n                    case 'positive':\n                        val = (val - el.clientWidth) + el.scrollWidth;\n                        break;\n                    case 'reverse':\n                        val = -val;\n                        break;\n                }\n            }\n            el.scrollLeft = val;\n        };\n        /*\n        Always returns the number of pixels scrolled from the leftmost position (even if RTL).\n        Always positive.\n        */\n        EnhancedScroller.prototype.getScrollFromLeft = function () {\n            var el = this.el;\n            var direction = window.getComputedStyle(el).direction;\n            var val = el.scrollLeft;\n            if (direction === 'rtl') {\n                switch (getRtlScrollSystem()) {\n                    case 'negative':\n                        val = (val - el.clientWidth) + el.scrollWidth;\n                        break;\n                    case 'reverse':\n                        val = (-val - el.clientWidth) + el.scrollWidth;\n                        break;\n                }\n            }\n            return val;\n        };\n        return EnhancedScroller;\n    }(core.ScrollComponent));\n    core.EmitterMixin.mixInto(EnhancedScroller);\n    // Horizontal Scroll System Detection\n    // ----------------------------------------------------------------------------------------------\n    var _rtlScrollSystem;\n    function getRtlScrollSystem() {\n        return _rtlScrollSystem || (_rtlScrollSystem = detectRtlScrollSystem());\n    }\n    function detectRtlScrollSystem() {\n        var el = core.htmlToElement(\"<div style=\\\" position: absolute; top: -1000px; width: 1px; height: 1px; overflow: scroll; direction: rtl; font-size: 100px; \\\">A</div>\");\n        document.body.appendChild(el);\n        var system;\n        if (el.scrollLeft > 0) {\n            system = 'positive';\n        }\n        else {\n            el.scrollLeft = 1;\n            if (el.scrollLeft > 0) {\n                system = 'reverse';\n            }\n            else {\n                system = 'negative';\n            }\n        }\n        core.removeElement(el);\n        return system;\n    }\n\n    /*\n    A Scroller, but with a wrapping div that allows \"clipping\" away of native scrollbars,\n    giving the appearance that there are no scrollbars.\n    */\n    var ClippedScroller = /** @class */ (function () {\n        /*\n        Received overflows can be set to 'clipped', meaning scrollbars shouldn't be visible\n        to the user, but the area should still scroll.\n        */\n        function ClippedScroller(overflowX, overflowY, parentEl) {\n            this.isHScrollbarsClipped = false;\n            this.isVScrollbarsClipped = false;\n            if (overflowX === 'clipped-scroll') {\n                overflowX = 'scroll';\n                this.isHScrollbarsClipped = true;\n            }\n            if (overflowY === 'clipped-scroll') {\n                overflowY = 'scroll';\n                this.isVScrollbarsClipped = true;\n            }\n            this.enhancedScroll = new EnhancedScroller(overflowX, overflowY);\n            parentEl.appendChild(this.el = core.createElement('div', {\n                className: 'fc-scroller-clip'\n            }));\n            this.el.appendChild(this.enhancedScroll.el);\n        }\n        ClippedScroller.prototype.destroy = function () {\n            core.removeElement(this.el);\n        };\n        ClippedScroller.prototype.updateSize = function () {\n            var enhancedScroll = this.enhancedScroll;\n            var scrollEl = enhancedScroll.el;\n            var edges = core.computeEdges(scrollEl);\n            var cssProps = { marginLeft: 0, marginRight: 0, marginTop: 0, marginBottom: 0 };\n            // give the inner scrolling div negative margins so that its scrollbars\n            // are nudged outside of the bounding box of the wrapper, which is overflow:hidden\n            if (this.isVScrollbarsClipped) {\n                cssProps.marginLeft = -edges.scrollbarLeft;\n                cssProps.marginRight = -edges.scrollbarRight;\n            }\n            if (this.isHScrollbarsClipped) {\n                cssProps.marginBottom = -edges.scrollbarBottom;\n            }\n            core.applyStyle(scrollEl, cssProps);\n            // if we are attempting to hide the scrollbars offscreen, OSX/iOS will still\n            // display the floating scrollbars. attach a className to force-hide them.\n            if ((this.isHScrollbarsClipped || (enhancedScroll.overflowX === 'hidden')) && // should never show?\n                (this.isVScrollbarsClipped || (enhancedScroll.overflowY === 'hidden')) && // should never show?\n                !( // doesn't have any scrollbar mass\n                edges.scrollbarLeft ||\n                    edges.scrollbarRight ||\n                    edges.scrollbarBottom)) {\n                scrollEl.classList.add('fc-no-scrollbars');\n            }\n            else {\n                scrollEl.classList.remove('fc-no-scrollbars');\n            }\n        };\n        ClippedScroller.prototype.setHeight = function (height) {\n            this.enhancedScroll.setHeight(height);\n        };\n        /*\n        Accounts for 'clipped' scrollbars\n        */\n        ClippedScroller.prototype.getScrollbarWidths = function () {\n            var widths = this.enhancedScroll.getScrollbarWidths();\n            if (this.isVScrollbarsClipped) {\n                widths.left = 0;\n                widths.right = 0;\n            }\n            if (this.isHScrollbarsClipped) {\n                widths.bottom = 0;\n            }\n            return widths;\n        };\n        return ClippedScroller;\n    }());\n\n    var ScrollJoiner = /** @class */ (function () {\n        function ScrollJoiner(axis, scrollers) {\n            this.axis = axis;\n            this.scrollers = scrollers;\n            for (var _i = 0, _a = this.scrollers; _i < _a.length; _i++) {\n                var scroller = _a[_i];\n                this.initScroller(scroller);\n            }\n        }\n        ScrollJoiner.prototype.initScroller = function (scroller) {\n            var _this = this;\n            var enhancedScroll = scroller.enhancedScroll;\n            // when the user scrolls via mousewheel, we know for sure the target\n            // scroller should be the master. capture the various x-browser events that fire.\n            var onScroll = function () {\n                _this.assignMasterScroller(scroller);\n            };\n            'wheel mousewheel DomMouseScroll MozMousePixelScroll'.split(' ').forEach(function (evName) {\n                enhancedScroll.el.addEventListener(evName, onScroll);\n            });\n            enhancedScroll\n                .on('scrollStart', function () {\n                if (!_this.masterScroller) {\n                    _this.assignMasterScroller(scroller);\n                }\n            })\n                .on('scroll', function () {\n                if (scroller === _this.masterScroller) {\n                    for (var _i = 0, _a = _this.scrollers; _i < _a.length; _i++) {\n                        var otherScroller = _a[_i];\n                        if (otherScroller !== scroller) {\n                            switch (_this.axis) {\n                                case 'horizontal':\n                                    otherScroller.enhancedScroll.el.scrollLeft = enhancedScroll.el.scrollLeft;\n                                    break;\n                                case 'vertical':\n                                    otherScroller.enhancedScroll.setScrollTop(enhancedScroll.getScrollTop());\n                                    break;\n                            }\n                        }\n                    }\n                }\n            })\n                .on('scrollEnd', function () {\n                if (scroller === _this.masterScroller) {\n                    _this.unassignMasterScroller();\n                }\n            });\n        };\n        ScrollJoiner.prototype.assignMasterScroller = function (scroller) {\n            this.unassignMasterScroller();\n            this.masterScroller = scroller;\n            for (var _i = 0, _a = this.scrollers; _i < _a.length; _i++) {\n                var otherScroller = _a[_i];\n                if (otherScroller !== scroller) {\n                    otherScroller.enhancedScroll.disableTouchScroll();\n                }\n            }\n        };\n        ScrollJoiner.prototype.unassignMasterScroller = function () {\n            if (this.masterScroller) {\n                for (var _i = 0, _a = this.scrollers; _i < _a.length; _i++) {\n                    var otherScroller = _a[_i];\n                    otherScroller.enhancedScroll.enableTouchScroll();\n                }\n                this.masterScroller = null;\n            }\n        };\n        ScrollJoiner.prototype.update = function () {\n            var allWidths = this.scrollers.map(function (scroller) { return scroller.getScrollbarWidths(); });\n            var maxLeft = 0;\n            var maxRight = 0;\n            var maxTop = 0;\n            var maxBottom = 0;\n            var widths;\n            var i;\n            for (var _i = 0, allWidths_1 = allWidths; _i < allWidths_1.length; _i++) {\n                widths = allWidths_1[_i];\n                maxLeft = Math.max(maxLeft, widths.left);\n                maxRight = Math.max(maxRight, widths.right);\n                maxTop = Math.max(maxTop, widths.top);\n                maxBottom = Math.max(maxBottom, widths.bottom);\n            }\n            for (i = 0; i < this.scrollers.length; i++) {\n                var scroller = this.scrollers[i];\n                widths = allWidths[i];\n                scroller.enhancedScroll.canvas.setGutters(this.axis === 'horizontal' ?\n                    {\n                        left: maxLeft - widths.left,\n                        right: maxRight - widths.right\n                    } :\n                    {\n                        top: maxTop - widths.top,\n                        bottom: maxBottom - widths.bottom\n                    });\n            }\n        };\n        return ScrollJoiner;\n    }());\n\n    var HeaderBodyLayout = /** @class */ (function () {\n        /*\n        verticalScroll = 'auto' | 'clipped-scroll'\n        */\n        function HeaderBodyLayout(headerContainerEl, bodyContainerEl, verticalScroll) {\n            this.headerScroller = new ClippedScroller('clipped-scroll', 'hidden', headerContainerEl);\n            this.bodyScroller = new ClippedScroller('auto', verticalScroll, bodyContainerEl);\n            this.scrollJoiner = new ScrollJoiner('horizontal', [\n                this.headerScroller,\n                this.bodyScroller\n            ]);\n        }\n        HeaderBodyLayout.prototype.destroy = function () {\n            this.headerScroller.destroy();\n            this.bodyScroller.destroy();\n        };\n        HeaderBodyLayout.prototype.setHeight = function (totalHeight, isAuto) {\n            var bodyHeight;\n            if (isAuto) {\n                bodyHeight = 'auto';\n            }\n            else {\n                bodyHeight = totalHeight - this.queryHeadHeight();\n            }\n            this.bodyScroller.setHeight(bodyHeight);\n            this.headerScroller.updateSize(); // adjusts gutters and classNames\n            this.bodyScroller.updateSize(); // adjusts gutters and classNames\n            this.scrollJoiner.update();\n        };\n        HeaderBodyLayout.prototype.queryHeadHeight = function () {\n            return this.headerScroller.enhancedScroll.canvas.contentEl.getBoundingClientRect().height;\n        };\n        return HeaderBodyLayout;\n    }());\n\n    var TimelineHeader = /** @class */ (function (_super) {\n        __extends(TimelineHeader, _super);\n        function TimelineHeader(parentEl) {\n            var _this = _super.call(this) || this;\n            _this.parentEl = parentEl;\n            return _this;\n        }\n        TimelineHeader.prototype.firstContext = function (context) {\n            this.parentEl.appendChild(this.tableEl = core.createElement('table', {\n                className: context.theme.getClass('tableGrid')\n            }));\n        };\n        TimelineHeader.prototype.destroy = function () {\n            core.removeElement(this.tableEl);\n            _super.prototype.destroy.call(this);\n        };\n        TimelineHeader.prototype.render = function (props) {\n            this.renderDates(props.tDateProfile);\n        };\n        TimelineHeader.prototype.renderDates = function (tDateProfile) {\n            var _a = this.context, dateEnv = _a.dateEnv, theme = _a.theme;\n            var cellRows = tDateProfile.cellRows;\n            var lastRow = cellRows[cellRows.length - 1];\n            var isChrono = core.asRoughMs(tDateProfile.labelInterval) > core.asRoughMs(tDateProfile.slotDuration);\n            var oneDay = core.isSingleDay(tDateProfile.slotDuration);\n            var html = '<colgroup>';\n            // needs to be a col for each body slat. header cells will have colspans\n            for (var i = tDateProfile.slotCnt - 1; i >= 0; i--) {\n                html += '<col/>';\n            }\n            html += '</colgroup>';\n            html += '<tbody>';\n            for (var _i = 0, cellRows_1 = cellRows; _i < cellRows_1.length; _i++) {\n                var rowCells = cellRows_1[_i];\n                var isLast = rowCells === lastRow;\n                html += '<tr' + (isChrono && isLast ? ' class=\"fc-chrono\"' : '') + '>';\n                for (var _b = 0, rowCells_1 = rowCells; _b < rowCells_1.length; _b++) {\n                    var cell = rowCells_1[_b];\n                    var headerCellClassNames = [theme.getClass('widgetHeader')];\n                    if (cell.isWeekStart) {\n                        headerCellClassNames.push('fc-em-cell');\n                    }\n                    if (oneDay) {\n                        headerCellClassNames = headerCellClassNames.concat(core.getDayClasses(cell.date, this.props.dateProfile, this.context, true) // adds \"today\" class and other day-based classes\n                        );\n                    }\n                    html +=\n                        '<th class=\"' + headerCellClassNames.join(' ') + '\"' +\n                            ' data-date=\"' + dateEnv.formatIso(cell.date, { omitTime: !tDateProfile.isTimeScale, omitTimeZoneOffset: true }) + '\"' +\n                            (cell.colspan > 1 ? ' colspan=\"' + cell.colspan + '\"' : '') +\n                            '>' +\n                            '<div class=\"fc-cell-content\">' +\n                            cell.spanHtml +\n                            '</div>' +\n                            '</th>';\n                }\n                html += '</tr>';\n            }\n            html += '</tbody>';\n            this.tableEl.innerHTML = html; // TODO: does this work cross-browser?\n            this.slatColEls = core.findElements(this.tableEl, 'col');\n            this.innerEls = core.findElements(this.tableEl.querySelector('tr:last-child'), // compound selector won't work because of query-root problem\n            'th .fc-cell-text');\n            core.findElements(this.tableEl.querySelectorAll('tr:not(:last-child)'), // compound selector won't work because of query-root problem\n            'th .fc-cell-text').forEach(function (innerEl) {\n                innerEl.classList.add('fc-sticky');\n            });\n        };\n        return TimelineHeader;\n    }(core.Component));\n\n    var TimelineSlats = /** @class */ (function (_super) {\n        __extends(TimelineSlats, _super);\n        function TimelineSlats(parentEl) {\n            var _this = _super.call(this) || this;\n            parentEl.appendChild(_this.el = core.createElement('div', { className: 'fc-slats' }));\n            return _this;\n        }\n        TimelineSlats.prototype.destroy = function () {\n            core.removeElement(this.el);\n            _super.prototype.destroy.call(this);\n        };\n        TimelineSlats.prototype.render = function (props) {\n            this.renderDates(props.tDateProfile);\n        };\n        TimelineSlats.prototype.renderDates = function (tDateProfile) {\n            var _a = this.context, calendar = _a.calendar, view = _a.view, theme = _a.theme, dateEnv = _a.dateEnv;\n            var slotDates = tDateProfile.slotDates, isWeekStarts = tDateProfile.isWeekStarts;\n            var html = '<table class=\"' + theme.getClass('tableGrid') + '\">' +\n                '<colgroup>';\n            for (var i = 0; i < slotDates.length; i++) {\n                html += '<col/>';\n            }\n            html += '</colgroup>';\n            html += '<tbody><tr>';\n            for (var i = 0; i < slotDates.length; i++) {\n                html += this.slatCellHtml(slotDates[i], isWeekStarts[i], tDateProfile);\n            }\n            html += '</tr></tbody></table>';\n            this.el.innerHTML = html;\n            this.slatColEls = core.findElements(this.el, 'col');\n            this.slatEls = core.findElements(this.el, 'td');\n            for (var i = 0; i < slotDates.length; i++) {\n                calendar.publiclyTrigger('dayRender', [\n                    {\n                        date: dateEnv.toDate(slotDates[i]),\n                        el: this.slatEls[i],\n                        view: view\n                    }\n                ]);\n            }\n            this.outerCoordCache = new core.PositionCache(this.el, this.slatEls, true, // isHorizontal\n            false // isVertical\n            );\n            // for the inner divs within the slats\n            // used for event rendering and scrollTime, to disregard slat border\n            this.innerCoordCache = new core.PositionCache(this.el, core.findChildren(this.slatEls, 'div'), true, // isHorizontal\n            false // isVertical\n            );\n        };\n        TimelineSlats.prototype.slatCellHtml = function (date, isEm, tDateProfile) {\n            var _a = this.context, theme = _a.theme, dateEnv = _a.dateEnv;\n            var classes;\n            if (tDateProfile.isTimeScale) {\n                classes = [];\n                classes.push(core.isInt(dateEnv.countDurationsBetween(tDateProfile.normalizedRange.start, date, tDateProfile.labelInterval)) ?\n                    'fc-major' :\n                    'fc-minor');\n            }\n            else {\n                classes = core.getDayClasses(date, this.props.dateProfile, this.context);\n                classes.push('fc-day');\n            }\n            classes.unshift(theme.getClass('widgetContent'));\n            if (isEm) {\n                classes.push('fc-em-cell');\n            }\n            return '<td class=\"' + classes.join(' ') + '\"' +\n                ' data-date=\"' + dateEnv.formatIso(date, { omitTime: !tDateProfile.isTimeScale, omitTimeZoneOffset: true }) + '\"' +\n                '><div></div></td>';\n        };\n        TimelineSlats.prototype.updateSize = function () {\n            this.outerCoordCache.build();\n            this.innerCoordCache.build();\n        };\n        TimelineSlats.prototype.positionToHit = function (leftPosition) {\n            var outerCoordCache = this.outerCoordCache;\n            var _a = this.context, dateEnv = _a.dateEnv, isRtl = _a.isRtl;\n            var tDateProfile = this.props.tDateProfile;\n            var slatIndex = outerCoordCache.leftToIndex(leftPosition);\n            if (slatIndex != null) {\n                // somewhat similar to what TimeGrid does. consolidate?\n                var slatWidth = outerCoordCache.getWidth(slatIndex);\n                var partial = isRtl ?\n                    (outerCoordCache.rights[slatIndex] - leftPosition) / slatWidth :\n                    (leftPosition - outerCoordCache.lefts[slatIndex]) / slatWidth;\n                var localSnapIndex = Math.floor(partial * tDateProfile.snapsPerSlot);\n                var start = dateEnv.add(tDateProfile.slotDates[slatIndex], core.multiplyDuration(tDateProfile.snapDuration, localSnapIndex));\n                var end = dateEnv.add(start, tDateProfile.snapDuration);\n                return {\n                    dateSpan: {\n                        range: { start: start, end: end },\n                        allDay: !this.props.tDateProfile.isTimeScale\n                    },\n                    dayEl: this.slatColEls[slatIndex],\n                    left: outerCoordCache.lefts[slatIndex],\n                    right: outerCoordCache.rights[slatIndex]\n                };\n            }\n            return null;\n        };\n        return TimelineSlats;\n    }(core.Component));\n\n    var MIN_AUTO_LABELS = 18; // more than `12` months but less that `24` hours\n    var MAX_AUTO_SLOTS_PER_LABEL = 6; // allows 6 10-min slots in an hour\n    var MAX_AUTO_CELLS = 200; // allows 4-days to have a :30 slot duration\n    core.config.MAX_TIMELINE_SLOTS = 1000;\n    // potential nice values for slot-duration and interval-duration\n    var STOCK_SUB_DURATIONS = [\n        { years: 1 },\n        { months: 1 },\n        { days: 1 },\n        { hours: 1 },\n        { minutes: 30 },\n        { minutes: 15 },\n        { minutes: 10 },\n        { minutes: 5 },\n        { minutes: 1 },\n        { seconds: 30 },\n        { seconds: 15 },\n        { seconds: 10 },\n        { seconds: 5 },\n        { seconds: 1 },\n        { milliseconds: 500 },\n        { milliseconds: 100 },\n        { milliseconds: 10 },\n        { milliseconds: 1 }\n    ];\n    function buildTimelineDateProfile(dateProfile, dateEnv, allOptions, dateProfileGenerator) {\n        var tDateProfile = {\n            labelInterval: queryDurationOption(allOptions, 'slotLabelInterval'),\n            slotDuration: queryDurationOption(allOptions, 'slotDuration')\n        };\n        validateLabelAndSlot(tDateProfile, dateProfile, dateEnv); // validate after computed grid duration\n        ensureLabelInterval(tDateProfile, dateProfile, dateEnv);\n        ensureSlotDuration(tDateProfile, dateProfile, dateEnv);\n        var input = allOptions.slotLabelFormat;\n        var rawFormats = Array.isArray(input) ?\n            input\n            : (input != null) ?\n                [input]\n                :\n                    computeHeaderFormats(tDateProfile, dateProfile, dateEnv, allOptions);\n        tDateProfile.headerFormats = rawFormats.map(function (rawFormat) {\n            return core.createFormatter(rawFormat);\n        });\n        tDateProfile.isTimeScale = Boolean(tDateProfile.slotDuration.milliseconds);\n        var largeUnit = null;\n        if (!tDateProfile.isTimeScale) {\n            var slotUnit = core.greatestDurationDenominator(tDateProfile.slotDuration).unit;\n            if (/year|month|week/.test(slotUnit)) {\n                largeUnit = slotUnit;\n            }\n        }\n        tDateProfile.largeUnit = largeUnit;\n        tDateProfile.emphasizeWeeks =\n            core.isSingleDay(tDateProfile.slotDuration) &&\n                currentRangeAs('weeks', dateProfile, dateEnv) >= 2 &&\n                !allOptions.businessHours;\n        /*\n        console.log('label interval =', timelineView.labelInterval.humanize())\n        console.log('slot duration =', timelineView.slotDuration.humanize())\n        console.log('header formats =', timelineView.headerFormats)\n        console.log('isTimeScale', timelineView.isTimeScale)\n        console.log('largeUnit', timelineView.largeUnit)\n        */\n        var rawSnapDuration = allOptions.snapDuration;\n        var snapDuration;\n        var snapsPerSlot;\n        if (rawSnapDuration) {\n            snapDuration = core.createDuration(rawSnapDuration);\n            snapsPerSlot = core.wholeDivideDurations(tDateProfile.slotDuration, snapDuration);\n            // ^ TODO: warning if not whole?\n        }\n        if (snapsPerSlot == null) {\n            snapDuration = tDateProfile.slotDuration;\n            snapsPerSlot = 1;\n        }\n        tDateProfile.snapDuration = snapDuration;\n        tDateProfile.snapsPerSlot = snapsPerSlot;\n        // more...\n        var timeWindowMs = core.asRoughMs(dateProfile.maxTime) - core.asRoughMs(dateProfile.minTime);\n        // TODO: why not use normalizeRange!?\n        var normalizedStart = normalizeDate(dateProfile.renderRange.start, tDateProfile, dateEnv);\n        var normalizedEnd = normalizeDate(dateProfile.renderRange.end, tDateProfile, dateEnv);\n        // apply minTime/maxTime\n        // TODO: View should be responsible.\n        if (tDateProfile.isTimeScale) {\n            normalizedStart = dateEnv.add(normalizedStart, dateProfile.minTime);\n            normalizedEnd = dateEnv.add(core.addDays(normalizedEnd, -1), dateProfile.maxTime);\n        }\n        tDateProfile.timeWindowMs = timeWindowMs;\n        tDateProfile.normalizedRange = { start: normalizedStart, end: normalizedEnd };\n        var slotDates = [];\n        var date = normalizedStart;\n        while (date < normalizedEnd) {\n            if (isValidDate(date, tDateProfile, dateProfile, dateProfileGenerator)) {\n                slotDates.push(date);\n            }\n            date = dateEnv.add(date, tDateProfile.slotDuration);\n        }\n        tDateProfile.slotDates = slotDates;\n        // more...\n        var snapIndex = -1;\n        var snapDiff = 0; // index of the diff :(\n        var snapDiffToIndex = [];\n        var snapIndexToDiff = [];\n        date = normalizedStart;\n        while (date < normalizedEnd) {\n            if (isValidDate(date, tDateProfile, dateProfile, dateProfileGenerator)) {\n                snapIndex++;\n                snapDiffToIndex.push(snapIndex);\n                snapIndexToDiff.push(snapDiff);\n            }\n            else {\n                snapDiffToIndex.push(snapIndex + 0.5);\n            }\n            date = dateEnv.add(date, tDateProfile.snapDuration);\n            snapDiff++;\n        }\n        tDateProfile.snapDiffToIndex = snapDiffToIndex;\n        tDateProfile.snapIndexToDiff = snapIndexToDiff;\n        tDateProfile.snapCnt = snapIndex + 1; // is always one behind\n        tDateProfile.slotCnt = tDateProfile.snapCnt / tDateProfile.snapsPerSlot;\n        // more...\n        tDateProfile.isWeekStarts = buildIsWeekStarts(tDateProfile, dateEnv);\n        tDateProfile.cellRows = buildCellRows(tDateProfile, dateEnv, allOptions);\n        return tDateProfile;\n    }\n    /*\n    snaps to appropriate unit\n    */\n    function normalizeDate(date, tDateProfile, dateEnv) {\n        var normalDate = date;\n        if (!tDateProfile.isTimeScale) {\n            normalDate = core.startOfDay(normalDate);\n            if (tDateProfile.largeUnit) {\n                normalDate = dateEnv.startOf(normalDate, tDateProfile.largeUnit);\n            }\n        }\n        return normalDate;\n    }\n    /*\n    snaps to appropriate unit\n    */\n    function normalizeRange(range, tDateProfile, dateEnv) {\n        if (!tDateProfile.isTimeScale) {\n            range = core.computeVisibleDayRange(range);\n            if (tDateProfile.largeUnit) {\n                var dayRange = range; // preserve original result\n                range = {\n                    start: dateEnv.startOf(range.start, tDateProfile.largeUnit),\n                    end: dateEnv.startOf(range.end, tDateProfile.largeUnit)\n                };\n                // if date is partially through the interval, or is in the same interval as the start,\n                // make the exclusive end be the *next* interval\n                if (range.end.valueOf() !== dayRange.end.valueOf() || range.end <= range.start) {\n                    range = {\n                        start: range.start,\n                        end: dateEnv.add(range.end, tDateProfile.slotDuration)\n                    };\n                }\n            }\n        }\n        return range;\n    }\n    function isValidDate(date, tDateProfile, dateProfile, dateProfileGenerator) {\n        if (dateProfileGenerator.isHiddenDay(date)) {\n            return false;\n        }\n        else if (tDateProfile.isTimeScale) {\n            // determine if the time is within minTime/maxTime, which may have wacky values\n            var day = core.startOfDay(date);\n            var timeMs = date.valueOf() - day.valueOf();\n            var ms = timeMs - core.asRoughMs(dateProfile.minTime); // milliseconds since minTime\n            ms = ((ms % 86400000) + 86400000) % 86400000; // make negative values wrap to 24hr clock\n            return ms < tDateProfile.timeWindowMs; // before the maxTime?\n        }\n        else {\n            return true;\n        }\n    }\n    function queryDurationOption(allOptions, name) {\n        var input = allOptions[name];\n        if (input != null) {\n            return core.createDuration(input);\n        }\n    }\n    function validateLabelAndSlot(tDateProfile, dateProfile, dateEnv) {\n        var currentRange = dateProfile.currentRange;\n        // make sure labelInterval doesn't exceed the max number of cells\n        if (tDateProfile.labelInterval) {\n            var labelCnt = dateEnv.countDurationsBetween(currentRange.start, currentRange.end, tDateProfile.labelInterval);\n            if (labelCnt > core.config.MAX_TIMELINE_SLOTS) {\n                console.warn('slotLabelInterval results in too many cells');\n                tDateProfile.labelInterval = null;\n            }\n        }\n        // make sure slotDuration doesn't exceed the maximum number of cells\n        if (tDateProfile.slotDuration) {\n            var slotCnt = dateEnv.countDurationsBetween(currentRange.start, currentRange.end, tDateProfile.slotDuration);\n            if (slotCnt > core.config.MAX_TIMELINE_SLOTS) {\n                console.warn('slotDuration results in too many cells');\n                tDateProfile.slotDuration = null;\n            }\n        }\n        // make sure labelInterval is a multiple of slotDuration\n        if (tDateProfile.labelInterval && tDateProfile.slotDuration) {\n            var slotsPerLabel = core.wholeDivideDurations(tDateProfile.labelInterval, tDateProfile.slotDuration);\n            if (slotsPerLabel === null || slotsPerLabel < 1) {\n                console.warn('slotLabelInterval must be a multiple of slotDuration');\n                tDateProfile.slotDuration = null;\n            }\n        }\n    }\n    function ensureLabelInterval(tDateProfile, dateProfile, dateEnv) {\n        var currentRange = dateProfile.currentRange;\n        var labelInterval = tDateProfile.labelInterval;\n        if (!labelInterval) {\n            // compute based off the slot duration\n            // find the largest label interval with an acceptable slots-per-label\n            var input = void 0;\n            if (tDateProfile.slotDuration) {\n                for (var _i = 0, STOCK_SUB_DURATIONS_1 = STOCK_SUB_DURATIONS; _i < STOCK_SUB_DURATIONS_1.length; _i++) {\n                    input = STOCK_SUB_DURATIONS_1[_i];\n                    var tryLabelInterval = core.createDuration(input);\n                    var slotsPerLabel = core.wholeDivideDurations(tryLabelInterval, tDateProfile.slotDuration);\n                    if (slotsPerLabel !== null && slotsPerLabel <= MAX_AUTO_SLOTS_PER_LABEL) {\n                        labelInterval = tryLabelInterval;\n                        break;\n                    }\n                }\n                // use the slot duration as a last resort\n                if (!labelInterval) {\n                    labelInterval = tDateProfile.slotDuration;\n                }\n                // compute based off the view's duration\n                // find the largest label interval that yields the minimum number of labels\n            }\n            else {\n                for (var _a = 0, STOCK_SUB_DURATIONS_2 = STOCK_SUB_DURATIONS; _a < STOCK_SUB_DURATIONS_2.length; _a++) {\n                    input = STOCK_SUB_DURATIONS_2[_a];\n                    labelInterval = core.createDuration(input);\n                    var labelCnt = dateEnv.countDurationsBetween(currentRange.start, currentRange.end, labelInterval);\n                    if (labelCnt >= MIN_AUTO_LABELS) {\n                        break;\n                    }\n                }\n            }\n            tDateProfile.labelInterval = labelInterval;\n        }\n        return labelInterval;\n    }\n    function ensureSlotDuration(tDateProfile, dateProfile, dateEnv) {\n        var currentRange = dateProfile.currentRange;\n        var slotDuration = tDateProfile.slotDuration;\n        if (!slotDuration) {\n            var labelInterval = ensureLabelInterval(tDateProfile, dateProfile, dateEnv); // will compute if necessary\n            // compute based off the label interval\n            // find the largest slot duration that is different from labelInterval, but still acceptable\n            for (var _i = 0, STOCK_SUB_DURATIONS_3 = STOCK_SUB_DURATIONS; _i < STOCK_SUB_DURATIONS_3.length; _i++) {\n                var input = STOCK_SUB_DURATIONS_3[_i];\n                var trySlotDuration = core.createDuration(input);\n                var slotsPerLabel = core.wholeDivideDurations(labelInterval, trySlotDuration);\n                if (slotsPerLabel !== null && slotsPerLabel > 1 && slotsPerLabel <= MAX_AUTO_SLOTS_PER_LABEL) {\n                    slotDuration = trySlotDuration;\n                    break;\n                }\n            }\n            // only allow the value if it won't exceed the view's # of slots limit\n            if (slotDuration) {\n                var slotCnt = dateEnv.countDurationsBetween(currentRange.start, currentRange.end, slotDuration);\n                if (slotCnt > MAX_AUTO_CELLS) {\n                    slotDuration = null;\n                }\n            }\n            // use the label interval as a last resort\n            if (!slotDuration) {\n                slotDuration = labelInterval;\n            }\n            tDateProfile.slotDuration = slotDuration;\n        }\n        return slotDuration;\n    }\n    function computeHeaderFormats(tDateProfile, dateProfile, dateEnv, allOptions) {\n        var format1;\n        var format2;\n        var labelInterval = tDateProfile.labelInterval;\n        var unit = core.greatestDurationDenominator(labelInterval).unit;\n        var weekNumbersVisible = allOptions.weekNumbers;\n        var format0 = (format1 = (format2 = null));\n        // NOTE: weekNumber computation function wont work\n        if ((unit === 'week') && !weekNumbersVisible) {\n            unit = 'day';\n        }\n        switch (unit) {\n            case 'year':\n                format0 = { year: 'numeric' }; // '2015'\n                break;\n            case 'month':\n                if (currentRangeAs('years', dateProfile, dateEnv) > 1) {\n                    format0 = { year: 'numeric' }; // '2015'\n                }\n                format1 = { month: 'short' }; // 'Jan'\n                break;\n            case 'week':\n                if (currentRangeAs('years', dateProfile, dateEnv) > 1) {\n                    format0 = { year: 'numeric' }; // '2015'\n                }\n                format1 = { week: 'narrow' }; // 'Wk4'\n                break;\n            case 'day':\n                if (currentRangeAs('years', dateProfile, dateEnv) > 1) {\n                    format0 = { year: 'numeric', month: 'long' }; // 'January 2014'\n                }\n                else if (currentRangeAs('months', dateProfile, dateEnv) > 1) {\n                    format0 = { month: 'long' }; // 'January'\n                }\n                if (weekNumbersVisible) {\n                    format1 = { week: 'short' }; // 'Wk 4'\n                }\n                format2 = { weekday: 'narrow', day: 'numeric' }; // 'Su 9'\n                break;\n            case 'hour':\n                if (weekNumbersVisible) {\n                    format0 = { week: 'short' }; // 'Wk 4'\n                }\n                if (currentRangeAs('days', dateProfile, dateEnv) > 1) {\n                    format1 = { weekday: 'short', day: 'numeric', month: 'numeric', omitCommas: true }; // Sat 4/7\n                }\n                format2 = {\n                    hour: 'numeric',\n                    minute: '2-digit',\n                    omitZeroMinute: true,\n                    meridiem: 'short'\n                };\n                break;\n            case 'minute':\n                // sufficiently large number of different minute cells?\n                if ((core.asRoughMinutes(labelInterval) / 60) >= MAX_AUTO_SLOTS_PER_LABEL) {\n                    format0 = {\n                        hour: 'numeric',\n                        meridiem: 'short'\n                    };\n                    format1 = function (params) {\n                        return ':' + core.padStart(params.date.minute, 2); // ':30'\n                    };\n                }\n                else {\n                    format0 = {\n                        hour: 'numeric',\n                        minute: 'numeric',\n                        meridiem: 'short'\n                    };\n                }\n                break;\n            case 'second':\n                // sufficiently large number of different second cells?\n                if ((core.asRoughSeconds(labelInterval) / 60) >= MAX_AUTO_SLOTS_PER_LABEL) {\n                    format0 = { hour: 'numeric', minute: '2-digit', meridiem: 'lowercase' }; // '8:30 PM'\n                    format1 = function (params) {\n                        return ':' + core.padStart(params.date.second, 2); // ':30'\n                    };\n                }\n                else {\n                    format0 = { hour: 'numeric', minute: '2-digit', second: '2-digit', meridiem: 'lowercase' }; // '8:30:45 PM'\n                }\n                break;\n            case 'millisecond':\n                format0 = { hour: 'numeric', minute: '2-digit', second: '2-digit', meridiem: 'lowercase' }; // '8:30:45 PM'\n                format1 = function (params) {\n                    return '.' + core.padStart(params.millisecond, 3);\n                };\n                break;\n        }\n        return [].concat(format0 || [], format1 || [], format2 || []);\n    }\n    // Compute the number of the give units in the \"current\" range.\n    // Won't go more precise than days.\n    // Will return `0` if there's not a clean whole interval.\n    function currentRangeAs(unit, dateProfile, dateEnv) {\n        var range = dateProfile.currentRange;\n        var res = null;\n        if (unit === 'years') {\n            res = dateEnv.diffWholeYears(range.start, range.end);\n        }\n        else if (unit === 'months') {\n            res = dateEnv.diffWholeMonths(range.start, range.end);\n        }\n        else if (unit === 'weeks') {\n            res = dateEnv.diffWholeMonths(range.start, range.end);\n        }\n        else if (unit === 'days') {\n            res = core.diffWholeDays(range.start, range.end);\n        }\n        return res || 0;\n    }\n    function buildIsWeekStarts(tDateProfile, dateEnv) {\n        var slotDates = tDateProfile.slotDates, emphasizeWeeks = tDateProfile.emphasizeWeeks;\n        var prevWeekNumber = null;\n        var isWeekStarts = [];\n        for (var _i = 0, slotDates_1 = slotDates; _i < slotDates_1.length; _i++) {\n            var slotDate = slotDates_1[_i];\n            var weekNumber = dateEnv.computeWeekNumber(slotDate);\n            var isWeekStart = emphasizeWeeks && (prevWeekNumber !== null) && (prevWeekNumber !== weekNumber);\n            prevWeekNumber = weekNumber;\n            isWeekStarts.push(isWeekStart);\n        }\n        return isWeekStarts;\n    }\n    function buildCellRows(tDateProfile, dateEnv, allOptions) {\n        var slotDates = tDateProfile.slotDates;\n        var formats = tDateProfile.headerFormats;\n        var cellRows = formats.map(function (format) { return []; }); // indexed by row,col\n        // specifically for navclicks\n        var rowUnits = formats.map(function (format) {\n            return format.getLargestUnit ? format.getLargestUnit() : null;\n        });\n        // builds cellRows and slotCells\n        for (var i = 0; i < slotDates.length; i++) {\n            var date = slotDates[i];\n            var isWeekStart = tDateProfile.isWeekStarts[i];\n            for (var row = 0; row < formats.length; row++) {\n                var format = formats[row];\n                var rowCells = cellRows[row];\n                var leadingCell = rowCells[rowCells.length - 1];\n                var isSuperRow = (formats.length > 1) && (row < (formats.length - 1)); // more than one row and not the last\n                var newCell = null;\n                if (isSuperRow) {\n                    var text = dateEnv.format(date, format);\n                    if (!leadingCell || (leadingCell.text !== text)) {\n                        newCell = buildCellObject(date, text, rowUnits[row], allOptions, dateEnv);\n                    }\n                    else {\n                        leadingCell.colspan += 1;\n                    }\n                }\n                else {\n                    if (!leadingCell ||\n                        core.isInt(dateEnv.countDurationsBetween(tDateProfile.normalizedRange.start, date, tDateProfile.labelInterval))) {\n                        var text = dateEnv.format(date, format);\n                        newCell = buildCellObject(date, text, rowUnits[row], allOptions, dateEnv);\n                    }\n                    else {\n                        leadingCell.colspan += 1;\n                    }\n                }\n                if (newCell) {\n                    newCell.weekStart = isWeekStart;\n                    rowCells.push(newCell);\n                }\n            }\n        }\n        return cellRows;\n    }\n    function buildCellObject(date, text, rowUnit, allOptions, dateEnv) {\n        var spanHtml = core.buildGotoAnchorHtml(allOptions, dateEnv, {\n            date: date,\n            type: rowUnit,\n            forceOff: !rowUnit\n        }, {\n            'class': 'fc-cell-text'\n        }, core.htmlEscape(text));\n        return { text: text, spanHtml: spanHtml, date: date, colspan: 1, isWeekStart: false };\n    }\n\n    var TimelineNowIndicator = /** @class */ (function () {\n        function TimelineNowIndicator(headParent, bodyParent) {\n            this.headParent = headParent;\n            this.bodyParent = bodyParent;\n        }\n        TimelineNowIndicator.prototype.render = function (coord, isRtl) {\n            var styleProps = isRtl ? { right: -coord } : { left: coord };\n            this.headParent.appendChild(this.arrowEl = core.createElement('div', {\n                className: 'fc-now-indicator fc-now-indicator-arrow',\n                style: styleProps\n            }));\n            this.bodyParent.appendChild(this.lineEl = core.createElement('div', {\n                className: 'fc-now-indicator fc-now-indicator-line',\n                style: styleProps\n            }));\n        };\n        TimelineNowIndicator.prototype.unrender = function () {\n            if (this.arrowEl) {\n                core.removeElement(this.arrowEl);\n            }\n            if (this.lineEl) {\n                core.removeElement(this.lineEl);\n            }\n        };\n        return TimelineNowIndicator;\n    }());\n\n    var STICKY_PROP_VAL = computeStickyPropVal(); // if null, means not supported at all\n    var IS_MS_EDGE = /Edge/.test(navigator.userAgent);\n    var IS_SAFARI = STICKY_PROP_VAL === '-webkit-sticky'; // good b/c doesn't confuse chrome\n    var STICKY_CLASSNAME = 'fc-sticky';\n    /*\n    useful beyond the native position:sticky for these reasons:\n    - support in IE11\n    - nice centering support\n    */\n    var StickyScroller = /** @class */ (function () {\n        function StickyScroller(scroller, isRtl, isVertical) {\n            var _this = this;\n            this.usingRelative = null;\n            /*\n            known bug: called twice on init. problem when mixing with ScrollJoiner\n            */\n            this.updateSize = function () {\n                var els = Array.prototype.slice.call(_this.scroller.canvas.el.querySelectorAll('.' + STICKY_CLASSNAME));\n                var elGeoms = _this.queryElGeoms(els);\n                var viewportWidth = _this.scroller.el.clientWidth;\n                if (_this.usingRelative) {\n                    var elDestinations = _this.computeElDestinations(elGeoms, viewportWidth); // read before prepPositioning\n                    assignRelativePositions(els, elGeoms, elDestinations);\n                }\n                else {\n                    assignStickyPositions(els, elGeoms, viewportWidth);\n                }\n            };\n            this.scroller = scroller;\n            this.usingRelative =\n                !STICKY_PROP_VAL || // IE11\n                    (IS_MS_EDGE && isRtl) || // https://developer.microsoft.com/en-us/microsoft-edge/platform/issues/18883305/\n                    ((IS_MS_EDGE || IS_SAFARI) && isVertical); // because doesn't work with rowspan in tables, our only vertial use\n            if (this.usingRelative) {\n                scroller.on('scrollEnd', this.updateSize);\n            }\n        }\n        StickyScroller.prototype.destroy = function () {\n            this.scroller.off('scrollEnd', this.updateSize);\n        };\n        StickyScroller.prototype.queryElGeoms = function (els) {\n            var canvasOrigin = this.scroller.canvas.el.getBoundingClientRect();\n            var elGeoms = [];\n            for (var _i = 0, els_1 = els; _i < els_1.length; _i++) {\n                var el = els_1[_i];\n                var parentBound = core.translateRect(el.parentNode.getBoundingClientRect(), -canvasOrigin.left, -canvasOrigin.top);\n                var elRect = el.getBoundingClientRect();\n                var computedStyles = window.getComputedStyle(el);\n                var computedTextAlign = window.getComputedStyle(el.parentNode).textAlign; // ask the parent\n                var intendedTextAlign = computedTextAlign;\n                var naturalBound = null;\n                if (computedStyles.position !== 'sticky') {\n                    naturalBound = core.translateRect(elRect, -canvasOrigin.left - (parseFloat(computedStyles.left) || 0), // could be 'auto'\n                    -canvasOrigin.top - (parseFloat(computedStyles.top) || 0));\n                }\n                if (el.hasAttribute('data-sticky-center')) {\n                    intendedTextAlign = 'center';\n                }\n                elGeoms.push({\n                    parentBound: parentBound,\n                    naturalBound: naturalBound,\n                    elWidth: elRect.width,\n                    elHeight: elRect.height,\n                    computedTextAlign: computedTextAlign,\n                    intendedTextAlign: intendedTextAlign\n                });\n            }\n            return elGeoms;\n        };\n        StickyScroller.prototype.computeElDestinations = function (elGeoms, viewportWidth) {\n            var viewportLeft = this.scroller.getScrollFromLeft();\n            var viewportTop = this.scroller.getScrollTop();\n            var viewportRight = viewportLeft + viewportWidth;\n            return elGeoms.map(function (elGeom) {\n                var elWidth = elGeom.elWidth, elHeight = elGeom.elHeight, parentBound = elGeom.parentBound, naturalBound = elGeom.naturalBound;\n                var destLeft; // relative to canvas topleft\n                var destTop; // \"\n                switch (elGeom.intendedTextAlign) {\n                    case 'left':\n                        destLeft = viewportLeft;\n                        break;\n                    case 'right':\n                        destLeft = viewportRight - elWidth;\n                        break;\n                    case 'center':\n                        destLeft = (viewportLeft + viewportRight) / 2 - elWidth / 2;\n                        break;\n                }\n                destLeft = Math.min(destLeft, parentBound.right - elWidth);\n                destLeft = Math.max(destLeft, parentBound.left);\n                destTop = viewportTop;\n                destTop = Math.min(destTop, parentBound.bottom - elHeight);\n                destTop = Math.max(destTop, naturalBound.top); // better to use natural top for upper bound\n                return { left: destLeft, top: destTop };\n            });\n        };\n        return StickyScroller;\n    }());\n    function assignRelativePositions(els, elGeoms, elDestinations) {\n        els.forEach(function (el, i) {\n            var naturalBound = elGeoms[i].naturalBound;\n            core.applyStyle(el, {\n                position: 'relative',\n                left: elDestinations[i].left - naturalBound.left,\n                top: elDestinations[i].top - naturalBound.top\n            });\n        });\n    }\n    function assignStickyPositions(els, elGeoms, viewportWidth) {\n        els.forEach(function (el, i) {\n            var stickyLeft = 0;\n            if (elGeoms[i].intendedTextAlign === 'center') {\n                stickyLeft = (viewportWidth - elGeoms[i].elWidth) / 2;\n                // needs to be forced to left?\n                if (elGeoms[i].computedTextAlign === 'center') {\n                    el.setAttribute('data-sticky-center', '') // remember for next queryElGeoms\n                    ;\n                    el.parentNode.style.textAlign = 'left';\n                }\n            }\n            core.applyStyle(el, {\n                position: STICKY_PROP_VAL,\n                left: stickyLeft,\n                right: 0,\n                top: 0\n            });\n        });\n    }\n    function computeStickyPropVal() {\n        var el = core.htmlToElement('<div style=\"position:-webkit-sticky;position:sticky\"></div>');\n        var val = el.style.position;\n        if (val.indexOf('sticky') !== -1) {\n            return val;\n        }\n        else {\n            return null;\n        }\n    }\n\n    var TimeAxis = /** @class */ (function (_super) {\n        __extends(TimeAxis, _super);\n        function TimeAxis(headerContainerEl, bodyContainerEl) {\n            var _this = _super.call(this) || this;\n            _this.renderSkeleton = core.memoizeRendering(_this._renderSkeleton, _this._unrenderSkeleton);\n            _this.layout = new HeaderBodyLayout(headerContainerEl, bodyContainerEl, 'auto');\n            return _this;\n        }\n        TimeAxis.prototype.render = function (props, context) {\n            var tDateProfile = this.tDateProfile =\n                buildTimelineDateProfile(props.dateProfile, context.dateEnv, context.options, props.dateProfileGenerator); // TODO: cache\n            this.renderSkeleton(context);\n            this.header.receiveProps({\n                dateProfile: props.dateProfile,\n                tDateProfile: tDateProfile\n            }, context);\n            this.slats.receiveProps({\n                dateProfile: props.dateProfile,\n                tDateProfile: tDateProfile\n            }, context);\n        };\n        TimeAxis.prototype.destroy = function () {\n            this.renderSkeleton.unrender();\n            this.layout.destroy();\n            _super.prototype.destroy.call(this);\n        };\n        TimeAxis.prototype._renderSkeleton = function (context) {\n            var layout = this.layout;\n            var headerEnhancedScroller = layout.headerScroller.enhancedScroll;\n            var bodyEnhancedScroller = layout.bodyScroller.enhancedScroll;\n            // needs to go after layout, which has ScrollJoiner\n            this.headStickyScroller = new StickyScroller(headerEnhancedScroller, context.isRtl, false); // isVertical=false\n            this.bodyStickyScroller = new StickyScroller(bodyEnhancedScroller, context.isRtl, false); // isVertical=false\n            this.header = new TimelineHeader(headerEnhancedScroller.canvas.contentEl);\n            this.slats = new TimelineSlats(bodyEnhancedScroller.canvas.bgEl);\n            this.nowIndicator = new TimelineNowIndicator(headerEnhancedScroller.canvas.el, bodyEnhancedScroller.canvas.el);\n        };\n        TimeAxis.prototype._unrenderSkeleton = function () {\n            this.header.destroy();\n            this.slats.destroy();\n            this.nowIndicator.unrender();\n            this.headStickyScroller.destroy();\n            this.bodyStickyScroller.destroy();\n        };\n        // Now Indicator\n        // ------------------------------------------------------------------------------------------\n        TimeAxis.prototype.getNowIndicatorUnit = function (dateProfile, dateProfileGenerator) {\n            var context = this.context;\n            // yuck\n            var tDateProfile = this.tDateProfile =\n                buildTimelineDateProfile(dateProfile, context.dateEnv, context.options, dateProfileGenerator); // TODO: cache\n            if (tDateProfile.isTimeScale) {\n                return core.greatestDurationDenominator(tDateProfile.slotDuration).unit;\n            }\n        };\n        // will only execute if isTimeScale\n        TimeAxis.prototype.renderNowIndicator = function (date) {\n            if (core.rangeContainsMarker(this.tDateProfile.normalizedRange, date)) {\n                this.nowIndicator.render(this.dateToCoord(date), this.context.isRtl);\n            }\n        };\n        // will only execute if isTimeScale\n        TimeAxis.prototype.unrenderNowIndicator = function () {\n            this.nowIndicator.unrender();\n        };\n        // Sizing\n        // ------------------------------------------------------------------------------------------\n        TimeAxis.prototype.updateSize = function (isResize, totalHeight, isAuto) {\n            this.applySlotWidth(this.computeSlotWidth());\n            // adjusts gutters. do after slot widths set\n            this.layout.setHeight(totalHeight, isAuto);\n            // pretty much just queries coords. do last\n            this.slats.updateSize();\n        };\n        TimeAxis.prototype.updateStickyScrollers = function () {\n            this.headStickyScroller.updateSize();\n            this.bodyStickyScroller.updateSize();\n        };\n        TimeAxis.prototype.computeSlotWidth = function () {\n            var slotWidth = this.context.options.slotWidth || '';\n            if (slotWidth === '') {\n                slotWidth = this.computeDefaultSlotWidth(this.tDateProfile);\n            }\n            return slotWidth;\n        };\n        TimeAxis.prototype.computeDefaultSlotWidth = function (tDateProfile) {\n            var maxInnerWidth = 0; // TODO: harness core's `matchCellWidths` for this\n            this.header.innerEls.forEach(function (innerEl, i) {\n                maxInnerWidth = Math.max(maxInnerWidth, innerEl.getBoundingClientRect().width);\n            });\n            var headingCellWidth = Math.ceil(maxInnerWidth) + 1; // assume no padding, and one pixel border\n            // in TimelineView.defaults we ensured that labelInterval is an interval of slotDuration\n            // TODO: rename labelDuration?\n            var slotsPerLabel = core.wholeDivideDurations(tDateProfile.labelInterval, tDateProfile.slotDuration);\n            var slotWidth = Math.ceil(headingCellWidth / slotsPerLabel);\n            var minWidth = window.getComputedStyle(this.header.slatColEls[0]).minWidth;\n            if (minWidth) {\n                minWidth = parseInt(minWidth, 10);\n                if (minWidth) {\n                    slotWidth = Math.max(slotWidth, minWidth);\n                }\n            }\n            return slotWidth;\n        };\n        TimeAxis.prototype.applySlotWidth = function (slotWidth) {\n            var _a = this, layout = _a.layout, tDateProfile = _a.tDateProfile;\n            var containerWidth = '';\n            var containerMinWidth = '';\n            var nonLastSlotWidth = '';\n            if (slotWidth !== '') {\n                slotWidth = Math.round(slotWidth);\n                containerWidth = slotWidth * tDateProfile.slotDates.length;\n                containerMinWidth = '';\n                nonLastSlotWidth = slotWidth;\n                var availableWidth = layout.bodyScroller.enhancedScroll.getClientWidth();\n                if (availableWidth > containerWidth) {\n                    containerMinWidth = availableWidth;\n                    containerWidth = '';\n                    nonLastSlotWidth = Math.floor(availableWidth / tDateProfile.slotDates.length);\n                }\n            }\n            layout.headerScroller.enhancedScroll.canvas.setWidth(containerWidth);\n            layout.headerScroller.enhancedScroll.canvas.setMinWidth(containerMinWidth);\n            layout.bodyScroller.enhancedScroll.canvas.setWidth(containerWidth);\n            layout.bodyScroller.enhancedScroll.canvas.setMinWidth(containerMinWidth);\n            if (nonLastSlotWidth !== '') {\n                this.header.slatColEls.slice(0, -1).concat(this.slats.slatColEls.slice(0, -1)).forEach(function (el) {\n                    el.style.width = nonLastSlotWidth + 'px';\n                    el.style.minWidth = '0'; // prevent the browser from using the actual min-width (issue #6338)\n                });\n            }\n        };\n        // returned value is between 0 and the number of snaps\n        TimeAxis.prototype.computeDateSnapCoverage = function (date) {\n            var tDateProfile = this.tDateProfile;\n            var dateEnv = this.context.dateEnv;\n            var snapDiff = dateEnv.countDurationsBetween(tDateProfile.normalizedRange.start, date, tDateProfile.snapDuration);\n            if (snapDiff < 0) {\n                return 0;\n            }\n            else if (snapDiff >= tDateProfile.snapDiffToIndex.length) {\n                return tDateProfile.snapCnt;\n            }\n            else {\n                var snapDiffInt = Math.floor(snapDiff);\n                var snapCoverage = tDateProfile.snapDiffToIndex[snapDiffInt];\n                if (core.isInt(snapCoverage)) { // not an in-between value\n                    snapCoverage += snapDiff - snapDiffInt; // add the remainder\n                }\n                else {\n                    // a fractional value, meaning the date is not visible\n                    // always round up in this case. works for start AND end dates in a range.\n                    snapCoverage = Math.ceil(snapCoverage);\n                }\n                return snapCoverage;\n            }\n        };\n        // for LTR, results range from 0 to width of area\n        // for RTL, results range from negative width of area to 0\n        TimeAxis.prototype.dateToCoord = function (date) {\n            var tDateProfile = this.tDateProfile;\n            var snapCoverage = this.computeDateSnapCoverage(date);\n            var slotCoverage = snapCoverage / tDateProfile.snapsPerSlot;\n            var slotIndex = Math.floor(slotCoverage);\n            slotIndex = Math.min(slotIndex, tDateProfile.slotCnt - 1);\n            var partial = slotCoverage - slotIndex;\n            var _a = this.slats, innerCoordCache = _a.innerCoordCache, outerCoordCache = _a.outerCoordCache;\n            if (this.context.isRtl) {\n                return (outerCoordCache.rights[slotIndex] -\n                    (innerCoordCache.getWidth(slotIndex) * partial)) - outerCoordCache.originClientRect.width;\n            }\n            else {\n                return (outerCoordCache.lefts[slotIndex] +\n                    (innerCoordCache.getWidth(slotIndex) * partial));\n            }\n        };\n        TimeAxis.prototype.rangeToCoords = function (range) {\n            if (this.context.isRtl) {\n                return { right: this.dateToCoord(range.start), left: this.dateToCoord(range.end) };\n            }\n            else {\n                return { left: this.dateToCoord(range.start), right: this.dateToCoord(range.end) };\n            }\n        };\n        // Scrolling\n        // ------------------------------------------------------------------------------------------\n        TimeAxis.prototype.computeDateScroll = function (duration) {\n            var _a = this.context, dateEnv = _a.dateEnv, isRtl = _a.isRtl;\n            var dateProfile = this.props.dateProfile;\n            var left = 0;\n            if (dateProfile) {\n                left = this.dateToCoord(dateEnv.add(core.startOfDay(dateProfile.activeRange.start), // startOfDay needed?\n                duration));\n                // hack to overcome the left borders of non-first slat\n                if (!isRtl && left) {\n                    left += 1;\n                }\n            }\n            return { left: left };\n        };\n        TimeAxis.prototype.queryDateScroll = function () {\n            var enhancedScroll = this.layout.bodyScroller.enhancedScroll;\n            return {\n                left: enhancedScroll.getScrollLeft()\n            };\n        };\n        TimeAxis.prototype.applyDateScroll = function (scroll) {\n            // TODO: lame we have to update both. use the scrolljoiner instead maybe\n            this.layout.bodyScroller.enhancedScroll.setScrollLeft(scroll.left || 0);\n            this.layout.headerScroller.enhancedScroll.setScrollLeft(scroll.left || 0);\n        };\n        return TimeAxis;\n    }(core.Component));\n\n    var TimelineLaneEventRenderer = /** @class */ (function (_super) {\n        __extends(TimelineLaneEventRenderer, _super);\n        function TimelineLaneEventRenderer(masterContainerEl, timeAxis) {\n            var _this = _super.call(this) || this;\n            _this.masterContainerEl = masterContainerEl;\n            _this.timeAxis = timeAxis;\n            return _this;\n        }\n        TimelineLaneEventRenderer.prototype.renderSegHtml = function (seg, mirrorInfo) {\n            var context = this.context;\n            var eventRange = seg.eventRange;\n            var eventDef = eventRange.def;\n            var eventUi = eventRange.ui;\n            var isDraggable = core.computeEventDraggable(context, eventDef, eventUi);\n            var isResizableFromStart = seg.isStart && core.computeEventStartResizable(context, eventDef, eventUi);\n            var isResizableFromEnd = seg.isEnd && core.computeEventEndResizable(context, eventDef, eventUi);\n            var classes = this.getSegClasses(seg, isDraggable, isResizableFromStart || isResizableFromEnd, mirrorInfo);\n            classes.unshift('fc-timeline-event', 'fc-h-event');\n            var timeText = this.getTimeText(eventRange);\n            return '<a class=\"' + classes.join(' ') + '\" style=\"' + core.cssToStr(this.getSkinCss(eventUi)) + '\"' +\n                (eventDef.url ?\n                    ' href=\"' + core.htmlEscape(eventDef.url) + '\"' :\n                    '') +\n                '>' +\n                (timeText ?\n                    '<span class=\"fc-time-wrap\">' +\n                        '<span class=\"fc-time\">' +\n                        core.htmlEscape(timeText) +\n                        '</span>' +\n                        '</span>'\n                    :\n                        '') +\n                '<span class=\"fc-title-wrap\">' +\n                '<span class=\"fc-title fc-sticky\">' +\n                (eventDef.title ? core.htmlEscape(eventDef.title) : '&nbsp;') +\n                '</span>' +\n                '</span>' +\n                (isResizableFromStart ?\n                    '<div class=\"fc-resizer fc-start-resizer\"></div>' :\n                    '') +\n                (isResizableFromEnd ?\n                    '<div class=\"fc-resizer fc-end-resizer\"></div>' :\n                    '') +\n                '</a>';\n        };\n        TimelineLaneEventRenderer.prototype.computeDisplayEventTime = function () {\n            return !this.timeAxis.tDateProfile.isTimeScale; // because times should be obvious via axis\n        };\n        TimelineLaneEventRenderer.prototype.computeDisplayEventEnd = function () {\n            return false;\n        };\n        // Computes a default event time formatting string if `timeFormat` is not explicitly defined\n        TimelineLaneEventRenderer.prototype.computeEventTimeFormat = function () {\n            return {\n                hour: 'numeric',\n                minute: '2-digit',\n                omitZeroMinute: true,\n                meridiem: 'narrow'\n            };\n        };\n        TimelineLaneEventRenderer.prototype.attachSegs = function (segs, mirrorInfo) {\n            if (!this.el && this.masterContainerEl) {\n                this.el = core.createElement('div', { className: 'fc-event-container' });\n                if (mirrorInfo) {\n                    this.el.classList.add('fc-mirror-container');\n                }\n                this.masterContainerEl.appendChild(this.el);\n            }\n            if (this.el) {\n                for (var _i = 0, segs_1 = segs; _i < segs_1.length; _i++) {\n                    var seg = segs_1[_i];\n                    this.el.appendChild(seg.el);\n                }\n            }\n        };\n        TimelineLaneEventRenderer.prototype.detachSegs = function (segs) {\n            for (var _i = 0, segs_2 = segs; _i < segs_2.length; _i++) {\n                var seg = segs_2[_i];\n                core.removeElement(seg.el);\n            }\n        };\n        // computes AND assigns (assigns the left/right at least). bad\n        TimelineLaneEventRenderer.prototype.computeSegSizes = function (segs) {\n            var timeAxis = this.timeAxis;\n            for (var _i = 0, segs_3 = segs; _i < segs_3.length; _i++) {\n                var seg = segs_3[_i];\n                var coords = timeAxis.rangeToCoords(seg); // works because Seg has start/end\n                core.applyStyle(seg.el, {\n                    left: (seg.left = coords.left),\n                    right: -(seg.right = coords.right)\n                });\n            }\n        };\n        TimelineLaneEventRenderer.prototype.assignSegSizes = function (segs) {\n            if (!this.el) {\n                return;\n            }\n            // compute seg verticals\n            for (var _i = 0, segs_4 = segs; _i < segs_4.length; _i++) {\n                var seg = segs_4[_i];\n                seg.height = core.computeHeightAndMargins(seg.el);\n            }\n            this.buildSegLevels(segs); // populates above/below props for computeOffsetForSegs\n            var totalHeight = computeOffsetForSegs(segs); // also assigns seg.top\n            core.applyStyleProp(this.el, 'height', totalHeight);\n            // assign seg verticals\n            for (var _a = 0, segs_5 = segs; _a < segs_5.length; _a++) {\n                var seg = segs_5[_a];\n                core.applyStyleProp(seg.el, 'top', seg.top);\n            }\n        };\n        TimelineLaneEventRenderer.prototype.buildSegLevels = function (segs) {\n            var segLevels = [];\n            segs = this.sortEventSegs(segs);\n            for (var _i = 0, segs_6 = segs; _i < segs_6.length; _i++) {\n                var unplacedSeg = segs_6[_i];\n                unplacedSeg.above = [];\n                // determine the first level with no collisions\n                var level = 0; // level index\n                while (level < segLevels.length) {\n                    var isLevelCollision = false;\n                    // determine collisions\n                    for (var _a = 0, _b = segLevels[level]; _a < _b.length; _a++) {\n                        var placedSeg = _b[_a];\n                        if (timeRowSegsCollide(unplacedSeg, placedSeg)) {\n                            unplacedSeg.above.push(placedSeg);\n                            isLevelCollision = true;\n                        }\n                    }\n                    if (isLevelCollision) {\n                        level += 1;\n                    }\n                    else {\n                        break;\n                    }\n                }\n                // insert into the first non-colliding level. create if necessary\n                (segLevels[level] || (segLevels[level] = []))\n                    .push(unplacedSeg);\n                // record possible colliding segments below (TODO: automated test for this)\n                level += 1;\n                while (level < segLevels.length) {\n                    for (var _c = 0, _d = segLevels[level]; _c < _d.length; _c++) {\n                        var belowSeg = _d[_c];\n                        if (timeRowSegsCollide(unplacedSeg, belowSeg)) {\n                            belowSeg.above.push(unplacedSeg);\n                        }\n                    }\n                    level += 1;\n                }\n            }\n            return segLevels;\n        };\n        return TimelineLaneEventRenderer;\n    }(core.FgEventRenderer));\n    function computeOffsetForSegs(segs) {\n        var max = 0;\n        for (var _i = 0, segs_7 = segs; _i < segs_7.length; _i++) {\n            var seg = segs_7[_i];\n            max = Math.max(max, computeOffsetForSeg(seg));\n        }\n        return max;\n    }\n    function computeOffsetForSeg(seg) {\n        if ((seg.top == null)) {\n            seg.top = computeOffsetForSegs(seg.above);\n        }\n        return seg.top + seg.height;\n    }\n    function timeRowSegsCollide(seg0, seg1) {\n        return (seg0.left < seg1.right) && (seg0.right > seg1.left);\n    }\n\n    var TimelineLaneFillRenderer = /** @class */ (function (_super) {\n        __extends(TimelineLaneFillRenderer, _super);\n        function TimelineLaneFillRenderer(masterContainerEl, timeAxis) {\n            var _this = _super.call(this) || this;\n            _this.masterContainerEl = masterContainerEl;\n            _this.timeAxis = timeAxis;\n            return _this;\n        }\n        TimelineLaneFillRenderer.prototype.attachSegs = function (type, segs) {\n            if (segs.length) {\n                var className = void 0;\n                if (type === 'businessHours') {\n                    className = 'bgevent';\n                }\n                else {\n                    className = type.toLowerCase();\n                }\n                // making a new container each time is OKAY\n                // all types of segs (background or business hours or whatever) are rendered in one pass\n                var containerEl = core.createElement('div', { className: 'fc-' + className + '-container' });\n                this.masterContainerEl.appendChild(containerEl);\n                for (var _i = 0, segs_1 = segs; _i < segs_1.length; _i++) {\n                    var seg = segs_1[_i];\n                    containerEl.appendChild(seg.el);\n                }\n                return [containerEl]; // return value\n            }\n        };\n        TimelineLaneFillRenderer.prototype.computeSegSizes = function (segs) {\n            var timeAxis = this.timeAxis;\n            for (var _i = 0, segs_2 = segs; _i < segs_2.length; _i++) {\n                var seg = segs_2[_i];\n                var coords = timeAxis.rangeToCoords(seg);\n                seg.left = coords.left;\n                seg.right = coords.right;\n            }\n        };\n        TimelineLaneFillRenderer.prototype.assignSegSizes = function (segs) {\n            for (var _i = 0, segs_3 = segs; _i < segs_3.length; _i++) {\n                var seg = segs_3[_i];\n                core.applyStyle(seg.el, {\n                    left: seg.left,\n                    right: -seg.right\n                });\n            }\n        };\n        return TimelineLaneFillRenderer;\n    }(core.FillRenderer));\n\n    var TimelineLane = /** @class */ (function (_super) {\n        __extends(TimelineLane, _super);\n        function TimelineLane(fgContainerEl, bgContainerEl, timeAxis) {\n            var _this = _super.call(this, bgContainerEl) || this;\n            _this.slicer = new TimelineLaneSlicer();\n            _this.renderEventDrag = core.memoizeRendering(_this._renderEventDrag, _this._unrenderEventDrag);\n            _this.renderEventResize = core.memoizeRendering(_this._renderEventResize, _this._unrenderEventResize);\n            _this.fgContainerEl = fgContainerEl;\n            _this.timeAxis = timeAxis;\n            var fillRenderer = _this.fillRenderer = new TimelineLaneFillRenderer(bgContainerEl, timeAxis);\n            var eventRenderer = _this.eventRenderer = new TimelineLaneEventRenderer(fgContainerEl, timeAxis);\n            _this.mirrorRenderer = new TimelineLaneEventRenderer(fgContainerEl, timeAxis);\n            _this.renderBusinessHours = core.memoizeRendering(fillRenderer.renderSegs.bind(fillRenderer, 'businessHours'), fillRenderer.unrender.bind(fillRenderer, 'businessHours'));\n            _this.renderDateSelection = core.memoizeRendering(fillRenderer.renderSegs.bind(fillRenderer, 'highlight'), fillRenderer.unrender.bind(fillRenderer, 'highlight'));\n            _this.renderBgEvents = core.memoizeRendering(fillRenderer.renderSegs.bind(fillRenderer, 'bgEvent'), fillRenderer.unrender.bind(fillRenderer, 'bgEvent'));\n            _this.renderFgEvents = core.memoizeRendering(eventRenderer.renderSegs.bind(eventRenderer), eventRenderer.unrender.bind(eventRenderer));\n            _this.renderEventSelection = core.memoizeRendering(eventRenderer.selectByInstanceId.bind(eventRenderer), eventRenderer.unselectByInstanceId.bind(eventRenderer), [_this.renderFgEvents]);\n            return _this;\n        }\n        TimelineLane.prototype.render = function (props, context) {\n            var timeAxis = this.timeAxis;\n            var slicedProps = this.slicer.sliceProps(props, props.dateProfile, timeAxis.tDateProfile.isTimeScale ? null : props.nextDayThreshold, context.calendar, this, timeAxis);\n            this.renderBusinessHours(context, slicedProps.businessHourSegs);\n            this.renderDateSelection(context, slicedProps.dateSelectionSegs);\n            this.renderBgEvents(context, slicedProps.bgEventSegs);\n            this.renderFgEvents(context, slicedProps.fgEventSegs);\n            this.renderEventSelection(slicedProps.eventSelection);\n            this.renderEventDrag(slicedProps.eventDrag);\n            this.renderEventResize(slicedProps.eventResize);\n        };\n        TimelineLane.prototype.destroy = function () {\n            _super.prototype.destroy.call(this);\n            this.renderBusinessHours.unrender();\n            this.renderDateSelection.unrender();\n            this.renderBgEvents.unrender();\n            this.renderFgEvents.unrender();\n            this.renderEventSelection.unrender();\n            this.renderEventDrag.unrender();\n            this.renderEventResize.unrender();\n        };\n        TimelineLane.prototype._renderEventDrag = function (state) {\n            if (state) {\n                this.eventRenderer.hideByHash(state.affectedInstances);\n                this.mirrorRenderer.renderSegs(this.context, state.segs, { isDragging: true, sourceSeg: state.sourceSeg });\n            }\n        };\n        TimelineLane.prototype._unrenderEventDrag = function (state) {\n            if (state) {\n                this.eventRenderer.showByHash(state.affectedInstances);\n                this.mirrorRenderer.unrender(this.context, state.segs, { isDragging: true, sourceSeg: state.sourceSeg });\n            }\n        };\n        TimelineLane.prototype._renderEventResize = function (state) {\n            if (state) {\n                // HACK. eventRenderer and fillRenderer both use these segs. would compete over seg.el\n                var segsForHighlight = state.segs.map(function (seg) {\n                    return __assign({}, seg);\n                });\n                this.eventRenderer.hideByHash(state.affectedInstances);\n                this.fillRenderer.renderSegs('highlight', this.context, segsForHighlight);\n                this.mirrorRenderer.renderSegs(this.context, state.segs, { isDragging: true, sourceSeg: state.sourceSeg });\n            }\n        };\n        TimelineLane.prototype._unrenderEventResize = function (state) {\n            if (state) {\n                this.eventRenderer.showByHash(state.affectedInstances);\n                this.fillRenderer.unrender('highlight', this.context);\n                this.mirrorRenderer.unrender(this.context, state.segs, { isDragging: true, sourceSeg: state.sourceSeg });\n            }\n        };\n        TimelineLane.prototype.updateSize = function (isResize) {\n            var _a = this, fillRenderer = _a.fillRenderer, eventRenderer = _a.eventRenderer, mirrorRenderer = _a.mirrorRenderer;\n            fillRenderer.computeSizes(isResize);\n            eventRenderer.computeSizes(isResize);\n            mirrorRenderer.computeSizes(isResize);\n            fillRenderer.assignSizes(isResize);\n            eventRenderer.assignSizes(isResize);\n            mirrorRenderer.assignSizes(isResize);\n        };\n        return TimelineLane;\n    }(core.DateComponent));\n    var TimelineLaneSlicer = /** @class */ (function (_super) {\n        __extends(TimelineLaneSlicer, _super);\n        function TimelineLaneSlicer() {\n            return _super !== null && _super.apply(this, arguments) || this;\n        }\n        TimelineLaneSlicer.prototype.sliceRange = function (origRange, timeAxis) {\n            var tDateProfile = timeAxis.tDateProfile;\n            var _a = timeAxis.props, dateProfile = _a.dateProfile, dateProfileGenerator = _a.dateProfileGenerator;\n            var dateEnv = timeAxis.context.dateEnv;\n            var normalRange = normalizeRange(origRange, tDateProfile, dateEnv);\n            var segs = [];\n            // protect against when the span is entirely in an invalid date region\n            if (timeAxis.computeDateSnapCoverage(normalRange.start) < timeAxis.computeDateSnapCoverage(normalRange.end)) {\n                // intersect the footprint's range with the grid's range\n                var slicedRange = core.intersectRanges(normalRange, tDateProfile.normalizedRange);\n                if (slicedRange) {\n                    segs.push({\n                        start: slicedRange.start,\n                        end: slicedRange.end,\n                        isStart: slicedRange.start.valueOf() === normalRange.start.valueOf() && isValidDate(slicedRange.start, tDateProfile, dateProfile, dateProfileGenerator),\n                        isEnd: slicedRange.end.valueOf() === normalRange.end.valueOf() && isValidDate(core.addMs(slicedRange.end, -1), tDateProfile, dateProfile, dateProfileGenerator)\n                    });\n                }\n            }\n            return segs;\n        };\n        return TimelineLaneSlicer;\n    }(core.Slicer));\n\n    var TimelineView = /** @class */ (function (_super) {\n        __extends(TimelineView, _super);\n        function TimelineView() {\n            var _this = _super !== null && _super.apply(this, arguments) || this;\n            _this.renderSkeleton = core.memoizeRendering(_this._renderSkeleton, _this._unrenderSkeleton);\n            _this.startInteractive = core.memoizeRendering(_this._startInteractive, _this._stopInteractive);\n            return _this;\n        }\n        TimelineView.prototype._startInteractive = function (timeAxisEl) {\n            this.context.calendar.registerInteractiveComponent(this, {\n                el: timeAxisEl\n            });\n        };\n        TimelineView.prototype._stopInteractive = function () {\n            this.context.calendar.unregisterInteractiveComponent(this);\n        };\n        TimelineView.prototype.render = function (props, context) {\n            _super.prototype.render.call(this, props, context); // flags for updateSize, addScroll. and _renderSkeleton/_unrenderSkeleton\n            this.renderSkeleton(this.context);\n            this.timeAxis.receiveProps({\n                dateProfileGenerator: props.dateProfileGenerator,\n                dateProfile: props.dateProfile\n            }, context);\n            this.startInteractive(this.timeAxis.slats.el);\n            this.lane.receiveProps(__assign({}, props, { nextDayThreshold: this.context.nextDayThreshold }), context);\n            this.startNowIndicator(props.dateProfile, props.dateProfileGenerator);\n        };\n        TimelineView.prototype.destroy = function () {\n            this.startInteractive.unrender(); // \"unrender\" a weird name\n            this.renderSkeleton.unrender();\n            _super.prototype.destroy.call(this);\n        };\n        TimelineView.prototype._renderSkeleton = function (context) {\n            this.el.classList.add('fc-timeline');\n            if (context.options.eventOverlap === false) {\n                this.el.classList.add('fc-no-overlap');\n            }\n            this.el.innerHTML = this.renderSkeletonHtml();\n            this.timeAxis = new TimeAxis(this.el.querySelector('thead .fc-time-area'), this.el.querySelector('tbody .fc-time-area'));\n            this.lane = new TimelineLane(this.timeAxis.layout.bodyScroller.enhancedScroll.canvas.contentEl, this.timeAxis.layout.bodyScroller.enhancedScroll.canvas.bgEl, this.timeAxis);\n        };\n        TimelineView.prototype._unrenderSkeleton = function () {\n            this.el.classList.remove('fc-timeline');\n            this.el.classList.remove('fc-no-overlap');\n            this.timeAxis.destroy();\n            this.lane.destroy();\n        };\n        TimelineView.prototype.renderSkeletonHtml = function () {\n            var theme = this.context.theme;\n            return \"<table class=\\\"\" + theme.getClass('tableGrid') + \"\\\"> <thead class=\\\"fc-head\\\"> <tr> <td class=\\\"fc-time-area \" + theme.getClass('widgetHeader') + \"\\\"></td> </tr> </thead> <tbody class=\\\"fc-body\\\"> <tr> <td class=\\\"fc-time-area \" + theme.getClass('widgetContent') + \"\\\"></td> </tr> </tbody> </table>\";\n        };\n        TimelineView.prototype.updateSize = function (isResize, totalHeight, isAuto) {\n            this.timeAxis.updateSize(isResize, totalHeight, isAuto);\n            this.lane.updateSize(isResize);\n        };\n        // Now Indicator\n        // ------------------------------------------------------------------------------------------\n        TimelineView.prototype.getNowIndicatorUnit = function (dateProfile, dateProfileGenerator) {\n            return this.timeAxis.getNowIndicatorUnit(dateProfile, dateProfileGenerator);\n        };\n        TimelineView.prototype.renderNowIndicator = function (date) {\n            this.timeAxis.renderNowIndicator(date);\n        };\n        TimelineView.prototype.unrenderNowIndicator = function () {\n            this.timeAxis.unrenderNowIndicator();\n        };\n        // Scroll System\n        // ------------------------------------------------------------------------------------------\n        TimelineView.prototype.computeDateScroll = function (duration) {\n            return this.timeAxis.computeDateScroll(duration);\n        };\n        TimelineView.prototype.applyScroll = function (scroll, isResize) {\n            _super.prototype.applyScroll.call(this, scroll, isResize); // will call applyDateScroll\n            var calendar = this.context.calendar;\n            // avoid updating stickyscroll too often\n            // TODO: repeat code as ResourceTimelineView::updateSize\n            if (isResize || calendar.isViewUpdated || calendar.isDatesUpdated || calendar.isEventsUpdated) {\n                this.timeAxis.updateStickyScrollers();\n            }\n        };\n        TimelineView.prototype.applyDateScroll = function (scroll) {\n            this.timeAxis.applyDateScroll(scroll);\n        };\n        TimelineView.prototype.queryScroll = function () {\n            var enhancedScroll = this.timeAxis.layout.bodyScroller.enhancedScroll;\n            return {\n                top: enhancedScroll.getScrollTop(),\n                left: enhancedScroll.getScrollLeft()\n            };\n        };\n        // Hit System\n        // ------------------------------------------------------------------------------------------\n        TimelineView.prototype.buildPositionCaches = function () {\n            this.timeAxis.slats.updateSize();\n        };\n        TimelineView.prototype.queryHit = function (positionLeft, positionTop, elWidth, elHeight) {\n            var slatHit = this.timeAxis.slats.positionToHit(positionLeft);\n            if (slatHit) {\n                return {\n                    component: this,\n                    dateSpan: slatHit.dateSpan,\n                    rect: {\n                        left: slatHit.left,\n                        right: slatHit.right,\n                        top: 0,\n                        bottom: elHeight\n                    },\n                    dayEl: slatHit.dayEl,\n                    layer: 0\n                };\n            }\n        };\n        return TimelineView;\n    }(core.View));\n\n    var main = core.createPlugin({\n        defaultView: 'timelineDay',\n        views: {\n            timeline: {\n                class: TimelineView,\n                eventResizableFromStart: true // how is this consumed for TimelineView tho?\n            },\n            timelineDay: {\n                type: 'timeline',\n                duration: { days: 1 }\n            },\n            timelineWeek: {\n                type: 'timeline',\n                duration: { weeks: 1 }\n            },\n            timelineMonth: {\n                type: 'timeline',\n                duration: { months: 1 }\n            },\n            timelineYear: {\n                type: 'timeline',\n                duration: { years: 1 }\n            }\n        }\n    });\n\n    exports.HeaderBodyLayout = HeaderBodyLayout;\n    exports.ScrollJoiner = ScrollJoiner;\n    exports.StickyScroller = StickyScroller;\n    exports.TimeAxis = TimeAxis;\n    exports.TimelineLane = TimelineLane;\n    exports.TimelineView = TimelineView;\n    exports.default = main;\n\n    Object.defineProperty(exports, '__esModule', { value: true });\n\n}));\n"

/***/ },

/***/ 1079
(module) {

module.exports = "(function webpackUniversalModuleDefinition(root, factory) {\n\tif(typeof exports === 'object' && typeof module === 'object')\n\t\tmodule.exports = factory();\n\telse if(typeof define === 'function' && define.amd)\n\t\tdefine([], factory);\n\telse if(typeof exports === 'object')\n\t\texports[\"rrule\"] = factory();\n\telse\n\t\troot[\"rrule\"] = factory();\n})(typeof self !== 'undefined' ? self : this, () => {\nreturn /******/ (() => { // webpackBootstrap\n/******/ \t\"use strict\";\n/******/ \t// The require scope\n/******/ \tvar __webpack_require__ = {};\n/******/ \t\n/************************************************************************/\n/******/ \t/* webpack/runtime/define property getters */\n/******/ \t(() => {\n/******/ \t\t// define getter functions for harmony exports\n/******/ \t\t__webpack_require__.d = (exports, definition) => {\n/******/ \t\t\tfor(var key in definition) {\n/******/ \t\t\t\tif(__webpack_require__.o(definition, key) && !__webpack_require__.o(exports, key)) {\n/******/ \t\t\t\t\tObject.defineProperty(exports, key, { enumerable: true, get: definition[key] });\n/******/ \t\t\t\t}\n/******/ \t\t\t}\n/******/ \t\t};\n/******/ \t})();\n/******/ \t\n/******/ \t/* webpack/runtime/hasOwnProperty shorthand */\n/******/ \t(() => {\n/******/ \t\t__webpack_require__.o = (obj, prop) => (Object.prototype.hasOwnProperty.call(obj, prop))\n/******/ \t})();\n/******/ \t\n/******/ \t/* webpack/runtime/make namespace object */\n/******/ \t(() => {\n/******/ \t\t// define __esModule on exports\n/******/ \t\t__webpack_require__.r = (exports) => {\n/******/ \t\t\tif(typeof Symbol !== 'undefined' && Symbol.toStringTag) {\n/******/ \t\t\t\tObject.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });\n/******/ \t\t\t}\n/******/ \t\t\tObject.defineProperty(exports, '__esModule', { value: true });\n/******/ \t\t};\n/******/ \t})();\n/******/ \t\n/************************************************************************/\nvar __webpack_exports__ = {};\n// ESM COMPAT FLAG\n__webpack_require__.r(__webpack_exports__);\n\n// EXPORTS\n__webpack_require__.d(__webpack_exports__, {\n  \"ALL_WEEKDAYS\": () => (/* reexport */ ALL_WEEKDAYS),\n  \"Frequency\": () => (/* reexport */ Frequency),\n  \"RRule\": () => (/* reexport */ RRule),\n  \"RRuleSet\": () => (/* reexport */ RRuleSet),\n  \"Weekday\": () => (/* reexport */ Weekday),\n  \"datetime\": () => (/* reexport */ datetime),\n  \"rrulestr\": () => (/* reexport */ rrulestr)\n});\n\n;// CONCATENATED MODULE: ./src/weekday.ts\n// =============================================================================\n// Weekday\n// =============================================================================\nvar ALL_WEEKDAYS = [\n    'MO',\n    'TU',\n    'WE',\n    'TH',\n    'FR',\n    'SA',\n    'SU',\n];\nvar Weekday = /** @class */ (function () {\n    function Weekday(weekday, n) {\n        if (n === 0)\n            throw new Error(\"Can't create weekday with n == 0\");\n        this.weekday = weekday;\n        this.n = n;\n    }\n    Weekday.fromStr = function (str) {\n        return new Weekday(ALL_WEEKDAYS.indexOf(str));\n    };\n    // __call__ - Cannot call the object directly, do it through\n    // e.g. RRule.TH.nth(-1) instead,\n    Weekday.prototype.nth = function (n) {\n        return this.n === n ? this : new Weekday(this.weekday, n);\n    };\n    // __eq__\n    Weekday.prototype.equals = function (other) {\n        return this.weekday === other.weekday && this.n === other.n;\n    };\n    // __repr__\n    Weekday.prototype.toString = function () {\n        var s = ALL_WEEKDAYS[this.weekday];\n        if (this.n)\n            s = (this.n > 0 ? '+' : '') + String(this.n) + s;\n        return s;\n    };\n    Weekday.prototype.getJsWeekday = function () {\n        return this.weekday === 6 ? 0 : this.weekday + 1;\n    };\n    return Weekday;\n}());\n\n\n;// CONCATENATED MODULE: ./src/helpers.ts\n// =============================================================================\n// Helper functions\n// =============================================================================\n\nvar isPresent = function (value) {\n    return value !== null && value !== undefined;\n};\nvar isNumber = function (value) {\n    return typeof value === 'number';\n};\nvar isWeekdayStr = function (value) {\n    return typeof value === 'string' && ALL_WEEKDAYS.includes(value);\n};\nvar isArray = Array.isArray;\n/**\n * Simplified version of python's range()\n */\nvar range = function (start, end) {\n    if (end === void 0) { end = start; }\n    if (arguments.length === 1) {\n        end = start;\n        start = 0;\n    }\n    var rang = [];\n    for (var i = start; i < end; i++)\n        rang.push(i);\n    return rang;\n};\nvar clone = function (array) {\n    return [].concat(array);\n};\nvar repeat = function (value, times) {\n    var i = 0;\n    var array = [];\n    if (isArray(value)) {\n        for (; i < times; i++)\n            array[i] = [].concat(value);\n    }\n    else {\n        for (; i < times; i++)\n            array[i] = value;\n    }\n    return array;\n};\nvar toArray = function (item) {\n    if (isArray(item)) {\n        return item;\n    }\n    return [item];\n};\nfunction padStart(item, targetLength, padString) {\n    if (padString === void 0) { padString = ' '; }\n    var str = String(item);\n    targetLength = targetLength >> 0;\n    if (str.length > targetLength) {\n        return String(str);\n    }\n    targetLength = targetLength - str.length;\n    if (targetLength > padString.length) {\n        padString += repeat(padString, targetLength / padString.length);\n    }\n    return padString.slice(0, targetLength) + String(str);\n}\n/**\n * Python like split\n */\nvar split = function (str, sep, num) {\n    var splits = str.split(sep);\n    return num\n        ? splits.slice(0, num).concat([splits.slice(num).join(sep)])\n        : splits;\n};\n/**\n * closure/goog/math/math.js:modulo\n * Copyright 2006 The Closure Library Authors.\n * The % operator in JavaScript returns the remainder of a / b, but differs from\n * some other languages in that the result will have the same sign as the\n * dividend. For example, -1 % 8 == -1, whereas in some other languages\n * (such as Python) the result would be 7. This function emulates the more\n * correct modulo behavior, which is useful for certain applications such as\n * calculating an offset index in a circular list.\n *\n * @param {number} a The dividend.\n * @param {number} b The divisor.\n * @return {number} a % b where the result is between 0 and b (either 0 <= x < b\n * or b < x <= 0, depending on the sign of b).\n */\nvar pymod = function (a, b) {\n    var r = a % b;\n    // If r and b differ in sign, add b to wrap the result to the correct sign.\n    return r * b < 0 ? r + b : r;\n};\n/**\n * @see: <http://docs.python.org/library/functions.html#divmod>\n */\nvar divmod = function (a, b) {\n    return { div: Math.floor(a / b), mod: pymod(a, b) };\n};\nvar empty = function (obj) {\n    return !isPresent(obj) || obj.length === 0;\n};\n/**\n * Python-like boolean\n *\n * @return {Boolean} value of an object/primitive, taking into account\n * the fact that in Python an empty list's/tuple's\n * boolean value is False, whereas in JS it's true\n */\nvar notEmpty = function (obj) {\n    return !empty(obj);\n};\n/**\n * Return true if a value is in an array\n */\nvar includes = function (arr, val) {\n    return notEmpty(arr) && arr.indexOf(val) !== -1;\n};\n\n;// CONCATENATED MODULE: ./src/dateutil.ts\n\nvar datetime = function (y, m, d, h, i, s) {\n    if (h === void 0) { h = 0; }\n    if (i === void 0) { i = 0; }\n    if (s === void 0) { s = 0; }\n    return new Date(Date.UTC(y, m - 1, d, h, i, s));\n};\n/**\n * General date-related utilities.\n * Also handles several incompatibilities between JavaScript and Python\n *\n */\nvar MONTH_DAYS = [31, 28, 31, 30, 31, 30, 31, 31, 30, 31, 30, 31];\n/**\n * Number of milliseconds of one day\n */\nvar ONE_DAY = 1000 * 60 * 60 * 24;\n/**\n * @see: <http://docs.python.org/library/datetime.html#datetime.MAXYEAR>\n */\nvar MAXYEAR = 9999;\n/**\n * Python uses 1-Jan-1 as the base for calculating ordinals but we don't\n * want to confuse the JS engine with milliseconds > Number.MAX_NUMBER,\n * therefore we use 1-Jan-1970 instead\n */\nvar ORDINAL_BASE = datetime(1970, 1, 1);\n/**\n * Python: MO-SU: 0 - 6\n * JS: SU-SAT 0 - 6\n */\nvar PY_WEEKDAYS = [6, 0, 1, 2, 3, 4, 5];\n/**\n * py_date.timetuple()[7]\n */\nvar getYearDay = function (date) {\n    var dateNoTime = new Date(date.getUTCFullYear(), date.getUTCMonth(), date.getUTCDate());\n    return (Math.ceil((dateNoTime.valueOf() - new Date(date.getUTCFullYear(), 0, 1).valueOf()) /\n        ONE_DAY) + 1);\n};\nvar isLeapYear = function (year) {\n    return (year % 4 === 0 && year % 100 !== 0) || year % 400 === 0;\n};\nvar isDate = function (value) {\n    return value instanceof Date;\n};\nvar isValidDate = function (value) {\n    return isDate(value) && !isNaN(value.getTime());\n};\n/**\n * @return {Number} the date's timezone offset in ms\n */\nvar tzOffset = function (date) {\n    return date.getTimezoneOffset() * 60 * 1000;\n};\n/**\n * @see: <http://www.mcfedries.com/JavaScript/DaysBetween.asp>\n */\nvar daysBetween = function (date1, date2) {\n    // The number of milliseconds in one day\n    // Convert both dates to milliseconds\n    var date1ms = date1.getTime();\n    var date2ms = date2.getTime();\n    // Calculate the difference in milliseconds\n    var differencems = date1ms - date2ms;\n    // Convert back to days and return\n    return Math.round(differencems / ONE_DAY);\n};\n/**\n * @see: <http://docs.python.org/library/datetime.html#datetime.date.toordinal>\n */\nvar toOrdinal = function (date) {\n    return daysBetween(date, ORDINAL_BASE);\n};\n/**\n * @see - <http://docs.python.org/library/datetime.html#datetime.date.fromordinal>\n */\nvar fromOrdinal = function (ordinal) {\n    return new Date(ORDINAL_BASE.getTime() + ordinal * ONE_DAY);\n};\nvar getMonthDays = function (date) {\n    var month = date.getUTCMonth();\n    return month === 1 && isLeapYear(date.getUTCFullYear())\n        ? 29\n        : MONTH_DAYS[month];\n};\n/**\n * @return {Number} python-like weekday\n */\nvar getWeekday = function (date) {\n    return PY_WEEKDAYS[date.getUTCDay()];\n};\n/**\n * @see: <http://docs.python.org/library/calendar.html#calendar.monthrange>\n */\nvar monthRange = function (year, month) {\n    var date = datetime(year, month + 1, 1);\n    return [getWeekday(date), getMonthDays(date)];\n};\n/**\n * @see: <http://docs.python.org/library/datetime.html#datetime.datetime.combine>\n */\nvar combine = function (date, time) {\n    time = time || date;\n    return new Date(Date.UTC(date.getUTCFullYear(), date.getUTCMonth(), date.getUTCDate(), time.getHours(), time.getMinutes(), time.getSeconds(), time.getMilliseconds()));\n};\nvar dateutil_clone = function (date) {\n    var dolly = new Date(date.getTime());\n    return dolly;\n};\nvar cloneDates = function (dates) {\n    var clones = [];\n    for (var i = 0; i < dates.length; i++) {\n        clones.push(dateutil_clone(dates[i]));\n    }\n    return clones;\n};\n/**\n * Sorts an array of Date or Time objects\n */\nvar sort = function (dates) {\n    dates.sort(function (a, b) {\n        return a.getTime() - b.getTime();\n    });\n};\nvar timeToUntilString = function (time, utc) {\n    if (utc === void 0) { utc = true; }\n    var date = new Date(time);\n    return [\n        padStart(date.getUTCFullYear().toString(), 4, '0'),\n        padStart(date.getUTCMonth() + 1, 2, '0'),\n        padStart(date.getUTCDate(), 2, '0'),\n        'T',\n        padStart(date.getUTCHours(), 2, '0'),\n        padStart(date.getUTCMinutes(), 2, '0'),\n        padStart(date.getUTCSeconds(), 2, '0'),\n        utc ? 'Z' : '',\n    ].join('');\n};\nvar untilStringToDate = function (until) {\n    var re = /^(\\d{4})(\\d{2})(\\d{2})(T(\\d{2})(\\d{2})(\\d{2})Z?)?$/;\n    var bits = re.exec(until);\n    if (!bits)\n        throw new Error(\"Invalid UNTIL value: \".concat(until));\n    return new Date(Date.UTC(parseInt(bits[1], 10), parseInt(bits[2], 10) - 1, parseInt(bits[3], 10), parseInt(bits[5], 10) || 0, parseInt(bits[6], 10) || 0, parseInt(bits[7], 10) || 0));\n};\nvar dateTZtoISO8601 = function (date, timeZone) {\n    // date format for sv-SE is almost ISO8601\n    var dateStr = date.toLocaleString('sv-SE', { timeZone: timeZone });\n    // '2023-02-07 10:41:36'\n    return dateStr.replace(' ', 'T') + 'Z';\n};\nvar dateInTimeZone = function (date, timeZone) {\n    var localTimeZone = Intl.DateTimeFormat().resolvedOptions().timeZone;\n    // Date constructor can only reliably parse dates in ISO8601 format\n    var dateInLocalTZ = new Date(dateTZtoISO8601(date, localTimeZone));\n    var dateInTargetTZ = new Date(dateTZtoISO8601(date, timeZone !== null && timeZone !== void 0 ? timeZone : 'UTC'));\n    var tzOffset = dateInTargetTZ.getTime() - dateInLocalTZ.getTime();\n    return new Date(date.getTime() - tzOffset);\n};\n\n;// CONCATENATED MODULE: ./src/iterresult.ts\n/**\n * This class helps us to emulate python's generators, sorta.\n */\nvar IterResult = /** @class */ (function () {\n    function IterResult(method, args) {\n        this.minDate = null;\n        this.maxDate = null;\n        this._result = [];\n        this.total = 0;\n        this.method = method;\n        this.args = args;\n        if (method === 'between') {\n            this.maxDate = args.inc\n                ? args.before\n                : new Date(args.before.getTime() - 1);\n            this.minDate = args.inc ? args.after : new Date(args.after.getTime() + 1);\n        }\n        else if (method === 'before') {\n            this.maxDate = args.inc ? args.dt : new Date(args.dt.getTime() - 1);\n        }\n        else if (method === 'after') {\n            this.minDate = args.inc ? args.dt : new Date(args.dt.getTime() + 1);\n        }\n    }\n    /**\n     * Possibly adds a date into the result.\n     *\n     * @param {Date} date - the date isn't necessarly added to the result\n     * list (if it is too late/too early)\n     * @return {Boolean} true if it makes sense to continue the iteration\n     * false if we're done.\n     */\n    IterResult.prototype.accept = function (date) {\n        ++this.total;\n        var tooEarly = this.minDate && date < this.minDate;\n        var tooLate = this.maxDate && date > this.maxDate;\n        if (this.method === 'between') {\n            if (tooEarly)\n                return true;\n            if (tooLate)\n                return false;\n        }\n        else if (this.method === 'before') {\n            if (tooLate)\n                return false;\n        }\n        else if (this.method === 'after') {\n            if (tooEarly)\n                return true;\n            this.add(date);\n            return false;\n        }\n        return this.add(date);\n    };\n    /**\n     *\n     * @param {Date} date that is part of the result.\n     * @return {Boolean} whether we are interested in more values.\n     */\n    IterResult.prototype.add = function (date) {\n        this._result.push(date);\n        return true;\n    };\n    /**\n     * 'before' and 'after' return only one date, whereas 'all'\n     * and 'between' an array.\n     *\n     * @return {Date,Array?}\n     */\n    IterResult.prototype.getValue = function () {\n        var res = this._result;\n        switch (this.method) {\n            case 'all':\n            case 'between':\n                return res;\n            case 'before':\n            case 'after':\n            default:\n                return (res.length ? res[res.length - 1] : null);\n        }\n    };\n    IterResult.prototype.clone = function () {\n        return new IterResult(this.method, this.args);\n    };\n    return IterResult;\n}());\n/* harmony default export */ const iterresult = (IterResult);\n\n;// CONCATENATED MODULE: ./node_modules/tslib/tslib.es6.js\n/******************************************************************************\r\nCopyright (c) Microsoft Corporation.\r\n\r\nPermission to use, copy, modify, and/or distribute this software for any\r\npurpose with or without fee is hereby granted.\r\n\r\nTHE SOFTWARE IS PROVIDED \"AS IS\" AND THE AUTHOR DISCLAIMS ALL WARRANTIES WITH\r\nREGARD TO THIS SOFTWARE INCLUDING ALL IMPLIED WARRANTIES OF MERCHANTABILITY\r\nAND FITNESS. IN NO EVENT SHALL THE AUTHOR BE LIABLE FOR ANY SPECIAL, DIRECT,\r\nINDIRECT, OR CONSEQUENTIAL DAMAGES OR ANY DAMAGES WHATSOEVER RESULTING FROM\r\nLOSS OF USE, DATA OR PROFITS, WHETHER IN AN ACTION OF CONTRACT, NEGLIGENCE OR\r\nOTHER TORTIOUS ACTION, ARISING OUT OF OR IN CONNECTION WITH THE USE OR\r\nPERFORMANCE OF THIS SOFTWARE.\r\n***************************************************************************** */\r\n/* global Reflect, Promise */\r\n\r\nvar extendStatics = function(d, b) {\r\n    extendStatics = Object.setPrototypeOf ||\r\n        ({ __proto__: [] } instanceof Array && function (d, b) { d.__proto__ = b; }) ||\r\n        function (d, b) { for (var p in b) if (Object.prototype.hasOwnProperty.call(b, p)) d[p] = b[p]; };\r\n    return extendStatics(d, b);\r\n};\r\n\r\nfunction __extends(d, b) {\r\n    if (typeof b !== \"function\" && b !== null)\r\n        throw new TypeError(\"Class extends value \" + String(b) + \" is not a constructor or null\");\r\n    extendStatics(d, b);\r\n    function __() { this.constructor = d; }\r\n    d.prototype = b === null ? Object.create(b) : (__.prototype = b.prototype, new __());\r\n}\r\n\r\nvar __assign = function() {\r\n    __assign = Object.assign || function __assign(t) {\r\n        for (var s, i = 1, n = arguments.length; i < n; i++) {\r\n            s = arguments[i];\r\n            for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p)) t[p] = s[p];\r\n        }\r\n        return t;\r\n    }\r\n    return __assign.apply(this, arguments);\r\n}\r\n\r\nfunction __rest(s, e) {\r\n    var t = {};\r\n    for (var p in s) if (Object.prototype.hasOwnProperty.call(s, p) && e.indexOf(p) < 0)\r\n        t[p] = s[p];\r\n    if (s != null && typeof Object.getOwnPropertySymbols === \"function\")\r\n        for (var i = 0, p = Object.getOwnPropertySymbols(s); i < p.length; i++) {\r\n            if (e.indexOf(p[i]) < 0 && Object.prototype.propertyIsEnumerable.call(s, p[i]))\r\n                t[p[i]] = s[p[i]];\r\n        }\r\n    return t;\r\n}\r\n\r\nfunction __decorate(decorators, target, key, desc) {\r\n    var c = arguments.length, r = c < 3 ? target : desc === null ? desc = Object.getOwnPropertyDescriptor(target, key) : desc, d;\r\n    if (typeof Reflect === \"object\" && typeof Reflect.decorate === \"function\") r = Reflect.decorate(decorators, target, key, desc);\r\n    else for (var i = decorators.length - 1; i >= 0; i--) if (d = decorators[i]) r = (c < 3 ? d(r) : c > 3 ? d(target, key, r) : d(target, key)) || r;\r\n    return c > 3 && r && Object.defineProperty(target, key, r), r;\r\n}\r\n\r\nfunction __param(paramIndex, decorator) {\r\n    return function (target, key) { decorator(target, key, paramIndex); }\r\n}\r\n\r\nfunction __metadata(metadataKey, metadataValue) {\r\n    if (typeof Reflect === \"object\" && typeof Reflect.metadata === \"function\") return Reflect.metadata(metadataKey, metadataValue);\r\n}\r\n\r\nfunction __awaiter(thisArg, _arguments, P, generator) {\r\n    function adopt(value) { return value instanceof P ? value : new P(function (resolve) { resolve(value); }); }\r\n    return new (P || (P = Promise))(function (resolve, reject) {\r\n        function fulfilled(value) { try { step(generator.next(value)); } catch (e) { reject(e); } }\r\n        function rejected(value) { try { step(generator[\"throw\"](value)); } catch (e) { reject(e); } }\r\n        function step(result) { result.done ? resolve(result.value) : adopt(result.value).then(fulfilled, rejected); }\r\n        step((generator = generator.apply(thisArg, _arguments || [])).next());\r\n    });\r\n}\r\n\r\nfunction __generator(thisArg, body) {\r\n    var _ = { label: 0, sent: function() { if (t[0] & 1) throw t[1]; return t[1]; }, trys: [], ops: [] }, f, y, t, g;\r\n    return g = { next: verb(0), \"throw\": verb(1), \"return\": verb(2) }, typeof Symbol === \"function\" && (g[Symbol.iterator] = function() { return this; }), g;\r\n    function verb(n) { return function (v) { return step([n, v]); }; }\r\n    function step(op) {\r\n        if (f) throw new TypeError(\"Generator is already executing.\");\r\n        while (_) try {\r\n            if (f = 1, y && (t = op[0] & 2 ? y[\"return\"] : op[0] ? y[\"throw\"] || ((t = y[\"return\"]) && t.call(y), 0) : y.next) && !(t = t.call(y, op[1])).done) return t;\r\n            if (y = 0, t) op = [op[0] & 2, t.value];\r\n            switch (op[0]) {\r\n                case 0: case 1: t = op; break;\r\n                case 4: _.label++; return { value: op[1], done: false };\r\n                case 5: _.label++; y = op[1]; op = [0]; continue;\r\n                case 7: op = _.ops.pop(); _.trys.pop(); continue;\r\n                default:\r\n                    if (!(t = _.trys, t = t.length > 0 && t[t.length - 1]) && (op[0] === 6 || op[0] === 2)) { _ = 0; continue; }\r\n                    if (op[0] === 3 && (!t || (op[1] > t[0] && op[1] < t[3]))) { _.label = op[1]; break; }\r\n                    if (op[0] === 6 && _.label < t[1]) { _.label = t[1]; t = op; break; }\r\n                    if (t && _.label < t[2]) { _.label = t[2]; _.ops.push(op); break; }\r\n                    if (t[2]) _.ops.pop();\r\n                    _.trys.pop(); continue;\r\n            }\r\n            op = body.call(thisArg, _);\r\n        } catch (e) { op = [6, e]; y = 0; } finally { f = t = 0; }\r\n        if (op[0] & 5) throw op[1]; return { value: op[0] ? op[1] : void 0, done: true };\r\n    }\r\n}\r\n\r\nvar __createBinding = Object.create ? (function(o, m, k, k2) {\r\n    if (k2 === undefined) k2 = k;\r\n    var desc = Object.getOwnPropertyDescriptor(m, k);\r\n    if (!desc || (\"get\" in desc ? !m.__esModule : desc.writable || desc.configurable)) {\r\n        desc = { enumerable: true, get: function() { return m[k]; } };\r\n    }\r\n    Object.defineProperty(o, k2, desc);\r\n}) : (function(o, m, k, k2) {\r\n    if (k2 === undefined) k2 = k;\r\n    o[k2] = m[k];\r\n});\r\n\r\nfunction __exportStar(m, o) {\r\n    for (var p in m) if (p !== \"default\" && !Object.prototype.hasOwnProperty.call(o, p)) __createBinding(o, m, p);\r\n}\r\n\r\nfunction __values(o) {\r\n    var s = typeof Symbol === \"function\" && Symbol.iterator, m = s && o[s], i = 0;\r\n    if (m) return m.call(o);\r\n    if (o && typeof o.length === \"number\") return {\r\n        next: function () {\r\n            if (o && i >= o.length) o = void 0;\r\n            return { value: o && o[i++], done: !o };\r\n        }\r\n    };\r\n    throw new TypeError(s ? \"Object is not iterable.\" : \"Symbol.iterator is not defined.\");\r\n}\r\n\r\nfunction __read(o, n) {\r\n    var m = typeof Symbol === \"function\" && o[Symbol.iterator];\r\n    if (!m) return o;\r\n    var i = m.call(o), r, ar = [], e;\r\n    try {\r\n        while ((n === void 0 || n-- > 0) && !(r = i.next()).done) ar.push(r.value);\r\n    }\r\n    catch (error) { e = { error: error }; }\r\n    finally {\r\n        try {\r\n            if (r && !r.done && (m = i[\"return\"])) m.call(i);\r\n        }\r\n        finally { if (e) throw e.error; }\r\n    }\r\n    return ar;\r\n}\r\n\r\n/** @deprecated */\r\nfunction __spread() {\r\n    for (var ar = [], i = 0; i < arguments.length; i++)\r\n        ar = ar.concat(__read(arguments[i]));\r\n    return ar;\r\n}\r\n\r\n/** @deprecated */\r\nfunction __spreadArrays() {\r\n    for (var s = 0, i = 0, il = arguments.length; i < il; i++) s += arguments[i].length;\r\n    for (var r = Array(s), k = 0, i = 0; i < il; i++)\r\n        for (var a = arguments[i], j = 0, jl = a.length; j < jl; j++, k++)\r\n            r[k] = a[j];\r\n    return r;\r\n}\r\n\r\nfunction __spreadArray(to, from, pack) {\r\n    if (pack || arguments.length === 2) for (var i = 0, l = from.length, ar; i < l; i++) {\r\n        if (ar || !(i in from)) {\r\n            if (!ar) ar = Array.prototype.slice.call(from, 0, i);\r\n            ar[i] = from[i];\r\n        }\r\n    }\r\n    return to.concat(ar || Array.prototype.slice.call(from));\r\n}\r\n\r\nfunction __await(v) {\r\n    return this instanceof __await ? (this.v = v, this) : new __await(v);\r\n}\r\n\r\nfunction __asyncGenerator(thisArg, _arguments, generator) {\r\n    if (!Symbol.asyncIterator) throw new TypeError(\"Symbol.asyncIterator is not defined.\");\r\n    var g = generator.apply(thisArg, _arguments || []), i, q = [];\r\n    return i = {}, verb(\"next\"), verb(\"throw\"), verb(\"return\"), i[Symbol.asyncIterator] = function () { return this; }, i;\r\n    function verb(n) { if (g[n]) i[n] = function (v) { return new Promise(function (a, b) { q.push([n, v, a, b]) > 1 || resume(n, v); }); }; }\r\n    function resume(n, v) { try { step(g[n](v)); } catch (e) { settle(q[0][3], e); } }\r\n    function step(r) { r.value instanceof __await ? Promise.resolve(r.value.v).then(fulfill, reject) : settle(q[0][2], r); }\r\n    function fulfill(value) { resume(\"next\", value); }\r\n    function reject(value) { resume(\"throw\", value); }\r\n    function settle(f, v) { if (f(v), q.shift(), q.length) resume(q[0][0], q[0][1]); }\r\n}\r\n\r\nfunction __asyncDelegator(o) {\r\n    var i, p;\r\n    return i = {}, verb(\"next\"), verb(\"throw\", function (e) { throw e; }), verb(\"return\"), i[Symbol.iterator] = function () { return this; }, i;\r\n    function verb(n, f) { i[n] = o[n] ? function (v) { return (p = !p) ? { value: __await(o[n](v)), done: n === \"return\" } : f ? f(v) : v; } : f; }\r\n}\r\n\r\nfunction __asyncValues(o) {\r\n    if (!Symbol.asyncIterator) throw new TypeError(\"Symbol.asyncIterator is not defined.\");\r\n    var m = o[Symbol.asyncIterator], i;\r\n    return m ? m.call(o) : (o = typeof __values === \"function\" ? __values(o) : o[Symbol.iterator](), i = {}, verb(\"next\"), verb(\"throw\"), verb(\"return\"), i[Symbol.asyncIterator] = function () { return this; }, i);\r\n    function verb(n) { i[n] = o[n] && function (v) { return new Promise(function (resolve, reject) { v = o[n](v), settle(resolve, reject, v.done, v.value); }); }; }\r\n    function settle(resolve, reject, d, v) { Promise.resolve(v).then(function(v) { resolve({ value: v, done: d }); }, reject); }\r\n}\r\n\r\nfunction __makeTemplateObject(cooked, raw) {\r\n    if (Object.defineProperty) { Object.defineProperty(cooked, \"raw\", { value: raw }); } else { cooked.raw = raw; }\r\n    return cooked;\r\n};\r\n\r\nvar __setModuleDefault = Object.create ? (function(o, v) {\r\n    Object.defineProperty(o, \"default\", { enumerable: true, value: v });\r\n}) : function(o, v) {\r\n    o[\"default\"] = v;\r\n};\r\n\r\nfunction __importStar(mod) {\r\n    if (mod && mod.__esModule) return mod;\r\n    var result = {};\r\n    if (mod != null) for (var k in mod) if (k !== \"default\" && Object.prototype.hasOwnProperty.call(mod, k)) __createBinding(result, mod, k);\r\n    __setModuleDefault(result, mod);\r\n    return result;\r\n}\r\n\r\nfunction __importDefault(mod) {\r\n    return (mod && mod.__esModule) ? mod : { default: mod };\r\n}\r\n\r\nfunction __classPrivateFieldGet(receiver, state, kind, f) {\r\n    if (kind === \"a\" && !f) throw new TypeError(\"Private accessor was defined without a getter\");\r\n    if (typeof state === \"function\" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError(\"Cannot read private member from an object whose class did not declare it\");\r\n    return kind === \"m\" ? f : kind === \"a\" ? f.call(receiver) : f ? f.value : state.get(receiver);\r\n}\r\n\r\nfunction __classPrivateFieldSet(receiver, state, value, kind, f) {\r\n    if (kind === \"m\") throw new TypeError(\"Private method is not writable\");\r\n    if (kind === \"a\" && !f) throw new TypeError(\"Private accessor was defined without a setter\");\r\n    if (typeof state === \"function\" ? receiver !== state || !f : !state.has(receiver)) throw new TypeError(\"Cannot write private member to an object whose class did not declare it\");\r\n    return (kind === \"a\" ? f.call(receiver, value) : f ? f.value = value : state.set(receiver, value)), value;\r\n}\r\n\r\nfunction __classPrivateFieldIn(state, receiver) {\r\n    if (receiver === null || (typeof receiver !== \"object\" && typeof receiver !== \"function\")) throw new TypeError(\"Cannot use 'in' operator on non-object\");\r\n    return typeof state === \"function\" ? receiver === state : state.has(receiver);\r\n}\r\n\n;// CONCATENATED MODULE: ./src/callbackiterresult.ts\n\n\n/**\n * IterResult subclass that calls a callback function on each add,\n * and stops iterating when the callback returns false.\n */\nvar CallbackIterResult = /** @class */ (function (_super) {\n    __extends(CallbackIterResult, _super);\n    function CallbackIterResult(method, args, iterator) {\n        var _this = _super.call(this, method, args) || this;\n        _this.iterator = iterator;\n        return _this;\n    }\n    CallbackIterResult.prototype.add = function (date) {\n        if (this.iterator(date, this._result.length)) {\n            this._result.push(date);\n            return true;\n        }\n        return false;\n    };\n    return CallbackIterResult;\n}(iterresult));\n/* harmony default export */ const callbackiterresult = (CallbackIterResult);\n\n;// CONCATENATED MODULE: ./src/nlp/i18n.ts\n// =============================================================================\n// i18n\n// =============================================================================\nvar ENGLISH = {\n    dayNames: [\n        'Sunday',\n        'Monday',\n        'Tuesday',\n        'Wednesday',\n        'Thursday',\n        'Friday',\n        'Saturday',\n    ],\n    monthNames: [\n        'January',\n        'February',\n        'March',\n        'April',\n        'May',\n        'June',\n        'July',\n        'August',\n        'September',\n        'October',\n        'November',\n        'December',\n    ],\n    tokens: {\n        SKIP: /^[ \\r\\n\\t]+|^\\.$/,\n        number: /^[1-9][0-9]*/,\n        numberAsText: /^(one|two|three)/i,\n        every: /^every/i,\n        'day(s)': /^days?/i,\n        'weekday(s)': /^weekdays?/i,\n        'week(s)': /^weeks?/i,\n        'hour(s)': /^hours?/i,\n        'minute(s)': /^minutes?/i,\n        'month(s)': /^months?/i,\n        'year(s)': /^years?/i,\n        on: /^(on|in)/i,\n        at: /^(at)/i,\n        the: /^the/i,\n        first: /^first/i,\n        second: /^second/i,\n        third: /^third/i,\n        nth: /^([1-9][0-9]*)(\\.|th|nd|rd|st)/i,\n        last: /^last/i,\n        for: /^for/i,\n        'time(s)': /^times?/i,\n        until: /^(un)?til/i,\n        monday: /^mo(n(day)?)?/i,\n        tuesday: /^tu(e(s(day)?)?)?/i,\n        wednesday: /^we(d(n(esday)?)?)?/i,\n        thursday: /^th(u(r(sday)?)?)?/i,\n        friday: /^fr(i(day)?)?/i,\n        saturday: /^sa(t(urday)?)?/i,\n        sunday: /^su(n(day)?)?/i,\n        january: /^jan(uary)?/i,\n        february: /^feb(ruary)?/i,\n        march: /^mar(ch)?/i,\n        april: /^apr(il)?/i,\n        may: /^may/i,\n        june: /^june?/i,\n        july: /^july?/i,\n        august: /^aug(ust)?/i,\n        september: /^sep(t(ember)?)?/i,\n        october: /^oct(ober)?/i,\n        november: /^nov(ember)?/i,\n        december: /^dec(ember)?/i,\n        comma: /^(,\\s*|(and|or)\\s*)+/i,\n    },\n};\n/* harmony default export */ const i18n = (ENGLISH);\n\n;// CONCATENATED MODULE: ./src/nlp/totext.ts\n\n\n\n// =============================================================================\n// Helper functions\n// =============================================================================\n/**\n * Return true if a value is in an array\n */\nvar contains = function (arr, val) {\n    return arr.indexOf(val) !== -1;\n};\nvar defaultGetText = function (id) { return id.toString(); };\nvar defaultDateFormatter = function (year, month, day) { return \"\".concat(month, \" \").concat(day, \", \").concat(year); };\n/**\n *\n * @param {RRule} rrule\n * Optional:\n * @param {Function} gettext function\n * @param {Object} language definition\n * @constructor\n */\nvar ToText = /** @class */ (function () {\n    function ToText(rrule, gettext, language, dateFormatter) {\n        if (gettext === void 0) { gettext = defaultGetText; }\n        if (language === void 0) { language = i18n; }\n        if (dateFormatter === void 0) { dateFormatter = defaultDateFormatter; }\n        this.text = [];\n        this.language = language || i18n;\n        this.gettext = gettext;\n        this.dateFormatter = dateFormatter;\n        this.rrule = rrule;\n        this.options = rrule.options;\n        this.origOptions = rrule.origOptions;\n        if (this.origOptions.bymonthday) {\n            var bymonthday = [].concat(this.options.bymonthday);\n            var bynmonthday = [].concat(this.options.bynmonthday);\n            bymonthday.sort(function (a, b) { return a - b; });\n            bynmonthday.sort(function (a, b) { return b - a; });\n            // 1, 2, 3, .., -5, -4, -3, ..\n            this.bymonthday = bymonthday.concat(bynmonthday);\n            if (!this.bymonthday.length)\n                this.bymonthday = null;\n        }\n        if (isPresent(this.origOptions.byweekday)) {\n            var byweekday = !isArray(this.origOptions.byweekday)\n                ? [this.origOptions.byweekday]\n                : this.origOptions.byweekday;\n            var days = String(byweekday);\n            this.byweekday = {\n                allWeeks: byweekday.filter(function (weekday) {\n                    return !weekday.n;\n                }),\n                someWeeks: byweekday.filter(function (weekday) {\n                    return Boolean(weekday.n);\n                }),\n                isWeekdays: days.indexOf('MO') !== -1 &&\n                    days.indexOf('TU') !== -1 &&\n                    days.indexOf('WE') !== -1 &&\n                    days.indexOf('TH') !== -1 &&\n                    days.indexOf('FR') !== -1 &&\n                    days.indexOf('SA') === -1 &&\n                    days.indexOf('SU') === -1,\n                isEveryDay: days.indexOf('MO') !== -1 &&\n                    days.indexOf('TU') !== -1 &&\n                    days.indexOf('WE') !== -1 &&\n                    days.indexOf('TH') !== -1 &&\n                    days.indexOf('FR') !== -1 &&\n                    days.indexOf('SA') !== -1 &&\n                    days.indexOf('SU') !== -1,\n            };\n            var sortWeekDays = function (a, b) {\n                return a.weekday - b.weekday;\n            };\n            this.byweekday.allWeeks.sort(sortWeekDays);\n            this.byweekday.someWeeks.sort(sortWeekDays);\n            if (!this.byweekday.allWeeks.length)\n                this.byweekday.allWeeks = null;\n            if (!this.byweekday.someWeeks.length)\n                this.byweekday.someWeeks = null;\n        }\n        else {\n            this.byweekday = null;\n        }\n    }\n    /**\n     * Test whether the rrule can be fully converted to text.\n     *\n     * @param {RRule} rrule\n     * @return {Boolean}\n     */\n    ToText.isFullyConvertible = function (rrule) {\n        var canConvert = true;\n        if (!(rrule.options.freq in ToText.IMPLEMENTED))\n            return false;\n        if (rrule.origOptions.until && rrule.origOptions.count)\n            return false;\n        for (var key in rrule.origOptions) {\n            if (contains(['dtstart', 'tzid', 'wkst', 'freq'], key))\n                return true;\n            if (!contains(ToText.IMPLEMENTED[rrule.options.freq], key))\n                return false;\n        }\n        return canConvert;\n    };\n    ToText.prototype.isFullyConvertible = function () {\n        return ToText.isFullyConvertible(this.rrule);\n    };\n    /**\n     * Perform the conversion. Only some of the frequencies are supported.\n     * If some of the rrule's options aren't supported, they'll\n     * be omitted from the output an \"(~ approximate)\" will be appended.\n     *\n     * @return {*}\n     */\n    ToText.prototype.toString = function () {\n        var gettext = this.gettext;\n        if (!(this.options.freq in ToText.IMPLEMENTED)) {\n            return gettext('RRule error: Unable to fully convert this rrule to text');\n        }\n        this.text = [gettext('every')];\n        // eslint-disable-next-line @typescript-eslint/ban-ts-comment\n        // @ts-ignore\n        this[RRule.FREQUENCIES[this.options.freq]]();\n        if (this.options.until) {\n            this.add(gettext('until'));\n            var until = this.options.until;\n            this.add(this.dateFormatter(until.getUTCFullYear(), this.language.monthNames[until.getUTCMonth()], until.getUTCDate()));\n        }\n        else if (this.options.count) {\n            this.add(gettext('for'))\n                .add(this.options.count.toString())\n                .add(this.plural(this.options.count) ? gettext('times') : gettext('time'));\n        }\n        if (!this.isFullyConvertible())\n            this.add(gettext('(~ approximate)'));\n        return this.text.join('');\n    };\n    ToText.prototype.HOURLY = function () {\n        var gettext = this.gettext;\n        if (this.options.interval !== 1)\n            this.add(this.options.interval.toString());\n        this.add(this.plural(this.options.interval) ? gettext('hours') : gettext('hour'));\n    };\n    ToText.prototype.MINUTELY = function () {\n        var gettext = this.gettext;\n        if (this.options.interval !== 1)\n            this.add(this.options.interval.toString());\n        this.add(this.plural(this.options.interval)\n            ? gettext('minutes')\n            : gettext('minute'));\n    };\n    ToText.prototype.DAILY = function () {\n        var gettext = this.gettext;\n        if (this.options.interval !== 1)\n            this.add(this.options.interval.toString());\n        if (this.byweekday && this.byweekday.isWeekdays) {\n            this.add(this.plural(this.options.interval)\n                ? gettext('weekdays')\n                : gettext('weekday'));\n        }\n        else {\n            this.add(this.plural(this.options.interval) ? gettext('days') : gettext('day'));\n        }\n        if (this.origOptions.bymonth) {\n            this.add(gettext('in'));\n            this._bymonth();\n        }\n        if (this.bymonthday) {\n            this._bymonthday();\n        }\n        else if (this.byweekday) {\n            this._byweekday();\n        }\n        else if (this.origOptions.byhour) {\n            this._byhour();\n        }\n    };\n    ToText.prototype.WEEKLY = function () {\n        var gettext = this.gettext;\n        if (this.options.interval !== 1) {\n            this.add(this.options.interval.toString()).add(this.plural(this.options.interval) ? gettext('weeks') : gettext('week'));\n        }\n        if (this.byweekday && this.byweekday.isWeekdays) {\n            if (this.options.interval === 1) {\n                this.add(this.plural(this.options.interval)\n                    ? gettext('weekdays')\n                    : gettext('weekday'));\n            }\n            else {\n                this.add(gettext('on')).add(gettext('weekdays'));\n            }\n        }\n        else if (this.byweekday && this.byweekday.isEveryDay) {\n            this.add(this.plural(this.options.interval) ? gettext('days') : gettext('day'));\n        }\n        else {\n            if (this.options.interval === 1)\n                this.add(gettext('week'));\n            if (this.origOptions.bymonth) {\n                this.add(gettext('in'));\n                this._bymonth();\n            }\n            if (this.bymonthday) {\n                this._bymonthday();\n            }\n            else if (this.byweekday) {\n                this._byweekday();\n            }\n            if (this.origOptions.byhour) {\n                this._byhour();\n            }\n        }\n    };\n    ToText.prototype.MONTHLY = function () {\n        var gettext = this.gettext;\n        if (this.origOptions.bymonth) {\n            if (this.options.interval !== 1) {\n                this.add(this.options.interval.toString()).add(gettext('months'));\n                if (this.plural(this.options.interval))\n                    this.add(gettext('in'));\n            }\n            else {\n                // this.add(gettext('MONTH'))\n            }\n            this._bymonth();\n        }\n        else {\n            if (this.options.interval !== 1) {\n                this.add(this.options.interval.toString());\n            }\n            this.add(this.plural(this.options.interval)\n                ? gettext('months')\n                : gettext('month'));\n        }\n        if (this.bymonthday) {\n            this._bymonthday();\n        }\n        else if (this.byweekday && this.byweekday.isWeekdays) {\n            this.add(gettext('on')).add(gettext('weekdays'));\n        }\n        else if (this.byweekday) {\n            this._byweekday();\n        }\n    };\n    ToText.prototype.YEARLY = function () {\n        var gettext = this.gettext;\n        if (this.origOptions.bymonth) {\n            if (this.options.interval !== 1) {\n                this.add(this.options.interval.toString());\n                this.add(gettext('years'));\n            }\n            else {\n                // this.add(gettext('YEAR'))\n            }\n            this._bymonth();\n        }\n        else {\n            if (this.options.interval !== 1) {\n                this.add(this.options.interval.toString());\n            }\n            this.add(this.plural(this.options.interval) ? gettext('years') : gettext('year'));\n        }\n        if (this.bymonthday) {\n            this._bymonthday();\n        }\n        else if (this.byweekday) {\n            this._byweekday();\n        }\n        if (this.options.byyearday) {\n            this.add(gettext('on the'))\n                .add(this.list(this.options.byyearday, this.nth, gettext('and')))\n                .add(gettext('day'));\n        }\n        if (this.options.byweekno) {\n            this.add(gettext('in'))\n                .add(this.plural(this.options.byweekno.length)\n                ? gettext('weeks')\n                : gettext('week'))\n                .add(this.list(this.options.byweekno, undefined, gettext('and')));\n        }\n    };\n    ToText.prototype._bymonthday = function () {\n        var gettext = this.gettext;\n        if (this.byweekday && this.byweekday.allWeeks) {\n            this.add(gettext('on'))\n                .add(this.list(this.byweekday.allWeeks, this.weekdaytext, gettext('or')))\n                .add(gettext('the'))\n                .add(this.list(this.bymonthday, this.nth, gettext('or')));\n        }\n        else {\n            this.add(gettext('on the')).add(this.list(this.bymonthday, this.nth, gettext('and')));\n        }\n        // this.add(gettext('DAY'))\n    };\n    ToText.prototype._byweekday = function () {\n        var gettext = this.gettext;\n        if (this.byweekday.allWeeks && !this.byweekday.isWeekdays) {\n            this.add(gettext('on')).add(this.list(this.byweekday.allWeeks, this.weekdaytext));\n        }\n        if (this.byweekday.someWeeks) {\n            if (this.byweekday.allWeeks)\n                this.add(gettext('and'));\n            this.add(gettext('on the')).add(this.list(this.byweekday.someWeeks, this.weekdaytext, gettext('and')));\n        }\n    };\n    ToText.prototype._byhour = function () {\n        var gettext = this.gettext;\n        this.add(gettext('at')).add(this.list(this.origOptions.byhour, undefined, gettext('and')));\n    };\n    ToText.prototype._bymonth = function () {\n        this.add(this.list(this.options.bymonth, this.monthtext, this.gettext('and')));\n    };\n    ToText.prototype.nth = function (n) {\n        n = parseInt(n.toString(), 10);\n        var nth;\n        var gettext = this.gettext;\n        if (n === -1)\n            return gettext('last');\n        var npos = Math.abs(n);\n        switch (npos) {\n            case 1:\n            case 21:\n            case 31:\n                nth = npos + gettext('st');\n                break;\n            case 2:\n            case 22:\n                nth = npos + gettext('nd');\n                break;\n            case 3:\n            case 23:\n                nth = npos + gettext('rd');\n                break;\n            default:\n                nth = npos + gettext('th');\n        }\n        return n < 0 ? nth + ' ' + gettext('last') : nth;\n    };\n    ToText.prototype.monthtext = function (m) {\n        return this.language.monthNames[m - 1];\n    };\n    ToText.prototype.weekdaytext = function (wday) {\n        var weekday = isNumber(wday) ? (wday + 1) % 7 : wday.getJsWeekday();\n        return ((wday.n ? this.nth(wday.n) + ' ' : '') +\n            this.language.dayNames[weekday]);\n    };\n    ToText.prototype.plural = function (n) {\n        return n % 100 !== 1;\n    };\n    ToText.prototype.add = function (s) {\n        this.text.push(' ');\n        this.text.push(s);\n        return this;\n    };\n    ToText.prototype.list = function (arr, callback, finalDelim, delim) {\n        var _this = this;\n        if (delim === void 0) { delim = ','; }\n        if (!isArray(arr)) {\n            arr = [arr];\n        }\n        var delimJoin = function (array, delimiter, finalDelimiter) {\n            var list = '';\n            for (var i = 0; i < array.length; i++) {\n                if (i !== 0) {\n                    if (i === array.length - 1) {\n                        list += ' ' + finalDelimiter + ' ';\n                    }\n                    else {\n                        list += delimiter + ' ';\n                    }\n                }\n                list += array[i];\n            }\n            return list;\n        };\n        callback =\n            callback ||\n                function (o) {\n                    return o.toString();\n                };\n        var realCallback = function (arg) {\n            return callback && callback.call(_this, arg);\n        };\n        if (finalDelim) {\n            return delimJoin(arr.map(realCallback), delim, finalDelim);\n        }\n        else {\n            return arr.map(realCallback).join(delim + ' ');\n        }\n    };\n    return ToText;\n}());\n/* harmony default export */ const totext = (ToText);\n\n;// CONCATENATED MODULE: ./src/nlp/parsetext.ts\n\n\n// =============================================================================\n// Parser\n// =============================================================================\nvar Parser = /** @class */ (function () {\n    function Parser(rules) {\n        this.done = true;\n        this.rules = rules;\n    }\n    Parser.prototype.start = function (text) {\n        this.text = text;\n        this.done = false;\n        return this.nextSymbol();\n    };\n    Parser.prototype.isDone = function () {\n        return this.done && this.symbol === null;\n    };\n    Parser.prototype.nextSymbol = function () {\n        var best;\n        var bestSymbol;\n        this.symbol = null;\n        this.value = null;\n        do {\n            if (this.done)\n                return false;\n            var rule = void 0;\n            best = null;\n            for (var name_1 in this.rules) {\n                rule = this.rules[name_1];\n                var match = rule.exec(this.text);\n                if (match) {\n                    if (best === null || match[0].length > best[0].length) {\n                        best = match;\n                        bestSymbol = name_1;\n                    }\n                }\n            }\n            if (best != null) {\n                this.text = this.text.substr(best[0].length);\n                if (this.text === '')\n                    this.done = true;\n            }\n            if (best == null) {\n                this.done = true;\n                this.symbol = null;\n                this.value = null;\n                return;\n            }\n        } while (bestSymbol === 'SKIP');\n        this.symbol = bestSymbol;\n        this.value = best;\n        return true;\n    };\n    Parser.prototype.accept = function (name) {\n        if (this.symbol === name) {\n            if (this.value) {\n                var v = this.value;\n                this.nextSymbol();\n                return v;\n            }\n            this.nextSymbol();\n            return true;\n        }\n        return false;\n    };\n    Parser.prototype.acceptNumber = function () {\n        return this.accept('number');\n    };\n    Parser.prototype.expect = function (name) {\n        if (this.accept(name))\n            return true;\n        throw new Error('expected ' + name + ' but found ' + this.symbol);\n    };\n    return Parser;\n}());\nfunction parseText(text, language) {\n    if (language === void 0) { language = i18n; }\n    var options = {};\n    var ttr = new Parser(language.tokens);\n    if (!ttr.start(text))\n        return null;\n    S();\n    return options;\n    function S() {\n        // every [n]\n        ttr.expect('every');\n        var n = ttr.acceptNumber();\n        if (n)\n            options.interval = parseInt(n[0], 10);\n        if (ttr.isDone())\n            throw new Error('Unexpected end');\n        switch (ttr.symbol) {\n            case 'day(s)':\n                options.freq = RRule.DAILY;\n                if (ttr.nextSymbol()) {\n                    AT();\n                    F();\n                }\n                break;\n            // FIXME Note: every 2 weekdays != every two weeks on weekdays.\n            // DAILY on weekdays is not a valid rule\n            case 'weekday(s)':\n                options.freq = RRule.WEEKLY;\n                options.byweekday = [RRule.MO, RRule.TU, RRule.WE, RRule.TH, RRule.FR];\n                ttr.nextSymbol();\n                AT();\n                F();\n                break;\n            case 'week(s)':\n                options.freq = RRule.WEEKLY;\n                if (ttr.nextSymbol()) {\n                    ON();\n                    AT();\n                    F();\n                }\n                break;\n            case 'hour(s)':\n                options.freq = RRule.HOURLY;\n                if (ttr.nextSymbol()) {\n                    ON();\n                    F();\n                }\n                break;\n            case 'minute(s)':\n                options.freq = RRule.MINUTELY;\n                if (ttr.nextSymbol()) {\n                    ON();\n                    F();\n                }\n                break;\n            case 'month(s)':\n                options.freq = RRule.MONTHLY;\n                if (ttr.nextSymbol()) {\n                    ON();\n                    F();\n                }\n                break;\n            case 'year(s)':\n                options.freq = RRule.YEARLY;\n                if (ttr.nextSymbol()) {\n                    ON();\n                    F();\n                }\n                break;\n            case 'monday':\n            case 'tuesday':\n            case 'wednesday':\n            case 'thursday':\n            case 'friday':\n            case 'saturday':\n            case 'sunday':\n                options.freq = RRule.WEEKLY;\n                var key = ttr.symbol\n                    .substr(0, 2)\n                    .toUpperCase();\n                options.byweekday = [RRule[key]];\n                if (!ttr.nextSymbol())\n                    return;\n                // TODO check for duplicates\n                while (ttr.accept('comma')) {\n                    if (ttr.isDone())\n                        throw new Error('Unexpected end');\n                    var wkd = decodeWKD();\n                    if (!wkd) {\n                        throw new Error('Unexpected symbol ' + ttr.symbol + ', expected weekday');\n                    }\n                    options.byweekday.push(RRule[wkd]);\n                    ttr.nextSymbol();\n                }\n                AT();\n                MDAYs();\n                F();\n                break;\n            case 'january':\n            case 'february':\n            case 'march':\n            case 'april':\n            case 'may':\n            case 'june':\n            case 'july':\n            case 'august':\n            case 'september':\n            case 'october':\n            case 'november':\n            case 'december':\n                options.freq = RRule.YEARLY;\n                options.bymonth = [decodeM()];\n                if (!ttr.nextSymbol())\n                    return;\n                // TODO check for duplicates\n                while (ttr.accept('comma')) {\n                    if (ttr.isDone())\n                        throw new Error('Unexpected end');\n                    var m = decodeM();\n                    if (!m) {\n                        throw new Error('Unexpected symbol ' + ttr.symbol + ', expected month');\n                    }\n                    options.bymonth.push(m);\n                    ttr.nextSymbol();\n                }\n                ON();\n                F();\n                break;\n            default:\n                throw new Error('Unknown symbol');\n        }\n    }\n    function ON() {\n        var on = ttr.accept('on');\n        var the = ttr.accept('the');\n        if (!(on || the))\n            return;\n        do {\n            var nth = decodeNTH();\n            var wkd = decodeWKD();\n            var m = decodeM();\n            // nth <weekday> | <weekday>\n            if (nth) {\n                // ttr.nextSymbol()\n                if (wkd) {\n                    ttr.nextSymbol();\n                    if (!options.byweekday)\n                        options.byweekday = [];\n                    options.byweekday.push(RRule[wkd].nth(nth));\n                }\n                else {\n                    if (!options.bymonthday)\n                        options.bymonthday = [];\n                    options.bymonthday.push(nth);\n                    ttr.accept('day(s)');\n                }\n                // <weekday>\n            }\n            else if (wkd) {\n                ttr.nextSymbol();\n                if (!options.byweekday)\n                    options.byweekday = [];\n                options.byweekday.push(RRule[wkd]);\n            }\n            else if (ttr.symbol === 'weekday(s)') {\n                ttr.nextSymbol();\n                if (!options.byweekday) {\n                    options.byweekday = [RRule.MO, RRule.TU, RRule.WE, RRule.TH, RRule.FR];\n                }\n            }\n            else if (ttr.symbol === 'week(s)') {\n                ttr.nextSymbol();\n                var n = ttr.acceptNumber();\n                if (!n) {\n                    throw new Error('Unexpected symbol ' + ttr.symbol + ', expected week number');\n                }\n                options.byweekno = [parseInt(n[0], 10)];\n                while (ttr.accept('comma')) {\n                    n = ttr.acceptNumber();\n                    if (!n) {\n                        throw new Error('Unexpected symbol ' + ttr.symbol + '; expected monthday');\n                    }\n                    options.byweekno.push(parseInt(n[0], 10));\n                }\n            }\n            else if (m) {\n                ttr.nextSymbol();\n                if (!options.bymonth)\n                    options.bymonth = [];\n                options.bymonth.push(m);\n            }\n            else {\n                return;\n            }\n        } while (ttr.accept('comma') || ttr.accept('the') || ttr.accept('on'));\n    }\n    function AT() {\n        var at = ttr.accept('at');\n        if (!at)\n            return;\n        do {\n            var n = ttr.acceptNumber();\n            if (!n) {\n                throw new Error('Unexpected symbol ' + ttr.symbol + ', expected hour');\n            }\n            options.byhour = [parseInt(n[0], 10)];\n            while (ttr.accept('comma')) {\n                n = ttr.acceptNumber();\n                if (!n) {\n                    throw new Error('Unexpected symbol ' + ttr.symbol + '; expected hour');\n                }\n                options.byhour.push(parseInt(n[0], 10));\n            }\n        } while (ttr.accept('comma') || ttr.accept('at'));\n    }\n    function decodeM() {\n        switch (ttr.symbol) {\n            case 'january':\n                return 1;\n            case 'february':\n                return 2;\n            case 'march':\n                return 3;\n            case 'april':\n                return 4;\n            case 'may':\n                return 5;\n            case 'june':\n                return 6;\n            case 'july':\n                return 7;\n            case 'august':\n                return 8;\n            case 'september':\n                return 9;\n            case 'october':\n                return 10;\n            case 'november':\n                return 11;\n            case 'december':\n                return 12;\n            default:\n                return false;\n        }\n    }\n    function decodeWKD() {\n        switch (ttr.symbol) {\n            case 'monday':\n            case 'tuesday':\n            case 'wednesday':\n            case 'thursday':\n            case 'friday':\n            case 'saturday':\n            case 'sunday':\n                return ttr.symbol.substr(0, 2).toUpperCase();\n            default:\n                return false;\n        }\n    }\n    function decodeNTH() {\n        switch (ttr.symbol) {\n            case 'last':\n                ttr.nextSymbol();\n                return -1;\n            case 'first':\n                ttr.nextSymbol();\n                return 1;\n            case 'second':\n                ttr.nextSymbol();\n                return ttr.accept('last') ? -2 : 2;\n            case 'third':\n                ttr.nextSymbol();\n                return ttr.accept('last') ? -3 : 3;\n            case 'nth':\n                var v = parseInt(ttr.value[1], 10);\n                if (v < -366 || v > 366)\n                    throw new Error('Nth out of range: ' + v);\n                ttr.nextSymbol();\n                return ttr.accept('last') ? -v : v;\n            default:\n                return false;\n        }\n    }\n    function MDAYs() {\n        ttr.accept('on');\n        ttr.accept('the');\n        var nth = decodeNTH();\n        if (!nth)\n            return;\n        options.bymonthday = [nth];\n        ttr.nextSymbol();\n        while (ttr.accept('comma')) {\n            nth = decodeNTH();\n            if (!nth) {\n                throw new Error('Unexpected symbol ' + ttr.symbol + '; expected monthday');\n            }\n            options.bymonthday.push(nth);\n            ttr.nextSymbol();\n        }\n    }\n    function F() {\n        if (ttr.symbol === 'until') {\n            var date = Date.parse(ttr.text);\n            if (!date)\n                throw new Error('Cannot parse until date:' + ttr.text);\n            options.until = new Date(date);\n        }\n        else if (ttr.accept('for')) {\n            options.count = parseInt(ttr.value[0], 10);\n            ttr.expect('number');\n            // ttr.expect('times')\n        }\n    }\n}\n\n;// CONCATENATED MODULE: ./src/types.ts\nvar Frequency;\n(function (Frequency) {\n    Frequency[Frequency[\"YEARLY\"] = 0] = \"YEARLY\";\n    Frequency[Frequency[\"MONTHLY\"] = 1] = \"MONTHLY\";\n    Frequency[Frequency[\"WEEKLY\"] = 2] = \"WEEKLY\";\n    Frequency[Frequency[\"DAILY\"] = 3] = \"DAILY\";\n    Frequency[Frequency[\"HOURLY\"] = 4] = \"HOURLY\";\n    Frequency[Frequency[\"MINUTELY\"] = 5] = \"MINUTELY\";\n    Frequency[Frequency[\"SECONDLY\"] = 6] = \"SECONDLY\";\n})(Frequency || (Frequency = {}));\nfunction freqIsDailyOrGreater(freq) {\n    return freq < Frequency.HOURLY;\n}\n\n;// CONCATENATED MODULE: ./src/nlp/index.ts\n\n\n\n\n\n/* !\n * rrule.js - Library for working with recurrence rules for calendar dates.\n * https://github.com/jakubroztocil/rrule\n *\n * Copyright 2010, Jakub Roztocil and Lars Schoning\n * Licenced under the BSD licence.\n * https://github.com/jakubroztocil/rrule/blob/master/LICENCE\n *\n */\n/**\n *\n * Implementation of RRule.fromText() and RRule::toText().\n *\n *\n * On the client side, this file needs to be included\n * when those functions are used.\n *\n */\n// =============================================================================\n// fromText\n// =============================================================================\n/**\n * Will be able to convert some of the below described rules from\n * text format to a rule object.\n *\n *\n * RULES\n *\n * Every ([n])\n * day(s)\n * | [weekday], ..., (and) [weekday]\n * | weekday(s)\n * | week(s)\n * | month(s)\n * | [month], ..., (and) [month]\n * | year(s)\n *\n *\n * Plus 0, 1, or multiple of these:\n *\n * on [weekday], ..., (or) [weekday] the [monthday], [monthday], ... (or) [monthday]\n *\n * on [weekday], ..., (and) [weekday]\n *\n * on the [monthday], [monthday], ... (and) [monthday] (day of the month)\n *\n * on the [nth-weekday], ..., (and) [nth-weekday] (of the month/year)\n *\n *\n * Plus 0 or 1 of these:\n *\n * for [n] time(s)\n *\n * until [date]\n *\n * Plus (.)\n *\n *\n * Definitely no supported for parsing:\n *\n * (for year):\n * in week(s) [n], ..., (and) [n]\n *\n * on the [yearday], ..., (and) [n] day of the year\n * on day [yearday], ..., (and) [n]\n *\n *\n * NON-TERMINALS\n *\n * [n]: 1, 2 ..., one, two, three ..\n * [month]: January, February, March, April, May, ... December\n * [weekday]: Monday, ... Sunday\n * [nth-weekday]: first [weekday], 2nd [weekday], ... last [weekday], ...\n * [monthday]: first, 1., 2., 1st, 2nd, second, ... 31st, last day, 2nd last day, ..\n * [date]:\n * - [month] (0-31(,) ([year])),\n * - (the) 0-31.(1-12.([year])),\n * - (the) 0-31/(1-12/([year])),\n * - [weekday]\n *\n * [year]: 0000, 0001, ... 01, 02, ..\n *\n * Definitely not supported for parsing:\n *\n * [yearday]: first, 1., 2., 1st, 2nd, second, ... 366th, last day, 2nd last day, ..\n *\n * @param {String} text\n * @return {Object, Boolean} the rule, or null.\n */\nvar fromText = function (text, language) {\n    if (language === void 0) { language = i18n; }\n    return new RRule(parseText(text, language) || undefined);\n};\nvar common = [\n    'count',\n    'until',\n    'interval',\n    'byweekday',\n    'bymonthday',\n    'bymonth',\n];\ntotext.IMPLEMENTED = [];\ntotext.IMPLEMENTED[Frequency.HOURLY] = common;\ntotext.IMPLEMENTED[Frequency.MINUTELY] = common;\ntotext.IMPLEMENTED[Frequency.DAILY] = ['byhour'].concat(common);\ntotext.IMPLEMENTED[Frequency.WEEKLY] = common;\ntotext.IMPLEMENTED[Frequency.MONTHLY] = common;\ntotext.IMPLEMENTED[Frequency.YEARLY] = ['byweekno', 'byyearday'].concat(common);\n// =============================================================================\n// Export\n// =============================================================================\nvar toText = function (rrule, gettext, language, dateFormatter) {\n    return new totext(rrule, gettext, language, dateFormatter).toString();\n};\nvar isFullyConvertible = totext.isFullyConvertible;\n\n\n;// CONCATENATED MODULE: ./src/datetime.ts\n\n\n\n\nvar Time = /** @class */ (function () {\n    function Time(hour, minute, second, millisecond) {\n        this.hour = hour;\n        this.minute = minute;\n        this.second = second;\n        this.millisecond = millisecond || 0;\n    }\n    Time.prototype.getHours = function () {\n        return this.hour;\n    };\n    Time.prototype.getMinutes = function () {\n        return this.minute;\n    };\n    Time.prototype.getSeconds = function () {\n        return this.second;\n    };\n    Time.prototype.getMilliseconds = function () {\n        return this.millisecond;\n    };\n    Time.prototype.getTime = function () {\n        return ((this.hour * 60 * 60 + this.minute * 60 + this.second) * 1000 +\n            this.millisecond);\n    };\n    return Time;\n}());\n\nvar DateTime = /** @class */ (function (_super) {\n    __extends(DateTime, _super);\n    function DateTime(year, month, day, hour, minute, second, millisecond) {\n        var _this = _super.call(this, hour, minute, second, millisecond) || this;\n        _this.year = year;\n        _this.month = month;\n        _this.day = day;\n        return _this;\n    }\n    DateTime.fromDate = function (date) {\n        return new this(date.getUTCFullYear(), date.getUTCMonth() + 1, date.getUTCDate(), date.getUTCHours(), date.getUTCMinutes(), date.getUTCSeconds(), date.valueOf() % 1000);\n    };\n    DateTime.prototype.getWeekday = function () {\n        return getWeekday(new Date(this.getTime()));\n    };\n    DateTime.prototype.getTime = function () {\n        return new Date(Date.UTC(this.year, this.month - 1, this.day, this.hour, this.minute, this.second, this.millisecond)).getTime();\n    };\n    DateTime.prototype.getDay = function () {\n        return this.day;\n    };\n    DateTime.prototype.getMonth = function () {\n        return this.month;\n    };\n    DateTime.prototype.getYear = function () {\n        return this.year;\n    };\n    DateTime.prototype.addYears = function (years) {\n        this.year += years;\n    };\n    DateTime.prototype.addMonths = function (months) {\n        this.month += months;\n        if (this.month > 12) {\n            var yearDiv = Math.floor(this.month / 12);\n            var monthMod = pymod(this.month, 12);\n            this.month = monthMod;\n            this.year += yearDiv;\n            if (this.month === 0) {\n                this.month = 12;\n                --this.year;\n            }\n        }\n    };\n    DateTime.prototype.addWeekly = function (days, wkst) {\n        if (wkst > this.getWeekday()) {\n            this.day += -(this.getWeekday() + 1 + (6 - wkst)) + days * 7;\n        }\n        else {\n            this.day += -(this.getWeekday() - wkst) + days * 7;\n        }\n        this.fixDay();\n    };\n    DateTime.prototype.addDaily = function (days) {\n        this.day += days;\n        this.fixDay();\n    };\n    DateTime.prototype.addHours = function (hours, filtered, byhour) {\n        if (filtered) {\n            // Jump to one iteration before next day\n            this.hour += Math.floor((23 - this.hour) / hours) * hours;\n        }\n        for (;;) {\n            this.hour += hours;\n            var _a = divmod(this.hour, 24), dayDiv = _a.div, hourMod = _a.mod;\n            if (dayDiv) {\n                this.hour = hourMod;\n                this.addDaily(dayDiv);\n            }\n            if (empty(byhour) || includes(byhour, this.hour))\n                break;\n        }\n    };\n    DateTime.prototype.addMinutes = function (minutes, filtered, byhour, byminute) {\n        if (filtered) {\n            // Jump to one iteration before next day\n            this.minute +=\n                Math.floor((1439 - (this.hour * 60 + this.minute)) / minutes) * minutes;\n        }\n        for (;;) {\n            this.minute += minutes;\n            var _a = divmod(this.minute, 60), hourDiv = _a.div, minuteMod = _a.mod;\n            if (hourDiv) {\n                this.minute = minuteMod;\n                this.addHours(hourDiv, false, byhour);\n            }\n            if ((empty(byhour) || includes(byhour, this.hour)) &&\n                (empty(byminute) || includes(byminute, this.minute))) {\n                break;\n            }\n        }\n    };\n    DateTime.prototype.addSeconds = function (seconds, filtered, byhour, byminute, bysecond) {\n        if (filtered) {\n            // Jump to one iteration before next day\n            this.second +=\n                Math.floor((86399 - (this.hour * 3600 + this.minute * 60 + this.second)) /\n                    seconds) * seconds;\n        }\n        for (;;) {\n            this.second += seconds;\n            var _a = divmod(this.second, 60), minuteDiv = _a.div, secondMod = _a.mod;\n            if (minuteDiv) {\n                this.second = secondMod;\n                this.addMinutes(minuteDiv, false, byhour, byminute);\n            }\n            if ((empty(byhour) || includes(byhour, this.hour)) &&\n                (empty(byminute) || includes(byminute, this.minute)) &&\n                (empty(bysecond) || includes(bysecond, this.second))) {\n                break;\n            }\n        }\n    };\n    DateTime.prototype.fixDay = function () {\n        if (this.day <= 28) {\n            return;\n        }\n        var daysinmonth = monthRange(this.year, this.month - 1)[1];\n        if (this.day <= daysinmonth) {\n            return;\n        }\n        while (this.day > daysinmonth) {\n            this.day -= daysinmonth;\n            ++this.month;\n            if (this.month === 13) {\n                this.month = 1;\n                ++this.year;\n                if (this.year > MAXYEAR) {\n                    return;\n                }\n            }\n            daysinmonth = monthRange(this.year, this.month - 1)[1];\n        }\n    };\n    DateTime.prototype.add = function (options, filtered) {\n        var freq = options.freq, interval = options.interval, wkst = options.wkst, byhour = options.byhour, byminute = options.byminute, bysecond = options.bysecond;\n        switch (freq) {\n            case Frequency.YEARLY:\n                return this.addYears(interval);\n            case Frequency.MONTHLY:\n                return this.addMonths(interval);\n            case Frequency.WEEKLY:\n                return this.addWeekly(interval, wkst);\n            case Frequency.DAILY:\n                return this.addDaily(interval);\n            case Frequency.HOURLY:\n                return this.addHours(interval, filtered, byhour);\n            case Frequency.MINUTELY:\n                return this.addMinutes(interval, filtered, byhour, byminute);\n            case Frequency.SECONDLY:\n                return this.addSeconds(interval, filtered, byhour, byminute, bysecond);\n        }\n    };\n    return DateTime;\n}(Time));\n\n\n;// CONCATENATED MODULE: ./src/parseoptions.ts\n\n\n\n\n\n\n\nfunction initializeOptions(options) {\n    var invalid = [];\n    var keys = Object.keys(options);\n    // Shallow copy for options and origOptions and check for invalid\n    for (var _i = 0, keys_1 = keys; _i < keys_1.length; _i++) {\n        var key = keys_1[_i];\n        if (!includes(defaultKeys, key))\n            invalid.push(key);\n        if (isDate(options[key]) && !isValidDate(options[key])) {\n            invalid.push(key);\n        }\n    }\n    if (invalid.length) {\n        throw new Error('Invalid options: ' + invalid.join(', '));\n    }\n    return __assign({}, options);\n}\nfunction parseOptions(options) {\n    var opts = __assign(__assign({}, DEFAULT_OPTIONS), initializeOptions(options));\n    if (isPresent(opts.byeaster))\n        opts.freq = RRule.YEARLY;\n    if (!(isPresent(opts.freq) && RRule.FREQUENCIES[opts.freq])) {\n        throw new Error(\"Invalid frequency: \".concat(opts.freq, \" \").concat(options.freq));\n    }\n    if (!opts.dtstart)\n        opts.dtstart = new Date(new Date().setMilliseconds(0));\n    if (!isPresent(opts.wkst)) {\n        opts.wkst = RRule.MO.weekday;\n    }\n    else if (isNumber(opts.wkst)) {\n        // cool, just keep it like that\n    }\n    else {\n        opts.wkst = opts.wkst.weekday;\n    }\n    if (isPresent(opts.bysetpos)) {\n        if (isNumber(opts.bysetpos))\n            opts.bysetpos = [opts.bysetpos];\n        for (var i = 0; i < opts.bysetpos.length; i++) {\n            var v = opts.bysetpos[i];\n            if (v === 0 || !(v >= -366 && v <= 366)) {\n                throw new Error('bysetpos must be between 1 and 366,' + ' or between -366 and -1');\n            }\n        }\n    }\n    if (!(Boolean(opts.byweekno) ||\n        notEmpty(opts.byweekno) ||\n        notEmpty(opts.byyearday) ||\n        Boolean(opts.bymonthday) ||\n        notEmpty(opts.bymonthday) ||\n        isPresent(opts.byweekday) ||\n        isPresent(opts.byeaster))) {\n        switch (opts.freq) {\n            case RRule.YEARLY:\n                if (!opts.bymonth)\n                    opts.bymonth = opts.dtstart.getUTCMonth() + 1;\n                opts.bymonthday = opts.dtstart.getUTCDate();\n                break;\n            case RRule.MONTHLY:\n                opts.bymonthday = opts.dtstart.getUTCDate();\n                break;\n            case RRule.WEEKLY:\n                opts.byweekday = [getWeekday(opts.dtstart)];\n                break;\n        }\n    }\n    // bymonth\n    if (isPresent(opts.bymonth) && !isArray(opts.bymonth)) {\n        opts.bymonth = [opts.bymonth];\n    }\n    // byyearday\n    if (isPresent(opts.byyearday) &&\n        !isArray(opts.byyearday) &&\n        isNumber(opts.byyearday)) {\n        opts.byyearday = [opts.byyearday];\n    }\n    // bymonthday\n    if (!isPresent(opts.bymonthday)) {\n        opts.bymonthday = [];\n        opts.bynmonthday = [];\n    }\n    else if (isArray(opts.bymonthday)) {\n        var bymonthday = [];\n        var bynmonthday = [];\n        for (var i = 0; i < opts.bymonthday.length; i++) {\n            var v = opts.bymonthday[i];\n            if (v > 0) {\n                bymonthday.push(v);\n            }\n            else if (v < 0) {\n                bynmonthday.push(v);\n            }\n        }\n        opts.bymonthday = bymonthday;\n        opts.bynmonthday = bynmonthday;\n    }\n    else if (opts.bymonthday < 0) {\n        opts.bynmonthday = [opts.bymonthday];\n        opts.bymonthday = [];\n    }\n    else {\n        opts.bynmonthday = [];\n        opts.bymonthday = [opts.bymonthday];\n    }\n    // byweekno\n    if (isPresent(opts.byweekno) && !isArray(opts.byweekno)) {\n        opts.byweekno = [opts.byweekno];\n    }\n    // byweekday / bynweekday\n    if (!isPresent(opts.byweekday)) {\n        opts.bynweekday = null;\n    }\n    else if (isNumber(opts.byweekday)) {\n        opts.byweekday = [opts.byweekday];\n        opts.bynweekday = null;\n    }\n    else if (isWeekdayStr(opts.byweekday)) {\n        opts.byweekday = [Weekday.fromStr(opts.byweekday).weekday];\n        opts.bynweekday = null;\n    }\n    else if (opts.byweekday instanceof Weekday) {\n        if (!opts.byweekday.n || opts.freq > RRule.MONTHLY) {\n            opts.byweekday = [opts.byweekday.weekday];\n            opts.bynweekday = null;\n        }\n        else {\n            opts.bynweekday = [[opts.byweekday.weekday, opts.byweekday.n]];\n            opts.byweekday = null;\n        }\n    }\n    else {\n        var byweekday = [];\n        var bynweekday = [];\n        for (var i = 0; i < opts.byweekday.length; i++) {\n            var wday = opts.byweekday[i];\n            if (isNumber(wday)) {\n                byweekday.push(wday);\n                continue;\n            }\n            else if (isWeekdayStr(wday)) {\n                byweekday.push(Weekday.fromStr(wday).weekday);\n                continue;\n            }\n            if (!wday.n || opts.freq > RRule.MONTHLY) {\n                byweekday.push(wday.weekday);\n            }\n            else {\n                bynweekday.push([wday.weekday, wday.n]);\n            }\n        }\n        opts.byweekday = notEmpty(byweekday) ? byweekday : null;\n        opts.bynweekday = notEmpty(bynweekday) ? bynweekday : null;\n    }\n    // byhour\n    if (!isPresent(opts.byhour)) {\n        opts.byhour = opts.freq < RRule.HOURLY ? [opts.dtstart.getUTCHours()] : null;\n    }\n    else if (isNumber(opts.byhour)) {\n        opts.byhour = [opts.byhour];\n    }\n    // byminute\n    if (!isPresent(opts.byminute)) {\n        opts.byminute =\n            opts.freq < RRule.MINUTELY ? [opts.dtstart.getUTCMinutes()] : null;\n    }\n    else if (isNumber(opts.byminute)) {\n        opts.byminute = [opts.byminute];\n    }\n    // bysecond\n    if (!isPresent(opts.bysecond)) {\n        opts.bysecond =\n            opts.freq < RRule.SECONDLY ? [opts.dtstart.getUTCSeconds()] : null;\n    }\n    else if (isNumber(opts.bysecond)) {\n        opts.bysecond = [opts.bysecond];\n    }\n    return { parsedOptions: opts };\n}\nfunction buildTimeset(opts) {\n    var millisecondModulo = opts.dtstart.getTime() % 1000;\n    if (!freqIsDailyOrGreater(opts.freq)) {\n        return [];\n    }\n    var timeset = [];\n    opts.byhour.forEach(function (hour) {\n        opts.byminute.forEach(function (minute) {\n            opts.bysecond.forEach(function (second) {\n                timeset.push(new Time(hour, minute, second, millisecondModulo));\n            });\n        });\n    });\n    return timeset;\n}\n\n;// CONCATENATED MODULE: ./src/parsestring.ts\n\n\n\n\n\nfunction parseString(rfcString) {\n    var options = rfcString\n        .split('\\n')\n        .map(parseLine)\n        .filter(function (x) { return x !== null; });\n    return __assign(__assign({}, options[0]), options[1]);\n}\nfunction parseDtstart(line) {\n    var options = {};\n    var dtstartWithZone = /DTSTART(?:;TZID=([^:=]+?))?(?::|=)([^;\\s]+)/i.exec(line);\n    if (!dtstartWithZone) {\n        return options;\n    }\n    var tzid = dtstartWithZone[1], dtstart = dtstartWithZone[2];\n    if (tzid) {\n        options.tzid = tzid;\n    }\n    options.dtstart = untilStringToDate(dtstart);\n    return options;\n}\nfunction parseLine(rfcString) {\n    rfcString = rfcString.replace(/^\\s+|\\s+$/, '');\n    if (!rfcString.length)\n        return null;\n    var header = /^([A-Z]+?)[:;]/.exec(rfcString.toUpperCase());\n    if (!header) {\n        return parseRrule(rfcString);\n    }\n    var key = header[1];\n    switch (key.toUpperCase()) {\n        case 'RRULE':\n        case 'EXRULE':\n            return parseRrule(rfcString);\n        case 'DTSTART':\n            return parseDtstart(rfcString);\n        default:\n            throw new Error(\"Unsupported RFC prop \".concat(key, \" in \").concat(rfcString));\n    }\n}\nfunction parseRrule(line) {\n    var strippedLine = line.replace(/^RRULE:/i, '');\n    var options = parseDtstart(strippedLine);\n    var attrs = line.replace(/^(?:RRULE|EXRULE):/i, '').split(';');\n    attrs.forEach(function (attr) {\n        var _a = attr.split('='), key = _a[0], value = _a[1];\n        switch (key.toUpperCase()) {\n            case 'FREQ':\n                options.freq = Frequency[value.toUpperCase()];\n                break;\n            case 'WKST':\n                options.wkst = Days[value.toUpperCase()];\n                break;\n            case 'COUNT':\n            case 'INTERVAL':\n            case 'BYSETPOS':\n            case 'BYMONTH':\n            case 'BYMONTHDAY':\n            case 'BYYEARDAY':\n            case 'BYWEEKNO':\n            case 'BYHOUR':\n            case 'BYMINUTE':\n            case 'BYSECOND':\n                var num = parseNumber(value);\n                var optionKey = key.toLowerCase();\n                // eslint-disable-next-line @typescript-eslint/ban-ts-comment\n                // @ts-ignore\n                options[optionKey] = num;\n                break;\n            case 'BYWEEKDAY':\n            case 'BYDAY':\n                options.byweekday = parseWeekday(value);\n                break;\n            case 'DTSTART':\n            case 'TZID':\n                // for backwards compatibility\n                var dtstart = parseDtstart(line);\n                options.tzid = dtstart.tzid;\n                options.dtstart = dtstart.dtstart;\n                break;\n            case 'UNTIL':\n                options.until = untilStringToDate(value);\n                break;\n            case 'BYEASTER':\n                options.byeaster = Number(value);\n                break;\n            default:\n                throw new Error(\"Unknown RRULE property '\" + key + \"'\");\n        }\n    });\n    return options;\n}\nfunction parseNumber(value) {\n    if (value.indexOf(',') !== -1) {\n        var values = value.split(',');\n        return values.map(parseIndividualNumber);\n    }\n    return parseIndividualNumber(value);\n}\nfunction parseIndividualNumber(value) {\n    if (/^[+-]?\\d+$/.test(value)) {\n        return Number(value);\n    }\n    return value;\n}\nfunction parseWeekday(value) {\n    var days = value.split(',');\n    return days.map(function (day) {\n        if (day.length === 2) {\n            // MO, TU, ...\n            return Days[day]; // wday instanceof Weekday\n        }\n        // -1MO, +3FR, 1SO, 13TU ...\n        var parts = day.match(/^([+-]?\\d{1,2})([A-Z]{2})$/);\n        if (!parts || parts.length < 3) {\n            throw new SyntaxError(\"Invalid weekday string: \".concat(day));\n        }\n        var n = Number(parts[1]);\n        var wdaypart = parts[2];\n        var wday = Days[wdaypart].weekday;\n        return new Weekday(wday, n);\n    });\n}\n\n;// CONCATENATED MODULE: ./src/datewithzone.ts\n\nvar DateWithZone = /** @class */ (function () {\n    function DateWithZone(date, tzid) {\n        if (isNaN(date.getTime())) {\n            throw new RangeError('Invalid date passed to DateWithZone');\n        }\n        this.date = date;\n        this.tzid = tzid;\n    }\n    Object.defineProperty(DateWithZone.prototype, \"isUTC\", {\n        get: function () {\n            return !this.tzid || this.tzid.toUpperCase() === 'UTC';\n        },\n        enumerable: false,\n        configurable: true\n    });\n    DateWithZone.prototype.toString = function () {\n        var datestr = timeToUntilString(this.date.getTime(), this.isUTC);\n        if (!this.isUTC) {\n            return \";TZID=\".concat(this.tzid, \":\").concat(datestr);\n        }\n        return \":\".concat(datestr);\n    };\n    DateWithZone.prototype.getTime = function () {\n        return this.date.getTime();\n    };\n    DateWithZone.prototype.rezonedDate = function () {\n        if (this.isUTC) {\n            return this.date;\n        }\n        return dateInTimeZone(this.date, this.tzid);\n    };\n    return DateWithZone;\n}());\n\n\n;// CONCATENATED MODULE: ./src/optionstostring.ts\n\n\n\n\n\nfunction optionsToString(options) {\n    var rrule = [];\n    var dtstart = '';\n    var keys = Object.keys(options);\n    var defaultKeys = Object.keys(DEFAULT_OPTIONS);\n    for (var i = 0; i < keys.length; i++) {\n        if (keys[i] === 'tzid')\n            continue;\n        if (!includes(defaultKeys, keys[i]))\n            continue;\n        var key = keys[i].toUpperCase();\n        var value = options[keys[i]];\n        var outValue = '';\n        if (!isPresent(value) || (isArray(value) && !value.length))\n            continue;\n        switch (key) {\n            case 'FREQ':\n                outValue = RRule.FREQUENCIES[options.freq];\n                break;\n            case 'WKST':\n                if (isNumber(value)) {\n                    outValue = new Weekday(value).toString();\n                }\n                else {\n                    outValue = value.toString();\n                }\n                break;\n            case 'BYWEEKDAY':\n                /*\n                  NOTE: BYWEEKDAY is a special case.\n                  RRule() deconstructs the rule.options.byweekday array\n                  into an array of Weekday arguments.\n                  On the other hand, rule.origOptions is an array of Weekdays.\n                  We need to handle both cases here.\n                  It might be worth change RRule to keep the Weekdays.\n        \n                  Also, BYWEEKDAY (used by RRule) vs. BYDAY (RFC)\n        \n                  */\n                key = 'BYDAY';\n                outValue = toArray(value)\n                    .map(function (wday) {\n                    if (wday instanceof Weekday) {\n                        return wday;\n                    }\n                    if (isArray(wday)) {\n                        return new Weekday(wday[0], wday[1]);\n                    }\n                    return new Weekday(wday);\n                })\n                    .toString();\n                break;\n            case 'DTSTART':\n                dtstart = buildDtstart(value, options.tzid);\n                break;\n            case 'UNTIL':\n                outValue = timeToUntilString(value, !options.tzid);\n                break;\n            default:\n                if (isArray(value)) {\n                    var strValues = [];\n                    for (var j = 0; j < value.length; j++) {\n                        strValues[j] = String(value[j]);\n                    }\n                    outValue = strValues.toString();\n                }\n                else {\n                    outValue = String(value);\n                }\n        }\n        if (outValue) {\n            rrule.push([key, outValue]);\n        }\n    }\n    var rules = rrule\n        .map(function (_a) {\n        var key = _a[0], value = _a[1];\n        return \"\".concat(key, \"=\").concat(value.toString());\n    })\n        .join(';');\n    var ruleString = '';\n    if (rules !== '') {\n        ruleString = \"RRULE:\".concat(rules);\n    }\n    return [dtstart, ruleString].filter(function (x) { return !!x; }).join('\\n');\n}\nfunction buildDtstart(dtstart, tzid) {\n    if (!dtstart) {\n        return '';\n    }\n    return 'DTSTART' + new DateWithZone(new Date(dtstart), tzid).toString();\n}\n\n;// CONCATENATED MODULE: ./src/cache.ts\n\n\n\nfunction argsMatch(left, right) {\n    if (Array.isArray(left)) {\n        if (!Array.isArray(right))\n            return false;\n        if (left.length !== right.length)\n            return false;\n        return left.every(function (date, i) { return date.getTime() === right[i].getTime(); });\n    }\n    if (left instanceof Date) {\n        return right instanceof Date && left.getTime() === right.getTime();\n    }\n    return left === right;\n}\nvar Cache = /** @class */ (function () {\n    function Cache() {\n        this.all = false;\n        this.before = [];\n        this.after = [];\n        this.between = [];\n    }\n    /**\n     * @param {String} what - all/before/after/between\n     * @param {Array,Date} value - an array of dates, one date, or null\n     * @param {Object?} args - _iter arguments\n     */\n    Cache.prototype._cacheAdd = function (what, value, args) {\n        if (value) {\n            value = value instanceof Date ? dateutil_clone(value) : cloneDates(value);\n        }\n        if (what === 'all') {\n            this.all = value;\n        }\n        else {\n            args._value = value;\n            this[what].push(args);\n        }\n    };\n    /**\n     * @return false - not in the cache\n     * @return null  - cached, but zero occurrences (before/after)\n     * @return Date  - cached (before/after)\n     * @return []    - cached, but zero occurrences (all/between)\n     * @return [Date1, DateN] - cached (all/between)\n     */\n    Cache.prototype._cacheGet = function (what, args) {\n        var cached = false;\n        var argsKeys = args ? Object.keys(args) : [];\n        var findCacheDiff = function (item) {\n            for (var i = 0; i < argsKeys.length; i++) {\n                var key = argsKeys[i];\n                if (!argsMatch(args[key], item[key])) {\n                    return true;\n                }\n            }\n            return false;\n        };\n        var cachedObject = this[what];\n        if (what === 'all') {\n            cached = this.all;\n        }\n        else if (isArray(cachedObject)) {\n            // Let's see whether we've already called the\n            // 'what' method with the same 'args'\n            for (var i = 0; i < cachedObject.length; i++) {\n                var item = cachedObject[i];\n                if (argsKeys.length && findCacheDiff(item))\n                    continue;\n                cached = item._value;\n                break;\n            }\n        }\n        if (!cached && this.all) {\n            // Not in the cache, but we already know all the occurrences,\n            // so we can find the correct dates from the cached ones.\n            var iterResult = new iterresult(what, args);\n            for (var i = 0; i < this.all.length; i++) {\n                if (!iterResult.accept(this.all[i]))\n                    break;\n            }\n            cached = iterResult.getValue();\n            this._cacheAdd(what, cached, args);\n        }\n        return isArray(cached)\n            ? cloneDates(cached)\n            : cached instanceof Date\n                ? dateutil_clone(cached)\n                : cached;\n    };\n    return Cache;\n}());\n\n\n;// CONCATENATED MODULE: ./src/masks.ts\n\n\n// =============================================================================\n// Date masks\n// =============================================================================\n// Every mask is 7 days longer to handle cross-year weekly periods.\nvar M365MASK = __spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray([], repeat(1, 31), true), repeat(2, 28), true), repeat(3, 31), true), repeat(4, 30), true), repeat(5, 31), true), repeat(6, 30), true), repeat(7, 31), true), repeat(8, 31), true), repeat(9, 30), true), repeat(10, 31), true), repeat(11, 30), true), repeat(12, 31), true), repeat(1, 7), true);\nvar M366MASK = __spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray([], repeat(1, 31), true), repeat(2, 29), true), repeat(3, 31), true), repeat(4, 30), true), repeat(5, 31), true), repeat(6, 30), true), repeat(7, 31), true), repeat(8, 31), true), repeat(9, 30), true), repeat(10, 31), true), repeat(11, 30), true), repeat(12, 31), true), repeat(1, 7), true);\nvar M28 = range(1, 29);\nvar M29 = range(1, 30);\nvar M30 = range(1, 31);\nvar M31 = range(1, 32);\nvar MDAY366MASK = __spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray([], M31, true), M29, true), M31, true), M30, true), M31, true), M30, true), M31, true), M31, true), M30, true), M31, true), M30, true), M31, true), M31.slice(0, 7), true);\nvar MDAY365MASK = __spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray([], M31, true), M28, true), M31, true), M30, true), M31, true), M30, true), M31, true), M31, true), M30, true), M31, true), M30, true), M31, true), M31.slice(0, 7), true);\nvar NM28 = range(-28, 0);\nvar NM29 = range(-29, 0);\nvar NM30 = range(-30, 0);\nvar NM31 = range(-31, 0);\nvar NMDAY366MASK = __spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray([], NM31, true), NM29, true), NM31, true), NM30, true), NM31, true), NM30, true), NM31, true), NM31, true), NM30, true), NM31, true), NM30, true), NM31, true), NM31.slice(0, 7), true);\nvar NMDAY365MASK = __spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray(__spreadArray([], NM31, true), NM28, true), NM31, true), NM30, true), NM31, true), NM30, true), NM31, true), NM31, true), NM30, true), NM31, true), NM30, true), NM31, true), NM31.slice(0, 7), true);\nvar M366RANGE = [0, 31, 60, 91, 121, 152, 182, 213, 244, 274, 305, 335, 366];\nvar M365RANGE = [0, 31, 59, 90, 120, 151, 181, 212, 243, 273, 304, 334, 365];\nvar WDAYMASK = (function () {\n    var wdaymask = [];\n    for (var i = 0; i < 55; i++)\n        wdaymask = wdaymask.concat(range(7));\n    return wdaymask;\n})();\n\n\n;// CONCATENATED MODULE: ./src/iterinfo/yearinfo.ts\n\n\n\n\nfunction rebuildYear(year, options) {\n    var firstyday = datetime(year, 1, 1);\n    var yearlen = isLeapYear(year) ? 366 : 365;\n    var nextyearlen = isLeapYear(year + 1) ? 366 : 365;\n    var yearordinal = toOrdinal(firstyday);\n    var yearweekday = getWeekday(firstyday);\n    var result = __assign(__assign({ yearlen: yearlen, nextyearlen: nextyearlen, yearordinal: yearordinal, yearweekday: yearweekday }, baseYearMasks(year)), { wnomask: null });\n    if (empty(options.byweekno)) {\n        return result;\n    }\n    result.wnomask = repeat(0, yearlen + 7);\n    var firstwkst;\n    var wyearlen;\n    var no1wkst = (firstwkst = pymod(7 - yearweekday + options.wkst, 7));\n    if (no1wkst >= 4) {\n        no1wkst = 0;\n        // Number of days in the year, plus the days we got\n        // from last year.\n        wyearlen = result.yearlen + pymod(yearweekday - options.wkst, 7);\n    }\n    else {\n        // Number of days in the year, minus the days we\n        // left in last year.\n        wyearlen = yearlen - no1wkst;\n    }\n    var div = Math.floor(wyearlen / 7);\n    var mod = pymod(wyearlen, 7);\n    var numweeks = Math.floor(div + mod / 4);\n    for (var j = 0; j < options.byweekno.length; j++) {\n        var n = options.byweekno[j];\n        if (n < 0) {\n            n += numweeks + 1;\n        }\n        if (!(n > 0 && n <= numweeks)) {\n            continue;\n        }\n        var i = void 0;\n        if (n > 1) {\n            i = no1wkst + (n - 1) * 7;\n            if (no1wkst !== firstwkst) {\n                i -= 7 - firstwkst;\n            }\n        }\n        else {\n            i = no1wkst;\n        }\n        for (var k = 0; k < 7; k++) {\n            result.wnomask[i] = 1;\n            i++;\n            if (result.wdaymask[i] === options.wkst)\n                break;\n        }\n    }\n    if (includes(options.byweekno, 1)) {\n        // Check week number 1 of next year as well\n        // orig-TODO : Check -numweeks for next year.\n        var i = no1wkst + numweeks * 7;\n        if (no1wkst !== firstwkst)\n            i -= 7 - firstwkst;\n        if (i < yearlen) {\n            // If week starts in next year, we\n            // don't care about it.\n            for (var j = 0; j < 7; j++) {\n                result.wnomask[i] = 1;\n                i += 1;\n                if (result.wdaymask[i] === options.wkst)\n                    break;\n            }\n        }\n    }\n    if (no1wkst) {\n        // Check last week number of last year as\n        // well. If no1wkst is 0, either the year\n        // started on week start, or week number 1\n        // got days from last year, so there are no\n        // days from last year's last week number in\n        // this year.\n        var lnumweeks = void 0;\n        if (!includes(options.byweekno, -1)) {\n            var lyearweekday = getWeekday(datetime(year - 1, 1, 1));\n            var lno1wkst = pymod(7 - lyearweekday.valueOf() + options.wkst, 7);\n            var lyearlen = isLeapYear(year - 1) ? 366 : 365;\n            var weekst = void 0;\n            if (lno1wkst >= 4) {\n                lno1wkst = 0;\n                weekst = lyearlen + pymod(lyearweekday - options.wkst, 7);\n            }\n            else {\n                weekst = yearlen - no1wkst;\n            }\n            lnumweeks = Math.floor(52 + pymod(weekst, 7) / 4);\n        }\n        else {\n            lnumweeks = -1;\n        }\n        if (includes(options.byweekno, lnumweeks)) {\n            for (var i = 0; i < no1wkst; i++)\n                result.wnomask[i] = 1;\n        }\n    }\n    return result;\n}\nfunction baseYearMasks(year) {\n    var yearlen = isLeapYear(year) ? 366 : 365;\n    var firstyday = datetime(year, 1, 1);\n    var wday = getWeekday(firstyday);\n    if (yearlen === 365) {\n        return {\n            mmask: M365MASK,\n            mdaymask: MDAY365MASK,\n            nmdaymask: NMDAY365MASK,\n            wdaymask: WDAYMASK.slice(wday),\n            mrange: M365RANGE,\n        };\n    }\n    return {\n        mmask: M366MASK,\n        mdaymask: MDAY366MASK,\n        nmdaymask: NMDAY366MASK,\n        wdaymask: WDAYMASK.slice(wday),\n        mrange: M366RANGE,\n    };\n}\n\n;// CONCATENATED MODULE: ./src/iterinfo/monthinfo.ts\n\n\nfunction rebuildMonth(year, month, yearlen, mrange, wdaymask, options) {\n    var result = {\n        lastyear: year,\n        lastmonth: month,\n        nwdaymask: [],\n    };\n    var ranges = [];\n    if (options.freq === RRule.YEARLY) {\n        if (empty(options.bymonth)) {\n            ranges = [[0, yearlen]];\n        }\n        else {\n            for (var j = 0; j < options.bymonth.length; j++) {\n                month = options.bymonth[j];\n                ranges.push(mrange.slice(month - 1, month + 1));\n            }\n        }\n    }\n    else if (options.freq === RRule.MONTHLY) {\n        ranges = [mrange.slice(month - 1, month + 1)];\n    }\n    if (empty(ranges)) {\n        return result;\n    }\n    // Weekly frequency won't get here, so we may not\n    // care about cross-year weekly periods.\n    result.nwdaymask = repeat(0, yearlen);\n    for (var j = 0; j < ranges.length; j++) {\n        var rang = ranges[j];\n        var first = rang[0];\n        var last = rang[1] - 1;\n        for (var k = 0; k < options.bynweekday.length; k++) {\n            var i = void 0;\n            var _a = options.bynweekday[k], wday = _a[0], n = _a[1];\n            if (n < 0) {\n                i = last + (n + 1) * 7;\n                i -= pymod(wdaymask[i] - wday, 7);\n            }\n            else {\n                i = first + (n - 1) * 7;\n                i += pymod(7 - wdaymask[i] + wday, 7);\n            }\n            if (first <= i && i <= last)\n                result.nwdaymask[i] = 1;\n        }\n    }\n    return result;\n}\n\n;// CONCATENATED MODULE: ./src/iterinfo/easter.ts\nfunction easter(y, offset) {\n    if (offset === void 0) { offset = 0; }\n    var a = y % 19;\n    var b = Math.floor(y / 100);\n    var c = y % 100;\n    var d = Math.floor(b / 4);\n    var e = b % 4;\n    var f = Math.floor((b + 8) / 25);\n    var g = Math.floor((b - f + 1) / 3);\n    var h = Math.floor(19 * a + b - d - g + 15) % 30;\n    var i = Math.floor(c / 4);\n    var k = c % 4;\n    var l = Math.floor(32 + 2 * e + 2 * i - h - k) % 7;\n    var m = Math.floor((a + 11 * h + 22 * l) / 451);\n    var month = Math.floor((h + l - 7 * m + 114) / 31);\n    var day = ((h + l - 7 * m + 114) % 31) + 1;\n    var date = Date.UTC(y, month - 1, day + offset);\n    var yearStart = Date.UTC(y, 0, 1);\n    return [Math.ceil((date - yearStart) / (1000 * 60 * 60 * 24))];\n}\n\n;// CONCATENATED MODULE: ./src/iterinfo/index.ts\n\n\n\n\n\n\n\n// =============================================================================\n// Iterinfo\n// =============================================================================\nvar Iterinfo = /** @class */ (function () {\n    // eslint-disable-next-line no-empty-function\n    function Iterinfo(options) {\n        this.options = options;\n    }\n    Iterinfo.prototype.rebuild = function (year, month) {\n        var options = this.options;\n        if (year !== this.lastyear) {\n            this.yearinfo = rebuildYear(year, options);\n        }\n        if (notEmpty(options.bynweekday) &&\n            (month !== this.lastmonth || year !== this.lastyear)) {\n            var _a = this.yearinfo, yearlen = _a.yearlen, mrange = _a.mrange, wdaymask = _a.wdaymask;\n            this.monthinfo = rebuildMonth(year, month, yearlen, mrange, wdaymask, options);\n        }\n        if (isPresent(options.byeaster)) {\n            this.eastermask = easter(year, options.byeaster);\n        }\n    };\n    Object.defineProperty(Iterinfo.prototype, \"lastyear\", {\n        get: function () {\n            return this.monthinfo ? this.monthinfo.lastyear : null;\n        },\n        enumerable: false,\n        configurable: true\n    });\n    Object.defineProperty(Iterinfo.prototype, \"lastmonth\", {\n        get: function () {\n            return this.monthinfo ? this.monthinfo.lastmonth : null;\n        },\n        enumerable: false,\n        configurable: true\n    });\n    Object.defineProperty(Iterinfo.prototype, \"yearlen\", {\n        get: function () {\n            return this.yearinfo.yearlen;\n        },\n        enumerable: false,\n        configurable: true\n    });\n    Object.defineProperty(Iterinfo.prototype, \"yearordinal\", {\n        get: function () {\n            return this.yearinfo.yearordinal;\n        },\n        enumerable: false,\n        configurable: true\n    });\n    Object.defineProperty(Iterinfo.prototype, \"mrange\", {\n        get: function () {\n            return this.yearinfo.mrange;\n        },\n        enumerable: false,\n        configurable: true\n    });\n    Object.defineProperty(Iterinfo.prototype, \"wdaymask\", {\n        get: function () {\n            return this.yearinfo.wdaymask;\n        },\n        enumerable: false,\n        configurable: true\n    });\n    Object.defineProperty(Iterinfo.prototype, \"mmask\", {\n        get: function () {\n            return this.yearinfo.mmask;\n        },\n        enumerable: false,\n        configurable: true\n    });\n    Object.defineProperty(Iterinfo.prototype, \"wnomask\", {\n        get: function () {\n            return this.yearinfo.wnomask;\n        },\n        enumerable: false,\n        configurable: true\n    });\n    Object.defineProperty(Iterinfo.prototype, \"nwdaymask\", {\n        get: function () {\n            return this.monthinfo ? this.monthinfo.nwdaymask : [];\n        },\n        enumerable: false,\n        configurable: true\n    });\n    Object.defineProperty(Iterinfo.prototype, \"nextyearlen\", {\n        get: function () {\n            return this.yearinfo.nextyearlen;\n        },\n        enumerable: false,\n        configurable: true\n    });\n    Object.defineProperty(Iterinfo.prototype, \"mdaymask\", {\n        get: function () {\n            return this.yearinfo.mdaymask;\n        },\n        enumerable: false,\n        configurable: true\n    });\n    Object.defineProperty(Iterinfo.prototype, \"nmdaymask\", {\n        get: function () {\n            return this.yearinfo.nmdaymask;\n        },\n        enumerable: false,\n        configurable: true\n    });\n    Iterinfo.prototype.ydayset = function () {\n        return [range(this.yearlen), 0, this.yearlen];\n    };\n    Iterinfo.prototype.mdayset = function (_, month) {\n        var start = this.mrange[month - 1];\n        var end = this.mrange[month];\n        var set = repeat(null, this.yearlen);\n        for (var i = start; i < end; i++)\n            set[i] = i;\n        return [set, start, end];\n    };\n    Iterinfo.prototype.wdayset = function (year, month, day) {\n        // We need to handle cross-year weeks here.\n        var set = repeat(null, this.yearlen + 7);\n        var i = toOrdinal(datetime(year, month, day)) - this.yearordinal;\n        var start = i;\n        for (var j = 0; j < 7; j++) {\n            set[i] = i;\n            ++i;\n            if (this.wdaymask[i] === this.options.wkst)\n                break;\n        }\n        return [set, start, i];\n    };\n    Iterinfo.prototype.ddayset = function (year, month, day) {\n        var set = repeat(null, this.yearlen);\n        var i = toOrdinal(datetime(year, month, day)) - this.yearordinal;\n        set[i] = i;\n        return [set, i, i + 1];\n    };\n    Iterinfo.prototype.htimeset = function (hour, _, second, millisecond) {\n        var _this = this;\n        var set = [];\n        this.options.byminute.forEach(function (minute) {\n            set = set.concat(_this.mtimeset(hour, minute, second, millisecond));\n        });\n        sort(set);\n        return set;\n    };\n    Iterinfo.prototype.mtimeset = function (hour, minute, _, millisecond) {\n        var set = this.options.bysecond.map(function (second) { return new Time(hour, minute, second, millisecond); });\n        sort(set);\n        return set;\n    };\n    Iterinfo.prototype.stimeset = function (hour, minute, second, millisecond) {\n        return [new Time(hour, minute, second, millisecond)];\n    };\n    Iterinfo.prototype.getdayset = function (freq) {\n        switch (freq) {\n            case Frequency.YEARLY:\n                return this.ydayset.bind(this);\n            case Frequency.MONTHLY:\n                return this.mdayset.bind(this);\n            case Frequency.WEEKLY:\n                return this.wdayset.bind(this);\n            case Frequency.DAILY:\n                return this.ddayset.bind(this);\n            default:\n                return this.ddayset.bind(this);\n        }\n    };\n    Iterinfo.prototype.gettimeset = function (freq) {\n        switch (freq) {\n            case Frequency.HOURLY:\n                return this.htimeset.bind(this);\n            case Frequency.MINUTELY:\n                return this.mtimeset.bind(this);\n            case Frequency.SECONDLY:\n                return this.stimeset.bind(this);\n        }\n    };\n    return Iterinfo;\n}());\n/* harmony default export */ const iterinfo = (Iterinfo);\n\n;// CONCATENATED MODULE: ./src/iter/poslist.ts\n\n\nfunction buildPoslist(bysetpos, timeset, start, end, ii, dayset) {\n    var poslist = [];\n    for (var j = 0; j < bysetpos.length; j++) {\n        var daypos = void 0;\n        var timepos = void 0;\n        var pos = bysetpos[j];\n        if (pos < 0) {\n            daypos = Math.floor(pos / timeset.length);\n            timepos = pymod(pos, timeset.length);\n        }\n        else {\n            daypos = Math.floor((pos - 1) / timeset.length);\n            timepos = pymod(pos - 1, timeset.length);\n        }\n        var tmp = [];\n        for (var k = start; k < end; k++) {\n            var val = dayset[k];\n            if (!isPresent(val))\n                continue;\n            tmp.push(val);\n        }\n        var i = void 0;\n        if (daypos < 0) {\n            i = tmp.slice(daypos)[0];\n        }\n        else {\n            i = tmp[daypos];\n        }\n        var time = timeset[timepos];\n        var date = fromOrdinal(ii.yearordinal + i);\n        var res = combine(date, time);\n        // XXX: can this ever be in the array?\n        // - compare the actual date instead?\n        if (!includes(poslist, res))\n            poslist.push(res);\n    }\n    sort(poslist);\n    return poslist;\n}\n\n;// CONCATENATED MODULE: ./src/iter/index.ts\n\n\n\n\n\n\n\n\n\nfunction iter(iterResult, options) {\n    var dtstart = options.dtstart, freq = options.freq, interval = options.interval, until = options.until, bysetpos = options.bysetpos;\n    var count = options.count;\n    if (count === 0 || interval === 0) {\n        return emitResult(iterResult);\n    }\n    var counterDate = DateTime.fromDate(dtstart);\n    var ii = new iterinfo(options);\n    ii.rebuild(counterDate.year, counterDate.month);\n    var timeset = makeTimeset(ii, counterDate, options);\n    for (;;) {\n        var _a = ii.getdayset(freq)(counterDate.year, counterDate.month, counterDate.day), dayset = _a[0], start = _a[1], end = _a[2];\n        var filtered = removeFilteredDays(dayset, start, end, ii, options);\n        if (notEmpty(bysetpos)) {\n            var poslist = buildPoslist(bysetpos, timeset, start, end, ii, dayset);\n            for (var j = 0; j < poslist.length; j++) {\n                var res = poslist[j];\n                if (until && res > until) {\n                    return emitResult(iterResult);\n                }\n                if (res >= dtstart) {\n                    var rezonedDate = rezoneIfNeeded(res, options);\n                    if (!iterResult.accept(rezonedDate)) {\n                        return emitResult(iterResult);\n                    }\n                    if (count) {\n                        --count;\n                        if (!count) {\n                            return emitResult(iterResult);\n                        }\n                    }\n                }\n            }\n        }\n        else {\n            for (var j = start; j < end; j++) {\n                var currentDay = dayset[j];\n                if (!isPresent(currentDay)) {\n                    continue;\n                }\n                var date = fromOrdinal(ii.yearordinal + currentDay);\n                for (var k = 0; k < timeset.length; k++) {\n                    var time = timeset[k];\n                    var res = combine(date, time);\n                    if (until && res > until) {\n                        return emitResult(iterResult);\n                    }\n                    if (res >= dtstart) {\n                        var rezonedDate = rezoneIfNeeded(res, options);\n                        if (!iterResult.accept(rezonedDate)) {\n                            return emitResult(iterResult);\n                        }\n                        if (count) {\n                            --count;\n                            if (!count) {\n                                return emitResult(iterResult);\n                            }\n                        }\n                    }\n                }\n            }\n        }\n        if (options.interval === 0) {\n            return emitResult(iterResult);\n        }\n        // Handle frequency and interval\n        counterDate.add(options, filtered);\n        if (counterDate.year > MAXYEAR) {\n            return emitResult(iterResult);\n        }\n        if (!freqIsDailyOrGreater(freq)) {\n            timeset = ii.gettimeset(freq)(counterDate.hour, counterDate.minute, counterDate.second, 0);\n        }\n        ii.rebuild(counterDate.year, counterDate.month);\n    }\n}\nfunction isFiltered(ii, currentDay, options) {\n    var bymonth = options.bymonth, byweekno = options.byweekno, byweekday = options.byweekday, byeaster = options.byeaster, bymonthday = options.bymonthday, bynmonthday = options.bynmonthday, byyearday = options.byyearday;\n    return ((notEmpty(bymonth) && !includes(bymonth, ii.mmask[currentDay])) ||\n        (notEmpty(byweekno) && !ii.wnomask[currentDay]) ||\n        (notEmpty(byweekday) && !includes(byweekday, ii.wdaymask[currentDay])) ||\n        (notEmpty(ii.nwdaymask) && !ii.nwdaymask[currentDay]) ||\n        (byeaster !== null && !includes(ii.eastermask, currentDay)) ||\n        ((notEmpty(bymonthday) || notEmpty(bynmonthday)) &&\n            !includes(bymonthday, ii.mdaymask[currentDay]) &&\n            !includes(bynmonthday, ii.nmdaymask[currentDay])) ||\n        (notEmpty(byyearday) &&\n            ((currentDay < ii.yearlen &&\n                !includes(byyearday, currentDay + 1) &&\n                !includes(byyearday, -ii.yearlen + currentDay)) ||\n                (currentDay >= ii.yearlen &&\n                    !includes(byyearday, currentDay + 1 - ii.yearlen) &&\n                    !includes(byyearday, -ii.nextyearlen + currentDay - ii.yearlen)))));\n}\nfunction rezoneIfNeeded(date, options) {\n    return new DateWithZone(date, options.tzid).rezonedDate();\n}\nfunction emitResult(iterResult) {\n    return iterResult.getValue();\n}\nfunction removeFilteredDays(dayset, start, end, ii, options) {\n    var filtered = false;\n    for (var dayCounter = start; dayCounter < end; dayCounter++) {\n        var currentDay = dayset[dayCounter];\n        filtered = isFiltered(ii, currentDay, options);\n        if (filtered)\n            dayset[currentDay] = null;\n    }\n    return filtered;\n}\nfunction makeTimeset(ii, counterDate, options) {\n    var freq = options.freq, byhour = options.byhour, byminute = options.byminute, bysecond = options.bysecond;\n    if (freqIsDailyOrGreater(freq)) {\n        return buildTimeset(options);\n    }\n    if ((freq >= RRule.HOURLY &&\n        notEmpty(byhour) &&\n        !includes(byhour, counterDate.hour)) ||\n        (freq >= RRule.MINUTELY &&\n            notEmpty(byminute) &&\n            !includes(byminute, counterDate.minute)) ||\n        (freq >= RRule.SECONDLY &&\n            notEmpty(bysecond) &&\n            !includes(bysecond, counterDate.second))) {\n        return [];\n    }\n    return ii.gettimeset(freq)(counterDate.hour, counterDate.minute, counterDate.second, counterDate.millisecond);\n}\n\n;// CONCATENATED MODULE: ./src/rrule.ts\n\n\n\n\n\n\n\n\n\n\n\n// =============================================================================\n// RRule\n// =============================================================================\nvar Days = {\n    MO: new Weekday(0),\n    TU: new Weekday(1),\n    WE: new Weekday(2),\n    TH: new Weekday(3),\n    FR: new Weekday(4),\n    SA: new Weekday(5),\n    SU: new Weekday(6),\n};\nvar DEFAULT_OPTIONS = {\n    freq: Frequency.YEARLY,\n    dtstart: null,\n    interval: 1,\n    wkst: Days.MO,\n    count: null,\n    until: null,\n    tzid: null,\n    bysetpos: null,\n    bymonth: null,\n    bymonthday: null,\n    bynmonthday: null,\n    byyearday: null,\n    byweekno: null,\n    byweekday: null,\n    bynweekday: null,\n    byhour: null,\n    byminute: null,\n    bysecond: null,\n    byeaster: null,\n};\nvar defaultKeys = Object.keys(DEFAULT_OPTIONS);\n/**\n *\n * @param {Options?} options - see <http://labix.org/python-dateutil/#head-cf004ee9a75592797e076752b2a889c10f445418>\n * - The only required option is `freq`, one of RRule.YEARLY, RRule.MONTHLY, ...\n * @constructor\n */\nvar RRule = /** @class */ (function () {\n    function RRule(options, noCache) {\n        if (options === void 0) { options = {}; }\n        if (noCache === void 0) { noCache = false; }\n        // RFC string\n        this._cache = noCache ? null : new Cache();\n        // used by toString()\n        this.origOptions = initializeOptions(options);\n        var parsedOptions = parseOptions(options).parsedOptions;\n        this.options = parsedOptions;\n    }\n    RRule.parseText = function (text, language) {\n        return parseText(text, language);\n    };\n    RRule.fromText = function (text, language) {\n        return fromText(text, language);\n    };\n    RRule.fromString = function (str) {\n        return new RRule(RRule.parseString(str) || undefined);\n    };\n    RRule.prototype._iter = function (iterResult) {\n        return iter(iterResult, this.options);\n    };\n    RRule.prototype._cacheGet = function (what, args) {\n        if (!this._cache)\n            return false;\n        return this._cache._cacheGet(what, args);\n    };\n    RRule.prototype._cacheAdd = function (what, value, args) {\n        if (!this._cache)\n            return;\n        return this._cache._cacheAdd(what, value, args);\n    };\n    /**\n     * @param {Function} iterator - optional function that will be called\n     * on each date that is added. It can return false\n     * to stop the iteration.\n     * @return Array containing all recurrences.\n     */\n    RRule.prototype.all = function (iterator) {\n        if (iterator) {\n            return this._iter(new callbackiterresult('all', {}, iterator));\n        }\n        var result = this._cacheGet('all');\n        if (result === false) {\n            result = this._iter(new iterresult('all', {}));\n            this._cacheAdd('all', result);\n        }\n        return result;\n    };\n    /**\n     * Returns all the occurrences of the rrule between after and before.\n     * The inc keyword defines what happens if after and/or before are\n     * themselves occurrences. With inc == True, they will be included in the\n     * list, if they are found in the recurrence set.\n     *\n     * @return Array\n     */\n    RRule.prototype.between = function (after, before, inc, iterator) {\n        if (inc === void 0) { inc = false; }\n        if (!isValidDate(after) || !isValidDate(before)) {\n            throw new Error('Invalid date passed in to RRule.between');\n        }\n        var args = {\n            before: before,\n            after: after,\n            inc: inc,\n        };\n        if (iterator) {\n            return this._iter(new callbackiterresult('between', args, iterator));\n        }\n        var result = this._cacheGet('between', args);\n        if (result === false) {\n            result = this._iter(new iterresult('between', args));\n            this._cacheAdd('between', result, args);\n        }\n        return result;\n    };\n    /**\n     * Returns the last recurrence before the given datetime instance.\n     * The inc keyword defines what happens if dt is an occurrence.\n     * With inc == True, if dt itself is an occurrence, it will be returned.\n     *\n     * @return Date or null\n     */\n    RRule.prototype.before = function (dt, inc) {\n        if (inc === void 0) { inc = false; }\n        if (!isValidDate(dt)) {\n            throw new Error('Invalid date passed in to RRule.before');\n        }\n        var args = { dt: dt, inc: inc };\n        var result = this._cacheGet('before', args);\n        if (result === false) {\n            result = this._iter(new iterresult('before', args));\n            this._cacheAdd('before', result, args);\n        }\n        return result;\n    };\n    /**\n     * Returns the first recurrence after the given datetime instance.\n     * The inc keyword defines what happens if dt is an occurrence.\n     * With inc == True, if dt itself is an occurrence, it will be returned.\n     *\n     * @return Date or null\n     */\n    RRule.prototype.after = function (dt, inc) {\n        if (inc === void 0) { inc = false; }\n        if (!isValidDate(dt)) {\n            throw new Error('Invalid date passed in to RRule.after');\n        }\n        var args = { dt: dt, inc: inc };\n        var result = this._cacheGet('after', args);\n        if (result === false) {\n            result = this._iter(new iterresult('after', args));\n            this._cacheAdd('after', result, args);\n        }\n        return result;\n    };\n    /**\n     * Returns the number of recurrences in this set. It will have go trough\n     * the whole recurrence, if this hasn't been done before.\n     */\n    RRule.prototype.count = function () {\n        return this.all().length;\n    };\n    /**\n     * Converts the rrule into its string representation\n     *\n     * @see <http://www.ietf.org/rfc/rfc2445.txt>\n     * @return String\n     */\n    RRule.prototype.toString = function () {\n        return optionsToString(this.origOptions);\n    };\n    /**\n     * Will convert all rules described in nlp:ToText\n     * to text.\n     */\n    RRule.prototype.toText = function (gettext, language, dateFormatter) {\n        return toText(this, gettext, language, dateFormatter);\n    };\n    RRule.prototype.isFullyConvertibleToText = function () {\n        return isFullyConvertible(this);\n    };\n    /**\n     * @return a RRule instance with the same freq and options\n     * as this one (cache is not cloned)\n     */\n    RRule.prototype.clone = function () {\n        return new RRule(this.origOptions);\n    };\n    // RRule class 'constants'\n    RRule.FREQUENCIES = [\n        'YEARLY',\n        'MONTHLY',\n        'WEEKLY',\n        'DAILY',\n        'HOURLY',\n        'MINUTELY',\n        'SECONDLY',\n    ];\n    RRule.YEARLY = Frequency.YEARLY;\n    RRule.MONTHLY = Frequency.MONTHLY;\n    RRule.WEEKLY = Frequency.WEEKLY;\n    RRule.DAILY = Frequency.DAILY;\n    RRule.HOURLY = Frequency.HOURLY;\n    RRule.MINUTELY = Frequency.MINUTELY;\n    RRule.SECONDLY = Frequency.SECONDLY;\n    RRule.MO = Days.MO;\n    RRule.TU = Days.TU;\n    RRule.WE = Days.WE;\n    RRule.TH = Days.TH;\n    RRule.FR = Days.FR;\n    RRule.SA = Days.SA;\n    RRule.SU = Days.SU;\n    RRule.parseString = parseString;\n    RRule.optionsToString = optionsToString;\n    return RRule;\n}());\n\n\n;// CONCATENATED MODULE: ./src/iterset.ts\n\n\n\nfunction iterSet(iterResult, _rrule, _exrule, _rdate, _exdate, tzid) {\n    var _exdateHash = {};\n    var _accept = iterResult.accept;\n    function evalExdate(after, before) {\n        _exrule.forEach(function (rrule) {\n            rrule.between(after, before, true).forEach(function (date) {\n                _exdateHash[Number(date)] = true;\n            });\n        });\n    }\n    _exdate.forEach(function (date) {\n        var zonedDate = new DateWithZone(date, tzid).rezonedDate();\n        _exdateHash[Number(zonedDate)] = true;\n    });\n    iterResult.accept = function (date) {\n        var dt = Number(date);\n        if (isNaN(dt))\n            return _accept.call(this, date);\n        if (!_exdateHash[dt]) {\n            evalExdate(new Date(dt - 1), new Date(dt + 1));\n            if (!_exdateHash[dt]) {\n                _exdateHash[dt] = true;\n                return _accept.call(this, date);\n            }\n        }\n        return true;\n    };\n    if (iterResult.method === 'between') {\n        evalExdate(iterResult.args.after, iterResult.args.before);\n        iterResult.accept = function (date) {\n            var dt = Number(date);\n            if (!_exdateHash[dt]) {\n                _exdateHash[dt] = true;\n                return _accept.call(this, date);\n            }\n            return true;\n        };\n    }\n    for (var i = 0; i < _rdate.length; i++) {\n        var zonedDate = new DateWithZone(_rdate[i], tzid).rezonedDate();\n        if (!iterResult.accept(new Date(zonedDate.getTime())))\n            break;\n    }\n    _rrule.forEach(function (rrule) {\n        iter(iterResult, rrule.options);\n    });\n    var res = iterResult._result;\n    sort(res);\n    switch (iterResult.method) {\n        case 'all':\n        case 'between':\n            return res;\n        case 'before':\n            return ((res.length && res[res.length - 1]) || null);\n        case 'after':\n        default:\n            return ((res.length && res[0]) || null);\n    }\n}\n\n;// CONCATENATED MODULE: ./src/rrulestr.ts\n\n\n\n\n\n\n/**\n * RRuleStr\n * To parse a set of rrule strings\n */\nvar rrulestr_DEFAULT_OPTIONS = {\n    dtstart: null,\n    cache: false,\n    unfold: false,\n    forceset: false,\n    compatible: false,\n    tzid: null,\n};\nfunction parseInput(s, options) {\n    var rrulevals = [];\n    var rdatevals = [];\n    var exrulevals = [];\n    var exdatevals = [];\n    var parsedDtstart = parseDtstart(s);\n    var dtstart = parsedDtstart.dtstart;\n    var tzid = parsedDtstart.tzid;\n    var lines = splitIntoLines(s, options.unfold);\n    lines.forEach(function (line) {\n        var _a;\n        if (!line)\n            return;\n        var _b = breakDownLine(line), name = _b.name, parms = _b.parms, value = _b.value;\n        switch (name.toUpperCase()) {\n            case 'RRULE':\n                if (parms.length) {\n                    throw new Error(\"unsupported RRULE parm: \".concat(parms.join(',')));\n                }\n                rrulevals.push(parseString(line));\n                break;\n            case 'RDATE':\n                var _c = (_a = /RDATE(?:;TZID=([^:=]+))?/i.exec(line)) !== null && _a !== void 0 ? _a : [], rdateTzid = _c[1];\n                if (rdateTzid && !tzid) {\n                    tzid = rdateTzid;\n                }\n                rdatevals = rdatevals.concat(parseRDate(value, parms));\n                break;\n            case 'EXRULE':\n                if (parms.length) {\n                    throw new Error(\"unsupported EXRULE parm: \".concat(parms.join(',')));\n                }\n                exrulevals.push(parseString(value));\n                break;\n            case 'EXDATE':\n                exdatevals = exdatevals.concat(parseRDate(value, parms));\n                break;\n            case 'DTSTART':\n                break;\n            default:\n                throw new Error('unsupported property: ' + name);\n        }\n    });\n    return {\n        dtstart: dtstart,\n        tzid: tzid,\n        rrulevals: rrulevals,\n        rdatevals: rdatevals,\n        exrulevals: exrulevals,\n        exdatevals: exdatevals,\n    };\n}\nfunction buildRule(s, options) {\n    var _a = parseInput(s, options), rrulevals = _a.rrulevals, rdatevals = _a.rdatevals, exrulevals = _a.exrulevals, exdatevals = _a.exdatevals, dtstart = _a.dtstart, tzid = _a.tzid;\n    var noCache = options.cache === false;\n    if (options.compatible) {\n        options.forceset = true;\n        options.unfold = true;\n    }\n    if (options.forceset ||\n        rrulevals.length > 1 ||\n        rdatevals.length ||\n        exrulevals.length ||\n        exdatevals.length) {\n        var rset_1 = new RRuleSet(noCache);\n        rset_1.dtstart(dtstart);\n        rset_1.tzid(tzid || undefined);\n        rrulevals.forEach(function (val) {\n            rset_1.rrule(new RRule(groomRruleOptions(val, dtstart, tzid), noCache));\n        });\n        rdatevals.forEach(function (date) {\n            rset_1.rdate(date);\n        });\n        exrulevals.forEach(function (val) {\n            rset_1.exrule(new RRule(groomRruleOptions(val, dtstart, tzid), noCache));\n        });\n        exdatevals.forEach(function (date) {\n            rset_1.exdate(date);\n        });\n        if (options.compatible && options.dtstart)\n            rset_1.rdate(dtstart);\n        return rset_1;\n    }\n    var val = rrulevals[0] || {};\n    return new RRule(groomRruleOptions(val, val.dtstart || options.dtstart || dtstart, val.tzid || options.tzid || tzid), noCache);\n}\nfunction rrulestr(s, options) {\n    if (options === void 0) { options = {}; }\n    return buildRule(s, rrulestr_initializeOptions(options));\n}\nfunction groomRruleOptions(val, dtstart, tzid) {\n    return __assign(__assign({}, val), { dtstart: dtstart, tzid: tzid });\n}\nfunction rrulestr_initializeOptions(options) {\n    var invalid = [];\n    var keys = Object.keys(options);\n    var defaultKeys = Object.keys(rrulestr_DEFAULT_OPTIONS);\n    keys.forEach(function (key) {\n        if (!includes(defaultKeys, key))\n            invalid.push(key);\n    });\n    if (invalid.length) {\n        throw new Error('Invalid options: ' + invalid.join(', '));\n    }\n    return __assign(__assign({}, rrulestr_DEFAULT_OPTIONS), options);\n}\nfunction extractName(line) {\n    if (line.indexOf(':') === -1) {\n        return {\n            name: 'RRULE',\n            value: line,\n        };\n    }\n    var _a = split(line, ':', 1), name = _a[0], value = _a[1];\n    return {\n        name: name,\n        value: value,\n    };\n}\nfunction breakDownLine(line) {\n    var _a = extractName(line), name = _a.name, value = _a.value;\n    var parms = name.split(';');\n    if (!parms)\n        throw new Error('empty property name');\n    return {\n        name: parms[0].toUpperCase(),\n        parms: parms.slice(1),\n        value: value,\n    };\n}\nfunction splitIntoLines(s, unfold) {\n    if (unfold === void 0) { unfold = false; }\n    s = s && s.trim();\n    if (!s)\n        throw new Error('Invalid empty string');\n    // More info about 'unfold' option\n    // Go head to http://www.ietf.org/rfc/rfc2445.txt\n    if (!unfold) {\n        return s.split(/\\s/);\n    }\n    var lines = s.split('\\n');\n    var i = 0;\n    while (i < lines.length) {\n        // TODO\n        var line = (lines[i] = lines[i].replace(/\\s+$/g, ''));\n        if (!line) {\n            lines.splice(i, 1);\n        }\n        else if (i > 0 && line[0] === ' ') {\n            lines[i - 1] += line.slice(1);\n            lines.splice(i, 1);\n        }\n        else {\n            i += 1;\n        }\n    }\n    return lines;\n}\nfunction validateDateParm(parms) {\n    parms.forEach(function (parm) {\n        if (!/(VALUE=DATE(-TIME)?)|(TZID=)/.test(parm)) {\n            throw new Error('unsupported RDATE/EXDATE parm: ' + parm);\n        }\n    });\n}\nfunction parseRDate(rdateval, parms) {\n    validateDateParm(parms);\n    return rdateval.split(',').map(function (datestr) { return untilStringToDate(datestr); });\n}\n\n;// CONCATENATED MODULE: ./src/rruleset.ts\n\n\n\n\n\n\n\nfunction createGetterSetter(fieldName) {\n    var _this = this;\n    return function (field) {\n        if (field !== undefined) {\n            _this[\"_\".concat(fieldName)] = field;\n        }\n        if (_this[\"_\".concat(fieldName)] !== undefined) {\n            return _this[\"_\".concat(fieldName)];\n        }\n        for (var i = 0; i < _this._rrule.length; i++) {\n            var field_1 = _this._rrule[i].origOptions[fieldName];\n            if (field_1) {\n                return field_1;\n            }\n        }\n    };\n}\nvar RRuleSet = /** @class */ (function (_super) {\n    __extends(RRuleSet, _super);\n    /**\n     *\n     * @param {Boolean?} noCache\n     * The same stratagy as RRule on cache, default to false\n     * @constructor\n     */\n    function RRuleSet(noCache) {\n        if (noCache === void 0) { noCache = false; }\n        var _this = _super.call(this, {}, noCache) || this;\n        _this.dtstart = createGetterSetter.apply(_this, ['dtstart']);\n        _this.tzid = createGetterSetter.apply(_this, ['tzid']);\n        _this._rrule = [];\n        _this._rdate = [];\n        _this._exrule = [];\n        _this._exdate = [];\n        return _this;\n    }\n    RRuleSet.prototype._iter = function (iterResult) {\n        return iterSet(iterResult, this._rrule, this._exrule, this._rdate, this._exdate, this.tzid());\n    };\n    /**\n     * Adds an RRule to the set\n     *\n     * @param {RRule}\n     */\n    RRuleSet.prototype.rrule = function (rrule) {\n        _addRule(rrule, this._rrule);\n    };\n    /**\n     * Adds an EXRULE to the set\n     *\n     * @param {RRule}\n     */\n    RRuleSet.prototype.exrule = function (rrule) {\n        _addRule(rrule, this._exrule);\n    };\n    /**\n     * Adds an RDate to the set\n     *\n     * @param {Date}\n     */\n    RRuleSet.prototype.rdate = function (date) {\n        _addDate(date, this._rdate);\n    };\n    /**\n     * Adds an EXDATE to the set\n     *\n     * @param {Date}\n     */\n    RRuleSet.prototype.exdate = function (date) {\n        _addDate(date, this._exdate);\n    };\n    /**\n     * Get list of included rrules in this recurrence set.\n     *\n     * @return List of rrules\n     */\n    RRuleSet.prototype.rrules = function () {\n        return this._rrule.map(function (e) { return rrulestr(e.toString()); });\n    };\n    /**\n     * Get list of excluded rrules in this recurrence set.\n     *\n     * @return List of exrules\n     */\n    RRuleSet.prototype.exrules = function () {\n        return this._exrule.map(function (e) { return rrulestr(e.toString()); });\n    };\n    /**\n     * Get list of included datetimes in this recurrence set.\n     *\n     * @return List of rdates\n     */\n    RRuleSet.prototype.rdates = function () {\n        return this._rdate.map(function (e) { return new Date(e.getTime()); });\n    };\n    /**\n     * Get list of included datetimes in this recurrence set.\n     *\n     * @return List of exdates\n     */\n    RRuleSet.prototype.exdates = function () {\n        return this._exdate.map(function (e) { return new Date(e.getTime()); });\n    };\n    RRuleSet.prototype.valueOf = function () {\n        var result = [];\n        if (!this._rrule.length && this._dtstart) {\n            result = result.concat(optionsToString({ dtstart: this._dtstart }));\n        }\n        this._rrule.forEach(function (rrule) {\n            result = result.concat(rrule.toString().split('\\n'));\n        });\n        this._exrule.forEach(function (exrule) {\n            result = result.concat(exrule\n                .toString()\n                .split('\\n')\n                .map(function (line) { return line.replace(/^RRULE:/, 'EXRULE:'); })\n                .filter(function (line) { return !/^DTSTART/.test(line); }));\n        });\n        if (this._rdate.length) {\n            result.push(rdatesToString('RDATE', this._rdate, this.tzid()));\n        }\n        if (this._exdate.length) {\n            result.push(rdatesToString('EXDATE', this._exdate, this.tzid()));\n        }\n        return result;\n    };\n    /**\n     * to generate recurrence field such as:\n     * DTSTART:19970902T010000Z\n     * RRULE:FREQ=YEARLY;COUNT=2;BYDAY=TU\n     * RRULE:FREQ=YEARLY;COUNT=1;BYDAY=TH\n     */\n    RRuleSet.prototype.toString = function () {\n        return this.valueOf().join('\\n');\n    };\n    /**\n     * Create a new RRuleSet Object completely base on current instance\n     */\n    RRuleSet.prototype.clone = function () {\n        var rrs = new RRuleSet(!!this._cache);\n        this._rrule.forEach(function (rule) { return rrs.rrule(rule.clone()); });\n        this._exrule.forEach(function (rule) { return rrs.exrule(rule.clone()); });\n        this._rdate.forEach(function (date) { return rrs.rdate(new Date(date.getTime())); });\n        this._exdate.forEach(function (date) { return rrs.exdate(new Date(date.getTime())); });\n        return rrs;\n    };\n    return RRuleSet;\n}(RRule));\n\nfunction _addRule(rrule, collection) {\n    if (!(rrule instanceof RRule)) {\n        throw new TypeError(String(rrule) + ' is not RRule instance');\n    }\n    if (!includes(collection.map(String), String(rrule))) {\n        collection.push(rrule);\n    }\n}\nfunction _addDate(date, collection) {\n    if (!(date instanceof Date)) {\n        throw new TypeError(String(date) + ' is not Date instance');\n    }\n    if (!includes(collection.map(Number), Number(date))) {\n        collection.push(date);\n        sort(collection);\n    }\n}\nfunction rdatesToString(param, rdates, tzid) {\n    var isUTC = !tzid || tzid.toUpperCase() === 'UTC';\n    var header = isUTC ? \"\".concat(param, \":\") : \"\".concat(param, \";TZID=\").concat(tzid, \":\");\n    var dateString = rdates\n        .map(function (rdate) { return timeToUntilString(rdate.valueOf(), isUTC); })\n        .join(',');\n    return \"\".concat(header).concat(dateString);\n}\n\n;// CONCATENATED MODULE: ./src/index.ts\n/* !\n * rrule.js - Library for working with recurrence rules for calendar dates.\n * https://github.com/jakubroztocil/rrule\n *\n * Copyright 2010, Jakub Roztocil and Lars Schoning\n * Licenced under the BSD licence.\n * https://github.com/jakubroztocil/rrule/blob/master/LICENCE\n *\n * Based on:\n * python-dateutil - Extensions to the standard Python datetime module.\n * Copyright (c) 2003-2011 - Gustavo Niemeyer <gustavo@niemeyer.net>\n * Copyright (c) 2012 - Tomi Pieviläinen <tomi.pievilainen@iki.fi>\n * https://github.com/jakubroztocil/rrule/blob/master/LICENCE\n *\n */\n\n\n\n\n\n\n\n/******/ \treturn __webpack_exports__;\n/******/ })()\n;\n});"

/***/ },

/***/ 1049
(module) {

/*
	MIT License http://www.opensource.org/licenses/mit-license.php
	Author Tobias Koppers @sokra
*/
module.exports = function(src) {
	function log(error) {
		(typeof console !== "undefined")
		&& (console.error || console.log)("[Script Loader]", error);
	}

	// Check for IE =< 8
	function isIE() {
		return typeof attachEvent !== "undefined" && typeof addEventListener === "undefined";
	}

	try {
		if (typeof execScript !== "undefined" && isIE()) {
			execScript(src);
		} else if (typeof eval !== "undefined") {
			eval.call(null, src);
		} else {
			log("EvalError: No eval function available");
		}
	} catch (error) {
		log(error);
	}
}


/***/ },

/***/ 1084
(__unused_webpack_module, __unused_webpack_exports, __webpack_require__) {

__webpack_require__(1049)(__webpack_require__(1085))

/***/ },

/***/ 1080
(__unused_webpack_module, __unused_webpack_exports, __webpack_require__) {

__webpack_require__(1049)(__webpack_require__(1081))

/***/ },

/***/ 1082
(__unused_webpack_module, __unused_webpack_exports, __webpack_require__) {

__webpack_require__(1049)(__webpack_require__(1083))

/***/ },

/***/ 1086
(__unused_webpack_module, __unused_webpack_exports, __webpack_require__) {

__webpack_require__(1049)(__webpack_require__(1087))

/***/ },

/***/ 1088
(__unused_webpack_module, __unused_webpack_exports, __webpack_require__) {

__webpack_require__(1049)(__webpack_require__(1089))

/***/ },

/***/ 1094
(__unused_webpack_module, __unused_webpack_exports, __webpack_require__) {

__webpack_require__(1049)(__webpack_require__(1095))

/***/ },

/***/ 1096
(__unused_webpack_module, __unused_webpack_exports, __webpack_require__) {

__webpack_require__(1049)(__webpack_require__(1097))

/***/ },

/***/ 1098
(__unused_webpack_module, __unused_webpack_exports, __webpack_require__) {

__webpack_require__(1049)(__webpack_require__(1099))

/***/ },

/***/ 1090
(__unused_webpack_module, __unused_webpack_exports, __webpack_require__) {

__webpack_require__(1049)(__webpack_require__(1091))

/***/ },

/***/ 1092
(__unused_webpack_module, __unused_webpack_exports, __webpack_require__) {

__webpack_require__(1049)(__webpack_require__(1093))

/***/ },

/***/ 1078
(__unused_webpack_module, __unused_webpack_exports, __webpack_require__) {

__webpack_require__(1049)(__webpack_require__(1079))

/***/ }

/******/ 	});
/************************************************************************/
/******/ 	// The module cache
/******/ 	var __webpack_module_cache__ = {};
/******/ 	
/******/ 	// The require function
/******/ 	function __webpack_require__(moduleId) {
/******/ 		// Check if module is in cache
/******/ 		var cachedModule = __webpack_module_cache__[moduleId];
/******/ 		if (cachedModule !== undefined) {
/******/ 			return cachedModule.exports;
/******/ 		}
/******/ 		// Create a new module (and put it into the cache)
/******/ 		var module = __webpack_module_cache__[moduleId] = {
/******/ 			// no module.id needed
/******/ 			// no module.loaded needed
/******/ 			exports: {}
/******/ 		};
/******/ 	
/******/ 		// Execute the module function
/******/ 		__webpack_modules__[moduleId](module, module.exports, __webpack_require__);
/******/ 	
/******/ 		// Return the exports of the module
/******/ 		return module.exports;
/******/ 	}
/******/ 	
/************************************************************************/
/******/ 	/* webpack/runtime/compat get default export */
/******/ 	(() => {
/******/ 		// getDefaultExport function for compatibility with non-harmony modules
/******/ 		__webpack_require__.n = (module) => {
/******/ 			var getter = module && module.__esModule ?
/******/ 				() => (module['default']) :
/******/ 				() => (module);
/******/ 			__webpack_require__.d(getter, { a: getter });
/******/ 			return getter;
/******/ 		};
/******/ 	})();
/******/ 	
/******/ 	/* webpack/runtime/define property getters */
/******/ 	(() => {
/******/ 		// define getter functions for harmony exports
/******/ 		__webpack_require__.d = (exports, definition) => {
/******/ 			for(var key in definition) {
/******/ 				if(__webpack_require__.o(definition, key) && !__webpack_require__.o(exports, key)) {
/******/ 					Object.defineProperty(exports, key, { enumerable: true, get: definition[key] });
/******/ 				}
/******/ 			}
/******/ 		};
/******/ 	})();
/******/ 	
/******/ 	/* webpack/runtime/hasOwnProperty shorthand */
/******/ 	(() => {
/******/ 		__webpack_require__.o = (obj, prop) => (Object.prototype.hasOwnProperty.call(obj, prop))
/******/ 	})();
/******/ 	
/******/ 	/* webpack/runtime/make namespace object */
/******/ 	(() => {
/******/ 		// define __esModule on exports
/******/ 		__webpack_require__.r = (exports) => {
/******/ 			if(typeof Symbol !== 'undefined' && Symbol.toStringTag) {
/******/ 				Object.defineProperty(exports, Symbol.toStringTag, { value: 'Module' });
/******/ 			}
/******/ 			Object.defineProperty(exports, '__esModule', { value: true });
/******/ 		};
/******/ 	})();
/******/ 	
/************************************************************************/
var __webpack_exports__ = {};
// This entry needs to be wrapped in an IIFE because it needs to be in strict mode.
(() => {
"use strict";
__webpack_require__.r(__webpack_exports__);
/* harmony import */ var rrule__WEBPACK_IMPORTED_MODULE_0__ = __webpack_require__(1078);
/* harmony import */ var rrule__WEBPACK_IMPORTED_MODULE_0___default = /*#__PURE__*/__webpack_require__.n(rrule__WEBPACK_IMPORTED_MODULE_0__);
/**
 * ---------------------------------------------------------------------
 *
 * GLPI - Gestionnaire Libre de Parc Informatique
 *
 * http://glpi-project.org
 *
 * @copyright 2015-2026 Teclib' and contributors.
 * @copyright 2003-2014 by the INDEPNET Development Team.
 * @licence   https://www.gnu.org/licenses/gpl-3.0.html
 *
 * ---------------------------------------------------------------------
 *
 * LICENSE
 *
 * This file is part of GLPI.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------
 */

// RRule dependency
// lib is exported as a module, so it must be imported as a module


// Fullcalendar library
__webpack_require__(1080);

// Plugins
__webpack_require__(1082);
__webpack_require__(1084);
__webpack_require__(1086);
__webpack_require__(1088);
__webpack_require__(1090);
__webpack_require__(1092);
__webpack_require__(1094);
__webpack_require__(1096);
__webpack_require__(1098);

__webpack_require__(1100);
__webpack_require__(1101);
__webpack_require__(1102);
__webpack_require__(1103);
__webpack_require__(1104);
__webpack_require__(1105);
__webpack_require__(1106);


})();

/******/ })()
;
//# sourceMappingURL=fullcalendar.js.map