@charset "UTF-8";
/*!
 * ---------------------------------------------------------------------
 *
 * GLPI - Gestionnaire Libre de Parc Informatique
 *
 * http://glpi-project.org
 *
 * @copyright 2015-2026 Teclib' and contributors.
 * @licence   https://www.gnu.org/licenses/gpl-3.0.html
 *
 * ---------------------------------------------------------------------
 *
 * LICENSE
 *
 * This file is part of GLPI.
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 *
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License
 * along with this program.  If not, see <https://www.gnu.org/licenses/>.
 *
 * ---------------------------------------------------------------------
 */
/**
 * Converts a given value to a percentage string.
 *
 * @param {Number} $value - The value to be converted to a percentage.
 * @return {String} - The percentage representation of the value.
 */
/**
 * Generates a transparent version of the given color.
 *
 * @param {Color} $color - The base color to be made transparent.
 * @param {Number} $alpha - The level of transparency, ranging from 0 (fully transparent) to 1 (fully opaque). Default is 1.
 * @return {Color} - The resulting color with the specified transparency.
 */
:root,
[data-bs-theme=light] {
  --tblr-blue: #066fd1;
  --tblr-indigo: #4263eb;
  --tblr-purple: #ae3ec9;
  --tblr-pink: #d6336c;
  --tblr-red: #d63939;
  --tblr-orange: #f76707;
  --tblr-yellow: #f59f00;
  --tblr-green: #2fb344;
  --tblr-teal: #0ca678;
  --tblr-cyan: #17a2b8;
  --tblr-black: #000000;
  --tblr-white: #ffffff;
  --tblr-gray: #4b5563;
  --tblr-gray-dark: #1f2937;
  --tblr-gray-100: #f3f4f6;
  --tblr-gray-200: #e5e7eb;
  --tblr-gray-300: #d1d5db;
  --tblr-gray-400: #9ca3af;
  --tblr-gray-500: #6b7280;
  --tblr-gray-600: #4b5563;
  --tblr-gray-700: #374151;
  --tblr-gray-800: #1f2937;
  --tblr-gray-900: #111827;
  --tblr-primary: #066fd1;
  --tblr-secondary: #6b7280;
  --tblr-success: #2fb344;
  --tblr-info: #4299e1;
  --tblr-warning: #f59f00;
  --tblr-danger: #d63939;
  --tblr-light: #f9fafb;
  --tblr-dark: #1f2937;
  --tblr-muted: #6b7280;
  --tblr-blue: #066fd1;
  --tblr-azure: #4299e1;
  --tblr-indigo: #4263eb;
  --tblr-purple: #ae3ec9;
  --tblr-pink: #d6336c;
  --tblr-red: #d63939;
  --tblr-orange: #f76707;
  --tblr-yellow: #f59f00;
  --tblr-lime: #74b816;
  --tblr-green: #2fb344;
  --tblr-teal: #0ca678;
  --tblr-cyan: #17a2b8;
  --tblr-primary-rgb: 6, 111, 209;
  --tblr-secondary-rgb: 107, 114, 128;
  --tblr-success-rgb: 47, 179, 68;
  --tblr-info-rgb: 66, 153, 225;
  --tblr-warning-rgb: 245, 159, 0;
  --tblr-danger-rgb: 214, 57, 57;
  --tblr-light-rgb: 249, 250, 251;
  --tblr-dark-rgb: 31, 41, 55;
  --tblr-muted-rgb: 107, 114, 128;
  --tblr-blue-rgb: 6, 111, 209;
  --tblr-azure-rgb: 66, 153, 225;
  --tblr-indigo-rgb: 66, 99, 235;
  --tblr-purple-rgb: 174, 62, 201;
  --tblr-pink-rgb: 214, 51, 108;
  --tblr-red-rgb: 214, 57, 57;
  --tblr-orange-rgb: 247, 103, 7;
  --tblr-yellow-rgb: 245, 159, 0;
  --tblr-lime-rgb: 116, 184, 22;
  --tblr-green-rgb: 47, 179, 68;
  --tblr-teal-rgb: 12, 166, 120;
  --tblr-cyan-rgb: 23, 162, 184;
  --tblr-primary-text-emphasis: rgb(2.4, 44.4, 83.6);
  --tblr-secondary-text-emphasis: rgb(42.8, 45.6, 51.2);
  --tblr-success-text-emphasis: rgb(18.8, 71.6, 27.2);
  --tblr-info-text-emphasis: rgb(26.4, 61.2, 90);
  --tblr-warning-text-emphasis: rgb(98, 63.6, 0);
  --tblr-danger-text-emphasis: rgb(85.6, 22.8, 22.8);
  --tblr-light-text-emphasis: #374151;
  --tblr-dark-text-emphasis: #374151;
  --tblr-primary-bg-subtle: rgb(205.2, 226.2, 245.8);
  --tblr-secondary-bg-subtle: rgb(225.4, 226.8, 229.6);
  --tblr-success-bg-subtle: rgb(213.4, 239.8, 217.6);
  --tblr-info-bg-subtle: rgb(217.2, 234.6, 249);
  --tblr-warning-bg-subtle: rgb(253, 235.8, 204);
  --tblr-danger-bg-subtle: rgb(246.8, 215.4, 215.4);
  --tblr-light-bg-subtle: rgb(249, 249.5, 250.5);
  --tblr-dark-bg-subtle: #9ca3af;
  --tblr-primary-border-subtle: rgb(155.4, 197.4, 236.6);
  --tblr-secondary-border-subtle: rgb(195.8, 198.6, 204.2);
  --tblr-success-border-subtle: rgb(171.8, 224.6, 180.2);
  --tblr-info-border-subtle: rgb(179.4, 214.2, 243);
  --tblr-warning-border-subtle: rgb(251, 216.6, 153);
  --tblr-danger-border-subtle: rgb(238.6, 175.8, 175.8);
  --tblr-light-border-subtle: #e5e7eb;
  --tblr-dark-border-subtle: #6b7280;
  --tblr-white-rgb: 255, 255, 255;
  --tblr-black-rgb: 0, 0, 0;
  --tblr-font-sans-serif:    "Inter Var", Inter, -apple-system, BlinkMacSystemFont, San Francisco, Segoe UI, Roboto, Helvetica Neue, sans-serif;
  --tblr-font-monospace:  Monaco, Consolas, Liberation Mono, Courier New, monospace;
  --tblr-gradient: linear-gradient(180deg, rgba(255, 255, 255, 0.15), rgba(255, 255, 255, 0));
  --tblr-body-font-family: var(--tblr-font-sans-serif);
  --tblr-body-font-size: 0.875rem;
  --tblr-body-font-weight: 400;
  --tblr-body-line-height: 1.4285714286;
  --tblr-body-color: #1f2937;
  --tblr-body-color-rgb: 31, 41, 55;
  --tblr-body-bg: #f9fafb;
  --tblr-body-bg-rgb: 249, 250, 251;
  --tblr-emphasis-color: #374151;
  --tblr-emphasis-color-rgb: 55, 65, 81;
  --tblr-secondary-color: rgba(31, 41, 55, 0.75);
  --tblr-secondary-color-rgb: 31, 41, 55;
  --tblr-secondary-bg: #e5e7eb;
  --tblr-secondary-bg-rgb: 229, 231, 235;
  --tblr-tertiary-color: rgba(31, 41, 55, 0.5);
  --tblr-tertiary-color-rgb: 31, 41, 55;
  --tblr-tertiary-bg: #f3f4f6;
  --tblr-tertiary-bg-rgb: 243, 244, 246;
  --tblr-heading-color: inherit;
  --tblr-link-color: #066fd1;
  --tblr-link-color-rgb: 6, 111, 209;
  --tblr-link-decoration: none;
  --tblr-link-hover-color: rgb(4.8, 88.8, 167.2);
  --tblr-link-hover-color-rgb: 5, 89, 167;
  --tblr-link-hover-decoration: none;
  --tblr-code-color: light-dark(var(--tblr-gray-600), var(--tblr-gray-400));
  --tblr-highlight-color: #1f2937;
  --tblr-highlight-bg: rgb(253, 235.8, 204);
  --tblr-border-width: 1px;
  --tblr-border-style: solid;
  --tblr-border-color: #e5e7eb;
  --tblr-border-color-translucent: rgba(4, 32, 69, 0.1);
  --tblr-border-radius: 6px;
  --tblr-border-radius-sm: 4px;
  --tblr-border-radius-lg: 8px;
  --tblr-border-radius-xl: 1rem;
  --tblr-border-radius-xxl: 2rem;
  --tblr-border-radius-2xl: var(--tblr-border-radius-xxl);
  --tblr-border-radius-pill: 100rem;
  --tblr-box-shadow: rgba(var(--tblr-body-color-rgb), 0.04) 0 2px 4px 0;
  --tblr-box-shadow-sm: 0 0.125rem 0.25rem rgba(0, 0, 0, 0.075);
  --tblr-box-shadow-lg: 0 1rem 3rem rgba(0, 0, 0, 0.175);
  --tblr-box-shadow-inset: 0 0 transparent;
  --tblr-focus-ring-width: 0.25rem;
  --tblr-focus-ring-opacity: 0.25;
  --tblr-focus-ring-color: rgba(var(--tblr-primary-rgb), 0.25);
  --tblr-form-valid-color: #2fb344;
  --tblr-form-valid-border-color: #2fb344;
  --tblr-form-invalid-color: #d63939;
  --tblr-form-invalid-border-color: #d63939;
}

[data-bs-theme=dark], body[data-bs-theme=dark] [data-bs-theme=light] {
  color-scheme: dark;
  --tblr-body-color: #e5e7eb;
  --tblr-body-color-rgb: 229, 231, 235;
  --tblr-body-bg: #111827;
  --tblr-body-bg-rgb: 17, 24, 39;
  --tblr-emphasis-color: #ffffff;
  --tblr-emphasis-color-rgb: 255, 255, 255;
  --tblr-secondary-color: rgba(229, 231, 235, 0.75);
  --tblr-secondary-color-rgb: 229, 231, 235;
  --tblr-secondary-bg: #1f2937;
  --tblr-secondary-bg-rgb: 31, 41, 55;
  --tblr-tertiary-color: rgba(229, 231, 235, 0.5);
  --tblr-tertiary-color-rgb: 229, 231, 235;
  --tblr-tertiary-bg: rgb(24, 32.5, 47);
  --tblr-tertiary-bg-rgb: 24, 33, 47;
  --tblr-primary-text-emphasis: rgb(105.6, 168.6, 227.4);
  --tblr-secondary-text-emphasis: rgb(166.2, 170.4, 178.8);
  --tblr-success-text-emphasis: rgb(130.2, 209.4, 142.8);
  --tblr-info-text-emphasis: rgb(141.6, 193.8, 237);
  --tblr-warning-text-emphasis: rgb(249, 197.4, 102);
  --tblr-danger-text-emphasis: rgb(230.4, 136.2, 136.2);
  --tblr-light-text-emphasis: #f3f4f6;
  --tblr-dark-text-emphasis: #d1d5db;
  --tblr-primary-bg-subtle: rgb(1.2, 22.2, 41.8);
  --tblr-secondary-bg-subtle: rgb(21.4, 22.8, 25.6);
  --tblr-success-bg-subtle: rgb(9.4, 35.8, 13.6);
  --tblr-info-bg-subtle: rgb(13.2, 30.6, 45);
  --tblr-warning-bg-subtle: rgb(49, 31.8, 0);
  --tblr-danger-bg-subtle: rgb(42.8, 11.4, 11.4);
  --tblr-light-bg-subtle: #1f2937;
  --tblr-dark-bg-subtle: rgb(15.5, 20.5, 27.5);
  --tblr-primary-border-subtle: rgb(3.6, 66.6, 125.4);
  --tblr-secondary-border-subtle: rgb(64.2, 68.4, 76.8);
  --tblr-success-border-subtle: rgb(28.2, 107.4, 40.8);
  --tblr-info-border-subtle: rgb(39.6, 91.8, 135);
  --tblr-warning-border-subtle: rgb(147, 95.4, 0);
  --tblr-danger-border-subtle: rgb(128.4, 34.2, 34.2);
  --tblr-light-border-subtle: #374151;
  --tblr-dark-border-subtle: #1f2937;
  --tblr-heading-color: inherit;
  --tblr-link-color: rgb(105.6, 168.6, 227.4);
  --tblr-link-hover-color: rgb(135.48, 185.88, 232.92);
  --tblr-link-color-rgb: 106, 169, 227;
  --tblr-link-hover-color-rgb: 135, 186, 233;
  --tblr-code-color: var(--tblr-gray-300);
  --tblr-highlight-color: #e5e7eb;
  --tblr-highlight-bg: rgb(98, 63.6, 0);
  --tblr-border-color: rgb(45.7069767442, 60.4511627907, 81.0930232558);
  --tblr-border-color-translucent: rgba(72, 110, 149, 0.14);
  --tblr-form-valid-color: rgb(130.2, 209.4, 142.8);
  --tblr-form-valid-border-color: rgb(130.2, 209.4, 142.8);
  --tblr-form-invalid-color: rgb(230.4, 136.2, 136.2);
  --tblr-form-invalid-border-color: rgb(230.4, 136.2, 136.2);
}

*,
*::before,
*::after {
  box-sizing: border-box;
}

@media (prefers-reduced-motion: no-preference) {
  :root {
    scroll-behavior: smooth;
  }
}

body {
  margin: 0;
  font-family: var(--tblr-body-font-family);
  font-size: var(--tblr-body-font-size);
  font-weight: var(--tblr-body-font-weight);
  line-height: var(--tblr-body-line-height);
  color: var(--tblr-body-color);
  text-align: var(--tblr-body-text-align);
  background-color: var(--tblr-body-bg);
  -webkit-text-size-adjust: 100%;
  -webkit-tap-highlight-color: rgba(0, 0, 0, 0);
}

hr, .hr {
  margin: 2rem 0;
  color: inherit;
  border: 0;
  border-top: var(--tblr-border-width) solid;
  opacity: 0.16;
}

h6, .h6, h5, .h5, h4, .h4, h3, .h3, h2, .h2, h1, .h1 {
  margin-top: 0;
  margin-bottom: var(--tblr-spacer);
  font-weight: var(--tblr-font-weight-bold);
  line-height: 1.2;
  color: var(--tblr-heading-color);
}

h1, .h1 {
  font-size: 1.5rem;
}

h2, .h2 {
  font-size: 1.25rem;
}

h3, .h3 {
  font-size: 1rem;
}

h4, .h4 {
  font-size: 0.875rem;
}

h5, .h5 {
  font-size: 0.75rem;
}

h6, .h6 {
  font-size: 0.625rem;
}

p {
  margin-top: 0;
  margin-bottom: 1rem;
}

abbr[title] {
  text-decoration: underline dotted;
  cursor: help;
  text-decoration-skip-ink: none;
}

address {
  margin-bottom: 1rem;
  font-style: normal;
  line-height: inherit;
}

ol,
ul {
  padding-right: 2rem;
}

ol,
ul,
dl {
  margin-top: 0;
  margin-bottom: 1rem;
}

ol ol,
ul ul,
ol ul,
ul ol {
  margin-bottom: 0;
}

dt {
  font-weight: 600;
}

dd {
  margin-bottom: 0.5rem;
  margin-right: 0;
}

blockquote {
  margin: 0 0 1rem;
}

b,
strong {
  font-weight: bolder;
}

small, .small {
  font-size: 0.875em;
}

mark, .mark {
  padding: 0.1875em;
  color: var(--tblr-highlight-color);
  background-color: var(--tblr-highlight-bg);
}

sub,
sup {
  position: relative;
  font-size: 0.75em;
  line-height: 0;
  vertical-align: baseline;
}

sub {
  bottom: -0.25em;
}

sup {
  top: -0.5em;
}

a {
  color: rgba(var(--tblr-link-color-rgb), var(--tblr-link-opacity, 1));
  text-decoration: none;
}
a:hover {
  --tblr-link-color-rgb: var(--tblr-link-hover-color-rgb);
  text-decoration: none;
}

a:not([href]):not([class]), a:not([href]):not([class]):hover {
  color: inherit;
  text-decoration: none;
}

pre,
code,
kbd,
samp {
  font-family: var(--tblr-font-monospace);
  font-size: 1em;
}

pre {
  display: block;
  margin-top: 0;
  margin-bottom: 1rem;
  overflow: auto;
  font-size: 0.85714285em;
  color: var(--tblr-light);
}
pre code {
  font-size: inherit;
  color: inherit;
  word-break: normal;
}

code {
  font-size: 0.85714285em;
  color: var(--tblr-code-color);
  word-wrap: break-word;
}
a > code {
  color: inherit;
}

kbd {
  padding: 0.25rem 0.5rem;
  font-size: var(--tblr-font-size-h5);
  color: var(--tblr-text-secondary-dark);
  background-color: var(--tblr-code-bg);
  border-radius: 4px;
}
kbd kbd {
  padding: 0;
  font-size: 1em;
}

figure {
  margin: 0 0 1rem;
}

img,
svg {
  vertical-align: middle;
}

table {
  caption-side: bottom;
  border-collapse: collapse;
}

caption {
  padding-top: 0.75rem;
  padding-bottom: 0.75rem;
  color: var(--tblr-secondary-color);
  text-align: right;
}

th {
  text-align: inherit;
  text-align: -webkit-match-parent;
}

thead,
tbody,
tfoot,
tr,
td,
th {
  border-color: inherit;
  border-style: solid;
  border-width: 0;
}

label {
  display: inline-block;
}

button {
  border-radius: 0;
}

button:focus:not(:focus-visible) {
  outline: 0;
}

input,
button,
select,
optgroup,
textarea {
  margin: 0;
  font-family: inherit;
  font-size: inherit;
  line-height: inherit;
}

button,
select {
  text-transform: none;
}

[role=button] {
  cursor: pointer;
}

select {
  word-wrap: normal;
}
select:disabled {
  opacity: 1;
}

[list]:not([type=date]):not([type=datetime-local]):not([type=month]):not([type=week]):not([type=time])::-webkit-calendar-picker-indicator {
  display: none !important;
}

button,
[type=button],
[type=reset],
[type=submit] {
  -webkit-appearance: button;
}
button:not(:disabled),
[type=button]:not(:disabled),
[type=reset]:not(:disabled),
[type=submit]:not(:disabled) {
  cursor: pointer;
}

::-moz-focus-inner {
  padding: 0;
  border-style: none;
}

textarea {
  resize: vertical;
}

fieldset {
  min-width: 0;
  padding: 0;
  margin: 0;
  border: 0;
}

legend {
  float: right;
  width: 100%;
  padding: 0;
  margin-bottom: 0.5rem;
  line-height: inherit;
  font-size: 1.5rem;
}
legend + * {
  clear: right;
}

::-webkit-datetime-edit-fields-wrapper,
::-webkit-datetime-edit-text,
::-webkit-datetime-edit-minute,
::-webkit-datetime-edit-hour-field,
::-webkit-datetime-edit-day-field,
::-webkit-datetime-edit-month-field,
::-webkit-datetime-edit-year-field {
  padding: 0;
}

::-webkit-inner-spin-button {
  height: auto;
}

[type=search] {
  -webkit-appearance: textfield;
  outline-offset: -2px;
}

[type="tel"],
[type="url"],
[type="email"],
[type="number"] {
  direction: ltr;
}
::-webkit-search-decoration {
  -webkit-appearance: none;
}

::-webkit-color-swatch-wrapper {
  padding: 0;
}

::file-selector-button {
  font: inherit;
  -webkit-appearance: button;
}

output {
  display: inline-block;
}

iframe {
  border: 0;
}

summary {
  display: list-item;
  cursor: pointer;
}

progress {
  vertical-align: baseline;
}

[hidden] {
  display: none !important;
}

.lead {
  font-size: 0.875rem;
  font-weight: var(--tblr-font-weight-normal);
}

.display-1 {
  font-weight: 300;
  line-height: 1.2;
  font-size: 5rem;
}

.display-2 {
  font-weight: 300;
  line-height: 1.2;
  font-size: 4.5rem;
}

.display-3 {
  font-weight: 300;
  line-height: 1.2;
  font-size: 4rem;
}

.display-4 {
  font-weight: 300;
  line-height: 1.2;
  font-size: 3.5rem;
}

.display-5 {
  font-weight: 300;
  line-height: 1.2;
  font-size: 3rem;
}

.display-6 {
  font-weight: 300;
  line-height: 1.2;
  font-size: 2rem;
}

.list-unstyled {
  padding-right: 0;
  list-style: none;
}

.list-inline {
  padding-right: 0;
  list-style: none;
}

.list-inline-item {
  display: inline-block;
}
.list-inline-item:not(:last-child) {
  margin-left: 0.5rem;
}

.initialism {
  font-size: 0.875em;
  text-transform: uppercase;
}

.blockquote {
  margin-bottom: 1rem;
  font-size: 0.875rem;
}
.blockquote > :last-child {
  margin-bottom: 0;
}

.blockquote-footer {
  margin-top: -1rem;
  margin-bottom: 1rem;
  font-size: 0.875em;
  color: #4b5563;
}
.blockquote-footer::before {
  content: "— ";
}

.img-fluid {
  max-width: 100%;
  height: auto;
}

.img-thumbnail {
  padding: 0.25rem;
  background-color: var(--tblr-body-bg);
  border: var(--tblr-border-width) solid var(--tblr-border-color);
  border-radius: var(--tblr-border-radius);
  box-shadow: var(--tblr-box-shadow-sm);
  max-width: 100%;
  height: auto;
}

.figure {
  display: inline-block;
}

.figure-img {
  margin-bottom: 0.5rem;
  line-height: 1;
}

.figure-caption {
  font-size: 0.875em;
  color: var(--tblr-secondary-color);
}

.container,
.container-fluid,
.container-xxl,
.container-xl,
.container-lg,
.container-md,
.container-sm {
  --tblr-gutter-x: calc(var(--tblr-page-padding) * 2);
  --tblr-gutter-y: 0;
  width: 100%;
  padding-left: calc(var(--tblr-gutter-x) * 0.5);
  padding-right: calc(var(--tblr-gutter-x) * 0.5);
  margin-left: auto;
  margin-right: auto;
}

@media (min-width: 576px) {
  .container-sm, .container {
    max-width: 540px;
  }
}
@media (min-width: 768px) {
  .container-md, .container-sm, .container {
    max-width: 720px;
  }
}
@media (min-width: 992px) {
  .container-lg, .container-md, .container-sm, .container {
    max-width: 960px;
  }
}
@media (min-width: 1200px) {
  .container-xl, .container-lg, .container-md, .container-sm, .container {
    max-width: 1140px;
  }
}
@media (min-width: 1400px) {
  .container-xxl, .container-xl, .container-lg, .container-md, .container-sm, .container {
    max-width: 1320px;
  }
}
:root {
  --tblr-breakpoint-xs: 0;
  --tblr-breakpoint-sm: 576px;
  --tblr-breakpoint-md: 768px;
  --tblr-breakpoint-lg: 992px;
  --tblr-breakpoint-xl: 1200px;
  --tblr-breakpoint-xxl: 1400px;
}

.row {
  --tblr-gutter-x: var(--tblr-page-padding);
  --tblr-gutter-y: 0;
  display: flex;
  flex-wrap: wrap;
  margin-top: calc(-1 * var(--tblr-gutter-y));
  margin-left: calc(-0.5 * var(--tblr-gutter-x));
  margin-right: calc(-0.5 * var(--tblr-gutter-x));
}
.row > * {
  flex-shrink: 0;
  width: 100%;
  max-width: 100%;
  padding-left: calc(var(--tblr-gutter-x) * 0.5);
  padding-right: calc(var(--tblr-gutter-x) * 0.5);
  margin-top: var(--tblr-gutter-y);
}

.grid {
  display: grid;
  grid-template-rows: repeat(var(--tblr-rows, 1), 1fr);
  grid-template-columns: repeat(var(--tblr-columns, 12), 1fr);
  gap: var(--tblr-gap, var(--tblr-page-padding));
}
.grid .g-col-1 {
  grid-column: auto/span 1;
}
.grid .g-col-2 {
  grid-column: auto/span 2;
}
.grid .g-col-3 {
  grid-column: auto/span 3;
}
.grid .g-col-4 {
  grid-column: auto/span 4;
}
.grid .g-col-5 {
  grid-column: auto/span 5;
}
.grid .g-col-6 {
  grid-column: auto/span 6;
}
.grid .g-col-7 {
  grid-column: auto/span 7;
}
.grid .g-col-8 {
  grid-column: auto/span 8;
}
.grid .g-col-9 {
  grid-column: auto/span 9;
}
.grid .g-col-10 {
  grid-column: auto/span 10;
}
.grid .g-col-11 {
  grid-column: auto/span 11;
}
.grid .g-col-12 {
  grid-column: auto/span 12;
}
.grid .g-start-1 {
  grid-column-start: 1;
}
.grid .g-start-2 {
  grid-column-start: 2;
}
.grid .g-start-3 {
  grid-column-start: 3;
}
.grid .g-start-4 {
  grid-column-start: 4;
}
.grid .g-start-5 {
  grid-column-start: 5;
}
.grid .g-start-6 {
  grid-column-start: 6;
}
.grid .g-start-7 {
  grid-column-start: 7;
}
.grid .g-start-8 {
  grid-column-start: 8;
}
.grid .g-start-9 {
  grid-column-start: 9;
}
.grid .g-start-10 {
  grid-column-start: 10;
}
.grid .g-start-11 {
  grid-column-start: 11;
}
@media (min-width: 576px) {
  .grid .g-col-sm-1 {
    grid-column: auto/span 1;
  }
  .grid .g-col-sm-2 {
    grid-column: auto/span 2;
  }
  .grid .g-col-sm-3 {
    grid-column: auto/span 3;
  }
  .grid .g-col-sm-4 {
    grid-column: auto/span 4;
  }
  .grid .g-col-sm-5 {
    grid-column: auto/span 5;
  }
  .grid .g-col-sm-6 {
    grid-column: auto/span 6;
  }
  .grid .g-col-sm-7 {
    grid-column: auto/span 7;
  }
  .grid .g-col-sm-8 {
    grid-column: auto/span 8;
  }
  .grid .g-col-sm-9 {
    grid-column: auto/span 9;
  }
  .grid .g-col-sm-10 {
    grid-column: auto/span 10;
  }
  .grid .g-col-sm-11 {
    grid-column: auto/span 11;
  }
  .grid .g-col-sm-12 {
    grid-column: auto/span 12;
  }
  .grid .g-start-sm-1 {
    grid-column-start: 1;
  }
  .grid .g-start-sm-2 {
    grid-column-start: 2;
  }
  .grid .g-start-sm-3 {
    grid-column-start: 3;
  }
  .grid .g-start-sm-4 {
    grid-column-start: 4;
  }
  .grid .g-start-sm-5 {
    grid-column-start: 5;
  }
  .grid .g-start-sm-6 {
    grid-column-start: 6;
  }
  .grid .g-start-sm-7 {
    grid-column-start: 7;
  }
  .grid .g-start-sm-8 {
    grid-column-start: 8;
  }
  .grid .g-start-sm-9 {
    grid-column-start: 9;
  }
  .grid .g-start-sm-10 {
    grid-column-start: 10;
  }
  .grid .g-start-sm-11 {
    grid-column-start: 11;
  }
}
@media (min-width: 768px) {
  .grid .g-col-md-1 {
    grid-column: auto/span 1;
  }
  .grid .g-col-md-2 {
    grid-column: auto/span 2;
  }
  .grid .g-col-md-3 {
    grid-column: auto/span 3;
  }
  .grid .g-col-md-4 {
    grid-column: auto/span 4;
  }
  .grid .g-col-md-5 {
    grid-column: auto/span 5;
  }
  .grid .g-col-md-6 {
    grid-column: auto/span 6;
  }
  .grid .g-col-md-7 {
    grid-column: auto/span 7;
  }
  .grid .g-col-md-8 {
    grid-column: auto/span 8;
  }
  .grid .g-col-md-9 {
    grid-column: auto/span 9;
  }
  .grid .g-col-md-10 {
    grid-column: auto/span 10;
  }
  .grid .g-col-md-11 {
    grid-column: auto/span 11;
  }
  .grid .g-col-md-12 {
    grid-column: auto/span 12;
  }
  .grid .g-start-md-1 {
    grid-column-start: 1;
  }
  .grid .g-start-md-2 {
    grid-column-start: 2;
  }
  .grid .g-start-md-3 {
    grid-column-start: 3;
  }
  .grid .g-start-md-4 {
    grid-column-start: 4;
  }
  .grid .g-start-md-5 {
    grid-column-start: 5;
  }
  .grid .g-start-md-6 {
    grid-column-start: 6;
  }
  .grid .g-start-md-7 {
    grid-column-start: 7;
  }
  .grid .g-start-md-8 {
    grid-column-start: 8;
  }
  .grid .g-start-md-9 {
    grid-column-start: 9;
  }
  .grid .g-start-md-10 {
    grid-column-start: 10;
  }
  .grid .g-start-md-11 {
    grid-column-start: 11;
  }
}
@media (min-width: 992px) {
  .grid .g-col-lg-1 {
    grid-column: auto/span 1;
  }
  .grid .g-col-lg-2 {
    grid-column: auto/span 2;
  }
  .grid .g-col-lg-3 {
    grid-column: auto/span 3;
  }
  .grid .g-col-lg-4 {
    grid-column: auto/span 4;
  }
  .grid .g-col-lg-5 {
    grid-column: auto/span 5;
  }
  .grid .g-col-lg-6 {
    grid-column: auto/span 6;
  }
  .grid .g-col-lg-7 {
    grid-column: auto/span 7;
  }
  .grid .g-col-lg-8 {
    grid-column: auto/span 8;
  }
  .grid .g-col-lg-9 {
    grid-column: auto/span 9;
  }
  .grid .g-col-lg-10 {
    grid-column: auto/span 10;
  }
  .grid .g-col-lg-11 {
    grid-column: auto/span 11;
  }
  .grid .g-col-lg-12 {
    grid-column: auto/span 12;
  }
  .grid .g-start-lg-1 {
    grid-column-start: 1;
  }
  .grid .g-start-lg-2 {
    grid-column-start: 2;
  }
  .grid .g-start-lg-3 {
    grid-column-start: 3;
  }
  .grid .g-start-lg-4 {
    grid-column-start: 4;
  }
  .grid .g-start-lg-5 {
    grid-column-start: 5;
  }
  .grid .g-start-lg-6 {
    grid-column-start: 6;
  }
  .grid .g-start-lg-7 {
    grid-column-start: 7;
  }
  .grid .g-start-lg-8 {
    grid-column-start: 8;
  }
  .grid .g-start-lg-9 {
    grid-column-start: 9;
  }
  .grid .g-start-lg-10 {
    grid-column-start: 10;
  }
  .grid .g-start-lg-11 {
    grid-column-start: 11;
  }
}
@media (min-width: 1200px) {
  .grid .g-col-xl-1 {
    grid-column: auto/span 1;
  }
  .grid .g-col-xl-2 {
    grid-column: auto/span 2;
  }
  .grid .g-col-xl-3 {
    grid-column: auto/span 3;
  }
  .grid .g-col-xl-4 {
    grid-column: auto/span 4;
  }
  .grid .g-col-xl-5 {
    grid-column: auto/span 5;
  }
  .grid .g-col-xl-6 {
    grid-column: auto/span 6;
  }
  .grid .g-col-xl-7 {
    grid-column: auto/span 7;
  }
  .grid .g-col-xl-8 {
    grid-column: auto/span 8;
  }
  .grid .g-col-xl-9 {
    grid-column: auto/span 9;
  }
  .grid .g-col-xl-10 {
    grid-column: auto/span 10;
  }
  .grid .g-col-xl-11 {
    grid-column: auto/span 11;
  }
  .grid .g-col-xl-12 {
    grid-column: auto/span 12;
  }
  .grid .g-start-xl-1 {
    grid-column-start: 1;
  }
  .grid .g-start-xl-2 {
    grid-column-start: 2;
  }
  .grid .g-start-xl-3 {
    grid-column-start: 3;
  }
  .grid .g-start-xl-4 {
    grid-column-start: 4;
  }
  .grid .g-start-xl-5 {
    grid-column-start: 5;
  }
  .grid .g-start-xl-6 {
    grid-column-start: 6;
  }
  .grid .g-start-xl-7 {
    grid-column-start: 7;
  }
  .grid .g-start-xl-8 {
    grid-column-start: 8;
  }
  .grid .g-start-xl-9 {
    grid-column-start: 9;
  }
  .grid .g-start-xl-10 {
    grid-column-start: 10;
  }
  .grid .g-start-xl-11 {
    grid-column-start: 11;
  }
}
@media (min-width: 1400px) {
  .grid .g-col-xxl-1 {
    grid-column: auto/span 1;
  }
  .grid .g-col-xxl-2 {
    grid-column: auto/span 2;
  }
  .grid .g-col-xxl-3 {
    grid-column: auto/span 3;
  }
  .grid .g-col-xxl-4 {
    grid-column: auto/span 4;
  }
  .grid .g-col-xxl-5 {
    grid-column: auto/span 5;
  }
  .grid .g-col-xxl-6 {
    grid-column: auto/span 6;
  }
  .grid .g-col-xxl-7 {
    grid-column: auto/span 7;
  }
  .grid .g-col-xxl-8 {
    grid-column: auto/span 8;
  }
  .grid .g-col-xxl-9 {
    grid-column: auto/span 9;
  }
  .grid .g-col-xxl-10 {
    grid-column: auto/span 10;
  }
  .grid .g-col-xxl-11 {
    grid-column: auto/span 11;
  }
  .grid .g-col-xxl-12 {
    grid-column: auto/span 12;
  }
  .grid .g-start-xxl-1 {
    grid-column-start: 1;
  }
  .grid .g-start-xxl-2 {
    grid-column-start: 2;
  }
  .grid .g-start-xxl-3 {
    grid-column-start: 3;
  }
  .grid .g-start-xxl-4 {
    grid-column-start: 4;
  }
  .grid .g-start-xxl-5 {
    grid-column-start: 5;
  }
  .grid .g-start-xxl-6 {
    grid-column-start: 6;
  }
  .grid .g-start-xxl-7 {
    grid-column-start: 7;
  }
  .grid .g-start-xxl-8 {
    grid-column-start: 8;
  }
  .grid .g-start-xxl-9 {
    grid-column-start: 9;
  }
  .grid .g-start-xxl-10 {
    grid-column-start: 10;
  }
  .grid .g-start-xxl-11 {
    grid-column-start: 11;
  }
}

.col {
  flex: 1 0 0;
}

.row-cols-auto > * {
  flex: 0 0 auto;
  width: auto;
}

.row-cols-1 > * {
  flex: 0 0 auto;
  width: 100%;
}

.row-cols-2 > * {
  flex: 0 0 auto;
  width: 50%;
}

.row-cols-3 > * {
  flex: 0 0 auto;
  width: 33.33333333%;
}

.row-cols-4 > * {
  flex: 0 0 auto;
  width: 25%;
}

.row-cols-5 > * {
  flex: 0 0 auto;
  width: 20%;
}

.row-cols-6 > * {
  flex: 0 0 auto;
  width: 16.66666667%;
}

.col-auto {
  flex: 0 0 auto;
  width: auto;
}

.col-1 {
  flex: 0 0 auto;
  width: 8.33333333%;
}

.col-2 {
  flex: 0 0 auto;
  width: 16.66666667%;
}

.col-3 {
  flex: 0 0 auto;
  width: 25%;
}

.col-4 {
  flex: 0 0 auto;
  width: 33.33333333%;
}

.col-5 {
  flex: 0 0 auto;
  width: 41.66666667%;
}

.col-6 {
  flex: 0 0 auto;
  width: 50%;
}

.col-7 {
  flex: 0 0 auto;
  width: 58.33333333%;
}

.col-8 {
  flex: 0 0 auto;
  width: 66.66666667%;
}

.col-9 {
  flex: 0 0 auto;
  width: 75%;
}

.col-10 {
  flex: 0 0 auto;
  width: 83.33333333%;
}

.col-11 {
  flex: 0 0 auto;
  width: 91.66666667%;
}

.col-12 {
  flex: 0 0 auto;
  width: 100%;
}

.offset-1 {
  margin-right: 8.33333333%;
}

.offset-2 {
  margin-right: 16.66666667%;
}

.offset-3 {
  margin-right: 25%;
}

.offset-4 {
  margin-right: 33.33333333%;
}

.offset-5 {
  margin-right: 41.66666667%;
}

.offset-6 {
  margin-right: 50%;
}

.offset-7 {
  margin-right: 58.33333333%;
}

.offset-8 {
  margin-right: 66.66666667%;
}

.offset-9 {
  margin-right: 75%;
}

.offset-10 {
  margin-right: 83.33333333%;
}

.offset-11 {
  margin-right: 91.66666667%;
}

.g-0,
.gx-0 {
  --tblr-gutter-x: 0;
}

.g-0,
.gy-0 {
  --tblr-gutter-y: 0;
}

.g-1,
.gx-1 {
  --tblr-gutter-x: 0.25rem;
}

.g-1,
.gy-1 {
  --tblr-gutter-y: 0.25rem;
}

.g-2,
.gx-2 {
  --tblr-gutter-x: 0.5rem;
}

.g-2,
.gy-2 {
  --tblr-gutter-y: 0.5rem;
}

.g-3,
.gx-3 {
  --tblr-gutter-x: 1rem;
}

.g-3,
.gy-3 {
  --tblr-gutter-y: 1rem;
}

.g-4,
.gx-4 {
  --tblr-gutter-x: 1.5rem;
}

.g-4,
.gy-4 {
  --tblr-gutter-y: 1.5rem;
}

.g-5,
.gx-5 {
  --tblr-gutter-x: 2rem;
}

.g-5,
.gy-5 {
  --tblr-gutter-y: 2rem;
}

.g-6,
.gx-6 {
  --tblr-gutter-x: 2.5rem;
}

.g-6,
.gy-6 {
  --tblr-gutter-y: 2.5rem;
}

@media (min-width: 576px) {
  .col-sm {
    flex: 1 0 0;
  }
  .row-cols-sm-auto > * {
    flex: 0 0 auto;
    width: auto;
  }
  .row-cols-sm-1 > * {
    flex: 0 0 auto;
    width: 100%;
  }
  .row-cols-sm-2 > * {
    flex: 0 0 auto;
    width: 50%;
  }
  .row-cols-sm-3 > * {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .row-cols-sm-4 > * {
    flex: 0 0 auto;
    width: 25%;
  }
  .row-cols-sm-5 > * {
    flex: 0 0 auto;
    width: 20%;
  }
  .row-cols-sm-6 > * {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-sm-auto {
    flex: 0 0 auto;
    width: auto;
  }
  .col-sm-1 {
    flex: 0 0 auto;
    width: 8.33333333%;
  }
  .col-sm-2 {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-sm-3 {
    flex: 0 0 auto;
    width: 25%;
  }
  .col-sm-4 {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .col-sm-5 {
    flex: 0 0 auto;
    width: 41.66666667%;
  }
  .col-sm-6 {
    flex: 0 0 auto;
    width: 50%;
  }
  .col-sm-7 {
    flex: 0 0 auto;
    width: 58.33333333%;
  }
  .col-sm-8 {
    flex: 0 0 auto;
    width: 66.66666667%;
  }
  .col-sm-9 {
    flex: 0 0 auto;
    width: 75%;
  }
  .col-sm-10 {
    flex: 0 0 auto;
    width: 83.33333333%;
  }
  .col-sm-11 {
    flex: 0 0 auto;
    width: 91.66666667%;
  }
  .col-sm-12 {
    flex: 0 0 auto;
    width: 100%;
  }
  .offset-sm-0 {
    margin-right: 0;
  }
  .offset-sm-1 {
    margin-right: 8.33333333%;
  }
  .offset-sm-2 {
    margin-right: 16.66666667%;
  }
  .offset-sm-3 {
    margin-right: 25%;
  }
  .offset-sm-4 {
    margin-right: 33.33333333%;
  }
  .offset-sm-5 {
    margin-right: 41.66666667%;
  }
  .offset-sm-6 {
    margin-right: 50%;
  }
  .offset-sm-7 {
    margin-right: 58.33333333%;
  }
  .offset-sm-8 {
    margin-right: 66.66666667%;
  }
  .offset-sm-9 {
    margin-right: 75%;
  }
  .offset-sm-10 {
    margin-right: 83.33333333%;
  }
  .offset-sm-11 {
    margin-right: 91.66666667%;
  }
  .g-sm-0,
  .gx-sm-0 {
    --tblr-gutter-x: 0;
  }
  .g-sm-0,
  .gy-sm-0 {
    --tblr-gutter-y: 0;
  }
  .g-sm-1,
  .gx-sm-1 {
    --tblr-gutter-x: 0.25rem;
  }
  .g-sm-1,
  .gy-sm-1 {
    --tblr-gutter-y: 0.25rem;
  }
  .g-sm-2,
  .gx-sm-2 {
    --tblr-gutter-x: 0.5rem;
  }
  .g-sm-2,
  .gy-sm-2 {
    --tblr-gutter-y: 0.5rem;
  }
  .g-sm-3,
  .gx-sm-3 {
    --tblr-gutter-x: 1rem;
  }
  .g-sm-3,
  .gy-sm-3 {
    --tblr-gutter-y: 1rem;
  }
  .g-sm-4,
  .gx-sm-4 {
    --tblr-gutter-x: 1.5rem;
  }
  .g-sm-4,
  .gy-sm-4 {
    --tblr-gutter-y: 1.5rem;
  }
  .g-sm-5,
  .gx-sm-5 {
    --tblr-gutter-x: 2rem;
  }
  .g-sm-5,
  .gy-sm-5 {
    --tblr-gutter-y: 2rem;
  }
  .g-sm-6,
  .gx-sm-6 {
    --tblr-gutter-x: 2.5rem;
  }
  .g-sm-6,
  .gy-sm-6 {
    --tblr-gutter-y: 2.5rem;
  }
}
@media (min-width: 768px) {
  .col-md {
    flex: 1 0 0;
  }
  .row-cols-md-auto > * {
    flex: 0 0 auto;
    width: auto;
  }
  .row-cols-md-1 > * {
    flex: 0 0 auto;
    width: 100%;
  }
  .row-cols-md-2 > * {
    flex: 0 0 auto;
    width: 50%;
  }
  .row-cols-md-3 > * {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .row-cols-md-4 > * {
    flex: 0 0 auto;
    width: 25%;
  }
  .row-cols-md-5 > * {
    flex: 0 0 auto;
    width: 20%;
  }
  .row-cols-md-6 > * {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-md-auto {
    flex: 0 0 auto;
    width: auto;
  }
  .col-md-1 {
    flex: 0 0 auto;
    width: 8.33333333%;
  }
  .col-md-2 {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-md-3 {
    flex: 0 0 auto;
    width: 25%;
  }
  .col-md-4 {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .col-md-5 {
    flex: 0 0 auto;
    width: 41.66666667%;
  }
  .col-md-6 {
    flex: 0 0 auto;
    width: 50%;
  }
  .col-md-7 {
    flex: 0 0 auto;
    width: 58.33333333%;
  }
  .col-md-8 {
    flex: 0 0 auto;
    width: 66.66666667%;
  }
  .col-md-9 {
    flex: 0 0 auto;
    width: 75%;
  }
  .col-md-10 {
    flex: 0 0 auto;
    width: 83.33333333%;
  }
  .col-md-11 {
    flex: 0 0 auto;
    width: 91.66666667%;
  }
  .col-md-12 {
    flex: 0 0 auto;
    width: 100%;
  }
  .offset-md-0 {
    margin-right: 0;
  }
  .offset-md-1 {
    margin-right: 8.33333333%;
  }
  .offset-md-2 {
    margin-right: 16.66666667%;
  }
  .offset-md-3 {
    margin-right: 25%;
  }
  .offset-md-4 {
    margin-right: 33.33333333%;
  }
  .offset-md-5 {
    margin-right: 41.66666667%;
  }
  .offset-md-6 {
    margin-right: 50%;
  }
  .offset-md-7 {
    margin-right: 58.33333333%;
  }
  .offset-md-8 {
    margin-right: 66.66666667%;
  }
  .offset-md-9 {
    margin-right: 75%;
  }
  .offset-md-10 {
    margin-right: 83.33333333%;
  }
  .offset-md-11 {
    margin-right: 91.66666667%;
  }
  .g-md-0,
  .gx-md-0 {
    --tblr-gutter-x: 0;
  }
  .g-md-0,
  .gy-md-0 {
    --tblr-gutter-y: 0;
  }
  .g-md-1,
  .gx-md-1 {
    --tblr-gutter-x: 0.25rem;
  }
  .g-md-1,
  .gy-md-1 {
    --tblr-gutter-y: 0.25rem;
  }
  .g-md-2,
  .gx-md-2 {
    --tblr-gutter-x: 0.5rem;
  }
  .g-md-2,
  .gy-md-2 {
    --tblr-gutter-y: 0.5rem;
  }
  .g-md-3,
  .gx-md-3 {
    --tblr-gutter-x: 1rem;
  }
  .g-md-3,
  .gy-md-3 {
    --tblr-gutter-y: 1rem;
  }
  .g-md-4,
  .gx-md-4 {
    --tblr-gutter-x: 1.5rem;
  }
  .g-md-4,
  .gy-md-4 {
    --tblr-gutter-y: 1.5rem;
  }
  .g-md-5,
  .gx-md-5 {
    --tblr-gutter-x: 2rem;
  }
  .g-md-5,
  .gy-md-5 {
    --tblr-gutter-y: 2rem;
  }
  .g-md-6,
  .gx-md-6 {
    --tblr-gutter-x: 2.5rem;
  }
  .g-md-6,
  .gy-md-6 {
    --tblr-gutter-y: 2.5rem;
  }
}
@media (min-width: 992px) {
  .col-lg {
    flex: 1 0 0;
  }
  .row-cols-lg-auto > * {
    flex: 0 0 auto;
    width: auto;
  }
  .row-cols-lg-1 > * {
    flex: 0 0 auto;
    width: 100%;
  }
  .row-cols-lg-2 > * {
    flex: 0 0 auto;
    width: 50%;
  }
  .row-cols-lg-3 > * {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .row-cols-lg-4 > * {
    flex: 0 0 auto;
    width: 25%;
  }
  .row-cols-lg-5 > * {
    flex: 0 0 auto;
    width: 20%;
  }
  .row-cols-lg-6 > * {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-lg-auto {
    flex: 0 0 auto;
    width: auto;
  }
  .col-lg-1 {
    flex: 0 0 auto;
    width: 8.33333333%;
  }
  .col-lg-2 {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-lg-3 {
    flex: 0 0 auto;
    width: 25%;
  }
  .col-lg-4 {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .col-lg-5 {
    flex: 0 0 auto;
    width: 41.66666667%;
  }
  .col-lg-6 {
    flex: 0 0 auto;
    width: 50%;
  }
  .col-lg-7 {
    flex: 0 0 auto;
    width: 58.33333333%;
  }
  .col-lg-8 {
    flex: 0 0 auto;
    width: 66.66666667%;
  }
  .col-lg-9 {
    flex: 0 0 auto;
    width: 75%;
  }
  .col-lg-10 {
    flex: 0 0 auto;
    width: 83.33333333%;
  }
  .col-lg-11 {
    flex: 0 0 auto;
    width: 91.66666667%;
  }
  .col-lg-12 {
    flex: 0 0 auto;
    width: 100%;
  }
  .offset-lg-0 {
    margin-right: 0;
  }
  .offset-lg-1 {
    margin-right: 8.33333333%;
  }
  .offset-lg-2 {
    margin-right: 16.66666667%;
  }
  .offset-lg-3 {
    margin-right: 25%;
  }
  .offset-lg-4 {
    margin-right: 33.33333333%;
  }
  .offset-lg-5 {
    margin-right: 41.66666667%;
  }
  .offset-lg-6 {
    margin-right: 50%;
  }
  .offset-lg-7 {
    margin-right: 58.33333333%;
  }
  .offset-lg-8 {
    margin-right: 66.66666667%;
  }
  .offset-lg-9 {
    margin-right: 75%;
  }
  .offset-lg-10 {
    margin-right: 83.33333333%;
  }
  .offset-lg-11 {
    margin-right: 91.66666667%;
  }
  .g-lg-0,
  .gx-lg-0 {
    --tblr-gutter-x: 0;
  }
  .g-lg-0,
  .gy-lg-0 {
    --tblr-gutter-y: 0;
  }
  .g-lg-1,
  .gx-lg-1 {
    --tblr-gutter-x: 0.25rem;
  }
  .g-lg-1,
  .gy-lg-1 {
    --tblr-gutter-y: 0.25rem;
  }
  .g-lg-2,
  .gx-lg-2 {
    --tblr-gutter-x: 0.5rem;
  }
  .g-lg-2,
  .gy-lg-2 {
    --tblr-gutter-y: 0.5rem;
  }
  .g-lg-3,
  .gx-lg-3 {
    --tblr-gutter-x: 1rem;
  }
  .g-lg-3,
  .gy-lg-3 {
    --tblr-gutter-y: 1rem;
  }
  .g-lg-4,
  .gx-lg-4 {
    --tblr-gutter-x: 1.5rem;
  }
  .g-lg-4,
  .gy-lg-4 {
    --tblr-gutter-y: 1.5rem;
  }
  .g-lg-5,
  .gx-lg-5 {
    --tblr-gutter-x: 2rem;
  }
  .g-lg-5,
  .gy-lg-5 {
    --tblr-gutter-y: 2rem;
  }
  .g-lg-6,
  .gx-lg-6 {
    --tblr-gutter-x: 2.5rem;
  }
  .g-lg-6,
  .gy-lg-6 {
    --tblr-gutter-y: 2.5rem;
  }
}
@media (min-width: 1200px) {
  .col-xl {
    flex: 1 0 0;
  }
  .row-cols-xl-auto > * {
    flex: 0 0 auto;
    width: auto;
  }
  .row-cols-xl-1 > * {
    flex: 0 0 auto;
    width: 100%;
  }
  .row-cols-xl-2 > * {
    flex: 0 0 auto;
    width: 50%;
  }
  .row-cols-xl-3 > * {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .row-cols-xl-4 > * {
    flex: 0 0 auto;
    width: 25%;
  }
  .row-cols-xl-5 > * {
    flex: 0 0 auto;
    width: 20%;
  }
  .row-cols-xl-6 > * {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-xl-auto {
    flex: 0 0 auto;
    width: auto;
  }
  .col-xl-1 {
    flex: 0 0 auto;
    width: 8.33333333%;
  }
  .col-xl-2 {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-xl-3 {
    flex: 0 0 auto;
    width: 25%;
  }
  .col-xl-4 {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .col-xl-5 {
    flex: 0 0 auto;
    width: 41.66666667%;
  }
  .col-xl-6 {
    flex: 0 0 auto;
    width: 50%;
  }
  .col-xl-7 {
    flex: 0 0 auto;
    width: 58.33333333%;
  }
  .col-xl-8 {
    flex: 0 0 auto;
    width: 66.66666667%;
  }
  .col-xl-9 {
    flex: 0 0 auto;
    width: 75%;
  }
  .col-xl-10 {
    flex: 0 0 auto;
    width: 83.33333333%;
  }
  .col-xl-11 {
    flex: 0 0 auto;
    width: 91.66666667%;
  }
  .col-xl-12 {
    flex: 0 0 auto;
    width: 100%;
  }
  .offset-xl-0 {
    margin-right: 0;
  }
  .offset-xl-1 {
    margin-right: 8.33333333%;
  }
  .offset-xl-2 {
    margin-right: 16.66666667%;
  }
  .offset-xl-3 {
    margin-right: 25%;
  }
  .offset-xl-4 {
    margin-right: 33.33333333%;
  }
  .offset-xl-5 {
    margin-right: 41.66666667%;
  }
  .offset-xl-6 {
    margin-right: 50%;
  }
  .offset-xl-7 {
    margin-right: 58.33333333%;
  }
  .offset-xl-8 {
    margin-right: 66.66666667%;
  }
  .offset-xl-9 {
    margin-right: 75%;
  }
  .offset-xl-10 {
    margin-right: 83.33333333%;
  }
  .offset-xl-11 {
    margin-right: 91.66666667%;
  }
  .g-xl-0,
  .gx-xl-0 {
    --tblr-gutter-x: 0;
  }
  .g-xl-0,
  .gy-xl-0 {
    --tblr-gutter-y: 0;
  }
  .g-xl-1,
  .gx-xl-1 {
    --tblr-gutter-x: 0.25rem;
  }
  .g-xl-1,
  .gy-xl-1 {
    --tblr-gutter-y: 0.25rem;
  }
  .g-xl-2,
  .gx-xl-2 {
    --tblr-gutter-x: 0.5rem;
  }
  .g-xl-2,
  .gy-xl-2 {
    --tblr-gutter-y: 0.5rem;
  }
  .g-xl-3,
  .gx-xl-3 {
    --tblr-gutter-x: 1rem;
  }
  .g-xl-3,
  .gy-xl-3 {
    --tblr-gutter-y: 1rem;
  }
  .g-xl-4,
  .gx-xl-4 {
    --tblr-gutter-x: 1.5rem;
  }
  .g-xl-4,
  .gy-xl-4 {
    --tblr-gutter-y: 1.5rem;
  }
  .g-xl-5,
  .gx-xl-5 {
    --tblr-gutter-x: 2rem;
  }
  .g-xl-5,
  .gy-xl-5 {
    --tblr-gutter-y: 2rem;
  }
  .g-xl-6,
  .gx-xl-6 {
    --tblr-gutter-x: 2.5rem;
  }
  .g-xl-6,
  .gy-xl-6 {
    --tblr-gutter-y: 2.5rem;
  }
}
@media (min-width: 1400px) {
  .col-xxl {
    flex: 1 0 0;
  }
  .row-cols-xxl-auto > * {
    flex: 0 0 auto;
    width: auto;
  }
  .row-cols-xxl-1 > * {
    flex: 0 0 auto;
    width: 100%;
  }
  .row-cols-xxl-2 > * {
    flex: 0 0 auto;
    width: 50%;
  }
  .row-cols-xxl-3 > * {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .row-cols-xxl-4 > * {
    flex: 0 0 auto;
    width: 25%;
  }
  .row-cols-xxl-5 > * {
    flex: 0 0 auto;
    width: 20%;
  }
  .row-cols-xxl-6 > * {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-xxl-auto {
    flex: 0 0 auto;
    width: auto;
  }
  .col-xxl-1 {
    flex: 0 0 auto;
    width: 8.33333333%;
  }
  .col-xxl-2 {
    flex: 0 0 auto;
    width: 16.66666667%;
  }
  .col-xxl-3 {
    flex: 0 0 auto;
    width: 25%;
  }
  .col-xxl-4 {
    flex: 0 0 auto;
    width: 33.33333333%;
  }
  .col-xxl-5 {
    flex: 0 0 auto;
    width: 41.66666667%;
  }
  .col-xxl-6 {
    flex: 0 0 auto;
    width: 50%;
  }
  .col-xxl-7 {
    flex: 0 0 auto;
    width: 58.33333333%;
  }
  .col-xxl-8 {
    flex: 0 0 auto;
    width: 66.66666667%;
  }
  .col-xxl-9 {
    flex: 0 0 auto;
    width: 75%;
  }
  .col-xxl-10 {
    flex: 0 0 auto;
    width: 83.33333333%;
  }
  .col-xxl-11 {
    flex: 0 0 auto;
    width: 91.66666667%;
  }
  .col-xxl-12 {
    flex: 0 0 auto;
    width: 100%;
  }
  .offset-xxl-0 {
    margin-right: 0;
  }
  .offset-xxl-1 {
    margin-right: 8.33333333%;
  }
  .offset-xxl-2 {
    margin-right: 16.66666667%;
  }
  .offset-xxl-3 {
    margin-right: 25%;
  }
  .offset-xxl-4 {
    margin-right: 33.33333333%;
  }
  .offset-xxl-5 {
    margin-right: 41.66666667%;
  }
  .offset-xxl-6 {
    margin-right: 50%;
  }
  .offset-xxl-7 {
    margin-right: 58.33333333%;
  }
  .offset-xxl-8 {
    margin-right: 66.66666667%;
  }
  .offset-xxl-9 {
    margin-right: 75%;
  }
  .offset-xxl-10 {
    margin-right: 83.33333333%;
  }
  .offset-xxl-11 {
    margin-right: 91.66666667%;
  }
  .g-xxl-0,
  .gx-xxl-0 {
    --tblr-gutter-x: 0;
  }
  .g-xxl-0,
  .gy-xxl-0 {
    --tblr-gutter-y: 0;
  }
  .g-xxl-1,
  .gx-xxl-1 {
    --tblr-gutter-x: 0.25rem;
  }
  .g-xxl-1,
  .gy-xxl-1 {
    --tblr-gutter-y: 0.25rem;
  }
  .g-xxl-2,
  .gx-xxl-2 {
    --tblr-gutter-x: 0.5rem;
  }
  .g-xxl-2,
  .gy-xxl-2 {
    --tblr-gutter-y: 0.5rem;
  }
  .g-xxl-3,
  .gx-xxl-3 {
    --tblr-gutter-x: 1rem;
  }
  .g-xxl-3,
  .gy-xxl-3 {
    --tblr-gutter-y: 1rem;
  }
  .g-xxl-4,
  .gx-xxl-4 {
    --tblr-gutter-x: 1.5rem;
  }
  .g-xxl-4,
  .gy-xxl-4 {
    --tblr-gutter-y: 1.5rem;
  }
  .g-xxl-5,
  .gx-xxl-5 {
    --tblr-gutter-x: 2rem;
  }
  .g-xxl-5,
  .gy-xxl-5 {
    --tblr-gutter-y: 2rem;
  }
  .g-xxl-6,
  .gx-xxl-6 {
    --tblr-gutter-x: 2.5rem;
  }
  .g-xxl-6,
  .gy-xxl-6 {
    --tblr-gutter-y: 2.5rem;
  }
}
.table, .markdown > table {
  --tblr-table-color-type: initial;
  --tblr-table-bg-type: initial;
  --tblr-table-color-state: initial;
  --tblr-table-bg-state: initial;
  --tblr-table-color: inherit;
  --tblr-table-bg: transparent;
  --tblr-table-border-color: var(--tblr-border-color-translucent);
  --tblr-table-accent-bg: transparent;
  --tblr-table-striped-color: inherit;
  --tblr-table-striped-bg: var(--tblr-bg-surface-tertiary);
  --tblr-table-active-color: inherit;
  --tblr-table-active-bg: var(--tblr-active-bg);
  --tblr-table-hover-color: inherit;
  --tblr-table-hover-bg: rgba(var(--tblr-emphasis-color-rgb), 0.075);
  width: 100%;
  margin-bottom: 1rem;
  vertical-align: top;
  border-color: var(--tblr-table-border-color);
}
.table > :not(caption) > * > *, .markdown > table > :not(caption) > * > * {
  padding: 0.75rem 0.75rem;
  color: var(--tblr-table-color-state, var(--tblr-table-color-type, var(--tblr-table-color)));
  background-color: var(--tblr-table-bg);
  border-bottom-width: var(--tblr-border-width);
  box-shadow: inset 0 0 0 9999px var(--tblr-table-bg-state, var(--tblr-table-bg-type, var(--tblr-table-accent-bg)));
}
.table > tbody, .markdown > table > tbody {
  vertical-align: inherit;
}
.table > thead, .markdown > table > thead {
  vertical-align: bottom;
}

.table-group-divider {
  border-top: calc(var(--tblr-border-width) * 2) solid var(--tblr-border-color-translucent);
}

.caption-top {
  caption-side: top;
}

.table-sm > :not(caption) > * > *, .markdown > table > :not(caption) > * > * {
  padding: 0.25rem 0.25rem;
}

.table-bordered > :not(caption) > *, .markdown > table > :not(caption) > * {
  border-width: var(--tblr-border-width) 0;
}
.table-bordered > :not(caption) > * > *, .markdown > table > :not(caption) > * > * {
  border-width: 0 var(--tblr-border-width);
}

.table-borderless > :not(caption) > * > * {
  border-bottom-width: 0;
}
.table-borderless > :not(:first-child) {
  border-top-width: 0;
}

.table-striped > tbody > tr:nth-of-type(even) > * {
  --tblr-table-color-type: var(--tblr-table-striped-color);
  --tblr-table-bg-type: var(--tblr-table-striped-bg);
}

.table-striped-columns > :not(caption) > tr > :nth-child(even) {
  --tblr-table-color-type: var(--tblr-table-striped-color);
  --tblr-table-bg-type: var(--tblr-table-striped-bg);
}

.table-active {
  --tblr-table-color-state: var(--tblr-table-active-color);
  --tblr-table-bg-state: var(--tblr-table-active-bg);
}

.table-hover > tbody > tr:hover > * {
  --tblr-table-color-state: var(--tblr-table-hover-color);
  --tblr-table-bg-state: var(--tblr-table-hover-bg);
}

.table-primary {
  --tblr-table-color: #1f2937;
  --tblr-table-bg: rgb(205.2, 226.2, 245.8);
  --tblr-table-border-color: rgb(170.36, 189.16, 207.64);
  --tblr-table-striped-bg: rgb(196.49, 216.94, 236.26);
  --tblr-table-striped-color: #1f2937;
  --tblr-table-active-bg: rgb(187.78, 207.68, 226.72);
  --tblr-table-active-color: #f9fafb;
  --tblr-table-hover-bg: rgb(192.135, 212.31, 231.49);
  --tblr-table-hover-color: #1f2937;
  color: var(--tblr-table-color);
  border-color: var(--tblr-table-border-color);
}

.table-secondary {
  --tblr-table-color: #1f2937;
  --tblr-table-bg: rgb(225.4, 226.8, 229.6);
  --tblr-table-border-color: rgb(186.52, 189.64, 194.68);
  --tblr-table-striped-bg: rgb(215.68, 217.51, 220.87);
  --tblr-table-striped-color: #1f2937;
  --tblr-table-active-bg: rgb(205.96, 208.22, 212.14);
  --tblr-table-active-color: #1f2937;
  --tblr-table-hover-bg: rgb(210.82, 212.865, 216.505);
  --tblr-table-hover-color: #1f2937;
  color: var(--tblr-table-color);
  border-color: var(--tblr-table-border-color);
}

.table-success {
  --tblr-table-color: #1f2937;
  --tblr-table-bg: rgb(213.4, 239.8, 217.6);
  --tblr-table-border-color: rgb(176.92, 200.04, 185.08);
  --tblr-table-striped-bg: rgb(204.28, 229.86, 209.47);
  --tblr-table-striped-color: #1f2937;
  --tblr-table-active-bg: rgb(195.16, 219.92, 201.34);
  --tblr-table-active-color: #1f2937;
  --tblr-table-hover-bg: rgb(199.72, 224.89, 205.405);
  --tblr-table-hover-color: #1f2937;
  color: var(--tblr-table-color);
  border-color: var(--tblr-table-border-color);
}

.table-info {
  --tblr-table-color: #1f2937;
  --tblr-table-bg: rgb(217.2, 234.6, 249);
  --tblr-table-border-color: rgb(179.96, 195.88, 210.2);
  --tblr-table-striped-bg: rgb(207.89, 224.92, 239.3);
  --tblr-table-striped-color: #1f2937;
  --tblr-table-active-bg: rgb(198.58, 215.24, 229.6);
  --tblr-table-active-color: #1f2937;
  --tblr-table-hover-bg: rgb(203.235, 220.08, 234.45);
  --tblr-table-hover-color: #1f2937;
  color: var(--tblr-table-color);
  border-color: var(--tblr-table-border-color);
}

.table-warning {
  --tblr-table-color: #1f2937;
  --tblr-table-bg: rgb(253, 235.8, 204);
  --tblr-table-border-color: rgb(208.6, 196.84, 174.2);
  --tblr-table-striped-bg: rgb(241.9, 226.06, 196.55);
  --tblr-table-striped-color: #1f2937;
  --tblr-table-active-bg: rgb(230.8, 216.32, 189.1);
  --tblr-table-active-color: #1f2937;
  --tblr-table-hover-bg: rgb(236.35, 221.19, 192.825);
  --tblr-table-hover-color: #1f2937;
  color: var(--tblr-table-color);
  border-color: var(--tblr-table-border-color);
}

.table-danger {
  --tblr-table-color: #1f2937;
  --tblr-table-bg: rgb(246.8, 215.4, 215.4);
  --tblr-table-border-color: rgb(203.64, 180.52, 183.32);
  --tblr-table-striped-bg: rgb(236.01, 206.68, 207.38);
  --tblr-table-striped-color: #1f2937;
  --tblr-table-active-bg: rgb(225.22, 197.96, 199.36);
  --tblr-table-active-color: #f9fafb;
  --tblr-table-hover-bg: rgb(230.615, 202.32, 203.37);
  --tblr-table-hover-color: #1f2937;
  color: var(--tblr-table-color);
  border-color: var(--tblr-table-border-color);
}

.table-light {
  --tblr-table-color: #1f2937;
  --tblr-table-bg: #f9fafb;
  --tblr-table-border-color: rgb(205.4, 208.2, 211.8);
  --tblr-table-striped-bg: rgb(238.1, 239.55, 241.2);
  --tblr-table-striped-color: #1f2937;
  --tblr-table-active-bg: rgb(227.2, 229.1, 231.4);
  --tblr-table-active-color: #1f2937;
  --tblr-table-hover-bg: rgb(232.65, 234.325, 236.3);
  --tblr-table-hover-color: #1f2937;
  color: var(--tblr-table-color);
  border-color: var(--tblr-table-border-color);
}

.table-dark {
  --tblr-table-color: #f9fafb;
  --tblr-table-bg: #1f2937;
  --tblr-table-border-color: rgb(74.6, 82.8, 94.2);
  --tblr-table-striped-bg: rgb(41.9, 51.45, 64.8);
  --tblr-table-striped-color: #f9fafb;
  --tblr-table-active-bg: rgb(52.8, 61.9, 74.6);
  --tblr-table-active-color: #f9fafb;
  --tblr-table-hover-bg: rgb(47.35, 56.675, 69.7);
  --tblr-table-hover-color: #f9fafb;
  color: var(--tblr-table-color);
  border-color: var(--tblr-table-border-color);
}

.table-responsive {
  overflow-x: auto;
  -webkit-overflow-scrolling: touch;
}

@media (max-width: 575.98px) {
  .table-responsive-sm {
    overflow-x: auto;
    -webkit-overflow-scrolling: touch;
  }
}
@media (max-width: 767.98px) {
  .table-responsive-md {
    overflow-x: auto;
    -webkit-overflow-scrolling: touch;
  }
}
@media (max-width: 991.98px) {
  .table-responsive-lg {
    overflow-x: auto;
    -webkit-overflow-scrolling: touch;
  }
}
@media (max-width: 1199.98px) {
  .table-responsive-xl {
    overflow-x: auto;
    -webkit-overflow-scrolling: touch;
  }
}
@media (max-width: 1399.98px) {
  .table-responsive-xxl {
    overflow-x: auto;
    -webkit-overflow-scrolling: touch;
  }
}
.form-label {
  margin-bottom: 0.5rem;
  font-size: 0.875rem;
  font-weight: var(--tblr-font-weight-medium);
}

.col-form-label {
  padding-top: calc(0.5625rem + var(--tblr-border-width));
  padding-bottom: calc(0.5625rem + var(--tblr-border-width));
  margin-bottom: 0;
  font-size: inherit;
  font-weight: var(--tblr-font-weight-medium);
  line-height: 1.25rem;
}

.col-form-label-lg {
  padding-top: calc(0.6875rem + var(--tblr-border-width));
  padding-bottom: calc(0.6875rem + var(--tblr-border-width));
  font-size: 1rem;
}

.col-form-label-sm {
  padding-top: calc(0.3125rem + var(--tblr-border-width));
  padding-bottom: calc(0.3125rem + var(--tblr-border-width));
  font-size: 0.75rem;
}

.form-text {
  margin-top: 0.25rem;
  font-size: 0.875em;
  color: var(--tblr-secondary-color);
}

.form-control {
  display: block;
  width: 100%;
  padding: 0.5625rem 1rem;
  font-family: var(--tblr-body-font-family);
  font-size: 0.875rem;
  font-weight: 400;
  line-height: 1.25rem;
  color: var(--tblr-body-color);
  appearance: none;
  background-color: var(--tblr-bg-forms);
  background-clip: padding-box;
  border: var(--tblr-border-width) solid var(--tblr-border-color);
  border-radius: var(--tblr-border-radius);
  box-shadow: var(--tblr-shadow-input);
  transition: border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .form-control {
    transition: none;
  }
}
.form-control[type=file] {
  overflow: hidden;
}
.form-control[type=file]:not(:disabled):not([readonly]) {
  cursor: pointer;
}
.form-control:focus {
  color: var(--tblr-body-color);
  background-color: var(--tblr-bg-forms);
  border-color: rgb(130.5, 183, 232);
  outline: 0;
  box-shadow: var(--tblr-shadow-input), 0 0 0 0.25rem rgba(var(--tblr-primary-rgb), 0.25);
}
.form-control::-webkit-date-and-time-value {
  min-width: 85px;
  height: 1.25rem;
  margin: 0;
}
.form-control::-webkit-datetime-edit {
  display: block;
  padding: 0;
}
.form-control::placeholder {
  color: var(--tblr-tertiary);
  opacity: 1;
}
.form-control:disabled {
  background-color: var(--tblr-bg-surface-secondary);
  opacity: 1;
}
.form-control::file-selector-button {
  padding: 0.5625rem 1rem;
  margin: -0.5625rem -1rem;
  margin-inline-end: 1rem;
  color: var(--tblr-body-color);
  background-color: var(--tblr-tertiary-bg);
  pointer-events: none;
  border-color: inherit;
  border-style: solid;
  border-width: 0;
  border-inline-end-width: var(--tblr-border-width);
  border-radius: 0;
  transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .form-control::file-selector-button {
    transition: none;
  }
}
.form-control:hover:not(:disabled):not([readonly])::file-selector-button {
  background-color: var(--tblr-secondary-bg);
}

.form-control-plaintext {
  display: block;
  width: 100%;
  padding: 0.5625rem 0;
  margin-bottom: 0;
  line-height: 1.25rem;
  color: var(--tblr-body-color);
  background-color: transparent;
  border: solid transparent;
  border-width: var(--tblr-border-width) 0;
}
.form-control-plaintext:focus {
  outline: 0;
}
.form-control-plaintext.form-control-sm, .form-control-plaintext.form-control-lg {
  padding-left: 0;
  padding-right: 0;
}

.form-control-sm {
  min-height: calc(1.25rem + 0.625rem + calc(var(--tblr-border-width) * 2));
  padding: 0.3125rem 0.5rem;
  font-size: 0.75rem;
  border-radius: var(--tblr-border-radius-sm);
}
.form-control-sm::file-selector-button {
  padding: 0.3125rem 0.5rem;
  margin: -0.3125rem -0.5rem;
  margin-inline-end: 0.5rem;
}

.form-control-lg {
  min-height: calc(1.25rem + 1.375rem + calc(var(--tblr-border-width) * 2));
  padding: 0.6875rem 1.5rem;
  font-size: 1rem;
  border-radius: var(--tblr-border-radius-lg);
}
.form-control-lg::file-selector-button {
  padding: 0.6875rem 1.5rem;
  margin: -0.6875rem -1.5rem;
  margin-inline-end: 1.5rem;
}

textarea.form-control {
  min-height: calc(1.25rem + 1.125rem + calc(var(--tblr-border-width) * 2));
}
textarea.form-control-sm {
  min-height: calc(1.25rem + 0.625rem + calc(var(--tblr-border-width) * 2));
}
textarea.form-control-lg {
  min-height: calc(1.25rem + 1.375rem + calc(var(--tblr-border-width) * 2));
}

.form-control-color {
  width: 3rem;
  height: calc(1.25rem + 1.125rem + calc(var(--tblr-border-width) * 2));
  padding: 0.5625rem;
}
.form-control-color:not(:disabled):not([readonly]) {
  cursor: pointer;
}
.form-control-color::-moz-color-swatch {
  border: 0 !important;
  border-radius: var(--tblr-border-radius);
}
.form-control-color::-webkit-color-swatch {
  border: 0 !important;
  border-radius: var(--tblr-border-radius);
}
.form-control-color.form-control-sm {
  height: calc(1.25rem + 0.625rem + calc(var(--tblr-border-width) * 2));
}
.form-control-color.form-control-lg {
  height: calc(1.25rem + 1.375rem + calc(var(--tblr-border-width) * 2));
}

.form-select {
  --tblr-form-select-bg-img: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%270 0 16 16%27%3e%3cpath fill=%27none%27 stroke=%27%239ca3af%27 stroke-linecap=%27round%27 stroke-linejoin=%27round%27 stroke-width=%272%27 d=%27m2 5 6 6 6-6%27/%3e%3c/svg%3e");
  display: block;
  width: 100%;
  padding: 0.5625rem 1rem 0.5625rem 3rem;
  font-family: var(--tblr-body-font-family);
  font-size: 0.875rem;
  font-weight: 400;
  line-height: 1.25rem;
  color: var(--tblr-body-color);
  appearance: none;
  background-color: var(--tblr-bg-forms);
  background-image: var(--tblr-form-select-bg-img), var(--tblr-form-select-bg-icon, none);
  background-repeat: no-repeat;
  background-position: left 1rem center;
  background-size: 16px 12px;
  border: var(--tblr-border-width) solid var(--tblr-border-color);
  border-radius: var(--tblr-border-radius);
  box-shadow: var(--tblr-shadow-input);
  transition: border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .form-select {
    transition: none;
  }
}
.form-select:focus {
  border-color: rgb(130.5, 183, 232);
  outline: 0;
  box-shadow: var(--tblr-shadow-input), 0 0 0 0.25rem rgba(var(--tblr-primary-rgb), 0.25);
}
.form-select[multiple], .form-select[size]:not([size="1"]) {
  padding-left: 1rem;
  background-image: none;
}
.form-select:disabled {
  background-color: var(--tblr-bg-surface-secondary);
}
.form-select:-moz-focusring {
  color: transparent;
  text-shadow: 0 0 0 var(--tblr-body-color);
}

.form-select-sm {
  padding-top: 0.3125rem;
  padding-bottom: 0.3125rem;
  padding-right: 0.5rem;
  font-size: 0.75rem;
  border-radius: var(--tblr-border-radius-sm);
}

.form-select-lg {
  padding-top: 0.6875rem;
  padding-bottom: 0.6875rem;
  padding-right: 1.5rem;
  font-size: 1rem;
  border-radius: var(--tblr-border-radius-lg);
}

[data-bs-theme=dark] .form-select, body[data-bs-theme=dark] [data-bs-theme=light] .form-select {
  --tblr-form-select-bg-img: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%270 0 16 16%27%3e%3cpath fill=%27none%27 stroke=%27%23e5e7eb%27 stroke-linecap=%27round%27 stroke-linejoin=%27round%27 stroke-width=%272%27 d=%27m2 5 6 6 6-6%27/%3e%3c/svg%3e");
}

.form-check {
  display: block;
  min-height: 1.25rem;
  padding-right: 2rem;
  margin-bottom: 0.75rem;
}
.form-check .form-check-input {
  float: right;
  margin-right: -2rem;
}

.form-check-reverse {
  padding-left: 2rem;
  padding-right: 0;
  text-align: left;
}
.form-check-reverse .form-check-input {
  float: left;
  margin-left: -2rem;
  margin-right: 0;
}

.form-check-input {
  --tblr-form-check-bg: var(--tblr-bg-forms);
  flex-shrink: 0;
  width: 1.25rem;
  height: 1.25rem;
  margin-top: 0.0892857143rem;
  vertical-align: top;
  appearance: none;
  background-color: var(--tblr-form-check-bg);
  background-image: var(--tblr-form-check-bg-image);
  background-repeat: no-repeat;
  background-position: center;
  background-size: contain;
  border: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent);
  print-color-adjust: exact;
}
.form-check-input[type=checkbox] {
  border-radius: var(--tblr-border-radius);
}
.form-check-input[type=radio] {
  border-radius: 50%;
}
.form-check-input:active {
  filter: brightness(90%);
}
.form-check-input:focus {
  border-color: rgb(130.5, 183, 232);
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(var(--tblr-primary-rgb), 0.25);
}
.form-check-input:checked {
  background-color: var(--tblr-primary);
  border-color: var(--tblr-border-color-translucent);
}
.form-check-input:checked[type=checkbox] {
  --tblr-form-check-bg-image: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%270 0 16 16%27 width=%2716%27 height=%2716%27%3e%3cpath fill=%27none%27 stroke=%27%23ffffff%27 stroke-linecap=%27round%27 stroke-linejoin=%27round%27 stroke-width=%272%27 d=%27M4 8.5l2.5 2.5l5.5 -5.5%27/%3e%3c/svg%3e");
}
.form-check-input:checked[type=radio] {
  --tblr-form-check-bg-image: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%270 0 16 16%27%3e%3ccircle r=%273%27 fill=%27%23ffffff%27 cx=%278%27 cy=%278%27 /%3e%3c/svg%3e");
}
.form-check-input[type=checkbox]:indeterminate {
  background-color: var(--tblr-primary);
  border-color: var(--tblr-primary);
  --tblr-form-check-bg-image: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%270 0 20 20%27%3e%3cpath fill=%27none%27 stroke=%27%23ffffff%27 stroke-linecap=%27round%27 stroke-linejoin=%27round%27 stroke-width=%273%27 d=%27M6 10h8%27/%3e%3c/svg%3e");
}
.form-check-input:disabled {
  pointer-events: none;
  filter: none;
  opacity: 0.5;
}
.form-check-input[disabled] ~ .form-check-label, .form-check-input:disabled ~ .form-check-label {
  cursor: default;
  opacity: 0.7;
}

.form-switch {
  padding-right: 2.5rem;
}
.form-switch .form-check-input {
  --tblr-form-switch-bg: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%27-4 -4 8 8%27%3e%3ccircle r=%273%27 fill=%27%23e5e7eb%27/%3e%3c/svg%3e");
  width: 2rem;
  margin-right: -2.5rem;
  background-image: var(--tblr-form-switch-bg);
  background-position: right center;
  border-radius: 2rem;
  transition: background-position 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .form-switch .form-check-input {
    transition: none;
  }
}
.form-switch .form-check-input:focus {
  --tblr-form-switch-bg: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%27-4 -4 8 8%27%3e%3ccircle r=%273%27 fill=%27rgb%28130.5, 183, 232%29%27/%3e%3c/svg%3e");
}
.form-switch .form-check-input:checked {
  background-position: left center;
  --tblr-form-switch-bg: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%27-4 -4 8 8%27%3e%3ccircle r=%273%27 fill=%27%23ffffff%27/%3e%3c/svg%3e");
}
.form-switch.form-check-reverse {
  padding-left: 2.5rem;
  padding-right: 0;
}
.form-switch.form-check-reverse .form-check-input {
  margin-left: -2.5rem;
  margin-right: 0;
}

.form-check-inline {
  display: inline-block;
  margin-left: 1rem;
}

.btn-check {
  position: absolute;
  clip: rect(0, 0, 0, 0);
  pointer-events: none;
}
.btn-check[disabled] + .btn, .btn-check:disabled + .btn {
  pointer-events: none;
  filter: none;
  opacity: 0.4;
}

[data-bs-theme=dark] .form-switch .form-check-input:not(:checked):not(:focus) {
  --tblr-form-switch-bg: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%27-4 -4 8 8%27%3e%3ccircle r=%273%27 fill=%27rgba%28255, 255, 255, 0.25%29%27/%3e%3c/svg%3e");
}

.form-range {
  width: 100%;
  height: 1.25rem;
  padding: 0;
  appearance: none;
  background-color: transparent;
}
.form-range:focus {
  outline: 0;
}
.form-range:focus::-webkit-slider-thumb {
  box-shadow: 0 0 0 1px #f9fafb, 0 0 0 0.25rem rgba(var(--tblr-primary-rgb), 0.25);
}
.form-range:focus::-moz-range-thumb {
  box-shadow: 0 0 0 1px #f9fafb, 0 0 0 0.25rem rgba(var(--tblr-primary-rgb), 0.25);
}
.form-range::-moz-focus-outer {
  border: 0;
}
.form-range::-webkit-slider-thumb {
  width: 1rem;
  height: 1rem;
  margin-top: -0.375rem;
  appearance: none;
  background-color: var(--tblr-primary);
  border: 2px var(--tblr-border-style) #ffffff;
  border-radius: 1rem;
  box-shadow: 0 0.1rem 0.25rem rgba(0, 0, 0, 0.1);
  transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .form-range::-webkit-slider-thumb {
    transition: none;
  }
}
.form-range::-webkit-slider-thumb:active {
  background-color: rgb(180.3, 211.8, 241.2);
}
.form-range::-webkit-slider-runnable-track {
  width: 100%;
  height: 0.25rem;
  color: transparent;
  cursor: pointer;
  background-color: var(--tblr-border-color);
  border-color: transparent;
  border-radius: 1rem;
  box-shadow: var(--tblr-box-shadow-inset);
}
.form-range::-moz-range-thumb {
  width: 1rem;
  height: 1rem;
  appearance: none;
  background-color: var(--tblr-primary);
  border: 2px var(--tblr-border-style) #ffffff;
  border-radius: 1rem;
  box-shadow: 0 0.1rem 0.25rem rgba(0, 0, 0, 0.1);
  transition: background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .form-range::-moz-range-thumb {
    transition: none;
  }
}
.form-range::-moz-range-thumb:active {
  background-color: rgb(180.3, 211.8, 241.2);
}
.form-range::-moz-range-track {
  width: 100%;
  height: 0.25rem;
  color: transparent;
  cursor: pointer;
  background-color: var(--tblr-border-color);
  border-color: transparent;
  border-radius: 1rem;
  box-shadow: var(--tblr-box-shadow-inset);
}
.form-range:disabled {
  pointer-events: none;
}
.form-range:disabled::-webkit-slider-thumb {
  background-color: var(--tblr-secondary-color);
}
.form-range:disabled::-moz-range-thumb {
  background-color: var(--tblr-secondary-color);
}

.form-floating {
  position: relative;
}
.form-floating > .form-control,
.form-floating > .form-control-plaintext,
.form-floating > .form-select {
  height: calc(3.5rem + calc(var(--tblr-border-width) * 2));
  min-height: calc(3.5rem + calc(var(--tblr-border-width) * 2));
  line-height: 1.25;
}
.form-floating > label {
  position: absolute;
  top: 0;
  right: 0;
  z-index: 2;
  max-width: 100%;
  height: 100%;
  padding: 1rem 1rem;
  overflow: hidden;
  color: rgba(var(--tblr-body-color-rgb), 0.65);
  text-align: start;
  text-overflow: ellipsis;
  white-space: nowrap;
  pointer-events: none;
  border: var(--tblr-border-width) solid transparent;
  transform-origin: 100% 0;
  transition: opacity 0.1s ease-in-out, transform 0.1s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .form-floating > label {
    transition: none;
  }
}
.form-floating > .form-control,
.form-floating > .form-control-plaintext {
  padding: 1rem 1rem;
}
.form-floating > .form-control::placeholder,
.form-floating > .form-control-plaintext::placeholder {
  color: transparent;
}
.form-floating > .form-control:focus, .form-floating > .form-control:not(:placeholder-shown),
.form-floating > .form-control-plaintext:focus,
.form-floating > .form-control-plaintext:not(:placeholder-shown) {
  padding-top: 1.625rem;
  padding-bottom: 0.625rem;
}
.form-floating > .form-control:-webkit-autofill,
.form-floating > .form-control-plaintext:-webkit-autofill {
  padding-top: 1.625rem;
  padding-bottom: 0.625rem;
}
.form-floating > .form-select {
  padding-top: 1.625rem;
  padding-bottom: 0.625rem;
  padding-right: 1rem;
}
.form-floating > .form-control:focus ~ label,
.form-floating > .form-control:not(:placeholder-shown) ~ label,
.form-floating > .form-control-plaintext ~ label,
.form-floating > .form-select ~ label {
  transform: scale(0.85) translateY(-0.5rem) translateX(-0.15rem);
}
.form-floating > .form-control:-webkit-autofill ~ label {
  transform: scale(0.85) translateY(-0.5rem) translateX(-0.15rem);
}
.form-floating > textarea:focus ~ label::after,
.form-floating > textarea:not(:placeholder-shown) ~ label::after {
  position: absolute;
  inset: 1rem 0.5rem;
  z-index: -1;
  height: 1.5em;
  content: "";
  background-color: var(--tblr-bg-forms);
  border-radius: var(--tblr-border-radius);
}
.form-floating > textarea:disabled ~ label::after {
  background-color: var(--tblr-bg-surface-secondary);
}
.form-floating > .form-control-plaintext ~ label {
  border-width: var(--tblr-border-width) 0;
}
.form-floating > :disabled ~ label,
.form-floating > .form-control:disabled ~ label {
  color: #4b5563;
}

.input-group {
  position: relative;
  display: flex;
  flex-wrap: wrap;
  align-items: stretch;
  width: 100%;
}
.input-group > .form-control,
.input-group > .form-select,
.input-group > .form-floating {
  position: relative;
  flex: 1 1 auto;
  width: 1%;
  min-width: 0;
}
.input-group > .form-control:focus,
.input-group > .form-select:focus,
.input-group > .form-floating:focus-within {
  z-index: 5;
}
.input-group .btn {
  position: relative;
  z-index: 2;
}
.input-group .btn:focus {
  z-index: 5;
}

.input-group-text {
  display: flex;
  align-items: center;
  padding: 0.5625rem 1rem;
  font-size: 0.875rem;
  font-weight: 400;
  line-height: 1.25rem;
  color: var(--tblr-gray-500);
  text-align: center;
  white-space: nowrap;
  background-color: var(--tblr-bg-surface-secondary);
  border: var(--tblr-border-width) solid var(--tblr-border-color);
  border-radius: var(--tblr-border-radius);
}

.input-group-lg > .form-control,
.input-group-lg > .form-select,
.input-group-lg > .input-group-text,
.input-group-lg > .btn {
  padding: 0.6875rem 1.5rem;
  font-size: 1rem;
  border-radius: var(--tblr-border-radius-lg);
}

.input-group-sm > .form-control,
.input-group-sm > .form-select,
.input-group-sm > .input-group-text,
.input-group-sm > .btn {
  padding: 0.3125rem 0.5rem;
  font-size: 0.75rem;
  border-radius: var(--tblr-border-radius-sm);
}

.input-group-lg > .form-select,
.input-group-sm > .form-select {
  padding-left: 4rem;
}

.input-group:not(.has-validation) > :not(:last-child):not(.dropdown-toggle):not(.dropdown-menu):not(.form-floating),
.input-group:not(.has-validation) > .dropdown-toggle:nth-last-child(n+3),
.input-group:not(.has-validation) > .form-floating:not(:last-child) > .form-control,
.input-group:not(.has-validation) > .form-floating:not(:last-child) > .form-select {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}
.input-group.has-validation > :nth-last-child(n+3):not(.dropdown-toggle):not(.dropdown-menu):not(.form-floating),
.input-group.has-validation > .dropdown-toggle:nth-last-child(n+4),
.input-group.has-validation > .form-floating:nth-last-child(n+3) > .form-control,
.input-group.has-validation > .form-floating:nth-last-child(n+3) > .form-select {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}
.input-group > :not(:first-child):not(.dropdown-menu):not(.valid-tooltip):not(.valid-feedback):not(.invalid-tooltip):not(.invalid-feedback) {
  margin-right: calc(-1 * var(--tblr-border-width));
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}
.input-group > .form-floating:not(:first-child) > .form-control,
.input-group > .form-floating:not(:first-child) > .form-select {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}

.valid-feedback {
  display: none;
  width: 100%;
  margin-top: 0.25rem;
  font-size: 0.875em;
  color: var(--tblr-form-valid-color);
}

.valid-tooltip {
  position: absolute;
  top: 100%;
  z-index: 5;
  display: none;
  max-width: 100%;
  padding: var(--tblr-spacer-1) var(--tblr-spacer-3);
  margin-top: 0.1rem;
  font-size: 0.765625rem;
  color: #fff;
  background-color: var(--tblr-success);
  border-radius: var(--tblr-border-radius);
}

.was-validated :valid ~ .valid-feedback,
.was-validated :valid ~ .valid-tooltip,
.is-valid ~ .valid-feedback,
.is-valid ~ .valid-tooltip {
  display: block;
}

.was-validated .form-control:valid, .form-control.is-valid {
  border-color: var(--tblr-form-valid-border-color);
  padding-left: 2.375rem;
  background-image: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2724%27 height=%2724%27 viewBox=%270 0 24 24%27 fill=%27none%27 stroke=%27%232fb344%27 stroke-width=%272%27 stroke-linecap=%27round%27 stroke-linejoin=%27round%27%3e%3cpolyline points=%2720 6 9 17 4 12%27%3e%3c/polyline%3e%3c/svg%3e");
  background-repeat: no-repeat;
  background-position: left 1.53125rem center;
  background-size: 1.8125rem 1.8125rem;
}
.was-validated .form-control:valid:focus, .form-control.is-valid:focus {
  border-color: var(--tblr-form-valid-border-color);
  box-shadow: var(--tblr-shadow-input), 0 0 0 0.25rem rgba(var(--tblr-success-rgb), 0.25);
}

.was-validated textarea.form-control:valid, textarea.form-control.is-valid {
  padding-left: 2.375rem;
  background-position: top 1.53125rem left 1.53125rem;
}

.was-validated .form-select:valid, .form-select.is-valid {
  border-color: var(--tblr-form-valid-border-color);
}
.was-validated .form-select:valid:not([multiple]):not([size]), .was-validated .form-select:valid:not([multiple])[size="1"], .form-select.is-valid:not([multiple]):not([size]), .form-select.is-valid:not([multiple])[size="1"] {
  --tblr-form-select-bg-icon: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2724%27 height=%2724%27 viewBox=%270 0 24 24%27 fill=%27none%27 stroke=%27%232fb344%27 stroke-width=%272%27 stroke-linecap=%27round%27 stroke-linejoin=%27round%27%3e%3cpolyline points=%2720 6 9 17 4 12%27%3e%3c/polyline%3e%3c/svg%3e");
  padding-left: 5.5rem;
  background-position: left 1rem center, center left 3rem;
  background-size: 16px 12px, 1.8125rem 1.8125rem;
}
.was-validated .form-select:valid:focus, .form-select.is-valid:focus {
  border-color: var(--tblr-form-valid-border-color);
  box-shadow: var(--tblr-shadow-input), 0 0 0 0.25rem rgba(var(--tblr-success-rgb), 0.25);
}

.was-validated .form-control-color:valid, .form-control-color.is-valid {
  width: 5.375rem;
}

.was-validated .form-check-input:valid, .form-check-input.is-valid {
  border-color: var(--tblr-form-valid-border-color);
}
.was-validated .form-check-input:valid:checked, .form-check-input.is-valid:checked {
  background-color: var(--tblr-form-valid-color);
}
.was-validated .form-check-input:valid:focus, .form-check-input.is-valid:focus {
  box-shadow: 0 0 0 0.25rem rgba(var(--tblr-success-rgb), 0.25);
}
.was-validated .form-check-input:valid ~ .form-check-label, .form-check-input.is-valid ~ .form-check-label {
  color: var(--tblr-form-valid-color);
}

.form-check-inline .form-check-input ~ .valid-feedback {
  margin-right: 0.5em;
}

.was-validated .input-group > .form-control:not(:focus):valid, .input-group > .form-control:not(:focus).is-valid,
.was-validated .input-group > .form-select:not(:focus):valid,
.input-group > .form-select:not(:focus).is-valid,
.was-validated .input-group > .form-floating:not(:focus-within):valid,
.input-group > .form-floating:not(:focus-within).is-valid {
  z-index: 3;
}

.invalid-feedback {
  display: none;
  width: 100%;
  margin-top: 0.25rem;
  font-size: 0.875em;
  color: var(--tblr-form-invalid-color);
}

.invalid-tooltip {
  position: absolute;
  top: 100%;
  z-index: 5;
  display: none;
  max-width: 100%;
  padding: var(--tblr-spacer-1) var(--tblr-spacer-3);
  margin-top: 0.1rem;
  font-size: 0.765625rem;
  color: #fff;
  background-color: var(--tblr-danger);
  border-radius: var(--tblr-border-radius);
}

.was-validated :invalid ~ .invalid-feedback,
.was-validated :invalid ~ .invalid-tooltip,
.is-invalid ~ .invalid-feedback,
.is-invalid ~ .invalid-tooltip {
  display: block;
}

.was-validated .form-control:invalid, .form-control.is-invalid {
  border-color: var(--tblr-form-invalid-border-color);
  padding-left: 2.375rem;
  background-image: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2724%27 height=%2724%27 viewBox=%270 0 24 24%27 fill=%27none%27 stroke=%27%23d63939%27 stroke-width=%272%27 stroke-linecap=%27round%27 stroke-linejoin=%27round%27%3e%3cline x1=%2718%27 y1=%276%27 x2=%276%27 y2=%2718%27%3e%3c/line%3e%3cline x1=%276%27 y1=%276%27 x2=%2718%27 y2=%2718%27%3e%3c/line%3e%3c/svg%3e");
  background-repeat: no-repeat;
  background-position: left 1.53125rem center;
  background-size: 1.8125rem 1.8125rem;
}
.was-validated .form-control:invalid:focus, .form-control.is-invalid:focus {
  border-color: var(--tblr-form-invalid-border-color);
  box-shadow: var(--tblr-shadow-input), 0 0 0 0.25rem rgba(var(--tblr-danger-rgb), 0.25);
}

.was-validated textarea.form-control:invalid, textarea.form-control.is-invalid {
  padding-left: 2.375rem;
  background-position: top 1.53125rem left 1.53125rem;
}

.was-validated .form-select:invalid, .form-select.is-invalid {
  border-color: var(--tblr-form-invalid-border-color);
}
.was-validated .form-select:invalid:not([multiple]):not([size]), .was-validated .form-select:invalid:not([multiple])[size="1"], .form-select.is-invalid:not([multiple]):not([size]), .form-select.is-invalid:not([multiple])[size="1"] {
  --tblr-form-select-bg-icon: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 width=%2724%27 height=%2724%27 viewBox=%270 0 24 24%27 fill=%27none%27 stroke=%27%23d63939%27 stroke-width=%272%27 stroke-linecap=%27round%27 stroke-linejoin=%27round%27%3e%3cline x1=%2718%27 y1=%276%27 x2=%276%27 y2=%2718%27%3e%3c/line%3e%3cline x1=%276%27 y1=%276%27 x2=%2718%27 y2=%2718%27%3e%3c/line%3e%3c/svg%3e");
  padding-left: 5.5rem;
  background-position: left 1rem center, center left 3rem;
  background-size: 16px 12px, 1.8125rem 1.8125rem;
}
.was-validated .form-select:invalid:focus, .form-select.is-invalid:focus {
  border-color: var(--tblr-form-invalid-border-color);
  box-shadow: var(--tblr-shadow-input), 0 0 0 0.25rem rgba(var(--tblr-danger-rgb), 0.25);
}

.was-validated .form-control-color:invalid, .form-control-color.is-invalid {
  width: 5.375rem;
}

.was-validated .form-check-input:invalid, .form-check-input.is-invalid {
  border-color: var(--tblr-form-invalid-border-color);
}
.was-validated .form-check-input:invalid:checked, .form-check-input.is-invalid:checked {
  background-color: var(--tblr-form-invalid-color);
}
.was-validated .form-check-input:invalid:focus, .form-check-input.is-invalid:focus {
  box-shadow: 0 0 0 0.25rem rgba(var(--tblr-danger-rgb), 0.25);
}
.was-validated .form-check-input:invalid ~ .form-check-label, .form-check-input.is-invalid ~ .form-check-label {
  color: var(--tblr-form-invalid-color);
}

.form-check-inline .form-check-input ~ .invalid-feedback {
  margin-right: 0.5em;
}

.was-validated .input-group > .form-control:not(:focus):invalid, .input-group > .form-control:not(:focus).is-invalid,
.was-validated .input-group > .form-select:not(:focus):invalid,
.input-group > .form-select:not(:focus).is-invalid,
.was-validated .input-group > .form-floating:not(:focus-within):invalid,
.input-group > .form-floating:not(:focus-within).is-invalid {
  z-index: 4;
}

.btn {
  --tblr-btn-padding-x: 1rem;
  --tblr-btn-padding-y: 0.5625rem;
  --tblr-btn-font-family: var(--tblr-body-font-family);
  --tblr-btn-font-size: 0.875rem;
  --tblr-btn-font-weight: var(--tblr-font-weight-medium);
  --tblr-btn-line-height: 1.25rem;
  --tblr-btn-color: var(--tblr-body-color);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-width: var(--tblr-border-width);
  --tblr-btn-border-color: transparent;
  --tblr-btn-border-radius: var(--tblr-border-radius);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
  --tblr-btn-disabled-opacity: 0.4;
  --tblr-btn-focus-box-shadow: 0 0 0 0.25rem rgba(var(--tblr-btn-focus-shadow-rgb), .5);
  display: inline-block;
  padding: var(--tblr-btn-padding-y) var(--tblr-btn-padding-x);
  font-family: var(--tblr-btn-font-family);
  font-size: var(--tblr-btn-font-size);
  font-weight: var(--tblr-btn-font-weight);
  line-height: var(--tblr-btn-line-height);
  color: var(--tblr-btn-color);
  text-align: center;
  vertical-align: middle;
  cursor: pointer;
  user-select: none;
  border: var(--tblr-btn-border-width) solid var(--tblr-btn-border-color);
  border-radius: var(--tblr-btn-border-radius);
  background-color: var(--tblr-btn-bg);
  box-shadow: var(--tblr-btn-box-shadow);
  transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .btn {
    transition: none;
  }
}
.btn:hover {
  color: var(--tblr-btn-hover-color);
  background-color: var(--tblr-btn-hover-bg);
  border-color: var(--tblr-btn-hover-border-color);
}
.btn-check + .btn:hover {
  color: var(--tblr-btn-color);
  background-color: var(--tblr-btn-bg);
  border-color: var(--tblr-btn-border-color);
}
.btn:focus-visible {
  color: var(--tblr-btn-hover-color);
  background-color: var(--tblr-btn-hover-bg);
  border-color: var(--tblr-btn-hover-border-color);
  outline: 0;
  box-shadow: var(--tblr-btn-box-shadow), var(--tblr-btn-focus-box-shadow);
}
.btn-check:focus-visible + .btn {
  border-color: var(--tblr-btn-hover-border-color);
  outline: 0;
  box-shadow: var(--tblr-btn-box-shadow), var(--tblr-btn-focus-box-shadow);
}
.btn-check:checked + .btn, :not(.btn-check) + .btn:active, .btn:first-child:active, .btn.active, .btn.show {
  color: var(--tblr-btn-active-color);
  background-color: var(--tblr-btn-active-bg);
  border-color: var(--tblr-btn-active-border-color);
  box-shadow: var(--tblr-btn-active-shadow);
}
.btn-check:checked + .btn:focus-visible, :not(.btn-check) + .btn:active:focus-visible, .btn:first-child:active:focus-visible, .btn.active:focus-visible, .btn.show:focus-visible {
  box-shadow: var(--tblr-btn-active-shadow), var(--tblr-btn-focus-box-shadow);
}
.btn-check:checked:focus-visible + .btn {
  box-shadow: var(--tblr-btn-active-shadow), var(--tblr-btn-focus-box-shadow);
}
.btn:disabled, .btn.disabled, fieldset:disabled .btn {
  color: var(--tblr-btn-disabled-color);
  pointer-events: none;
  background-color: var(--tblr-btn-disabled-bg);
  border-color: var(--tblr-btn-disabled-border-color);
  opacity: var(--tblr-btn-disabled-opacity);
  box-shadow: none;
}

.btn-link {
  --tblr-btn-font-weight: 400;
  --tblr-btn-color: var(--tblr-link-color);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-link-hover-color);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-color: var(--tblr-link-hover-color);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-disabled-color: #4b5563;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-btn-box-shadow: 0 0 0 #000;
  --tblr-btn-focus-shadow-rgb: 42, 132, 215;
  text-decoration: none;
}
.btn-link:hover, .btn-link:focus-visible {
  text-decoration: none;
}
.btn-link:focus-visible {
  color: var(--tblr-btn-color);
}
.btn-link:hover {
  color: var(--tblr-btn-hover-color);
}

.btn-lg, .btn-group-lg > .btn {
  --tblr-btn-padding-y: 0.6875rem;
  --tblr-btn-padding-x: 1.5rem;
  --tblr-btn-font-size: 1rem;
  --tblr-btn-border-radius: var(--tblr-border-radius-lg);
}

.btn-sm, .btn-group-sm > .btn {
  --tblr-btn-padding-y: 0.3125rem;
  --tblr-btn-padding-x: 0.5rem;
  --tblr-btn-font-size: 0.75rem;
  --tblr-btn-border-radius: var(--tblr-border-radius-sm);
}

.fade {
  transition: opacity 0.15s linear;
}
@media (prefers-reduced-motion: reduce) {
  .fade {
    transition: none;
  }
}
.fade:not(.show) {
  opacity: 0;
}

.collapse:not(.show) {
  display: none;
}

.collapsing {
  height: 0;
  overflow: hidden;
  transition: height 0.35s ease;
}
@media (prefers-reduced-motion: reduce) {
  .collapsing {
    transition: none;
  }
}
.collapsing.collapse-horizontal {
  width: 0;
  height: auto;
  transition: width 0.35s ease;
}
@media (prefers-reduced-motion: reduce) {
  .collapsing.collapse-horizontal {
    transition: none;
  }
}

.dropup,
.dropend,
.dropdown,
.dropstart,
.dropup-center,
.dropdown-center {
  position: relative;
}

.dropdown-toggle {
  white-space: nowrap;
}
.dropdown-toggle:after {
  content: "";
  display: inline-block;
  vertical-align: 0.306em;
  width: 0.36em;
  height: 0.36em;
  border-bottom: 1px var(--tblr-border-style);
  border-right: 1px var(--tblr-border-style);
  margin-left: 0.1em;
  margin-right: 0.4em;
  transform: rotate(45deg);
}

.dropdown-menu {
  --tblr-dropdown-zindex: 1000;
  --tblr-dropdown-min-width: 11rem;
  --tblr-dropdown-padding-x: 0;
  --tblr-dropdown-padding-y: 0.25rem;
  --tblr-dropdown-spacer: 1px;
  --tblr-dropdown-font-size: 0.875rem;
  --tblr-dropdown-color: var(--tblr-body-color);
  --tblr-dropdown-bg: var(--tblr-bg-surface);
  --tblr-dropdown-border-color: var(--tblr-border-color-translucent);
  --tblr-dropdown-border-radius: var(--tblr-border-radius);
  --tblr-dropdown-border-width: var(--tblr-border-width);
  --tblr-dropdown-inner-border-radius: calc(var(--tblr-border-radius) - var(--tblr-border-width));
  --tblr-dropdown-divider-bg: var(--tblr-border-color-translucent);
  --tblr-dropdown-divider-margin-y: var(--tblr-spacer-2);
  --tblr-dropdown-box-shadow: var(--tblr-shadow-dropdown);
  --tblr-dropdown-link-color: inherit;
  --tblr-dropdown-link-hover-color: inherit;
  --tblr-dropdown-link-hover-bg: rgba(var(--tblr-secondary-rgb), 0.08);
  --tblr-dropdown-link-active-color: var(--tblr-primary);
  --tblr-dropdown-link-active-bg: var(--tblr-active-bg);
  --tblr-dropdown-link-disabled-color: var(--tblr-tertiary-color);
  --tblr-dropdown-item-padding-x: 0.75rem;
  --tblr-dropdown-item-padding-y: 0.5rem;
  --tblr-dropdown-header-color: #4b5563;
  --tblr-dropdown-header-padding-x: 0.75rem;
  --tblr-dropdown-header-padding-y: 0.25rem;
  position: absolute;
  z-index: var(--tblr-dropdown-zindex);
  display: none;
  min-width: var(--tblr-dropdown-min-width);
  padding: var(--tblr-dropdown-padding-y) var(--tblr-dropdown-padding-x);
  margin: 0;
  font-size: var(--tblr-dropdown-font-size);
  color: var(--tblr-dropdown-color);
  text-align: right;
  list-style: none;
  background-color: var(--tblr-dropdown-bg);
  background-clip: padding-box;
  border: var(--tblr-dropdown-border-width) solid var(--tblr-dropdown-border-color);
  border-radius: var(--tblr-dropdown-border-radius);
  box-shadow: var(--tblr-dropdown-box-shadow);
}
.dropdown-menu[data-bs-popper] {
  top: 100%;
  right: 0;
  margin-top: var(--tblr-dropdown-spacer);
}

.dropdown-menu-start {
  --bs-position: start;
}
.dropdown-menu-start[data-bs-popper] {
  left: auto;
  right: 0;
}

.dropdown-menu-end {
  --bs-position: end;
}
.dropdown-menu-end[data-bs-popper] {
  left: 0;
  right: auto;
}

@media (min-width: 576px) {
  .dropdown-menu-sm-start {
    --bs-position: start;
  }
  .dropdown-menu-sm-start[data-bs-popper] {
    left: auto;
    right: 0;
  }
  .dropdown-menu-sm-end {
    --bs-position: end;
  }
  .dropdown-menu-sm-end[data-bs-popper] {
    left: 0;
    right: auto;
  }
}
@media (min-width: 768px) {
  .dropdown-menu-md-start {
    --bs-position: start;
  }
  .dropdown-menu-md-start[data-bs-popper] {
    left: auto;
    right: 0;
  }
  .dropdown-menu-md-end {
    --bs-position: end;
  }
  .dropdown-menu-md-end[data-bs-popper] {
    left: 0;
    right: auto;
  }
}
@media (min-width: 992px) {
  .dropdown-menu-lg-start {
    --bs-position: start;
  }
  .dropdown-menu-lg-start[data-bs-popper] {
    left: auto;
    right: 0;
  }
  .dropdown-menu-lg-end {
    --bs-position: end;
  }
  .dropdown-menu-lg-end[data-bs-popper] {
    left: 0;
    right: auto;
  }
}
@media (min-width: 1200px) {
  .dropdown-menu-xl-start {
    --bs-position: start;
  }
  .dropdown-menu-xl-start[data-bs-popper] {
    left: auto;
    right: 0;
  }
  .dropdown-menu-xl-end {
    --bs-position: end;
  }
  .dropdown-menu-xl-end[data-bs-popper] {
    left: 0;
    right: auto;
  }
}
@media (min-width: 1400px) {
  .dropdown-menu-xxl-start {
    --bs-position: start;
  }
  .dropdown-menu-xxl-start[data-bs-popper] {
    left: auto;
    right: 0;
  }
  .dropdown-menu-xxl-end {
    --bs-position: end;
  }
  .dropdown-menu-xxl-end[data-bs-popper] {
    left: 0;
    right: auto;
  }
}
.dropup .dropdown-menu[data-bs-popper] {
  top: auto;
  bottom: 100%;
  margin-top: 0;
  margin-bottom: var(--tblr-dropdown-spacer);
}
.dropup .dropdown-toggle:after {
  content: "";
  display: inline-block;
  vertical-align: 0.306em;
  width: 0.36em;
  height: 0.36em;
  border-bottom: 1px var(--tblr-border-style);
  border-right: 1px var(--tblr-border-style);
  margin-left: 0.1em;
  margin-right: 0.4em;
  transform: rotate(-135deg);
}

.dropend .dropdown-menu[data-bs-popper] {
  top: 0;
  left: auto;
  right: 100%;
  margin-top: 0;
  margin-right: var(--tblr-dropdown-spacer);
}
.dropend .dropdown-toggle:after {
  content: "";
  display: inline-block;
  vertical-align: 0.306em;
  width: 0.36em;
  height: 0.36em;
  border-bottom: 1px var(--tblr-border-style);
  border-right: 1px var(--tblr-border-style);
  margin-left: 0.1em;
  margin-right: 0.4em;
  transform: rotate(135deg);
}
.dropend .dropdown-toggle::after {
  vertical-align: 0;
}

.dropstart .dropdown-menu[data-bs-popper] {
  top: 0;
  left: 100%;
  right: auto;
  margin-top: 0;
  margin-left: var(--tblr-dropdown-spacer);
}
.dropstart .dropdown-toggle:after {
  content: "";
  display: inline-block;
  vertical-align: 0.306em;
  width: 0.36em;
  height: 0.36em;
  border-bottom: 1px var(--tblr-border-style);
  border-right: 1px var(--tblr-border-style);
  margin-left: 0.1em;
  margin-right: 0.4em;
  transform: rotate(-45deg);
}
.dropstart .dropdown-toggle::before {
  vertical-align: 0;
}

.dropdown-divider {
  height: 0;
  margin: var(--tblr-dropdown-divider-margin-y) 0;
  overflow: hidden;
  border-top: 1px solid var(--tblr-dropdown-divider-bg);
  opacity: 1;
}

.dropdown-item {
  display: block;
  width: 100%;
  padding: var(--tblr-dropdown-item-padding-y) var(--tblr-dropdown-item-padding-x);
  clear: both;
  font-weight: 400;
  color: var(--tblr-dropdown-link-color);
  text-align: inherit;
  white-space: nowrap;
  background-color: transparent;
  border: 0;
  border-radius: var(--tblr-dropdown-item-border-radius, 0);
}
.dropdown-item:hover, .dropdown-item:focus {
  color: var(--tblr-dropdown-link-hover-color);
  background-color: var(--tblr-dropdown-link-hover-bg);
}
.dropdown-item.active, .dropdown-item:active {
  color: var(--tblr-dropdown-link-active-color);
  text-decoration: none;
  background-color: var(--tblr-dropdown-link-active-bg);
}
.dropdown-item.disabled, .dropdown-item:disabled {
  color: var(--tblr-dropdown-link-disabled-color);
  pointer-events: none;
  background-color: transparent;
}

.dropdown-menu.show {
  display: block;
}

.dropdown-header {
  display: block;
  padding: var(--tblr-dropdown-header-padding-y) var(--tblr-dropdown-header-padding-x);
  margin-bottom: 0;
  font-size: 0.765625rem;
  color: var(--tblr-dropdown-header-color);
  white-space: nowrap;
}

.dropdown-item-text {
  display: block;
  padding: var(--tblr-dropdown-item-padding-y) var(--tblr-dropdown-item-padding-x);
  color: var(--tblr-dropdown-link-color);
}

.dropdown-menu-dark {
  --tblr-dropdown-color: #d1d5db;
  --tblr-dropdown-bg: #1f2937;
  --tblr-dropdown-border-color: var(--tblr-border-color-translucent);
  --tblr-dropdown-box-shadow: ;
  --tblr-dropdown-link-color: #d1d5db;
  --tblr-dropdown-link-hover-color: #ffffff;
  --tblr-dropdown-divider-bg: var(--tblr-border-color-translucent);
  --tblr-dropdown-link-hover-bg: rgba(255, 255, 255, 0.15);
  --tblr-dropdown-link-active-color: var(--tblr-primary);
  --tblr-dropdown-link-active-bg: var(--tblr-active-bg);
  --tblr-dropdown-link-disabled-color: #6b7280;
  --tblr-dropdown-header-color: #6b7280;
}

.btn-group,
.btn-group-vertical {
  position: relative;
  display: inline-flex;
  vertical-align: middle;
}
.btn-group > .btn,
.btn-group-vertical > .btn {
  position: relative;
  flex: 1 1 auto;
}
.btn-group > .btn-check:checked + .btn,
.btn-group > .btn-check:focus + .btn,
.btn-group > .btn:hover,
.btn-group > .btn:focus,
.btn-group > .btn:active,
.btn-group > .btn.active,
.btn-group-vertical > .btn-check:checked + .btn,
.btn-group-vertical > .btn-check:focus + .btn,
.btn-group-vertical > .btn:hover,
.btn-group-vertical > .btn:focus,
.btn-group-vertical > .btn:active,
.btn-group-vertical > .btn.active {
  z-index: 1;
}

.btn-toolbar {
  display: flex;
  flex-wrap: wrap;
  justify-content: flex-start;
}
.btn-toolbar .input-group {
  width: auto;
}

.btn-group {
  border-radius: var(--tblr-border-radius);
}
.btn-group > :not(.btn-check:first-child) + .btn,
.btn-group > .btn-group:not(:first-child) {
  margin-right: calc(-1 * var(--tblr-border-width));
}
.btn-group > .btn:not(:last-child):not(.dropdown-toggle),
.btn-group > .btn.dropdown-toggle-split:first-child,
.btn-group > .btn-group:not(:last-child) > .btn {
  border-top-left-radius: 0;
  border-bottom-left-radius: 0;
}
.btn-group > .btn:nth-child(n+3),
.btn-group > :not(.btn-check) + .btn,
.btn-group > .btn-group:not(:first-child) > .btn {
  border-top-right-radius: 0;
  border-bottom-right-radius: 0;
}

.dropdown-toggle-split {
  padding-left: 0.75rem;
  padding-right: 0.75rem;
}
.dropdown-toggle-split::after, .dropup .dropdown-toggle-split::after, .dropend .dropdown-toggle-split::after {
  margin-right: 0;
}
.dropstart .dropdown-toggle-split::before {
  margin-left: 0;
}

.btn-sm + .dropdown-toggle-split, .btn-group-sm > .btn + .dropdown-toggle-split {
  padding-left: 0.375rem;
  padding-right: 0.375rem;
}

.btn-lg + .dropdown-toggle-split, .btn-group-lg > .btn + .dropdown-toggle-split {
  padding-left: 1.125rem;
  padding-right: 1.125rem;
}

.btn-group.show .dropdown-toggle {
  box-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
}
.btn-group.show .dropdown-toggle.btn-link {
  box-shadow: none;
}

.btn-group-vertical {
  flex-direction: column;
  align-items: flex-start;
  justify-content: center;
}
.btn-group-vertical > .btn,
.btn-group-vertical > .btn-group {
  width: 100%;
}
.btn-group-vertical > .btn:not(:first-child),
.btn-group-vertical > .btn-group:not(:first-child) {
  margin-top: calc(-1 * var(--tblr-border-width));
}
.btn-group-vertical > .btn:not(:last-child):not(.dropdown-toggle),
.btn-group-vertical > .btn-group:not(:last-child) > .btn {
  border-bottom-left-radius: 0;
  border-bottom-right-radius: 0;
}
.btn-group-vertical > .btn:nth-child(n+3),
.btn-group-vertical > :not(.btn-check) + .btn,
.btn-group-vertical > .btn-group:not(:first-child) > .btn {
  border-top-right-radius: 0;
  border-top-left-radius: 0;
}

.nav {
  --tblr-nav-link-padding-x: 0.75rem;
  --tblr-nav-link-padding-y: 0.5rem;
  --tblr-nav-link-font-weight: ;
  --tblr-nav-link-color: var(--tblr-gray-500);
  --tblr-nav-link-hover-color: var(--tblr-link-hover-color);
  --tblr-nav-link-disabled-color: var(--tblr-disabled-color);
  display: flex;
  flex-wrap: wrap;
  padding-right: 0;
  margin-bottom: 0;
  list-style: none;
}

.nav-link {
  display: block;
  padding: var(--tblr-nav-link-padding-y) var(--tblr-nav-link-padding-x);
  font-size: var(--tblr-nav-link-font-size);
  font-weight: var(--tblr-nav-link-font-weight);
  color: var(--tblr-nav-link-color);
  background: none;
  border: 0;
  transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .nav-link {
    transition: none;
  }
}
.nav-link:hover, .nav-link:focus {
  color: var(--tblr-nav-link-hover-color);
}
.nav-link:focus-visible {
  outline: 0;
  box-shadow: 0 0 0 0.25rem rgba(var(--tblr-primary-rgb), 0.25);
}
.nav-link.disabled, .nav-link:disabled {
  color: var(--tblr-nav-link-disabled-color);
  pointer-events: none;
  cursor: default;
}

.nav-tabs {
  --tblr-nav-tabs-border-width: var(--tblr-border-width);
  --tblr-nav-tabs-border-color: var(--tblr-border-color);
  --tblr-nav-tabs-border-radius: var(--tblr-border-radius);
  --tblr-nav-tabs-link-hover-border-color: var(--tblr-border-color) var(--tblr-border-color) var(--tblr-border-color);
  --tblr-nav-tabs-link-active-color: var(--tblr-body-color);
  --tblr-nav-tabs-link-active-bg: var(--tblr-body-bg);
  --tblr-nav-tabs-link-active-border-color: var(--tblr-border-color) var(--tblr-border-color) var(--tblr-border-color);
  border-bottom: var(--tblr-nav-tabs-border-width) solid var(--tblr-nav-tabs-border-color);
}
.nav-tabs .nav-link {
  margin-bottom: calc(-1 * var(--tblr-nav-tabs-border-width));
  border: var(--tblr-nav-tabs-border-width) solid transparent;
  border-top-right-radius: var(--tblr-nav-tabs-border-radius);
  border-top-left-radius: var(--tblr-nav-tabs-border-radius);
}
.nav-tabs .nav-link:hover, .nav-tabs .nav-link:focus {
  isolation: isolate;
  border-color: var(--tblr-nav-tabs-link-hover-border-color);
}
.nav-tabs .nav-link.active,
.nav-tabs .nav-item.show .nav-link {
  color: var(--tblr-nav-tabs-link-active-color);
  background-color: var(--tblr-nav-tabs-link-active-bg);
  border-color: var(--tblr-nav-tabs-link-active-border-color);
}
.nav-tabs .dropdown-menu {
  margin-top: calc(-1 * var(--tblr-nav-tabs-border-width));
  border-top-right-radius: 0;
  border-top-left-radius: 0;
}

.nav-pills {
  --tblr-nav-pills-border-radius: var(--tblr-border-radius);
  --tblr-nav-pills-link-active-color: var(--tblr-primary);
  --tblr-nav-pills-link-active-bg: var(--tblr-active-bg);
}
.nav-pills .nav-link {
  border-radius: var(--tblr-nav-pills-border-radius);
}
.nav-pills .nav-link.active,
.nav-pills .show > .nav-link {
  color: var(--tblr-nav-pills-link-active-color);
  background-color: var(--tblr-nav-pills-link-active-bg);
}

.nav-underline {
  --tblr-nav-underline-gap: 1rem;
  --tblr-nav-underline-border-width: 0.125rem;
  --tblr-nav-underline-link-active-color: var(--tblr-emphasis-color);
  gap: var(--tblr-nav-underline-gap);
}
.nav-underline .nav-link {
  padding-left: 0;
  padding-right: 0;
  border-bottom: var(--tblr-nav-underline-border-width) solid transparent;
}
.nav-underline .nav-link:hover, .nav-underline .nav-link:focus {
  border-bottom-color: currentcolor;
}
.nav-underline .nav-link.active,
.nav-underline .show > .nav-link {
  font-weight: 600;
  color: var(--tblr-nav-underline-link-active-color);
  border-bottom-color: currentcolor;
}

.nav-fill > .nav-link,
.nav-fill .nav-item {
  flex: 1 1 auto;
  text-align: center;
}

.nav-justified > .nav-link,
.nav-justified .nav-item {
  flex-grow: 1;
  flex-basis: 0;
  text-align: center;
}

.nav-fill .nav-item .nav-link,
.nav-justified .nav-item .nav-link {
  width: 100%;
}

.tab-content > .tab-pane {
  display: none;
}
.tab-content > .active {
  display: block;
}

.navbar {
  --tblr-navbar-padding-x: 0;
  --tblr-navbar-padding-y: 0.25rem;
  --tblr-navbar-color: var(--tblr-secondary);
  --tblr-navbar-hover-color: var(--tblr-body-color);
  --tblr-navbar-disabled-color: var(--tblr-disabled-color);
  --tblr-navbar-active-color: var(--tblr-body-color);
  --tblr-navbar-brand-padding-y: 0.5rem;
  --tblr-navbar-brand-margin-end: 1rem;
  --tblr-navbar-brand-font-size: 1.25rem;
  --tblr-navbar-brand-color: var(--tblr-body-color);
  --tblr-navbar-brand-hover-color: var(--tblr-body-color);
  --tblr-navbar-nav-link-padding-x: 0.75rem;
  --tblr-navbar-toggler-padding-y: 0;
  --tblr-navbar-toggler-padding-x: 0;
  --tblr-navbar-toggler-font-size: 1rem;
  --tblr-navbar-toggler-icon-bg: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%270 0 30 30%27%3e%3cpath stroke=%27rgba%2831, 41, 55, 0.75%29%27 stroke-linecap=%27round%27 stroke-miterlimit=%2710%27 stroke-width=%272%27 d=%27M4 7h22M4 15h22M4 23h22%27/%3e%3c/svg%3e");
  --tblr-navbar-toggler-border-color: rgba(var(--tblr-emphasis-color-rgb), 0.15);
  --tblr-navbar-toggler-border-radius: var(--tblr-border-radius);
  --tblr-navbar-toggler-focus-width: 0;
  --tblr-navbar-toggler-transition: box-shadow 0.15s ease-in-out;
  position: relative;
  display: flex;
  flex-wrap: wrap;
  align-items: center;
  justify-content: space-between;
  padding: var(--tblr-navbar-padding-y) var(--tblr-navbar-padding-x);
}
.navbar > .container,
.navbar > .container-fluid,
.navbar > .container-sm,
.navbar > .container-md,
.navbar > .container-lg,
.navbar > .container-xl,
.navbar > .container-xxl {
  display: flex;
  flex-wrap: inherit;
  align-items: center;
  justify-content: space-between;
}
.navbar-brand {
  padding-top: var(--tblr-navbar-brand-padding-y);
  padding-bottom: var(--tblr-navbar-brand-padding-y);
  margin-left: var(--tblr-navbar-brand-margin-end);
  font-size: var(--tblr-navbar-brand-font-size);
  color: var(--tblr-navbar-brand-color);
  white-space: nowrap;
}
.navbar-brand:hover, .navbar-brand:focus {
  color: var(--tblr-navbar-brand-hover-color);
}

.navbar-nav {
  --tblr-nav-link-padding-x: 0;
  --tblr-nav-link-padding-y: 0.5rem;
  --tblr-nav-link-font-weight: ;
  --tblr-nav-link-color: var(--tblr-navbar-color);
  --tblr-nav-link-hover-color: var(--tblr-navbar-hover-color);
  --tblr-nav-link-disabled-color: var(--tblr-navbar-disabled-color);
  display: flex;
  flex-direction: column;
  padding-right: 0;
  margin-bottom: 0;
  list-style: none;
}
.navbar-nav .nav-link.active, .navbar-nav .nav-link.show {
  color: var(--tblr-navbar-active-color);
}
.navbar-nav .dropdown-menu {
  position: static;
}

.navbar-text {
  padding-top: 0.5rem;
  padding-bottom: 0.5rem;
  color: var(--tblr-navbar-color);
}
.navbar-text a,
.navbar-text a:hover,
.navbar-text a:focus {
  color: var(--tblr-navbar-active-color);
}

.navbar-collapse {
  flex-grow: 1;
  flex-basis: 100%;
  align-items: center;
}

.navbar-toggler {
  padding: var(--tblr-navbar-toggler-padding-y) var(--tblr-navbar-toggler-padding-x);
  font-size: var(--tblr-navbar-toggler-font-size);
  line-height: 1;
  color: var(--tblr-navbar-color);
  background-color: transparent;
  border: var(--tblr-border-width) solid var(--tblr-navbar-toggler-border-color);
  border-radius: var(--tblr-navbar-toggler-border-radius);
  transition: var(--tblr-navbar-toggler-transition);
}
@media (prefers-reduced-motion: reduce) {
  .navbar-toggler {
    transition: none;
  }
}
.navbar-toggler:hover {
  text-decoration: none;
}
.navbar-toggler:focus {
  text-decoration: none;
  outline: 0;
  box-shadow: 0 0 0 var(--tblr-navbar-toggler-focus-width);
}

.navbar-toggler-icon {
  display: inline-block;
  width: 1.5em;
  height: 1.5em;
  vertical-align: middle;
  background-image: var(--tblr-navbar-toggler-icon-bg);
  background-repeat: no-repeat;
  background-position: center;
  background-size: 100%;
}

.navbar-nav-scroll {
  max-height: var(--tblr-scroll-height, 75vh);
  overflow-y: auto;
}

@media (min-width: 576px) {
  .navbar-expand-sm {
    flex-wrap: nowrap;
    justify-content: flex-start;
  }
  .navbar-expand-sm .navbar-nav {
    flex-direction: row;
  }
  .navbar-expand-sm .navbar-nav .dropdown-menu {
    position: absolute;
  }
  .navbar-expand-sm .navbar-nav .nav-link {
    padding-left: var(--tblr-navbar-nav-link-padding-x);
    padding-right: var(--tblr-navbar-nav-link-padding-x);
  }
  .navbar-expand-sm .navbar-nav-scroll {
    overflow: visible;
  }
  .navbar-expand-sm .navbar-collapse {
    display: flex !important;
    flex-basis: auto;
  }
  .navbar-expand-sm .navbar-toggler {
    display: none;
  }
  .navbar-expand-sm .offcanvas {
    position: static;
    z-index: auto;
    flex-grow: 1;
    width: auto !important;
    height: auto !important;
    visibility: visible !important;
    background-color: transparent !important;
    border: 0 !important;
    transform: none !important;
    box-shadow: none;
    transition: none;
  }
  .navbar-expand-sm .offcanvas .offcanvas-header {
    display: none;
  }
  .navbar-expand-sm .offcanvas .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
  }
}
@media (min-width: 768px) {
  .navbar-expand-md {
    flex-wrap: nowrap;
    justify-content: flex-start;
  }
  .navbar-expand-md .navbar-nav {
    flex-direction: row;
  }
  .navbar-expand-md .navbar-nav .dropdown-menu {
    position: absolute;
  }
  .navbar-expand-md .navbar-nav .nav-link {
    padding-left: var(--tblr-navbar-nav-link-padding-x);
    padding-right: var(--tblr-navbar-nav-link-padding-x);
  }
  .navbar-expand-md .navbar-nav-scroll {
    overflow: visible;
  }
  .navbar-expand-md .navbar-collapse {
    display: flex !important;
    flex-basis: auto;
  }
  .navbar-expand-md .navbar-toggler {
    display: none;
  }
  .navbar-expand-md .offcanvas {
    position: static;
    z-index: auto;
    flex-grow: 1;
    width: auto !important;
    height: auto !important;
    visibility: visible !important;
    background-color: transparent !important;
    border: 0 !important;
    transform: none !important;
    box-shadow: none;
    transition: none;
  }
  .navbar-expand-md .offcanvas .offcanvas-header {
    display: none;
  }
  .navbar-expand-md .offcanvas .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
  }
}
@media (min-width: 992px) {
  .navbar-expand-lg {
    flex-wrap: nowrap;
    justify-content: flex-start;
  }
  .navbar-expand-lg .navbar-nav {
    flex-direction: row;
  }
  .navbar-expand-lg .navbar-nav .dropdown-menu {
    position: absolute;
  }
  .navbar-expand-lg .navbar-nav .nav-link {
    padding-left: var(--tblr-navbar-nav-link-padding-x);
    padding-right: var(--tblr-navbar-nav-link-padding-x);
  }
  .navbar-expand-lg .navbar-nav-scroll {
    overflow: visible;
  }
  .navbar-expand-lg .navbar-collapse {
    display: flex !important;
    flex-basis: auto;
  }
  .navbar-expand-lg .navbar-toggler {
    display: none;
  }
  .navbar-expand-lg .offcanvas {
    position: static;
    z-index: auto;
    flex-grow: 1;
    width: auto !important;
    height: auto !important;
    visibility: visible !important;
    background-color: transparent !important;
    border: 0 !important;
    transform: none !important;
    box-shadow: none;
    transition: none;
  }
  .navbar-expand-lg .offcanvas .offcanvas-header {
    display: none;
  }
  .navbar-expand-lg .offcanvas .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
  }
}
@media (min-width: 1200px) {
  .navbar-expand-xl {
    flex-wrap: nowrap;
    justify-content: flex-start;
  }
  .navbar-expand-xl .navbar-nav {
    flex-direction: row;
  }
  .navbar-expand-xl .navbar-nav .dropdown-menu {
    position: absolute;
  }
  .navbar-expand-xl .navbar-nav .nav-link {
    padding-left: var(--tblr-navbar-nav-link-padding-x);
    padding-right: var(--tblr-navbar-nav-link-padding-x);
  }
  .navbar-expand-xl .navbar-nav-scroll {
    overflow: visible;
  }
  .navbar-expand-xl .navbar-collapse {
    display: flex !important;
    flex-basis: auto;
  }
  .navbar-expand-xl .navbar-toggler {
    display: none;
  }
  .navbar-expand-xl .offcanvas {
    position: static;
    z-index: auto;
    flex-grow: 1;
    width: auto !important;
    height: auto !important;
    visibility: visible !important;
    background-color: transparent !important;
    border: 0 !important;
    transform: none !important;
    box-shadow: none;
    transition: none;
  }
  .navbar-expand-xl .offcanvas .offcanvas-header {
    display: none;
  }
  .navbar-expand-xl .offcanvas .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
  }
}
@media (min-width: 1400px) {
  .navbar-expand-xxl {
    flex-wrap: nowrap;
    justify-content: flex-start;
  }
  .navbar-expand-xxl .navbar-nav {
    flex-direction: row;
  }
  .navbar-expand-xxl .navbar-nav .dropdown-menu {
    position: absolute;
  }
  .navbar-expand-xxl .navbar-nav .nav-link {
    padding-left: var(--tblr-navbar-nav-link-padding-x);
    padding-right: var(--tblr-navbar-nav-link-padding-x);
  }
  .navbar-expand-xxl .navbar-nav-scroll {
    overflow: visible;
  }
  .navbar-expand-xxl .navbar-collapse {
    display: flex !important;
    flex-basis: auto;
  }
  .navbar-expand-xxl .navbar-toggler {
    display: none;
  }
  .navbar-expand-xxl .offcanvas {
    position: static;
    z-index: auto;
    flex-grow: 1;
    width: auto !important;
    height: auto !important;
    visibility: visible !important;
    background-color: transparent !important;
    border: 0 !important;
    transform: none !important;
    box-shadow: none;
    transition: none;
  }
  .navbar-expand-xxl .offcanvas .offcanvas-header {
    display: none;
  }
  .navbar-expand-xxl .offcanvas .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
  }
}
.navbar-expand {
  flex-wrap: nowrap;
  justify-content: flex-start;
}
.navbar-expand .navbar-nav {
  flex-direction: row;
}
.navbar-expand .navbar-nav .dropdown-menu {
  position: absolute;
}
.navbar-expand .navbar-nav .nav-link {
  padding-left: var(--tblr-navbar-nav-link-padding-x);
  padding-right: var(--tblr-navbar-nav-link-padding-x);
}
.navbar-expand .navbar-nav-scroll {
  overflow: visible;
}
.navbar-expand .navbar-collapse {
  display: flex !important;
  flex-basis: auto;
}
.navbar-expand .navbar-toggler {
  display: none;
}
.navbar-expand .offcanvas {
  position: static;
  z-index: auto;
  flex-grow: 1;
  width: auto !important;
  height: auto !important;
  visibility: visible !important;
  background-color: transparent !important;
  border: 0 !important;
  transform: none !important;
  box-shadow: none;
  transition: none;
}
.navbar-expand .offcanvas .offcanvas-header {
  display: none;
}
.navbar-expand .offcanvas .offcanvas-body {
  display: flex;
  flex-grow: 0;
  padding: 0;
  overflow-y: visible;
}

.navbar-dark,
.navbar[data-bs-theme=dark],
body[data-bs-theme=dark] .navbar[data-bs-theme=light] {
  --tblr-navbar-color: rgba(255, 255, 255, 0.7);
  --tblr-navbar-hover-color: rgba(255, 255, 255, 0.75);
  --tblr-navbar-disabled-color: var(--tblr-disabled-color);
  --tblr-navbar-active-color: #ffffff;
  --tblr-navbar-brand-color: #ffffff;
  --tblr-navbar-brand-hover-color: #ffffff;
  --tblr-navbar-toggler-border-color: rgba(255, 255, 255, 0.1);
  --tblr-navbar-toggler-icon-bg: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%270 0 30 30%27%3e%3cpath stroke=%27rgba%28255, 255, 255, 0.7%29%27 stroke-linecap=%27round%27 stroke-miterlimit=%2710%27 stroke-width=%272%27 d=%27M4 7h22M4 15h22M4 23h22%27/%3e%3c/svg%3e");
}

[data-bs-theme=dark] .navbar-toggler-icon, body[data-bs-theme=dark] [data-bs-theme=light] .navbar-toggler-icon {
  --tblr-navbar-toggler-icon-bg: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%270 0 30 30%27%3e%3cpath stroke=%27rgba%28255, 255, 255, 0.7%29%27 stroke-linecap=%27round%27 stroke-miterlimit=%2710%27 stroke-width=%272%27 d=%27M4 7h22M4 15h22M4 23h22%27/%3e%3c/svg%3e");
}

.card {
  --tblr-card-spacer-y: 1rem;
  --tblr-card-spacer-x: 1.25rem;
  --tblr-card-title-spacer-y: 1.25rem;
  --tblr-card-title-color: ;
  --tblr-card-subtitle-color: ;
  --tblr-card-border-width: var(--tblr-border-width);
  --tblr-card-border-color: var(--tblr-border-color-translucent);
  --tblr-card-border-radius: var(--tblr-border-radius-lg);
  --tblr-card-box-shadow: var(--tblr-shadow-card);
  --tblr-card-inner-border-radius: calc(var(--tblr-border-radius-lg) - (var(--tblr-border-width)));
  --tblr-card-cap-padding-y: 1rem;
  --tblr-card-cap-padding-x: 1.25rem;
  --tblr-card-cap-bg: var(--tblr-bg-surface-tertiary);
  --tblr-card-cap-color: inherit;
  --tblr-card-height: ;
  --tblr-card-color: inherit;
  --tblr-card-bg: var(--tblr-bg-surface);
  --tblr-card-img-overlay-padding: 1rem;
  --tblr-card-group-margin: 1.5rem;
  position: relative;
  display: flex;
  flex-direction: column;
  min-width: 0;
  height: var(--tblr-card-height);
  color: var(--tblr-body-color);
  word-wrap: break-word;
  background-color: var(--tblr-card-bg);
  background-clip: border-box;
  border: var(--tblr-card-border-width) solid var(--tblr-card-border-color);
  border-radius: var(--tblr-card-border-radius);
  box-shadow: var(--tblr-card-box-shadow);
}
.card > hr, .card > .hr {
  margin-left: 0;
  margin-right: 0;
}
.card > .list-group {
  border-top: inherit;
  border-bottom: inherit;
}
.card > .list-group:first-child {
  border-top-width: 0;
  border-top-right-radius: var(--tblr-card-inner-border-radius);
  border-top-left-radius: var(--tblr-card-inner-border-radius);
}
.card > .list-group:last-child {
  border-bottom-width: 0;
  border-bottom-left-radius: var(--tblr-card-inner-border-radius);
  border-bottom-right-radius: var(--tblr-card-inner-border-radius);
}
.card > .card-header + .list-group,
.card > .list-group + .card-footer {
  border-top: 0;
}

.card-body {
  flex: 1 1 auto;
  padding: var(--tblr-card-spacer-y) var(--tblr-card-spacer-x);
  color: var(--tblr-card-color);
}

.card-title {
  margin-bottom: var(--tblr-card-title-spacer-y);
  color: var(--tblr-card-title-color);
}

.card-subtitle {
  margin-top: calc(-0.5 * var(--tblr-card-title-spacer-y));
  margin-bottom: 0;
  color: var(--tblr-card-subtitle-color);
}

.card-text:last-child {
  margin-bottom: 0;
}

.card-link + .card-link {
  margin-right: var(--tblr-card-spacer-x);
}

.card-header {
  padding: var(--tblr-card-cap-padding-y) var(--tblr-card-cap-padding-x);
  margin-bottom: 0;
  color: var(--tblr-card-cap-color);
  background-color: var(--tblr-card-cap-bg);
  border-bottom: var(--tblr-card-border-width) solid var(--tblr-card-border-color);
}
.card-header:first-child {
  border-radius: var(--tblr-card-inner-border-radius) var(--tblr-card-inner-border-radius) 0 0;
}

.card-footer {
  padding: var(--tblr-card-cap-padding-y) var(--tblr-card-cap-padding-x);
  color: var(--tblr-card-cap-color);
  background-color: var(--tblr-card-cap-bg);
  border-top: var(--tblr-card-border-width) solid var(--tblr-card-border-color);
}
.card-footer:last-child {
  border-radius: 0 0 var(--tblr-card-inner-border-radius) var(--tblr-card-inner-border-radius);
}

.card-header-tabs {
  margin-left: calc(-0.5 * var(--tblr-card-cap-padding-x));
  margin-bottom: calc(-1 * var(--tblr-card-cap-padding-y));
  margin-right: calc(-0.5 * var(--tblr-card-cap-padding-x));
  border-bottom: 0;
}
.card-header-tabs .nav-link.active {
  background-color: var(--tblr-card-bg);
  border-bottom-color: var(--tblr-card-bg);
}

.card-header-pills {
  margin-left: calc(-0.5 * var(--tblr-card-cap-padding-x));
  margin-right: calc(-0.5 * var(--tblr-card-cap-padding-x));
}

.card-img-overlay {
  position: absolute;
  top: 0;
  left: 0;
  bottom: 0;
  right: 0;
  padding: var(--tblr-card-img-overlay-padding);
  border-radius: var(--tblr-card-inner-border-radius);
}

.card-img,
.card-img-top,
.card-img-bottom {
  width: 100%;
}

.card-img,
.card-img-top {
  border-top-right-radius: var(--tblr-card-inner-border-radius);
  border-top-left-radius: var(--tblr-card-inner-border-radius);
}

.card-img,
.card-img-bottom {
  border-bottom-left-radius: var(--tblr-card-inner-border-radius);
  border-bottom-right-radius: var(--tblr-card-inner-border-radius);
}

.card-group > .card {
  margin-bottom: var(--tblr-card-group-margin);
}
@media (min-width: 576px) {
  .card-group {
    display: flex;
    flex-flow: row wrap;
  }
  .card-group > .card {
    flex: 1 0 0;
    margin-bottom: 0;
  }
  .card-group > .card + .card {
    margin-right: 0;
    border-right: 0;
  }
  .card-group > .card:not(:last-child) {
    border-top-left-radius: 0;
    border-bottom-left-radius: 0;
  }
  .card-group > .card:not(:last-child) > .card-img-top,
  .card-group > .card:not(:last-child) > .card-header {
    border-top-left-radius: 0;
  }
  .card-group > .card:not(:last-child) > .card-img-bottom,
  .card-group > .card:not(:last-child) > .card-footer {
    border-bottom-left-radius: 0;
  }
  .card-group > .card:not(:first-child) {
    border-top-right-radius: 0;
    border-bottom-right-radius: 0;
  }
  .card-group > .card:not(:first-child) > .card-img-top,
  .card-group > .card:not(:first-child) > .card-header {
    border-top-right-radius: 0;
  }
  .card-group > .card:not(:first-child) > .card-img-bottom,
  .card-group > .card:not(:first-child) > .card-footer {
    border-bottom-right-radius: 0;
  }
}

.pagination {
  --tblr-pagination-padding-x: 0.25rem;
  --tblr-pagination-padding-y: calc(0.25rem + 1px);
  --tblr-pagination-font-size: 0.875rem;
  --tblr-pagination-color: var(--tblr-body-color);
  --tblr-pagination-bg: transparent;
  --tblr-pagination-border-width: 1px;
  --tblr-pagination-border-color: transparent;
  --tblr-pagination-border-radius: var(--tblr-border-radius);
  --tblr-pagination-hover-color: var(--tblr-link-hover-color);
  --tblr-pagination-hover-bg: var(--tblr-active-bg);
  --tblr-pagination-hover-border-color: var(--tblr-pagination-border-color);
  --tblr-pagination-focus-color: var(--tblr-link-hover-color);
  --tblr-pagination-focus-bg: var(--tblr-secondary-bg);
  --tblr-pagination-focus-box-shadow: 0 0 0 0.25rem rgba(var(--tblr-primary-rgb), 0.25);
  --tblr-pagination-active-color: #ffffff;
  --tblr-pagination-active-bg: var(--tblr-primary);
  --tblr-pagination-active-border-color: var(--tblr-primary);
  --tblr-pagination-disabled-color: var(--tblr-disabled-color);
  --tblr-pagination-disabled-bg: transparent;
  --tblr-pagination-disabled-border-color: var(--tblr-pagination-border-color);
  display: flex;
  padding-right: 0;
  list-style: none;
}

.page-link {
  position: relative;
  display: block;
  padding: var(--tblr-pagination-padding-y) var(--tblr-pagination-padding-x);
  font-size: var(--tblr-pagination-font-size);
  color: var(--tblr-pagination-color);
  background-color: var(--tblr-pagination-bg);
  border: var(--tblr-pagination-border-width) solid var(--tblr-pagination-border-color);
  transition: color 0.15s ease-in-out, background-color 0.15s ease-in-out, border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .page-link {
    transition: none;
  }
}
.page-link:hover {
  z-index: 2;
  color: var(--tblr-pagination-hover-color);
  background-color: var(--tblr-pagination-hover-bg);
  border-color: var(--tblr-pagination-hover-border-color);
}
.page-link:focus {
  z-index: 3;
  color: var(--tblr-pagination-focus-color);
  background-color: var(--tblr-pagination-focus-bg);
  outline: 0;
  box-shadow: var(--tblr-pagination-focus-box-shadow);
}
.page-link.active, .active > .page-link {
  z-index: 3;
  color: var(--tblr-pagination-active-color);
  background-color: var(--tblr-pagination-active-bg);
  border-color: var(--tblr-pagination-active-border-color);
}
.page-link.disabled, .disabled > .page-link {
  color: var(--tblr-pagination-disabled-color);
  pointer-events: none;
  background-color: var(--tblr-pagination-disabled-bg);
  border-color: var(--tblr-pagination-disabled-border-color);
}

.page-item:not(:first-child) .page-link {
  margin-right: calc(-1 * 1px);
}
.page-item:first-child .page-link {
  border-top-right-radius: var(--tblr-pagination-border-radius);
  border-bottom-right-radius: var(--tblr-pagination-border-radius);
}
.page-item:last-child .page-link {
  border-top-left-radius: var(--tblr-pagination-border-radius);
  border-bottom-left-radius: var(--tblr-pagination-border-radius);
}

.pagination-lg {
  --tblr-pagination-padding-x: 1.5rem;
  --tblr-pagination-padding-y: 0.75rem;
  --tblr-pagination-font-size: 1.09375rem;
  --tblr-pagination-border-radius: var(--tblr-border-radius-lg);
}

.pagination-sm {
  --tblr-pagination-padding-x: 0.5rem;
  --tblr-pagination-padding-y: 0.25rem;
  --tblr-pagination-font-size: 0.765625rem;
  --tblr-pagination-border-radius: var(--tblr-border-radius-sm);
}

@keyframes progress-bar-stripes {
  0% {
    background-position-x: var(--tblr-progress-height);
  }
}
.progress,
.progress-stacked {
  --tblr-progress-height: 0.5rem;
  --tblr-progress-font-size: 0.65625rem;
  --tblr-progress-bg: var(--tblr-border-color);
  --tblr-progress-border-radius: var(--tblr-border-radius);
  --tblr-progress-box-shadow: var(--tblr-box-shadow-inset);
  --tblr-progress-bar-color: #ffffff;
  --tblr-progress-bar-bg: var(--tblr-primary);
  --tblr-progress-bar-transition: width 0.6s ease;
  display: flex;
  height: var(--tblr-progress-height);
  overflow: hidden;
  font-size: var(--tblr-progress-font-size);
  background-color: var(--tblr-progress-bg);
  border-radius: var(--tblr-progress-border-radius);
  box-shadow: var(--tblr-progress-box-shadow);
}

.progress-bar {
  display: flex;
  flex-direction: column;
  justify-content: center;
  overflow: hidden;
  color: var(--tblr-progress-bar-color);
  text-align: center;
  white-space: nowrap;
  background-color: var(--tblr-progress-bar-bg);
  transition: var(--tblr-progress-bar-transition);
}
@media (prefers-reduced-motion: reduce) {
  .progress-bar {
    transition: none;
  }
}

.progress-bar-striped {
  background-image: linear-gradient(-45deg, rgba(255, 255, 255, 0.15) 25%, transparent 25%, transparent 50%, rgba(255, 255, 255, 0.15) 50%, rgba(255, 255, 255, 0.15) 75%, transparent 75%, transparent);
  background-size: var(--tblr-progress-height) var(--tblr-progress-height);
}

.progress-stacked > .progress {
  overflow: visible;
}

.progress-stacked > .progress > .progress-bar {
  width: 100%;
}

.progress-bar-animated {
  animation: 1s linear infinite progress-bar-stripes;
}
@media (prefers-reduced-motion: reduce) {
  .progress-bar-animated {
    animation: none;
  }
}

.list-group {
  --tblr-list-group-color: var(--tblr-body-color);
  --tblr-list-group-bg: inherit;
  --tblr-list-group-border-color: var(--tblr-border-color);
  --tblr-list-group-border-width: var(--tblr-border-width);
  --tblr-list-group-border-radius: var(--tblr-border-radius);
  --tblr-list-group-item-padding-x: 1.25rem;
  --tblr-list-group-item-padding-y: 1rem;
  --tblr-list-group-action-color: inherit;
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: rgba(var(--tblr-secondary-rgb), 0.08);
  --tblr-list-group-action-active-color: var(--tblr-body-color);
  --tblr-list-group-action-active-bg: var(--tblr-secondary-bg);
  --tblr-list-group-disabled-color: var(--tblr-secondary-color);
  --tblr-list-group-disabled-bg: inherit;
  --tblr-list-group-active-color: inherit;
  --tblr-list-group-active-bg: var(--tblr-active-bg);
  --tblr-list-group-active-border-color: var(--tblr-border-color);
  display: flex;
  flex-direction: column;
  padding-right: 0;
  margin-bottom: 0;
  border-radius: var(--tblr-list-group-border-radius);
}

.list-group-numbered {
  list-style-type: none;
  counter-reset: section;
}
.list-group-numbered > .list-group-item::before {
  content: counters(section, ".") ". ";
  counter-increment: section;
}

.list-group-item {
  position: relative;
  display: block;
  padding: var(--tblr-list-group-item-padding-y) var(--tblr-list-group-item-padding-x);
  color: var(--tblr-list-group-color);
  background-color: var(--tblr-list-group-bg);
  border: var(--tblr-list-group-border-width) solid var(--tblr-list-group-border-color);
}
.list-group-item:first-child {
  border-top-right-radius: inherit;
  border-top-left-radius: inherit;
}
.list-group-item:last-child {
  border-bottom-left-radius: inherit;
  border-bottom-right-radius: inherit;
}
.list-group-item.disabled, .list-group-item:disabled {
  color: var(--tblr-list-group-disabled-color);
  pointer-events: none;
  background-color: var(--tblr-list-group-disabled-bg);
}
.list-group-item.active {
  z-index: 2;
  color: var(--tblr-list-group-active-color);
  background-color: var(--tblr-list-group-active-bg);
  border-color: var(--tblr-list-group-active-border-color);
}
.list-group-item + .list-group-item {
  border-top-width: 0;
}
.list-group-item + .list-group-item.active {
  margin-top: calc(-1 * var(--tblr-list-group-border-width));
  border-top-width: var(--tblr-list-group-border-width);
}

.list-group-item-action {
  width: 100%;
  color: var(--tblr-list-group-action-color);
  text-align: inherit;
}
.list-group-item-action:not(.active):hover, .list-group-item-action:not(.active):focus {
  z-index: 1;
  color: var(--tblr-list-group-action-hover-color);
  text-decoration: none;
  background-color: var(--tblr-list-group-action-hover-bg);
}
.list-group-item-action:not(.active):active {
  color: var(--tblr-list-group-action-active-color);
  background-color: var(--tblr-list-group-action-active-bg);
}

.list-group-horizontal {
  flex-direction: row;
}
.list-group-horizontal > .list-group-item:first-child:not(:last-child) {
  border-bottom-right-radius: var(--tblr-list-group-border-radius);
  border-top-left-radius: 0;
}
.list-group-horizontal > .list-group-item:last-child:not(:first-child) {
  border-top-left-radius: var(--tblr-list-group-border-radius);
  border-bottom-right-radius: 0;
}
.list-group-horizontal > .list-group-item.active {
  margin-top: 0;
}
.list-group-horizontal > .list-group-item + .list-group-item {
  border-top-width: var(--tblr-list-group-border-width);
  border-right-width: 0;
}
.list-group-horizontal > .list-group-item + .list-group-item.active {
  margin-right: calc(-1 * var(--tblr-list-group-border-width));
  border-right-width: var(--tblr-list-group-border-width);
}

@media (min-width: 576px) {
  .list-group-horizontal-sm {
    flex-direction: row;
  }
  .list-group-horizontal-sm > .list-group-item:first-child:not(:last-child) {
    border-bottom-right-radius: var(--tblr-list-group-border-radius);
    border-top-left-radius: 0;
  }
  .list-group-horizontal-sm > .list-group-item:last-child:not(:first-child) {
    border-top-left-radius: var(--tblr-list-group-border-radius);
    border-bottom-right-radius: 0;
  }
  .list-group-horizontal-sm > .list-group-item.active {
    margin-top: 0;
  }
  .list-group-horizontal-sm > .list-group-item + .list-group-item {
    border-top-width: var(--tblr-list-group-border-width);
    border-right-width: 0;
  }
  .list-group-horizontal-sm > .list-group-item + .list-group-item.active {
    margin-right: calc(-1 * var(--tblr-list-group-border-width));
    border-right-width: var(--tblr-list-group-border-width);
  }
}
@media (min-width: 768px) {
  .list-group-horizontal-md {
    flex-direction: row;
  }
  .list-group-horizontal-md > .list-group-item:first-child:not(:last-child) {
    border-bottom-right-radius: var(--tblr-list-group-border-radius);
    border-top-left-radius: 0;
  }
  .list-group-horizontal-md > .list-group-item:last-child:not(:first-child) {
    border-top-left-radius: var(--tblr-list-group-border-radius);
    border-bottom-right-radius: 0;
  }
  .list-group-horizontal-md > .list-group-item.active {
    margin-top: 0;
  }
  .list-group-horizontal-md > .list-group-item + .list-group-item {
    border-top-width: var(--tblr-list-group-border-width);
    border-right-width: 0;
  }
  .list-group-horizontal-md > .list-group-item + .list-group-item.active {
    margin-right: calc(-1 * var(--tblr-list-group-border-width));
    border-right-width: var(--tblr-list-group-border-width);
  }
}
@media (min-width: 992px) {
  .list-group-horizontal-lg {
    flex-direction: row;
  }
  .list-group-horizontal-lg > .list-group-item:first-child:not(:last-child) {
    border-bottom-right-radius: var(--tblr-list-group-border-radius);
    border-top-left-radius: 0;
  }
  .list-group-horizontal-lg > .list-group-item:last-child:not(:first-child) {
    border-top-left-radius: var(--tblr-list-group-border-radius);
    border-bottom-right-radius: 0;
  }
  .list-group-horizontal-lg > .list-group-item.active {
    margin-top: 0;
  }
  .list-group-horizontal-lg > .list-group-item + .list-group-item {
    border-top-width: var(--tblr-list-group-border-width);
    border-right-width: 0;
  }
  .list-group-horizontal-lg > .list-group-item + .list-group-item.active {
    margin-right: calc(-1 * var(--tblr-list-group-border-width));
    border-right-width: var(--tblr-list-group-border-width);
  }
}
@media (min-width: 1200px) {
  .list-group-horizontal-xl {
    flex-direction: row;
  }
  .list-group-horizontal-xl > .list-group-item:first-child:not(:last-child) {
    border-bottom-right-radius: var(--tblr-list-group-border-radius);
    border-top-left-radius: 0;
  }
  .list-group-horizontal-xl > .list-group-item:last-child:not(:first-child) {
    border-top-left-radius: var(--tblr-list-group-border-radius);
    border-bottom-right-radius: 0;
  }
  .list-group-horizontal-xl > .list-group-item.active {
    margin-top: 0;
  }
  .list-group-horizontal-xl > .list-group-item + .list-group-item {
    border-top-width: var(--tblr-list-group-border-width);
    border-right-width: 0;
  }
  .list-group-horizontal-xl > .list-group-item + .list-group-item.active {
    margin-right: calc(-1 * var(--tblr-list-group-border-width));
    border-right-width: var(--tblr-list-group-border-width);
  }
}
@media (min-width: 1400px) {
  .list-group-horizontal-xxl {
    flex-direction: row;
  }
  .list-group-horizontal-xxl > .list-group-item:first-child:not(:last-child) {
    border-bottom-right-radius: var(--tblr-list-group-border-radius);
    border-top-left-radius: 0;
  }
  .list-group-horizontal-xxl > .list-group-item:last-child:not(:first-child) {
    border-top-left-radius: var(--tblr-list-group-border-radius);
    border-bottom-right-radius: 0;
  }
  .list-group-horizontal-xxl > .list-group-item.active {
    margin-top: 0;
  }
  .list-group-horizontal-xxl > .list-group-item + .list-group-item {
    border-top-width: var(--tblr-list-group-border-width);
    border-right-width: 0;
  }
  .list-group-horizontal-xxl > .list-group-item + .list-group-item.active {
    margin-right: calc(-1 * var(--tblr-list-group-border-width));
    border-right-width: var(--tblr-list-group-border-width);
  }
}
.list-group-flush {
  border-radius: 0;
}
.list-group-flush > .list-group-item {
  border-width: 0 0 var(--tblr-list-group-border-width);
}
.list-group-flush > .list-group-item:last-child {
  border-bottom-width: 0;
}

.list-group-item-primary {
  --tblr-list-group-color: var(--tblr-primary-text-emphasis);
  --tblr-list-group-bg: var(--tblr-primary-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-primary-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-primary-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-primary-border-subtle);
  --tblr-list-group-active-color: var(--tblr-primary-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-primary-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-primary-text-emphasis);
}

.list-group-item-secondary {
  --tblr-list-group-color: var(--tblr-secondary-text-emphasis);
  --tblr-list-group-bg: var(--tblr-secondary-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-secondary-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-secondary-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-secondary-border-subtle);
  --tblr-list-group-active-color: var(--tblr-secondary-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-secondary-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-secondary-text-emphasis);
}

.list-group-item-success {
  --tblr-list-group-color: var(--tblr-success-text-emphasis);
  --tblr-list-group-bg: var(--tblr-success-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-success-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-success-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-success-border-subtle);
  --tblr-list-group-active-color: var(--tblr-success-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-success-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-success-text-emphasis);
}

.list-group-item-info {
  --tblr-list-group-color: var(--tblr-info-text-emphasis);
  --tblr-list-group-bg: var(--tblr-info-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-info-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-info-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-info-border-subtle);
  --tblr-list-group-active-color: var(--tblr-info-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-info-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-info-text-emphasis);
}

.list-group-item-warning {
  --tblr-list-group-color: var(--tblr-warning-text-emphasis);
  --tblr-list-group-bg: var(--tblr-warning-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-warning-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-warning-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-warning-border-subtle);
  --tblr-list-group-active-color: var(--tblr-warning-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-warning-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-warning-text-emphasis);
}

.list-group-item-danger {
  --tblr-list-group-color: var(--tblr-danger-text-emphasis);
  --tblr-list-group-bg: var(--tblr-danger-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-danger-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-danger-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-danger-border-subtle);
  --tblr-list-group-active-color: var(--tblr-danger-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-danger-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-danger-text-emphasis);
}

.list-group-item-light {
  --tblr-list-group-color: var(--tblr-light-text-emphasis);
  --tblr-list-group-bg: var(--tblr-light-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-light-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-light-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-light-border-subtle);
  --tblr-list-group-active-color: var(--tblr-light-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-light-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-light-text-emphasis);
}

.list-group-item-dark {
  --tblr-list-group-color: var(--tblr-dark-text-emphasis);
  --tblr-list-group-bg: var(--tblr-dark-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-dark-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-dark-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-dark-border-subtle);
  --tblr-list-group-active-color: var(--tblr-dark-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-dark-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-dark-text-emphasis);
}

.list-group-item-muted {
  --tblr-list-group-color: var(--tblr-muted-text-emphasis);
  --tblr-list-group-bg: var(--tblr-muted-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-muted-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-muted-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-muted-border-subtle);
  --tblr-list-group-active-color: var(--tblr-muted-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-muted-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-muted-text-emphasis);
}

.list-group-item-blue {
  --tblr-list-group-color: var(--tblr-blue-text-emphasis);
  --tblr-list-group-bg: var(--tblr-blue-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-blue-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-blue-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-blue-border-subtle);
  --tblr-list-group-active-color: var(--tblr-blue-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-blue-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-blue-text-emphasis);
}

.list-group-item-azure {
  --tblr-list-group-color: var(--tblr-azure-text-emphasis);
  --tblr-list-group-bg: var(--tblr-azure-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-azure-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-azure-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-azure-border-subtle);
  --tblr-list-group-active-color: var(--tblr-azure-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-azure-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-azure-text-emphasis);
}

.list-group-item-indigo {
  --tblr-list-group-color: var(--tblr-indigo-text-emphasis);
  --tblr-list-group-bg: var(--tblr-indigo-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-indigo-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-indigo-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-indigo-border-subtle);
  --tblr-list-group-active-color: var(--tblr-indigo-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-indigo-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-indigo-text-emphasis);
}

.list-group-item-purple {
  --tblr-list-group-color: var(--tblr-purple-text-emphasis);
  --tblr-list-group-bg: var(--tblr-purple-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-purple-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-purple-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-purple-border-subtle);
  --tblr-list-group-active-color: var(--tblr-purple-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-purple-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-purple-text-emphasis);
}

.list-group-item-pink {
  --tblr-list-group-color: var(--tblr-pink-text-emphasis);
  --tblr-list-group-bg: var(--tblr-pink-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-pink-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-pink-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-pink-border-subtle);
  --tblr-list-group-active-color: var(--tblr-pink-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-pink-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-pink-text-emphasis);
}

.list-group-item-red {
  --tblr-list-group-color: var(--tblr-red-text-emphasis);
  --tblr-list-group-bg: var(--tblr-red-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-red-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-red-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-red-border-subtle);
  --tblr-list-group-active-color: var(--tblr-red-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-red-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-red-text-emphasis);
}

.list-group-item-orange {
  --tblr-list-group-color: var(--tblr-orange-text-emphasis);
  --tblr-list-group-bg: var(--tblr-orange-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-orange-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-orange-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-orange-border-subtle);
  --tblr-list-group-active-color: var(--tblr-orange-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-orange-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-orange-text-emphasis);
}

.list-group-item-yellow {
  --tblr-list-group-color: var(--tblr-yellow-text-emphasis);
  --tblr-list-group-bg: var(--tblr-yellow-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-yellow-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-yellow-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-yellow-border-subtle);
  --tblr-list-group-active-color: var(--tblr-yellow-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-yellow-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-yellow-text-emphasis);
}

.list-group-item-lime {
  --tblr-list-group-color: var(--tblr-lime-text-emphasis);
  --tblr-list-group-bg: var(--tblr-lime-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-lime-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-lime-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-lime-border-subtle);
  --tblr-list-group-active-color: var(--tblr-lime-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-lime-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-lime-text-emphasis);
}

.list-group-item-green {
  --tblr-list-group-color: var(--tblr-green-text-emphasis);
  --tblr-list-group-bg: var(--tblr-green-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-green-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-green-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-green-border-subtle);
  --tblr-list-group-active-color: var(--tblr-green-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-green-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-green-text-emphasis);
}

.list-group-item-teal {
  --tblr-list-group-color: var(--tblr-teal-text-emphasis);
  --tblr-list-group-bg: var(--tblr-teal-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-teal-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-teal-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-teal-border-subtle);
  --tblr-list-group-active-color: var(--tblr-teal-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-teal-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-teal-text-emphasis);
}

.list-group-item-cyan {
  --tblr-list-group-color: var(--tblr-cyan-text-emphasis);
  --tblr-list-group-bg: var(--tblr-cyan-bg-subtle);
  --tblr-list-group-border-color: var(--tblr-cyan-border-subtle);
  --tblr-list-group-action-hover-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-hover-bg: var(--tblr-cyan-border-subtle);
  --tblr-list-group-action-active-color: var(--tblr-emphasis-color);
  --tblr-list-group-action-active-bg: var(--tblr-cyan-border-subtle);
  --tblr-list-group-active-color: var(--tblr-cyan-bg-subtle);
  --tblr-list-group-active-bg: var(--tblr-cyan-text-emphasis);
  --tblr-list-group-active-border-color: var(--tblr-cyan-text-emphasis);
}

.toast {
  --tblr-toast-zindex: 1090;
  --tblr-toast-padding-x: 0.75rem;
  --tblr-toast-padding-y: 0.5rem;
  --tblr-toast-spacing: calc(var(--tblr-page-padding) * 2);
  --tblr-toast-max-width: 350px;
  --tblr-toast-font-size: 0.875rem;
  --tblr-toast-color: ;
  --tblr-toast-bg: var(--tblr-bg-surface);
  --tblr-toast-border-width: var(--tblr-border-width);
  --tblr-toast-border-color: var(--tblr-border-color);
  --tblr-toast-border-radius: var(--tblr-border-radius);
  --tblr-toast-box-shadow: var(--tblr-box-shadow);
  --tblr-toast-header-color: var(--tblr-gray-500);
  --tblr-toast-header-bg: rgba(var(--tblr-body-bg-rgb), 0.85);
  --tblr-toast-header-border-color: var(--tblr-border-color);
  width: var(--tblr-toast-max-width);
  max-width: 100%;
  font-size: var(--tblr-toast-font-size);
  color: var(--tblr-toast-color);
  pointer-events: auto;
  background-color: var(--tblr-toast-bg);
  background-clip: padding-box;
  border: var(--tblr-toast-border-width) solid var(--tblr-toast-border-color);
  box-shadow: var(--tblr-toast-box-shadow);
  border-radius: var(--tblr-toast-border-radius);
}
.toast.showing {
  opacity: 0;
}
.toast:not(.show) {
  display: none;
}

.toast-container {
  --tblr-toast-zindex: 1090;
  position: absolute;
  z-index: var(--tblr-toast-zindex);
  width: max-content;
  max-width: 100%;
  pointer-events: none;
}
.toast-container > :not(:last-child) {
  margin-bottom: var(--tblr-toast-spacing);
}

.toast-header {
  display: flex;
  align-items: center;
  padding: var(--tblr-toast-padding-y) var(--tblr-toast-padding-x);
  color: var(--tblr-toast-header-color);
  background-color: var(--tblr-toast-header-bg);
  background-clip: padding-box;
  border-bottom: var(--tblr-toast-border-width) solid var(--tblr-toast-header-border-color);
  border-top-right-radius: calc(var(--tblr-toast-border-radius) - var(--tblr-toast-border-width));
  border-top-left-radius: calc(var(--tblr-toast-border-radius) - var(--tblr-toast-border-width));
}
.toast-header .btn-close {
  margin-left: calc(-0.5 * var(--tblr-toast-padding-x));
  margin-right: var(--tblr-toast-padding-x);
}

.toast-body {
  padding: var(--tblr-toast-padding-x);
  word-wrap: break-word;
}

.modal {
  --tblr-modal-zindex: 1055;
  --tblr-modal-width: 540px;
  --tblr-modal-padding: 1.5rem;
  --tblr-modal-margin: 0.5rem;
  --tblr-modal-color: var(--tblr-body-color);
  --tblr-modal-bg: var(--tblr-bg-surface);
  --tblr-modal-border-color: transparent;
  --tblr-modal-border-width: var(--tblr-border-width);
  --tblr-modal-border-radius: var(--tblr-border-radius-lg);
  --tblr-modal-box-shadow: var(--tblr-box-shadow-sm);
  --tblr-modal-inner-border-radius: calc(var(--tblr-modal-border-radius) - 1px);
  --tblr-modal-header-padding-x: 1.5rem;
  --tblr-modal-header-padding-y: 1.5rem;
  --tblr-modal-header-padding: 1.5rem;
  --tblr-modal-header-border-color: var(--tblr-border-color);
  --tblr-modal-header-border-width: var(--tblr-border-width);
  --tblr-modal-title-line-height: 1.4285714286;
  --tblr-modal-footer-gap: 0.75rem;
  --tblr-modal-footer-bg: var(--tblr-bg-surface-tertiary);
  --tblr-modal-footer-border-color: var(--tblr-border-color);
  --tblr-modal-footer-border-width: var(--tblr-border-width);
  position: fixed;
  top: 0;
  right: 0;
  z-index: var(--tblr-modal-zindex);
  display: none;
  width: 100%;
  height: 100%;
  overflow-x: hidden;
  overflow-y: auto;
  outline: 0;
}

.modal-dialog {
  position: relative;
  width: auto;
  margin: var(--tblr-modal-margin);
  pointer-events: none;
}
.modal.fade .modal-dialog {
  transform: translate(0, -1rem);
  transition: transform 0.3s ease-out;
}
@media (prefers-reduced-motion: reduce) {
  .modal.fade .modal-dialog {
    transition: none;
  }
}
.modal.show .modal-dialog {
  transform: none;
}
.modal.modal-static .modal-dialog {
  transform: scale(1.02);
}

.modal-dialog-scrollable {
  height: calc(100% - var(--tblr-modal-margin) * 2);
}
.modal-dialog-scrollable .modal-content {
  max-height: 100%;
  overflow: hidden;
}
.modal-dialog-scrollable .modal-body {
  overflow-y: auto;
}

.modal-dialog-centered {
  display: flex;
  align-items: center;
  min-height: calc(100% - var(--tblr-modal-margin) * 2);
}

.modal-content {
  position: relative;
  display: flex;
  flex-direction: column;
  width: 100%;
  color: var(--tblr-modal-color);
  pointer-events: auto;
  background-color: var(--tblr-modal-bg);
  background-clip: padding-box;
  border: var(--tblr-modal-border-width) solid var(--tblr-modal-border-color);
  border-radius: var(--tblr-modal-border-radius);
  box-shadow: var(--tblr-modal-box-shadow);
  outline: 0;
}

.modal-backdrop {
  --tblr-backdrop-zindex: 1050;
  --tblr-backdrop-bg: var(--tblr-gray-800);
  --tblr-backdrop-opacity: 0.24;
  position: fixed;
  top: 0;
  right: 0;
  z-index: var(--tblr-backdrop-zindex);
  width: 100vw;
  height: 100vh;
  background-color: var(--tblr-backdrop-bg);
}
.modal-backdrop.fade {
  opacity: 0;
}
.modal-backdrop.show {
  opacity: var(--tblr-backdrop-opacity);
}

.modal-header {
  display: flex;
  flex-shrink: 0;
  align-items: center;
  padding: var(--tblr-modal-header-padding);
  border-bottom: var(--tblr-modal-header-border-width) solid var(--tblr-modal-header-border-color);
  border-top-right-radius: var(--tblr-modal-inner-border-radius);
  border-top-left-radius: var(--tblr-modal-inner-border-radius);
}
.modal-header .btn-close {
  padding: calc(var(--tblr-modal-header-padding-y) * 0.5) calc(var(--tblr-modal-header-padding-x) * 0.5);
  margin-top: calc(-0.5 * var(--tblr-modal-header-padding-y));
  margin-left: calc(-0.5 * var(--tblr-modal-header-padding-x));
  margin-bottom: calc(-0.5 * var(--tblr-modal-header-padding-y));
  margin-right: auto;
}

.modal-title {
  margin-bottom: 0;
  line-height: var(--tblr-modal-title-line-height);
}

.modal-body {
  position: relative;
  flex: 1 1 auto;
  padding: var(--tblr-modal-padding);
}

.modal-footer {
  display: flex;
  flex-shrink: 0;
  flex-wrap: wrap;
  align-items: center;
  justify-content: flex-end;
  padding: calc(var(--tblr-modal-padding) - var(--tblr-modal-footer-gap) * 0.5);
  background-color: var(--tblr-modal-footer-bg);
  border-top: var(--tblr-modal-footer-border-width) solid var(--tblr-modal-footer-border-color);
  border-bottom-left-radius: var(--tblr-modal-inner-border-radius);
  border-bottom-right-radius: var(--tblr-modal-inner-border-radius);
}
.modal-footer > * {
  margin: calc(var(--tblr-modal-footer-gap) * 0.5);
}

@media (min-width: 576px) {
  .modal {
    --tblr-modal-margin: 1.75rem;
    --tblr-modal-box-shadow: var(--tblr-box-shadow);
  }
  .modal-dialog {
    max-width: var(--tblr-modal-width);
    margin-left: auto;
    margin-right: auto;
  }
  .modal-sm {
    --tblr-modal-width: 380px;
  }
}
@media (min-width: 992px) {
  .modal-lg,
  .modal-xl {
    --tblr-modal-width: 720px;
  }
}
@media (min-width: 1200px) {
  .modal-xl {
    --tblr-modal-width: 1140px;
  }
}
.modal-fullscreen {
  width: 100vw;
  max-width: none;
  height: 100%;
  margin: 0;
}
.modal-fullscreen .modal-content {
  height: 100%;
  border: 0;
  border-radius: 0;
}
.modal-fullscreen .modal-header,
.modal-fullscreen .modal-footer {
  border-radius: 0;
}
.modal-fullscreen .modal-body {
  overflow-y: auto;
}

@media (max-width: 575.98px) {
  .modal-fullscreen-sm-down {
    width: 100vw;
    max-width: none;
    height: 100%;
    margin: 0;
  }
  .modal-fullscreen-sm-down .modal-content {
    height: 100%;
    border: 0;
    border-radius: 0;
  }
  .modal-fullscreen-sm-down .modal-header,
  .modal-fullscreen-sm-down .modal-footer {
    border-radius: 0;
  }
  .modal-fullscreen-sm-down .modal-body {
    overflow-y: auto;
  }
}
@media (max-width: 767.98px) {
  .modal-fullscreen-md-down {
    width: 100vw;
    max-width: none;
    height: 100%;
    margin: 0;
  }
  .modal-fullscreen-md-down .modal-content {
    height: 100%;
    border: 0;
    border-radius: 0;
  }
  .modal-fullscreen-md-down .modal-header,
  .modal-fullscreen-md-down .modal-footer {
    border-radius: 0;
  }
  .modal-fullscreen-md-down .modal-body {
    overflow-y: auto;
  }
}
@media (max-width: 991.98px) {
  .modal-fullscreen-lg-down {
    width: 100vw;
    max-width: none;
    height: 100%;
    margin: 0;
  }
  .modal-fullscreen-lg-down .modal-content {
    height: 100%;
    border: 0;
    border-radius: 0;
  }
  .modal-fullscreen-lg-down .modal-header,
  .modal-fullscreen-lg-down .modal-footer {
    border-radius: 0;
  }
  .modal-fullscreen-lg-down .modal-body {
    overflow-y: auto;
  }
}
@media (max-width: 1199.98px) {
  .modal-fullscreen-xl-down {
    width: 100vw;
    max-width: none;
    height: 100%;
    margin: 0;
  }
  .modal-fullscreen-xl-down .modal-content {
    height: 100%;
    border: 0;
    border-radius: 0;
  }
  .modal-fullscreen-xl-down .modal-header,
  .modal-fullscreen-xl-down .modal-footer {
    border-radius: 0;
  }
  .modal-fullscreen-xl-down .modal-body {
    overflow-y: auto;
  }
}
@media (max-width: 1399.98px) {
  .modal-fullscreen-xxl-down {
    width: 100vw;
    max-width: none;
    height: 100%;
    margin: 0;
  }
  .modal-fullscreen-xxl-down .modal-content {
    height: 100%;
    border: 0;
    border-radius: 0;
  }
  .modal-fullscreen-xxl-down .modal-header,
  .modal-fullscreen-xxl-down .modal-footer {
    border-radius: 0;
  }
  .modal-fullscreen-xxl-down .modal-body {
    overflow-y: auto;
  }
}
.tooltip {
  --tblr-tooltip-zindex: 1080;
  --tblr-tooltip-max-width: 200px;
  --tblr-tooltip-padding-x: var(--tblr-spacer-3);
  --tblr-tooltip-padding-y: var(--tblr-spacer-1);
  --tblr-tooltip-margin: ;
  --tblr-tooltip-font-size: 0.765625rem;
  --tblr-tooltip-color: var(--tblr-text-inverted);
  --tblr-tooltip-bg: var(--tblr-bg-surface-inverted);
  --tblr-tooltip-border-radius: var(--tblr-border-radius);
  --tblr-tooltip-opacity: 0.9;
  --tblr-tooltip-arrow-width: 0.8rem;
  --tblr-tooltip-arrow-height: 0.4rem;
  z-index: var(--tblr-tooltip-zindex);
  display: block;
  margin: var(--tblr-tooltip-margin);
  font-family: var(--tblr-font-sans-serif);
  font-style: normal;
  font-weight: 400;
  line-height: 1.4285714286;
  text-align: right;
  text-align: start;
  text-decoration: none;
  text-shadow: none;
  text-transform: none;
  letter-spacing: normal;
  word-break: normal;
  white-space: normal;
  word-spacing: normal;
  line-break: auto;
  font-size: var(--tblr-tooltip-font-size);
  word-wrap: break-word;
  opacity: 0;
}
.tooltip.show {
  opacity: var(--tblr-tooltip-opacity);
}
.tooltip .tooltip-arrow {
  display: block;
  width: var(--tblr-tooltip-arrow-width);
  height: var(--tblr-tooltip-arrow-height);
}
.tooltip .tooltip-arrow::before {
  position: absolute;
  content: "";
  border-color: transparent;
  border-style: solid;
}

.bs-tooltip-top .tooltip-arrow, .bs-tooltip-auto[data-popper-placement^=top] .tooltip-arrow {
  bottom: calc(-1 * var(--tblr-tooltip-arrow-height));
}
.bs-tooltip-top .tooltip-arrow::before, .bs-tooltip-auto[data-popper-placement^=top] .tooltip-arrow::before {
  top: -1px;
  border-width: var(--tblr-tooltip-arrow-height) calc(var(--tblr-tooltip-arrow-width) * 0.5) 0;
  border-top-color: var(--tblr-tooltip-bg);
}
.bs-tooltip-end .tooltip-arrow, .bs-tooltip-auto[data-popper-placement^=right] .tooltip-arrow {
  left: calc(-1 * var(--tblr-tooltip-arrow-height));
  width: var(--tblr-tooltip-arrow-height);
  height: var(--tblr-tooltip-arrow-width);
}
.bs-tooltip-end .tooltip-arrow::before, .bs-tooltip-auto[data-popper-placement^=right] .tooltip-arrow::before {
  right: -1px;
  border-width: calc(var(--tblr-tooltip-arrow-width) * 0.5) var(--tblr-tooltip-arrow-height) calc(var(--tblr-tooltip-arrow-width) * 0.5) 0;
  border-right-color: var(--tblr-tooltip-bg);
}
.bs-tooltip-bottom .tooltip-arrow, .bs-tooltip-auto[data-popper-placement^=bottom] .tooltip-arrow {
  top: calc(-1 * var(--tblr-tooltip-arrow-height));
}
.bs-tooltip-bottom .tooltip-arrow::before, .bs-tooltip-auto[data-popper-placement^=bottom] .tooltip-arrow::before {
  bottom: -1px;
  border-width: 0 calc(var(--tblr-tooltip-arrow-width) * 0.5) var(--tblr-tooltip-arrow-height);
  border-bottom-color: var(--tblr-tooltip-bg);
}
.bs-tooltip-start .tooltip-arrow, .bs-tooltip-auto[data-popper-placement^=left] .tooltip-arrow {
  right: calc(-1 * var(--tblr-tooltip-arrow-height));
  width: var(--tblr-tooltip-arrow-height);
  height: var(--tblr-tooltip-arrow-width);
}
.bs-tooltip-start .tooltip-arrow::before, .bs-tooltip-auto[data-popper-placement^=left] .tooltip-arrow::before {
  left: -1px;
  border-width: calc(var(--tblr-tooltip-arrow-width) * 0.5) 0 calc(var(--tblr-tooltip-arrow-width) * 0.5) var(--tblr-tooltip-arrow-height);
  border-left-color: var(--tblr-tooltip-bg);
}
.tooltip-inner {
  max-width: var(--tblr-tooltip-max-width);
  padding: var(--tblr-tooltip-padding-y) var(--tblr-tooltip-padding-x);
  color: var(--tblr-tooltip-color);
  text-align: center;
  background-color: var(--tblr-tooltip-bg);
  border-radius: var(--tblr-tooltip-border-radius);
}

.popover {
  --tblr-popover-zindex: 1070;
  --tblr-popover-max-width: 276px;
  --tblr-popover-font-size: 0.765625rem;
  --tblr-popover-bg: var(--tblr-bg-surface);
  --tblr-popover-border-width: var(--tblr-border-width);
  --tblr-popover-border-color: var(--tblr-border-color);
  --tblr-popover-border-radius: var(--tblr-border-radius-lg);
  --tblr-popover-inner-border-radius: calc(var(--tblr-border-radius-lg) - var(--tblr-border-width));
  --tblr-popover-box-shadow: var(--tblr-shadow-lg);
  --tblr-popover-header-padding-x: 1rem;
  --tblr-popover-header-padding-y: 0.5rem;
  --tblr-popover-header-font-size: 0.875rem;
  --tblr-popover-header-color: inherit;
  --tblr-popover-header-bg: transparent;
  --tblr-popover-body-padding-x: 0.5rem;
  --tblr-popover-body-padding-y: 0.5rem;
  --tblr-popover-body-color: inherit;
  --tblr-popover-arrow-width: 1rem;
  --tblr-popover-arrow-height: 0.5rem;
  --tblr-popover-arrow-border: var(--tblr-popover-border-color);
  z-index: var(--tblr-popover-zindex);
  display: block;
  max-width: var(--tblr-popover-max-width);
  font-family: var(--tblr-font-sans-serif);
  font-style: normal;
  font-weight: 400;
  line-height: 1.4285714286;
  text-align: right;
  text-align: start;
  text-decoration: none;
  text-shadow: none;
  text-transform: none;
  letter-spacing: normal;
  word-break: normal;
  white-space: normal;
  word-spacing: normal;
  line-break: auto;
  font-size: var(--tblr-popover-font-size);
  word-wrap: break-word;
  background-color: var(--tblr-popover-bg);
  background-clip: padding-box;
  border: var(--tblr-popover-border-width) solid var(--tblr-popover-border-color);
  border-radius: var(--tblr-popover-border-radius);
  box-shadow: var(--tblr-popover-box-shadow);
}
.popover .popover-arrow {
  display: block;
  width: var(--tblr-popover-arrow-width);
  height: var(--tblr-popover-arrow-height);
}
.popover .popover-arrow::before, .popover .popover-arrow::after {
  position: absolute;
  display: block;
  content: "";
  border-color: transparent;
  border-style: solid;
  border-width: 0;
}

.bs-popover-top > .popover-arrow, .bs-popover-auto[data-popper-placement^=top] > .popover-arrow {
  bottom: calc(-1 * (var(--tblr-popover-arrow-height)) - var(--tblr-popover-border-width));
}
.bs-popover-top > .popover-arrow::before, .bs-popover-auto[data-popper-placement^=top] > .popover-arrow::before, .bs-popover-top > .popover-arrow::after, .bs-popover-auto[data-popper-placement^=top] > .popover-arrow::after {
  border-width: var(--tblr-popover-arrow-height) calc(var(--tblr-popover-arrow-width) * 0.5) 0;
}
.bs-popover-top > .popover-arrow::before, .bs-popover-auto[data-popper-placement^=top] > .popover-arrow::before {
  bottom: 0;
  border-top-color: var(--tblr-popover-arrow-border);
}
.bs-popover-top > .popover-arrow::after, .bs-popover-auto[data-popper-placement^=top] > .popover-arrow::after {
  bottom: var(--tblr-popover-border-width);
  border-top-color: var(--tblr-popover-bg);
}
.bs-popover-end > .popover-arrow, .bs-popover-auto[data-popper-placement^=right] > .popover-arrow {
  left: calc(-1 * (var(--tblr-popover-arrow-height)) - var(--tblr-popover-border-width));
  width: var(--tblr-popover-arrow-height);
  height: var(--tblr-popover-arrow-width);
}
.bs-popover-end > .popover-arrow::before, .bs-popover-auto[data-popper-placement^=right] > .popover-arrow::before, .bs-popover-end > .popover-arrow::after, .bs-popover-auto[data-popper-placement^=right] > .popover-arrow::after {
  border-width: calc(var(--tblr-popover-arrow-width) * 0.5) var(--tblr-popover-arrow-height) calc(var(--tblr-popover-arrow-width) * 0.5) 0;
}
.bs-popover-end > .popover-arrow::before, .bs-popover-auto[data-popper-placement^=right] > .popover-arrow::before {
  left: 0;
  border-right-color: var(--tblr-popover-arrow-border);
}
.bs-popover-end > .popover-arrow::after, .bs-popover-auto[data-popper-placement^=right] > .popover-arrow::after {
  left: var(--tblr-popover-border-width);
  border-right-color: var(--tblr-popover-bg);
}
.bs-popover-bottom > .popover-arrow, .bs-popover-auto[data-popper-placement^=bottom] > .popover-arrow {
  top: calc(-1 * (var(--tblr-popover-arrow-height)) - var(--tblr-popover-border-width));
}
.bs-popover-bottom > .popover-arrow::before, .bs-popover-auto[data-popper-placement^=bottom] > .popover-arrow::before, .bs-popover-bottom > .popover-arrow::after, .bs-popover-auto[data-popper-placement^=bottom] > .popover-arrow::after {
  border-width: 0 calc(var(--tblr-popover-arrow-width) * 0.5) var(--tblr-popover-arrow-height);
}
.bs-popover-bottom > .popover-arrow::before, .bs-popover-auto[data-popper-placement^=bottom] > .popover-arrow::before {
  top: 0;
  border-bottom-color: var(--tblr-popover-arrow-border);
}
.bs-popover-bottom > .popover-arrow::after, .bs-popover-auto[data-popper-placement^=bottom] > .popover-arrow::after {
  top: var(--tblr-popover-border-width);
  border-bottom-color: var(--tblr-popover-bg);
}
.bs-popover-bottom .popover-header::before, .bs-popover-auto[data-popper-placement^=bottom] .popover-header::before {
  position: absolute;
  top: 0;
  right: 50%;
  display: block;
  width: var(--tblr-popover-arrow-width);
  margin-right: calc(-0.5 * var(--tblr-popover-arrow-width));
  content: "";
  border-bottom: var(--tblr-popover-border-width) solid var(--tblr-popover-header-bg);
}
.bs-popover-start > .popover-arrow, .bs-popover-auto[data-popper-placement^=left] > .popover-arrow {
  right: calc(-1 * (var(--tblr-popover-arrow-height)) - var(--tblr-popover-border-width));
  width: var(--tblr-popover-arrow-height);
  height: var(--tblr-popover-arrow-width);
}
.bs-popover-start > .popover-arrow::before, .bs-popover-auto[data-popper-placement^=left] > .popover-arrow::before, .bs-popover-start > .popover-arrow::after, .bs-popover-auto[data-popper-placement^=left] > .popover-arrow::after {
  border-width: calc(var(--tblr-popover-arrow-width) * 0.5) 0 calc(var(--tblr-popover-arrow-width) * 0.5) var(--tblr-popover-arrow-height);
}
.bs-popover-start > .popover-arrow::before, .bs-popover-auto[data-popper-placement^=left] > .popover-arrow::before {
  right: 0;
  border-left-color: var(--tblr-popover-arrow-border);
}
.bs-popover-start > .popover-arrow::after, .bs-popover-auto[data-popper-placement^=left] > .popover-arrow::after {
  right: var(--tblr-popover-border-width);
  border-left-color: var(--tblr-popover-bg);
}
.popover-header {
  padding: var(--tblr-popover-header-padding-y) var(--tblr-popover-header-padding-x);
  margin-bottom: 0;
  font-size: var(--tblr-popover-header-font-size);
  color: var(--tblr-popover-header-color);
  background-color: var(--tblr-popover-header-bg);
  border-bottom: var(--tblr-popover-border-width) solid var(--tblr-popover-border-color);
  border-top-right-radius: var(--tblr-popover-inner-border-radius);
  border-top-left-radius: var(--tblr-popover-inner-border-radius);
}
.popover-header:empty {
  display: none;
}

.popover-body {
  padding: var(--tblr-popover-body-padding-y) var(--tblr-popover-body-padding-x);
  color: var(--tblr-popover-body-color);
}

.carousel {
  position: relative;
}

.carousel.pointer-event {
  touch-action: pan-y;
}

.carousel-inner {
  position: relative;
  width: 100%;
  overflow: hidden;
}
.carousel-inner::after {
  display: block;
  clear: both;
  content: "";
}

.carousel-item {
  position: relative;
  display: none;
  float: right;
  width: 100%;
  margin-left: -100%;
  backface-visibility: hidden;
  transition: transform 0.6s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .carousel-item {
    transition: none;
  }
}

.carousel-item.active,
.carousel-item-next,
.carousel-item-prev {
  display: block;
}

.carousel-item-next:not(.carousel-item-start),
.active.carousel-item-end {
  transform: translateX(-100%);
}

.carousel-item-prev:not(.carousel-item-end),
.active.carousel-item-start {
  transform: translateX(100%);
}

.carousel-fade .carousel-item {
  opacity: 0;
  transition-property: opacity;
  transform: none;
}
.carousel-fade .carousel-item.active,
.carousel-fade .carousel-item-next.carousel-item-start,
.carousel-fade .carousel-item-prev.carousel-item-end {
  z-index: 1;
  opacity: 1;
}
.carousel-fade .active.carousel-item-start,
.carousel-fade .active.carousel-item-end {
  z-index: 0;
  opacity: 0;
  transition: opacity 0s 0.6s;
}
@media (prefers-reduced-motion: reduce) {
  .carousel-fade .active.carousel-item-start,
  .carousel-fade .active.carousel-item-end {
    transition: none;
  }
}

.carousel-control-prev,
.carousel-control-next {
  position: absolute;
  top: 0;
  bottom: 0;
  z-index: 1;
  display: flex;
  align-items: center;
  justify-content: center;
  width: 15%;
  padding: 0;
  color: #ffffff;
  text-align: center;
  background: none;
  filter: var(--tblr-carousel-control-icon-filter);
  border: 0;
  opacity: 0.5;
  transition: opacity 0.15s ease;
}
@media (prefers-reduced-motion: reduce) {
  .carousel-control-prev,
  .carousel-control-next {
    transition: none;
  }
}
.carousel-control-prev:hover, .carousel-control-prev:focus,
.carousel-control-next:hover,
.carousel-control-next:focus {
  color: #ffffff;
  text-decoration: none;
  outline: 0;
  opacity: 0.9;
}

.carousel-control-prev {
  right: 0;
}

.carousel-control-next {
  left: 0;
}

.carousel-control-prev-icon,
.carousel-control-next-icon {
  display: inline-block;
  width: 1.5rem;
  height: 1.5rem;
  background-repeat: no-repeat;
  background-position: 50%;
  background-size: 100% 100%;
}

.carousel-control-prev-icon {
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' width='24' height='24' viewBox='0 0 24 24' fill='none' stroke='%23ffffff' stroke-width='2' stroke-linecap='round' stroke-linejoin='round'%3e%3cpolyline points='9 18 15 12 9 6'%3e%3c/polyline%3e%3c/svg%3e");
}

.carousel-control-next-icon {
  background-image: url("data:image/svg+xml,%3csvg xmlns='http://www.w3.org/2000/svg' width='24' height='24' viewBox='0 0 24 24' fill='none' stroke='%23ffffff' stroke-width='2' stroke-linecap='round' stroke-linejoin='round'%3e%3cpolyline points='15 18 9 12 15 6'%3e%3c/polyline%3e%3c/svg%3e");
}

.carousel-indicators {
  position: absolute;
  left: 0;
  bottom: 0;
  right: 0;
  z-index: 2;
  display: flex;
  justify-content: center;
  padding: 0;
  margin-left: 15%;
  margin-bottom: 1rem;
  margin-right: 15%;
}
.carousel-indicators [data-bs-target] {
  box-sizing: content-box;
  flex: 0 1 auto;
  width: 30px;
  height: 3px;
  padding: 0;
  margin-left: 3px;
  margin-right: 3px;
  text-indent: -999px;
  cursor: pointer;
  background-color: var(--tblr-carousel-indicator-active-bg);
  background-clip: padding-box;
  border: 0;
  border-top: 10px solid transparent;
  border-bottom: 10px solid transparent;
  opacity: 0.5;
  transition: opacity 0.6s ease;
}
@media (prefers-reduced-motion: reduce) {
  .carousel-indicators [data-bs-target] {
    transition: none;
  }
}
.carousel-indicators .active {
  opacity: 1;
}

.carousel-caption {
  position: absolute;
  left: 15%;
  bottom: 1.25rem;
  right: 15%;
  padding-top: 1.25rem;
  padding-bottom: 1.25rem;
  color: var(--tblr-carousel-caption-color);
  text-align: center;
}

.carousel-dark {
  --tblr-carousel-indicator-active-bg: #000000;
  --tblr-carousel-caption-color: #000000;
  --tblr-carousel-control-icon-filter: invert(1) grayscale(100);
}

:root,
[data-bs-theme=light] {
  --tblr-carousel-indicator-active-bg: #ffffff;
  --tblr-carousel-caption-color: #ffffff;
  --tblr-carousel-control-icon-filter: ;
}

[data-bs-theme=dark], body[data-bs-theme=dark] [data-bs-theme=light] {
  --tblr-carousel-indicator-active-bg: #000000;
  --tblr-carousel-caption-color: #000000;
  --tblr-carousel-control-icon-filter: invert(1) grayscale(100);
}

.spinner-grow,
.spinner-border {
  display: inline-block;
  width: var(--tblr-spinner-width);
  height: var(--tblr-spinner-height);
  vertical-align: var(--tblr-spinner-vertical-align);
  border-radius: 50%;
  animation: var(--tblr-spinner-animation-speed) linear infinite var(--tblr-spinner-animation-name);
}

@keyframes spinner-border {
  to {
    transform: rotate(360deg) ;
  }
}
.spinner-border {
  --tblr-spinner-width: 1.5rem;
  --tblr-spinner-height: 1.5rem;
  --tblr-spinner-vertical-align: -0.125em;
  --tblr-spinner-border-width: 2px;
  --tblr-spinner-animation-speed: 0.75s;
  --tblr-spinner-animation-name: spinner-border;
  border: var(--tblr-spinner-border-width) solid currentcolor;
  border-left-color: transparent;
}

.spinner-border-sm {
  --tblr-spinner-width: 1rem;
  --tblr-spinner-height: 1rem;
  --tblr-spinner-border-width: 1px;
}

@keyframes spinner-grow {
  0% {
    transform: scale(0);
  }
  50% {
    opacity: 1;
    transform: none;
  }
}
.spinner-grow {
  --tblr-spinner-width: 1.5rem;
  --tblr-spinner-height: 1.5rem;
  --tblr-spinner-vertical-align: -0.125em;
  --tblr-spinner-animation-speed: 0.75s;
  --tblr-spinner-animation-name: spinner-grow;
  background-color: currentcolor;
  opacity: 0;
}

.spinner-grow-sm {
  --tblr-spinner-width: 1rem;
  --tblr-spinner-height: 1rem;
}

@media (prefers-reduced-motion: reduce) {
  .spinner-border,
  .spinner-grow {
    --tblr-spinner-animation-speed: 1.5s;
  }
}
.offcanvas, .offcanvas-xxl, .offcanvas-xl, .offcanvas-lg, .offcanvas-md, .offcanvas-sm {
  --tblr-offcanvas-zindex: 1045;
  --tblr-offcanvas-width: 400px;
  --tblr-offcanvas-height: 30vh;
  --tblr-offcanvas-padding-x: 1.5rem;
  --tblr-offcanvas-padding-y: 1.5rem;
  --tblr-offcanvas-color: var(--tblr-body-color);
  --tblr-offcanvas-bg: var(--tblr-bg-surface);
  --tblr-offcanvas-border-width: var(--tblr-border-width);
  --tblr-offcanvas-border-color: var(--tblr-border-color);
  --tblr-offcanvas-box-shadow: var(--tblr-box-shadow-sm);
  --tblr-offcanvas-transition: transform 0.3s ease-in-out;
  --tblr-offcanvas-title-line-height: 1.4285714286;
}

@media (max-width: 575.98px) {
  .offcanvas-sm {
    position: fixed;
    bottom: 0;
    z-index: var(--tblr-offcanvas-zindex);
    display: flex;
    flex-direction: column;
    max-width: 100%;
    color: var(--tblr-offcanvas-color);
    visibility: hidden;
    background-color: var(--tblr-offcanvas-bg);
    background-clip: padding-box;
    outline: 0;
    box-shadow: var(--tblr-offcanvas-box-shadow);
    transition: var(--tblr-offcanvas-transition);
  }
}
@media (max-width: 575.98px) and (prefers-reduced-motion: reduce) {
  .offcanvas-sm {
    transition: none;
  }
}
@media (max-width: 575.98px) {
  .offcanvas-sm.offcanvas-start {
    top: 0;
    right: 0;
    width: var(--tblr-offcanvas-width);
    border-left: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateX(100%);
  }
  .offcanvas-sm.offcanvas-end {
    top: 0;
    left: 0;
    width: var(--tblr-offcanvas-width);
    border-right: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateX(-100%);
  }
  .offcanvas-sm.offcanvas-top {
    top: 0;
    left: 0;
    right: 0;
    height: var(--tblr-offcanvas-height);
    max-height: 100%;
    border-bottom: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateY(-100%);
  }
  .offcanvas-sm.offcanvas-bottom {
    left: 0;
    right: 0;
    height: var(--tblr-offcanvas-height);
    max-height: 100%;
    border-top: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateY(100%);
  }
  .offcanvas-sm.showing, .offcanvas-sm.show:not(.hiding) {
    transform: none;
  }
  .offcanvas-sm.showing, .offcanvas-sm.hiding, .offcanvas-sm.show {
    visibility: visible;
  }
}
@media (min-width: 576px) {
  .offcanvas-sm {
    --tblr-offcanvas-height: auto;
    --tblr-offcanvas-border-width: 0;
    background-color: transparent !important;
  }
  .offcanvas-sm .offcanvas-header {
    display: none;
  }
  .offcanvas-sm .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
    background-color: transparent !important;
  }
}

@media (max-width: 767.98px) {
  .offcanvas-md {
    position: fixed;
    bottom: 0;
    z-index: var(--tblr-offcanvas-zindex);
    display: flex;
    flex-direction: column;
    max-width: 100%;
    color: var(--tblr-offcanvas-color);
    visibility: hidden;
    background-color: var(--tblr-offcanvas-bg);
    background-clip: padding-box;
    outline: 0;
    box-shadow: var(--tblr-offcanvas-box-shadow);
    transition: var(--tblr-offcanvas-transition);
  }
}
@media (max-width: 767.98px) and (prefers-reduced-motion: reduce) {
  .offcanvas-md {
    transition: none;
  }
}
@media (max-width: 767.98px) {
  .offcanvas-md.offcanvas-start {
    top: 0;
    right: 0;
    width: var(--tblr-offcanvas-width);
    border-left: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateX(100%);
  }
  .offcanvas-md.offcanvas-end {
    top: 0;
    left: 0;
    width: var(--tblr-offcanvas-width);
    border-right: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateX(-100%);
  }
  .offcanvas-md.offcanvas-top {
    top: 0;
    left: 0;
    right: 0;
    height: var(--tblr-offcanvas-height);
    max-height: 100%;
    border-bottom: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateY(-100%);
  }
  .offcanvas-md.offcanvas-bottom {
    left: 0;
    right: 0;
    height: var(--tblr-offcanvas-height);
    max-height: 100%;
    border-top: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateY(100%);
  }
  .offcanvas-md.showing, .offcanvas-md.show:not(.hiding) {
    transform: none;
  }
  .offcanvas-md.showing, .offcanvas-md.hiding, .offcanvas-md.show {
    visibility: visible;
  }
}
@media (min-width: 768px) {
  .offcanvas-md {
    --tblr-offcanvas-height: auto;
    --tblr-offcanvas-border-width: 0;
    background-color: transparent !important;
  }
  .offcanvas-md .offcanvas-header {
    display: none;
  }
  .offcanvas-md .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
    background-color: transparent !important;
  }
}

@media (max-width: 991.98px) {
  .offcanvas-lg {
    position: fixed;
    bottom: 0;
    z-index: var(--tblr-offcanvas-zindex);
    display: flex;
    flex-direction: column;
    max-width: 100%;
    color: var(--tblr-offcanvas-color);
    visibility: hidden;
    background-color: var(--tblr-offcanvas-bg);
    background-clip: padding-box;
    outline: 0;
    box-shadow: var(--tblr-offcanvas-box-shadow);
    transition: var(--tblr-offcanvas-transition);
  }
}
@media (max-width: 991.98px) and (prefers-reduced-motion: reduce) {
  .offcanvas-lg {
    transition: none;
  }
}
@media (max-width: 991.98px) {
  .offcanvas-lg.offcanvas-start {
    top: 0;
    right: 0;
    width: var(--tblr-offcanvas-width);
    border-left: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateX(100%);
  }
  .offcanvas-lg.offcanvas-end {
    top: 0;
    left: 0;
    width: var(--tblr-offcanvas-width);
    border-right: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateX(-100%);
  }
  .offcanvas-lg.offcanvas-top {
    top: 0;
    left: 0;
    right: 0;
    height: var(--tblr-offcanvas-height);
    max-height: 100%;
    border-bottom: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateY(-100%);
  }
  .offcanvas-lg.offcanvas-bottom {
    left: 0;
    right: 0;
    height: var(--tblr-offcanvas-height);
    max-height: 100%;
    border-top: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateY(100%);
  }
  .offcanvas-lg.showing, .offcanvas-lg.show:not(.hiding) {
    transform: none;
  }
  .offcanvas-lg.showing, .offcanvas-lg.hiding, .offcanvas-lg.show {
    visibility: visible;
  }
}
@media (min-width: 992px) {
  .offcanvas-lg {
    --tblr-offcanvas-height: auto;
    --tblr-offcanvas-border-width: 0;
    background-color: transparent !important;
  }
  .offcanvas-lg .offcanvas-header {
    display: none;
  }
  .offcanvas-lg .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
    background-color: transparent !important;
  }
}

@media (max-width: 1199.98px) {
  .offcanvas-xl {
    position: fixed;
    bottom: 0;
    z-index: var(--tblr-offcanvas-zindex);
    display: flex;
    flex-direction: column;
    max-width: 100%;
    color: var(--tblr-offcanvas-color);
    visibility: hidden;
    background-color: var(--tblr-offcanvas-bg);
    background-clip: padding-box;
    outline: 0;
    box-shadow: var(--tblr-offcanvas-box-shadow);
    transition: var(--tblr-offcanvas-transition);
  }
}
@media (max-width: 1199.98px) and (prefers-reduced-motion: reduce) {
  .offcanvas-xl {
    transition: none;
  }
}
@media (max-width: 1199.98px) {
  .offcanvas-xl.offcanvas-start {
    top: 0;
    right: 0;
    width: var(--tblr-offcanvas-width);
    border-left: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateX(100%);
  }
  .offcanvas-xl.offcanvas-end {
    top: 0;
    left: 0;
    width: var(--tblr-offcanvas-width);
    border-right: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateX(-100%);
  }
  .offcanvas-xl.offcanvas-top {
    top: 0;
    left: 0;
    right: 0;
    height: var(--tblr-offcanvas-height);
    max-height: 100%;
    border-bottom: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateY(-100%);
  }
  .offcanvas-xl.offcanvas-bottom {
    left: 0;
    right: 0;
    height: var(--tblr-offcanvas-height);
    max-height: 100%;
    border-top: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateY(100%);
  }
  .offcanvas-xl.showing, .offcanvas-xl.show:not(.hiding) {
    transform: none;
  }
  .offcanvas-xl.showing, .offcanvas-xl.hiding, .offcanvas-xl.show {
    visibility: visible;
  }
}
@media (min-width: 1200px) {
  .offcanvas-xl {
    --tblr-offcanvas-height: auto;
    --tblr-offcanvas-border-width: 0;
    background-color: transparent !important;
  }
  .offcanvas-xl .offcanvas-header {
    display: none;
  }
  .offcanvas-xl .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
    background-color: transparent !important;
  }
}

@media (max-width: 1399.98px) {
  .offcanvas-xxl {
    position: fixed;
    bottom: 0;
    z-index: var(--tblr-offcanvas-zindex);
    display: flex;
    flex-direction: column;
    max-width: 100%;
    color: var(--tblr-offcanvas-color);
    visibility: hidden;
    background-color: var(--tblr-offcanvas-bg);
    background-clip: padding-box;
    outline: 0;
    box-shadow: var(--tblr-offcanvas-box-shadow);
    transition: var(--tblr-offcanvas-transition);
  }
}
@media (max-width: 1399.98px) and (prefers-reduced-motion: reduce) {
  .offcanvas-xxl {
    transition: none;
  }
}
@media (max-width: 1399.98px) {
  .offcanvas-xxl.offcanvas-start {
    top: 0;
    right: 0;
    width: var(--tblr-offcanvas-width);
    border-left: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateX(100%);
  }
  .offcanvas-xxl.offcanvas-end {
    top: 0;
    left: 0;
    width: var(--tblr-offcanvas-width);
    border-right: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateX(-100%);
  }
  .offcanvas-xxl.offcanvas-top {
    top: 0;
    left: 0;
    right: 0;
    height: var(--tblr-offcanvas-height);
    max-height: 100%;
    border-bottom: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateY(-100%);
  }
  .offcanvas-xxl.offcanvas-bottom {
    left: 0;
    right: 0;
    height: var(--tblr-offcanvas-height);
    max-height: 100%;
    border-top: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
    transform: translateY(100%);
  }
  .offcanvas-xxl.showing, .offcanvas-xxl.show:not(.hiding) {
    transform: none;
  }
  .offcanvas-xxl.showing, .offcanvas-xxl.hiding, .offcanvas-xxl.show {
    visibility: visible;
  }
}
@media (min-width: 1400px) {
  .offcanvas-xxl {
    --tblr-offcanvas-height: auto;
    --tblr-offcanvas-border-width: 0;
    background-color: transparent !important;
  }
  .offcanvas-xxl .offcanvas-header {
    display: none;
  }
  .offcanvas-xxl .offcanvas-body {
    display: flex;
    flex-grow: 0;
    padding: 0;
    overflow-y: visible;
    background-color: transparent !important;
  }
}

.offcanvas {
  position: fixed;
  bottom: 0;
  z-index: var(--tblr-offcanvas-zindex);
  display: flex;
  flex-direction: column;
  max-width: 100%;
  color: var(--tblr-offcanvas-color);
  visibility: hidden;
  background-color: var(--tblr-offcanvas-bg);
  background-clip: padding-box;
  outline: 0;
  box-shadow: var(--tblr-offcanvas-box-shadow);
  transition: var(--tblr-offcanvas-transition);
}
@media (prefers-reduced-motion: reduce) {
  .offcanvas {
    transition: none;
  }
}
.offcanvas.offcanvas-start {
  top: 0;
  right: 0;
  width: var(--tblr-offcanvas-width);
  border-left: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
  transform: translateX(100%);
}
.offcanvas.offcanvas-end {
  top: 0;
  left: 0;
  width: var(--tblr-offcanvas-width);
  border-right: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
  transform: translateX(-100%);
}
.offcanvas.offcanvas-top {
  top: 0;
  left: 0;
  right: 0;
  height: var(--tblr-offcanvas-height);
  max-height: 100%;
  border-bottom: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
  transform: translateY(-100%);
}
.offcanvas.offcanvas-bottom {
  left: 0;
  right: 0;
  height: var(--tblr-offcanvas-height);
  max-height: 100%;
  border-top: var(--tblr-offcanvas-border-width) solid var(--tblr-offcanvas-border-color);
  transform: translateY(100%);
}
.offcanvas.showing, .offcanvas.show:not(.hiding) {
  transform: none;
}
.offcanvas.showing, .offcanvas.hiding, .offcanvas.show {
  visibility: visible;
}

.offcanvas-backdrop {
  position: fixed;
  top: 0;
  right: 0;
  z-index: 1040;
  width: 100vw;
  height: 100vh;
  background-color: var(--tblr-gray-800);
}
.offcanvas-backdrop.fade {
  opacity: 0;
}
.offcanvas-backdrop.show {
  opacity: 0.24;
}

.offcanvas-header {
  display: flex;
  align-items: center;
  padding: var(--tblr-offcanvas-padding-y) var(--tblr-offcanvas-padding-x);
}
.offcanvas-header .btn-close {
  padding: calc(var(--tblr-offcanvas-padding-y) * 0.5) calc(var(--tblr-offcanvas-padding-x) * 0.5);
  margin-top: calc(-0.5 * var(--tblr-offcanvas-padding-y));
  margin-left: calc(-0.5 * var(--tblr-offcanvas-padding-x));
  margin-bottom: calc(-0.5 * var(--tblr-offcanvas-padding-y));
  margin-right: auto;
}

.offcanvas-title {
  margin-bottom: 0;
  line-height: var(--tblr-offcanvas-title-line-height);
}

.offcanvas-body {
  flex-grow: 1;
  padding: var(--tblr-offcanvas-padding-y) var(--tblr-offcanvas-padding-x);
  overflow-y: auto;
}

.placeholder {
  display: inline-block;
  min-height: 1em;
  vertical-align: middle;
  cursor: wait;
  background-color: currentcolor;
  opacity: 0.2;
}
.placeholder.btn::before {
  display: inline-block;
  content: "";
}

.placeholder-xs {
  min-height: 0.6em;
}

.placeholder-sm {
  min-height: 0.8em;
}

.placeholder-lg {
  min-height: 1.2em;
}

.placeholder-glow .placeholder {
  animation: placeholder-glow 2s ease-in-out infinite;
}

@keyframes placeholder-glow {
  50% {
    opacity: 0.1;
  }
}
.placeholder-wave {
  mask-image: linear-gradient(130deg, #000000 55%, rgba(0, 0, 0, 0.9) 75%, #000000 95%);
  mask-size: 200% 100%;
  animation: placeholder-wave 2s linear infinite;
}

@keyframes placeholder-wave {
  100% {
    mask-position: -200% 0%;
  }
}
.align-baseline {
  vertical-align: baseline !important;
}

.align-top {
  vertical-align: top !important;
}

.align-middle {
  vertical-align: middle !important;
}

.align-bottom {
  vertical-align: bottom !important;
}

.align-text-bottom {
  vertical-align: text-bottom !important;
}

.align-text-top {
  vertical-align: text-top !important;
}

.float-start {
  float: right !important;
}

.float-end {
  float: left !important;
}

.float-none {
  float: none !important;
}

.object-fit-contain {
  object-fit: contain !important;
}

.object-fit-cover {
  object-fit: cover !important;
}

.object-fit-fill {
  object-fit: fill !important;
}

.object-fit-scale {
  object-fit: scale-down !important;
}

.object-fit-none {
  object-fit: none !important;
}

.opacity-0 {
  opacity: 0 !important;
}

.opacity-25 {
  opacity: 0.25 !important;
}

.opacity-50 {
  opacity: 0.5 !important;
}

.opacity-75 {
  opacity: 0.75 !important;
}

.opacity-100 {
  opacity: 1 !important;
}

.overflow-auto {
  overflow: auto !important;
}

.overflow-hidden {
  overflow: hidden !important;
}

.overflow-visible {
  overflow: visible !important;
}

.overflow-scroll {
  overflow: scroll !important;
}

.overflow-x-auto {
  overflow-x: auto !important;
}

.overflow-x-hidden {
  overflow-x: hidden !important;
}

.overflow-x-visible {
  overflow-x: visible !important;
}

.overflow-x-scroll {
  overflow-x: scroll !important;
}

.overflow-y-auto {
  overflow-y: auto !important;
}

.overflow-y-hidden {
  overflow-y: hidden !important;
}

.overflow-y-visible {
  overflow-y: visible !important;
}

.overflow-y-scroll {
  overflow-y: scroll !important;
}

.d-inline {
  display: inline !important;
}

.d-inline-block {
  display: inline-block !important;
}

.d-block {
  display: block !important;
}

.d-grid {
  display: grid !important;
}

.d-inline-grid {
  display: inline-grid !important;
}

.d-table {
  display: table !important;
}

.d-table-row {
  display: table-row !important;
}

.d-table-cell {
  display: table-cell !important;
}

.d-flex {
  display: flex !important;
}

.d-inline-flex {
  display: inline-flex !important;
}

.d-none {
  display: none !important;
}

.shadow {
  box-shadow: var(--tblr-box-shadow) !important;
}

.shadow-sm {
  box-shadow: var(--tblr-box-shadow-sm) !important;
}

.shadow-lg {
  box-shadow: var(--tblr-box-shadow-lg) !important;
}

.shadow-none {
  box-shadow: none !important;
}

.focus-ring-primary {
  --tblr-focus-ring-color: rgba(var(--tblr-primary-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-secondary {
  --tblr-focus-ring-color: rgba(var(--tblr-secondary-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-success {
  --tblr-focus-ring-color: rgba(var(--tblr-success-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-info {
  --tblr-focus-ring-color: rgba(var(--tblr-info-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-warning {
  --tblr-focus-ring-color: rgba(var(--tblr-warning-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-danger {
  --tblr-focus-ring-color: rgba(var(--tblr-danger-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-light {
  --tblr-focus-ring-color: rgba(var(--tblr-light-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-dark {
  --tblr-focus-ring-color: rgba(var(--tblr-dark-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-muted {
  --tblr-focus-ring-color: rgba(var(--tblr-muted-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-blue {
  --tblr-focus-ring-color: rgba(var(--tblr-blue-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-azure {
  --tblr-focus-ring-color: rgba(var(--tblr-azure-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-indigo {
  --tblr-focus-ring-color: rgba(var(--tblr-indigo-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-purple {
  --tblr-focus-ring-color: rgba(var(--tblr-purple-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-pink {
  --tblr-focus-ring-color: rgba(var(--tblr-pink-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-red {
  --tblr-focus-ring-color: rgba(var(--tblr-red-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-orange {
  --tblr-focus-ring-color: rgba(var(--tblr-orange-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-yellow {
  --tblr-focus-ring-color: rgba(var(--tblr-yellow-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-lime {
  --tblr-focus-ring-color: rgba(var(--tblr-lime-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-green {
  --tblr-focus-ring-color: rgba(var(--tblr-green-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-teal {
  --tblr-focus-ring-color: rgba(var(--tblr-teal-rgb), var(--tblr-focus-ring-opacity));
}

.focus-ring-cyan {
  --tblr-focus-ring-color: rgba(var(--tblr-cyan-rgb), var(--tblr-focus-ring-opacity));
}

.position-static {
  position: static !important;
}

.position-relative {
  position: relative !important;
}

.position-absolute {
  position: absolute !important;
}

.position-fixed {
  position: fixed !important;
}

.position-sticky {
  position: sticky !important;
}

.top-0 {
  top: 0 !important;
}

.top-50 {
  top: 50% !important;
}

.top-100 {
  top: 100% !important;
}

.bottom-0 {
  bottom: 0 !important;
}

.bottom-50 {
  bottom: 50% !important;
}

.bottom-100 {
  bottom: 100% !important;
}

.start-0 {
  right: 0 !important;
}

.start-50 {
  right: 50% !important;
}

.start-100 {
  right: 100% !important;
}

.end-0 {
  left: 0 !important;
}

.end-50 {
  left: 50% !important;
}

.end-100 {
  left: 100% !important;
}

.translate-middle {
  transform: translate(50%, -50%) !important;
}

.translate-middle-x {
  transform: translateX(50%) !important;
}

.translate-middle-y {
  transform: translateY(-50%) !important;
}

.border {
  border: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}

.border-wide {
  border: 2px var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}

.border-0 {
  border: 0 !important;
}

.border-top {
  border-top: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}

.border-top-wide {
  border-top: 2px var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}

.border-top-0 {
  border-top: 0 !important;
}

.border-end {
  border-left: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}

.border-end-wide {
  border-left: 2px var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}

.border-end-0 {
  border-left: 0 !important;
}

.border-bottom {
  border-bottom: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}

.border-bottom-wide {
  border-bottom: 2px var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}

.border-bottom-0 {
  border-bottom: 0 !important;
}

.border-start {
  border-right: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}

.border-start-wide {
  border-right: 2px var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}

.border-start-0 {
  border-right: 0 !important;
}

.border-red {
  --tblr-border-opacity: 1;
  border-color: rgba(var(--tblr-red-rgb), var(--tblr-border-opacity)) !important;
}

.border-green {
  --tblr-border-opacity: 1;
  border-color: rgba(var(--tblr-green-rgb), var(--tblr-border-opacity)) !important;
}

.border-primary-subtle {
  border-color: var(--tblr-primary-border-subtle) !important;
}

.border-secondary-subtle {
  border-color: var(--tblr-secondary-border-subtle) !important;
}

.border-success-subtle {
  border-color: var(--tblr-success-border-subtle) !important;
}

.border-info-subtle {
  border-color: var(--tblr-info-border-subtle) !important;
}

.border-warning-subtle {
  border-color: var(--tblr-warning-border-subtle) !important;
}

.border-danger-subtle {
  border-color: var(--tblr-danger-border-subtle) !important;
}

.border-light-subtle {
  border-color: var(--tblr-light-border-subtle) !important;
}

.border-dark-subtle {
  border-color: var(--tblr-dark-border-subtle) !important;
}

.border-1 {
  border-width: 1px !important;
}

.border-2 {
  border-width: 2px !important;
}

.border-3 {
  border-width: 3px !important;
}

.border-4 {
  border-width: 4px !important;
}

.border-5 {
  border-width: 5px !important;
}

.border-opacity-10 {
  --tblr-border-opacity: 0.1;
}

.border-opacity-25 {
  --tblr-border-opacity: 0.25;
}

.border-opacity-50 {
  --tblr-border-opacity: 0.5;
}

.border-opacity-75 {
  --tblr-border-opacity: 0.75;
}

.border-opacity-100 {
  --tblr-border-opacity: 1;
}

.w-0 {
  width: 0 !important;
}

.w-1 {
  width: 0.25rem !important;
}

.w-2 {
  width: 0.5rem !important;
}

.w-3 {
  width: 1rem !important;
}

.w-4 {
  width: 1.5rem !important;
}

.w-5 {
  width: 2rem !important;
}

.w-6 {
  width: 2.5rem !important;
}

.w-25 {
  width: 25% !important;
}

.w-33 {
  width: 33.33333% !important;
}

.w-50 {
  width: 50% !important;
}

.w-66 {
  width: 66.66666% !important;
}

.w-75 {
  width: 75% !important;
}

.w-100 {
  width: 100% !important;
}

.w-auto {
  width: auto !important;
}

.mw-100 {
  max-width: 100% !important;
}

.vw-100 {
  width: 100vw !important;
}

.min-vw-100 {
  min-width: 100vw !important;
}

.h-0 {
  height: 0 !important;
}

.h-1 {
  height: 0.25rem !important;
}

.h-2 {
  height: 0.5rem !important;
}

.h-3 {
  height: 1rem !important;
}

.h-4 {
  height: 1.5rem !important;
}

.h-5 {
  height: 2rem !important;
}

.h-6 {
  height: 2.5rem !important;
}

.h-25 {
  height: 25% !important;
}

.h-33 {
  height: 33.33333% !important;
}

.h-50 {
  height: 50% !important;
}

.h-66 {
  height: 66.66666% !important;
}

.h-75 {
  height: 75% !important;
}

.h-100 {
  height: 100% !important;
}

.h-auto {
  height: auto !important;
}

.mh-100 {
  max-height: 100% !important;
}

.vh-100 {
  height: 100vh !important;
}

.min-vh-100 {
  min-height: 100vh !important;
}

.flex-fill {
  flex: 1 1 auto !important;
}

.flex-row {
  flex-direction: row !important;
}

.flex-column {
  flex-direction: column !important;
}

.flex-row-reverse {
  flex-direction: row-reverse !important;
}

.flex-column-reverse {
  flex-direction: column-reverse !important;
}

.flex-grow-0 {
  flex-grow: 0 !important;
}

.flex-grow-1 {
  flex-grow: 1 !important;
}

.flex-shrink-0 {
  flex-shrink: 0 !important;
}

.flex-shrink-1 {
  flex-shrink: 1 !important;
}

.flex-wrap {
  flex-wrap: wrap !important;
}

.flex-nowrap {
  flex-wrap: nowrap !important;
}

.flex-wrap-reverse {
  flex-wrap: wrap-reverse !important;
}

.justify-content-start {
  justify-content: flex-start !important;
}

.justify-content-end {
  justify-content: flex-end !important;
}

.justify-content-center {
  justify-content: center !important;
}

.justify-content-between {
  justify-content: space-between !important;
}

.justify-content-around {
  justify-content: space-around !important;
}

.justify-content-evenly {
  justify-content: space-evenly !important;
}

.align-items-start {
  align-items: flex-start !important;
}

.align-items-end {
  align-items: flex-end !important;
}

.align-items-center {
  align-items: center !important;
}

.align-items-baseline {
  align-items: baseline !important;
}

.align-items-stretch {
  align-items: stretch !important;
}

.align-content-start {
  align-content: flex-start !important;
}

.align-content-end {
  align-content: flex-end !important;
}

.align-content-center {
  align-content: center !important;
}

.align-content-between {
  align-content: space-between !important;
}

.align-content-around {
  align-content: space-around !important;
}

.align-content-stretch {
  align-content: stretch !important;
}

.align-self-auto {
  align-self: auto !important;
}

.align-self-start {
  align-self: flex-start !important;
}

.align-self-end {
  align-self: flex-end !important;
}

.align-self-center {
  align-self: center !important;
}

.align-self-baseline {
  align-self: baseline !important;
}

.align-self-stretch {
  align-self: stretch !important;
}

.order-first {
  order: -1 !important;
}

.order-0 {
  order: 0 !important;
}

.order-1 {
  order: 1 !important;
}

.order-2 {
  order: 2 !important;
}

.order-3 {
  order: 3 !important;
}

.order-4 {
  order: 4 !important;
}

.order-5 {
  order: 5 !important;
}

.order-last {
  order: 6 !important;
}

.m-0 {
  margin: 0 !important;
}

.m-1 {
  margin: 0.25rem !important;
}

.m-2 {
  margin: 0.5rem !important;
}

.m-3 {
  margin: 1rem !important;
}

.m-4 {
  margin: 1.5rem !important;
}

.m-5 {
  margin: 2rem !important;
}

.m-6 {
  margin: 2.5rem !important;
}

.m-auto {
  margin: auto !important;
}

.mx-0 {
  margin-left: 0 !important;
  margin-right: 0 !important;
}

.mx-1 {
  margin-left: 0.25rem !important;
  margin-right: 0.25rem !important;
}

.mx-2 {
  margin-left: 0.5rem !important;
  margin-right: 0.5rem !important;
}

.mx-3 {
  margin-left: 1rem !important;
  margin-right: 1rem !important;
}

.mx-4 {
  margin-left: 1.5rem !important;
  margin-right: 1.5rem !important;
}

.mx-5 {
  margin-left: 2rem !important;
  margin-right: 2rem !important;
}

.mx-6 {
  margin-left: 2.5rem !important;
  margin-right: 2.5rem !important;
}

.mx-auto {
  margin-left: auto !important;
  margin-right: auto !important;
}

.my-0 {
  margin-top: 0 !important;
  margin-bottom: 0 !important;
}

.my-1 {
  margin-top: 0.25rem !important;
  margin-bottom: 0.25rem !important;
}

.my-2 {
  margin-top: 0.5rem !important;
  margin-bottom: 0.5rem !important;
}

.my-3 {
  margin-top: 1rem !important;
  margin-bottom: 1rem !important;
}

.my-4 {
  margin-top: 1.5rem !important;
  margin-bottom: 1.5rem !important;
}

.my-5 {
  margin-top: 2rem !important;
  margin-bottom: 2rem !important;
}

.my-6 {
  margin-top: 2.5rem !important;
  margin-bottom: 2.5rem !important;
}

.my-auto {
  margin-top: auto !important;
  margin-bottom: auto !important;
}

.mt-0 {
  margin-top: 0 !important;
}

.mt-1 {
  margin-top: 0.25rem !important;
}

.mt-2 {
  margin-top: 0.5rem !important;
}

.mt-3 {
  margin-top: 1rem !important;
}

.mt-4 {
  margin-top: 1.5rem !important;
}

.mt-5 {
  margin-top: 2rem !important;
}

.mt-6 {
  margin-top: 2.5rem !important;
}

.mt-auto {
  margin-top: auto !important;
}

.me-0 {
  margin-left: 0 !important;
}

.me-1 {
  margin-left: 0.25rem !important;
}

.me-2 {
  margin-left: 0.5rem !important;
}

.me-3 {
  margin-left: 1rem !important;
}

.me-4 {
  margin-left: 1.5rem !important;
}

.me-5 {
  margin-left: 2rem !important;
}

.me-6 {
  margin-left: 2.5rem !important;
}

.me-auto {
  margin-left: auto !important;
}

.mb-0 {
  margin-bottom: 0 !important;
}

.mb-1 {
  margin-bottom: 0.25rem !important;
}

.mb-2 {
  margin-bottom: 0.5rem !important;
}

.mb-3 {
  margin-bottom: 1rem !important;
}

.mb-4 {
  margin-bottom: 1.5rem !important;
}

.mb-5 {
  margin-bottom: 2rem !important;
}

.mb-6 {
  margin-bottom: 2.5rem !important;
}

.mb-auto {
  margin-bottom: auto !important;
}

.ms-0 {
  margin-right: 0 !important;
}

.ms-1 {
  margin-right: 0.25rem !important;
}

.ms-2 {
  margin-right: 0.5rem !important;
}

.ms-3 {
  margin-right: 1rem !important;
}

.ms-4 {
  margin-right: 1.5rem !important;
}

.ms-5 {
  margin-right: 2rem !important;
}

.ms-6 {
  margin-right: 2.5rem !important;
}

.ms-auto {
  margin-right: auto !important;
}

.m-n1 {
  margin: -0.25rem !important;
}

.m-n2 {
  margin: -0.5rem !important;
}

.m-n3 {
  margin: -1rem !important;
}

.m-n4 {
  margin: -1.5rem !important;
}

.m-n5 {
  margin: -2rem !important;
}

.m-n6 {
  margin: -2.5rem !important;
}

.mx-n1 {
  margin-left: -0.25rem !important;
  margin-right: -0.25rem !important;
}

.mx-n2 {
  margin-left: -0.5rem !important;
  margin-right: -0.5rem !important;
}

.mx-n3 {
  margin-left: -1rem !important;
  margin-right: -1rem !important;
}

.mx-n4 {
  margin-left: -1.5rem !important;
  margin-right: -1.5rem !important;
}

.mx-n5 {
  margin-left: -2rem !important;
  margin-right: -2rem !important;
}

.mx-n6 {
  margin-left: -2.5rem !important;
  margin-right: -2.5rem !important;
}

.my-n1 {
  margin-top: -0.25rem !important;
  margin-bottom: -0.25rem !important;
}

.my-n2 {
  margin-top: -0.5rem !important;
  margin-bottom: -0.5rem !important;
}

.my-n3 {
  margin-top: -1rem !important;
  margin-bottom: -1rem !important;
}

.my-n4 {
  margin-top: -1.5rem !important;
  margin-bottom: -1.5rem !important;
}

.my-n5 {
  margin-top: -2rem !important;
  margin-bottom: -2rem !important;
}

.my-n6 {
  margin-top: -2.5rem !important;
  margin-bottom: -2.5rem !important;
}

.mt-n1 {
  margin-top: -0.25rem !important;
}

.mt-n2 {
  margin-top: -0.5rem !important;
}

.mt-n3 {
  margin-top: -1rem !important;
}

.mt-n4 {
  margin-top: -1.5rem !important;
}

.mt-n5 {
  margin-top: -2rem !important;
}

.mt-n6 {
  margin-top: -2.5rem !important;
}

.me-n1 {
  margin-left: -0.25rem !important;
}

.me-n2 {
  margin-left: -0.5rem !important;
}

.me-n3 {
  margin-left: -1rem !important;
}

.me-n4 {
  margin-left: -1.5rem !important;
}

.me-n5 {
  margin-left: -2rem !important;
}

.me-n6 {
  margin-left: -2.5rem !important;
}

.mb-n1 {
  margin-bottom: -0.25rem !important;
}

.mb-n2 {
  margin-bottom: -0.5rem !important;
}

.mb-n3 {
  margin-bottom: -1rem !important;
}

.mb-n4 {
  margin-bottom: -1.5rem !important;
}

.mb-n5 {
  margin-bottom: -2rem !important;
}

.mb-n6 {
  margin-bottom: -2.5rem !important;
}

.ms-n1 {
  margin-right: -0.25rem !important;
}

.ms-n2 {
  margin-right: -0.5rem !important;
}

.ms-n3 {
  margin-right: -1rem !important;
}

.ms-n4 {
  margin-right: -1.5rem !important;
}

.ms-n5 {
  margin-right: -2rem !important;
}

.ms-n6 {
  margin-right: -2.5rem !important;
}

.p-0 {
  padding: 0 !important;
}

.p-1 {
  padding: 0.25rem !important;
}

.p-2 {
  padding: 0.5rem !important;
}

.p-3 {
  padding: 1rem !important;
}

.p-4 {
  padding: 1.5rem !important;
}

.p-5 {
  padding: 2rem !important;
}

.p-6 {
  padding: 2.5rem !important;
}

.px-0 {
  padding-left: 0 !important;
  padding-right: 0 !important;
}

.px-1 {
  padding-left: 0.25rem !important;
  padding-right: 0.25rem !important;
}

.px-2 {
  padding-left: 0.5rem !important;
  padding-right: 0.5rem !important;
}

.px-3 {
  padding-left: 1rem !important;
  padding-right: 1rem !important;
}

.px-4 {
  padding-left: 1.5rem !important;
  padding-right: 1.5rem !important;
}

.px-5 {
  padding-left: 2rem !important;
  padding-right: 2rem !important;
}

.px-6 {
  padding-left: 2.5rem !important;
  padding-right: 2.5rem !important;
}

.py-0 {
  padding-top: 0 !important;
  padding-bottom: 0 !important;
}

.py-1 {
  padding-top: 0.25rem !important;
  padding-bottom: 0.25rem !important;
}

.py-2 {
  padding-top: 0.5rem !important;
  padding-bottom: 0.5rem !important;
}

.py-3 {
  padding-top: 1rem !important;
  padding-bottom: 1rem !important;
}

.py-4 {
  padding-top: 1.5rem !important;
  padding-bottom: 1.5rem !important;
}

.py-5 {
  padding-top: 2rem !important;
  padding-bottom: 2rem !important;
}

.py-6 {
  padding-top: 2.5rem !important;
  padding-bottom: 2.5rem !important;
}

.pt-0 {
  padding-top: 0 !important;
}

.pt-1 {
  padding-top: 0.25rem !important;
}

.pt-2 {
  padding-top: 0.5rem !important;
}

.pt-3 {
  padding-top: 1rem !important;
}

.pt-4 {
  padding-top: 1.5rem !important;
}

.pt-5 {
  padding-top: 2rem !important;
}

.pt-6 {
  padding-top: 2.5rem !important;
}

.pe-0 {
  padding-left: 0 !important;
}

.pe-1 {
  padding-left: 0.25rem !important;
}

.pe-2 {
  padding-left: 0.5rem !important;
}

.pe-3 {
  padding-left: 1rem !important;
}

.pe-4 {
  padding-left: 1.5rem !important;
}

.pe-5 {
  padding-left: 2rem !important;
}

.pe-6 {
  padding-left: 2.5rem !important;
}

.pb-0 {
  padding-bottom: 0 !important;
}

.pb-1 {
  padding-bottom: 0.25rem !important;
}

.pb-2 {
  padding-bottom: 0.5rem !important;
}

.pb-3 {
  padding-bottom: 1rem !important;
}

.pb-4 {
  padding-bottom: 1.5rem !important;
}

.pb-5 {
  padding-bottom: 2rem !important;
}

.pb-6 {
  padding-bottom: 2.5rem !important;
}

.ps-0 {
  padding-right: 0 !important;
}

.ps-1 {
  padding-right: 0.25rem !important;
}

.ps-2 {
  padding-right: 0.5rem !important;
}

.ps-3 {
  padding-right: 1rem !important;
}

.ps-4 {
  padding-right: 1.5rem !important;
}

.ps-5 {
  padding-right: 2rem !important;
}

.ps-6 {
  padding-right: 2.5rem !important;
}

.gap-0 {
  gap: 0 !important;
}

.gap-1 {
  gap: 0.25rem !important;
}

.gap-2 {
  gap: 0.5rem !important;
}

.gap-3 {
  gap: 1rem !important;
}

.gap-4 {
  gap: 1.5rem !important;
}

.gap-5 {
  gap: 2rem !important;
}

.gap-6 {
  gap: 2.5rem !important;
}

.row-gap-0 {
  row-gap: 0 !important;
}

.row-gap-1 {
  row-gap: 0.25rem !important;
}

.row-gap-2 {
  row-gap: 0.5rem !important;
}

.row-gap-3 {
  row-gap: 1rem !important;
}

.row-gap-4 {
  row-gap: 1.5rem !important;
}

.row-gap-5 {
  row-gap: 2rem !important;
}

.row-gap-6 {
  row-gap: 2.5rem !important;
}

.column-gap-0 {
  column-gap: 0 !important;
}

.column-gap-1 {
  column-gap: 0.25rem !important;
}

.column-gap-2 {
  column-gap: 0.5rem !important;
}

.column-gap-3 {
  column-gap: 1rem !important;
}

.column-gap-4 {
  column-gap: 1.5rem !important;
}

.column-gap-5 {
  column-gap: 2rem !important;
}

.column-gap-6 {
  column-gap: 2.5rem !important;
}

.font-monospace {
  font-family: var(--tblr-font-monospace) !important;
}

.fs-1 {
  font-size: 1.5rem !important;
}

.fs-2 {
  font-size: 1.25rem !important;
}

.fs-3 {
  font-size: 1rem !important;
}

.fs-4 {
  font-size: 0.875rem !important;
}

.fs-5 {
  font-size: 0.75rem !important;
}

.fs-6 {
  font-size: 0.625rem !important;
}

.fst-italic {
  font-style: italic !important;
}

.fst-normal {
  font-style: normal !important;
}

.fw-lighter {
  font-weight: lighter !important;
}

.fw-light {
  font-weight: 300 !important;
}

.fw-normal {
  font-weight: 400 !important;
}

.fw-medium {
  font-weight: 500 !important;
}

.fw-semibold {
  font-weight: 600 !important;
}

.fw-bold {
  font-weight: 600 !important;
}

.fw-bolder {
  font-weight: bolder !important;
}

.lh-1 {
  line-height: 1 !important;
}

.lh-sm {
  line-height: 1.1428571429 !important;
}

.lh-base {
  line-height: 1.4285714286 !important;
}

.lh-lg {
  line-height: 1.7142857143 !important;
}

.text-start {
  text-align: right !important;
}

.text-end {
  text-align: left !important;
}

.text-center {
  text-align: center !important;
}

.text-decoration-none {
  text-decoration: none !important;
}

.text-decoration-underline {
  text-decoration: underline !important;
}

.text-decoration-line-through {
  text-decoration: line-through !important;
}

.text-lowercase {
  text-transform: lowercase !important;
}

.text-uppercase {
  text-transform: uppercase !important;
}

.text-capitalize {
  text-transform: capitalize !important;
}

.text-wrap {
  white-space: normal !important;
}

.text-nowrap {
  white-space: nowrap !important;
}
.text-primary {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-primary-rgb), var(--tblr-text-opacity)) !important;
}

.text-secondary {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-secondary-rgb), var(--tblr-text-opacity)) !important;
}

.text-success {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-success-rgb), var(--tblr-text-opacity)) !important;
}

.text-info {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-info-rgb), var(--tblr-text-opacity)) !important;
}

.text-warning {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-warning-rgb), var(--tblr-text-opacity)) !important;
}

.text-danger {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-danger-rgb), var(--tblr-text-opacity)) !important;
}

.text-light {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-light-rgb), var(--tblr-text-opacity)) !important;
}

.text-dark {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-dark-rgb), var(--tblr-text-opacity)) !important;
}

.text-muted {
  --tblr-text-opacity: 1;
  color: var(--tblr-secondary-color) !important;
}

.text-blue {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-blue-rgb), var(--tblr-text-opacity)) !important;
}

.text-azure {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-azure-rgb), var(--tblr-text-opacity)) !important;
}

.text-indigo {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-indigo-rgb), var(--tblr-text-opacity)) !important;
}

.text-purple {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-purple-rgb), var(--tblr-text-opacity)) !important;
}

.text-pink {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-pink-rgb), var(--tblr-text-opacity)) !important;
}

.text-red {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-red-rgb), var(--tblr-text-opacity)) !important;
}

.text-orange {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-orange-rgb), var(--tblr-text-opacity)) !important;
}

.text-yellow {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-yellow-rgb), var(--tblr-text-opacity)) !important;
}

.text-lime {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-lime-rgb), var(--tblr-text-opacity)) !important;
}

.text-green {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-green-rgb), var(--tblr-text-opacity)) !important;
}

.text-teal {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-teal-rgb), var(--tblr-text-opacity)) !important;
}

.text-cyan {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-cyan-rgb), var(--tblr-text-opacity)) !important;
}

.text-black {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-black-rgb), var(--tblr-text-opacity)) !important;
}

.text-white {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-white-rgb), var(--tblr-text-opacity)) !important;
}

.text-body {
  --tblr-text-opacity: 1;
  color: rgba(var(--tblr-body-color-rgb), var(--tblr-text-opacity)) !important;
}

.text-black-50 {
  --tblr-text-opacity: 1;
  color: rgba(0, 0, 0, 0.5) !important;
}

.text-white-50 {
  --tblr-text-opacity: 1;
  color: rgba(255, 255, 255, 0.5) !important;
}

.text-body-secondary {
  --tblr-text-opacity: 1;
  color: var(--tblr-secondary-color) !important;
}

.text-body-tertiary {
  --tblr-text-opacity: 1;
  color: var(--tblr-tertiary-color) !important;
}

.text-body-emphasis {
  --tblr-text-opacity: 1;
  color: var(--tblr-emphasis-color) !important;
}

.text-reset {
  --tblr-text-opacity: 1;
  color: inherit !important;
}

.text-opacity-25 {
  --tblr-text-opacity: 0.25;
}

.text-opacity-50 {
  --tblr-text-opacity: 0.5;
}

.text-opacity-75 {
  --tblr-text-opacity: 0.75;
}

.text-opacity-100 {
  --tblr-text-opacity: 1;
}

.text-primary-emphasis {
  color: var(--tblr-primary-text-emphasis) !important;
}

.text-secondary-emphasis {
  color: var(--tblr-secondary-text-emphasis) !important;
}

.text-success-emphasis {
  color: var(--tblr-success-text-emphasis) !important;
}

.text-info-emphasis {
  color: var(--tblr-info-text-emphasis) !important;
}

.text-warning-emphasis {
  color: var(--tblr-warning-text-emphasis) !important;
}

.text-danger-emphasis {
  color: var(--tblr-danger-text-emphasis) !important;
}

.text-light-emphasis {
  color: var(--tblr-light-text-emphasis) !important;
}

.text-dark-emphasis {
  color: var(--tblr-dark-text-emphasis) !important;
}

.link-opacity-10 {
  --tblr-link-opacity: 0.1;
}

.link-opacity-10-hover:hover {
  --tblr-link-opacity: 0.1;
}

.link-opacity-25 {
  --tblr-link-opacity: 0.25;
}

.link-opacity-25-hover:hover {
  --tblr-link-opacity: 0.25;
}

.link-opacity-50 {
  --tblr-link-opacity: 0.5;
}

.link-opacity-50-hover:hover {
  --tblr-link-opacity: 0.5;
}

.link-opacity-75 {
  --tblr-link-opacity: 0.75;
}

.link-opacity-75-hover:hover {
  --tblr-link-opacity: 0.75;
}

.link-opacity-100 {
  --tblr-link-opacity: 1;
}

.link-opacity-100-hover:hover {
  --tblr-link-opacity: 1;
}

.link-offset-1 {
  text-underline-offset: 0.125em !important;
}

.link-offset-1-hover:hover {
  text-underline-offset: 0.125em !important;
}

.link-offset-2 {
  text-underline-offset: 0.25em !important;
}

.link-offset-2-hover:hover {
  text-underline-offset: 0.25em !important;
}

.link-offset-3 {
  text-underline-offset: 0.375em !important;
}

.link-offset-3-hover:hover {
  text-underline-offset: 0.375em !important;
}

.link-underline-primary {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-primary-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-secondary {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-secondary-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-success {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-success-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-info {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-info-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-warning {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-warning-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-danger {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-danger-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-light {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-light-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-dark {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-dark-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-muted {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-muted-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-blue {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-blue-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-azure {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-azure-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-indigo {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-indigo-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-purple {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-purple-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-pink {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-pink-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-red {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-red-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-orange {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-orange-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-yellow {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-yellow-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-lime {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-lime-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-green {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-green-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-teal {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-teal-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline-cyan {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-cyan-rgb), var(--tblr-link-underline-opacity)) !important;
}

.link-underline {
  --tblr-link-underline-opacity: 1;
  text-decoration-color: rgba(var(--tblr-link-color-rgb), var(--tblr-link-underline-opacity, 1)) !important;
}

.link-underline-opacity-0 {
  --tblr-link-underline-opacity: 0;
}

.link-underline-opacity-0-hover:hover {
  --tblr-link-underline-opacity: 0;
}

.link-underline-opacity-10 {
  --tblr-link-underline-opacity: 0.1;
}

.link-underline-opacity-10-hover:hover {
  --tblr-link-underline-opacity: 0.1;
}

.link-underline-opacity-25 {
  --tblr-link-underline-opacity: 0.25;
}

.link-underline-opacity-25-hover:hover {
  --tblr-link-underline-opacity: 0.25;
}

.link-underline-opacity-50 {
  --tblr-link-underline-opacity: 0.5;
}

.link-underline-opacity-50-hover:hover {
  --tblr-link-underline-opacity: 0.5;
}

.link-underline-opacity-75 {
  --tblr-link-underline-opacity: 0.75;
}

.link-underline-opacity-75-hover:hover {
  --tblr-link-underline-opacity: 0.75;
}

.link-underline-opacity-100 {
  --tblr-link-underline-opacity: 1;
}

.link-underline-opacity-100-hover:hover {
  --tblr-link-underline-opacity: 1;
}

.bg-primary {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-primary-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-secondary {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-secondary-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-success {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-success-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-info {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-info-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-warning {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-warning-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-danger {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-danger-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-light {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-light-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-dark {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-dark-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-muted {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-muted-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-blue {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-blue-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-azure {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-azure-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-indigo {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-indigo-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-purple {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-purple-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-pink {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-pink-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-red {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-red-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-orange {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-orange-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-yellow {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-yellow-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-lime {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-lime-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-green {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-green-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-teal {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-teal-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-cyan {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-cyan-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-black {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-black-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-white {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-white-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-body {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-body-bg-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-transparent {
  --tblr-bg-opacity: 1;
  background-color: transparent !important;
}

.bg-body-secondary {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-secondary-bg-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-body-tertiary {
  --tblr-bg-opacity: 1;
  background-color: rgba(var(--tblr-tertiary-bg-rgb), var(--tblr-bg-opacity)) !important;
}

.bg-opacity-10 {
  --tblr-bg-opacity: 0.1;
}

.bg-opacity-25 {
  --tblr-bg-opacity: 0.25;
}

.bg-opacity-50 {
  --tblr-bg-opacity: 0.5;
}

.bg-opacity-75 {
  --tblr-bg-opacity: 0.75;
}

.bg-opacity-100 {
  --tblr-bg-opacity: 1;
}

.bg-primary-subtle {
  background-color: var(--tblr-primary-bg-subtle) !important;
}

.bg-secondary-subtle {
  background-color: var(--tblr-secondary-bg-subtle) !important;
}

.bg-success-subtle {
  background-color: var(--tblr-success-bg-subtle) !important;
}

.bg-info-subtle {
  background-color: var(--tblr-info-bg-subtle) !important;
}

.bg-warning-subtle {
  background-color: var(--tblr-warning-bg-subtle) !important;
}

.bg-danger-subtle {
  background-color: var(--tblr-danger-bg-subtle) !important;
}

.bg-light-subtle {
  background-color: var(--tblr-light-bg-subtle) !important;
}

.bg-dark-subtle {
  background-color: var(--tblr-dark-bg-subtle) !important;
}

.bg-gradient {
  background-image: var(--tblr-gradient) !important;
}

.user-select-all {
  user-select: all !important;
}

.user-select-auto {
  user-select: auto !important;
}

.user-select-none {
  user-select: none !important;
}

.pe-none {
  pointer-events: none !important;
}

.pe-auto {
  pointer-events: auto !important;
}

.rounded {
  border-radius: var(--tblr-border-radius) !important;
}

.rounded-0 {
  border-radius: 0 !important;
}

.rounded-1 {
  border-radius: var(--tblr-border-radius-sm) !important;
}

.rounded-2 {
  border-radius: var(--tblr-border-radius) !important;
}

.rounded-3 {
  border-radius: var(--tblr-border-radius-lg) !important;
}

.rounded-4 {
  border-radius: var(--tblr-border-radius-xl) !important;
}

.rounded-5 {
  border-radius: var(--tblr-border-radius-xxl) !important;
}

.rounded-circle {
  border-radius: 50% !important;
}

.rounded-pill {
  border-radius: var(--tblr-border-radius-pill) !important;
}

.rounded-top {
  border-top-right-radius: var(--tblr-border-radius) !important;
  border-top-left-radius: var(--tblr-border-radius) !important;
}

.rounded-top-0 {
  border-top-right-radius: 0 !important;
  border-top-left-radius: 0 !important;
}

.rounded-top-1 {
  border-top-right-radius: var(--tblr-border-radius-sm) !important;
  border-top-left-radius: var(--tblr-border-radius-sm) !important;
}

.rounded-top-2 {
  border-top-right-radius: var(--tblr-border-radius) !important;
  border-top-left-radius: var(--tblr-border-radius) !important;
}

.rounded-top-3 {
  border-top-right-radius: var(--tblr-border-radius-lg) !important;
  border-top-left-radius: var(--tblr-border-radius-lg) !important;
}

.rounded-top-4 {
  border-top-right-radius: var(--tblr-border-radius-xl) !important;
  border-top-left-radius: var(--tblr-border-radius-xl) !important;
}

.rounded-top-5 {
  border-top-right-radius: var(--tblr-border-radius-xxl) !important;
  border-top-left-radius: var(--tblr-border-radius-xxl) !important;
}

.rounded-top-circle {
  border-top-right-radius: 50% !important;
  border-top-left-radius: 50% !important;
}

.rounded-top-pill {
  border-top-right-radius: var(--tblr-border-radius-pill) !important;
  border-top-left-radius: var(--tblr-border-radius-pill) !important;
}

.rounded-end {
  border-top-left-radius: var(--tblr-border-radius) !important;
  border-bottom-left-radius: var(--tblr-border-radius) !important;
}

.rounded-end-0 {
  border-top-left-radius: 0 !important;
  border-bottom-left-radius: 0 !important;
}

.rounded-end-1 {
  border-top-left-radius: var(--tblr-border-radius-sm) !important;
  border-bottom-left-radius: var(--tblr-border-radius-sm) !important;
}

.rounded-end-2 {
  border-top-left-radius: var(--tblr-border-radius) !important;
  border-bottom-left-radius: var(--tblr-border-radius) !important;
}

.rounded-end-3 {
  border-top-left-radius: var(--tblr-border-radius-lg) !important;
  border-bottom-left-radius: var(--tblr-border-radius-lg) !important;
}

.rounded-end-4 {
  border-top-left-radius: var(--tblr-border-radius-xl) !important;
  border-bottom-left-radius: var(--tblr-border-radius-xl) !important;
}

.rounded-end-5 {
  border-top-left-radius: var(--tblr-border-radius-xxl) !important;
  border-bottom-left-radius: var(--tblr-border-radius-xxl) !important;
}

.rounded-end-circle {
  border-top-left-radius: 50% !important;
  border-bottom-left-radius: 50% !important;
}

.rounded-end-pill {
  border-top-left-radius: var(--tblr-border-radius-pill) !important;
  border-bottom-left-radius: var(--tblr-border-radius-pill) !important;
}

.rounded-bottom {
  border-bottom-left-radius: var(--tblr-border-radius) !important;
  border-bottom-right-radius: var(--tblr-border-radius) !important;
}

.rounded-bottom-0 {
  border-bottom-left-radius: 0 !important;
  border-bottom-right-radius: 0 !important;
}

.rounded-bottom-1 {
  border-bottom-left-radius: var(--tblr-border-radius-sm) !important;
  border-bottom-right-radius: var(--tblr-border-radius-sm) !important;
}

.rounded-bottom-2 {
  border-bottom-left-radius: var(--tblr-border-radius) !important;
  border-bottom-right-radius: var(--tblr-border-radius) !important;
}

.rounded-bottom-3 {
  border-bottom-left-radius: var(--tblr-border-radius-lg) !important;
  border-bottom-right-radius: var(--tblr-border-radius-lg) !important;
}

.rounded-bottom-4 {
  border-bottom-left-radius: var(--tblr-border-radius-xl) !important;
  border-bottom-right-radius: var(--tblr-border-radius-xl) !important;
}

.rounded-bottom-5 {
  border-bottom-left-radius: var(--tblr-border-radius-xxl) !important;
  border-bottom-right-radius: var(--tblr-border-radius-xxl) !important;
}

.rounded-bottom-circle {
  border-bottom-left-radius: 50% !important;
  border-bottom-right-radius: 50% !important;
}

.rounded-bottom-pill {
  border-bottom-left-radius: var(--tblr-border-radius-pill) !important;
  border-bottom-right-radius: var(--tblr-border-radius-pill) !important;
}

.rounded-start {
  border-bottom-right-radius: var(--tblr-border-radius) !important;
  border-top-right-radius: var(--tblr-border-radius) !important;
}

.rounded-start-0 {
  border-bottom-right-radius: 0 !important;
  border-top-right-radius: 0 !important;
}

.rounded-start-1 {
  border-bottom-right-radius: var(--tblr-border-radius-sm) !important;
  border-top-right-radius: var(--tblr-border-radius-sm) !important;
}

.rounded-start-2 {
  border-bottom-right-radius: var(--tblr-border-radius) !important;
  border-top-right-radius: var(--tblr-border-radius) !important;
}

.rounded-start-3 {
  border-bottom-right-radius: var(--tblr-border-radius-lg) !important;
  border-top-right-radius: var(--tblr-border-radius-lg) !important;
}

.rounded-start-4 {
  border-bottom-right-radius: var(--tblr-border-radius-xl) !important;
  border-top-right-radius: var(--tblr-border-radius-xl) !important;
}

.rounded-start-5 {
  border-bottom-right-radius: var(--tblr-border-radius-xxl) !important;
  border-top-right-radius: var(--tblr-border-radius-xxl) !important;
}

.rounded-start-circle {
  border-bottom-right-radius: 50% !important;
  border-top-right-radius: 50% !important;
}

.rounded-start-pill {
  border-bottom-right-radius: var(--tblr-border-radius-pill) !important;
  border-top-right-radius: var(--tblr-border-radius-pill) !important;
}

.visible {
  visibility: visible !important;
}

.invisible {
  visibility: hidden !important;
}

.z-n1 {
  z-index: -1 !important;
}

.z-0 {
  z-index: 0 !important;
}

.z-1 {
  z-index: 1 !important;
}

.z-2 {
  z-index: 2 !important;
}

.z-3 {
  z-index: 3 !important;
}

.object-contain {
  object-fit: contain !important;
}

.object-cover {
  object-fit: cover !important;
}

.object-fill {
  object-fit: fill !important;
}

.object-scale-down {
  object-fit: scale-down !important;
}

.object-none {
  object-fit: none !important;
}

.cursor-auto {
  cursor: auto !important;
}

.cursor-pointer {
  cursor: pointer !important;
}

.cursor-move {
  cursor: move !important;
}

.cursor-not-allowed {
  cursor: not-allowed !important;
}

.cursor-zoom-in {
  cursor: zoom-in !important;
}

.cursor-zoom-out {
  cursor: zoom-out !important;
}

.cursor-default {
  cursor: default !important;
}

.cursor-none {
  cursor: none !important;
}

.cursor-help {
  cursor: help !important;
}

.cursor-progress {
  cursor: progress !important;
}

.cursor-wait {
  cursor: wait !important;
}

.cursor-text {
  cursor: text !important;
}

.cursor-v-text {
  cursor: vertical-text !important;
}

.cursor-grab {
  cursor: grab !important;
}

.cursor-grabbing {
  cursor: grabbing !important;
}

.cursor-crosshair {
  cursor: crosshair !important;
}

.border-x {
  border-right: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
  border-left: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}

.border-x-wide {
  border-right: 2px var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
  border-left: 2px var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}

.border-x-0 {
  border-right: 0 !important;
  border-left: 0 !important;
}

.border-y {
  border-top: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
  border-bottom: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}

.border-y-wide {
  border-top: 2px var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
  border-bottom: 2px var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}

.border-y-0 {
  border-top: 0 !important;
  border-bottom: 0 !important;
}

.columns-2 {
  columns: 2 !important;
}

.columns-3 {
  columns: 3 !important;
}

.columns-4 {
  columns: 4 !important;
}

.bg-pattern-transparent {
  background: url("data:image/svg+xml;charset=UTF-8,<svg xmlns=\"http://www.w3.org/2000/svg\" width=\"16\" height=\"16\" viewBox=\"0 0 16 16\"><rect x=\"0\" y=\"0\" width=\"8\" height=\"8\" fill=\"rgba%28130, 130, 130, .1%29\" /><rect x=\"8\" y=\"8\" width=\"8\" height=\"8\" fill=\"rgba%28130, 130, 130, .1%29\" /></svg>") repeat center/16px 16px !important;
}

.bg-gradient {
  background: linear-gradient(var(--tblr-gradient-direction, to right), var(--tblr-gradient-stops, var(--tblr-gradient-from, transparent), var(--tblr-gradient-to, transparent))) no-repeat !important;
}

.bg-gradient-to-t {
  --tblr-gradient-direction: to top !important;
}

.bg-gradient-to-te {
  --tblr-gradient-direction: to top right !important;
}

.bg-gradient-to-e {
  --tblr-gradient-direction: to right !important;
}

.bg-gradient-to-be {
  --tblr-gradient-direction: to bottom right !important;
}

.bg-gradient-to-b {
  --tblr-gradient-direction: to bottom !important;
}

.bg-gradient-to-bs {
  --tblr-gradient-direction: to bottom left !important;
}

.bg-gradient-to-s {
  --tblr-gradient-direction: to left !important;
}

.bg-gradient-to-ts {
  --tblr-gradient-direction: to top left !important;
}

.table-auto {
  table-layout: auto !important;
}

.table-fixed {
  table-layout: fixed !important;
}

@media (min-width: 576px) {
  .float-sm-start {
    float: right !important;
  }
  .float-sm-end {
    float: left !important;
  }
  .float-sm-none {
    float: none !important;
  }
  .object-fit-sm-contain {
    object-fit: contain !important;
  }
  .object-fit-sm-cover {
    object-fit: cover !important;
  }
  .object-fit-sm-fill {
    object-fit: fill !important;
  }
  .object-fit-sm-scale {
    object-fit: scale-down !important;
  }
  .object-fit-sm-none {
    object-fit: none !important;
  }
  .d-sm-inline {
    display: inline !important;
  }
  .d-sm-inline-block {
    display: inline-block !important;
  }
  .d-sm-block {
    display: block !important;
  }
  .d-sm-grid {
    display: grid !important;
  }
  .d-sm-inline-grid {
    display: inline-grid !important;
  }
  .d-sm-table {
    display: table !important;
  }
  .d-sm-table-row {
    display: table-row !important;
  }
  .d-sm-table-cell {
    display: table-cell !important;
  }
  .d-sm-flex {
    display: flex !important;
  }
  .d-sm-inline-flex {
    display: inline-flex !important;
  }
  .d-sm-none {
    display: none !important;
  }
  .flex-sm-fill {
    flex: 1 1 auto !important;
  }
  .flex-sm-row {
    flex-direction: row !important;
  }
  .flex-sm-column {
    flex-direction: column !important;
  }
  .flex-sm-row-reverse {
    flex-direction: row-reverse !important;
  }
  .flex-sm-column-reverse {
    flex-direction: column-reverse !important;
  }
  .flex-sm-grow-0 {
    flex-grow: 0 !important;
  }
  .flex-sm-grow-1 {
    flex-grow: 1 !important;
  }
  .flex-sm-shrink-0 {
    flex-shrink: 0 !important;
  }
  .flex-sm-shrink-1 {
    flex-shrink: 1 !important;
  }
  .flex-sm-wrap {
    flex-wrap: wrap !important;
  }
  .flex-sm-nowrap {
    flex-wrap: nowrap !important;
  }
  .flex-sm-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }
  .justify-content-sm-start {
    justify-content: flex-start !important;
  }
  .justify-content-sm-end {
    justify-content: flex-end !important;
  }
  .justify-content-sm-center {
    justify-content: center !important;
  }
  .justify-content-sm-between {
    justify-content: space-between !important;
  }
  .justify-content-sm-around {
    justify-content: space-around !important;
  }
  .justify-content-sm-evenly {
    justify-content: space-evenly !important;
  }
  .align-items-sm-start {
    align-items: flex-start !important;
  }
  .align-items-sm-end {
    align-items: flex-end !important;
  }
  .align-items-sm-center {
    align-items: center !important;
  }
  .align-items-sm-baseline {
    align-items: baseline !important;
  }
  .align-items-sm-stretch {
    align-items: stretch !important;
  }
  .align-content-sm-start {
    align-content: flex-start !important;
  }
  .align-content-sm-end {
    align-content: flex-end !important;
  }
  .align-content-sm-center {
    align-content: center !important;
  }
  .align-content-sm-between {
    align-content: space-between !important;
  }
  .align-content-sm-around {
    align-content: space-around !important;
  }
  .align-content-sm-stretch {
    align-content: stretch !important;
  }
  .align-self-sm-auto {
    align-self: auto !important;
  }
  .align-self-sm-start {
    align-self: flex-start !important;
  }
  .align-self-sm-end {
    align-self: flex-end !important;
  }
  .align-self-sm-center {
    align-self: center !important;
  }
  .align-self-sm-baseline {
    align-self: baseline !important;
  }
  .align-self-sm-stretch {
    align-self: stretch !important;
  }
  .order-sm-first {
    order: -1 !important;
  }
  .order-sm-0 {
    order: 0 !important;
  }
  .order-sm-1 {
    order: 1 !important;
  }
  .order-sm-2 {
    order: 2 !important;
  }
  .order-sm-3 {
    order: 3 !important;
  }
  .order-sm-4 {
    order: 4 !important;
  }
  .order-sm-5 {
    order: 5 !important;
  }
  .order-sm-last {
    order: 6 !important;
  }
  .m-sm-0 {
    margin: 0 !important;
  }
  .m-sm-1 {
    margin: 0.25rem !important;
  }
  .m-sm-2 {
    margin: 0.5rem !important;
  }
  .m-sm-3 {
    margin: 1rem !important;
  }
  .m-sm-4 {
    margin: 1.5rem !important;
  }
  .m-sm-5 {
    margin: 2rem !important;
  }
  .m-sm-6 {
    margin: 2.5rem !important;
  }
  .m-sm-auto {
    margin: auto !important;
  }
  .mx-sm-0 {
    margin-left: 0 !important;
    margin-right: 0 !important;
  }
  .mx-sm-1 {
    margin-left: 0.25rem !important;
    margin-right: 0.25rem !important;
  }
  .mx-sm-2 {
    margin-left: 0.5rem !important;
    margin-right: 0.5rem !important;
  }
  .mx-sm-3 {
    margin-left: 1rem !important;
    margin-right: 1rem !important;
  }
  .mx-sm-4 {
    margin-left: 1.5rem !important;
    margin-right: 1.5rem !important;
  }
  .mx-sm-5 {
    margin-left: 2rem !important;
    margin-right: 2rem !important;
  }
  .mx-sm-6 {
    margin-left: 2.5rem !important;
    margin-right: 2.5rem !important;
  }
  .mx-sm-auto {
    margin-left: auto !important;
    margin-right: auto !important;
  }
  .my-sm-0 {
    margin-top: 0 !important;
    margin-bottom: 0 !important;
  }
  .my-sm-1 {
    margin-top: 0.25rem !important;
    margin-bottom: 0.25rem !important;
  }
  .my-sm-2 {
    margin-top: 0.5rem !important;
    margin-bottom: 0.5rem !important;
  }
  .my-sm-3 {
    margin-top: 1rem !important;
    margin-bottom: 1rem !important;
  }
  .my-sm-4 {
    margin-top: 1.5rem !important;
    margin-bottom: 1.5rem !important;
  }
  .my-sm-5 {
    margin-top: 2rem !important;
    margin-bottom: 2rem !important;
  }
  .my-sm-6 {
    margin-top: 2.5rem !important;
    margin-bottom: 2.5rem !important;
  }
  .my-sm-auto {
    margin-top: auto !important;
    margin-bottom: auto !important;
  }
  .mt-sm-0 {
    margin-top: 0 !important;
  }
  .mt-sm-1 {
    margin-top: 0.25rem !important;
  }
  .mt-sm-2 {
    margin-top: 0.5rem !important;
  }
  .mt-sm-3 {
    margin-top: 1rem !important;
  }
  .mt-sm-4 {
    margin-top: 1.5rem !important;
  }
  .mt-sm-5 {
    margin-top: 2rem !important;
  }
  .mt-sm-6 {
    margin-top: 2.5rem !important;
  }
  .mt-sm-auto {
    margin-top: auto !important;
  }
  .me-sm-0 {
    margin-left: 0 !important;
  }
  .me-sm-1 {
    margin-left: 0.25rem !important;
  }
  .me-sm-2 {
    margin-left: 0.5rem !important;
  }
  .me-sm-3 {
    margin-left: 1rem !important;
  }
  .me-sm-4 {
    margin-left: 1.5rem !important;
  }
  .me-sm-5 {
    margin-left: 2rem !important;
  }
  .me-sm-6 {
    margin-left: 2.5rem !important;
  }
  .me-sm-auto {
    margin-left: auto !important;
  }
  .mb-sm-0 {
    margin-bottom: 0 !important;
  }
  .mb-sm-1 {
    margin-bottom: 0.25rem !important;
  }
  .mb-sm-2 {
    margin-bottom: 0.5rem !important;
  }
  .mb-sm-3 {
    margin-bottom: 1rem !important;
  }
  .mb-sm-4 {
    margin-bottom: 1.5rem !important;
  }
  .mb-sm-5 {
    margin-bottom: 2rem !important;
  }
  .mb-sm-6 {
    margin-bottom: 2.5rem !important;
  }
  .mb-sm-auto {
    margin-bottom: auto !important;
  }
  .ms-sm-0 {
    margin-right: 0 !important;
  }
  .ms-sm-1 {
    margin-right: 0.25rem !important;
  }
  .ms-sm-2 {
    margin-right: 0.5rem !important;
  }
  .ms-sm-3 {
    margin-right: 1rem !important;
  }
  .ms-sm-4 {
    margin-right: 1.5rem !important;
  }
  .ms-sm-5 {
    margin-right: 2rem !important;
  }
  .ms-sm-6 {
    margin-right: 2.5rem !important;
  }
  .ms-sm-auto {
    margin-right: auto !important;
  }
  .m-sm-n1 {
    margin: -0.25rem !important;
  }
  .m-sm-n2 {
    margin: -0.5rem !important;
  }
  .m-sm-n3 {
    margin: -1rem !important;
  }
  .m-sm-n4 {
    margin: -1.5rem !important;
  }
  .m-sm-n5 {
    margin: -2rem !important;
  }
  .m-sm-n6 {
    margin: -2.5rem !important;
  }
  .mx-sm-n1 {
    margin-left: -0.25rem !important;
    margin-right: -0.25rem !important;
  }
  .mx-sm-n2 {
    margin-left: -0.5rem !important;
    margin-right: -0.5rem !important;
  }
  .mx-sm-n3 {
    margin-left: -1rem !important;
    margin-right: -1rem !important;
  }
  .mx-sm-n4 {
    margin-left: -1.5rem !important;
    margin-right: -1.5rem !important;
  }
  .mx-sm-n5 {
    margin-left: -2rem !important;
    margin-right: -2rem !important;
  }
  .mx-sm-n6 {
    margin-left: -2.5rem !important;
    margin-right: -2.5rem !important;
  }
  .my-sm-n1 {
    margin-top: -0.25rem !important;
    margin-bottom: -0.25rem !important;
  }
  .my-sm-n2 {
    margin-top: -0.5rem !important;
    margin-bottom: -0.5rem !important;
  }
  .my-sm-n3 {
    margin-top: -1rem !important;
    margin-bottom: -1rem !important;
  }
  .my-sm-n4 {
    margin-top: -1.5rem !important;
    margin-bottom: -1.5rem !important;
  }
  .my-sm-n5 {
    margin-top: -2rem !important;
    margin-bottom: -2rem !important;
  }
  .my-sm-n6 {
    margin-top: -2.5rem !important;
    margin-bottom: -2.5rem !important;
  }
  .mt-sm-n1 {
    margin-top: -0.25rem !important;
  }
  .mt-sm-n2 {
    margin-top: -0.5rem !important;
  }
  .mt-sm-n3 {
    margin-top: -1rem !important;
  }
  .mt-sm-n4 {
    margin-top: -1.5rem !important;
  }
  .mt-sm-n5 {
    margin-top: -2rem !important;
  }
  .mt-sm-n6 {
    margin-top: -2.5rem !important;
  }
  .me-sm-n1 {
    margin-left: -0.25rem !important;
  }
  .me-sm-n2 {
    margin-left: -0.5rem !important;
  }
  .me-sm-n3 {
    margin-left: -1rem !important;
  }
  .me-sm-n4 {
    margin-left: -1.5rem !important;
  }
  .me-sm-n5 {
    margin-left: -2rem !important;
  }
  .me-sm-n6 {
    margin-left: -2.5rem !important;
  }
  .mb-sm-n1 {
    margin-bottom: -0.25rem !important;
  }
  .mb-sm-n2 {
    margin-bottom: -0.5rem !important;
  }
  .mb-sm-n3 {
    margin-bottom: -1rem !important;
  }
  .mb-sm-n4 {
    margin-bottom: -1.5rem !important;
  }
  .mb-sm-n5 {
    margin-bottom: -2rem !important;
  }
  .mb-sm-n6 {
    margin-bottom: -2.5rem !important;
  }
  .ms-sm-n1 {
    margin-right: -0.25rem !important;
  }
  .ms-sm-n2 {
    margin-right: -0.5rem !important;
  }
  .ms-sm-n3 {
    margin-right: -1rem !important;
  }
  .ms-sm-n4 {
    margin-right: -1.5rem !important;
  }
  .ms-sm-n5 {
    margin-right: -2rem !important;
  }
  .ms-sm-n6 {
    margin-right: -2.5rem !important;
  }
  .p-sm-0 {
    padding: 0 !important;
  }
  .p-sm-1 {
    padding: 0.25rem !important;
  }
  .p-sm-2 {
    padding: 0.5rem !important;
  }
  .p-sm-3 {
    padding: 1rem !important;
  }
  .p-sm-4 {
    padding: 1.5rem !important;
  }
  .p-sm-5 {
    padding: 2rem !important;
  }
  .p-sm-6 {
    padding: 2.5rem !important;
  }
  .px-sm-0 {
    padding-left: 0 !important;
    padding-right: 0 !important;
  }
  .px-sm-1 {
    padding-left: 0.25rem !important;
    padding-right: 0.25rem !important;
  }
  .px-sm-2 {
    padding-left: 0.5rem !important;
    padding-right: 0.5rem !important;
  }
  .px-sm-3 {
    padding-left: 1rem !important;
    padding-right: 1rem !important;
  }
  .px-sm-4 {
    padding-left: 1.5rem !important;
    padding-right: 1.5rem !important;
  }
  .px-sm-5 {
    padding-left: 2rem !important;
    padding-right: 2rem !important;
  }
  .px-sm-6 {
    padding-left: 2.5rem !important;
    padding-right: 2.5rem !important;
  }
  .py-sm-0 {
    padding-top: 0 !important;
    padding-bottom: 0 !important;
  }
  .py-sm-1 {
    padding-top: 0.25rem !important;
    padding-bottom: 0.25rem !important;
  }
  .py-sm-2 {
    padding-top: 0.5rem !important;
    padding-bottom: 0.5rem !important;
  }
  .py-sm-3 {
    padding-top: 1rem !important;
    padding-bottom: 1rem !important;
  }
  .py-sm-4 {
    padding-top: 1.5rem !important;
    padding-bottom: 1.5rem !important;
  }
  .py-sm-5 {
    padding-top: 2rem !important;
    padding-bottom: 2rem !important;
  }
  .py-sm-6 {
    padding-top: 2.5rem !important;
    padding-bottom: 2.5rem !important;
  }
  .pt-sm-0 {
    padding-top: 0 !important;
  }
  .pt-sm-1 {
    padding-top: 0.25rem !important;
  }
  .pt-sm-2 {
    padding-top: 0.5rem !important;
  }
  .pt-sm-3 {
    padding-top: 1rem !important;
  }
  .pt-sm-4 {
    padding-top: 1.5rem !important;
  }
  .pt-sm-5 {
    padding-top: 2rem !important;
  }
  .pt-sm-6 {
    padding-top: 2.5rem !important;
  }
  .pe-sm-0 {
    padding-left: 0 !important;
  }
  .pe-sm-1 {
    padding-left: 0.25rem !important;
  }
  .pe-sm-2 {
    padding-left: 0.5rem !important;
  }
  .pe-sm-3 {
    padding-left: 1rem !important;
  }
  .pe-sm-4 {
    padding-left: 1.5rem !important;
  }
  .pe-sm-5 {
    padding-left: 2rem !important;
  }
  .pe-sm-6 {
    padding-left: 2.5rem !important;
  }
  .pb-sm-0 {
    padding-bottom: 0 !important;
  }
  .pb-sm-1 {
    padding-bottom: 0.25rem !important;
  }
  .pb-sm-2 {
    padding-bottom: 0.5rem !important;
  }
  .pb-sm-3 {
    padding-bottom: 1rem !important;
  }
  .pb-sm-4 {
    padding-bottom: 1.5rem !important;
  }
  .pb-sm-5 {
    padding-bottom: 2rem !important;
  }
  .pb-sm-6 {
    padding-bottom: 2.5rem !important;
  }
  .ps-sm-0 {
    padding-right: 0 !important;
  }
  .ps-sm-1 {
    padding-right: 0.25rem !important;
  }
  .ps-sm-2 {
    padding-right: 0.5rem !important;
  }
  .ps-sm-3 {
    padding-right: 1rem !important;
  }
  .ps-sm-4 {
    padding-right: 1.5rem !important;
  }
  .ps-sm-5 {
    padding-right: 2rem !important;
  }
  .ps-sm-6 {
    padding-right: 2.5rem !important;
  }
  .gap-sm-0 {
    gap: 0 !important;
  }
  .gap-sm-1 {
    gap: 0.25rem !important;
  }
  .gap-sm-2 {
    gap: 0.5rem !important;
  }
  .gap-sm-3 {
    gap: 1rem !important;
  }
  .gap-sm-4 {
    gap: 1.5rem !important;
  }
  .gap-sm-5 {
    gap: 2rem !important;
  }
  .gap-sm-6 {
    gap: 2.5rem !important;
  }
  .row-gap-sm-0 {
    row-gap: 0 !important;
  }
  .row-gap-sm-1 {
    row-gap: 0.25rem !important;
  }
  .row-gap-sm-2 {
    row-gap: 0.5rem !important;
  }
  .row-gap-sm-3 {
    row-gap: 1rem !important;
  }
  .row-gap-sm-4 {
    row-gap: 1.5rem !important;
  }
  .row-gap-sm-5 {
    row-gap: 2rem !important;
  }
  .row-gap-sm-6 {
    row-gap: 2.5rem !important;
  }
  .column-gap-sm-0 {
    column-gap: 0 !important;
  }
  .column-gap-sm-1 {
    column-gap: 0.25rem !important;
  }
  .column-gap-sm-2 {
    column-gap: 0.5rem !important;
  }
  .column-gap-sm-3 {
    column-gap: 1rem !important;
  }
  .column-gap-sm-4 {
    column-gap: 1.5rem !important;
  }
  .column-gap-sm-5 {
    column-gap: 2rem !important;
  }
  .column-gap-sm-6 {
    column-gap: 2.5rem !important;
  }
  .text-sm-start {
    text-align: right !important;
  }
  .text-sm-end {
    text-align: left !important;
  }
  .text-sm-center {
    text-align: center !important;
  }
  .columns-sm-2 {
    columns: 2 !important;
  }
  .columns-sm-3 {
    columns: 3 !important;
  }
  .columns-sm-4 {
    columns: 4 !important;
  }
}
@media (min-width: 768px) {
  .float-md-start {
    float: right !important;
  }
  .float-md-end {
    float: left !important;
  }
  .float-md-none {
    float: none !important;
  }
  .object-fit-md-contain {
    object-fit: contain !important;
  }
  .object-fit-md-cover {
    object-fit: cover !important;
  }
  .object-fit-md-fill {
    object-fit: fill !important;
  }
  .object-fit-md-scale {
    object-fit: scale-down !important;
  }
  .object-fit-md-none {
    object-fit: none !important;
  }
  .d-md-inline {
    display: inline !important;
  }
  .d-md-inline-block {
    display: inline-block !important;
  }
  .d-md-block {
    display: block !important;
  }
  .d-md-grid {
    display: grid !important;
  }
  .d-md-inline-grid {
    display: inline-grid !important;
  }
  .d-md-table {
    display: table !important;
  }
  .d-md-table-row {
    display: table-row !important;
  }
  .d-md-table-cell {
    display: table-cell !important;
  }
  .d-md-flex {
    display: flex !important;
  }
  .d-md-inline-flex {
    display: inline-flex !important;
  }
  .d-md-none {
    display: none !important;
  }
  .flex-md-fill {
    flex: 1 1 auto !important;
  }
  .flex-md-row {
    flex-direction: row !important;
  }
  .flex-md-column {
    flex-direction: column !important;
  }
  .flex-md-row-reverse {
    flex-direction: row-reverse !important;
  }
  .flex-md-column-reverse {
    flex-direction: column-reverse !important;
  }
  .flex-md-grow-0 {
    flex-grow: 0 !important;
  }
  .flex-md-grow-1 {
    flex-grow: 1 !important;
  }
  .flex-md-shrink-0 {
    flex-shrink: 0 !important;
  }
  .flex-md-shrink-1 {
    flex-shrink: 1 !important;
  }
  .flex-md-wrap {
    flex-wrap: wrap !important;
  }
  .flex-md-nowrap {
    flex-wrap: nowrap !important;
  }
  .flex-md-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }
  .justify-content-md-start {
    justify-content: flex-start !important;
  }
  .justify-content-md-end {
    justify-content: flex-end !important;
  }
  .justify-content-md-center {
    justify-content: center !important;
  }
  .justify-content-md-between {
    justify-content: space-between !important;
  }
  .justify-content-md-around {
    justify-content: space-around !important;
  }
  .justify-content-md-evenly {
    justify-content: space-evenly !important;
  }
  .align-items-md-start {
    align-items: flex-start !important;
  }
  .align-items-md-end {
    align-items: flex-end !important;
  }
  .align-items-md-center {
    align-items: center !important;
  }
  .align-items-md-baseline {
    align-items: baseline !important;
  }
  .align-items-md-stretch {
    align-items: stretch !important;
  }
  .align-content-md-start {
    align-content: flex-start !important;
  }
  .align-content-md-end {
    align-content: flex-end !important;
  }
  .align-content-md-center {
    align-content: center !important;
  }
  .align-content-md-between {
    align-content: space-between !important;
  }
  .align-content-md-around {
    align-content: space-around !important;
  }
  .align-content-md-stretch {
    align-content: stretch !important;
  }
  .align-self-md-auto {
    align-self: auto !important;
  }
  .align-self-md-start {
    align-self: flex-start !important;
  }
  .align-self-md-end {
    align-self: flex-end !important;
  }
  .align-self-md-center {
    align-self: center !important;
  }
  .align-self-md-baseline {
    align-self: baseline !important;
  }
  .align-self-md-stretch {
    align-self: stretch !important;
  }
  .order-md-first {
    order: -1 !important;
  }
  .order-md-0 {
    order: 0 !important;
  }
  .order-md-1 {
    order: 1 !important;
  }
  .order-md-2 {
    order: 2 !important;
  }
  .order-md-3 {
    order: 3 !important;
  }
  .order-md-4 {
    order: 4 !important;
  }
  .order-md-5 {
    order: 5 !important;
  }
  .order-md-last {
    order: 6 !important;
  }
  .m-md-0 {
    margin: 0 !important;
  }
  .m-md-1 {
    margin: 0.25rem !important;
  }
  .m-md-2 {
    margin: 0.5rem !important;
  }
  .m-md-3 {
    margin: 1rem !important;
  }
  .m-md-4 {
    margin: 1.5rem !important;
  }
  .m-md-5 {
    margin: 2rem !important;
  }
  .m-md-6 {
    margin: 2.5rem !important;
  }
  .m-md-auto {
    margin: auto !important;
  }
  .mx-md-0 {
    margin-left: 0 !important;
    margin-right: 0 !important;
  }
  .mx-md-1 {
    margin-left: 0.25rem !important;
    margin-right: 0.25rem !important;
  }
  .mx-md-2 {
    margin-left: 0.5rem !important;
    margin-right: 0.5rem !important;
  }
  .mx-md-3 {
    margin-left: 1rem !important;
    margin-right: 1rem !important;
  }
  .mx-md-4 {
    margin-left: 1.5rem !important;
    margin-right: 1.5rem !important;
  }
  .mx-md-5 {
    margin-left: 2rem !important;
    margin-right: 2rem !important;
  }
  .mx-md-6 {
    margin-left: 2.5rem !important;
    margin-right: 2.5rem !important;
  }
  .mx-md-auto {
    margin-left: auto !important;
    margin-right: auto !important;
  }
  .my-md-0 {
    margin-top: 0 !important;
    margin-bottom: 0 !important;
  }
  .my-md-1 {
    margin-top: 0.25rem !important;
    margin-bottom: 0.25rem !important;
  }
  .my-md-2 {
    margin-top: 0.5rem !important;
    margin-bottom: 0.5rem !important;
  }
  .my-md-3 {
    margin-top: 1rem !important;
    margin-bottom: 1rem !important;
  }
  .my-md-4 {
    margin-top: 1.5rem !important;
    margin-bottom: 1.5rem !important;
  }
  .my-md-5 {
    margin-top: 2rem !important;
    margin-bottom: 2rem !important;
  }
  .my-md-6 {
    margin-top: 2.5rem !important;
    margin-bottom: 2.5rem !important;
  }
  .my-md-auto {
    margin-top: auto !important;
    margin-bottom: auto !important;
  }
  .mt-md-0 {
    margin-top: 0 !important;
  }
  .mt-md-1 {
    margin-top: 0.25rem !important;
  }
  .mt-md-2 {
    margin-top: 0.5rem !important;
  }
  .mt-md-3 {
    margin-top: 1rem !important;
  }
  .mt-md-4 {
    margin-top: 1.5rem !important;
  }
  .mt-md-5 {
    margin-top: 2rem !important;
  }
  .mt-md-6 {
    margin-top: 2.5rem !important;
  }
  .mt-md-auto {
    margin-top: auto !important;
  }
  .me-md-0 {
    margin-left: 0 !important;
  }
  .me-md-1 {
    margin-left: 0.25rem !important;
  }
  .me-md-2 {
    margin-left: 0.5rem !important;
  }
  .me-md-3 {
    margin-left: 1rem !important;
  }
  .me-md-4 {
    margin-left: 1.5rem !important;
  }
  .me-md-5 {
    margin-left: 2rem !important;
  }
  .me-md-6 {
    margin-left: 2.5rem !important;
  }
  .me-md-auto {
    margin-left: auto !important;
  }
  .mb-md-0 {
    margin-bottom: 0 !important;
  }
  .mb-md-1 {
    margin-bottom: 0.25rem !important;
  }
  .mb-md-2 {
    margin-bottom: 0.5rem !important;
  }
  .mb-md-3 {
    margin-bottom: 1rem !important;
  }
  .mb-md-4 {
    margin-bottom: 1.5rem !important;
  }
  .mb-md-5 {
    margin-bottom: 2rem !important;
  }
  .mb-md-6 {
    margin-bottom: 2.5rem !important;
  }
  .mb-md-auto {
    margin-bottom: auto !important;
  }
  .ms-md-0 {
    margin-right: 0 !important;
  }
  .ms-md-1 {
    margin-right: 0.25rem !important;
  }
  .ms-md-2 {
    margin-right: 0.5rem !important;
  }
  .ms-md-3 {
    margin-right: 1rem !important;
  }
  .ms-md-4 {
    margin-right: 1.5rem !important;
  }
  .ms-md-5 {
    margin-right: 2rem !important;
  }
  .ms-md-6 {
    margin-right: 2.5rem !important;
  }
  .ms-md-auto {
    margin-right: auto !important;
  }
  .m-md-n1 {
    margin: -0.25rem !important;
  }
  .m-md-n2 {
    margin: -0.5rem !important;
  }
  .m-md-n3 {
    margin: -1rem !important;
  }
  .m-md-n4 {
    margin: -1.5rem !important;
  }
  .m-md-n5 {
    margin: -2rem !important;
  }
  .m-md-n6 {
    margin: -2.5rem !important;
  }
  .mx-md-n1 {
    margin-left: -0.25rem !important;
    margin-right: -0.25rem !important;
  }
  .mx-md-n2 {
    margin-left: -0.5rem !important;
    margin-right: -0.5rem !important;
  }
  .mx-md-n3 {
    margin-left: -1rem !important;
    margin-right: -1rem !important;
  }
  .mx-md-n4 {
    margin-left: -1.5rem !important;
    margin-right: -1.5rem !important;
  }
  .mx-md-n5 {
    margin-left: -2rem !important;
    margin-right: -2rem !important;
  }
  .mx-md-n6 {
    margin-left: -2.5rem !important;
    margin-right: -2.5rem !important;
  }
  .my-md-n1 {
    margin-top: -0.25rem !important;
    margin-bottom: -0.25rem !important;
  }
  .my-md-n2 {
    margin-top: -0.5rem !important;
    margin-bottom: -0.5rem !important;
  }
  .my-md-n3 {
    margin-top: -1rem !important;
    margin-bottom: -1rem !important;
  }
  .my-md-n4 {
    margin-top: -1.5rem !important;
    margin-bottom: -1.5rem !important;
  }
  .my-md-n5 {
    margin-top: -2rem !important;
    margin-bottom: -2rem !important;
  }
  .my-md-n6 {
    margin-top: -2.5rem !important;
    margin-bottom: -2.5rem !important;
  }
  .mt-md-n1 {
    margin-top: -0.25rem !important;
  }
  .mt-md-n2 {
    margin-top: -0.5rem !important;
  }
  .mt-md-n3 {
    margin-top: -1rem !important;
  }
  .mt-md-n4 {
    margin-top: -1.5rem !important;
  }
  .mt-md-n5 {
    margin-top: -2rem !important;
  }
  .mt-md-n6 {
    margin-top: -2.5rem !important;
  }
  .me-md-n1 {
    margin-left: -0.25rem !important;
  }
  .me-md-n2 {
    margin-left: -0.5rem !important;
  }
  .me-md-n3 {
    margin-left: -1rem !important;
  }
  .me-md-n4 {
    margin-left: -1.5rem !important;
  }
  .me-md-n5 {
    margin-left: -2rem !important;
  }
  .me-md-n6 {
    margin-left: -2.5rem !important;
  }
  .mb-md-n1 {
    margin-bottom: -0.25rem !important;
  }
  .mb-md-n2 {
    margin-bottom: -0.5rem !important;
  }
  .mb-md-n3 {
    margin-bottom: -1rem !important;
  }
  .mb-md-n4 {
    margin-bottom: -1.5rem !important;
  }
  .mb-md-n5 {
    margin-bottom: -2rem !important;
  }
  .mb-md-n6 {
    margin-bottom: -2.5rem !important;
  }
  .ms-md-n1 {
    margin-right: -0.25rem !important;
  }
  .ms-md-n2 {
    margin-right: -0.5rem !important;
  }
  .ms-md-n3 {
    margin-right: -1rem !important;
  }
  .ms-md-n4 {
    margin-right: -1.5rem !important;
  }
  .ms-md-n5 {
    margin-right: -2rem !important;
  }
  .ms-md-n6 {
    margin-right: -2.5rem !important;
  }
  .p-md-0 {
    padding: 0 !important;
  }
  .p-md-1 {
    padding: 0.25rem !important;
  }
  .p-md-2 {
    padding: 0.5rem !important;
  }
  .p-md-3 {
    padding: 1rem !important;
  }
  .p-md-4 {
    padding: 1.5rem !important;
  }
  .p-md-5 {
    padding: 2rem !important;
  }
  .p-md-6 {
    padding: 2.5rem !important;
  }
  .px-md-0 {
    padding-left: 0 !important;
    padding-right: 0 !important;
  }
  .px-md-1 {
    padding-left: 0.25rem !important;
    padding-right: 0.25rem !important;
  }
  .px-md-2 {
    padding-left: 0.5rem !important;
    padding-right: 0.5rem !important;
  }
  .px-md-3 {
    padding-left: 1rem !important;
    padding-right: 1rem !important;
  }
  .px-md-4 {
    padding-left: 1.5rem !important;
    padding-right: 1.5rem !important;
  }
  .px-md-5 {
    padding-left: 2rem !important;
    padding-right: 2rem !important;
  }
  .px-md-6 {
    padding-left: 2.5rem !important;
    padding-right: 2.5rem !important;
  }
  .py-md-0 {
    padding-top: 0 !important;
    padding-bottom: 0 !important;
  }
  .py-md-1 {
    padding-top: 0.25rem !important;
    padding-bottom: 0.25rem !important;
  }
  .py-md-2 {
    padding-top: 0.5rem !important;
    padding-bottom: 0.5rem !important;
  }
  .py-md-3 {
    padding-top: 1rem !important;
    padding-bottom: 1rem !important;
  }
  .py-md-4 {
    padding-top: 1.5rem !important;
    padding-bottom: 1.5rem !important;
  }
  .py-md-5 {
    padding-top: 2rem !important;
    padding-bottom: 2rem !important;
  }
  .py-md-6 {
    padding-top: 2.5rem !important;
    padding-bottom: 2.5rem !important;
  }
  .pt-md-0 {
    padding-top: 0 !important;
  }
  .pt-md-1 {
    padding-top: 0.25rem !important;
  }
  .pt-md-2 {
    padding-top: 0.5rem !important;
  }
  .pt-md-3 {
    padding-top: 1rem !important;
  }
  .pt-md-4 {
    padding-top: 1.5rem !important;
  }
  .pt-md-5 {
    padding-top: 2rem !important;
  }
  .pt-md-6 {
    padding-top: 2.5rem !important;
  }
  .pe-md-0 {
    padding-left: 0 !important;
  }
  .pe-md-1 {
    padding-left: 0.25rem !important;
  }
  .pe-md-2 {
    padding-left: 0.5rem !important;
  }
  .pe-md-3 {
    padding-left: 1rem !important;
  }
  .pe-md-4 {
    padding-left: 1.5rem !important;
  }
  .pe-md-5 {
    padding-left: 2rem !important;
  }
  .pe-md-6 {
    padding-left: 2.5rem !important;
  }
  .pb-md-0 {
    padding-bottom: 0 !important;
  }
  .pb-md-1 {
    padding-bottom: 0.25rem !important;
  }
  .pb-md-2 {
    padding-bottom: 0.5rem !important;
  }
  .pb-md-3 {
    padding-bottom: 1rem !important;
  }
  .pb-md-4 {
    padding-bottom: 1.5rem !important;
  }
  .pb-md-5 {
    padding-bottom: 2rem !important;
  }
  .pb-md-6 {
    padding-bottom: 2.5rem !important;
  }
  .ps-md-0 {
    padding-right: 0 !important;
  }
  .ps-md-1 {
    padding-right: 0.25rem !important;
  }
  .ps-md-2 {
    padding-right: 0.5rem !important;
  }
  .ps-md-3 {
    padding-right: 1rem !important;
  }
  .ps-md-4 {
    padding-right: 1.5rem !important;
  }
  .ps-md-5 {
    padding-right: 2rem !important;
  }
  .ps-md-6 {
    padding-right: 2.5rem !important;
  }
  .gap-md-0 {
    gap: 0 !important;
  }
  .gap-md-1 {
    gap: 0.25rem !important;
  }
  .gap-md-2 {
    gap: 0.5rem !important;
  }
  .gap-md-3 {
    gap: 1rem !important;
  }
  .gap-md-4 {
    gap: 1.5rem !important;
  }
  .gap-md-5 {
    gap: 2rem !important;
  }
  .gap-md-6 {
    gap: 2.5rem !important;
  }
  .row-gap-md-0 {
    row-gap: 0 !important;
  }
  .row-gap-md-1 {
    row-gap: 0.25rem !important;
  }
  .row-gap-md-2 {
    row-gap: 0.5rem !important;
  }
  .row-gap-md-3 {
    row-gap: 1rem !important;
  }
  .row-gap-md-4 {
    row-gap: 1.5rem !important;
  }
  .row-gap-md-5 {
    row-gap: 2rem !important;
  }
  .row-gap-md-6 {
    row-gap: 2.5rem !important;
  }
  .column-gap-md-0 {
    column-gap: 0 !important;
  }
  .column-gap-md-1 {
    column-gap: 0.25rem !important;
  }
  .column-gap-md-2 {
    column-gap: 0.5rem !important;
  }
  .column-gap-md-3 {
    column-gap: 1rem !important;
  }
  .column-gap-md-4 {
    column-gap: 1.5rem !important;
  }
  .column-gap-md-5 {
    column-gap: 2rem !important;
  }
  .column-gap-md-6 {
    column-gap: 2.5rem !important;
  }
  .text-md-start {
    text-align: right !important;
  }
  .text-md-end {
    text-align: left !important;
  }
  .text-md-center {
    text-align: center !important;
  }
  .columns-md-2 {
    columns: 2 !important;
  }
  .columns-md-3 {
    columns: 3 !important;
  }
  .columns-md-4 {
    columns: 4 !important;
  }
}
@media (min-width: 992px) {
  .float-lg-start {
    float: right !important;
  }
  .float-lg-end {
    float: left !important;
  }
  .float-lg-none {
    float: none !important;
  }
  .object-fit-lg-contain {
    object-fit: contain !important;
  }
  .object-fit-lg-cover {
    object-fit: cover !important;
  }
  .object-fit-lg-fill {
    object-fit: fill !important;
  }
  .object-fit-lg-scale {
    object-fit: scale-down !important;
  }
  .object-fit-lg-none {
    object-fit: none !important;
  }
  .d-lg-inline {
    display: inline !important;
  }
  .d-lg-inline-block {
    display: inline-block !important;
  }
  .d-lg-block {
    display: block !important;
  }
  .d-lg-grid {
    display: grid !important;
  }
  .d-lg-inline-grid {
    display: inline-grid !important;
  }
  .d-lg-table {
    display: table !important;
  }
  .d-lg-table-row {
    display: table-row !important;
  }
  .d-lg-table-cell {
    display: table-cell !important;
  }
  .d-lg-flex {
    display: flex !important;
  }
  .d-lg-inline-flex {
    display: inline-flex !important;
  }
  .d-lg-none {
    display: none !important;
  }
  .flex-lg-fill {
    flex: 1 1 auto !important;
  }
  .flex-lg-row {
    flex-direction: row !important;
  }
  .flex-lg-column {
    flex-direction: column !important;
  }
  .flex-lg-row-reverse {
    flex-direction: row-reverse !important;
  }
  .flex-lg-column-reverse {
    flex-direction: column-reverse !important;
  }
  .flex-lg-grow-0 {
    flex-grow: 0 !important;
  }
  .flex-lg-grow-1 {
    flex-grow: 1 !important;
  }
  .flex-lg-shrink-0 {
    flex-shrink: 0 !important;
  }
  .flex-lg-shrink-1 {
    flex-shrink: 1 !important;
  }
  .flex-lg-wrap {
    flex-wrap: wrap !important;
  }
  .flex-lg-nowrap {
    flex-wrap: nowrap !important;
  }
  .flex-lg-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }
  .justify-content-lg-start {
    justify-content: flex-start !important;
  }
  .justify-content-lg-end {
    justify-content: flex-end !important;
  }
  .justify-content-lg-center {
    justify-content: center !important;
  }
  .justify-content-lg-between {
    justify-content: space-between !important;
  }
  .justify-content-lg-around {
    justify-content: space-around !important;
  }
  .justify-content-lg-evenly {
    justify-content: space-evenly !important;
  }
  .align-items-lg-start {
    align-items: flex-start !important;
  }
  .align-items-lg-end {
    align-items: flex-end !important;
  }
  .align-items-lg-center {
    align-items: center !important;
  }
  .align-items-lg-baseline {
    align-items: baseline !important;
  }
  .align-items-lg-stretch {
    align-items: stretch !important;
  }
  .align-content-lg-start {
    align-content: flex-start !important;
  }
  .align-content-lg-end {
    align-content: flex-end !important;
  }
  .align-content-lg-center {
    align-content: center !important;
  }
  .align-content-lg-between {
    align-content: space-between !important;
  }
  .align-content-lg-around {
    align-content: space-around !important;
  }
  .align-content-lg-stretch {
    align-content: stretch !important;
  }
  .align-self-lg-auto {
    align-self: auto !important;
  }
  .align-self-lg-start {
    align-self: flex-start !important;
  }
  .align-self-lg-end {
    align-self: flex-end !important;
  }
  .align-self-lg-center {
    align-self: center !important;
  }
  .align-self-lg-baseline {
    align-self: baseline !important;
  }
  .align-self-lg-stretch {
    align-self: stretch !important;
  }
  .order-lg-first {
    order: -1 !important;
  }
  .order-lg-0 {
    order: 0 !important;
  }
  .order-lg-1 {
    order: 1 !important;
  }
  .order-lg-2 {
    order: 2 !important;
  }
  .order-lg-3 {
    order: 3 !important;
  }
  .order-lg-4 {
    order: 4 !important;
  }
  .order-lg-5 {
    order: 5 !important;
  }
  .order-lg-last {
    order: 6 !important;
  }
  .m-lg-0 {
    margin: 0 !important;
  }
  .m-lg-1 {
    margin: 0.25rem !important;
  }
  .m-lg-2 {
    margin: 0.5rem !important;
  }
  .m-lg-3 {
    margin: 1rem !important;
  }
  .m-lg-4 {
    margin: 1.5rem !important;
  }
  .m-lg-5 {
    margin: 2rem !important;
  }
  .m-lg-6 {
    margin: 2.5rem !important;
  }
  .m-lg-auto {
    margin: auto !important;
  }
  .mx-lg-0 {
    margin-left: 0 !important;
    margin-right: 0 !important;
  }
  .mx-lg-1 {
    margin-left: 0.25rem !important;
    margin-right: 0.25rem !important;
  }
  .mx-lg-2 {
    margin-left: 0.5rem !important;
    margin-right: 0.5rem !important;
  }
  .mx-lg-3 {
    margin-left: 1rem !important;
    margin-right: 1rem !important;
  }
  .mx-lg-4 {
    margin-left: 1.5rem !important;
    margin-right: 1.5rem !important;
  }
  .mx-lg-5 {
    margin-left: 2rem !important;
    margin-right: 2rem !important;
  }
  .mx-lg-6 {
    margin-left: 2.5rem !important;
    margin-right: 2.5rem !important;
  }
  .mx-lg-auto {
    margin-left: auto !important;
    margin-right: auto !important;
  }
  .my-lg-0 {
    margin-top: 0 !important;
    margin-bottom: 0 !important;
  }
  .my-lg-1 {
    margin-top: 0.25rem !important;
    margin-bottom: 0.25rem !important;
  }
  .my-lg-2 {
    margin-top: 0.5rem !important;
    margin-bottom: 0.5rem !important;
  }
  .my-lg-3 {
    margin-top: 1rem !important;
    margin-bottom: 1rem !important;
  }
  .my-lg-4 {
    margin-top: 1.5rem !important;
    margin-bottom: 1.5rem !important;
  }
  .my-lg-5 {
    margin-top: 2rem !important;
    margin-bottom: 2rem !important;
  }
  .my-lg-6 {
    margin-top: 2.5rem !important;
    margin-bottom: 2.5rem !important;
  }
  .my-lg-auto {
    margin-top: auto !important;
    margin-bottom: auto !important;
  }
  .mt-lg-0 {
    margin-top: 0 !important;
  }
  .mt-lg-1 {
    margin-top: 0.25rem !important;
  }
  .mt-lg-2 {
    margin-top: 0.5rem !important;
  }
  .mt-lg-3 {
    margin-top: 1rem !important;
  }
  .mt-lg-4 {
    margin-top: 1.5rem !important;
  }
  .mt-lg-5 {
    margin-top: 2rem !important;
  }
  .mt-lg-6 {
    margin-top: 2.5rem !important;
  }
  .mt-lg-auto {
    margin-top: auto !important;
  }
  .me-lg-0 {
    margin-left: 0 !important;
  }
  .me-lg-1 {
    margin-left: 0.25rem !important;
  }
  .me-lg-2 {
    margin-left: 0.5rem !important;
  }
  .me-lg-3 {
    margin-left: 1rem !important;
  }
  .me-lg-4 {
    margin-left: 1.5rem !important;
  }
  .me-lg-5 {
    margin-left: 2rem !important;
  }
  .me-lg-6 {
    margin-left: 2.5rem !important;
  }
  .me-lg-auto {
    margin-left: auto !important;
  }
  .mb-lg-0 {
    margin-bottom: 0 !important;
  }
  .mb-lg-1 {
    margin-bottom: 0.25rem !important;
  }
  .mb-lg-2 {
    margin-bottom: 0.5rem !important;
  }
  .mb-lg-3 {
    margin-bottom: 1rem !important;
  }
  .mb-lg-4 {
    margin-bottom: 1.5rem !important;
  }
  .mb-lg-5 {
    margin-bottom: 2rem !important;
  }
  .mb-lg-6 {
    margin-bottom: 2.5rem !important;
  }
  .mb-lg-auto {
    margin-bottom: auto !important;
  }
  .ms-lg-0 {
    margin-right: 0 !important;
  }
  .ms-lg-1 {
    margin-right: 0.25rem !important;
  }
  .ms-lg-2 {
    margin-right: 0.5rem !important;
  }
  .ms-lg-3 {
    margin-right: 1rem !important;
  }
  .ms-lg-4 {
    margin-right: 1.5rem !important;
  }
  .ms-lg-5 {
    margin-right: 2rem !important;
  }
  .ms-lg-6 {
    margin-right: 2.5rem !important;
  }
  .ms-lg-auto {
    margin-right: auto !important;
  }
  .m-lg-n1 {
    margin: -0.25rem !important;
  }
  .m-lg-n2 {
    margin: -0.5rem !important;
  }
  .m-lg-n3 {
    margin: -1rem !important;
  }
  .m-lg-n4 {
    margin: -1.5rem !important;
  }
  .m-lg-n5 {
    margin: -2rem !important;
  }
  .m-lg-n6 {
    margin: -2.5rem !important;
  }
  .mx-lg-n1 {
    margin-left: -0.25rem !important;
    margin-right: -0.25rem !important;
  }
  .mx-lg-n2 {
    margin-left: -0.5rem !important;
    margin-right: -0.5rem !important;
  }
  .mx-lg-n3 {
    margin-left: -1rem !important;
    margin-right: -1rem !important;
  }
  .mx-lg-n4 {
    margin-left: -1.5rem !important;
    margin-right: -1.5rem !important;
  }
  .mx-lg-n5 {
    margin-left: -2rem !important;
    margin-right: -2rem !important;
  }
  .mx-lg-n6 {
    margin-left: -2.5rem !important;
    margin-right: -2.5rem !important;
  }
  .my-lg-n1 {
    margin-top: -0.25rem !important;
    margin-bottom: -0.25rem !important;
  }
  .my-lg-n2 {
    margin-top: -0.5rem !important;
    margin-bottom: -0.5rem !important;
  }
  .my-lg-n3 {
    margin-top: -1rem !important;
    margin-bottom: -1rem !important;
  }
  .my-lg-n4 {
    margin-top: -1.5rem !important;
    margin-bottom: -1.5rem !important;
  }
  .my-lg-n5 {
    margin-top: -2rem !important;
    margin-bottom: -2rem !important;
  }
  .my-lg-n6 {
    margin-top: -2.5rem !important;
    margin-bottom: -2.5rem !important;
  }
  .mt-lg-n1 {
    margin-top: -0.25rem !important;
  }
  .mt-lg-n2 {
    margin-top: -0.5rem !important;
  }
  .mt-lg-n3 {
    margin-top: -1rem !important;
  }
  .mt-lg-n4 {
    margin-top: -1.5rem !important;
  }
  .mt-lg-n5 {
    margin-top: -2rem !important;
  }
  .mt-lg-n6 {
    margin-top: -2.5rem !important;
  }
  .me-lg-n1 {
    margin-left: -0.25rem !important;
  }
  .me-lg-n2 {
    margin-left: -0.5rem !important;
  }
  .me-lg-n3 {
    margin-left: -1rem !important;
  }
  .me-lg-n4 {
    margin-left: -1.5rem !important;
  }
  .me-lg-n5 {
    margin-left: -2rem !important;
  }
  .me-lg-n6 {
    margin-left: -2.5rem !important;
  }
  .mb-lg-n1 {
    margin-bottom: -0.25rem !important;
  }
  .mb-lg-n2 {
    margin-bottom: -0.5rem !important;
  }
  .mb-lg-n3 {
    margin-bottom: -1rem !important;
  }
  .mb-lg-n4 {
    margin-bottom: -1.5rem !important;
  }
  .mb-lg-n5 {
    margin-bottom: -2rem !important;
  }
  .mb-lg-n6 {
    margin-bottom: -2.5rem !important;
  }
  .ms-lg-n1 {
    margin-right: -0.25rem !important;
  }
  .ms-lg-n2 {
    margin-right: -0.5rem !important;
  }
  .ms-lg-n3 {
    margin-right: -1rem !important;
  }
  .ms-lg-n4 {
    margin-right: -1.5rem !important;
  }
  .ms-lg-n5 {
    margin-right: -2rem !important;
  }
  .ms-lg-n6 {
    margin-right: -2.5rem !important;
  }
  .p-lg-0 {
    padding: 0 !important;
  }
  .p-lg-1 {
    padding: 0.25rem !important;
  }
  .p-lg-2 {
    padding: 0.5rem !important;
  }
  .p-lg-3 {
    padding: 1rem !important;
  }
  .p-lg-4 {
    padding: 1.5rem !important;
  }
  .p-lg-5 {
    padding: 2rem !important;
  }
  .p-lg-6 {
    padding: 2.5rem !important;
  }
  .px-lg-0 {
    padding-left: 0 !important;
    padding-right: 0 !important;
  }
  .px-lg-1 {
    padding-left: 0.25rem !important;
    padding-right: 0.25rem !important;
  }
  .px-lg-2 {
    padding-left: 0.5rem !important;
    padding-right: 0.5rem !important;
  }
  .px-lg-3 {
    padding-left: 1rem !important;
    padding-right: 1rem !important;
  }
  .px-lg-4 {
    padding-left: 1.5rem !important;
    padding-right: 1.5rem !important;
  }
  .px-lg-5 {
    padding-left: 2rem !important;
    padding-right: 2rem !important;
  }
  .px-lg-6 {
    padding-left: 2.5rem !important;
    padding-right: 2.5rem !important;
  }
  .py-lg-0 {
    padding-top: 0 !important;
    padding-bottom: 0 !important;
  }
  .py-lg-1 {
    padding-top: 0.25rem !important;
    padding-bottom: 0.25rem !important;
  }
  .py-lg-2 {
    padding-top: 0.5rem !important;
    padding-bottom: 0.5rem !important;
  }
  .py-lg-3 {
    padding-top: 1rem !important;
    padding-bottom: 1rem !important;
  }
  .py-lg-4 {
    padding-top: 1.5rem !important;
    padding-bottom: 1.5rem !important;
  }
  .py-lg-5 {
    padding-top: 2rem !important;
    padding-bottom: 2rem !important;
  }
  .py-lg-6 {
    padding-top: 2.5rem !important;
    padding-bottom: 2.5rem !important;
  }
  .pt-lg-0 {
    padding-top: 0 !important;
  }
  .pt-lg-1 {
    padding-top: 0.25rem !important;
  }
  .pt-lg-2 {
    padding-top: 0.5rem !important;
  }
  .pt-lg-3 {
    padding-top: 1rem !important;
  }
  .pt-lg-4 {
    padding-top: 1.5rem !important;
  }
  .pt-lg-5 {
    padding-top: 2rem !important;
  }
  .pt-lg-6 {
    padding-top: 2.5rem !important;
  }
  .pe-lg-0 {
    padding-left: 0 !important;
  }
  .pe-lg-1 {
    padding-left: 0.25rem !important;
  }
  .pe-lg-2 {
    padding-left: 0.5rem !important;
  }
  .pe-lg-3 {
    padding-left: 1rem !important;
  }
  .pe-lg-4 {
    padding-left: 1.5rem !important;
  }
  .pe-lg-5 {
    padding-left: 2rem !important;
  }
  .pe-lg-6 {
    padding-left: 2.5rem !important;
  }
  .pb-lg-0 {
    padding-bottom: 0 !important;
  }
  .pb-lg-1 {
    padding-bottom: 0.25rem !important;
  }
  .pb-lg-2 {
    padding-bottom: 0.5rem !important;
  }
  .pb-lg-3 {
    padding-bottom: 1rem !important;
  }
  .pb-lg-4 {
    padding-bottom: 1.5rem !important;
  }
  .pb-lg-5 {
    padding-bottom: 2rem !important;
  }
  .pb-lg-6 {
    padding-bottom: 2.5rem !important;
  }
  .ps-lg-0 {
    padding-right: 0 !important;
  }
  .ps-lg-1 {
    padding-right: 0.25rem !important;
  }
  .ps-lg-2 {
    padding-right: 0.5rem !important;
  }
  .ps-lg-3 {
    padding-right: 1rem !important;
  }
  .ps-lg-4 {
    padding-right: 1.5rem !important;
  }
  .ps-lg-5 {
    padding-right: 2rem !important;
  }
  .ps-lg-6 {
    padding-right: 2.5rem !important;
  }
  .gap-lg-0 {
    gap: 0 !important;
  }
  .gap-lg-1 {
    gap: 0.25rem !important;
  }
  .gap-lg-2 {
    gap: 0.5rem !important;
  }
  .gap-lg-3 {
    gap: 1rem !important;
  }
  .gap-lg-4 {
    gap: 1.5rem !important;
  }
  .gap-lg-5 {
    gap: 2rem !important;
  }
  .gap-lg-6 {
    gap: 2.5rem !important;
  }
  .row-gap-lg-0 {
    row-gap: 0 !important;
  }
  .row-gap-lg-1 {
    row-gap: 0.25rem !important;
  }
  .row-gap-lg-2 {
    row-gap: 0.5rem !important;
  }
  .row-gap-lg-3 {
    row-gap: 1rem !important;
  }
  .row-gap-lg-4 {
    row-gap: 1.5rem !important;
  }
  .row-gap-lg-5 {
    row-gap: 2rem !important;
  }
  .row-gap-lg-6 {
    row-gap: 2.5rem !important;
  }
  .column-gap-lg-0 {
    column-gap: 0 !important;
  }
  .column-gap-lg-1 {
    column-gap: 0.25rem !important;
  }
  .column-gap-lg-2 {
    column-gap: 0.5rem !important;
  }
  .column-gap-lg-3 {
    column-gap: 1rem !important;
  }
  .column-gap-lg-4 {
    column-gap: 1.5rem !important;
  }
  .column-gap-lg-5 {
    column-gap: 2rem !important;
  }
  .column-gap-lg-6 {
    column-gap: 2.5rem !important;
  }
  .text-lg-start {
    text-align: right !important;
  }
  .text-lg-end {
    text-align: left !important;
  }
  .text-lg-center {
    text-align: center !important;
  }
  .columns-lg-2 {
    columns: 2 !important;
  }
  .columns-lg-3 {
    columns: 3 !important;
  }
  .columns-lg-4 {
    columns: 4 !important;
  }
}
@media (min-width: 1200px) {
  .float-xl-start {
    float: right !important;
  }
  .float-xl-end {
    float: left !important;
  }
  .float-xl-none {
    float: none !important;
  }
  .object-fit-xl-contain {
    object-fit: contain !important;
  }
  .object-fit-xl-cover {
    object-fit: cover !important;
  }
  .object-fit-xl-fill {
    object-fit: fill !important;
  }
  .object-fit-xl-scale {
    object-fit: scale-down !important;
  }
  .object-fit-xl-none {
    object-fit: none !important;
  }
  .d-xl-inline {
    display: inline !important;
  }
  .d-xl-inline-block {
    display: inline-block !important;
  }
  .d-xl-block {
    display: block !important;
  }
  .d-xl-grid {
    display: grid !important;
  }
  .d-xl-inline-grid {
    display: inline-grid !important;
  }
  .d-xl-table {
    display: table !important;
  }
  .d-xl-table-row {
    display: table-row !important;
  }
  .d-xl-table-cell {
    display: table-cell !important;
  }
  .d-xl-flex {
    display: flex !important;
  }
  .d-xl-inline-flex {
    display: inline-flex !important;
  }
  .d-xl-none {
    display: none !important;
  }
  .flex-xl-fill {
    flex: 1 1 auto !important;
  }
  .flex-xl-row {
    flex-direction: row !important;
  }
  .flex-xl-column {
    flex-direction: column !important;
  }
  .flex-xl-row-reverse {
    flex-direction: row-reverse !important;
  }
  .flex-xl-column-reverse {
    flex-direction: column-reverse !important;
  }
  .flex-xl-grow-0 {
    flex-grow: 0 !important;
  }
  .flex-xl-grow-1 {
    flex-grow: 1 !important;
  }
  .flex-xl-shrink-0 {
    flex-shrink: 0 !important;
  }
  .flex-xl-shrink-1 {
    flex-shrink: 1 !important;
  }
  .flex-xl-wrap {
    flex-wrap: wrap !important;
  }
  .flex-xl-nowrap {
    flex-wrap: nowrap !important;
  }
  .flex-xl-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }
  .justify-content-xl-start {
    justify-content: flex-start !important;
  }
  .justify-content-xl-end {
    justify-content: flex-end !important;
  }
  .justify-content-xl-center {
    justify-content: center !important;
  }
  .justify-content-xl-between {
    justify-content: space-between !important;
  }
  .justify-content-xl-around {
    justify-content: space-around !important;
  }
  .justify-content-xl-evenly {
    justify-content: space-evenly !important;
  }
  .align-items-xl-start {
    align-items: flex-start !important;
  }
  .align-items-xl-end {
    align-items: flex-end !important;
  }
  .align-items-xl-center {
    align-items: center !important;
  }
  .align-items-xl-baseline {
    align-items: baseline !important;
  }
  .align-items-xl-stretch {
    align-items: stretch !important;
  }
  .align-content-xl-start {
    align-content: flex-start !important;
  }
  .align-content-xl-end {
    align-content: flex-end !important;
  }
  .align-content-xl-center {
    align-content: center !important;
  }
  .align-content-xl-between {
    align-content: space-between !important;
  }
  .align-content-xl-around {
    align-content: space-around !important;
  }
  .align-content-xl-stretch {
    align-content: stretch !important;
  }
  .align-self-xl-auto {
    align-self: auto !important;
  }
  .align-self-xl-start {
    align-self: flex-start !important;
  }
  .align-self-xl-end {
    align-self: flex-end !important;
  }
  .align-self-xl-center {
    align-self: center !important;
  }
  .align-self-xl-baseline {
    align-self: baseline !important;
  }
  .align-self-xl-stretch {
    align-self: stretch !important;
  }
  .order-xl-first {
    order: -1 !important;
  }
  .order-xl-0 {
    order: 0 !important;
  }
  .order-xl-1 {
    order: 1 !important;
  }
  .order-xl-2 {
    order: 2 !important;
  }
  .order-xl-3 {
    order: 3 !important;
  }
  .order-xl-4 {
    order: 4 !important;
  }
  .order-xl-5 {
    order: 5 !important;
  }
  .order-xl-last {
    order: 6 !important;
  }
  .m-xl-0 {
    margin: 0 !important;
  }
  .m-xl-1 {
    margin: 0.25rem !important;
  }
  .m-xl-2 {
    margin: 0.5rem !important;
  }
  .m-xl-3 {
    margin: 1rem !important;
  }
  .m-xl-4 {
    margin: 1.5rem !important;
  }
  .m-xl-5 {
    margin: 2rem !important;
  }
  .m-xl-6 {
    margin: 2.5rem !important;
  }
  .m-xl-auto {
    margin: auto !important;
  }
  .mx-xl-0 {
    margin-left: 0 !important;
    margin-right: 0 !important;
  }
  .mx-xl-1 {
    margin-left: 0.25rem !important;
    margin-right: 0.25rem !important;
  }
  .mx-xl-2 {
    margin-left: 0.5rem !important;
    margin-right: 0.5rem !important;
  }
  .mx-xl-3 {
    margin-left: 1rem !important;
    margin-right: 1rem !important;
  }
  .mx-xl-4 {
    margin-left: 1.5rem !important;
    margin-right: 1.5rem !important;
  }
  .mx-xl-5 {
    margin-left: 2rem !important;
    margin-right: 2rem !important;
  }
  .mx-xl-6 {
    margin-left: 2.5rem !important;
    margin-right: 2.5rem !important;
  }
  .mx-xl-auto {
    margin-left: auto !important;
    margin-right: auto !important;
  }
  .my-xl-0 {
    margin-top: 0 !important;
    margin-bottom: 0 !important;
  }
  .my-xl-1 {
    margin-top: 0.25rem !important;
    margin-bottom: 0.25rem !important;
  }
  .my-xl-2 {
    margin-top: 0.5rem !important;
    margin-bottom: 0.5rem !important;
  }
  .my-xl-3 {
    margin-top: 1rem !important;
    margin-bottom: 1rem !important;
  }
  .my-xl-4 {
    margin-top: 1.5rem !important;
    margin-bottom: 1.5rem !important;
  }
  .my-xl-5 {
    margin-top: 2rem !important;
    margin-bottom: 2rem !important;
  }
  .my-xl-6 {
    margin-top: 2.5rem !important;
    margin-bottom: 2.5rem !important;
  }
  .my-xl-auto {
    margin-top: auto !important;
    margin-bottom: auto !important;
  }
  .mt-xl-0 {
    margin-top: 0 !important;
  }
  .mt-xl-1 {
    margin-top: 0.25rem !important;
  }
  .mt-xl-2 {
    margin-top: 0.5rem !important;
  }
  .mt-xl-3 {
    margin-top: 1rem !important;
  }
  .mt-xl-4 {
    margin-top: 1.5rem !important;
  }
  .mt-xl-5 {
    margin-top: 2rem !important;
  }
  .mt-xl-6 {
    margin-top: 2.5rem !important;
  }
  .mt-xl-auto {
    margin-top: auto !important;
  }
  .me-xl-0 {
    margin-left: 0 !important;
  }
  .me-xl-1 {
    margin-left: 0.25rem !important;
  }
  .me-xl-2 {
    margin-left: 0.5rem !important;
  }
  .me-xl-3 {
    margin-left: 1rem !important;
  }
  .me-xl-4 {
    margin-left: 1.5rem !important;
  }
  .me-xl-5 {
    margin-left: 2rem !important;
  }
  .me-xl-6 {
    margin-left: 2.5rem !important;
  }
  .me-xl-auto {
    margin-left: auto !important;
  }
  .mb-xl-0 {
    margin-bottom: 0 !important;
  }
  .mb-xl-1 {
    margin-bottom: 0.25rem !important;
  }
  .mb-xl-2 {
    margin-bottom: 0.5rem !important;
  }
  .mb-xl-3 {
    margin-bottom: 1rem !important;
  }
  .mb-xl-4 {
    margin-bottom: 1.5rem !important;
  }
  .mb-xl-5 {
    margin-bottom: 2rem !important;
  }
  .mb-xl-6 {
    margin-bottom: 2.5rem !important;
  }
  .mb-xl-auto {
    margin-bottom: auto !important;
  }
  .ms-xl-0 {
    margin-right: 0 !important;
  }
  .ms-xl-1 {
    margin-right: 0.25rem !important;
  }
  .ms-xl-2 {
    margin-right: 0.5rem !important;
  }
  .ms-xl-3 {
    margin-right: 1rem !important;
  }
  .ms-xl-4 {
    margin-right: 1.5rem !important;
  }
  .ms-xl-5 {
    margin-right: 2rem !important;
  }
  .ms-xl-6 {
    margin-right: 2.5rem !important;
  }
  .ms-xl-auto {
    margin-right: auto !important;
  }
  .m-xl-n1 {
    margin: -0.25rem !important;
  }
  .m-xl-n2 {
    margin: -0.5rem !important;
  }
  .m-xl-n3 {
    margin: -1rem !important;
  }
  .m-xl-n4 {
    margin: -1.5rem !important;
  }
  .m-xl-n5 {
    margin: -2rem !important;
  }
  .m-xl-n6 {
    margin: -2.5rem !important;
  }
  .mx-xl-n1 {
    margin-left: -0.25rem !important;
    margin-right: -0.25rem !important;
  }
  .mx-xl-n2 {
    margin-left: -0.5rem !important;
    margin-right: -0.5rem !important;
  }
  .mx-xl-n3 {
    margin-left: -1rem !important;
    margin-right: -1rem !important;
  }
  .mx-xl-n4 {
    margin-left: -1.5rem !important;
    margin-right: -1.5rem !important;
  }
  .mx-xl-n5 {
    margin-left: -2rem !important;
    margin-right: -2rem !important;
  }
  .mx-xl-n6 {
    margin-left: -2.5rem !important;
    margin-right: -2.5rem !important;
  }
  .my-xl-n1 {
    margin-top: -0.25rem !important;
    margin-bottom: -0.25rem !important;
  }
  .my-xl-n2 {
    margin-top: -0.5rem !important;
    margin-bottom: -0.5rem !important;
  }
  .my-xl-n3 {
    margin-top: -1rem !important;
    margin-bottom: -1rem !important;
  }
  .my-xl-n4 {
    margin-top: -1.5rem !important;
    margin-bottom: -1.5rem !important;
  }
  .my-xl-n5 {
    margin-top: -2rem !important;
    margin-bottom: -2rem !important;
  }
  .my-xl-n6 {
    margin-top: -2.5rem !important;
    margin-bottom: -2.5rem !important;
  }
  .mt-xl-n1 {
    margin-top: -0.25rem !important;
  }
  .mt-xl-n2 {
    margin-top: -0.5rem !important;
  }
  .mt-xl-n3 {
    margin-top: -1rem !important;
  }
  .mt-xl-n4 {
    margin-top: -1.5rem !important;
  }
  .mt-xl-n5 {
    margin-top: -2rem !important;
  }
  .mt-xl-n6 {
    margin-top: -2.5rem !important;
  }
  .me-xl-n1 {
    margin-left: -0.25rem !important;
  }
  .me-xl-n2 {
    margin-left: -0.5rem !important;
  }
  .me-xl-n3 {
    margin-left: -1rem !important;
  }
  .me-xl-n4 {
    margin-left: -1.5rem !important;
  }
  .me-xl-n5 {
    margin-left: -2rem !important;
  }
  .me-xl-n6 {
    margin-left: -2.5rem !important;
  }
  .mb-xl-n1 {
    margin-bottom: -0.25rem !important;
  }
  .mb-xl-n2 {
    margin-bottom: -0.5rem !important;
  }
  .mb-xl-n3 {
    margin-bottom: -1rem !important;
  }
  .mb-xl-n4 {
    margin-bottom: -1.5rem !important;
  }
  .mb-xl-n5 {
    margin-bottom: -2rem !important;
  }
  .mb-xl-n6 {
    margin-bottom: -2.5rem !important;
  }
  .ms-xl-n1 {
    margin-right: -0.25rem !important;
  }
  .ms-xl-n2 {
    margin-right: -0.5rem !important;
  }
  .ms-xl-n3 {
    margin-right: -1rem !important;
  }
  .ms-xl-n4 {
    margin-right: -1.5rem !important;
  }
  .ms-xl-n5 {
    margin-right: -2rem !important;
  }
  .ms-xl-n6 {
    margin-right: -2.5rem !important;
  }
  .p-xl-0 {
    padding: 0 !important;
  }
  .p-xl-1 {
    padding: 0.25rem !important;
  }
  .p-xl-2 {
    padding: 0.5rem !important;
  }
  .p-xl-3 {
    padding: 1rem !important;
  }
  .p-xl-4 {
    padding: 1.5rem !important;
  }
  .p-xl-5 {
    padding: 2rem !important;
  }
  .p-xl-6 {
    padding: 2.5rem !important;
  }
  .px-xl-0 {
    padding-left: 0 !important;
    padding-right: 0 !important;
  }
  .px-xl-1 {
    padding-left: 0.25rem !important;
    padding-right: 0.25rem !important;
  }
  .px-xl-2 {
    padding-left: 0.5rem !important;
    padding-right: 0.5rem !important;
  }
  .px-xl-3 {
    padding-left: 1rem !important;
    padding-right: 1rem !important;
  }
  .px-xl-4 {
    padding-left: 1.5rem !important;
    padding-right: 1.5rem !important;
  }
  .px-xl-5 {
    padding-left: 2rem !important;
    padding-right: 2rem !important;
  }
  .px-xl-6 {
    padding-left: 2.5rem !important;
    padding-right: 2.5rem !important;
  }
  .py-xl-0 {
    padding-top: 0 !important;
    padding-bottom: 0 !important;
  }
  .py-xl-1 {
    padding-top: 0.25rem !important;
    padding-bottom: 0.25rem !important;
  }
  .py-xl-2 {
    padding-top: 0.5rem !important;
    padding-bottom: 0.5rem !important;
  }
  .py-xl-3 {
    padding-top: 1rem !important;
    padding-bottom: 1rem !important;
  }
  .py-xl-4 {
    padding-top: 1.5rem !important;
    padding-bottom: 1.5rem !important;
  }
  .py-xl-5 {
    padding-top: 2rem !important;
    padding-bottom: 2rem !important;
  }
  .py-xl-6 {
    padding-top: 2.5rem !important;
    padding-bottom: 2.5rem !important;
  }
  .pt-xl-0 {
    padding-top: 0 !important;
  }
  .pt-xl-1 {
    padding-top: 0.25rem !important;
  }
  .pt-xl-2 {
    padding-top: 0.5rem !important;
  }
  .pt-xl-3 {
    padding-top: 1rem !important;
  }
  .pt-xl-4 {
    padding-top: 1.5rem !important;
  }
  .pt-xl-5 {
    padding-top: 2rem !important;
  }
  .pt-xl-6 {
    padding-top: 2.5rem !important;
  }
  .pe-xl-0 {
    padding-left: 0 !important;
  }
  .pe-xl-1 {
    padding-left: 0.25rem !important;
  }
  .pe-xl-2 {
    padding-left: 0.5rem !important;
  }
  .pe-xl-3 {
    padding-left: 1rem !important;
  }
  .pe-xl-4 {
    padding-left: 1.5rem !important;
  }
  .pe-xl-5 {
    padding-left: 2rem !important;
  }
  .pe-xl-6 {
    padding-left: 2.5rem !important;
  }
  .pb-xl-0 {
    padding-bottom: 0 !important;
  }
  .pb-xl-1 {
    padding-bottom: 0.25rem !important;
  }
  .pb-xl-2 {
    padding-bottom: 0.5rem !important;
  }
  .pb-xl-3 {
    padding-bottom: 1rem !important;
  }
  .pb-xl-4 {
    padding-bottom: 1.5rem !important;
  }
  .pb-xl-5 {
    padding-bottom: 2rem !important;
  }
  .pb-xl-6 {
    padding-bottom: 2.5rem !important;
  }
  .ps-xl-0 {
    padding-right: 0 !important;
  }
  .ps-xl-1 {
    padding-right: 0.25rem !important;
  }
  .ps-xl-2 {
    padding-right: 0.5rem !important;
  }
  .ps-xl-3 {
    padding-right: 1rem !important;
  }
  .ps-xl-4 {
    padding-right: 1.5rem !important;
  }
  .ps-xl-5 {
    padding-right: 2rem !important;
  }
  .ps-xl-6 {
    padding-right: 2.5rem !important;
  }
  .gap-xl-0 {
    gap: 0 !important;
  }
  .gap-xl-1 {
    gap: 0.25rem !important;
  }
  .gap-xl-2 {
    gap: 0.5rem !important;
  }
  .gap-xl-3 {
    gap: 1rem !important;
  }
  .gap-xl-4 {
    gap: 1.5rem !important;
  }
  .gap-xl-5 {
    gap: 2rem !important;
  }
  .gap-xl-6 {
    gap: 2.5rem !important;
  }
  .row-gap-xl-0 {
    row-gap: 0 !important;
  }
  .row-gap-xl-1 {
    row-gap: 0.25rem !important;
  }
  .row-gap-xl-2 {
    row-gap: 0.5rem !important;
  }
  .row-gap-xl-3 {
    row-gap: 1rem !important;
  }
  .row-gap-xl-4 {
    row-gap: 1.5rem !important;
  }
  .row-gap-xl-5 {
    row-gap: 2rem !important;
  }
  .row-gap-xl-6 {
    row-gap: 2.5rem !important;
  }
  .column-gap-xl-0 {
    column-gap: 0 !important;
  }
  .column-gap-xl-1 {
    column-gap: 0.25rem !important;
  }
  .column-gap-xl-2 {
    column-gap: 0.5rem !important;
  }
  .column-gap-xl-3 {
    column-gap: 1rem !important;
  }
  .column-gap-xl-4 {
    column-gap: 1.5rem !important;
  }
  .column-gap-xl-5 {
    column-gap: 2rem !important;
  }
  .column-gap-xl-6 {
    column-gap: 2.5rem !important;
  }
  .text-xl-start {
    text-align: right !important;
  }
  .text-xl-end {
    text-align: left !important;
  }
  .text-xl-center {
    text-align: center !important;
  }
  .columns-xl-2 {
    columns: 2 !important;
  }
  .columns-xl-3 {
    columns: 3 !important;
  }
  .columns-xl-4 {
    columns: 4 !important;
  }
}
@media (min-width: 1400px) {
  .float-xxl-start {
    float: right !important;
  }
  .float-xxl-end {
    float: left !important;
  }
  .float-xxl-none {
    float: none !important;
  }
  .object-fit-xxl-contain {
    object-fit: contain !important;
  }
  .object-fit-xxl-cover {
    object-fit: cover !important;
  }
  .object-fit-xxl-fill {
    object-fit: fill !important;
  }
  .object-fit-xxl-scale {
    object-fit: scale-down !important;
  }
  .object-fit-xxl-none {
    object-fit: none !important;
  }
  .d-xxl-inline {
    display: inline !important;
  }
  .d-xxl-inline-block {
    display: inline-block !important;
  }
  .d-xxl-block {
    display: block !important;
  }
  .d-xxl-grid {
    display: grid !important;
  }
  .d-xxl-inline-grid {
    display: inline-grid !important;
  }
  .d-xxl-table {
    display: table !important;
  }
  .d-xxl-table-row {
    display: table-row !important;
  }
  .d-xxl-table-cell {
    display: table-cell !important;
  }
  .d-xxl-flex {
    display: flex !important;
  }
  .d-xxl-inline-flex {
    display: inline-flex !important;
  }
  .d-xxl-none {
    display: none !important;
  }
  .flex-xxl-fill {
    flex: 1 1 auto !important;
  }
  .flex-xxl-row {
    flex-direction: row !important;
  }
  .flex-xxl-column {
    flex-direction: column !important;
  }
  .flex-xxl-row-reverse {
    flex-direction: row-reverse !important;
  }
  .flex-xxl-column-reverse {
    flex-direction: column-reverse !important;
  }
  .flex-xxl-grow-0 {
    flex-grow: 0 !important;
  }
  .flex-xxl-grow-1 {
    flex-grow: 1 !important;
  }
  .flex-xxl-shrink-0 {
    flex-shrink: 0 !important;
  }
  .flex-xxl-shrink-1 {
    flex-shrink: 1 !important;
  }
  .flex-xxl-wrap {
    flex-wrap: wrap !important;
  }
  .flex-xxl-nowrap {
    flex-wrap: nowrap !important;
  }
  .flex-xxl-wrap-reverse {
    flex-wrap: wrap-reverse !important;
  }
  .justify-content-xxl-start {
    justify-content: flex-start !important;
  }
  .justify-content-xxl-end {
    justify-content: flex-end !important;
  }
  .justify-content-xxl-center {
    justify-content: center !important;
  }
  .justify-content-xxl-between {
    justify-content: space-between !important;
  }
  .justify-content-xxl-around {
    justify-content: space-around !important;
  }
  .justify-content-xxl-evenly {
    justify-content: space-evenly !important;
  }
  .align-items-xxl-start {
    align-items: flex-start !important;
  }
  .align-items-xxl-end {
    align-items: flex-end !important;
  }
  .align-items-xxl-center {
    align-items: center !important;
  }
  .align-items-xxl-baseline {
    align-items: baseline !important;
  }
  .align-items-xxl-stretch {
    align-items: stretch !important;
  }
  .align-content-xxl-start {
    align-content: flex-start !important;
  }
  .align-content-xxl-end {
    align-content: flex-end !important;
  }
  .align-content-xxl-center {
    align-content: center !important;
  }
  .align-content-xxl-between {
    align-content: space-between !important;
  }
  .align-content-xxl-around {
    align-content: space-around !important;
  }
  .align-content-xxl-stretch {
    align-content: stretch !important;
  }
  .align-self-xxl-auto {
    align-self: auto !important;
  }
  .align-self-xxl-start {
    align-self: flex-start !important;
  }
  .align-self-xxl-end {
    align-self: flex-end !important;
  }
  .align-self-xxl-center {
    align-self: center !important;
  }
  .align-self-xxl-baseline {
    align-self: baseline !important;
  }
  .align-self-xxl-stretch {
    align-self: stretch !important;
  }
  .order-xxl-first {
    order: -1 !important;
  }
  .order-xxl-0 {
    order: 0 !important;
  }
  .order-xxl-1 {
    order: 1 !important;
  }
  .order-xxl-2 {
    order: 2 !important;
  }
  .order-xxl-3 {
    order: 3 !important;
  }
  .order-xxl-4 {
    order: 4 !important;
  }
  .order-xxl-5 {
    order: 5 !important;
  }
  .order-xxl-last {
    order: 6 !important;
  }
  .m-xxl-0 {
    margin: 0 !important;
  }
  .m-xxl-1 {
    margin: 0.25rem !important;
  }
  .m-xxl-2 {
    margin: 0.5rem !important;
  }
  .m-xxl-3 {
    margin: 1rem !important;
  }
  .m-xxl-4 {
    margin: 1.5rem !important;
  }
  .m-xxl-5 {
    margin: 2rem !important;
  }
  .m-xxl-6 {
    margin: 2.5rem !important;
  }
  .m-xxl-auto {
    margin: auto !important;
  }
  .mx-xxl-0 {
    margin-left: 0 !important;
    margin-right: 0 !important;
  }
  .mx-xxl-1 {
    margin-left: 0.25rem !important;
    margin-right: 0.25rem !important;
  }
  .mx-xxl-2 {
    margin-left: 0.5rem !important;
    margin-right: 0.5rem !important;
  }
  .mx-xxl-3 {
    margin-left: 1rem !important;
    margin-right: 1rem !important;
  }
  .mx-xxl-4 {
    margin-left: 1.5rem !important;
    margin-right: 1.5rem !important;
  }
  .mx-xxl-5 {
    margin-left: 2rem !important;
    margin-right: 2rem !important;
  }
  .mx-xxl-6 {
    margin-left: 2.5rem !important;
    margin-right: 2.5rem !important;
  }
  .mx-xxl-auto {
    margin-left: auto !important;
    margin-right: auto !important;
  }
  .my-xxl-0 {
    margin-top: 0 !important;
    margin-bottom: 0 !important;
  }
  .my-xxl-1 {
    margin-top: 0.25rem !important;
    margin-bottom: 0.25rem !important;
  }
  .my-xxl-2 {
    margin-top: 0.5rem !important;
    margin-bottom: 0.5rem !important;
  }
  .my-xxl-3 {
    margin-top: 1rem !important;
    margin-bottom: 1rem !important;
  }
  .my-xxl-4 {
    margin-top: 1.5rem !important;
    margin-bottom: 1.5rem !important;
  }
  .my-xxl-5 {
    margin-top: 2rem !important;
    margin-bottom: 2rem !important;
  }
  .my-xxl-6 {
    margin-top: 2.5rem !important;
    margin-bottom: 2.5rem !important;
  }
  .my-xxl-auto {
    margin-top: auto !important;
    margin-bottom: auto !important;
  }
  .mt-xxl-0 {
    margin-top: 0 !important;
  }
  .mt-xxl-1 {
    margin-top: 0.25rem !important;
  }
  .mt-xxl-2 {
    margin-top: 0.5rem !important;
  }
  .mt-xxl-3 {
    margin-top: 1rem !important;
  }
  .mt-xxl-4 {
    margin-top: 1.5rem !important;
  }
  .mt-xxl-5 {
    margin-top: 2rem !important;
  }
  .mt-xxl-6 {
    margin-top: 2.5rem !important;
  }
  .mt-xxl-auto {
    margin-top: auto !important;
  }
  .me-xxl-0 {
    margin-left: 0 !important;
  }
  .me-xxl-1 {
    margin-left: 0.25rem !important;
  }
  .me-xxl-2 {
    margin-left: 0.5rem !important;
  }
  .me-xxl-3 {
    margin-left: 1rem !important;
  }
  .me-xxl-4 {
    margin-left: 1.5rem !important;
  }
  .me-xxl-5 {
    margin-left: 2rem !important;
  }
  .me-xxl-6 {
    margin-left: 2.5rem !important;
  }
  .me-xxl-auto {
    margin-left: auto !important;
  }
  .mb-xxl-0 {
    margin-bottom: 0 !important;
  }
  .mb-xxl-1 {
    margin-bottom: 0.25rem !important;
  }
  .mb-xxl-2 {
    margin-bottom: 0.5rem !important;
  }
  .mb-xxl-3 {
    margin-bottom: 1rem !important;
  }
  .mb-xxl-4 {
    margin-bottom: 1.5rem !important;
  }
  .mb-xxl-5 {
    margin-bottom: 2rem !important;
  }
  .mb-xxl-6 {
    margin-bottom: 2.5rem !important;
  }
  .mb-xxl-auto {
    margin-bottom: auto !important;
  }
  .ms-xxl-0 {
    margin-right: 0 !important;
  }
  .ms-xxl-1 {
    margin-right: 0.25rem !important;
  }
  .ms-xxl-2 {
    margin-right: 0.5rem !important;
  }
  .ms-xxl-3 {
    margin-right: 1rem !important;
  }
  .ms-xxl-4 {
    margin-right: 1.5rem !important;
  }
  .ms-xxl-5 {
    margin-right: 2rem !important;
  }
  .ms-xxl-6 {
    margin-right: 2.5rem !important;
  }
  .ms-xxl-auto {
    margin-right: auto !important;
  }
  .m-xxl-n1 {
    margin: -0.25rem !important;
  }
  .m-xxl-n2 {
    margin: -0.5rem !important;
  }
  .m-xxl-n3 {
    margin: -1rem !important;
  }
  .m-xxl-n4 {
    margin: -1.5rem !important;
  }
  .m-xxl-n5 {
    margin: -2rem !important;
  }
  .m-xxl-n6 {
    margin: -2.5rem !important;
  }
  .mx-xxl-n1 {
    margin-left: -0.25rem !important;
    margin-right: -0.25rem !important;
  }
  .mx-xxl-n2 {
    margin-left: -0.5rem !important;
    margin-right: -0.5rem !important;
  }
  .mx-xxl-n3 {
    margin-left: -1rem !important;
    margin-right: -1rem !important;
  }
  .mx-xxl-n4 {
    margin-left: -1.5rem !important;
    margin-right: -1.5rem !important;
  }
  .mx-xxl-n5 {
    margin-left: -2rem !important;
    margin-right: -2rem !important;
  }
  .mx-xxl-n6 {
    margin-left: -2.5rem !important;
    margin-right: -2.5rem !important;
  }
  .my-xxl-n1 {
    margin-top: -0.25rem !important;
    margin-bottom: -0.25rem !important;
  }
  .my-xxl-n2 {
    margin-top: -0.5rem !important;
    margin-bottom: -0.5rem !important;
  }
  .my-xxl-n3 {
    margin-top: -1rem !important;
    margin-bottom: -1rem !important;
  }
  .my-xxl-n4 {
    margin-top: -1.5rem !important;
    margin-bottom: -1.5rem !important;
  }
  .my-xxl-n5 {
    margin-top: -2rem !important;
    margin-bottom: -2rem !important;
  }
  .my-xxl-n6 {
    margin-top: -2.5rem !important;
    margin-bottom: -2.5rem !important;
  }
  .mt-xxl-n1 {
    margin-top: -0.25rem !important;
  }
  .mt-xxl-n2 {
    margin-top: -0.5rem !important;
  }
  .mt-xxl-n3 {
    margin-top: -1rem !important;
  }
  .mt-xxl-n4 {
    margin-top: -1.5rem !important;
  }
  .mt-xxl-n5 {
    margin-top: -2rem !important;
  }
  .mt-xxl-n6 {
    margin-top: -2.5rem !important;
  }
  .me-xxl-n1 {
    margin-left: -0.25rem !important;
  }
  .me-xxl-n2 {
    margin-left: -0.5rem !important;
  }
  .me-xxl-n3 {
    margin-left: -1rem !important;
  }
  .me-xxl-n4 {
    margin-left: -1.5rem !important;
  }
  .me-xxl-n5 {
    margin-left: -2rem !important;
  }
  .me-xxl-n6 {
    margin-left: -2.5rem !important;
  }
  .mb-xxl-n1 {
    margin-bottom: -0.25rem !important;
  }
  .mb-xxl-n2 {
    margin-bottom: -0.5rem !important;
  }
  .mb-xxl-n3 {
    margin-bottom: -1rem !important;
  }
  .mb-xxl-n4 {
    margin-bottom: -1.5rem !important;
  }
  .mb-xxl-n5 {
    margin-bottom: -2rem !important;
  }
  .mb-xxl-n6 {
    margin-bottom: -2.5rem !important;
  }
  .ms-xxl-n1 {
    margin-right: -0.25rem !important;
  }
  .ms-xxl-n2 {
    margin-right: -0.5rem !important;
  }
  .ms-xxl-n3 {
    margin-right: -1rem !important;
  }
  .ms-xxl-n4 {
    margin-right: -1.5rem !important;
  }
  .ms-xxl-n5 {
    margin-right: -2rem !important;
  }
  .ms-xxl-n6 {
    margin-right: -2.5rem !important;
  }
  .p-xxl-0 {
    padding: 0 !important;
  }
  .p-xxl-1 {
    padding: 0.25rem !important;
  }
  .p-xxl-2 {
    padding: 0.5rem !important;
  }
  .p-xxl-3 {
    padding: 1rem !important;
  }
  .p-xxl-4 {
    padding: 1.5rem !important;
  }
  .p-xxl-5 {
    padding: 2rem !important;
  }
  .p-xxl-6 {
    padding: 2.5rem !important;
  }
  .px-xxl-0 {
    padding-left: 0 !important;
    padding-right: 0 !important;
  }
  .px-xxl-1 {
    padding-left: 0.25rem !important;
    padding-right: 0.25rem !important;
  }
  .px-xxl-2 {
    padding-left: 0.5rem !important;
    padding-right: 0.5rem !important;
  }
  .px-xxl-3 {
    padding-left: 1rem !important;
    padding-right: 1rem !important;
  }
  .px-xxl-4 {
    padding-left: 1.5rem !important;
    padding-right: 1.5rem !important;
  }
  .px-xxl-5 {
    padding-left: 2rem !important;
    padding-right: 2rem !important;
  }
  .px-xxl-6 {
    padding-left: 2.5rem !important;
    padding-right: 2.5rem !important;
  }
  .py-xxl-0 {
    padding-top: 0 !important;
    padding-bottom: 0 !important;
  }
  .py-xxl-1 {
    padding-top: 0.25rem !important;
    padding-bottom: 0.25rem !important;
  }
  .py-xxl-2 {
    padding-top: 0.5rem !important;
    padding-bottom: 0.5rem !important;
  }
  .py-xxl-3 {
    padding-top: 1rem !important;
    padding-bottom: 1rem !important;
  }
  .py-xxl-4 {
    padding-top: 1.5rem !important;
    padding-bottom: 1.5rem !important;
  }
  .py-xxl-5 {
    padding-top: 2rem !important;
    padding-bottom: 2rem !important;
  }
  .py-xxl-6 {
    padding-top: 2.5rem !important;
    padding-bottom: 2.5rem !important;
  }
  .pt-xxl-0 {
    padding-top: 0 !important;
  }
  .pt-xxl-1 {
    padding-top: 0.25rem !important;
  }
  .pt-xxl-2 {
    padding-top: 0.5rem !important;
  }
  .pt-xxl-3 {
    padding-top: 1rem !important;
  }
  .pt-xxl-4 {
    padding-top: 1.5rem !important;
  }
  .pt-xxl-5 {
    padding-top: 2rem !important;
  }
  .pt-xxl-6 {
    padding-top: 2.5rem !important;
  }
  .pe-xxl-0 {
    padding-left: 0 !important;
  }
  .pe-xxl-1 {
    padding-left: 0.25rem !important;
  }
  .pe-xxl-2 {
    padding-left: 0.5rem !important;
  }
  .pe-xxl-3 {
    padding-left: 1rem !important;
  }
  .pe-xxl-4 {
    padding-left: 1.5rem !important;
  }
  .pe-xxl-5 {
    padding-left: 2rem !important;
  }
  .pe-xxl-6 {
    padding-left: 2.5rem !important;
  }
  .pb-xxl-0 {
    padding-bottom: 0 !important;
  }
  .pb-xxl-1 {
    padding-bottom: 0.25rem !important;
  }
  .pb-xxl-2 {
    padding-bottom: 0.5rem !important;
  }
  .pb-xxl-3 {
    padding-bottom: 1rem !important;
  }
  .pb-xxl-4 {
    padding-bottom: 1.5rem !important;
  }
  .pb-xxl-5 {
    padding-bottom: 2rem !important;
  }
  .pb-xxl-6 {
    padding-bottom: 2.5rem !important;
  }
  .ps-xxl-0 {
    padding-right: 0 !important;
  }
  .ps-xxl-1 {
    padding-right: 0.25rem !important;
  }
  .ps-xxl-2 {
    padding-right: 0.5rem !important;
  }
  .ps-xxl-3 {
    padding-right: 1rem !important;
  }
  .ps-xxl-4 {
    padding-right: 1.5rem !important;
  }
  .ps-xxl-5 {
    padding-right: 2rem !important;
  }
  .ps-xxl-6 {
    padding-right: 2.5rem !important;
  }
  .gap-xxl-0 {
    gap: 0 !important;
  }
  .gap-xxl-1 {
    gap: 0.25rem !important;
  }
  .gap-xxl-2 {
    gap: 0.5rem !important;
  }
  .gap-xxl-3 {
    gap: 1rem !important;
  }
  .gap-xxl-4 {
    gap: 1.5rem !important;
  }
  .gap-xxl-5 {
    gap: 2rem !important;
  }
  .gap-xxl-6 {
    gap: 2.5rem !important;
  }
  .row-gap-xxl-0 {
    row-gap: 0 !important;
  }
  .row-gap-xxl-1 {
    row-gap: 0.25rem !important;
  }
  .row-gap-xxl-2 {
    row-gap: 0.5rem !important;
  }
  .row-gap-xxl-3 {
    row-gap: 1rem !important;
  }
  .row-gap-xxl-4 {
    row-gap: 1.5rem !important;
  }
  .row-gap-xxl-5 {
    row-gap: 2rem !important;
  }
  .row-gap-xxl-6 {
    row-gap: 2.5rem !important;
  }
  .column-gap-xxl-0 {
    column-gap: 0 !important;
  }
  .column-gap-xxl-1 {
    column-gap: 0.25rem !important;
  }
  .column-gap-xxl-2 {
    column-gap: 0.5rem !important;
  }
  .column-gap-xxl-3 {
    column-gap: 1rem !important;
  }
  .column-gap-xxl-4 {
    column-gap: 1.5rem !important;
  }
  .column-gap-xxl-5 {
    column-gap: 2rem !important;
  }
  .column-gap-xxl-6 {
    column-gap: 2.5rem !important;
  }
  .text-xxl-start {
    text-align: right !important;
  }
  .text-xxl-end {
    text-align: left !important;
  }
  .text-xxl-center {
    text-align: center !important;
  }
  .columns-xxl-2 {
    columns: 2 !important;
  }
  .columns-xxl-3 {
    columns: 3 !important;
  }
  .columns-xxl-4 {
    columns: 4 !important;
  }
}
@media print {
  .d-print-inline {
    display: inline !important;
  }
  .d-print-inline-block {
    display: inline-block !important;
  }
  .d-print-block {
    display: block !important;
  }
  .d-print-grid {
    display: grid !important;
  }
  .d-print-inline-grid {
    display: inline-grid !important;
  }
  .d-print-table {
    display: table !important;
  }
  .d-print-table-row {
    display: table-row !important;
  }
  .d-print-table-cell {
    display: table-cell !important;
  }
  .d-print-flex {
    display: flex !important;
  }
  .d-print-inline-flex {
    display: inline-flex !important;
  }
  .d-print-none {
    display: none !important;
  }
}
/**
 * Converts a given value to a percentage string.
 *
 * @param {Number} $value - The value to be converted to a percentage.
 * @return {String} - The percentage representation of the value.
 */
/**
 * Generates a transparent version of the given color.
 *
 * @param {Color} $color - The base color to be made transparent.
 * @param {Number} $alpha - The level of transparency, ranging from 0 (fully transparent) to 1 (fully opaque). Default is 1.
 * @return {Color} - The resulting color with the specified transparency.
 */
:root,
:host {
  /** Fonts */
  --tblr-font-monospace: Monaco, Consolas, Liberation Mono, Courier New, monospace;
  --tblr-font-sans-serif:    Inter Var, Inter, -apple-system, BlinkMacSystemFont, San Francisco, Segoe UI, Roboto, Helvetica Neue, sans-serif;
  --tblr-font-serif: Georgia, Times New Roman, times, serif;
  --tblr-font-comic: Comic Sans MS, Comic Sans, Chalkboard SE, Comic Neue, sans-serif, cursive;
  /** Gray colors */
  --tblr-gray-50: #f9fafb;
  --tblr-gray-100: #f3f4f6;
  --tblr-gray-200: #e5e7eb;
  --tblr-gray-300: #d1d5db;
  --tblr-gray-400: #9ca3af;
  --tblr-gray-500: #6b7280;
  --tblr-gray-600: #4b5563;
  --tblr-gray-700: #374151;
  --tblr-gray-800: #1f2937;
  --tblr-gray-900: #111827;
  --tblr-gray-950: #030712;
  --tblr-white: #ffffff;
  --tblr-black: #000000;
  --tblr-dark: #1f2937;
  --tblr-light: #f9fafb;
  /** Brand colors */
  --tblr-brand: #066fd1;
  /** Theme colors */
  --tblr-primary: #066fd1;
  --tblr-primary-rgb: 6, 111, 209;
  --tblr-primary-fg: var(--tblr-light);
  --tblr-primary-darken: rgb(5.4, 99.9, 188.1);
  --tblr-primary-darken: color-mix(in oklab, var(--tblr-primary), transparent 20%);
  --tblr-primary-lt: rgb(230.1, 240.6, 250.4);
  --tblr-primary-lt: color-mix(in oklab, var(--tblr-primary) 10%, transparent);
  --tblr-primary-200: color-mix(in oklab, var(--tblr-primary) 20%, transparent);
  --tblr-primary-lt-rgb: 230, 241, 250;
  --tblr-secondary: #6b7280;
  --tblr-secondary-rgb: 107, 114, 128;
  --tblr-secondary-fg: var(--tblr-light);
  --tblr-secondary-darken: rgb(96.3, 102.6, 115.2);
  --tblr-secondary-darken: color-mix(in oklab, var(--tblr-secondary), transparent 20%);
  --tblr-secondary-lt: rgb(240.2, 240.9, 242.3);
  --tblr-secondary-lt: color-mix(in oklab, var(--tblr-secondary) 10%, transparent);
  --tblr-secondary-200: color-mix(in oklab, var(--tblr-secondary) 20%, transparent);
  --tblr-secondary-lt-rgb: 240, 241, 242;
  --tblr-success: #2fb344;
  --tblr-success-rgb: 47, 179, 68;
  --tblr-success-fg: var(--tblr-light);
  --tblr-success-darken: rgb(42.3, 161.1, 61.2);
  --tblr-success-darken: color-mix(in oklab, var(--tblr-success), transparent 20%);
  --tblr-success-lt: rgb(234.2, 247.4, 236.3);
  --tblr-success-lt: color-mix(in oklab, var(--tblr-success) 10%, transparent);
  --tblr-success-200: color-mix(in oklab, var(--tblr-success) 20%, transparent);
  --tblr-success-lt-rgb: 234, 247, 236;
  --tblr-info: #4299e1;
  --tblr-info-rgb: 66, 153, 225;
  --tblr-info-fg: var(--tblr-light);
  --tblr-info-darken: rgb(59.4, 137.7, 202.5);
  --tblr-info-darken: color-mix(in oklab, var(--tblr-info), transparent 20%);
  --tblr-info-lt: rgb(236.1, 244.8, 252);
  --tblr-info-lt: color-mix(in oklab, var(--tblr-info) 10%, transparent);
  --tblr-info-200: color-mix(in oklab, var(--tblr-info) 20%, transparent);
  --tblr-info-lt-rgb: 236, 245, 252;
  --tblr-warning: #f59f00;
  --tblr-warning-rgb: 245, 159, 0;
  --tblr-warning-fg: var(--tblr-light);
  --tblr-warning-darken: rgb(220.5, 143.1, 0);
  --tblr-warning-darken: color-mix(in oklab, var(--tblr-warning), transparent 20%);
  --tblr-warning-lt: rgb(254, 245.4, 229.5);
  --tblr-warning-lt: color-mix(in oklab, var(--tblr-warning) 10%, transparent);
  --tblr-warning-200: color-mix(in oklab, var(--tblr-warning) 20%, transparent);
  --tblr-warning-lt-rgb: 254, 245, 230;
  --tblr-danger: #d63939;
  --tblr-danger-rgb: 214, 57, 57;
  --tblr-danger-fg: var(--tblr-light);
  --tblr-danger-darken: rgb(192.6, 51.3, 51.3);
  --tblr-danger-darken: color-mix(in oklab, var(--tblr-danger), transparent 20%);
  --tblr-danger-lt: rgb(250.9, 235.2, 235.2);
  --tblr-danger-lt: color-mix(in oklab, var(--tblr-danger) 10%, transparent);
  --tblr-danger-200: color-mix(in oklab, var(--tblr-danger) 20%, transparent);
  --tblr-danger-lt-rgb: 251, 235, 235;
  --tblr-light: #f9fafb;
  --tblr-light-rgb: 249, 250, 251;
  --tblr-light-fg: var(--tblr-dark);
  --tblr-light-darken: rgb(224.1, 225, 225.9);
  --tblr-light-darken: color-mix(in oklab, var(--tblr-light), transparent 20%);
  --tblr-light-lt: rgb(254.4, 254.5, 254.6);
  --tblr-light-lt: color-mix(in oklab, var(--tblr-light) 10%, transparent);
  --tblr-light-200: color-mix(in oklab, var(--tblr-light) 20%, transparent);
  --tblr-light-lt-rgb: 254, 255, 255;
  --tblr-dark: #1f2937;
  --tblr-dark-rgb: 31, 41, 55;
  --tblr-dark-fg: var(--tblr-light);
  --tblr-dark-darken: rgb(27.9, 36.9, 49.5);
  --tblr-dark-darken: color-mix(in oklab, var(--tblr-dark), transparent 20%);
  --tblr-dark-lt: rgb(232.6, 233.6, 235);
  --tblr-dark-lt: color-mix(in oklab, var(--tblr-dark) 10%, transparent);
  --tblr-dark-200: color-mix(in oklab, var(--tblr-dark) 20%, transparent);
  --tblr-dark-lt-rgb: 233, 234, 235;
  --tblr-muted: #6b7280;
  --tblr-muted-rgb: 107, 114, 128;
  --tblr-muted-fg: var(--tblr-light);
  --tblr-muted-darken: rgb(96.3, 102.6, 115.2);
  --tblr-muted-darken: color-mix(in oklab, var(--tblr-muted), transparent 20%);
  --tblr-muted-lt: rgb(240.2, 240.9, 242.3);
  --tblr-muted-lt: color-mix(in oklab, var(--tblr-muted) 10%, transparent);
  --tblr-muted-200: color-mix(in oklab, var(--tblr-muted) 20%, transparent);
  --tblr-muted-lt-rgb: 240, 241, 242;
  --tblr-blue: #066fd1;
  --tblr-blue-rgb: 6, 111, 209;
  --tblr-blue-fg: var(--tblr-light);
  --tblr-blue-darken: rgb(5.4, 99.9, 188.1);
  --tblr-blue-darken: color-mix(in oklab, var(--tblr-blue), transparent 20%);
  --tblr-blue-lt: rgb(230.1, 240.6, 250.4);
  --tblr-blue-lt: color-mix(in oklab, var(--tblr-blue) 10%, transparent);
  --tblr-blue-200: color-mix(in oklab, var(--tblr-blue) 20%, transparent);
  --tblr-blue-lt-rgb: 230, 241, 250;
  --tblr-azure: #4299e1;
  --tblr-azure-rgb: 66, 153, 225;
  --tblr-azure-fg: var(--tblr-light);
  --tblr-azure-darken: rgb(59.4, 137.7, 202.5);
  --tblr-azure-darken: color-mix(in oklab, var(--tblr-azure), transparent 20%);
  --tblr-azure-lt: rgb(236.1, 244.8, 252);
  --tblr-azure-lt: color-mix(in oklab, var(--tblr-azure) 10%, transparent);
  --tblr-azure-200: color-mix(in oklab, var(--tblr-azure) 20%, transparent);
  --tblr-azure-lt-rgb: 236, 245, 252;
  --tblr-indigo: #4263eb;
  --tblr-indigo-rgb: 66, 99, 235;
  --tblr-indigo-fg: var(--tblr-light);
  --tblr-indigo-darken: rgb(59.4, 89.1, 211.5);
  --tblr-indigo-darken: color-mix(in oklab, var(--tblr-indigo), transparent 20%);
  --tblr-indigo-lt: rgb(236.1, 239.4, 253);
  --tblr-indigo-lt: color-mix(in oklab, var(--tblr-indigo) 10%, transparent);
  --tblr-indigo-200: color-mix(in oklab, var(--tblr-indigo) 20%, transparent);
  --tblr-indigo-lt-rgb: 236, 239, 253;
  --tblr-purple: #ae3ec9;
  --tblr-purple-rgb: 174, 62, 201;
  --tblr-purple-fg: var(--tblr-light);
  --tblr-purple-darken: rgb(156.6, 55.8, 180.9);
  --tblr-purple-darken: color-mix(in oklab, var(--tblr-purple), transparent 20%);
  --tblr-purple-lt: rgb(246.9, 235.7, 249.6);
  --tblr-purple-lt: color-mix(in oklab, var(--tblr-purple) 10%, transparent);
  --tblr-purple-200: color-mix(in oklab, var(--tblr-purple) 20%, transparent);
  --tblr-purple-lt-rgb: 247, 236, 250;
  --tblr-pink: #d6336c;
  --tblr-pink-rgb: 214, 51, 108;
  --tblr-pink-fg: var(--tblr-light);
  --tblr-pink-darken: rgb(192.6, 45.9, 97.2);
  --tblr-pink-darken: color-mix(in oklab, var(--tblr-pink), transparent 20%);
  --tblr-pink-lt: rgb(250.9, 234.6, 240.3);
  --tblr-pink-lt: color-mix(in oklab, var(--tblr-pink) 10%, transparent);
  --tblr-pink-200: color-mix(in oklab, var(--tblr-pink) 20%, transparent);
  --tblr-pink-lt-rgb: 251, 235, 240;
  --tblr-red: #d63939;
  --tblr-red-rgb: 214, 57, 57;
  --tblr-red-fg: var(--tblr-light);
  --tblr-red-darken: rgb(192.6, 51.3, 51.3);
  --tblr-red-darken: color-mix(in oklab, var(--tblr-red), transparent 20%);
  --tblr-red-lt: rgb(250.9, 235.2, 235.2);
  --tblr-red-lt: color-mix(in oklab, var(--tblr-red) 10%, transparent);
  --tblr-red-200: color-mix(in oklab, var(--tblr-red) 20%, transparent);
  --tblr-red-lt-rgb: 251, 235, 235;
  --tblr-orange: #f76707;
  --tblr-orange-rgb: 247, 103, 7;
  --tblr-orange-fg: var(--tblr-light);
  --tblr-orange-darken: rgb(222.3, 92.7, 6.3);
  --tblr-orange-darken: color-mix(in oklab, var(--tblr-orange), transparent 20%);
  --tblr-orange-lt: rgb(254.2, 239.8, 230.2);
  --tblr-orange-lt: color-mix(in oklab, var(--tblr-orange) 10%, transparent);
  --tblr-orange-200: color-mix(in oklab, var(--tblr-orange) 20%, transparent);
  --tblr-orange-lt-rgb: 254, 240, 230;
  --tblr-yellow: #f59f00;
  --tblr-yellow-rgb: 245, 159, 0;
  --tblr-yellow-fg: var(--tblr-light);
  --tblr-yellow-darken: rgb(220.5, 143.1, 0);
  --tblr-yellow-darken: color-mix(in oklab, var(--tblr-yellow), transparent 20%);
  --tblr-yellow-lt: rgb(254, 245.4, 229.5);
  --tblr-yellow-lt: color-mix(in oklab, var(--tblr-yellow) 10%, transparent);
  --tblr-yellow-200: color-mix(in oklab, var(--tblr-yellow) 20%, transparent);
  --tblr-yellow-lt-rgb: 254, 245, 230;
  --tblr-lime: #74b816;
  --tblr-lime-rgb: 116, 184, 22;
  --tblr-lime-fg: var(--tblr-light);
  --tblr-lime-darken: rgb(104.4, 165.6, 19.8);
  --tblr-lime-darken: color-mix(in oklab, var(--tblr-lime), transparent 20%);
  --tblr-lime-lt: rgb(241.1, 247.9, 231.7);
  --tblr-lime-lt: color-mix(in oklab, var(--tblr-lime) 10%, transparent);
  --tblr-lime-200: color-mix(in oklab, var(--tblr-lime) 20%, transparent);
  --tblr-lime-lt-rgb: 241, 248, 232;
  --tblr-green: #2fb344;
  --tblr-green-rgb: 47, 179, 68;
  --tblr-green-fg: var(--tblr-light);
  --tblr-green-darken: rgb(42.3, 161.1, 61.2);
  --tblr-green-darken: color-mix(in oklab, var(--tblr-green), transparent 20%);
  --tblr-green-lt: rgb(234.2, 247.4, 236.3);
  --tblr-green-lt: color-mix(in oklab, var(--tblr-green) 10%, transparent);
  --tblr-green-200: color-mix(in oklab, var(--tblr-green) 20%, transparent);
  --tblr-green-lt-rgb: 234, 247, 236;
  --tblr-teal: #0ca678;
  --tblr-teal-rgb: 12, 166, 120;
  --tblr-teal-fg: var(--tblr-light);
  --tblr-teal-darken: rgb(10.8, 149.4, 108);
  --tblr-teal-darken: color-mix(in oklab, var(--tblr-teal), transparent 20%);
  --tblr-teal-lt: rgb(230.7, 246.1, 241.5);
  --tblr-teal-lt: color-mix(in oklab, var(--tblr-teal) 10%, transparent);
  --tblr-teal-200: color-mix(in oklab, var(--tblr-teal) 20%, transparent);
  --tblr-teal-lt-rgb: 231, 246, 242;
  --tblr-cyan: #17a2b8;
  --tblr-cyan-rgb: 23, 162, 184;
  --tblr-cyan-fg: var(--tblr-light);
  --tblr-cyan-darken: rgb(20.7, 145.8, 165.6);
  --tblr-cyan-darken: color-mix(in oklab, var(--tblr-cyan), transparent 20%);
  --tblr-cyan-lt: rgb(231.8, 245.7, 247.9);
  --tblr-cyan-lt: color-mix(in oklab, var(--tblr-cyan) 10%, transparent);
  --tblr-cyan-200: color-mix(in oklab, var(--tblr-cyan) 20%, transparent);
  --tblr-cyan-lt-rgb: 232, 246, 248;
  --tblr-x: #000000;
  --tblr-x-rgb: 0, 0, 0;
  --tblr-x-fg: var(--tblr-light);
  --tblr-x-darken: black;
  --tblr-x-darken: color-mix(in oklab, var(--tblr-x), transparent 20%);
  --tblr-x-lt: rgb(229.5, 229.5, 229.5);
  --tblr-x-lt: color-mix(in oklab, var(--tblr-x) 10%, transparent);
  --tblr-x-200: color-mix(in oklab, var(--tblr-x) 20%, transparent);
  --tblr-x-lt-rgb: 230, 230, 230;
  --tblr-facebook: #1877f2;
  --tblr-facebook-rgb: 24, 119, 242;
  --tblr-facebook-fg: var(--tblr-light);
  --tblr-facebook-darken: rgb(21.6, 107.1, 217.8);
  --tblr-facebook-darken: color-mix(in oklab, var(--tblr-facebook), transparent 20%);
  --tblr-facebook-lt: rgb(231.9, 241.4, 253.7);
  --tblr-facebook-lt: color-mix(in oklab, var(--tblr-facebook) 10%, transparent);
  --tblr-facebook-200: color-mix(in oklab, var(--tblr-facebook) 20%, transparent);
  --tblr-facebook-lt-rgb: 232, 241, 254;
  --tblr-twitter: #1da1f2;
  --tblr-twitter-rgb: 29, 161, 242;
  --tblr-twitter-fg: var(--tblr-light);
  --tblr-twitter-darken: rgb(26.1, 144.9, 217.8);
  --tblr-twitter-darken: color-mix(in oklab, var(--tblr-twitter), transparent 20%);
  --tblr-twitter-lt: rgb(232.4, 245.6, 253.7);
  --tblr-twitter-lt: color-mix(in oklab, var(--tblr-twitter) 10%, transparent);
  --tblr-twitter-200: color-mix(in oklab, var(--tblr-twitter) 20%, transparent);
  --tblr-twitter-lt-rgb: 232, 246, 254;
  --tblr-linkedin: #0a66c2;
  --tblr-linkedin-rgb: 10, 102, 194;
  --tblr-linkedin-fg: var(--tblr-light);
  --tblr-linkedin-darken: rgb(9, 91.8, 174.6);
  --tblr-linkedin-darken: color-mix(in oklab, var(--tblr-linkedin), transparent 20%);
  --tblr-linkedin-lt: rgb(230.5, 239.7, 248.9);
  --tblr-linkedin-lt: color-mix(in oklab, var(--tblr-linkedin) 10%, transparent);
  --tblr-linkedin-200: color-mix(in oklab, var(--tblr-linkedin) 20%, transparent);
  --tblr-linkedin-lt-rgb: 231, 240, 249;
  --tblr-google: #dc4e41;
  --tblr-google-rgb: 220, 78, 65;
  --tblr-google-fg: var(--tblr-light);
  --tblr-google-darken: rgb(198, 70.2, 58.5);
  --tblr-google-darken: color-mix(in oklab, var(--tblr-google), transparent 20%);
  --tblr-google-lt: rgb(251.5, 237.3, 236);
  --tblr-google-lt: color-mix(in oklab, var(--tblr-google) 10%, transparent);
  --tblr-google-200: color-mix(in oklab, var(--tblr-google) 20%, transparent);
  --tblr-google-lt-rgb: 252, 237, 236;
  --tblr-youtube: #ff0000;
  --tblr-youtube-rgb: 255, 0, 0;
  --tblr-youtube-fg: var(--tblr-light);
  --tblr-youtube-darken: rgb(229.5, 0, 0);
  --tblr-youtube-darken: color-mix(in oklab, var(--tblr-youtube), transparent 20%);
  --tblr-youtube-lt: rgb(255, 229.5, 229.5);
  --tblr-youtube-lt: color-mix(in oklab, var(--tblr-youtube) 10%, transparent);
  --tblr-youtube-200: color-mix(in oklab, var(--tblr-youtube) 20%, transparent);
  --tblr-youtube-lt-rgb: 255, 230, 230;
  --tblr-vimeo: #1ab7ea;
  --tblr-vimeo-rgb: 26, 183, 234;
  --tblr-vimeo-fg: var(--tblr-light);
  --tblr-vimeo-darken: rgb(23.4, 164.7, 210.6);
  --tblr-vimeo-darken: color-mix(in oklab, var(--tblr-vimeo), transparent 20%);
  --tblr-vimeo-lt: rgb(232.1, 247.8, 252.9);
  --tblr-vimeo-lt: color-mix(in oklab, var(--tblr-vimeo) 10%, transparent);
  --tblr-vimeo-200: color-mix(in oklab, var(--tblr-vimeo) 20%, transparent);
  --tblr-vimeo-lt-rgb: 232, 248, 253;
  --tblr-dribbble: #ea4c89;
  --tblr-dribbble-rgb: 234, 76, 137;
  --tblr-dribbble-fg: var(--tblr-light);
  --tblr-dribbble-darken: rgb(210.6, 68.4, 123.3);
  --tblr-dribbble-darken: color-mix(in oklab, var(--tblr-dribbble), transparent 20%);
  --tblr-dribbble-lt: rgb(252.9, 237.1, 243.2);
  --tblr-dribbble-lt: color-mix(in oklab, var(--tblr-dribbble) 10%, transparent);
  --tblr-dribbble-200: color-mix(in oklab, var(--tblr-dribbble) 20%, transparent);
  --tblr-dribbble-lt-rgb: 253, 237, 243;
  --tblr-github: #181717;
  --tblr-github-rgb: 24, 23, 23;
  --tblr-github-fg: var(--tblr-light);
  --tblr-github-darken: rgb(21.6, 20.7, 20.7);
  --tblr-github-darken: color-mix(in oklab, var(--tblr-github), transparent 20%);
  --tblr-github-lt: rgb(231.9, 231.8, 231.8);
  --tblr-github-lt: color-mix(in oklab, var(--tblr-github) 10%, transparent);
  --tblr-github-200: color-mix(in oklab, var(--tblr-github) 20%, transparent);
  --tblr-github-lt-rgb: 232, 232, 232;
  --tblr-instagram: #e4405f;
  --tblr-instagram-rgb: 228, 64, 95;
  --tblr-instagram-fg: var(--tblr-light);
  --tblr-instagram-darken: rgb(205.2, 57.6, 85.5);
  --tblr-instagram-darken: color-mix(in oklab, var(--tblr-instagram), transparent 20%);
  --tblr-instagram-lt: rgb(252.3, 235.9, 239);
  --tblr-instagram-lt: color-mix(in oklab, var(--tblr-instagram) 10%, transparent);
  --tblr-instagram-200: color-mix(in oklab, var(--tblr-instagram) 20%, transparent);
  --tblr-instagram-lt-rgb: 252, 236, 239;
  --tblr-pinterest: #bd081c;
  --tblr-pinterest-rgb: 189, 8, 28;
  --tblr-pinterest-fg: var(--tblr-light);
  --tblr-pinterest-darken: rgb(170.1, 7.2, 25.2);
  --tblr-pinterest-darken: color-mix(in oklab, var(--tblr-pinterest), transparent 20%);
  --tblr-pinterest-lt: rgb(248.4, 230.3, 232.3);
  --tblr-pinterest-lt: color-mix(in oklab, var(--tblr-pinterest) 10%, transparent);
  --tblr-pinterest-200: color-mix(in oklab, var(--tblr-pinterest) 20%, transparent);
  --tblr-pinterest-lt-rgb: 248, 230, 232;
  --tblr-vk: #6383a8;
  --tblr-vk-rgb: 99, 131, 168;
  --tblr-vk-fg: var(--tblr-light);
  --tblr-vk-darken: rgb(89.1, 117.9, 151.2);
  --tblr-vk-darken: color-mix(in oklab, var(--tblr-vk), transparent 20%);
  --tblr-vk-lt: rgb(239.4, 242.6, 246.3);
  --tblr-vk-lt: color-mix(in oklab, var(--tblr-vk) 10%, transparent);
  --tblr-vk-200: color-mix(in oklab, var(--tblr-vk) 20%, transparent);
  --tblr-vk-lt-rgb: 239, 243, 246;
  --tblr-rss: #ffa500;
  --tblr-rss-rgb: 255, 165, 0;
  --tblr-rss-fg: var(--tblr-light);
  --tblr-rss-darken: rgb(229.5, 148.5, 0);
  --tblr-rss-darken: color-mix(in oklab, var(--tblr-rss), transparent 20%);
  --tblr-rss-lt: rgb(255, 246, 229.5);
  --tblr-rss-lt: color-mix(in oklab, var(--tblr-rss) 10%, transparent);
  --tblr-rss-200: color-mix(in oklab, var(--tblr-rss) 20%, transparent);
  --tblr-rss-lt-rgb: 255, 246, 230;
  --tblr-flickr: #0063dc;
  --tblr-flickr-rgb: 0, 99, 220;
  --tblr-flickr-fg: var(--tblr-light);
  --tblr-flickr-darken: rgb(0, 89.1, 198);
  --tblr-flickr-darken: color-mix(in oklab, var(--tblr-flickr), transparent 20%);
  --tblr-flickr-lt: rgb(229.5, 239.4, 251.5);
  --tblr-flickr-lt: color-mix(in oklab, var(--tblr-flickr) 10%, transparent);
  --tblr-flickr-200: color-mix(in oklab, var(--tblr-flickr) 20%, transparent);
  --tblr-flickr-lt-rgb: 230, 239, 252;
  --tblr-bitbucket: #0052cc;
  --tblr-bitbucket-rgb: 0, 82, 204;
  --tblr-bitbucket-fg: var(--tblr-light);
  --tblr-bitbucket-darken: rgb(0, 73.8, 183.6);
  --tblr-bitbucket-darken: color-mix(in oklab, var(--tblr-bitbucket), transparent 20%);
  --tblr-bitbucket-lt: rgb(229.5, 237.7, 249.9);
  --tblr-bitbucket-lt: color-mix(in oklab, var(--tblr-bitbucket) 10%, transparent);
  --tblr-bitbucket-200: color-mix(in oklab, var(--tblr-bitbucket) 20%, transparent);
  --tblr-bitbucket-lt-rgb: 230, 238, 250;
  --tblr-tabler: #066fd1;
  --tblr-tabler-rgb: 6, 111, 209;
  --tblr-tabler-fg: var(--tblr-light);
  --tblr-tabler-darken: rgb(5.4, 99.9, 188.1);
  --tblr-tabler-darken: color-mix(in oklab, var(--tblr-tabler), transparent 20%);
  --tblr-tabler-lt: rgb(230.1, 240.6, 250.4);
  --tblr-tabler-lt: color-mix(in oklab, var(--tblr-tabler) 10%, transparent);
  --tblr-tabler-200: color-mix(in oklab, var(--tblr-tabler) 20%, transparent);
  --tblr-tabler-lt-rgb: 230, 241, 250;
  /** Gray colors */
  --tblr-gray-50-fg: var(--tblr-body-color);
  --tblr-gray-100-fg: var(--tblr-body-color);
  --tblr-gray-200-fg: var(--tblr-body-color);
  --tblr-gray-300-fg: var(--tblr-body-color);
  --tblr-gray-400-fg: var(--tblr-white);
  --tblr-gray-500-fg: var(--tblr-white);
  --tblr-gray-600-fg: var(--tblr-white);
  --tblr-gray-700-fg: var(--tblr-white);
  --tblr-gray-800-fg: var(--tblr-white);
  --tblr-gray-900-fg: var(--tblr-white);
  --tblr-gray-950-fg: var(--tblr-white);
  /** Spacers */
  --tblr-spacer-0: 0;
  --tblr-spacer-1: 0.25rem;
  --tblr-spacer-2: 0.5rem;
  --tblr-spacer-3: 1rem;
  --tblr-spacer-4: 1.5rem;
  --tblr-spacer-5: 2rem;
  --tblr-spacer-6: 2.5rem;
  /** Font sizes */
  --tblr-font-weight-light: 300;
  --tblr-font-weight-normal: 400;
  --tblr-font-weight-medium: 500;
  --tblr-font-weight-bold: 600;
  --tblr-font-weight-black: 700;
  --tblr-font-weight-headings: var(--tblr-font-weight-bold);
  --tblr-font-size-h1: 1.5rem;
  --tblr-font-size-h2: 1.25rem;
  --tblr-font-size-h3: 1rem;
  --tblr-font-size-h4: 0.875rem;
  --tblr-font-size-h5: 0.75rem;
  --tblr-font-size-h6: 0.625rem;
  --tblr-line-height-h1: 2rem;
  --tblr-line-height-h2: 1.75rem;
  --tblr-line-height-h3: 1.5rem;
  --tblr-line-height-h4: 1.25rem;
  --tblr-line-height-h5: 1rem;
  --tblr-line-height-h6: 1rem;
  /** Shadows */
  --tblr-shadow: rgba(var(--tblr-body-color-rgb), 0.04) 0 2px 4px 0;
  --tblr-shadow-border: inset 0 0 0 1px var(--tblr-border-color-translucent);
  --tblr-shadow-transparent: 0 0 0 0 transparent;
  --tblr-shadow-input: 0 1px 1px rgba(var(--tblr-body-color-rgb), 0.06);
  --tblr-shadow-card: 0 0 4px rgba(var(--tblr-body-color-rgb), 0.04);
  --tblr-shadow-card-hover: rgba(var(--tblr-body-color-rgb), 0.16) 0 2px 16px 0;
  --tblr-shadow-dropdown: 0 16px 24px 2px rgba(0, 0, 0, 0.07), 0 6px 30px 5px rgba(0, 0, 0, 0.06), 0 8px 10px -5px rgba(0, 0, 0, 0.1);
  /** Border radiuses */
  --tblr-border-radius-scale: 1;
  --tblr-border-radius-0: calc(0 * var(--tblr-border-radius-scale, 1));
  --tblr-border-radius-sm: calc(4px * var(--tblr-border-radius-scale, 1));
  --tblr-border-radius-md: calc(6px * var(--tblr-border-radius-scale, 1));
  --tblr-border-radius-lg: calc(8px * var(--tblr-border-radius-scale, 1));
  --tblr-border-radius-pill: calc(100rem * var(--tblr-border-radius-scale, 1));
  --tblr-border-radius: var(--tblr-border-radius-md);
  /** Backdrops */
  --tblr-backdrop-opacity: 24%;
  --tblr-backdrop-bg: var(--tblr-bg-surface-dark);
  --tblr-backdrop-bg-dark: color-mix(in srgb, var(--tblr-color-dark), transparent var(--tblr-backdrop-opacity));
  --tblr-backdrop-bg-light: color-mix(in srgb, var(--tblr-color-light), transparent var(--tblr-backdrop-opacity));
  --tblr-backdrop-blur: 4px;
  --tblr-backdrop-filter: blur(var(--tblr-backdrop-blur));
}

:root,
:host {
  font-size: 16px;
  height: 100%;
}
@media (min-width: 992px) {
  :root,
  :host {
    margin-right: calc(100vw - 100%);
    margin-left: 0;
  }
}

:root,
:host,
[data-bs-theme=light] {
  color-scheme: light;
  --tblr-spacer: var(--tblr-spacer-2);
  --tblr-bg-surface: var(--tblr-bg-surface-primary);
  --tblr-bg-surface-primary: var(--tblr-white);
  --tblr-bg-surface-secondary: var(--tblr-gray-50);
  --tblr-bg-surface-tertiary: var(--tblr-gray-50);
  --tblr-bg-surface-dark: var(--tblr-gray-900);
  --tblr-bg-surface-inverted: var(--tblr-gray-900);
  --tblr-bg-forms: var(--tblr-bg-surface);
  --tblr-text-inverted: var(--tblr-gray-100);
  --tblr-body-color: var(--tblr-gray-700);
  --tblr-body-bg: var(--tblr-bg-surface-secondary);
  --tblr-link-color: var(--tblr-primary);
  --tblr-link-hover-color: color-mix(in srgb, var(--tblr-primary), #000 20%);
  --tblr-secondary: var(--tblr-gray-500);
  --tblr-tertiary: var(--tblr-gray-400);
  --tblr-border-color: #e5e7eb;
  --tblr-border-color-translucent: rgba(4, 32, 69, 0.1);
  --tblr-border-dark-color: #9ca3af;
  --tblr-border-dark-color-translucent: rgba(4, 32, 69, 0.27);
  --tblr-border-active-color: rgb(169.16, 173.22, 181.34);
  --tblr-icon-color: var(--tblr-gray-400);
  --tblr-active-bg: rgba(var(--tblr-primary-rgb), 0.04);
  --tblr-disabled-bg: var(--tblr-bg-surface-secondary);
  --tblr-disabled-color: color-mix(in srgb, var(--tblr-body-color) 40%, transparent);
  --tblr-code-color: light-dark(var(--tblr-gray-600), var(--tblr-gray-400));
  --tblr-code-bg: light-dark(var(--tblr-gray-100), var(--tblr-gray-900));
  --tblr-dark-mode-border-color: rgb(45.7069767442, 60.4511627907, 81.0930232558);
  --tblr-dark-mode-border-color-translucent: rgba(72, 110, 149, 0.14);
  --tblr-dark-mode-border-active-color: rgb(53.0604651163, 70.176744186, 94.1395348837);
  --tblr-dark-mode-border-dark-color: rgb(38.3534883721, 50.7255813953, 68.0465116279);
  --tblr-page-padding: var(--tblr-spacer-3);
  --tblr-page-padding-y: var(--tblr-spacer-4);
}
@media (max-width: 991.98px) {
  :root,
  :host,
  [data-bs-theme=light] {
    --tblr-page-padding: var(--tblr-spacer-2);
  }
}

@keyframes pulse {
  0% {
    transform: scale(1);
  }
  14% {
    transform: scale(1.25);
  }
  28% {
    transform: scale(1);
  }
  42% {
    transform: scale(1.25);
  }
  70% {
    transform: scale(1);
  }
}
@keyframes tada {
  0% {
    transform: scale3d(1, 1, 1);
  }
  10%, 5% {
    transform: scale3d(0.9, 0.9, 0.9) rotate3d(0, 0, 1, 5deg);
  }
  15%, 25%, 35%, 45% {
    transform: scale3d(1.1, 1.1, 1.1) rotate3d(0, 0, 1, -5deg);
  }
  20%, 30%, 40% {
    transform: scale3d(1.1, 1.1, 1.1) rotate3d(0, 0, 1, 5deg);
  }
  50% {
    transform: scale3d(1, 1, 1);
  }
}
@keyframes rotate-360 {
  from {
    transform: rotate(0deg);
  }
  to {
    transform: rotate(-360deg);
  }
}
@keyframes blink {
  from {
    opacity: 0;
  }
  50% {
    opacity: 1;
  }
  to {
    opacity: 0;
  }
}
@keyframes shake {
  0% {
    transform: scaleX(1);
  }
  20% {
    transform: scale3d(0.9, 0.9, 0.9) rotate(5deg);
  }
  50%, 70%, 90% {
    transform: scale3d(1.25, 1.25, 1.25) rotate(-5deg);
  }
  60%, 80% {
    transform: scale3d(1.25, 1.25, 1.25) rotate(5deg);
  }
  to {
    transform: scaleX(1);
  }
}
body {
  letter-spacing: 0;
  touch-action: manipulation;
  text-rendering: optimizeLegibility;
  font-feature-settings: "liga" 0, "cv03", "cv04", "cv11";
  position: relative;
  min-height: 100%;
  height: 100%;
  padding: 0;
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}
@media print {
  body {
    background: transparent;
  }
}

* {
  scrollbar-color: color-mix(in srgb, var(--tblr-scrollbar-color, var(--tblr-body-color)) 20%, transparent) transparent;
}

*::-webkit-scrollbar {
  width: 1rem;
  height: 1rem;
  transition: background 0.3s;
}
@media (prefers-reduced-motion: reduce) {
  *::-webkit-scrollbar {
    transition: none;
  }
}

*::-webkit-scrollbar-thumb {
  border-radius: 1rem;
  border: 5px solid transparent;
  box-shadow: inset 0 0 0 1rem color-mix(in srgb, var(--tblr-scrollbar-color, var(--tblr-body-color)) 20%, transparent);
}

*::-webkit-scrollbar-track {
  background: transparent;
}

*:hover::-webkit-scrollbar-thumb {
  box-shadow: inset 0 0 0 1rem color-mix(in srgb, var(--tblr-scrollbar-color, var(--tblr-body-color)) 40%, transparent);
}

*::-webkit-scrollbar-corner {
  background: transparent;
}

.layout-fluid .container,
.layout-fluid [class^=container-],
.layout-fluid [class*=" container-"] {
  max-width: 100%;
}

.layout-boxed {
  --tblr-theme-boxed-border-radius: 0;
  --tblr-theme-boxed-width: 1320px;
}
@media (min-width: 768px) {
  .layout-boxed {
    background: #1f2937 linear-gradient(to left, rgba(255, 255, 255, 0.1), transparent) fixed;
    padding: 1rem;
    --tblr-theme-boxed-border-radius: 6px;
  }
}
.layout-boxed .page {
  margin: 0 auto;
  max-width: var(--tblr-theme-boxed-width);
  border-radius: var(--tblr-theme-boxed-border-radius);
  color: var(--tblr-body-color);
}
@media (min-width: 768px) {
  .layout-boxed .page {
    border: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
    background: var(--tblr-body-bg);
  }
}
.layout-boxed .page > .navbar:first-child {
  border-top-right-radius: var(--tblr-theme-boxed-border-radius);
  border-top-left-radius: var(--tblr-theme-boxed-border-radius);
}

/**
Navbar
 */
.navbar {
  --tblr-navbar-bg: var(--tblr-bg-surface);
  --tblr-navbar-border-width: var(--tblr-border-width);
  --tblr-navbar-active-border-color: var(--tblr-primary);
  --tblr-navbar-active-bg: rgba(0, 0, 0, 0.2);
  --tblr-navbar-border-color: var(--tblr-border-color);
  --tblr-navbar-hover-color: var(--tblr-body-color);
  align-items: stretch;
  min-height: 3.5rem;
  box-shadow: inset 0 calc(-1 * var(--tblr-navbar-border-width)) 0 0 var(--tblr-navbar-border-color);
  background: var(--tblr-navbar-bg);
  color: var(--tblr-navbar-color);
}
.navbar-collapse .navbar {
  flex-grow: 1;
}
.navbar.collapsing {
  min-height: 0;
}
.navbar .dropdown-menu {
  position: absolute;
  z-index: 1030;
}
.navbar .navbar-nav {
  min-height: 3rem;
}
.navbar .navbar-nav .nav-link {
  position: relative;
  min-width: 2.5rem;
  min-height: 2.5rem;
  justify-content: center;
  border-radius: var(--tblr-border-radius);
}
.navbar .navbar-nav .nav-link .badge {
  position: absolute;
  top: 0.375rem;
  left: 0.375rem;
  transform: translate(-50%, -50%);
}

@media (max-width: 575.98px) {
  .navbar-expand-sm .navbar-collapse {
    flex-direction: column;
  }
  .navbar-expand-sm .navbar-collapse [class^=container] {
    flex-direction: column;
    align-items: stretch;
    padding: 0;
  }
  .navbar-expand-sm .navbar-collapse .navbar-nav {
    margin-right: 0;
    margin-left: 0;
  }
  .navbar-expand-sm .navbar-collapse .navbar-nav .nav-link {
    padding: 0.5rem calc(calc(var(--tblr-page-padding) * 2) / 2);
    justify-content: flex-start;
  }
  .navbar-expand-sm .navbar-collapse .dropdown-menu-columns {
    flex-direction: column;
  }
  .navbar-expand-sm .navbar-collapse .dropdown-menu {
    padding: 0;
    background: transparent;
    position: static;
    color: inherit;
    box-shadow: none;
    border: none;
    min-width: 0;
    margin: 0;
  }
  .navbar-expand-sm .navbar-collapse .dropdown-menu .dropdown-item {
    min-width: 0;
    display: flex;
    width: auto;
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 1.75rem);
    color: inherit;
  }
  .navbar-expand-sm .navbar-collapse .dropdown-menu .dropdown-item.disabled {
    color: var(--tblr-disabled-color);
    pointer-events: none;
    background-color: transparent;
  }
  .navbar-expand-sm .navbar-collapse .dropdown-menu .dropdown-item.active, .navbar-expand-sm .navbar-collapse .dropdown-menu .dropdown-item:active {
    background: var(--tblr-navbar-active-bg);
  }
  .navbar-expand-sm .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 3.25rem);
  }
  .navbar-expand-sm .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 4.75rem);
  }
  .navbar-expand-sm .navbar-collapse .dropdown-toggle:after {
    margin-right: auto;
  }
  .navbar-expand-sm .navbar-collapse .nav-item.active:after {
    border-bottom-width: 0;
    border-right-width: 3px;
    left: auto;
    top: 0;
    bottom: 0;
  }
}
@media (min-width: 576px) {
  .navbar-expand-sm .navbar-collapse {
    width: auto;
    flex: 1 1 auto;
  }
  .navbar-expand-sm .nav-item.active {
    position: relative;
  }
  .navbar-expand-sm .nav-item.active .nav-link {
    color: var(--tblr-navbar-active-color);
  }
  .navbar-expand-sm .nav-item.active:after {
    content: "";
    position: absolute;
    right: 0;
    left: 0;
    bottom: -0.25rem;
    border: 0 var(--tblr-border-style) var(--tblr-navbar-active-border-color);
    border-bottom-width: 2px;
  }
  .navbar-expand-sm.navbar-vertical {
    box-shadow: inset calc(-1*(-1 * var(--tblr-navbar-border-width))) 0 0 0 var(--tblr-navbar-border-color);
  }
  .navbar-expand-sm.navbar-vertical.navbar-right, .navbar-expand-sm.navbar-vertical.navbar-end {
    box-shadow: inset calc(-1*(1 * var(--tblr-navbar-border-width))) 0 0 0 var(--tblr-navbar-border-color);
  }
  .navbar-expand-sm.navbar-vertical ~ .navbar,
  .navbar-expand-sm.navbar-vertical ~ .page-wrapper {
    margin-right: 15rem;
  }
  .navbar-expand-sm.navbar-vertical.navbar-right ~ .navbar,
  .navbar-expand-sm.navbar-vertical.navbar-right ~ .page-wrapper, .navbar-expand-sm.navbar-vertical.navbar-end ~ .navbar,
  .navbar-expand-sm.navbar-vertical.navbar-end ~ .page-wrapper {
    margin-right: 0;
    margin-left: 15rem;
  }
}
@media (max-width: 767.98px) {
  .navbar-expand-md .navbar-collapse {
    flex-direction: column;
  }
  .navbar-expand-md .navbar-collapse [class^=container] {
    flex-direction: column;
    align-items: stretch;
    padding: 0;
  }
  .navbar-expand-md .navbar-collapse .navbar-nav {
    margin-right: 0;
    margin-left: 0;
  }
  .navbar-expand-md .navbar-collapse .navbar-nav .nav-link {
    padding: 0.5rem calc(calc(var(--tblr-page-padding) * 2) / 2);
    justify-content: flex-start;
  }
  .navbar-expand-md .navbar-collapse .dropdown-menu-columns {
    flex-direction: column;
  }
  .navbar-expand-md .navbar-collapse .dropdown-menu {
    padding: 0;
    background: transparent;
    position: static;
    color: inherit;
    box-shadow: none;
    border: none;
    min-width: 0;
    margin: 0;
  }
  .navbar-expand-md .navbar-collapse .dropdown-menu .dropdown-item {
    min-width: 0;
    display: flex;
    width: auto;
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 1.75rem);
    color: inherit;
  }
  .navbar-expand-md .navbar-collapse .dropdown-menu .dropdown-item.disabled {
    color: var(--tblr-disabled-color);
    pointer-events: none;
    background-color: transparent;
  }
  .navbar-expand-md .navbar-collapse .dropdown-menu .dropdown-item.active, .navbar-expand-md .navbar-collapse .dropdown-menu .dropdown-item:active {
    background: var(--tblr-navbar-active-bg);
  }
  .navbar-expand-md .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 3.25rem);
  }
  .navbar-expand-md .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 4.75rem);
  }
  .navbar-expand-md .navbar-collapse .dropdown-toggle:after {
    margin-right: auto;
  }
  .navbar-expand-md .navbar-collapse .nav-item.active:after {
    border-bottom-width: 0;
    border-right-width: 3px;
    left: auto;
    top: 0;
    bottom: 0;
  }
}
@media (min-width: 768px) {
  .navbar-expand-md .navbar-collapse {
    width: auto;
    flex: 1 1 auto;
  }
  .navbar-expand-md .nav-item.active {
    position: relative;
  }
  .navbar-expand-md .nav-item.active .nav-link {
    color: var(--tblr-navbar-active-color);
  }
  .navbar-expand-md .nav-item.active:after {
    content: "";
    position: absolute;
    right: 0;
    left: 0;
    bottom: -0.25rem;
    border: 0 var(--tblr-border-style) var(--tblr-navbar-active-border-color);
    border-bottom-width: 2px;
  }
  .navbar-expand-md.navbar-vertical {
    box-shadow: inset calc(-1*(-1 * var(--tblr-navbar-border-width))) 0 0 0 var(--tblr-navbar-border-color);
  }
  .navbar-expand-md.navbar-vertical.navbar-right, .navbar-expand-md.navbar-vertical.navbar-end {
    box-shadow: inset calc(-1*(1 * var(--tblr-navbar-border-width))) 0 0 0 var(--tblr-navbar-border-color);
  }
  .navbar-expand-md.navbar-vertical ~ .navbar,
  .navbar-expand-md.navbar-vertical ~ .page-wrapper {
    margin-right: 15rem;
  }
  .navbar-expand-md.navbar-vertical.navbar-right ~ .navbar,
  .navbar-expand-md.navbar-vertical.navbar-right ~ .page-wrapper, .navbar-expand-md.navbar-vertical.navbar-end ~ .navbar,
  .navbar-expand-md.navbar-vertical.navbar-end ~ .page-wrapper {
    margin-right: 0;
    margin-left: 15rem;
  }
}
@media (max-width: 991.98px) {
  .navbar-expand-lg .navbar-collapse {
    flex-direction: column;
  }
  .navbar-expand-lg .navbar-collapse [class^=container] {
    flex-direction: column;
    align-items: stretch;
    padding: 0;
  }
  .navbar-expand-lg .navbar-collapse .navbar-nav {
    margin-right: 0;
    margin-left: 0;
  }
  .navbar-expand-lg .navbar-collapse .navbar-nav .nav-link {
    padding: 0.5rem calc(calc(var(--tblr-page-padding) * 2) / 2);
    justify-content: flex-start;
  }
  .navbar-expand-lg .navbar-collapse .dropdown-menu-columns {
    flex-direction: column;
  }
  .navbar-expand-lg .navbar-collapse .dropdown-menu {
    padding: 0;
    background: transparent;
    position: static;
    color: inherit;
    box-shadow: none;
    border: none;
    min-width: 0;
    margin: 0;
  }
  .navbar-expand-lg .navbar-collapse .dropdown-menu .dropdown-item {
    min-width: 0;
    display: flex;
    width: auto;
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 1.75rem);
    color: inherit;
  }
  .navbar-expand-lg .navbar-collapse .dropdown-menu .dropdown-item.disabled {
    color: var(--tblr-disabled-color);
    pointer-events: none;
    background-color: transparent;
  }
  .navbar-expand-lg .navbar-collapse .dropdown-menu .dropdown-item.active, .navbar-expand-lg .navbar-collapse .dropdown-menu .dropdown-item:active {
    background: var(--tblr-navbar-active-bg);
  }
  .navbar-expand-lg .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 3.25rem);
  }
  .navbar-expand-lg .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 4.75rem);
  }
  .navbar-expand-lg .navbar-collapse .dropdown-toggle:after {
    margin-right: auto;
  }
  .navbar-expand-lg .navbar-collapse .nav-item.active:after {
    border-bottom-width: 0;
    border-right-width: 3px;
    left: auto;
    top: 0;
    bottom: 0;
  }
}
@media (min-width: 992px) {
  .navbar-expand-lg .navbar-collapse {
    width: auto;
    flex: 1 1 auto;
  }
  .navbar-expand-lg .nav-item.active {
    position: relative;
  }
  .navbar-expand-lg .nav-item.active .nav-link {
    color: var(--tblr-navbar-active-color);
  }
  .navbar-expand-lg .nav-item.active:after {
    content: "";
    position: absolute;
    right: 0;
    left: 0;
    bottom: -0.25rem;
    border: 0 var(--tblr-border-style) var(--tblr-navbar-active-border-color);
    border-bottom-width: 2px;
  }
  .navbar-expand-lg.navbar-vertical {
    box-shadow: inset calc(-1*(-1 * var(--tblr-navbar-border-width))) 0 0 0 var(--tblr-navbar-border-color);
  }
  .navbar-expand-lg.navbar-vertical.navbar-right, .navbar-expand-lg.navbar-vertical.navbar-end {
    box-shadow: inset calc(-1*(1 * var(--tblr-navbar-border-width))) 0 0 0 var(--tblr-navbar-border-color);
  }
  .navbar-expand-lg.navbar-vertical ~ .navbar,
  .navbar-expand-lg.navbar-vertical ~ .page-wrapper {
    margin-right: 15rem;
  }
  .navbar-expand-lg.navbar-vertical.navbar-right ~ .navbar,
  .navbar-expand-lg.navbar-vertical.navbar-right ~ .page-wrapper, .navbar-expand-lg.navbar-vertical.navbar-end ~ .navbar,
  .navbar-expand-lg.navbar-vertical.navbar-end ~ .page-wrapper {
    margin-right: 0;
    margin-left: 15rem;
  }
}
@media (max-width: 1199.98px) {
  .navbar-expand-xl .navbar-collapse {
    flex-direction: column;
  }
  .navbar-expand-xl .navbar-collapse [class^=container] {
    flex-direction: column;
    align-items: stretch;
    padding: 0;
  }
  .navbar-expand-xl .navbar-collapse .navbar-nav {
    margin-right: 0;
    margin-left: 0;
  }
  .navbar-expand-xl .navbar-collapse .navbar-nav .nav-link {
    padding: 0.5rem calc(calc(var(--tblr-page-padding) * 2) / 2);
    justify-content: flex-start;
  }
  .navbar-expand-xl .navbar-collapse .dropdown-menu-columns {
    flex-direction: column;
  }
  .navbar-expand-xl .navbar-collapse .dropdown-menu {
    padding: 0;
    background: transparent;
    position: static;
    color: inherit;
    box-shadow: none;
    border: none;
    min-width: 0;
    margin: 0;
  }
  .navbar-expand-xl .navbar-collapse .dropdown-menu .dropdown-item {
    min-width: 0;
    display: flex;
    width: auto;
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 1.75rem);
    color: inherit;
  }
  .navbar-expand-xl .navbar-collapse .dropdown-menu .dropdown-item.disabled {
    color: var(--tblr-disabled-color);
    pointer-events: none;
    background-color: transparent;
  }
  .navbar-expand-xl .navbar-collapse .dropdown-menu .dropdown-item.active, .navbar-expand-xl .navbar-collapse .dropdown-menu .dropdown-item:active {
    background: var(--tblr-navbar-active-bg);
  }
  .navbar-expand-xl .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 3.25rem);
  }
  .navbar-expand-xl .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 4.75rem);
  }
  .navbar-expand-xl .navbar-collapse .dropdown-toggle:after {
    margin-right: auto;
  }
  .navbar-expand-xl .navbar-collapse .nav-item.active:after {
    border-bottom-width: 0;
    border-right-width: 3px;
    left: auto;
    top: 0;
    bottom: 0;
  }
}
@media (min-width: 1200px) {
  .navbar-expand-xl .navbar-collapse {
    width: auto;
    flex: 1 1 auto;
  }
  .navbar-expand-xl .nav-item.active {
    position: relative;
  }
  .navbar-expand-xl .nav-item.active .nav-link {
    color: var(--tblr-navbar-active-color);
  }
  .navbar-expand-xl .nav-item.active:after {
    content: "";
    position: absolute;
    right: 0;
    left: 0;
    bottom: -0.25rem;
    border: 0 var(--tblr-border-style) var(--tblr-navbar-active-border-color);
    border-bottom-width: 2px;
  }
  .navbar-expand-xl.navbar-vertical {
    box-shadow: inset calc(-1*(-1 * var(--tblr-navbar-border-width))) 0 0 0 var(--tblr-navbar-border-color);
  }
  .navbar-expand-xl.navbar-vertical.navbar-right, .navbar-expand-xl.navbar-vertical.navbar-end {
    box-shadow: inset calc(-1*(1 * var(--tblr-navbar-border-width))) 0 0 0 var(--tblr-navbar-border-color);
  }
  .navbar-expand-xl.navbar-vertical ~ .navbar,
  .navbar-expand-xl.navbar-vertical ~ .page-wrapper {
    margin-right: 15rem;
  }
  .navbar-expand-xl.navbar-vertical.navbar-right ~ .navbar,
  .navbar-expand-xl.navbar-vertical.navbar-right ~ .page-wrapper, .navbar-expand-xl.navbar-vertical.navbar-end ~ .navbar,
  .navbar-expand-xl.navbar-vertical.navbar-end ~ .page-wrapper {
    margin-right: 0;
    margin-left: 15rem;
  }
}
@media (max-width: 1399.98px) {
  .navbar-expand-xxl .navbar-collapse {
    flex-direction: column;
  }
  .navbar-expand-xxl .navbar-collapse [class^=container] {
    flex-direction: column;
    align-items: stretch;
    padding: 0;
  }
  .navbar-expand-xxl .navbar-collapse .navbar-nav {
    margin-right: 0;
    margin-left: 0;
  }
  .navbar-expand-xxl .navbar-collapse .navbar-nav .nav-link {
    padding: 0.5rem calc(calc(var(--tblr-page-padding) * 2) / 2);
    justify-content: flex-start;
  }
  .navbar-expand-xxl .navbar-collapse .dropdown-menu-columns {
    flex-direction: column;
  }
  .navbar-expand-xxl .navbar-collapse .dropdown-menu {
    padding: 0;
    background: transparent;
    position: static;
    color: inherit;
    box-shadow: none;
    border: none;
    min-width: 0;
    margin: 0;
  }
  .navbar-expand-xxl .navbar-collapse .dropdown-menu .dropdown-item {
    min-width: 0;
    display: flex;
    width: auto;
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 1.75rem);
    color: inherit;
  }
  .navbar-expand-xxl .navbar-collapse .dropdown-menu .dropdown-item.disabled {
    color: var(--tblr-disabled-color);
    pointer-events: none;
    background-color: transparent;
  }
  .navbar-expand-xxl .navbar-collapse .dropdown-menu .dropdown-item.active, .navbar-expand-xxl .navbar-collapse .dropdown-menu .dropdown-item:active {
    background: var(--tblr-navbar-active-bg);
  }
  .navbar-expand-xxl .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 3.25rem);
  }
  .navbar-expand-xxl .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 4.75rem);
  }
  .navbar-expand-xxl .navbar-collapse .dropdown-toggle:after {
    margin-right: auto;
  }
  .navbar-expand-xxl .navbar-collapse .nav-item.active:after {
    border-bottom-width: 0;
    border-right-width: 3px;
    left: auto;
    top: 0;
    bottom: 0;
  }
}
@media (min-width: 1400px) {
  .navbar-expand-xxl .navbar-collapse {
    width: auto;
    flex: 1 1 auto;
  }
  .navbar-expand-xxl .nav-item.active {
    position: relative;
  }
  .navbar-expand-xxl .nav-item.active .nav-link {
    color: var(--tblr-navbar-active-color);
  }
  .navbar-expand-xxl .nav-item.active:after {
    content: "";
    position: absolute;
    right: 0;
    left: 0;
    bottom: -0.25rem;
    border: 0 var(--tblr-border-style) var(--tblr-navbar-active-border-color);
    border-bottom-width: 2px;
  }
  .navbar-expand-xxl.navbar-vertical {
    box-shadow: inset calc(-1*(-1 * var(--tblr-navbar-border-width))) 0 0 0 var(--tblr-navbar-border-color);
  }
  .navbar-expand-xxl.navbar-vertical.navbar-right, .navbar-expand-xxl.navbar-vertical.navbar-end {
    box-shadow: inset calc(-1*(1 * var(--tblr-navbar-border-width))) 0 0 0 var(--tblr-navbar-border-color);
  }
  .navbar-expand-xxl.navbar-vertical ~ .navbar,
  .navbar-expand-xxl.navbar-vertical ~ .page-wrapper {
    margin-right: 15rem;
  }
  .navbar-expand-xxl.navbar-vertical.navbar-right ~ .navbar,
  .navbar-expand-xxl.navbar-vertical.navbar-right ~ .page-wrapper, .navbar-expand-xxl.navbar-vertical.navbar-end ~ .navbar,
  .navbar-expand-xxl.navbar-vertical.navbar-end ~ .page-wrapper {
    margin-right: 0;
    margin-left: 15rem;
  }
}
.navbar-expand .navbar-collapse {
  flex-direction: column;
}
.navbar-expand .navbar-collapse [class^=container] {
  flex-direction: column;
  align-items: stretch;
  padding: 0;
}
.navbar-expand .navbar-collapse .navbar-nav {
  margin-right: 0;
  margin-left: 0;
}
.navbar-expand .navbar-collapse .navbar-nav .nav-link {
  padding: 0.5rem calc(calc(var(--tblr-page-padding) * 2) / 2);
  justify-content: flex-start;
}
.navbar-expand .navbar-collapse .dropdown-menu-columns {
  flex-direction: column;
}
.navbar-expand .navbar-collapse .dropdown-menu {
  padding: 0;
  background: transparent;
  position: static;
  color: inherit;
  box-shadow: none;
  border: none;
  min-width: 0;
  margin: 0;
}
.navbar-expand .navbar-collapse .dropdown-menu .dropdown-item {
  min-width: 0;
  display: flex;
  width: auto;
  padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 1.75rem);
  color: inherit;
}
.navbar-expand .navbar-collapse .dropdown-menu .dropdown-item.disabled {
  color: var(--tblr-disabled-color);
  pointer-events: none;
  background-color: transparent;
}
.navbar-expand .navbar-collapse .dropdown-menu .dropdown-item.active, .navbar-expand .navbar-collapse .dropdown-menu .dropdown-item:active {
  background: var(--tblr-navbar-active-bg);
}
.navbar-expand .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-item {
  padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 3.25rem);
}
.navbar-expand .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-menu .dropdown-item {
  padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 4.75rem);
}
.navbar-expand .navbar-collapse .dropdown-toggle:after {
  margin-right: auto;
}
.navbar-expand .navbar-collapse .nav-item.active:after {
  border-bottom-width: 0;
  border-right-width: 3px;
  left: auto;
  top: 0;
  bottom: 0;
}
.navbar-expand .navbar-collapse {
  width: auto;
  flex: 1 1 auto;
}
.navbar-expand .nav-item.active {
  position: relative;
}
.navbar-expand .nav-item.active .nav-link {
  color: var(--tblr-navbar-active-color);
}
.navbar-expand .nav-item.active:after {
  content: "";
  position: absolute;
  right: 0;
  left: 0;
  bottom: -0.25rem;
  border: 0 var(--tblr-border-style) var(--tblr-navbar-active-border-color);
  border-bottom-width: 2px;
}
.navbar-expand.navbar-vertical {
  box-shadow: inset calc(-1*(-1 * var(--tblr-navbar-border-width))) 0 0 0 var(--tblr-navbar-border-color);
}
.navbar-expand.navbar-vertical.navbar-right, .navbar-expand.navbar-vertical.navbar-end {
  box-shadow: inset calc(-1*(1 * var(--tblr-navbar-border-width))) 0 0 0 var(--tblr-navbar-border-color);
}
.navbar-expand.navbar-vertical ~ .navbar,
.navbar-expand.navbar-vertical ~ .page-wrapper {
  margin-right: 15rem;
}
.navbar-expand.navbar-vertical.navbar-right ~ .navbar,
.navbar-expand.navbar-vertical.navbar-right ~ .page-wrapper, .navbar-expand.navbar-vertical.navbar-end ~ .navbar,
.navbar-expand.navbar-vertical.navbar-end ~ .page-wrapper {
  margin-right: 0;
  margin-left: 15rem;
}

/**
Navbar brand
 */
.navbar-brand {
  display: inline-flex;
  align-items: center;
  font-weight: var(--tblr-font-weight-bold);
  margin: 0;
  line-height: 1;
  gap: 0.5rem;
}

.navbar-brand-image {
  height: 2rem;
  width: auto;
}

/**
Navbar toggler
 */
.navbar-toggler {
  border: 0;
  width: 2rem;
  height: 2rem;
  position: relative;
  display: flex;
  align-items: center;
  justify-content: center;
}

.navbar-toggler-icon {
  height: 2px;
  width: 1.25em;
  background: currentColor;
  border-radius: 10px;
  transition: top 0.2s 0.2s, bottom 0.2s 0.2s, transform 0.2s, opacity 0s 0.2s;
  position: relative;
}
@media (prefers-reduced-motion: reduce) {
  .navbar-toggler-icon {
    transition: none;
  }
}
.navbar-toggler-icon:before, .navbar-toggler-icon:after {
  content: "";
  display: block;
  height: inherit;
  width: inherit;
  border-radius: inherit;
  background: inherit;
  position: absolute;
  right: 0;
  transition: inherit;
}
@media (prefers-reduced-motion: reduce) {
  .navbar-toggler-icon:before, .navbar-toggler-icon:after {
    transition: none;
  }
}
.navbar-toggler-icon:before {
  top: -0.45em;
}
.navbar-toggler-icon:after {
  bottom: -0.45em;
}
.navbar-toggler[aria-expanded=true] .navbar-toggler-icon {
  transform: rotate(-45deg);
  transition: top 0.3s, bottom 0.3s, transform 0.3s 0.3s, opacity 0s 0.3s;
}
@media (prefers-reduced-motion: reduce) {
  .navbar-toggler[aria-expanded=true] .navbar-toggler-icon {
    transition: none;
  }
}
.navbar-toggler[aria-expanded=true] .navbar-toggler-icon:before {
  top: 0;
  transform: rotate(90deg);
}
.navbar-toggler[aria-expanded=true] .navbar-toggler-icon:after {
  bottom: 0;
  opacity: 0;
}

/**
Navbar transparent
 */
.navbar-transparent {
  --tblr-navbar-border-color: transparent !important;
  background: transparent !important;
}

/**
Navbar nav
 */
.navbar-nav {
  --tblr-nav-link-hover-bg: color-mix(in srgb, var(--tblr-nav-link-color) 4%, transparent);
  margin: 0;
  padding: 0;
  align-items: stretch;
}
.navbar-nav .nav-item {
  display: flex;
  flex-direction: column;
  justify-content: center;
}

/**
Navbar side
 */
.navbar-side {
  margin: 0;
  display: flex;
  flex-direction: row;
  align-items: center;
  justify-content: space-around;
}

/**
Navbar vertical
 */
@media (min-width: 576px) {
  .navbar-vertical.navbar-expand-sm {
    width: 15rem;
    position: fixed;
    top: 0;
    right: 0;
    bottom: 0;
    z-index: 1030;
    align-items: flex-start;
    transition: transform 0.3s;
    overflow-y: scroll;
    padding: 0;
  }
}
@media (min-width: 576px) and (prefers-reduced-motion: reduce) {
  .navbar-vertical.navbar-expand-sm {
    transition: none;
  }
}
@media (min-width: 576px) {
  .navbar-vertical.navbar-expand-sm.navbar-right, .navbar-vertical.navbar-expand-sm.navbar-end {
    right: auto;
    left: 0;
  }
  .navbar-vertical.navbar-expand-sm .navbar-brand {
    padding: 0.75rem 0;
    justify-content: center;
  }
  .navbar-vertical.navbar-expand-sm .navbar-collapse {
    align-items: stretch;
  }
  .navbar-vertical.navbar-expand-sm .navbar-nav {
    flex-direction: column;
    flex-grow: 1;
    min-height: auto;
  }
  .navbar-vertical.navbar-expand-sm .navbar-nav .nav-link {
    padding-top: 0.5rem;
    padding-bottom: 0.5rem;
  }
  .navbar-vertical.navbar-expand-sm > [class^=container] {
    flex-direction: column;
    align-items: stretch;
    min-height: 100%;
    justify-content: flex-start;
    padding: 0;
  }
  .navbar-vertical.navbar-expand-sm ~ .page {
    padding-right: 15rem;
  }
  .navbar-vertical.navbar-expand-sm ~ .page [class^=container] {
    padding-right: 1.5rem;
    padding-left: 1.5rem;
  }
  .navbar-vertical.navbar-expand-sm.navbar-right ~ .page, .navbar-vertical.navbar-expand-sm.navbar-end ~ .page {
    padding-right: 0;
    padding-left: 15rem;
  }
  .navbar-vertical.navbar-expand-sm .navbar-collapse {
    flex-direction: column;
  }
  .navbar-vertical.navbar-expand-sm .navbar-collapse [class^=container] {
    flex-direction: column;
    align-items: stretch;
    padding: 0;
  }
  .navbar-vertical.navbar-expand-sm .navbar-collapse .navbar-nav {
    margin-right: 0;
    margin-left: 0;
  }
  .navbar-vertical.navbar-expand-sm .navbar-collapse .navbar-nav .nav-link {
    padding: 0.5rem calc(calc(var(--tblr-page-padding) * 2) / 2);
    justify-content: flex-start;
  }
  .navbar-vertical.navbar-expand-sm .navbar-collapse .dropdown-menu-columns {
    flex-direction: column;
  }
  .navbar-vertical.navbar-expand-sm .navbar-collapse .dropdown-menu {
    padding: 0;
    background: transparent;
    position: static;
    color: inherit;
    box-shadow: none;
    border: none;
    min-width: 0;
    margin: 0;
  }
  .navbar-vertical.navbar-expand-sm .navbar-collapse .dropdown-menu .dropdown-item {
    min-width: 0;
    display: flex;
    width: auto;
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 1.75rem);
    color: inherit;
  }
  .navbar-vertical.navbar-expand-sm .navbar-collapse .dropdown-menu .dropdown-item.disabled {
    color: var(--tblr-disabled-color);
    pointer-events: none;
    background-color: transparent;
  }
  .navbar-vertical.navbar-expand-sm .navbar-collapse .dropdown-menu .dropdown-item.active, .navbar-vertical.navbar-expand-sm .navbar-collapse .dropdown-menu .dropdown-item:active {
    background: var(--tblr-navbar-active-bg);
  }
  .navbar-vertical.navbar-expand-sm .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 3.25rem);
  }
  .navbar-vertical.navbar-expand-sm .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 4.75rem);
  }
  .navbar-vertical.navbar-expand-sm .navbar-collapse .dropdown-toggle:after {
    margin-right: auto;
  }
  .navbar-vertical.navbar-expand-sm .navbar-collapse .nav-item.active:after {
    border-bottom-width: 0;
    border-right-width: 3px;
    left: auto;
    top: 0;
    bottom: 0;
  }
}
@media (min-width: 768px) {
  .navbar-vertical.navbar-expand-md {
    width: 15rem;
    position: fixed;
    top: 0;
    right: 0;
    bottom: 0;
    z-index: 1030;
    align-items: flex-start;
    transition: transform 0.3s;
    overflow-y: scroll;
    padding: 0;
  }
}
@media (min-width: 768px) and (prefers-reduced-motion: reduce) {
  .navbar-vertical.navbar-expand-md {
    transition: none;
  }
}
@media (min-width: 768px) {
  .navbar-vertical.navbar-expand-md.navbar-right, .navbar-vertical.navbar-expand-md.navbar-end {
    right: auto;
    left: 0;
  }
  .navbar-vertical.navbar-expand-md .navbar-brand {
    padding: 0.75rem 0;
    justify-content: center;
  }
  .navbar-vertical.navbar-expand-md .navbar-collapse {
    align-items: stretch;
  }
  .navbar-vertical.navbar-expand-md .navbar-nav {
    flex-direction: column;
    flex-grow: 1;
    min-height: auto;
  }
  .navbar-vertical.navbar-expand-md .navbar-nav .nav-link {
    padding-top: 0.5rem;
    padding-bottom: 0.5rem;
  }
  .navbar-vertical.navbar-expand-md > [class^=container] {
    flex-direction: column;
    align-items: stretch;
    min-height: 100%;
    justify-content: flex-start;
    padding: 0;
  }
  .navbar-vertical.navbar-expand-md ~ .page {
    padding-right: 15rem;
  }
  .navbar-vertical.navbar-expand-md ~ .page [class^=container] {
    padding-right: 1.5rem;
    padding-left: 1.5rem;
  }
  .navbar-vertical.navbar-expand-md.navbar-right ~ .page, .navbar-vertical.navbar-expand-md.navbar-end ~ .page {
    padding-right: 0;
    padding-left: 15rem;
  }
  .navbar-vertical.navbar-expand-md .navbar-collapse {
    flex-direction: column;
  }
  .navbar-vertical.navbar-expand-md .navbar-collapse [class^=container] {
    flex-direction: column;
    align-items: stretch;
    padding: 0;
  }
  .navbar-vertical.navbar-expand-md .navbar-collapse .navbar-nav {
    margin-right: 0;
    margin-left: 0;
  }
  .navbar-vertical.navbar-expand-md .navbar-collapse .navbar-nav .nav-link {
    padding: 0.5rem calc(calc(var(--tblr-page-padding) * 2) / 2);
    justify-content: flex-start;
  }
  .navbar-vertical.navbar-expand-md .navbar-collapse .dropdown-menu-columns {
    flex-direction: column;
  }
  .navbar-vertical.navbar-expand-md .navbar-collapse .dropdown-menu {
    padding: 0;
    background: transparent;
    position: static;
    color: inherit;
    box-shadow: none;
    border: none;
    min-width: 0;
    margin: 0;
  }
  .navbar-vertical.navbar-expand-md .navbar-collapse .dropdown-menu .dropdown-item {
    min-width: 0;
    display: flex;
    width: auto;
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 1.75rem);
    color: inherit;
  }
  .navbar-vertical.navbar-expand-md .navbar-collapse .dropdown-menu .dropdown-item.disabled {
    color: var(--tblr-disabled-color);
    pointer-events: none;
    background-color: transparent;
  }
  .navbar-vertical.navbar-expand-md .navbar-collapse .dropdown-menu .dropdown-item.active, .navbar-vertical.navbar-expand-md .navbar-collapse .dropdown-menu .dropdown-item:active {
    background: var(--tblr-navbar-active-bg);
  }
  .navbar-vertical.navbar-expand-md .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 3.25rem);
  }
  .navbar-vertical.navbar-expand-md .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 4.75rem);
  }
  .navbar-vertical.navbar-expand-md .navbar-collapse .dropdown-toggle:after {
    margin-right: auto;
  }
  .navbar-vertical.navbar-expand-md .navbar-collapse .nav-item.active:after {
    border-bottom-width: 0;
    border-right-width: 3px;
    left: auto;
    top: 0;
    bottom: 0;
  }
}
@media (min-width: 992px) {
  .navbar-vertical.navbar-expand-lg {
    width: 15rem;
    position: fixed;
    top: 0;
    right: 0;
    bottom: 0;
    z-index: 1030;
    align-items: flex-start;
    transition: transform 0.3s;
    overflow-y: scroll;
    padding: 0;
  }
}
@media (min-width: 992px) and (prefers-reduced-motion: reduce) {
  .navbar-vertical.navbar-expand-lg {
    transition: none;
  }
}
@media (min-width: 992px) {
  .navbar-vertical.navbar-expand-lg.navbar-right, .navbar-vertical.navbar-expand-lg.navbar-end {
    right: auto;
    left: 0;
  }
  .navbar-vertical.navbar-expand-lg .navbar-brand {
    padding: 0.75rem 0;
    justify-content: center;
  }
  .navbar-vertical.navbar-expand-lg .navbar-collapse {
    align-items: stretch;
  }
  .navbar-vertical.navbar-expand-lg .navbar-nav {
    flex-direction: column;
    flex-grow: 1;
    min-height: auto;
  }
  .navbar-vertical.navbar-expand-lg .navbar-nav .nav-link {
    padding-top: 0.5rem;
    padding-bottom: 0.5rem;
  }
  .navbar-vertical.navbar-expand-lg > [class^=container] {
    flex-direction: column;
    align-items: stretch;
    min-height: 100%;
    justify-content: flex-start;
    padding: 0;
  }
  .navbar-vertical.navbar-expand-lg ~ .page {
    padding-right: 15rem;
  }
  .navbar-vertical.navbar-expand-lg ~ .page [class^=container] {
    padding-right: 1.5rem;
    padding-left: 1.5rem;
  }
  .navbar-vertical.navbar-expand-lg.navbar-right ~ .page, .navbar-vertical.navbar-expand-lg.navbar-end ~ .page {
    padding-right: 0;
    padding-left: 15rem;
  }
  .navbar-vertical.navbar-expand-lg .navbar-collapse {
    flex-direction: column;
  }
  .navbar-vertical.navbar-expand-lg .navbar-collapse [class^=container] {
    flex-direction: column;
    align-items: stretch;
    padding: 0;
  }
  .navbar-vertical.navbar-expand-lg .navbar-collapse .navbar-nav {
    margin-right: 0;
    margin-left: 0;
  }
  .navbar-vertical.navbar-expand-lg .navbar-collapse .navbar-nav .nav-link {
    padding: 0.5rem calc(calc(var(--tblr-page-padding) * 2) / 2);
    justify-content: flex-start;
  }
  .navbar-vertical.navbar-expand-lg .navbar-collapse .dropdown-menu-columns {
    flex-direction: column;
  }
  .navbar-vertical.navbar-expand-lg .navbar-collapse .dropdown-menu {
    padding: 0;
    background: transparent;
    position: static;
    color: inherit;
    box-shadow: none;
    border: none;
    min-width: 0;
    margin: 0;
  }
  .navbar-vertical.navbar-expand-lg .navbar-collapse .dropdown-menu .dropdown-item {
    min-width: 0;
    display: flex;
    width: auto;
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 1.75rem);
    color: inherit;
  }
  .navbar-vertical.navbar-expand-lg .navbar-collapse .dropdown-menu .dropdown-item.disabled {
    color: var(--tblr-disabled-color);
    pointer-events: none;
    background-color: transparent;
  }
  .navbar-vertical.navbar-expand-lg .navbar-collapse .dropdown-menu .dropdown-item.active, .navbar-vertical.navbar-expand-lg .navbar-collapse .dropdown-menu .dropdown-item:active {
    background: var(--tblr-navbar-active-bg);
  }
  .navbar-vertical.navbar-expand-lg .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 3.25rem);
  }
  .navbar-vertical.navbar-expand-lg .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 4.75rem);
  }
  .navbar-vertical.navbar-expand-lg .navbar-collapse .dropdown-toggle:after {
    margin-right: auto;
  }
  .navbar-vertical.navbar-expand-lg .navbar-collapse .nav-item.active:after {
    border-bottom-width: 0;
    border-right-width: 3px;
    left: auto;
    top: 0;
    bottom: 0;
  }
}
@media (min-width: 1200px) {
  .navbar-vertical.navbar-expand-xl {
    width: 15rem;
    position: fixed;
    top: 0;
    right: 0;
    bottom: 0;
    z-index: 1030;
    align-items: flex-start;
    transition: transform 0.3s;
    overflow-y: scroll;
    padding: 0;
  }
}
@media (min-width: 1200px) and (prefers-reduced-motion: reduce) {
  .navbar-vertical.navbar-expand-xl {
    transition: none;
  }
}
@media (min-width: 1200px) {
  .navbar-vertical.navbar-expand-xl.navbar-right, .navbar-vertical.navbar-expand-xl.navbar-end {
    right: auto;
    left: 0;
  }
  .navbar-vertical.navbar-expand-xl .navbar-brand {
    padding: 0.75rem 0;
    justify-content: center;
  }
  .navbar-vertical.navbar-expand-xl .navbar-collapse {
    align-items: stretch;
  }
  .navbar-vertical.navbar-expand-xl .navbar-nav {
    flex-direction: column;
    flex-grow: 1;
    min-height: auto;
  }
  .navbar-vertical.navbar-expand-xl .navbar-nav .nav-link {
    padding-top: 0.5rem;
    padding-bottom: 0.5rem;
  }
  .navbar-vertical.navbar-expand-xl > [class^=container] {
    flex-direction: column;
    align-items: stretch;
    min-height: 100%;
    justify-content: flex-start;
    padding: 0;
  }
  .navbar-vertical.navbar-expand-xl ~ .page {
    padding-right: 15rem;
  }
  .navbar-vertical.navbar-expand-xl ~ .page [class^=container] {
    padding-right: 1.5rem;
    padding-left: 1.5rem;
  }
  .navbar-vertical.navbar-expand-xl.navbar-right ~ .page, .navbar-vertical.navbar-expand-xl.navbar-end ~ .page {
    padding-right: 0;
    padding-left: 15rem;
  }
  .navbar-vertical.navbar-expand-xl .navbar-collapse {
    flex-direction: column;
  }
  .navbar-vertical.navbar-expand-xl .navbar-collapse [class^=container] {
    flex-direction: column;
    align-items: stretch;
    padding: 0;
  }
  .navbar-vertical.navbar-expand-xl .navbar-collapse .navbar-nav {
    margin-right: 0;
    margin-left: 0;
  }
  .navbar-vertical.navbar-expand-xl .navbar-collapse .navbar-nav .nav-link {
    padding: 0.5rem calc(calc(var(--tblr-page-padding) * 2) / 2);
    justify-content: flex-start;
  }
  .navbar-vertical.navbar-expand-xl .navbar-collapse .dropdown-menu-columns {
    flex-direction: column;
  }
  .navbar-vertical.navbar-expand-xl .navbar-collapse .dropdown-menu {
    padding: 0;
    background: transparent;
    position: static;
    color: inherit;
    box-shadow: none;
    border: none;
    min-width: 0;
    margin: 0;
  }
  .navbar-vertical.navbar-expand-xl .navbar-collapse .dropdown-menu .dropdown-item {
    min-width: 0;
    display: flex;
    width: auto;
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 1.75rem);
    color: inherit;
  }
  .navbar-vertical.navbar-expand-xl .navbar-collapse .dropdown-menu .dropdown-item.disabled {
    color: var(--tblr-disabled-color);
    pointer-events: none;
    background-color: transparent;
  }
  .navbar-vertical.navbar-expand-xl .navbar-collapse .dropdown-menu .dropdown-item.active, .navbar-vertical.navbar-expand-xl .navbar-collapse .dropdown-menu .dropdown-item:active {
    background: var(--tblr-navbar-active-bg);
  }
  .navbar-vertical.navbar-expand-xl .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 3.25rem);
  }
  .navbar-vertical.navbar-expand-xl .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 4.75rem);
  }
  .navbar-vertical.navbar-expand-xl .navbar-collapse .dropdown-toggle:after {
    margin-right: auto;
  }
  .navbar-vertical.navbar-expand-xl .navbar-collapse .nav-item.active:after {
    border-bottom-width: 0;
    border-right-width: 3px;
    left: auto;
    top: 0;
    bottom: 0;
  }
}
@media (min-width: 1400px) {
  .navbar-vertical.navbar-expand-xxl {
    width: 15rem;
    position: fixed;
    top: 0;
    right: 0;
    bottom: 0;
    z-index: 1030;
    align-items: flex-start;
    transition: transform 0.3s;
    overflow-y: scroll;
    padding: 0;
  }
}
@media (min-width: 1400px) and (prefers-reduced-motion: reduce) {
  .navbar-vertical.navbar-expand-xxl {
    transition: none;
  }
}
@media (min-width: 1400px) {
  .navbar-vertical.navbar-expand-xxl.navbar-right, .navbar-vertical.navbar-expand-xxl.navbar-end {
    right: auto;
    left: 0;
  }
  .navbar-vertical.navbar-expand-xxl .navbar-brand {
    padding: 0.75rem 0;
    justify-content: center;
  }
  .navbar-vertical.navbar-expand-xxl .navbar-collapse {
    align-items: stretch;
  }
  .navbar-vertical.navbar-expand-xxl .navbar-nav {
    flex-direction: column;
    flex-grow: 1;
    min-height: auto;
  }
  .navbar-vertical.navbar-expand-xxl .navbar-nav .nav-link {
    padding-top: 0.5rem;
    padding-bottom: 0.5rem;
  }
  .navbar-vertical.navbar-expand-xxl > [class^=container] {
    flex-direction: column;
    align-items: stretch;
    min-height: 100%;
    justify-content: flex-start;
    padding: 0;
  }
  .navbar-vertical.navbar-expand-xxl ~ .page {
    padding-right: 15rem;
  }
  .navbar-vertical.navbar-expand-xxl ~ .page [class^=container] {
    padding-right: 1.5rem;
    padding-left: 1.5rem;
  }
  .navbar-vertical.navbar-expand-xxl.navbar-right ~ .page, .navbar-vertical.navbar-expand-xxl.navbar-end ~ .page {
    padding-right: 0;
    padding-left: 15rem;
  }
  .navbar-vertical.navbar-expand-xxl .navbar-collapse {
    flex-direction: column;
  }
  .navbar-vertical.navbar-expand-xxl .navbar-collapse [class^=container] {
    flex-direction: column;
    align-items: stretch;
    padding: 0;
  }
  .navbar-vertical.navbar-expand-xxl .navbar-collapse .navbar-nav {
    margin-right: 0;
    margin-left: 0;
  }
  .navbar-vertical.navbar-expand-xxl .navbar-collapse .navbar-nav .nav-link {
    padding: 0.5rem calc(calc(var(--tblr-page-padding) * 2) / 2);
    justify-content: flex-start;
  }
  .navbar-vertical.navbar-expand-xxl .navbar-collapse .dropdown-menu-columns {
    flex-direction: column;
  }
  .navbar-vertical.navbar-expand-xxl .navbar-collapse .dropdown-menu {
    padding: 0;
    background: transparent;
    position: static;
    color: inherit;
    box-shadow: none;
    border: none;
    min-width: 0;
    margin: 0;
  }
  .navbar-vertical.navbar-expand-xxl .navbar-collapse .dropdown-menu .dropdown-item {
    min-width: 0;
    display: flex;
    width: auto;
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 1.75rem);
    color: inherit;
  }
  .navbar-vertical.navbar-expand-xxl .navbar-collapse .dropdown-menu .dropdown-item.disabled {
    color: var(--tblr-disabled-color);
    pointer-events: none;
    background-color: transparent;
  }
  .navbar-vertical.navbar-expand-xxl .navbar-collapse .dropdown-menu .dropdown-item.active, .navbar-vertical.navbar-expand-xxl .navbar-collapse .dropdown-menu .dropdown-item:active {
    background: var(--tblr-navbar-active-bg);
  }
  .navbar-vertical.navbar-expand-xxl .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 3.25rem);
  }
  .navbar-vertical.navbar-expand-xxl .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-menu .dropdown-item {
    padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 4.75rem);
  }
  .navbar-vertical.navbar-expand-xxl .navbar-collapse .dropdown-toggle:after {
    margin-right: auto;
  }
  .navbar-vertical.navbar-expand-xxl .navbar-collapse .nav-item.active:after {
    border-bottom-width: 0;
    border-right-width: 3px;
    left: auto;
    top: 0;
    bottom: 0;
  }
}
.navbar-vertical.navbar-expand {
  width: 15rem;
  position: fixed;
  top: 0;
  right: 0;
  bottom: 0;
  z-index: 1030;
  align-items: flex-start;
  transition: transform 0.3s;
  overflow-y: scroll;
  padding: 0;
}
@media (prefers-reduced-motion: reduce) {
  .navbar-vertical.navbar-expand {
    transition: none;
  }
}
.navbar-vertical.navbar-expand.navbar-right, .navbar-vertical.navbar-expand.navbar-end {
  right: auto;
  left: 0;
}
.navbar-vertical.navbar-expand .navbar-brand {
  padding: 0.75rem 0;
  justify-content: center;
}
.navbar-vertical.navbar-expand .navbar-collapse {
  align-items: stretch;
}
.navbar-vertical.navbar-expand .navbar-nav {
  flex-direction: column;
  flex-grow: 1;
  min-height: auto;
}
.navbar-vertical.navbar-expand .navbar-nav .nav-link {
  padding-top: 0.5rem;
  padding-bottom: 0.5rem;
}
.navbar-vertical.navbar-expand > [class^=container] {
  flex-direction: column;
  align-items: stretch;
  min-height: 100%;
  justify-content: flex-start;
  padding: 0;
}
.navbar-vertical.navbar-expand ~ .page {
  padding-right: 15rem;
}
.navbar-vertical.navbar-expand ~ .page [class^=container] {
  padding-right: 1.5rem;
  padding-left: 1.5rem;
}
.navbar-vertical.navbar-expand.navbar-right ~ .page, .navbar-vertical.navbar-expand.navbar-end ~ .page {
  padding-right: 0;
  padding-left: 15rem;
}
.navbar-vertical.navbar-expand .navbar-collapse {
  flex-direction: column;
}
.navbar-vertical.navbar-expand .navbar-collapse [class^=container] {
  flex-direction: column;
  align-items: stretch;
  padding: 0;
}
.navbar-vertical.navbar-expand .navbar-collapse .navbar-nav {
  margin-right: 0;
  margin-left: 0;
}
.navbar-vertical.navbar-expand .navbar-collapse .navbar-nav .nav-link {
  padding: 0.5rem calc(calc(var(--tblr-page-padding) * 2) / 2);
  justify-content: flex-start;
}
.navbar-vertical.navbar-expand .navbar-collapse .dropdown-menu-columns {
  flex-direction: column;
}
.navbar-vertical.navbar-expand .navbar-collapse .dropdown-menu {
  padding: 0;
  background: transparent;
  position: static;
  color: inherit;
  box-shadow: none;
  border: none;
  min-width: 0;
  margin: 0;
}
.navbar-vertical.navbar-expand .navbar-collapse .dropdown-menu .dropdown-item {
  min-width: 0;
  display: flex;
  width: auto;
  padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 1.75rem);
  color: inherit;
}
.navbar-vertical.navbar-expand .navbar-collapse .dropdown-menu .dropdown-item.disabled {
  color: var(--tblr-disabled-color);
  pointer-events: none;
  background-color: transparent;
}
.navbar-vertical.navbar-expand .navbar-collapse .dropdown-menu .dropdown-item.active, .navbar-vertical.navbar-expand .navbar-collapse .dropdown-menu .dropdown-item:active {
  background: var(--tblr-navbar-active-bg);
}
.navbar-vertical.navbar-expand .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-item {
  padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 3.25rem);
}
.navbar-vertical.navbar-expand .navbar-collapse .dropdown-menu .dropdown-menu .dropdown-menu .dropdown-item {
  padding-right: calc(calc(calc(var(--tblr-page-padding) * 2) / 2) + 4.75rem);
}
.navbar-vertical.navbar-expand .navbar-collapse .dropdown-toggle:after {
  margin-right: auto;
}
.navbar-vertical.navbar-expand .navbar-collapse .nav-item.active:after {
  border-bottom-width: 0;
  border-right-width: 3px;
  left: auto;
  top: 0;
  bottom: 0;
}

.navbar-overlap:after {
  content: "";
  height: 9rem;
  position: absolute;
  top: 100%;
  right: 0;
  left: 0;
  background: inherit;
  z-index: -1;
  box-shadow: inherit;
}

.page {
  display: flex;
  flex-direction: column;
  position: relative;
  min-height: 100%;
}

.page-center {
  justify-content: center;
}

.page-wrapper {
  flex: 1;
  display: flex;
  flex-direction: column;
}
@media print {
  .page-wrapper {
    margin: 0 !important;
  }
}

.page-wrapper-full .page-body:first-child {
  margin: 0;
  border-top: 0;
}

.page-body {
  margin-top: var(--tblr-page-padding-y);
  margin-bottom: var(--tblr-page-padding-y);
  display: flex;
  flex-direction: column;
  flex: 1;
}

.page-body-card {
  background: var(--tblr-bg-surface);
  border-top: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent);
  padding: var(--tblr-page-padding) 0;
  margin-bottom: 0;
  flex: 1;
}
.page-body ~ .page-body-card {
  margin-top: 0;
}

.page-cover {
  background: no-repeat center/cover;
  min-height: 9rem;
}
@media (min-width: 768px) {
  .page-cover {
    min-height: 12rem;
  }
}
@media (min-width: 992px) {
  .page-cover {
    min-height: 15rem;
  }
}

.page-cover-overlay {
  position: relative;
}
.page-cover-overlay:after {
  content: "";
  position: absolute;
  top: 0;
  right: 0;
  left: 0;
  bottom: 0;
  background-image: linear-gradient(-180deg, rgba(0, 0, 0, 0) 0%, rgba(0, 0, 0, 0.6) 100%);
}

.page-header {
  display: flex;
  flex-wrap: wrap;
  min-height: 2.25rem;
  flex-direction: column;
  justify-content: center;
  max-width: 100%;
}
.page-wrapper .page-header {
  margin: var(--tblr-page-padding-y) 0 0;
}

.page-header-border {
  border-bottom: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
  padding: var(--tblr-page-padding-y) 0;
  margin: 0 !important;
  background-color: var(--tblr-bg-surface);
}

.page-pretitle {
  font-size: 0.75rem;
  font-weight: var(--tblr-font-weight-medium);
  text-transform: uppercase;
  letter-spacing: 0.04em;
  line-height: 1rem;
  color: var(--tblr-secondary);
}

.page-title {
  margin: 0;
  font-size: var(--tblr-font-size-h2);
  line-height: var(--tblr-line-height-h2);
  font-weight: var(--tblr-font-weight-headings);
  color: inherit;
  display: flex;
  align-items: center;
}
.page-title svg {
  width: 1.5rem;
  height: 1.5rem;
  margin-left: 0.25rem;
}

.page-title-lg {
  font-size: 1.5rem;
  line-height: 2rem;
}

.page-subtitle {
  margin-top: 0.25rem;
  color: var(--tblr-secondary);
}

.page-cover {
  --tblr-page-cover-blur: 20px;
  --tblr-page-cover-padding: 1rem;
  min-height: 6rem;
  padding: var(--tblr-page-cover-padding) 0;
  position: relative;
  overflow: hidden;
}

.page-cover-img {
  position: absolute;
  top: calc(-2 * var(--tblr-page-cover-blur, 0));
  right: calc(-2 * var(--tblr-page-cover-blur, 0));
  left: calc(-2 * var(--tblr-page-cover-blur, 0));
  bottom: calc(-2 * var(--tblr-page-cover-blur, 0));
  pointer-events: none;
  filter: blur(var(--tblr-page-cover-blur));
  object-fit: cover;
  background-size: cover;
  background-position: center;
  z-index: -1;
}

.page-tabs {
  margin-top: 0.5rem;
  position: relative;
}

.page-header-tabs .nav-bordered {
  border: 0;
}
.page-header-tabs + .page-body-card {
  margin-top: 0;
}

.footer {
  border-top: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
  background-color: var(--tblr-bg-surface);
  padding: 2rem 0;
  color: var(--tblr-gray-500);
  margin-top: auto;
}

.footer-transparent {
  background-color: transparent;
  border-top: 0;
}

:root:not(.theme-dark):not([data-bs-theme=dark]) .hide-theme-light {
  display: none !important;
}
:root:not(.theme-dark):not([data-bs-theme=dark]) .img-dark {
  display: none !important;
}
:root.theme-dark .hide-theme-dark, :root[data-bs-theme=dark] .hide-theme-dark, body[data-bs-theme=dark] [data-bs-theme=light]:root .hide-theme-dark {
  display: none !important;
}
:root.theme-dark .img-light, :root[data-bs-theme=dark] .img-light, body[data-bs-theme=dark] [data-bs-theme=light]:root .img-light {
  display: none !important;
}

[data-bs-theme=dark], body[data-bs-theme=dark] [data-bs-theme=light] {
  color-scheme: dark;
  --tblr-body-color: var(--tblr-gray-200);
  --tblr-secondary: var(--tblr-gray-400);
  --tblr-body-bg: var(--tblr-gray-900);
  --tblr-emphasis-color: #ffffff;
  --tblr-emphasis-color-rgb: 255, 255, 255;
  --tblr-bg-forms: var(--tblr-gray-900);
  --tblr-bg-surface: var(--tblr-gray-800);
  --tblr-bg-surface-inverted: var(--tblr-gray-100);
  --tblr-bg-surface-secondary: var(--tblr-gray-900);
  --tblr-bg-surface-tertiary: var(--tblr-gray-800);
  --tblr-text-inverted: var(--tblr-gray-800);
  --tblr-link-color: var(--tblr-primary);
  --tblr-link-hover-color: color-mix(in srgb, var(--tblr-primary), black 20%);
  --tblr-active-bg: rgb(34.676744186, 45.8627906977, 61.523255814);
  --tblr-disabled-color: color-mix(in srgb, var(--tblr-body-color) 40%, transparent);
  --tblr-border-color: var(--tblr-gray-700);
  --tblr-border-color-translucent: var(
    --tblr-dark-mode-border-color-translucent
  );
  --tblr-border-dark-color: var(--tblr-dark-mode-border-dark-color);
  --tblr-border-active-color: var(
    --tblr-dark-mode-border-active-color
  );
  --tblr-btn-color: rgb(27.323255814, 36.1372093023, 48.476744186);
}
[data-bs-theme=dark] .navbar-brand-autodark .navbar-brand-image {
  filter: brightness(0) invert(1);
}

.accordion {
  --tblr-accordion-color: var(--tblr-body-color);
  --tblr-accordion-border-color: var(--tblr-border-color);
  --tblr-accordion-border-radius: var(--tblr-border-radius);
  --tblr-accordion-inner-border-radius: calc(var(--tblr-border-radius) - (var(--tblr-border-width)));
  --tblr-accordion-padding-x: 1.25rem;
  --tblr-accordion-gap: 0;
  --tblr-accordion-active-color: inherit;
  --tblr-accordion-btn-color: var(--tblr-accordion-color);
  --tblr-accordion-btn-bg: transparent;
  --tblr-accordion-btn-toggle-width: 1.25rem;
  --tblr-accordion-btn-padding-x: var(--tblr-accordion-padding-x);
  --tblr-accordion-btn-padding-y: 1rem;
  --tblr-accordion-btn-font-weight: var(--tblr-font-weight-medium);
  --tblr-accordion-body-padding-x: var(--tblr-accordion-padding-x);
  --tblr-accordion-body-padding-y: 1rem;
  display: flex;
  flex-direction: column;
  gap: var(--tblr-accordion-gap);
}

.accordion-button {
  position: relative;
  display: flex;
  align-items: center;
  width: 100%;
  padding: var(--tblr-accordion-btn-padding-y) var(--tblr-accordion-padding-x);
  color: inherit;
  text-align: inherit;
  background-color: transparent;
  border: 0;
  font-size: inherit;
  font-weight: var(--tblr-accordion-btn-font-weight);
  gap: 0.75rem;
}
.accordion-button:not(.collapsed) {
  border-bottom-color: transparent;
  box-shadow: none;
  color: var(--tblr-accordion-active-color);
}

.accordion-header {
  margin: 0;
  position: relative;
  display: flex;
  gap: 1rem;
  align-items: center;
  width: 100%;
  color: var(--tblr-accordion-btn-color);
  text-align: right;
  background-color: transparent;
  border: 0;
  overflow-anchor: none;
  transition: transform 0.3s;
}
.accordion-header:hover {
  z-index: 2;
}
.accordion-header:focus {
  z-index: 3;
  outline: 0;
  box-shadow: var(--tblr-accordion-btn-focus-box-shadow);
}
.accordion-header:focus:not(:focus-visible) {
  outline: none;
  box-shadow: none;
}

.accordion-button-icon {
  color: var(--tblr-secondary);
}

.accordion-button-toggle {
  display: flex;
  line-height: 1;
  transition: 0.3s transform;
  margin-right: auto;
  margin-left: 0;
  color: var(--tblr-secondary);
  width: var(--tblr-accordion-btn-toggle-width);
  height: var(--tblr-accordion-btn-toggle-width);
}
.accordion-button:not(.collapsed) .accordion-button-toggle {
  transform: rotate(180deg);
  color: var(--tblr-accordion-active-color);
}
.accordion-button-toggle path {
  transition: 0.3s opacity;
}

.accordion-button:not(.collapsed) .accordion-button-toggle-plus path:first-child {
  opacity: 0;
}

.accordion-item {
  color: var(--tblr-accordion-color);
  border: var(--tblr-border-width) solid var(--tblr-accordion-border-color);
}
.accordion-item:first-of-type {
  border-top-right-radius: var(--tblr-accordion-border-radius);
  border-top-left-radius: var(--tblr-accordion-border-radius);
}
.accordion-item:first-of-type > .accordion-header {
  border-top-right-radius: var(--tblr-accordion-inner-border-radius);
  border-top-left-radius: var(--tblr-accordion-inner-border-radius);
}
.accordion-item:not(:first-of-type) {
  border-top: 0;
}
.accordion-item:last-of-type {
  border-bottom-left-radius: var(--tblr-accordion-border-radius);
  border-bottom-right-radius: var(--tblr-accordion-border-radius);
}
.accordion-item:last-of-type > .accordion-header.collapsed {
  border-bottom-left-radius: var(--tblr-accordion-inner-border-radius);
  border-bottom-right-radius: var(--tblr-accordion-inner-border-radius);
}
.accordion-item:last-of-type > .accordion-collapse {
  border-bottom-left-radius: var(--tblr-accordion-border-radius);
  border-bottom-right-radius: var(--tblr-accordion-border-radius);
}

.accordion-body {
  color: var(--tblr-secondary);
  padding: 0 var(--tblr-accordion-body-padding-x) var(--tblr-accordion-body-padding-y);
}

.accordion-flush > .accordion-item {
  border-left: 0;
  border-right: 0;
  border-radius: 0;
}
.accordion-flush > .accordion-item:first-child {
  border-top: 0;
}
.accordion-flush > .accordion-item:last-child {
  border-bottom: 0;
}
.accordion-flush > .accordion-item > .accordion-collapse,
.accordion-flush > .accordion-item > .accordion-header .accordion-button,
.accordion-flush > .accordion-item > .accordion-header .accordion-button.collapsed {
  border-radius: 0;
}

.accordion-tabs {
  --tblr-accordion-gap: 0.75rem;
}
.accordion-tabs > .accordion-item {
  border: var(--tblr-border-width) solid var(--tblr-accordion-border-color);
  border-radius: var(--tblr-accordion-border-radius);
}

.accordion-inverted .accordion-button-toggle {
  order: -1;
  margin-right: 0;
}

.alert {
  --tblr-alert-color: var(--tblr-body-color);
  --tblr-alert-bg: color-mix(in srgb, var(--tblr-alert-color) 10%, transparent);
  --tblr-alert-padding-x: 1rem;
  --tblr-alert-padding-y: 0.75rem;
  --tblr-alert-margin-bottom: 1rem;
  --tblr-alert-border-color: color-mix(in srgb, var(--tblr-alert-color) 20%, transparent);
  --tblr-alert-border: var(--tblr-border-width) solid var(--tblr-alert-border-color);
  --tblr-alert-border-radius: var(--tblr-border-radius);
  --tblr-alert-link-color: inherit;
  --tblr-alert-heading-font-weight: var(--tblr-font-weight-medium);
  position: relative;
  padding: var(--tblr-alert-padding-y) var(--tblr-alert-padding-x);
  margin-bottom: var(--tblr-alert-margin-bottom);
  background-color: color-mix(in srgb, var(--tblr-alert-bg), var(--tblr-bg-surface));
  border-radius: var(--tblr-alert-border-radius);
  border: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-alert-border-color);
  display: flex;
  flex-direction: row;
  gap: 1rem;
}

.alert-heading {
  color: inherit;
  margin-bottom: 0.25rem;
  font-weight: var(--tblr-alert-heading-font-weight);
}

.alert-description {
  color: var(--tblr-secondary);
}

.alert-icon {
  color: var(--tblr-alert-color);
  width: 1.25rem !important;
  height: 1.25rem !important;
}

.alert-action {
  color: var(--tblr-alert-color);
  text-decoration: underline;
}
.alert-action:hover {
  text-decoration: none;
}

.alert-list {
  margin: 0;
}

.alert-link {
  font-weight: var(--tblr-font-weight-bold);
  color: var(--tblr-alert-link-color);
}
.alert-link, .alert-link:hover {
  color: var(--tblr-alert-color);
}

.alert-dismissible {
  padding-left: 3rem;
}
.alert-dismissible .btn-close {
  position: absolute;
  top: calc(var(--tblr-alert-padding-x) / 2 - 1px);
  left: calc(var(--tblr-alert-padding-y) / 2 - 1px);
  z-index: 1;
  padding: calc(var(--tblr-alert-padding-y) * 1.25) var(--tblr-alert-padding-x);
}

.alert-important {
  border-color: var(--tblr-alert-color);
  background-color: var(--tblr-alert-color);
  color: var(--tblr-white);
}
.alert-important .alert-description {
  color: inherit;
}
.alert-important .alert-icon {
  color: inherit;
}

.alert-minor {
  background: transparent;
  border-color: var(--tblr-border-color);
}

.alert-primary {
  --tblr-alert-color: var(--tblr-primary);
}

.alert-secondary {
  --tblr-alert-color: var(--tblr-secondary);
}

.alert-success {
  --tblr-alert-color: var(--tblr-success);
}

.alert-info {
  --tblr-alert-color: var(--tblr-info);
}

.alert-warning {
  --tblr-alert-color: var(--tblr-warning);
}

.alert-danger {
  --tblr-alert-color: var(--tblr-danger);
}

.alert-light {
  --tblr-alert-color: var(--tblr-light);
}

.alert-dark {
  --tblr-alert-color: var(--tblr-dark);
}

.alert-muted {
  --tblr-alert-color: var(--tblr-muted);
}

.alert-blue {
  --tblr-alert-color: var(--tblr-blue);
}

.alert-azure {
  --tblr-alert-color: var(--tblr-azure);
}

.alert-indigo {
  --tblr-alert-color: var(--tblr-indigo);
}

.alert-purple {
  --tblr-alert-color: var(--tblr-purple);
}

.alert-pink {
  --tblr-alert-color: var(--tblr-pink);
}

.alert-red {
  --tblr-alert-color: var(--tblr-red);
}

.alert-orange {
  --tblr-alert-color: var(--tblr-orange);
}

.alert-yellow {
  --tblr-alert-color: var(--tblr-yellow);
}

.alert-lime {
  --tblr-alert-color: var(--tblr-lime);
}

.alert-green {
  --tblr-alert-color: var(--tblr-green);
}

.alert-teal {
  --tblr-alert-color: var(--tblr-teal);
}

.alert-cyan {
  --tblr-alert-color: var(--tblr-cyan);
}

.avatar {
  --tblr-avatar-size: var(--tblr-avatar-list-size, 2.5rem);
  --tblr-avatar-status-size: 0.75rem;
  --tblr-avatar-bg: var(--tblr-bg-surface-secondary);
  --tblr-avatar-box-shadow-color: var(--tblr-border-color-translucent);
  --tblr-avatar-box-shadow: inset 0 0 0 1px var(--tblr-avatar-box-shadow-color);
  --tblr-avatar-font-size: 1rem;
  --tblr-avatar-icon-size: 1.5rem;
  --tblr-avatar-brand-size: 1.25rem;
  position: relative;
  width: var(--tblr-avatar-size);
  height: var(--tblr-avatar-size);
  font-size: var(--tblr-avatar-font-size);
  font-weight: var(--tblr-font-weight-medium);
  line-height: 1;
  display: inline-flex;
  align-items: center;
  justify-content: center;
  color: var(--tblr-secondary);
  text-align: center;
  text-transform: uppercase;
  vertical-align: bottom;
  user-select: none;
  background: var(--tblr-avatar-bg) no-repeat center/cover;
  border-radius: var(--tblr-border-radius);
  box-shadow: var(--tblr-avatar-box-shadow);
  transition: color 0.3s, background-color 0.3s, box-shadow 0.3s;
}
.avatar .icon {
  width: var(--tblr-avatar-icon-size);
  height: var(--tblr-avatar-icon-size);
}
.avatar .badge {
  position: absolute;
  left: 0;
  bottom: 0;
  border-radius: 100rem;
  box-shadow: 0 0 0 calc(var(--tblr-avatar-status-size) / 4) var(--tblr-bg-surface);
}
a.avatar {
  cursor: pointer;
}
a.avatar:hover {
  color: var(--tblr-primary);
  --tblr-avatar-box-shadow-color: var(--tblr-primary);
}

.avatar-rounded {
  border-radius: 100rem;
}

.avatar-xxs {
  --tblr-avatar-size: 1rem;
  --tblr-avatar-status-size: 0.25rem;
  --tblr-avatar-font-size: 0.5rem;
  --tblr-avatar-icon-size: 0.5rem;
  --tblr-avatar-brand-size: 0.5rem;
}
.avatar-xxs .badge:empty {
  width: 0.25rem;
  height: 0.25rem;
}

.avatar-xs {
  --tblr-avatar-size: 1.25rem;
  --tblr-avatar-status-size: 0.375rem;
  --tblr-avatar-font-size: 0.625rem;
  --tblr-avatar-icon-size: 0.75rem;
  --tblr-avatar-brand-size: 0.75rem;
}
.avatar-xs .badge:empty {
  width: 0.375rem;
  height: 0.375rem;
}

.avatar-sm {
  --tblr-avatar-size: 2rem;
  --tblr-avatar-status-size: 0.5rem;
  --tblr-avatar-font-size: 0.75rem;
  --tblr-avatar-icon-size: 1.5rem;
  --tblr-avatar-brand-size: 1rem;
}
.avatar-sm .badge:empty {
  width: 0.5rem;
  height: 0.5rem;
}

.avatar-md {
  --tblr-avatar-size: 2.5rem;
  --tblr-avatar-status-size: 0.75rem;
  --tblr-avatar-font-size: 0.875rem;
  --tblr-avatar-icon-size: 1.5rem;
  --tblr-avatar-brand-size: 1.25rem;
}
.avatar-md .badge:empty {
  width: 0.75rem;
  height: 0.75rem;
}

.avatar-lg {
  --tblr-avatar-size: 3rem;
  --tblr-avatar-status-size: 0.75rem;
  --tblr-avatar-font-size: 1.25rem;
  --tblr-avatar-icon-size: 2rem;
  --tblr-avatar-brand-size: 1.25rem;
}
.avatar-lg .badge:empty {
  width: 0.75rem;
  height: 0.75rem;
}

.avatar-xl {
  --tblr-avatar-size: 5rem;
  --tblr-avatar-status-size: 1rem;
  --tblr-avatar-font-size: 2rem;
  --tblr-avatar-icon-size: 3rem;
  --tblr-avatar-brand-size: 1.25rem;
}
.avatar-xl .badge:empty {
  width: 1rem;
  height: 1rem;
}

.avatar-2xl {
  --tblr-avatar-size: 7rem;
  --tblr-avatar-status-size: 1rem;
  --tblr-avatar-font-size: 3rem;
  --tblr-avatar-icon-size: 5rem;
  --tblr-avatar-brand-size: 2rem;
}
.avatar-2xl .badge:empty {
  width: 1rem;
  height: 1rem;
}

.avatar-list {
  --tblr-avatar-list-size: 2.5rem;
  --tblr-list-gap: 0.5rem;
  display: flex;
  flex-wrap: wrap;
  gap: var(--tblr-list-gap);
}
.avatar-list a.avatar:hover {
  z-index: 1;
}

.avatar-list-stacked {
  display: block;
  --tblr-list-gap: 0;
}
.avatar-list-stacked .avatar {
  margin-left: calc(-0.5 * var(--tblr-avatar-size)) !important;
  box-shadow: var(--tblr-avatar-box-shadow), 0 0 0 2px var(--tblr-card-bg, var(--tblr-bg-surface));
}

.avatar-list-xxs {
  --tblr-avatar-list-size: 1rem;
}

.avatar-list-xs {
  --tblr-avatar-list-size: 1.25rem;
}

.avatar-list-sm {
  --tblr-avatar-list-size: 2rem;
}

.avatar-list-md {
  --tblr-avatar-list-size: 2.5rem;
}

.avatar-list-lg {
  --tblr-avatar-list-size: 3rem;
}

.avatar-list-xl {
  --tblr-avatar-list-size: 5rem;
}

.avatar-list-2xl {
  --tblr-avatar-list-size: 7rem;
}

.avatar-upload {
  border: var(--tblr-border-width) dashed var(--tblr-border-color);
  background: var(--tblr-bg-forms);
  box-shadow: none;
  flex-direction: column;
  transition: color 0.3s, background-color 0.3s;
}
@media (prefers-reduced-motion: reduce) {
  .avatar-upload {
    transition: none;
  }
}
.avatar-upload svg {
  width: 1.5rem;
  height: 1.5rem;
  stroke-width: 1;
}
.avatar-upload:hover {
  border-color: var(--tblr-primary);
  color: var(--tblr-primary);
  text-decoration: none;
}

.avatar-upload-text {
  font-size: 0.625rem;
  line-height: 1;
  margin-top: 0.25rem;
}

.avatar-cover {
  margin-top: calc(-0.5 * var(--tblr-avatar-size));
  box-shadow: 0 0 0 0.25rem var(--tblr-card-bg, var(--tblr-body-bg));
}

.avatar-brand {
  width: var(--tblr-avatar-brand-size);
  height: var(--tblr-avatar-brand-size);
  position: absolute;
  left: -2px;
  bottom: -2px;
  z-index: 1000;
  background: var(--tblr-bg-surface);
  border-radius: var(--tblr-border-radius);
  border: 1px solid var(--tblr-border-color);
}

.badge {
  --tblr-badge-padding-x: 0.5em;
  --tblr-badge-padding-y: 0.25em;
  --tblr-badge-font-size: 0.85714285em;
  --tblr-badge-font-weight: var(--tblr-font-weight-medium);
  --tblr-badge-color: var(--tblr-secondary);
  --tblr-badge-border-radius: var(--tblr-border-radius);
  --tblr-badge-icon-size: 1em;
  --tblr-badge-line-height: 1;
  display: inline-flex;
  padding: var(--tblr-badge-padding-y) var(--tblr-badge-padding-x);
  font-weight: var(--tblr-badge-font-weight);
  font-size: var(--tblr-badge-font-size);
  color: var(--tblr-badge-color);
  text-align: center;
  white-space: nowrap;
  justify-content: center;
  align-items: center;
  gap: 0.25rem;
  background: var(--tblr-bg-surface-secondary);
  overflow: hidden;
  user-select: none;
  border: var(--tblr-border-width) var(--tblr-border-style) transparent;
  border-radius: var(--tblr-badge-border-radius);
  min-width: calc(1em + var(--tblr-badge-padding-y) * 2 + 2px);
  letter-spacing: 0.04em;
  vertical-align: bottom;
  line-height: var(--tblr-badge-line-height);
}
a.badge {
  background: var(--tblr-bg-surface-secondary);
}

.badge .icon {
  width: 1em;
  height: 1em;
  font-size: var(--tblr-badge-icon-size);
  stroke-width: 2;
}

.badge:empty,
.badge-dot {
  display: inline-block;
  width: 10px;
  height: 10px;
  min-width: 0;
  min-height: auto;
  padding: 0;
  border-radius: 100rem;
  vertical-align: baseline;
}

.badge-outline {
  background-color: transparent;
  border: var(--tblr-border-width) var(--tblr-border-style) currentColor;
}

.badge-pill {
  border-radius: 100rem;
}

.badges-list {
  --tblr-list-gap: 0.5rem;
  display: flex;
  flex-wrap: wrap;
  gap: var(--tblr-list-gap);
}

.badge-notification {
  position: absolute !important;
  top: 0 !important;
  left: 0 !important;
  transform: translate(-50%, -50%);
  z-index: 1;
}

.badge-blink {
  animation: blink 2s infinite;
}

.badge-sm {
  --tblr-badge-font-size: 0.71428571em;
  --tblr-badge-icon-size: 1em;
  --tblr-badge-padding-y: 2px;
  --tblr-badge-padding-x: 0.25rem;
}

.badge-lg {
  --tblr-badge-font-size: 1em;
  --tblr-badge-icon-size: 1em;
  --tblr-badge-padding-y: 0.25rem;
  --tblr-badge-padding-x: 0.5rem;
}

.badge-icononly {
  --tblr-badge-padding-x: 0;
}

.breadcrumb {
  --tblr-breadcrumb-padding-x: 0;
  --tblr-breadcrumb-padding-y: 0;
  --tblr-breadcrumb-margin-bottom: 1rem;
  --tblr-breadcrumb-font-size: ;
  --tblr-breadcrumb-bg: ;
  --tblr-breadcrumb-border-radius: ;
  --tblr-breadcrumb-divider-color: var(--tblr-gray-500);
  --tblr-breadcrumb-item-padding-x: 0.5rem;
  --tblr-breadcrumb-item-active-color: inherit;
  --tblr-breadcrumb-item-active-font-weight: var(--tblr-font-weight-bold);
  --tblr-breadcrumb-item-disabled-color: var(--tblr-disabled-color);
  --tblr-breadcrumb-link-color: var(--tblr-link-color);
  display: flex;
  flex-wrap: wrap;
  font-size: var(--tblr-breadcrumb-font-size);
  list-style: none;
  background-color: var(--tblr-breadcrumb-bg);
  border-radius: var(--tblr-breadcrumb-border-radius);
  padding: 0;
  margin: 0;
  background: transparent;
}
.breadcrumb a {
  color: var(--tblr-breadcrumb-link-color);
}
.breadcrumb a:hover {
  text-decoration: underline;
}

.breadcrumb-muted {
  --tblr-breadcrumb-link-color: var(--tblr-secondary);
}

.breadcrumb-item.active {
  color: var(--tblr-breadcrumb-item-active-color);
  font-weight: var(--tblr-breadcrumb-item-active-font-weight);
}
.breadcrumb-item.active a {
  color: inherit;
  pointer-events: none;
}
.breadcrumb-item.disabled {
  color: var(--tblr-breadcrumb-item-disabled-color);
}
.breadcrumb-item.disabled:before {
  color: inherit;
}
.breadcrumb-item.disabled a {
  color: inherit;
  pointer-events: none;
}
.breadcrumb-item + .breadcrumb-item {
  padding-right: var(--tblr-breadcrumb-item-padding-x);
}
.breadcrumb-item + .breadcrumb-item::before {
  float: right;
  padding-left: var(--tblr-breadcrumb-item-padding-x);
  color: var(--tblr-breadcrumb-divider-color);
  content: var(--tblr-breadcrumb-divider, "/");
  transform: scaleX(-1);
}

.breadcrumb-dots {
  --tblr-breadcrumb-divider: "·";
}

.breadcrumb-arrows {
  --tblr-breadcrumb-divider: "›";
}

.breadcrumb-bullets {
  --tblr-breadcrumb-divider: "•";
}

.btn {
  --tblr-btn-icon-size: 1.25rem;
  --tblr-btn-icon-color: inherit;
  --tblr-btn-bg: var(--tblr-bg-surface);
  --tblr-btn-color: var(--tblr-body-color);
  --tblr-btn-border-color: var(--tblr-border-color);
  --tblr-btn-hover-bg: var(--tblr-btn-bg);
  --tblr-btn-hover-border-color: var(--tblr-border-active-color);
  --tblr-btn-active-color: var(--tblr-primary);
  --tblr-btn-active-bg: rgba(var(--tblr-primary-rgb), 0.04);
  --tblr-btn-active-border-color: var(--tblr-primary);
  display: inline-flex;
  align-items: center;
  justify-content: center;
  white-space: nowrap;
  box-shadow: var(--tblr-btn-box-shadow);
  position: relative;
  min-width: calc(var(--tblr-btn-line-height) * 1 + var(--tblr-btn-padding-y) * 2 + var(--tblr-btn-border-width) * 2);
  min-height: calc(var(--tblr-btn-line-height) * 1 + var(--tblr-btn-padding-y) * 2 + var(--tblr-btn-border-width) * 2);
}
.btn .icon {
  width: var(--tblr-btn-icon-size);
  height: var(--tblr-btn-icon-size);
  min-width: var(--tblr-btn-icon-size);
  font-size: var(--tblr-btn-icon-size);
  margin: 0 calc(var(--tblr-btn-padding-x) / -4) 0 calc(var(--tblr-btn-padding-x) / 2);
  vertical-align: bottom;
  color: var(--tblr-btn-icon-color);
}
.btn .avatar {
  width: var(--tblr-btn-icon-size);
  height: var(--tblr-btn-icon-size);
  margin: 0 calc(var(--tblr-btn-padding-x) / -4) 0 calc(var(--tblr-btn-padding-x) / 2);
}
.btn .icon-right,
.btn .icon-end {
  margin: 0 calc(var(--tblr-btn-padding-x) / 2) 0 calc(var(--tblr-btn-padding-x) / -4);
}
.btn .badge {
  top: auto;
}
.btn-check + .btn:hover {
  color: var(--tblr-btn-hover-color);
  background-color: var(--tblr-btn-hover-bg);
  border-color: var(--tblr-btn-hover-border-color);
}

.btn-link {
  color: rgb(6.711627907, 124.1651162791, 233.788372093);
  background-color: transparent;
  border-color: transparent;
  box-shadow: none;
}
.btn-link .icon {
  color: inherit;
}
.btn-link:hover {
  color: rgb(4.8, 88.8, 167.2);
  border-color: transparent;
}

.btn-primary {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-primary-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-primary);
  --tblr-btn-hover-color: var(--tblr-primary-fg);
  --tblr-btn-hover-bg: var(--tblr-primary-darken);
  --tblr-btn-active-color: var(--tblr-primary-fg);
  --tblr-btn-active-bg: var(--tblr-primary-darken);
  --tblr-btn-disabled-bg: var(--tblr-primary);
  --tblr-btn-disabled-color: var(--tblr-primary-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-primary,
.btn-outline.btn-primary {
  --tblr-btn-color: var(--tblr-primary);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-primary);
  --tblr-btn-hover-color: var(--tblr-primary-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-primary);
  --tblr-btn-active-color: var(--tblr-primary-fg);
  --tblr-btn-active-bg: var(--tblr-primary);
  --tblr-btn-active-border-color: var(--tblr-primary);
  --tblr-btn-disabled-color: var(--tblr-primary);
  --tblr-btn-disabled-border-color: var(--tblr-primary);
}

.btn-ghost-primary,
.btn-ghost.btn-primary {
  --tblr-btn-color: var(--tblr-primary);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-primary-fg);
  --tblr-btn-hover-bg: var(--tblr-primary);
  --tblr-btn-hover-border-color: var(--tblr-primary);
  --tblr-btn-active-color: var(--tblr-primary-fg);
  --tblr-btn-active-bg: var(--tblr-primary);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-primary);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-secondary {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-secondary-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-secondary);
  --tblr-btn-hover-color: var(--tblr-secondary-fg);
  --tblr-btn-hover-bg: var(--tblr-secondary-darken);
  --tblr-btn-active-color: var(--tblr-secondary-fg);
  --tblr-btn-active-bg: var(--tblr-secondary-darken);
  --tblr-btn-disabled-bg: var(--tblr-secondary);
  --tblr-btn-disabled-color: var(--tblr-secondary-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-secondary,
.btn-outline.btn-secondary {
  --tblr-btn-color: var(--tblr-secondary);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-secondary);
  --tblr-btn-hover-color: var(--tblr-secondary-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-secondary);
  --tblr-btn-active-color: var(--tblr-secondary-fg);
  --tblr-btn-active-bg: var(--tblr-secondary);
  --tblr-btn-active-border-color: var(--tblr-secondary);
  --tblr-btn-disabled-color: var(--tblr-secondary);
  --tblr-btn-disabled-border-color: var(--tblr-secondary);
}

.btn-ghost-secondary,
.btn-ghost.btn-secondary {
  --tblr-btn-color: var(--tblr-secondary);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-secondary-fg);
  --tblr-btn-hover-bg: var(--tblr-secondary);
  --tblr-btn-hover-border-color: var(--tblr-secondary);
  --tblr-btn-active-color: var(--tblr-secondary-fg);
  --tblr-btn-active-bg: var(--tblr-secondary);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-secondary);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-success {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-success-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-success);
  --tblr-btn-hover-color: var(--tblr-success-fg);
  --tblr-btn-hover-bg: var(--tblr-success-darken);
  --tblr-btn-active-color: var(--tblr-success-fg);
  --tblr-btn-active-bg: var(--tblr-success-darken);
  --tblr-btn-disabled-bg: var(--tblr-success);
  --tblr-btn-disabled-color: var(--tblr-success-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-success,
.btn-outline.btn-success {
  --tblr-btn-color: var(--tblr-success);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-success);
  --tblr-btn-hover-color: var(--tblr-success-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-success);
  --tblr-btn-active-color: var(--tblr-success-fg);
  --tblr-btn-active-bg: var(--tblr-success);
  --tblr-btn-active-border-color: var(--tblr-success);
  --tblr-btn-disabled-color: var(--tblr-success);
  --tblr-btn-disabled-border-color: var(--tblr-success);
}

.btn-ghost-success,
.btn-ghost.btn-success {
  --tblr-btn-color: var(--tblr-success);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-success-fg);
  --tblr-btn-hover-bg: var(--tblr-success);
  --tblr-btn-hover-border-color: var(--tblr-success);
  --tblr-btn-active-color: var(--tblr-success-fg);
  --tblr-btn-active-bg: var(--tblr-success);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-success);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-info {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-info-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-info);
  --tblr-btn-hover-color: var(--tblr-info-fg);
  --tblr-btn-hover-bg: var(--tblr-info-darken);
  --tblr-btn-active-color: var(--tblr-info-fg);
  --tblr-btn-active-bg: var(--tblr-info-darken);
  --tblr-btn-disabled-bg: var(--tblr-info);
  --tblr-btn-disabled-color: var(--tblr-info-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-info,
.btn-outline.btn-info {
  --tblr-btn-color: var(--tblr-info);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-info);
  --tblr-btn-hover-color: var(--tblr-info-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-info);
  --tblr-btn-active-color: var(--tblr-info-fg);
  --tblr-btn-active-bg: var(--tblr-info);
  --tblr-btn-active-border-color: var(--tblr-info);
  --tblr-btn-disabled-color: var(--tblr-info);
  --tblr-btn-disabled-border-color: var(--tblr-info);
}

.btn-ghost-info,
.btn-ghost.btn-info {
  --tblr-btn-color: var(--tblr-info);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-info-fg);
  --tblr-btn-hover-bg: var(--tblr-info);
  --tblr-btn-hover-border-color: var(--tblr-info);
  --tblr-btn-active-color: var(--tblr-info-fg);
  --tblr-btn-active-bg: var(--tblr-info);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-info);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-warning {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-warning-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-warning);
  --tblr-btn-hover-color: var(--tblr-warning-fg);
  --tblr-btn-hover-bg: var(--tblr-warning-darken);
  --tblr-btn-active-color: var(--tblr-warning-fg);
  --tblr-btn-active-bg: var(--tblr-warning-darken);
  --tblr-btn-disabled-bg: var(--tblr-warning);
  --tblr-btn-disabled-color: var(--tblr-warning-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-warning,
.btn-outline.btn-warning {
  --tblr-btn-color: var(--tblr-warning);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-warning);
  --tblr-btn-hover-color: var(--tblr-warning-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-warning);
  --tblr-btn-active-color: var(--tblr-warning-fg);
  --tblr-btn-active-bg: var(--tblr-warning);
  --tblr-btn-active-border-color: var(--tblr-warning);
  --tblr-btn-disabled-color: var(--tblr-warning);
  --tblr-btn-disabled-border-color: var(--tblr-warning);
}

.btn-ghost-warning,
.btn-ghost.btn-warning {
  --tblr-btn-color: var(--tblr-warning);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-warning-fg);
  --tblr-btn-hover-bg: var(--tblr-warning);
  --tblr-btn-hover-border-color: var(--tblr-warning);
  --tblr-btn-active-color: var(--tblr-warning-fg);
  --tblr-btn-active-bg: var(--tblr-warning);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-warning);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-danger {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-danger-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-danger);
  --tblr-btn-hover-color: var(--tblr-danger-fg);
  --tblr-btn-hover-bg: var(--tblr-danger-darken);
  --tblr-btn-active-color: var(--tblr-danger-fg);
  --tblr-btn-active-bg: var(--tblr-danger-darken);
  --tblr-btn-disabled-bg: var(--tblr-danger);
  --tblr-btn-disabled-color: var(--tblr-danger-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-danger,
.btn-outline.btn-danger {
  --tblr-btn-color: var(--tblr-danger);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-danger);
  --tblr-btn-hover-color: var(--tblr-danger-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-danger);
  --tblr-btn-active-color: var(--tblr-danger-fg);
  --tblr-btn-active-bg: var(--tblr-danger);
  --tblr-btn-active-border-color: var(--tblr-danger);
  --tblr-btn-disabled-color: var(--tblr-danger);
  --tblr-btn-disabled-border-color: var(--tblr-danger);
}

.btn-ghost-danger,
.btn-ghost.btn-danger {
  --tblr-btn-color: var(--tblr-danger);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-danger-fg);
  --tblr-btn-hover-bg: var(--tblr-danger);
  --tblr-btn-hover-border-color: var(--tblr-danger);
  --tblr-btn-active-color: var(--tblr-danger-fg);
  --tblr-btn-active-bg: var(--tblr-danger);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-danger);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-light {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-light-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-light);
  --tblr-btn-hover-color: var(--tblr-light-fg);
  --tblr-btn-hover-bg: var(--tblr-light-darken);
  --tblr-btn-active-color: var(--tblr-light-fg);
  --tblr-btn-active-bg: var(--tblr-light-darken);
  --tblr-btn-disabled-bg: var(--tblr-light);
  --tblr-btn-disabled-color: var(--tblr-light-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-light,
.btn-outline.btn-light {
  --tblr-btn-color: var(--tblr-light);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-light);
  --tblr-btn-hover-color: var(--tblr-light-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-light);
  --tblr-btn-active-color: var(--tblr-light-fg);
  --tblr-btn-active-bg: var(--tblr-light);
  --tblr-btn-active-border-color: var(--tblr-light);
  --tblr-btn-disabled-color: var(--tblr-light);
  --tblr-btn-disabled-border-color: var(--tblr-light);
}

.btn-ghost-light,
.btn-ghost.btn-light {
  --tblr-btn-color: var(--tblr-light);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-light-fg);
  --tblr-btn-hover-bg: var(--tblr-light);
  --tblr-btn-hover-border-color: var(--tblr-light);
  --tblr-btn-active-color: var(--tblr-light-fg);
  --tblr-btn-active-bg: var(--tblr-light);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-light);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-dark {
  --tblr-btn-border-color: var(--tblr-dark-mode-border-color);
  --tblr-btn-hover-border-color: var(--tblr-dark-mode-border-active-color);
  --tblr-btn-active-border-color: var(--tblr-dark-mode-border-active-color);
  --tblr-btn-color: var(--tblr-dark-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-dark);
  --tblr-btn-hover-color: var(--tblr-dark-fg);
  --tblr-btn-hover-bg: var(--tblr-dark-darken);
  --tblr-btn-active-color: var(--tblr-dark-fg);
  --tblr-btn-active-bg: var(--tblr-dark-darken);
  --tblr-btn-disabled-bg: var(--tblr-dark);
  --tblr-btn-disabled-color: var(--tblr-dark-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-dark,
.btn-outline.btn-dark {
  --tblr-btn-color: var(--tblr-dark);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-dark);
  --tblr-btn-hover-color: var(--tblr-dark-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-dark);
  --tblr-btn-active-color: var(--tblr-dark-fg);
  --tblr-btn-active-bg: var(--tblr-dark);
  --tblr-btn-active-border-color: var(--tblr-dark);
  --tblr-btn-disabled-color: var(--tblr-dark);
  --tblr-btn-disabled-border-color: var(--tblr-dark);
}

.btn-ghost-dark,
.btn-ghost.btn-dark {
  --tblr-btn-color: var(--tblr-dark);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-dark-fg);
  --tblr-btn-hover-bg: var(--tblr-dark);
  --tblr-btn-hover-border-color: var(--tblr-dark);
  --tblr-btn-active-color: var(--tblr-dark-fg);
  --tblr-btn-active-bg: var(--tblr-dark);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-dark);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-muted {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-muted-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-muted);
  --tblr-btn-hover-color: var(--tblr-muted-fg);
  --tblr-btn-hover-bg: var(--tblr-muted-darken);
  --tblr-btn-active-color: var(--tblr-muted-fg);
  --tblr-btn-active-bg: var(--tblr-muted-darken);
  --tblr-btn-disabled-bg: var(--tblr-muted);
  --tblr-btn-disabled-color: var(--tblr-muted-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-muted,
.btn-outline.btn-muted {
  --tblr-btn-color: var(--tblr-muted);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-muted);
  --tblr-btn-hover-color: var(--tblr-muted-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-muted);
  --tblr-btn-active-color: var(--tblr-muted-fg);
  --tblr-btn-active-bg: var(--tblr-muted);
  --tblr-btn-active-border-color: var(--tblr-muted);
  --tblr-btn-disabled-color: var(--tblr-muted);
  --tblr-btn-disabled-border-color: var(--tblr-muted);
}

.btn-ghost-muted,
.btn-ghost.btn-muted {
  --tblr-btn-color: var(--tblr-muted);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-muted-fg);
  --tblr-btn-hover-bg: var(--tblr-muted);
  --tblr-btn-hover-border-color: var(--tblr-muted);
  --tblr-btn-active-color: var(--tblr-muted-fg);
  --tblr-btn-active-bg: var(--tblr-muted);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-muted);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-blue {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-blue-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-blue);
  --tblr-btn-hover-color: var(--tblr-blue-fg);
  --tblr-btn-hover-bg: var(--tblr-blue-darken);
  --tblr-btn-active-color: var(--tblr-blue-fg);
  --tblr-btn-active-bg: var(--tblr-blue-darken);
  --tblr-btn-disabled-bg: var(--tblr-blue);
  --tblr-btn-disabled-color: var(--tblr-blue-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-blue,
.btn-outline.btn-blue {
  --tblr-btn-color: var(--tblr-blue);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-blue);
  --tblr-btn-hover-color: var(--tblr-blue-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-blue);
  --tblr-btn-active-color: var(--tblr-blue-fg);
  --tblr-btn-active-bg: var(--tblr-blue);
  --tblr-btn-active-border-color: var(--tblr-blue);
  --tblr-btn-disabled-color: var(--tblr-blue);
  --tblr-btn-disabled-border-color: var(--tblr-blue);
}

.btn-ghost-blue,
.btn-ghost.btn-blue {
  --tblr-btn-color: var(--tblr-blue);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-blue-fg);
  --tblr-btn-hover-bg: var(--tblr-blue);
  --tblr-btn-hover-border-color: var(--tblr-blue);
  --tblr-btn-active-color: var(--tblr-blue-fg);
  --tblr-btn-active-bg: var(--tblr-blue);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-blue);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-azure {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-azure-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-azure);
  --tblr-btn-hover-color: var(--tblr-azure-fg);
  --tblr-btn-hover-bg: var(--tblr-azure-darken);
  --tblr-btn-active-color: var(--tblr-azure-fg);
  --tblr-btn-active-bg: var(--tblr-azure-darken);
  --tblr-btn-disabled-bg: var(--tblr-azure);
  --tblr-btn-disabled-color: var(--tblr-azure-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-azure,
.btn-outline.btn-azure {
  --tblr-btn-color: var(--tblr-azure);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-azure);
  --tblr-btn-hover-color: var(--tblr-azure-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-azure);
  --tblr-btn-active-color: var(--tblr-azure-fg);
  --tblr-btn-active-bg: var(--tblr-azure);
  --tblr-btn-active-border-color: var(--tblr-azure);
  --tblr-btn-disabled-color: var(--tblr-azure);
  --tblr-btn-disabled-border-color: var(--tblr-azure);
}

.btn-ghost-azure,
.btn-ghost.btn-azure {
  --tblr-btn-color: var(--tblr-azure);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-azure-fg);
  --tblr-btn-hover-bg: var(--tblr-azure);
  --tblr-btn-hover-border-color: var(--tblr-azure);
  --tblr-btn-active-color: var(--tblr-azure-fg);
  --tblr-btn-active-bg: var(--tblr-azure);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-azure);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-indigo {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-indigo-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-indigo);
  --tblr-btn-hover-color: var(--tblr-indigo-fg);
  --tblr-btn-hover-bg: var(--tblr-indigo-darken);
  --tblr-btn-active-color: var(--tblr-indigo-fg);
  --tblr-btn-active-bg: var(--tblr-indigo-darken);
  --tblr-btn-disabled-bg: var(--tblr-indigo);
  --tblr-btn-disabled-color: var(--tblr-indigo-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-indigo,
.btn-outline.btn-indigo {
  --tblr-btn-color: var(--tblr-indigo);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-indigo);
  --tblr-btn-hover-color: var(--tblr-indigo-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-indigo);
  --tblr-btn-active-color: var(--tblr-indigo-fg);
  --tblr-btn-active-bg: var(--tblr-indigo);
  --tblr-btn-active-border-color: var(--tblr-indigo);
  --tblr-btn-disabled-color: var(--tblr-indigo);
  --tblr-btn-disabled-border-color: var(--tblr-indigo);
}

.btn-ghost-indigo,
.btn-ghost.btn-indigo {
  --tblr-btn-color: var(--tblr-indigo);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-indigo-fg);
  --tblr-btn-hover-bg: var(--tblr-indigo);
  --tblr-btn-hover-border-color: var(--tblr-indigo);
  --tblr-btn-active-color: var(--tblr-indigo-fg);
  --tblr-btn-active-bg: var(--tblr-indigo);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-indigo);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-purple {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-purple-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-purple);
  --tblr-btn-hover-color: var(--tblr-purple-fg);
  --tblr-btn-hover-bg: var(--tblr-purple-darken);
  --tblr-btn-active-color: var(--tblr-purple-fg);
  --tblr-btn-active-bg: var(--tblr-purple-darken);
  --tblr-btn-disabled-bg: var(--tblr-purple);
  --tblr-btn-disabled-color: var(--tblr-purple-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-purple,
.btn-outline.btn-purple {
  --tblr-btn-color: var(--tblr-purple);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-purple);
  --tblr-btn-hover-color: var(--tblr-purple-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-purple);
  --tblr-btn-active-color: var(--tblr-purple-fg);
  --tblr-btn-active-bg: var(--tblr-purple);
  --tblr-btn-active-border-color: var(--tblr-purple);
  --tblr-btn-disabled-color: var(--tblr-purple);
  --tblr-btn-disabled-border-color: var(--tblr-purple);
}

.btn-ghost-purple,
.btn-ghost.btn-purple {
  --tblr-btn-color: var(--tblr-purple);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-purple-fg);
  --tblr-btn-hover-bg: var(--tblr-purple);
  --tblr-btn-hover-border-color: var(--tblr-purple);
  --tblr-btn-active-color: var(--tblr-purple-fg);
  --tblr-btn-active-bg: var(--tblr-purple);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-purple);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-pink {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-pink-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-pink);
  --tblr-btn-hover-color: var(--tblr-pink-fg);
  --tblr-btn-hover-bg: var(--tblr-pink-darken);
  --tblr-btn-active-color: var(--tblr-pink-fg);
  --tblr-btn-active-bg: var(--tblr-pink-darken);
  --tblr-btn-disabled-bg: var(--tblr-pink);
  --tblr-btn-disabled-color: var(--tblr-pink-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-pink,
.btn-outline.btn-pink {
  --tblr-btn-color: var(--tblr-pink);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-pink);
  --tblr-btn-hover-color: var(--tblr-pink-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-pink);
  --tblr-btn-active-color: var(--tblr-pink-fg);
  --tblr-btn-active-bg: var(--tblr-pink);
  --tblr-btn-active-border-color: var(--tblr-pink);
  --tblr-btn-disabled-color: var(--tblr-pink);
  --tblr-btn-disabled-border-color: var(--tblr-pink);
}

.btn-ghost-pink,
.btn-ghost.btn-pink {
  --tblr-btn-color: var(--tblr-pink);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-pink-fg);
  --tblr-btn-hover-bg: var(--tblr-pink);
  --tblr-btn-hover-border-color: var(--tblr-pink);
  --tblr-btn-active-color: var(--tblr-pink-fg);
  --tblr-btn-active-bg: var(--tblr-pink);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-pink);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-red {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-red-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-red);
  --tblr-btn-hover-color: var(--tblr-red-fg);
  --tblr-btn-hover-bg: var(--tblr-red-darken);
  --tblr-btn-active-color: var(--tblr-red-fg);
  --tblr-btn-active-bg: var(--tblr-red-darken);
  --tblr-btn-disabled-bg: var(--tblr-red);
  --tblr-btn-disabled-color: var(--tblr-red-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-red,
.btn-outline.btn-red {
  --tblr-btn-color: var(--tblr-red);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-red);
  --tblr-btn-hover-color: var(--tblr-red-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-red);
  --tblr-btn-active-color: var(--tblr-red-fg);
  --tblr-btn-active-bg: var(--tblr-red);
  --tblr-btn-active-border-color: var(--tblr-red);
  --tblr-btn-disabled-color: var(--tblr-red);
  --tblr-btn-disabled-border-color: var(--tblr-red);
}

.btn-ghost-red,
.btn-ghost.btn-red {
  --tblr-btn-color: var(--tblr-red);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-red-fg);
  --tblr-btn-hover-bg: var(--tblr-red);
  --tblr-btn-hover-border-color: var(--tblr-red);
  --tblr-btn-active-color: var(--tblr-red-fg);
  --tblr-btn-active-bg: var(--tblr-red);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-red);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-orange {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-orange-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-orange);
  --tblr-btn-hover-color: var(--tblr-orange-fg);
  --tblr-btn-hover-bg: var(--tblr-orange-darken);
  --tblr-btn-active-color: var(--tblr-orange-fg);
  --tblr-btn-active-bg: var(--tblr-orange-darken);
  --tblr-btn-disabled-bg: var(--tblr-orange);
  --tblr-btn-disabled-color: var(--tblr-orange-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-orange,
.btn-outline.btn-orange {
  --tblr-btn-color: var(--tblr-orange);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-orange);
  --tblr-btn-hover-color: var(--tblr-orange-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-orange);
  --tblr-btn-active-color: var(--tblr-orange-fg);
  --tblr-btn-active-bg: var(--tblr-orange);
  --tblr-btn-active-border-color: var(--tblr-orange);
  --tblr-btn-disabled-color: var(--tblr-orange);
  --tblr-btn-disabled-border-color: var(--tblr-orange);
}

.btn-ghost-orange,
.btn-ghost.btn-orange {
  --tblr-btn-color: var(--tblr-orange);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-orange-fg);
  --tblr-btn-hover-bg: var(--tblr-orange);
  --tblr-btn-hover-border-color: var(--tblr-orange);
  --tblr-btn-active-color: var(--tblr-orange-fg);
  --tblr-btn-active-bg: var(--tblr-orange);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-orange);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-yellow {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-yellow-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-yellow);
  --tblr-btn-hover-color: var(--tblr-yellow-fg);
  --tblr-btn-hover-bg: var(--tblr-yellow-darken);
  --tblr-btn-active-color: var(--tblr-yellow-fg);
  --tblr-btn-active-bg: var(--tblr-yellow-darken);
  --tblr-btn-disabled-bg: var(--tblr-yellow);
  --tblr-btn-disabled-color: var(--tblr-yellow-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-yellow,
.btn-outline.btn-yellow {
  --tblr-btn-color: var(--tblr-yellow);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-yellow);
  --tblr-btn-hover-color: var(--tblr-yellow-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-yellow);
  --tblr-btn-active-color: var(--tblr-yellow-fg);
  --tblr-btn-active-bg: var(--tblr-yellow);
  --tblr-btn-active-border-color: var(--tblr-yellow);
  --tblr-btn-disabled-color: var(--tblr-yellow);
  --tblr-btn-disabled-border-color: var(--tblr-yellow);
}

.btn-ghost-yellow,
.btn-ghost.btn-yellow {
  --tblr-btn-color: var(--tblr-yellow);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-yellow-fg);
  --tblr-btn-hover-bg: var(--tblr-yellow);
  --tblr-btn-hover-border-color: var(--tblr-yellow);
  --tblr-btn-active-color: var(--tblr-yellow-fg);
  --tblr-btn-active-bg: var(--tblr-yellow);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-yellow);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-lime {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-lime-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-lime);
  --tblr-btn-hover-color: var(--tblr-lime-fg);
  --tblr-btn-hover-bg: var(--tblr-lime-darken);
  --tblr-btn-active-color: var(--tblr-lime-fg);
  --tblr-btn-active-bg: var(--tblr-lime-darken);
  --tblr-btn-disabled-bg: var(--tblr-lime);
  --tblr-btn-disabled-color: var(--tblr-lime-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-lime,
.btn-outline.btn-lime {
  --tblr-btn-color: var(--tblr-lime);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-lime);
  --tblr-btn-hover-color: var(--tblr-lime-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-lime);
  --tblr-btn-active-color: var(--tblr-lime-fg);
  --tblr-btn-active-bg: var(--tblr-lime);
  --tblr-btn-active-border-color: var(--tblr-lime);
  --tblr-btn-disabled-color: var(--tblr-lime);
  --tblr-btn-disabled-border-color: var(--tblr-lime);
}

.btn-ghost-lime,
.btn-ghost.btn-lime {
  --tblr-btn-color: var(--tblr-lime);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-lime-fg);
  --tblr-btn-hover-bg: var(--tblr-lime);
  --tblr-btn-hover-border-color: var(--tblr-lime);
  --tblr-btn-active-color: var(--tblr-lime-fg);
  --tblr-btn-active-bg: var(--tblr-lime);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-lime);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-green {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-green-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-green);
  --tblr-btn-hover-color: var(--tblr-green-fg);
  --tblr-btn-hover-bg: var(--tblr-green-darken);
  --tblr-btn-active-color: var(--tblr-green-fg);
  --tblr-btn-active-bg: var(--tblr-green-darken);
  --tblr-btn-disabled-bg: var(--tblr-green);
  --tblr-btn-disabled-color: var(--tblr-green-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-green,
.btn-outline.btn-green {
  --tblr-btn-color: var(--tblr-green);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-green);
  --tblr-btn-hover-color: var(--tblr-green-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-green);
  --tblr-btn-active-color: var(--tblr-green-fg);
  --tblr-btn-active-bg: var(--tblr-green);
  --tblr-btn-active-border-color: var(--tblr-green);
  --tblr-btn-disabled-color: var(--tblr-green);
  --tblr-btn-disabled-border-color: var(--tblr-green);
}

.btn-ghost-green,
.btn-ghost.btn-green {
  --tblr-btn-color: var(--tblr-green);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-green-fg);
  --tblr-btn-hover-bg: var(--tblr-green);
  --tblr-btn-hover-border-color: var(--tblr-green);
  --tblr-btn-active-color: var(--tblr-green-fg);
  --tblr-btn-active-bg: var(--tblr-green);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-green);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-teal {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-teal-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-teal);
  --tblr-btn-hover-color: var(--tblr-teal-fg);
  --tblr-btn-hover-bg: var(--tblr-teal-darken);
  --tblr-btn-active-color: var(--tblr-teal-fg);
  --tblr-btn-active-bg: var(--tblr-teal-darken);
  --tblr-btn-disabled-bg: var(--tblr-teal);
  --tblr-btn-disabled-color: var(--tblr-teal-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-teal,
.btn-outline.btn-teal {
  --tblr-btn-color: var(--tblr-teal);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-teal);
  --tblr-btn-hover-color: var(--tblr-teal-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-teal);
  --tblr-btn-active-color: var(--tblr-teal-fg);
  --tblr-btn-active-bg: var(--tblr-teal);
  --tblr-btn-active-border-color: var(--tblr-teal);
  --tblr-btn-disabled-color: var(--tblr-teal);
  --tblr-btn-disabled-border-color: var(--tblr-teal);
}

.btn-ghost-teal,
.btn-ghost.btn-teal {
  --tblr-btn-color: var(--tblr-teal);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-teal-fg);
  --tblr-btn-hover-bg: var(--tblr-teal);
  --tblr-btn-hover-border-color: var(--tblr-teal);
  --tblr-btn-active-color: var(--tblr-teal-fg);
  --tblr-btn-active-bg: var(--tblr-teal);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-teal);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-cyan {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-cyan-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-cyan);
  --tblr-btn-hover-color: var(--tblr-cyan-fg);
  --tblr-btn-hover-bg: var(--tblr-cyan-darken);
  --tblr-btn-active-color: var(--tblr-cyan-fg);
  --tblr-btn-active-bg: var(--tblr-cyan-darken);
  --tblr-btn-disabled-bg: var(--tblr-cyan);
  --tblr-btn-disabled-color: var(--tblr-cyan-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-cyan,
.btn-outline.btn-cyan {
  --tblr-btn-color: var(--tblr-cyan);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-cyan);
  --tblr-btn-hover-color: var(--tblr-cyan-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-cyan);
  --tblr-btn-active-color: var(--tblr-cyan-fg);
  --tblr-btn-active-bg: var(--tblr-cyan);
  --tblr-btn-active-border-color: var(--tblr-cyan);
  --tblr-btn-disabled-color: var(--tblr-cyan);
  --tblr-btn-disabled-border-color: var(--tblr-cyan);
}

.btn-ghost-cyan,
.btn-ghost.btn-cyan {
  --tblr-btn-color: var(--tblr-cyan);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-cyan-fg);
  --tblr-btn-hover-bg: var(--tblr-cyan);
  --tblr-btn-hover-border-color: var(--tblr-cyan);
  --tblr-btn-active-color: var(--tblr-cyan-fg);
  --tblr-btn-active-bg: var(--tblr-cyan);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-cyan);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-x {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-x-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-x);
  --tblr-btn-hover-color: var(--tblr-x-fg);
  --tblr-btn-hover-bg: var(--tblr-x-darken);
  --tblr-btn-active-color: var(--tblr-x-fg);
  --tblr-btn-active-bg: var(--tblr-x-darken);
  --tblr-btn-disabled-bg: var(--tblr-x);
  --tblr-btn-disabled-color: var(--tblr-x-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-x,
.btn-outline.btn-x {
  --tblr-btn-color: var(--tblr-x);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-x);
  --tblr-btn-hover-color: var(--tblr-x-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-x);
  --tblr-btn-active-color: var(--tblr-x-fg);
  --tblr-btn-active-bg: var(--tblr-x);
  --tblr-btn-active-border-color: var(--tblr-x);
  --tblr-btn-disabled-color: var(--tblr-x);
  --tblr-btn-disabled-border-color: var(--tblr-x);
}

.btn-ghost-x,
.btn-ghost.btn-x {
  --tblr-btn-color: var(--tblr-x);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-x-fg);
  --tblr-btn-hover-bg: var(--tblr-x);
  --tblr-btn-hover-border-color: var(--tblr-x);
  --tblr-btn-active-color: var(--tblr-x-fg);
  --tblr-btn-active-bg: var(--tblr-x);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-x);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-facebook {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-facebook-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-facebook);
  --tblr-btn-hover-color: var(--tblr-facebook-fg);
  --tblr-btn-hover-bg: var(--tblr-facebook-darken);
  --tblr-btn-active-color: var(--tblr-facebook-fg);
  --tblr-btn-active-bg: var(--tblr-facebook-darken);
  --tblr-btn-disabled-bg: var(--tblr-facebook);
  --tblr-btn-disabled-color: var(--tblr-facebook-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-facebook,
.btn-outline.btn-facebook {
  --tblr-btn-color: var(--tblr-facebook);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-facebook);
  --tblr-btn-hover-color: var(--tblr-facebook-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-facebook);
  --tblr-btn-active-color: var(--tblr-facebook-fg);
  --tblr-btn-active-bg: var(--tblr-facebook);
  --tblr-btn-active-border-color: var(--tblr-facebook);
  --tblr-btn-disabled-color: var(--tblr-facebook);
  --tblr-btn-disabled-border-color: var(--tblr-facebook);
}

.btn-ghost-facebook,
.btn-ghost.btn-facebook {
  --tblr-btn-color: var(--tblr-facebook);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-facebook-fg);
  --tblr-btn-hover-bg: var(--tblr-facebook);
  --tblr-btn-hover-border-color: var(--tblr-facebook);
  --tblr-btn-active-color: var(--tblr-facebook-fg);
  --tblr-btn-active-bg: var(--tblr-facebook);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-facebook);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-twitter {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-twitter-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-twitter);
  --tblr-btn-hover-color: var(--tblr-twitter-fg);
  --tblr-btn-hover-bg: var(--tblr-twitter-darken);
  --tblr-btn-active-color: var(--tblr-twitter-fg);
  --tblr-btn-active-bg: var(--tblr-twitter-darken);
  --tblr-btn-disabled-bg: var(--tblr-twitter);
  --tblr-btn-disabled-color: var(--tblr-twitter-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-twitter,
.btn-outline.btn-twitter {
  --tblr-btn-color: var(--tblr-twitter);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-twitter);
  --tblr-btn-hover-color: var(--tblr-twitter-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-twitter);
  --tblr-btn-active-color: var(--tblr-twitter-fg);
  --tblr-btn-active-bg: var(--tblr-twitter);
  --tblr-btn-active-border-color: var(--tblr-twitter);
  --tblr-btn-disabled-color: var(--tblr-twitter);
  --tblr-btn-disabled-border-color: var(--tblr-twitter);
}

.btn-ghost-twitter,
.btn-ghost.btn-twitter {
  --tblr-btn-color: var(--tblr-twitter);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-twitter-fg);
  --tblr-btn-hover-bg: var(--tblr-twitter);
  --tblr-btn-hover-border-color: var(--tblr-twitter);
  --tblr-btn-active-color: var(--tblr-twitter-fg);
  --tblr-btn-active-bg: var(--tblr-twitter);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-twitter);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-linkedin {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-linkedin-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-linkedin);
  --tblr-btn-hover-color: var(--tblr-linkedin-fg);
  --tblr-btn-hover-bg: var(--tblr-linkedin-darken);
  --tblr-btn-active-color: var(--tblr-linkedin-fg);
  --tblr-btn-active-bg: var(--tblr-linkedin-darken);
  --tblr-btn-disabled-bg: var(--tblr-linkedin);
  --tblr-btn-disabled-color: var(--tblr-linkedin-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-linkedin,
.btn-outline.btn-linkedin {
  --tblr-btn-color: var(--tblr-linkedin);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-linkedin);
  --tblr-btn-hover-color: var(--tblr-linkedin-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-linkedin);
  --tblr-btn-active-color: var(--tblr-linkedin-fg);
  --tblr-btn-active-bg: var(--tblr-linkedin);
  --tblr-btn-active-border-color: var(--tblr-linkedin);
  --tblr-btn-disabled-color: var(--tblr-linkedin);
  --tblr-btn-disabled-border-color: var(--tblr-linkedin);
}

.btn-ghost-linkedin,
.btn-ghost.btn-linkedin {
  --tblr-btn-color: var(--tblr-linkedin);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-linkedin-fg);
  --tblr-btn-hover-bg: var(--tblr-linkedin);
  --tblr-btn-hover-border-color: var(--tblr-linkedin);
  --tblr-btn-active-color: var(--tblr-linkedin-fg);
  --tblr-btn-active-bg: var(--tblr-linkedin);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-linkedin);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-google {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-google-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-google);
  --tblr-btn-hover-color: var(--tblr-google-fg);
  --tblr-btn-hover-bg: var(--tblr-google-darken);
  --tblr-btn-active-color: var(--tblr-google-fg);
  --tblr-btn-active-bg: var(--tblr-google-darken);
  --tblr-btn-disabled-bg: var(--tblr-google);
  --tblr-btn-disabled-color: var(--tblr-google-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-google,
.btn-outline.btn-google {
  --tblr-btn-color: var(--tblr-google);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-google);
  --tblr-btn-hover-color: var(--tblr-google-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-google);
  --tblr-btn-active-color: var(--tblr-google-fg);
  --tblr-btn-active-bg: var(--tblr-google);
  --tblr-btn-active-border-color: var(--tblr-google);
  --tblr-btn-disabled-color: var(--tblr-google);
  --tblr-btn-disabled-border-color: var(--tblr-google);
}

.btn-ghost-google,
.btn-ghost.btn-google {
  --tblr-btn-color: var(--tblr-google);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-google-fg);
  --tblr-btn-hover-bg: var(--tblr-google);
  --tblr-btn-hover-border-color: var(--tblr-google);
  --tblr-btn-active-color: var(--tblr-google-fg);
  --tblr-btn-active-bg: var(--tblr-google);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-google);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-youtube {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-youtube-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-youtube);
  --tblr-btn-hover-color: var(--tblr-youtube-fg);
  --tblr-btn-hover-bg: var(--tblr-youtube-darken);
  --tblr-btn-active-color: var(--tblr-youtube-fg);
  --tblr-btn-active-bg: var(--tblr-youtube-darken);
  --tblr-btn-disabled-bg: var(--tblr-youtube);
  --tblr-btn-disabled-color: var(--tblr-youtube-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-youtube,
.btn-outline.btn-youtube {
  --tblr-btn-color: var(--tblr-youtube);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-youtube);
  --tblr-btn-hover-color: var(--tblr-youtube-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-youtube);
  --tblr-btn-active-color: var(--tblr-youtube-fg);
  --tblr-btn-active-bg: var(--tblr-youtube);
  --tblr-btn-active-border-color: var(--tblr-youtube);
  --tblr-btn-disabled-color: var(--tblr-youtube);
  --tblr-btn-disabled-border-color: var(--tblr-youtube);
}

.btn-ghost-youtube,
.btn-ghost.btn-youtube {
  --tblr-btn-color: var(--tblr-youtube);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-youtube-fg);
  --tblr-btn-hover-bg: var(--tblr-youtube);
  --tblr-btn-hover-border-color: var(--tblr-youtube);
  --tblr-btn-active-color: var(--tblr-youtube-fg);
  --tblr-btn-active-bg: var(--tblr-youtube);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-youtube);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-vimeo {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-vimeo-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-vimeo);
  --tblr-btn-hover-color: var(--tblr-vimeo-fg);
  --tblr-btn-hover-bg: var(--tblr-vimeo-darken);
  --tblr-btn-active-color: var(--tblr-vimeo-fg);
  --tblr-btn-active-bg: var(--tblr-vimeo-darken);
  --tblr-btn-disabled-bg: var(--tblr-vimeo);
  --tblr-btn-disabled-color: var(--tblr-vimeo-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-vimeo,
.btn-outline.btn-vimeo {
  --tblr-btn-color: var(--tblr-vimeo);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-vimeo);
  --tblr-btn-hover-color: var(--tblr-vimeo-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-vimeo);
  --tblr-btn-active-color: var(--tblr-vimeo-fg);
  --tblr-btn-active-bg: var(--tblr-vimeo);
  --tblr-btn-active-border-color: var(--tblr-vimeo);
  --tblr-btn-disabled-color: var(--tblr-vimeo);
  --tblr-btn-disabled-border-color: var(--tblr-vimeo);
}

.btn-ghost-vimeo,
.btn-ghost.btn-vimeo {
  --tblr-btn-color: var(--tblr-vimeo);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-vimeo-fg);
  --tblr-btn-hover-bg: var(--tblr-vimeo);
  --tblr-btn-hover-border-color: var(--tblr-vimeo);
  --tblr-btn-active-color: var(--tblr-vimeo-fg);
  --tblr-btn-active-bg: var(--tblr-vimeo);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-vimeo);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-dribbble {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-dribbble-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-dribbble);
  --tblr-btn-hover-color: var(--tblr-dribbble-fg);
  --tblr-btn-hover-bg: var(--tblr-dribbble-darken);
  --tblr-btn-active-color: var(--tblr-dribbble-fg);
  --tblr-btn-active-bg: var(--tblr-dribbble-darken);
  --tblr-btn-disabled-bg: var(--tblr-dribbble);
  --tblr-btn-disabled-color: var(--tblr-dribbble-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-dribbble,
.btn-outline.btn-dribbble {
  --tblr-btn-color: var(--tblr-dribbble);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-dribbble);
  --tblr-btn-hover-color: var(--tblr-dribbble-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-dribbble);
  --tblr-btn-active-color: var(--tblr-dribbble-fg);
  --tblr-btn-active-bg: var(--tblr-dribbble);
  --tblr-btn-active-border-color: var(--tblr-dribbble);
  --tblr-btn-disabled-color: var(--tblr-dribbble);
  --tblr-btn-disabled-border-color: var(--tblr-dribbble);
}

.btn-ghost-dribbble,
.btn-ghost.btn-dribbble {
  --tblr-btn-color: var(--tblr-dribbble);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-dribbble-fg);
  --tblr-btn-hover-bg: var(--tblr-dribbble);
  --tblr-btn-hover-border-color: var(--tblr-dribbble);
  --tblr-btn-active-color: var(--tblr-dribbble-fg);
  --tblr-btn-active-bg: var(--tblr-dribbble);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-dribbble);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-github {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-github-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-github);
  --tblr-btn-hover-color: var(--tblr-github-fg);
  --tblr-btn-hover-bg: var(--tblr-github-darken);
  --tblr-btn-active-color: var(--tblr-github-fg);
  --tblr-btn-active-bg: var(--tblr-github-darken);
  --tblr-btn-disabled-bg: var(--tblr-github);
  --tblr-btn-disabled-color: var(--tblr-github-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-github,
.btn-outline.btn-github {
  --tblr-btn-color: var(--tblr-github);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-github);
  --tblr-btn-hover-color: var(--tblr-github-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-github);
  --tblr-btn-active-color: var(--tblr-github-fg);
  --tblr-btn-active-bg: var(--tblr-github);
  --tblr-btn-active-border-color: var(--tblr-github);
  --tblr-btn-disabled-color: var(--tblr-github);
  --tblr-btn-disabled-border-color: var(--tblr-github);
}

.btn-ghost-github,
.btn-ghost.btn-github {
  --tblr-btn-color: var(--tblr-github);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-github-fg);
  --tblr-btn-hover-bg: var(--tblr-github);
  --tblr-btn-hover-border-color: var(--tblr-github);
  --tblr-btn-active-color: var(--tblr-github-fg);
  --tblr-btn-active-bg: var(--tblr-github);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-github);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-instagram {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-instagram-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-instagram);
  --tblr-btn-hover-color: var(--tblr-instagram-fg);
  --tblr-btn-hover-bg: var(--tblr-instagram-darken);
  --tblr-btn-active-color: var(--tblr-instagram-fg);
  --tblr-btn-active-bg: var(--tblr-instagram-darken);
  --tblr-btn-disabled-bg: var(--tblr-instagram);
  --tblr-btn-disabled-color: var(--tblr-instagram-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-instagram,
.btn-outline.btn-instagram {
  --tblr-btn-color: var(--tblr-instagram);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-instagram);
  --tblr-btn-hover-color: var(--tblr-instagram-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-instagram);
  --tblr-btn-active-color: var(--tblr-instagram-fg);
  --tblr-btn-active-bg: var(--tblr-instagram);
  --tblr-btn-active-border-color: var(--tblr-instagram);
  --tblr-btn-disabled-color: var(--tblr-instagram);
  --tblr-btn-disabled-border-color: var(--tblr-instagram);
}

.btn-ghost-instagram,
.btn-ghost.btn-instagram {
  --tblr-btn-color: var(--tblr-instagram);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-instagram-fg);
  --tblr-btn-hover-bg: var(--tblr-instagram);
  --tblr-btn-hover-border-color: var(--tblr-instagram);
  --tblr-btn-active-color: var(--tblr-instagram-fg);
  --tblr-btn-active-bg: var(--tblr-instagram);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-instagram);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-pinterest {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-pinterest-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-pinterest);
  --tblr-btn-hover-color: var(--tblr-pinterest-fg);
  --tblr-btn-hover-bg: var(--tblr-pinterest-darken);
  --tblr-btn-active-color: var(--tblr-pinterest-fg);
  --tblr-btn-active-bg: var(--tblr-pinterest-darken);
  --tblr-btn-disabled-bg: var(--tblr-pinterest);
  --tblr-btn-disabled-color: var(--tblr-pinterest-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-pinterest,
.btn-outline.btn-pinterest {
  --tblr-btn-color: var(--tblr-pinterest);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-pinterest);
  --tblr-btn-hover-color: var(--tblr-pinterest-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-pinterest);
  --tblr-btn-active-color: var(--tblr-pinterest-fg);
  --tblr-btn-active-bg: var(--tblr-pinterest);
  --tblr-btn-active-border-color: var(--tblr-pinterest);
  --tblr-btn-disabled-color: var(--tblr-pinterest);
  --tblr-btn-disabled-border-color: var(--tblr-pinterest);
}

.btn-ghost-pinterest,
.btn-ghost.btn-pinterest {
  --tblr-btn-color: var(--tblr-pinterest);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-pinterest-fg);
  --tblr-btn-hover-bg: var(--tblr-pinterest);
  --tblr-btn-hover-border-color: var(--tblr-pinterest);
  --tblr-btn-active-color: var(--tblr-pinterest-fg);
  --tblr-btn-active-bg: var(--tblr-pinterest);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-pinterest);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-vk {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-vk-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-vk);
  --tblr-btn-hover-color: var(--tblr-vk-fg);
  --tblr-btn-hover-bg: var(--tblr-vk-darken);
  --tblr-btn-active-color: var(--tblr-vk-fg);
  --tblr-btn-active-bg: var(--tblr-vk-darken);
  --tblr-btn-disabled-bg: var(--tblr-vk);
  --tblr-btn-disabled-color: var(--tblr-vk-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-vk,
.btn-outline.btn-vk {
  --tblr-btn-color: var(--tblr-vk);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-vk);
  --tblr-btn-hover-color: var(--tblr-vk-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-vk);
  --tblr-btn-active-color: var(--tblr-vk-fg);
  --tblr-btn-active-bg: var(--tblr-vk);
  --tblr-btn-active-border-color: var(--tblr-vk);
  --tblr-btn-disabled-color: var(--tblr-vk);
  --tblr-btn-disabled-border-color: var(--tblr-vk);
}

.btn-ghost-vk,
.btn-ghost.btn-vk {
  --tblr-btn-color: var(--tblr-vk);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-vk-fg);
  --tblr-btn-hover-bg: var(--tblr-vk);
  --tblr-btn-hover-border-color: var(--tblr-vk);
  --tblr-btn-active-color: var(--tblr-vk-fg);
  --tblr-btn-active-bg: var(--tblr-vk);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-vk);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-rss {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-rss-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-rss);
  --tblr-btn-hover-color: var(--tblr-rss-fg);
  --tblr-btn-hover-bg: var(--tblr-rss-darken);
  --tblr-btn-active-color: var(--tblr-rss-fg);
  --tblr-btn-active-bg: var(--tblr-rss-darken);
  --tblr-btn-disabled-bg: var(--tblr-rss);
  --tblr-btn-disabled-color: var(--tblr-rss-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-rss,
.btn-outline.btn-rss {
  --tblr-btn-color: var(--tblr-rss);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-rss);
  --tblr-btn-hover-color: var(--tblr-rss-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-rss);
  --tblr-btn-active-color: var(--tblr-rss-fg);
  --tblr-btn-active-bg: var(--tblr-rss);
  --tblr-btn-active-border-color: var(--tblr-rss);
  --tblr-btn-disabled-color: var(--tblr-rss);
  --tblr-btn-disabled-border-color: var(--tblr-rss);
}

.btn-ghost-rss,
.btn-ghost.btn-rss {
  --tblr-btn-color: var(--tblr-rss);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-rss-fg);
  --tblr-btn-hover-bg: var(--tblr-rss);
  --tblr-btn-hover-border-color: var(--tblr-rss);
  --tblr-btn-active-color: var(--tblr-rss-fg);
  --tblr-btn-active-bg: var(--tblr-rss);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-rss);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-flickr {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-flickr-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-flickr);
  --tblr-btn-hover-color: var(--tblr-flickr-fg);
  --tblr-btn-hover-bg: var(--tblr-flickr-darken);
  --tblr-btn-active-color: var(--tblr-flickr-fg);
  --tblr-btn-active-bg: var(--tblr-flickr-darken);
  --tblr-btn-disabled-bg: var(--tblr-flickr);
  --tblr-btn-disabled-color: var(--tblr-flickr-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-flickr,
.btn-outline.btn-flickr {
  --tblr-btn-color: var(--tblr-flickr);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-flickr);
  --tblr-btn-hover-color: var(--tblr-flickr-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-flickr);
  --tblr-btn-active-color: var(--tblr-flickr-fg);
  --tblr-btn-active-bg: var(--tblr-flickr);
  --tblr-btn-active-border-color: var(--tblr-flickr);
  --tblr-btn-disabled-color: var(--tblr-flickr);
  --tblr-btn-disabled-border-color: var(--tblr-flickr);
}

.btn-ghost-flickr,
.btn-ghost.btn-flickr {
  --tblr-btn-color: var(--tblr-flickr);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-flickr-fg);
  --tblr-btn-hover-bg: var(--tblr-flickr);
  --tblr-btn-hover-border-color: var(--tblr-flickr);
  --tblr-btn-active-color: var(--tblr-flickr-fg);
  --tblr-btn-active-bg: var(--tblr-flickr);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-flickr);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-bitbucket {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-bitbucket-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-bitbucket);
  --tblr-btn-hover-color: var(--tblr-bitbucket-fg);
  --tblr-btn-hover-bg: var(--tblr-bitbucket-darken);
  --tblr-btn-active-color: var(--tblr-bitbucket-fg);
  --tblr-btn-active-bg: var(--tblr-bitbucket-darken);
  --tblr-btn-disabled-bg: var(--tblr-bitbucket);
  --tblr-btn-disabled-color: var(--tblr-bitbucket-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-bitbucket,
.btn-outline.btn-bitbucket {
  --tblr-btn-color: var(--tblr-bitbucket);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-bitbucket);
  --tblr-btn-hover-color: var(--tblr-bitbucket-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-bitbucket);
  --tblr-btn-active-color: var(--tblr-bitbucket-fg);
  --tblr-btn-active-bg: var(--tblr-bitbucket);
  --tblr-btn-active-border-color: var(--tblr-bitbucket);
  --tblr-btn-disabled-color: var(--tblr-bitbucket);
  --tblr-btn-disabled-border-color: var(--tblr-bitbucket);
}

.btn-ghost-bitbucket,
.btn-ghost.btn-bitbucket {
  --tblr-btn-color: var(--tblr-bitbucket);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-bitbucket-fg);
  --tblr-btn-hover-bg: var(--tblr-bitbucket);
  --tblr-btn-hover-border-color: var(--tblr-bitbucket);
  --tblr-btn-active-color: var(--tblr-bitbucket-fg);
  --tblr-btn-active-bg: var(--tblr-bitbucket);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-bitbucket);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-tabler {
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-color: var(--tblr-tabler-fg, #ffffff);
  --tblr-btn-bg: var(--tblr-tabler);
  --tblr-btn-hover-color: var(--tblr-tabler-fg);
  --tblr-btn-hover-bg: var(--tblr-tabler-darken);
  --tblr-btn-active-color: var(--tblr-tabler-fg);
  --tblr-btn-active-bg: var(--tblr-tabler-darken);
  --tblr-btn-disabled-bg: var(--tblr-tabler);
  --tblr-btn-disabled-color: var(--tblr-tabler-fg);
  --tblr-btn-box-shadow: var(--tblr-shadow-input);
}

.btn-outline-tabler,
.btn-outline.btn-tabler {
  --tblr-btn-color: var(--tblr-tabler);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: var(--tblr-tabler);
  --tblr-btn-hover-color: var(--tblr-tabler-fg);
  --tblr-btn-hover-border-color: transparent;
  --tblr-btn-hover-bg: var(--tblr-tabler);
  --tblr-btn-active-color: var(--tblr-tabler-fg);
  --tblr-btn-active-bg: var(--tblr-tabler);
  --tblr-btn-active-border-color: var(--tblr-tabler);
  --tblr-btn-disabled-color: var(--tblr-tabler);
  --tblr-btn-disabled-border-color: var(--tblr-tabler);
}

.btn-ghost-tabler,
.btn-ghost.btn-tabler {
  --tblr-btn-color: var(--tblr-tabler);
  --tblr-btn-bg: transparent;
  --tblr-btn-border-color: transparent;
  --tblr-btn-hover-color: var(--tblr-tabler-fg);
  --tblr-btn-hover-bg: var(--tblr-tabler);
  --tblr-btn-hover-border-color: var(--tblr-tabler);
  --tblr-btn-active-color: var(--tblr-tabler-fg);
  --tblr-btn-active-bg: var(--tblr-tabler);
  --tblr-btn-active-border-color: transparent;
  --tblr-btn-active-shadow: inset 0 3px 5px rgba(0, 0, 0, 0.125);
  --tblr-btn-disabled-color: var(--tblr-tabler);
  --tblr-btn-disabled-bg: transparent;
  --tblr-btn-disabled-border-color: transparent;
  --tblr-gradient: none;
  --tblr-btn-box-shadow: none;
}

.btn-sm,
.btn-group-sm > .btn {
  --tblr-btn-line-height: 1.3333333333;
  --tblr-btn-icon-size: 1rem;
}

.btn-lg,
.btn-group-lg > .btn {
  --tblr-btn-line-height: 1.5rem;
  --tblr-btn-icon-size: 1.5rem;
}

.btn-xl,
.btn-group-xl > .btn {
  --tblr-btn-line-height: 2;
  --tblr-btn-icon-size: 2rem;
  --tblr-btn-padding-y: 0.6875rem;
  --tblr-btn-padding-x: 2rem;
  --tblr-btn-font-size: 1.5rem;
}

.btn-pill {
  padding-left: 1.5em;
  padding-right: 1.5em;
  border-radius: 10rem;
}
.btn-pill[class*=btn-icon] {
  padding: 0.375rem 15px;
}

.btn-square {
  border-radius: 0;
}

.btn-icon,
.btn-action {
  padding-right: 0;
  padding-left: 0;
}
.btn-icon .icon,
.btn-action .icon {
  margin: calc(-1 * var(--tblr-btn-padding-x));
}

.btn-list {
  --tblr-list-gap: 0.5rem;
  display: flex;
  flex-wrap: wrap;
  gap: var(--tblr-list-gap);
}

.btn-floating {
  position: fixed;
  z-index: 1030;
  bottom: 1rem;
  right: 1rem;
  box-shadow: var(--tblr-shadow-dropdown);
}

.btn-loading {
  position: relative;
  color: transparent !important;
  text-shadow: none !important;
  pointer-events: none;
}
.btn-loading > * {
  opacity: 0;
}
.btn-loading:after {
  content: "";
  display: inline-block;
  vertical-align: text-bottom;
  border: 2px var(--tblr-border-style) currentColor;
  border-left-color: transparent;
  border-radius: 100rem;
  color: var(--tblr-btn-color);
  position: absolute;
  width: var(--tblr-btn-icon-size);
  height: var(--tblr-btn-icon-size);
  right: calc(50% - var(--tblr-btn-icon-size) / 2);
  top: calc(50% - var(--tblr-btn-icon-size) / 2);
  animation: spinner-border 0.75s linear infinite;
}

.btn-action {
  --tblr-border-color: transparent;
  color: var(--tblr-secondary);
  border-radius: var(--tblr-border-radius);
  background: transparent;
  box-shadow: none;
}
.btn-action:after {
  content: none;
}
.btn-action:focus {
  outline: none;
  box-shadow: none;
}
.btn-action:hover, .btn-action.show {
  color: var(--tblr-body-color);
  background: var(--tblr-active-bg);
  border-color: transparent;
}
.btn-action.show {
  color: var(--tblr-primary);
}

.btn-actions {
  display: flex;
}

.btn-animate-icon .icon {
  transition: transform 0.3s ease;
}
.btn-animate-icon:hover .icon, .btn-animate-icon:focus-visible .icon {
  transform: translateX(-4px);
}
.btn-animate-icon.btn-animate-icon-rotate:hover .icon, .btn-animate-icon.btn-animate-icon-rotate:focus-visible .icon {
  transform: rotate(-90deg);
}
.btn-animate-icon.btn-animate-icon-move-start:hover .icon, .btn-animate-icon.btn-animate-icon-move-start:focus-visible .icon {
  transform: translateX(4px);
}
.btn-animate-icon.btn-animate-icon-pulse:hover .icon, .btn-animate-icon.btn-animate-icon-pulse:focus-visible .icon {
  transform: none;
  animation: pulse 0.9s;
}
.btn-animate-icon.btn-animate-icon-shake:hover .icon, .btn-animate-icon.btn-animate-icon-shake:focus-visible .icon {
  transform: none;
  animation: shake 0.9s;
}
.btn-animate-icon.btn-animate-icon-tada:hover .icon, .btn-animate-icon.btn-animate-icon-tada:focus-visible .icon {
  transform: none;
  animation: tada 0.9s;
}

.btn-group,
.btn-group-vertical {
  box-shadow: var(--tblr-shadow-input);
}
.btn-group > .btn-check:checked + .btn,
.btn-group > .btn:active,
.btn-group > .btn.active,
.btn-group-vertical > .btn-check:checked + .btn,
.btn-group-vertical > .btn:active,
.btn-group-vertical > .btn.active {
  z-index: 5;
}
.btn-group > .btn-check:focus + .btn,
.btn-group > .btn:hover,
.btn-group > .btn:focus,
.btn-group-vertical > .btn-check:focus + .btn,
.btn-group-vertical > .btn:hover,
.btn-group-vertical > .btn:focus {
  z-index: 1;
}

.calendar {
  display: block;
  font-size: 0.765625rem;
  border: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
  border-radius: var(--tblr-border-radius);
}

.calendar-nav {
  display: flex;
  align-items: center;
}

.calendar-title {
  flex: 1;
  text-align: center;
}

.calendar-body,
.calendar-header {
  display: flex;
  flex-wrap: wrap;
  justify-content: flex-start;
  padding: 0.5rem 0;
}

.calendar-header {
  color: var(--tblr-secondary);
}

.calendar-date {
  flex: 0 0 14.2857142857%;
  max-width: 14.2857142857%;
  padding: 0.2rem;
  text-align: center;
  border: 0;
}
.calendar-date.prev-month, .calendar-date.next-month {
  opacity: 0.25;
}
.calendar-date .date-item {
  position: relative;
  display: inline-block;
  width: 1.4rem;
  height: 1.4rem;
  line-height: 1.4rem;
  color: #66758c;
  text-align: center;
  text-decoration: none;
  white-space: nowrap;
  vertical-align: middle;
  cursor: pointer;
  background: 100% 0;
  border: var(--tblr-border-width) var(--tblr-border-style) transparent;
  border-radius: 100rem;
  outline: 0;
  transition: background 0.3s, border 0.3s, box-shadow 0.32s, color 0.3s;
}
@media (prefers-reduced-motion: reduce) {
  .calendar-date .date-item {
    transition: none;
  }
}
.calendar-date .date-item:hover {
  color: var(--tblr-primary);
  text-decoration: none;
  background: #fefeff;
  border-color: var(--tblr-border-color);
}
.calendar-date .date-today {
  color: var(--tblr-primary);
  border-color: var(--tblr-border-color);
}

.calendar-range {
  position: relative;
}
.calendar-range:before {
  position: absolute;
  top: 50%;
  left: 0;
  right: 0;
  height: 1.4rem;
  content: "";
  background: rgba(var(--tblr-primary-rgb), 0.1);
  transform: translateY(-50%);
}
.calendar-range.range-start .date-item, .calendar-range.range-end .date-item {
  color: #ffffff;
  background: var(--tblr-primary);
  border-color: var(--tblr-primary);
}
.calendar-range.range-start:before {
  right: 50%;
}
.calendar-range.range-end:before {
  left: 50%;
}

.carousel-indicators-vertical {
  right: auto;
  top: 0;
  margin: 0 0 0 1rem;
  flex-direction: column;
}
.carousel-indicators-vertical [data-bs-target] {
  margin: 3px 0 3px;
  width: 3px;
  height: 30px;
  border: 0;
  border-right: 10px var(--tblr-border-style) transparent;
  border-left: 10px var(--tblr-border-style) transparent;
}

.carousel-indicators-dot [data-bs-target] {
  width: 0.5rem;
  height: 0.5rem;
  border-radius: 100rem;
  border: 10px var(--tblr-border-style) transparent;
  margin: 0;
}

.carousel-indicators-thumb [data-bs-target] {
  width: 2rem;
  height: auto;
  background: no-repeat center/cover;
  border: 0;
  border-radius: var(--tblr-border-radius);
  box-shadow: rgba(var(--tblr-body-color-rgb), 0.04) 0 2px 4px 0;
  margin: 0 3px;
  opacity: 0.75;
}
@media (min-width: 992px) {
  .carousel-indicators-thumb [data-bs-target] {
    width: 4rem;
  }
}
.carousel-indicators-thumb [data-bs-target]:before {
  content: "";
  padding-top: var(--tblr-aspect-ratio, 100%);
  display: block;
}
.carousel-indicators-thumb.carousel-indicators-vertical [data-bs-target] {
  margin: 3px 0;
}

.carousel-caption-background {
  background: red;
  position: absolute;
  right: 0;
  left: 0;
  bottom: 0;
  height: 90%;
  background: linear-gradient(0deg, rgba(31, 41, 55, 0.9), rgba(31, 41, 55, 0));
}

.card {
  transition: transform 0.3s ease-out, opacity 0.3s ease-out, box-shadow 0.3s ease-out;
}
@media (prefers-reduced-motion: reduce) {
  .card {
    transition: none;
  }
}
@media print {
  .card {
    border: none;
    box-shadow: none;
  }
}
a.card {
  color: inherit;
}
a.card:hover {
  text-decoration: none;
  box-shadow: rgba(var(--tblr-body-color-rgb), 0.16) 0 2px 16px 0;
}

.card .card {
  box-shadow: none;
}

.card-borderless,
.card-borderless .card-header,
.card-borderless .card-footer {
  border-color: transparent;
}

.card-stamp {
  --tblr-stamp-size: 7rem;
  position: absolute;
  top: 0;
  left: 0;
  width: calc(var(--tblr-stamp-size) * 1);
  height: calc(var(--tblr-stamp-size) * 1);
  max-height: 100%;
  border-top-left-radius: 6px;
  opacity: 0.2;
  overflow: hidden;
  pointer-events: none;
}

.card-stamp-lg {
  --tblr-stamp-size: 13rem;
}

.card-stamp-icon {
  background: var(--tblr-secondary);
  color: var(--tblr-card-bg, var(--tblr-bg-surface));
  display: flex;
  align-items: center;
  justify-content: center;
  border-radius: 100rem;
  width: calc(var(--tblr-stamp-size) * 1);
  height: calc(var(--tblr-stamp-size) * 1);
  position: relative;
  top: calc(var(--tblr-stamp-size) * -0.25);
  left: calc(var(--tblr-stamp-size) * -0.25);
  font-size: calc(var(--tblr-stamp-size) * 0.75);
  transform: rotate(-10deg);
}
.card-stamp-icon .icon {
  stroke-width: 2;
  width: calc(var(--tblr-stamp-size) * 0.75);
  height: calc(var(--tblr-stamp-size) * 0.75);
}

.card-img,
.card-img-start {
  border-top-right-radius: calc(var(--tblr-border-radius-lg) - (var(--tblr-border-width)));
  border-bottom-right-radius: calc(var(--tblr-border-radius-lg) - (var(--tblr-border-width)));
}

.card-img,
.card-img-end {
  border-top-left-radius: calc(var(--tblr-border-radius-lg) - (var(--tblr-border-width)));
  border-bottom-left-radius: calc(var(--tblr-border-radius-lg) - (var(--tblr-border-width)));
}

.card-img-overlay {
  display: flex;
  flex-direction: column;
  justify-content: flex-end;
}

.card-img-overlay-dark {
  background-image: linear-gradient(-180deg, rgba(0, 0, 0, 0) 0%, rgba(0, 0, 0, 0.6) 100%);
}

.card-inactive {
  pointer-events: none;
  box-shadow: none;
}
.card-inactive .card-body {
  opacity: 0.64;
}

.card-active {
  --tblr-card-border-color: var(--tblr-primary);
  --tblr-card-bg: var(--tblr-active-bg);
}

.card-btn {
  display: flex;
  align-items: center;
  justify-content: center;
  padding: 1rem 1.25rem;
  text-align: center;
  transition: background 0.3s;
  border-top: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
  flex: 1;
  color: inherit;
  font-weight: var(--tblr-font-weight-medium);
}
@media (prefers-reduced-motion: reduce) {
  .card-btn {
    transition: none;
  }
}
.card-btn:hover {
  text-decoration: none;
  background: rgba(var(--tblr-primary-rgb), 0.04);
}
.card-btn + .card-btn {
  border-right: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
}

/**
Stacked card
 */
.card-stacked {
  --tblr-card-stacked-offset: .25rem;
  position: relative;
}
.card-stacked:after {
  position: absolute;
  top: calc(-1 * var(--tblr-card-stacked-offset));
  left: var(--tblr-card-stacked-offset);
  right: var(--tblr-card-stacked-offset);
  height: var(--tblr-card-stacked-offset);
  content: "";
  background: var(--tblr-card-bg, var(--tblr-bg-surface));
  border: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-card-border-color);
  border-radius: var(--tblr-card-border-radius) var(--tblr-card-border-radius) 0 0;
}

.card-cover {
  position: relative;
  padding: 1rem 1.25rem;
  background: #666666 no-repeat center/cover;
}
.card-cover:before {
  position: absolute;
  top: 0;
  left: 0;
  bottom: 0;
  right: 0;
  content: "";
  background: rgba(31, 41, 55, 0.48);
}
.card-cover:first-child, .card-cover:first-child:before {
  border-radius: 6px 6px 0 0;
}

.card-cover-blurred:before {
  backdrop-filter: blur(2px);
}

.card-actions {
  margin: -0.5rem auto -0.5rem -0.5rem;
  padding-right: 0.5rem;
}
.card-actions a {
  text-decoration: none;
}

.card-header {
  color: inherit;
  display: flex;
  align-items: center;
  background: transparent;
}
.card-header:first-child {
  border-radius: var(--tblr-card-border-radius) var(--tblr-card-border-radius) 0 0;
}

.card-header-light {
  border-bottom-color: transparent;
  background: var(--tblr-bg-surface-tertiary);
}

.card-header-tabs {
  background: var(--tblr-bg-surface-tertiary);
  flex: 1;
  margin: calc(var(--tblr-card-cap-padding-y) * -1) calc(var(--tblr-card-cap-padding-x) * -1) calc(var(--tblr-card-cap-padding-y) * -1);
  padding: calc(var(--tblr-card-cap-padding-y) * 0.5) calc(var(--tblr-card-cap-padding-x) * 0.5) 0;
  border-radius: var(--tblr-card-border-radius) var(--tblr-card-border-radius) 0 0;
}

.card-header-pills {
  flex: 1;
  margin-top: -0.5rem;
  margin-bottom: -0.5rem;
}

.card-rotate-left,
.card-rotate-start {
  transform: rotate(1.5deg);
}

.card-rotate-right,
.card-rotate-end {
  transform: rotate(-1.5deg);
}

.card-link {
  color: inherit;
}
.card-link:hover {
  color: inherit;
  text-decoration: none;
  box-shadow: 0 1px 6px 0 rgba(0, 0, 0, 0.08);
}

.card-link-rotate:hover {
  transform: rotate(-1.5deg);
  opacity: 1;
}

.card-link-pop:hover {
  transform: translateY(-2px);
  opacity: 1;
}

.card-footer {
  margin-top: auto;
}
.card-footer:last-child {
  border-radius: 0 0 var(--tblr-card-border-radius) var(--tblr-card-border-radius);
}

.card-footer-transparent {
  background: transparent;
  border-color: transparent;
  padding-top: 0;
}

.card-footer-borderless {
  border-top: none;
}

.card-progress {
  height: 0.25rem;
}
.card-progress:last-child {
  border-radius: 0 0 2px 2px;
}
.card-progress:first-child {
  border-radius: 2px 2px 0 0;
}

.card-meta {
  color: var(--tblr-secondary);
}

.card-title {
  display: block;
  margin: 0 0 1rem;
  font-size: 1rem;
  font-weight: var(--tblr-font-weight-medium);
  color: inherit;
  line-height: 1.5rem;
}
a.card-title:hover {
  color: inherit;
}

.card-header .card-title {
  margin: 0;
}

.card-subtitle {
  margin-bottom: 1.25rem;
  color: var(--tblr-secondary);
  font-weight: normal;
}
.card-header .card-subtitle {
  margin: 0;
}
.card-title .card-subtitle {
  margin: 0 0.25rem 0 0;
  font-size: 0.875rem;
}

.card-body {
  position: relative;
}
.card-body > :last-child {
  margin-bottom: 0;
}
.card-sm > .card-body {
  padding: 1rem;
}
@media (min-width: 768px) {
  .card-md > .card-body {
    padding: 2.5rem;
  }
}
@media (min-width: 768px) {
  .card-lg > .card-body {
    padding: 2rem;
  }
}
@media (min-width: 992px) {
  .card-lg > .card-body {
    padding: 4rem;
  }
}
@media print {
  .card-body {
    padding: 0;
  }
}
.card-body + .card-body {
  border-top: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
}

.card-body-scrollable {
  overflow: auto;
}

/**
Card optinos
 */
.card-options {
  top: 1.5rem;
  left: 0.75rem;
  display: flex;
  margin-right: auto;
}

.card-options-link {
  display: inline-block;
  min-width: 1rem;
  margin-right: 0.25rem;
  color: var(--tblr-secondary);
}

/**
Card status
 */
.card-status-top {
  position: absolute;
  top: 0;
  left: 0;
  right: 0;
  height: 2px;
  border-radius: var(--tblr-card-border-radius) var(--tblr-card-border-radius) 0 0;
}

.card-status-start {
  position: absolute;
  left: auto;
  bottom: 0;
  width: 2px;
  height: 100%;
  border-radius: 0 var(--tblr-card-border-radius) var(--tblr-card-border-radius) 0;
}

.card-status-bottom {
  position: absolute;
  top: initial;
  bottom: 0;
  width: 100%;
  height: 2px;
  border-radius: 0 0 var(--tblr-card-border-radius) var(--tblr-card-border-radius);
}

/**
Card table
 */
.card-table {
  margin-bottom: 0 !important;
}
.card-table tr td:first-child,
.card-table tr th:first-child {
  padding-right: 1.25rem;
  border-right: 0;
}
.card-table tr td:last-child,
.card-table tr th:last-child {
  padding-left: 1.25rem;
  border-left: 0;
}
.card-table thead tr:first-child,
.card-table tbody tr:first-child,
.card-table tfoot tr:first-child {
  border-top: 0;
}
.card-table thead tr:first-child td,
.card-table thead tr:first-child th,
.card-table tbody tr:first-child td,
.card-table tbody tr:first-child th,
.card-table tfoot tr:first-child td,
.card-table tfoot tr:first-child th {
  border-top: 0;
}
.card-body + .card-table {
  border-top: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-table-border-color);
}

/*
Card code
 */
.card-code {
  padding: 0;
}
.card-code .highlight {
  margin: 0;
  border: 0;
}
.card-code pre {
  margin: 0 !important;
  border: 0 !important;
}

/*
Card chart
 */
.card-chart {
  position: relative;
  z-index: 1;
  height: 3.5rem;
}

/**
Card avatar
 */
.card-avatar {
  margin-right: auto;
  margin-left: auto;
  box-shadow: 0 0 0 0.25rem var(--tblr-card-bg, var(--tblr-bg-surface));
  margin-top: calc(-1 * var(--tblr-avatar-size) * 0.5);
}

/*
Card list group
 */
.card-body + .card-list-group {
  border-top: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
}
.card-list-group .list-group-item {
  padding-left: 1.25rem;
  padding-right: 1.25rem;
  border-left: 0;
  border-right: 0;
  border-radius: 0;
}
.card-list-group .list-group-item:last-child {
  border-bottom: 0;
}
.card-list-group .list-group-item:first-child {
  border-top: 0;
}

.card-tabs .nav-tabs {
  position: relative;
  z-index: 1000;
  border-bottom: 0;
}
.card-tabs .nav-tabs .nav-link {
  background: var(--tblr-bg-surface-tertiary);
  border: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent);
}
.card-tabs .nav-tabs .nav-link.active, .card-tabs .nav-tabs .nav-link:active, .card-tabs .nav-tabs .nav-link:hover {
  border-color: var(--tblr-border-color-translucent);
  color: var(--tblr-body-color);
}
.card-tabs .nav-tabs .nav-link.active {
  color: inherit;
  background: var(--tblr-card-bg, var(--tblr-bg-surface));
  border-bottom-color: transparent;
}
.card-tabs .nav-tabs .nav-item:not(:first-child) .nav-link {
  border-top-right-radius: 0;
}
.card-tabs .nav-tabs .nav-item:not(:last-child) .nav-link {
  border-top-left-radius: 0;
}
.card-tabs .nav-tabs .nav-item + .nav-item {
  margin-right: calc(-1 * var(--tblr-border-width));
}
.card-tabs .nav-tabs-bottom {
  margin-bottom: 0;
}
.card-tabs .nav-tabs-bottom .nav-link {
  margin-bottom: 0;
}
.card-tabs .nav-tabs-bottom .nav-link.active {
  border-top-color: transparent;
}
.card-tabs .nav-tabs-bottom .nav-item {
  margin-top: calc(-1 * var(--tblr-border-width));
  margin-bottom: 0;
}
.card-tabs .nav-tabs-bottom .nav-item .nav-link {
  border-bottom: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent);
  border-radius: 0 0 var(--tblr-border-radius-lg) var(--tblr-border-radius-lg);
}
.card-tabs .nav-tabs-bottom .nav-item:not(:first-child) .nav-link {
  border-bottom-right-radius: 0;
}
.card-tabs .nav-tabs-bottom .nav-item:not(:last-child) .nav-link {
  border-bottom-left-radius: 0;
}
.card-tabs .card {
  border-bottom-right-radius: 0;
}
.card-tabs .nav-tabs + .tab-content .card {
  border-bottom-right-radius: var(--tblr-card-border-radius);
  border-top-right-radius: 0;
}

/**
Card note
*/
.card-note {
  --tblr-card-bg: #fff7dd;
  --tblr-card-border-color: #fff1c9;
}

.btn-close {
  --tblr-btn-close-color: currentColor;
  --tblr-btn-close-bg: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%270 0 16 16%27 fill=%27%231f2937%27%3e%3cpath d=%27M.293.293a1 1 0 0 1 1.414 0L8 6.586 14.293.293a1 1 0 1 1 1.414 1.414L9.414 8l6.293 6.293a1 1 0 0 1-1.414 1.414L8 9.414l-6.293 6.293a1 1 0 0 1-1.414-1.414L6.586 8 .293 1.707a1 1 0 0 1 0-1.414%27/%3e%3c/svg%3e");
  --tblr-btn-close-opacity: 0.4;
  --tblr-btn-close-hover-opacity: 0.75;
  --tblr-btn-close-focus-shadow: 0 0 0 0.25rem rgba(var(--tblr-primary-rgb), 0.25);
  --tblr-btn-close-focus-opacity: 1;
  --tblr-btn-close-disabled-opacity: 0.25;
  --tblr-btn-close-size: 1em;
  width: var(--tblr-btn-close-size);
  height: var(--tblr-btn-close-size);
  padding: 0.25em 0.25em;
  color: var(--tblr-btn-close-color);
  mask: var(--tblr-btn-close-bg) no-repeat center/calc(var(--tblr-btn-close-size) * 0.75);
  background-color: var(--tblr-btn-close-color);
  border: 0;
  border-radius: var(--tblr-border-radius);
  opacity: var(--tblr-btn-close-opacity);
  cursor: pointer;
  display: block;
}
.btn-close:hover {
  color: var(--tblr-btn-close-color);
  text-decoration: none;
  opacity: var(--tblr-btn-close-hover-opacity);
}
.btn-close:focus {
  outline: 0;
  box-shadow: var(--tblr-btn-close-focus-shadow);
  opacity: var(--tblr-btn-close-focus-opacity);
}
.btn-close:disabled, .btn-close.disabled {
  pointer-events: none;
  user-select: none;
  opacity: var(--tblr-btn-close-disabled-opacity);
}

.dropdown-menu {
  user-select: none;
  background-clip: border-box;
}
.dropdown-menu.card {
  padding: 0;
  min-width: 25rem;
  display: none;
}
.dropdown-menu.card.show {
  display: flex;
}

.dropdown-item {
  min-width: 11rem;
  display: flex;
  align-items: center;
  margin: 0;
  line-height: 1.4285714286;
  gap: 0.5rem;
}

.dropdown-item-icon {
  width: 1.25rem !important;
  height: 1.25rem !important;
  margin-left: 0.5rem;
  color: var(--tblr-secondary);
  opacity: 0.7;
  text-align: center;
}

.dropdown-item-indicator {
  margin-left: 0.5rem;
  margin-right: -0.25rem;
  height: 1.25rem;
  display: inline-flex;
  line-height: 1;
  vertical-align: bottom;
  align-items: center;
}

.dropdown-header {
  font-size: 0.75rem;
  font-weight: var(--tblr-font-weight-medium);
  text-transform: uppercase;
  letter-spacing: 0.04em;
  line-height: 1rem;
  color: var(--tblr-secondary);
  padding-bottom: 0.25rem;
  pointer-events: none;
}

.dropdown-menu-scrollable {
  height: auto;
  max-height: 13rem;
  overflow-x: hidden;
}

.dropdown-menu-column {
  min-width: 11rem;
}
.dropdown-menu-column .dropdown-item {
  min-width: 0;
}

.dropdown-menu-columns {
  display: flex;
  flex: 0 0.25rem;
}

.dropdown-menu-arrow:before {
  content: "";
  position: absolute;
  top: -0.25rem;
  right: 0.75rem;
  display: block;
  background: inherit;
  width: 14px;
  height: 14px;
  transform: rotate(45deg);
  transform-origin: center;
  border: 1px solid;
  border-color: inherit;
  z-index: -1;
  clip: rect(0px, 9px, 9px, 0px);
}
.dropdown-menu-arrow.dropdown-menu-end:before {
  left: 0.75rem;
  right: auto;
}

.dropend > .dropdown-menu {
  margin-top: calc(-0.25rem - 1px);
  margin-right: -0.25rem;
}
.dropend .dropdown-toggle:after {
  margin-right: auto;
}

.dropdown-menu-card {
  padding: 0;
  min-width: 20rem;
}
.dropdown-menu-card > .card {
  margin: 0;
  border: 0;
  box-shadow: none;
}

.datagrid {
  --tblr-datagrid-padding: 1.5rem;
  --tblr-datagrid-item-width: 15rem;
  display: grid;
  grid-gap: var(--tblr-datagrid-padding);
  grid-template-columns: repeat(auto-fit, minmax(var(--tblr-datagrid-item-width), 1fr));
}

.datagrid-title {
  font-size: 0.75rem;
  font-weight: var(--tblr-font-weight-medium);
  text-transform: uppercase;
  letter-spacing: 0.04em;
  line-height: 1rem;
  color: var(--tblr-secondary);
  margin-bottom: 0.25rem;
}

.empty {
  display: flex;
  flex-direction: column;
  align-items: center;
  justify-content: center;
  height: 100%;
  padding: 1rem;
  text-align: center;
}
@media (min-width: 768px) {
  .empty {
    padding: 3rem;
  }
}

.empty-icon {
  margin: 0 0 1rem;
  width: 3rem;
  height: 3rem;
  line-height: 1;
  color: var(--tblr-secondary);
}
.empty-icon svg {
  width: 100%;
  height: 100%;
}

.empty-img {
  margin: 0 0 2rem;
  line-height: 1;
}

.empty-header {
  margin: 0 0 1rem;
  font-size: 4rem;
  font-weight: var(--tblr-font-weight-light);
  line-height: 1;
  color: var(--tblr-secondary);
}

.empty-title {
  font-size: 1.25rem;
  line-height: 1.75rem;
  font-weight: var(--tblr-font-weight-bold);
}

.empty-title,
.empty-subtitle {
  margin: 0 0 0.5rem;
}

.empty-action {
  margin-top: 1.5rem;
}

.empty-bordered {
  border: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
  border-radius: var(--tblr-border-radius);
}

.row > * {
  min-width: 0;
}

.col-separator {
  border-right: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
}

.container-slim {
  --tblr-gutter-x: calc(var(--tblr-page-padding) * 2);
  --tblr-gutter-y: 0;
  width: 100%;
  padding-left: calc(var(--tblr-gutter-x) * 0.5);
  padding-right: calc(var(--tblr-gutter-x) * 0.5);
  margin-left: auto;
  margin-right: auto;
  max-width: 16rem;
}
.container-tight {
  --tblr-gutter-x: calc(var(--tblr-page-padding) * 2);
  --tblr-gutter-y: 0;
  width: 100%;
  padding-left: calc(var(--tblr-gutter-x) * 0.5);
  padding-right: calc(var(--tblr-gutter-x) * 0.5);
  margin-left: auto;
  margin-right: auto;
  max-width: 30rem;
}
.container-narrow {
  --tblr-gutter-x: calc(var(--tblr-page-padding) * 2);
  --tblr-gutter-y: 0;
  width: 100%;
  padding-left: calc(var(--tblr-gutter-x) * 0.5);
  padding-right: calc(var(--tblr-gutter-x) * 0.5);
  margin-left: auto;
  margin-right: auto;
  max-width: 61.875rem;
}

.row-0 {
  margin-left: 0;
  margin-right: 0;
}
.row-0 > .col,
.row-0 > [class*=col-] {
  padding-left: 0;
  padding-right: 0;
}
.row-0 .card {
  margin-bottom: 0;
}

.row-sm {
  margin-left: -0.375rem;
  margin-right: -0.375rem;
}
.row-sm > .col,
.row-sm > [class*=col-] {
  padding-left: 0.375rem;
  padding-right: 0.375rem;
}
.row-sm .card {
  margin-bottom: 0.75rem;
}

.row-md {
  margin-left: -1.5rem;
  margin-right: -1.5rem;
}
.row-md > .col,
.row-md > [class*=col-] {
  padding-left: 1.5rem;
  padding-right: 1.5rem;
}
.row-md .card {
  margin-bottom: 3rem;
}

.row-lg {
  margin-left: -3rem;
  margin-right: -3rem;
}
.row-lg > .col,
.row-lg > [class*=col-] {
  padding-left: 3rem;
  padding-right: 3rem;
}
.row-lg .card {
  margin-bottom: 6rem;
}

.row-deck > .col,
.row-deck > [class*=col-] {
  display: flex;
  align-items: stretch;
}
.row-deck > .col .card,
.row-deck > [class*=col-] .card {
  flex: 1 1 auto;
}

.row-cards {
  --tblr-gutter-x: var(--tblr-page-padding);
  --tblr-gutter-y: var(--tblr-page-padding);
  min-width: 0;
}
.row-cards .row-cards {
  flex: 1;
}

.space-y {
  display: flex;
  flex-direction: column;
  gap: 1rem;
}

.space-x {
  display: flex;
  gap: 1rem;
}

.space-y-0 {
  display: flex;
  flex-direction: column;
  gap: 0;
}

.space-x-0 {
  display: flex;
  gap: 0;
}

.space-y-1 {
  display: flex;
  flex-direction: column;
  gap: 0.25rem;
}

.space-x-1 {
  display: flex;
  gap: 0.25rem;
}

.space-y-2 {
  display: flex;
  flex-direction: column;
  gap: 0.5rem;
}

.space-x-2 {
  display: flex;
  gap: 0.5rem;
}

.space-y-3 {
  display: flex;
  flex-direction: column;
  gap: 1rem;
}

.space-x-3 {
  display: flex;
  gap: 1rem;
}

.space-y-4 {
  display: flex;
  flex-direction: column;
  gap: 1.5rem;
}

.space-x-4 {
  display: flex;
  gap: 1.5rem;
}

.space-y-5 {
  display: flex;
  flex-direction: column;
  gap: 2rem;
}

.space-x-5 {
  display: flex;
  gap: 2rem;
}

.space-y-6 {
  display: flex;
  flex-direction: column;
  gap: 2.5rem;
}

.space-x-6 {
  display: flex;
  gap: 2.5rem;
}

.divide-y > :not(template) ~ :not(template) {
  border-top: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}
.divide-y > :not(template):not(:first-child) {
  padding-top: 1rem !important;
}
.divide-y > :not(template):not(:last-child) {
  padding-bottom: 1rem !important;
}

.divide-x > :not(template) ~ :not(template) {
  border-right: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}
.divide-x > :not(template):not(:first-child) {
  padding-right: 1rem !important;
}
.divide-x > :not(template):not(:last-child) {
  padding-left: 1rem !important;
}

.divide-y-0 > :not(template) ~ :not(template) {
  border-top: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}
.divide-y-0 > :not(template):not(:first-child) {
  padding-top: 0 !important;
}
.divide-y-0 > :not(template):not(:last-child) {
  padding-bottom: 0 !important;
}

.divide-x-0 > :not(template) ~ :not(template) {
  border-right: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}
.divide-x-0 > :not(template):not(:first-child) {
  padding-right: 0 !important;
}
.divide-x-0 > :not(template):not(:last-child) {
  padding-left: 0 !important;
}

.divide-y-1 > :not(template) ~ :not(template) {
  border-top: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}
.divide-y-1 > :not(template):not(:first-child) {
  padding-top: 0.25rem !important;
}
.divide-y-1 > :not(template):not(:last-child) {
  padding-bottom: 0.25rem !important;
}

.divide-x-1 > :not(template) ~ :not(template) {
  border-right: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}
.divide-x-1 > :not(template):not(:first-child) {
  padding-right: 0.25rem !important;
}
.divide-x-1 > :not(template):not(:last-child) {
  padding-left: 0.25rem !important;
}

.divide-y-2 > :not(template) ~ :not(template) {
  border-top: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}
.divide-y-2 > :not(template):not(:first-child) {
  padding-top: 0.5rem !important;
}
.divide-y-2 > :not(template):not(:last-child) {
  padding-bottom: 0.5rem !important;
}

.divide-x-2 > :not(template) ~ :not(template) {
  border-right: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}
.divide-x-2 > :not(template):not(:first-child) {
  padding-right: 0.5rem !important;
}
.divide-x-2 > :not(template):not(:last-child) {
  padding-left: 0.5rem !important;
}

.divide-y-3 > :not(template) ~ :not(template) {
  border-top: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}
.divide-y-3 > :not(template):not(:first-child) {
  padding-top: 1rem !important;
}
.divide-y-3 > :not(template):not(:last-child) {
  padding-bottom: 1rem !important;
}

.divide-x-3 > :not(template) ~ :not(template) {
  border-right: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}
.divide-x-3 > :not(template):not(:first-child) {
  padding-right: 1rem !important;
}
.divide-x-3 > :not(template):not(:last-child) {
  padding-left: 1rem !important;
}

.divide-y-4 > :not(template) ~ :not(template) {
  border-top: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}
.divide-y-4 > :not(template):not(:first-child) {
  padding-top: 1.5rem !important;
}
.divide-y-4 > :not(template):not(:last-child) {
  padding-bottom: 1.5rem !important;
}

.divide-x-4 > :not(template) ~ :not(template) {
  border-right: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}
.divide-x-4 > :not(template):not(:first-child) {
  padding-right: 1.5rem !important;
}
.divide-x-4 > :not(template):not(:last-child) {
  padding-left: 1.5rem !important;
}

.divide-y-5 > :not(template) ~ :not(template) {
  border-top: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}
.divide-y-5 > :not(template):not(:first-child) {
  padding-top: 2rem !important;
}
.divide-y-5 > :not(template):not(:last-child) {
  padding-bottom: 2rem !important;
}

.divide-x-5 > :not(template) ~ :not(template) {
  border-right: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}
.divide-x-5 > :not(template):not(:first-child) {
  padding-right: 2rem !important;
}
.divide-x-5 > :not(template):not(:last-child) {
  padding-left: 2rem !important;
}

.divide-y-6 > :not(template) ~ :not(template) {
  border-top: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}
.divide-y-6 > :not(template):not(:first-child) {
  padding-top: 2.5rem !important;
}
.divide-y-6 > :not(template):not(:last-child) {
  padding-bottom: 2.5rem !important;
}

.divide-x-6 > :not(template) ~ :not(template) {
  border-right: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent) !important;
}
.divide-x-6 > :not(template):not(:first-child) {
  padding-right: 2.5rem !important;
}
.divide-x-6 > :not(template):not(:last-child) {
  padding-left: 2.5rem !important;
}

.divide-y-fill {
  display: flex;
  flex-direction: column;
  height: 100%;
}
.divide-y-fill > :not(template) {
  flex: 1;
  display: flex;
  justify-content: center;
  flex-direction: column;
}

.icon {
  --tblr-icon-size: 1.25rem;
  width: var(--tblr-icon-size);
  height: var(--tblr-icon-size);
  font-size: var(--tblr-icon-size);
  vertical-align: bottom;
  stroke-width: 1.5;
}
.icon:hover {
  text-decoration: none;
}

.icon-inline {
  --tblr-icon-size: 1rem;
  vertical-align: -0.2rem;
}

.icon-filled {
  fill: currentColor;
}

.icon-sm {
  --tblr-icon-size: 1rem;
  stroke-width: 1;
}

.icon-md {
  --tblr-icon-size: 2.5rem;
  stroke-width: 1;
}

.icon-lg {
  --tblr-icon-size: 3.5rem;
  stroke-width: 1;
}

.icon-pulse {
  transition: all 0.15s ease 0s;
  animation: pulse 2s ease infinite;
  animation-fill-mode: both;
}

.icon-tada {
  transition: all 0.15s ease 0s;
  animation: tada 3s ease infinite;
  animation-fill-mode: both;
}

.icon-rotate {
  transition: all 0.15s ease 0s;
  animation: rotate-360 3s linear infinite;
  animation-fill-mode: both;
}

.img-responsive {
  --tblr-img-responsive-ratio: 75%;
  background: no-repeat center/cover;
  padding-top: var(--tblr-img-responsive-ratio);
}

.img-responsive-grid {
  padding-top: calc(var(--tblr-img-responsive-ratio) - var(--tblr-gutter-y) / 2);
}

.img-responsive-1x1 {
  --tblr-img-responsive-ratio: 100%;
}

.img-responsive-2x1 {
  --tblr-img-responsive-ratio: 50%;
}

.img-responsive-1x2 {
  --tblr-img-responsive-ratio: 200%;
}

.img-responsive-3x1 {
  --tblr-img-responsive-ratio: 33.3333333333%;
}

.img-responsive-1x3 {
  --tblr-img-responsive-ratio: 300%;
}

.img-responsive-4x1 {
  --tblr-img-responsive-ratio: 25%;
}

.img-responsive-1x4 {
  --tblr-img-responsive-ratio: 400%;
}

.img-responsive-4x3 {
  --tblr-img-responsive-ratio: 75%;
}

.img-responsive-3x4 {
  --tblr-img-responsive-ratio: 133.3333333333%;
}

.img-responsive-16x9 {
  --tblr-img-responsive-ratio: 56.25%;
}

.img-responsive-9x16 {
  --tblr-img-responsive-ratio: 177.7777777778%;
}

.img-responsive-21x9 {
  --tblr-img-responsive-ratio: 42.8571428571%;
}

.img-responsive-9x21 {
  --tblr-img-responsive-ratio: 233.3333333333%;
}

.img-bg {
  background: no-repeat center/cover;
}

textarea[cols] {
  height: auto;
}

/**
Form label
 */
.col-form-label,
.form-label {
  display: block;
  font-weight: var(--tblr-font-weight-medium);
}
.col-form-label.required:after,
.form-label.required:after {
  content: "*";
  margin-right: 0.25rem;
  color: #d63939;
}

.form-label-description {
  float: left;
  font-weight: var(--tblr-font-weight-normal);
  color: var(--tblr-secondary);
}

/**
Form hint
 */
.form-hint {
  display: block;
  color: var(--tblr-secondary);
}
.form-hint:last-child {
  margin-bottom: 0;
}
.form-hint + .form-control {
  margin-top: 0.25rem;
}
.form-label + .form-hint {
  margin-top: -0.25rem;
}
.input-group + .form-hint, .form-control + .form-hint, .form-select + .form-hint {
  margin-top: 0.5rem;
  color: var(--tblr-secondary);
}

/**
Form select
 */
.form-select:-moz-focusring {
  color: var(--tblr-body-color);
}

/**
Form control
 */
.form-control:-webkit-autofill {
  box-shadow: 0 0 0 1000px var(--tblr-bg-surface-secondary) inset;
  color: var(--tblr-body-color);
  -webkit-text-fill-color: var(--tblr-body-color);
}
.form-control:disabled, .form-control.disabled {
  color: var(--tblr-secondary);
  user-select: none;
}
.form-control[size] {
  width: auto;
}

.form-control-light {
  background-color: var(--tblr-gray-100);
  border-color: transparent;
}

.form-control-dark {
  background-color: rgba(0, 0, 0, 0.1);
  color: #ffffff;
  border-color: transparent;
}
.form-control-dark:focus {
  background-color: rgba(0, 0, 0, 0.1);
  box-shadow: none;
  border-color: rgba(255, 255, 255, 0.24);
}
.form-control-dark::placeholder {
  color: rgba(255, 255, 255, 0.6);
}

.form-control-rounded {
  border-radius: 10rem;
}

.form-control-flush {
  padding: 0;
  background: none !important;
  border-color: transparent !important;
  resize: none;
  box-shadow: none !important;
  line-height: inherit;
}

.form-footer {
  margin-top: 2rem;
}

.form-fieldset {
  padding: 1rem;
  margin-bottom: 1rem;
  background: var(--tblr-bg-surface-secondary);
  border: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
  border-radius: var(--tblr-border-radius);
}

fieldset:empty {
  display: none;
}

/**
Form help
 */
.form-help {
  display: inline-flex;
  font-weight: var(--tblr-font-weight-bold);
  align-items: center;
  justify-content: center;
  width: 1.125rem;
  height: 1.125rem;
  font-size: 0.75rem;
  color: var(--tblr-secondary);
  text-align: center;
  text-decoration: none;
  cursor: pointer;
  user-select: none;
  background: var(--tblr-gray-100);
  border-radius: 100rem;
  transition: background-color 0.3s, color 0.3s;
}
@media (prefers-reduced-motion: reduce) {
  .form-help {
    transition: none;
  }
}
.form-help:hover, .form-help[aria-describedby] {
  color: #ffffff;
  background: var(--tblr-primary);
}

/**
Input group
 */
.input-group {
  box-shadow: var(--tblr-shadow-input);
  border-radius: var(--tblr-border-radius);
}
.input-group .form-control,
.input-group .btn {
  box-shadow: none;
}

.input-group-link {
  font-size: 0.75rem;
}

.input-group-flat:focus-within {
  box-shadow: 0 0 0 0.25rem rgba(var(--tblr-primary-rgb), 0.25);
  border-radius: var(--tblr-border-radius);
}
.input-group-flat:focus-within .form-control,
.input-group-flat:focus-within .input-group-text {
  border-color: rgb(130.5, 183, 232) !important;
}
.input-group-flat .form-control:focus {
  border-color: var(--tblr-border-color);
  box-shadow: none;
}
.input-group-flat .form-control:not(:last-child) {
  border-left: 0;
}
.input-group-flat .form-control:not(:first-child) {
  border-right: 0;
}
.input-group-flat .input-group-text {
  background: var(--tblr-bg-forms);
  transition: border-color 0.15s ease-in-out, box-shadow 0.15s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .input-group-flat .input-group-text {
    transition: none;
  }
}
.input-group-flat .input-group-text:first-child {
  padding-left: 0;
}
.input-group-flat .input-group-text:last-child {
  padding-right: 0;
}

/**
Upload files
 */
.form-file-button {
  margin-right: 0;
  border-right: 0;
}

/**
Floating inputs
 */
label[for=floating-input] {
  max-width: 100%;
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap;
}

/**
Forms on mobile devices 
  */
@media (max-width: 575.98px) {
  .form-control,
  .form-select {
    font-size: 1rem;
  }
}

/**
Icon input
 */
.input-icon {
  position: relative;
}
.input-icon .form-control:not(:last-child),
.input-icon .form-select:not(:last-child) {
  padding-left: 2.5rem;
}
.input-icon .form-control:not(:first-child),
.input-icon .form-select:not(:last-child) {
  padding-right: 2.5rem;
}

.input-icon-addon {
  position: absolute;
  top: 0;
  bottom: 0;
  right: 0;
  display: flex;
  align-items: center;
  justify-content: center;
  min-width: 2.5rem;
  color: var(--tblr-icon-color);
  pointer-events: none;
  font-size: 1.2em;
}
.input-icon-addon:last-child {
  left: 0;
  right: auto;
}

/*
Color Input
 */
.form-colorinput {
  position: relative;
  display: inline-block;
  margin: 0;
  line-height: 1;
  cursor: pointer;
}

.form-colorinput-input {
  position: absolute;
  z-index: -1;
  opacity: 0;
}

.form-colorinput-color {
  display: block;
  width: 1.5rem;
  height: 1.5rem;
  color: #ffffff;
  border: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent);
  border-radius: var(--tblr-border-radius);
  box-shadow: 0 1px 2px 0 rgba(0, 0, 0, 0.05);
}
.form-colorinput-color:before {
  position: absolute;
  top: 0;
  right: 0;
  width: 100%;
  height: 100%;
  content: "";
  background: no-repeat center center/1.25rem;
  background-image: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%270 0 16 16%27 width=%2716%27 height=%2716%27%3e%3cpath fill=%27none%27 stroke=%27%23ffffff%27 stroke-linecap=%27round%27 stroke-linejoin=%27round%27 stroke-width=%272%27 d=%27M4 8.5l2.5 2.5l5.5 -5.5%27/%3e%3c/svg%3e");
  opacity: 0;
  transition: opacity 0.3s;
}
@media (prefers-reduced-motion: reduce) {
  .form-colorinput-color:before {
    transition: none;
  }
}
.form-colorinput-input:checked ~ .form-colorinput-color:before {
  opacity: 1;
}
.form-colorinput-input:focus ~ .form-colorinput-color {
  border-color: var(--tblr-primary);
  box-shadow: 0 0 0 0.25rem rgba(var(--tblr-primary-rgb), 0.25);
}
.form-colorinput-light .form-colorinput-color:before {
  background-image: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%270 0 16 16%27 width=%2716%27 height=%2716%27%3e%3cpath fill=%27none%27 stroke=%27%231f2937%27 stroke-linecap=%27round%27 stroke-linejoin=%27round%27 stroke-width=%272%27 d=%27M4 8.5l2.5 2.5l5.5 -5.5%27/%3e%3c/svg%3e");
}

/**
Image check
 */
.form-imagecheck {
  --tblr-form-imagecheck-radius: var(--tblr-border-radius);
  position: relative;
  margin: 0;
  cursor: pointer;
}

.form-imagecheck-input {
  position: absolute;
  z-index: -1;
  opacity: 0;
}

.form-imagecheck-figure {
  position: relative;
  display: block;
  margin: 0;
  user-select: none;
  border: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
  border-radius: var(--tblr-form-imagecheck-radius);
}
.form-imagecheck-input:focus ~ .form-imagecheck-figure {
  border-color: var(--tblr-primary);
  box-shadow: 0 0 0 0.25rem rgba(var(--tblr-primary-rgb), 0.25);
}
.form-imagecheck-input:checked ~ .form-imagecheck-figure {
  border-color: var(--tblr-primary);
}
.form-imagecheck-figure:before {
  position: absolute;
  top: 0.25rem;
  right: 0.25rem;
  z-index: 1;
  display: block;
  width: 1.25rem;
  height: 1.25rem;
  color: #ffffff;
  pointer-events: none;
  content: "";
  user-select: none;
  background: var(--tblr-bg-forms);
  border: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
  border-radius: var(--tblr-border-radius);
  transition: opacity 0.3s;
}
@media (prefers-reduced-motion: reduce) {
  .form-imagecheck-figure:before {
    transition: none;
  }
}
.form-imagecheck-input:checked ~ .form-imagecheck-figure:before {
  background-color: var(--tblr-primary);
  background-image: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%270 0 16 16%27 width=%2716%27 height=%2716%27%3e%3cpath fill=%27none%27 stroke=%27%23ffffff%27 stroke-linecap=%27round%27 stroke-linejoin=%27round%27 stroke-width=%272%27 d=%27M4 8.5l2.5 2.5l5.5 -5.5%27/%3e%3c/svg%3e");
  background-repeat: repeat;
  background-position: center;
  background-size: 1.25rem;
  border-color: var(--tblr-border-color-translucent);
}
.form-imagecheck-input[type=radio] ~ .form-imagecheck-figure:before {
  border-radius: 50%;
}
.form-imagecheck-input[type=radio]:checked ~ .form-imagecheck-figure:before {
  background-image: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%270 0 16 16%27%3e%3ccircle r=%273%27 fill=%27%23ffffff%27 cx=%278%27 cy=%278%27 /%3e%3c/svg%3e");
}

.form-imagecheck-image {
  max-width: 100%;
  display: block;
  opacity: 0.64;
  transition: opacity 0.3s;
}
@media (prefers-reduced-motion: reduce) {
  .form-imagecheck-image {
    transition: none;
  }
}
.form-imagecheck-image:first-child {
  border-top-right-radius: calc(var(--tblr-form-imagecheck-radius) - 1px);
  border-top-left-radius: calc(var(--tblr-form-imagecheck-radius) - 1px);
}
.form-imagecheck-image:last-child {
  border-bottom-left-radius: calc(var(--tblr-form-imagecheck-radius) - 1px);
  border-bottom-right-radius: calc(var(--tblr-form-imagecheck-radius) - 1px);
}
.form-imagecheck:hover .form-imagecheck-image, .form-imagecheck-input:focus ~ .form-imagecheck-figure .form-imagecheck-image, .form-imagecheck-input:checked ~ .form-imagecheck-figure .form-imagecheck-image {
  opacity: 1;
}

.form-imagecheck-caption {
  padding: 0.25rem;
  font-size: 0.765625rem;
  color: var(--tblr-secondary);
  text-align: center;
  transition: color 0.3s;
}
@media (prefers-reduced-motion: reduce) {
  .form-imagecheck-caption {
    transition: none;
  }
}
.form-imagecheck:hover .form-imagecheck-caption, .form-imagecheck-input:focus ~ .form-imagecheck-figure .form-imagecheck-caption, .form-imagecheck-input:checked ~ .form-imagecheck-figure .form-imagecheck-caption {
  color: var(--tblr-body-color);
}

/*
Select group
 */
.form-selectgroup {
  display: inline-flex;
  margin: 0 0 -0.5rem -0.5rem;
  flex-wrap: wrap;
}
.form-selectgroup .form-selectgroup-item {
  margin: 0 0 0.5rem 0.5rem;
}

.form-selectgroup-vertical {
  flex-direction: column;
}

.form-selectgroup-item {
  display: block;
  position: relative;
}

.form-selectgroup-input {
  position: absolute;
  top: 0;
  right: 0;
  z-index: -1;
  opacity: 0;
}

.form-selectgroup-label {
  position: relative;
  display: block;
  min-width: calc(1.25rem + 1.125rem + calc(var(--tblr-border-width) * 2));
  margin: 0;
  padding: 0.5625rem 1rem;
  font-size: 0.875rem;
  line-height: 1.25rem;
  color: var(--tblr-secondary);
  background: var(--tblr-bg-forms);
  text-align: center;
  cursor: pointer;
  user-select: none;
  border: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
  border-radius: var(--tblr-border-radius);
  box-shadow: var(--tblr-shadow-input);
  transition: border-color 0.3s, background 0.3s, color 0.3s;
}
@media (prefers-reduced-motion: reduce) {
  .form-selectgroup-label {
    transition: none;
  }
}
.form-selectgroup-label .icon:only-child {
  margin: 0 -0.25rem;
}
.form-selectgroup-label:hover {
  color: var(--tblr-body-color);
}

.form-selectgroup-check {
  display: inline-block;
  width: 1.25rem;
  height: 1.25rem;
  border: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent);
  vertical-align: middle;
  box-shadow: var(--tblr-shadow-input);
}
.form-selectgroup-input[type=checkbox] + .form-selectgroup-label .form-selectgroup-check {
  border-radius: var(--tblr-border-radius);
}
.form-selectgroup-input[type=radio] + .form-selectgroup-label .form-selectgroup-check {
  border-radius: 50%;
}
.form-selectgroup-input:checked + .form-selectgroup-label .form-selectgroup-check {
  background-color: var(--tblr-primary);
  background-repeat: repeat;
  background-position: center;
  background-size: 1.25rem;
  border-color: var(--tblr-border-color-translucent);
}
.form-selectgroup-input[type=checkbox]:checked + .form-selectgroup-label .form-selectgroup-check {
  background-image: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%270 0 16 16%27 width=%2716%27 height=%2716%27%3e%3cpath fill=%27none%27 stroke=%27%23ffffff%27 stroke-linecap=%27round%27 stroke-linejoin=%27round%27 stroke-width=%272%27 d=%27M4 8.5l2.5 2.5l5.5 -5.5%27/%3e%3c/svg%3e");
}
.form-selectgroup-input[type=radio]:checked + .form-selectgroup-label .form-selectgroup-check {
  background-image: url("data:image/svg+xml,%3csvg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%270 0 16 16%27%3e%3ccircle r=%273%27 fill=%27%23ffffff%27 cx=%278%27 cy=%278%27 /%3e%3c/svg%3e");
}

.form-selectgroup-check-floated {
  position: absolute;
  top: 0.5625rem;
  left: 0.5625rem;
}

.form-selectgroup-input:checked + .form-selectgroup-label {
  z-index: 1;
  color: var(--tblr-primary);
  background: rgba(var(--tblr-primary-rgb), 0.04);
  border-color: var(--tblr-primary);
}

.form-selectgroup-input:focus + .form-selectgroup-label {
  z-index: 2;
  color: var(--tblr-primary);
  border-color: var(--tblr-primary);
  box-shadow: 0 0 0 0.25rem rgba(var(--tblr-primary-rgb), 0.25);
}

/**
Alternate version of form select group
 */
.form-selectgroup-boxes .form-selectgroup-label {
  text-align: right;
  padding: 1.25rem 1rem;
  color: inherit;
}
.form-selectgroup-boxes .form-selectgroup-input:checked + .form-selectgroup-label {
  color: inherit;
}
.form-selectgroup-boxes .form-selectgroup-input:checked + .form-selectgroup-label .form-selectgroup-title {
  color: var(--tblr-primary);
}
.form-selectgroup-boxes .form-selectgroup-input:checked + .form-selectgroup-label .form-selectgroup-label-content {
  opacity: 1;
}

/**
Select group
 */
.form-selectgroup-pills {
  flex-wrap: wrap;
  align-items: flex-start;
}
.form-selectgroup-pills .form-selectgroup-item {
  flex-grow: 0;
}
.form-selectgroup-pills .form-selectgroup-label {
  border-radius: 50px;
}

/**
Bootstrap color input
 */
.form-control-color::-webkit-color-swatch {
  border: none;
}

/**
Remove the cancel buttons in Chrome and Safari on macOS.
 */
[type=search]::-webkit-search-cancel-button {
  -webkit-appearance: none;
}

/**
Form control dark theme fix
 */
.form-control::file-selector-button {
  background-color: var(--tblr-btn-color, var(--tblr-tertiary-bg));
}
.form-control:hover:not(:disabled):not([readonly])::file-selector-button {
  background-color: var(--tblr-btn-color, var(--tblr-secondary-bg));
}

/*
Form check
 */
.form-check {
  user-select: none;
}
.form-check.form-check-highlight .form-check-input:not(:checked) ~ .form-check-label {
  color: var(--tblr-secondary);
}
.form-check .form-check-label-off {
  color: var(--tblr-secondary);
}
.form-check .form-check-input:checked ~ .form-check-label-off {
  display: none;
}
.form-check .form-check-input:not(:checked) ~ .form-check-label-on {
  display: none;
}

.form-check-input {
  background-size: 1.25rem;
  margin-top: 0rem;
  box-shadow: var(--tblr-shadow-input);
}
.form-switch .form-check-input {
  transition: background-color 0.3s, background-position 0.3s;
}
@media (prefers-reduced-motion: reduce) {
  .form-switch .form-check-input {
    transition: none;
  }
}

.form-check-label {
  display: block;
}
.form-check-label.required:after {
  content: "*";
  margin-right: 0.25rem;
  color: #d63939;
}

.form-check-description {
  display: block;
  color: var(--tblr-secondary);
  font-size: 0.75rem;
  margin-top: 0.25rem;
}

.form-check-single {
  margin: 0;
}
.form-check-single .form-check-input {
  margin: 0;
}

/*
Form switch
 */
.form-switch .form-check-input {
  height: 1.25rem;
  margin-top: 0rem;
}

.form-switch-lg {
  padding-right: 3.5rem;
  min-height: 1.5rem;
}
.form-switch-lg .form-check-input {
  height: 1.5rem;
  width: 2.75rem;
  background-size: 1.5rem;
  margin-right: -3.5rem;
}
.form-switch-lg .form-check-label {
  padding-top: 0.125rem;
}

/*Correction of Form-check position*/
.form-check-input:checked {
  border: none;
}

.form-select.is-invalid-lite, .form-control.is-invalid-lite, .form-select.is-valid-lite, .form-control.is-valid-lite {
  border-color: var(--tblr-border-color) !important;
}

/**
Legend
 */
.legend {
  --tblr-legend-size: 0.75em;
  display: inline-block;
  background: var(--tblr-border-color);
  width: var(--tblr-legend-size);
  height: var(--tblr-legend-size);
  border-radius: var(--tblr-border-radius-sm);
  border: 1px solid var(--tblr-border-color-translucent);
}

.list-group {
  margin-right: 0;
  margin-left: 0;
}

.list-group-header {
  background: var(--tblr-bg-surface-tertiary);
  padding: 0.5rem 1.25rem;
  font-size: 0.75rem;
  font-weight: var(--tblr-font-weight-medium);
  line-height: 1;
  text-transform: uppercase;
  color: var(--tblr-gray-500);
  border-bottom: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
}
.list-group-flush > .list-group-header:last-child {
  border-bottom-width: 0;
}

.list-group-item {
  background-color: inherit;
}

.list-group-item.active {
  background-color: rgba(var(--tblr-secondary-rgb), 0.08);
  border-right-color: #066fd1;
  border-right-width: 2px;
}

.list-group-item.disabled, .list-group-item:disabled {
  color: #6b7280;
  background-color: rgba(var(--tblr-secondary-rgb), 0.08);
}

.list-bordered .list-item {
  border-top: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
  margin-top: -1px;
}
.list-bordered .list-item:first-child {
  border-top: none;
}

.list-group-hoverable .list-group-item:active, .list-group-hoverable .list-group-item:focus, .list-group-hoverable .list-group-item:hover {
  background-color: rgba(var(--tblr-secondary-rgb), 0.08);
}
.list-group-hoverable .list-group-item-actions {
  opacity: 0;
  transition: opacity 0.3s;
}
@media (prefers-reduced-motion: reduce) {
  .list-group-hoverable .list-group-item-actions {
    transition: none;
  }
}
.list-group-hoverable .list-group-item:hover .list-group-item-actions,
.list-group-hoverable .list-group-item-actions.show {
  opacity: 1;
}

.list-group-transparent {
  --tblr-list-group-border-radius: 0;
  margin: 0 -1.25rem;
}
.list-group-transparent .list-group-item {
  background: none;
  border: 0;
}
.list-group-transparent .list-group-item .icon {
  color: var(--tblr-secondary);
}
.list-group-transparent .list-group-item.active {
  font-weight: var(--tblr-font-weight-bold);
  color: inherit;
  background: var(--tblr-active-bg);
}
.list-group-transparent .list-group-item.active .icon {
  color: inherit;
}

/**
Seprated list
*/
.list-separated {
  display: flex;
  flex-direction: column;
  gap: 1rem;
}

/**
Inline list
 */
.list-inline {
  margin: 0;
}

.list-inline-item:not(:last-child) {
  margin-left: auto;
  margin-inline-end: 0.5rem;
}

.list-inline-dots .list-inline-item + .list-inline-item:before {
  content: " · ";
  margin-inline-end: 0.5rem;
}

.loader {
  position: relative;
  display: block;
  width: 2.5rem;
  height: 2.5rem;
  color: #066fd1;
  vertical-align: middle;
}
.loader:after {
  position: absolute;
  top: 0;
  right: 0;
  width: 100%;
  height: 100%;
  content: "";
  border: 1px var(--tblr-border-style);
  border-color: transparent;
  border-top-color: currentColor;
  border-right-color: currentColor;
  border-radius: 100rem;
  animation: rotate-360 0.6s linear;
  animation-iteration-count: infinite;
}

/**
Dimmer
*/
.dimmer {
  position: relative;
}
.dimmer .loader {
  position: absolute;
  top: 50%;
  left: 0;
  right: 0;
  display: none;
  margin: 0 auto;
  transform: translateY(-50%);
}
.dimmer.active .loader {
  display: block;
}
.dimmer.active .dimmer-content {
  pointer-events: none;
  opacity: 0.1;
}

@keyframes animated-dots {
  0% {
    transform: translateX(100%);
  }
}
.animated-dots {
  display: inline-block;
  overflow: hidden;
  vertical-align: bottom;
}
.animated-dots:after {
  display: inline-block;
  content: "...";
  animation: animated-dots 1.2s steps(4, jump-none) infinite;
}

.modal-content > .btn-close, .modal-header > .btn-close {
  position: absolute;
  top: 0;
  left: 0;
  width: 3.5rem;
  height: 3.5rem;
  margin: 0;
  padding: 0;
  z-index: 10;
}

.modal-body {
  scrollbar-color: color-mix(in srgb, var(--tblr-scrollbar-color, var(--tblr-body-color)) 20%, transparent) transparent;
}
.modal-body::-webkit-scrollbar {
  width: 1rem;
  height: 1rem;
  transition: background 0.3s;
}
@media (prefers-reduced-motion: reduce) {
  .modal-body::-webkit-scrollbar {
    transition: none;
  }
}
.modal-body::-webkit-scrollbar-thumb {
  border-radius: 1rem;
  border: 5px solid transparent;
  box-shadow: inset 0 0 0 1rem color-mix(in srgb, var(--tblr-scrollbar-color, var(--tblr-body-color)) 20%, transparent);
}
.modal-body::-webkit-scrollbar-track {
  background: transparent;
}
.modal-body:hover::-webkit-scrollbar-thumb {
  box-shadow: inset 0 0 0 1rem color-mix(in srgb, var(--tblr-scrollbar-color, var(--tblr-body-color)) 40%, transparent);
}
.modal-body::-webkit-scrollbar-corner {
  background: transparent;
}
.modal-body .modal-title {
  margin-bottom: 1rem;
}
.modal-body + .modal-body {
  border-top: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
}

.modal-status {
  position: absolute;
  top: 0;
  right: 0;
  left: 0;
  height: 2px;
  background: var(--tblr-secondary);
  border-radius: var(--tblr-border-radius-lg) var(--tblr-border-radius-lg) 0 0;
}

.modal-header {
  align-items: center;
  min-height: 3.5rem;
  background: transparent;
  padding: 0 1.5rem 0 3.5rem;
}

.modal-title {
  font-size: 1rem;
  font-weight: var(--tblr-font-weight-bold);
  color: inherit;
  line-height: 1.4285714286;
}

.modal-footer {
  padding-top: 0.75rem;
  padding-bottom: 0.75rem;
}

.modal-blur {
  backdrop-filter: blur(4px);
}

.modal-full-width {
  max-width: none;
  margin: 0 0.5rem;
}

.nav {
  --tblr-nav-link-hover-bg: color-mix(in srgb, var(--tblr-nav-link-color) 4%, transparent);
}

.nav-vertical,
.nav-vertical .nav {
  flex-direction: column;
  flex-wrap: nowrap;
}
.nav-vertical .nav {
  margin-right: 1.25rem;
  border-right: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
  padding-right: 0.5rem;
}
.nav-vertical .nav-link.active,
.nav-vertical .nav-item.show .nav-link {
  font-weight: var(--tblr-font-weight-bold);
  color: var(--tblr-nav-link-active-color);
}
.nav-vertical.nav-pills {
  margin: 0 -0.75rem;
}

.nav-bordered {
  border-bottom: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
}
.nav-bordered .nav-item + .nav-item {
  margin-right: 1.25rem;
}
.nav-bordered .nav-link {
  padding-right: 0;
  padding-left: 0;
  margin: 0 0 calc(-1 * var(--tblr-border-width));
  border: 0;
  border-bottom: 2px var(--tblr-border-style) transparent;
}
.nav-bordered .nav-link:hover {
  background-color: transparent;
}
.nav-bordered .nav-link.active,
.nav-bordered .nav-item.show .nav-link {
  color: var(--tblr-primary);
  border-color: var(--tblr-primary);
}

.nav-underline .nav-link {
  border-radius: 0;
}

.nav-link {
  display: flex;
  transition: color 0.3s, background-color 0.3s;
  align-items: center;
}
@media (prefers-reduced-motion: reduce) {
  .nav-link {
    transition: none;
  }
}
.nav-link:hover, .nav-link:focus {
  background-color: var(--tblr-nav-link-hover-bg);
}

.nav-link-toggle {
  margin-right: auto;
  padding: 0 0.25rem;
  transition: transform 0.3s;
}
@media (prefers-reduced-motion: reduce) {
  .nav-link-toggle {
    transition: none;
  }
}
.nav-link-toggle:after {
  content: "";
  display: inline-block;
  vertical-align: 0.306em;
  width: 0.36em;
  height: 0.36em;
  border-bottom: 1px var(--tblr-border-style);
  border-right: 1px var(--tblr-border-style);
  margin-left: 0.1em;
  margin-right: 0.4em;
  transform: rotate(45deg);
}
.nav-link-toggle:after {
  margin: 0;
}
.nav-link[aria-expanded=true] .nav-link-toggle {
  transform: rotate(-180deg);
}

.nav-link-icon {
  width: 1.25rem;
  height: 1.25rem;
  margin-left: 0.5rem;
  color: inherit;
}
.nav-link-icon svg {
  display: block;
  height: 100%;
}

.nav-fill .nav-item .nav-link {
  justify-content: center;
}

.stars {
  display: inline-flex;
  color: #9ca3af;
  font-size: 0.75rem;
}
.stars .star:not(:first-child) {
  margin-right: 0.25rem;
}

.pagination {
  margin: 0;
  --tblr-pagination-gap: .25rem;
  user-select: none;
  gap: var(--tblr-pagination-gap);
  line-height: var(--tblr-body-line-height);
}

.page-link {
  min-width: 2rem;
  border-radius: var(--tblr-pagination-border-radius);
}

.page-item:not(.active) .page-link:hover {
  background: var(--tblr-pagination-hover-bg);
}

.page-text {
  padding-right: 0.5rem;
  padding-left: 0.5rem;
}

.page-item {
  text-align: center;
}
.page-item.page-prev, .page-item.page-next {
  flex: 0 0 50%;
  text-align: right;
}
.page-item.page-next {
  margin-right: auto;
  text-align: left;
}

.page-item-subtitle {
  margin-bottom: 2px;
  font-size: 12px;
  color: var(--tblr-secondary);
  text-transform: uppercase;
}
.page-item.disabled .page-item-subtitle {
  color: var(--tblr-disabled-color);
}

.page-item-title {
  font-size: 1rem;
  font-weight: var(--tblr-font-weight-normal);
  color: var(--tblr-body-color);
}
.page-link:hover .page-item-title {
  color: #066fd1;
}
.page-item.disabled .page-item-title {
  color: var(--tblr-disabled-color);
}

.pagination-outline {
  --tblr-pagination-border-color: var(--tblr-border-color);
  --tblr-pagination-disabled-border-color: var(--tblr-border-color);
  --tblr-pagination-border-width: 1px;
}

.pagination-circle {
  --tblr-pagination-border-radius: var(--tblr-border-radius-pill);
}

@keyframes progress-indeterminate {
  0% {
    left: 100%;
    right: -35%;
  }
  100%, 60% {
    left: -90%;
    right: 100%;
  }
}
/**
Progress
 */
.progress {
  position: relative;
  width: 100%;
  line-height: 0.5rem;
  appearance: none;
}
.progress::-webkit-progress-bar {
  background: var(--tblr-progress-bg);
}
.progress::-webkit-progress-value {
  background-color: var(--tblr-primary);
}
.progress::-moz-progress-bar {
  background-color: var(--tblr-primary);
}
.progress::-ms-fill {
  background-color: var(--tblr-primary);
  border: none;
}

.progress-sm {
  height: 0.25rem;
}

/**
Progress bar
 */
.progress-bar {
  height: 100%;
}

.progress-bar-indeterminate:after, .progress-bar-indeterminate:before {
  position: absolute;
  top: 0;
  bottom: 0;
  right: 0;
  content: "";
  background-color: inherit;
  will-change: left, right;
}
.progress-bar-indeterminate:before {
  animation: progress-indeterminate 1.5s cubic-bezier(0.65, 0.815, 0.735, 0.395) infinite;
}

.progress-separated .progress-bar {
  box-shadow: 0 0 0 2px var(--tblr-card-bg, var(--tblr-bg-surface));
}

/**
Progressbg
 */
.progressbg {
  position: relative;
  padding: 0.25rem 0.5rem;
  display: flex;
}

.progressbg-text {
  position: relative;
  z-index: 1;
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap;
}

.progressbg-progress {
  position: absolute;
  top: 0;
  left: 0;
  bottom: 0;
  right: 0;
  z-index: 0;
  height: 100%;
  background: transparent;
  pointer-events: none;
}

.progressbg-value {
  font-weight: var(--tblr-font-weight-medium);
  margin-right: auto;
  padding-right: 2rem;
}

.ribbon {
  --tblr-ribbon-margin: 0.25rem;
  --tblr-ribbon-border-radius: var(--tblr-border-radius);
  position: absolute;
  top: 0.75rem;
  left: calc(-1 * var(--tblr-ribbon-margin));
  z-index: 1;
  padding: 0.25rem 0.75rem;
  font-size: 0.625rem;
  font-weight: var(--tblr-font-weight-bold);
  line-height: 1;
  color: #ffffff;
  text-align: center;
  text-transform: uppercase;
  background: var(--tblr-primary);
  border-color: var(--tblr-primary);
  border-radius: 0 var(--tblr-ribbon-border-radius) var(--tblr-ribbon-border-radius) var(--tblr-ribbon-border-radius);
  display: inline-flex;
  align-items: center;
  justify-content: center;
  min-height: 2rem;
  min-width: 2rem;
}
.ribbon:before {
  position: absolute;
  left: 0;
  bottom: 100%;
  width: 0;
  height: 0;
  content: "";
  filter: brightness(70%);
  border: calc(var(--tblr-ribbon-margin) * 0.5) var(--tblr-border-style);
  border-color: inherit;
  border-top-color: transparent;
  border-left-color: transparent;
}
.ribbon.bg-blue {
  border-color: var(--tblr-blue);
}
.ribbon.bg-blue-lt {
  border-color: rgba(var(--tblr-blue-rgb), 0.1) !important;
}
.ribbon.bg-azure {
  border-color: var(--tblr-azure);
}
.ribbon.bg-azure-lt {
  border-color: rgba(var(--tblr-azure-rgb), 0.1) !important;
}
.ribbon.bg-indigo {
  border-color: var(--tblr-indigo);
}
.ribbon.bg-indigo-lt {
  border-color: rgba(var(--tblr-indigo-rgb), 0.1) !important;
}
.ribbon.bg-purple {
  border-color: var(--tblr-purple);
}
.ribbon.bg-purple-lt {
  border-color: rgba(var(--tblr-purple-rgb), 0.1) !important;
}
.ribbon.bg-pink {
  border-color: var(--tblr-pink);
}
.ribbon.bg-pink-lt {
  border-color: rgba(var(--tblr-pink-rgb), 0.1) !important;
}
.ribbon.bg-red {
  border-color: var(--tblr-red);
}
.ribbon.bg-red-lt {
  border-color: rgba(var(--tblr-red-rgb), 0.1) !important;
}
.ribbon.bg-orange {
  border-color: var(--tblr-orange);
}
.ribbon.bg-orange-lt {
  border-color: rgba(var(--tblr-orange-rgb), 0.1) !important;
}
.ribbon.bg-yellow {
  border-color: var(--tblr-yellow);
}
.ribbon.bg-yellow-lt {
  border-color: rgba(var(--tblr-yellow-rgb), 0.1) !important;
}
.ribbon.bg-lime {
  border-color: var(--tblr-lime);
}
.ribbon.bg-lime-lt {
  border-color: rgba(var(--tblr-lime-rgb), 0.1) !important;
}
.ribbon.bg-green {
  border-color: var(--tblr-green);
}
.ribbon.bg-green-lt {
  border-color: rgba(var(--tblr-green-rgb), 0.1) !important;
}
.ribbon.bg-teal {
  border-color: var(--tblr-teal);
}
.ribbon.bg-teal-lt {
  border-color: rgba(var(--tblr-teal-rgb), 0.1) !important;
}
.ribbon.bg-cyan {
  border-color: var(--tblr-cyan);
}
.ribbon.bg-cyan-lt {
  border-color: rgba(var(--tblr-cyan-rgb), 0.1) !important;
}
.ribbon .icon {
  width: 1.25rem;
  height: 1.25rem;
  font-size: 1.25rem;
}

.ribbon-top {
  top: calc(-1 * var(--tblr-ribbon-margin));
  left: 0.75rem;
  width: 2rem;
  padding: 0.5rem 0;
  border-radius: var(--tblr-ribbon-border-radius) 0 var(--tblr-ribbon-border-radius) var(--tblr-ribbon-border-radius);
}
.ribbon-top:before {
  top: 0;
  left: 100%;
  bottom: auto;
  border-color: inherit;
  border-top-color: transparent;
  border-right-color: transparent;
}
.ribbon-top.ribbon-start {
  left: auto;
  right: 0.75rem;
}
.ribbon-top.ribbon-start:before {
  top: 0;
  left: 100%;
  right: auto;
}

.ribbon-start {
  left: auto;
  right: calc(-1 * var(--tblr-ribbon-margin));
  border-radius: var(--tblr-ribbon-border-radius) 0 var(--tblr-ribbon-border-radius) var(--tblr-ribbon-border-radius);
}
.ribbon-start:before {
  top: auto;
  bottom: 100%;
  right: 0;
  border-color: inherit;
  border-top-color: transparent;
  border-right-color: transparent;
}

.ribbon-bottom {
  top: auto;
  bottom: 0.75rem;
}

.ribbon-bookmark {
  padding-right: 0.25rem;
  border-radius: 0 0 0 var(--tblr-ribbon-border-radius);
}
.ribbon-bookmark:after {
  position: absolute;
  top: 0;
  left: 100%;
  display: block;
  width: 0;
  height: 0;
  content: "";
  border: 1rem var(--tblr-border-style);
  border-color: inherit;
  border-left-width: 0;
  border-right-color: transparent;
  border-right-width: 0.5rem;
}
.ribbon-bookmark.ribbon-left {
  padding-left: 0.5rem;
}
.ribbon-bookmark.ribbon-left:after {
  left: auto;
  right: 100%;
  border-left-color: transparent;
  border-left-width: 0.5rem;
  border-right-width: 0;
}
.ribbon-bookmark.ribbon-top {
  padding-left: 0;
  padding-bottom: 0.25rem;
  padding-right: 0;
  border-radius: var(--tblr-ribbon-border-radius) 0 0 0;
}
.ribbon-bookmark.ribbon-top:after {
  top: 100%;
  left: 0;
  right: 0;
  border-color: inherit;
  border-width: 1rem;
  border-top-width: 0;
  border-bottom-color: transparent;
  border-bottom-width: 0.5rem;
}

/**
Markdown
 */
.markdown {
  line-height: 2;
}
.markdown > :first-child {
  margin-top: 0;
}
.markdown > :last-child,
.markdown > :last-child .highlight {
  margin-bottom: 0;
}
@media (min-width: 768px) {
  .markdown > hr, .markdown > .hr {
    margin-top: 3em;
    margin-bottom: 3em;
  }
}
.markdown > h1, .markdown > .h1, .markdown > h2, .markdown > .h2, .markdown > h3, .markdown > .h3, .markdown > h4, .markdown > .h4, .markdown > h5, .markdown > .h5, .markdown > h6, .markdown > .h6 {
  font-weight: var(--tblr-font-weight-bold);
}
.markdown > h2, .markdown > .h2, .markdown > h3, .markdown > .h3, .markdown > h4, .markdown > .h4, .markdown > h5, .markdown > .h5, .markdown > h6, .markdown > .h6 {
  margin-top: 2.5rem;
}
.markdown > table {
  font-size: var(--tblr-body-font-size);
}
.markdown > blockquote {
  font-size: 1rem;
  margin: 1.5rem 0;
  padding: 0.5rem 1.5rem;
}
.markdown > img,
.markdown > p > img {
  border-radius: var(--tblr-border-radius);
  border: 1px solid var(--tblr-border-color);
}
.markdown pre {
  max-height: 20rem;
}

.placeholder:not(.btn):not([class*=bg-]) {
  background-color: currentColor !important;
}
.placeholder:not(.avatar):not([class*=card-img-]) {
  border-radius: var(--tblr-border-radius);
}

.nav-segmented {
  --tblr-nav-bg: var(--tblr-bg-surface-tertiary);
  --tblr-nav-padding: 2px;
  --tblr-nav-height: 2.5rem;
  --tblr-nav-gap: .25rem;
  --tblr-nav-active-bg: var(--tblr-bg-surface);
  --tblr-nav-font-size: inherit;
  --tblr-nav-radius: 6px;
  --tblr-nav-link-disabled-color: var(--tblr-disabled-color);
  --tblr-nav-link-gap: .25rem;
  --tblr-nav-link-padding-x: .75rem;
  --tblr-nav-link-icon-size: 1.25rem;
  display: inline-flex;
  flex-wrap: wrap;
  gap: var(--tblr-nav-gap);
  padding: var(--tblr-nav-padding);
  list-style: none;
  background: var(--tblr-nav-bg);
  border-radius: calc(var(--tblr-nav-radius) + var(--tblr-nav-padding));
  box-shadow: inset 0 0 0 1px rgba(0, 0, 0, 0.04);
}
.nav-segmented .nav-link {
  display: inline-flex;
  gap: calc(0.25rem + var(--tblr-nav-link-gap));
  align-items: center;
  margin: 0;
  font-size: var(--tblr-nav-font-size);
  min-width: calc(var(--tblr-nav-height) - 2 * var(--tblr-nav-padding));
  height: calc(var(--tblr-nav-height) - 2 * var(--tblr-nav-padding));
  padding: 0 calc(var(--tblr-nav-link-padding-x) - 2px);
  border: 1px solid transparent;
  background: transparent;
  color: var(--tblr-secondary);
  text-align: center;
  text-decoration: none;
  white-space: nowrap;
  cursor: pointer;
  transition: background-color 0.3s, color 0.3s;
  border-radius: var(--tblr-nav-radius);
  flex-grow: 1;
  justify-content: center;
}
.nav-segmented .nav-link:hover, .nav-segmented .nav-link.hover {
  background: rgba(0, 0, 0, 0.04);
  color: var(--tblr-body-color);
}
.nav-segmented .nav-link.disabled, .nav-segmented .nav-link:disabled {
  color: var(--tblr-nav-link-disabled-color);
  cursor: not-allowed;
}
.nav-segmented .nav-link-input:checked + .nav-link,
.nav-segmented .nav-link.active {
  color: var(--tblr-body-color);
  background: var(--tblr-nav-active-bg);
  border-color: var(--tblr-border-color);
}
.nav-segmented .nav-link-input {
  display: none;
}
.nav-segmented .nav-link-icon {
  width: var(--tblr-nav-link-icon-size);
  height: var(--tblr-nav-link-icon-size);
  margin: 0 -0.25rem;
  color: inherit;
}

.nav-segmented-vertical {
  flex-direction: column;
}
.nav-segmented-vertical .nav-link {
  justify-content: flex-start;
}

.nav-sm {
  --tblr-nav-height: 2rem;
  --tblr-nav-font-size: var(--tblr-font-size-h5);
  --tblr-nav-radius: 4px;
  --tblr-nav-link-padding-x: .5rem;
  --tblr-nav-link-gap: .25rem;
  --tblr-nav-link-icon-size: 1rem;
}

.nav-lg {
  --tblr-nav-height: 3rem;
  --tblr-nav-font-size: var(--tblr-font-size-h3);
  --tblr-nav-radius: 8px;
  --tblr-nav-link-padding-x: 1rem;
  --tblr-nav-link-gap: .5rem;
  --tblr-nav-link-icon-size: 1.5rem;
}

.steps {
  --tblr-steps-color: var(--tblr-primary);
  --tblr-steps-inactive-color: var(--tblr-border-color);
  --tblr-steps-dot-size: .5rem;
  --tblr-steps-border-width: 2px;
  display: flex;
  flex-wrap: nowrap;
  width: 100%;
  padding: 0;
  margin: 0;
  list-style: none;
}

.steps-blue {
  --tblr-steps-color: var(--tblr-blue);
}
.steps-blue-lt {
  --tblr-steps-color: var(--tblr-blue-lt);
}

.steps-azure {
  --tblr-steps-color: var(--tblr-azure);
}
.steps-azure-lt {
  --tblr-steps-color: var(--tblr-azure-lt);
}

.steps-indigo {
  --tblr-steps-color: var(--tblr-indigo);
}
.steps-indigo-lt {
  --tblr-steps-color: var(--tblr-indigo-lt);
}

.steps-purple {
  --tblr-steps-color: var(--tblr-purple);
}
.steps-purple-lt {
  --tblr-steps-color: var(--tblr-purple-lt);
}

.steps-pink {
  --tblr-steps-color: var(--tblr-pink);
}
.steps-pink-lt {
  --tblr-steps-color: var(--tblr-pink-lt);
}

.steps-red {
  --tblr-steps-color: var(--tblr-red);
}
.steps-red-lt {
  --tblr-steps-color: var(--tblr-red-lt);
}

.steps-orange {
  --tblr-steps-color: var(--tblr-orange);
}
.steps-orange-lt {
  --tblr-steps-color: var(--tblr-orange-lt);
}

.steps-yellow {
  --tblr-steps-color: var(--tblr-yellow);
}
.steps-yellow-lt {
  --tblr-steps-color: var(--tblr-yellow-lt);
}

.steps-lime {
  --tblr-steps-color: var(--tblr-lime);
}
.steps-lime-lt {
  --tblr-steps-color: var(--tblr-lime-lt);
}

.steps-green {
  --tblr-steps-color: var(--tblr-green);
}
.steps-green-lt {
  --tblr-steps-color: var(--tblr-green-lt);
}

.steps-teal {
  --tblr-steps-color: var(--tblr-teal);
}
.steps-teal-lt {
  --tblr-steps-color: var(--tblr-teal-lt);
}

.steps-cyan {
  --tblr-steps-color: var(--tblr-cyan);
}
.steps-cyan-lt {
  --tblr-steps-color: var(--tblr-cyan-lt);
}

.step-item {
  position: relative;
  flex: 1 1 0;
  min-height: 1rem;
  margin-top: 0;
  color: inherit;
  text-align: center;
  cursor: default;
  padding-top: calc(var(--tblr-steps-dot-size));
}
a.step-item {
  cursor: pointer;
}
a.step-item:hover {
  color: inherit;
}

.step-item:after, .step-item:before {
  background: var(--tblr-steps-color);
}
.step-item:not(:last-child):after {
  position: absolute;
  right: 50%;
  width: 100%;
  content: "";
  transform: translateY(-50%);
}
.step-item:after {
  top: calc(var(--tblr-steps-dot-size) * 0.5);
  height: var(--tblr-steps-border-width);
}
.step-item:before {
  content: "";
  position: absolute;
  top: 0;
  right: 50%;
  z-index: 1;
  box-sizing: content-box;
  display: flex;
  align-items: center;
  justify-content: center;
  border-radius: 100rem;
  transform: translateX(50%);
  color: var(--tblr-white);
  width: var(--tblr-steps-dot-size);
  height: var(--tblr-steps-dot-size);
}
.step-item.active {
  font-weight: var(--tblr-font-weight-bold);
}
.step-item.active:after {
  background: var(--tblr-steps-inactive-color);
}
.step-item.active ~ .step-item {
  color: var(--tblr-disabled-color);
}
.step-item.active ~ .step-item:after, .step-item.active ~ .step-item:before {
  background: var(--tblr-steps-inactive-color);
}

.steps-counter {
  --tblr-steps-dot-size: 1.5rem;
  counter-reset: steps;
}
.steps-counter .step-item {
  counter-increment: steps;
}
.steps-counter .step-item:before {
  content: counter(steps);
}

.steps-vertical {
  --tblr-steps-dot-offset: 6px;
  flex-direction: column;
}
.steps-vertical.steps-counter {
  --tblr-steps-dot-offset: -2px;
}
.steps-vertical .step-item {
  text-align: right;
  padding-top: 0;
  padding-right: calc(var(--tblr-steps-dot-size) + 1rem);
  min-height: auto;
}
.steps-vertical .step-item:not(:first-child) {
  margin-top: 1rem;
}
.steps-vertical .step-item:before {
  top: var(--tblr-steps-dot-offset);
  right: 0;
  transform: translate(0, 0);
}
.steps-vertical .step-item:not(:last-child):after {
  position: absolute;
  content: "";
  transform: translateX(50%);
  top: var(--tblr-steps-dot-offset);
  right: calc(var(--tblr-steps-dot-size) * 0.5);
  width: var(--tblr-steps-border-width);
  height: calc(100% + 1rem);
}

@keyframes status-pulsate-main {
  40% {
    transform: scale(1.25, 1.25);
  }
  60% {
    transform: scale(1.25, 1.25);
  }
}
@keyframes status-pulsate-secondary {
  10% {
    transform: scale(1, 1);
  }
  30% {
    transform: scale(3, 3);
  }
  80% {
    transform: scale(3, 3);
  }
  100% {
    transform: scale(1, 1);
  }
}
@keyframes status-pulsate-tertiary {
  25% {
    transform: scale(1, 1);
  }
  80% {
    transform: scale(3, 3);
    opacity: 0;
  }
  100% {
    transform: scale(3, 3);
    opacity: 0;
  }
}
.status {
  --tblr-status-height: 1.5rem;
  --tblr-status-color: #6b7280;
  --tblr-status-color-rgb: 107, 114, 128;
  display: inline-flex;
  align-items: center;
  height: var(--tblr-status-height);
  padding: 0.25rem 0.75rem;
  gap: 0.5rem;
  color: var(--tblr-status-color);
  background: rgba(var(--tblr-status-color-rgb), 0.1);
  font-size: 0.875rem;
  text-transform: none;
  letter-spacing: normal;
  border-radius: 100rem;
  font-weight: var(--tblr-font-weight-medium);
  line-height: 1;
  margin: 0;
}
.status .status-dot {
  background: var(--tblr-status-color);
}
.status .icon {
  font-size: 1.25rem;
}

.status-lite {
  border: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color) !important;
  background: transparent !important;
  color: var(--tblr-body-color) !important;
}

.status-primary {
  --tblr-status-color: #066fd1;
  --tblr-status-color-rgb: 6, 111, 209;
}

.status-secondary {
  --tblr-status-color: #6b7280;
  --tblr-status-color-rgb: 107, 114, 128;
}

.status-success {
  --tblr-status-color: #2fb344;
  --tblr-status-color-rgb: 47, 179, 68;
}

.status-info {
  --tblr-status-color: #4299e1;
  --tblr-status-color-rgb: 66, 153, 225;
}

.status-warning {
  --tblr-status-color: #f59f00;
  --tblr-status-color-rgb: 245, 159, 0;
}

.status-danger {
  --tblr-status-color: #d63939;
  --tblr-status-color-rgb: 214, 57, 57;
}

.status-light {
  --tblr-status-color: #f9fafb;
  --tblr-status-color-rgb: 249, 250, 251;
}

.status-dark {
  --tblr-status-color: #1f2937;
  --tblr-status-color-rgb: 31, 41, 55;
}

.status-muted {
  --tblr-status-color: #6b7280;
  --tblr-status-color-rgb: 107, 114, 128;
}

.status-blue {
  --tblr-status-color: #066fd1;
  --tblr-status-color-rgb: 6, 111, 209;
}

.status-azure {
  --tblr-status-color: #4299e1;
  --tblr-status-color-rgb: 66, 153, 225;
}

.status-indigo {
  --tblr-status-color: #4263eb;
  --tblr-status-color-rgb: 66, 99, 235;
}

.status-purple {
  --tblr-status-color: #ae3ec9;
  --tblr-status-color-rgb: 174, 62, 201;
}

.status-pink {
  --tblr-status-color: #d6336c;
  --tblr-status-color-rgb: 214, 51, 108;
}

.status-red {
  --tblr-status-color: #d63939;
  --tblr-status-color-rgb: 214, 57, 57;
}

.status-orange {
  --tblr-status-color: #f76707;
  --tblr-status-color-rgb: 247, 103, 7;
}

.status-yellow {
  --tblr-status-color: #f59f00;
  --tblr-status-color-rgb: 245, 159, 0;
}

.status-lime {
  --tblr-status-color: #74b816;
  --tblr-status-color-rgb: 116, 184, 22;
}

.status-green {
  --tblr-status-color: #2fb344;
  --tblr-status-color-rgb: 47, 179, 68;
}

.status-teal {
  --tblr-status-color: #0ca678;
  --tblr-status-color-rgb: 12, 166, 120;
}

.status-cyan {
  --tblr-status-color: #17a2b8;
  --tblr-status-color-rgb: 23, 162, 184;
}

.status-dot {
  --tblr-status-dot-color: var(--tblr-status-color, #6b7280);
  --tblr-status-size: 0.5rem;
  position: relative;
  display: inline-block;
  width: var(--tblr-status-size);
  height: var(--tblr-status-size);
  background: var(--tblr-status-dot-color);
  border-radius: 100rem;
}

.status-dot-animated:before {
  content: "";
  position: absolute;
  inset: 0;
  z-index: 0;
  background: inherit;
  border-radius: inherit;
  opacity: 0.6;
  animation: 1s linear 2s backwards infinite status-pulsate-tertiary;
}

.status-indicator {
  --tblr-status-indicator-size: 2.5rem;
  --tblr-status-indicator-color: var(--tblr-status-color, #6b7280);
  display: block;
  position: relative;
  width: var(--tblr-status-indicator-size);
  height: var(--tblr-status-indicator-size);
}

.status-indicator-circle {
  --tblr-status-circle-size: .75rem;
  position: absolute;
  right: 50%;
  top: 50%;
  margin: calc(var(--tblr-status-circle-size) / -2) calc(var(--tblr-status-circle-size) / -2) 0 0;
  width: var(--tblr-status-circle-size);
  height: var(--tblr-status-circle-size);
  border-radius: 100rem;
  background: var(--tblr-status-color);
}
.status-indicator-circle:nth-child(1) {
  z-index: 3;
}
.status-indicator-circle:nth-child(2) {
  z-index: 2;
  opacity: 0.1;
}
.status-indicator-circle:nth-child(3) {
  z-index: 1;
  opacity: 0.3;
}

.status-indicator-animated .status-indicator-circle:nth-child(1) {
  animation: 2s linear 1s infinite backwards status-pulsate-main;
}
.status-indicator-animated .status-indicator-circle:nth-child(2) {
  animation: 2s linear 1s infinite backwards status-pulsate-secondary;
}
.status-indicator-animated .status-indicator-circle:nth-child(3) {
  animation: 2s linear 1s infinite backwards status-pulsate-tertiary;
}

.switch-icon {
  display: inline-block;
  line-height: 1;
  border: 0;
  padding: 0;
  background: transparent;
  width: 1.25rem;
  height: 1.25rem;
  vertical-align: bottom;
  position: relative;
  cursor: pointer;
}
.switch-icon.disabled {
  pointer-events: none;
  opacity: 0.4;
}
.switch-icon:focus {
  outline: none;
}
.switch-icon svg {
  display: block;
  width: 100%;
  height: 100%;
}
.switch-icon .switch-icon-a,
.switch-icon .switch-icon-b {
  display: block;
  width: 100%;
  height: 100%;
}
.switch-icon .switch-icon-a {
  opacity: 1;
}
.switch-icon .switch-icon-b {
  position: absolute;
  top: 0;
  right: 0;
  opacity: 0;
}
.switch-icon.active .switch-icon-a {
  opacity: 0;
}
.switch-icon.active .switch-icon-b {
  opacity: 1;
}

.switch-icon-fade .switch-icon-a,
.switch-icon-fade .switch-icon-b {
  transition: opacity 0.5s;
}
@media (prefers-reduced-motion: reduce) {
  .switch-icon-fade .switch-icon-a,
  .switch-icon-fade .switch-icon-b {
    transition: none;
  }
}

.switch-icon-scale .switch-icon-a,
.switch-icon-scale .switch-icon-b {
  transition: opacity 0.5s, transform 0s 0.5s;
}
@media (prefers-reduced-motion: reduce) {
  .switch-icon-scale .switch-icon-a,
  .switch-icon-scale .switch-icon-b {
    transition: none;
  }
}
.switch-icon-scale .switch-icon-b {
  transform: scale(1.5);
}
.switch-icon-scale.active .switch-icon-a,
.switch-icon-scale.active .switch-icon-b {
  transition: opacity 0s, transform 0.5s;
}
@media (prefers-reduced-motion: reduce) {
  .switch-icon-scale.active .switch-icon-a,
  .switch-icon-scale.active .switch-icon-b {
    transition: none;
  }
}
.switch-icon-scale.active .switch-icon-b {
  transform: scale(1);
}

.switch-icon-flip {
  perspective: 10em;
}
.switch-icon-flip .switch-icon-a,
.switch-icon-flip .switch-icon-b {
  backface-visibility: hidden;
  transform-style: preserve-3d;
  transition: opacity 0s 0.2s, transform 0.4s ease-in-out;
}
@media (prefers-reduced-motion: reduce) {
  .switch-icon-flip .switch-icon-a,
  .switch-icon-flip .switch-icon-b {
    transition: none;
  }
}
.switch-icon-flip .switch-icon-a {
  opacity: 1;
  transform: rotateY(0deg);
}
.switch-icon-flip .switch-icon-b {
  opacity: 1;
  transform: rotateY(180deg);
}
.switch-icon-flip.active .switch-icon-a {
  opacity: 1;
  transform: rotateY(-180deg);
}
.switch-icon-flip.active .switch-icon-b {
  opacity: 1;
  transform: rotateY(0deg);
}

.switch-icon-slide-up,
.switch-icon-slide-left,
.switch-icon-slide-start,
.switch-icon-slide-right,
.switch-icon-slide-end,
.switch-icon-slide-down {
  overflow: hidden;
}
.switch-icon-slide-up .switch-icon-a,
.switch-icon-slide-up .switch-icon-b,
.switch-icon-slide-left .switch-icon-a,
.switch-icon-slide-left .switch-icon-b,
.switch-icon-slide-start .switch-icon-a,
.switch-icon-slide-start .switch-icon-b,
.switch-icon-slide-right .switch-icon-a,
.switch-icon-slide-right .switch-icon-b,
.switch-icon-slide-end .switch-icon-a,
.switch-icon-slide-end .switch-icon-b,
.switch-icon-slide-down .switch-icon-a,
.switch-icon-slide-down .switch-icon-b {
  transition: opacity 0.3s, transform 0.3s;
}
@media (prefers-reduced-motion: reduce) {
  .switch-icon-slide-up .switch-icon-a,
  .switch-icon-slide-up .switch-icon-b,
  .switch-icon-slide-left .switch-icon-a,
  .switch-icon-slide-left .switch-icon-b,
  .switch-icon-slide-start .switch-icon-a,
  .switch-icon-slide-start .switch-icon-b,
  .switch-icon-slide-right .switch-icon-a,
  .switch-icon-slide-right .switch-icon-b,
  .switch-icon-slide-end .switch-icon-a,
  .switch-icon-slide-end .switch-icon-b,
  .switch-icon-slide-down .switch-icon-a,
  .switch-icon-slide-down .switch-icon-b {
    transition: none;
  }
}
.switch-icon-slide-up .switch-icon-a,
.switch-icon-slide-left .switch-icon-a,
.switch-icon-slide-start .switch-icon-a,
.switch-icon-slide-right .switch-icon-a,
.switch-icon-slide-end .switch-icon-a,
.switch-icon-slide-down .switch-icon-a {
  transform: translateY(0);
}
.switch-icon-slide-up .switch-icon-b,
.switch-icon-slide-left .switch-icon-b,
.switch-icon-slide-start .switch-icon-b,
.switch-icon-slide-right .switch-icon-b,
.switch-icon-slide-end .switch-icon-b,
.switch-icon-slide-down .switch-icon-b {
  transform: translateY(100%);
}
.switch-icon-slide-up.active .switch-icon-a,
.switch-icon-slide-left.active .switch-icon-a,
.switch-icon-slide-start.active .switch-icon-a,
.switch-icon-slide-right.active .switch-icon-a,
.switch-icon-slide-end.active .switch-icon-a,
.switch-icon-slide-down.active .switch-icon-a {
  transform: translateY(-100%);
}
.switch-icon-slide-up.active .switch-icon-b,
.switch-icon-slide-left.active .switch-icon-b,
.switch-icon-slide-start.active .switch-icon-b,
.switch-icon-slide-right.active .switch-icon-b,
.switch-icon-slide-end.active .switch-icon-b,
.switch-icon-slide-down.active .switch-icon-b {
  transform: translateY(0);
}

.switch-icon-slide-left .switch-icon-a,
.switch-icon-slide-start .switch-icon-a {
  transform: translateX(0);
}
.switch-icon-slide-left .switch-icon-b,
.switch-icon-slide-start .switch-icon-b {
  transform: translateX(-100%);
}
.switch-icon-slide-left.active .switch-icon-a,
.switch-icon-slide-start.active .switch-icon-a {
  transform: translateX(100%);
}
.switch-icon-slide-left.active .switch-icon-b,
.switch-icon-slide-start.active .switch-icon-b {
  transform: translateX(0);
}

.switch-icon-slide-right .switch-icon-a,
.switch-icon-slide-end .switch-icon-a {
  transform: translateX(0);
}
.switch-icon-slide-right .switch-icon-b,
.switch-icon-slide-end .switch-icon-b {
  transform: translateX(100%);
}
.switch-icon-slide-right.active .switch-icon-a,
.switch-icon-slide-end.active .switch-icon-a {
  transform: translateX(-100%);
}
.switch-icon-slide-right.active .switch-icon-b,
.switch-icon-slide-end.active .switch-icon-b {
  transform: translateX(0);
}

.switch-icon-slide-down .switch-icon-a {
  transform: translateY(0);
}
.switch-icon-slide-down .switch-icon-b {
  transform: translateY(-100%);
}
.switch-icon-slide-down.active .switch-icon-a {
  transform: translateY(100%);
}
.switch-icon-slide-down.active .switch-icon-b {
  transform: translateY(0);
}

.table thead th, .markdown > table thead th {
  background: var(--tblr-bg-surface-tertiary);
  font-size: 0.75rem;
  font-weight: var(--tblr-font-weight-medium);
  text-transform: uppercase;
  letter-spacing: 0.04em;
  line-height: 1rem;
  color: var(--tblr-secondary);
  padding-top: 0.5rem;
  padding-bottom: 0.5rem;
  white-space: nowrap;
}
@media print {
  .table thead th, .markdown > table thead th {
    background: transparent;
  }
}

.table-responsive .table, .table-responsive .markdown > table {
  margin-bottom: 0;
}
.table-responsive + .card-footer {
  border-top: 0;
}

.table-transparent thead th {
  background: transparent;
}

.table-nowrap > :not(caption) > * > * {
  white-space: nowrap;
}

.table-vcenter > :not(caption) > * > * {
  vertical-align: middle;
}

.table-center > :not(caption) > * > * {
  text-align: center;
}

.td-truncate {
  max-width: 1px;
  width: 100%;
}

.table-mobile {
  display: block;
}
.table-mobile thead {
  display: none;
}
.table-mobile tbody,
.table-mobile tr {
  display: flex;
  flex-direction: column;
}
.table-mobile td {
  display: block;
  padding: 0.75rem 0.75rem !important;
  border: none;
  color: var(--tblr-body-color) !important;
}
.table-mobile td[data-label]:before {
  font-size: 0.75rem;
  font-weight: var(--tblr-font-weight-medium);
  text-transform: uppercase;
  letter-spacing: 0.04em;
  line-height: 1rem;
  color: var(--tblr-secondary);
  content: attr(data-label);
  display: block;
}
.table-mobile tr {
  border-bottom: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent);
}
.table-mobile .btn {
  display: block;
}
@media (max-width: 575.98px) {
  .table-mobile-sm {
    display: block;
  }
  .table-mobile-sm thead {
    display: none;
  }
  .table-mobile-sm tbody,
  .table-mobile-sm tr {
    display: flex;
    flex-direction: column;
  }
  .table-mobile-sm td {
    display: block;
    padding: 0.75rem 0.75rem !important;
    border: none;
    color: var(--tblr-body-color) !important;
  }
  .table-mobile-sm td[data-label]:before {
    font-size: 0.75rem;
    font-weight: var(--tblr-font-weight-medium);
    text-transform: uppercase;
    letter-spacing: 0.04em;
    line-height: 1rem;
    color: var(--tblr-secondary);
    content: attr(data-label);
    display: block;
  }
  .table-mobile-sm tr {
    border-bottom: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent);
  }
  .table-mobile-sm .btn {
    display: block;
  }
}
@media (max-width: 767.98px) {
  .table-mobile-md {
    display: block;
  }
  .table-mobile-md thead {
    display: none;
  }
  .table-mobile-md tbody,
  .table-mobile-md tr {
    display: flex;
    flex-direction: column;
  }
  .table-mobile-md td {
    display: block;
    padding: 0.75rem 0.75rem !important;
    border: none;
    color: var(--tblr-body-color) !important;
  }
  .table-mobile-md td[data-label]:before {
    font-size: 0.75rem;
    font-weight: var(--tblr-font-weight-medium);
    text-transform: uppercase;
    letter-spacing: 0.04em;
    line-height: 1rem;
    color: var(--tblr-secondary);
    content: attr(data-label);
    display: block;
  }
  .table-mobile-md tr {
    border-bottom: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent);
  }
  .table-mobile-md .btn {
    display: block;
  }
}
@media (max-width: 991.98px) {
  .table-mobile-lg {
    display: block;
  }
  .table-mobile-lg thead {
    display: none;
  }
  .table-mobile-lg tbody,
  .table-mobile-lg tr {
    display: flex;
    flex-direction: column;
  }
  .table-mobile-lg td {
    display: block;
    padding: 0.75rem 0.75rem !important;
    border: none;
    color: var(--tblr-body-color) !important;
  }
  .table-mobile-lg td[data-label]:before {
    font-size: 0.75rem;
    font-weight: var(--tblr-font-weight-medium);
    text-transform: uppercase;
    letter-spacing: 0.04em;
    line-height: 1rem;
    color: var(--tblr-secondary);
    content: attr(data-label);
    display: block;
  }
  .table-mobile-lg tr {
    border-bottom: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent);
  }
  .table-mobile-lg .btn {
    display: block;
  }
}
@media (max-width: 1199.98px) {
  .table-mobile-xl {
    display: block;
  }
  .table-mobile-xl thead {
    display: none;
  }
  .table-mobile-xl tbody,
  .table-mobile-xl tr {
    display: flex;
    flex-direction: column;
  }
  .table-mobile-xl td {
    display: block;
    padding: 0.75rem 0.75rem !important;
    border: none;
    color: var(--tblr-body-color) !important;
  }
  .table-mobile-xl td[data-label]:before {
    font-size: 0.75rem;
    font-weight: var(--tblr-font-weight-medium);
    text-transform: uppercase;
    letter-spacing: 0.04em;
    line-height: 1rem;
    color: var(--tblr-secondary);
    content: attr(data-label);
    display: block;
  }
  .table-mobile-xl tr {
    border-bottom: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent);
  }
  .table-mobile-xl .btn {
    display: block;
  }
}
@media (max-width: 1399.98px) {
  .table-mobile-xxl {
    display: block;
  }
  .table-mobile-xxl thead {
    display: none;
  }
  .table-mobile-xxl tbody,
  .table-mobile-xxl tr {
    display: flex;
    flex-direction: column;
  }
  .table-mobile-xxl td {
    display: block;
    padding: 0.75rem 0.75rem !important;
    border: none;
    color: var(--tblr-body-color) !important;
  }
  .table-mobile-xxl td[data-label]:before {
    font-size: 0.75rem;
    font-weight: var(--tblr-font-weight-medium);
    text-transform: uppercase;
    letter-spacing: 0.04em;
    line-height: 1rem;
    color: var(--tblr-secondary);
    content: attr(data-label);
    display: block;
  }
  .table-mobile-xxl tr {
    border-bottom: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent);
  }
  .table-mobile-xxl .btn {
    display: block;
  }
}

/**
Table sort
 */
.table-sort {
  font: inherit;
  color: inherit;
  text-transform: inherit;
  letter-spacing: inherit;
  border: 0;
  background: inherit;
  display: block;
  width: 100%;
  text-align: inherit;
  transition: color 0.3s;
  margin: -0.5rem -0.75rem;
  padding: 0.5rem 0.75rem;
}
@media (prefers-reduced-motion: reduce) {
  .table-sort {
    transition: none;
  }
}
.table-sort:hover, .table-sort.asc, .table-sort.desc {
  color: var(--tblr-body-color);
}
.table-sort:after {
  content: "";
  display: inline-flex;
  width: 1rem;
  height: 1rem;
  vertical-align: bottom;
  mask-image: url("data:image/svg+xml,<svg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%270 0 16 16%27 width=%2716%27 height=%2716%27 fill=%27none%27 stroke=%27currentColor%27 stroke-linecap=%27round%27 stroke-linejoin=%27round%27 stroke-width=%271%27><path d=%27M5 7l3 -3l3 3%27/><path d=%27M5 10l3 3l3 -3%27/></svg>");
  background: currentColor;
  margin-right: 0.25rem;
}
.table-sort.asc:after {
  mask-image: url("data:image/svg+xml,<svg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%270 0 16 16%27 width=%2716%27 height=%2716%27><path fill=%27none%27 stroke=%27currentColor%27 stroke-linecap=%27round%27 stroke-linejoin=%27round%27 stroke-width=%271%27 d=%27M5 10l3 -3l3 3%27/></svg>");
}
.table-sort.desc:after {
  mask-image: url("data:image/svg+xml,<svg xmlns=%27http://www.w3.org/2000/svg%27 viewBox=%270 0 16 16%27 width=%2716%27 height=%2716%27><path fill=%27none%27 stroke=%27currentColor%27 stroke-linecap=%27round%27 stroke-linejoin=%27round%27 stroke-width=%271%27 d=%27M5 7l3 3l3 -3%27/></svg>");
}

.table-borderless thead th {
  background: transparent;
}

.table-selectable tbody tr .on-checked {
  display: none;
}
.table-selectable tbody tr .on-unchecked {
  display: initial;
}
.table-selectable tbody tr:has(.table-selectable-check:checked) {
  background-color: var(--tblr-active-bg);
}
.table-selectable tbody tr:has(.table-selectable-check:checked) .on-checked {
  display: initial;
}
.table-selectable tbody tr:has(.table-selectable-check:checked) .on-unchecked {
  display: none;
}

.tag {
  --tblr-tag-height: 1.5rem;
  border: 1px solid var(--tblr-border-color);
  display: inline-flex;
  align-items: center;
  height: var(--tblr-tag-height);
  border-radius: var(--tblr-border-radius);
  padding: 0 0.5rem;
  background: var(--tblr-bg-surface);
  box-shadow: var(--tblr-shadow-input);
  gap: 0.5rem;
}
.tag .btn-close {
  margin-left: -0.25rem;
  margin-right: -0.125rem;
  padding: 0;
  width: 1rem;
  height: 1rem;
  font-size: 0.5rem;
}

.tag-badge {
  --tblr-badge-font-size: 0.625rem;
  --tblr-badge-padding-x: .25rem;
  --tblr-badge-padding-y: .125rem;
  margin-left: -0.25rem;
}

.tag-avatar,
.tag-flag,
.tag-payment,
.tag-icon,
.tag-check {
  margin-right: -0.25rem;
}

.tag-icon {
  color: var(--tblr-secondary);
  margin-left: -0.125rem;
  width: 1rem;
  height: 1rem;
}

.tag-check {
  width: 1rem;
  height: 1rem;
  background-size: 1rem;
}

.tags-list {
  --tblr-list-gap: 0.5rem;
  display: flex;
  flex-wrap: wrap;
  gap: var(--tblr-list-gap);
}

.toast {
  border: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color-translucent);
  box-shadow: rgba(31, 41, 55, 0.04) 0 2px 4px 0;
}
.toast .toast-header {
  user-select: none;
}
.toast button[data-bs-dismiss=toast] {
  outline: none;
}

.toast-primary {
  --tblr-toast-color: #066fd1;
}

.toast-secondary {
  --tblr-toast-color: #6b7280;
}

.toast-success {
  --tblr-toast-color: #2fb344;
}

.toast-info {
  --tblr-toast-color: #4299e1;
}

.toast-warning {
  --tblr-toast-color: #f59f00;
}

.toast-danger {
  --tblr-toast-color: #d63939;
}

.toast-light {
  --tblr-toast-color: #f9fafb;
}

.toast-dark {
  --tblr-toast-color: #1f2937;
}

.toast-muted {
  --tblr-toast-color: #6b7280;
}

.toast-blue {
  --tblr-toast-color: #066fd1;
}

.toast-azure {
  --tblr-toast-color: #4299e1;
}

.toast-indigo {
  --tblr-toast-color: #4263eb;
}

.toast-purple {
  --tblr-toast-color: #ae3ec9;
}

.toast-pink {
  --tblr-toast-color: #d6336c;
}

.toast-red {
  --tblr-toast-color: #d63939;
}

.toast-orange {
  --tblr-toast-color: #f76707;
}

.toast-yellow {
  --tblr-toast-color: #f59f00;
}

.toast-lime {
  --tblr-toast-color: #74b816;
}

.toast-green {
  --tblr-toast-color: #2fb344;
}

.toast-teal {
  --tblr-toast-color: #0ca678;
}

.toast-cyan {
  --tblr-toast-color: #17a2b8;
}

.toolbar {
  display: flex;
  flex-wrap: nowrap;
  flex-shrink: 0;
  margin: 0 -0.5rem;
}
.toolbar > * {
  margin: 0 0.5rem;
}

.tracking {
  --tblr-tracking-height: 1.5rem;
  --tblr-tracking-gap-width: 0.125rem;
  --tblr-tracking-block-border-radius: var(--tblr-border-radius);
  display: flex;
  gap: var(--tblr-tracking-gap-width);
}

.tracking-squares {
  --tblr-tracking-block-border-radius: var(--tblr-border-radius-sm);
}
.tracking-squares .tracking-block {
  height: auto;
}
.tracking-squares .tracking-block:before {
  content: "";
  display: block;
  padding-top: 100%;
}

.tracking-block {
  flex: 1;
  border-radius: var(--tblr-tracking-block-border-radius);
  height: var(--tblr-tracking-height);
  min-width: 0.25rem;
  background: var(--tblr-border-color);
}

.timeline {
  --tblr-timeline-icon-size: 2.5rem;
  position: relative;
  list-style: none;
  padding: 0;
}

.timeline-event {
  position: relative;
}
.timeline-event:not(:last-child) {
  margin-bottom: var(--tblr-page-padding);
}
.timeline-event:not(:last-child):before {
  content: "";
  position: absolute;
  top: var(--tblr-timeline-icon-size);
  right: calc(var(--tblr-timeline-icon-size) / 2);
  bottom: calc(-1 * var(--tblr-page-padding));
  width: var(--tblr-border-width);
  background-color: var(--tblr-border-color);
  border-radius: var(--tblr-border-radius);
}

.timeline-event-icon {
  position: absolute;
  display: flex;
  align-items: center;
  justify-content: center;
  width: var(--tblr-timeline-icon-size, 2.5rem);
  height: var(--tblr-timeline-icon-size, 2.5rem);
  background: var(--tblr-bg-surface-secondary);
  color: var(--tblr-secondary);
  border-radius: var(--tblr-border-radius);
  z-index: 5;
}

.timeline-event-card {
  margin-right: calc(var(--tblr-timeline-icon-size, 2.5rem) + var(--tblr-page-padding));
}

.timeline-simple .timeline-event-icon {
  display: none;
}
.timeline-simple .timeline-event-card {
  margin-right: 0;
}

/**
Horizontal rules
 */
/**
Hr text
 */
.hr-text {
  display: flex;
  align-items: center;
  margin: 2rem 0;
  font-size: 0.75rem;
  font-weight: var(--tblr-font-weight-medium);
  text-transform: uppercase;
  letter-spacing: 0.04em;
  line-height: 1rem;
  color: var(--tblr-secondary);
  height: 1px;
}
.hr-text:after, .hr-text:before {
  flex: 1 1 auto;
  height: 1px;
  background-color: var(--tblr-border-color);
}
.hr-text:before {
  content: "";
  margin-left: 0.5rem;
}
.hr-text:after {
  content: "";
  margin-right: 0.5rem;
}
.hr-text > *:first-child {
  padding-left: 0.5rem;
  padding-right: 0;
  color: var(--tblr-secondary);
}
.hr-text.hr-text-left:before, .hr-text.hr-text-start:before {
  content: none;
}
.hr-text.hr-text-left > *:first-child, .hr-text.hr-text-start > *:first-child {
  padding-left: 0.5rem;
  padding-right: 0.5rem;
}
.hr-text.hr-text-right:before, .hr-text.hr-text-end:before {
  content: "";
}
.hr-text.hr-text-right:after, .hr-text.hr-text-end:after {
  content: none;
}
.hr-text.hr-text-right > *:first-child, .hr-text.hr-text-end > *:first-child {
  padding-left: 0;
  padding-right: 0.5rem;
}
.card > .hr-text {
  margin: 0;
}

.hr-text-spaceless {
  margin: -0.5rem 0;
}

.lead {
  color: var(--tblr-secondary);
  font-size: inherit;
}

a {
  text-decoration-skip-ink: auto;
  color: color-mix(in srgb, transparent, var(--tblr-link-color) var(--tblr-link-opacity, 100%));
}
a:hover {
  color: color-mix(in srgb, transparent, var(--tblr-link-hover-color) var(--tblr-link-opacity, 100%));
}

h1 a,
h2 a,
h3 a,
h4 a,
h5 a,
h6 a,
.h1 a,
.h2 a,
.h3 a,
.h4 a,
.h5 a,
.h6 a {
  color: inherit;
}
h1 a:hover,
h2 a:hover,
h3 a:hover,
h4 a:hover,
h5 a:hover,
h6 a:hover,
.h1 a:hover,
.h2 a:hover,
.h3 a:hover,
.h4 a:hover,
.h5 a:hover,
.h6 a:hover {
  color: inherit;
}

h1,
.h1 {
  font-size: var(--tblr-font-size-h1);
  line-height: var(--tblr-line-height-h1);
}

h2,
.h2 {
  font-size: var(--tblr-font-size-h2);
  line-height: var(--tblr-line-height-h2);
}

h3,
.h3 {
  font-size: var(--tblr-font-size-h3);
  line-height: var(--tblr-line-height-h3);
}

h4,
.h4 {
  font-size: var(--tblr-font-size-h4);
  line-height: var(--tblr-line-height-h4);
}

h5,
.h5 {
  font-size: var(--tblr-font-size-h5);
  line-height: var(--tblr-line-height-h5);
}

h6,
.h6 {
  font-size: var(--tblr-font-size-h6);
  line-height: var(--tblr-line-height-h6);
}

.fs-base {
  font-size: var(--tblr-body-font-size);
}

strong,
.strong,
b {
  font-weight: var(--tblr-font-weight-bold);
}

blockquote {
  padding: 1rem 1rem 1rem;
  border-right: 2px var(--tblr-border-style) var(--tblr-border-color);
}
blockquote p {
  margin-bottom: 1rem;
}
blockquote cite {
  display: block;
  text-align: left;
}
blockquote cite:before {
  content: "— ";
}

ul,
ol {
  padding-right: 1.5rem;
}

hr, .hr {
  margin: 2rem 0;
}

dl dd:last-child {
  margin-bottom: 0;
}

pre {
  --tblr-scrollbar-color: var(--tblr-light);
  padding: 1rem;
  background: var(--tblr-bg-surface-dark);
  color: var(--tblr-light);
  border-radius: var(--tblr-border-radius);
  line-height: 1.4285714286;
}
pre {
  scrollbar-color: color-mix(in srgb, var(--tblr-scrollbar-color, var(--tblr-body-color)) 20%, transparent) transparent;
}
pre::-webkit-scrollbar {
  width: 1rem;
  height: 1rem;
  transition: background 0.3s;
}
@media (prefers-reduced-motion: reduce) {
  pre::-webkit-scrollbar {
    transition: none;
  }
}
pre::-webkit-scrollbar-thumb {
  border-radius: 1rem;
  border: 5px solid transparent;
  box-shadow: inset 0 0 0 1rem color-mix(in srgb, var(--tblr-scrollbar-color, var(--tblr-body-color)) 20%, transparent);
}
pre::-webkit-scrollbar-track {
  background: transparent;
}
pre:hover::-webkit-scrollbar-thumb {
  box-shadow: inset 0 0 0 1rem color-mix(in srgb, var(--tblr-scrollbar-color, var(--tblr-body-color)) 40%, transparent);
}
pre::-webkit-scrollbar-corner {
  background: transparent;
}
pre code {
  background: transparent;
  padding: 0;
}

code {
  background: var(--tblr-code-bg);
  padding: 2px 4px;
  border-radius: var(--tblr-border-radius);
}

abbr {
  text-decoration: underline dotted;
  cursor: help;
  text-decoration-skip-ink: none;
}

kbd,
.kbd {
  border: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
  display: inline-block;
  box-sizing: border-box;
  max-width: 100%;
  font-size: var(--tblr-font-size-h5);
  font-weight: var(--tblr-font-weight-medium);
  line-height: 1;
  vertical-align: baseline;
  border-radius: var(--tblr-border-radius);
}

img {
  max-width: 100%;
  height: auto;
}

.list-unstyled {
  margin-right: 0;
}

/**
Selection
 */
::selection,
.text-selected {
  background-color: color-mix(in srgb, var(--tblr-primary) 10%, transparent);
}

.text-selected {
  display: inline-block;
}

/**
Links
 */
[class^=link-].disabled,
[class*=" link-"].disabled {
  color: var(--tblr-disabled-color) !important;
  pointer-events: none;
}

a:hover:has(.icon) {
  text-decoration: none;
}

.link-hoverable {
  border-radius: var(--tblr-border-radius);
  transition: background-color 0.15s ease-in-out;
}
.link-hoverable:hover {
  text-decoration: none;
  color: var(--tblr-primary);
  background: color-mix(in srgb, var(--tblr-secondary) 4%, transparent);
}

/**
Subheader
 */
.subheader {
  font-size: 0.75rem;
  font-weight: var(--tblr-font-weight-medium);
  text-transform: uppercase;
  letter-spacing: 0.04em;
  line-height: 1rem;
  color: var(--tblr-secondary);
}

/**
Mentions
 */
.mention {
  display: inline-block;
  box-shadow: var(--tblr-shadow-border);
  border-radius: var(--tblr-border-radius-pill);
  line-height: 1.3333333333em;
  font-size: 0.8571428571em;
  color: var(--tblr-body-color);
  background: var(--tblr-bg-surface-tertiary);
  padding: 0.1666666667em 0.6666666667em;
  font-weight: var(--tblr-font-weight-medium);
}
a.mention {
  cursor: pointer;
}
a.mention:hover, a.mention.hover {
  background: var(--tblr-bg-surface-secondary);
  text-decoration: underline;
}

.mention-avatar,
.mention-app,
.mention-color {
  width: 1.1666666667em;
  height: 1.1666666667em;
  border-radius: var(--tblr-border-radius-pill);
  margin: -0.1666666667em -0.3333333333em 0 0.3333333333em;
  display: inline-flex;
  background: no-repeat center center/cover;
  box-shadow: var(--tblr-shadow-border);
  vertical-align: middle;
  text-align: center;
}

.mention-app {
  box-shadow: none;
  background: none;
  border-radius: 0;
}

.mention-count {
  color: var(--tblr-secondary);
  margin-right: 0.6666666667em;
}

.text-incorrect {
  background: color-mix(in srgb, var(--tblr-red) 4%, transparent);
  background: color-mix(in srgb, var(--tblr-red) 4%, transparent);
  text-decoration: underline;
  text-decoration-thickness: 1px;
  text-decoration-color: var(--tblr-red);
}

.text-correct {
  background: color-mix(in srgb, var(--tblr-green) 4%, transparent);
  background: color-mix(in srgb, var(--tblr-green) 4%, transparent);
  text-decoration: underline;
  text-decoration-thickness: 1px;
  text-decoration-color: var(--tblr-green);
}

.steps {
  --tblr-steps-padding: 2rem;
  --tblr-steps-item-size: 1.5rem;
  margin-right: 1rem;
  padding-right: var(--tblr-steps-padding);
  counter-reset: step;
  border-right: 1px solid var(--tblr-border-color);
  margin-bottom: 2rem;
}
.steps h3, .steps .h3 {
  counter-increment: step;
}
.steps h3:not(:first-child), .steps .h3:not(:first-child) {
  margin-top: 2.5rem !important;
}
.steps h3:before, .steps .h3:before {
  content: counter(step);
  display: inline-block;
  position: absolute;
  margin-top: 1px;
  margin-right: calc(-1 * var(--tblr-steps-padding) - var(--tblr-steps-item-size) / 2);
  width: var(--tblr-steps-item-size);
  height: var(--tblr-steps-item-size);
  text-align: center;
  color: var(--tblr-body-color);
  border: 1px solid var(--tblr-border-color);
  background: var(--tblr-bg-surface);
  border-radius: var(--tblr-border-radius);
  line-height: calc(var(--tblr-steps-item-size) - 2px);
  font-size: var(--tblr-font-size-h4);
  font-weight: var(--tblr-font-weight-bold);
}
.steps > :last-child {
  margin-bottom: 0;
}

.callout {
  margin-bottom: 1.5rem;
  border: 1px solid var(--tblr-primary-200);
  border-radius: var(--tblr-border-radius);
  padding: 0.5rem 1rem;
  background: var(--tblr-primary-lt);
}
.callout > :last-child {
  margin-bottom: 0;
}

.chart {
  display: block;
  min-height: 10rem;
}
.chart text {
  font-family: inherit;
}

.chart-sm {
  height: 2.5rem;
}

.chart-lg {
  height: 15rem;
}

.chart-square {
  height: 5.75rem;
}

/**
Chart sparkline
 */
.chart-sparkline {
  position: relative;
  width: 4rem;
  height: 2.5rem;
  line-height: 1;
  min-height: 0 !important;
}

.chart-sparkline-sm {
  height: 1.5rem;
}

.chart-sparkline-square {
  width: 2.5rem;
}

.chart-sparkline-wide {
  width: 6rem;
}

.chart-sparkline-label {
  position: absolute;
  top: 0;
  left: 0;
  bottom: 0;
  right: 0;
  display: flex;
  align-items: center;
  justify-content: center;
  font-size: 0.625rem;
}
.chart-sparkline-label .icon {
  width: 1rem;
  height: 1rem;
  font-size: 1rem;
}

.offcanvas-header {
  border-bottom: var(--tblr-border-width) var(--tblr-border-style) var(--tblr-border-color);
}

.offcanvas-footer {
  padding: 1.5rem 1.5rem;
}

.offcanvas-title {
  font-size: 1rem;
  font-weight: var(--tblr-font-weight-medium);
  line-height: 1.5rem;
}

.offcanvas-narrow {
  width: 20rem;
}

.chat-bubbles {
  display: flex;
  flex-direction: column;
  gap: 1rem;
}

.chat-bubble {
  background: var(--tblr-bg-surface-secondary);
  border-radius: var(--tblr-border-radius-lg);
  padding: 1rem;
  position: relative;
}

.chat-bubble-me {
  background-color: var(--tblr-primary-lt);
  box-shadow: none;
}

.chat-bubble-title {
  margin-bottom: 0.25rem;
}

.chat-bubble-author {
  font-weight: 600;
}

.chat-bubble-date {
  color: var(--tblr-secondary);
}

.chat-bubble-body > *:last-child {
  margin-bottom: 0;
}

.signature {
  --tblr-signature-padding: var(--tblr-spacer-1);
  --tblr-signature-border-radius: var(--tblr-border-radius);
  border: var(--tblr-border-width) solid var(--tblr-border-color);
  padding: var(--tblr-signature-padding);
  border-radius: var(--tblr-border-radius);
}

.signature-canvas {
  border: var(--tblr-border-width) dashed var(--tblr-border-color);
  border-radius: calc(var(--tblr-signature-border-radius) - var(--tblr-signature-padding));
  display: block;
  cursor: crosshair;
  width: 100%;
}

.clearfix::after {
  display: block;
  clear: both;
  content: "";
}

.text-truncate {
  overflow: hidden;
  text-overflow: ellipsis;
  white-space: nowrap;
}

.vr {
  display: inline-block;
  align-self: stretch;
  width: var(--tblr-border-width);
  min-height: 1em;
  background-color: currentcolor;
  opacity: 0.16;
}

.stretched-link::after {
  position: absolute;
  top: 0;
  left: 0;
  bottom: 0;
  right: 0;
  z-index: 1;
  content: "";
}

.visually-hidden,
.visually-hidden-focusable:not(:focus):not(:focus-within) {
  width: 1px !important;
  height: 1px !important;
  padding: 0 !important;
  margin: -1px !important;
  overflow: hidden !important;
  clip: rect(0, 0, 0, 0) !important;
  white-space: nowrap !important;
  border: 0 !important;
}
.visually-hidden:not(caption),
.visually-hidden-focusable:not(:focus):not(:focus-within):not(caption) {
  position: absolute !important;
}
.visually-hidden *,
.visually-hidden-focusable:not(:focus):not(:focus-within) * {
  overflow: hidden !important;
}

.hstack {
  display: flex;
  flex-direction: row;
  align-items: center;
  align-self: stretch;
}

.vstack {
  display: flex;
  flex: 1 1 auto;
  flex-direction: column;
  align-self: stretch;
}

.fixed-top {
  position: fixed;
  top: 0;
  left: 0;
  right: 0;
  z-index: 1030;
}

.fixed-bottom {
  position: fixed;
  left: 0;
  bottom: 0;
  right: 0;
  z-index: 1030;
}

.sticky-top {
  position: sticky;
  top: 0;
  z-index: 1020;
}

.sticky-bottom {
  position: sticky;
  bottom: 0;
  z-index: 1020;
}

@media (min-width: 576px) {
  .sticky-sm-top {
    position: sticky;
    top: 0;
    z-index: 1020;
  }
  .sticky-sm-bottom {
    position: sticky;
    bottom: 0;
    z-index: 1020;
  }
}
@media (min-width: 768px) {
  .sticky-md-top {
    position: sticky;
    top: 0;
    z-index: 1020;
  }
  .sticky-md-bottom {
    position: sticky;
    bottom: 0;
    z-index: 1020;
  }
}
@media (min-width: 992px) {
  .sticky-lg-top {
    position: sticky;
    top: 0;
    z-index: 1020;
  }
  .sticky-lg-bottom {
    position: sticky;
    bottom: 0;
    z-index: 1020;
  }
}
@media (min-width: 1200px) {
  .sticky-xl-top {
    position: sticky;
    top: 0;
    z-index: 1020;
  }
  .sticky-xl-bottom {
    position: sticky;
    bottom: 0;
    z-index: 1020;
  }
}
@media (min-width: 1400px) {
  .sticky-xxl-top {
    position: sticky;
    top: 0;
    z-index: 1020;
  }
  .sticky-xxl-bottom {
    position: sticky;
    bottom: 0;
    z-index: 1020;
  }
}
.ratio {
  position: relative;
  width: 100%;
}
.ratio::before {
  display: block;
  padding-top: var(--tblr-aspect-ratio);
  content: "";
}
.ratio > * {
  position: absolute;
  top: 0;
  right: 0;
  width: 100%;
  height: 100%;
}

.ratio-1x1 {
  --tblr-aspect-ratio: 100%;
}

.ratio-2x1 {
  --tblr-aspect-ratio: 50%;
}

.ratio-1x2 {
  --tblr-aspect-ratio: 200%;
}

.ratio-3x1 {
  --tblr-aspect-ratio: 33.3333333333%;
}

.ratio-1x3 {
  --tblr-aspect-ratio: 300%;
}

.ratio-4x1 {
  --tblr-aspect-ratio: 25%;
}

.ratio-1x4 {
  --tblr-aspect-ratio: 400%;
}

.ratio-4x3 {
  --tblr-aspect-ratio: 75%;
}

.ratio-3x4 {
  --tblr-aspect-ratio: 133.3333333333%;
}

.ratio-16x9 {
  --tblr-aspect-ratio: 56.25%;
}

.ratio-9x16 {
  --tblr-aspect-ratio: 177.7777777778%;
}

.ratio-21x9 {
  --tblr-aspect-ratio: 42.8571428571%;
}

.ratio-9x21 {
  --tblr-aspect-ratio: 233.3333333333%;
}

.focus-ring:focus {
  outline: 0;
  box-shadow: var(--tblr-focus-ring-x, 0) var(--tblr-focus-ring-y, 0) var(--tblr-focus-ring-blur, 0) var(--tblr-focus-ring-width) var(--tblr-focus-ring-color);
}

.bg-white-overlay {
  color: #ffffff;
  background-color: rgba(249, 250, 251, 0.24);
}

.bg-dark-overlay {
  color: #ffffff;
  background-color: rgba(31, 41, 55, 0.24);
}

.bg-cover {
  background-repeat: no-repeat;
  background-size: cover;
  background-position: center;
}

.bg-primary {
  background-color: color-mix(in srgb, var(--tblr-primary) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-primary-lt {
  color: color-mix(in srgb, var(--tblr-primary) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-primary-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-primary {
  border-color: color-mix(in srgb, var(--tblr-primary) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-primary {
  --tblr-gradient-from: var(--tblr-primary);
}

.bg-gradient-to-primary {
  --tblr-gradient-to: var(--tblr-primary);
}

.bg-gradient-via-primary {
  --tblr-gradient-via: var(--tblr-primary);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-primary {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-primary-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-primary {
  color: color-mix(in srgb, var(--tblr-primary) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-primary) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-primary:hover, .link-primary:focus {
  color: RGBA(5, 89, 167, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(5, 89, 167, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-secondary {
  background-color: color-mix(in srgb, var(--tblr-secondary) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-secondary-lt {
  color: color-mix(in srgb, var(--tblr-secondary) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-secondary-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-secondary {
  border-color: color-mix(in srgb, var(--tblr-secondary) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-secondary {
  --tblr-gradient-from: var(--tblr-secondary);
}

.bg-gradient-to-secondary {
  --tblr-gradient-to: var(--tblr-secondary);
}

.bg-gradient-via-secondary {
  --tblr-gradient-via: var(--tblr-secondary);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-secondary {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-secondary-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-secondary {
  color: color-mix(in srgb, var(--tblr-secondary) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-secondary) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-secondary:hover, .link-secondary:focus {
  color: RGBA(86, 91, 102, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(86, 91, 102, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-success {
  background-color: color-mix(in srgb, var(--tblr-success) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-success-lt {
  color: color-mix(in srgb, var(--tblr-success) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-success-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-success {
  border-color: color-mix(in srgb, var(--tblr-success) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-success {
  --tblr-gradient-from: var(--tblr-success);
}

.bg-gradient-to-success {
  --tblr-gradient-to: var(--tblr-success);
}

.bg-gradient-via-success {
  --tblr-gradient-via: var(--tblr-success);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-success {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-success-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-success {
  color: color-mix(in srgb, var(--tblr-success) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-success) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-success:hover, .link-success:focus {
  color: RGBA(38, 143, 54, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(38, 143, 54, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-info {
  background-color: color-mix(in srgb, var(--tblr-info) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-info-lt {
  color: color-mix(in srgb, var(--tblr-info) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-info-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-info {
  border-color: color-mix(in srgb, var(--tblr-info) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-info {
  --tblr-gradient-from: var(--tblr-info);
}

.bg-gradient-to-info {
  --tblr-gradient-to: var(--tblr-info);
}

.bg-gradient-via-info {
  --tblr-gradient-via: var(--tblr-info);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-info {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-info-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-info {
  color: color-mix(in srgb, var(--tblr-info) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-info) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-info:hover, .link-info:focus {
  color: RGBA(53, 122, 180, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(53, 122, 180, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-warning {
  background-color: color-mix(in srgb, var(--tblr-warning) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-warning-lt {
  color: color-mix(in srgb, var(--tblr-warning) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-warning-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-warning {
  border-color: color-mix(in srgb, var(--tblr-warning) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-warning {
  --tblr-gradient-from: var(--tblr-warning);
}

.bg-gradient-to-warning {
  --tblr-gradient-to: var(--tblr-warning);
}

.bg-gradient-via-warning {
  --tblr-gradient-via: var(--tblr-warning);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-warning {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-warning-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-warning {
  color: color-mix(in srgb, var(--tblr-warning) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-warning) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-warning:hover, .link-warning:focus {
  color: RGBA(196, 127, 0, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(196, 127, 0, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-danger {
  background-color: color-mix(in srgb, var(--tblr-danger) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-danger-lt {
  color: color-mix(in srgb, var(--tblr-danger) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-danger-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-danger {
  border-color: color-mix(in srgb, var(--tblr-danger) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-danger {
  --tblr-gradient-from: var(--tblr-danger);
}

.bg-gradient-to-danger {
  --tblr-gradient-to: var(--tblr-danger);
}

.bg-gradient-via-danger {
  --tblr-gradient-via: var(--tblr-danger);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-danger {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-danger-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-danger {
  color: color-mix(in srgb, var(--tblr-danger) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-danger) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-danger:hover, .link-danger:focus {
  color: RGBA(171, 46, 46, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(171, 46, 46, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-light {
  background-color: color-mix(in srgb, var(--tblr-light) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-light-lt {
  color: color-mix(in srgb, var(--tblr-light) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-light-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-light {
  border-color: color-mix(in srgb, var(--tblr-light) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-light {
  --tblr-gradient-from: var(--tblr-light);
}

.bg-gradient-to-light {
  --tblr-gradient-to: var(--tblr-light);
}

.bg-gradient-via-light {
  --tblr-gradient-via: var(--tblr-light);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-light {
  color: #1f2937 !important;
  background-color: RGBA(var(--tblr-light-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-light {
  color: color-mix(in srgb, var(--tblr-light) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-light) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-light:hover, .link-light:focus {
  color: RGBA(250, 251, 252, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(250, 251, 252, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-dark {
  background-color: color-mix(in srgb, var(--tblr-dark) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-dark-lt {
  color: color-mix(in srgb, var(--tblr-dark) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-dark-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-dark {
  border-color: color-mix(in srgb, var(--tblr-dark) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-dark {
  --tblr-gradient-from: var(--tblr-dark);
}

.bg-gradient-to-dark {
  --tblr-gradient-to: var(--tblr-dark);
}

.bg-gradient-via-dark {
  --tblr-gradient-via: var(--tblr-dark);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-dark {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-dark-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-dark {
  color: color-mix(in srgb, var(--tblr-dark) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-dark) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-dark:hover, .link-dark:focus {
  color: RGBA(25, 33, 44, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(25, 33, 44, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-muted {
  background-color: color-mix(in srgb, var(--tblr-muted) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-muted-lt {
  color: color-mix(in srgb, var(--tblr-muted) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-muted-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-muted {
  border-color: color-mix(in srgb, var(--tblr-muted) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-muted {
  --tblr-gradient-from: var(--tblr-muted);
}

.bg-gradient-to-muted {
  --tblr-gradient-to: var(--tblr-muted);
}

.bg-gradient-via-muted {
  --tblr-gradient-via: var(--tblr-muted);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-muted {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-muted-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-muted {
  color: color-mix(in srgb, var(--tblr-muted) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-muted) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-muted:hover, .link-muted:focus {
  color: RGBA(86, 91, 102, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(86, 91, 102, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-blue {
  background-color: color-mix(in srgb, var(--tblr-blue) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-blue-lt {
  color: color-mix(in srgb, var(--tblr-blue) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-blue-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-blue {
  border-color: color-mix(in srgb, var(--tblr-blue) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-blue {
  --tblr-gradient-from: var(--tblr-blue);
}

.bg-gradient-to-blue {
  --tblr-gradient-to: var(--tblr-blue);
}

.bg-gradient-via-blue {
  --tblr-gradient-via: var(--tblr-blue);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-blue {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-blue-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-blue {
  color: color-mix(in srgb, var(--tblr-blue) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-blue) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-blue:hover, .link-blue:focus {
  color: RGBA(5, 89, 167, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(5, 89, 167, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-azure {
  background-color: color-mix(in srgb, var(--tblr-azure) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-azure-lt {
  color: color-mix(in srgb, var(--tblr-azure) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-azure-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-azure {
  border-color: color-mix(in srgb, var(--tblr-azure) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-azure {
  --tblr-gradient-from: var(--tblr-azure);
}

.bg-gradient-to-azure {
  --tblr-gradient-to: var(--tblr-azure);
}

.bg-gradient-via-azure {
  --tblr-gradient-via: var(--tblr-azure);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-azure {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-azure-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-azure {
  color: color-mix(in srgb, var(--tblr-azure) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-azure) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-azure:hover, .link-azure:focus {
  color: RGBA(53, 122, 180, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(53, 122, 180, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-indigo {
  background-color: color-mix(in srgb, var(--tblr-indigo) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-indigo-lt {
  color: color-mix(in srgb, var(--tblr-indigo) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-indigo-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-indigo {
  border-color: color-mix(in srgb, var(--tblr-indigo) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-indigo {
  --tblr-gradient-from: var(--tblr-indigo);
}

.bg-gradient-to-indigo {
  --tblr-gradient-to: var(--tblr-indigo);
}

.bg-gradient-via-indigo {
  --tblr-gradient-via: var(--tblr-indigo);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-indigo {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-indigo-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-indigo {
  color: color-mix(in srgb, var(--tblr-indigo) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-indigo) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-indigo:hover, .link-indigo:focus {
  color: RGBA(53, 79, 188, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(53, 79, 188, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-purple {
  background-color: color-mix(in srgb, var(--tblr-purple) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-purple-lt {
  color: color-mix(in srgb, var(--tblr-purple) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-purple-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-purple {
  border-color: color-mix(in srgb, var(--tblr-purple) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-purple {
  --tblr-gradient-from: var(--tblr-purple);
}

.bg-gradient-to-purple {
  --tblr-gradient-to: var(--tblr-purple);
}

.bg-gradient-via-purple {
  --tblr-gradient-via: var(--tblr-purple);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-purple {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-purple-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-purple {
  color: color-mix(in srgb, var(--tblr-purple) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-purple) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-purple:hover, .link-purple:focus {
  color: RGBA(139, 50, 161, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(139, 50, 161, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-pink {
  background-color: color-mix(in srgb, var(--tblr-pink) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-pink-lt {
  color: color-mix(in srgb, var(--tblr-pink) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-pink-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-pink {
  border-color: color-mix(in srgb, var(--tblr-pink) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-pink {
  --tblr-gradient-from: var(--tblr-pink);
}

.bg-gradient-to-pink {
  --tblr-gradient-to: var(--tblr-pink);
}

.bg-gradient-via-pink {
  --tblr-gradient-via: var(--tblr-pink);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-pink {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-pink-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-pink {
  color: color-mix(in srgb, var(--tblr-pink) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-pink) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-pink:hover, .link-pink:focus {
  color: RGBA(171, 41, 86, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(171, 41, 86, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-red {
  background-color: color-mix(in srgb, var(--tblr-red) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-red-lt {
  color: color-mix(in srgb, var(--tblr-red) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-red-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-red {
  border-color: color-mix(in srgb, var(--tblr-red) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-red {
  --tblr-gradient-from: var(--tblr-red);
}

.bg-gradient-to-red {
  --tblr-gradient-to: var(--tblr-red);
}

.bg-gradient-via-red {
  --tblr-gradient-via: var(--tblr-red);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-red {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-red-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-red {
  color: color-mix(in srgb, var(--tblr-red) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-red) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-red:hover, .link-red:focus {
  color: RGBA(171, 46, 46, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(171, 46, 46, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-orange {
  background-color: color-mix(in srgb, var(--tblr-orange) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-orange-lt {
  color: color-mix(in srgb, var(--tblr-orange) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-orange-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-orange {
  border-color: color-mix(in srgb, var(--tblr-orange) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-orange {
  --tblr-gradient-from: var(--tblr-orange);
}

.bg-gradient-to-orange {
  --tblr-gradient-to: var(--tblr-orange);
}

.bg-gradient-via-orange {
  --tblr-gradient-via: var(--tblr-orange);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-orange {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-orange-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-orange {
  color: color-mix(in srgb, var(--tblr-orange) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-orange) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-orange:hover, .link-orange:focus {
  color: RGBA(198, 82, 6, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(198, 82, 6, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-yellow {
  background-color: color-mix(in srgb, var(--tblr-yellow) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-yellow-lt {
  color: color-mix(in srgb, var(--tblr-yellow) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-yellow-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-yellow {
  border-color: color-mix(in srgb, var(--tblr-yellow) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-yellow {
  --tblr-gradient-from: var(--tblr-yellow);
}

.bg-gradient-to-yellow {
  --tblr-gradient-to: var(--tblr-yellow);
}

.bg-gradient-via-yellow {
  --tblr-gradient-via: var(--tblr-yellow);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-yellow {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-yellow-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-yellow {
  color: color-mix(in srgb, var(--tblr-yellow) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-yellow) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-yellow:hover, .link-yellow:focus {
  color: RGBA(196, 127, 0, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(196, 127, 0, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-lime {
  background-color: color-mix(in srgb, var(--tblr-lime) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-lime-lt {
  color: color-mix(in srgb, var(--tblr-lime) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-lime-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-lime {
  border-color: color-mix(in srgb, var(--tblr-lime) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-lime {
  --tblr-gradient-from: var(--tblr-lime);
}

.bg-gradient-to-lime {
  --tblr-gradient-to: var(--tblr-lime);
}

.bg-gradient-via-lime {
  --tblr-gradient-via: var(--tblr-lime);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-lime {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-lime-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-lime {
  color: color-mix(in srgb, var(--tblr-lime) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-lime) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-lime:hover, .link-lime:focus {
  color: RGBA(93, 147, 18, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(93, 147, 18, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-green {
  background-color: color-mix(in srgb, var(--tblr-green) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-green-lt {
  color: color-mix(in srgb, var(--tblr-green) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-green-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-green {
  border-color: color-mix(in srgb, var(--tblr-green) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-green {
  --tblr-gradient-from: var(--tblr-green);
}

.bg-gradient-to-green {
  --tblr-gradient-to: var(--tblr-green);
}

.bg-gradient-via-green {
  --tblr-gradient-via: var(--tblr-green);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-green {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-green-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-green {
  color: color-mix(in srgb, var(--tblr-green) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-green) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-green:hover, .link-green:focus {
  color: RGBA(38, 143, 54, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(38, 143, 54, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-teal {
  background-color: color-mix(in srgb, var(--tblr-teal) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-teal-lt {
  color: color-mix(in srgb, var(--tblr-teal) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-teal-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-teal {
  border-color: color-mix(in srgb, var(--tblr-teal) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-teal {
  --tblr-gradient-from: var(--tblr-teal);
}

.bg-gradient-to-teal {
  --tblr-gradient-to: var(--tblr-teal);
}

.bg-gradient-via-teal {
  --tblr-gradient-via: var(--tblr-teal);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-teal {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-teal-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-teal {
  color: color-mix(in srgb, var(--tblr-teal) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-teal) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-teal:hover, .link-teal:focus {
  color: RGBA(10, 133, 96, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(10, 133, 96, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-cyan {
  background-color: color-mix(in srgb, var(--tblr-cyan) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-cyan-lt {
  color: color-mix(in srgb, var(--tblr-cyan) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-cyan-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-cyan {
  border-color: color-mix(in srgb, var(--tblr-cyan) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-cyan {
  --tblr-gradient-from: var(--tblr-cyan);
}

.bg-gradient-to-cyan {
  --tblr-gradient-to: var(--tblr-cyan);
}

.bg-gradient-via-cyan {
  --tblr-gradient-via: var(--tblr-cyan);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-cyan {
  color: #f9fafb !important;
  background-color: RGBA(var(--tblr-cyan-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-cyan {
  color: color-mix(in srgb, var(--tblr-cyan) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-cyan) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-cyan:hover, .link-cyan:focus {
  color: RGBA(18, 130, 147, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(18, 130, 147, var(--tblr-link-underline-opacity, 1)) !important;
}

.bg-white {
  background-color: color-mix(in srgb, var(--tblr-white) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.bg-white-lt {
  color: color-mix(in srgb, var(--tblr-white) calc(var(--tblr-text-opacity, 1) * 100%), transparent) !important;
  background-color: color-mix(in srgb, var(--tblr-white-lt) calc(var(--tblr-bg-opacity, 1) * 100%), transparent) !important;
}

.border-white {
  border-color: color-mix(in srgb, var(--tblr-white) calc(var(--tblr-border-opacity, 1) * 100%), transparent) !important;
}

.bg-gradient-from-white {
  --tblr-gradient-from: var(--tblr-white);
}

.bg-gradient-to-white {
  --tblr-gradient-to: var(--tblr-white);
}

.bg-gradient-via-white {
  --tblr-gradient-via: var(--tblr-white);
  --tblr-gradient-stops: var(--tblr-gradient-from, transparent), var(--tblr-gradient-via, transparent), var(--tblr-gradient-to, transparent);
}

.text-bg-white {
  color: #1f2937 !important;
  background-color: RGBA(var(--tblr-white-rgb), var(--tblr-bg-opacity, 1)) !important;
}

.link-white {
  color: color-mix(in srgb, var(--tblr-white) calc(var(--tblr-link-opacity, 1) * 100%), transparent) !important;
  text-decoration-color: color-mix(in srgb, var(--tblr-white) calc(var(--tblr-link-underline-opacity, 1) * 100%), transparent) !important;
}
.link-white:hover, .link-white:focus {
  color: RGBA(255, 255, 255, var(--tblr-link-opacity, 1)) !important;
  text-decoration-color: RGBA(255, 255, 255, var(--tblr-link-underline-opacity, 1)) !important;
}

.text-primary {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-primary) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-primary-fg {
  color: var(--tblr-primary-fg) !important;
}

.text-secondary {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-secondary) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-secondary-fg {
  color: var(--tblr-secondary-fg) !important;
}

.text-success {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-success) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-success-fg {
  color: var(--tblr-success-fg) !important;
}

.text-info {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-info) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-info-fg {
  color: var(--tblr-info-fg) !important;
}

.text-warning {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-warning) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-warning-fg {
  color: var(--tblr-warning-fg) !important;
}

.text-danger {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-danger) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-danger-fg {
  color: var(--tblr-danger-fg) !important;
}

.text-light {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-light) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-light-fg {
  color: var(--tblr-light-fg) !important;
}

.text-dark {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-dark) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-dark-fg {
  color: var(--tblr-dark-fg) !important;
}

.text-muted {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-muted) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-muted-fg {
  color: var(--tblr-muted-fg) !important;
}

.text-blue {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-blue) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-blue-fg {
  color: var(--tblr-blue-fg) !important;
}

.text-azure {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-azure) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-azure-fg {
  color: var(--tblr-azure-fg) !important;
}

.text-indigo {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-indigo) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-indigo-fg {
  color: var(--tblr-indigo-fg) !important;
}

.text-purple {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-purple) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-purple-fg {
  color: var(--tblr-purple-fg) !important;
}

.text-pink {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-pink) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-pink-fg {
  color: var(--tblr-pink-fg) !important;
}

.text-red {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-red) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-red-fg {
  color: var(--tblr-red-fg) !important;
}

.text-orange {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-orange) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-orange-fg {
  color: var(--tblr-orange-fg) !important;
}

.text-yellow {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-yellow) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-yellow-fg {
  color: var(--tblr-yellow-fg) !important;
}

.text-lime {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-lime) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-lime-fg {
  color: var(--tblr-lime-fg) !important;
}

.text-green {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-green) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-green-fg {
  color: var(--tblr-green-fg) !important;
}

.text-teal {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-teal) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-teal-fg {
  color: var(--tblr-teal-fg) !important;
}

.text-cyan {
  --tblr-text-opacity: 1;
  color: color-mix(in srgb, var(--tblr-cyan) calc(var(--tblr-text-opacity) * 100%), transparent) !important;
}

.text-cyan-fg {
  color: var(--tblr-cyan-fg) !important;
}

.bg-gray-50 {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-gray-50) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-gray-50-fg {
  color: var(--tblr-gray-50-fg) !important;
}

.bg-gray-100 {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-gray-100) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-gray-100-fg {
  color: var(--tblr-gray-100-fg) !important;
}

.bg-gray-200 {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-gray-200) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-gray-200-fg {
  color: var(--tblr-gray-200-fg) !important;
}

.bg-gray-300 {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-gray-300) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-gray-300-fg {
  color: var(--tblr-gray-300-fg) !important;
}

.bg-gray-400 {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-gray-400) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-gray-400-fg {
  color: var(--tblr-gray-400-fg) !important;
}

.bg-gray-500 {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-gray-500) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-gray-500-fg {
  color: var(--tblr-gray-500-fg) !important;
}

.bg-gray-600 {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-gray-600) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-gray-600-fg {
  color: var(--tblr-gray-600-fg) !important;
}

.bg-gray-700 {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-gray-700) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-gray-700-fg {
  color: var(--tblr-gray-700-fg) !important;
}

.bg-gray-800 {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-gray-800) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-gray-800-fg {
  color: var(--tblr-gray-800-fg) !important;
}

.bg-gray-900 {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-gray-900) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-gray-900-fg {
  color: var(--tblr-gray-900-fg) !important;
}

.bg-gray-950 {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-gray-950) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-gray-950-fg {
  color: var(--tblr-gray-950-fg) !important;
}

.bg-x {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-x) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-x-fg {
  color: var(--tblr-x-fg) !important;
}

.bg-facebook {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-facebook) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-facebook-fg {
  color: var(--tblr-facebook-fg) !important;
}

.bg-twitter {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-twitter) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-twitter-fg {
  color: var(--tblr-twitter-fg) !important;
}

.bg-linkedin {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-linkedin) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-linkedin-fg {
  color: var(--tblr-linkedin-fg) !important;
}

.bg-google {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-google) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-google-fg {
  color: var(--tblr-google-fg) !important;
}

.bg-youtube {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-youtube) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-youtube-fg {
  color: var(--tblr-youtube-fg) !important;
}

.bg-vimeo {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-vimeo) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-vimeo-fg {
  color: var(--tblr-vimeo-fg) !important;
}

.bg-dribbble {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-dribbble) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-dribbble-fg {
  color: var(--tblr-dribbble-fg) !important;
}

.bg-github {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-github) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-github-fg {
  color: var(--tblr-github-fg) !important;
}

.bg-instagram {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-instagram) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-instagram-fg {
  color: var(--tblr-instagram-fg) !important;
}

.bg-pinterest {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-pinterest) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-pinterest-fg {
  color: var(--tblr-pinterest-fg) !important;
}

.bg-vk {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-vk) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-vk-fg {
  color: var(--tblr-vk-fg) !important;
}

.bg-rss {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-rss) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-rss-fg {
  color: var(--tblr-rss-fg) !important;
}

.bg-flickr {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-flickr) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-flickr-fg {
  color: var(--tblr-flickr-fg) !important;
}

.bg-bitbucket {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-bitbucket) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-bitbucket-fg {
  color: var(--tblr-bitbucket-fg) !important;
}

.bg-tabler {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-tabler) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.text-tabler-fg {
  color: var(--tblr-tabler-fg) !important;
}

.bg-inverted {
  --tblr-bg-opacity: 1;
  background-color: color-mix(in srgb, var(--tblr-bg-surface-inverted) calc(var(--tblr-bg-opacity) * 100%), transparent) !important;
}

.bg-surface {
  background-color: var(--tblr-bg-surface) !important;
}

.bg-surface-secondary {
  background-color: var(--tblr-bg-surface-secondary) !important;
}

.bg-surface-tertiary {
  background-color: var(--tblr-bg-surface-tertiary) !important;
}

.bg-surface-backdrop {
  background-color: color-mix(in srgb, var(--tblr-gray-800) 24%, transparent) !important;
}

/*
Scrollable
*/
.scrollable {
  overflow-x: hidden;
  overflow-y: auto;
  -webkit-overflow-scrolling: touch;
}
.scrollable.hover {
  overflow-y: hidden;
}
.scrollable.hover > * {
  margin-top: -1px;
}
.scrollable.hover:hover, .scrollable.hover:focus, .scrollable.hover:active {
  overflow: visible;
  overflow-y: auto;
}
.touch .scrollable {
  overflow-y: auto !important;
}

.scroll-x,
.scroll-y {
  overflow: hidden;
  -webkit-overflow-scrolling: touch;
}

.scroll-y {
  overflow-y: auto;
}

.scroll-x {
  overflow-x: auto;
}

.no-scroll {
  overflow: hidden;
}

.w-0 {
  width: 0 !important;
}

.h-0 {
  height: 0 !important;
}

.w-1 {
  width: 0.25rem !important;
}

.h-1 {
  height: 0.25rem !important;
}

.w-2 {
  width: 0.5rem !important;
}

.h-2 {
  height: 0.5rem !important;
}

.w-3 {
  width: 1rem !important;
}

.h-3 {
  height: 1rem !important;
}

.w-4 {
  width: 1.5rem !important;
}

.h-4 {
  height: 1.5rem !important;
}

.w-5 {
  width: 2rem !important;
}

.h-5 {
  height: 2rem !important;
}

.w-6 {
  width: 2.5rem !important;
}

.h-6 {
  height: 2.5rem !important;
}

.w-auto {
  width: auto !important;
}

.h-auto {
  height: auto !important;
}

.w-px {
  width: 1px !important;
}

.h-px {
  height: 1px !important;
}

.w-full {
  width: 100% !important;
}

.h-full {
  height: 100% !important;
}

.opacity-0 {
  opacity: calc(0 / 100) !important;
}

.opacity-5 {
  opacity: calc(5 / 100) !important;
}

.opacity-10 {
  opacity: calc(10 / 100) !important;
}

.opacity-15 {
  opacity: calc(15 / 100) !important;
}

.opacity-20 {
  opacity: calc(20 / 100) !important;
}

.opacity-25 {
  opacity: calc(25 / 100) !important;
}

.opacity-30 {
  opacity: calc(30 / 100) !important;
}

.opacity-35 {
  opacity: calc(35 / 100) !important;
}

.opacity-40 {
  opacity: calc(40 / 100) !important;
}

.opacity-45 {
  opacity: calc(45 / 100) !important;
}

.opacity-50 {
  opacity: calc(50 / 100) !important;
}

.opacity-55 {
  opacity: calc(55 / 100) !important;
}

.opacity-60 {
  opacity: calc(60 / 100) !important;
}

.opacity-65 {
  opacity: calc(65 / 100) !important;
}

.opacity-70 {
  opacity: calc(70 / 100) !important;
}

.opacity-75 {
  opacity: calc(75 / 100) !important;
}

.opacity-80 {
  opacity: calc(80 / 100) !important;
}

.opacity-85 {
  opacity: calc(85 / 100) !important;
}

.opacity-90 {
  opacity: calc(90 / 100) !important;
}

.opacity-95 {
  opacity: calc(95 / 100) !important;
}

.opacity-100 {
  opacity: calc(100 / 100) !important;
}

.hover-shadow-sm:hover {
  box-shadow: 0 0.125rem 0.25rem rgba(0, 0, 0, 0.075) !important;
}

.hover-shadow:hover {
  box-shadow: rgba(var(--tblr-body-color-rgb), 0.04) 0 2px 4px 0 !important;
}

.hover-shadow-lg:hover {
  box-shadow: 0 1rem 3rem rgba(0, 0, 0, 0.175) !important;
}

.hover-shadow-none:hover {
  box-shadow: none !important;
}

/**
Antialiasing
 */
.antialiased {
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}

.subpixel-antialiased {
  -webkit-font-smoothing: auto;
  -moz-osx-font-smoothing: auto;
}

.hover-rotate-start, .hover-rotate-end, .hover-scale, .hover-elevate-down, .hover-elevate-up {
  transition: transform 0.3s ease;
}
.hover-rotate-start:hover, .hover-rotate-end:hover, .hover-scale:hover, .hover-elevate-down:hover, .hover-elevate-up:hover {
  will-change: transform;
}

.hover-elevate-up:hover {
  transform: translateY(-4px);
}

.hover-elevate-down:hover {
  transform: translateY(4px);
}

.hover-scale:hover {
  transform: scale(1.1);
}

.hover-rotate-end:hover {
  transform: rotate(-4deg);
}

.hover-rotate-start:hover {
  transform: rotate(4deg);
}

@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 100;
  src: url(fontsource/inter/files/inter-cyrillic-ext-100-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-100-normal.woff) format("woff");
  unicode-range: U+0460-052F, U+1C80-1C88, U+20B4, U+2DE0-2DFF, U+A640-A69F, U+FE2E-FE2F;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 100;
  src: url(fontsource/inter/files/inter-cyrillic-100-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-100-normal.woff) format("woff");
  unicode-range: U+0301, U+0400-045F, U+0490-0491, U+04B0-04B1, U+2116;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 100;
  src: url(fontsource/inter/files/inter-greek-ext-100-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-100-normal.woff) format("woff");
  unicode-range: U+1F00-1FFF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 100;
  src: url(fontsource/inter/files/inter-greek-100-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-100-normal.woff) format("woff");
  unicode-range: U+0370-03FF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 100;
  src: url(fontsource/inter/files/inter-vietnamese-100-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-100-normal.woff) format("woff");
  unicode-range: U+0102-0103, U+0110-0111, U+0128-0129, U+0168-0169, U+01A0-01A1, U+01AF-01B0, U+1EA0-1EF9, U+20AB;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 100;
  src: url(fontsource/inter/files/inter-latin-ext-100-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-100-normal.woff) format("woff");
  unicode-range: U+0100-024F, U+0259, U+1E00-1EFF, U+2020, U+20A0-20AB, U+20AD-20CF, U+2113, U+2C60-2C7F, U+A720-A7FF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 100;
  src: url(fontsource/inter/files/inter-latin-100-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-100-normal.woff) format("woff");
  unicode-range: U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+2000-206F, U+2074, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 200;
  src: url(fontsource/inter/files/inter-cyrillic-ext-200-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-200-normal.woff) format("woff");
  unicode-range: U+0460-052F, U+1C80-1C88, U+20B4, U+2DE0-2DFF, U+A640-A69F, U+FE2E-FE2F;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 200;
  src: url(fontsource/inter/files/inter-cyrillic-200-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-200-normal.woff) format("woff");
  unicode-range: U+0301, U+0400-045F, U+0490-0491, U+04B0-04B1, U+2116;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 200;
  src: url(fontsource/inter/files/inter-greek-ext-200-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-200-normal.woff) format("woff");
  unicode-range: U+1F00-1FFF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 200;
  src: url(fontsource/inter/files/inter-greek-200-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-200-normal.woff) format("woff");
  unicode-range: U+0370-03FF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 200;
  src: url(fontsource/inter/files/inter-vietnamese-200-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-200-normal.woff) format("woff");
  unicode-range: U+0102-0103, U+0110-0111, U+0128-0129, U+0168-0169, U+01A0-01A1, U+01AF-01B0, U+1EA0-1EF9, U+20AB;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 200;
  src: url(fontsource/inter/files/inter-latin-ext-200-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-200-normal.woff) format("woff");
  unicode-range: U+0100-024F, U+0259, U+1E00-1EFF, U+2020, U+20A0-20AB, U+20AD-20CF, U+2113, U+2C60-2C7F, U+A720-A7FF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 200;
  src: url(fontsource/inter/files/inter-latin-200-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-200-normal.woff) format("woff");
  unicode-range: U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+2000-206F, U+2074, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 300;
  src: url(fontsource/inter/files/inter-cyrillic-ext-300-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-300-normal.woff) format("woff");
  unicode-range: U+0460-052F, U+1C80-1C88, U+20B4, U+2DE0-2DFF, U+A640-A69F, U+FE2E-FE2F;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 300;
  src: url(fontsource/inter/files/inter-cyrillic-300-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-300-normal.woff) format("woff");
  unicode-range: U+0301, U+0400-045F, U+0490-0491, U+04B0-04B1, U+2116;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 300;
  src: url(fontsource/inter/files/inter-greek-ext-300-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-300-normal.woff) format("woff");
  unicode-range: U+1F00-1FFF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 300;
  src: url(fontsource/inter/files/inter-greek-300-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-300-normal.woff) format("woff");
  unicode-range: U+0370-03FF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 300;
  src: url(fontsource/inter/files/inter-vietnamese-300-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-300-normal.woff) format("woff");
  unicode-range: U+0102-0103, U+0110-0111, U+0128-0129, U+0168-0169, U+01A0-01A1, U+01AF-01B0, U+1EA0-1EF9, U+20AB;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 300;
  src: url(fontsource/inter/files/inter-latin-ext-300-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-300-normal.woff) format("woff");
  unicode-range: U+0100-024F, U+0259, U+1E00-1EFF, U+2020, U+20A0-20AB, U+20AD-20CF, U+2113, U+2C60-2C7F, U+A720-A7FF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 300;
  src: url(fontsource/inter/files/inter-latin-300-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-300-normal.woff) format("woff");
  unicode-range: U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+2000-206F, U+2074, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 400;
  src: url(fontsource/inter/files/inter-cyrillic-ext-400-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-400-normal.woff) format("woff");
  unicode-range: U+0460-052F, U+1C80-1C88, U+20B4, U+2DE0-2DFF, U+A640-A69F, U+FE2E-FE2F;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 400;
  src: url(fontsource/inter/files/inter-cyrillic-400-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-400-normal.woff) format("woff");
  unicode-range: U+0301, U+0400-045F, U+0490-0491, U+04B0-04B1, U+2116;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 400;
  src: url(fontsource/inter/files/inter-greek-ext-400-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-400-normal.woff) format("woff");
  unicode-range: U+1F00-1FFF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 400;
  src: url(fontsource/inter/files/inter-greek-400-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-400-normal.woff) format("woff");
  unicode-range: U+0370-03FF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 400;
  src: url(fontsource/inter/files/inter-vietnamese-400-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-400-normal.woff) format("woff");
  unicode-range: U+0102-0103, U+0110-0111, U+0128-0129, U+0168-0169, U+01A0-01A1, U+01AF-01B0, U+1EA0-1EF9, U+20AB;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 400;
  src: url(fontsource/inter/files/inter-latin-ext-400-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-400-normal.woff) format("woff");
  unicode-range: U+0100-024F, U+0259, U+1E00-1EFF, U+2020, U+20A0-20AB, U+20AD-20CF, U+2113, U+2C60-2C7F, U+A720-A7FF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 400;
  src: url(fontsource/inter/files/inter-latin-400-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-400-normal.woff) format("woff");
  unicode-range: U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+2000-206F, U+2074, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 500;
  src: url(fontsource/inter/files/inter-cyrillic-ext-500-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-500-normal.woff) format("woff");
  unicode-range: U+0460-052F, U+1C80-1C88, U+20B4, U+2DE0-2DFF, U+A640-A69F, U+FE2E-FE2F;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 500;
  src: url(fontsource/inter/files/inter-cyrillic-500-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-500-normal.woff) format("woff");
  unicode-range: U+0301, U+0400-045F, U+0490-0491, U+04B0-04B1, U+2116;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 500;
  src: url(fontsource/inter/files/inter-greek-ext-500-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-500-normal.woff) format("woff");
  unicode-range: U+1F00-1FFF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 500;
  src: url(fontsource/inter/files/inter-greek-500-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-500-normal.woff) format("woff");
  unicode-range: U+0370-03FF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 500;
  src: url(fontsource/inter/files/inter-vietnamese-500-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-500-normal.woff) format("woff");
  unicode-range: U+0102-0103, U+0110-0111, U+0128-0129, U+0168-0169, U+01A0-01A1, U+01AF-01B0, U+1EA0-1EF9, U+20AB;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 500;
  src: url(fontsource/inter/files/inter-latin-ext-500-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-500-normal.woff) format("woff");
  unicode-range: U+0100-024F, U+0259, U+1E00-1EFF, U+2020, U+20A0-20AB, U+20AD-20CF, U+2113, U+2C60-2C7F, U+A720-A7FF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 500;
  src: url(fontsource/inter/files/inter-latin-500-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-500-normal.woff) format("woff");
  unicode-range: U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+2000-206F, U+2074, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 600;
  src: url(fontsource/inter/files/inter-cyrillic-ext-600-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-600-normal.woff) format("woff");
  unicode-range: U+0460-052F, U+1C80-1C88, U+20B4, U+2DE0-2DFF, U+A640-A69F, U+FE2E-FE2F;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 600;
  src: url(fontsource/inter/files/inter-cyrillic-600-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-600-normal.woff) format("woff");
  unicode-range: U+0301, U+0400-045F, U+0490-0491, U+04B0-04B1, U+2116;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 600;
  src: url(fontsource/inter/files/inter-greek-ext-600-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-600-normal.woff) format("woff");
  unicode-range: U+1F00-1FFF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 600;
  src: url(fontsource/inter/files/inter-greek-600-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-600-normal.woff) format("woff");
  unicode-range: U+0370-03FF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 600;
  src: url(fontsource/inter/files/inter-vietnamese-600-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-600-normal.woff) format("woff");
  unicode-range: U+0102-0103, U+0110-0111, U+0128-0129, U+0168-0169, U+01A0-01A1, U+01AF-01B0, U+1EA0-1EF9, U+20AB;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 600;
  src: url(fontsource/inter/files/inter-latin-ext-600-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-600-normal.woff) format("woff");
  unicode-range: U+0100-024F, U+0259, U+1E00-1EFF, U+2020, U+20A0-20AB, U+20AD-20CF, U+2113, U+2C60-2C7F, U+A720-A7FF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 600;
  src: url(fontsource/inter/files/inter-latin-600-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-600-normal.woff) format("woff");
  unicode-range: U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+2000-206F, U+2074, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 700;
  src: url(fontsource/inter/files/inter-cyrillic-ext-700-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-700-normal.woff) format("woff");
  unicode-range: U+0460-052F, U+1C80-1C88, U+20B4, U+2DE0-2DFF, U+A640-A69F, U+FE2E-FE2F;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 700;
  src: url(fontsource/inter/files/inter-cyrillic-700-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-700-normal.woff) format("woff");
  unicode-range: U+0301, U+0400-045F, U+0490-0491, U+04B0-04B1, U+2116;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 700;
  src: url(fontsource/inter/files/inter-greek-ext-700-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-700-normal.woff) format("woff");
  unicode-range: U+1F00-1FFF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 700;
  src: url(fontsource/inter/files/inter-greek-700-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-700-normal.woff) format("woff");
  unicode-range: U+0370-03FF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 700;
  src: url(fontsource/inter/files/inter-vietnamese-700-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-700-normal.woff) format("woff");
  unicode-range: U+0102-0103, U+0110-0111, U+0128-0129, U+0168-0169, U+01A0-01A1, U+01AF-01B0, U+1EA0-1EF9, U+20AB;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 700;
  src: url(fontsource/inter/files/inter-latin-ext-700-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-700-normal.woff) format("woff");
  unicode-range: U+0100-024F, U+0259, U+1E00-1EFF, U+2020, U+20A0-20AB, U+20AD-20CF, U+2113, U+2C60-2C7F, U+A720-A7FF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 700;
  src: url(fontsource/inter/files/inter-latin-700-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-700-normal.woff) format("woff");
  unicode-range: U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+2000-206F, U+2074, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 800;
  src: url(fontsource/inter/files/inter-cyrillic-ext-800-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-800-normal.woff) format("woff");
  unicode-range: U+0460-052F, U+1C80-1C88, U+20B4, U+2DE0-2DFF, U+A640-A69F, U+FE2E-FE2F;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 800;
  src: url(fontsource/inter/files/inter-cyrillic-800-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-800-normal.woff) format("woff");
  unicode-range: U+0301, U+0400-045F, U+0490-0491, U+04B0-04B1, U+2116;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 800;
  src: url(fontsource/inter/files/inter-greek-ext-800-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-800-normal.woff) format("woff");
  unicode-range: U+1F00-1FFF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 800;
  src: url(fontsource/inter/files/inter-greek-800-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-800-normal.woff) format("woff");
  unicode-range: U+0370-03FF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 800;
  src: url(fontsource/inter/files/inter-vietnamese-800-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-800-normal.woff) format("woff");
  unicode-range: U+0102-0103, U+0110-0111, U+0128-0129, U+0168-0169, U+01A0-01A1, U+01AF-01B0, U+1EA0-1EF9, U+20AB;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 800;
  src: url(fontsource/inter/files/inter-latin-ext-800-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-800-normal.woff) format("woff");
  unicode-range: U+0100-024F, U+0259, U+1E00-1EFF, U+2020, U+20A0-20AB, U+20AD-20CF, U+2113, U+2C60-2C7F, U+A720-A7FF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 800;
  src: url(fontsource/inter/files/inter-latin-800-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-800-normal.woff) format("woff");
  unicode-range: U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+2000-206F, U+2074, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 900;
  src: url(fontsource/inter/files/inter-cyrillic-ext-900-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-900-normal.woff) format("woff");
  unicode-range: U+0460-052F, U+1C80-1C88, U+20B4, U+2DE0-2DFF, U+A640-A69F, U+FE2E-FE2F;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 900;
  src: url(fontsource/inter/files/inter-cyrillic-900-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-900-normal.woff) format("woff");
  unicode-range: U+0301, U+0400-045F, U+0490-0491, U+04B0-04B1, U+2116;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 900;
  src: url(fontsource/inter/files/inter-greek-ext-900-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-900-normal.woff) format("woff");
  unicode-range: U+1F00-1FFF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 900;
  src: url(fontsource/inter/files/inter-greek-900-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-900-normal.woff) format("woff");
  unicode-range: U+0370-03FF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 900;
  src: url(fontsource/inter/files/inter-vietnamese-900-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-900-normal.woff) format("woff");
  unicode-range: U+0102-0103, U+0110-0111, U+0128-0129, U+0168-0169, U+01A0-01A1, U+01AF-01B0, U+1EA0-1EF9, U+20AB;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 900;
  src: url(fontsource/inter/files/inter-latin-ext-900-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-900-normal.woff) format("woff");
  unicode-range: U+0100-024F, U+0259, U+1E00-1EFF, U+2020, U+20A0-20AB, U+20AD-20CF, U+2113, U+2C60-2C7F, U+A720-A7FF;
}
@font-face {
  font-family: "Inter";
  font-style: normal;
  font-display: swap;
  font-weight: 900;
  src: url(fontsource/inter/files/inter-latin-900-normal.woff2) format("woff2"), url(fontsource/inter/files/inter-all-900-normal.woff) format("woff");
  unicode-range: U+0000-00FF, U+0131, U+0152-0153, U+02BB-02BC, U+02C6, U+02DA, U+02DC, U+2000-206F, U+2074, U+20AC, U+2122, U+2191, U+2193, U+2212, U+2215, U+FEFF, U+FFFD;
}
:root {
  --tblr-font-sans-serif: inter, -apple-system, blinkmacsystemfont, san francisco, segoe ui, roboto, helvetica neue, sans-serif;
}

/*!
 * Tabler Icons 3.34.1 by tabler - https://tabler.io
 * License - https://github.com/tabler/tabler-icons/blob/master/LICENSE
 */
@font-face {
  font-family: "tabler-icons";
  font-style: normal;
  font-weight: 400;
  src: url(tabler/icons-webfont/dist/fonts/tabler-icons.woff2?v3.34.1) format("woff2"), url(tabler/icons-webfont/dist/fonts/tabler-icons.woff?) format("woff"), url(tabler/icons-webfont/dist/fonts/tabler-icons.ttf?v3.34.1) format("truetype");
}
.ti {
  font-family: "tabler-icons" !important;
  speak: none;
  font-style: normal;
  font-weight: normal;
  font-variant: normal;
  text-transform: none;
  line-height: 1;
  /* Better Font Rendering */
  -webkit-font-smoothing: antialiased;
  -moz-osx-font-smoothing: grayscale;
}

.ti-a-b:before {
  content: "\ec36";
}

.ti-a-b-2:before {
  content: "\f25f";
}

.ti-a-b-off:before {
  content: "\f0a6";
}

.ti-abacus:before {
  content: "\f05c";
}

.ti-abacus-off:before {
  content: "\f3b6";
}

.ti-abc:before {
  content: "\f567";
}

.ti-access-point:before {
  content: "\ed1b";
}

.ti-access-point-off:before {
  content: "\ed1a";
}

.ti-accessible:before {
  content: "\eba9";
}

.ti-accessible-filled:before {
  content: "\f6ea";
}

.ti-accessible-off:before {
  content: "\f0a7";
}

.ti-activity:before {
  content: "\ed23";
}

.ti-activity-heartbeat:before {
  content: "\f0db";
}

.ti-ad:before {
  content: "\ea02";
}

.ti-ad-2:before {
  content: "\ef1f";
}

.ti-ad-circle:before {
  content: "\f79e";
}

.ti-ad-circle-filled:before {
  content: "\f7d3";
}

.ti-ad-circle-off:before {
  content: "\f79d";
}

.ti-ad-filled:before {
  content: "\f6eb";
}

.ti-ad-off:before {
  content: "\f3b7";
}

.ti-address-book:before {
  content: "\f021";
}

.ti-address-book-off:before {
  content: "\f3b8";
}

.ti-adjustments:before {
  content: "\ea03";
}

.ti-adjustments-alt:before {
  content: "\ec37";
}

.ti-adjustments-bolt:before {
  content: "\f7fb";
}

.ti-adjustments-cancel:before {
  content: "\f7fc";
}

.ti-adjustments-check:before {
  content: "\f7fd";
}

.ti-adjustments-code:before {
  content: "\f7fe";
}

.ti-adjustments-cog:before {
  content: "\f7ff";
}

.ti-adjustments-dollar:before {
  content: "\f800";
}

.ti-adjustments-down:before {
  content: "\f801";
}

.ti-adjustments-exclamation:before {
  content: "\f802";
}

.ti-adjustments-filled:before {
  content: "\f6ec";
}

.ti-adjustments-heart:before {
  content: "\f803";
}

.ti-adjustments-horizontal:before {
  content: "\ec38";
}

.ti-adjustments-minus:before {
  content: "\f804";
}

.ti-adjustments-off:before {
  content: "\f0a8";
}

.ti-adjustments-pause:before {
  content: "\f805";
}

.ti-adjustments-pin:before {
  content: "\f806";
}

.ti-adjustments-plus:before {
  content: "\f807";
}

.ti-adjustments-question:before {
  content: "\f808";
}

.ti-adjustments-search:before {
  content: "\f809";
}

.ti-adjustments-share:before {
  content: "\f80a";
}

.ti-adjustments-spark:before {
  content: "\ffbe";
}

.ti-adjustments-star:before {
  content: "\f80b";
}

.ti-adjustments-up:before {
  content: "\f80c";
}

.ti-adjustments-x:before {
  content: "\f80d";
}

.ti-aerial-lift:before {
  content: "\edfe";
}

.ti-aerial-lift-filled:before {
  content: "\10101";
}

.ti-affiliate:before {
  content: "\edff";
}

.ti-affiliate-filled:before {
  content: "\f6ed";
}

.ti-ai:before {
  content: "\fee7";
}

.ti-air-balloon:before {
  content: "\f4a6";
}

.ti-air-balloon-filled:before {
  content: "\10100";
}

.ti-air-conditioning:before {
  content: "\f3a2";
}

.ti-air-conditioning-disabled:before {
  content: "\f542";
}

.ti-air-traffic-control:before {
  content: "\fb01";
}

.ti-alarm:before {
  content: "\ea04";
}

.ti-alarm-average:before {
  content: "\fc9e";
}

.ti-alarm-filled:before {
  content: "\f709";
}

.ti-alarm-minus:before {
  content: "\f630";
}

.ti-alarm-minus-filled:before {
  content: "\f70a";
}

.ti-alarm-off:before {
  content: "\f0a9";
}

.ti-alarm-plus:before {
  content: "\f631";
}

.ti-alarm-plus-filled:before {
  content: "\f70b";
}

.ti-alarm-smoke:before {
  content: "\100b6";
}

.ti-alarm-snooze:before {
  content: "\f632";
}

.ti-alarm-snooze-filled:before {
  content: "\f70c";
}

.ti-album:before {
  content: "\f022";
}

.ti-album-off:before {
  content: "\f3b9";
}

.ti-alert-circle:before {
  content: "\ea05";
}

.ti-alert-circle-filled:before {
  content: "\f6ee";
}

.ti-alert-circle-off:before {
  content: "\fc65";
}

.ti-alert-hexagon:before {
  content: "\f80e";
}

.ti-alert-hexagon-filled:before {
  content: "\fa34";
}

.ti-alert-hexagon-off:before {
  content: "\fc66";
}

.ti-alert-octagon:before {
  content: "\ecc6";
}

.ti-alert-octagon-filled:before {
  content: "\f6ef";
}

.ti-alert-small:before {
  content: "\f80f";
}

.ti-alert-small-off:before {
  content: "\fc67";
}

.ti-alert-square:before {
  content: "\f811";
}

.ti-alert-square-filled:before {
  content: "\fa35";
}

.ti-alert-square-rounded:before {
  content: "\f810";
}

.ti-alert-square-rounded-filled:before {
  content: "\fa36";
}

.ti-alert-square-rounded-off:before {
  content: "\fc68";
}

.ti-alert-triangle:before {
  content: "\ea06";
}

.ti-alert-triangle-filled:before {
  content: "\f6f0";
}

.ti-alert-triangle-off:before {
  content: "\fc69";
}

.ti-alien:before {
  content: "\ebde";
}

.ti-alien-filled:before {
  content: "\f70d";
}

.ti-align-box-bottom-center:before {
  content: "\f530";
}

.ti-align-box-bottom-center-filled:before {
  content: "\f70e";
}

.ti-align-box-bottom-left:before {
  content: "\f531";
}

.ti-align-box-bottom-left-filled:before {
  content: "\f70f";
}

.ti-align-box-bottom-right:before {
  content: "\f532";
}

.ti-align-box-bottom-right-filled:before {
  content: "\f710";
}

.ti-align-box-center-bottom:before {
  content: "\facb";
}

.ti-align-box-center-middle:before {
  content: "\f79f";
}

.ti-align-box-center-middle-filled:before {
  content: "\f7d4";
}

.ti-align-box-center-stretch:before {
  content: "\facc";
}

.ti-align-box-center-top:before {
  content: "\facd";
}

.ti-align-box-left-bottom:before {
  content: "\f533";
}

.ti-align-box-left-bottom-filled:before {
  content: "\f711";
}

.ti-align-box-left-middle:before {
  content: "\f534";
}

.ti-align-box-left-middle-filled:before {
  content: "\f712";
}

.ti-align-box-left-stretch:before {
  content: "\face";
}

.ti-align-box-left-top:before {
  content: "\f535";
}

.ti-align-box-left-top-filled:before {
  content: "\f713";
}

.ti-align-box-right-bottom:before {
  content: "\f536";
}

.ti-align-box-right-bottom-filled:before {
  content: "\f714";
}

.ti-align-box-right-middle:before {
  content: "\f537";
}

.ti-align-box-right-middle-filled:before {
  content: "\f7d5";
}

.ti-align-box-right-stretch:before {
  content: "\facf";
}

.ti-align-box-right-top:before {
  content: "\f538";
}

.ti-align-box-right-top-filled:before {
  content: "\f715";
}

.ti-align-box-top-center:before {
  content: "\f539";
}

.ti-align-box-top-center-filled:before {
  content: "\f716";
}

.ti-align-box-top-left:before {
  content: "\f53a";
}

.ti-align-box-top-left-filled:before {
  content: "\f717";
}

.ti-align-box-top-right:before {
  content: "\f53b";
}

.ti-align-box-top-right-filled:before {
  content: "\f718";
}

.ti-align-center:before {
  content: "\ea07";
}

.ti-align-justified:before {
  content: "\ea08";
}

.ti-align-left:before {
  content: "\ea09";
}

.ti-align-left-2:before {
  content: "\ff00";
}

.ti-align-right:before {
  content: "\ea0a";
}

.ti-align-right-2:before {
  content: "\feff";
}

.ti-alpha:before {
  content: "\f543";
}

.ti-alphabet-arabic:before {
  content: "\ff2f";
}

.ti-alphabet-bangla:before {
  content: "\ff2e";
}

.ti-alphabet-cyrillic:before {
  content: "\f1df";
}

.ti-alphabet-greek:before {
  content: "\f1e0";
}

.ti-alphabet-hebrew:before {
  content: "\ff2d";
}

.ti-alphabet-korean:before {
  content: "\ff2c";
}

.ti-alphabet-latin:before {
  content: "\f1e1";
}

.ti-alphabet-thai:before {
  content: "\ff2b";
}

.ti-alt:before {
  content: "\fc54";
}

.ti-ambulance:before {
  content: "\ebf5";
}

.ti-ampersand:before {
  content: "\f229";
}

.ti-analyze:before {
  content: "\f3a3";
}

.ti-analyze-filled:before {
  content: "\f719";
}

.ti-analyze-off:before {
  content: "\f3ba";
}

.ti-anchor:before {
  content: "\eb76";
}

.ti-anchor-off:before {
  content: "\f0f7";
}

.ti-angle:before {
  content: "\ef20";
}

.ti-ankh:before {
  content: "\f1cd";
}

.ti-antenna:before {
  content: "\f094";
}

.ti-antenna-bars-1:before {
  content: "\ecc7";
}

.ti-antenna-bars-2:before {
  content: "\ecc8";
}

.ti-antenna-bars-3:before {
  content: "\ecc9";
}

.ti-antenna-bars-4:before {
  content: "\ecca";
}

.ti-antenna-bars-5:before {
  content: "\eccb";
}

.ti-antenna-bars-off:before {
  content: "\f0aa";
}

.ti-antenna-off:before {
  content: "\f3bb";
}

.ti-aperture:before {
  content: "\eb58";
}

.ti-aperture-off:before {
  content: "\f3bc";
}

.ti-api:before {
  content: "\effd";
}

.ti-api-app:before {
  content: "\effc";
}

.ti-api-app-off:before {
  content: "\f0ab";
}

.ti-api-off:before {
  content: "\f0f8";
}

.ti-app-window:before {
  content: "\efe6";
}

.ti-app-window-filled:before {
  content: "\f71a";
}

.ti-apple:before {
  content: "\ef21";
}

.ti-apple-filled:before {
  content: "\10017";
}

.ti-apps:before {
  content: "\ebb6";
}

.ti-apps-filled:before {
  content: "\f6f1";
}

.ti-apps-off:before {
  content: "\f0ac";
}

.ti-archery-arrow:before {
  content: "\fc55";
}

.ti-archive:before {
  content: "\ea0b";
}

.ti-archive-filled:before {
  content: "\fa82";
}

.ti-archive-off:before {
  content: "\f0ad";
}

.ti-armchair:before {
  content: "\ef9e";
}

.ti-armchair-2:before {
  content: "\efe7";
}

.ti-armchair-2-off:before {
  content: "\f3bd";
}

.ti-armchair-off:before {
  content: "\f3be";
}

.ti-arrow-autofit-content:before {
  content: "\ef31";
}

.ti-arrow-autofit-content-filled:before {
  content: "\f6f2";
}

.ti-arrow-autofit-down:before {
  content: "\ef32";
}

.ti-arrow-autofit-down-filled:before {
  content: "\10113";
}

.ti-arrow-autofit-height:before {
  content: "\ef33";
}

.ti-arrow-autofit-height-filled:before {
  content: "\10112";
}

.ti-arrow-autofit-left:before {
  content: "\ef34";
}

.ti-arrow-autofit-left-filled:before {
  content: "\10111";
}

.ti-arrow-autofit-right:before {
  content: "\ef35";
}

.ti-arrow-autofit-right-filled:before {
  content: "\10110";
}

.ti-arrow-autofit-up:before {
  content: "\ef36";
}

.ti-arrow-autofit-up-filled:before {
  content: "\1010f";
}

.ti-arrow-autofit-width:before {
  content: "\ef37";
}

.ti-arrow-autofit-width-filled:before {
  content: "\1010e";
}

.ti-arrow-back:before {
  content: "\ea0c";
}

.ti-arrow-back-up:before {
  content: "\eb77";
}

.ti-arrow-back-up-double:before {
  content: "\f9ec";
}

.ti-arrow-badge-down:before {
  content: "\f60b";
}

.ti-arrow-badge-down-filled:before {
  content: "\f7d6";
}

.ti-arrow-badge-left:before {
  content: "\f60c";
}

.ti-arrow-badge-left-filled:before {
  content: "\f7d7";
}

.ti-arrow-badge-right:before {
  content: "\f60d";
}

.ti-arrow-badge-right-filled:before {
  content: "\f7d8";
}

.ti-arrow-badge-up:before {
  content: "\f60e";
}

.ti-arrow-badge-up-filled:before {
  content: "\f7d9";
}

.ti-arrow-bar-both:before {
  content: "\fadd";
}

.ti-arrow-bar-down:before {
  content: "\ea0d";
}

.ti-arrow-bar-left:before {
  content: "\ea0e";
}

.ti-arrow-bar-right:before {
  content: "\ea0f";
}

.ti-arrow-bar-to-down:before {
  content: "\ec88";
}

.ti-arrow-bar-to-down-dashed:before {
  content: "\10164";
}

.ti-arrow-bar-to-left:before {
  content: "\ec89";
}

.ti-arrow-bar-to-left-dashed:before {
  content: "\10163";
}

.ti-arrow-bar-to-right:before {
  content: "\ec8a";
}

.ti-arrow-bar-to-right-dashed:before {
  content: "\10162";
}

.ti-arrow-bar-to-up:before {
  content: "\ec8b";
}

.ti-arrow-bar-to-up-dashed:before {
  content: "\10161";
}

.ti-arrow-bar-up:before {
  content: "\ea10";
}

.ti-arrow-bear-left:before {
  content: "\f045";
}

.ti-arrow-bear-left-2:before {
  content: "\f044";
}

.ti-arrow-bear-right:before {
  content: "\f047";
}

.ti-arrow-bear-right-2:before {
  content: "\f046";
}

.ti-arrow-big-down:before {
  content: "\edda";
}

.ti-arrow-big-down-filled:before {
  content: "\f6c6";
}

.ti-arrow-big-down-line:before {
  content: "\efe8";
}

.ti-arrow-big-down-line-filled:before {
  content: "\f6c7";
}

.ti-arrow-big-down-lines:before {
  content: "\efe9";
}

.ti-arrow-big-down-lines-filled:before {
  content: "\f6c8";
}

.ti-arrow-big-left:before {
  content: "\eddb";
}

.ti-arrow-big-left-filled:before {
  content: "\f6c9";
}

.ti-arrow-big-left-line:before {
  content: "\efea";
}

.ti-arrow-big-left-line-filled:before {
  content: "\f6ca";
}

.ti-arrow-big-left-lines:before {
  content: "\efeb";
}

.ti-arrow-big-left-lines-filled:before {
  content: "\f6cb";
}

.ti-arrow-big-right:before {
  content: "\eddc";
}

.ti-arrow-big-right-filled:before {
  content: "\f6cc";
}

.ti-arrow-big-right-line:before {
  content: "\efec";
}

.ti-arrow-big-right-line-filled:before {
  content: "\f6cd";
}

.ti-arrow-big-right-lines:before {
  content: "\efed";
}

.ti-arrow-big-right-lines-filled:before {
  content: "\f6ce";
}

.ti-arrow-big-up:before {
  content: "\eddd";
}

.ti-arrow-big-up-filled:before {
  content: "\f6cf";
}

.ti-arrow-big-up-line:before {
  content: "\efee";
}

.ti-arrow-big-up-line-filled:before {
  content: "\f6d0";
}

.ti-arrow-big-up-lines:before {
  content: "\efef";
}

.ti-arrow-big-up-lines-filled:before {
  content: "\f6d1";
}

.ti-arrow-bounce:before {
  content: "\f3a4";
}

.ti-arrow-capsule:before {
  content: "\fade";
}

.ti-arrow-curve-left:before {
  content: "\f048";
}

.ti-arrow-curve-right:before {
  content: "\f049";
}

.ti-arrow-down:before {
  content: "\ea16";
}

.ti-arrow-down-bar:before {
  content: "\ed98";
}

.ti-arrow-down-circle:before {
  content: "\ea11";
}

.ti-arrow-down-circle-filled:before {
  content: "\1003b";
}

.ti-arrow-down-dashed:before {
  content: "\1006a";
}

.ti-arrow-down-from-arc:before {
  content: "\fd86";
}

.ti-arrow-down-left:before {
  content: "\ea13";
}

.ti-arrow-down-left-circle:before {
  content: "\ea12";
}

.ti-arrow-down-rhombus:before {
  content: "\f61d";
}

.ti-arrow-down-rhombus-filled:before {
  content: "\1003a";
}

.ti-arrow-down-right:before {
  content: "\ea15";
}

.ti-arrow-down-right-circle:before {
  content: "\ea14";
}

.ti-arrow-down-square:before {
  content: "\ed9a";
}

.ti-arrow-down-square-filled:before {
  content: "\10039";
}

.ti-arrow-down-tail:before {
  content: "\ed9b";
}

.ti-arrow-down-to-arc:before {
  content: "\fd87";
}

.ti-arrow-elbow-left:before {
  content: "\f9ed";
}

.ti-arrow-elbow-right:before {
  content: "\f9ee";
}

.ti-arrow-fork:before {
  content: "\f04a";
}

.ti-arrow-forward:before {
  content: "\ea17";
}

.ti-arrow-forward-up:before {
  content: "\eb78";
}

.ti-arrow-forward-up-double:before {
  content: "\f9ef";
}

.ti-arrow-guide:before {
  content: "\f22a";
}

.ti-arrow-guide-filled:before {
  content: "\10038";
}

.ti-arrow-iteration:before {
  content: "\f578";
}

.ti-arrow-left:before {
  content: "\ea19";
}

.ti-arrow-left-bar:before {
  content: "\ed9c";
}

.ti-arrow-left-circle:before {
  content: "\ea18";
}

.ti-arrow-left-circle-filled:before {
  content: "\10037";
}

.ti-arrow-left-dashed:before {
  content: "\10069";
}

.ti-arrow-left-from-arc:before {
  content: "\fd88";
}

.ti-arrow-left-rhombus:before {
  content: "\f61e";
}

.ti-arrow-left-rhombus-filled:before {
  content: "\10036";
}

.ti-arrow-left-right:before {
  content: "\f04b";
}

.ti-arrow-left-square:before {
  content: "\ed9d";
}

.ti-arrow-left-square-filled:before {
  content: "\10035";
}

.ti-arrow-left-tail:before {
  content: "\ed9e";
}

.ti-arrow-left-to-arc:before {
  content: "\fd89";
}

.ti-arrow-loop-left:before {
  content: "\ed9f";
}

.ti-arrow-loop-left-2:before {
  content: "\f04c";
}

.ti-arrow-loop-right:before {
  content: "\eda0";
}

.ti-arrow-loop-right-2:before {
  content: "\f04d";
}

.ti-arrow-merge:before {
  content: "\f04e";
}

.ti-arrow-merge-alt-left:before {
  content: "\fc9f";
}

.ti-arrow-merge-alt-right:before {
  content: "\fca0";
}

.ti-arrow-merge-both:before {
  content: "\f23b";
}

.ti-arrow-merge-left:before {
  content: "\f23c";
}

.ti-arrow-merge-right:before {
  content: "\f23d";
}

.ti-arrow-move-down:before {
  content: "\f2ba";
}

.ti-arrow-move-down-filled:before {
  content: "\10034";
}

.ti-arrow-move-left:before {
  content: "\f2bb";
}

.ti-arrow-move-left-filled:before {
  content: "\10033";
}

.ti-arrow-move-right:before {
  content: "\f2bc";
}

.ti-arrow-move-right-filled:before {
  content: "\10032";
}

.ti-arrow-move-up:before {
  content: "\f2bd";
}

.ti-arrow-move-up-filled:before {
  content: "\10031";
}

.ti-arrow-narrow-down:before {
  content: "\ea1a";
}

.ti-arrow-narrow-down-dashed:before {
  content: "\10068";
}

.ti-arrow-narrow-left:before {
  content: "\ea1b";
}

.ti-arrow-narrow-left-dashed:before {
  content: "\10067";
}

.ti-arrow-narrow-right:before {
  content: "\ea1c";
}

.ti-arrow-narrow-right-dashed:before {
  content: "\10066";
}

.ti-arrow-narrow-up:before {
  content: "\ea1d";
}

.ti-arrow-narrow-up-dashed:before {
  content: "\10065";
}

.ti-arrow-ramp-left:before {
  content: "\ed3c";
}

.ti-arrow-ramp-left-2:before {
  content: "\f04f";
}

.ti-arrow-ramp-left-3:before {
  content: "\f050";
}

.ti-arrow-ramp-right:before {
  content: "\ed3d";
}

.ti-arrow-ramp-right-2:before {
  content: "\f051";
}

.ti-arrow-ramp-right-3:before {
  content: "\f052";
}

.ti-arrow-right:before {
  content: "\ea1f";
}

.ti-arrow-right-bar:before {
  content: "\eda1";
}

.ti-arrow-right-circle:before {
  content: "\ea1e";
}

.ti-arrow-right-circle-filled:before {
  content: "\10030";
}

.ti-arrow-right-dashed:before {
  content: "\10064";
}

.ti-arrow-right-from-arc:before {
  content: "\fd8a";
}

.ti-arrow-right-rhombus:before {
  content: "\f61f";
}

.ti-arrow-right-rhombus-filled:before {
  content: "\1002f";
}

.ti-arrow-right-square:before {
  content: "\eda2";
}

.ti-arrow-right-square-filled:before {
  content: "\1002e";
}

.ti-arrow-right-tail:before {
  content: "\eda3";
}

.ti-arrow-right-to-arc:before {
  content: "\fd8b";
}

.ti-arrow-rotary-first-left:before {
  content: "\f053";
}

.ti-arrow-rotary-first-right:before {
  content: "\f054";
}

.ti-arrow-rotary-last-left:before {
  content: "\f055";
}

.ti-arrow-rotary-last-right:before {
  content: "\f056";
}

.ti-arrow-rotary-left:before {
  content: "\f057";
}

.ti-arrow-rotary-right:before {
  content: "\f058";
}

.ti-arrow-rotary-straight:before {
  content: "\f059";
}

.ti-arrow-roundabout-left:before {
  content: "\f22b";
}

.ti-arrow-roundabout-right:before {
  content: "\f22c";
}

.ti-arrow-sharp-turn-left:before {
  content: "\f05a";
}

.ti-arrow-sharp-turn-right:before {
  content: "\f05b";
}

.ti-arrow-up:before {
  content: "\ea25";
}

.ti-arrow-up-bar:before {
  content: "\eda4";
}

.ti-arrow-up-circle:before {
  content: "\ea20";
}

.ti-arrow-up-circle-filled:before {
  content: "\1002d";
}

.ti-arrow-up-dashed:before {
  content: "\10063";
}

.ti-arrow-up-from-arc:before {
  content: "\fd8c";
}

.ti-arrow-up-left:before {
  content: "\ea22";
}

.ti-arrow-up-left-circle:before {
  content: "\ea21";
}

.ti-arrow-up-rhombus:before {
  content: "\f620";
}

.ti-arrow-up-rhombus-filled:before {
  content: "\1002c";
}

.ti-arrow-up-right:before {
  content: "\ea24";
}

.ti-arrow-up-right-circle:before {
  content: "\ea23";
}

.ti-arrow-up-square:before {
  content: "\eda6";
}

.ti-arrow-up-square-filled:before {
  content: "\1002b";
}

.ti-arrow-up-tail:before {
  content: "\eda7";
}

.ti-arrow-up-to-arc:before {
  content: "\fd8d";
}

.ti-arrow-wave-left-down:before {
  content: "\eda8";
}

.ti-arrow-wave-left-up:before {
  content: "\eda9";
}

.ti-arrow-wave-right-down:before {
  content: "\edaa";
}

.ti-arrow-wave-right-up:before {
  content: "\edab";
}

.ti-arrow-zig-zag:before {
  content: "\f4a7";
}

.ti-arrows-cross:before {
  content: "\effe";
}

.ti-arrows-diagonal:before {
  content: "\ea27";
}

.ti-arrows-diagonal-2:before {
  content: "\ea26";
}

.ti-arrows-diagonal-minimize:before {
  content: "\ef39";
}

.ti-arrows-diagonal-minimize-2:before {
  content: "\ef38";
}

.ti-arrows-diff:before {
  content: "\f296";
}

.ti-arrows-double-ne-sw:before {
  content: "\edde";
}

.ti-arrows-double-nw-se:before {
  content: "\eddf";
}

.ti-arrows-double-se-nw:before {
  content: "\ede0";
}

.ti-arrows-double-sw-ne:before {
  content: "\ede1";
}

.ti-arrows-down:before {
  content: "\edad";
}

.ti-arrows-down-up:before {
  content: "\edac";
}

.ti-arrows-exchange:before {
  content: "\f1f4";
}

.ti-arrows-exchange-2:before {
  content: "\f1f3";
}

.ti-arrows-horizontal:before {
  content: "\eb59";
}

.ti-arrows-join:before {
  content: "\edaf";
}

.ti-arrows-join-2:before {
  content: "\edae";
}

.ti-arrows-left:before {
  content: "\edb1";
}

.ti-arrows-left-down:before {
  content: "\ee00";
}

.ti-arrows-left-right:before {
  content: "\edb0";
}

.ti-arrows-maximize:before {
  content: "\ea28";
}

.ti-arrows-minimize:before {
  content: "\ea29";
}

.ti-arrows-move:before {
  content: "\f22f";
}

.ti-arrows-move-horizontal:before {
  content: "\f22d";
}

.ti-arrows-move-vertical:before {
  content: "\f22e";
}

.ti-arrows-random:before {
  content: "\f095";
}

.ti-arrows-right:before {
  content: "\edb3";
}

.ti-arrows-right-down:before {
  content: "\ee01";
}

.ti-arrows-right-left:before {
  content: "\edb2";
}

.ti-arrows-shuffle:before {
  content: "\f000";
}

.ti-arrows-shuffle-2:before {
  content: "\efff";
}

.ti-arrows-sort:before {
  content: "\eb5a";
}

.ti-arrows-split:before {
  content: "\edb5";
}

.ti-arrows-split-2:before {
  content: "\edb4";
}

.ti-arrows-transfer-down:before {
  content: "\f2cc";
}

.ti-arrows-transfer-up:before {
  content: "\f2cd";
}

.ti-arrows-transfer-up-down:before {
  content: "\ffac";
}

.ti-arrows-up:before {
  content: "\edb7";
}

.ti-arrows-up-down:before {
  content: "\edb6";
}

.ti-arrows-up-left:before {
  content: "\ee02";
}

.ti-arrows-up-right:before {
  content: "\ee03";
}

.ti-arrows-vertical:before {
  content: "\eb5b";
}

.ti-artboard:before {
  content: "\ea2a";
}

.ti-artboard-filled:before {
  content: "\fa83";
}

.ti-artboard-off:before {
  content: "\f0ae";
}

.ti-article:before {
  content: "\f1e2";
}

.ti-article-filled:before {
  content: "\f7da";
}

.ti-article-off:before {
  content: "\f3bf";
}

.ti-aspect-ratio:before {
  content: "\ed30";
}

.ti-aspect-ratio-filled:before {
  content: "\f7db";
}

.ti-aspect-ratio-off:before {
  content: "\f0af";
}

.ti-assembly:before {
  content: "\f24d";
}

.ti-assembly-filled:before {
  content: "\fe9e";
}

.ti-assembly-off:before {
  content: "\f3c0";
}

.ti-asset:before {
  content: "\f1ce";
}

.ti-asset-filled:before {
  content: "\fe9d";
}

.ti-asterisk:before {
  content: "\efd5";
}

.ti-asterisk-simple:before {
  content: "\efd4";
}

.ti-at:before {
  content: "\ea2b";
}

.ti-at-off:before {
  content: "\f0b0";
}

.ti-atom:before {
  content: "\eb79";
}

.ti-atom-2:before {
  content: "\ebdf";
}

.ti-atom-2-filled:before {
  content: "\f71b";
}

.ti-atom-off:before {
  content: "\f0f9";
}

.ti-augmented-reality:before {
  content: "\f023";
}

.ti-augmented-reality-2:before {
  content: "\f37e";
}

.ti-augmented-reality-off:before {
  content: "\f3c1";
}

.ti-auth-2fa:before {
  content: "\eca0";
}

.ti-automatic-gearbox:before {
  content: "\fc89";
}

.ti-automatic-gearbox-filled:before {
  content: "\1002a";
}

.ti-automation:before {
  content: "\fef8";
}

.ti-avocado:before {
  content: "\fd8e";
}

.ti-award:before {
  content: "\ea2c";
}

.ti-award-filled:before {
  content: "\f71c";
}

.ti-award-off:before {
  content: "\f0fa";
}

.ti-axe:before {
  content: "\ef9f";
}

.ti-axis-x:before {
  content: "\ef45";
}

.ti-axis-y:before {
  content: "\ef46";
}

.ti-baby-bottle:before {
  content: "\f5d2";
}

.ti-baby-carriage:before {
  content: "\f05d";
}

.ti-baby-carriage-filled:before {
  content: "\fe9c";
}

.ti-background:before {
  content: "\fd2c";
}

.ti-backhoe:before {
  content: "\ed86";
}

.ti-backpack:before {
  content: "\ef47";
}

.ti-backpack-off:before {
  content: "\f3c2";
}

.ti-backslash:before {
  content: "\fab9";
}

.ti-backspace:before {
  content: "\ea2d";
}

.ti-backspace-filled:before {
  content: "\f7dc";
}

.ti-badge:before {
  content: "\efc2";
}

.ti-badge-2k:before {
  content: "\100b5";
}

.ti-badge-3d:before {
  content: "\f555";
}

.ti-badge-3d-filled:before {
  content: "\fe9b";
}

.ti-badge-3k:before {
  content: "\100b4";
}

.ti-badge-4k:before {
  content: "\f556";
}

.ti-badge-4k-filled:before {
  content: "\fe9a";
}

.ti-badge-5k:before {
  content: "\100b3";
}

.ti-badge-8k:before {
  content: "\f557";
}

.ti-badge-8k-filled:before {
  content: "\fe99";
}

.ti-badge-ad:before {
  content: "\f558";
}

.ti-badge-ad-filled:before {
  content: "\fe98";
}

.ti-badge-ad-off:before {
  content: "\fd8f";
}

.ti-badge-ar:before {
  content: "\f559";
}

.ti-badge-ar-filled:before {
  content: "\fe97";
}

.ti-badge-cc:before {
  content: "\f55a";
}

.ti-badge-cc-filled:before {
  content: "\fe96";
}

.ti-badge-filled:before {
  content: "\f667";
}

.ti-badge-hd:before {
  content: "\f55b";
}

.ti-badge-hd-filled:before {
  content: "\fe95";
}

.ti-badge-off:before {
  content: "\f0fb";
}

.ti-badge-sd:before {
  content: "\f55c";
}

.ti-badge-sd-filled:before {
  content: "\fe94";
}

.ti-badge-tm:before {
  content: "\f55d";
}

.ti-badge-tm-filled:before {
  content: "\fe93";
}

.ti-badge-vo:before {
  content: "\f55e";
}

.ti-badge-vo-filled:before {
  content: "\fe92";
}

.ti-badge-vr:before {
  content: "\f55f";
}

.ti-badge-vr-filled:before {
  content: "\fe91";
}

.ti-badge-wc:before {
  content: "\f560";
}

.ti-badge-wc-filled:before {
  content: "\fe90";
}

.ti-badges:before {
  content: "\efc3";
}

.ti-badges-filled:before {
  content: "\f7dd";
}

.ti-badges-off:before {
  content: "\f0fc";
}

.ti-baguette:before {
  content: "\f3a5";
}

.ti-ball-american-football:before {
  content: "\ee04";
}

.ti-ball-american-football-off:before {
  content: "\f3c3";
}

.ti-ball-baseball:before {
  content: "\efa0";
}

.ti-ball-basketball:before {
  content: "\ec28";
}

.ti-ball-bowling:before {
  content: "\ec29";
}

.ti-ball-football:before {
  content: "\ee06";
}

.ti-ball-football-off:before {
  content: "\ee05";
}

.ti-ball-tennis:before {
  content: "\ec2a";
}

.ti-ball-volleyball:before {
  content: "\ec2b";
}

.ti-balloon:before {
  content: "\ef3a";
}

.ti-balloon-filled:before {
  content: "\fa84";
}

.ti-balloon-off:before {
  content: "\f0fd";
}

.ti-ballpen:before {
  content: "\f06e";
}

.ti-ballpen-filled:before {
  content: "\fa85";
}

.ti-ballpen-off:before {
  content: "\f0b1";
}

.ti-ban:before {
  content: "\ea2e";
}

.ti-bandage:before {
  content: "\eb7a";
}

.ti-bandage-filled:before {
  content: "\f7de";
}

.ti-bandage-off:before {
  content: "\f3c4";
}

.ti-barbell:before {
  content: "\eff0";
}

.ti-barbell-filled:before {
  content: "\fe8f";
}

.ti-barbell-off:before {
  content: "\f0b2";
}

.ti-barcode:before {
  content: "\ebc6";
}

.ti-barcode-off:before {
  content: "\f0b3";
}

.ti-barrel:before {
  content: "\f0b4";
}

.ti-barrel-off:before {
  content: "\f0fe";
}

.ti-barrier-block:before {
  content: "\f00e";
}

.ti-barrier-block-filled:before {
  content: "\fe8e";
}

.ti-barrier-block-off:before {
  content: "\f0b5";
}

.ti-baseline:before {
  content: "\f024";
}

.ti-baseline-density-large:before {
  content: "\f9f0";
}

.ti-baseline-density-medium:before {
  content: "\f9f1";
}

.ti-baseline-density-small:before {
  content: "\f9f2";
}

.ti-basket:before {
  content: "\ebe1";
}

.ti-basket-bolt:before {
  content: "\fb43";
}

.ti-basket-cancel:before {
  content: "\fb44";
}

.ti-basket-check:before {
  content: "\fb45";
}

.ti-basket-code:before {
  content: "\fb46";
}

.ti-basket-cog:before {
  content: "\fb47";
}

.ti-basket-discount:before {
  content: "\fb48";
}

.ti-basket-dollar:before {
  content: "\fb49";
}

.ti-basket-down:before {
  content: "\fb4a";
}

.ti-basket-exclamation:before {
  content: "\fb4b";
}

.ti-basket-filled:before {
  content: "\f7df";
}

.ti-basket-heart:before {
  content: "\fb4c";
}

.ti-basket-minus:before {
  content: "\fb4d";
}

.ti-basket-off:before {
  content: "\f0b6";
}

.ti-basket-pause:before {
  content: "\fb4e";
}

.ti-basket-pin:before {
  content: "\fb4f";
}

.ti-basket-plus:before {
  content: "\fb50";
}

.ti-basket-question:before {
  content: "\fb51";
}

.ti-basket-search:before {
  content: "\fb52";
}

.ti-basket-share:before {
  content: "\fb53";
}

.ti-basket-star:before {
  content: "\fb54";
}

.ti-basket-up:before {
  content: "\fb55";
}

.ti-basket-x:before {
  content: "\fb56";
}

.ti-bat:before {
  content: "\f284";
}

.ti-bath:before {
  content: "\ef48";
}

.ti-bath-filled:before {
  content: "\f71d";
}

.ti-bath-off:before {
  content: "\f0ff";
}

.ti-battery:before {
  content: "\ea34";
}

.ti-battery-1:before {
  content: "\ea2f";
}

.ti-battery-1-filled:before {
  content: "\f71e";
}

.ti-battery-2:before {
  content: "\ea30";
}

.ti-battery-2-filled:before {
  content: "\f71f";
}

.ti-battery-3:before {
  content: "\ea31";
}

.ti-battery-3-filled:before {
  content: "\f720";
}

.ti-battery-4:before {
  content: "\ea32";
}

.ti-battery-4-filled:before {
  content: "\f721";
}

.ti-battery-automotive:before {
  content: "\ee07";
}

.ti-battery-automotive-filled:before {
  content: "\10029";
}

.ti-battery-charging:before {
  content: "\ea33";
}

.ti-battery-charging-2:before {
  content: "\ef3b";
}

.ti-battery-eco:before {
  content: "\ef3c";
}

.ti-battery-exclamation:before {
  content: "\ff1d";
}

.ti-battery-filled:before {
  content: "\f668";
}

.ti-battery-off:before {
  content: "\ed1c";
}

.ti-battery-spark:before {
  content: "\ffbd";
}

.ti-battery-vertical:before {
  content: "\ff13";
}

.ti-battery-vertical-1:before {
  content: "\ff1c";
}

.ti-battery-vertical-1-filled:before {
  content: "\10028";
}

.ti-battery-vertical-2:before {
  content: "\ff1b";
}

.ti-battery-vertical-2-filled:before {
  content: "\10027";
}

.ti-battery-vertical-3:before {
  content: "\ff1a";
}

.ti-battery-vertical-3-filled:before {
  content: "\10026";
}

.ti-battery-vertical-4:before {
  content: "\ff19";
}

.ti-battery-vertical-4-filled:before {
  content: "\10025";
}

.ti-battery-vertical-charging:before {
  content: "\ff17";
}

.ti-battery-vertical-charging-2:before {
  content: "\ff18";
}

.ti-battery-vertical-eco:before {
  content: "\ff16";
}

.ti-battery-vertical-exclamation:before {
  content: "\ff15";
}

.ti-battery-vertical-filled:before {
  content: "\10024";
}

.ti-battery-vertical-off:before {
  content: "\ff14";
}

.ti-beach:before {
  content: "\ef3d";
}

.ti-beach-off:before {
  content: "\f0b7";
}

.ti-bed:before {
  content: "\eb5c";
}

.ti-bed-filled:before {
  content: "\f7e0";
}

.ti-bed-flat:before {
  content: "\fca1";
}

.ti-bed-flat-filled:before {
  content: "\fe8d";
}

.ti-bed-off:before {
  content: "\f100";
}

.ti-beer:before {
  content: "\efa1";
}

.ti-beer-filled:before {
  content: "\f7e1";
}

.ti-beer-off:before {
  content: "\f101";
}

.ti-bell:before {
  content: "\ea35";
}

.ti-bell-bolt:before {
  content: "\f812";
}

.ti-bell-cancel:before {
  content: "\f813";
}

.ti-bell-check:before {
  content: "\f814";
}

.ti-bell-code:before {
  content: "\f815";
}

.ti-bell-cog:before {
  content: "\f816";
}

.ti-bell-dollar:before {
  content: "\f817";
}

.ti-bell-down:before {
  content: "\f818";
}

.ti-bell-exclamation:before {
  content: "\f819";
}

.ti-bell-filled:before {
  content: "\f669";
}

.ti-bell-heart:before {
  content: "\f81a";
}

.ti-bell-minus:before {
  content: "\ede2";
}

.ti-bell-minus-filled:before {
  content: "\f722";
}

.ti-bell-off:before {
  content: "\ece9";
}

.ti-bell-pause:before {
  content: "\f81b";
}

.ti-bell-pin:before {
  content: "\f81c";
}

.ti-bell-plus:before {
  content: "\ede3";
}

.ti-bell-plus-filled:before {
  content: "\f723";
}

.ti-bell-question:before {
  content: "\f81d";
}

.ti-bell-ringing:before {
  content: "\ed07";
}

.ti-bell-ringing-2:before {
  content: "\ede4";
}

.ti-bell-ringing-2-filled:before {
  content: "\f724";
}

.ti-bell-ringing-filled:before {
  content: "\f725";
}

.ti-bell-school:before {
  content: "\f05e";
}

.ti-bell-search:before {
  content: "\f81e";
}

.ti-bell-share:before {
  content: "\f81f";
}

.ti-bell-star:before {
  content: "\f820";
}

.ti-bell-up:before {
  content: "\f821";
}

.ti-bell-x:before {
  content: "\ede5";
}

.ti-bell-x-filled:before {
  content: "\f726";
}

.ti-bell-z:before {
  content: "\eff1";
}

.ti-bell-z-filled:before {
  content: "\f727";
}

.ti-beta:before {
  content: "\f544";
}

.ti-bible:before {
  content: "\efc4";
}

.ti-bike:before {
  content: "\ea36";
}

.ti-bike-filled:before {
  content: "\10023";
}

.ti-bike-off:before {
  content: "\f0b8";
}

.ti-binary:before {
  content: "\ee08";
}

.ti-binary-off:before {
  content: "\f3c5";
}

.ti-binary-tree:before {
  content: "\f5d4";
}

.ti-binary-tree-2:before {
  content: "\f5d3";
}

.ti-binary-tree-2-filled:before {
  content: "\ff65";
}

.ti-binary-tree-filled:before {
  content: "\ff64";
}

.ti-binoculars:before {
  content: "\fefe";
}

.ti-binoculars-filled:before {
  content: "\ff0b";
}

.ti-biohazard:before {
  content: "\ecb8";
}

.ti-biohazard-filled:before {
  content: "\fe8c";
}

.ti-biohazard-off:before {
  content: "\f0b9";
}

.ti-blade:before {
  content: "\f4bd";
}

.ti-blade-filled:before {
  content: "\f7e2";
}

.ti-bleach:before {
  content: "\f2f3";
}

.ti-bleach-chlorine:before {
  content: "\f2f0";
}

.ti-bleach-no-chlorine:before {
  content: "\f2f1";
}

.ti-bleach-off:before {
  content: "\f2f2";
}

.ti-blend-mode:before {
  content: "\feb0";
}

.ti-blender:before {
  content: "\fca2";
}

.ti-blender-filled:before {
  content: "\10022";
}

.ti-blob:before {
  content: "\feaf";
}

.ti-blob-filled:before {
  content: "\feb1";
}

.ti-blockquote:before {
  content: "\ee09";
}

.ti-blocks:before {
  content: "\100b2";
}

.ti-bluetooth:before {
  content: "\ea37";
}

.ti-bluetooth-connected:before {
  content: "\ecea";
}

.ti-bluetooth-off:before {
  content: "\eceb";
}

.ti-bluetooth-x:before {
  content: "\f081";
}

.ti-blur:before {
  content: "\ef8c";
}

.ti-blur-off:before {
  content: "\f3c6";
}

.ti-bmp:before {
  content: "\f3a6";
}

.ti-body-scan:before {
  content: "\fca3";
}

.ti-bold:before {
  content: "\eb7b";
}

.ti-bold-off:before {
  content: "\f0ba";
}

.ti-bolt:before {
  content: "\ea38";
}

.ti-bolt-filled:before {
  content: "\10021";
}

.ti-bolt-off:before {
  content: "\ecec";
}

.ti-bomb:before {
  content: "\f59c";
}

.ti-bomb-filled:before {
  content: "\fa86";
}

.ti-bone:before {
  content: "\edb8";
}

.ti-bone-filled:before {
  content: "\fe8b";
}

.ti-bone-off:before {
  content: "\f0bb";
}

.ti-bong:before {
  content: "\f3a7";
}

.ti-bong-filled:before {
  content: "\10020";
}

.ti-bong-off:before {
  content: "\f3c7";
}

.ti-book:before {
  content: "\ea39";
}

.ti-book-2:before {
  content: "\efc5";
}

.ti-book-download:before {
  content: "\f070";
}

.ti-book-filled:before {
  content: "\fa87";
}

.ti-book-off:before {
  content: "\f0bc";
}

.ti-book-upload:before {
  content: "\f071";
}

.ti-bookmark:before {
  content: "\ea3a";
}

.ti-bookmark-ai:before {
  content: "\fc8a";
}

.ti-bookmark-edit:before {
  content: "\fa5e";
}

.ti-bookmark-filled:before {
  content: "\fa88";
}

.ti-bookmark-minus:before {
  content: "\fa5f";
}

.ti-bookmark-off:before {
  content: "\eced";
}

.ti-bookmark-plus:before {
  content: "\fa60";
}

.ti-bookmark-question:before {
  content: "\fa61";
}

.ti-bookmarks:before {
  content: "\ed08";
}

.ti-bookmarks-filled:before {
  content: "\fb1f";
}

.ti-bookmarks-off:before {
  content: "\f0bd";
}

.ti-books:before {
  content: "\eff2";
}

.ti-books-off:before {
  content: "\f0be";
}

.ti-boom:before {
  content: "\fdbe";
}

.ti-boom-filled:before {
  content: "\fe8a";
}

.ti-border-all:before {
  content: "\ea3b";
}

.ti-border-bottom:before {
  content: "\ea3c";
}

.ti-border-bottom-plus:before {
  content: "\fdbd";
}

.ti-border-corner-ios:before {
  content: "\fd98";
}

.ti-border-corner-pill:before {
  content: "\fd62";
}

.ti-border-corner-rounded:before {
  content: "\fd63";
}

.ti-border-corner-square:before {
  content: "\fd64";
}

.ti-border-corners:before {
  content: "\f7a0";
}

.ti-border-horizontal:before {
  content: "\ea3d";
}

.ti-border-inner:before {
  content: "\ea3e";
}

.ti-border-left:before {
  content: "\ea3f";
}

.ti-border-left-plus:before {
  content: "\fdbc";
}

.ti-border-none:before {
  content: "\ea40";
}

.ti-border-outer:before {
  content: "\ea41";
}

.ti-border-radius:before {
  content: "\eb7c";
}

.ti-border-right:before {
  content: "\ea42";
}

.ti-border-right-plus:before {
  content: "\fdbb";
}

.ti-border-sides:before {
  content: "\f7a1";
}

.ti-border-style:before {
  content: "\ee0a";
}

.ti-border-style-2:before {
  content: "\ef22";
}

.ti-border-top:before {
  content: "\ea43";
}

.ti-border-top-plus:before {
  content: "\fdba";
}

.ti-border-vertical:before {
  content: "\ea44";
}

.ti-bottle:before {
  content: "\ef0b";
}

.ti-bottle-filled:before {
  content: "\fa89";
}

.ti-bottle-off:before {
  content: "\f3c8";
}

.ti-bounce-left:before {
  content: "\f59d";
}

.ti-bounce-left-filled:before {
  content: "\fb20";
}

.ti-bounce-right:before {
  content: "\f59e";
}

.ti-bounce-right-filled:before {
  content: "\fb21";
}

.ti-bow:before {
  content: "\f096";
}

.ti-bow-filled:before {
  content: "\fe89";
}

.ti-bowl:before {
  content: "\f4fa";
}

.ti-bowl-chopsticks:before {
  content: "\fd90";
}

.ti-bowl-chopsticks-filled:before {
  content: "\fe88";
}

.ti-bowl-filled:before {
  content: "\fb22";
}

.ti-bowl-spoon:before {
  content: "\fd91";
}

.ti-bowl-spoon-filled:before {
  content: "\fe87";
}

.ti-bowling:before {
  content: "\100b1";
}

.ti-box:before {
  content: "\ea45";
}

.ti-box-align-bottom:before {
  content: "\f2a8";
}

.ti-box-align-bottom-filled:before {
  content: "\fa8a";
}

.ti-box-align-bottom-left:before {
  content: "\f2ce";
}

.ti-box-align-bottom-left-filled:before {
  content: "\fa8b";
}

.ti-box-align-bottom-right:before {
  content: "\f2cf";
}

.ti-box-align-bottom-right-filled:before {
  content: "\fa8c";
}

.ti-box-align-left:before {
  content: "\f2a9";
}

.ti-box-align-left-filled:before {
  content: "\fa8d";
}

.ti-box-align-right:before {
  content: "\f2aa";
}

.ti-box-align-right-filled:before {
  content: "\fa8e";
}

.ti-box-align-top:before {
  content: "\f2ab";
}

.ti-box-align-top-filled:before {
  content: "\fa8f";
}

.ti-box-align-top-left:before {
  content: "\f2d0";
}

.ti-box-align-top-left-filled:before {
  content: "\fa90";
}

.ti-box-align-top-right:before {
  content: "\f2d1";
}

.ti-box-align-top-right-filled:before {
  content: "\fa91";
}

.ti-box-margin:before {
  content: "\ee0b";
}

.ti-box-model:before {
  content: "\ee0c";
}

.ti-box-model-2:before {
  content: "\ef23";
}

.ti-box-model-2-off:before {
  content: "\f3c9";
}

.ti-box-model-off:before {
  content: "\f3ca";
}

.ti-box-multiple:before {
  content: "\ee17";
}

.ti-box-multiple-0:before {
  content: "\ee0d";
}

.ti-box-multiple-1:before {
  content: "\ee0e";
}

.ti-box-multiple-2:before {
  content: "\ee0f";
}

.ti-box-multiple-3:before {
  content: "\ee10";
}

.ti-box-multiple-4:before {
  content: "\ee11";
}

.ti-box-multiple-5:before {
  content: "\ee12";
}

.ti-box-multiple-6:before {
  content: "\ee13";
}

.ti-box-multiple-7:before {
  content: "\ee14";
}

.ti-box-multiple-8:before {
  content: "\ee15";
}

.ti-box-multiple-9:before {
  content: "\ee16";
}

.ti-box-multiple-filled:before {
  content: "\1001f";
}

.ti-box-off:before {
  content: "\f102";
}

.ti-box-padding:before {
  content: "\ee18";
}

.ti-braces:before {
  content: "\ebcc";
}

.ti-braces-off:before {
  content: "\f0bf";
}

.ti-brackets:before {
  content: "\ebcd";
}

.ti-brackets-angle:before {
  content: "\fcb2";
}

.ti-brackets-angle-off:before {
  content: "\fcb1";
}

.ti-brackets-contain:before {
  content: "\f1e5";
}

.ti-brackets-contain-end:before {
  content: "\f1e3";
}

.ti-brackets-contain-start:before {
  content: "\f1e4";
}

.ti-brackets-off:before {
  content: "\f0c0";
}

.ti-braille:before {
  content: "\f545";
}

.ti-brain:before {
  content: "\f59f";
}

.ti-brand-4chan:before {
  content: "\f494";
}

.ti-brand-abstract:before {
  content: "\f495";
}

.ti-brand-adobe:before {
  content: "\f0dc";
}

.ti-brand-adobe-after-effect:before {
  content: "\ff2a";
}

.ti-brand-adobe-illustrator:before {
  content: "\ff29";
}

.ti-brand-adobe-indesign:before {
  content: "\ff28";
}

.ti-brand-adobe-photoshop:before {
  content: "\ff27";
}

.ti-brand-adobe-premier:before {
  content: "\ff26";
}

.ti-brand-adobe-xd:before {
  content: "\ff25";
}

.ti-brand-adonis-js:before {
  content: "\f496";
}

.ti-brand-airbnb:before {
  content: "\ed68";
}

.ti-brand-airtable:before {
  content: "\ef6a";
}

.ti-brand-algolia:before {
  content: "\f390";
}

.ti-brand-alipay:before {
  content: "\f7a2";
}

.ti-brand-alpine-js:before {
  content: "\f324";
}

.ti-brand-amazon:before {
  content: "\f230";
}

.ti-brand-amd:before {
  content: "\f653";
}

.ti-brand-amie:before {
  content: "\ffab";
}

.ti-brand-amigo:before {
  content: "\f5f9";
}

.ti-brand-among-us:before {
  content: "\f205";
}

.ti-brand-android:before {
  content: "\ec16";
}

.ti-brand-angular:before {
  content: "\ef6b";
}

.ti-brand-angular-filled:before {
  content: "\10095";
}

.ti-brand-ansible:before {
  content: "\fa70";
}

.ti-brand-ao3:before {
  content: "\f5e8";
}

.ti-brand-appgallery:before {
  content: "\f231";
}

.ti-brand-apple:before {
  content: "\ec17";
}

.ti-brand-apple-arcade:before {
  content: "\ed69";
}

.ti-brand-apple-filled:before {
  content: "\fd74";
}

.ti-brand-apple-news:before {
  content: "\ff24";
}

.ti-brand-apple-podcast:before {
  content: "\f1e6";
}

.ti-brand-appstore:before {
  content: "\ed24";
}

.ti-brand-arc:before {
  content: "\feae";
}

.ti-brand-asana:before {
  content: "\edc5";
}

.ti-brand-astro:before {
  content: "\fdb9";
}

.ti-brand-auth0:before {
  content: "\fcb3";
}

.ti-brand-aws:before {
  content: "\fa4c";
}

.ti-brand-azure:before {
  content: "\fa4d";
}

.ti-brand-backbone:before {
  content: "\f325";
}

.ti-brand-badoo:before {
  content: "\f206";
}

.ti-brand-baidu:before {
  content: "\f5e9";
}

.ti-brand-bandcamp:before {
  content: "\f207";
}

.ti-brand-bandlab:before {
  content: "\f5fa";
}

.ti-brand-beats:before {
  content: "\f208";
}

.ti-brand-bebo:before {
  content: "\ffaa";
}

.ti-brand-behance:before {
  content: "\ec6e";
}

.ti-brand-bilibili:before {
  content: "\f6d2";
}

.ti-brand-binance:before {
  content: "\f5a0";
}

.ti-brand-bing:before {
  content: "\edc6";
}

.ti-brand-bitbucket:before {
  content: "\edc7";
}

.ti-brand-bitbucket-filled:before {
  content: "\100cb";
}

.ti-brand-blackberry:before {
  content: "\f568";
}

.ti-brand-blender:before {
  content: "\f326";
}

.ti-brand-blogger:before {
  content: "\f35a";
}

.ti-brand-bluesky:before {
  content: "\fd75";
}

.ti-brand-booking:before {
  content: "\edc8";
}

.ti-brand-bootstrap:before {
  content: "\ef3e";
}

.ti-brand-bulma:before {
  content: "\f327";
}

.ti-brand-bumble:before {
  content: "\f5fb";
}

.ti-brand-bunpo:before {
  content: "\f4cf";
}

.ti-brand-c-sharp:before {
  content: "\f003";
}

.ti-brand-cake:before {
  content: "\f7a3";
}

.ti-brand-cakephp:before {
  content: "\f7af";
}

.ti-brand-campaignmonitor:before {
  content: "\f328";
}

.ti-brand-carbon:before {
  content: "\f348";
}

.ti-brand-cashapp:before {
  content: "\f391";
}

.ti-brand-chrome:before {
  content: "\ec18";
}

.ti-brand-cinema-4d:before {
  content: "\fa71";
}

.ti-brand-citymapper:before {
  content: "\f5fc";
}

.ti-brand-cloudflare:before {
  content: "\fa4e";
}

.ti-brand-codecov:before {
  content: "\f329";
}

.ti-brand-codepen:before {
  content: "\ec6f";
}

.ti-brand-codesandbox:before {
  content: "\ed6a";
}

.ti-brand-cohost:before {
  content: "\f5d5";
}

.ti-brand-coinbase:before {
  content: "\f209";
}

.ti-brand-comedy-central:before {
  content: "\f217";
}

.ti-brand-coreos:before {
  content: "\f5fd";
}

.ti-brand-couchdb:before {
  content: "\f60f";
}

.ti-brand-couchsurfing:before {
  content: "\f392";
}

.ti-brand-cpp:before {
  content: "\f5fe";
}

.ti-brand-craft:before {
  content: "\fa72";
}

.ti-brand-crunchbase:before {
  content: "\f7e3";
}

.ti-brand-css3:before {
  content: "\ed6b";
}

.ti-brand-ctemplar:before {
  content: "\f4d0";
}

.ti-brand-cucumber:before {
  content: "\ef6c";
}

.ti-brand-cupra:before {
  content: "\f4d1";
}

.ti-brand-cypress:before {
  content: "\f333";
}

.ti-brand-d3:before {
  content: "\f24e";
}

.ti-brand-databricks:before {
  content: "\fc41";
}

.ti-brand-days-counter:before {
  content: "\f4d2";
}

.ti-brand-dcos:before {
  content: "\f32a";
}

.ti-brand-debian:before {
  content: "\ef57";
}

.ti-brand-deezer:before {
  content: "\f78b";
}

.ti-brand-deliveroo:before {
  content: "\f4d3";
}

.ti-brand-deno:before {
  content: "\f24f";
}

.ti-brand-denodo:before {
  content: "\f610";
}

.ti-brand-deviantart:before {
  content: "\ecfb";
}

.ti-brand-digg:before {
  content: "\fa73";
}

.ti-brand-dingtalk:before {
  content: "\f5ea";
}

.ti-brand-discord:before {
  content: "\ece3";
}

.ti-brand-discord-filled:before {
  content: "\f7e4";
}

.ti-brand-disney:before {
  content: "\f20a";
}

.ti-brand-disqus:before {
  content: "\edc9";
}

.ti-brand-django:before {
  content: "\f349";
}

.ti-brand-docker:before {
  content: "\edca";
}

.ti-brand-doctrine:before {
  content: "\ef6d";
}

.ti-brand-dolby-digital:before {
  content: "\f4d4";
}

.ti-brand-douban:before {
  content: "\f5ff";
}

.ti-brand-dribbble:before {
  content: "\ec19";
}

.ti-brand-dribbble-filled:before {
  content: "\f7e5";
}

.ti-brand-dropbox:before {
  content: "\1018a";
}

.ti-brand-drops:before {
  content: "\f4d5";
}

.ti-brand-drupal:before {
  content: "\f393";
}

.ti-brand-edge:before {
  content: "\ecfc";
}

.ti-brand-elastic:before {
  content: "\f611";
}

.ti-brand-electronic-arts:before {
  content: "\fa74";
}

.ti-brand-ember:before {
  content: "\f497";
}

.ti-brand-envato:before {
  content: "\f394";
}

.ti-brand-etsy:before {
  content: "\f654";
}

.ti-brand-evernote:before {
  content: "\f600";
}

.ti-brand-facebook:before {
  content: "\ec1a";
}

.ti-brand-facebook-filled:before {
  content: "\f7e6";
}

.ti-brand-feedly:before {
  content: "\fa75";
}

.ti-brand-figma:before {
  content: "\ec93";
}

.ti-brand-filezilla:before {
  content: "\fa76";
}

.ti-brand-finder:before {
  content: "\f218";
}

.ti-brand-firebase:before {
  content: "\ef6e";
}

.ti-brand-firefox:before {
  content: "\ecfd";
}

.ti-brand-fiverr:before {
  content: "\f7a4";
}

.ti-brand-flickr:before {
  content: "\ecfe";
}

.ti-brand-flightradar24:before {
  content: "\f4d6";
}

.ti-brand-flipboard:before {
  content: "\f20b";
}

.ti-brand-flutter:before {
  content: "\f395";
}

.ti-brand-fortnite:before {
  content: "\f260";
}

.ti-brand-foursquare:before {
  content: "\ecff";
}

.ti-brand-framer:before {
  content: "\ec1b";
}

.ti-brand-framer-motion:before {
  content: "\f78c";
}

.ti-brand-funimation:before {
  content: "\f655";
}

.ti-brand-gatsby:before {
  content: "\f396";
}

.ti-brand-git:before {
  content: "\ef6f";
}

.ti-brand-github:before {
  content: "\ec1c";
}

.ti-brand-github-copilot:before {
  content: "\f4a8";
}

.ti-brand-github-filled:before {
  content: "\f7e7";
}

.ti-brand-gitlab:before {
  content: "\ec1d";
}

.ti-brand-gmail:before {
  content: "\efa2";
}

.ti-brand-golang:before {
  content: "\f78d";
}

.ti-brand-google:before {
  content: "\ec1f";
}

.ti-brand-google-analytics:before {
  content: "\edcb";
}

.ti-brand-google-big-query:before {
  content: "\f612";
}

.ti-brand-google-drive:before {
  content: "\ec1e";
}

.ti-brand-google-filled:before {
  content: "\fd1a";
}

.ti-brand-google-fit:before {
  content: "\f297";
}

.ti-brand-google-home:before {
  content: "\f601";
}

.ti-brand-google-maps:before {
  content: "\fa4f";
}

.ti-brand-google-one:before {
  content: "\f232";
}

.ti-brand-google-photos:before {
  content: "\f20c";
}

.ti-brand-google-play:before {
  content: "\ed25";
}

.ti-brand-google-podcasts:before {
  content: "\f656";
}

.ti-brand-grammarly:before {
  content: "\f32b";
}

.ti-brand-graphql:before {
  content: "\f32c";
}

.ti-brand-gravatar:before {
  content: "\edcc";
}

.ti-brand-grindr:before {
  content: "\f20d";
}

.ti-brand-guardian:before {
  content: "\f4fb";
}

.ti-brand-gumroad:before {
  content: "\f5d6";
}

.ti-brand-hackerrank:before {
  content: "\ff23";
}

.ti-brand-hbo:before {
  content: "\f657";
}

.ti-brand-headlessui:before {
  content: "\f32d";
}

.ti-brand-hexo:before {
  content: "\fa50";
}

.ti-brand-hipchat:before {
  content: "\edcd";
}

.ti-brand-html5:before {
  content: "\ed6c";
}

.ti-brand-inertia:before {
  content: "\f34a";
}

.ti-brand-instagram:before {
  content: "\ec20";
}

.ti-brand-instagram-filled:before {
  content: "\10094";
}

.ti-brand-intercom:before {
  content: "\f1cf";
}

.ti-brand-itch:before {
  content: "\fa22";
}

.ti-brand-javascript:before {
  content: "\ef0c";
}

.ti-brand-juejin:before {
  content: "\f7b0";
}

.ti-brand-kako-talk:before {
  content: "\fd2d";
}

.ti-brand-kbin:before {
  content: "\fad0";
}

.ti-brand-kick:before {
  content: "\fa23";
}

.ti-brand-kick-filled:before {
  content: "\10093";
}

.ti-brand-kickstarter:before {
  content: "\edce";
}

.ti-brand-kotlin:before {
  content: "\ed6d";
}

.ti-brand-laravel:before {
  content: "\f34b";
}

.ti-brand-lastfm:before {
  content: "\f001";
}

.ti-brand-leetcode:before {
  content: "\fa51";
}

.ti-brand-letterboxd:before {
  content: "\fa24";
}

.ti-brand-line:before {
  content: "\f7e8";
}

.ti-brand-linkedin:before {
  content: "\ec8c";
}

.ti-brand-linkedin-filled:before {
  content: "\10092";
}

.ti-brand-linktree:before {
  content: "\f1e7";
}

.ti-brand-linqpad:before {
  content: "\f562";
}

.ti-brand-livewire:before {
  content: "\fd76";
}

.ti-brand-loom:before {
  content: "\ef70";
}

.ti-brand-mailgun:before {
  content: "\f32e";
}

.ti-brand-mantine:before {
  content: "\f32f";
}

.ti-brand-mastercard:before {
  content: "\ef49";
}

.ti-brand-mastodon:before {
  content: "\f250";
}

.ti-brand-matrix:before {
  content: "\f5eb";
}

.ti-brand-mcdonalds:before {
  content: "\f251";
}

.ti-brand-medium:before {
  content: "\ec70";
}

.ti-brand-meetup:before {
  content: "\fc6a";
}

.ti-brand-mercedes:before {
  content: "\f072";
}

.ti-brand-messenger:before {
  content: "\ec71";
}

.ti-brand-messenger-filled:before {
  content: "\100a7";
}

.ti-brand-meta:before {
  content: "\efb0";
}

.ti-brand-metabrainz:before {
  content: "\ff12";
}

.ti-brand-minecraft:before {
  content: "\faef";
}

.ti-brand-miniprogram:before {
  content: "\f602";
}

.ti-brand-mixpanel:before {
  content: "\f397";
}

.ti-brand-monday:before {
  content: "\f219";
}

.ti-brand-mongodb:before {
  content: "\f613";
}

.ti-brand-my-oppo:before {
  content: "\f4d7";
}

.ti-brand-mysql:before {
  content: "\f614";
}

.ti-brand-national-geographic:before {
  content: "\f603";
}

.ti-brand-nem:before {
  content: "\f5a1";
}

.ti-brand-netbeans:before {
  content: "\ef71";
}

.ti-brand-netease-music:before {
  content: "\f604";
}

.ti-brand-netflix:before {
  content: "\edcf";
}

.ti-brand-nexo:before {
  content: "\f5a2";
}

.ti-brand-nextcloud:before {
  content: "\f4d8";
}

.ti-brand-nextjs:before {
  content: "\f0dd";
}

.ti-brand-nodejs:before {
  content: "\fae0";
}

.ti-brand-nord-vpn:before {
  content: "\f37f";
}

.ti-brand-notion:before {
  content: "\ef7b";
}

.ti-brand-npm:before {
  content: "\f569";
}

.ti-brand-nuxt:before {
  content: "\f0de";
}

.ti-brand-nytimes:before {
  content: "\ef8d";
}

.ti-brand-oauth:before {
  content: "\fa52";
}

.ti-brand-office:before {
  content: "\f398";
}

.ti-brand-ok-ru:before {
  content: "\f399";
}

.ti-brand-onedrive:before {
  content: "\f5d7";
}

.ti-brand-onlyfans:before {
  content: "\f605";
}

.ti-brand-open-source:before {
  content: "\edd0";
}

.ti-brand-open-source-filled:before {
  content: "\10091";
}

.ti-brand-openai:before {
  content: "\f78e";
}

.ti-brand-openvpn:before {
  content: "\f39a";
}

.ti-brand-opera:before {
  content: "\ec21";
}

.ti-brand-opera-filled:before {
  content: "\10090";
}

.ti-brand-pagekit:before {
  content: "\edd1";
}

.ti-brand-parsinta:before {
  content: "\fc42";
}

.ti-brand-patreon:before {
  content: "\edd2";
}

.ti-brand-patreon-filled:before {
  content: "\fcff";
}

.ti-brand-paypal:before {
  content: "\ec22";
}

.ti-brand-paypal-filled:before {
  content: "\f7e9";
}

.ti-brand-paypay:before {
  content: "\f5ec";
}

.ti-brand-peanut:before {
  content: "\f39b";
}

.ti-brand-pepsi:before {
  content: "\f261";
}

.ti-brand-php:before {
  content: "\ef72";
}

.ti-brand-picsart:before {
  content: "\f4d9";
}

.ti-brand-pinterest:before {
  content: "\ec8d";
}

.ti-brand-pinterest-filled:before {
  content: "\1008f";
}

.ti-brand-planetscale:before {
  content: "\f78f";
}

.ti-brand-pnpm:before {
  content: "\fd77";
}

.ti-brand-pocket:before {
  content: "\ed00";
}

.ti-brand-polymer:before {
  content: "\f498";
}

.ti-brand-powershell:before {
  content: "\f5ed";
}

.ti-brand-printables:before {
  content: "\fd1b";
}

.ti-brand-prisma:before {
  content: "\f499";
}

.ti-brand-producthunt:before {
  content: "\edd3";
}

.ti-brand-pushbullet:before {
  content: "\f330";
}

.ti-brand-pushover:before {
  content: "\f20e";
}

.ti-brand-python:before {
  content: "\ed01";
}

.ti-brand-qq:before {
  content: "\f606";
}

.ti-brand-radix-ui:before {
  content: "\f790";
}

.ti-brand-react:before {
  content: "\f34c";
}

.ti-brand-react-native:before {
  content: "\ef73";
}

.ti-brand-reason:before {
  content: "\f49a";
}

.ti-brand-reddit:before {
  content: "\ec8e";
}

.ti-brand-redhat:before {
  content: "\f331";
}

.ti-brand-redux:before {
  content: "\f3a8";
}

.ti-brand-revolut:before {
  content: "\f4da";
}

.ti-brand-rumble:before {
  content: "\fad1";
}

.ti-brand-rust:before {
  content: "\fa53";
}

.ti-brand-safari:before {
  content: "\ec23";
}

.ti-brand-samsungpass:before {
  content: "\f4db";
}

.ti-brand-sass:before {
  content: "\edd4";
}

.ti-brand-sentry:before {
  content: "\edd5";
}

.ti-brand-sharik:before {
  content: "\f4dc";
}

.ti-brand-shazam:before {
  content: "\edd6";
}

.ti-brand-shopee:before {
  content: "\f252";
}

.ti-brand-sketch:before {
  content: "\ec24";
}

.ti-brand-sketch-filled:before {
  content: "\1008e";
}

.ti-brand-skype:before {
  content: "\ed02";
}

.ti-brand-slack:before {
  content: "\ec72";
}

.ti-brand-snapchat:before {
  content: "\ec25";
}

.ti-brand-snapchat-filled:before {
  content: "\1008d";
}

.ti-brand-snapseed:before {
  content: "\f253";
}

.ti-brand-snowflake:before {
  content: "\f615";
}

.ti-brand-socket-io:before {
  content: "\f49b";
}

.ti-brand-solidjs:before {
  content: "\f5ee";
}

.ti-brand-soundcloud:before {
  content: "\ed6e";
}

.ti-brand-spacehey:before {
  content: "\f4fc";
}

.ti-brand-speedtest:before {
  content: "\fa77";
}

.ti-brand-spotify:before {
  content: "\ed03";
}

.ti-brand-spotify-filled:before {
  content: "\fe86";
}

.ti-brand-stackoverflow:before {
  content: "\ef58";
}

.ti-brand-stackshare:before {
  content: "\f607";
}

.ti-brand-steam:before {
  content: "\ed6f";
}

.ti-brand-steam-filled:before {
  content: "\1008c";
}

.ti-brand-stocktwits:before {
  content: "\fd78";
}

.ti-brand-storj:before {
  content: "\fa54";
}

.ti-brand-storybook:before {
  content: "\f332";
}

.ti-brand-storytel:before {
  content: "\f608";
}

.ti-brand-strava:before {
  content: "\f254";
}

.ti-brand-stripe:before {
  content: "\edd7";
}

.ti-brand-stripe-filled:before {
  content: "\1008b";
}

.ti-brand-sublime-text:before {
  content: "\ef74";
}

.ti-brand-sugarizer:before {
  content: "\f7a5";
}

.ti-brand-supabase:before {
  content: "\f6d3";
}

.ti-brand-superhuman:before {
  content: "\f50c";
}

.ti-brand-supernova:before {
  content: "\f49c";
}

.ti-brand-surfshark:before {
  content: "\f255";
}

.ti-brand-svelte:before {
  content: "\f0df";
}

.ti-brand-swift:before {
  content: "\fa55";
}

.ti-brand-symfony:before {
  content: "\f616";
}

.ti-brand-tabler:before {
  content: "\ec8f";
}

.ti-brand-tabler-filled:before {
  content: "\1008a";
}

.ti-brand-tailwind:before {
  content: "\eca1";
}

.ti-brand-taobao:before {
  content: "\f5ef";
}

.ti-brand-teams:before {
  content: "\fadf";
}

.ti-brand-ted:before {
  content: "\f658";
}

.ti-brand-telegram:before {
  content: "\ec26";
}

.ti-brand-terraform:before {
  content: "\fa56";
}

.ti-brand-tesla:before {
  content: "\10099";
}

.ti-brand-tether:before {
  content: "\f5a3";
}

.ti-brand-thingiverse:before {
  content: "\fd1c";
}

.ti-brand-threads:before {
  content: "\fb02";
}

.ti-brand-threejs:before {
  content: "\f5f0";
}

.ti-brand-tidal:before {
  content: "\ed70";
}

.ti-brand-tiktok:before {
  content: "\ec73";
}

.ti-brand-tiktok-filled:before {
  content: "\f7ea";
}

.ti-brand-tinder:before {
  content: "\ed71";
}

.ti-brand-tinder-filled:before {
  content: "\10089";
}

.ti-brand-topbuzz:before {
  content: "\f50d";
}

.ti-brand-torchain:before {
  content: "\f5a4";
}

.ti-brand-toyota:before {
  content: "\f262";
}

.ti-brand-trello:before {
  content: "\f39d";
}

.ti-brand-tripadvisor:before {
  content: "\f002";
}

.ti-brand-tumblr:before {
  content: "\ed04";
}

.ti-brand-tumblr-filled:before {
  content: "\10088";
}

.ti-brand-twilio:before {
  content: "\f617";
}

.ti-brand-twitch:before {
  content: "\ed05";
}

.ti-brand-twitter:before {
  content: "\ec27";
}

.ti-brand-twitter-filled:before {
  content: "\f7eb";
}

.ti-brand-typescript:before {
  content: "\f5f1";
}

.ti-brand-uber:before {
  content: "\ef75";
}

.ti-brand-ubuntu:before {
  content: "\ef59";
}

.ti-brand-unity:before {
  content: "\f49d";
}

.ti-brand-unsplash:before {
  content: "\edd8";
}

.ti-brand-upwork:before {
  content: "\f39e";
}

.ti-brand-valorant:before {
  content: "\f39f";
}

.ti-brand-vercel:before {
  content: "\ef24";
}

.ti-brand-vercel-filled:before {
  content: "\10087";
}

.ti-brand-vimeo:before {
  content: "\ed06";
}

.ti-brand-vimeo-filled:before {
  content: "\10086";
}

.ti-brand-vinted:before {
  content: "\f20f";
}

.ti-brand-visa:before {
  content: "\f380";
}

.ti-brand-visual-studio:before {
  content: "\ef76";
}

.ti-brand-vite:before {
  content: "\f5f2";
}

.ti-brand-vivaldi:before {
  content: "\f210";
}

.ti-brand-vk:before {
  content: "\ed72";
}

.ti-brand-vlc:before {
  content: "\fa78";
}

.ti-brand-volkswagen:before {
  content: "\f50e";
}

.ti-brand-vsco:before {
  content: "\f334";
}

.ti-brand-vscode:before {
  content: "\f3a0";
}

.ti-brand-vue:before {
  content: "\f0e0";
}

.ti-brand-walmart:before {
  content: "\f211";
}

.ti-brand-waze:before {
  content: "\f5d8";
}

.ti-brand-webflow:before {
  content: "\f2d2";
}

.ti-brand-wechat:before {
  content: "\f5f3";
}

.ti-brand-weibo:before {
  content: "\f609";
}

.ti-brand-weibo-filled:before {
  content: "\100a6";
}

.ti-brand-whatsapp:before {
  content: "\ec74";
}

.ti-brand-whatsapp-filled:before {
  content: "\10085";
}

.ti-brand-wikipedia:before {
  content: "\fa79";
}

.ti-brand-windows:before {
  content: "\ecd8";
}

.ti-brand-windows-filled:before {
  content: "\10084";
}

.ti-brand-windy:before {
  content: "\f4dd";
}

.ti-brand-wish:before {
  content: "\f212";
}

.ti-brand-wix:before {
  content: "\f3a1";
}

.ti-brand-wordpress:before {
  content: "\f2d3";
}

.ti-brand-x:before {
  content: "\fc0f";
}

.ti-brand-x-filled:before {
  content: "\fc21";
}

.ti-brand-xamarin:before {
  content: "\fa7a";
}

.ti-brand-xbox:before {
  content: "\f298";
}

.ti-brand-xdeep:before {
  content: "\fc10";
}

.ti-brand-xing:before {
  content: "\f21a";
}

.ti-brand-yahoo:before {
  content: "\ed73";
}

.ti-brand-yandex:before {
  content: "\fae1";
}

.ti-brand-yarn:before {
  content: "\fd79";
}

.ti-brand-yatse:before {
  content: "\f213";
}

.ti-brand-ycombinator:before {
  content: "\edd9";
}

.ti-brand-youtube:before {
  content: "\ec90";
}

.ti-brand-youtube-filled:before {
  content: "\fc22";
}

.ti-brand-youtube-kids:before {
  content: "\f214";
}

.ti-brand-zalando:before {
  content: "\f49e";
}

.ti-brand-zapier:before {
  content: "\f49f";
}

.ti-brand-zeit:before {
  content: "\f335";
}

.ti-brand-zhihu:before {
  content: "\f60a";
}

.ti-brand-zoom:before {
  content: "\f215";
}

.ti-brand-zulip:before {
  content: "\f4de";
}

.ti-brand-zwift:before {
  content: "\f216";
}

.ti-bread:before {
  content: "\efa3";
}

.ti-bread-filled:before {
  content: "\fe85";
}

.ti-bread-off:before {
  content: "\f3cb";
}

.ti-briefcase:before {
  content: "\ea46";
}

.ti-briefcase-2:before {
  content: "\fb03";
}

.ti-briefcase-2-filled:before {
  content: "\fe84";
}

.ti-briefcase-filled:before {
  content: "\fd00";
}

.ti-briefcase-off:before {
  content: "\f3cc";
}

.ti-brightness:before {
  content: "\eb7f";
}

.ti-brightness-2:before {
  content: "\ee19";
}

.ti-brightness-auto:before {
  content: "\fd99";
}

.ti-brightness-auto-filled:before {
  content: "\fe83";
}

.ti-brightness-down:before {
  content: "\eb7d";
}

.ti-brightness-down-filled:before {
  content: "\fb23";
}

.ti-brightness-filled:before {
  content: "\fe82";
}

.ti-brightness-half:before {
  content: "\ee1a";
}

.ti-brightness-off:before {
  content: "\f3cd";
}

.ti-brightness-up:before {
  content: "\eb7e";
}

.ti-brightness-up-filled:before {
  content: "\fb24";
}

.ti-broadcast:before {
  content: "\f1e9";
}

.ti-broadcast-off:before {
  content: "\f1e8";
}

.ti-browser:before {
  content: "\ebb7";
}

.ti-browser-check:before {
  content: "\efd6";
}

.ti-browser-maximize:before {
  content: "\100b0";
}

.ti-browser-minus:before {
  content: "\100af";
}

.ti-browser-off:before {
  content: "\f0c1";
}

.ti-browser-plus:before {
  content: "\efd7";
}

.ti-browser-share:before {
  content: "\100ae";
}

.ti-browser-x:before {
  content: "\efd8";
}

.ti-brush:before {
  content: "\ebb8";
}

.ti-brush-off:before {
  content: "\f0c2";
}

.ti-bubble:before {
  content: "\feba";
}

.ti-bubble-filled:before {
  content: "\fec3";
}

.ti-bubble-minus:before {
  content: "\febe";
}

.ti-bubble-plus:before {
  content: "\febd";
}

.ti-bubble-tea:before {
  content: "\ff51";
}

.ti-bubble-tea-2:before {
  content: "\ff52";
}

.ti-bubble-text:before {
  content: "\febc";
}

.ti-bubble-text-filled:before {
  content: "\100a5";
}

.ti-bubble-x:before {
  content: "\febb";
}

.ti-bucket:before {
  content: "\ea47";
}

.ti-bucket-droplet:before {
  content: "\f56a";
}

.ti-bucket-off:before {
  content: "\f103";
}

.ti-bug:before {
  content: "\ea48";
}

.ti-bug-filled:before {
  content: "\fd01";
}

.ti-bug-off:before {
  content: "\f0c3";
}

.ti-building:before {
  content: "\ea4f";
}

.ti-building-airport:before {
  content: "\ffa9";
}

.ti-building-arch:before {
  content: "\ea49";
}

.ti-building-bank:before {
  content: "\ebe2";
}

.ti-building-bridge:before {
  content: "\ea4b";
}

.ti-building-bridge-2:before {
  content: "\ea4a";
}

.ti-building-bridge-2-filled:before {
  content: "\10189";
}

.ti-building-broadcast-tower:before {
  content: "\f4be";
}

.ti-building-broadcast-tower-filled:before {
  content: "\fe81";
}

.ti-building-burj-al-arab:before {
  content: "\ff50";
}

.ti-building-carousel:before {
  content: "\ed87";
}

.ti-building-castle:before {
  content: "\ed88";
}

.ti-building-church:before {
  content: "\ea4c";
}

.ti-building-circus:before {
  content: "\f4bf";
}

.ti-building-cog:before {
  content: "\10062";
}

.ti-building-community:before {
  content: "\ebf6";
}

.ti-building-cottage:before {
  content: "\ee1b";
}

.ti-building-estate:before {
  content: "\f5a5";
}

.ti-building-factory:before {
  content: "\ee1c";
}

.ti-building-factory-2:before {
  content: "\f082";
}

.ti-building-fortress:before {
  content: "\ed89";
}

.ti-building-hospital:before {
  content: "\ea4d";
}

.ti-building-lighthouse:before {
  content: "\ed8a";
}

.ti-building-minus:before {
  content: "\10061";
}

.ti-building-monument:before {
  content: "\ed26";
}

.ti-building-mosque:before {
  content: "\fa57";
}

.ti-building-off:before {
  content: "\fefd";
}

.ti-building-pavilion:before {
  content: "\ebf7";
}

.ti-building-plus:before {
  content: "\10060";
}

.ti-building-skyscraper:before {
  content: "\ec39";
}

.ti-building-stadium:before {
  content: "\f641";
}

.ti-building-store:before {
  content: "\ea4e";
}

.ti-building-tunnel:before {
  content: "\f5a6";
}

.ti-building-warehouse:before {
  content: "\ebe3";
}

.ti-building-wind-turbine:before {
  content: "\f4c0";
}

.ti-buildings:before {
  content: "\ff40";
}

.ti-bulb:before {
  content: "\ea51";
}

.ti-bulb-filled:before {
  content: "\f66a";
}

.ti-bulb-off:before {
  content: "\ea50";
}

.ti-bulldozer:before {
  content: "\ee1d";
}

.ti-burger:before {
  content: "\fcb4";
}

.ti-bus:before {
  content: "\ebe4";
}

.ti-bus-filled:before {
  content: "\100ff";
}

.ti-bus-off:before {
  content: "\f3ce";
}

.ti-bus-stop:before {
  content: "\f2d4";
}

.ti-businessplan:before {
  content: "\ee1e";
}

.ti-butterfly:before {
  content: "\efd9";
}

.ti-butterfly-filled:before {
  content: "\10016";
}

.ti-cactus:before {
  content: "\f21b";
}

.ti-cactus-filled:before {
  content: "\fb25";
}

.ti-cactus-off:before {
  content: "\f3cf";
}

.ti-cake:before {
  content: "\f00f";
}

.ti-cake-off:before {
  content: "\f104";
}

.ti-cake-roll:before {
  content: "\100bd";
}

.ti-calculator:before {
  content: "\eb80";
}

.ti-calculator-filled:before {
  content: "\fb26";
}

.ti-calculator-off:before {
  content: "\f0c4";
}

.ti-calendar:before {
  content: "\ea53";
}

.ti-calendar-bolt:before {
  content: "\f822";
}

.ti-calendar-cancel:before {
  content: "\f823";
}

.ti-calendar-check:before {
  content: "\f824";
}

.ti-calendar-clock:before {
  content: "\fd2e";
}

.ti-calendar-code:before {
  content: "\f825";
}

.ti-calendar-cog:before {
  content: "\f826";
}

.ti-calendar-dollar:before {
  content: "\f827";
}

.ti-calendar-dot:before {
  content: "\fd3e";
}

.ti-calendar-down:before {
  content: "\f828";
}

.ti-calendar-due:before {
  content: "\f621";
}

.ti-calendar-event:before {
  content: "\ea52";
}

.ti-calendar-event-filled:before {
  content: "\100b9";
}

.ti-calendar-exclamation:before {
  content: "\f829";
}

.ti-calendar-filled:before {
  content: "\fb27";
}

.ti-calendar-heart:before {
  content: "\f82a";
}

.ti-calendar-minus:before {
  content: "\ebb9";
}

.ti-calendar-month:before {
  content: "\fd2f";
}

.ti-calendar-month-filled:before {
  content: "\100b8";
}

.ti-calendar-off:before {
  content: "\ee1f";
}

.ti-calendar-pause:before {
  content: "\f82b";
}

.ti-calendar-pin:before {
  content: "\f82c";
}

.ti-calendar-plus:before {
  content: "\ebba";
}

.ti-calendar-question:before {
  content: "\f82d";
}

.ti-calendar-repeat:before {
  content: "\fad2";
}

.ti-calendar-sad:before {
  content: "\fd1d";
}

.ti-calendar-search:before {
  content: "\f82e";
}

.ti-calendar-share:before {
  content: "\f82f";
}

.ti-calendar-smile:before {
  content: "\fd1e";
}

.ti-calendar-star:before {
  content: "\f830";
}

.ti-calendar-stats:before {
  content: "\ee20";
}

.ti-calendar-time:before {
  content: "\ee21";
}

.ti-calendar-up:before {
  content: "\f831";
}

.ti-calendar-user:before {
  content: "\fd1f";
}

.ti-calendar-week:before {
  content: "\fd30";
}

.ti-calendar-week-filled:before {
  content: "\100b7";
}

.ti-calendar-x:before {
  content: "\f832";
}

.ti-camera:before {
  content: "\ea54";
}

.ti-camera-ai:before {
  content: "\ffa8";
}

.ti-camera-bitcoin:before {
  content: "\ffa7";
}

.ti-camera-bolt:before {
  content: "\f833";
}

.ti-camera-cancel:before {
  content: "\f834";
}

.ti-camera-check:before {
  content: "\f835";
}

.ti-camera-code:before {
  content: "\f836";
}

.ti-camera-cog:before {
  content: "\f837";
}

.ti-camera-dollar:before {
  content: "\f838";
}

.ti-camera-down:before {
  content: "\f839";
}

.ti-camera-exclamation:before {
  content: "\f83a";
}

.ti-camera-filled:before {
  content: "\fa37";
}

.ti-camera-heart:before {
  content: "\f83b";
}

.ti-camera-minus:before {
  content: "\ec3a";
}

.ti-camera-moon:before {
  content: "\ffa6";
}

.ti-camera-off:before {
  content: "\ecee";
}

.ti-camera-pause:before {
  content: "\f83c";
}

.ti-camera-pin:before {
  content: "\f83d";
}

.ti-camera-plus:before {
  content: "\ec3b";
}

.ti-camera-question:before {
  content: "\f83e";
}

.ti-camera-rotate:before {
  content: "\ee22";
}

.ti-camera-search:before {
  content: "\f83f";
}

.ti-camera-selfie:before {
  content: "\ee23";
}

.ti-camera-share:before {
  content: "\f840";
}

.ti-camera-spark:before {
  content: "\ffbc";
}

.ti-camera-star:before {
  content: "\f841";
}

.ti-camera-up:before {
  content: "\f842";
}

.ti-camera-x:before {
  content: "\f843";
}

.ti-camper:before {
  content: "\fa25";
}

.ti-campfire:before {
  content: "\f5a7";
}

.ti-campfire-filled:before {
  content: "\fb28";
}

.ti-cancel:before {
  content: "\ff11";
}

.ti-candle:before {
  content: "\efc6";
}

.ti-candle-filled:before {
  content: "\fc23";
}

.ti-candy:before {
  content: "\ef0d";
}

.ti-candy-off:before {
  content: "\f0c5";
}

.ti-cane:before {
  content: "\f50f";
}

.ti-cannabis:before {
  content: "\f4c1";
}

.ti-cannabis-filled:before {
  content: "\10015";
}

.ti-cap-projecting:before {
  content: "\ff22";
}

.ti-cap-rounded:before {
  content: "\ff21";
}

.ti-cap-straight:before {
  content: "\ff20";
}

.ti-capsule:before {
  content: "\fae3";
}

.ti-capsule-filled:before {
  content: "\fc24";
}

.ti-capsule-horizontal:before {
  content: "\fae2";
}

.ti-capsule-horizontal-filled:before {
  content: "\fc25";
}

.ti-capture:before {
  content: "\ec3c";
}

.ti-capture-filled:before {
  content: "\fb29";
}

.ti-capture-off:before {
  content: "\f0c6";
}

.ti-car:before {
  content: "\ebbb";
}

.ti-car-4wd:before {
  content: "\fdb8";
}

.ti-car-4wd-filled:before {
  content: "\1001e";
}

.ti-car-crane:before {
  content: "\ef25";
}

.ti-car-crane-filled:before {
  content: "\100fe";
}

.ti-car-crash:before {
  content: "\efa4";
}

.ti-car-fan:before {
  content: "\fdb3";
}

.ti-car-fan-1:before {
  content: "\fdb7";
}

.ti-car-fan-2:before {
  content: "\fdb6";
}

.ti-car-fan-3:before {
  content: "\fdb5";
}

.ti-car-fan-auto:before {
  content: "\fdb4";
}

.ti-car-fan-filled:before {
  content: "\1001d";
}

.ti-car-filled:before {
  content: "\1004c";
}

.ti-car-garage:before {
  content: "\fc77";
}

.ti-car-off:before {
  content: "\f0c7";
}

.ti-car-suv:before {
  content: "\fc8b";
}

.ti-car-suv-filled:before {
  content: "\1004d";
}

.ti-car-turbine:before {
  content: "\f4fd";
}

.ti-carambola:before {
  content: "\feb9";
}

.ti-carambola-filled:before {
  content: "\10014";
}

.ti-caravan:before {
  content: "\ec7c";
}

.ti-caravan-filled:before {
  content: "\100fd";
}

.ti-cardboards:before {
  content: "\ed74";
}

.ti-cardboards-filled:before {
  content: "\1001c";
}

.ti-cardboards-off:before {
  content: "\f0c8";
}

.ti-cards:before {
  content: "\f510";
}

.ti-cards-filled:before {
  content: "\fc26";
}

.ti-caret-down:before {
  content: "\eb5d";
}

.ti-caret-down-filled:before {
  content: "\fb2a";
}

.ti-caret-left:before {
  content: "\eb5e";
}

.ti-caret-left-filled:before {
  content: "\fb2b";
}

.ti-caret-left-right:before {
  content: "\fc43";
}

.ti-caret-left-right-filled:before {
  content: "\fd02";
}

.ti-caret-right:before {
  content: "\eb5f";
}

.ti-caret-right-filled:before {
  content: "\fb2c";
}

.ti-caret-up:before {
  content: "\eb60";
}

.ti-caret-up-down:before {
  content: "\fc44";
}

.ti-caret-up-down-filled:before {
  content: "\fd03";
}

.ti-caret-up-filled:before {
  content: "\fb2d";
}

.ti-carousel-horizontal:before {
  content: "\f659";
}

.ti-carousel-horizontal-filled:before {
  content: "\fa92";
}

.ti-carousel-vertical:before {
  content: "\f65a";
}

.ti-carousel-vertical-filled:before {
  content: "\fa93";
}

.ti-carrot:before {
  content: "\f21c";
}

.ti-carrot-off:before {
  content: "\f3d0";
}

.ti-cash:before {
  content: "\ea55";
}

.ti-cash-banknote:before {
  content: "\ee25";
}

.ti-cash-banknote-edit:before {
  content: "\10149";
}

.ti-cash-banknote-filled:before {
  content: "\fe80";
}

.ti-cash-banknote-heart:before {
  content: "\10148";
}

.ti-cash-banknote-minus:before {
  content: "\10147";
}

.ti-cash-banknote-move:before {
  content: "\10145";
}

.ti-cash-banknote-move-back:before {
  content: "\10146";
}

.ti-cash-banknote-off:before {
  content: "\ee24";
}

.ti-cash-banknote-plus:before {
  content: "\10144";
}

.ti-cash-edit:before {
  content: "\10143";
}

.ti-cash-heart:before {
  content: "\10142";
}

.ti-cash-minus:before {
  content: "\10141";
}

.ti-cash-move:before {
  content: "\1013f";
}

.ti-cash-move-back:before {
  content: "\10140";
}

.ti-cash-off:before {
  content: "\f105";
}

.ti-cash-plus:before {
  content: "\1013e";
}

.ti-cash-register:before {
  content: "\fee6";
}

.ti-cast:before {
  content: "\ea56";
}

.ti-cast-off:before {
  content: "\f0c9";
}

.ti-cat:before {
  content: "\f65b";
}

.ti-category:before {
  content: "\f1f6";
}

.ti-category-2:before {
  content: "\f1f5";
}

.ti-category-filled:before {
  content: "\fb2e";
}

.ti-category-minus:before {
  content: "\fd20";
}

.ti-category-plus:before {
  content: "\fd21";
}

.ti-ce:before {
  content: "\ed75";
}

.ti-ce-off:before {
  content: "\f0ca";
}

.ti-cell:before {
  content: "\f05f";
}

.ti-cell-signal-1:before {
  content: "\f083";
}

.ti-cell-signal-2:before {
  content: "\f084";
}

.ti-cell-signal-3:before {
  content: "\f085";
}

.ti-cell-signal-4:before {
  content: "\f086";
}

.ti-cell-signal-5:before {
  content: "\f087";
}

.ti-cell-signal-off:before {
  content: "\f088";
}

.ti-certificate:before {
  content: "\ed76";
}

.ti-certificate-2:before {
  content: "\f073";
}

.ti-certificate-2-off:before {
  content: "\f0cb";
}

.ti-certificate-off:before {
  content: "\f0cc";
}

.ti-chair-director:before {
  content: "\f2d5";
}

.ti-chalkboard:before {
  content: "\f34d";
}

.ti-chalkboard-off:before {
  content: "\f3d1";
}

.ti-chalkboard-teacher:before {
  content: "\10160";
}

.ti-charging-pile:before {
  content: "\ee26";
}

.ti-charging-pile-filled:before {
  content: "\1001b";
}

.ti-chart-arcs:before {
  content: "\ee28";
}

.ti-chart-arcs-3:before {
  content: "\ee27";
}

.ti-chart-area:before {
  content: "\ea58";
}

.ti-chart-area-filled:before {
  content: "\f66b";
}

.ti-chart-area-line:before {
  content: "\ea57";
}

.ti-chart-area-line-filled:before {
  content: "\f66c";
}

.ti-chart-arrows:before {
  content: "\ee2a";
}

.ti-chart-arrows-vertical:before {
  content: "\ee29";
}

.ti-chart-bar:before {
  content: "\ea59";
}

.ti-chart-bar-off:before {
  content: "\f3d2";
}

.ti-chart-bar-popular:before {
  content: "\fef7";
}

.ti-chart-bubble:before {
  content: "\ec75";
}

.ti-chart-bubble-filled:before {
  content: "\f66d";
}

.ti-chart-candle:before {
  content: "\ea5a";
}

.ti-chart-candle-filled:before {
  content: "\f66e";
}

.ti-chart-circles:before {
  content: "\ee2b";
}

.ti-chart-cohort:before {
  content: "\fef6";
}

.ti-chart-column:before {
  content: "\ffa5";
}

.ti-chart-covariate:before {
  content: "\ffa4";
}

.ti-chart-donut:before {
  content: "\ea5b";
}

.ti-chart-donut-2:before {
  content: "\ee2c";
}

.ti-chart-donut-3:before {
  content: "\ee2d";
}

.ti-chart-donut-4:before {
  content: "\ee2e";
}

.ti-chart-donut-filled:before {
  content: "\f66f";
}

.ti-chart-dots:before {
  content: "\ee2f";
}

.ti-chart-dots-2:before {
  content: "\f097";
}

.ti-chart-dots-2-filled:before {
  content: "\100dd";
}

.ti-chart-dots-3:before {
  content: "\f098";
}

.ti-chart-dots-3-filled:before {
  content: "\100dc";
}

.ti-chart-dots-filled:before {
  content: "\fd04";
}

.ti-chart-funnel:before {
  content: "\fef5";
}

.ti-chart-funnel-filled:before {
  content: "\100db";
}

.ti-chart-grid-dots:before {
  content: "\f4c2";
}

.ti-chart-grid-dots-filled:before {
  content: "\fd05";
}

.ti-chart-histogram:before {
  content: "\f65c";
}

.ti-chart-infographic:before {
  content: "\ee30";
}

.ti-chart-line:before {
  content: "\ea5c";
}

.ti-chart-pie:before {
  content: "\ea5d";
}

.ti-chart-pie-2:before {
  content: "\ee31";
}

.ti-chart-pie-2-filled:before {
  content: "\100da";
}

.ti-chart-pie-3:before {
  content: "\ee32";
}

.ti-chart-pie-3-filled:before {
  content: "\100d9";
}

.ti-chart-pie-4:before {
  content: "\ee33";
}

.ti-chart-pie-4-filled:before {
  content: "\100d8";
}

.ti-chart-pie-filled:before {
  content: "\f670";
}

.ti-chart-pie-off:before {
  content: "\f3d3";
}

.ti-chart-ppf:before {
  content: "\f618";
}

.ti-chart-radar:before {
  content: "\ed77";
}

.ti-chart-sankey:before {
  content: "\f619";
}

.ti-chart-scatter:before {
  content: "\fd93";
}

.ti-chart-scatter-3d:before {
  content: "\fd92";
}

.ti-chart-treemap:before {
  content: "\f381";
}

.ti-check:before {
  content: "\ea5e";
}

.ti-checkbox:before {
  content: "\eba6";
}

.ti-checklist:before {
  content: "\f074";
}

.ti-checks:before {
  content: "\ebaa";
}

.ti-checkup-list:before {
  content: "\ef5a";
}

.ti-cheese:before {
  content: "\ef26";
}

.ti-chef-hat:before {
  content: "\f21d";
}

.ti-chef-hat-filled:before {
  content: "\100d7";
}

.ti-chef-hat-off:before {
  content: "\f3d4";
}

.ti-cherry:before {
  content: "\f511";
}

.ti-cherry-filled:before {
  content: "\f728";
}

.ti-chess:before {
  content: "\f382";
}

.ti-chess-bishop:before {
  content: "\f56b";
}

.ti-chess-bishop-filled:before {
  content: "\f729";
}

.ti-chess-filled:before {
  content: "\f72a";
}

.ti-chess-king:before {
  content: "\f56c";
}

.ti-chess-king-filled:before {
  content: "\f72b";
}

.ti-chess-knight:before {
  content: "\f56d";
}

.ti-chess-knight-filled:before {
  content: "\f72c";
}

.ti-chess-queen:before {
  content: "\f56e";
}

.ti-chess-queen-filled:before {
  content: "\f72d";
}

.ti-chess-rook:before {
  content: "\f56f";
}

.ti-chess-rook-filled:before {
  content: "\f72e";
}

.ti-chevron-compact-down:before {
  content: "\faf0";
}

.ti-chevron-compact-left:before {
  content: "\faf1";
}

.ti-chevron-compact-right:before {
  content: "\faf2";
}

.ti-chevron-compact-up:before {
  content: "\faf3";
}

.ti-chevron-down:before {
  content: "\ea5f";
}

.ti-chevron-down-left:before {
  content: "\ed09";
}

.ti-chevron-down-right:before {
  content: "\ed0a";
}

.ti-chevron-left:before {
  content: "\ea60";
}

.ti-chevron-left-pipe:before {
  content: "\fae4";
}

.ti-chevron-right:before {
  content: "\ea61";
}

.ti-chevron-right-pipe:before {
  content: "\fae5";
}

.ti-chevron-up:before {
  content: "\ea62";
}

.ti-chevron-up-left:before {
  content: "\ed0b";
}

.ti-chevron-up-right:before {
  content: "\ed0c";
}

.ti-chevrons-down:before {
  content: "\ea63";
}

.ti-chevrons-down-left:before {
  content: "\ed0d";
}

.ti-chevrons-down-right:before {
  content: "\ed0e";
}

.ti-chevrons-left:before {
  content: "\ea64";
}

.ti-chevrons-right:before {
  content: "\ea65";
}

.ti-chevrons-up:before {
  content: "\ea66";
}

.ti-chevrons-up-left:before {
  content: "\ed0f";
}

.ti-chevrons-up-right:before {
  content: "\ed10";
}

.ti-chisel:before {
  content: "\f383";
}

.ti-christmas-ball:before {
  content: "\fd31";
}

.ti-christmas-tree:before {
  content: "\ed78";
}

.ti-christmas-tree-filled:before {
  content: "\1001a";
}

.ti-christmas-tree-off:before {
  content: "\f3d5";
}

.ti-circle:before {
  content: "\ea6b";
}

.ti-circle-arrow-down:before {
  content: "\f6f9";
}

.ti-circle-arrow-down-filled:before {
  content: "\f6f4";
}

.ti-circle-arrow-down-left:before {
  content: "\f6f6";
}

.ti-circle-arrow-down-left-filled:before {
  content: "\f6f5";
}

.ti-circle-arrow-down-right:before {
  content: "\f6f8";
}

.ti-circle-arrow-down-right-filled:before {
  content: "\f6f7";
}

.ti-circle-arrow-left:before {
  content: "\f6fb";
}

.ti-circle-arrow-left-filled:before {
  content: "\f6fa";
}

.ti-circle-arrow-right:before {
  content: "\f6fd";
}

.ti-circle-arrow-right-filled:before {
  content: "\f6fc";
}

.ti-circle-arrow-up:before {
  content: "\f703";
}

.ti-circle-arrow-up-filled:before {
  content: "\f6fe";
}

.ti-circle-arrow-up-left:before {
  content: "\f700";
}

.ti-circle-arrow-up-left-filled:before {
  content: "\f6ff";
}

.ti-circle-arrow-up-right:before {
  content: "\f702";
}

.ti-circle-arrow-up-right-filled:before {
  content: "\f701";
}

.ti-circle-caret-down:before {
  content: "\f4a9";
}

.ti-circle-caret-down-filled:before {
  content: "\100d6";
}

.ti-circle-caret-left:before {
  content: "\f4aa";
}

.ti-circle-caret-left-filled:before {
  content: "\100d5";
}

.ti-circle-caret-right:before {
  content: "\f4ab";
}

.ti-circle-caret-right-filled:before {
  content: "\100d4";
}

.ti-circle-caret-up:before {
  content: "\f4ac";
}

.ti-circle-caret-up-filled:before {
  content: "\100d3";
}

.ti-circle-check:before {
  content: "\ea67";
}

.ti-circle-check-filled:before {
  content: "\f704";
}

.ti-circle-chevron-down:before {
  content: "\f622";
}

.ti-circle-chevron-down-filled:before {
  content: "\100d2";
}

.ti-circle-chevron-left:before {
  content: "\f623";
}

.ti-circle-chevron-left-filled:before {
  content: "\100d1";
}

.ti-circle-chevron-right:before {
  content: "\f624";
}

.ti-circle-chevron-right-filled:before {
  content: "\100d0";
}

.ti-circle-chevron-up:before {
  content: "\f625";
}

.ti-circle-chevron-up-filled:before {
  content: "\100cf";
}

.ti-circle-chevrons-down:before {
  content: "\f642";
}

.ti-circle-chevrons-down-filled:before {
  content: "\100ef";
}

.ti-circle-chevrons-left:before {
  content: "\f643";
}

.ti-circle-chevrons-left-filled:before {
  content: "\100ee";
}

.ti-circle-chevrons-right:before {
  content: "\f644";
}

.ti-circle-chevrons-right-filled:before {
  content: "\100ed";
}

.ti-circle-chevrons-up:before {
  content: "\f645";
}

.ti-circle-chevrons-up-filled:before {
  content: "\100ec";
}

.ti-circle-dashed:before {
  content: "\ed27";
}

.ti-circle-dashed-check:before {
  content: "\feb8";
}

.ti-circle-dashed-letter-a:before {
  content: "\ff9a";
}

.ti-circle-dashed-letter-b:before {
  content: "\ff99";
}

.ti-circle-dashed-letter-c:before {
  content: "\ff98";
}

.ti-circle-dashed-letter-d:before {
  content: "\ff97";
}

.ti-circle-dashed-letter-e:before {
  content: "\ff96";
}

.ti-circle-dashed-letter-f:before {
  content: "\ff95";
}

.ti-circle-dashed-letter-g:before {
  content: "\ff94";
}

.ti-circle-dashed-letter-h:before {
  content: "\ff93";
}

.ti-circle-dashed-letter-i:before {
  content: "\ff92";
}

.ti-circle-dashed-letter-j:before {
  content: "\ff91";
}

.ti-circle-dashed-letter-k:before {
  content: "\ff90";
}

.ti-circle-dashed-letter-l:before {
  content: "\ff8f";
}

.ti-circle-dashed-letter-m:before {
  content: "\ff8d";
}

.ti-circle-dashed-letter-n:before {
  content: "\ff8c";
}

.ti-circle-dashed-letter-o:before {
  content: "\ff8b";
}

.ti-circle-dashed-letter-p:before {
  content: "\ff8a";
}

.ti-circle-dashed-letter-q:before {
  content: "\ff89";
}

.ti-circle-dashed-letter-r:before {
  content: "\ff88";
}

.ti-circle-dashed-letter-s:before {
  content: "\ff87";
}

.ti-circle-dashed-letter-t:before {
  content: "\ff86";
}

.ti-circle-dashed-letter-u:before {
  content: "\ff85";
}

.ti-circle-dashed-letter-v:before {
  content: "\ff84";
}

.ti-circle-dashed-letter-w:before {
  content: "\ff83";
}

.ti-circle-dashed-letter-x:before {
  content: "\ff82";
}

.ti-circle-dashed-letter-y:before {
  content: "\ff81";
}

.ti-circle-dashed-letter-z:before {
  content: "\ff80";
}

.ti-circle-dashed-minus:before {
  content: "\feb7";
}

.ti-circle-dashed-number-0:before {
  content: "\fc6b";
}

.ti-circle-dashed-number-1:before {
  content: "\fc6c";
}

.ti-circle-dashed-number-2:before {
  content: "\fc6d";
}

.ti-circle-dashed-number-3:before {
  content: "\fc6e";
}

.ti-circle-dashed-number-4:before {
  content: "\fc6f";
}

.ti-circle-dashed-number-5:before {
  content: "\fc70";
}

.ti-circle-dashed-number-6:before {
  content: "\fc71";
}

.ti-circle-dashed-number-7:before {
  content: "\fc72";
}

.ti-circle-dashed-number-8:before {
  content: "\fc73";
}

.ti-circle-dashed-number-9:before {
  content: "\fc74";
}

.ti-circle-dashed-percentage:before {
  content: "\fd7a";
}

.ti-circle-dashed-plus:before {
  content: "\feb6";
}

.ti-circle-dashed-x:before {
  content: "\fc75";
}

.ti-circle-dot:before {
  content: "\efb1";
}

.ti-circle-dot-filled:before {
  content: "\f705";
}

.ti-circle-dotted:before {
  content: "\ed28";
}

.ti-circle-dotted-letter-a:before {
  content: "\ff7f";
}

.ti-circle-dotted-letter-b:before {
  content: "\ff7e";
}

.ti-circle-dotted-letter-c:before {
  content: "\ff7d";
}

.ti-circle-dotted-letter-d:before {
  content: "\ff7c";
}

.ti-circle-dotted-letter-e:before {
  content: "\ff7b";
}

.ti-circle-dotted-letter-f:before {
  content: "\ff7a";
}

.ti-circle-dotted-letter-g:before {
  content: "\ff79";
}

.ti-circle-dotted-letter-h:before {
  content: "\ff78";
}

.ti-circle-dotted-letter-i:before {
  content: "\ff77";
}

.ti-circle-dotted-letter-j:before {
  content: "\ff76";
}

.ti-circle-dotted-letter-k:before {
  content: "\ff75";
}

.ti-circle-dotted-letter-l:before {
  content: "\ff74";
}

.ti-circle-dotted-letter-m:before {
  content: "\ff73";
}

.ti-circle-dotted-letter-n:before {
  content: "\ff72";
}

.ti-circle-dotted-letter-o:before {
  content: "\ff71";
}

.ti-circle-dotted-letter-p:before {
  content: "\ff70";
}

.ti-circle-dotted-letter-q:before {
  content: "\ff6f";
}

.ti-circle-dotted-letter-r:before {
  content: "\ff6e";
}

.ti-circle-dotted-letter-s:before {
  content: "\ff6d";
}

.ti-circle-dotted-letter-t:before {
  content: "\ff6c";
}

.ti-circle-dotted-letter-u:before {
  content: "\ff6b";
}

.ti-circle-dotted-letter-v:before {
  content: "\ff6a";
}

.ti-circle-dotted-letter-w:before {
  content: "\ff69";
}

.ti-circle-dotted-letter-x:before {
  content: "\ff68";
}

.ti-circle-dotted-letter-y:before {
  content: "\ff67";
}

.ti-circle-dotted-letter-z:before {
  content: "\ff66";
}

.ti-circle-filled:before {
  content: "\f671";
}

.ti-circle-half:before {
  content: "\ee3f";
}

.ti-circle-half-2:before {
  content: "\eff3";
}

.ti-circle-half-vertical:before {
  content: "\ee3e";
}

.ti-circle-key:before {
  content: "\f633";
}

.ti-circle-key-filled:before {
  content: "\f706";
}

.ti-circle-letter-a:before {
  content: "\f441";
}

.ti-circle-letter-a-filled:before {
  content: "\fe7f";
}

.ti-circle-letter-b:before {
  content: "\f442";
}

.ti-circle-letter-b-filled:before {
  content: "\fe7e";
}

.ti-circle-letter-c:before {
  content: "\f443";
}

.ti-circle-letter-c-filled:before {
  content: "\fe7d";
}

.ti-circle-letter-d:before {
  content: "\f444";
}

.ti-circle-letter-d-filled:before {
  content: "\fe7c";
}

.ti-circle-letter-e:before {
  content: "\f445";
}

.ti-circle-letter-e-filled:before {
  content: "\fe7b";
}

.ti-circle-letter-f:before {
  content: "\f446";
}

.ti-circle-letter-f-filled:before {
  content: "\fe7a";
}

.ti-circle-letter-g:before {
  content: "\f447";
}

.ti-circle-letter-g-filled:before {
  content: "\fe79";
}

.ti-circle-letter-h:before {
  content: "\f448";
}

.ti-circle-letter-h-filled:before {
  content: "\fe78";
}

.ti-circle-letter-i:before {
  content: "\f449";
}

.ti-circle-letter-i-filled:before {
  content: "\fe77";
}

.ti-circle-letter-j:before {
  content: "\f44a";
}

.ti-circle-letter-j-filled:before {
  content: "\fe76";
}

.ti-circle-letter-k:before {
  content: "\f44b";
}

.ti-circle-letter-k-filled:before {
  content: "\fe75";
}

.ti-circle-letter-l:before {
  content: "\f44c";
}

.ti-circle-letter-l-filled:before {
  content: "\fe74";
}

.ti-circle-letter-m:before {
  content: "\f44d";
}

.ti-circle-letter-m-filled:before {
  content: "\fe73";
}

.ti-circle-letter-n:before {
  content: "\f44e";
}

.ti-circle-letter-n-filled:before {
  content: "\fe72";
}

.ti-circle-letter-o:before {
  content: "\f44f";
}

.ti-circle-letter-o-filled:before {
  content: "\fe71";
}

.ti-circle-letter-p:before {
  content: "\f450";
}

.ti-circle-letter-p-filled:before {
  content: "\fe70";
}

.ti-circle-letter-q:before {
  content: "\f451";
}

.ti-circle-letter-q-filled:before {
  content: "\fe6f";
}

.ti-circle-letter-r:before {
  content: "\f452";
}

.ti-circle-letter-r-filled:before {
  content: "\fe6e";
}

.ti-circle-letter-s:before {
  content: "\f453";
}

.ti-circle-letter-s-filled:before {
  content: "\fe6d";
}

.ti-circle-letter-t:before {
  content: "\f454";
}

.ti-circle-letter-t-filled:before {
  content: "\fe6c";
}

.ti-circle-letter-u:before {
  content: "\f455";
}

.ti-circle-letter-u-filled:before {
  content: "\fe6b";
}

.ti-circle-letter-v:before {
  content: "\f4ad";
}

.ti-circle-letter-v-filled:before {
  content: "\fe6a";
}

.ti-circle-letter-w:before {
  content: "\f456";
}

.ti-circle-letter-w-filled:before {
  content: "\fe69";
}

.ti-circle-letter-x:before {
  content: "\f4ae";
}

.ti-circle-letter-x-filled:before {
  content: "\fe68";
}

.ti-circle-letter-y:before {
  content: "\f457";
}

.ti-circle-letter-y-filled:before {
  content: "\fe67";
}

.ti-circle-letter-z:before {
  content: "\f458";
}

.ti-circle-letter-z-filled:before {
  content: "\fe66";
}

.ti-circle-minus:before {
  content: "\ea68";
}

.ti-circle-minus-2:before {
  content: "\fc8c";
}

.ti-circle-number-0:before {
  content: "\ee34";
}

.ti-circle-number-0-filled:before {
  content: "\f72f";
}

.ti-circle-number-1:before {
  content: "\ee35";
}

.ti-circle-number-1-filled:before {
  content: "\f730";
}

.ti-circle-number-2:before {
  content: "\ee36";
}

.ti-circle-number-2-filled:before {
  content: "\f731";
}

.ti-circle-number-3:before {
  content: "\ee37";
}

.ti-circle-number-3-filled:before {
  content: "\f732";
}

.ti-circle-number-4:before {
  content: "\ee38";
}

.ti-circle-number-4-filled:before {
  content: "\f733";
}

.ti-circle-number-5:before {
  content: "\ee39";
}

.ti-circle-number-5-filled:before {
  content: "\f734";
}

.ti-circle-number-6:before {
  content: "\ee3a";
}

.ti-circle-number-6-filled:before {
  content: "\f735";
}

.ti-circle-number-7:before {
  content: "\ee3b";
}

.ti-circle-number-7-filled:before {
  content: "\f736";
}

.ti-circle-number-8:before {
  content: "\ee3c";
}

.ti-circle-number-8-filled:before {
  content: "\f737";
}

.ti-circle-number-9:before {
  content: "\ee3d";
}

.ti-circle-number-9-filled:before {
  content: "\f738";
}

.ti-circle-off:before {
  content: "\ee40";
}

.ti-circle-percentage:before {
  content: "\fd7b";
}

.ti-circle-percentage-filled:before {
  content: "\fed5";
}

.ti-circle-plus:before {
  content: "\ea69";
}

.ti-circle-plus-2:before {
  content: "\fc8d";
}

.ti-circle-plus-filled:before {
  content: "\fef9";
}

.ti-circle-rectangle:before {
  content: "\f010";
}

.ti-circle-rectangle-filled:before {
  content: "\ff63";
}

.ti-circle-rectangle-off:before {
  content: "\f0cd";
}

.ti-circle-square:before {
  content: "\ece4";
}

.ti-circle-triangle:before {
  content: "\f011";
}

.ti-circle-x:before {
  content: "\ea6a";
}

.ti-circle-x-filled:before {
  content: "\f739";
}

.ti-circles:before {
  content: "\ece5";
}

.ti-circles-filled:before {
  content: "\f672";
}

.ti-circles-relation:before {
  content: "\f4c3";
}

.ti-circuit-ammeter:before {
  content: "\f271";
}

.ti-circuit-battery:before {
  content: "\f272";
}

.ti-circuit-bulb:before {
  content: "\f273";
}

.ti-circuit-capacitor:before {
  content: "\f275";
}

.ti-circuit-capacitor-polarized:before {
  content: "\f274";
}

.ti-circuit-cell:before {
  content: "\f277";
}

.ti-circuit-cell-plus:before {
  content: "\f276";
}

.ti-circuit-changeover:before {
  content: "\f278";
}

.ti-circuit-diode:before {
  content: "\f27a";
}

.ti-circuit-diode-zener:before {
  content: "\f279";
}

.ti-circuit-ground:before {
  content: "\f27c";
}

.ti-circuit-ground-digital:before {
  content: "\f27b";
}

.ti-circuit-inductor:before {
  content: "\f27d";
}

.ti-circuit-motor:before {
  content: "\f27e";
}

.ti-circuit-pushbutton:before {
  content: "\f27f";
}

.ti-circuit-resistor:before {
  content: "\f280";
}

.ti-circuit-switch-closed:before {
  content: "\f281";
}

.ti-circuit-switch-open:before {
  content: "\f282";
}

.ti-circuit-voltmeter:before {
  content: "\f283";
}

.ti-clear-all:before {
  content: "\ee41";
}

.ti-clear-formatting:before {
  content: "\ebe5";
}

.ti-click:before {
  content: "\ebbc";
}

.ti-cliff-jumping:before {
  content: "\fefc";
}

.ti-clipboard:before {
  content: "\ea6f";
}

.ti-clipboard-check:before {
  content: "\ea6c";
}

.ti-clipboard-check-filled:before {
  content: "\100ce";
}

.ti-clipboard-copy:before {
  content: "\f299";
}

.ti-clipboard-data:before {
  content: "\f563";
}

.ti-clipboard-data-filled:before {
  content: "\100eb";
}

.ti-clipboard-filled:before {
  content: "\100cc";
}

.ti-clipboard-heart:before {
  content: "\f34e";
}

.ti-clipboard-list:before {
  content: "\ea6d";
}

.ti-clipboard-list-filled:before {
  content: "\100ea";
}

.ti-clipboard-off:before {
  content: "\f0ce";
}

.ti-clipboard-plus:before {
  content: "\efb2";
}

.ti-clipboard-plus-filled:before {
  content: "\10176";
}

.ti-clipboard-search:before {
  content: "\10098";
}

.ti-clipboard-smile:before {
  content: "\fd9a";
}

.ti-clipboard-smile-filled:before {
  content: "\10175";
}

.ti-clipboard-text:before {
  content: "\f089";
}

.ti-clipboard-text-filled:before {
  content: "\100e9";
}

.ti-clipboard-typography:before {
  content: "\f34f";
}

.ti-clipboard-typography-filled:before {
  content: "\100e8";
}

.ti-clipboard-x:before {
  content: "\ea6e";
}

.ti-clipboard-x-filled:before {
  content: "\100cd";
}

.ti-clock:before {
  content: "\ea70";
}

.ti-clock-12:before {
  content: "\fc56";
}

.ti-clock-2:before {
  content: "\f099";
}

.ti-clock-24:before {
  content: "\fc57";
}

.ti-clock-bitcoin:before {
  content: "\ff3f";
}

.ti-clock-bolt:before {
  content: "\f844";
}

.ti-clock-cancel:before {
  content: "\f546";
}

.ti-clock-check:before {
  content: "\f7c1";
}

.ti-clock-code:before {
  content: "\f845";
}

.ti-clock-cog:before {
  content: "\f7c2";
}

.ti-clock-dollar:before {
  content: "\f846";
}

.ti-clock-down:before {
  content: "\f7c3";
}

.ti-clock-edit:before {
  content: "\f547";
}

.ti-clock-exclamation:before {
  content: "\f847";
}

.ti-clock-filled:before {
  content: "\f73a";
}

.ti-clock-heart:before {
  content: "\f7c4";
}

.ti-clock-hour-1:before {
  content: "\f313";
}

.ti-clock-hour-1-filled:before {
  content: "\fe65";
}

.ti-clock-hour-10:before {
  content: "\f314";
}

.ti-clock-hour-10-filled:before {
  content: "\fe64";
}

.ti-clock-hour-11:before {
  content: "\f315";
}

.ti-clock-hour-11-filled:before {
  content: "\fe63";
}

.ti-clock-hour-12:before {
  content: "\f316";
}

.ti-clock-hour-12-filled:before {
  content: "\fe62";
}

.ti-clock-hour-2:before {
  content: "\f317";
}

.ti-clock-hour-2-filled:before {
  content: "\fe61";
}

.ti-clock-hour-3:before {
  content: "\f318";
}

.ti-clock-hour-3-filled:before {
  content: "\fe60";
}

.ti-clock-hour-4:before {
  content: "\f319";
}

.ti-clock-hour-4-filled:before {
  content: "\fe5f";
}

.ti-clock-hour-5:before {
  content: "\f31a";
}

.ti-clock-hour-5-filled:before {
  content: "\fe5e";
}

.ti-clock-hour-6:before {
  content: "\f31b";
}

.ti-clock-hour-6-filled:before {
  content: "\fe5d";
}

.ti-clock-hour-7:before {
  content: "\f31c";
}

.ti-clock-hour-7-filled:before {
  content: "\fe5c";
}

.ti-clock-hour-8:before {
  content: "\f31d";
}

.ti-clock-hour-8-filled:before {
  content: "\fe5b";
}

.ti-clock-hour-9:before {
  content: "\f31e";
}

.ti-clock-hour-9-filled:before {
  content: "\fe5a";
}

.ti-clock-minus:before {
  content: "\f848";
}

.ti-clock-off:before {
  content: "\f0cf";
}

.ti-clock-pause:before {
  content: "\f548";
}

.ti-clock-pin:before {
  content: "\f849";
}

.ti-clock-play:before {
  content: "\f549";
}

.ti-clock-plus:before {
  content: "\f7c5";
}

.ti-clock-question:before {
  content: "\f7c6";
}

.ti-clock-record:before {
  content: "\f54a";
}

.ti-clock-search:before {
  content: "\f7c7";
}

.ti-clock-share:before {
  content: "\f84a";
}

.ti-clock-shield:before {
  content: "\f7c8";
}

.ti-clock-star:before {
  content: "\f7c9";
}

.ti-clock-stop:before {
  content: "\f54b";
}

.ti-clock-up:before {
  content: "\f7ca";
}

.ti-clock-x:before {
  content: "\f7cb";
}

.ti-clothes-rack:before {
  content: "\f285";
}

.ti-clothes-rack-off:before {
  content: "\f3d6";
}

.ti-cloud:before {
  content: "\ea76";
}

.ti-cloud-bitcoin:before {
  content: "\ff3e";
}

.ti-cloud-bolt:before {
  content: "\f84b";
}

.ti-cloud-cancel:before {
  content: "\f84c";
}

.ti-cloud-check:before {
  content: "\f84d";
}

.ti-cloud-code:before {
  content: "\f84e";
}

.ti-cloud-cog:before {
  content: "\f84f";
}

.ti-cloud-computing:before {
  content: "\f1d0";
}

.ti-cloud-computing-filled:before {
  content: "\1010d";
}

.ti-cloud-data-connection:before {
  content: "\f1d1";
}

.ti-cloud-data-connection-filled:before {
  content: "\1010c";
}

.ti-cloud-dollar:before {
  content: "\f850";
}

.ti-cloud-down:before {
  content: "\f851";
}

.ti-cloud-download:before {
  content: "\ea71";
}

.ti-cloud-exclamation:before {
  content: "\f852";
}

.ti-cloud-filled:before {
  content: "\f673";
}

.ti-cloud-fog:before {
  content: "\ecd9";
}

.ti-cloud-heart:before {
  content: "\f853";
}

.ti-cloud-lock:before {
  content: "\efdb";
}

.ti-cloud-lock-open:before {
  content: "\efda";
}

.ti-cloud-minus:before {
  content: "\f854";
}

.ti-cloud-network:before {
  content: "\fc78";
}

.ti-cloud-off:before {
  content: "\ed3e";
}

.ti-cloud-pause:before {
  content: "\f855";
}

.ti-cloud-pin:before {
  content: "\f856";
}

.ti-cloud-plus:before {
  content: "\f857";
}

.ti-cloud-question:before {
  content: "\f858";
}

.ti-cloud-rain:before {
  content: "\ea72";
}

.ti-cloud-search:before {
  content: "\f859";
}

.ti-cloud-share:before {
  content: "\f85a";
}

.ti-cloud-snow:before {
  content: "\ea73";
}

.ti-cloud-star:before {
  content: "\f85b";
}

.ti-cloud-storm:before {
  content: "\ea74";
}

.ti-cloud-up:before {
  content: "\f85c";
}

.ti-cloud-upload:before {
  content: "\ea75";
}

.ti-cloud-x:before {
  content: "\f85d";
}

.ti-clover:before {
  content: "\f1ea";
}

.ti-clover-2:before {
  content: "\f21e";
}

.ti-clover-filled:before {
  content: "\10013";
}

.ti-clubs:before {
  content: "\eff4";
}

.ti-clubs-filled:before {
  content: "\f674";
}

.ti-code:before {
  content: "\ea77";
}

.ti-code-asterisk:before {
  content: "\f312";
}

.ti-code-circle:before {
  content: "\f4ff";
}

.ti-code-circle-2:before {
  content: "\f4fe";
}

.ti-code-circle-2-filled:before {
  content: "\fed4";
}

.ti-code-circle-filled:before {
  content: "\fed3";
}

.ti-code-dots:before {
  content: "\f61a";
}

.ti-code-minus:before {
  content: "\ee42";
}

.ti-code-off:before {
  content: "\f0d0";
}

.ti-code-plus:before {
  content: "\ee43";
}

.ti-code-variable:before {
  content: "\100ab";
}

.ti-code-variable-minus:before {
  content: "\100ad";
}

.ti-code-variable-plus:before {
  content: "\100ac";
}

.ti-coffee:before {
  content: "\ef0e";
}

.ti-coffee-off:before {
  content: "\f106";
}

.ti-coffin:before {
  content: "\f579";
}

.ti-coin:before {
  content: "\eb82";
}

.ti-coin-bitcoin:before {
  content: "\f2be";
}

.ti-coin-bitcoin-filled:before {
  content: "\fd06";
}

.ti-coin-euro:before {
  content: "\f2bf";
}

.ti-coin-euro-filled:before {
  content: "\fd07";
}

.ti-coin-filled:before {
  content: "\fd08";
}

.ti-coin-monero:before {
  content: "\f4a0";
}

.ti-coin-monero-filled:before {
  content: "\fd09";
}

.ti-coin-off:before {
  content: "\f0d1";
}

.ti-coin-pound:before {
  content: "\f2c0";
}

.ti-coin-pound-filled:before {
  content: "\fd0a";
}

.ti-coin-rupee:before {
  content: "\f2c1";
}

.ti-coin-rupee-filled:before {
  content: "\fd0b";
}

.ti-coin-taka:before {
  content: "\fd0d";
}

.ti-coin-taka-filled:before {
  content: "\fd0c";
}

.ti-coin-yen:before {
  content: "\f2c2";
}

.ti-coin-yen-filled:before {
  content: "\fd0e";
}

.ti-coin-yuan:before {
  content: "\f2c3";
}

.ti-coin-yuan-filled:before {
  content: "\fd0f";
}

.ti-coins:before {
  content: "\f65d";
}

.ti-color-filter:before {
  content: "\f5a8";
}

.ti-color-picker:before {
  content: "\ebe6";
}

.ti-color-picker-off:before {
  content: "\f0d2";
}

.ti-color-swatch:before {
  content: "\eb61";
}

.ti-color-swatch-off:before {
  content: "\f0d3";
}

.ti-column-insert-left:before {
  content: "\ee44";
}

.ti-column-insert-right:before {
  content: "\ee45";
}

.ti-column-remove:before {
  content: "\faf4";
}

.ti-columns:before {
  content: "\eb83";
}

.ti-columns-1:before {
  content: "\f6d4";
}

.ti-columns-1-filled:before {
  content: "\10188";
}

.ti-columns-2:before {
  content: "\f6d5";
}

.ti-columns-2-filled:before {
  content: "\10187";
}

.ti-columns-3:before {
  content: "\f6d6";
}

.ti-columns-3-filled:before {
  content: "\10186";
}

.ti-columns-off:before {
  content: "\f0d4";
}

.ti-comet:before {
  content: "\ec76";
}

.ti-command:before {
  content: "\ea78";
}

.ti-command-off:before {
  content: "\f3d7";
}

.ti-compass:before {
  content: "\ea79";
}

.ti-compass-filled:before {
  content: "\fd10";
}

.ti-compass-off:before {
  content: "\f0d5";
}

.ti-components:before {
  content: "\efa5";
}

.ti-components-off:before {
  content: "\f0d6";
}

.ti-cone:before {
  content: "\efdd";
}

.ti-cone-2:before {
  content: "\efdc";
}

.ti-cone-2-filled:before {
  content: "\fe59";
}

.ti-cone-filled:before {
  content: "\fe58";
}

.ti-cone-off:before {
  content: "\f3d8";
}

.ti-cone-plus:before {
  content: "\fa94";
}

.ti-confetti:before {
  content: "\ee46";
}

.ti-confetti-filled:before {
  content: "\10185";
}

.ti-confetti-off:before {
  content: "\f3d9";
}

.ti-confucius:before {
  content: "\f58a";
}

.ti-congruent-to:before {
  content: "\ffa3";
}

.ti-container:before {
  content: "\ee47";
}

.ti-container-filled:before {
  content: "\10184";
}

.ti-container-off:before {
  content: "\f107";
}

.ti-contract:before {
  content: "\fefb";
}

.ti-contrast:before {
  content: "\ec4e";
}

.ti-contrast-2:before {
  content: "\efc7";
}

.ti-contrast-2-filled:before {
  content: "\fe57";
}

.ti-contrast-2-off:before {
  content: "\f3da";
}

.ti-contrast-filled:before {
  content: "\fe56";
}

.ti-contrast-off:before {
  content: "\f3db";
}

.ti-cooker:before {
  content: "\f57a";
}

.ti-cookie:before {
  content: "\fdb1";
}

.ti-cookie-filled:before {
  content: "\fe54";
}

.ti-cookie-man:before {
  content: "\fdb2";
}

.ti-cookie-man-filled:before {
  content: "\fe55";
}

.ti-cookie-off:before {
  content: "\f0d7";
}

.ti-copy:before {
  content: "\ea7a";
}

.ti-copy-check:before {
  content: "\fdb0";
}

.ti-copy-check-filled:before {
  content: "\fe53";
}

.ti-copy-minus:before {
  content: "\fdaf";
}

.ti-copy-minus-filled:before {
  content: "\fe52";
}

.ti-copy-off:before {
  content: "\f0d8";
}

.ti-copy-plus:before {
  content: "\fdae";
}

.ti-copy-plus-filled:before {
  content: "\fe51";
}

.ti-copy-x:before {
  content: "\fdad";
}

.ti-copy-x-filled:before {
  content: "\fe50";
}

.ti-copyleft:before {
  content: "\ec3d";
}

.ti-copyleft-filled:before {
  content: "\f73b";
}

.ti-copyleft-off:before {
  content: "\f0d9";
}

.ti-copyright:before {
  content: "\ea7b";
}

.ti-copyright-filled:before {
  content: "\f73c";
}

.ti-copyright-off:before {
  content: "\f0da";
}

.ti-corner-down-left:before {
  content: "\ea7c";
}

.ti-corner-down-left-double:before {
  content: "\ee48";
}

.ti-corner-down-right:before {
  content: "\ea7d";
}

.ti-corner-down-right-double:before {
  content: "\ee49";
}

.ti-corner-left-down:before {
  content: "\ea7e";
}

.ti-corner-left-down-double:before {
  content: "\ee4a";
}

.ti-corner-left-up:before {
  content: "\ea7f";
}

.ti-corner-left-up-double:before {
  content: "\ee4b";
}

.ti-corner-right-down:before {
  content: "\ea80";
}

.ti-corner-right-down-double:before {
  content: "\ee4c";
}

.ti-corner-right-up:before {
  content: "\ea81";
}

.ti-corner-right-up-double:before {
  content: "\ee4d";
}

.ti-corner-up-left:before {
  content: "\ea82";
}

.ti-corner-up-left-double:before {
  content: "\ee4e";
}

.ti-corner-up-right:before {
  content: "\ea83";
}

.ti-corner-up-right-double:before {
  content: "\ee4f";
}

.ti-cpu:before {
  content: "\ef8e";
}

.ti-cpu-2:before {
  content: "\f075";
}

.ti-cpu-off:before {
  content: "\f108";
}

.ti-crane:before {
  content: "\ef27";
}

.ti-crane-off:before {
  content: "\f109";
}

.ti-creative-commons:before {
  content: "\efb3";
}

.ti-creative-commons-by:before {
  content: "\f21f";
}

.ti-creative-commons-nc:before {
  content: "\f220";
}

.ti-creative-commons-nd:before {
  content: "\f221";
}

.ti-creative-commons-off:before {
  content: "\f10a";
}

.ti-creative-commons-sa:before {
  content: "\f222";
}

.ti-creative-commons-zero:before {
  content: "\f223";
}

.ti-credit-card:before {
  content: "\ea84";
}

.ti-credit-card-filled:before {
  content: "\fd11";
}

.ti-credit-card-off:before {
  content: "\ed11";
}

.ti-credit-card-pay:before {
  content: "\fd32";
}

.ti-credit-card-refund:before {
  content: "\fd33";
}

.ti-cricket:before {
  content: "\f09a";
}

.ti-crop:before {
  content: "\ea85";
}

.ti-crop-1-1:before {
  content: "\fd50";
}

.ti-crop-1-1-filled:before {
  content: "\fe4f";
}

.ti-crop-16-9:before {
  content: "\fd51";
}

.ti-crop-16-9-filled:before {
  content: "\fe4e";
}

.ti-crop-3-2:before {
  content: "\fd52";
}

.ti-crop-3-2-filled:before {
  content: "\fe4d";
}

.ti-crop-5-4:before {
  content: "\fd53";
}

.ti-crop-5-4-filled:before {
  content: "\fe4c";
}

.ti-crop-7-5:before {
  content: "\fd54";
}

.ti-crop-7-5-filled:before {
  content: "\fe4b";
}

.ti-crop-landscape:before {
  content: "\fd55";
}

.ti-crop-landscape-filled:before {
  content: "\fe4a";
}

.ti-crop-portrait:before {
  content: "\fd56";
}

.ti-crop-portrait-filled:before {
  content: "\fe49";
}

.ti-cross:before {
  content: "\ef8f";
}

.ti-cross-filled:before {
  content: "\f675";
}

.ti-cross-off:before {
  content: "\f10b";
}

.ti-crosshair:before {
  content: "\ec3e";
}

.ti-crown:before {
  content: "\ed12";
}

.ti-crown-off:before {
  content: "\ee50";
}

.ti-crutches:before {
  content: "\ef5b";
}

.ti-crutches-off:before {
  content: "\f10c";
}

.ti-crystal-ball:before {
  content: "\f57b";
}

.ti-csv:before {
  content: "\f791";
}

.ti-cube:before {
  content: "\fa97";
}

.ti-cube-3d-sphere:before {
  content: "\ecd7";
}

.ti-cube-3d-sphere-off:before {
  content: "\f3b5";
}

.ti-cube-off:before {
  content: "\fa95";
}

.ti-cube-plus:before {
  content: "\fa96";
}

.ti-cube-send:before {
  content: "\f61b";
}

.ti-cube-spark:before {
  content: "\ffbb";
}

.ti-cube-unfolded:before {
  content: "\f61c";
}

.ti-cup:before {
  content: "\ef28";
}

.ti-cup-off:before {
  content: "\f10d";
}

.ti-curling:before {
  content: "\efc8";
}

.ti-curly-loop:before {
  content: "\ecda";
}

.ti-currency:before {
  content: "\efa6";
}

.ti-currency-afghani:before {
  content: "\f65e";
}

.ti-currency-bahraini:before {
  content: "\ee51";
}

.ti-currency-baht:before {
  content: "\f08a";
}

.ti-currency-bitcoin:before {
  content: "\ebab";
}

.ti-currency-cent:before {
  content: "\ee53";
}

.ti-currency-dinar:before {
  content: "\ee54";
}

.ti-currency-dirham:before {
  content: "\ee55";
}

.ti-currency-dogecoin:before {
  content: "\ef4b";
}

.ti-currency-dollar:before {
  content: "\eb84";
}

.ti-currency-dollar-australian:before {
  content: "\ee56";
}

.ti-currency-dollar-brunei:before {
  content: "\f36c";
}

.ti-currency-dollar-canadian:before {
  content: "\ee57";
}

.ti-currency-dollar-guyanese:before {
  content: "\f36d";
}

.ti-currency-dollar-off:before {
  content: "\f3dc";
}

.ti-currency-dollar-singapore:before {
  content: "\ee58";
}

.ti-currency-dollar-zimbabwean:before {
  content: "\f36e";
}

.ti-currency-dong:before {
  content: "\f36f";
}

.ti-currency-dram:before {
  content: "\f370";
}

.ti-currency-ethereum:before {
  content: "\ee59";
}

.ti-currency-euro:before {
  content: "\eb85";
}

.ti-currency-euro-off:before {
  content: "\f3dd";
}

.ti-currency-florin:before {
  content: "\faf5";
}

.ti-currency-forint:before {
  content: "\ee5a";
}

.ti-currency-frank:before {
  content: "\ee5b";
}

.ti-currency-guarani:before {
  content: "\f371";
}

.ti-currency-hryvnia:before {
  content: "\f372";
}

.ti-currency-iranian-rial:before {
  content: "\fa58";
}

.ti-currency-kip:before {
  content: "\f373";
}

.ti-currency-krone-czech:before {
  content: "\ee5c";
}

.ti-currency-krone-danish:before {
  content: "\ee5d";
}

.ti-currency-krone-swedish:before {
  content: "\ee5e";
}

.ti-currency-lari:before {
  content: "\f374";
}

.ti-currency-leu:before {
  content: "\ee5f";
}

.ti-currency-lira:before {
  content: "\ee60";
}

.ti-currency-litecoin:before {
  content: "\ee61";
}

.ti-currency-lyd:before {
  content: "\f375";
}

.ti-currency-manat:before {
  content: "\f376";
}

.ti-currency-monero:before {
  content: "\f377";
}

.ti-currency-naira:before {
  content: "\ee62";
}

.ti-currency-nano:before {
  content: "\f7a6";
}

.ti-currency-off:before {
  content: "\f3de";
}

.ti-currency-paanga:before {
  content: "\f378";
}

.ti-currency-peso:before {
  content: "\f65f";
}

.ti-currency-pound:before {
  content: "\ebac";
}

.ti-currency-pound-off:before {
  content: "\f3df";
}

.ti-currency-quetzal:before {
  content: "\f379";
}

.ti-currency-real:before {
  content: "\ee63";
}

.ti-currency-renminbi:before {
  content: "\ee64";
}

.ti-currency-ripple:before {
  content: "\ee65";
}

.ti-currency-riyal:before {
  content: "\ee66";
}

.ti-currency-rubel:before {
  content: "\ee67";
}

.ti-currency-rufiyaa:before {
  content: "\f37a";
}

.ti-currency-rupee:before {
  content: "\ebad";
}

.ti-currency-rupee-nepalese:before {
  content: "\f37b";
}

.ti-currency-shekel:before {
  content: "\ee68";
}

.ti-currency-solana:before {
  content: "\f4a1";
}

.ti-currency-som:before {
  content: "\f37c";
}

.ti-currency-taka:before {
  content: "\ee69";
}

.ti-currency-tenge:before {
  content: "\f37d";
}

.ti-currency-tugrik:before {
  content: "\ee6a";
}

.ti-currency-won:before {
  content: "\ee6b";
}

.ti-currency-xrp:before {
  content: "\fd34";
}

.ti-currency-yen:before {
  content: "\ebae";
}

.ti-currency-yen-off:before {
  content: "\f3e0";
}

.ti-currency-yuan:before {
  content: "\f29a";
}

.ti-currency-zloty:before {
  content: "\ee6c";
}

.ti-current-location:before {
  content: "\ecef";
}

.ti-current-location-filled:before {
  content: "\10125";
}

.ti-current-location-off:before {
  content: "\f10e";
}

.ti-cursor-off:before {
  content: "\f10f";
}

.ti-cursor-text:before {
  content: "\ee6d";
}

.ti-cut:before {
  content: "\ea86";
}

.ti-cylinder:before {
  content: "\f54c";
}

.ti-cylinder-off:before {
  content: "\fa98";
}

.ti-cylinder-plus:before {
  content: "\fa99";
}

.ti-dashboard:before {
  content: "\ea87";
}

.ti-dashboard-filled:before {
  content: "\10019";
}

.ti-dashboard-off:before {
  content: "\f3e1";
}

.ti-database:before {
  content: "\ea88";
}

.ti-database-cog:before {
  content: "\fa10";
}

.ti-database-dollar:before {
  content: "\fa11";
}

.ti-database-edit:before {
  content: "\fa12";
}

.ti-database-exclamation:before {
  content: "\fa13";
}

.ti-database-export:before {
  content: "\ee6e";
}

.ti-database-heart:before {
  content: "\fa14";
}

.ti-database-import:before {
  content: "\ee6f";
}

.ti-database-leak:before {
  content: "\fa15";
}

.ti-database-minus:before {
  content: "\fa16";
}

.ti-database-off:before {
  content: "\ee70";
}

.ti-database-plus:before {
  content: "\fa17";
}

.ti-database-search:before {
  content: "\fa18";
}

.ti-database-share:before {
  content: "\fa19";
}

.ti-database-smile:before {
  content: "\fd9b";
}

.ti-database-star:before {
  content: "\fa1a";
}

.ti-database-x:before {
  content: "\fa1b";
}

.ti-decimal:before {
  content: "\fa26";
}

.ti-deer:before {
  content: "\f4c5";
}

.ti-delta:before {
  content: "\f53c";
}

.ti-dental:before {
  content: "\f025";
}

.ti-dental-broken:before {
  content: "\f286";
}

.ti-dental-off:before {
  content: "\f110";
}

.ti-deselect:before {
  content: "\f9f3";
}

.ti-desk:before {
  content: "\fd35";
}

.ti-details:before {
  content: "\ee71";
}

.ti-details-off:before {
  content: "\f3e2";
}

.ti-device-airpods:before {
  content: "\f5a9";
}

.ti-device-airpods-case:before {
  content: "\f646";
}

.ti-device-airtag:before {
  content: "\fae6";
}

.ti-device-analytics:before {
  content: "\ee72";
}

.ti-device-audio-tape:before {
  content: "\ee73";
}

.ti-device-camera-phone:before {
  content: "\f233";
}

.ti-device-cctv:before {
  content: "\ee74";
}

.ti-device-cctv-filled:before {
  content: "\1004b";
}

.ti-device-cctv-off:before {
  content: "\f3e3";
}

.ti-device-computer-camera:before {
  content: "\ee76";
}

.ti-device-computer-camera-off:before {
  content: "\ee75";
}

.ti-device-desktop:before {
  content: "\ea89";
}

.ti-device-desktop-analytics:before {
  content: "\ee77";
}

.ti-device-desktop-bolt:before {
  content: "\f85e";
}

.ti-device-desktop-cancel:before {
  content: "\f85f";
}

.ti-device-desktop-check:before {
  content: "\f860";
}

.ti-device-desktop-code:before {
  content: "\f861";
}

.ti-device-desktop-cog:before {
  content: "\f862";
}

.ti-device-desktop-dollar:before {
  content: "\f863";
}

.ti-device-desktop-down:before {
  content: "\f864";
}

.ti-device-desktop-exclamation:before {
  content: "\f865";
}

.ti-device-desktop-filled:before {
  content: "\1004a";
}

.ti-device-desktop-heart:before {
  content: "\f866";
}

.ti-device-desktop-minus:before {
  content: "\f867";
}

.ti-device-desktop-off:before {
  content: "\ee78";
}

.ti-device-desktop-pause:before {
  content: "\f868";
}

.ti-device-desktop-pin:before {
  content: "\f869";
}

.ti-device-desktop-plus:before {
  content: "\f86a";
}

.ti-device-desktop-question:before {
  content: "\f86b";
}

.ti-device-desktop-search:before {
  content: "\f86c";
}

.ti-device-desktop-share:before {
  content: "\f86d";
}

.ti-device-desktop-star:before {
  content: "\f86e";
}

.ti-device-desktop-up:before {
  content: "\f86f";
}

.ti-device-desktop-x:before {
  content: "\f870";
}

.ti-device-floppy:before {
  content: "\eb62";
}

.ti-device-gamepad:before {
  content: "\eb63";
}

.ti-device-gamepad-2:before {
  content: "\f1d2";
}

.ti-device-gamepad-3:before {
  content: "\fc58";
}

.ti-device-gamepad-3-filled:before {
  content: "\10049";
}

.ti-device-heart-monitor:before {
  content: "\f060";
}

.ti-device-heart-monitor-filled:before {
  content: "\fa38";
}

.ti-device-imac:before {
  content: "\f7a7";
}

.ti-device-imac-bolt:before {
  content: "\f871";
}

.ti-device-imac-cancel:before {
  content: "\f872";
}

.ti-device-imac-check:before {
  content: "\f873";
}

.ti-device-imac-code:before {
  content: "\f874";
}

.ti-device-imac-cog:before {
  content: "\f875";
}

.ti-device-imac-dollar:before {
  content: "\f876";
}

.ti-device-imac-down:before {
  content: "\f877";
}

.ti-device-imac-exclamation:before {
  content: "\f878";
}

.ti-device-imac-filled:before {
  content: "\10048";
}

.ti-device-imac-heart:before {
  content: "\f879";
}

.ti-device-imac-minus:before {
  content: "\f87a";
}

.ti-device-imac-off:before {
  content: "\f87b";
}

.ti-device-imac-pause:before {
  content: "\f87c";
}

.ti-device-imac-pin:before {
  content: "\f87d";
}

.ti-device-imac-plus:before {
  content: "\f87e";
}

.ti-device-imac-question:before {
  content: "\f87f";
}

.ti-device-imac-search:before {
  content: "\f880";
}

.ti-device-imac-share:before {
  content: "\f881";
}

.ti-device-imac-star:before {
  content: "\f882";
}

.ti-device-imac-up:before {
  content: "\f883";
}

.ti-device-imac-x:before {
  content: "\f884";
}

.ti-device-ipad:before {
  content: "\f648";
}

.ti-device-ipad-bolt:before {
  content: "\f885";
}

.ti-device-ipad-cancel:before {
  content: "\f886";
}

.ti-device-ipad-check:before {
  content: "\f887";
}

.ti-device-ipad-code:before {
  content: "\f888";
}

.ti-device-ipad-cog:before {
  content: "\f889";
}

.ti-device-ipad-dollar:before {
  content: "\f88a";
}

.ti-device-ipad-down:before {
  content: "\f88b";
}

.ti-device-ipad-exclamation:before {
  content: "\f88c";
}

.ti-device-ipad-filled:before {
  content: "\10047";
}

.ti-device-ipad-heart:before {
  content: "\f88d";
}

.ti-device-ipad-horizontal:before {
  content: "\f647";
}

.ti-device-ipad-horizontal-bolt:before {
  content: "\f88e";
}

.ti-device-ipad-horizontal-cancel:before {
  content: "\f88f";
}

.ti-device-ipad-horizontal-check:before {
  content: "\f890";
}

.ti-device-ipad-horizontal-code:before {
  content: "\f891";
}

.ti-device-ipad-horizontal-cog:before {
  content: "\f892";
}

.ti-device-ipad-horizontal-dollar:before {
  content: "\f893";
}

.ti-device-ipad-horizontal-down:before {
  content: "\f894";
}

.ti-device-ipad-horizontal-exclamation:before {
  content: "\f895";
}

.ti-device-ipad-horizontal-heart:before {
  content: "\f896";
}

.ti-device-ipad-horizontal-minus:before {
  content: "\f897";
}

.ti-device-ipad-horizontal-off:before {
  content: "\f898";
}

.ti-device-ipad-horizontal-pause:before {
  content: "\f899";
}

.ti-device-ipad-horizontal-pin:before {
  content: "\f89a";
}

.ti-device-ipad-horizontal-plus:before {
  content: "\f89b";
}

.ti-device-ipad-horizontal-question:before {
  content: "\f89c";
}

.ti-device-ipad-horizontal-search:before {
  content: "\f89d";
}

.ti-device-ipad-horizontal-share:before {
  content: "\f89e";
}

.ti-device-ipad-horizontal-star:before {
  content: "\f89f";
}

.ti-device-ipad-horizontal-up:before {
  content: "\f8a0";
}

.ti-device-ipad-horizontal-x:before {
  content: "\f8a1";
}

.ti-device-ipad-minus:before {
  content: "\f8a2";
}

.ti-device-ipad-off:before {
  content: "\f8a3";
}

.ti-device-ipad-pause:before {
  content: "\f8a4";
}

.ti-device-ipad-pin:before {
  content: "\f8a5";
}

.ti-device-ipad-plus:before {
  content: "\f8a6";
}

.ti-device-ipad-question:before {
  content: "\f8a7";
}

.ti-device-ipad-search:before {
  content: "\f8a8";
}

.ti-device-ipad-share:before {
  content: "\f8a9";
}

.ti-device-ipad-star:before {
  content: "\f8aa";
}

.ti-device-ipad-up:before {
  content: "\f8ab";
}

.ti-device-ipad-x:before {
  content: "\f8ac";
}

.ti-device-landline-phone:before {
  content: "\f649";
}

.ti-device-laptop:before {
  content: "\eb64";
}

.ti-device-laptop-off:before {
  content: "\f061";
}

.ti-device-mobile:before {
  content: "\ea8a";
}

.ti-device-mobile-bolt:before {
  content: "\f8ad";
}

.ti-device-mobile-cancel:before {
  content: "\f8ae";
}

.ti-device-mobile-charging:before {
  content: "\f224";
}

.ti-device-mobile-check:before {
  content: "\f8af";
}

.ti-device-mobile-code:before {
  content: "\f8b0";
}

.ti-device-mobile-cog:before {
  content: "\f8b1";
}

.ti-device-mobile-dollar:before {
  content: "\f8b2";
}

.ti-device-mobile-down:before {
  content: "\f8b3";
}

.ti-device-mobile-exclamation:before {
  content: "\f8b4";
}

.ti-device-mobile-filled:before {
  content: "\fa39";
}

.ti-device-mobile-heart:before {
  content: "\f8b5";
}

.ti-device-mobile-message:before {
  content: "\ee79";
}

.ti-device-mobile-minus:before {
  content: "\f8b6";
}

.ti-device-mobile-off:before {
  content: "\f062";
}

.ti-device-mobile-pause:before {
  content: "\f8b7";
}

.ti-device-mobile-pin:before {
  content: "\f8b8";
}

.ti-device-mobile-plus:before {
  content: "\f8b9";
}

.ti-device-mobile-question:before {
  content: "\f8ba";
}

.ti-device-mobile-rotated:before {
  content: "\ecdb";
}

.ti-device-mobile-search:before {
  content: "\f8bb";
}

.ti-device-mobile-share:before {
  content: "\f8bc";
}

.ti-device-mobile-star:before {
  content: "\f8bd";
}

.ti-device-mobile-up:before {
  content: "\f8be";
}

.ti-device-mobile-vibration:before {
  content: "\eb86";
}

.ti-device-mobile-x:before {
  content: "\f8bf";
}

.ti-device-nintendo:before {
  content: "\f026";
}

.ti-device-nintendo-off:before {
  content: "\f111";
}

.ti-device-projector:before {
  content: "\fc11";
}

.ti-device-remote:before {
  content: "\f792";
}

.ti-device-remote-filled:before {
  content: "\10046";
}

.ti-device-sd-card:before {
  content: "\f384";
}

.ti-device-sim:before {
  content: "\f4b2";
}

.ti-device-sim-1:before {
  content: "\f4af";
}

.ti-device-sim-2:before {
  content: "\f4b0";
}

.ti-device-sim-3:before {
  content: "\f4b1";
}

.ti-device-speaker:before {
  content: "\ea8b";
}

.ti-device-speaker-filled:before {
  content: "\10045";
}

.ti-device-speaker-off:before {
  content: "\f112";
}

.ti-device-tablet:before {
  content: "\ea8c";
}

.ti-device-tablet-bolt:before {
  content: "\f8c0";
}

.ti-device-tablet-cancel:before {
  content: "\f8c1";
}

.ti-device-tablet-check:before {
  content: "\f8c2";
}

.ti-device-tablet-code:before {
  content: "\f8c3";
}

.ti-device-tablet-cog:before {
  content: "\f8c4";
}

.ti-device-tablet-dollar:before {
  content: "\f8c5";
}

.ti-device-tablet-down:before {
  content: "\f8c6";
}

.ti-device-tablet-exclamation:before {
  content: "\f8c7";
}

.ti-device-tablet-filled:before {
  content: "\fa3a";
}

.ti-device-tablet-heart:before {
  content: "\f8c8";
}

.ti-device-tablet-minus:before {
  content: "\f8c9";
}

.ti-device-tablet-off:before {
  content: "\f063";
}

.ti-device-tablet-pause:before {
  content: "\f8ca";
}

.ti-device-tablet-pin:before {
  content: "\f8cb";
}

.ti-device-tablet-plus:before {
  content: "\f8cc";
}

.ti-device-tablet-question:before {
  content: "\f8cd";
}

.ti-device-tablet-search:before {
  content: "\f8ce";
}

.ti-device-tablet-share:before {
  content: "\f8cf";
}

.ti-device-tablet-star:before {
  content: "\f8d0";
}

.ti-device-tablet-up:before {
  content: "\f8d1";
}

.ti-device-tablet-x:before {
  content: "\f8d2";
}

.ti-device-tv:before {
  content: "\ea8d";
}

.ti-device-tv-filled:before {
  content: "\10043";
}

.ti-device-tv-off:before {
  content: "\f064";
}

.ti-device-tv-old:before {
  content: "\f1d3";
}

.ti-device-tv-old-filled:before {
  content: "\10044";
}

.ti-device-unknown:before {
  content: "\fef4";
}

.ti-device-unknown-filled:before {
  content: "\10018";
}

.ti-device-usb:before {
  content: "\fc59";
}

.ti-device-usb-filled:before {
  content: "\10042";
}

.ti-device-vision-pro:before {
  content: "\fae7";
}

.ti-device-vision-pro-filled:before {
  content: "\10041";
}

.ti-device-watch:before {
  content: "\ebf9";
}

.ti-device-watch-bolt:before {
  content: "\f8d3";
}

.ti-device-watch-cancel:before {
  content: "\f8d4";
}

.ti-device-watch-check:before {
  content: "\f8d5";
}

.ti-device-watch-code:before {
  content: "\f8d6";
}

.ti-device-watch-cog:before {
  content: "\f8d7";
}

.ti-device-watch-dollar:before {
  content: "\f8d8";
}

.ti-device-watch-down:before {
  content: "\f8d9";
}

.ti-device-watch-exclamation:before {
  content: "\f8da";
}

.ti-device-watch-filled:before {
  content: "\10040";
}

.ti-device-watch-heart:before {
  content: "\f8db";
}

.ti-device-watch-minus:before {
  content: "\f8dc";
}

.ti-device-watch-off:before {
  content: "\f065";
}

.ti-device-watch-pause:before {
  content: "\f8dd";
}

.ti-device-watch-pin:before {
  content: "\f8de";
}

.ti-device-watch-plus:before {
  content: "\f8df";
}

.ti-device-watch-question:before {
  content: "\f8e0";
}

.ti-device-watch-search:before {
  content: "\f8e1";
}

.ti-device-watch-share:before {
  content: "\f8e2";
}

.ti-device-watch-star:before {
  content: "\f8e3";
}

.ti-device-watch-stats:before {
  content: "\ef7d";
}

.ti-device-watch-stats-2:before {
  content: "\ef7c";
}

.ti-device-watch-up:before {
  content: "\f8e4";
}

.ti-device-watch-x:before {
  content: "\f8e5";
}

.ti-devices:before {
  content: "\eb87";
}

.ti-devices-2:before {
  content: "\ed29";
}

.ti-devices-bolt:before {
  content: "\f8e6";
}

.ti-devices-cancel:before {
  content: "\f8e7";
}

.ti-devices-check:before {
  content: "\f8e8";
}

.ti-devices-code:before {
  content: "\f8e9";
}

.ti-devices-cog:before {
  content: "\f8ea";
}

.ti-devices-dollar:before {
  content: "\f8eb";
}

.ti-devices-down:before {
  content: "\f8ec";
}

.ti-devices-exclamation:before {
  content: "\f8ed";
}

.ti-devices-heart:before {
  content: "\f8ee";
}

.ti-devices-minus:before {
  content: "\f8ef";
}

.ti-devices-off:before {
  content: "\f3e4";
}

.ti-devices-pause:before {
  content: "\f8f0";
}

.ti-devices-pc:before {
  content: "\ee7a";
}

.ti-devices-pc-off:before {
  content: "\f113";
}

.ti-devices-pin:before {
  content: "\f8f1";
}

.ti-devices-plus:before {
  content: "\f8f2";
}

.ti-devices-question:before {
  content: "\f8f3";
}

.ti-devices-search:before {
  content: "\f8f4";
}

.ti-devices-share:before {
  content: "\f8f5";
}

.ti-devices-star:before {
  content: "\f8f6";
}

.ti-devices-up:before {
  content: "\f8f7";
}

.ti-devices-x:before {
  content: "\f8f8";
}

.ti-diabolo:before {
  content: "\fa9c";
}

.ti-diabolo-off:before {
  content: "\fa9a";
}

.ti-diabolo-plus:before {
  content: "\fa9b";
}

.ti-dialpad:before {
  content: "\f067";
}

.ti-dialpad-filled:before {
  content: "\fa3b";
}

.ti-dialpad-off:before {
  content: "\f114";
}

.ti-diamond:before {
  content: "\eb65";
}

.ti-diamond-filled:before {
  content: "\f73d";
}

.ti-diamond-off:before {
  content: "\f115";
}

.ti-diamonds:before {
  content: "\eff5";
}

.ti-diamonds-filled:before {
  content: "\f676";
}

.ti-diaper:before {
  content: "\ffa2";
}

.ti-dice:before {
  content: "\eb66";
}

.ti-dice-1:before {
  content: "\f08b";
}

.ti-dice-1-filled:before {
  content: "\f73e";
}

.ti-dice-2:before {
  content: "\f08c";
}

.ti-dice-2-filled:before {
  content: "\f73f";
}

.ti-dice-3:before {
  content: "\f08d";
}

.ti-dice-3-filled:before {
  content: "\f740";
}

.ti-dice-4:before {
  content: "\f08e";
}

.ti-dice-4-filled:before {
  content: "\f741";
}

.ti-dice-5:before {
  content: "\f08f";
}

.ti-dice-5-filled:before {
  content: "\f742";
}

.ti-dice-6:before {
  content: "\f090";
}

.ti-dice-6-filled:before {
  content: "\f743";
}

.ti-dice-filled:before {
  content: "\f744";
}

.ti-dimensions:before {
  content: "\ee7b";
}

.ti-direction:before {
  content: "\ebfb";
}

.ti-direction-arrows:before {
  content: "\fd36";
}

.ti-direction-arrows-filled:before {
  content: "\100ca";
}

.ti-direction-horizontal:before {
  content: "\ebfa";
}

.ti-direction-sign:before {
  content: "\f1f7";
}

.ti-direction-sign-filled:before {
  content: "\f745";
}

.ti-direction-sign-off:before {
  content: "\f3e5";
}

.ti-directions:before {
  content: "\ea8e";
}

.ti-directions-filled:before {
  content: "\1003f";
}

.ti-directions-off:before {
  content: "\f116";
}

.ti-disabled:before {
  content: "\ea8f";
}

.ti-disabled-2:before {
  content: "\ebaf";
}

.ti-disabled-off:before {
  content: "\f117";
}

.ti-disc:before {
  content: "\ea90";
}

.ti-disc-filled:before {
  content: "\1003e";
}

.ti-disc-golf:before {
  content: "\f385";
}

.ti-disc-off:before {
  content: "\f118";
}

.ti-discount:before {
  content: "\ebbd";
}

.ti-discount-filled:before {
  content: "\1003d";
}

.ti-discount-off:before {
  content: "\f3e7";
}

.ti-divide:before {
  content: "\ed5c";
}

.ti-dna:before {
  content: "\ee7d";
}

.ti-dna-2:before {
  content: "\ef5c";
}

.ti-dna-2-off:before {
  content: "\f119";
}

.ti-dna-off:before {
  content: "\f11a";
}

.ti-dog:before {
  content: "\f660";
}

.ti-dog-bowl:before {
  content: "\ef29";
}

.ti-door:before {
  content: "\ef4e";
}

.ti-door-enter:before {
  content: "\ef4c";
}

.ti-door-exit:before {
  content: "\ef4d";
}

.ti-door-off:before {
  content: "\f11b";
}

.ti-dots:before {
  content: "\ea95";
}

.ti-dots-circle-horizontal:before {
  content: "\ea91";
}

.ti-dots-diagonal:before {
  content: "\ea93";
}

.ti-dots-diagonal-2:before {
  content: "\ea92";
}

.ti-dots-vertical:before {
  content: "\ea94";
}

.ti-download:before {
  content: "\ea96";
}

.ti-download-off:before {
  content: "\f11c";
}

.ti-drag-drop:before {
  content: "\eb89";
}

.ti-drag-drop-2:before {
  content: "\eb88";
}

.ti-drone:before {
  content: "\ed79";
}

.ti-drone-off:before {
  content: "\ee7e";
}

.ti-drop-circle:before {
  content: "\efde";
}

.ti-drop-circle-filled:before {
  content: "\10137";
}

.ti-droplet:before {
  content: "\ea97";
}

.ti-droplet-bolt:before {
  content: "\f8f9";
}

.ti-droplet-cancel:before {
  content: "\f8fa";
}

.ti-droplet-check:before {
  content: "\f8fb";
}

.ti-droplet-code:before {
  content: "\f8fc";
}

.ti-droplet-cog:before {
  content: "\f8fd";
}

.ti-droplet-dollar:before {
  content: "\f8fe";
}

.ti-droplet-down:before {
  content: "\f8ff";
}

.ti-droplet-exclamation:before {
  content: "\f900";
}

.ti-droplet-filled:before {
  content: "\ee80";
}

.ti-droplet-half:before {
  content: "\ee82";
}

.ti-droplet-half-2:before {
  content: "\ee81";
}

.ti-droplet-half-2-filled:before {
  content: "\fb6c";
}

.ti-droplet-half-filled:before {
  content: "\f6c5";
}

.ti-droplet-heart:before {
  content: "\f901";
}

.ti-droplet-minus:before {
  content: "\f902";
}

.ti-droplet-off:before {
  content: "\ee83";
}

.ti-droplet-pause:before {
  content: "\f903";
}

.ti-droplet-pin:before {
  content: "\f904";
}

.ti-droplet-plus:before {
  content: "\f905";
}

.ti-droplet-question:before {
  content: "\f906";
}

.ti-droplet-search:before {
  content: "\f907";
}

.ti-droplet-share:before {
  content: "\f908";
}

.ti-droplet-star:before {
  content: "\f909";
}

.ti-droplet-up:before {
  content: "\f90a";
}

.ti-droplet-x:before {
  content: "\f90b";
}

.ti-droplets:before {
  content: "\fc12";
}

.ti-droplets-filled:before {
  content: "\100c9";
}

.ti-dual-screen:before {
  content: "\fa59";
}

.ti-dual-screen-filled:before {
  content: "\10136";
}

.ti-dumpling:before {
  content: "\feb5";
}

.ti-dumpling-filled:before {
  content: "\10135";
}

.ti-e-passport:before {
  content: "\f4df";
}

.ti-ear:before {
  content: "\ebce";
}

.ti-ear-off:before {
  content: "\ee84";
}

.ti-ear-scan:before {
  content: "\fd57";
}

.ti-ease-in:before {
  content: "\f573";
}

.ti-ease-in-control-point:before {
  content: "\f570";
}

.ti-ease-in-control-point-filled:before {
  content: "\10174";
}

.ti-ease-in-out:before {
  content: "\f572";
}

.ti-ease-in-out-control-points:before {
  content: "\f571";
}

.ti-ease-in-out-control-points-filled:before {
  content: "\10173";
}

.ti-ease-out:before {
  content: "\f575";
}

.ti-ease-out-control-point:before {
  content: "\f574";
}

.ti-ease-out-control-point-filled:before {
  content: "\10172";
}

.ti-edit:before {
  content: "\ea98";
}

.ti-edit-circle:before {
  content: "\ee85";
}

.ti-edit-circle-off:before {
  content: "\f11d";
}

.ti-edit-off:before {
  content: "\f11e";
}

.ti-egg:before {
  content: "\eb8a";
}

.ti-egg-cracked:before {
  content: "\f2d6";
}

.ti-egg-cracked-filled:before {
  content: "\10012";
}

.ti-egg-filled:before {
  content: "\f678";
}

.ti-egg-fried:before {
  content: "\f386";
}

.ti-egg-fried-filled:before {
  content: "\10134";
}

.ti-egg-off:before {
  content: "\f11f";
}

.ti-eggs:before {
  content: "\f500";
}

.ti-elevator:before {
  content: "\efdf";
}

.ti-elevator-filled:before {
  content: "\1003c";
}

.ti-elevator-off:before {
  content: "\f3e8";
}

.ti-emergency-bed:before {
  content: "\ef5d";
}

.ti-empathize:before {
  content: "\f29b";
}

.ti-empathize-off:before {
  content: "\f3e9";
}

.ti-emphasis:before {
  content: "\ebcf";
}

.ti-engine:before {
  content: "\ef7e";
}

.ti-engine-filled:before {
  content: "\100fc";
}

.ti-engine-off:before {
  content: "\f120";
}

.ti-equal:before {
  content: "\ee87";
}

.ti-equal-double:before {
  content: "\f4e1";
}

.ti-equal-not:before {
  content: "\ee86";
}

.ti-eraser:before {
  content: "\eb8b";
}

.ti-eraser-off:before {
  content: "\f121";
}

.ti-error-404:before {
  content: "\f027";
}

.ti-error-404-off:before {
  content: "\f122";
}

.ti-escalator:before {
  content: "\fb06";
}

.ti-escalator-down:before {
  content: "\fb04";
}

.ti-escalator-down-filled:before {
  content: "\10133";
}

.ti-escalator-filled:before {
  content: "\10131";
}

.ti-escalator-up:before {
  content: "\fb05";
}

.ti-escalator-up-filled:before {
  content: "\10132";
}

.ti-exchange:before {
  content: "\ebe7";
}

.ti-exchange-filled:before {
  content: "\10130";
}

.ti-exchange-off:before {
  content: "\f123";
}

.ti-exclamation-circle:before {
  content: "\f634";
}

.ti-exclamation-circle-filled:before {
  content: "\ff62";
}

.ti-exclamation-mark:before {
  content: "\efb4";
}

.ti-exclamation-mark-off:before {
  content: "\f124";
}

.ti-explicit:before {
  content: "\f256";
}

.ti-explicit-filled:before {
  content: "\1012f";
}

.ti-explicit-off:before {
  content: "\f3ea";
}

.ti-exposure:before {
  content: "\eb8c";
}

.ti-exposure-0:before {
  content: "\f29c";
}

.ti-exposure-filled:before {
  content: "\10124";
}

.ti-exposure-minus-1:before {
  content: "\f29d";
}

.ti-exposure-minus-2:before {
  content: "\f29e";
}

.ti-exposure-off:before {
  content: "\f3eb";
}

.ti-exposure-plus-1:before {
  content: "\f29f";
}

.ti-exposure-plus-2:before {
  content: "\f2a0";
}

.ti-external-link:before {
  content: "\ea99";
}

.ti-external-link-off:before {
  content: "\f125";
}

.ti-eye:before {
  content: "\ea9a";
}

.ti-eye-bitcoin:before {
  content: "\ff3d";
}

.ti-eye-bolt:before {
  content: "\fb6d";
}

.ti-eye-cancel:before {
  content: "\fb6e";
}

.ti-eye-check:before {
  content: "\ee88";
}

.ti-eye-closed:before {
  content: "\f7ec";
}

.ti-eye-code:before {
  content: "\fb6f";
}

.ti-eye-cog:before {
  content: "\f7ed";
}

.ti-eye-discount:before {
  content: "\fb70";
}

.ti-eye-dollar:before {
  content: "\fb71";
}

.ti-eye-dotted:before {
  content: "\fead";
}

.ti-eye-down:before {
  content: "\fb72";
}

.ti-eye-edit:before {
  content: "\f7ee";
}

.ti-eye-exclamation:before {
  content: "\f7ef";
}

.ti-eye-filled:before {
  content: "\f679";
}

.ti-eye-heart:before {
  content: "\f7f0";
}

.ti-eye-minus:before {
  content: "\fb73";
}

.ti-eye-off:before {
  content: "\ecf0";
}

.ti-eye-pause:before {
  content: "\fb74";
}

.ti-eye-pin:before {
  content: "\fb75";
}

.ti-eye-plus:before {
  content: "\fb76";
}

.ti-eye-question:before {
  content: "\fb77";
}

.ti-eye-search:before {
  content: "\fb78";
}

.ti-eye-share:before {
  content: "\fb79";
}

.ti-eye-spark:before {
  content: "\ffba";
}

.ti-eye-star:before {
  content: "\fb7a";
}

.ti-eye-table:before {
  content: "\ef5e";
}

.ti-eye-table-filled:before {
  content: "\10123";
}

.ti-eye-up:before {
  content: "\fb7b";
}

.ti-eye-x:before {
  content: "\f7f1";
}

.ti-eyeglass:before {
  content: "\ee8a";
}

.ti-eyeglass-2:before {
  content: "\ee89";
}

.ti-eyeglass-2-filled:before {
  content: "\10122";
}

.ti-eyeglass-filled:before {
  content: "\100c8";
}

.ti-eyeglass-off:before {
  content: "\f126";
}

.ti-face-id:before {
  content: "\ea9b";
}

.ti-face-id-error:before {
  content: "\efa7";
}

.ti-face-mask:before {
  content: "\efb5";
}

.ti-face-mask-filled:before {
  content: "\10121";
}

.ti-face-mask-off:before {
  content: "\f127";
}

.ti-fall:before {
  content: "\ecb9";
}

.ti-favicon:before {
  content: "\fd65";
}

.ti-favicon-filled:before {
  content: "\10071";
}

.ti-feather:before {
  content: "\ee8b";
}

.ti-feather-filled:before {
  content: "\10011";
}

.ti-feather-off:before {
  content: "\f128";
}

.ti-fence:before {
  content: "\ef2a";
}

.ti-fence-filled:before {
  content: "\10120";
}

.ti-fence-off:before {
  content: "\f129";
}

.ti-ferry:before {
  content: "\10074";
}

.ti-ferry-filled:before {
  content: "\100fb";
}

.ti-fidget-spinner:before {
  content: "\f068";
}

.ti-fidget-spinner-filled:before {
  content: "\1011f";
}

.ti-file:before {
  content: "\eaa4";
}

.ti-file-3d:before {
  content: "\f032";
}

.ti-file-ai:before {
  content: "\ffa1";
}

.ti-file-alert:before {
  content: "\ede6";
}

.ti-file-analytics:before {
  content: "\ede7";
}

.ti-file-analytics-filled:before {
  content: "\10171";
}

.ti-file-arrow-left:before {
  content: "\f033";
}

.ti-file-arrow-right:before {
  content: "\f034";
}

.ti-file-barcode:before {
  content: "\f035";
}

.ti-file-bitcoin:before {
  content: "\ffa0";
}

.ti-file-broken:before {
  content: "\f501";
}

.ti-file-certificate:before {
  content: "\ed4d";
}

.ti-file-chart:before {
  content: "\f036";
}

.ti-file-check:before {
  content: "\ea9c";
}

.ti-file-check-filled:before {
  content: "\1012e";
}

.ti-file-code:before {
  content: "\ebd0";
}

.ti-file-code-2:before {
  content: "\ede8";
}

.ti-file-code-2-filled:before {
  content: "\1012d";
}

.ti-file-code-filled:before {
  content: "\10170";
}

.ti-file-cv:before {
  content: "\fa5a";
}

.ti-file-cv-filled:before {
  content: "\1012c";
}

.ti-file-database:before {
  content: "\f037";
}

.ti-file-delta:before {
  content: "\f53d";
}

.ti-file-delta-filled:before {
  content: "\1012b";
}

.ti-file-description:before {
  content: "\f028";
}

.ti-file-description-filled:before {
  content: "\1011e";
}

.ti-file-diff:before {
  content: "\ecf1";
}

.ti-file-diff-filled:before {
  content: "\1016f";
}

.ti-file-digit:before {
  content: "\efa8";
}

.ti-file-digit-filled:before {
  content: "\1011d";
}

.ti-file-dislike:before {
  content: "\ed2a";
}

.ti-file-dollar:before {
  content: "\efe0";
}

.ti-file-dots:before {
  content: "\f038";
}

.ti-file-dots-filled:before {
  content: "\1016e";
}

.ti-file-download:before {
  content: "\ea9d";
}

.ti-file-download-filled:before {
  content: "\1012a";
}

.ti-file-euro:before {
  content: "\efe1";
}

.ti-file-excel:before {
  content: "\fef3";
}

.ti-file-export:before {
  content: "\ede9";
}

.ti-file-filled:before {
  content: "\f747";
}

.ti-file-function:before {
  content: "\f53e";
}

.ti-file-function-filled:before {
  content: "\1016d";
}

.ti-file-horizontal:before {
  content: "\ebb0";
}

.ti-file-horizontal-filled:before {
  content: "\1011c";
}

.ti-file-import:before {
  content: "\edea";
}

.ti-file-infinity:before {
  content: "\f502";
}

.ti-file-info:before {
  content: "\edec";
}

.ti-file-info-filled:before {
  content: "\1016c";
}

.ti-file-invoice:before {
  content: "\eb67";
}

.ti-file-invoice-filled:before {
  content: "\1011b";
}

.ti-file-isr:before {
  content: "\feac";
}

.ti-file-lambda:before {
  content: "\f53f";
}

.ti-file-lambda-filled:before {
  content: "\10129";
}

.ti-file-like:before {
  content: "\ed2b";
}

.ti-file-minus:before {
  content: "\ea9e";
}

.ti-file-minus-filled:before {
  content: "\1011a";
}

.ti-file-music:before {
  content: "\ea9f";
}

.ti-file-neutral:before {
  content: "\fd22";
}

.ti-file-neutral-filled:before {
  content: "\10119";
}

.ti-file-off:before {
  content: "\ecf2";
}

.ti-file-orientation:before {
  content: "\f2a1";
}

.ti-file-pencil:before {
  content: "\f039";
}

.ti-file-percent:before {
  content: "\f540";
}

.ti-file-percent-filled:before {
  content: "\10128";
}

.ti-file-phone:before {
  content: "\ecdc";
}

.ti-file-phone-filled:before {
  content: "\10127";
}

.ti-file-plus:before {
  content: "\eaa0";
}

.ti-file-power:before {
  content: "\f03a";
}

.ti-file-power-filled:before {
  content: "\10118";
}

.ti-file-report:before {
  content: "\eded";
}

.ti-file-rss:before {
  content: "\f03b";
}

.ti-file-rss-filled:before {
  content: "\10126";
}

.ti-file-sad:before {
  content: "\fd23";
}

.ti-file-sad-filled:before {
  content: "\10117";
}

.ti-file-scissors:before {
  content: "\f03c";
}

.ti-file-search:before {
  content: "\ed5d";
}

.ti-file-settings:before {
  content: "\f029";
}

.ti-file-shredder:before {
  content: "\eaa1";
}

.ti-file-signal:before {
  content: "\f03d";
}

.ti-file-smile:before {
  content: "\fd24";
}

.ti-file-smile-filled:before {
  content: "\10116";
}

.ti-file-spark:before {
  content: "\ffb9";
}

.ti-file-spreadsheet:before {
  content: "\f03e";
}

.ti-file-stack:before {
  content: "\f503";
}

.ti-file-star:before {
  content: "\f03f";
}

.ti-file-star-filled:before {
  content: "\10115";
}

.ti-file-symlink:before {
  content: "\ed53";
}

.ti-file-text:before {
  content: "\eaa2";
}

.ti-file-text-ai:before {
  content: "\fa27";
}

.ti-file-text-filled:before {
  content: "\10114";
}

.ti-file-text-shield:before {
  content: "\100f2";
}

.ti-file-text-spark:before {
  content: "\ffb8";
}

.ti-file-time:before {
  content: "\f040";
}

.ti-file-type-bmp:before {
  content: "\fb07";
}

.ti-file-type-css:before {
  content: "\fb08";
}

.ti-file-type-csv:before {
  content: "\fb09";
}

.ti-file-type-doc:before {
  content: "\fb0a";
}

.ti-file-type-docx:before {
  content: "\fb0b";
}

.ti-file-type-html:before {
  content: "\fb0c";
}

.ti-file-type-jpg:before {
  content: "\fb0d";
}

.ti-file-type-js:before {
  content: "\fb0e";
}

.ti-file-type-jsx:before {
  content: "\fb0f";
}

.ti-file-type-pdf:before {
  content: "\fb10";
}

.ti-file-type-php:before {
  content: "\fb11";
}

.ti-file-type-png:before {
  content: "\fb12";
}

.ti-file-type-ppt:before {
  content: "\fb13";
}

.ti-file-type-rs:before {
  content: "\fb14";
}

.ti-file-type-sql:before {
  content: "\fb15";
}

.ti-file-type-svg:before {
  content: "\fb16";
}

.ti-file-type-ts:before {
  content: "\fb17";
}

.ti-file-type-tsx:before {
  content: "\fb18";
}

.ti-file-type-txt:before {
  content: "\fb19";
}

.ti-file-type-vue:before {
  content: "\fb1a";
}

.ti-file-type-xls:before {
  content: "\fb1b";
}

.ti-file-type-xml:before {
  content: "\fb1c";
}

.ti-file-type-zip:before {
  content: "\fb1d";
}

.ti-file-typography:before {
  content: "\f041";
}

.ti-file-typography-filled:before {
  content: "\1016b";
}

.ti-file-unknown:before {
  content: "\f042";
}

.ti-file-upload:before {
  content: "\ec91";
}

.ti-file-vector:before {
  content: "\f043";
}

.ti-file-word:before {
  content: "\fef2";
}

.ti-file-x:before {
  content: "\eaa3";
}

.ti-file-x-filled:before {
  content: "\f748";
}

.ti-file-zip:before {
  content: "\ed4e";
}

.ti-files:before {
  content: "\edef";
}

.ti-files-off:before {
  content: "\edee";
}

.ti-filter:before {
  content: "\eaa5";
}

.ti-filter-2:before {
  content: "\1014b";
}

.ti-filter-2-bolt:before {
  content: "\1015f";
}

.ti-filter-2-cancel:before {
  content: "\1015e";
}

.ti-filter-2-check:before {
  content: "\1015d";
}

.ti-filter-2-code:before {
  content: "\1015c";
}

.ti-filter-2-cog:before {
  content: "\1015b";
}

.ti-filter-2-discount:before {
  content: "\1015a";
}

.ti-filter-2-dollar:before {
  content: "\10159";
}

.ti-filter-2-down:before {
  content: "\10158";
}

.ti-filter-2-edit:before {
  content: "\10157";
}

.ti-filter-2-exclamation:before {
  content: "\10156";
}

.ti-filter-2-minus:before {
  content: "\10155";
}

.ti-filter-2-pause:before {
  content: "\10154";
}

.ti-filter-2-pin:before {
  content: "\10153";
}

.ti-filter-2-plus:before {
  content: "\10152";
}

.ti-filter-2-question:before {
  content: "\10151";
}

.ti-filter-2-search:before {
  content: "\10150";
}

.ti-filter-2-share:before {
  content: "\1014f";
}

.ti-filter-2-spark:before {
  content: "\1014e";
}

.ti-filter-2-up:before {
  content: "\1014d";
}

.ti-filter-2-x:before {
  content: "\1014c";
}

.ti-filter-bolt:before {
  content: "\fb7c";
}

.ti-filter-cancel:before {
  content: "\fb7d";
}

.ti-filter-check:before {
  content: "\fb7e";
}

.ti-filter-code:before {
  content: "\fb7f";
}

.ti-filter-cog:before {
  content: "\f9fe";
}

.ti-filter-discount:before {
  content: "\fb80";
}

.ti-filter-dollar:before {
  content: "\f9ff";
}

.ti-filter-down:before {
  content: "\fb81";
}

.ti-filter-edit:before {
  content: "\fa00";
}

.ti-filter-exclamation:before {
  content: "\fb82";
}

.ti-filter-filled:before {
  content: "\fc27";
}

.ti-filter-heart:before {
  content: "\fb83";
}

.ti-filter-minus:before {
  content: "\fa01";
}

.ti-filter-off:before {
  content: "\ed2c";
}

.ti-filter-pause:before {
  content: "\fb84";
}

.ti-filter-pin:before {
  content: "\fb85";
}

.ti-filter-plus:before {
  content: "\fa02";
}

.ti-filter-question:before {
  content: "\fb86";
}

.ti-filter-search:before {
  content: "\fb87";
}

.ti-filter-share:before {
  content: "\fb88";
}

.ti-filter-spark:before {
  content: "\1014a";
}

.ti-filter-star:before {
  content: "\fa03";
}

.ti-filter-up:before {
  content: "\fb89";
}

.ti-filter-x:before {
  content: "\fa04";
}

.ti-filters:before {
  content: "\f793";
}

.ti-filters-filled:before {
  content: "\100c7";
}

.ti-fingerprint:before {
  content: "\ebd1";
}

.ti-fingerprint-off:before {
  content: "\f12a";
}

.ti-fingerprint-scan:before {
  content: "\fcb5";
}

.ti-fire-extinguisher:before {
  content: "\faf6";
}

.ti-fire-hydrant:before {
  content: "\f3a9";
}

.ti-fire-hydrant-off:before {
  content: "\f3ec";
}

.ti-firetruck:before {
  content: "\ebe8";
}

.ti-first-aid-kit:before {
  content: "\ef5f";
}

.ti-first-aid-kit-off:before {
  content: "\f3ed";
}

.ti-fish:before {
  content: "\ef2b";
}

.ti-fish-bone:before {
  content: "\f287";
}

.ti-fish-bone-filled:before {
  content: "\1010b";
}

.ti-fish-christianity:before {
  content: "\f58b";
}

.ti-fish-hook:before {
  content: "\f1f9";
}

.ti-fish-hook-off:before {
  content: "\f3ee";
}

.ti-fish-off:before {
  content: "\f12b";
}

.ti-flag:before {
  content: "\eaa6";
}

.ti-flag-2:before {
  content: "\ee8c";
}

.ti-flag-2-filled:before {
  content: "\f707";
}

.ti-flag-2-off:before {
  content: "\f12c";
}

.ti-flag-3:before {
  content: "\ee8d";
}

.ti-flag-3-filled:before {
  content: "\f708";
}

.ti-flag-bitcoin:before {
  content: "\ff3c";
}

.ti-flag-bolt:before {
  content: "\fb8a";
}

.ti-flag-cancel:before {
  content: "\fb8b";
}

.ti-flag-check:before {
  content: "\fb8c";
}

.ti-flag-code:before {
  content: "\fb8d";
}

.ti-flag-cog:before {
  content: "\fb8e";
}

.ti-flag-discount:before {
  content: "\fb8f";
}

.ti-flag-dollar:before {
  content: "\fb90";
}

.ti-flag-down:before {
  content: "\fb91";
}

.ti-flag-exclamation:before {
  content: "\fb92";
}

.ti-flag-filled:before {
  content: "\f67a";
}

.ti-flag-heart:before {
  content: "\fb93";
}

.ti-flag-minus:before {
  content: "\fb94";
}

.ti-flag-off:before {
  content: "\f12d";
}

.ti-flag-pause:before {
  content: "\fb95";
}

.ti-flag-pin:before {
  content: "\fb96";
}

.ti-flag-plus:before {
  content: "\fb97";
}

.ti-flag-question:before {
  content: "\fb98";
}

.ti-flag-search:before {
  content: "\fb99";
}

.ti-flag-share:before {
  content: "\fb9a";
}

.ti-flag-spark:before {
  content: "\ffb7";
}

.ti-flag-star:before {
  content: "\fb9b";
}

.ti-flag-up:before {
  content: "\fb9c";
}

.ti-flag-x:before {
  content: "\fb9d";
}

.ti-flame:before {
  content: "\ec2c";
}

.ti-flame-filled:before {
  content: "\100c6";
}

.ti-flame-off:before {
  content: "\f12e";
}

.ti-flare:before {
  content: "\ee8e";
}

.ti-flare-filled:before {
  content: "\100c5";
}

.ti-flask:before {
  content: "\ebd2";
}

.ti-flask-2:before {
  content: "\ef60";
}

.ti-flask-2-filled:before {
  content: "\fd12";
}

.ti-flask-2-off:before {
  content: "\f12f";
}

.ti-flask-filled:before {
  content: "\fd13";
}

.ti-flask-off:before {
  content: "\f130";
}

.ti-flip-flops:before {
  content: "\f564";
}

.ti-flip-horizontal:before {
  content: "\eaa7";
}

.ti-flip-vertical:before {
  content: "\eaa8";
}

.ti-float-center:before {
  content: "\ebb1";
}

.ti-float-left:before {
  content: "\ebb2";
}

.ti-float-none:before {
  content: "\ed13";
}

.ti-float-right:before {
  content: "\ebb3";
}

.ti-flower:before {
  content: "\eff6";
}

.ti-flower-filled:before {
  content: "\10010";
}

.ti-flower-off:before {
  content: "\f131";
}

.ti-focus:before {
  content: "\eb8d";
}

.ti-focus-2:before {
  content: "\ebd3";
}

.ti-focus-auto:before {
  content: "\fa62";
}

.ti-focus-centered:before {
  content: "\f02a";
}

.ti-fold:before {
  content: "\ed56";
}

.ti-fold-down:before {
  content: "\ed54";
}

.ti-fold-up:before {
  content: "\ed55";
}

.ti-folder:before {
  content: "\eaad";
}

.ti-folder-bolt:before {
  content: "\f90c";
}

.ti-folder-cancel:before {
  content: "\f90d";
}

.ti-folder-check:before {
  content: "\f90e";
}

.ti-folder-code:before {
  content: "\f90f";
}

.ti-folder-cog:before {
  content: "\f910";
}

.ti-folder-dollar:before {
  content: "\f911";
}

.ti-folder-down:before {
  content: "\f912";
}

.ti-folder-exclamation:before {
  content: "\f913";
}

.ti-folder-filled:before {
  content: "\f749";
}

.ti-folder-heart:before {
  content: "\f914";
}

.ti-folder-minus:before {
  content: "\eaaa";
}

.ti-folder-off:before {
  content: "\ed14";
}

.ti-folder-open:before {
  content: "\faf7";
}

.ti-folder-pause:before {
  content: "\f915";
}

.ti-folder-pin:before {
  content: "\f916";
}

.ti-folder-plus:before {
  content: "\eaab";
}

.ti-folder-question:before {
  content: "\f917";
}

.ti-folder-root:before {
  content: "\fd43";
}

.ti-folder-search:before {
  content: "\f918";
}

.ti-folder-share:before {
  content: "\f919";
}

.ti-folder-star:before {
  content: "\f91a";
}

.ti-folder-symlink:before {
  content: "\f91b";
}

.ti-folder-up:before {
  content: "\f91c";
}

.ti-folder-x:before {
  content: "\eaac";
}

.ti-folders:before {
  content: "\eaae";
}

.ti-folders-filled:before {
  content: "\100c4";
}

.ti-folders-off:before {
  content: "\f133";
}

.ti-forbid:before {
  content: "\ebd5";
}

.ti-forbid-2:before {
  content: "\ebd4";
}

.ti-forbid-2-filled:before {
  content: "\fc28";
}

.ti-forbid-filled:before {
  content: "\fc29";
}

.ti-forklift:before {
  content: "\ebe9";
}

.ti-forms:before {
  content: "\ee8f";
}

.ti-fountain:before {
  content: "\f09b";
}

.ti-fountain-filled:before {
  content: "\fc2a";
}

.ti-fountain-off:before {
  content: "\f134";
}

.ti-frame:before {
  content: "\eaaf";
}

.ti-frame-off:before {
  content: "\f135";
}

.ti-free-rights:before {
  content: "\efb6";
}

.ti-freeze-column:before {
  content: "\fa63";
}

.ti-freeze-row:before {
  content: "\fa65";
}

.ti-freeze-row-column:before {
  content: "\fa64";
}

.ti-fridge:before {
  content: "\f1fa";
}

.ti-fridge-off:before {
  content: "\f3ef";
}

.ti-friends:before {
  content: "\eab0";
}

.ti-friends-off:before {
  content: "\f136";
}

.ti-frustum:before {
  content: "\fa9f";
}

.ti-frustum-off:before {
  content: "\fa9d";
}

.ti-frustum-plus:before {
  content: "\fa9e";
}

.ti-function:before {
  content: "\f225";
}

.ti-function-filled:before {
  content: "\fc2b";
}

.ti-function-off:before {
  content: "\f3f0";
}

.ti-galaxy:before {
  content: "\fcb6";
}

.ti-garden-cart:before {
  content: "\f23e";
}

.ti-garden-cart-filled:before {
  content: "\100c3";
}

.ti-garden-cart-off:before {
  content: "\f3f1";
}

.ti-gas-station:before {
  content: "\ec7d";
}

.ti-gas-station-filled:before {
  content: "\100fa";
}

.ti-gas-station-off:before {
  content: "\f137";
}

.ti-gauge:before {
  content: "\eab1";
}

.ti-gauge-filled:before {
  content: "\fc2c";
}

.ti-gauge-off:before {
  content: "\f138";
}

.ti-gavel:before {
  content: "\ef90";
}

.ti-gender-agender:before {
  content: "\f0e1";
}

.ti-gender-androgyne:before {
  content: "\f0e2";
}

.ti-gender-bigender:before {
  content: "\f0e3";
}

.ti-gender-demiboy:before {
  content: "\f0e4";
}

.ti-gender-demigirl:before {
  content: "\f0e5";
}

.ti-gender-epicene:before {
  content: "\f0e6";
}

.ti-gender-female:before {
  content: "\f0e7";
}

.ti-gender-femme:before {
  content: "\f0e8";
}

.ti-gender-genderfluid:before {
  content: "\f0e9";
}

.ti-gender-genderless:before {
  content: "\f0ea";
}

.ti-gender-genderqueer:before {
  content: "\f0eb";
}

.ti-gender-hermaphrodite:before {
  content: "\f0ec";
}

.ti-gender-intergender:before {
  content: "\f0ed";
}

.ti-gender-male:before {
  content: "\f0ee";
}

.ti-gender-neutrois:before {
  content: "\f0ef";
}

.ti-gender-third:before {
  content: "\f0f0";
}

.ti-gender-transgender:before {
  content: "\f0f1";
}

.ti-gender-trasvesti:before {
  content: "\f0f2";
}

.ti-geometry:before {
  content: "\ee90";
}

.ti-ghost:before {
  content: "\eb8e";
}

.ti-ghost-2:before {
  content: "\f57c";
}

.ti-ghost-2-filled:before {
  content: "\f74a";
}

.ti-ghost-3:before {
  content: "\fc13";
}

.ti-ghost-3-filled:before {
  content: "\100a4";
}

.ti-ghost-filled:before {
  content: "\f74b";
}

.ti-ghost-off:before {
  content: "\f3f2";
}

.ti-gif:before {
  content: "\f257";
}

.ti-gift:before {
  content: "\eb68";
}

.ti-gift-card:before {
  content: "\f3aa";
}

.ti-gift-card-filled:before {
  content: "\fc2d";
}

.ti-gift-filled:before {
  content: "\fd14";
}

.ti-gift-off:before {
  content: "\f3f3";
}

.ti-git-branch:before {
  content: "\eab2";
}

.ti-git-branch-deleted:before {
  content: "\f57d";
}

.ti-git-cherry-pick:before {
  content: "\f57e";
}

.ti-git-commit:before {
  content: "\eab3";
}

.ti-git-compare:before {
  content: "\eab4";
}

.ti-git-fork:before {
  content: "\eb8f";
}

.ti-git-merge:before {
  content: "\eab5";
}

.ti-git-pull-request:before {
  content: "\eab6";
}

.ti-git-pull-request-closed:before {
  content: "\ef7f";
}

.ti-git-pull-request-draft:before {
  content: "\efb7";
}

.ti-gizmo:before {
  content: "\f02b";
}

.ti-glass:before {
  content: "\eab8";
}

.ti-glass-champagne:before {
  content: "\fd9c";
}

.ti-glass-cocktail:before {
  content: "\fd9d";
}

.ti-glass-filled:before {
  content: "\1000f";
}

.ti-glass-full:before {
  content: "\eab7";
}

.ti-glass-full-filled:before {
  content: "\fc2e";
}

.ti-glass-gin:before {
  content: "\fd9e";
}

.ti-glass-off:before {
  content: "\ee91";
}

.ti-globe:before {
  content: "\eab9";
}

.ti-globe-filled:before {
  content: "\fc2f";
}

.ti-globe-off:before {
  content: "\f139";
}

.ti-go-game:before {
  content: "\f512";
}

.ti-golf:before {
  content: "\ed8c";
}

.ti-golf-filled:before {
  content: "\100a3";
}

.ti-golf-off:before {
  content: "\f13a";
}

.ti-gps:before {
  content: "\ed7a";
}

.ti-gps-filled:before {
  content: "\fe48";
}

.ti-gradienter:before {
  content: "\f3ab";
}

.ti-grain:before {
  content: "\ee92";
}

.ti-graph:before {
  content: "\f288";
}

.ti-graph-filled:before {
  content: "\fd15";
}

.ti-graph-off:before {
  content: "\f3f4";
}

.ti-grave:before {
  content: "\f580";
}

.ti-grave-2:before {
  content: "\f57f";
}

.ti-grid-3x3:before {
  content: "\fca4";
}

.ti-grid-4x4:before {
  content: "\fca5";
}

.ti-grid-dots:before {
  content: "\eaba";
}

.ti-grid-goldenratio:before {
  content: "\fca6";
}

.ti-grid-pattern:before {
  content: "\efc9";
}

.ti-grid-pattern-filled:before {
  content: "\100c2";
}

.ti-grid-scan:before {
  content: "\fca7";
}

.ti-grill:before {
  content: "\efa9";
}

.ti-grill-fork:before {
  content: "\f35b";
}

.ti-grill-off:before {
  content: "\f3f5";
}

.ti-grill-spatula:before {
  content: "\f35c";
}

.ti-grip-horizontal:before {
  content: "\ec00";
}

.ti-grip-vertical:before {
  content: "\ec01";
}

.ti-growth:before {
  content: "\ee93";
}

.ti-guitar-pick:before {
  content: "\f4c6";
}

.ti-guitar-pick-filled:before {
  content: "\f67b";
}

.ti-gymnastics:before {
  content: "\fd44";
}

.ti-h-1:before {
  content: "\ec94";
}

.ti-h-2:before {
  content: "\ec95";
}

.ti-h-3:before {
  content: "\ec96";
}

.ti-h-4:before {
  content: "\ec97";
}

.ti-h-5:before {
  content: "\ec98";
}

.ti-h-6:before {
  content: "\ec99";
}

.ti-hammer:before {
  content: "\ef91";
}

.ti-hammer-off:before {
  content: "\f13c";
}

.ti-hand-click:before {
  content: "\ef4f";
}

.ti-hand-click-off:before {
  content: "\100f1";
}

.ti-hand-finger:before {
  content: "\ee94";
}

.ti-hand-finger-down:before {
  content: "\ff4f";
}

.ti-hand-finger-left:before {
  content: "\ff4e";
}

.ti-hand-finger-off:before {
  content: "\f13d";
}

.ti-hand-finger-right:before {
  content: "\ff4d";
}

.ti-hand-grab:before {
  content: "\f091";
}

.ti-hand-little-finger:before {
  content: "\ee95";
}

.ti-hand-love-you:before {
  content: "\ee97";
}

.ti-hand-middle-finger:before {
  content: "\ec2d";
}

.ti-hand-move:before {
  content: "\ef50";
}

.ti-hand-off:before {
  content: "\ed15";
}

.ti-hand-ring-finger:before {
  content: "\ee96";
}

.ti-hand-sanitizer:before {
  content: "\f5f4";
}

.ti-hand-stop:before {
  content: "\ec2e";
}

.ti-hand-three-fingers:before {
  content: "\ee98";
}

.ti-hand-two-fingers:before {
  content: "\ee99";
}

.ti-hanger:before {
  content: "\ee9a";
}

.ti-hanger-2:before {
  content: "\f09c";
}

.ti-hanger-2-filled:before {
  content: "\ff61";
}

.ti-hanger-off:before {
  content: "\f13e";
}

.ti-hash:before {
  content: "\eabc";
}

.ti-haze:before {
  content: "\efaa";
}

.ti-haze-moon:before {
  content: "\faf8";
}

.ti-hdr:before {
  content: "\fa7b";
}

.ti-heading:before {
  content: "\ee9b";
}

.ti-heading-off:before {
  content: "\f13f";
}

.ti-headphones:before {
  content: "\eabd";
}

.ti-headphones-filled:before {
  content: "\fa3c";
}

.ti-headphones-off:before {
  content: "\ed1d";
}

.ti-headset:before {
  content: "\eb90";
}

.ti-headset-off:before {
  content: "\f3f6";
}

.ti-health-recognition:before {
  content: "\f1fb";
}

.ti-heart:before {
  content: "\eabe";
}

.ti-heart-bitcoin:before {
  content: "\ff3b";
}

.ti-heart-bolt:before {
  content: "\fb9e";
}

.ti-heart-broken:before {
  content: "\ecba";
}

.ti-heart-broken-filled:before {
  content: "\1016a";
}

.ti-heart-cancel:before {
  content: "\fb9f";
}

.ti-heart-check:before {
  content: "\fba0";
}

.ti-heart-code:before {
  content: "\fba1";
}

.ti-heart-cog:before {
  content: "\fba2";
}

.ti-heart-discount:before {
  content: "\fba3";
}

.ti-heart-dollar:before {
  content: "\fba4";
}

.ti-heart-down:before {
  content: "\fba5";
}

.ti-heart-exclamation:before {
  content: "\fba6";
}

.ti-heart-filled:before {
  content: "\f67c";
}

.ti-heart-handshake:before {
  content: "\f0f3";
}

.ti-heart-minus:before {
  content: "\f140";
}

.ti-heart-off:before {
  content: "\f141";
}

.ti-heart-pause:before {
  content: "\fba7";
}

.ti-heart-pin:before {
  content: "\fba8";
}

.ti-heart-plus:before {
  content: "\f142";
}

.ti-heart-question:before {
  content: "\fba9";
}

.ti-heart-rate-monitor:before {
  content: "\ef61";
}

.ti-heart-search:before {
  content: "\fbaa";
}

.ti-heart-share:before {
  content: "\fbab";
}

.ti-heart-spark:before {
  content: "\ffb6";
}

.ti-heart-star:before {
  content: "\fbac";
}

.ti-heart-up:before {
  content: "\fbad";
}

.ti-heart-x:before {
  content: "\fbae";
}

.ti-heartbeat:before {
  content: "\ef92";
}

.ti-hearts:before {
  content: "\f387";
}

.ti-hearts-off:before {
  content: "\f3f7";
}

.ti-helicopter:before {
  content: "\ed8e";
}

.ti-helicopter-filled:before {
  content: "\100f9";
}

.ti-helicopter-landing:before {
  content: "\ed8d";
}

.ti-helicopter-landing-filled:before {
  content: "\100c1";
}

.ti-helmet:before {
  content: "\efca";
}

.ti-helmet-off:before {
  content: "\f143";
}

.ti-help:before {
  content: "\eabf";
}

.ti-help-circle:before {
  content: "\f91d";
}

.ti-help-circle-filled:before {
  content: "\fa3d";
}

.ti-help-hexagon:before {
  content: "\f7a8";
}

.ti-help-hexagon-filled:before {
  content: "\fa3e";
}

.ti-help-octagon:before {
  content: "\f7a9";
}

.ti-help-octagon-filled:before {
  content: "\fa3f";
}

.ti-help-off:before {
  content: "\f3f8";
}

.ti-help-small:before {
  content: "\f91e";
}

.ti-help-square:before {
  content: "\f920";
}

.ti-help-square-filled:before {
  content: "\fa40";
}

.ti-help-square-rounded:before {
  content: "\f91f";
}

.ti-help-square-rounded-filled:before {
  content: "\fa41";
}

.ti-help-triangle:before {
  content: "\f921";
}

.ti-help-triangle-filled:before {
  content: "\fa42";
}

.ti-hemisphere:before {
  content: "\faa2";
}

.ti-hemisphere-off:before {
  content: "\faa0";
}

.ti-hemisphere-plus:before {
  content: "\faa1";
}

.ti-hexagon:before {
  content: "\ec02";
}

.ti-hexagon-3d:before {
  content: "\f4c7";
}

.ti-hexagon-filled:before {
  content: "\f67d";
}

.ti-hexagon-letter-a:before {
  content: "\f463";
}

.ti-hexagon-letter-a-filled:before {
  content: "\fe47";
}

.ti-hexagon-letter-b:before {
  content: "\f464";
}

.ti-hexagon-letter-b-filled:before {
  content: "\fe46";
}

.ti-hexagon-letter-c:before {
  content: "\f465";
}

.ti-hexagon-letter-c-filled:before {
  content: "\fe45";
}

.ti-hexagon-letter-d:before {
  content: "\f466";
}

.ti-hexagon-letter-d-filled:before {
  content: "\fe44";
}

.ti-hexagon-letter-e:before {
  content: "\f467";
}

.ti-hexagon-letter-e-filled:before {
  content: "\fe43";
}

.ti-hexagon-letter-f:before {
  content: "\f468";
}

.ti-hexagon-letter-f-filled:before {
  content: "\fe42";
}

.ti-hexagon-letter-g:before {
  content: "\f469";
}

.ti-hexagon-letter-g-filled:before {
  content: "\fe41";
}

.ti-hexagon-letter-h:before {
  content: "\f46a";
}

.ti-hexagon-letter-h-filled:before {
  content: "\fe40";
}

.ti-hexagon-letter-i:before {
  content: "\f46b";
}

.ti-hexagon-letter-i-filled:before {
  content: "\fe3f";
}

.ti-hexagon-letter-j:before {
  content: "\f46c";
}

.ti-hexagon-letter-j-filled:before {
  content: "\fe3e";
}

.ti-hexagon-letter-k:before {
  content: "\f46d";
}

.ti-hexagon-letter-k-filled:before {
  content: "\fe3d";
}

.ti-hexagon-letter-l:before {
  content: "\f46e";
}

.ti-hexagon-letter-l-filled:before {
  content: "\fe3c";
}

.ti-hexagon-letter-m:before {
  content: "\f46f";
}

.ti-hexagon-letter-m-filled:before {
  content: "\fe3b";
}

.ti-hexagon-letter-n:before {
  content: "\f470";
}

.ti-hexagon-letter-n-filled:before {
  content: "\fe3a";
}

.ti-hexagon-letter-o:before {
  content: "\f471";
}

.ti-hexagon-letter-o-filled:before {
  content: "\fe39";
}

.ti-hexagon-letter-p:before {
  content: "\f472";
}

.ti-hexagon-letter-p-filled:before {
  content: "\fe38";
}

.ti-hexagon-letter-q:before {
  content: "\f473";
}

.ti-hexagon-letter-q-filled:before {
  content: "\fe37";
}

.ti-hexagon-letter-r:before {
  content: "\f474";
}

.ti-hexagon-letter-r-filled:before {
  content: "\fe36";
}

.ti-hexagon-letter-s:before {
  content: "\f475";
}

.ti-hexagon-letter-s-filled:before {
  content: "\fe35";
}

.ti-hexagon-letter-t:before {
  content: "\f476";
}

.ti-hexagon-letter-t-filled:before {
  content: "\fe34";
}

.ti-hexagon-letter-u:before {
  content: "\f477";
}

.ti-hexagon-letter-u-filled:before {
  content: "\fe33";
}

.ti-hexagon-letter-v:before {
  content: "\f4b3";
}

.ti-hexagon-letter-v-filled:before {
  content: "\fe32";
}

.ti-hexagon-letter-w:before {
  content: "\f478";
}

.ti-hexagon-letter-w-filled:before {
  content: "\fe31";
}

.ti-hexagon-letter-x:before {
  content: "\f479";
}

.ti-hexagon-letter-x-filled:before {
  content: "\fe30";
}

.ti-hexagon-letter-y:before {
  content: "\f47a";
}

.ti-hexagon-letter-y-filled:before {
  content: "\fe2f";
}

.ti-hexagon-letter-z:before {
  content: "\f47b";
}

.ti-hexagon-letter-z-filled:before {
  content: "\fe2e";
}

.ti-hexagon-minus:before {
  content: "\fc8f";
}

.ti-hexagon-minus-2:before {
  content: "\fc8e";
}

.ti-hexagon-minus-filled:before {
  content: "\fe2d";
}

.ti-hexagon-number-0:before {
  content: "\f459";
}

.ti-hexagon-number-0-filled:before {
  content: "\f74c";
}

.ti-hexagon-number-1:before {
  content: "\f45a";
}

.ti-hexagon-number-1-filled:before {
  content: "\f74d";
}

.ti-hexagon-number-2:before {
  content: "\f45b";
}

.ti-hexagon-number-2-filled:before {
  content: "\f74e";
}

.ti-hexagon-number-3:before {
  content: "\f45c";
}

.ti-hexagon-number-3-filled:before {
  content: "\f74f";
}

.ti-hexagon-number-4:before {
  content: "\f45d";
}

.ti-hexagon-number-4-filled:before {
  content: "\f750";
}

.ti-hexagon-number-5:before {
  content: "\f45e";
}

.ti-hexagon-number-5-filled:before {
  content: "\f751";
}

.ti-hexagon-number-6:before {
  content: "\f45f";
}

.ti-hexagon-number-6-filled:before {
  content: "\f752";
}

.ti-hexagon-number-7:before {
  content: "\f460";
}

.ti-hexagon-number-7-filled:before {
  content: "\f753";
}

.ti-hexagon-number-8:before {
  content: "\f461";
}

.ti-hexagon-number-8-filled:before {
  content: "\f754";
}

.ti-hexagon-number-9:before {
  content: "\f462";
}

.ti-hexagon-number-9-filled:before {
  content: "\f755";
}

.ti-hexagon-off:before {
  content: "\ee9c";
}

.ti-hexagon-plus:before {
  content: "\fc45";
}

.ti-hexagon-plus-2:before {
  content: "\fc90";
}

.ti-hexagon-plus-filled:before {
  content: "\fe2c";
}

.ti-hexagonal-prism:before {
  content: "\faa5";
}

.ti-hexagonal-prism-off:before {
  content: "\faa3";
}

.ti-hexagonal-prism-plus:before {
  content: "\faa4";
}

.ti-hexagonal-pyramid:before {
  content: "\faa8";
}

.ti-hexagonal-pyramid-off:before {
  content: "\faa6";
}

.ti-hexagonal-pyramid-plus:before {
  content: "\faa7";
}

.ti-hexagons:before {
  content: "\f09d";
}

.ti-hexagons-off:before {
  content: "\f3f9";
}

.ti-hierarchy:before {
  content: "\ee9e";
}

.ti-hierarchy-2:before {
  content: "\ee9d";
}

.ti-hierarchy-3:before {
  content: "\f289";
}

.ti-hierarchy-off:before {
  content: "\f3fa";
}

.ti-highlight:before {
  content: "\ef3f";
}

.ti-highlight-off:before {
  content: "\f144";
}

.ti-history:before {
  content: "\ebea";
}

.ti-history-off:before {
  content: "\f3fb";
}

.ti-history-toggle:before {
  content: "\f1fc";
}

.ti-home:before {
  content: "\eac1";
}

.ti-home-2:before {
  content: "\eac0";
}

.ti-home-bitcoin:before {
  content: "\ff3a";
}

.ti-home-bolt:before {
  content: "\f336";
}

.ti-home-cancel:before {
  content: "\f350";
}

.ti-home-check:before {
  content: "\f337";
}

.ti-home-cog:before {
  content: "\f338";
}

.ti-home-dollar:before {
  content: "\f339";
}

.ti-home-dot:before {
  content: "\f33a";
}

.ti-home-down:before {
  content: "\f33b";
}

.ti-home-eco:before {
  content: "\f351";
}

.ti-home-edit:before {
  content: "\f352";
}

.ti-home-exclamation:before {
  content: "\f33c";
}

.ti-home-filled:before {
  content: "\fe2b";
}

.ti-home-hand:before {
  content: "\f504";
}

.ti-home-heart:before {
  content: "\f353";
}

.ti-home-infinity:before {
  content: "\f505";
}

.ti-home-link:before {
  content: "\f354";
}

.ti-home-minus:before {
  content: "\f33d";
}

.ti-home-move:before {
  content: "\f33e";
}

.ti-home-off:before {
  content: "\f145";
}

.ti-home-plus:before {
  content: "\f33f";
}

.ti-home-question:before {
  content: "\f340";
}

.ti-home-ribbon:before {
  content: "\f355";
}

.ti-home-search:before {
  content: "\f341";
}

.ti-home-share:before {
  content: "\f342";
}

.ti-home-shield:before {
  content: "\f343";
}

.ti-home-signal:before {
  content: "\f356";
}

.ti-home-spark:before {
  content: "\ffb5";
}

.ti-home-star:before {
  content: "\f344";
}

.ti-home-stats:before {
  content: "\f345";
}

.ti-home-up:before {
  content: "\f346";
}

.ti-home-x:before {
  content: "\f347";
}

.ti-horse:before {
  content: "\fc46";
}

.ti-horse-toy:before {
  content: "\f28a";
}

.ti-horseshoe:before {
  content: "\fcb7";
}

.ti-hospital:before {
  content: "\fd59";
}

.ti-hospital-circle:before {
  content: "\fd58";
}

.ti-hospital-circle-filled:before {
  content: "\fed2";
}

.ti-hotel-service:before {
  content: "\ef80";
}

.ti-hourglass:before {
  content: "\ef93";
}

.ti-hourglass-empty:before {
  content: "\f146";
}

.ti-hourglass-filled:before {
  content: "\f756";
}

.ti-hourglass-high:before {
  content: "\f092";
}

.ti-hourglass-low:before {
  content: "\f093";
}

.ti-hourglass-off:before {
  content: "\f147";
}

.ti-hours-12:before {
  content: "\fc53";
}

.ti-hours-24:before {
  content: "\f5e7";
}

.ti-html:before {
  content: "\f7b1";
}

.ti-http-connect:before {
  content: "\fa28";
}

.ti-http-connect-off:before {
  content: "\100e7";
}

.ti-http-delete:before {
  content: "\fa29";
}

.ti-http-delete-off:before {
  content: "\100e6";
}

.ti-http-get:before {
  content: "\fa2a";
}

.ti-http-get-off:before {
  content: "\100e5";
}

.ti-http-head:before {
  content: "\fa2b";
}

.ti-http-head-off:before {
  content: "\100e4";
}

.ti-http-options:before {
  content: "\fa2c";
}

.ti-http-options-off:before {
  content: "\100e3";
}

.ti-http-patch:before {
  content: "\fa2d";
}

.ti-http-patch-off:before {
  content: "\100e2";
}

.ti-http-post:before {
  content: "\fa2e";
}

.ti-http-post-off:before {
  content: "\100e1";
}

.ti-http-put:before {
  content: "\fa2f";
}

.ti-http-put-off:before {
  content: "\100e0";
}

.ti-http-que:before {
  content: "\fa5b";
}

.ti-http-que-off:before {
  content: "\100df";
}

.ti-http-trace:before {
  content: "\fa30";
}

.ti-http-trace-off:before {
  content: "\100de";
}

.ti-ice-cream:before {
  content: "\eac2";
}

.ti-ice-cream-2:before {
  content: "\ee9f";
}

.ti-ice-cream-off:before {
  content: "\f148";
}

.ti-ice-skating:before {
  content: "\efcb";
}

.ti-icons:before {
  content: "\f1d4";
}

.ti-icons-filled:before {
  content: "\10070";
}

.ti-icons-off:before {
  content: "\f3fc";
}

.ti-id:before {
  content: "\eac3";
}

.ti-id-badge:before {
  content: "\eff7";
}

.ti-id-badge-2:before {
  content: "\f076";
}

.ti-id-badge-off:before {
  content: "\f3fd";
}

.ti-id-off:before {
  content: "\f149";
}

.ti-ikosaedr:before {
  content: "\fec6";
}

.ti-image-in-picture:before {
  content: "\fd9f";
}

.ti-inbox:before {
  content: "\eac4";
}

.ti-inbox-off:before {
  content: "\f14a";
}

.ti-indent-decrease:before {
  content: "\eb91";
}

.ti-indent-increase:before {
  content: "\eb92";
}

.ti-infinity:before {
  content: "\eb69";
}

.ti-infinity-off:before {
  content: "\f3fe";
}

.ti-info-circle:before {
  content: "\eac5";
}

.ti-info-circle-filled:before {
  content: "\f6d8";
}

.ti-info-hexagon:before {
  content: "\f7aa";
}

.ti-info-hexagon-filled:before {
  content: "\fa43";
}

.ti-info-octagon:before {
  content: "\f7ab";
}

.ti-info-octagon-filled:before {
  content: "\fa44";
}

.ti-info-small:before {
  content: "\f922";
}

.ti-info-square:before {
  content: "\eac6";
}

.ti-info-square-filled:before {
  content: "\fa45";
}

.ti-info-square-rounded:before {
  content: "\f635";
}

.ti-info-square-rounded-filled:before {
  content: "\f6d9";
}

.ti-info-triangle:before {
  content: "\f923";
}

.ti-info-triangle-filled:before {
  content: "\fa46";
}

.ti-inner-shadow-bottom:before {
  content: "\f520";
}

.ti-inner-shadow-bottom-filled:before {
  content: "\f757";
}

.ti-inner-shadow-bottom-left:before {
  content: "\f51e";
}

.ti-inner-shadow-bottom-left-filled:before {
  content: "\f758";
}

.ti-inner-shadow-bottom-right:before {
  content: "\f51f";
}

.ti-inner-shadow-bottom-right-filled:before {
  content: "\f759";
}

.ti-inner-shadow-left:before {
  content: "\f521";
}

.ti-inner-shadow-left-filled:before {
  content: "\f75a";
}

.ti-inner-shadow-right:before {
  content: "\f522";
}

.ti-inner-shadow-right-filled:before {
  content: "\f75b";
}

.ti-inner-shadow-top:before {
  content: "\f525";
}

.ti-inner-shadow-top-filled:before {
  content: "\f75c";
}

.ti-inner-shadow-top-left:before {
  content: "\f523";
}

.ti-inner-shadow-top-left-filled:before {
  content: "\f75d";
}

.ti-inner-shadow-top-right:before {
  content: "\f524";
}

.ti-inner-shadow-top-right-filled:before {
  content: "\f75e";
}

.ti-input-ai:before {
  content: "\fc5a";
}

.ti-input-check:before {
  content: "\fc5b";
}

.ti-input-search:before {
  content: "\f2a2";
}

.ti-input-spark:before {
  content: "\ffb4";
}

.ti-input-x:before {
  content: "\fc5c";
}

.ti-invoice:before {
  content: "\feab";
}

.ti-ironing:before {
  content: "\fa7c";
}

.ti-ironing-1:before {
  content: "\f2f4";
}

.ti-ironing-1-filled:before {
  content: "\1006f";
}

.ti-ironing-2:before {
  content: "\f2f5";
}

.ti-ironing-2-filled:before {
  content: "\1006e";
}

.ti-ironing-3:before {
  content: "\f2f6";
}

.ti-ironing-3-filled:before {
  content: "\1006d";
}

.ti-ironing-filled:before {
  content: "\fe2a";
}

.ti-ironing-off:before {
  content: "\f2f7";
}

.ti-ironing-steam:before {
  content: "\f2f9";
}

.ti-ironing-steam-filled:before {
  content: "\1006c";
}

.ti-ironing-steam-off:before {
  content: "\f2f8";
}

.ti-irregular-polyhedron:before {
  content: "\faab";
}

.ti-irregular-polyhedron-off:before {
  content: "\faa9";
}

.ti-irregular-polyhedron-plus:before {
  content: "\faaa";
}

.ti-italic:before {
  content: "\eb93";
}

.ti-jacket:before {
  content: "\f661";
}

.ti-jetpack:before {
  content: "\f581";
}

.ti-jetpack-filled:before {
  content: "\fe29";
}

.ti-jewish-star:before {
  content: "\f3ff";
}

.ti-jewish-star-filled:before {
  content: "\f67e";
}

.ti-join-bevel:before {
  content: "\ff4c";
}

.ti-join-round:before {
  content: "\ff4b";
}

.ti-join-straight:before {
  content: "\ff4a";
}

.ti-joker:before {
  content: "\1005f";
}

.ti-jpg:before {
  content: "\f3ac";
}

.ti-json:before {
  content: "\f7b2";
}

.ti-jump-rope:before {
  content: "\ed8f";
}

.ti-karate:before {
  content: "\ed32";
}

.ti-kayak:before {
  content: "\f1d6";
}

.ti-kerning:before {
  content: "\efb8";
}

.ti-key:before {
  content: "\eac7";
}

.ti-key-filled:before {
  content: "\fe28";
}

.ti-key-off:before {
  content: "\f14b";
}

.ti-keyboard:before {
  content: "\ebd6";
}

.ti-keyboard-filled:before {
  content: "\100a2";
}

.ti-keyboard-hide:before {
  content: "\ec7e";
}

.ti-keyboard-off:before {
  content: "\eea0";
}

.ti-keyboard-show:before {
  content: "\ec7f";
}

.ti-keyframe:before {
  content: "\f576";
}

.ti-keyframe-align-center:before {
  content: "\f582";
}

.ti-keyframe-align-center-filled:before {
  content: "\fc30";
}

.ti-keyframe-align-horizontal:before {
  content: "\f583";
}

.ti-keyframe-align-horizontal-filled:before {
  content: "\fc31";
}

.ti-keyframe-align-vertical:before {
  content: "\f584";
}

.ti-keyframe-align-vertical-filled:before {
  content: "\fc32";
}

.ti-keyframe-filled:before {
  content: "\fc33";
}

.ti-keyframes:before {
  content: "\f585";
}

.ti-keyframes-filled:before {
  content: "\fc34";
}

.ti-label:before {
  content: "\ff38";
}

.ti-label-filled:before {
  content: "\ff41";
}

.ti-label-important:before {
  content: "\ff49";
}

.ti-label-important-filled:before {
  content: "\ff60";
}

.ti-label-off:before {
  content: "\ff39";
}

.ti-ladder:before {
  content: "\efe2";
}

.ti-ladder-off:before {
  content: "\f14c";
}

.ti-ladle:before {
  content: "\fc14";
}

.ti-lambda:before {
  content: "\f541";
}

.ti-lamp:before {
  content: "\efab";
}

.ti-lamp-2:before {
  content: "\f09e";
}

.ti-lamp-off:before {
  content: "\f14d";
}

.ti-lane:before {
  content: "\faf9";
}

.ti-language:before {
  content: "\ebbe";
}

.ti-language-hiragana:before {
  content: "\ef77";
}

.ti-language-katakana:before {
  content: "\ef78";
}

.ti-language-off:before {
  content: "\f14e";
}

.ti-lasso:before {
  content: "\efac";
}

.ti-lasso-off:before {
  content: "\f14f";
}

.ti-lasso-polygon:before {
  content: "\f388";
}

.ti-lasso-polygon-filled:before {
  content: "\ff5f";
}

.ti-laurel-wreath:before {
  content: "\ff45";
}

.ti-laurel-wreath-1:before {
  content: "\ff48";
}

.ti-laurel-wreath-1-filled:before {
  content: "\10169";
}

.ti-laurel-wreath-2:before {
  content: "\ff47";
}

.ti-laurel-wreath-2-filled:before {
  content: "\10168";
}

.ti-laurel-wreath-3:before {
  content: "\ff46";
}

.ti-laurel-wreath-3-filled:before {
  content: "\10167";
}

.ti-laurel-wreath-filled:before {
  content: "\100c0";
}

.ti-layers-difference:before {
  content: "\eac8";
}

.ti-layers-intersect:before {
  content: "\eac9";
}

.ti-layers-intersect-2:before {
  content: "\eff8";
}

.ti-layers-linked:before {
  content: "\eea1";
}

.ti-layers-off:before {
  content: "\f150";
}

.ti-layers-selected:before {
  content: "\fea9";
}

.ti-layers-selected-bottom:before {
  content: "\feaa";
}

.ti-layers-subtract:before {
  content: "\eaca";
}

.ti-layers-union:before {
  content: "\eacb";
}

.ti-layout:before {
  content: "\eadb";
}

.ti-layout-2:before {
  content: "\eacc";
}

.ti-layout-2-filled:before {
  content: "\fe27";
}

.ti-layout-align-bottom:before {
  content: "\eacd";
}

.ti-layout-align-bottom-filled:before {
  content: "\fe26";
}

.ti-layout-align-center:before {
  content: "\eace";
}

.ti-layout-align-center-filled:before {
  content: "\fe25";
}

.ti-layout-align-left:before {
  content: "\eacf";
}

.ti-layout-align-left-filled:before {
  content: "\fe24";
}

.ti-layout-align-middle:before {
  content: "\ead0";
}

.ti-layout-align-middle-filled:before {
  content: "\fe23";
}

.ti-layout-align-right:before {
  content: "\ead1";
}

.ti-layout-align-right-filled:before {
  content: "\fe22";
}

.ti-layout-align-top:before {
  content: "\ead2";
}

.ti-layout-align-top-filled:before {
  content: "\fe21";
}

.ti-layout-board:before {
  content: "\ef95";
}

.ti-layout-board-filled:before {
  content: "\10182";
}

.ti-layout-board-split:before {
  content: "\ef94";
}

.ti-layout-board-split-filled:before {
  content: "\10183";
}

.ti-layout-bottombar:before {
  content: "\ead3";
}

.ti-layout-bottombar-collapse:before {
  content: "\f28b";
}

.ti-layout-bottombar-collapse-filled:before {
  content: "\fc35";
}

.ti-layout-bottombar-expand:before {
  content: "\f28c";
}

.ti-layout-bottombar-expand-filled:before {
  content: "\fc36";
}

.ti-layout-bottombar-filled:before {
  content: "\fc37";
}

.ti-layout-bottombar-inactive:before {
  content: "\fd45";
}

.ti-layout-cards:before {
  content: "\ec13";
}

.ti-layout-cards-filled:before {
  content: "\fe20";
}

.ti-layout-collage:before {
  content: "\f389";
}

.ti-layout-columns:before {
  content: "\ead4";
}

.ti-layout-dashboard:before {
  content: "\f02c";
}

.ti-layout-dashboard-filled:before {
  content: "\fe1f";
}

.ti-layout-distribute-horizontal:before {
  content: "\ead5";
}

.ti-layout-distribute-horizontal-filled:before {
  content: "\fe1e";
}

.ti-layout-distribute-vertical:before {
  content: "\ead6";
}

.ti-layout-distribute-vertical-filled:before {
  content: "\fe1d";
}

.ti-layout-filled:before {
  content: "\fe17";
}

.ti-layout-grid:before {
  content: "\edba";
}

.ti-layout-grid-add:before {
  content: "\edb9";
}

.ti-layout-grid-filled:before {
  content: "\fe1c";
}

.ti-layout-grid-remove:before {
  content: "\fa7d";
}

.ti-layout-kanban:before {
  content: "\ec3f";
}

.ti-layout-kanban-filled:before {
  content: "\fe1b";
}

.ti-layout-list:before {
  content: "\ec14";
}

.ti-layout-list-filled:before {
  content: "\fe1a";
}

.ti-layout-navbar:before {
  content: "\ead7";
}

.ti-layout-navbar-collapse:before {
  content: "\f28d";
}

.ti-layout-navbar-collapse-filled:before {
  content: "\fc38";
}

.ti-layout-navbar-expand:before {
  content: "\f28e";
}

.ti-layout-navbar-expand-filled:before {
  content: "\fc39";
}

.ti-layout-navbar-filled:before {
  content: "\fc3a";
}

.ti-layout-navbar-inactive:before {
  content: "\fd46";
}

.ti-layout-off:before {
  content: "\f151";
}

.ti-layout-rows:before {
  content: "\ead8";
}

.ti-layout-sidebar:before {
  content: "\eada";
}

.ti-layout-sidebar-filled:before {
  content: "\fe18";
}

.ti-layout-sidebar-inactive:before {
  content: "\fd47";
}

.ti-layout-sidebar-left-collapse:before {
  content: "\f004";
}

.ti-layout-sidebar-left-collapse-filled:before {
  content: "\fc3b";
}

.ti-layout-sidebar-left-expand:before {
  content: "\f005";
}

.ti-layout-sidebar-left-expand-filled:before {
  content: "\fc3c";
}

.ti-layout-sidebar-right:before {
  content: "\ead9";
}

.ti-layout-sidebar-right-collapse:before {
  content: "\f006";
}

.ti-layout-sidebar-right-collapse-filled:before {
  content: "\fc3d";
}

.ti-layout-sidebar-right-expand:before {
  content: "\f007";
}

.ti-layout-sidebar-right-expand-filled:before {
  content: "\fc3e";
}

.ti-layout-sidebar-right-filled:before {
  content: "\fe19";
}

.ti-layout-sidebar-right-inactive:before {
  content: "\fd48";
}

.ti-leaf:before {
  content: "\ed4f";
}

.ti-leaf-2:before {
  content: "\ff44";
}

.ti-leaf-off:before {
  content: "\f400";
}

.ti-lego:before {
  content: "\eadc";
}

.ti-lego-filled:before {
  content: "\fe16";
}

.ti-lego-off:before {
  content: "\f401";
}

.ti-lemon:before {
  content: "\ef10";
}

.ti-lemon-2:before {
  content: "\ef81";
}

.ti-lemon-2-filled:before {
  content: "\100bf";
}

.ti-letter-a:before {
  content: "\ec50";
}

.ti-letter-a-small:before {
  content: "\fcc7";
}

.ti-letter-b:before {
  content: "\ec51";
}

.ti-letter-b-small:before {
  content: "\fcc8";
}

.ti-letter-c:before {
  content: "\ec52";
}

.ti-letter-c-small:before {
  content: "\fcc9";
}

.ti-letter-case:before {
  content: "\eea5";
}

.ti-letter-case-lower:before {
  content: "\eea2";
}

.ti-letter-case-toggle:before {
  content: "\eea3";
}

.ti-letter-case-upper:before {
  content: "\eea4";
}

.ti-letter-d:before {
  content: "\ec53";
}

.ti-letter-d-small:before {
  content: "\fcca";
}

.ti-letter-e:before {
  content: "\ec54";
}

.ti-letter-e-small:before {
  content: "\fccb";
}

.ti-letter-f:before {
  content: "\ec55";
}

.ti-letter-f-small:before {
  content: "\fccc";
}

.ti-letter-g:before {
  content: "\ec56";
}

.ti-letter-g-small:before {
  content: "\fccd";
}

.ti-letter-h:before {
  content: "\ec57";
}

.ti-letter-h-small:before {
  content: "\fcce";
}

.ti-letter-i:before {
  content: "\ec58";
}

.ti-letter-i-small:before {
  content: "\fccf";
}

.ti-letter-j:before {
  content: "\ec59";
}

.ti-letter-j-small:before {
  content: "\fcd0";
}

.ti-letter-k:before {
  content: "\ec5a";
}

.ti-letter-k-small:before {
  content: "\fcd1";
}

.ti-letter-l:before {
  content: "\ec5b";
}

.ti-letter-l-small:before {
  content: "\fcd2";
}

.ti-letter-m:before {
  content: "\ec5c";
}

.ti-letter-m-small:before {
  content: "\fcd3";
}

.ti-letter-n:before {
  content: "\ec5d";
}

.ti-letter-n-small:before {
  content: "\fcd4";
}

.ti-letter-o:before {
  content: "\ec5e";
}

.ti-letter-o-small:before {
  content: "\fcd5";
}

.ti-letter-p:before {
  content: "\ec5f";
}

.ti-letter-p-small:before {
  content: "\fcd6";
}

.ti-letter-q:before {
  content: "\ec60";
}

.ti-letter-q-small:before {
  content: "\fcd7";
}

.ti-letter-r:before {
  content: "\ec61";
}

.ti-letter-r-small:before {
  content: "\fcd8";
}

.ti-letter-s:before {
  content: "\ec62";
}

.ti-letter-s-small:before {
  content: "\fcd9";
}

.ti-letter-spacing:before {
  content: "\eea6";
}

.ti-letter-t:before {
  content: "\ec63";
}

.ti-letter-t-small:before {
  content: "\fcda";
}

.ti-letter-u:before {
  content: "\ec64";
}

.ti-letter-u-small:before {
  content: "\fcdb";
}

.ti-letter-v:before {
  content: "\ec65";
}

.ti-letter-v-small:before {
  content: "\fcdc";
}

.ti-letter-w:before {
  content: "\ec66";
}

.ti-letter-w-small:before {
  content: "\fcdd";
}

.ti-letter-x:before {
  content: "\ec67";
}

.ti-letter-x-small:before {
  content: "\fcde";
}

.ti-letter-y:before {
  content: "\ec68";
}

.ti-letter-y-small:before {
  content: "\fcdf";
}

.ti-letter-z:before {
  content: "\ec69";
}

.ti-letter-z-small:before {
  content: "\fce0";
}

.ti-library:before {
  content: "\fd4c";
}

.ti-library-filled:before {
  content: "\10180";
}

.ti-library-minus:before {
  content: "\fd49";
}

.ti-library-photo:before {
  content: "\fd4a";
}

.ti-library-plus:before {
  content: "\fd4b";
}

.ti-library-plus-filled:before {
  content: "\10181";
}

.ti-license:before {
  content: "\ebc0";
}

.ti-license-off:before {
  content: "\f153";
}

.ti-lifebuoy:before {
  content: "\eadd";
}

.ti-lifebuoy-filled:before {
  content: "\100be";
}

.ti-lifebuoy-off:before {
  content: "\f154";
}

.ti-lighter:before {
  content: "\f794";
}

.ti-line:before {
  content: "\ec40";
}

.ti-line-dashed:before {
  content: "\eea7";
}

.ti-line-dotted:before {
  content: "\eea8";
}

.ti-line-height:before {
  content: "\eb94";
}

.ti-line-scan:before {
  content: "\fcb8";
}

.ti-link:before {
  content: "\eade";
}

.ti-link-minus:before {
  content: "\fd16";
}

.ti-link-off:before {
  content: "\f402";
}

.ti-link-plus:before {
  content: "\fd17";
}

.ti-list:before {
  content: "\eb6b";
}

.ti-list-check:before {
  content: "\eb6a";
}

.ti-list-details:before {
  content: "\ef40";
}

.ti-list-letters:before {
  content: "\fc47";
}

.ti-list-numbers:before {
  content: "\ef11";
}

.ti-list-search:before {
  content: "\eea9";
}

.ti-list-tree:before {
  content: "\fafa";
}

.ti-live-photo:before {
  content: "\eadf";
}

.ti-live-photo-filled:before {
  content: "\fed1";
}

.ti-live-photo-off:before {
  content: "\f403";
}

.ti-live-view:before {
  content: "\ec6b";
}

.ti-live-view-filled:before {
  content: "\100a1";
}

.ti-load-balancer:before {
  content: "\fa5c";
}

.ti-loader:before {
  content: "\eca3";
}

.ti-loader-2:before {
  content: "\f226";
}

.ti-loader-3:before {
  content: "\f513";
}

.ti-loader-quarter:before {
  content: "\eca2";
}

.ti-location:before {
  content: "\eae0";
}

.ti-location-bolt:before {
  content: "\fbaf";
}

.ti-location-broken:before {
  content: "\f2c4";
}

.ti-location-cancel:before {
  content: "\fbb0";
}

.ti-location-check:before {
  content: "\fbb1";
}

.ti-location-code:before {
  content: "\fbb2";
}

.ti-location-cog:before {
  content: "\fbb3";
}

.ti-location-discount:before {
  content: "\fbb4";
}

.ti-location-dollar:before {
  content: "\fbb5";
}

.ti-location-down:before {
  content: "\fbb6";
}

.ti-location-exclamation:before {
  content: "\fbb7";
}

.ti-location-filled:before {
  content: "\f67f";
}

.ti-location-heart:before {
  content: "\fbb8";
}

.ti-location-minus:before {
  content: "\fbb9";
}

.ti-location-off:before {
  content: "\f155";
}

.ti-location-pause:before {
  content: "\fbba";
}

.ti-location-pin:before {
  content: "\fbbb";
}

.ti-location-plus:before {
  content: "\fbbc";
}

.ti-location-question:before {
  content: "\fbbd";
}

.ti-location-search:before {
  content: "\fbbe";
}

.ti-location-share:before {
  content: "\fbbf";
}

.ti-location-star:before {
  content: "\fbc0";
}

.ti-location-up:before {
  content: "\fbc1";
}

.ti-location-x:before {
  content: "\fbc2";
}

.ti-lock:before {
  content: "\eae2";
}

.ti-lock-access:before {
  content: "\eeaa";
}

.ti-lock-access-off:before {
  content: "\f404";
}

.ti-lock-bitcoin:before {
  content: "\ff37";
}

.ti-lock-bolt:before {
  content: "\f924";
}

.ti-lock-cancel:before {
  content: "\f925";
}

.ti-lock-check:before {
  content: "\f926";
}

.ti-lock-code:before {
  content: "\f927";
}

.ti-lock-cog:before {
  content: "\f928";
}

.ti-lock-dollar:before {
  content: "\f929";
}

.ti-lock-down:before {
  content: "\f92a";
}

.ti-lock-exclamation:before {
  content: "\f92b";
}

.ti-lock-filled:before {
  content: "\fe15";
}

.ti-lock-heart:before {
  content: "\f92c";
}

.ti-lock-minus:before {
  content: "\f92d";
}

.ti-lock-off:before {
  content: "\ed1e";
}

.ti-lock-open:before {
  content: "\eae1";
}

.ti-lock-open-2:before {
  content: "\fea8";
}

.ti-lock-open-off:before {
  content: "\f156";
}

.ti-lock-password:before {
  content: "\ff9f";
}

.ti-lock-pause:before {
  content: "\f92e";
}

.ti-lock-pin:before {
  content: "\f92f";
}

.ti-lock-plus:before {
  content: "\f930";
}

.ti-lock-question:before {
  content: "\f931";
}

.ti-lock-search:before {
  content: "\f932";
}

.ti-lock-share:before {
  content: "\f933";
}

.ti-lock-square:before {
  content: "\ef51";
}

.ti-lock-square-rounded:before {
  content: "\f636";
}

.ti-lock-square-rounded-filled:before {
  content: "\f6da";
}

.ti-lock-star:before {
  content: "\f934";
}

.ti-lock-up:before {
  content: "\f935";
}

.ti-lock-x:before {
  content: "\f936";
}

.ti-logic-and:before {
  content: "\f240";
}

.ti-logic-buffer:before {
  content: "\f241";
}

.ti-logic-nand:before {
  content: "\f242";
}

.ti-logic-nor:before {
  content: "\f243";
}

.ti-logic-not:before {
  content: "\f244";
}

.ti-logic-or:before {
  content: "\f245";
}

.ti-logic-xnor:before {
  content: "\f246";
}

.ti-logic-xor:before {
  content: "\f247";
}

.ti-login:before {
  content: "\eba7";
}

.ti-login-2:before {
  content: "\fc76";
}

.ti-logout:before {
  content: "\eba8";
}

.ti-logout-2:before {
  content: "\fa7e";
}

.ti-logs:before {
  content: "\fea7";
}

.ti-lollipop:before {
  content: "\efcc";
}

.ti-lollipop-off:before {
  content: "\f157";
}

.ti-luggage:before {
  content: "\efad";
}

.ti-luggage-off:before {
  content: "\f158";
}

.ti-lungs:before {
  content: "\ef62";
}

.ti-lungs-filled:before {
  content: "\fe14";
}

.ti-lungs-off:before {
  content: "\f405";
}

.ti-macro:before {
  content: "\eeab";
}

.ti-macro-filled:before {
  content: "\fe13";
}

.ti-macro-off:before {
  content: "\f406";
}

.ti-magnet:before {
  content: "\eae3";
}

.ti-magnet-filled:before {
  content: "\fe12";
}

.ti-magnet-off:before {
  content: "\f159";
}

.ti-magnetic:before {
  content: "\fcb9";
}

.ti-mail:before {
  content: "\eae5";
}

.ti-mail-ai:before {
  content: "\fa31";
}

.ti-mail-bitcoin:before {
  content: "\ff36";
}

.ti-mail-bolt:before {
  content: "\f937";
}

.ti-mail-cancel:before {
  content: "\f938";
}

.ti-mail-check:before {
  content: "\f939";
}

.ti-mail-code:before {
  content: "\f93a";
}

.ti-mail-cog:before {
  content: "\f93b";
}

.ti-mail-dollar:before {
  content: "\f93c";
}

.ti-mail-down:before {
  content: "\f93d";
}

.ti-mail-exclamation:before {
  content: "\f93e";
}

.ti-mail-fast:before {
  content: "\f069";
}

.ti-mail-filled:before {
  content: "\fa47";
}

.ti-mail-forward:before {
  content: "\eeac";
}

.ti-mail-heart:before {
  content: "\f93f";
}

.ti-mail-minus:before {
  content: "\f940";
}

.ti-mail-off:before {
  content: "\f15a";
}

.ti-mail-opened:before {
  content: "\eae4";
}

.ti-mail-opened-filled:before {
  content: "\fa48";
}

.ti-mail-pause:before {
  content: "\f941";
}

.ti-mail-pin:before {
  content: "\f942";
}

.ti-mail-plus:before {
  content: "\f943";
}

.ti-mail-question:before {
  content: "\f944";
}

.ti-mail-search:before {
  content: "\f945";
}

.ti-mail-share:before {
  content: "\f946";
}

.ti-mail-spark:before {
  content: "\ffb3";
}

.ti-mail-star:before {
  content: "\f947";
}

.ti-mail-up:before {
  content: "\f948";
}

.ti-mail-x:before {
  content: "\f949";
}

.ti-mailbox:before {
  content: "\eead";
}

.ti-mailbox-off:before {
  content: "\f15b";
}

.ti-man:before {
  content: "\eae6";
}

.ti-man-filled:before {
  content: "\fe11";
}

.ti-manual-gearbox:before {
  content: "\ed7b";
}

.ti-manual-gearbox-filled:before {
  content: "\fe10";
}

.ti-map:before {
  content: "\eae9";
}

.ti-map-2:before {
  content: "\eae7";
}

.ti-map-bolt:before {
  content: "\fbc3";
}

.ti-map-cancel:before {
  content: "\fbc4";
}

.ti-map-check:before {
  content: "\fbc5";
}

.ti-map-code:before {
  content: "\fbc6";
}

.ti-map-cog:before {
  content: "\fbc7";
}

.ti-map-discount:before {
  content: "\fbc8";
}

.ti-map-dollar:before {
  content: "\fbc9";
}

.ti-map-down:before {
  content: "\fbca";
}

.ti-map-east:before {
  content: "\fc5d";
}

.ti-map-exclamation:before {
  content: "\fbcb";
}

.ti-map-heart:before {
  content: "\fbcc";
}

.ti-map-minus:before {
  content: "\fbcd";
}

.ti-map-north:before {
  content: "\fc5e";
}

.ti-map-off:before {
  content: "\f15c";
}

.ti-map-pause:before {
  content: "\fbce";
}

.ti-map-pin:before {
  content: "\eae8";
}

.ti-map-pin-2:before {
  content: "\fc48";
}

.ti-map-pin-bolt:before {
  content: "\f94a";
}

.ti-map-pin-cancel:before {
  content: "\f94b";
}

.ti-map-pin-check:before {
  content: "\f94c";
}

.ti-map-pin-code:before {
  content: "\f94d";
}

.ti-map-pin-cog:before {
  content: "\f94e";
}

.ti-map-pin-dollar:before {
  content: "\f94f";
}

.ti-map-pin-down:before {
  content: "\f950";
}

.ti-map-pin-exclamation:before {
  content: "\f951";
}

.ti-map-pin-filled:before {
  content: "\f680";
}

.ti-map-pin-heart:before {
  content: "\f952";
}

.ti-map-pin-minus:before {
  content: "\f953";
}

.ti-map-pin-off:before {
  content: "\ecf3";
}

.ti-map-pin-pause:before {
  content: "\f954";
}

.ti-map-pin-pin:before {
  content: "\f955";
}

.ti-map-pin-plus:before {
  content: "\f956";
}

.ti-map-pin-question:before {
  content: "\f957";
}

.ti-map-pin-search:before {
  content: "\f958";
}

.ti-map-pin-share:before {
  content: "\f795";
}

.ti-map-pin-star:before {
  content: "\f959";
}

.ti-map-pin-up:before {
  content: "\f95a";
}

.ti-map-pin-x:before {
  content: "\f95b";
}

.ti-map-pins:before {
  content: "\ed5e";
}

.ti-map-plus:before {
  content: "\fbcf";
}

.ti-map-question:before {
  content: "\fbd0";
}

.ti-map-route:before {
  content: "\fc79";
}

.ti-map-search:before {
  content: "\ef82";
}

.ti-map-share:before {
  content: "\fbd1";
}

.ti-map-south:before {
  content: "\fc5f";
}

.ti-map-star:before {
  content: "\fbd2";
}

.ti-map-up:before {
  content: "\fbd3";
}

.ti-map-west:before {
  content: "\fc60";
}

.ti-map-x:before {
  content: "\fbd4";
}

.ti-markdown:before {
  content: "\ec41";
}

.ti-markdown-off:before {
  content: "\f407";
}

.ti-marquee:before {
  content: "\ec77";
}

.ti-marquee-2:before {
  content: "\eeae";
}

.ti-marquee-off:before {
  content: "\f15d";
}

.ti-mars:before {
  content: "\ec80";
}

.ti-mask:before {
  content: "\eeb0";
}

.ti-mask-off:before {
  content: "\eeaf";
}

.ti-masks-theater:before {
  content: "\f263";
}

.ti-masks-theater-off:before {
  content: "\f408";
}

.ti-massage:before {
  content: "\eeb1";
}

.ti-matchstick:before {
  content: "\f577";
}

.ti-math:before {
  content: "\ebeb";
}

.ti-math-1-divide-2:before {
  content: "\f4e2";
}

.ti-math-1-divide-3:before {
  content: "\f4e3";
}

.ti-math-avg:before {
  content: "\f0f4";
}

.ti-math-cos:before {
  content: "\ff1f";
}

.ti-math-ctg:before {
  content: "\ff35";
}

.ti-math-equal-greater:before {
  content: "\f4e4";
}

.ti-math-equal-lower:before {
  content: "\f4e5";
}

.ti-math-function:before {
  content: "\eeb2";
}

.ti-math-function-off:before {
  content: "\f15e";
}

.ti-math-function-y:before {
  content: "\f4e6";
}

.ti-math-greater:before {
  content: "\f4e7";
}

.ti-math-integral:before {
  content: "\f4e9";
}

.ti-math-integral-x:before {
  content: "\f4e8";
}

.ti-math-integrals:before {
  content: "\f4ea";
}

.ti-math-lower:before {
  content: "\f4eb";
}

.ti-math-max:before {
  content: "\f0f5";
}

.ti-math-max-min:before {
  content: "\fda0";
}

.ti-math-min:before {
  content: "\f0f6";
}

.ti-math-not:before {
  content: "\f4ec";
}

.ti-math-off:before {
  content: "\f409";
}

.ti-math-pi:before {
  content: "\f4ee";
}

.ti-math-pi-divide-2:before {
  content: "\f4ed";
}

.ti-math-sec:before {
  content: "\ff34";
}

.ti-math-sin:before {
  content: "\ff1e";
}

.ti-math-symbols:before {
  content: "\eeb3";
}

.ti-math-tg:before {
  content: "\ff33";
}

.ti-math-x-divide-2:before {
  content: "\f4ef";
}

.ti-math-x-divide-y:before {
  content: "\f4f1";
}

.ti-math-x-divide-y-2:before {
  content: "\f4f0";
}

.ti-math-x-floor-divide-y:before {
  content: "\10073";
}

.ti-math-x-minus-x:before {
  content: "\f4f2";
}

.ti-math-x-minus-y:before {
  content: "\f4f3";
}

.ti-math-x-plus-x:before {
  content: "\f4f4";
}

.ti-math-x-plus-y:before {
  content: "\f4f5";
}

.ti-math-xy:before {
  content: "\f4f6";
}

.ti-math-y-minus-y:before {
  content: "\f4f7";
}

.ti-math-y-plus-y:before {
  content: "\f4f8";
}

.ti-matrix:before {
  content: "\100bc";
}

.ti-maximize:before {
  content: "\eaea";
}

.ti-maximize-off:before {
  content: "\f15f";
}

.ti-meat:before {
  content: "\ef12";
}

.ti-meat-off:before {
  content: "\f40a";
}

.ti-medal:before {
  content: "\ec78";
}

.ti-medal-2:before {
  content: "\efcd";
}

.ti-medical-cross:before {
  content: "\ec2f";
}

.ti-medical-cross-circle:before {
  content: "\fae8";
}

.ti-medical-cross-filled:before {
  content: "\f681";
}

.ti-medical-cross-off:before {
  content: "\f160";
}

.ti-medicine-syrup:before {
  content: "\ef63";
}

.ti-meeple:before {
  content: "\f514";
}

.ti-meeple-filled:before {
  content: "\100a0";
}

.ti-melon:before {
  content: "\fc7a";
}

.ti-melon-filled:before {
  content: "\1000e";
}

.ti-menorah:before {
  content: "\f58c";
}

.ti-menu:before {
  content: "\eaeb";
}

.ti-menu-2:before {
  content: "\ec42";
}

.ti-menu-3:before {
  content: "\ff43";
}

.ti-menu-4:before {
  content: "\ff42";
}

.ti-menu-deep:before {
  content: "\fafb";
}

.ti-menu-order:before {
  content: "\f5f5";
}

.ti-message:before {
  content: "\eaef";
}

.ti-message-2:before {
  content: "\eaec";
}

.ti-message-2-bolt:before {
  content: "\f95c";
}

.ti-message-2-cancel:before {
  content: "\f95d";
}

.ti-message-2-check:before {
  content: "\f95e";
}

.ti-message-2-code:before {
  content: "\f012";
}

.ti-message-2-cog:before {
  content: "\f95f";
}

.ti-message-2-dollar:before {
  content: "\f960";
}

.ti-message-2-down:before {
  content: "\f961";
}

.ti-message-2-exclamation:before {
  content: "\f962";
}

.ti-message-2-filled:before {
  content: "\1009f";
}

.ti-message-2-heart:before {
  content: "\f963";
}

.ti-message-2-minus:before {
  content: "\f964";
}

.ti-message-2-off:before {
  content: "\f40b";
}

.ti-message-2-pause:before {
  content: "\f965";
}

.ti-message-2-pin:before {
  content: "\f966";
}

.ti-message-2-plus:before {
  content: "\f967";
}

.ti-message-2-question:before {
  content: "\f968";
}

.ti-message-2-search:before {
  content: "\f969";
}

.ti-message-2-share:before {
  content: "\f077";
}

.ti-message-2-star:before {
  content: "\f96a";
}

.ti-message-2-up:before {
  content: "\f96b";
}

.ti-message-2-x:before {
  content: "\f96c";
}

.ti-message-bolt:before {
  content: "\f96d";
}

.ti-message-cancel:before {
  content: "\f96e";
}

.ti-message-chatbot:before {
  content: "\f38a";
}

.ti-message-chatbot-filled:before {
  content: "\fed0";
}

.ti-message-check:before {
  content: "\f96f";
}

.ti-message-circle:before {
  content: "\eaed";
}

.ti-message-circle-bolt:before {
  content: "\f970";
}

.ti-message-circle-cancel:before {
  content: "\f971";
}

.ti-message-circle-check:before {
  content: "\f972";
}

.ti-message-circle-code:before {
  content: "\f973";
}

.ti-message-circle-cog:before {
  content: "\f974";
}

.ti-message-circle-dollar:before {
  content: "\f975";
}

.ti-message-circle-down:before {
  content: "\f976";
}

.ti-message-circle-exclamation:before {
  content: "\f977";
}

.ti-message-circle-filled:before {
  content: "\fecf";
}

.ti-message-circle-heart:before {
  content: "\f978";
}

.ti-message-circle-minus:before {
  content: "\f979";
}

.ti-message-circle-off:before {
  content: "\ed40";
}

.ti-message-circle-pause:before {
  content: "\f97a";
}

.ti-message-circle-pin:before {
  content: "\f97b";
}

.ti-message-circle-plus:before {
  content: "\f97c";
}

.ti-message-circle-question:before {
  content: "\f97d";
}

.ti-message-circle-search:before {
  content: "\f97e";
}

.ti-message-circle-share:before {
  content: "\f97f";
}

.ti-message-circle-star:before {
  content: "\f980";
}

.ti-message-circle-up:before {
  content: "\f981";
}

.ti-message-circle-user:before {
  content: "\fec5";
}

.ti-message-circle-x:before {
  content: "\f982";
}

.ti-message-code:before {
  content: "\f013";
}

.ti-message-cog:before {
  content: "\f983";
}

.ti-message-dollar:before {
  content: "\f984";
}

.ti-message-dots:before {
  content: "\eaee";
}

.ti-message-down:before {
  content: "\f985";
}

.ti-message-exclamation:before {
  content: "\f986";
}

.ti-message-filled:before {
  content: "\fecd";
}

.ti-message-forward:before {
  content: "\f28f";
}

.ti-message-heart:before {
  content: "\f987";
}

.ti-message-language:before {
  content: "\efae";
}

.ti-message-minus:before {
  content: "\f988";
}

.ti-message-off:before {
  content: "\ed41";
}

.ti-message-pause:before {
  content: "\f989";
}

.ti-message-pin:before {
  content: "\f98a";
}

.ti-message-plus:before {
  content: "\ec9a";
}

.ti-message-question:before {
  content: "\f98b";
}

.ti-message-reply:before {
  content: "\fd4d";
}

.ti-message-report:before {
  content: "\ec9b";
}

.ti-message-report-filled:before {
  content: "\fece";
}

.ti-message-search:before {
  content: "\f98c";
}

.ti-message-share:before {
  content: "\f078";
}

.ti-message-star:before {
  content: "\f98d";
}

.ti-message-up:before {
  content: "\f98e";
}

.ti-message-user:before {
  content: "\fec4";
}

.ti-message-x:before {
  content: "\f98f";
}

.ti-messages:before {
  content: "\eb6c";
}

.ti-messages-off:before {
  content: "\ed42";
}

.ti-meteor:before {
  content: "\f1fd";
}

.ti-meteor-filled:before {
  content: "\1000d";
}

.ti-meteor-off:before {
  content: "\f40c";
}

.ti-meter-cube:before {
  content: "\fd7c";
}

.ti-meter-square:before {
  content: "\fd7d";
}

.ti-metronome:before {
  content: "\fd25";
}

.ti-michelin-bib-gourmand:before {
  content: "\fae9";
}

.ti-michelin-star:before {
  content: "\faeb";
}

.ti-michelin-star-filled:before {
  content: "\1000c";
}

.ti-michelin-star-green:before {
  content: "\faea";
}

.ti-mickey:before {
  content: "\f2a3";
}

.ti-mickey-filled:before {
  content: "\f683";
}

.ti-microphone:before {
  content: "\eaf0";
}

.ti-microphone-2:before {
  content: "\ef2c";
}

.ti-microphone-2-off:before {
  content: "\f40d";
}

.ti-microphone-filled:before {
  content: "\fe0f";
}

.ti-microphone-off:before {
  content: "\ed16";
}

.ti-microscope:before {
  content: "\ef64";
}

.ti-microscope-filled:before {
  content: "\10166";
}

.ti-microscope-off:before {
  content: "\f40e";
}

.ti-microwave:before {
  content: "\f248";
}

.ti-microwave-filled:before {
  content: "\fe0e";
}

.ti-microwave-off:before {
  content: "\f264";
}

.ti-military-award:before {
  content: "\f079";
}

.ti-military-rank:before {
  content: "\efcf";
}

.ti-military-rank-filled:before {
  content: "\ff5e";
}

.ti-milk:before {
  content: "\ef13";
}

.ti-milk-filled:before {
  content: "\1000b";
}

.ti-milk-off:before {
  content: "\f40f";
}

.ti-milkshake:before {
  content: "\f4c8";
}

.ti-minimize:before {
  content: "\eaf1";
}

.ti-minus:before {
  content: "\eaf2";
}

.ti-minus-vertical:before {
  content: "\eeb4";
}

.ti-mist:before {
  content: "\ec30";
}

.ti-mist-off:before {
  content: "\f410";
}

.ti-mobiledata:before {
  content: "\f9f5";
}

.ti-mobiledata-off:before {
  content: "\f9f4";
}

.ti-moneybag:before {
  content: "\f506";
}

.ti-moneybag-edit:before {
  content: "\1013d";
}

.ti-moneybag-heart:before {
  content: "\1013c";
}

.ti-moneybag-minus:before {
  content: "\1013b";
}

.ti-moneybag-move:before {
  content: "\10139";
}

.ti-moneybag-move-back:before {
  content: "\1013a";
}

.ti-moneybag-plus:before {
  content: "\10138";
}

.ti-monkeybar:before {
  content: "\feb4";
}

.ti-mood-angry:before {
  content: "\f2de";
}

.ti-mood-angry-filled:before {
  content: "\ff0a";
}

.ti-mood-annoyed:before {
  content: "\f2e0";
}

.ti-mood-annoyed-2:before {
  content: "\f2df";
}

.ti-mood-bitcoin:before {
  content: "\ff32";
}

.ti-mood-boy:before {
  content: "\ed2d";
}

.ti-mood-check:before {
  content: "\f7b3";
}

.ti-mood-cog:before {
  content: "\f7b4";
}

.ti-mood-confuzed:before {
  content: "\eaf3";
}

.ti-mood-confuzed-filled:before {
  content: "\f7f2";
}

.ti-mood-crazy-happy:before {
  content: "\ed90";
}

.ti-mood-crazy-happy-filled:before {
  content: "\ff09";
}

.ti-mood-cry:before {
  content: "\ecbb";
}

.ti-mood-dollar:before {
  content: "\f7b5";
}

.ti-mood-edit:before {
  content: "\fa05";
}

.ti-mood-empty:before {
  content: "\eeb5";
}

.ti-mood-empty-filled:before {
  content: "\f7f3";
}

.ti-mood-happy:before {
  content: "\eaf4";
}

.ti-mood-happy-filled:before {
  content: "\f7f4";
}

.ti-mood-heart:before {
  content: "\f7b6";
}

.ti-mood-kid:before {
  content: "\ec03";
}

.ti-mood-kid-filled:before {
  content: "\f7f5";
}

.ti-mood-look-down:before {
  content: "\fd37";
}

.ti-mood-look-left:before {
  content: "\f2c5";
}

.ti-mood-look-right:before {
  content: "\f2c6";
}

.ti-mood-look-up:before {
  content: "\fd38";
}

.ti-mood-minus:before {
  content: "\f7b7";
}

.ti-mood-nerd:before {
  content: "\f2e1";
}

.ti-mood-nervous:before {
  content: "\ef96";
}

.ti-mood-neutral:before {
  content: "\eaf5";
}

.ti-mood-neutral-filled:before {
  content: "\f7f6";
}

.ti-mood-off:before {
  content: "\f161";
}

.ti-mood-pin:before {
  content: "\f7b8";
}

.ti-mood-plus:before {
  content: "\f7b9";
}

.ti-mood-puzzled:before {
  content: "\fd39";
}

.ti-mood-sad:before {
  content: "\eaf6";
}

.ti-mood-sad-2:before {
  content: "\f2e2";
}

.ti-mood-sad-dizzy:before {
  content: "\f2e3";
}

.ti-mood-sad-filled:before {
  content: "\f7f7";
}

.ti-mood-sad-squint:before {
  content: "\f2e4";
}

.ti-mood-search:before {
  content: "\f7ba";
}

.ti-mood-share:before {
  content: "\fa06";
}

.ti-mood-sick:before {
  content: "\f2e5";
}

.ti-mood-silence:before {
  content: "\f2e6";
}

.ti-mood-sing:before {
  content: "\f2c7";
}

.ti-mood-smile:before {
  content: "\eaf7";
}

.ti-mood-smile-beam:before {
  content: "\f2e7";
}

.ti-mood-smile-dizzy:before {
  content: "\f2e8";
}

.ti-mood-smile-filled:before {
  content: "\f7f8";
}

.ti-mood-spark:before {
  content: "\ffb2";
}

.ti-mood-surprised:before {
  content: "\ec04";
}

.ti-mood-tongue:before {
  content: "\eb95";
}

.ti-mood-tongue-wink:before {
  content: "\f2ea";
}

.ti-mood-tongue-wink-2:before {
  content: "\f2e9";
}

.ti-mood-unamused:before {
  content: "\f2eb";
}

.ti-mood-up:before {
  content: "\f7bb";
}

.ti-mood-wink:before {
  content: "\f2ed";
}

.ti-mood-wink-2:before {
  content: "\f2ec";
}

.ti-mood-wrrr:before {
  content: "\f2ee";
}

.ti-mood-wrrr-filled:before {
  content: "\ff08";
}

.ti-mood-x:before {
  content: "\f7bc";
}

.ti-mood-xd:before {
  content: "\f2ef";
}

.ti-moon:before {
  content: "\eaf8";
}

.ti-moon-2:before {
  content: "\ece6";
}

.ti-moon-filled:before {
  content: "\f684";
}

.ti-moon-off:before {
  content: "\f162";
}

.ti-moon-stars:before {
  content: "\ece7";
}

.ti-moped:before {
  content: "\ecbc";
}

.ti-motorbike:before {
  content: "\eeb6";
}

.ti-motorbike-filled:before {
  content: "\100f8";
}

.ti-mountain:before {
  content: "\ef97";
}

.ti-mountain-filled:before {
  content: "\1000a";
}

.ti-mountain-off:before {
  content: "\f411";
}

.ti-mouse:before {
  content: "\eaf9";
}

.ti-mouse-2:before {
  content: "\f1d7";
}

.ti-mouse-filled:before {
  content: "\fb2f";
}

.ti-mouse-off:before {
  content: "\f163";
}

.ti-moustache:before {
  content: "\f4c9";
}

.ti-movie:before {
  content: "\eafa";
}

.ti-movie-off:before {
  content: "\f164";
}

.ti-mug:before {
  content: "\eafb";
}

.ti-mug-filled:before {
  content: "\10009";
}

.ti-mug-off:before {
  content: "\f165";
}

.ti-multiplier-0-5x:before {
  content: "\ef41";
}

.ti-multiplier-1-5x:before {
  content: "\ef42";
}

.ti-multiplier-1x:before {
  content: "\ef43";
}

.ti-multiplier-2x:before {
  content: "\ef44";
}

.ti-mushroom:before {
  content: "\ef14";
}

.ti-mushroom-filled:before {
  content: "\f7f9";
}

.ti-mushroom-off:before {
  content: "\f412";
}

.ti-music:before {
  content: "\eafc";
}

.ti-music-bolt:before {
  content: "\fbd5";
}

.ti-music-cancel:before {
  content: "\fbd6";
}

.ti-music-check:before {
  content: "\fbd7";
}

.ti-music-code:before {
  content: "\fbd8";
}

.ti-music-cog:before {
  content: "\fbd9";
}

.ti-music-discount:before {
  content: "\fbda";
}

.ti-music-dollar:before {
  content: "\fbdb";
}

.ti-music-down:before {
  content: "\fbdc";
}

.ti-music-exclamation:before {
  content: "\fbdd";
}

.ti-music-heart:before {
  content: "\fbde";
}

.ti-music-minus:before {
  content: "\fbdf";
}

.ti-music-off:before {
  content: "\f166";
}

.ti-music-pause:before {
  content: "\fbe0";
}

.ti-music-pin:before {
  content: "\fbe1";
}

.ti-music-plus:before {
  content: "\fbe2";
}

.ti-music-question:before {
  content: "\fbe3";
}

.ti-music-search:before {
  content: "\fbe4";
}

.ti-music-share:before {
  content: "\fbe5";
}

.ti-music-star:before {
  content: "\fbe6";
}

.ti-music-up:before {
  content: "\fbe7";
}

.ti-music-x:before {
  content: "\fbe8";
}

.ti-navigation:before {
  content: "\f2c8";
}

.ti-navigation-bolt:before {
  content: "\fbe9";
}

.ti-navigation-cancel:before {
  content: "\fbea";
}

.ti-navigation-check:before {
  content: "\fbeb";
}

.ti-navigation-code:before {
  content: "\fbec";
}

.ti-navigation-cog:before {
  content: "\fbed";
}

.ti-navigation-discount:before {
  content: "\fbee";
}

.ti-navigation-dollar:before {
  content: "\fbef";
}

.ti-navigation-down:before {
  content: "\fbf0";
}

.ti-navigation-east:before {
  content: "\fcba";
}

.ti-navigation-exclamation:before {
  content: "\fbf1";
}

.ti-navigation-filled:before {
  content: "\f685";
}

.ti-navigation-heart:before {
  content: "\fbf2";
}

.ti-navigation-minus:before {
  content: "\fbf3";
}

.ti-navigation-north:before {
  content: "\fcbb";
}

.ti-navigation-off:before {
  content: "\f413";
}

.ti-navigation-pause:before {
  content: "\fbf4";
}

.ti-navigation-pin:before {
  content: "\fbf5";
}

.ti-navigation-plus:before {
  content: "\fbf6";
}

.ti-navigation-question:before {
  content: "\fbf7";
}

.ti-navigation-search:before {
  content: "\fbf8";
}

.ti-navigation-share:before {
  content: "\fbf9";
}

.ti-navigation-south:before {
  content: "\fcbc";
}

.ti-navigation-star:before {
  content: "\fbfa";
}

.ti-navigation-top:before {
  content: "\faec";
}

.ti-navigation-up:before {
  content: "\fbfb";
}

.ti-navigation-west:before {
  content: "\fcbd";
}

.ti-navigation-x:before {
  content: "\fbfc";
}

.ti-needle:before {
  content: "\f508";
}

.ti-needle-thread:before {
  content: "\f507";
}

.ti-network:before {
  content: "\f09f";
}

.ti-network-off:before {
  content: "\f414";
}

.ti-new-section:before {
  content: "\ebc1";
}

.ti-news:before {
  content: "\eafd";
}

.ti-news-off:before {
  content: "\f167";
}

.ti-nfc:before {
  content: "\eeb7";
}

.ti-nfc-off:before {
  content: "\f168";
}

.ti-no-copyright:before {
  content: "\efb9";
}

.ti-no-creative-commons:before {
  content: "\efba";
}

.ti-no-derivatives:before {
  content: "\efbb";
}

.ti-north-star:before {
  content: "\f014";
}

.ti-note:before {
  content: "\eb6d";
}

.ti-note-off:before {
  content: "\f169";
}

.ti-notebook:before {
  content: "\eb96";
}

.ti-notebook-off:before {
  content: "\f415";
}

.ti-notes:before {
  content: "\eb6e";
}

.ti-notes-off:before {
  content: "\f16a";
}

.ti-notification:before {
  content: "\eafe";
}

.ti-notification-off:before {
  content: "\f16b";
}

.ti-number:before {
  content: "\f1fe";
}

.ti-number-0:before {
  content: "\edf0";
}

.ti-number-0-small:before {
  content: "\fce1";
}

.ti-number-1:before {
  content: "\edf1";
}

.ti-number-1-small:before {
  content: "\fce2";
}

.ti-number-10:before {
  content: "\1005e";
}

.ti-number-10-small:before {
  content: "\fce3";
}

.ti-number-100-small:before {
  content: "\10005";
}

.ti-number-11:before {
  content: "\1005d";
}

.ti-number-11-small:before {
  content: "\fce4";
}

.ti-number-12-small:before {
  content: "\fce5";
}

.ti-number-123:before {
  content: "\f554";
}

.ti-number-13-small:before {
  content: "\fce6";
}

.ti-number-14-small:before {
  content: "\fce7";
}

.ti-number-15-small:before {
  content: "\fce8";
}

.ti-number-16-small:before {
  content: "\fce9";
}

.ti-number-17-small:before {
  content: "\fcea";
}

.ti-number-18-small:before {
  content: "\fceb";
}

.ti-number-19-small:before {
  content: "\fcec";
}

.ti-number-2:before {
  content: "\edf2";
}

.ti-number-2-small:before {
  content: "\fced";
}

.ti-number-20-small:before {
  content: "\fcee";
}

.ti-number-21-small:before {
  content: "\fcef";
}

.ti-number-22-small:before {
  content: "\fcf0";
}

.ti-number-23-small:before {
  content: "\fcf1";
}

.ti-number-24-small:before {
  content: "\fcf2";
}

.ti-number-25-small:before {
  content: "\fcf3";
}

.ti-number-26-small:before {
  content: "\fcf4";
}

.ti-number-27-small:before {
  content: "\fcf5";
}

.ti-number-28-small:before {
  content: "\fcf6";
}

.ti-number-29-small:before {
  content: "\fcf7";
}

.ti-number-3:before {
  content: "\edf3";
}

.ti-number-3-small:before {
  content: "\fcf8";
}

.ti-number-30-small:before {
  content: "\10004";
}

.ti-number-31-small:before {
  content: "\10003";
}

.ti-number-32-small:before {
  content: "\10002";
}

.ti-number-33-small:before {
  content: "\10001";
}

.ti-number-34-small:before {
  content: "\10000";
}

.ti-number-35-small:before {
  content: "\ffff";
}

.ti-number-36-small:before {
  content: "\fffe";
}

.ti-number-37-small:before {
  content: "\fffd";
}

.ti-number-38-small:before {
  content: "\fffc";
}

.ti-number-39-small:before {
  content: "\fffb";
}

.ti-number-4:before {
  content: "\edf4";
}

.ti-number-4-small:before {
  content: "\fcf9";
}

.ti-number-40-small:before {
  content: "\fffa";
}

.ti-number-41-small:before {
  content: "\fff9";
}

.ti-number-42-small:before {
  content: "\fff8";
}

.ti-number-43-small:before {
  content: "\fff7";
}

.ti-number-44-small:before {
  content: "\fff6";
}

.ti-number-45-small:before {
  content: "\fff5";
}

.ti-number-46-small:before {
  content: "\fff4";
}

.ti-number-47-small:before {
  content: "\fff3";
}

.ti-number-48-small:before {
  content: "\fff2";
}

.ti-number-49-small:before {
  content: "\fff1";
}

.ti-number-5:before {
  content: "\edf5";
}

.ti-number-5-small:before {
  content: "\fcfa";
}

.ti-number-50-small:before {
  content: "\fff0";
}

.ti-number-51-small:before {
  content: "\ffef";
}

.ti-number-52-small:before {
  content: "\ffee";
}

.ti-number-53-small:before {
  content: "\ffed";
}

.ti-number-54-small:before {
  content: "\ffec";
}

.ti-number-55-small:before {
  content: "\ffeb";
}

.ti-number-56-small:before {
  content: "\ffea";
}

.ti-number-57-small:before {
  content: "\ffe9";
}

.ti-number-58-small:before {
  content: "\ffe8";
}

.ti-number-59-small:before {
  content: "\ffe7";
}

.ti-number-6:before {
  content: "\edf6";
}

.ti-number-6-small:before {
  content: "\fcfb";
}

.ti-number-60-small:before {
  content: "\ffe6";
}

.ti-number-61-small:before {
  content: "\ffe5";
}

.ti-number-62-small:before {
  content: "\ffe4";
}

.ti-number-63-small:before {
  content: "\ffe3";
}

.ti-number-64-small:before {
  content: "\ffe2";
}

.ti-number-65-small:before {
  content: "\ffe1";
}

.ti-number-66-small:before {
  content: "\ffe0";
}

.ti-number-67-small:before {
  content: "\ffdf";
}

.ti-number-68-small:before {
  content: "\ffde";
}

.ti-number-69-small:before {
  content: "\ffdd";
}

.ti-number-7:before {
  content: "\edf7";
}

.ti-number-7-small:before {
  content: "\fcfc";
}

.ti-number-70-small:before {
  content: "\ffdc";
}

.ti-number-71-small:before {
  content: "\ffdb";
}

.ti-number-72-small:before {
  content: "\ffda";
}

.ti-number-73-small:before {
  content: "\ffd9";
}

.ti-number-74-small:before {
  content: "\ffd8";
}

.ti-number-75-small:before {
  content: "\ffd7";
}

.ti-number-76-small:before {
  content: "\ffd6";
}

.ti-number-77-small:before {
  content: "\ffd5";
}

.ti-number-78-small:before {
  content: "\ffd4";
}

.ti-number-79-small:before {
  content: "\ffd3";
}

.ti-number-8:before {
  content: "\edf8";
}

.ti-number-8-small:before {
  content: "\fcfd";
}

.ti-number-80-small:before {
  content: "\ffd2";
}

.ti-number-81-small:before {
  content: "\ffd1";
}

.ti-number-82-small:before {
  content: "\ffd0";
}

.ti-number-83-small:before {
  content: "\ffcf";
}

.ti-number-84-small:before {
  content: "\ffce";
}

.ti-number-85-small:before {
  content: "\ffcd";
}

.ti-number-86-small:before {
  content: "\ffcc";
}

.ti-number-87-small:before {
  content: "\ffcb";
}

.ti-number-88-small:before {
  content: "\ffca";
}

.ti-number-89-small:before {
  content: "\ffc9";
}

.ti-number-9:before {
  content: "\edf9";
}

.ti-number-9-small:before {
  content: "\fcfe";
}

.ti-number-90-small:before {
  content: "\ffc8";
}

.ti-number-91-small:before {
  content: "\ffc7";
}

.ti-number-92-small:before {
  content: "\ffc6";
}

.ti-number-93-small:before {
  content: "\ffc5";
}

.ti-number-94-small:before {
  content: "\ffc4";
}

.ti-number-95-small:before {
  content: "\ffc3";
}

.ti-number-96-small:before {
  content: "\ffc2";
}

.ti-number-97-small:before {
  content: "\ffc1";
}

.ti-number-98-small:before {
  content: "\ffc0";
}

.ti-number-99-small:before {
  content: "\ffbf";
}

.ti-numbers:before {
  content: "\f015";
}

.ti-nurse:before {
  content: "\ef65";
}

.ti-nurse-filled:before {
  content: "\1009e";
}

.ti-nut:before {
  content: "\fc61";
}

.ti-object-scan:before {
  content: "\fef1";
}

.ti-octagon:before {
  content: "\ecbd";
}

.ti-octagon-filled:before {
  content: "\f686";
}

.ti-octagon-minus:before {
  content: "\fc92";
}

.ti-octagon-minus-2:before {
  content: "\fc91";
}

.ti-octagon-minus-filled:before {
  content: "\1017f";
}

.ti-octagon-off:before {
  content: "\eeb8";
}

.ti-octagon-plus:before {
  content: "\fc94";
}

.ti-octagon-plus-2:before {
  content: "\fc93";
}

.ti-octagon-plus-filled:before {
  content: "\1017e";
}

.ti-octahedron:before {
  content: "\faae";
}

.ti-octahedron-off:before {
  content: "\faac";
}

.ti-octahedron-plus:before {
  content: "\faad";
}

.ti-old:before {
  content: "\eeb9";
}

.ti-olympics:before {
  content: "\eeba";
}

.ti-olympics-off:before {
  content: "\f416";
}

.ti-om:before {
  content: "\f58d";
}

.ti-omega:before {
  content: "\eb97";
}

.ti-outbound:before {
  content: "\f249";
}

.ti-outlet:before {
  content: "\ebd7";
}

.ti-oval:before {
  content: "\f02e";
}

.ti-oval-filled:before {
  content: "\f687";
}

.ti-oval-vertical:before {
  content: "\f02d";
}

.ti-oval-vertical-filled:before {
  content: "\f688";
}

.ti-overline:before {
  content: "\eebb";
}

.ti-package:before {
  content: "\eaff";
}

.ti-package-export:before {
  content: "\f07a";
}

.ti-package-import:before {
  content: "\f07b";
}

.ti-package-off:before {
  content: "\f16c";
}

.ti-packages:before {
  content: "\f2c9";
}

.ti-pacman:before {
  content: "\eebc";
}

.ti-page-break:before {
  content: "\ec81";
}

.ti-paint:before {
  content: "\eb00";
}

.ti-paint-filled:before {
  content: "\f75f";
}

.ti-paint-off:before {
  content: "\f16d";
}

.ti-palette:before {
  content: "\eb01";
}

.ti-palette-filled:before {
  content: "\1009d";
}

.ti-palette-off:before {
  content: "\f16e";
}

.ti-panorama-horizontal:before {
  content: "\ed33";
}

.ti-panorama-horizontal-filled:before {
  content: "\fecc";
}

.ti-panorama-horizontal-off:before {
  content: "\f417";
}

.ti-panorama-vertical:before {
  content: "\ed34";
}

.ti-panorama-vertical-filled:before {
  content: "\fecb";
}

.ti-panorama-vertical-off:before {
  content: "\f418";
}

.ti-paper-bag:before {
  content: "\f02f";
}

.ti-paper-bag-off:before {
  content: "\f16f";
}

.ti-paperclip:before {
  content: "\eb02";
}

.ti-parachute:before {
  content: "\ed7c";
}

.ti-parachute-off:before {
  content: "\f170";
}

.ti-parentheses:before {
  content: "\ebd8";
}

.ti-parentheses-off:before {
  content: "\f171";
}

.ti-parking:before {
  content: "\eb03";
}

.ti-parking-circle:before {
  content: "\fd5a";
}

.ti-parking-circle-filled:before {
  content: "\feca";
}

.ti-parking-off:before {
  content: "\f172";
}

.ti-password:before {
  content: "\f4ca";
}

.ti-password-fingerprint:before {
  content: "\fc7b";
}

.ti-password-mobile-phone:before {
  content: "\fc7c";
}

.ti-password-user:before {
  content: "\fc7d";
}

.ti-paw:before {
  content: "\eff9";
}

.ti-paw-filled:before {
  content: "\f689";
}

.ti-paw-off:before {
  content: "\f419";
}

.ti-paywall:before {
  content: "\fd7e";
}

.ti-pdf:before {
  content: "\f7ac";
}

.ti-peace:before {
  content: "\ecbe";
}

.ti-pencil:before {
  content: "\eb04";
}

.ti-pencil-bolt:before {
  content: "\fbfd";
}

.ti-pencil-cancel:before {
  content: "\fbfe";
}

.ti-pencil-check:before {
  content: "\fbff";
}

.ti-pencil-code:before {
  content: "\fc00";
}

.ti-pencil-cog:before {
  content: "\fc01";
}

.ti-pencil-discount:before {
  content: "\fc02";
}

.ti-pencil-dollar:before {
  content: "\fc03";
}

.ti-pencil-down:before {
  content: "\fc04";
}

.ti-pencil-exclamation:before {
  content: "\fc05";
}

.ti-pencil-heart:before {
  content: "\fc06";
}

.ti-pencil-minus:before {
  content: "\f1eb";
}

.ti-pencil-off:before {
  content: "\f173";
}

.ti-pencil-pause:before {
  content: "\fc07";
}

.ti-pencil-pin:before {
  content: "\fc08";
}

.ti-pencil-plus:before {
  content: "\f1ec";
}

.ti-pencil-question:before {
  content: "\fc09";
}

.ti-pencil-search:before {
  content: "\fc0a";
}

.ti-pencil-share:before {
  content: "\fc0b";
}

.ti-pencil-star:before {
  content: "\fc0c";
}

.ti-pencil-up:before {
  content: "\fc0d";
}

.ti-pencil-x:before {
  content: "\fc0e";
}

.ti-pennant:before {
  content: "\ed7d";
}

.ti-pennant-2:before {
  content: "\f06a";
}

.ti-pennant-2-filled:before {
  content: "\f68a";
}

.ti-pennant-filled:before {
  content: "\f68b";
}

.ti-pennant-off:before {
  content: "\f174";
}

.ti-pentagon:before {
  content: "\efe3";
}

.ti-pentagon-filled:before {
  content: "\f68c";
}

.ti-pentagon-minus:before {
  content: "\feb3";
}

.ti-pentagon-number-0:before {
  content: "\fc7e";
}

.ti-pentagon-number-1:before {
  content: "\fc7f";
}

.ti-pentagon-number-2:before {
  content: "\fc80";
}

.ti-pentagon-number-3:before {
  content: "\fc81";
}

.ti-pentagon-number-4:before {
  content: "\fc82";
}

.ti-pentagon-number-5:before {
  content: "\fc83";
}

.ti-pentagon-number-6:before {
  content: "\fc84";
}

.ti-pentagon-number-7:before {
  content: "\fc85";
}

.ti-pentagon-number-8:before {
  content: "\fc86";
}

.ti-pentagon-number-9:before {
  content: "\fc87";
}

.ti-pentagon-off:before {
  content: "\f41a";
}

.ti-pentagon-plus:before {
  content: "\fc49";
}

.ti-pentagon-x:before {
  content: "\fc88";
}

.ti-pentagram:before {
  content: "\f586";
}

.ti-pepper:before {
  content: "\ef15";
}

.ti-pepper-off:before {
  content: "\f175";
}

.ti-percentage:before {
  content: "\ecf4";
}

.ti-percentage-0:before {
  content: "\fee5";
}

.ti-percentage-10:before {
  content: "\fee4";
}

.ti-percentage-100:before {
  content: "\fee3";
}

.ti-percentage-20:before {
  content: "\fee2";
}

.ti-percentage-25:before {
  content: "\fee1";
}

.ti-percentage-30:before {
  content: "\fee0";
}

.ti-percentage-33:before {
  content: "\fedf";
}

.ti-percentage-40:before {
  content: "\fede";
}

.ti-percentage-50:before {
  content: "\fedd";
}

.ti-percentage-60:before {
  content: "\fedc";
}

.ti-percentage-66:before {
  content: "\fedb";
}

.ti-percentage-70:before {
  content: "\feda";
}

.ti-percentage-75:before {
  content: "\fed9";
}

.ti-percentage-80:before {
  content: "\fed8";
}

.ti-percentage-90:before {
  content: "\fed7";
}

.ti-perfume:before {
  content: "\f509";
}

.ti-perspective:before {
  content: "\eebd";
}

.ti-perspective-off:before {
  content: "\f176";
}

.ti-phone:before {
  content: "\eb09";
}

.ti-phone-call:before {
  content: "\eb05";
}

.ti-phone-calling:before {
  content: "\ec43";
}

.ti-phone-check:before {
  content: "\ec05";
}

.ti-phone-done:before {
  content: "\ff9e";
}

.ti-phone-end:before {
  content: "\ff9d";
}

.ti-phone-filled:before {
  content: "\fa49";
}

.ti-phone-incoming:before {
  content: "\eb06";
}

.ti-phone-off:before {
  content: "\ecf5";
}

.ti-phone-outgoing:before {
  content: "\eb07";
}

.ti-phone-pause:before {
  content: "\eb08";
}

.ti-phone-plus:before {
  content: "\ec06";
}

.ti-phone-ringing:before {
  content: "\ff9c";
}

.ti-phone-spark:before {
  content: "\ffb1";
}

.ti-phone-x:before {
  content: "\ec07";
}

.ti-photo:before {
  content: "\eb0a";
}

.ti-photo-ai:before {
  content: "\fa32";
}

.ti-photo-bitcoin:before {
  content: "\ff31";
}

.ti-photo-bolt:before {
  content: "\f990";
}

.ti-photo-cancel:before {
  content: "\f35d";
}

.ti-photo-check:before {
  content: "\f35e";
}

.ti-photo-circle:before {
  content: "\fc4a";
}

.ti-photo-circle-minus:before {
  content: "\fc62";
}

.ti-photo-circle-plus:before {
  content: "\fc63";
}

.ti-photo-code:before {
  content: "\f991";
}

.ti-photo-cog:before {
  content: "\f992";
}

.ti-photo-dollar:before {
  content: "\f993";
}

.ti-photo-down:before {
  content: "\f35f";
}

.ti-photo-edit:before {
  content: "\f360";
}

.ti-photo-exclamation:before {
  content: "\f994";
}

.ti-photo-filled:before {
  content: "\fa4a";
}

.ti-photo-heart:before {
  content: "\f361";
}

.ti-photo-hexagon:before {
  content: "\fc4b";
}

.ti-photo-minus:before {
  content: "\f362";
}

.ti-photo-off:before {
  content: "\ecf6";
}

.ti-photo-pause:before {
  content: "\f995";
}

.ti-photo-pentagon:before {
  content: "\fc4c";
}

.ti-photo-pin:before {
  content: "\f996";
}

.ti-photo-plus:before {
  content: "\f363";
}

.ti-photo-question:before {
  content: "\f997";
}

.ti-photo-scan:before {
  content: "\fca8";
}

.ti-photo-search:before {
  content: "\f364";
}

.ti-photo-sensor:before {
  content: "\f798";
}

.ti-photo-sensor-2:before {
  content: "\f796";
}

.ti-photo-sensor-3:before {
  content: "\f797";
}

.ti-photo-share:before {
  content: "\f998";
}

.ti-photo-shield:before {
  content: "\f365";
}

.ti-photo-spark:before {
  content: "\ffb0";
}

.ti-photo-square-rounded:before {
  content: "\fc4d";
}

.ti-photo-star:before {
  content: "\f366";
}

.ti-photo-up:before {
  content: "\f38b";
}

.ti-photo-video:before {
  content: "\fc95";
}

.ti-photo-x:before {
  content: "\f367";
}

.ti-physotherapist:before {
  content: "\eebe";
}

.ti-piano:before {
  content: "\fad3";
}

.ti-pick:before {
  content: "\fafc";
}

.ti-picnic-table:before {
  content: "\fed6";
}

.ti-picture-in-picture:before {
  content: "\ed35";
}

.ti-picture-in-picture-filled:before {
  content: "\fec1";
}

.ti-picture-in-picture-off:before {
  content: "\ed43";
}

.ti-picture-in-picture-on:before {
  content: "\ed44";
}

.ti-picture-in-picture-top:before {
  content: "\efe4";
}

.ti-picture-in-picture-top-filled:before {
  content: "\fec2";
}

.ti-pig:before {
  content: "\ef52";
}

.ti-pig-filled:before {
  content: "\1010a";
}

.ti-pig-money:before {
  content: "\f38c";
}

.ti-pig-off:before {
  content: "\f177";
}

.ti-pilcrow:before {
  content: "\f5f6";
}

.ti-pilcrow-left:before {
  content: "\fd7f";
}

.ti-pilcrow-right:before {
  content: "\fd80";
}

.ti-pill:before {
  content: "\ec44";
}

.ti-pill-filled:before {
  content: "\ff07";
}

.ti-pill-off:before {
  content: "\f178";
}

.ti-pills:before {
  content: "\ef66";
}

.ti-pin:before {
  content: "\ec9c";
}

.ti-pin-end:before {
  content: "\fd5b";
}

.ti-pin-filled:before {
  content: "\f68d";
}

.ti-pin-invoke:before {
  content: "\fd5c";
}

.ti-ping-pong:before {
  content: "\f38d";
}

.ti-pinned:before {
  content: "\ed60";
}

.ti-pinned-filled:before {
  content: "\f68e";
}

.ti-pinned-off:before {
  content: "\ed5f";
}

.ti-pizza:before {
  content: "\edbb";
}

.ti-pizza-filled:before {
  content: "\10008";
}

.ti-pizza-off:before {
  content: "\f179";
}

.ti-placeholder:before {
  content: "\f626";
}

.ti-plane:before {
  content: "\eb6f";
}

.ti-plane-arrival:before {
  content: "\eb99";
}

.ti-plane-departure:before {
  content: "\eb9a";
}

.ti-plane-inflight:before {
  content: "\ef98";
}

.ti-plane-off:before {
  content: "\f17a";
}

.ti-plane-tilt:before {
  content: "\f1ed";
}

.ti-planet:before {
  content: "\ec08";
}

.ti-planet-off:before {
  content: "\f17b";
}

.ti-plant:before {
  content: "\ed50";
}

.ti-plant-2:before {
  content: "\ed7e";
}

.ti-plant-2-off:before {
  content: "\f17c";
}

.ti-plant-off:before {
  content: "\f17d";
}

.ti-play-basketball:before {
  content: "\fa66";
}

.ti-play-card:before {
  content: "\eebf";
}

.ti-play-card-1:before {
  content: "\1005c";
}

.ti-play-card-1-filled:before {
  content: "\10083";
}

.ti-play-card-10:before {
  content: "\1005b";
}

.ti-play-card-10-filled:before {
  content: "\10082";
}

.ti-play-card-2:before {
  content: "\1005a";
}

.ti-play-card-2-filled:before {
  content: "\10081";
}

.ti-play-card-3:before {
  content: "\10059";
}

.ti-play-card-3-filled:before {
  content: "\10080";
}

.ti-play-card-4:before {
  content: "\10058";
}

.ti-play-card-4-filled:before {
  content: "\1007f";
}

.ti-play-card-5:before {
  content: "\10057";
}

.ti-play-card-5-filled:before {
  content: "\1007e";
}

.ti-play-card-6:before {
  content: "\10056";
}

.ti-play-card-6-filled:before {
  content: "\1007d";
}

.ti-play-card-7:before {
  content: "\10055";
}

.ti-play-card-7-filled:before {
  content: "\1007c";
}

.ti-play-card-8:before {
  content: "\10054";
}

.ti-play-card-8-filled:before {
  content: "\1007b";
}

.ti-play-card-9:before {
  content: "\10053";
}

.ti-play-card-9-filled:before {
  content: "\1007a";
}

.ti-play-card-a:before {
  content: "\10052";
}

.ti-play-card-a-filled:before {
  content: "\10079";
}

.ti-play-card-j:before {
  content: "\10051";
}

.ti-play-card-j-filled:before {
  content: "\10078";
}

.ti-play-card-k:before {
  content: "\10050";
}

.ti-play-card-k-filled:before {
  content: "\10077";
}

.ti-play-card-off:before {
  content: "\f17e";
}

.ti-play-card-q:before {
  content: "\1004f";
}

.ti-play-card-q-filled:before {
  content: "\10076";
}

.ti-play-card-star:before {
  content: "\1004e";
}

.ti-play-card-star-filled:before {
  content: "\10075";
}

.ti-play-football:before {
  content: "\fa67";
}

.ti-play-handball:before {
  content: "\fa68";
}

.ti-play-volleyball:before {
  content: "\fa69";
}

.ti-player-eject:before {
  content: "\efbc";
}

.ti-player-eject-filled:before {
  content: "\f68f";
}

.ti-player-pause:before {
  content: "\ed45";
}

.ti-player-pause-filled:before {
  content: "\f690";
}

.ti-player-play:before {
  content: "\ed46";
}

.ti-player-play-filled:before {
  content: "\f691";
}

.ti-player-record:before {
  content: "\ed47";
}

.ti-player-record-filled:before {
  content: "\f692";
}

.ti-player-skip-back:before {
  content: "\ed48";
}

.ti-player-skip-back-filled:before {
  content: "\f693";
}

.ti-player-skip-forward:before {
  content: "\ed49";
}

.ti-player-skip-forward-filled:before {
  content: "\f694";
}

.ti-player-stop:before {
  content: "\ed4a";
}

.ti-player-stop-filled:before {
  content: "\f695";
}

.ti-player-track-next:before {
  content: "\ed4b";
}

.ti-player-track-next-filled:before {
  content: "\f696";
}

.ti-player-track-prev:before {
  content: "\ed4c";
}

.ti-player-track-prev-filled:before {
  content: "\f697";
}

.ti-playlist:before {
  content: "\eec0";
}

.ti-playlist-add:before {
  content: "\f008";
}

.ti-playlist-off:before {
  content: "\f17f";
}

.ti-playlist-x:before {
  content: "\f009";
}

.ti-playstation-circle:before {
  content: "\f2ad";
}

.ti-playstation-square:before {
  content: "\f2ae";
}

.ti-playstation-triangle:before {
  content: "\f2af";
}

.ti-playstation-x:before {
  content: "\f2b0";
}

.ti-plug:before {
  content: "\ebd9";
}

.ti-plug-connected:before {
  content: "\f00a";
}

.ti-plug-connected-x:before {
  content: "\f0a0";
}

.ti-plug-off:before {
  content: "\f180";
}

.ti-plug-x:before {
  content: "\f0a1";
}

.ti-plus:before {
  content: "\eb0b";
}

.ti-plus-equal:before {
  content: "\f7ad";
}

.ti-plus-minus:before {
  content: "\f7ae";
}

.ti-png:before {
  content: "\f3ad";
}

.ti-podium:before {
  content: "\f1d8";
}

.ti-podium-off:before {
  content: "\f41b";
}

.ti-point:before {
  content: "\eb0c";
}

.ti-point-filled:before {
  content: "\f698";
}

.ti-point-off:before {
  content: "\f181";
}

.ti-pointer:before {
  content: "\f265";
}

.ti-pointer-bolt:before {
  content: "\f999";
}

.ti-pointer-cancel:before {
  content: "\f99a";
}

.ti-pointer-check:before {
  content: "\f99b";
}

.ti-pointer-code:before {
  content: "\f99c";
}

.ti-pointer-cog:before {
  content: "\f99d";
}

.ti-pointer-dollar:before {
  content: "\f99e";
}

.ti-pointer-down:before {
  content: "\f99f";
}

.ti-pointer-exclamation:before {
  content: "\f9a0";
}

.ti-pointer-filled:before {
  content: "\fb30";
}

.ti-pointer-heart:before {
  content: "\f9a1";
}

.ti-pointer-minus:before {
  content: "\f9a2";
}

.ti-pointer-off:before {
  content: "\f9a3";
}

.ti-pointer-pause:before {
  content: "\f9a4";
}

.ti-pointer-pin:before {
  content: "\f9a5";
}

.ti-pointer-plus:before {
  content: "\f9a6";
}

.ti-pointer-question:before {
  content: "\f9a7";
}

.ti-pointer-search:before {
  content: "\f9a8";
}

.ti-pointer-share:before {
  content: "\f9a9";
}

.ti-pointer-star:before {
  content: "\f9aa";
}

.ti-pointer-up:before {
  content: "\f9ab";
}

.ti-pointer-x:before {
  content: "\f9ac";
}

.ti-pokeball:before {
  content: "\eec1";
}

.ti-pokeball-off:before {
  content: "\f41c";
}

.ti-poker-chip:before {
  content: "\f515";
}

.ti-polaroid:before {
  content: "\eec2";
}

.ti-polaroid-filled:before {
  content: "\fa4b";
}

.ti-polygon:before {
  content: "\efd0";
}

.ti-polygon-off:before {
  content: "\f182";
}

.ti-poo:before {
  content: "\f258";
}

.ti-poo-filled:before {
  content: "\fec9";
}

.ti-pool:before {
  content: "\ed91";
}

.ti-pool-off:before {
  content: "\f41d";
}

.ti-power:before {
  content: "\eb0d";
}

.ti-pray:before {
  content: "\ecbf";
}

.ti-premium-rights:before {
  content: "\efbd";
}

.ti-prescription:before {
  content: "\ef99";
}

.ti-presentation:before {
  content: "\eb70";
}

.ti-presentation-analytics:before {
  content: "\eec3";
}

.ti-presentation-analytics-filled:before {
  content: "\ff5d";
}

.ti-presentation-filled:before {
  content: "\ff5c";
}

.ti-presentation-off:before {
  content: "\f183";
}

.ti-printer:before {
  content: "\eb0e";
}

.ti-printer-off:before {
  content: "\f184";
}

.ti-prism:before {
  content: "\fab1";
}

.ti-prism-light:before {
  content: "\fea6";
}

.ti-prism-off:before {
  content: "\faaf";
}

.ti-prism-plus:before {
  content: "\fab0";
}

.ti-prison:before {
  content: "\ef79";
}

.ti-progress:before {
  content: "\fa0d";
}

.ti-progress-alert:before {
  content: "\fa07";
}

.ti-progress-bolt:before {
  content: "\fa08";
}

.ti-progress-check:before {
  content: "\fa09";
}

.ti-progress-down:before {
  content: "\fa0a";
}

.ti-progress-help:before {
  content: "\fa0b";
}

.ti-progress-x:before {
  content: "\fa0c";
}

.ti-prompt:before {
  content: "\eb0f";
}

.ti-prong:before {
  content: "\fda1";
}

.ti-propeller:before {
  content: "\eec4";
}

.ti-propeller-off:before {
  content: "\f185";
}

.ti-protocol:before {
  content: "\fd81";
}

.ti-pumpkin-scary:before {
  content: "\f587";
}

.ti-puzzle:before {
  content: "\eb10";
}

.ti-puzzle-2:before {
  content: "\ef83";
}

.ti-puzzle-filled:before {
  content: "\f699";
}

.ti-puzzle-off:before {
  content: "\f186";
}

.ti-pyramid:before {
  content: "\eec5";
}

.ti-pyramid-off:before {
  content: "\f187";
}

.ti-pyramid-plus:before {
  content: "\fab2";
}

.ti-qrcode:before {
  content: "\eb11";
}

.ti-qrcode-off:before {
  content: "\f41e";
}

.ti-question-mark:before {
  content: "\ec9d";
}

.ti-quote:before {
  content: "\efbe";
}

.ti-quote-filled:before {
  content: "\1009c";
}

.ti-quote-off:before {
  content: "\f188";
}

.ti-quotes:before {
  content: "\fb1e";
}

.ti-radar:before {
  content: "\f017";
}

.ti-radar-2:before {
  content: "\f016";
}

.ti-radar-filled:before {
  content: "\fe0d";
}

.ti-radar-off:before {
  content: "\f41f";
}

.ti-radio:before {
  content: "\ef2d";
}

.ti-radio-off:before {
  content: "\f420";
}

.ti-radioactive:before {
  content: "\ecc0";
}

.ti-radioactive-filled:before {
  content: "\f760";
}

.ti-radioactive-off:before {
  content: "\f189";
}

.ti-radius-bottom-left:before {
  content: "\eec6";
}

.ti-radius-bottom-right:before {
  content: "\eec7";
}

.ti-radius-top-left:before {
  content: "\eec8";
}

.ti-radius-top-right:before {
  content: "\eec9";
}

.ti-rainbow:before {
  content: "\edbc";
}

.ti-rainbow-off:before {
  content: "\f18a";
}

.ti-rating-12-plus:before {
  content: "\f266";
}

.ti-rating-14-plus:before {
  content: "\f267";
}

.ti-rating-16-plus:before {
  content: "\f268";
}

.ti-rating-18-plus:before {
  content: "\f269";
}

.ti-rating-21-plus:before {
  content: "\f26a";
}

.ti-razor:before {
  content: "\f4b5";
}

.ti-razor-electric:before {
  content: "\f4b4";
}

.ti-receipt:before {
  content: "\edfd";
}

.ti-receipt-2:before {
  content: "\edfa";
}

.ti-receipt-bitcoin:before {
  content: "\fd66";
}

.ti-receipt-dollar:before {
  content: "\fd67";
}

.ti-receipt-dollar-filled:before {
  content: "\1017d";
}

.ti-receipt-euro:before {
  content: "\fd68";
}

.ti-receipt-euro-filled:before {
  content: "\1017c";
}

.ti-receipt-filled:before {
  content: "\ff06";
}

.ti-receipt-off:before {
  content: "\edfb";
}

.ti-receipt-pound:before {
  content: "\fd69";
}

.ti-receipt-pound-filled:before {
  content: "\1017b";
}

.ti-receipt-refund:before {
  content: "\edfc";
}

.ti-receipt-rupee:before {
  content: "\fd82";
}

.ti-receipt-rupee-filled:before {
  content: "\1017a";
}

.ti-receipt-tax:before {
  content: "\edbd";
}

.ti-receipt-yen:before {
  content: "\fd6a";
}

.ti-receipt-yen-filled:before {
  content: "\10179";
}

.ti-receipt-yuan:before {
  content: "\fd6b";
}

.ti-receipt-yuan-filled:before {
  content: "\10178";
}

.ti-recharging:before {
  content: "\eeca";
}

.ti-record-mail:before {
  content: "\eb12";
}

.ti-record-mail-off:before {
  content: "\f18b";
}

.ti-rectangle:before {
  content: "\ed37";
}

.ti-rectangle-filled:before {
  content: "\f69a";
}

.ti-rectangle-rounded-bottom:before {
  content: "\faed";
}

.ti-rectangle-rounded-top:before {
  content: "\faee";
}

.ti-rectangle-vertical:before {
  content: "\ed36";
}

.ti-rectangle-vertical-filled:before {
  content: "\f69b";
}

.ti-rectangular-prism:before {
  content: "\fab5";
}

.ti-rectangular-prism-off:before {
  content: "\fab3";
}

.ti-rectangular-prism-plus:before {
  content: "\fab4";
}

.ti-recycle:before {
  content: "\eb9b";
}

.ti-recycle-off:before {
  content: "\f18c";
}

.ti-refresh:before {
  content: "\eb13";
}

.ti-refresh-alert:before {
  content: "\ed57";
}

.ti-refresh-dot:before {
  content: "\efbf";
}

.ti-refresh-off:before {
  content: "\f18d";
}

.ti-regex:before {
  content: "\f31f";
}

.ti-regex-off:before {
  content: "\f421";
}

.ti-registered:before {
  content: "\eb14";
}

.ti-relation-many-to-many:before {
  content: "\ed7f";
}

.ti-relation-many-to-many-filled:before {
  content: "\fe0c";
}

.ti-relation-one-to-many:before {
  content: "\ed80";
}

.ti-relation-one-to-many-filled:before {
  content: "\fe0b";
}

.ti-relation-one-to-one:before {
  content: "\ed81";
}

.ti-relation-one-to-one-filled:before {
  content: "\fe0a";
}

.ti-reload:before {
  content: "\f3ae";
}

.ti-reorder:before {
  content: "\fc15";
}

.ti-repeat:before {
  content: "\eb72";
}

.ti-repeat-off:before {
  content: "\f18e";
}

.ti-repeat-once:before {
  content: "\eb71";
}

.ti-replace:before {
  content: "\ebc7";
}

.ti-replace-filled:before {
  content: "\f69c";
}

.ti-replace-off:before {
  content: "\f422";
}

.ti-replace-user:before {
  content: "\100f0";
}

.ti-report:before {
  content: "\eece";
}

.ti-report-analytics:before {
  content: "\eecb";
}

.ti-report-medical:before {
  content: "\eecc";
}

.ti-report-money:before {
  content: "\eecd";
}

.ti-report-off:before {
  content: "\f18f";
}

.ti-report-search:before {
  content: "\ef84";
}

.ti-reserved-line:before {
  content: "\f9f6";
}

.ti-resize:before {
  content: "\eecf";
}

.ti-restore:before {
  content: "\fafd";
}

.ti-rewind-backward-10:before {
  content: "\faba";
}

.ti-rewind-backward-15:before {
  content: "\fabb";
}

.ti-rewind-backward-20:before {
  content: "\fabc";
}

.ti-rewind-backward-30:before {
  content: "\fabd";
}

.ti-rewind-backward-40:before {
  content: "\fabe";
}

.ti-rewind-backward-5:before {
  content: "\fabf";
}

.ti-rewind-backward-50:before {
  content: "\fac0";
}

.ti-rewind-backward-60:before {
  content: "\fac1";
}

.ti-rewind-forward-10:before {
  content: "\fac2";
}

.ti-rewind-forward-15:before {
  content: "\fac3";
}

.ti-rewind-forward-20:before {
  content: "\fac4";
}

.ti-rewind-forward-30:before {
  content: "\fac5";
}

.ti-rewind-forward-40:before {
  content: "\fac6";
}

.ti-rewind-forward-5:before {
  content: "\fac7";
}

.ti-rewind-forward-50:before {
  content: "\fac8";
}

.ti-rewind-forward-60:before {
  content: "\fac9";
}

.ti-ribbon-health:before {
  content: "\f58e";
}

.ti-rings:before {
  content: "\fa6a";
}

.ti-ripple:before {
  content: "\ed82";
}

.ti-ripple-off:before {
  content: "\f190";
}

.ti-road:before {
  content: "\f018";
}

.ti-road-off:before {
  content: "\f191";
}

.ti-road-sign:before {
  content: "\ecdd";
}

.ti-robot:before {
  content: "\f00b";
}

.ti-robot-face:before {
  content: "\fcbe";
}

.ti-robot-off:before {
  content: "\f192";
}

.ti-rocket:before {
  content: "\ec45";
}

.ti-rocket-off:before {
  content: "\f193";
}

.ti-roller-skating:before {
  content: "\efd1";
}

.ti-rollercoaster:before {
  content: "\f0a2";
}

.ti-rollercoaster-filled:before {
  content: "\100f7";
}

.ti-rollercoaster-off:before {
  content: "\f423";
}

.ti-rosette:before {
  content: "\f599";
}

.ti-rosette-discount:before {
  content: "\ee7c";
}

.ti-rosette-discount-check:before {
  content: "\f1f8";
}

.ti-rosette-discount-check-filled:before {
  content: "\f746";
}

.ti-rosette-discount-check-off:before {
  content: "\ff10";
}

.ti-rosette-discount-filled:before {
  content: "\ff05";
}

.ti-rosette-discount-off:before {
  content: "\f3e6";
}

.ti-rosette-filled:before {
  content: "\f69d";
}

.ti-rosette-number-0:before {
  content: "\f58f";
}

.ti-rosette-number-1:before {
  content: "\f590";
}

.ti-rosette-number-2:before {
  content: "\f591";
}

.ti-rosette-number-3:before {
  content: "\f592";
}

.ti-rosette-number-4:before {
  content: "\f593";
}

.ti-rosette-number-5:before {
  content: "\f594";
}

.ti-rosette-number-6:before {
  content: "\f595";
}

.ti-rosette-number-7:before {
  content: "\f596";
}

.ti-rosette-number-8:before {
  content: "\f597";
}

.ti-rosette-number-9:before {
  content: "\f598";
}

.ti-rotate:before {
  content: "\eb16";
}

.ti-rotate-2:before {
  content: "\ebb4";
}

.ti-rotate-360:before {
  content: "\ef85";
}

.ti-rotate-3d:before {
  content: "\f020";
}

.ti-rotate-clockwise:before {
  content: "\eb15";
}

.ti-rotate-clockwise-2:before {
  content: "\ebb5";
}

.ti-rotate-dot:before {
  content: "\efe5";
}

.ti-rotate-rectangle:before {
  content: "\ec15";
}

.ti-route:before {
  content: "\eb17";
}

.ti-route-2:before {
  content: "\f4b6";
}

.ti-route-alt-left:before {
  content: "\fca9";
}

.ti-route-alt-right:before {
  content: "\fcaa";
}

.ti-route-off:before {
  content: "\f194";
}

.ti-route-scan:before {
  content: "\fcbf";
}

.ti-route-square:before {
  content: "\fcac";
}

.ti-route-square-2:before {
  content: "\fcab";
}

.ti-route-x:before {
  content: "\fcae";
}

.ti-route-x-2:before {
  content: "\fcad";
}

.ti-router:before {
  content: "\eb18";
}

.ti-router-off:before {
  content: "\f424";
}

.ti-row-insert-bottom:before {
  content: "\eed0";
}

.ti-row-insert-top:before {
  content: "\eed1";
}

.ti-row-remove:before {
  content: "\fafe";
}

.ti-rss:before {
  content: "\eb19";
}

.ti-rubber-stamp:before {
  content: "\f5ab";
}

.ti-rubber-stamp-off:before {
  content: "\f5aa";
}

.ti-ruler:before {
  content: "\eb1a";
}

.ti-ruler-2:before {
  content: "\eed2";
}

.ti-ruler-2-off:before {
  content: "\f195";
}

.ti-ruler-3:before {
  content: "\f290";
}

.ti-ruler-measure:before {
  content: "\f291";
}

.ti-ruler-measure-2:before {
  content: "\ff0f";
}

.ti-ruler-off:before {
  content: "\f196";
}

.ti-run:before {
  content: "\ec82";
}

.ti-rv-truck:before {
  content: "\fcc0";
}

.ti-s-turn-down:before {
  content: "\f516";
}

.ti-s-turn-left:before {
  content: "\f517";
}

.ti-s-turn-right:before {
  content: "\f518";
}

.ti-s-turn-up:before {
  content: "\f519";
}

.ti-sailboat:before {
  content: "\ec83";
}

.ti-sailboat-2:before {
  content: "\f5f7";
}

.ti-sailboat-off:before {
  content: "\f425";
}

.ti-salad:before {
  content: "\f50a";
}

.ti-salad-filled:before {
  content: "\10007";
}

.ti-salt:before {
  content: "\ef16";
}

.ti-sandbox:before {
  content: "\fd6c";
}

.ti-satellite:before {
  content: "\eed3";
}

.ti-satellite-off:before {
  content: "\f197";
}

.ti-sausage:before {
  content: "\ef17";
}

.ti-scale:before {
  content: "\ebc2";
}

.ti-scale-off:before {
  content: "\f198";
}

.ti-scale-outline:before {
  content: "\ef53";
}

.ti-scale-outline-off:before {
  content: "\f199";
}

.ti-scan:before {
  content: "\ebc8";
}

.ti-scan-eye:before {
  content: "\f1ff";
}

.ti-scan-position:before {
  content: "\fdac";
}

.ti-schema:before {
  content: "\f200";
}

.ti-schema-off:before {
  content: "\f426";
}

.ti-school:before {
  content: "\ecf7";
}

.ti-school-bell:before {
  content: "\f64a";
}

.ti-school-off:before {
  content: "\f19a";
}

.ti-scissors:before {
  content: "\eb1b";
}

.ti-scissors-off:before {
  content: "\f19b";
}

.ti-scooter:before {
  content: "\ec6c";
}

.ti-scooter-electric:before {
  content: "\ecc1";
}

.ti-scoreboard:before {
  content: "\fa6b";
}

.ti-screen-share:before {
  content: "\ed18";
}

.ti-screen-share-off:before {
  content: "\ed17";
}

.ti-screenshot:before {
  content: "\f201";
}

.ti-scribble:before {
  content: "\f0a3";
}

.ti-scribble-off:before {
  content: "\f427";
}

.ti-script:before {
  content: "\f2da";
}

.ti-script-minus:before {
  content: "\f2d7";
}

.ti-script-plus:before {
  content: "\f2d8";
}

.ti-script-x:before {
  content: "\f2d9";
}

.ti-scuba-diving:before {
  content: "\fd4e";
}

.ti-scuba-diving-tank:before {
  content: "\fefa";
}

.ti-scuba-diving-tank-filled:before {
  content: "\ff04";
}

.ti-scuba-mask:before {
  content: "\eed4";
}

.ti-scuba-mask-off:before {
  content: "\f428";
}

.ti-sdk:before {
  content: "\f3af";
}

.ti-search:before {
  content: "\eb1c";
}

.ti-search-off:before {
  content: "\f19c";
}

.ti-section:before {
  content: "\eed5";
}

.ti-section-filled:before {
  content: "\fe09";
}

.ti-section-sign:before {
  content: "\f019";
}

.ti-seedling:before {
  content: "\ed51";
}

.ti-seedling-filled:before {
  content: "\10006";
}

.ti-seedling-off:before {
  content: "\f19d";
}

.ti-select:before {
  content: "\ec9e";
}

.ti-select-all:before {
  content: "\f9f7";
}

.ti-selector:before {
  content: "\eb1d";
}

.ti-send:before {
  content: "\eb1e";
}

.ti-send-2:before {
  content: "\fd5d";
}

.ti-send-off:before {
  content: "\f429";
}

.ti-seo:before {
  content: "\f26b";
}

.ti-separator:before {
  content: "\ebda";
}

.ti-separator-horizontal:before {
  content: "\ec79";
}

.ti-separator-vertical:before {
  content: "\ec7a";
}

.ti-server:before {
  content: "\eb1f";
}

.ti-server-2:before {
  content: "\f07c";
}

.ti-server-bolt:before {
  content: "\f320";
}

.ti-server-cog:before {
  content: "\f321";
}

.ti-server-off:before {
  content: "\f19e";
}

.ti-server-spark:before {
  content: "\ffaf";
}

.ti-servicemark:before {
  content: "\ec09";
}

.ti-settings:before {
  content: "\eb20";
}

.ti-settings-2:before {
  content: "\f5ac";
}

.ti-settings-automation:before {
  content: "\eed6";
}

.ti-settings-bolt:before {
  content: "\f9ad";
}

.ti-settings-cancel:before {
  content: "\f9ae";
}

.ti-settings-check:before {
  content: "\f9af";
}

.ti-settings-code:before {
  content: "\f9b0";
}

.ti-settings-cog:before {
  content: "\f9b1";
}

.ti-settings-dollar:before {
  content: "\f9b2";
}

.ti-settings-down:before {
  content: "\f9b3";
}

.ti-settings-exclamation:before {
  content: "\f9b4";
}

.ti-settings-filled:before {
  content: "\f69e";
}

.ti-settings-heart:before {
  content: "\f9b5";
}

.ti-settings-minus:before {
  content: "\f9b6";
}

.ti-settings-off:before {
  content: "\f19f";
}

.ti-settings-pause:before {
  content: "\f9b7";
}

.ti-settings-pin:before {
  content: "\f9b8";
}

.ti-settings-plus:before {
  content: "\f9b9";
}

.ti-settings-question:before {
  content: "\f9ba";
}

.ti-settings-search:before {
  content: "\f9bb";
}

.ti-settings-share:before {
  content: "\f9bc";
}

.ti-settings-spark:before {
  content: "\ffae";
}

.ti-settings-star:before {
  content: "\f9bd";
}

.ti-settings-up:before {
  content: "\f9be";
}

.ti-settings-x:before {
  content: "\f9bf";
}

.ti-shadow:before {
  content: "\eed8";
}

.ti-shadow-off:before {
  content: "\eed7";
}

.ti-shape:before {
  content: "\eb9c";
}

.ti-shape-2:before {
  content: "\eed9";
}

.ti-shape-3:before {
  content: "\eeda";
}

.ti-shape-off:before {
  content: "\f1a0";
}

.ti-share:before {
  content: "\eb21";
}

.ti-share-2:before {
  content: "\f799";
}

.ti-share-3:before {
  content: "\f7bd";
}

.ti-share-off:before {
  content: "\f1a1";
}

.ti-shareplay:before {
  content: "\fea5";
}

.ti-shield:before {
  content: "\eb24";
}

.ti-shield-bolt:before {
  content: "\f9c0";
}

.ti-shield-cancel:before {
  content: "\f9c1";
}

.ti-shield-check:before {
  content: "\eb22";
}

.ti-shield-check-filled:before {
  content: "\f761";
}

.ti-shield-checkered:before {
  content: "\ef9a";
}

.ti-shield-checkered-filled:before {
  content: "\f762";
}

.ti-shield-chevron:before {
  content: "\ef9b";
}

.ti-shield-code:before {
  content: "\f9c2";
}

.ti-shield-cog:before {
  content: "\f9c3";
}

.ti-shield-dollar:before {
  content: "\f9c4";
}

.ti-shield-down:before {
  content: "\f9c5";
}

.ti-shield-exclamation:before {
  content: "\f9c6";
}

.ti-shield-filled:before {
  content: "\f69f";
}

.ti-shield-half:before {
  content: "\f358";
}

.ti-shield-half-filled:before {
  content: "\f357";
}

.ti-shield-heart:before {
  content: "\f9c7";
}

.ti-shield-lock:before {
  content: "\ed58";
}

.ti-shield-lock-filled:before {
  content: "\f763";
}

.ti-shield-minus:before {
  content: "\f9c8";
}

.ti-shield-off:before {
  content: "\ecf8";
}

.ti-shield-pause:before {
  content: "\f9c9";
}

.ti-shield-pin:before {
  content: "\f9ca";
}

.ti-shield-plus:before {
  content: "\f9cb";
}

.ti-shield-question:before {
  content: "\f9cc";
}

.ti-shield-search:before {
  content: "\f9cd";
}

.ti-shield-share:before {
  content: "\f9ce";
}

.ti-shield-star:before {
  content: "\f9cf";
}

.ti-shield-up:before {
  content: "\f9d0";
}

.ti-shield-x:before {
  content: "\eb23";
}

.ti-ship:before {
  content: "\ec84";
}

.ti-ship-off:before {
  content: "\f42a";
}

.ti-shirt:before {
  content: "\ec0a";
}

.ti-shirt-filled:before {
  content: "\f6a0";
}

.ti-shirt-off:before {
  content: "\f1a2";
}

.ti-shirt-sport:before {
  content: "\f26c";
}

.ti-shoe:before {
  content: "\efd2";
}

.ti-shoe-off:before {
  content: "\f1a4";
}

.ti-shopping-bag:before {
  content: "\f5f8";
}

.ti-shopping-bag-check:before {
  content: "\fc16";
}

.ti-shopping-bag-discount:before {
  content: "\fc17";
}

.ti-shopping-bag-edit:before {
  content: "\fc18";
}

.ti-shopping-bag-exclamation:before {
  content: "\fc19";
}

.ti-shopping-bag-heart:before {
  content: "\fda2";
}

.ti-shopping-bag-minus:before {
  content: "\fc1a";
}

.ti-shopping-bag-plus:before {
  content: "\fc1b";
}

.ti-shopping-bag-search:before {
  content: "\fc1c";
}

.ti-shopping-bag-x:before {
  content: "\fc1d";
}

.ti-shopping-cart:before {
  content: "\eb25";
}

.ti-shopping-cart-bolt:before {
  content: "\fb57";
}

.ti-shopping-cart-cancel:before {
  content: "\fb58";
}

.ti-shopping-cart-check:before {
  content: "\fb59";
}

.ti-shopping-cart-code:before {
  content: "\fb5a";
}

.ti-shopping-cart-cog:before {
  content: "\fb5b";
}

.ti-shopping-cart-copy:before {
  content: "\fb5c";
}

.ti-shopping-cart-discount:before {
  content: "\fb5d";
}

.ti-shopping-cart-dollar:before {
  content: "\fb5e";
}

.ti-shopping-cart-down:before {
  content: "\fb5f";
}

.ti-shopping-cart-exclamation:before {
  content: "\fb60";
}

.ti-shopping-cart-filled:before {
  content: "\fc3f";
}

.ti-shopping-cart-heart:before {
  content: "\fb61";
}

.ti-shopping-cart-minus:before {
  content: "\fb62";
}

.ti-shopping-cart-off:before {
  content: "\eedc";
}

.ti-shopping-cart-pause:before {
  content: "\fb63";
}

.ti-shopping-cart-pin:before {
  content: "\fb64";
}

.ti-shopping-cart-plus:before {
  content: "\fb65";
}

.ti-shopping-cart-question:before {
  content: "\fb66";
}

.ti-shopping-cart-search:before {
  content: "\fb67";
}

.ti-shopping-cart-share:before {
  content: "\fb68";
}

.ti-shopping-cart-star:before {
  content: "\fb69";
}

.ti-shopping-cart-up:before {
  content: "\fb6a";
}

.ti-shopping-cart-x:before {
  content: "\fb6b";
}

.ti-shovel:before {
  content: "\f1d9";
}

.ti-shovel-pitchforks:before {
  content: "\fd3a";
}

.ti-shredder:before {
  content: "\eedf";
}

.ti-sign-left:before {
  content: "\f06b";
}

.ti-sign-left-filled:before {
  content: "\f6a1";
}

.ti-sign-right:before {
  content: "\f06c";
}

.ti-sign-right-filled:before {
  content: "\f6a2";
}

.ti-signal-2g:before {
  content: "\f79a";
}

.ti-signal-3g:before {
  content: "\f1ee";
}

.ti-signal-4g:before {
  content: "\f1ef";
}

.ti-signal-4g-plus:before {
  content: "\f259";
}

.ti-signal-5g:before {
  content: "\f1f0";
}

.ti-signal-6g:before {
  content: "\f9f8";
}

.ti-signal-e:before {
  content: "\f9f9";
}

.ti-signal-g:before {
  content: "\f9fa";
}

.ti-signal-h:before {
  content: "\f9fc";
}

.ti-signal-h-plus:before {
  content: "\f9fb";
}

.ti-signal-lte:before {
  content: "\f9fd";
}

.ti-signature:before {
  content: "\eee0";
}

.ti-signature-off:before {
  content: "\f1a5";
}

.ti-sitemap:before {
  content: "\eb9d";
}

.ti-sitemap-filled:before {
  content: "\1006b";
}

.ti-sitemap-off:before {
  content: "\f1a6";
}

.ti-skateboard:before {
  content: "\ecc2";
}

.ti-skateboard-off:before {
  content: "\f42b";
}

.ti-skateboarding:before {
  content: "\faca";
}

.ti-skew-x:before {
  content: "\fd3b";
}

.ti-skew-y:before {
  content: "\fd3c";
}

.ti-ski-jumping:before {
  content: "\fa6c";
}

.ti-skull:before {
  content: "\f292";
}

.ti-slash:before {
  content: "\f4f9";
}

.ti-slashes:before {
  content: "\f588";
}

.ti-sleigh:before {
  content: "\ef9c";
}

.ti-slice:before {
  content: "\ebdb";
}

.ti-slideshow:before {
  content: "\ebc9";
}

.ti-smart-home:before {
  content: "\ecde";
}

.ti-smart-home-off:before {
  content: "\f1a7";
}

.ti-smoking:before {
  content: "\ecc4";
}

.ti-smoking-no:before {
  content: "\ecc3";
}

.ti-snowboarding:before {
  content: "\fd4f";
}

.ti-snowflake:before {
  content: "\ec0b";
}

.ti-snowflake-off:before {
  content: "\f1a8";
}

.ti-snowman:before {
  content: "\f26d";
}

.ti-soccer-field:before {
  content: "\ed92";
}

.ti-social:before {
  content: "\ebec";
}

.ti-social-off:before {
  content: "\f1a9";
}

.ti-sock:before {
  content: "\eee1";
}

.ti-sofa:before {
  content: "\efaf";
}

.ti-sofa-off:before {
  content: "\f42c";
}

.ti-solar-electricity:before {
  content: "\fcc1";
}

.ti-solar-panel:before {
  content: "\f7bf";
}

.ti-solar-panel-2:before {
  content: "\f7be";
}

.ti-sort-0-9:before {
  content: "\f54d";
}

.ti-sort-9-0:before {
  content: "\f54e";
}

.ti-sort-a-z:before {
  content: "\f54f";
}

.ti-sort-ascending:before {
  content: "\eb26";
}

.ti-sort-ascending-2:before {
  content: "\eee2";
}

.ti-sort-ascending-2-filled:before {
  content: "\ff5b";
}

.ti-sort-ascending-letters:before {
  content: "\ef18";
}

.ti-sort-ascending-numbers:before {
  content: "\ef19";
}

.ti-sort-ascending-shapes:before {
  content: "\fd94";
}

.ti-sort-ascending-shapes-filled:before {
  content: "\ff5a";
}

.ti-sort-ascending-small-big:before {
  content: "\fd95";
}

.ti-sort-descending:before {
  content: "\eb27";
}

.ti-sort-descending-2:before {
  content: "\eee3";
}

.ti-sort-descending-2-filled:before {
  content: "\ff59";
}

.ti-sort-descending-letters:before {
  content: "\ef1a";
}

.ti-sort-descending-numbers:before {
  content: "\ef1b";
}

.ti-sort-descending-shapes:before {
  content: "\fd97";
}

.ti-sort-descending-shapes-filled:before {
  content: "\ff58";
}

.ti-sort-descending-small-big:before {
  content: "\fd96";
}

.ti-sort-z-a:before {
  content: "\f550";
}

.ti-sos:before {
  content: "\f24a";
}

.ti-soup:before {
  content: "\ef2e";
}

.ti-soup-filled:before {
  content: "\fe08";
}

.ti-soup-off:before {
  content: "\f42d";
}

.ti-source-code:before {
  content: "\f4a2";
}

.ti-space:before {
  content: "\ec0c";
}

.ti-space-off:before {
  content: "\f1aa";
}

.ti-spaces:before {
  content: "\fea4";
}

.ti-spacing-horizontal:before {
  content: "\ef54";
}

.ti-spacing-vertical:before {
  content: "\ef55";
}

.ti-spade:before {
  content: "\effa";
}

.ti-spade-filled:before {
  content: "\f6a3";
}

.ti-sparkles:before {
  content: "\f6d7";
}

.ti-speakerphone:before {
  content: "\ed61";
}

.ti-speedboat:before {
  content: "\ed93";
}

.ti-speedboat-filled:before {
  content: "\100f6";
}

.ti-sphere:before {
  content: "\fab8";
}

.ti-sphere-off:before {
  content: "\fab6";
}

.ti-sphere-plus:before {
  content: "\fab7";
}

.ti-spider:before {
  content: "\f293";
}

.ti-spider-filled:before {
  content: "\10109";
}

.ti-spiral:before {
  content: "\f294";
}

.ti-spiral-off:before {
  content: "\f42e";
}

.ti-sport-billard:before {
  content: "\eee4";
}

.ti-spray:before {
  content: "\f50b";
}

.ti-spy:before {
  content: "\f227";
}

.ti-spy-off:before {
  content: "\f42f";
}

.ti-sql:before {
  content: "\f7c0";
}

.ti-square:before {
  content: "\eb2c";
}

.ti-square-arrow-down:before {
  content: "\f4b7";
}

.ti-square-arrow-down-filled:before {
  content: "\fb31";
}

.ti-square-arrow-left:before {
  content: "\f4b8";
}

.ti-square-arrow-left-filled:before {
  content: "\fb32";
}

.ti-square-arrow-right:before {
  content: "\f4b9";
}

.ti-square-arrow-right-filled:before {
  content: "\fb33";
}

.ti-square-arrow-up:before {
  content: "\f4ba";
}

.ti-square-arrow-up-filled:before {
  content: "\fb34";
}

.ti-square-asterisk:before {
  content: "\f01a";
}

.ti-square-asterisk-filled:before {
  content: "\fb35";
}

.ti-square-check:before {
  content: "\eb28";
}

.ti-square-check-filled:before {
  content: "\f76d";
}

.ti-square-chevron-down:before {
  content: "\f627";
}

.ti-square-chevron-down-filled:before {
  content: "\fb36";
}

.ti-square-chevron-left:before {
  content: "\f628";
}

.ti-square-chevron-left-filled:before {
  content: "\fb37";
}

.ti-square-chevron-right:before {
  content: "\f629";
}

.ti-square-chevron-right-filled:before {
  content: "\fb38";
}

.ti-square-chevron-up:before {
  content: "\f62a";
}

.ti-square-chevron-up-filled:before {
  content: "\fb39";
}

.ti-square-chevrons-down:before {
  content: "\f64b";
}

.ti-square-chevrons-down-filled:before {
  content: "\fb3a";
}

.ti-square-chevrons-left:before {
  content: "\f64c";
}

.ti-square-chevrons-left-filled:before {
  content: "\fb3b";
}

.ti-square-chevrons-right:before {
  content: "\f64d";
}

.ti-square-chevrons-right-filled:before {
  content: "\fb3c";
}

.ti-square-chevrons-up:before {
  content: "\f64e";
}

.ti-square-chevrons-up-filled:before {
  content: "\fb3d";
}

.ti-square-dashed:before {
  content: "\100bb";
}

.ti-square-dot:before {
  content: "\ed59";
}

.ti-square-dot-filled:before {
  content: "\fb3e";
}

.ti-square-f0:before {
  content: "\f526";
}

.ti-square-f0-filled:before {
  content: "\f76e";
}

.ti-square-f1:before {
  content: "\f527";
}

.ti-square-f1-filled:before {
  content: "\f76f";
}

.ti-square-f2:before {
  content: "\f528";
}

.ti-square-f2-filled:before {
  content: "\f770";
}

.ti-square-f3:before {
  content: "\f529";
}

.ti-square-f3-filled:before {
  content: "\f771";
}

.ti-square-f4:before {
  content: "\f52a";
}

.ti-square-f4-filled:before {
  content: "\f772";
}

.ti-square-f5:before {
  content: "\f52b";
}

.ti-square-f5-filled:before {
  content: "\f773";
}

.ti-square-f6:before {
  content: "\f52c";
}

.ti-square-f6-filled:before {
  content: "\f774";
}

.ti-square-f7:before {
  content: "\f52d";
}

.ti-square-f7-filled:before {
  content: "\f775";
}

.ti-square-f8:before {
  content: "\f52e";
}

.ti-square-f8-filled:before {
  content: "\f776";
}

.ti-square-f9:before {
  content: "\f52f";
}

.ti-square-f9-filled:before {
  content: "\f777";
}

.ti-square-filled:before {
  content: "\fc40";
}

.ti-square-forbid:before {
  content: "\ed5b";
}

.ti-square-forbid-2:before {
  content: "\ed5a";
}

.ti-square-half:before {
  content: "\effb";
}

.ti-square-key:before {
  content: "\f638";
}

.ti-square-letter-a:before {
  content: "\f47c";
}

.ti-square-letter-a-filled:before {
  content: "\fe07";
}

.ti-square-letter-b:before {
  content: "\f47d";
}

.ti-square-letter-b-filled:before {
  content: "\fe06";
}

.ti-square-letter-c:before {
  content: "\f47e";
}

.ti-square-letter-c-filled:before {
  content: "\fe05";
}

.ti-square-letter-d:before {
  content: "\f47f";
}

.ti-square-letter-d-filled:before {
  content: "\fe04";
}

.ti-square-letter-e:before {
  content: "\f480";
}

.ti-square-letter-e-filled:before {
  content: "\fe03";
}

.ti-square-letter-f:before {
  content: "\f481";
}

.ti-square-letter-f-filled:before {
  content: "\fe02";
}

.ti-square-letter-g:before {
  content: "\f482";
}

.ti-square-letter-g-filled:before {
  content: "\fe01";
}

.ti-square-letter-h:before {
  content: "\f483";
}

.ti-square-letter-h-filled:before {
  content: "\fe00";
}

.ti-square-letter-i:before {
  content: "\f484";
}

.ti-square-letter-i-filled:before {
  content: "\fdff";
}

.ti-square-letter-j:before {
  content: "\f485";
}

.ti-square-letter-j-filled:before {
  content: "\fdfe";
}

.ti-square-letter-k:before {
  content: "\f486";
}

.ti-square-letter-k-filled:before {
  content: "\fdfd";
}

.ti-square-letter-l:before {
  content: "\f487";
}

.ti-square-letter-l-filled:before {
  content: "\fdfc";
}

.ti-square-letter-m:before {
  content: "\f488";
}

.ti-square-letter-m-filled:before {
  content: "\fdfb";
}

.ti-square-letter-n:before {
  content: "\f489";
}

.ti-square-letter-n-filled:before {
  content: "\fdfa";
}

.ti-square-letter-o:before {
  content: "\f48a";
}

.ti-square-letter-o-filled:before {
  content: "\fdf9";
}

.ti-square-letter-p:before {
  content: "\f48b";
}

.ti-square-letter-p-filled:before {
  content: "\fdf8";
}

.ti-square-letter-q:before {
  content: "\f48c";
}

.ti-square-letter-q-filled:before {
  content: "\fdf7";
}

.ti-square-letter-r:before {
  content: "\f48d";
}

.ti-square-letter-r-filled:before {
  content: "\fdf6";
}

.ti-square-letter-s:before {
  content: "\f48e";
}

.ti-square-letter-s-filled:before {
  content: "\fdf5";
}

.ti-square-letter-t:before {
  content: "\f48f";
}

.ti-square-letter-t-filled:before {
  content: "\fdf4";
}

.ti-square-letter-u:before {
  content: "\f490";
}

.ti-square-letter-u-filled:before {
  content: "\fdf3";
}

.ti-square-letter-v:before {
  content: "\f4bb";
}

.ti-square-letter-v-filled:before {
  content: "\fdf2";
}

.ti-square-letter-w:before {
  content: "\f491";
}

.ti-square-letter-w-filled:before {
  content: "\fdf1";
}

.ti-square-letter-x:before {
  content: "\f4bc";
}

.ti-square-letter-x-filled:before {
  content: "\fdf0";
}

.ti-square-letter-y:before {
  content: "\f492";
}

.ti-square-letter-y-filled:before {
  content: "\fdef";
}

.ti-square-letter-z:before {
  content: "\f493";
}

.ti-square-letter-z-filled:before {
  content: "\fdee";
}

.ti-square-minus:before {
  content: "\eb29";
}

.ti-square-minus-filled:before {
  content: "\fb3f";
}

.ti-square-number-0:before {
  content: "\eee5";
}

.ti-square-number-0-filled:before {
  content: "\f764";
}

.ti-square-number-1:before {
  content: "\eee6";
}

.ti-square-number-1-filled:before {
  content: "\f765";
}

.ti-square-number-2:before {
  content: "\eee7";
}

.ti-square-number-2-filled:before {
  content: "\f7fa";
}

.ti-square-number-3:before {
  content: "\eee8";
}

.ti-square-number-3-filled:before {
  content: "\f766";
}

.ti-square-number-4:before {
  content: "\eee9";
}

.ti-square-number-4-filled:before {
  content: "\f767";
}

.ti-square-number-5:before {
  content: "\eeea";
}

.ti-square-number-5-filled:before {
  content: "\f768";
}

.ti-square-number-6:before {
  content: "\eeeb";
}

.ti-square-number-6-filled:before {
  content: "\f769";
}

.ti-square-number-7:before {
  content: "\eeec";
}

.ti-square-number-7-filled:before {
  content: "\f76a";
}

.ti-square-number-8:before {
  content: "\eeed";
}

.ti-square-number-8-filled:before {
  content: "\f76b";
}

.ti-square-number-9:before {
  content: "\eeee";
}

.ti-square-number-9-filled:before {
  content: "\f76c";
}

.ti-square-off:before {
  content: "\eeef";
}

.ti-square-percentage:before {
  content: "\fd83";
}

.ti-square-plus:before {
  content: "\eb2a";
}

.ti-square-plus-2:before {
  content: "\fc96";
}

.ti-square-root:before {
  content: "\eef1";
}

.ti-square-root-2:before {
  content: "\eef0";
}

.ti-square-rotated:before {
  content: "\ecdf";
}

.ti-square-rotated-filled:before {
  content: "\f6a4";
}

.ti-square-rotated-forbid:before {
  content: "\f01c";
}

.ti-square-rotated-forbid-2:before {
  content: "\f01b";
}

.ti-square-rotated-off:before {
  content: "\eef2";
}

.ti-square-rounded:before {
  content: "\f59a";
}

.ti-square-rounded-arrow-down:before {
  content: "\f639";
}

.ti-square-rounded-arrow-down-filled:before {
  content: "\f6db";
}

.ti-square-rounded-arrow-left:before {
  content: "\f63a";
}

.ti-square-rounded-arrow-left-filled:before {
  content: "\f6dc";
}

.ti-square-rounded-arrow-right:before {
  content: "\f63b";
}

.ti-square-rounded-arrow-right-filled:before {
  content: "\f6dd";
}

.ti-square-rounded-arrow-up:before {
  content: "\f63c";
}

.ti-square-rounded-arrow-up-filled:before {
  content: "\f6de";
}

.ti-square-rounded-check:before {
  content: "\f63d";
}

.ti-square-rounded-check-filled:before {
  content: "\f6df";
}

.ti-square-rounded-chevron-down:before {
  content: "\f62b";
}

.ti-square-rounded-chevron-down-filled:before {
  content: "\f6e0";
}

.ti-square-rounded-chevron-left:before {
  content: "\f62c";
}

.ti-square-rounded-chevron-left-filled:before {
  content: "\f6e1";
}

.ti-square-rounded-chevron-right:before {
  content: "\f62d";
}

.ti-square-rounded-chevron-right-filled:before {
  content: "\f6e2";
}

.ti-square-rounded-chevron-up:before {
  content: "\f62e";
}

.ti-square-rounded-chevron-up-filled:before {
  content: "\f6e3";
}

.ti-square-rounded-chevrons-down:before {
  content: "\f64f";
}

.ti-square-rounded-chevrons-down-filled:before {
  content: "\f6e4";
}

.ti-square-rounded-chevrons-left:before {
  content: "\f650";
}

.ti-square-rounded-chevrons-left-filled:before {
  content: "\f6e5";
}

.ti-square-rounded-chevrons-right:before {
  content: "\f651";
}

.ti-square-rounded-chevrons-right-filled:before {
  content: "\f6e6";
}

.ti-square-rounded-chevrons-up:before {
  content: "\f652";
}

.ti-square-rounded-chevrons-up-filled:before {
  content: "\f6e7";
}

.ti-square-rounded-filled:before {
  content: "\f6a5";
}

.ti-square-rounded-letter-a:before {
  content: "\f5ae";
}

.ti-square-rounded-letter-a-filled:before {
  content: "\fded";
}

.ti-square-rounded-letter-b:before {
  content: "\f5af";
}

.ti-square-rounded-letter-b-filled:before {
  content: "\fdec";
}

.ti-square-rounded-letter-c:before {
  content: "\f5b0";
}

.ti-square-rounded-letter-c-filled:before {
  content: "\fdeb";
}

.ti-square-rounded-letter-d:before {
  content: "\f5b1";
}

.ti-square-rounded-letter-d-filled:before {
  content: "\fdea";
}

.ti-square-rounded-letter-e:before {
  content: "\f5b2";
}

.ti-square-rounded-letter-e-filled:before {
  content: "\fde9";
}

.ti-square-rounded-letter-f:before {
  content: "\f5b3";
}

.ti-square-rounded-letter-f-filled:before {
  content: "\fde8";
}

.ti-square-rounded-letter-g:before {
  content: "\f5b4";
}

.ti-square-rounded-letter-g-filled:before {
  content: "\fde7";
}

.ti-square-rounded-letter-h:before {
  content: "\f5b5";
}

.ti-square-rounded-letter-h-filled:before {
  content: "\fde6";
}

.ti-square-rounded-letter-i:before {
  content: "\f5b6";
}

.ti-square-rounded-letter-i-filled:before {
  content: "\fde5";
}

.ti-square-rounded-letter-j:before {
  content: "\f5b7";
}

.ti-square-rounded-letter-j-filled:before {
  content: "\fde4";
}

.ti-square-rounded-letter-k:before {
  content: "\f5b8";
}

.ti-square-rounded-letter-k-filled:before {
  content: "\fde3";
}

.ti-square-rounded-letter-l:before {
  content: "\f5b9";
}

.ti-square-rounded-letter-l-filled:before {
  content: "\fde2";
}

.ti-square-rounded-letter-m:before {
  content: "\f5ba";
}

.ti-square-rounded-letter-m-filled:before {
  content: "\fde1";
}

.ti-square-rounded-letter-n:before {
  content: "\f5bb";
}

.ti-square-rounded-letter-n-filled:before {
  content: "\fde0";
}

.ti-square-rounded-letter-o:before {
  content: "\f5bc";
}

.ti-square-rounded-letter-o-filled:before {
  content: "\fddf";
}

.ti-square-rounded-letter-p:before {
  content: "\f5bd";
}

.ti-square-rounded-letter-p-filled:before {
  content: "\fdde";
}

.ti-square-rounded-letter-q:before {
  content: "\f5be";
}

.ti-square-rounded-letter-q-filled:before {
  content: "\fddd";
}

.ti-square-rounded-letter-r:before {
  content: "\f5bf";
}

.ti-square-rounded-letter-r-filled:before {
  content: "\fddc";
}

.ti-square-rounded-letter-s:before {
  content: "\f5c0";
}

.ti-square-rounded-letter-s-filled:before {
  content: "\fddb";
}

.ti-square-rounded-letter-t:before {
  content: "\f5c1";
}

.ti-square-rounded-letter-t-filled:before {
  content: "\fdda";
}

.ti-square-rounded-letter-u:before {
  content: "\f5c2";
}

.ti-square-rounded-letter-u-filled:before {
  content: "\fdd9";
}

.ti-square-rounded-letter-v:before {
  content: "\f5c3";
}

.ti-square-rounded-letter-v-filled:before {
  content: "\fdd8";
}

.ti-square-rounded-letter-w:before {
  content: "\f5c4";
}

.ti-square-rounded-letter-w-filled:before {
  content: "\fdd7";
}

.ti-square-rounded-letter-x:before {
  content: "\f5c5";
}

.ti-square-rounded-letter-x-filled:before {
  content: "\fdd6";
}

.ti-square-rounded-letter-y:before {
  content: "\f5c6";
}

.ti-square-rounded-letter-y-filled:before {
  content: "\fdd5";
}

.ti-square-rounded-letter-z:before {
  content: "\f5c7";
}

.ti-square-rounded-letter-z-filled:before {
  content: "\fdd4";
}

.ti-square-rounded-minus:before {
  content: "\f63e";
}

.ti-square-rounded-minus-2:before {
  content: "\fc97";
}

.ti-square-rounded-minus-filled:before {
  content: "\fb40";
}

.ti-square-rounded-number-0:before {
  content: "\f5c8";
}

.ti-square-rounded-number-0-filled:before {
  content: "\f778";
}

.ti-square-rounded-number-1:before {
  content: "\f5c9";
}

.ti-square-rounded-number-1-filled:before {
  content: "\f779";
}

.ti-square-rounded-number-2:before {
  content: "\f5ca";
}

.ti-square-rounded-number-2-filled:before {
  content: "\f77a";
}

.ti-square-rounded-number-3:before {
  content: "\f5cb";
}

.ti-square-rounded-number-3-filled:before {
  content: "\f77b";
}

.ti-square-rounded-number-4:before {
  content: "\f5cc";
}

.ti-square-rounded-number-4-filled:before {
  content: "\f77c";
}

.ti-square-rounded-number-5:before {
  content: "\f5cd";
}

.ti-square-rounded-number-5-filled:before {
  content: "\f77d";
}

.ti-square-rounded-number-6:before {
  content: "\f5ce";
}

.ti-square-rounded-number-6-filled:before {
  content: "\f77e";
}

.ti-square-rounded-number-7:before {
  content: "\f5cf";
}

.ti-square-rounded-number-7-filled:before {
  content: "\f77f";
}

.ti-square-rounded-number-8:before {
  content: "\f5d0";
}

.ti-square-rounded-number-8-filled:before {
  content: "\f780";
}

.ti-square-rounded-number-9:before {
  content: "\f5d1";
}

.ti-square-rounded-number-9-filled:before {
  content: "\f781";
}

.ti-square-rounded-percentage:before {
  content: "\fd84";
}

.ti-square-rounded-plus:before {
  content: "\f63f";
}

.ti-square-rounded-plus-2:before {
  content: "\fc98";
}

.ti-square-rounded-plus-filled:before {
  content: "\f6e8";
}

.ti-square-rounded-x:before {
  content: "\f640";
}

.ti-square-rounded-x-filled:before {
  content: "\f6e9";
}

.ti-square-toggle:before {
  content: "\eef4";
}

.ti-square-toggle-horizontal:before {
  content: "\eef3";
}

.ti-square-x:before {
  content: "\eb2b";
}

.ti-square-x-filled:before {
  content: "\fb41";
}

.ti-squares:before {
  content: "\eef6";
}

.ti-squares-diagonal:before {
  content: "\eef5";
}

.ti-squares-filled:before {
  content: "\fe9f";
}

.ti-squares-selected:before {
  content: "\fea3";
}

.ti-stack:before {
  content: "\eb2d";
}

.ti-stack-2:before {
  content: "\eef7";
}

.ti-stack-2-filled:before {
  content: "\fdd3";
}

.ti-stack-3:before {
  content: "\ef9d";
}

.ti-stack-3-filled:before {
  content: "\fdd2";
}

.ti-stack-back:before {
  content: "\fd26";
}

.ti-stack-backward:before {
  content: "\fd27";
}

.ti-stack-filled:before {
  content: "\fdd1";
}

.ti-stack-forward:before {
  content: "\fd28";
}

.ti-stack-front:before {
  content: "\fd29";
}

.ti-stack-middle:before {
  content: "\fd2a";
}

.ti-stack-pop:before {
  content: "\f234";
}

.ti-stack-push:before {
  content: "\f235";
}

.ti-stairs:before {
  content: "\eca6";
}

.ti-stairs-down:before {
  content: "\eca4";
}

.ti-stairs-up:before {
  content: "\eca5";
}

.ti-star:before {
  content: "\eb2e";
}

.ti-star-filled:before {
  content: "\f6a6";
}

.ti-star-half:before {
  content: "\ed19";
}

.ti-star-half-filled:before {
  content: "\f6a7";
}

.ti-star-off:before {
  content: "\ed62";
}

.ti-stars:before {
  content: "\ed38";
}

.ti-stars-filled:before {
  content: "\f6a8";
}

.ti-stars-off:before {
  content: "\f430";
}

.ti-status-change:before {
  content: "\f3b0";
}

.ti-steam:before {
  content: "\f24b";
}

.ti-steering-wheel:before {
  content: "\ec7b";
}

.ti-steering-wheel-filled:before {
  content: "\ff03";
}

.ti-steering-wheel-off:before {
  content: "\f431";
}

.ti-step-into:before {
  content: "\ece0";
}

.ti-step-out:before {
  content: "\ece1";
}

.ti-stereo-glasses:before {
  content: "\f4cb";
}

.ti-stethoscope:before {
  content: "\edbe";
}

.ti-stethoscope-off:before {
  content: "\f432";
}

.ti-sticker:before {
  content: "\eb2f";
}

.ti-sticker-2:before {
  content: "\fd3d";
}

.ti-stopwatch:before {
  content: "\ff9b";
}

.ti-storm:before {
  content: "\f24c";
}

.ti-storm-off:before {
  content: "\f433";
}

.ti-stretching:before {
  content: "\f2db";
}

.ti-stretching-2:before {
  content: "\fa6d";
}

.ti-strikethrough:before {
  content: "\eb9e";
}

.ti-submarine:before {
  content: "\ed94";
}

.ti-subscript:before {
  content: "\eb9f";
}

.ti-subtask:before {
  content: "\ec9f";
}

.ti-sum:before {
  content: "\eb73";
}

.ti-sum-off:before {
  content: "\f1ab";
}

.ti-sun:before {
  content: "\eb30";
}

.ti-sun-electricity:before {
  content: "\fcc2";
}

.ti-sun-filled:before {
  content: "\f6a9";
}

.ti-sun-high:before {
  content: "\f236";
}

.ti-sun-high-filled:before {
  content: "\10108";
}

.ti-sun-low:before {
  content: "\f237";
}

.ti-sun-low-filled:before {
  content: "\10107";
}

.ti-sun-moon:before {
  content: "\f4a3";
}

.ti-sun-off:before {
  content: "\ed63";
}

.ti-sun-wind:before {
  content: "\f238";
}

.ti-sunglasses:before {
  content: "\f239";
}

.ti-sunglasses-filled:before {
  content: "\fec8";
}

.ti-sunrise:before {
  content: "\ef1c";
}

.ti-sunrise-filled:before {
  content: "\10106";
}

.ti-sunset:before {
  content: "\ec31";
}

.ti-sunset-2:before {
  content: "\f23a";
}

.ti-sunset-2-filled:before {
  content: "\10105";
}

.ti-sunset-filled:before {
  content: "\10104";
}

.ti-superscript:before {
  content: "\eba0";
}

.ti-svg:before {
  content: "\f25a";
}

.ti-swimming:before {
  content: "\ec92";
}

.ti-swipe:before {
  content: "\f551";
}

.ti-swipe-down:before {
  content: "\fd5e";
}

.ti-swipe-down-filled:before {
  content: "\ff57";
}

.ti-swipe-left:before {
  content: "\fd5f";
}

.ti-swipe-left-filled:before {
  content: "\ff56";
}

.ti-swipe-right:before {
  content: "\fd60";
}

.ti-swipe-right-filled:before {
  content: "\ff55";
}

.ti-swipe-up:before {
  content: "\fd61";
}

.ti-swipe-up-filled:before {
  content: "\ff54";
}

.ti-switch:before {
  content: "\eb33";
}

.ti-switch-2:before {
  content: "\edbf";
}

.ti-switch-3:before {
  content: "\edc0";
}

.ti-switch-horizontal:before {
  content: "\eb31";
}

.ti-switch-vertical:before {
  content: "\eb32";
}

.ti-sword:before {
  content: "\f030";
}

.ti-sword-off:before {
  content: "\f434";
}

.ti-swords:before {
  content: "\f132";
}

.ti-table:before {
  content: "\eba1";
}

.ti-table-alias:before {
  content: "\f25b";
}

.ti-table-column:before {
  content: "\faff";
}

.ti-table-dashed:before {
  content: "\100ba";
}

.ti-table-down:before {
  content: "\fa1c";
}

.ti-table-export:before {
  content: "\eef8";
}

.ti-table-filled:before {
  content: "\f782";
}

.ti-table-heart:before {
  content: "\fa1d";
}

.ti-table-import:before {
  content: "\eef9";
}

.ti-table-minus:before {
  content: "\fa1e";
}

.ti-table-off:before {
  content: "\eefa";
}

.ti-table-options:before {
  content: "\f25c";
}

.ti-table-plus:before {
  content: "\fa1f";
}

.ti-table-row:before {
  content: "\fb00";
}

.ti-table-share:before {
  content: "\fa20";
}

.ti-table-shortcut:before {
  content: "\f25d";
}

.ti-table-spark:before {
  content: "\ffad";
}

.ti-tag:before {
  content: "\10096";
}

.ti-tag-filled:before {
  content: "\ff02";
}

.ti-tag-minus:before {
  content: "\eb34";
}

.ti-tag-off:before {
  content: "\efc0";
}

.ti-tag-plus:before {
  content: "\10097";
}

.ti-tag-starred:before {
  content: "\fc99";
}

.ti-tags:before {
  content: "\ef86";
}

.ti-tags-filled:before {
  content: "\ff01";
}

.ti-tags-off:before {
  content: "\efc1";
}

.ti-tallymark-1:before {
  content: "\ec46";
}

.ti-tallymark-2:before {
  content: "\ec47";
}

.ti-tallymark-3:before {
  content: "\ec48";
}

.ti-tallymark-4:before {
  content: "\ec49";
}

.ti-tallymarks:before {
  content: "\ec4a";
}

.ti-tank:before {
  content: "\ed95";
}

.ti-target:before {
  content: "\eb35";
}

.ti-target-arrow:before {
  content: "\f51a";
}

.ti-target-off:before {
  content: "\f1ad";
}

.ti-tax:before {
  content: "\feee";
}

.ti-tax-euro:before {
  content: "\fef0";
}

.ti-tax-pound:before {
  content: "\feef";
}

.ti-teapot:before {
  content: "\f552";
}

.ti-telescope:before {
  content: "\f07d";
}

.ti-telescope-off:before {
  content: "\f1ae";
}

.ti-temperature:before {
  content: "\eb38";
}

.ti-temperature-celsius:before {
  content: "\eb36";
}

.ti-temperature-fahrenheit:before {
  content: "\eb37";
}

.ti-temperature-minus:before {
  content: "\ebed";
}

.ti-temperature-minus-filled:before {
  content: "\10103";
}

.ti-temperature-off:before {
  content: "\f1af";
}

.ti-temperature-plus:before {
  content: "\ebee";
}

.ti-temperature-plus-filled:before {
  content: "\10102";
}

.ti-temperature-snow:before {
  content: "\fda3";
}

.ti-temperature-sun:before {
  content: "\fda4";
}

.ti-template:before {
  content: "\eb39";
}

.ti-template-filled:before {
  content: "\10177";
}

.ti-template-off:before {
  content: "\f1b0";
}

.ti-tent:before {
  content: "\eefb";
}

.ti-tent-off:before {
  content: "\f435";
}

.ti-terminal:before {
  content: "\ebdc";
}

.ti-terminal-2:before {
  content: "\ebef";
}

.ti-test-pipe:before {
  content: "\eb3a";
}

.ti-test-pipe-2:before {
  content: "\f0a4";
}

.ti-test-pipe-2-filled:before {
  content: "\ff53";
}

.ti-test-pipe-off:before {
  content: "\f1b1";
}

.ti-tex:before {
  content: "\f4e0";
}

.ti-text-caption:before {
  content: "\f4a4";
}

.ti-text-color:before {
  content: "\f2dc";
}

.ti-text-decrease:before {
  content: "\f202";
}

.ti-text-direction-ltr:before {
  content: "\eefc";
}

.ti-text-direction-rtl:before {
  content: "\eefd";
}

.ti-text-grammar:before {
  content: "\fd6d";
}

.ti-text-increase:before {
  content: "\f203";
}

.ti-text-orientation:before {
  content: "\f2a4";
}

.ti-text-plus:before {
  content: "\f2a5";
}

.ti-text-recognition:before {
  content: "\f204";
}

.ti-text-resize:before {
  content: "\ef87";
}

.ti-text-scan-2:before {
  content: "\fcc3";
}

.ti-text-size:before {
  content: "\f2b1";
}

.ti-text-spellcheck:before {
  content: "\f2a6";
}

.ti-text-wrap:before {
  content: "\ebdd";
}

.ti-text-wrap-column:before {
  content: "\feb2";
}

.ti-text-wrap-disabled:before {
  content: "\eca7";
}

.ti-texture:before {
  content: "\f51b";
}

.ti-theater:before {
  content: "\f79b";
}

.ti-thermometer:before {
  content: "\ef67";
}

.ti-thumb-down:before {
  content: "\eb3b";
}

.ti-thumb-down-filled:before {
  content: "\f6aa";
}

.ti-thumb-down-off:before {
  content: "\f436";
}

.ti-thumb-up:before {
  content: "\eb3c";
}

.ti-thumb-up-filled:before {
  content: "\f6ab";
}

.ti-thumb-up-off:before {
  content: "\f437";
}

.ti-tic-tac:before {
  content: "\f51c";
}

.ti-ticket:before {
  content: "\eb3d";
}

.ti-ticket-off:before {
  content: "\f1b2";
}

.ti-tie:before {
  content: "\f07e";
}

.ti-tilde:before {
  content: "\f4a5";
}

.ti-tilt-shift:before {
  content: "\eefe";
}

.ti-tilt-shift-filled:before {
  content: "\fec7";
}

.ti-tilt-shift-off:before {
  content: "\f1b3";
}

.ti-time-duration-0:before {
  content: "\fad4";
}

.ti-time-duration-10:before {
  content: "\fad5";
}

.ti-time-duration-15:before {
  content: "\fad6";
}

.ti-time-duration-30:before {
  content: "\fad7";
}

.ti-time-duration-45:before {
  content: "\fad8";
}

.ti-time-duration-5:before {
  content: "\fad9";
}

.ti-time-duration-60:before {
  content: "\fada";
}

.ti-time-duration-90:before {
  content: "\fadb";
}

.ti-time-duration-off:before {
  content: "\fadc";
}

.ti-timeline:before {
  content: "\f031";
}

.ti-timeline-event:before {
  content: "\f553";
}

.ti-timeline-event-exclamation:before {
  content: "\f662";
}

.ti-timeline-event-filled:before {
  content: "\fd18";
}

.ti-timeline-event-minus:before {
  content: "\f663";
}

.ti-timeline-event-plus:before {
  content: "\f664";
}

.ti-timeline-event-text:before {
  content: "\f665";
}

.ti-timeline-event-x:before {
  content: "\f666";
}

.ti-timezone:before {
  content: "\feed";
}

.ti-tip-jar:before {
  content: "\feea";
}

.ti-tip-jar-euro:before {
  content: "\feec";
}

.ti-tip-jar-pound:before {
  content: "\feeb";
}

.ti-tir:before {
  content: "\ebf0";
}

.ti-toggle-left:before {
  content: "\eb3e";
}

.ti-toggle-left-filled:before {
  content: "\fec0";
}

.ti-toggle-right:before {
  content: "\eb3f";
}

.ti-toggle-right-filled:before {
  content: "\febf";
}

.ti-toilet-paper:before {
  content: "\efd3";
}

.ti-toilet-paper-off:before {
  content: "\f1b4";
}

.ti-toml:before {
  content: "\fa5d";
}

.ti-tool:before {
  content: "\eb40";
}

.ti-tools:before {
  content: "\ebca";
}

.ti-tools-kitchen:before {
  content: "\ed64";
}

.ti-tools-kitchen-2:before {
  content: "\eeff";
}

.ti-tools-kitchen-2-off:before {
  content: "\f1b5";
}

.ti-tools-kitchen-3:before {
  content: "\fd2b";
}

.ti-tools-kitchen-off:before {
  content: "\f1b6";
}

.ti-tools-off:before {
  content: "\f1b7";
}

.ti-tooltip:before {
  content: "\f2dd";
}

.ti-topology-bus:before {
  content: "\f5d9";
}

.ti-topology-complex:before {
  content: "\f5da";
}

.ti-topology-full:before {
  content: "\f5dc";
}

.ti-topology-full-hierarchy:before {
  content: "\f5db";
}

.ti-topology-ring:before {
  content: "\f5df";
}

.ti-topology-ring-2:before {
  content: "\f5dd";
}

.ti-topology-ring-3:before {
  content: "\f5de";
}

.ti-topology-star:before {
  content: "\f5e5";
}

.ti-topology-star-2:before {
  content: "\f5e0";
}

.ti-topology-star-3:before {
  content: "\f5e1";
}

.ti-topology-star-ring:before {
  content: "\f5e4";
}

.ti-topology-star-ring-2:before {
  content: "\f5e2";
}

.ti-topology-star-ring-3:before {
  content: "\f5e3";
}

.ti-torii:before {
  content: "\f59b";
}

.ti-tornado:before {
  content: "\ece2";
}

.ti-tournament:before {
  content: "\ecd0";
}

.ti-tower:before {
  content: "\f2cb";
}

.ti-tower-off:before {
  content: "\f2ca";
}

.ti-track:before {
  content: "\ef00";
}

.ti-tractor:before {
  content: "\ec0d";
}

.ti-trademark:before {
  content: "\ec0e";
}

.ti-traffic-cone:before {
  content: "\ec0f";
}

.ti-traffic-cone-off:before {
  content: "\f1b8";
}

.ti-traffic-lights:before {
  content: "\ed39";
}

.ti-traffic-lights-off:before {
  content: "\f1b9";
}

.ti-train:before {
  content: "\ed96";
}

.ti-train-filled:before {
  content: "\100f5";
}

.ti-transaction-bitcoin:before {
  content: "\fd6e";
}

.ti-transaction-dollar:before {
  content: "\fd6f";
}

.ti-transaction-euro:before {
  content: "\fd70";
}

.ti-transaction-pound:before {
  content: "\fd71";
}

.ti-transaction-rupee:before {
  content: "\fd85";
}

.ti-transaction-yen:before {
  content: "\fd72";
}

.ti-transaction-yuan:before {
  content: "\fd73";
}

.ti-transfer:before {
  content: "\fc1f";
}

.ti-transfer-in:before {
  content: "\ef2f";
}

.ti-transfer-out:before {
  content: "\ef30";
}

.ti-transfer-vertical:before {
  content: "\fc1e";
}

.ti-transform:before {
  content: "\f38e";
}

.ti-transform-filled:before {
  content: "\f6ac";
}

.ti-transform-point:before {
  content: "\fda9";
}

.ti-transform-point-bottom-left:before {
  content: "\fda5";
}

.ti-transform-point-bottom-right:before {
  content: "\fda6";
}

.ti-transform-point-top-left:before {
  content: "\fda7";
}

.ti-transform-point-top-right:before {
  content: "\fda8";
}

.ti-transition-bottom:before {
  content: "\f2b2";
}

.ti-transition-bottom-filled:before {
  content: "\fdd0";
}

.ti-transition-left:before {
  content: "\f2b3";
}

.ti-transition-left-filled:before {
  content: "\fdcf";
}

.ti-transition-right:before {
  content: "\f2b4";
}

.ti-transition-right-filled:before {
  content: "\fdce";
}

.ti-transition-top:before {
  content: "\f2b5";
}

.ti-transition-top-filled:before {
  content: "\fdcd";
}

.ti-trash:before {
  content: "\eb41";
}

.ti-trash-filled:before {
  content: "\f783";
}

.ti-trash-off:before {
  content: "\ed65";
}

.ti-trash-x:before {
  content: "\ef88";
}

.ti-trash-x-filled:before {
  content: "\f784";
}

.ti-treadmill:before {
  content: "\fa6e";
}

.ti-tree:before {
  content: "\ef01";
}

.ti-trees:before {
  content: "\ec10";
}

.ti-trekking:before {
  content: "\f5ad";
}

.ti-trending-down:before {
  content: "\eb42";
}

.ti-trending-down-2:before {
  content: "\edc1";
}

.ti-trending-down-3:before {
  content: "\edc2";
}

.ti-trending-up:before {
  content: "\eb43";
}

.ti-trending-up-2:before {
  content: "\edc3";
}

.ti-trending-up-3:before {
  content: "\edc4";
}

.ti-triangle:before {
  content: "\eb44";
}

.ti-triangle-filled:before {
  content: "\f6ad";
}

.ti-triangle-inverted:before {
  content: "\f01d";
}

.ti-triangle-inverted-filled:before {
  content: "\f6ae";
}

.ti-triangle-minus:before {
  content: "\fc9b";
}

.ti-triangle-minus-2:before {
  content: "\fc9a";
}

.ti-triangle-off:before {
  content: "\ef02";
}

.ti-triangle-plus:before {
  content: "\fc9d";
}

.ti-triangle-plus-2:before {
  content: "\fc9c";
}

.ti-triangle-square-circle:before {
  content: "\ece8";
}

.ti-triangle-square-circle-filled:before {
  content: "\fb42";
}

.ti-triangles:before {
  content: "\f0a5";
}

.ti-trident:before {
  content: "\ecc5";
}

.ti-trolley:before {
  content: "\f4cc";
}

.ti-trolley-filled:before {
  content: "\100f4";
}

.ti-trophy:before {
  content: "\eb45";
}

.ti-trophy-filled:before {
  content: "\f6af";
}

.ti-trophy-off:before {
  content: "\f438";
}

.ti-trowel:before {
  content: "\f368";
}

.ti-truck:before {
  content: "\ebc4";
}

.ti-truck-delivery:before {
  content: "\ec4b";
}

.ti-truck-filled:before {
  content: "\100f3";
}

.ti-truck-loading:before {
  content: "\f1da";
}

.ti-truck-off:before {
  content: "\ef03";
}

.ti-truck-return:before {
  content: "\ec4c";
}

.ti-txt:before {
  content: "\f3b1";
}

.ti-typeface:before {
  content: "\fdab";
}

.ti-typography:before {
  content: "\ebc5";
}

.ti-typography-off:before {
  content: "\f1ba";
}

.ti-u-turn-left:before {
  content: "\fea2";
}

.ti-u-turn-right:before {
  content: "\fea1";
}

.ti-ufo:before {
  content: "\f26f";
}

.ti-ufo-filled:before {
  content: "\10165";
}

.ti-ufo-off:before {
  content: "\f26e";
}

.ti-uhd:before {
  content: "\100aa";
}

.ti-umbrella:before {
  content: "\ebf1";
}

.ti-umbrella-2:before {
  content: "\ff0e";
}

.ti-umbrella-closed:before {
  content: "\ff0c";
}

.ti-umbrella-closed-2:before {
  content: "\ff0d";
}

.ti-umbrella-filled:before {
  content: "\f6b0";
}

.ti-umbrella-off:before {
  content: "\f1bb";
}

.ti-underline:before {
  content: "\eba2";
}

.ti-universe:before {
  content: "\fcc4";
}

.ti-unlink:before {
  content: "\eb46";
}

.ti-upload:before {
  content: "\eb47";
}

.ti-urgent:before {
  content: "\eb48";
}

.ti-usb:before {
  content: "\f00c";
}

.ti-user:before {
  content: "\eb4d";
}

.ti-user-bitcoin:before {
  content: "\ff30";
}

.ti-user-bolt:before {
  content: "\f9d1";
}

.ti-user-cancel:before {
  content: "\f9d2";
}

.ti-user-check:before {
  content: "\eb49";
}

.ti-user-circle:before {
  content: "\ef68";
}

.ti-user-code:before {
  content: "\f9d3";
}

.ti-user-cog:before {
  content: "\f9d4";
}

.ti-user-dollar:before {
  content: "\f9d5";
}

.ti-user-down:before {
  content: "\f9d6";
}

.ti-user-edit:before {
  content: "\f7cc";
}

.ti-user-exclamation:before {
  content: "\ec12";
}

.ti-user-filled:before {
  content: "\fd19";
}

.ti-user-heart:before {
  content: "\f7cd";
}

.ti-user-hexagon:before {
  content: "\fc4e";
}

.ti-user-minus:before {
  content: "\eb4a";
}

.ti-user-off:before {
  content: "\ecf9";
}

.ti-user-pause:before {
  content: "\f9d7";
}

.ti-user-pentagon:before {
  content: "\fc4f";
}

.ti-user-pin:before {
  content: "\f7ce";
}

.ti-user-plus:before {
  content: "\eb4b";
}

.ti-user-question:before {
  content: "\f7cf";
}

.ti-user-scan:before {
  content: "\fcaf";
}

.ti-user-screen:before {
  content: "\fea0";
}

.ti-user-search:before {
  content: "\ef89";
}

.ti-user-share:before {
  content: "\f9d8";
}

.ti-user-shield:before {
  content: "\f7d0";
}

.ti-user-square:before {
  content: "\fc51";
}

.ti-user-square-rounded:before {
  content: "\fc50";
}

.ti-user-star:before {
  content: "\f7d1";
}

.ti-user-up:before {
  content: "\f7d2";
}

.ti-user-x:before {
  content: "\eb4c";
}

.ti-users:before {
  content: "\ebf2";
}

.ti-users-group:before {
  content: "\fa21";
}

.ti-users-minus:before {
  content: "\fa0e";
}

.ti-users-plus:before {
  content: "\fa0f";
}

.ti-uv-index:before {
  content: "\f3b2";
}

.ti-ux-circle:before {
  content: "\f369";
}

.ti-vaccine:before {
  content: "\ef04";
}

.ti-vaccine-bottle:before {
  content: "\ef69";
}

.ti-vaccine-bottle-off:before {
  content: "\f439";
}

.ti-vaccine-off:before {
  content: "\f1bc";
}

.ti-vacuum-cleaner:before {
  content: "\f5e6";
}

.ti-variable:before {
  content: "\ef05";
}

.ti-variable-minus:before {
  content: "\f36a";
}

.ti-variable-off:before {
  content: "\f1bd";
}

.ti-variable-plus:before {
  content: "\f36b";
}

.ti-vector:before {
  content: "\eca9";
}

.ti-vector-bezier:before {
  content: "\ef1d";
}

.ti-vector-bezier-2:before {
  content: "\f1a3";
}

.ti-vector-bezier-arc:before {
  content: "\f4cd";
}

.ti-vector-bezier-circle:before {
  content: "\f4ce";
}

.ti-vector-off:before {
  content: "\f1be";
}

.ti-vector-spline:before {
  content: "\f565";
}

.ti-vector-triangle:before {
  content: "\eca8";
}

.ti-vector-triangle-off:before {
  content: "\f1bf";
}

.ti-venus:before {
  content: "\ec86";
}

.ti-versions:before {
  content: "\ed52";
}

.ti-versions-filled:before {
  content: "\f6b1";
}

.ti-versions-off:before {
  content: "\f1c0";
}

.ti-video:before {
  content: "\ed22";
}

.ti-video-filled:before {
  content: "\1009b";
}

.ti-video-minus:before {
  content: "\ed1f";
}

.ti-video-off:before {
  content: "\ed20";
}

.ti-video-plus:before {
  content: "\ed21";
}

.ti-view-360:before {
  content: "\ed84";
}

.ti-view-360-arrow:before {
  content: "\f62f";
}

.ti-view-360-number:before {
  content: "\f566";
}

.ti-view-360-off:before {
  content: "\f1c1";
}

.ti-viewfinder:before {
  content: "\eb4e";
}

.ti-viewfinder-off:before {
  content: "\f1c2";
}

.ti-viewport-narrow:before {
  content: "\ebf3";
}

.ti-viewport-short:before {
  content: "\fee9";
}

.ti-viewport-tall:before {
  content: "\fee8";
}

.ti-viewport-wide:before {
  content: "\ebf4";
}

.ti-vinyl:before {
  content: "\f00d";
}

.ti-vip:before {
  content: "\f3b3";
}

.ti-vip-off:before {
  content: "\f43a";
}

.ti-virus:before {
  content: "\eb74";
}

.ti-virus-off:before {
  content: "\ed66";
}

.ti-virus-search:before {
  content: "\ed67";
}

.ti-vocabulary:before {
  content: "\ef1e";
}

.ti-vocabulary-off:before {
  content: "\f43b";
}

.ti-volcano:before {
  content: "\f79c";
}

.ti-volume:before {
  content: "\eb51";
}

.ti-volume-2:before {
  content: "\eb4f";
}

.ti-volume-3:before {
  content: "\eb50";
}

.ti-volume-off:before {
  content: "\f1c3";
}

.ti-vs:before {
  content: "\fc52";
}

.ti-walk:before {
  content: "\ec87";
}

.ti-wall:before {
  content: "\ef7a";
}

.ti-wall-off:before {
  content: "\f43c";
}

.ti-wallet:before {
  content: "\eb75";
}

.ti-wallet-off:before {
  content: "\f1c4";
}

.ti-wallpaper:before {
  content: "\ef56";
}

.ti-wallpaper-off:before {
  content: "\f1c5";
}

.ti-wand:before {
  content: "\ebcb";
}

.ti-wand-off:before {
  content: "\f1c6";
}

.ti-wash:before {
  content: "\f311";
}

.ti-wash-dry:before {
  content: "\f304";
}

.ti-wash-dry-1:before {
  content: "\f2fa";
}

.ti-wash-dry-2:before {
  content: "\f2fb";
}

.ti-wash-dry-3:before {
  content: "\f2fc";
}

.ti-wash-dry-a:before {
  content: "\f2fd";
}

.ti-wash-dry-dip:before {
  content: "\f2fe";
}

.ti-wash-dry-f:before {
  content: "\f2ff";
}

.ti-wash-dry-flat:before {
  content: "\fa7f";
}

.ti-wash-dry-hang:before {
  content: "\f300";
}

.ti-wash-dry-off:before {
  content: "\f301";
}

.ti-wash-dry-p:before {
  content: "\f302";
}

.ti-wash-dry-shade:before {
  content: "\f303";
}

.ti-wash-dry-w:before {
  content: "\f322";
}

.ti-wash-dryclean:before {
  content: "\f305";
}

.ti-wash-dryclean-off:before {
  content: "\f323";
}

.ti-wash-eco:before {
  content: "\fa80";
}

.ti-wash-gentle:before {
  content: "\f306";
}

.ti-wash-hand:before {
  content: "\fa81";
}

.ti-wash-machine:before {
  content: "\f25e";
}

.ti-wash-off:before {
  content: "\f307";
}

.ti-wash-press:before {
  content: "\f308";
}

.ti-wash-temperature-1:before {
  content: "\f309";
}

.ti-wash-temperature-2:before {
  content: "\f30a";
}

.ti-wash-temperature-3:before {
  content: "\f30b";
}

.ti-wash-temperature-4:before {
  content: "\f30c";
}

.ti-wash-temperature-5:before {
  content: "\f30d";
}

.ti-wash-temperature-6:before {
  content: "\f30e";
}

.ti-wash-tumble-dry:before {
  content: "\f30f";
}

.ti-wash-tumble-off:before {
  content: "\f310";
}

.ti-waterpolo:before {
  content: "\fa6f";
}

.ti-wave-saw-tool:before {
  content: "\ecd3";
}

.ti-wave-sine:before {
  content: "\ecd4";
}

.ti-wave-square:before {
  content: "\ecd5";
}

.ti-waves-electricity:before {
  content: "\fcc5";
}

.ti-webhook:before {
  content: "\f01e";
}

.ti-webhook-off:before {
  content: "\f43d";
}

.ti-weight:before {
  content: "\f589";
}

.ti-wheat:before {
  content: "\100a8";
}

.ti-wheat-off:before {
  content: "\100a9";
}

.ti-wheel:before {
  content: "\fc64";
}

.ti-wheelchair:before {
  content: "\f1db";
}

.ti-wheelchair-off:before {
  content: "\f43e";
}

.ti-whirl:before {
  content: "\f51d";
}

.ti-wifi:before {
  content: "\eb52";
}

.ti-wifi-0:before {
  content: "\eba3";
}

.ti-wifi-1:before {
  content: "\eba4";
}

.ti-wifi-2:before {
  content: "\eba5";
}

.ti-wifi-off:before {
  content: "\ecfa";
}

.ti-wind:before {
  content: "\ec34";
}

.ti-wind-electricity:before {
  content: "\fcc6";
}

.ti-wind-off:before {
  content: "\f1c7";
}

.ti-windmill:before {
  content: "\ed85";
}

.ti-windmill-filled:before {
  content: "\f6b2";
}

.ti-windmill-off:before {
  content: "\f1c8";
}

.ti-window:before {
  content: "\ef06";
}

.ti-window-maximize:before {
  content: "\f1f1";
}

.ti-window-minimize:before {
  content: "\f1f2";
}

.ti-window-off:before {
  content: "\f1c9";
}

.ti-windsock:before {
  content: "\f06d";
}

.ti-windsock-filled:before {
  content: "\1009a";
}

.ti-wiper:before {
  content: "\ecab";
}

.ti-wiper-wash:before {
  content: "\ecaa";
}

.ti-woman:before {
  content: "\eb53";
}

.ti-woman-filled:before {
  content: "\fdcc";
}

.ti-wood:before {
  content: "\f359";
}

.ti-world:before {
  content: "\eb54";
}

.ti-world-bolt:before {
  content: "\f9d9";
}

.ti-world-cancel:before {
  content: "\f9da";
}

.ti-world-check:before {
  content: "\f9db";
}

.ti-world-code:before {
  content: "\f9dc";
}

.ti-world-cog:before {
  content: "\f9dd";
}

.ti-world-dollar:before {
  content: "\f9de";
}

.ti-world-down:before {
  content: "\f9df";
}

.ti-world-download:before {
  content: "\ef8a";
}

.ti-world-exclamation:before {
  content: "\f9e0";
}

.ti-world-heart:before {
  content: "\f9e1";
}

.ti-world-latitude:before {
  content: "\ed2e";
}

.ti-world-longitude:before {
  content: "\ed2f";
}

.ti-world-minus:before {
  content: "\f9e2";
}

.ti-world-off:before {
  content: "\f1ca";
}

.ti-world-pause:before {
  content: "\f9e3";
}

.ti-world-pin:before {
  content: "\f9e4";
}

.ti-world-plus:before {
  content: "\f9e5";
}

.ti-world-question:before {
  content: "\f9e6";
}

.ti-world-search:before {
  content: "\f9e7";
}

.ti-world-share:before {
  content: "\f9e8";
}

.ti-world-star:before {
  content: "\f9e9";
}

.ti-world-up:before {
  content: "\f9ea";
}

.ti-world-upload:before {
  content: "\ef8b";
}

.ti-world-www:before {
  content: "\f38f";
}

.ti-world-x:before {
  content: "\f9eb";
}

.ti-wrecking-ball:before {
  content: "\ed97";
}

.ti-writing:before {
  content: "\ef08";
}

.ti-writing-off:before {
  content: "\f1cb";
}

.ti-writing-sign:before {
  content: "\ef07";
}

.ti-writing-sign-off:before {
  content: "\f1cc";
}

.ti-x:before {
  content: "\eb55";
}

.ti-x-power-y:before {
  content: "\10072";
}

.ti-xbox-a:before {
  content: "\f2b6";
}

.ti-xbox-a-filled:before {
  content: "\fdcb";
}

.ti-xbox-b:before {
  content: "\f2b7";
}

.ti-xbox-b-filled:before {
  content: "\fdca";
}

.ti-xbox-x:before {
  content: "\f2b8";
}

.ti-xbox-x-filled:before {
  content: "\fdc9";
}

.ti-xbox-y:before {
  content: "\f2b9";
}

.ti-xbox-y-filled:before {
  content: "\fdc8";
}

.ti-xd:before {
  content: "\fa33";
}

.ti-xxx:before {
  content: "\fc20";
}

.ti-yin-yang:before {
  content: "\ec35";
}

.ti-yin-yang-filled:before {
  content: "\f785";
}

.ti-yoga:before {
  content: "\f01f";
}

.ti-zeppelin:before {
  content: "\f270";
}

.ti-zeppelin-filled:before {
  content: "\fdc7";
}

.ti-zeppelin-off:before {
  content: "\f43f";
}

.ti-zip:before {
  content: "\f3b4";
}

.ti-zodiac-aquarius:before {
  content: "\ecac";
}

.ti-zodiac-aries:before {
  content: "\ecad";
}

.ti-zodiac-cancer:before {
  content: "\ecae";
}

.ti-zodiac-capricorn:before {
  content: "\ecaf";
}

.ti-zodiac-gemini:before {
  content: "\ecb0";
}

.ti-zodiac-leo:before {
  content: "\ecb1";
}

.ti-zodiac-libra:before {
  content: "\ecb2";
}

.ti-zodiac-pisces:before {
  content: "\ecb3";
}

.ti-zodiac-sagittarius:before {
  content: "\ecb4";
}

.ti-zodiac-scorpio:before {
  content: "\ecb5";
}

.ti-zodiac-taurus:before {
  content: "\ecb6";
}

.ti-zodiac-virgo:before {
  content: "\ecb7";
}

.ti-zoom:before {
  content: "\fdaa";
}

.ti-zoom-cancel:before {
  content: "\ec4d";
}

.ti-zoom-cancel-filled:before {
  content: "\fdc6";
}

.ti-zoom-check:before {
  content: "\ef09";
}

.ti-zoom-check-filled:before {
  content: "\f786";
}

.ti-zoom-code:before {
  content: "\f07f";
}

.ti-zoom-code-filled:before {
  content: "\fdc5";
}

.ti-zoom-exclamation:before {
  content: "\f080";
}

.ti-zoom-exclamation-filled:before {
  content: "\fdc4";
}

.ti-zoom-filled:before {
  content: "\f787";
}

.ti-zoom-in:before {
  content: "\eb56";
}

.ti-zoom-in-area:before {
  content: "\f1dc";
}

.ti-zoom-in-area-filled:before {
  content: "\f788";
}

.ti-zoom-in-filled:before {
  content: "\f789";
}

.ti-zoom-money:before {
  content: "\ef0a";
}

.ti-zoom-money-filled:before {
  content: "\fdc3";
}

.ti-zoom-out:before {
  content: "\eb57";
}

.ti-zoom-out-area:before {
  content: "\f1dd";
}

.ti-zoom-out-area-filled:before {
  content: "\fdc2";
}

.ti-zoom-out-filled:before {
  content: "\f78a";
}

.ti-zoom-pan:before {
  content: "\f1de";
}

.ti-zoom-pan-filled:before {
  content: "\fdc1";
}

.ti-zoom-question:before {
  content: "\edeb";
}

.ti-zoom-question-filled:before {
  content: "\fdc0";
}

.ti-zoom-replace:before {
  content: "\f2a7";
}

.ti-zoom-reset:before {
  content: "\f295";
}

.ti-zoom-scan:before {
  content: "\fcb0";
}

.ti-zoom-scan-filled:before {
  content: "\fdbf";
}

.ti-zzz:before {
  content: "\f228";
}

.ti-zzz-off:before {
  content: "\f440";
}

.ti-123:before {
  content: "\f554";
}

.ti-360:before {
  content: "\f62f";
}

.ti-code-asterix:before {
  content: "\f312";
}

.ti-discount-2:before {
  content: "\ee7c";
}

.ti-discount-2-off:before {
  content: "\f3e6";
}

.ti-discount-check:before {
  content: "\f1f8";
}

.ti-hand-rock:before {
  content: "\ee97";
}

.ti-sort-deacending-small-big:before {
  content: "\fd96";
}

.ti-shi-jumping:before {
  content: "\fa6c";
}

.ti-box-seam:before {
  content: "\eaff";
}

.ti-kering:before {
  content: "\efb8";
}

.ti-2fa:before {
  content: "\eca0";
}

.ti-3d-cube-sphere:before {
  content: "\ecd7";
}

.ti-3d-cube-sphere-off:before {
  content: "\f3b5";
}

.ti-3d-rotate:before {
  content: "\f020";
}

.ti-12-hours:before {
  content: "\fc53";
}

.ti-24-hours:before {
  content: "\f5e7";
}

.ti-360-view:before {
  content: "\f566";
}

.ti-circle-0:before {
  content: "\ee34";
}

.ti-circle-1:before {
  content: "\ee35";
}

.ti-circle-2:before {
  content: "\ee36";
}

.ti-circle-3:before {
  content: "\ee37";
}

.ti-circle-4:before {
  content: "\ee38";
}

.ti-circle-5:before {
  content: "\ee39";
}

.ti-circle-6:before {
  content: "\ee3a";
}

.ti-circle-7:before {
  content: "\ee3b";
}

.ti-circle-8:before {
  content: "\ee3c";
}

.ti-circle-9:before {
  content: "\ee3d";
}

.ti-hexagon-0:before {
  content: "\f459";
}

.ti-hexagon-1:before {
  content: "\f45a";
}

.ti-hexagon-2:before {
  content: "\f45b";
}

.ti-hexagon-3:before {
  content: "\f45c";
}

.ti-hexagon-4:before {
  content: "\f45d";
}

.ti-hexagon-5:before {
  content: "\f45e";
}

.ti-hexagon-6:before {
  content: "\f45f";
}

.ti-hexagon-7:before {
  content: "\f460";
}

.ti-hexagon-8:before {
  content: "\f461";
}

.ti-hexagon-9:before {
  content: "\f462";
}

.ti-square-0:before {
  content: "\eee5";
}

.ti-square-1:before {
  content: "\eee6";
}

.ti-square-2:before {
  content: "\eee7";
}

.ti-square-3:before {
  content: "\eee8";
}

.ti-square-4:before {
  content: "\eee9";
}

.ti-square-5:before {
  content: "\eeea";
}

.ti-square-6:before {
  content: "\eeeb";
}

.ti-square-7:before {
  content: "\eeec";
}

.ti-square-8:before {
  content: "\eeed";
}

.ti-square-9:before {
  content: "\eeee";
}

.ti-message-circle-2:before {
  content: "\eaed";
}

.ti-mood-suprised:before {
  content: "\ec04";
}

.ti-circle-dashed-letter-letter-v:before {
  content: "\ff84";
}

.ti-seeding:before {
  content: "\ed51";
}

.ti-seeding-off:before {
  content: "\f19d";
}

.ti-discount-check-filled:before {
  content: "\f746";
}

.ti-message-circle-2-filled:before {
  content: "\fecf";
}

.ti-seeding-filled:before {
  content: "\10006";
}
